package it.asterisk.fastagi.internal;

import java.util.List;
import java.util.concurrent.BlockingQueue;

import it.asterisk.fastagi.AgiException;
import it.asterisk.fastagi.AgiReader;
import it.asterisk.fastagi.AgiRequest;
import it.asterisk.fastagi.reply.AgiReply;
import it.asterisk.manager.ManagerConnection;
import it.asterisk.manager.event.AsyncAgiEvent;

public class AsyncAgiReader implements AgiReader
{
    private final AgiRequestImpl request;
    private final BlockingQueue<AsyncAgiEvent> asyncAgiEvents;

    public AsyncAgiReader(ManagerConnection connection, List<String> environment, BlockingQueue<AsyncAgiEvent> asyncAgiEvents)
    {
        this.request = new AgiRequestImpl(environment);
        this.asyncAgiEvents = asyncAgiEvents;

        request.setLocalAddress(connection.getLocalAddress());
        request.setLocalPort(connection.getLocalPort());
        request.setRemoteAddress(connection.getRemoteAddress());
        request.setRemotePort(connection.getRemotePort());
    }

    public AgiRequest readRequest() throws AgiException
    {
        return request;
    }

    public AgiReply readReply() throws AgiException
    {
        try
        {
            return new AgiReplyImpl(asyncAgiEvents.take().decodeResult());
        }
        catch (InterruptedException e)
        {
            Thread.currentThread().interrupt();
            throw new AgiException("Interrupted while waiting for AsyncAgiEvent", e);
        }
    }
}
