package it.asterisk.fastagi.internal;

import java.io.IOException;

import it.asterisk.fastagi.AgiException;
import it.asterisk.fastagi.AgiWriter;
import it.asterisk.fastagi.command.AgiCommand;
import it.asterisk.manager.ManagerConnection;
import it.asterisk.manager.TimeoutException;
import it.asterisk.manager.action.AgiAction;
import it.asterisk.manager.response.ManagerError;
import it.asterisk.manager.response.ManagerResponse;

/**
 * Implementation of AgiWriter that uses a {@link it.asterisk.manager.ManagerConnection} to send
 * {@link it.asterisk.fastagi.command.AgiCommand AgiCommands} as part of an AsyncAgi conversation.
 *
 * @see it.asterisk.manager.ManagerConnection
 * @see it.asterisk.manager.action.AgiAction
 * @since 1.0.0 
 */
public class AsyncAgiWriter implements AgiWriter
{
    private final ManagerConnection connection;
    private volatile String channelName;

    public AsyncAgiWriter(ManagerConnection connection, String channelName)
    {
        this.connection = connection;
        this.channelName = channelName;
    }

    public void sendCommand(AgiCommand command) throws AgiException
    {
        final AgiAction agiAction;
        final ManagerResponse response;

        agiAction = new AgiAction(channelName, command.buildCommand());

        try
        {
            response = connection.sendAction(agiAction);
        }
        catch (IOException e)
        {
            throw new AgiException("Unable to send AsyncAGI command to " + connection.getHostname() +
                    " for channel " + channelName, e);
        }
        catch (TimeoutException e)
        {
            throw new AgiException("Timeout while sending AsyncAGI command to " + connection.getHostname() +
                    " for channel " + channelName , e);
        }

        if (response instanceof ManagerError)
        {
            throw new AgiException("Unable to send AsyncAGI command to " + connection.getHostname() +
                    " for channel " + channelName + ": " + response.getMessage());
        }
    }

    public void updateChannelName(String channelName)
    {
        this.channelName = channelName;
    }
}
