/*
 *  Copyright 2004-2006 Stefan Reuter
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
package it.asterisk.manager;

import java.util.Collection;

import it.asterisk.manager.event.ResponseEvent;
import it.asterisk.manager.response.ManagerResponse;


/**
 * Contains the result of executing an 
 * {@link it.asterisk.manager.action.EventGeneratingAction}, that is the
 * {@link it.asterisk.manager.response.ManagerResponse} and any received 
 * {@link it.asterisk.manager.event.ManagerEvent}s.
 * 
 * @see it.asterisk.manager.action.EventGeneratingAction
 * @author srt
 * @version $Id$
 * @since 0.2
 */
public interface ResponseEvents
{
    /**
     * Returns the response received.
     * 
     * @return the response received.
     */
    ManagerResponse getResponse();

    /**
     * Returns a Collection of ManagerEvents that have been received including
     * the last one that indicates completion.
     * 
     * @return a Collection of ManagerEvents received.
     */
    Collection<ResponseEvent> getEvents();
}
