package it.asterisk.pbx.activities;

import it.asterisk.pbx.Activity;
import it.asterisk.pbx.Call;
import it.asterisk.pbx.CallerID;
import it.asterisk.pbx.Channel;
import it.asterisk.pbx.EndPoint;
import it.asterisk.pbx.PBXException;

public interface BlindTransferActivity extends Activity
{
    /**
     * A bridged call is a successful transfer all other options are a failed
     * transfer.
     * 
     * @author bsutton
     */
    enum CompletionCause
    {
        // The call was bridged.
        BRIDGED("Connected")
        // The call hungup
        , HANGUP("Transfer Failed, Caller Hungup or Destination Busy")
        // A timeout occur during the transfer
        // which essentially means the transferTarget didn't answer the phone
        , TIMEOUT("Transfer Failed, Timeout")
        // The cancel method was called.
        , CANCELLED("Transfer Cancelled")
        // Failed
        , FAILED("Transfer Failed");

        String message;

        CompletionCause(String message)
        {
            this.message = message;
        }

        public String getMessage()
        {
            return message;
        }
    }

    Channel getChannelToTransfer();

    CallerID getTransferTargetCallerID();

    EndPoint getTransferTarget();

    Channel getTransferTargetChannel();

    CompletionCause getCompletionCause();

    void cancel();

    /**
     * The call created as a result of the blind transfer being answered.
     * 
     * @return
     * @throws PBXException
     */
    Call getNewCall() throws PBXException;

}
