package it.asterisk.pbx.agi;

import java.io.IOException;
import java.util.concurrent.CountDownLatch;

import it.asterisk.fastagi.AgiChannel;
import it.asterisk.fastagi.AgiException;
import it.asterisk.manager.TimeoutException;
import it.asterisk.pbx.AgiChannelActivityAction;
import it.asterisk.pbx.AsteriskSettings;
import it.asterisk.pbx.Channel;
import it.asterisk.pbx.PBXFactory;
import it.asterisk.pbx.asterisk.wrap.actions.RedirectAction;
import it.asterisk.pbx.internal.core.AsteriskPBX;
import it.callite.log.AsteriskLog;

public class AgiChannelActivityMeetme implements AgiChannelActivityAction
{

    CountDownLatch latch = new CountDownLatch(1);
    private String room;
    private volatile boolean hangup = true;

    private Channel ichannel;

    private String bridgeProfile;

    private String userProfile;

    public AgiChannelActivityMeetme(String room, String bridgeProfile, String userProfile)
    {
        this.room = room;
        this.bridgeProfile = bridgeProfile;
        this.userProfile = userProfile;
    }

    @Override
    public void execute(AgiChannel channel, Channel ichannel) throws AgiException, InterruptedException
    {
        if (ichannel == null)
        {
            throw new NullPointerException("ichannel cannot be null");
        }
        this.ichannel = ichannel;
        channel.confbridge(room, bridgeProfile + "," + userProfile);

        if (hangup)
        {
            try
            {
                channel.hangup();
            }
            catch (Exception e)
            {
                AsteriskLog.warn(e);
            }
        }
        AsteriskLog.info(ichannel + " left the meetme");

    }

    @Override
    public boolean isDisconnect(ActivityAgi activityAgi)
    {
        return false;
    }

    @Override
    public void cancel()
    {
        hangup = false;
        final AsteriskSettings profile = PBXFactory.getActiveProfile();

        AsteriskPBX pbx = (AsteriskPBX) PBXFactory.getActivePBX();
        final RedirectAction redirect = new RedirectAction(ichannel, profile.getManagementContext(), pbx.getExtensionAgi(),
                1);
        try
        {
            pbx.sendAction(redirect, 1000);
        }
        catch (IllegalArgumentException | IllegalStateException | IOException | TimeoutException e)
        {
            AsteriskLog.error(e, e);
        }

    }
}
