package it.asterisk.pbx.agi.config;

import it.asterisk.fastagi.AgiChannel;
import it.asterisk.fastagi.AgiException;
import it.asterisk.fastagi.AgiRequest;
import it.asterisk.fastagi.BaseAgiScript;

/**
 * Allows for auto configuration of the AGI script.<br>
 * <br>
 * Provides a getParameter(..) method which verifies the existence of the
 * parameter. <br>
 * <br>
 * Requires an implementing agi to provide it's name and parameter list via
 * getScriptName() and getParameters.
 * 
 * @author rsutton
 */
abstract public class ServiceAgiScriptImpl extends BaseAgiScript implements ServiceAgiScript
{
    protected AgiRequest request;
    protected AgiChannel channel;

    public synchronized void service(AgiRequest request, AgiChannel channel) throws AgiException
    {
        this.request = request;
        this.channel = channel;
        service();
    }

    /**
     * field variables request and channel will be set prior to calling
     * service() AgiRequest request AgiChannel channel
     * 
     * @throws AgiException
     */

    public abstract void service() throws AgiException;

    protected String required(String name, String parameter) throws AgiException
    {
        if (parameter == null || parameter.length() == 0)
            throw new AgiException("Required parameter " + name + " missing.");
        return parameter;
    }

    protected String getParameter(String name) throws AgiException
    {
        return required(name, request.getParameter(name));
    }

    public abstract String getScriptName();

    public abstract String[] getParameters();

}
