package it.asterisk.pbx.asterisk.wrap.events;

import it.asterisk.pbx.Channel;
import it.asterisk.pbx.InvalidChannelName;
import it.asterisk.pbx.PBXFactory;
import it.asterisk.pbx.internal.core.AsteriskPBX;
import it.asterisk.pbx.internal.core.ChannelProxy;
import it.callite.log.AsteriskLog;

public class RenameEvent extends ManagerEvent implements ChannelEvent
{
    private static final long serialVersionUID = 1L;

    /**
     * The existing channel which is about to be renamed.
     */
    private final Channel _channel;

    /**
     * New name of the channel after renaming occurred.
     */
    private final String _newName;

    private String uniqueId;

    public RenameEvent(final it.asterisk.manager.event.RenameEvent event) throws InvalidChannelName
    {
        super(event);
        final AsteriskPBX pbx = (AsteriskPBX) PBXFactory.getActivePBX();

        this._channel = pbx.internalRegisterChannel(event.getChannel(), event.getUniqueId());
        this._newName = event.getNewname();

        String uniqueId = ((ChannelProxy) this._channel).getRealChannel().getUniqueId();
        AsteriskLog.debug("Renaming :" + uniqueId + " " + event.getUniqueId());

        this.uniqueId = event.getUniqueId();

        assert uniqueId.equalsIgnoreCase("-1")
                || uniqueId.compareToIgnoreCase(event.getUniqueId()) == 0 : "Rename registered against incorrect channel"; //$NON-NLS-1$

    }

    @Override
    public final Channel getChannel()
    {
        return this._channel;
    }

    public final String getNewName()
    {
        return this._newName;
    }

    public final String getUniqueId()
    {
        return uniqueId;
    }

    @Override
    public String toString()
    {
        it.asterisk.manager.event.RenameEvent rawEvent = (it.asterisk.manager.event.RenameEvent) getSource();
        return "RenameEvent: existing channel: " + this._channel + " newname=" + this._newName //$NON-NLS-1$ //$NON-NLS-2$
                + "rawExisting:" + rawEvent.getChannel() + " rawNewname:" + rawEvent.getNewname() //$NON-NLS-1$ //$NON-NLS-2$
                + "rawUniquID:" + rawEvent.getUniqueId(); //$NON-NLS-1$
    }

}
