package it.asterisk.pbx.internal.core;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.TimeUnit;

import it.asterisk.pbx.Activity;
import it.asterisk.pbx.ActivityCallback;
import it.asterisk.pbx.ActivityStatusEnum;
import it.asterisk.pbx.AsteriskSettings;
import it.asterisk.pbx.CallerID;
import it.asterisk.pbx.Channel;
import it.asterisk.pbx.EndPoint;
import it.asterisk.pbx.PBX;
import it.asterisk.pbx.PBXException;
import it.asterisk.pbx.PBXFactory;
import it.asterisk.pbx.agi.ActivityAgi;
import it.asterisk.pbx.agi.ActivityArrivalListener;
import it.asterisk.pbx.asterisk.wrap.actions.OriginateAction;
import it.asterisk.pbx.asterisk.wrap.response.ManagerResponse;
import it.callite.log.AsteriskLog;

public class DialLocalToAgiActivity implements Runnable, Activity
{

    private EndPoint from;
    private CallerID fromCallerID;
    private Thread thread;

    CountDownLatch latch = new CountDownLatch(1);
    private List<Channel> channels = new LinkedList<>();
    private ActivityCallback<DialLocalToAgiActivity> callback;
    private Map<String, String> channelVarsToSet;

    public DialLocalToAgiActivity(EndPoint from, CallerID fromCallerID, ActivityCallback<DialLocalToAgiActivity> callback,
            Map<String, String> channelVarsToSet)
    {
        this.from = from;
        this.fromCallerID = fromCallerID;
        this.callback = callback;
        this.channelVarsToSet = channelVarsToSet;

        thread = new Thread(this, "Dial " + from + " to AGI");
        thread.start();
    }
    // Logger logger = LogManager.getLogger();

    @Override
    public void run()
    {
        AsteriskLog.debug("*******************************************************************************", null);
        AsteriskLog.info("***********                    begin dial local to AGI                  ****************", null);
        AsteriskLog.debug("***********                                                      ****************", null);
        AsteriskLog.debug("*******************************************************************************", null);
        final AsteriskSettings settings = PBXFactory.getActiveProfile();

        AsteriskPBX pbx = (AsteriskPBX) PBXFactory.getActivePBX();

        final OriginateAction originate = new OriginateAction();
        originate.setEndPoint(from);
        originate.setContext(settings.getManagementContext());
        originate.setExten(pbx.getExtensionAgi());
        originate.setPriority(1);
        originate.setCallerId(fromCallerID);
        originate.setTimeout(30000);

        Map<String, String> myVars = new HashMap<>();

        if (channelVarsToSet != null)
        {
            myVars.putAll(channelVarsToSet);
        }

        originate.setVariables(myVars);

        ActivityArrivalListener listener = new ActivityArrivalListener()
        {

            @Override
            public void channelArrived(Channel channel)
            {
                channels.add(channel);
                if (isSuccess())
                {
                    latch.countDown();
                }
            }
        };
        try (AutoCloseable closeable = ActivityAgi.addArrivalListener(originate, listener);)
        {

            ManagerResponse response = pbx.sendAction(originate, 5000);
            if (response.getResponse().compareToIgnoreCase("Success") != 0)
            {
                callback.progress(this, ActivityStatusEnum.FAILURE, ActivityStatusEnum.FAILURE.getDefaultMessage());
            }
            else
            {
                if (latch.await(5, TimeUnit.SECONDS))
                {
                    callback.progress(this, ActivityStatusEnum.SUCCESS, ActivityStatusEnum.SUCCESS.getDefaultMessage());
                }
                else
                {
                    callback.progress(this, ActivityStatusEnum.FAILURE, ActivityStatusEnum.FAILURE.getDefaultMessage());
                }
            }
        }
        catch (Exception e)
        {
            AsteriskLog.error(e, e);
            callback.progress(this, ActivityStatusEnum.FAILURE, ActivityStatusEnum.FAILURE.getDefaultMessage());

        }

    }

    public void abort(final String reason)
    {
        AsteriskLog.warn("Aborting originate ", null);

        for (Channel channel : channels)
        {
            PBX pbx = PBXFactory.getActivePBX();
            try
            {
                pbx.hangup(channel);
            }
            catch (IllegalArgumentException | IllegalStateException | PBXException e)
            {
                AsteriskLog.error(e, e);

            }
        }
        latch.countDown();

    }

    @Override
    public Throwable getLastException()
    {
        return null;
    }

    @Override
    public boolean isSuccess()
    {
        return channels.size() == 2;
    }

    public Channel getChannel1()
    {
        return channels.get(0);

    }

    public Channel getChannel2()
    {
        return channels.get(1);

    }
}
