package it.callite.action;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.faces.model.SelectItem;

import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;

import it.callite.chiamate.Chiamate;
import it.callite.dao.AutodialerDao;
import it.callite.dao.ImpostazioneDao;
import it.callite.model.Utente;
import it.callite.model.sip.SipBuddies;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.Buddy;
import it.callite.utility.UtilChat;
import it.callite.utility.chat.ReceiveBuddy;
import it.callite.utility.item.InternoSI;
@SuppressWarnings("serial")
public class BuddyBean implements Serializable {

	@Autowired
	private ImpostazioneDao impostazioneDao;
	@Autowired
	private AutodialerDao autodialerDao;

	private Buddy buddy;
	private ReceiveBuddy rb = null;
	private List<Buddy> buddyList = new ArrayList<Buddy>();
	private Boolean chatAbilitata = Boolean.FALSE;

	public void apriBuddy() {
		updateBuddy();
	}

	public void chiudiRisposta() {
		buddy = null;
	}

	public Buddy getBuddy() {
		return buddy;
	}
	public void inviaRisposta() {
		if(buddy != null) {
			if(buddy.getInvia()) {
				autodialerDao.sendMessage(buddy.getTo(), buddy.getFrom(), buddy.getBody());
			} else {
				if(rb != null) {
					rb.sendMessage(buddy.getTo(), buddy.getBody());
				}
			}
			buddy.setLetto(true);
			updateBuddy();
		}
		buddy = null;
	}
	public void leggiBuddys() {
		List<Buddy> removeList = new ArrayList<Buddy>();
		for(Buddy b : buddyList) {
			if(!b.isLetto()) {
				b.setLetto(true);
				removeList.add(b);
			}
		}
		if(ApplicationUtil.notEmpty(removeList)) {
			buddyList.removeAll(removeList);
		}
		updateBuddy();
	}
	public void leggiBuddy() {
		Map<String, String> map = ApplicationUtil.getRequestParameterMap();
		if(map != null) {
			String mid = ApplicationUtil.trimToNull((String) map.get("id"));
			if(mid != null) {
				List<Buddy> removeList = new ArrayList<Buddy>();
				for(Buddy b : buddyList) {
					if(!b.isLetto() && ApplicationUtil.equals(mid, b.getId())) {
						b.setLetto(true);
						removeList.add(b);
					}
				}
				if(ApplicationUtil.notEmpty(removeList)) {
					buddyList.removeAll(removeList);
				}
				updateBuddy();
			}
		}
	}

	public void rispondi() {
		Map<String, String> map = ApplicationUtil.getRequestParameterMap();
		if(map != null) {
			String mid = ApplicationUtil.trimToNull((String) map.get("id"));
			if(mid != null) {
				List<Buddy> removeList = new ArrayList<Buddy>();
				for(Buddy b : buddyList) {
					if(ApplicationUtil.equals(mid, b.getId())) {
						b.setLetto(true);
						removeList.add(b);
						buddy = new Buddy(b.getTo(), b.getFrom(), "", false);
						break;
					}
				}
				if(ApplicationUtil.notEmpty(removeList)) {
					buddyList.removeAll(removeList);
				}
				updateBuddy();
			} 
		}
	}

	public void nuova() {
		if(rb != null) {
			buddy = new Buddy(rb.getUtente(), "", "", true);
		}
	}

	public void setBuddy(Buddy buddy) {
		this.buddy = buddy;
	}
	public BuddyBean() {
	}

	public void imposta(Utente u) {
		chatAbilitata = ApplicationUtil.isTrue(impostazioneDao.chatPopup()) && u.isLocale();
		if(u != null && chatAbilitata) {
			UtilChat uc = impostazioneDao.getUtilChat();
			rb = Chiamate.getCodaBuddy().get(u.getInternoString());
			if(rb == null || (rb != null && !rb.isConnected())) {
				rb = new ReceiveBuddy(uc, u.getInternoString());
				Chiamate.getCodaBuddy().put(u.getInternoString(), rb);
			}
			if(rb != null) {
				rb.setUtilChat(uc);
			}
		}
	}

	public boolean isConnected() {
		return rb != null && rb.isConnected();
	}

	public List<Buddy> getBuddyList() {
		return buddyList;
	}

	public Boolean getChatAbilitata() {
		return chatAbilitata;
	}
	public void updateBuddy() {
		if(rb != null && ApplicationUtil.isTrue(chatAbilitata)) {
			List<Buddy> ms = rb.getMessaggi();
			if(ApplicationUtil.notEmpty(ms)) {
				for(Buddy b : ms) {
					if(!buddyList.contains(b) && !b.isLetto()) {
						buddyList.add(b);
					}
				}
			}
		}
	}


	public List<SelectItem> getInterniSI() {
		DetachedCriteria criteria = DetachedCriteria.forClass(SipBuddies.class);
		criteria.add(Restrictions.eq("tipologia", ApplicationUtil.convertBS(true)));
		if(buddy != null && !buddy.isTutto()) {
			criteria.createCriteria("utenti").add(Restrictions.eq("enabled", true));
		}
		//criteria.add(Restrictions.ne("name", rb != null ? rb.getUtente() : ""));
		List<InternoSI> lis = impostazioneDao.getInternoSI(criteria);
		return impostazioneDao.getInternoOnline(lis);
	}
	
	public boolean isValid() {
		if(buddy != null) {
			String m = ApplicationUtil.trimToNull(buddy.getBody());
			if(m != null) {
				return true;
			}
		}
		return false;
	}


}