package it.callite.action;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.faces.component.UIInput;
import javax.faces.event.ValueChangeEvent;

import it.callite.constants.ConstantsGlobal;
import it.callite.model.EsitoArchiviazione;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.ProdottoCheckbox;
import it.callite.utility.Ricerca;

@SuppressWarnings("serial")
public class FilteringBean implements Serializable  {

	private String filterProdotto;
	private String filterCategoria;

	private List<Long> filter = new ArrayList<Long>();
	private List<Long> selezionati = new ArrayList<Long>();

	public boolean filterProdotto(Object current) {
		ProdottoCheckbox currentProdotto = (ProdottoCheckbox) current;
		return filterType(currentProdotto);
	}

	private boolean filterType(ProdottoCheckbox currentProdotto) {
		String prodotto = ApplicationUtil.upperCaseEmpty(currentProdotto.getProdotto().getNomeTipo());
		String categoria = ApplicationUtil.upperCaseEmpty(currentProdotto.getProdotto().getCategoriaDenominazione());
		boolean ricercaProdotto = ApplicationUtil.isBlank(filterProdotto) || ricerca(filterProdotto, prodotto);
		boolean ricercaCategoria = ApplicationUtil.isBlank(filterCategoria) || ricerca(filterCategoria, categoria);
		if (ricercaProdotto && ricercaCategoria) {
			if(ApplicationUtil.gtZero(currentProdotto.getProdottoId()) && !filter.contains(currentProdotto.getProdottoId())) {
				filter.add(currentProdotto.getProdottoId());
			}
			return true;
		} else {
			if(ApplicationUtil.gtZero(currentProdotto.getProdottoId()) && filter.contains(currentProdotto.getProdottoId())) {
				filter.remove(currentProdotto.getProdottoId());
			}
			return false; 
		}
	}


	public void valueChanged(ValueChangeEvent event){
		UIInput component = (UIInput) event.getComponent();
		ProdottoCheckbox prd = (ProdottoCheckbox) component.getAttributes().get("prodotto");
		Long prodottoId = prd.getProdottoId();
		if(event.getNewValue().equals(Boolean.FALSE) && selezionati.contains(prodottoId)) {
			selezionati.remove(prodottoId);
		}
		if(event.getNewValue().equals(Boolean.TRUE) && !selezionati.contains(prodottoId)) {
			selezionati.add(prodottoId);
		}

	}

	public void doFilterReset(List<ProdottoCheckbox> prodotti, EsitoArchiviazione es) {
		filterProdotto = es.getContratto_filtro_prodotto();
		filterCategoria = es.getContratto_filtro_categoria();
		filter.clear();
		selezionati.clear();
		List<ProdottoCheckbox> prodottiDaRimuovere = new ArrayList<ProdottoCheckbox>();
		if(ApplicationUtil.notEmpty(prodotti)) {
			Iterator<ProdottoCheckbox> objIter = prodotti.iterator();
			while(objIter.hasNext()) {
				ProdottoCheckbox currentProdotto = objIter.next();
				if(filterType(currentProdotto)) {
					if(ApplicationUtil.eq(es.getContratto_selezione(), 1)) {
						currentProdotto.setSelezionato(true);
						selezionati.add(currentProdotto.getProdottoId());
					} else {
						currentProdotto.setSelezionato(false);
						selezionati.remove(currentProdotto.getProdottoId());
					}
				} else {
					if(!ApplicationUtil.isTrue(es.getContratto_ricerca())) {
						prodottiDaRimuovere.add(currentProdotto);
					}
				}
			}
			if(ApplicationUtil.notEmpty(prodottiDaRimuovere) && !ApplicationUtil.isTrue(es.getContratto_ricerca())) {
				prodotti.removeAll(prodottiDaRimuovere);
			}
		}
	}

	public List<Long> getFilter() {
		return filter;
	}

	public List<Long> getSelezionati() {
		return selezionati;
	}


	public FilteringBean() {
	}


	public String getFilterCategoria() {
		return ApplicationUtil.trimToEmpty(filterCategoria);
	}

	public void setFilterCategoria(String filterCategoria) {
		this.filterCategoria = ApplicationUtil.trimToEmpty(filterCategoria);
	}

	public String getFilterProdotto() {
		return ApplicationUtil.trimToEmpty(filterProdotto);
	}

	public void setFilterProdotto(String filterProdotto) {
		this.filterProdotto = ApplicationUtil.trimToEmpty(filterProdotto);
	}

	public boolean ricerca(String v, String elemento) {
		if(ApplicationUtil.isNotBlank(elemento)) {
			List<String> valori = ApplicationUtil.tokenU(v, ConstantsGlobal.colonSeparator);
			for(String valore : valori) {
				Ricerca m = ApplicationUtil.match(valore);
				if(m.getValore() == 1) {
					if(m.isInverti()) {
						if(m.getRicerca() != null && !elemento.equals(ApplicationUtil.upperCase(m.getRicerca()))) {
							return true;
						}
					} else {
						if(m.getRicerca() != null && elemento.equals(ApplicationUtil.upperCase(m.getRicerca()))) {
							return true;
						}
					}
				}
				if(m.getValore() == 2) {
					if(m.isInverti()) {
						if(m.getRicerca() != null && !elemento.startsWith(ApplicationUtil.upperCase(m.getRicerca()))) {
							return true;
						}
					} else {
						if(m.getRicerca() != null && elemento.startsWith(ApplicationUtil.upperCase(m.getRicerca()))) {
							return true;
						}
					}
				}
				if(m.getValore() == 3) {
					if(m.isInverti()) {
						if(m.getRicerca() != null && !elemento.endsWith(ApplicationUtil.upperCase(m.getRicerca()))) {
							return true;
						}
					} else {
						if(m.getRicerca() != null && elemento.endsWith(ApplicationUtil.upperCase(m.getRicerca()))) {
							return true;
						}
					}
				}
				if(m.getValore() == 4) {
					if(m.isInverti()) {
						if(m.getRicerca() != null && !elemento.contains(ApplicationUtil.upperCase(m.getRicerca()))) {
							return true;
						}
					} else {
						if(m.getRicerca() != null && elemento.contains(ApplicationUtil.upperCase(m.getRicerca()))) {
							return true;
						}
					}
				}
			}
		}
		return false;
	}

}