package it.callite.action;
import java.io.Serializable;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;

import it.callite.dao.MessaggioDao;
import it.callite.model.Utente;
import it.callite.model.messaggio.Destinatario;
import it.callite.model.messaggio.Messaggio;
import it.callite.utility.ApplicationUtil;
@SuppressWarnings("serial")
public class MessaggioBean implements Serializable {


	private Boolean lettura;
	private Integer messaggi;
	private List<Destinatario> messaggiList;
	private Messaggio messaggio;
	private Integer notifica;
	private Integer statistiche;
	private Utente utente;
	@Autowired
	private DownloadBean downloadBean;
	@Autowired
	private MessaggioDao messaggioDao;

	public MessaggioBean() {
	}
	public void apriMessaggi() {
		lettura = true;
		updateMessaggi();
	}
	public void chiudiMessaggi() {
		lettura = false;
		updateMessaggi();
	}
	public void chiudiRisposta() {
		messaggio = null;
	}
	public Boolean getLettura() {
		return lettura;
	}
	public Integer getMessaggi() {
		if(messaggi == null && utente != null) {
			messaggi = messaggioDao.trovaMessaggioInArrivoNuovi(utente.getId());
		}
		return messaggi;
	}
	public List<Destinatario> getMessaggiList() {
		return messaggiList;
	}
	public Messaggio getMessaggio() {
		return messaggio;
	}
	public Integer getStatistiche() {
		return statistiche;
	}
	public void imposta(Integer notifica, Integer statistiche, Utente utente) {
		this.lettura = false;
		this.notifica = notifica;
		this.utente = utente;
		this.statistiche = statistiche;
	}
	public void inviaRisposta() {
		if(messaggio != null) {
			messaggioDao.salvaMessaggioNuovoInvia(messaggio, utente);
		}
		messaggio = null;
	}
	public boolean isMessaggioPopup() {
		return (isNotifica3() && ApplicationUtil.gtZero(messaggi)) || (ApplicationUtil.isTrue(lettura) && isNotifica2() && ApplicationUtil.gtZero(messaggi));
	}
	public boolean isNotifica0() {
		return ApplicationUtil.eq(notifica, 0);
	}
	public boolean isNotifica1() {
		return ApplicationUtil.eq(notifica, 1);
	}
	public boolean isNotifica2() {
		return ApplicationUtil.eq(notifica, 2);
	}
	public boolean isNotifica3() {
		return ApplicationUtil.eq(notifica, 3);
	}

	public boolean isNotifica() {
		return ApplicationUtil.ge(notifica, 0);
	}

	public boolean isVisibile1() {
		return isNotifica1();
	}

	public boolean isVisibile2() {
		return (isNotifica2() || isNotifica3());
	}
	public void leggiMessaggi() {
		if(utente != null) {
			messaggioDao.leggiMessaggi(utente.getId(), null);
			lettura = false;
			updateMessaggi();
		}
	}
	public void leggiMessaggio() {
		Map<String, String> map = ApplicationUtil.getRequestParameterMap();
		if(map != null) {
			String mid = ApplicationUtil.trimToNull((String) map.get("id"));
			if(mid != null) {
				Long messID = Long.parseLong(mid);
				if(messID != null && utente != null) {
					messaggioDao.leggiMessaggi(utente.getId(), messID);
					updateMessaggi();
				}
			}
		}

	}
	public void rispondi() {
		Map<String, String> map = ApplicationUtil.getRequestParameterMap();
		if(map != null) {
			String mid = ApplicationUtil.trimToNull((String) map.get("id"));
			String singolo = ApplicationUtil.trimToNull((String) map.get("singolo"));
			if(mid != null) {
				Long messID = Long.parseLong(mid);
				if(messID != null){
					Boolean s = Boolean.valueOf(singolo);
					messaggio = messaggioDao.messaggioRispondiRapido(s, utente, messID);
					messaggioDao.leggiMessaggi(utente.getId(), messID);
					updateMessaggi();
				}
			}
		}

	}

	public void nuova() {
		messaggio = messaggioDao.messaggioNuovo(utente);
	}
	public void scarica() {
		Map<String, String> map = ApplicationUtil.getRequestParameterMap();
		if(map != null) {
			String mid = ApplicationUtil.trimToNull((String) map.get("id"));
			if(mid != null) {
				Long messID = Long.parseLong(mid);
				if(messID != null) {
					messaggioDao.scaricaAllegatiT(messID, downloadBean);
				}
			}
		}

	}
	public void setLettura(Boolean lettura) {
		this.lettura = lettura;
	}
	public void setMessaggio(Messaggio messaggio) {
		this.messaggio = messaggio;
	}
	public void setStatistiche(Integer statistiche) {
		this.statistiche = statistiche;
	}
	public void updateMessaggi() {
		if(utente != null) {
			messaggi = messaggioDao.trovaMessaggioInArrivoNuovi(utente.getId());
			if(isMessaggioPopup()) {
				messaggiList =  messaggioDao.trovaMessaggioInArrivoNuoviList(utente.getId(), null);
			} else {
				messaggiList = null;
			}
			messaggioDao.impostaConteggioPosta(messaggiList);
		}
	}

	public boolean isValid() {
		if(messaggio != null) {
			String m = ApplicationUtil.trimToNull(messaggio.getDestinatariString());
			if(m != null) {
				return true;
			}
		}
		return false;
	}
}