package it.callite.action;
import java.io.File;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

import javax.faces.model.SelectItem;

import org.richfaces.component.UITree;
import org.richfaces.component.state.TreeStateAdvisor;
import org.richfaces.event.NodeSelectedEvent;
import org.richfaces.model.TreeRowKey;
import org.springframework.beans.factory.annotation.Autowired;

import com.google.common.net.InternetDomainName;

import it.callite.comparator.MonitorSIComparator;
import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsMap;
import it.callite.dao.ImpostazioneDao;
import it.callite.message.Message;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.item.MonitorSI;

@SuppressWarnings("serial")
public class MonitorStorageBean implements TreeStateAdvisor, Serializable {

	@Autowired
	private ImpostazioneDao impostazioneDao;
	
	private String tipo = ConstantsMap.all;
	private String [] tipi = {ConstantsMap.all, ConstantsMap.day, ConstantsMap.week, ConstantsMap.month, ConstantsMap.year};

	private List<MonitorSI> files = new ArrayList<MonitorSI>();

	private MonitorSI selected;

	public List<SelectItem> getTipiSI() {
		List<SelectItem> sis = new ArrayList<SelectItem>();
		if (ApplicationUtil.notEmpty(tipi)) {
			for(String tipo : tipi) {
				sis.add(new SelectItem(tipo,  Message.addCatalogText(tipo)));
			}
		}
		return sis;
	}


	public boolean isExists() {
		String node = impostazioneDao.getUrlBase();
		InternetDomainName it = InternetDomainName.from(node);
		File f = new File(ConstantsGlobal.path_munin + it.topPrivateDomain() + "/"+it.toString()+"/");
		return f.exists();
	}



	@SuppressWarnings("unchecked")
	public Boolean adviseNodeOpened(UITree tree) {
		Object key = tree.getRowKey();
		TreeRowKey<MonitorSI> treeRowKey = (TreeRowKey<MonitorSI>) key;
		if (treeRowKey == null || treeRowKey.depth() <= 0) {
			return Boolean.TRUE;
		}
		return null;
	}

	public Boolean adviseNodeSelected(UITree tree) {
		MonitorSI file = (MonitorSI) tree.getRowData();
		File ls = new File(file.getPath());
		File [] fileArrays = ls.listFiles();
		if(ApplicationUtil.notEmpty(fileArrays)) {
			List<MonitorSI> list = getChildren(fileArrays);
			file.setNodes(list);
		}
		return null;
	}

	public List<MonitorSI> getChildren(File [] fileArrays) {
		List<MonitorSI> filesSI = new ArrayList<MonitorSI>();
		if(ApplicationUtil.notEmpty(fileArrays)) {
			List <File> fileList = new ArrayList<File>(Arrays.asList(fileArrays));
			Iterator<File> it = fileList.iterator();
			while (it.hasNext()) {
				File file = (File) it.next();
				String type =  "-" +  tipo + "." + ConstantsMap.png;
				String end = isAll(tipo) ? ConstantsMap.png : type;
				String remove = isAll(tipo) ? "-" : type;
				if(file.isDirectory() || file.getName().endsWith(end)) {
					MonitorSI fs = new MonitorSI(file.getAbsolutePath(), remove);
					if(!filesSI.contains(fs)) {
						filesSI.add(fs);
					}
				}
			}
		}
		Collections.sort(filesSI, new MonitorSIComparator());
		return filesSI; 
	}

	public List<MonitorSI> getRoots() {
		if(ApplicationUtil.empty(files)) {
			String node = impostazioneDao.getUrlBase();
			InternetDomainName it = InternetDomainName.from(node);
			File ls = new File(ConstantsGlobal.path_munin + it.topPrivateDomain() + "/"+it.toString()+"/");
			if(ls.exists() && ls.isDirectory()) {
				List<MonitorSI> list = getChildren(ls.listFiles());
				files.addAll(list);
			}

		}
		return files;
	}

	public void onSelect(final NodeSelectedEvent event) {
		//		HtmlTree theTree = (HtmlTree) event.getComponent();
		//		if (null == theTree) {
		//			return;
		//		}
		//		MonitorSI file = (MonitorSI) theTree.getRowData();
	}

	public void reset() {
		files.clear();
	}


	public String getTipo() {
		return tipo;
	}


	public void setTipo(String tipo) {
		this.tipo = tipo;
	}


	public MonitorSI getSelected() {
		return selected;
	}


	public void setSelected(MonitorSI selected) {
		this.selected = selected;
	}

	public List<MonitorSI> getMonitorList() {
		List<MonitorSI> sis = new ArrayList<MonitorSI>();
		if (selected != null && ApplicationUtil.notEmpty(tipi)) {
			String path = ApplicationUtil.normalizeNoEndSeparator(ApplicationUtil.getPathN(selected.getPath()));
			if(selected.isType()) {
				if(isAll(tipo)) {
					for(String tipo : tipi) {
						addFile(sis, selected.getNome(), path, tipo);
					}
				} else {
					addFile(sis, selected.getNome(), path, tipo);
				}
			} else {
				File fl = new File("/" +path+ "/" + selected.getNome());
				if(fl.exists()) {
					File [] fileList = fl.listFiles();
					if(ApplicationUtil.notEmpty(fileList)) {
						String type =  "-" +  tipo + "." + ConstantsMap.png;
						String end = isAll(tipo) ? ConstantsMap.png : type;
						String remove = isAll(tipo) ? "-" : type;
						for (File file : fileList) {
							if(file.getName().endsWith(end)) {
								sis.add(new MonitorSI(file.getAbsolutePath(), remove));
							}
						}
					}
				}
			}

		}
		return sis;
	}
	
	private void addFile(List<MonitorSI> sis, String nome, String path, String tipo) {
		File fl = new File("/" + path + "/" + nome + "-" +  tipo + "." + ConstantsMap.png);
		if(fl.exists()) {
			String type =  "-" +  tipo + "." + ConstantsMap.png;
			String remove = isAll(tipo) ? "-" : type;
			sis.add(new MonitorSI(fl.getAbsolutePath(), remove));
		}
	}
	
	private boolean isAll(String tipo) {
		return tipo == null || tipo.equals(ConstantsMap.all);
	}
}