package it.callite.autodialer.coda;


import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import it.callite.model.AutoContatto;
import it.callite.model.Autodialer;
import it.callite.model.Campagna;
import it.callite.model.Contatto;
import it.callite.utility.ApplicationUtil;


public class ChiamateInCorso {

	private Map<Long, AutoContatto> data;
	private int capacity;

	public ChiamateInCorso() {
		data = new HashMap<Long, AutoContatto>();
		capacity = 0;
	}

	public synchronized void clear() {
		data.clear();
		capacity = 0;
	}

	public synchronized boolean contains(Long id){
		return data.containsKey(id);
	}

	public synchronized AutoContatto get(Long id){
		return data.get(id);
	}

	public synchronized boolean dialStatus(Long id){
		AutoContatto auto = data.get(id);
		if(auto != null) {
			return auto.isDial();
		}
		return false;
	}
	
	public synchronized boolean hangupStatus(Long id){
		AutoContatto auto = data.get(id);
		if(auto != null) {
			return auto.isHangup();
		}
		return false;
	}
	
	public synchronized AutoContatto remove(Long id) {
		return data.remove(id);
	}

	public synchronized void put(AutoContatto auto) {
		if(auto!=null) {
			data.put(auto.getContatto().getId(), auto);
		}
	}
	public synchronized List<AutoContatto> get(Campagna campagna) {
		List<AutoContatto> lsch = new ArrayList<AutoContatto>();
		if(ApplicationUtil.notEmpty(data)) {
			Set<Long> lngList = data.keySet();
			Iterator<Long> lnt = lngList.iterator();
			while (lnt.hasNext()) {
				AutoContatto tmp = data.get(lnt.next());
				if(tmp!=null && tmp.equalsCampagna(campagna)) {
					lsch.add(tmp);
				}
			}
		}
		return lsch;
	}

	public synchronized boolean isEmpty(Campagna campagna) {
		if(ApplicationUtil.notEmpty(data)) {
			Set<Long> lngList = data.keySet();
			Iterator<Long> lnt = lngList.iterator();
			while (lnt.hasNext()) {
				AutoContatto tmp = data.get(lnt.next());
				if(tmp!=null && tmp.equalsCampagna(campagna)) {
					return false;
				}
			}
		}
		return true;
	}

	public synchronized void remove(Campagna campagna) {
		if(ApplicationUtil.notEmpty(data)) {
			Set<Long> lngList = data.keySet();
			Set<Long> removeList = new HashSet<Long>();
			Iterator<Long> lnt = lngList.iterator();
			while (lnt.hasNext()) {
				AutoContatto tmp = data.get(lnt.next());
				if(tmp!=null && tmp.equalsCampagna(campagna)) {
					removeList.add(tmp.getContatto().getId());
				}
			}
			if(ApplicationUtil.notEmpty(removeList)) {
				data.keySet().removeAll(removeList);
			}
		}
	}

	public synchronized void removeContatti(List<Contatto> contatti) {
		if(ApplicationUtil.notEmpty(data) && ApplicationUtil.notEmpty(contatti)) {
			Set<Long> lngList = data.keySet();
			Set<Long> removeList = new HashSet<Long>();
			Iterator<Contatto> lnt = contatti.iterator();
			while (lnt.hasNext()) {
				Contatto tmp = lnt.next();
				if(lngList.contains(tmp.getId())) {
					removeList.add(tmp.getId());
				}
			}
			if(ApplicationUtil.notEmpty(removeList)) {
				data.keySet().removeAll(removeList);
			}
		}
	}

	public synchronized void remove(List<AutoContatto> contatti) {
		if(ApplicationUtil.notEmpty(data) && ApplicationUtil.notEmpty(contatti)) {
			Set<Long> lngList = data.keySet();
			Set<Long> removeList = new HashSet<Long>();
			Iterator<AutoContatto> lnt = contatti.iterator();
			while (lnt.hasNext()) {
				AutoContatto tmp = lnt.next();
				if(lngList.contains(tmp.getContatto().getId())) {
					removeList.add(tmp.getContatto().getId());
				}
			}
			if(ApplicationUtil.notEmpty(removeList)) {
				data.keySet().removeAll(removeList);
			}
		}
	}

	public synchronized AutoContatto getUniqueId(String uniqueId){
		if(ApplicationUtil.notEmpty(data)) {
			Iterator<Long> lnt = data.keySet().iterator();
			while (lnt.hasNext()) {
				AutoContatto auto = data.get(lnt.next());
				if(auto!= null && (auto.equalsUniqueId1(uniqueId) || auto.equalsUniqueId2(uniqueId))){
					return auto;
				}		
			}
		}
		return null;
	}

	public synchronized List<AutoContatto> timeout(Date date){
		List<AutoContatto> lsch = new ArrayList<AutoContatto>();
		if(ApplicationUtil.notEmpty(data)) {
			Set<Long> lngList = data.keySet();
			Iterator<Long> lnt = lngList.iterator();
			while (lnt.hasNext()) {
				AutoContatto tmp = data.get(lnt.next());
				if (tmp!= null && (tmp.getData_chiamata() != null && (tmp.getData_rifiuto() == null && tmp.getData_aggancio() == null && tmp.getData_risposta() == null))) {
					long tempo_timeout = date.getTime() - tmp.getData_chiamata().getTime();
					Autodialer auto = tmp.getAutodialer();
					if(auto != null) {
						long tempo = ApplicationUtil.isTrue(tmp.getContatto().getInbound()) ? auto.getAbbattimento() : auto.getTimeoutAmi();
						if (tempo_timeout > (tempo * auto.getTempoAmi())) {
							lsch.add(tmp);
						}
					}
				}
			}
		}
		return lsch;
	}

	public synchronized void setCapacity(int cap) {
		capacity = cap;
	}

	public synchronized int getSize() {
		return data.size();
	}

	public synchronized int getCapacity() {
		return capacity;
	}

	public synchronized boolean isEmpty() {
		return data.size() == 0;
	}

	public synchronized boolean isFull() {
		return data.size() >= capacity;
	}

	public synchronized int getFree() {
		return capacity -  data.size();
	}

	public synchronized List<AutoContatto> list(){
		return new ArrayList<AutoContatto>(data.values());
	}


}