package it.callite.autodialer.controllo;

import java.util.Date;

import it.callite.autodialer.coda.ChiamateAttive;
import it.callite.autodialer.coda.ChiamateInAttesa;
import it.callite.autodialer.coda.ChiamateInCorso;
import it.callite.autodialer.coda.ChiamateSchedulate;
import it.callite.model.Autodialer;
import it.callite.model.Campagna;
import it.callite.model.report.AutodialerContatti;
import it.callite.model.report.AutodialerReport;
import it.callite.model.report.AutodialerStatistiche;
import it.callite.utility.ApplicationUtil;



public class CodaManager {

	private AutodialerStatistiche autodialerStatistiche;
	private AutodialerContatti autodialerContatti;
	private Campagna campagna;
	private ChiamateAttive chiamateAttive;
	private ChiamateInAttesa chiamateInAttesa;
	private ChiamateInCorso chiamateInCorso;
	private ChiamateSchedulate chiamateSchedulate;
	private int correzione;
	private boolean avviato;
	private boolean caricato;
	private boolean esaurito;
	private boolean ricarica;
	private boolean sospeso;
	private boolean started1;
	private boolean started2;
	private boolean finito;
	private boolean stato;
	private String criterio;
	private Date data;

	public CodaManager(Campagna campagna) {
		this.campagna = campagna;
		this.chiamateAttive = new ChiamateAttive();
		this.chiamateInAttesa = new ChiamateInAttesa();
		this.chiamateInCorso = new ChiamateInCorso();
		this.chiamateSchedulate = new ChiamateSchedulate();
		this.autodialerStatistiche = new AutodialerStatistiche();
		this.autodialerContatti = new AutodialerContatti();
		this.sospeso = true;
		this.avviato = true;
		this.caricato = false;
		this.ricarica = false;
		this.esaurito = false;
		this.started1 = false;
		this.data = null;
		this.started2 = false;
		this.finito = false;
		this.stato = false;
		this.criterio = null;
		this.correzione = 0;
	}

	public AutodialerStatistiche getAutodialerStatistiche() {
		return autodialerStatistiche;
	}

	public AutodialerContatti getAutodialerContatti() {
		return autodialerContatti;
	}

	public Campagna getCampagna() {
		return campagna;
	}

	public ChiamateAttive getChiamateAttive() {
		return chiamateAttive;
	}

	public ChiamateInAttesa getChiamateInAttesa() {
		return chiamateInAttesa;
	}

	public ChiamateInCorso getChiamateInCorso() {
		return chiamateInCorso;
	}

	public ChiamateSchedulate getChiamateSchedulate() {
		return chiamateSchedulate;
	}

	public boolean isAvviabile() {
		return !isStato();
	}

	public boolean isAvviato() {
		return avviato;
	}

	public boolean isCaricato() {
		return caricato;
	}

	public boolean isEsaurito() {
		return esaurito;
	}

	public boolean isFermabile() {
		return isStato();
	}

	public boolean isRicarica() {
		return ricarica;
	}

	public boolean isSospeso() {
		return sospeso;
	}

	public boolean isStarted() {
		return started1 && started2;
	}

	public boolean isFinito() {
		return finito;
	}

	public void setFinito(boolean finito) {
		this.finito = finito;
	}

	public boolean isStato() {
		return stato;
	}


	public void criterio(String tipo, String proprieta, Integer limite, String query_proprieta, String query_valore, Boolean nuovo, Boolean producer) {
		criterio = ApplicationUtil.criterio(tipo, proprieta, limite, query_proprieta, query_valore, nuovo, producer);
	}

	public void clear() {
		this.chiamateAttive.clear();
		this.chiamateInAttesa.clear();
		this.chiamateInCorso.clear();
		this.chiamateSchedulate.clear();
	}

	public void reset() {
		clear();
		this.autodialerStatistiche.resetAll();
		this.autodialerContatti.resetAll();
		this.ricarica = true;
		this.avviato = true;
		this.correzione = 0;
	}

	public void setAvviato(boolean avviato) {
		this.avviato = avviato;
	}

	public void setCampagna(Campagna campagna) {
		this.campagna = campagna;
	}

	public void setCaricato(boolean caricato) {
		this.caricato = caricato;
	}

	public void setEsaurito(boolean esaurito) {
		this.esaurito = esaurito;
	}

	public void setRicarica(boolean ricarica) {
		this.ricarica = ricarica;
	}

	public void setSospeso(boolean sospeso) {
		this.sospeso = sospeso;
	}

	public void setStarted1(boolean started1) {
		this.started1 = started1;
	}

	public void setStarted2(boolean started2) {
		this.started2 = started2;
	}

	public void setStato(boolean stato) {
		this.stato = stato;
	}

	public String getCriterio() {
		return ApplicationUtil.trimToEmpty(criterio);
	}

	public void setCriterio(String criterio) {
		this.criterio = criterio;
	}

	public Date getData() {
		return data;
	}

	public void setData(Date data) {
		this.data = data;
	}

	public int getCorrezione() {
		return correzione;
	}

	public void setCorrezione(int correzione) {
		this.correzione = correzione;
	}
	public int incCorrezione() {
		return correzione++;
	}
	public int decCorrezione() {
		return correzione--;
	}

	public void updateReport(AutodialerReport ad, CodaManager codaManager) {
		Campagna campagna = codaManager.getCampagna();
		if(campagna != null) {
			Autodialer auto = campagna.getAutodialer();
			if(auto != null) {
				Integer a_fallite = auto.getA_fallite();
				Integer a_saltate = auto.getA_saltate();
				Integer a_rimosse = auto.getA_rimosse();
				ad.setA_fallite(ApplicationUtil.gtZero(a_fallite) ? autodialerStatistiche.getA_fallite() > a_fallite.intValue() : false);
				ad.setA_saltate(ApplicationUtil.gtZero(a_saltate) ? autodialerStatistiche.getA_saltate() > a_saltate.intValue() : false);
				ad.setA_rimosse(ApplicationUtil.gtZero(a_rimosse) ? autodialerStatistiche.getA_rimosse() > a_rimosse.intValue() : false);
			}
		}
	}
}
