package it.callite.autodialer.controllo;


import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import it.callite.constants.ConstantsGlobal;
import it.callite.model.Campagna;
import it.callite.model.Contatto;
import it.callite.model.Utente;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.InfoUtente;


public class CodaTeam {

	private Map<Campagna, List<InfoUtente>> data;

	public CodaTeam() {
		data = new HashMap<Campagna, List<InfoUtente>>();
	}

	public synchronized void clear() {
		data.clear();
	}

	public synchronized List<InfoUtente> get(Campagna campagna){
		List<InfoUtente> users =  data.get(campagna);
		if(users == null) {
			users = new ArrayList<InfoUtente>();
		}
		return users;
	}

	public void addUser(Campagna campagna, InfoUtente utente, Integer priorita){
		List<InfoUtente> users =  data.get(campagna);
		if(users == null) {
			users = new ArrayList<InfoUtente>();
		}
		if(!users.contains(utente)) {
			if(ApplicationUtil.gt(priorita, 0)) {
				int m = users.size() > priorita.intValue() ? (users.size()-priorita.intValue()) : 0;
				users.add(m, utente);
			} else {
				users.add(utente);
			}
			
		}
		data.put(campagna, users);
	}


	public synchronized void reset(Campagna campagna){
		data.put(campagna, new ArrayList<InfoUtente>());
	}

	public synchronized int getSizeAll(){
		int size = 0;
		if(ApplicationUtil.notEmpty(data)) {
			Set<Campagna> lngList = data.keySet();
			Iterator<Campagna> lnt = lngList.iterator();
			while (lnt.hasNext()) {
				Campagna campagna = lnt.next();
				List<InfoUtente> users =  data.get(campagna);
				if(ApplicationUtil.notEmpty(users)){
					size += users.size();
				}
			}
		}
		return size;
	}

	public synchronized boolean update(Campagna campagna, Utente user) {
		List<InfoUtente> users =  data.get(campagna);
		if(ApplicationUtil.notEmpty(users)){
			Iterator<InfoUtente> iuIt = users.iterator();
			while (iuIt.hasNext()) {
				InfoUtente iu = iuIt.next();
				if(iu!= null && iu.getOperatore().equals(user)) {
					long time = System.currentTimeMillis();
					iu.setTempoApertura(time);
					return true;
				}
			}
		}
		return false;
	}

	public synchronized boolean updateReady(Contatto contatto, Campagna campagna, Utente user) {
		List<InfoUtente> users =  data.get(campagna);
		if(ApplicationUtil.notEmpty(users)){
			Iterator<InfoUtente> iuIt = users.iterator();
			while (iuIt.hasNext()) {
				InfoUtente iu = iuIt.next();
				if(iu!= null && iu.getOperatore().equals(user) && contatto == null && !iu.isReady()) {
					iu.setReady(true);
					return true;
				} 
			}
		}
		return false;
	}
	
	public synchronized boolean ready(Contatto contatto, Campagna campagna, Utente user) {
		List<InfoUtente> users =  data.get(campagna);
		if(ApplicationUtil.notEmpty(users)){
			Iterator<InfoUtente> iuIt = users.iterator();
			while (iuIt.hasNext()) {
				InfoUtente iu = iuIt.next();
				if(iu!= null && iu.getOperatore().equals(user)) {
					return iu.isReady();
				} 
			}
		}
		return false;
	}

	public synchronized void remove(Campagna campagna){
		data.remove(campagna);
	}

	public synchronized void timeout(){
		if(ApplicationUtil.notEmpty(data)) {
			Set<Campagna> lngList = data.keySet();
			long time = System.currentTimeMillis();
			Iterator<Campagna> lnt = lngList.iterator();
			while (lnt.hasNext()) {
				Campagna campagna = lnt.next();
				List<InfoUtente> removeList =  new ArrayList<InfoUtente>();
				List<InfoUtente> users =  data.get(campagna);
				if(ApplicationUtil.notEmpty(users)) {
					Iterator<InfoUtente> utIter = users.iterator();
					while (utIter.hasNext()) {
						InfoUtente info = utIter.next();
						if (info!= null && info.getTempoApertura() != null && ((time - info.getTempoApertura().longValue()) > ConstantsGlobal.timeoutUser)) {
							removeList.add(info);
						} 
					}
				}
				if(ApplicationUtil.notEmpty(removeList)) {
					users.removeAll(removeList);
				}
			}
		}
	}

	public synchronized void rimuoviUtente(Campagna campagna, InfoUtente utente){
		List<InfoUtente> users =  data.get(campagna);
		if(ApplicationUtil.notEmpty(users) && users.contains(utente)) {
			users.remove(utente);
			//data.put(campagna, users);
		}
	}

	public synchronized void rimuoviUtenteT(InfoUtente utente){
		if(ApplicationUtil.notEmpty(data)) {
			Set<Campagna> lngList = data.keySet();
			Iterator<Campagna> lnt = lngList.iterator();
			while (lnt.hasNext()) {
				Campagna campagna = lnt.next();
				List<InfoUtente> removeList =  new ArrayList<InfoUtente>();
				List<InfoUtente> users =  data.get(campagna);
				if(ApplicationUtil.notEmpty(users) && users.contains(utente)) {
					Iterator<InfoUtente> utIter = users.iterator();
					while (utIter.hasNext()) {
						InfoUtente infoUtente = utIter.next();
						if(!infoUtente.isDisco() && infoUtente.equals(utente)) {
							removeList.add(utente);
						}
					}
				}
				if(ApplicationUtil.notEmpty(removeList)) {
					users.removeAll(removeList);
				}
			}
		}
	}
	
	
	public synchronized void rimuoviUtenteAll(InfoUtente utente){
		if(ApplicationUtil.notEmpty(data)) {
			Set<Campagna> lngList = data.keySet();
			Iterator<Campagna> lnt = lngList.iterator();
			while (lnt.hasNext()) {
				Campagna campagna = lnt.next();
				List<InfoUtente> users =  data.get(campagna);
				if(ApplicationUtil.notEmpty(users) && users.contains(utente)) {
					users.remove(utente);
					//data.put(campagna, users);
				}
			}
		}
	}
	
	public synchronized List<Utente> getUtenti(Campagna campagna){
		List<Utente> utenti = new ArrayList<Utente>();
		if(campagna != null && ApplicationUtil.notEmpty(data)) {
			List<InfoUtente> users = get(campagna);
			Iterator<InfoUtente> lnt = users.iterator();
			while (lnt.hasNext()) {
				InfoUtente user = lnt.next();
				if(!user.isDisco()) {
					utenti.add(user.getOperatore());
				}
			}
		}
		return utenti;
	}


	public synchronized int getSize() {
		return data.size();
	}

	public synchronized boolean isEmpty() {
		return data.size() == 0;
	}
}