package it.callite.autodialer.pubblico;

import java.util.Iterator;
import java.util.List;
import java.util.concurrent.Semaphore;

import it.callite.autodialer.controllo.CodaManager;
import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsMap;
import it.callite.dao.AutodialerDao;
import it.callite.model.AutoContatto;
import it.callite.model.Campagna;
import it.callite.model.Contatto;
import it.callite.utility.ApplicationUtil;


public class PubblicoProduttoreContattoRichiamo extends Thread {


	private Semaphore mutex = new Semaphore(1);
	private CodaManager codaManager;
	private AutodialerDao autodialerDao;


	public PubblicoProduttoreContattoRichiamo(CodaManager codaManager, AutodialerDao autodialerDao) {
		this.codaManager = codaManager;
		this.autodialerDao = autodialerDao;
	}

	public void run() {

		while(true) {
			try {
				if(ConstantsGlobal.semaforo) {
					mutex.acquire();
				}
				boolean test = !codaManager.isSospeso() && codaManager.isAvviato() && !codaManager.getChiamateAttive().isFull();
				if(test) {
					Campagna campagna = codaManager.getCampagna();
					if(campagna != null) {
						if(ApplicationUtil.isTrue(campagna.getAutodialer().getProducer_modalita())) {
							Integer v =  campagna.getAutodialer().getProducer_profondita();
							if(ApplicationUtil.gtZero(v)) {
								int free = codaManager.getChiamateAttive().getFree();
								int a = (free > v) ? v : free;
								if(ApplicationUtil.gtZero(a)) {
									List<Contatto> contatti	= autodialerDao.getContatto(campagna, a, null);
									if(ApplicationUtil.notEmpty(contatti)) {
										codaManager.setFinito(false);
										Iterator<Contatto> cIt = contatti.iterator();
										while (cIt.hasNext()) {
											Contatto contatto = (Contatto) cIt.next();
											addContatto(contatto, campagna);
										}
									} else {
										codaManager.setFinito(true);
										codaManager.criterio(campagna.getAutodialer().getTipologia(), campagna.getProprieta(), campagna.getLimite(), campagna.getQuery_proprieta(), campagna.getQuery_valore(), campagna.getAutodialer().getNuovo(), campagna.getAutodialer().getProducer_modalita());
									}						
								}
							} else {
								Contatto contatto = autodialerDao.getContatto(campagna, null);
								if(contatto!=null) {
									codaManager.setFinito(false);
									addContatto(contatto, campagna);
								} else {
									codaManager.setFinito(true);
									codaManager.criterio(campagna.getAutodialer().getTipologia(), campagna.getProprieta(), campagna.getLimite(), campagna.getQuery_proprieta(), campagna.getQuery_valore(), campagna.getAutodialer().getNuovo(), campagna.getAutodialer().getProducer_modalita());
								}
							}
						} else {
							codaManager.setFinito(true);
							codaManager.criterio(campagna.getAutodialer().getTipologia(), campagna.getProprieta(), campagna.getLimite(), campagna.getQuery_proprieta(), campagna.getQuery_valore(), campagna.getAutodialer().getNuovo(), campagna.getAutodialer().getProducer_modalita());
						}
					}
				} 
				codaManager.setStarted1(true);
				sleep(autodialerDao.sleepProduttore());

			} catch (Exception e) {
			} finally {
				if(ConstantsGlobal.semaforo) {
					mutex.release();
				}
			}
		}
	}


	private void addContatto(Contatto contatto, Campagna campagna) {
		AutoContatto auto = new AutoContatto(contatto, ConstantsMap.CHIAMATA_ATTIVA, contatto.getTelefono_chiamata(), false);
		auto.impostaUserfield();
		contatto.setTelefono(auto.getTelefono());
		contatto.setAutodialer(true);
		contatto.setInbound(false);
		auto.setAutodialer(campagna.getAutodialer());
		codaManager.getChiamateAttive().put(auto);
	}

}


