package it.callite.autodialer.pubblico;


import java.util.List;
import java.util.concurrent.Semaphore;

import it.callite.autodialer.controllo.CodaManager;
import it.callite.autodialer.controllo.CodaStatic;
import it.callite.chiamate.Chiamate;
import it.callite.constants.ConstantsGlobal;
import it.callite.dao.AutodialerDao;
import it.callite.model.Autodialer;
import it.callite.model.Campagna;
import it.callite.model.sip.SipBuddies;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.InfoUtente;


public class PubblicoSospendi extends Thread {


	private Semaphore mutex = new Semaphore(1);
	private CodaManager codaManager;
	private AutodialerDao autodialerDao;
	private Integer numCanali;

	public PubblicoSospendi(CodaManager codaManager, AutodialerDao autodialerDao, Integer numCanali) {
		this.codaManager = codaManager;
		this.autodialerDao = autodialerDao;
		this.numCanali = numCanali;

	}

	public void run() {
		while(true) {
			try {
				if(ConstantsGlobal.semaforo) {
					mutex.acquire();
				}
				Campagna campagna = codaManager.getCampagna();
				if(campagna != null) {
					if(codaManager.isRicarica()) {
						codaManager.setRicarica(false);
						numCanali = autodialerDao.getNumCanali();
						Campagna c = autodialerDao.getCampagna(campagna);
						codaManager.setCampagna(c);
						Chiamate.getCodaProcessi().remove(campagna);
						if(c != null) {
							Chiamate.getCodaProcessi().put(c, codaManager);
						} 
						campagna = c;
					}
					if(campagna != null) {
						List<InfoUtente> users = CodaStatic.getCodaTeam().get(campagna);
						Autodialer auto = campagna.getAutodialer();
						if(!codaManager.isSospeso() && codaManager.isAvviato()) {
							CodaStatic.getChiamateInCorso().setCapacity(numCanali);
							codaCampagna(auto, users, numCanali);
						}
						if(codaManager.isStarted() && codaManager.getChiamateAttive().isEmpty() && codaManager.getChiamateInCorso().isEmpty()) {
							codaManager.setEsaurito(true);
						} else {
							codaManager.setEsaurito(false);
						}

						if (ApplicationUtil.notEmpty(users) && (auto!= null && auto.getStatoAttiva()) && (campagna!= null && campagna.getStatoAttiva())){
							codaManager.setSospeso(false);
						} else {
							codaManager.setSospeso(true);
						}
					} else {
						codaManager.setSospeso(true);
					}
				}
				sleep(autodialerDao.sleepSospendi());
			} catch (Exception e) {
			} finally {
				if(ConstantsGlobal.semaforo) {
					mutex.release();
				}
			}
		}
	}

	private void codaCampagna(Autodialer auto, List<InfoUtente> users, int numCanali) {
		if(auto!=null){
			SipBuddies sp = auto.getTrunk();
			Integer callSize = 0;
			if(sp != null) {
				callSize = sp.getMax_contacts();
			}
			double userSize = users.size();
			double aggressivita = auto.getAggressivita();
			double attesa = 1;
			if(auto.getCalcola_attesa()) {
				double media = codaManager.getAutodialerStatistiche().getMediaAttesa()/(ConstantsGlobal.media * ConstantsGlobal.millis);
				attesa += media;
			}
			Double agg = Double.valueOf(userSize * attesa * ((aggressivita + 100)/100));
			int valore = agg.intValue();
			int capacity = ((valore > callSize) ? callSize : valore) + codaManager.getCorrezione();
			codaManager.getChiamateAttive().setCapacity(capacity);
			codaManager.getChiamateInCorso().setCapacity((capacity > numCanali) ? numCanali : capacity);
		}
	}

}

