package it.callite.chiamate;



import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import it.callite.model.Campagna;
import it.callite.model.Contatto;
import it.callite.model.EsitoArchiviazione;
import it.callite.model.report.ElencoCampagnaReport;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.ElencoCampagna;


public class CodaCampagne {


	private List<ElencoCampagna> data;

	public CodaCampagne() {
		data = new ArrayList<ElencoCampagna>();
	}

	public synchronized void clear() {
		data.clear();
	}
	
	public synchronized void add(Contatto contatto){
		if(contatto!= null) {
			String proprieta = contatto.getCampagna().getProprieta();
			Integer limite = contatto.getCampagna().getLimite();
			if(ApplicationUtil.gtZero(limite) && ApplicationUtil.isNotBlank(proprieta)) {
				EsitoArchiviazione ea = contatto.getEsitoArchiviazione();
				if(ApplicationUtil.isTrue(ea.getUtile()) && ApplicationUtil.isTrue(ea.getPositivo())) {
					String elenco = ApplicationUtil.trimToNull(ApplicationUtil.metodo(contatto.getAnagrafica(), proprieta));
					if(elenco != null) {
						ElencoCampagna e = new ElencoCampagna(elenco, proprieta, contatto);
						int i = data.indexOf(e);
						if(i > -1) {
							ElencoCampagna a = data.get(i);
							a.incConteggio();
						} else {
							data.add(e);
						}
					}
				}
			}
		}
	}

	public synchronized List<String> get(Campagna campagna) {	
		String proprieta = campagna.getProprieta();
		Integer limite = campagna.getLimite();
		List<String> ls = new ArrayList<String>();
		if(ApplicationUtil.gtZero(limite) && ApplicationUtil.isNotBlank(proprieta)) {
			if(ApplicationUtil.notEmpty(data)) {
				Iterator<ElencoCampagna> lnt = data.iterator();
				while (lnt.hasNext()) {
					ElencoCampagna e = lnt.next();
					if(ApplicationUtil.isNotBlank(e.getProprieta()) && proprieta.equals(e.getProprieta()) ) {
						if(campagna != null && campagna.getId().equals(e.getCampagna().getId())) {
							if(ApplicationUtil.ge(e.getConteggio(), limite)) {
								String elenco = ApplicationUtil.trimToNull(e.getElenco());
								if(elenco!= null && !ls.contains(elenco)) {
									ls.add(elenco);
								}
							}
						}
					}
				}		
			}
		}
		return ls;
	}
	
	public synchronized List<ElencoCampagnaReport> get(List<Long> campagne) {	
		List<ElencoCampagnaReport> lsch = new ArrayList<ElencoCampagnaReport>();
		if(ApplicationUtil.notEmpty(data)) {
			Iterator<ElencoCampagna> lnt = data.iterator();
			while (lnt.hasNext()) {
				ElencoCampagna e = lnt.next();
				if(campagne != null && campagne.contains(e.getCampagna().getId())) {
					ElencoCampagnaReport c = new ElencoCampagnaReport(e);
					if(!lsch.contains(c)) {
						lsch.add(c);
					} else {
						int i = lsch.indexOf(c);
						if(i > -1) {
							ElencoCampagnaReport a = lsch.get(i);
							a.setConteggio(a.getConteggio() + c.getConteggio());
						}
					}
				}
			}		
		}
		return lsch;
	}


	public synchronized void reset(List<Long> campagne) {
		if(ApplicationUtil.notEmpty(data)) {
			Set<ElencoCampagna> removeList = new HashSet<ElencoCampagna>();
			Iterator<ElencoCampagna> lnt = data.iterator();
			while (lnt.hasNext()) {
				ElencoCampagna e = lnt.next();
				if(campagne != null && campagne.contains(e.getCampagna().getId())) {
					removeList.add(e);
				}
			}
			if(ApplicationUtil.notEmpty(removeList)) {
				data.removeAll(removeList);
			}
		}
	}

	public synchronized List<ElencoCampagna> list(){
		return data;
	}

}