package it.callite.chiamate;



import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import it.callite.model.Campagna;
import it.callite.model.Utente;
import it.callite.model.report.MonitorOperatore;
import it.callite.utility.ApplicationUtil;


public class CodaOperatori {

	private Map<Utente, MonitorOperatore> data;

	public CodaOperatori() {
		data = new HashMap<Utente, MonitorOperatore>();
	}

	public synchronized void clear() {
		data.clear();
	}

	public synchronized void remove(Utente utente) {
		data.remove(utente);
	}

	public synchronized void removeList(Set<Utente> utenti) {
		data.keySet().removeAll(utenti);
	}

	public synchronized MonitorOperatore get(Utente utente) {
		return	data.get(utente);
	}

	public synchronized void put(Utente utente, MonitorOperatore ic) {
		data.put(utente, ic);
	}

	public void replace(Utente user) {
		MonitorOperatore mop = data.remove(user);
		if(mop != null) { 
			mop.setOperatore(user);
			data.put(user, mop);
		} 
	}

	public void visibile(Utente user, boolean visibile) {
		MonitorOperatore mop = data.get(user);
		if(mop != null) { 
			mop.setVisibile(visibile);
		} 
	}

	public synchronized Integer hasPrecedenza(Utente utente, Campagna campagna) {
		if(ApplicationUtil.isTrue(campagna.getAutodialer().getPrecedenza())) {
			MonitorOperatore mop = data.get(utente);
			if(mop != null) {
				return mop.getPrecedenza();
			}
		}
		return null;
	}

	public void endorse(Utente user) {
		MonitorOperatore mop = data.get(user);
		if(mop != null) { 
			mop.setEndorse(System.currentTimeMillis());
		} 
	}

	public void online(Utente user, boolean online) {
		MonitorOperatore mop = data.get(user);
		if(mop != null) { 
			mop.setOnline(online);
		} 
	}

	public synchronized int getSize() {
		return data.size();
	}

	public synchronized Set<Utente>  keySet() {
		return data.keySet();
	}


	public synchronized boolean contains(Utente utente){
		if (ApplicationUtil.notEmpty(data)) {
			return data.containsKey(utente);
		}
		return false;
	}

	public synchronized boolean hasOperatore(Utente utente){
		if (ApplicationUtil.notEmpty(data)) {
			MonitorOperatore u = get(utente);
			if(u != null && u.isLoggato()) {
				return true;
			} 
		}
		return false;
	}

	public synchronized List<MonitorOperatore> list(){
		return new ArrayList<MonitorOperatore>(data.values());
	}

	public synchronized Map<Utente, MonitorOperatore> getData() {
		return data;
	}


	public synchronized Utente getID(Long id) {
		if (ApplicationUtil.notEmpty(data)) {
			Set<Utente> ls = data.keySet();
			for(Utente op : ls) {
				if(ApplicationUtil.eq(op.getId(), id)) {
					return op;
				}
			} 
		}
		return null;
	}

	public synchronized List<String> getIP(boolean locale) {
		Set<String> IP = new HashSet<String>();
		List<MonitorOperatore> ls = new ArrayList<MonitorOperatore>(data.values());
		for(MonitorOperatore op : ls) {
			String ip = ApplicationUtil.trimToNull(op.getIp());
			if(ip != null && !IP.contains(ip)) {
				if(!locale || (locale && op.getOperatore().isLocale())) {
					IP.add(ip);
				}
			}
		} 
		return new ArrayList<String>(IP);
	}


	public synchronized List<MonitorOperatore> monitor(Campagna campagna, List<Long> campagne, boolean c) {	
		List<MonitorOperatore> lsch = new ArrayList<MonitorOperatore>();
		if(ApplicationUtil.notEmpty(data)) {
			Set<Utente> lngList = data.keySet();
			Iterator<Utente> lnt = lngList.iterator();
			while (lnt.hasNext()) {
				Utente utente = lnt.next();
				MonitorOperatore tmp = data.get(utente);
				boolean contatto = !c || (c && tmp.getContatto() != null);
				if(tmp!=null && contatto && tmp.getCampagna() != null && campagne.contains(tmp.getCampagna().getId())) {
					if(campagna == null || (campagna != null && tmp.getCampagna().equals(campagna))) {
						lsch.add(tmp);
					}
				}
			}
		}
		return lsch;
	}


}