package it.callite.controller;
import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.richfaces.json.JSONArray;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.servlet.ModelAndView;

import it.callite.constants.ConstantsFormat;
import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsMap;
import it.callite.dao.AnagraficaDao;
import it.callite.dao.ControllerDao;
import it.callite.dao.ImpostazioneDao;
import it.callite.log.ErrorLog;
import it.callite.model.Anagrafica;
import it.callite.model.CampoAggiuntivoAnagrafica;
import it.callite.model.anagrafica.AnagraficaXML;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.JSONArrayView;
import it.callite.utility.ModelImport;
@RestController
@RequestMapping(value = ConstantsGlobal.path_web_importa)
public class AnagraficaController {
	@Autowired
	private ControllerDao controllerDao;
	@Autowired
	public ImpostazioneDao impostazioneDao;
	@Autowired
	public AnagraficaDao anagraficaDao;

	//https://BASE_PATH/importa/send/campagna/lista/password/xml
	@RequestMapping(value= "/send/{campagna}/{lista}/{password}/{ext}", method = RequestMethod.POST) 
	public ModelAndView sendAnagrafica(HttpServletRequest request, HttpServletResponse response, @PathVariable final Long campagna, @PathVariable final Long lista, @PathVariable final String password, @PathVariable final String ext, @RequestBody String dto){
		Map<String, List<String>> messageS = new HashMap<String, List<String>>();
		String ex = ApplicationUtil.trimToNull(ext);
		if(ex != null && impostazioneDao.comparePassword(password)) {
			if(ex.equalsIgnoreCase(ConstantsMap.xml) || ex.equalsIgnoreCase(ConstantsMap.json)) {
				try {
					InputStream stream = new ByteArrayInputStream(dto.getBytes(ConstantsGlobal.CHARSET));
					List<AnagraficaXML> anaList = new ArrayList<AnagraficaXML>();
					if(ex.equalsIgnoreCase(ConstantsMap.xml)) {
						ApplicationUtil.xmlDecoder(anaList, stream);
					} 
					if(ex.equalsIgnoreCase(ConstantsMap.json)) {
						ApplicationUtil.jsonDecoder(anaList, stream);
					}
					stream.close();
					for(AnagraficaXML anagrafica : anaList) {
						List<String> m = controllerDao.inviaContatto(anagrafica, campagna, lista, null, null);
						messageS.put(ApplicationUtil.id(anagrafica.getId(), anagrafica.getTipo(), anagrafica.getNome_completo(), anagrafica.getRagione_sociale(), anagrafica.getCognome(), anagrafica.getNome(), null), m);
					}
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				}
			}
		}
		ModelAndView mav = new ModelAndView();
		JSONArray jsarray = new JSONArray();
		jsarray.put(messageS);
		mav.setView(new JSONArrayView());
		mav.addObject("array", jsarray);
		return mav;
	}


	//https://BASE_PATH/importa/send/campagna/lista/password
	@RequestMapping(value= "/send/{campagna}/{lista}/{password}") 
	public ModelAndView sendContatto(HttpServletRequest request, HttpServletResponse response, @PathVariable final Long campagna, @PathVariable final Long lista, @PathVariable final String password){
		Map<String, List<String>> messageS = new HashMap<String, List<String>>();
		if(impostazioneDao.comparePassword(password)) {
			try {
				Anagrafica a = new Anagrafica();
				List<String> parameters = Collections.list(request.getParameterNames());
				if(ApplicationUtil.notEmpty(parameters)) {
					List<CampoAggiuntivoAnagrafica> dati = anagraficaDao.getAllCampiAggiuntivi();
					Iterator<String> it = parameters.iterator();
					while (it.hasNext()) {
						String e = ApplicationUtil.trimToNull(it.next());
						if(e != null) {
							String obj = request.getParameter(e);
							if(obj != null) {
								ModelImport.importAnagrafica(a, e, obj, dati);
							}
						}
					}
				}
				a.prepareDati();
				AnagraficaXML anagrafica = new AnagraficaXML(a);
				List<String> m = controllerDao.inviaContatto(anagrafica, campagna, lista, null, null);
				messageS.put(ApplicationUtil.id(anagrafica.getId(), anagrafica.getTipo(), anagrafica.getNome_completo(), anagrafica.getRagione_sociale(), anagrafica.getCognome(), anagrafica.getNome(), null), m);
			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		}
		ModelAndView mav = new ModelAndView();
		JSONArray jsarray = new JSONArray();
		jsarray.put(messageS);
		mav.setView(new JSONArrayView());
		mav.addObject("array", jsarray);
		return mav;
	}

	//https://BASE_PATH/importa/send/campagna/lista/password/tipo/telefono
	@RequestMapping(value= "/send/{campagna}/{lista}/{password}/{tipo}/{telefono}") 
	public ModelAndView sendTelefono(HttpServletRequest request, HttpServletResponse response, @PathVariable final Long campagna, @PathVariable final Long lista, @PathVariable final String password, @PathVariable final String tipo, @PathVariable final String telefono){
		Map<String, List<String>> messageS = new HashMap<String, List<String>>();
		if(ApplicationUtil.isNotBlank(telefono) && tipo != null && ConstantsMap.telefonoFMap.contains(tipo) && impostazioneDao.comparePassword(password)) {
			try {
				Anagrafica a = new Anagrafica();
				List<String> parameters = Collections.list(request.getParameterNames());
				if(ApplicationUtil.notEmpty(parameters)) {
					List<CampoAggiuntivoAnagrafica> dati = anagraficaDao.getAllCampiAggiuntivi();
					Iterator<String> it = parameters.iterator();
					while (it.hasNext()) {
						String e = ApplicationUtil.trimToNull(it.next());
						if(e != null) {
							String obj = request.getParameter(e);
							if(obj != null) {
								ModelImport.importAnagrafica(a, e, obj, dati);
							}
						}
					}
				}
				a.prepareDati();
				AnagraficaXML anagrafica = new AnagraficaXML(a);
				if(tipo.equals(ConstantsMap.telefono_fisso)) {
					anagrafica.setTelefono_fisso(telefono);
				}
				if(tipo.equals(ConstantsMap.telefono_mobile)) {
					anagrafica.setTelefono_mobile(telefono);
				}
				if(tipo.equals(ConstantsMap.telefono_fax)) {
					anagrafica.setTelefono_fax(telefono);
				}
				if(tipo.equals(ConstantsMap.telefono_altro)) {
					anagrafica.setTelefono_altro(telefono);
				}
				if(tipo.equals(ConstantsMap.telefono_altro)) {
					anagrafica.setTelefono_altro(telefono);
				}
				List<String> m = controllerDao.inviaContatto(anagrafica, campagna, lista, null, null);
				messageS.put(ApplicationUtil.id(anagrafica.getId(), anagrafica.getTipo(), anagrafica.getNome_completo(), anagrafica.getRagione_sociale(), anagrafica.getCognome(), anagrafica.getNome(), null), m);
			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		}
		ModelAndView mav = new ModelAndView();
		JSONArray jsarray = new JSONArray();
		jsarray.put(messageS);
		mav.setView(new JSONArrayView());
		mav.addObject("array", jsarray);
		return mav;
	}


	//https://BASE_PATH/importa/send/data/campagna/lista/password/xml
	@RequestMapping(value= "/send/{data}/{campagna}/{lista}/{password}/{ext}", method = RequestMethod.POST) 
	public ModelAndView sendAnagrafica(HttpServletRequest request, HttpServletResponse response, @PathVariable final String data, @PathVariable final Long campagna, @PathVariable final Long lista, @PathVariable final String password, @PathVariable final String ext, @RequestBody String dto){
		Map<String, List<String>> messageS = new HashMap<String, List<String>>();
		String ex = ApplicationUtil.trimToNull(ext);
		Date date = ApplicationUtil.parseData(ConstantsFormat.inputDate.stampa(), data);
		if(ex != null && impostazioneDao.comparePassword(password)) {
			if(ex.equalsIgnoreCase(ConstantsMap.xml) || ex.equalsIgnoreCase(ConstantsMap.json)) {
				try {
					InputStream stream = new ByteArrayInputStream(dto.getBytes(ConstantsGlobal.CHARSET));
					List<AnagraficaXML> anaList = new ArrayList<AnagraficaXML>();
					if(ex.equalsIgnoreCase(ConstantsMap.xml)) {
						ApplicationUtil.xmlDecoder(anaList, stream);
					} 
					if(ex.equalsIgnoreCase(ConstantsMap.json)) {
						ApplicationUtil.jsonDecoder(anaList, stream);
					}
					stream.close();
					for(AnagraficaXML anagrafica : anaList) {
						List<String> m = controllerDao.inviaContatto(anagrafica, campagna, lista, null, date);
						messageS.put(ApplicationUtil.id(anagrafica.getId(), anagrafica.getTipo(), anagrafica.getNome_completo(), anagrafica.getRagione_sociale(), anagrafica.getCognome(), anagrafica.getNome(), null), m);
					}
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				}
			}
		}
		ModelAndView mav = new ModelAndView();
		JSONArray jsarray = new JSONArray();
		jsarray.put(messageS);
		mav.setView(new JSONArrayView());
		mav.addObject("array", jsarray);
		return mav;
	}


	//https://BASE_PATH/importa/send/data/campagna/lista/password
	@RequestMapping(value= "/send/{data}/{campagna}/{lista}/{password}") 
	public ModelAndView sendContatto(HttpServletRequest request, HttpServletResponse response, @PathVariable final String data, @PathVariable final Long campagna, @PathVariable final Long lista, @PathVariable final String password){
		Map<String, List<String>> messageS = new HashMap<String, List<String>>();
		Date date = ApplicationUtil.parseData(ConstantsFormat.inputDate.stampa(), data);
		if(impostazioneDao.comparePassword(password)) {
			try {
				Anagrafica a = new Anagrafica();
				List<String> parameters = Collections.list(request.getParameterNames());
				if(ApplicationUtil.notEmpty(parameters)) {
					List<CampoAggiuntivoAnagrafica> dati = anagraficaDao.getAllCampiAggiuntivi();
					Iterator<String> it = parameters.iterator();
					while (it.hasNext()) {
						String e = ApplicationUtil.trimToNull(it.next());
						if(e != null) {
							String obj = request.getParameter(e);
							if(obj != null) {
								ModelImport.importAnagrafica(a, e, obj, dati);
							}
						}
					}
				}
				a.prepareDati();
				AnagraficaXML anagrafica = new AnagraficaXML(a);
				List<String> m = controllerDao.inviaContatto(anagrafica, campagna, lista, null, date);
				messageS.put(ApplicationUtil.id(anagrafica.getId(), anagrafica.getTipo(), anagrafica.getNome_completo(), anagrafica.getRagione_sociale(), anagrafica.getCognome(), anagrafica.getNome(), null), m);
			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		}
		ModelAndView mav = new ModelAndView();
		JSONArray jsarray = new JSONArray();
		jsarray.put(messageS);
		mav.setView(new JSONArrayView());
		mav.addObject("array", jsarray);
		return mav;
	}

	//https://BASE_PATH/importa/send/data/campagna/lista/password/tipo/telefono
	@RequestMapping(value= "/send/{data}/{campagna}/{lista}/{password}/{tipo}/{telefono}") 
	public ModelAndView sendTelefono(HttpServletRequest request, HttpServletResponse response, @PathVariable final String data, @PathVariable final Long campagna, @PathVariable final Long lista, @PathVariable final String password, @PathVariable final String tipo, @PathVariable final String telefono){
		Map<String, List<String>> messageS = new HashMap<String, List<String>>();
		Date date = ApplicationUtil.parseData(ConstantsFormat.inputDate.stampa(), data);
		if(ApplicationUtil.isNotBlank(telefono) && tipo != null && ConstantsMap.telefonoFMap.contains(tipo) && impostazioneDao.comparePassword(password)) {
			try {
				Anagrafica a = new Anagrafica();
				List<String> parameters = Collections.list(request.getParameterNames());
				if(ApplicationUtil.notEmpty(parameters)) {
					List<CampoAggiuntivoAnagrafica> dati = anagraficaDao.getAllCampiAggiuntivi();
					Iterator<String> it = parameters.iterator();
					while (it.hasNext()) {
						String e = ApplicationUtil.trimToNull(it.next());
						if(e != null) {
							String obj = request.getParameter(e);
							if(obj != null) {
								ModelImport.importAnagrafica(a, e, obj, dati);
							}
						}
					}
				}
				a.prepareDati();
				AnagraficaXML anagrafica = new AnagraficaXML(a);
				if(tipo.equals(ConstantsMap.telefono_fisso)) {
					anagrafica.setTelefono_fisso(telefono);
				}
				if(tipo.equals(ConstantsMap.telefono_mobile)) {
					anagrafica.setTelefono_mobile(telefono);
				}
				if(tipo.equals(ConstantsMap.telefono_fax)) {
					anagrafica.setTelefono_fax(telefono);
				}
				if(tipo.equals(ConstantsMap.telefono_altro)) {
					anagrafica.setTelefono_altro(telefono);
				}
				if(tipo.equals(ConstantsMap.telefono_altro)) {
					anagrafica.setTelefono_altro(telefono);
				}
				List<String> m = controllerDao.inviaContatto(anagrafica, campagna, lista, null, date);
				messageS.put(ApplicationUtil.id(anagrafica.getId(), anagrafica.getTipo(), anagrafica.getNome_completo(), anagrafica.getRagione_sociale(), anagrafica.getCognome(), anagrafica.getNome(), null), m);
			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		}
		ModelAndView mav = new ModelAndView();
		JSONArray jsarray = new JSONArray();
		jsarray.put(messageS);
		mav.setView(new JSONArrayView());
		mav.addObject("array", jsarray);
		return mav;
	}

}