package it.callite.controller;
import java.io.ByteArrayInputStream;
import java.io.Serializable;
import java.util.List;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;

import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.constants.ConstantsMap;
import it.callite.dao.EsportaDao;
import it.callite.dao.ImpostazioneDao;
import it.callite.log.ErrorLog;
import it.callite.utility.ApplicationUtil;
@SuppressWarnings("serial")
@Controller
public class ExportController  implements Serializable {

	@Autowired
	public EsportaDao esportaDao;
	@Autowired
	public ImpostazioneDao impostazioneDao;

	//https://BASE_PATH/anagrafica/ids/password/xml
	@RequestMapping(value = ConstantsGlobal.path_web_anagrafica + "/{ids}/{password}/{ext}",  method = RequestMethod.GET)
	@ResponseBody 
	public void getFileAnagrafica(HttpServletRequest request, HttpServletResponse response, @PathVariable final String ids, @PathVariable final String password, @PathVariable final String ext)  {
		if(impostazioneDao.comparePassword(password)) {
			if(ApplicationUtil.isNotBlank(ids)) {
				String itc = ApplicationUtil.base64Decoder(ids);
				List<Long> id = ApplicationUtil.tokenL(itc, ConstantsGlobal.colonSeparator);
				String t = ApplicationUtil.trimToNull(ext);
				if(ApplicationUtil.notEmpty(id) && t != null) {
					String result = null;
					String nomeFile = null;
					if(t.equals(ConstantsMap.xml)) {
						result = ApplicationUtil.trimToNull(esportaDao.preparaAnagraficaXML(id.toArray(new Long[id.size()])));
						nomeFile =  ConstantsLanguage.ANAGRAFICA.stampa() + " " + id  + "." + ConstantsMap.xml;
					}
					if(t.equals(ConstantsMap.json)) {
						result = ApplicationUtil.trimToNull(esportaDao.preparaAnagraficaJson(id.toArray(new Long[id.size()])));
						nomeFile =  ConstantsLanguage.ANAGRAFICA.stampa() + " " + id  + "." + ConstantsMap.json;
					}
					try {
						if(result != null && nomeFile != null) {
							ByteArrayInputStream istream = new ByteArrayInputStream(result.getBytes(ConstantsGlobal.CHARSET));
							ApplicationUtil.setResponse(null, impostazioneDao.getDownloadLogType(), true, null, request,  response, nomeFile);
							ServletOutputStream out = response.getOutputStream();

							ApplicationUtil.copy(istream, out);
							out.flush();
							out.close();
							istream.close();
						}
						response.flushBuffer();
					} catch (Exception e) {
						ErrorLog.log(null, getClass(), e);
					}
				}
			}
		}
	}

	//https://BASE_PATH/contatto/ids/password/xml
	@RequestMapping(value = ConstantsGlobal.path_web_contatto + "/{ids}/{password}/{ext}",  method = RequestMethod.GET)
	@ResponseBody 
	public void getFileContatto(HttpServletRequest request, HttpServletResponse response, @PathVariable final String ids, @PathVariable final String password, @PathVariable final String ext)  {
		if(impostazioneDao.comparePassword(password)) {
			if(ApplicationUtil.isNotBlank(ids)) {
				String itc = ApplicationUtil.base64Decoder(ids);
				List<Long> id = ApplicationUtil.tokenL(itc, ConstantsGlobal.colonSeparator);
				String t = ApplicationUtil.trimToNull(ext);
				if(ApplicationUtil.notEmpty(id) && t != null) {
					String result = null;
					String nomeFile = null;
					if(t.equals(ConstantsMap.xml)) {
						result = ApplicationUtil.trimToNull(esportaDao.preparaContattoXML(id.toArray(new Long[id.size()])));
						nomeFile = ConstantsLanguage.CONTATTO.stampa() + " " + id  + "." + ConstantsMap.xml;
					}
					if(t.equals(ConstantsMap.json)) {
						result = ApplicationUtil.trimToNull(esportaDao.preparaContattoJson(id.toArray(new Long[id.size()])));
						nomeFile = ConstantsLanguage.CONTATTO.stampa() + " " + id  + "." + ConstantsMap.json;
					}
					try {
						if(result != null && nomeFile != null) {
							ByteArrayInputStream istream = new ByteArrayInputStream(result.getBytes(ConstantsGlobal.CHARSET));
							ApplicationUtil.setResponse(null, impostazioneDao.getDownloadLogType(), true, null, request,  response, nomeFile);
							ServletOutputStream out = response.getOutputStream();

							ApplicationUtil.copy(istream, out);
							out.flush();
							out.close();
							istream.close();
						}
						response.flushBuffer();
					} catch (Exception e) {
						ErrorLog.log(null, getClass(), e);
					}
				}
			}
		}
	}

}