package it.callite.criteria.generic;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.hibernate.criterion.Criterion;
import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Restrictions;

import it.callite.constants.ConstantsColumn;
import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsMap;
import it.callite.dao.ImpostazioneDao;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.Ricerca;
import it.callite.utility.intervallo.Intervallo;
import it.callite.utility.intervallo.Intervallo1;
import it.callite.utility.intervallo.Intervallo2;
import it.callite.utility.intervallo.Intervallo3;
import it.callite.utility.intervallo.Intervallo4;
import it.callite.utility.intervallo.Intervallo5;

public abstract class GenericAllCriteria extends UnitedCriteria implements Serializable {
	
	private static final long serialVersionUID = 1L;

	private List<String> dati1 = new ArrayList<String>();
	private List<String> dati2 = new ArrayList<String>();
	private List<String> dati3 = new ArrayList<String>();
	private List<String> dati4 = new ArrayList<String>();
	private List<String> dati5 = new ArrayList<String>();
	private List<String> defaults = new ArrayList<String>();
	private Map<String, Intervallo> mappa = new HashMap<String, Intervallo>();
	private String o_colonna = "";
	private String o_tipo	= ConstantsGlobal.asc;
	private Boolean tasto = false;
	private Boolean visibileId = false;
	private String ids;
	
	public boolean isRicercabile() {
		return mappa.containsKey(ConstantsColumn.id);
	}

	public String getIds() {
		return ApplicationUtil.trimToNull(ids);
	}

	public void setIds(String ids) {
		this.ids = ApplicationUtil.trimToNull(ids);
	}



	public Boolean getVisibileId() {
		return visibileId;
	}

	public void setVisibileId(Boolean visibileId) {
		this.visibileId = visibileId;
	}

	public Boolean getTasto() {
		return tasto;
	}

	public void add(String key, int a, Boolean visibile, int tipo) {
		visibile = (visibile == null) ? a < ConstantsGlobal.ricercaMaxSize : visibile;
		Intervallo i = null;
		if(ApplicationUtil.eq(tipo, ConstantsGlobal.ricerca1)) {
			i =  new Intervallo1();
		}
		if(ApplicationUtil.eq(tipo, ConstantsGlobal.ricerca2)) {
			i =  new Intervallo2();
		}
		if(ApplicationUtil.eq(tipo, ConstantsGlobal.ricerca3)) {
			i =  new Intervallo3();
		}
		if(ApplicationUtil.eq(tipo, ConstantsGlobal.ricerca4)) {
			i =  new Intervallo4();
		}
		if(ApplicationUtil.eq(tipo, ConstantsGlobal.ricerca5)) {
			i =  new Intervallo5();
		}
		if(i != null) {
			i.setVisibile(visibile);
			i.setTipo(tipo);
			mappa.put(key, i);
			if(ApplicationUtil.eq(tipo, ConstantsGlobal.ricerca1)) {
				dati1.add(key);
			}
			if(ApplicationUtil.eq(tipo, ConstantsGlobal.ricerca2)) {
				dati2.add(key);
			}
			if(ApplicationUtil.eq(tipo, ConstantsGlobal.ricerca3)) {
				dati3.add(key);
			}
			if(ApplicationUtil.eq(tipo, ConstantsGlobal.ricerca4)) {
				dati4.add(key);
			}
			if(ApplicationUtil.eq(tipo, ConstantsGlobal.ricerca5)) {
				dati5.add(key);
			}
			if(visibile) {
				defaults.add(key);
			}
		}
	}

	public void addAll(String datiA [], String datiB [], String datiC [], String datiD [], String datiE []) {
		if(datiA != null) {
			for(int i = 0; i < datiA.length; i++) {
				add(datiA[i], i, null, ConstantsGlobal.ricerca1);
			}
		}
		if(datiB != null) {
			for(int i = 0; i < datiB.length; i++) {
				add(datiB[i], i, false, ConstantsGlobal.ricerca2);
			}
		}
		if(datiC != null) {
			for(int i = 0; i < datiC.length; i++) {
				add(datiC[i], i, false, ConstantsGlobal.ricerca3);
			}
		}
		if(datiD != null) {
			for(int i = 0; i < datiD.length; i++) {
				add(datiD[i], i, false, ConstantsGlobal.ricerca4);
			}
		}
		if(datiE != null) {
			for(int i = 0; i < datiE.length; i++) {
				add(datiE[i], i, false, ConstantsGlobal.ricerca5);
			}
		}
	}

	public void doAzzera(){
		if (ApplicationUtil.notEmpty(mappa)) {
			Iterator<String> objIter = mappa.keySet().iterator();
			while(objIter.hasNext()) {
				String s = objIter.next();
				Intervallo i = mappa.get(s);
				if(i != null) {
					i.setVisibile(false);
				}
			}
		}
	}

	public void doImposta(){
		if (ApplicationUtil.notEmpty(dati1)) {
			Iterator<String> objIter = dati1.iterator();
			while(objIter.hasNext()) {
				String s = objIter.next();
				Intervallo1 i = (Intervallo1) mappa.get(s);
				if(i != null && ApplicationUtil.isTrue(i.getVisibile())) {
					if(ApplicationUtil.isNotBlank(i.getPeriodo())) {
						if(i.getPeriodo().equals("reset")) {
							i.reset();
						} else {
							String [] p = i.getPeriodo().split(ConstantsGlobal.colonSeparator);
							String periodo = p[0];
							Integer ii = Integer.parseInt(p[1]);
							if(periodo.equals(ConstantsMap.p_g)){
								i.setInizio(ApplicationUtil.inizioGiornata(ii));
								i.setFine(ApplicationUtil.fineGiornata(ii));
							}
							if(periodo.equals(ConstantsMap.p_s)){
								i.setInizio(ApplicationUtil.inizioSettimana(ii));
								i.setFine(ApplicationUtil.fineSettimana(ii));
							}
							if(periodo.equals(ConstantsMap.p_m)){
								i.setInizio(ApplicationUtil.inizioMese(ii));
								i.setFine(ApplicationUtil.fineMese(ii));
							}
							if(periodo.equals(ConstantsMap.p_a)){
								i.setInizio(ApplicationUtil.inizioAnno(ii));
								i.setFine(ApplicationUtil.fineAnno(ii));
							}
						}
					} 
				}
			}
		}
	}

	public void doInverti(){
		if (ApplicationUtil.notEmpty(mappa)) {
			Iterator<String> objIter = mappa.keySet().iterator();
			while(objIter.hasNext()) {
				String s = objIter.next();
				Intervallo i = mappa.get(s);
				if(i != null) {
					if(ApplicationUtil.isTrue(i.getVisibile())){
						i.setVisibile(false);
					} else {
						i.setVisibile(true);
					}
				}
			}
		}
	}

	public void doReset(){
		if (ApplicationUtil.notEmpty(mappa)) {
			Iterator<String> objIter = mappa.keySet().iterator();
			while(objIter.hasNext()) {
				String s = objIter.next();
				Intervallo i = mappa.get(s);
				if(i != null) {
					if(defaults.contains(s)){
						i.setVisibile(true);
					} else {
						i.setVisibile(false);
					}
				}
			}
		}
	}

	public void doTutti(){
		if (ApplicationUtil.notEmpty(mappa)) {
			Iterator<String> objIter = mappa.keySet().iterator();
			while(objIter.hasNext()) {
				String s = objIter.next();
				Intervallo i = mappa.get(s);
				if(i != null) {
					i.setVisibile(true);
				}
			}
		}
	}

	public boolean getAzzera(){
		if (ApplicationUtil.notEmpty(mappa)) {
			Iterator<String> objIter = mappa.keySet().iterator();
			while(objIter.hasNext()) {
				String s = objIter.next();
				Intervallo i = mappa.get(s);
				if(i != null && ApplicationUtil.isTrue(i.getVisibile())) {
					return false;
				}
			}
		}
		return true;
	}

	public boolean getAzzeraDate(){
		if (ApplicationUtil.notEmpty(dati1)) {
			Iterator<String> objIter = dati1.iterator();
			while(objIter.hasNext()) {
				String s = objIter.next();
				Intervallo i = mappa.get(s);
				if(i != null && ApplicationUtil.isTrue(i.getVisibile())) {
					return false;
				}
			}
		}
		return true;
	}

	public Map<String, Intervallo> getMappa() {
		return mappa;
	}

	public int getMaxSize() {
		return ConstantsGlobal.ricercaMaxSize;
	}

	public String getO_colonna() {
		return o_colonna;
	}

	public String getO_tipo() {
		return o_tipo;
	}

	public Boolean getRendered() {
		return dati1.size() > 0 || dati2.size() > 0 || dati3.size() > 0 || dati4.size() > 0 || dati5.size() > 0;
	}



	public List<String> getSet1() {
		return dati1;
	}

	public List<String> getSet2() {
		return dati2;
	}

	public List<String> getSet3() {
		return dati3;
	}

	public List<String> getSet4() {
		return dati4;
	}

	public List<String> getSet5() {
		return dati5;
	}

	public boolean getTutti(){
		if (ApplicationUtil.notEmpty(mappa)) {
			Iterator<String> objIter = mappa.keySet().iterator();
			while(objIter.hasNext()) {
				String s = objIter.next();
				Intervallo i = mappa.get(s);
				if(i != null && ApplicationUtil.isFalse(i.getVisibile())) {
					return true;
				}
			}
		}
		return false;
	}

	public Boolean getValid() {
		return ApplicationUtil.isNotBlank(o_colonna);
	}

	public void impostaRicercaDate(DetachedCriteria criteria) {
		if (ApplicationUtil.notEmpty(mappa)) {
			Iterator<String> objIter = mappa.keySet().iterator();
			while(objIter.hasNext()) {
				String s = objIter.next();
				Intervallo i = mappa.get(s);
				if(ApplicationUtil.isTrue(i.getVisibile())) {
					if(i.getTipo() == ConstantsGlobal.ricerca1) {
						ApplicationUtil.dataIntervalD((Date) i.getInizio(), (Date) i.getFine(), s, criteria);
					}
					if(i.getTipo() == ConstantsGlobal.ricerca2) {
						ApplicationUtil.dataIntervalL((Long) i.getInizio(), (Long) i.getFine(), s, criteria);
					}
					if(i.getTipo() == ConstantsGlobal.ricerca3) {
						ApplicationUtil.dataIntervalI((Integer) i.getInizio(), (Integer) i.getFine(), s, criteria);
					}
					if(i.getTipo() == ConstantsGlobal.ricerca4) {
						ApplicationUtil.dataIntervalT((Date) i.getInizio(), (Date) i.getFine(), s, criteria);
					}
					if(i.getTipo() == ConstantsGlobal.ricerca5) {
						ApplicationUtil.dataIntervalW((Long) i.getInizio(), (Long) i.getFine(), s, criteria);
					}
				}
			}
			if(getVisibileId()) {
				String ricerca = getIds();
				List<String> rs = ApplicationUtil.token(ricerca, ConstantsGlobal.colonSeparator);
				if(ApplicationUtil.notEmpty(rs)) {
					List<Criterion> c = new ArrayList<Criterion>();
					for(String rad : rs) {
						String ra = ApplicationUtil.trimToNull(rad);
						if(ra != null) {
							Ricerca m = ApplicationUtil.match(ra);
							String r = m.getRicerca();
							String search = "like";
							String alias = "{alias}.id";
							if(m.getValore() == 1) {
								if(m.isInverti()) {
									c.add(Restrictions.sqlRestriction(alias + " not "+search+" '"  + r + "'"));
								} else {
									c.add(Restrictions.sqlRestriction(alias + " "+search+" '"  + r + "'"));
								}
							}
							if(m.getValore() == 2) {
								if(m.isInverti()) {
									c.add(Restrictions.sqlRestriction(alias + " not "+search+" '%"  + r + "'"));
								} else {
									c.add(Restrictions.sqlRestriction(alias + " "+search+" '%"  + r + "'"));
								}
							}
							if(m.getValore() == 3) {
								if(m.isInverti()) {
									c.add(Restrictions.sqlRestriction(alias + " not "+search+" '"  + r + "%'"));
								} else {
									c.add(Restrictions.sqlRestriction(alias + " "+search+" '"  + r + "%'"));
								}
							}
							if(m.getValore() == 4) {
								if(m.isInverti()) {
									c.add(Restrictions.sqlRestriction(alias + " not "+search+" '%"  + r + "%'"));
								} else {
									c.add(Restrictions.sqlRestriction(alias + " "+search+" '%"  + r + "%'"));
								}
							}
						}
					}
					if(ApplicationUtil.notEmpty(c)) {
						criteria.add(Restrictions.or(c.toArray(new Criterion[c.size()])));
					}
				}
			}
		}
	}

	public GenericAllCriteria loadCriteria() {
		GenericAllCriteria c = (GenericAllCriteria) ApplicationUtil.deserialize(getSerialize());
		if(c != null) {
			c.doImposta();
			return c;
		}
		return this;
	}

	public void replace(String key, String periodo) {
		Intervallo1 i = (Intervallo1) mappa.get(key);
		if(periodo.equals(ConstantsMap.p_g)){
			i.setInizio(ApplicationUtil.inizioGiornata(0));
			i.setFine(ApplicationUtil.fineGiornata(0));
		}
		if(periodo.equals(ConstantsMap.p_s)){
			i.setInizio(ApplicationUtil.inizioSettimana(0));
			i.setFine(ApplicationUtil.fineSettimana(0));
		}
		if(periodo.equals(ConstantsMap.p_m)){
			i.setInizio(ApplicationUtil.inizioMese(0));
			i.setFine(ApplicationUtil.fineMese(0));
		}
		if(periodo.equals(ConstantsMap.p_a)){
			i.setInizio(ApplicationUtil.inizioAnno(0));
			i.setFine(ApplicationUtil.fineAnno(0));
		}
	}

	public void salvaCriteria(ImpostazioneDao impostazioneDao, Integer b)  {
		if(ApplicationUtil.isTrue(impostazioneDao.getAbilitaRicercaManuale())) {
			setCerca(true);
			tasto = false;
		} else {
			setCerca(ApplicationUtil.ne(b, 1));
			tasto = ApplicationUtil.eq(b, 1);
		}
		if(ApplicationUtil.isTrue(impostazioneDao.getAbilitaRicercaTasto())) {
			if(ApplicationUtil.gtZero(b)) {
				this.setSerialize(null);
				setSerialize(ApplicationUtil.serialize(this));
			}
		} else {
			setSerialize(null);
		}
	}
	
	private String serialize;

	public String getSerialize() {
		return serialize;
	}

	public void setSerialize(String serialize) {
		this.serialize = serialize;
	}
	
	public boolean isRicarica() {
		if(serialize != null) {
			try {
				String [] escluse = {"serialize"};
				Map<String, Object> a = ApplicationUtil.proprieta(this, escluse);
				GenericAllCriteria c = (GenericAllCriteria) ApplicationUtil.deserialize(serialize);
				if(c != null) {
					c.doImposta();
				}
				Map<String, Object> b = ApplicationUtil.proprieta(c, escluse);
				return !ApplicationUtil.equals(a, b);
			} catch (Exception e) {
			}
		}
		return false;
	}



	public void setO_colonna(String o_colonna) {
		this.o_colonna = o_colonna;
	}

	public void setO_tipo(String o_tipo) {
		this.o_tipo = o_tipo;
	}

}
