package it.callite.dao;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.MatchMode;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Projections;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsMap;
import it.callite.criteria.AnagraficaPagineCriteria;
import it.callite.dao.generic.GenericDao;
import it.callite.log.ErrorLog;
import it.callite.model.Comune;
import it.callite.model.Provincia;
import it.callite.utility.AnagraficaPagine;
import it.callite.utility.AnagraficaPagineUtil;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.importa.ImportModelAnagrafica;
@SuppressWarnings("serial")
@Repository("anagraficaPagineDao")
public class AnagraficaPagineDao extends GenericDao {
	@Autowired
	private AnagraficaDao anagraficaDao;
	@Autowired
	private ImpostazioneDao impostazioneDao;
	
	@Autowired
	private DataDao dataDao;
	
	public ImportModelAnagrafica nuovoPagine() {
		return new ImportModelAnagrafica();
	}
	public ImportModelAnagrafica nuovoPagine(ImportModelAnagrafica a) {
		return a != null ? a : new ImportModelAnagrafica();
	}
	public ImportModelAnagrafica getAnagraficheByPagineBianche(Long campagnaId, AnagraficaPagineCriteria pag) {
		if(pag != null)  pag = (AnagraficaPagineCriteria) pag.loadCriteria();
		ImportModelAnagrafica importModel = new ImportModelAnagrafica();
		if(pag != null && pag.getCerca()) {
			if(ApplicationUtil.isNotBlank(pag.getRicerca())) {
				Map<String, String> province = anagraficaDao.trovaProvince();
				Map<String, String> comuni = anagraficaDao.trovaComuni();
				List<String> opposizioni = new ArrayList<String>();
				Integer oC = impostazioneDao.getOpposizioneTemplate();
				if(ApplicationUtil.gt(oC, 0)) {
					importModel.setOpposizione(true);
					if(ApplicationUtil.gtZero(campagnaId)) {
						opposizioni.addAll(impostazioneDao.trovaOpposizioni(campagnaId));
					} 
					opposizioni.addAll(impostazioneDao.trovaOpposizioni(null));
				}
				List<String> duplicati_fisso = new ArrayList<String>();
				String dFisso = impostazioneDao.getDuplicatoFissoTemplate(); 
				if(!dFisso.equals(ConstantsMap.no)) {
					importModel.setDuplicato_fisso(true);
					if(ConstantsMap.getDuplicatoList().contains(dFisso)) {
						duplicati_fisso.addAll(impostazioneDao.trovaDuplicati("telefono_fisso", dFisso, campagnaId, null));
					}
				}
				List<String> duplicati_mobile = new ArrayList<String>();
				String dMobile = impostazioneDao.getDuplicatoMobileTemplate(); 
				if(!dMobile.equals(ConstantsMap.no)) {
					importModel.setDuplicato_mobile(true);
					if(ConstantsMap.getDuplicatoList().contains(dMobile)) {
						duplicati_mobile.addAll(impostazioneDao.trovaDuplicati("telefono_mobile", dMobile, campagnaId, null));
					}
				}
				List<String> duplicati_fax = new ArrayList<String>();
				String dFax = impostazioneDao.getDuplicatoFaxTemplate(); 
				if(!dFax.equals(ConstantsMap.no)) {
					importModel.setDuplicato_fax(true);
					if(ConstantsMap.getDuplicatoList().contains(dFax)) {
						duplicati_fax.addAll(impostazioneDao.trovaDuplicati("telefono_fax", dFax, campagnaId, null));
					}
				}
				List<String> duplicati_altro = new ArrayList<String>();
				String dAltro = impostazioneDao.getDuplicatoAltroTemplate(); 
				if(!dAltro.equals(ConstantsMap.no)) {
					importModel.setDuplicato_altro(true);
					if(ConstantsMap.getDuplicatoList().contains(dAltro)) {
						duplicati_altro.addAll(impostazioneDao.trovaDuplicati("telefono_altro", dAltro, campagnaId, null));
					}
				}
				List<String> duplicati_email = new ArrayList<String>();
				List<String> duplicati_link = new ArrayList<String>();
				List<String> duplicati_codice = new ArrayList<String>();
				List<String> duplicati_partita = new ArrayList<String>();
				importModel.setDuplicato_email(false);
				importModel.setDuplicato_link(false);
				importModel.setDuplicato_codice(false);
				importModel.setDuplicato_partita(false);
				if(impostazioneDao.getRichiestoFaxTemplate()) {
					importModel.setRichiesto_fax(true);
				}
				if(impostazioneDao.getRichiestoAltroTemplate()) {
					importModel.setRichiesto_altro(true);
				}
				if(impostazioneDao.getRichiestoMobileTemplate()) {
					importModel.setRichiesto_mobile(true);
				}
				if(impostazioneDao.getRichiestoFissoTemplate()) {
					importModel.setRichiesto_fisso(true);
				}
				importModel.setRichiesto_email(false);
				importModel.setRichiesto_link(false);
				importModel.setRichiesto_codice(false);
				importModel.setRichiesto_partita(false);
				importModel.setImportCondition(impostazioneDao.getTemplateImportCondition());
				List<AnagraficaPagine> lista = new ArrayList<AnagraficaPagine>();
				importModel.setRegulare_fisso(impostazioneDao.getRegulareFissoTemplate());
				importModel.setRegulare_mobile(impostazioneDao.getRegulareMobileTemplate());
				importModel.setRegulare_fax(impostazioneDao.getRegulareFaxTemplate());
				importModel.setRegulare_altro(impostazioneDao.getRegulareAltroTemplate());
				importModel.setRegulare_email(impostazioneDao.getRegulareEmailTemplate());
				importModel.setRegulare_link(impostazioneDao.getRegulareLinkTemplate());
				importModel.setRegulare_codice(impostazioneDao.getRegulareCodiceTemplate());
				importModel.setRegulare_partita(impostazioneDao.getRegularePartitaTemplate());
				importModel.setValidatore_fisso(impostazioneDao.validatore_fisso());
				importModel.setValidatore_mobile(impostazioneDao.validatore_mobile());
				importModel.setValidatore_fax(impostazioneDao.validatore_fax());
				importModel.setValidatore_altro(impostazioneDao.validatore_altro());
				importModel.setValidatore_email(impostazioneDao.validatore_email());
				importModel.setValidatore_link(impostazioneDao.validatore_link());
				importModel.setValidatore_codice(impostazioneDao.validatore_codice());
				importModel.setValidatore_partita(impostazioneDao.validatore_partita());
				
				List<String> zone = ApplicationUtil.token(pag.getZona(), ConstantsGlobal.colonSeparator);
				List<String> ricerche = ApplicationUtil.token(pag.getRicerca(), ConstantsGlobal.colonSeparator);
				if(ApplicationUtil.gt(pag.getTipo(), 0)) {
					if(ApplicationUtil.empty(zone)) {
						zone = new ArrayList<String>();
						zone.add("");
					}
				}
				for(String zona : zone) {
					for(String ricerca : ricerche) {
						String ricercaN = ApplicationUtil.trimToEmpty(ricerca);
						String zonaN = ApplicationUtil.trimToEmpty(zona);
						List<String> sb = new ArrayList<String>();
						if(ApplicationUtil.isNotBlank(ricercaN)) {
							sb.add(ricercaN);
						}
						if(ApplicationUtil.isNotBlank(zonaN)) {
							sb.add(zonaN);
						}
						String importaFile = ApplicationUtil.detoken(sb, " - ");
						if(ApplicationUtil.isNotBlank(ricercaN)) {
							try {
								lista.addAll(AnagraficaPagineUtil.pagineBianche(pag.isEsegui(), pag.getTipo(), ricercaN,  zonaN, pag.getSize()));
								importModel.addAnagrafiche(AnagraficaPagineUtil.aggiustaAnagrafica(oC, importaFile, lista, opposizioni,  duplicati_fisso, duplicati_mobile, duplicati_fax, duplicati_altro, duplicati_email, duplicati_link, duplicati_codice, duplicati_partita,  importModel, province, comuni));
							} catch (Exception e) {
								ErrorLog.log(null, getClass(), e);
							}
							lista.clear();
						}
					}
				}
				Boolean s = impostazioneDao.getBloccaImportazione();
				importModel.setRighe(s ? dataDao.getRigheSize() : importModel.getAnagraficheSize());
			}
		}
		return importModel;
	}
	public ImportModelAnagrafica getAnagraficheByPagineGialle(Long campagnaId, AnagraficaPagineCriteria pag) {
		if(pag != null)  pag = (AnagraficaPagineCriteria) pag.loadCriteria();
		ImportModelAnagrafica importModel = new ImportModelAnagrafica();
		if(pag != null && pag.getCerca()) {
			if(ApplicationUtil.isNotBlank(pag.getRicerca())) {
				Map<String, String> province = anagraficaDao.trovaProvince();
				Map<String, String> comuni = anagraficaDao.trovaComuni();
				List<String> opposizioni = new ArrayList<String>();
				Integer oC = impostazioneDao.getOpposizioneTemplate();
				if(ApplicationUtil.gt(oC, 0)) {
					importModel.setOpposizione(true);
					if(ApplicationUtil.gtZero(campagnaId)) {
						opposizioni.addAll(impostazioneDao.trovaOpposizioni(campagnaId));
					} 
					opposizioni.addAll(impostazioneDao.trovaOpposizioni(null));
				}
				List<String> duplicati_fisso = new ArrayList<String>();
				String dFisso = impostazioneDao.getDuplicatoFissoTemplate(); 
				if(!dFisso.equals(ConstantsMap.no)) {
					importModel.setDuplicato_fisso(true);
					if(ConstantsMap.getDuplicatoList().contains(dFisso)) {
						duplicati_fisso.addAll(impostazioneDao.trovaDuplicati("telefono_fisso", dFisso, campagnaId, null));
					}
				}
				List<String> duplicati_mobile = new ArrayList<String>();
				String dMobile = impostazioneDao.getDuplicatoMobileTemplate(); 
				if(!dMobile.equals(ConstantsMap.no)) {
					importModel.setDuplicato_mobile(true);
					if(ConstantsMap.getDuplicatoList().contains(dMobile)) {
						duplicati_mobile.addAll(impostazioneDao.trovaDuplicati("telefono_mobile", dMobile, campagnaId, null));
					}
				}
				List<String> duplicati_fax = new ArrayList<String>();
				String dFax = impostazioneDao.getDuplicatoFaxTemplate(); 
				if(!dFax.equals(ConstantsMap.no)) {
					importModel.setDuplicato_fax(true);
					if(ConstantsMap.getDuplicatoList().contains(dFax)) {
						duplicati_fax.addAll(impostazioneDao.trovaDuplicati("telefono_fax", dFax, campagnaId, null));
					}
				}
				List<String> duplicati_altro = new ArrayList<String>();
				String dAltro = impostazioneDao.getDuplicatoAltroTemplate(); 
				if(!dAltro.equals(ConstantsMap.no)) {
					importModel.setDuplicato_altro(true);
					if(ConstantsMap.getDuplicatoList().contains(dAltro)) {
						duplicati_altro.addAll(impostazioneDao.trovaDuplicati("telefono_altro", dAltro, campagnaId, null));
					}
				}
				List<String> duplicati_email = new ArrayList<String>();
				List<String> duplicati_link = new ArrayList<String>();
				List<String> duplicati_codice = new ArrayList<String>();
				List<String> duplicati_partita = new ArrayList<String>();
				importModel.setDuplicato_email(false);
				importModel.setDuplicato_link(false);
				importModel.setDuplicato_codice(false);
				importModel.setDuplicato_partita(false);
				if(impostazioneDao.getRichiestoFaxTemplate()) {
					importModel.setRichiesto_fax(true);
				}
				if(impostazioneDao.getRichiestoAltroTemplate()) {
					importModel.setRichiesto_altro(true);
				}
				if(impostazioneDao.getRichiestoMobileTemplate()) {
					importModel.setRichiesto_mobile(true);
				}
				if(impostazioneDao.getRichiestoFissoTemplate()) {
					importModel.setRichiesto_fisso(true);
				}
				importModel.setRichiesto_email(false);
				importModel.setRichiesto_link(false);
				importModel.setRichiesto_codice(false);
				importModel.setRichiesto_partita(false);
				importModel.setImportCondition(impostazioneDao.getTemplateImportCondition());
				List<AnagraficaPagine> lista = new ArrayList<AnagraficaPagine>();
				importModel.setRegulare_fisso(impostazioneDao.getRegulareFissoTemplate());
				importModel.setRegulare_mobile(impostazioneDao.getRegulareMobileTemplate());
				importModel.setRegulare_fax(impostazioneDao.getRegulareFaxTemplate());
				importModel.setRegulare_altro(impostazioneDao.getRegulareAltroTemplate());
				importModel.setRegulare_email(impostazioneDao.getRegulareEmailTemplate());
				importModel.setRegulare_link(impostazioneDao.getRegulareLinkTemplate());
				importModel.setRegulare_codice(impostazioneDao.getRegulareCodiceTemplate());
				importModel.setRegulare_partita(impostazioneDao.getRegularePartitaTemplate());
				importModel.setValidatore_fisso(impostazioneDao.validatore_fisso());
				importModel.setValidatore_mobile(impostazioneDao.validatore_mobile());
				importModel.setValidatore_fax(impostazioneDao.validatore_fax());
				importModel.setValidatore_altro(impostazioneDao.validatore_altro());
				importModel.setValidatore_email(impostazioneDao.validatore_email());
				importModel.setValidatore_link(impostazioneDao.validatore_link());
				importModel.setValidatore_codice(impostazioneDao.validatore_codice());
				importModel.setValidatore_partita(impostazioneDao.validatore_partita());
				List<String> zone = ApplicationUtil.token(pag.getZona(), ConstantsGlobal.colonSeparator);
				List<String> ricerche = ApplicationUtil.token(pag.getRicerca(), ConstantsGlobal.colonSeparator);
				if(ApplicationUtil.gt(pag.getTipo(), 0)) {
					if(ApplicationUtil.empty(zone)) {
						zone = new ArrayList<String>();
						zone.add("");
					}
				}
				for(String zona : zone) {
					for(String ricerca : ricerche) {
						String ricercaN = ApplicationUtil.trimToEmpty(ricerca);
						String zonaN = ApplicationUtil.trimToEmpty(zona);
						List<String> sb = new ArrayList<String>();
						if(ApplicationUtil.isNotBlank(ricercaN)) {
							sb.add(ricercaN);
						}
						if(ApplicationUtil.isNotBlank(zonaN)) {
							sb.add(zonaN);
						}
						String importaFile = ApplicationUtil.detoken(sb, " - ");
						if(ApplicationUtil.isNotBlank(ricercaN)) {
							try {
								lista.addAll(AnagraficaPagineUtil.pagineGialle(pag.isEsegui(), pag.getTipo(), ricercaN,  zonaN, pag.getSize()));
								importModel.addAnagrafiche(AnagraficaPagineUtil.aggiustaAnagrafica(oC, importaFile, lista, opposizioni,  duplicati_fisso, duplicati_mobile, duplicati_fax, duplicati_altro, duplicati_email, duplicati_link, duplicati_codice, duplicati_partita,  importModel, province, comuni));
							} catch (Exception e) {
								ErrorLog.log(null, getClass(), e);
							}
							lista.clear();
						}
					}
				}
				Boolean s = impostazioneDao.getBloccaImportazione();
				importModel.setRighe(s ? dataDao.getRigheSize() : importModel.getAnagraficheSize());
			}
		}
		return importModel;
	}
	
	public String getLista(AnagraficaPagineCriteria pag) {
		if(pag != null)  pag = (AnagraficaPagineCriteria) pag.loadCriteria();
		String lista = "";
		if(pag != null && pag.getCerca()) {
			String zonaN = ApplicationUtil.trimToNull(pag.getZona());
			String denominazioneN = ApplicationUtil.trimToNull(pag.getRicerca());
			String telefonoN = ApplicationUtil.trimToNull(pag.getTelefono());
			if(zonaN != null || denominazioneN != null || telefonoN != null) {
				List<String> sb = new ArrayList<String>();
				if(zonaN != null) {
					sb.add(zonaN);
				}
				if(denominazioneN != null) {
					sb.add(denominazioneN);
				}
				if(telefonoN != null) {
					sb.add(telefonoN);
				}
				long t = System.currentTimeMillis();
				String importaFile = ApplicationUtil.detoken(sb, " - ");
				lista = t + ": " +   importaFile;
			}
		}
		return lista;
	}
	@SuppressWarnings("unchecked")
	public List<String> autocompletePagineComune(Object q) {
		String query = ApplicationUtil.trimToNull((String) q);
		if(query != null) {
			DetachedCriteria criteria = DetachedCriteria.forClass(Comune.class);
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, query, criteria, MatchMode.ANYWHERE, "nome");
			criteria.setProjection(Projections.distinct(Projections.property("nome")));
			criteria.addOrder(Order.asc("nome"));
			return getList(criteria);
		}
		return null;
	}
	@SuppressWarnings("unchecked")
	public List<String> autocompletePagineProvincia(Object q) {
		String query = ApplicationUtil.trimToNull((String) q);
		if(query != null) {
			DetachedCriteria criteria = DetachedCriteria.forClass(Provincia.class);
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, query, criteria, MatchMode.ANYWHERE, "codice");
			criteria.setProjection(Projections.distinct(Projections.property("codice")));
			criteria.addOrder(Order.asc("codice"));
			return getList(criteria);
		}
		return null;
	}
}