package it.callite.dao;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Random;
import java.util.Set;
import java.util.concurrent.atomic.AtomicInteger;

import javax.faces.model.SelectItem;

import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.ProjectionList;
import org.hibernate.criterion.Projections;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.DisposableBean;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import it.callite.autoinvio.email.ContattoAvvioProcessoEmail;
import it.callite.autoinvio.rmail.ContattoAvvioProcessoRmail;
import it.callite.autoinvio.telecom.ContattoAvvioProcessoTelecom;
import it.callite.chiamate.Chiamate;
import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsMap;
import it.callite.criteria.EmailCriteria;
import it.callite.criteria.TelecomCriteria;
import it.callite.criteria.base.ExtendedCriteria;
import it.callite.dao.generic.GenericDao;
import it.callite.model.AutoProcessEmail;
import it.callite.model.AutoProcessRmail;
import it.callite.model.AutoProcessTelecom;
import it.callite.model.Campagna;
import it.callite.model.Contatto;
import it.callite.model.Email;
import it.callite.model.EmailTemplate;
import it.callite.model.Rmail;
import it.callite.model.RmailTemplate;
import it.callite.model.Lista;
import it.callite.model.Telecom;
import it.callite.model.TelecomTemplate;
import it.callite.model.report.AutoInvioReportList;
import it.callite.model.report.ProcessoReportEmail;
import it.callite.model.report.ProcessoReportRmail;
import it.callite.model.report.ProcessoReportTelecom;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.ListaRandom;
import it.callite.utility.MailSupport;
import it.callite.utility.item.CampagnaSI;
import it.callite.utility.item.GenericSI;
@SuppressWarnings("serial")
@Repository("autoInvioDao")
public class AutoInvioDao extends GenericDao implements InitializingBean, DisposableBean {
	@Autowired	
	private PannelloDao pannelloDao;
	@Autowired
	private ComunicazioneDao comunicazioneDao;
	@Autowired
	private ImpostazioneDao impostazioneDao;
	private Map<Campagna, ContattoAvvioProcessoEmail> processiEmail = new HashMap<Campagna, ContattoAvvioProcessoEmail>();
	private Map<Campagna, ContattoAvvioProcessoRmail> processiRmail = new HashMap<Campagna, ContattoAvvioProcessoRmail>();
	private Map<Campagna, ContattoAvvioProcessoTelecom> processiTelecom = new HashMap<Campagna, ContattoAvvioProcessoTelecom>();
	public synchronized void inserisciCampagnaEmail(AutoProcessEmail ape)  {
		Campagna campagna = ape.getCampagna();
		if(!processiEmail.containsKey(campagna)) {
			ContattoAvvioProcessoEmail avv = new ContattoAvvioProcessoEmail(this, ape);
			if(ape.getAttivo())
				avv.avvia(null);
			else 
				avv.sospendi();
			avv.start();
			processiEmail.put(campagna, avv);
		}
		salvaAggiorna(ape);
	}
	
	public synchronized void modificaCampagnaEmail(AutoProcessEmail ape)  {
		Campagna campagna = ape.getCampagna();
		if(processiEmail.containsKey(campagna)) {
			ContattoAvvioProcessoEmail avv = processiEmail.get(campagna);
			avv.replace(ape);
			if(ape.getAttivo())
				avv.avvia(null);
			else 
				avv.sospendi();
			processiEmail.put(campagna, avv);
		}
		salvaAggiorna(ape);
	}
	
	public synchronized void caricaCampagnaEmail(AutoProcessEmail ape, boolean stato)  {
		Campagna campagna = ape.getCampagna();
		if(!processiEmail.containsKey(campagna)) {
			ContattoAvvioProcessoEmail avv = new ContattoAvvioProcessoEmail(this, ape);
			if(stato)
				avv.avvia(null);
			else
				avv.sospendi();
			avv.start();
			processiEmail.put(campagna, avv);
		}
	}
	
	public synchronized void inserisciCampagnaRmail(AutoProcessRmail ape)  {
		Campagna campagna = ape.getCampagna();
		if(!processiRmail.containsKey(campagna)) {
			ContattoAvvioProcessoRmail avv = new ContattoAvvioProcessoRmail(this, ape);
			if(ape.getAttivo())
				avv.avvia(null);
			else 
				avv.sospendi();
			avv.start();
			processiRmail.put(campagna, avv);
		}
		salvaAggiorna(ape);
	}
	public synchronized void modificaCampagnaRmail(AutoProcessRmail ape)  {
		Campagna campagna = ape.getCampagna();
		if(processiRmail.containsKey(campagna)) {
			ContattoAvvioProcessoRmail avv = processiRmail.get(campagna);
			avv.replace(ape);
			if(ape.getAttivo())
				avv.avvia(null);
			else 
				avv.sospendi();
			processiRmail.put(campagna, avv);
		}
		salvaAggiorna(ape);
	}
	public synchronized void caricaCampagnaRmail(AutoProcessRmail ape, boolean stato)  {
		Campagna campagna = ape.getCampagna();
		if(!processiRmail.containsKey(campagna)) {
			ContattoAvvioProcessoRmail avv = new ContattoAvvioProcessoRmail(this, ape);
			if(stato)
				avv.avvia(null);
			else
				avv.sospendi();
			avv.start();
			processiRmail.put(campagna, avv);
		}
	}
		
	public synchronized void ricaricaCampagnaEmail(AutoProcessEmail apeO)  {
		AutoProcessEmail ape = (AutoProcessEmail) get(AutoProcessEmail.class, apeO.getId());
		Campagna campagna = ape.getCampagna();
		if(processiEmail.containsKey(campagna)) {
			ContattoAvvioProcessoEmail avv = processiEmail.get(campagna);
			avv.ricarica(ape);
		}
	}
	
	public synchronized void sospendiCampagnaEmail(AutoProcessEmail ape)  {
		Campagna campagna = ape.getCampagna();
		if(processiEmail.containsKey(campagna)) {
			ContattoAvvioProcessoEmail avv = processiEmail.get(campagna);
			avv.sospendi();
		}
		ape.setAttivo(false);
		salvaAggiorna(ape);
	}
	public synchronized void avviaCampagnaEmail(AutoProcessEmail apeO)  {
		AutoProcessEmail ape = (AutoProcessEmail) get(AutoProcessEmail.class, apeO.getId());
		Campagna campagna = ape.getCampagna();
		if(processiEmail.containsKey(campagna)) {
			ContattoAvvioProcessoEmail avv = processiEmail.get(campagna);
			avv.avvia(ape);
		}
		ape.setAttivo(true);
		salvaAggiorna(ape);
	}
	public synchronized void removeCampagnaEmail(AutoProcessEmail ape)  {
		Campagna campagna = ape.getCampagna();
		if(processiEmail.containsKey(campagna)) {
			ContattoAvvioProcessoEmail avv = processiEmail.remove(campagna);
			avv.sospendi();
			avv.interrupt();
		}
		cancellaAggiorna(ape);
	}
	public EmailTemplate getEmailTemplate(Long id) {
		return (EmailTemplate) get(EmailTemplate.class, id);
	}
	
	public Campagna getCampagna(Long id) {
		return (Campagna) get(Campagna.class, id);
	}
	public synchronized void rimuoviDaAttiveEmail(Contatto contatto) {
		Chiamate.getContattiAttiviEmail().remove(contatto.getId());
	}
	public synchronized AutoInvioReportList contaContattiList(AutoProcessEmail ape)  {
		Campagna campagna = ape.getCampagna();
		AutoInvioReportList ad = new AutoInvioReportList();
		ContattoAvvioProcessoEmail avv = processiEmail.get(campagna);
		if(avv!=null) {
			ad.setCodaContattiRiusciti(avv.getCodaManagerContatti().getCodaContattiRiusciti().list());
			ad.setCodaContattiFalliti(avv.getCodaManagerContatti().getCodaContattiFalliti().list());
			ad.setCodaContattiTotali(avv.getCodaManagerContatti().getCodaContattiTotali().list());
		}
		return ad;
	} 
	public Integer inviaEmail(Contatto contatto, EmailTemplate template, MailSupport ms) {
		if (template != null && ApplicationUtil.isTrue(template.getAttivo())) {
			Email email = pannelloDao.creaEmailDaTemplate(template, contatto);
			email.setDestinatario(contatto.getAnagrafica().getEmail());
			email.setContatto(contatto);
			return comunicazioneDao.inviaEmail(email, null, ms);
		}
		return null;
	}
	public synchronized List<ProcessoReportEmail> getProcessiEmail() {
		List<ProcessoReportEmail> processiList = new ArrayList<ProcessoReportEmail>();
		if(ApplicationUtil.notEmpty(processiEmail)) {
			Set<Campagna> lngList = processiEmail.keySet();
			if(ApplicationUtil.notEmpty(lngList)) {
				Iterator<Campagna> lnt = lngList.iterator();
				while (lnt.hasNext()) {
					ContattoAvvioProcessoEmail avv = processiEmail.get(lnt.next());
					ProcessoReportEmail pr = new ProcessoReportEmail();
					pr.setAuto(avv.getAutoProcessEmail());
					pr.setSospeso(avv.getCodaManagerContatti().isSospeso());
					pr.setCaricato(avv.getCodaManagerContatti().isCaricato());
					pr.setCodaContattiFalliti(avv.getCodaManagerContatti().getCodaContattiFalliti().getSize());
					pr.setCodaContattiTotali(avv.getCodaManagerContatti().getCodaContattiTotali().getSize());
					pr.setCodaContattiRiusciti(avv.getCodaManagerContatti().getCodaContattiRiusciti().getSize());
					processiList.add(pr);
				}
			}
		}
		return processiList;
	}
	@SuppressWarnings("unchecked")
	public List<AutoProcessEmail> getAutoProcessEmail() {
		DetachedCriteria critCE = DetachedCriteria.forClass(AutoProcessEmail.class);
		return getList(critCE);
	}
	public void rimuoviContattoTotali(AutoProcessEmail ape, Long id)  {
		ContattoAvvioProcessoEmail avv = processiEmail.get(ape.getCampagna());
		if(avv!=null) {
			avv.getCodaManagerContatti().getCodaContattiTotali().remove(id);
		}
	}
	public void rimuoviContattoFalliti(AutoProcessEmail ape, Long id)  {
		ContattoAvvioProcessoEmail avv = processiEmail.get(ape.getCampagna());
		if(avv!=null) {
			avv.getCodaManagerContatti().getCodaContattiFalliti().remove(id);
		}
	}
	public void rimuoviContattoRiusciti(AutoProcessEmail ape, Long id)  {
		ContattoAvvioProcessoEmail avv = processiEmail.get(ape.getCampagna());
		if(avv!=null) {
			avv.getCodaManagerContatti().getCodaContattiRiusciti().remove(id);
		}
	}
	public void rimuoviContattoTotali(AutoProcessEmail ape)  {
		ContattoAvvioProcessoEmail avv = processiEmail.get(ape.getCampagna());
		if(avv!=null) {
			avv.getCodaManagerContatti().getCodaContattiTotali().clear();
		}
	}
	public void rimuoviContattoFalliti(AutoProcessEmail ape)  {
		ContattoAvvioProcessoEmail avv = processiEmail.get(ape.getCampagna());
		if(avv!=null) {
			avv.getCodaManagerContatti().getCodaContattiFalliti().clear();
		}
	}
	public void rimuoviContattoRiusciti(AutoProcessEmail ape)  {
		ContattoAvvioProcessoEmail avv = processiEmail.get(ape.getCampagna());
		if(avv!=null) {
			avv.getCodaManagerContatti().getCodaContattiRiusciti().clear();
		}
	}
	
	public void rimuoviContattoTotali(AutoProcessRmail ape)  {
		ContattoAvvioProcessoRmail avv = processiRmail.get(ape.getCampagna());
		if(avv!=null) {
			avv.getCodaManagerContatti().getCodaContattiTotali().clear();
		}
	}
	public void rimuoviContattoFalliti(AutoProcessRmail ape)  {
		ContattoAvvioProcessoRmail avv = processiRmail.get(ape.getCampagna());
		if(avv!=null) {
			avv.getCodaManagerContatti().getCodaContattiFalliti().clear();
		}
	}
	public void rimuoviContattoRiusciti(AutoProcessRmail ape)  {
		ContattoAvvioProcessoRmail avv = processiRmail.get(ape.getCampagna());
		if(avv!=null) {
			avv.getCodaManagerContatti().getCodaContattiRiusciti().clear();
		}
	}

	public synchronized Contatto getContattoEmail(Campagna campagna) {
		DetachedCriteria critCE = DetachedCriteria.forClass(Email.class);
		DetachedCriteria critC = critCE.createCriteria("contatto");
		critC.setProjection(Projections.distinct(Projections.property("id")));
		DetachedCriteria criteria = getCriteriaContatto(campagna, critC, 1);
		Contatto out =  pannelloDao.getEstrazioneContatto(criteria);
		if (out != null) {
			Chiamate.getContattiAttiviEmail().put(out);
		} 
		return out;
	}
	
	public List<SelectItem> getTemplateEmailSI() {
		DetachedCriteria criteria = DetachedCriteria.forClass(EmailTemplate.class);
		List<GenericSI> lis = getGenericSI(criteria, "id", "nome");
		return impostazioneDao.getGeneric(lis, false);
	}
	public List<SelectItem> getCampagneEmailSI() {
		DetachedCriteria critCE = DetachedCriteria.forClass(AutoProcessEmail.class);
		DetachedCriteria critC = critCE.createCriteria("campagna");
		critC.setProjection(Projections.distinct(Projections.property("id")));
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		impostazioneDao.subPropertyNotInS(criteria, critC, "id");
		List<CampagnaSI> lis = getCampagnaSI(criteria);
		return impostazioneDao.getCampagna(lis, false, false, false, null);
	}
	
	public Email getEmail(Long id){
		return (Email) get(Email.class, id);
	}
	public void eliminaEmail(Email id) {
		cancellaAggiorna(id);
	}
	private ExtendedCriteria trovaCriteriaEmail(EmailCriteria ema) {
		if(ema != null)  ema = (EmailCriteria) ema.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Email.class);
		DetachedCriteria criteriaCo = criteria.createCriteria("contatto");
		if (ApplicationUtil.isNotBlank(ema.getTesto())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, ema.getTesto(), criteria, "oggetto", "testo");
		}
		if (ApplicationUtil.isNotBlank(ema.getRicerca_anagrafica())) {
			DetachedCriteria criteriaA = criteriaCo.createCriteria("anagrafica");
			ApplicationUtil.anagrafica(ema.getRicerca_anagraficaV(), criteriaA, ema.getRicerca_anagrafica());
		}
		if (ApplicationUtil.gtZero(ema.getCampagna())) {
			criteriaCo.add(Restrictions.eq("campagna.id", ema.getCampagna()));
		}
		if (ApplicationUtil.gtZero(ema.getCodice())) {
			criteria.add(Restrictions.eq("codice", ema.getCodice()));
		}
		ema.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, ema);
	}
	@SuppressWarnings("unchecked")
	public List<Email> trovaEmail(EmailCriteria ema,int start, int size) {
		ExtendedCriteria criteria = trovaCriteriaEmail(ema);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int trovaEmailSize(EmailCriteria ema) {
		ExtendedCriteria criteria = trovaCriteriaEmail(ema);
		return getListSize(criteria);
	}
	
	public void afterPropertiesSet() throws Exception {
		List<AutoProcessEmail> autoEmail = getAutoProcessEmail();
		Iterator<AutoProcessEmail> itEmail = autoEmail.iterator();
		while (itEmail.hasNext()) {
			AutoProcessEmail email = (AutoProcessEmail) itEmail.next();
			caricaCampagnaEmail(email, email.getAttivo());
		}
		List<AutoProcessRmail> autoRmail = getAutoProcessRmail();
		Iterator<AutoProcessRmail> itRmail = autoRmail.iterator();
		while (itRmail.hasNext()) {
			AutoProcessRmail rmail = (AutoProcessRmail) itRmail.next();
			caricaCampagnaRmail(rmail, rmail.getAttivo());
		}
		List<AutoProcessTelecom> autoTelecom = getAutoProcessTelecom();
		Iterator<AutoProcessTelecom> itTelecom = autoTelecom.iterator();
		while (itTelecom.hasNext()) {
			AutoProcessTelecom telecom = (AutoProcessTelecom) itTelecom.next();
			caricaCampagnaTelecom(telecom, telecom.getAttivo());
		}
	}
	@SuppressWarnings("unchecked")
	public List<AutoProcessEmail> getAutoProcessEmailByCampagnaId(Long campagnaId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(AutoProcessEmail.class);
		criteria.add(Restrictions.eq("campagna.id",campagnaId));
		return getList(criteria);
	}
	
	@SuppressWarnings("unchecked")
	public List<AutoProcessEmail> getAutoProcessByEmailId(Long id) {
		DetachedCriteria criteria = DetachedCriteria.forClass(AutoProcessEmail.class);
		criteria.createCriteria("emailTemplate").add(Restrictions.idEq(id));
		return getList(criteria);
	}
	public RmailTemplate getRmailTemplate(Long id) {
		return (RmailTemplate) get(RmailTemplate.class, id);
	}
	
	public synchronized void rimuoviDaAttiveRmail(Contatto contatto) {
		Chiamate.getContattiAttiviRmail().remove(contatto.getId());
	}
	public synchronized AutoInvioReportList contaContattiList(AutoProcessRmail ape)  {
		Campagna campagna = ape.getCampagna();
		AutoInvioReportList ad = new AutoInvioReportList();
		ContattoAvvioProcessoRmail avv = processiRmail.get(campagna);
		if(avv!=null) {
			ad.setCodaContattiRiusciti(avv.getCodaManagerContatti().getCodaContattiRiusciti().list());
			ad.setCodaContattiFalliti(avv.getCodaManagerContatti().getCodaContattiFalliti().list());
			ad.setCodaContattiTotali(avv.getCodaManagerContatti().getCodaContattiTotali().list());
		}
		return ad;
	}
	public synchronized void ricaricaCampagnaRmail(AutoProcessRmail apeO)  {
		AutoProcessRmail ape = (AutoProcessRmail) get(AutoProcessRmail.class, apeO.getId());
		Campagna campagna = ape.getCampagna();
		if(processiRmail.containsKey(campagna)) {
			ContattoAvvioProcessoRmail avv = processiRmail.get(campagna);
			avv.ricarica(ape);
		}
	}
	public synchronized void sospendiCampagnaRmail(AutoProcessRmail ape)  {
		Campagna campagna = ape.getCampagna();
		if(processiRmail.containsKey(campagna)) {
			ContattoAvvioProcessoRmail avv = processiRmail.get(campagna);
			avv.sospendi();
		}
		ape.setAttivo(false);
		salvaAggiorna(ape);
	}
	public synchronized void avviaCampagnaRmail(AutoProcessRmail apeO)  {
		AutoProcessRmail ape = (AutoProcessRmail) get(AutoProcessRmail.class, apeO.getId());
		Campagna campagna = ape.getCampagna();
		if(processiRmail.containsKey(campagna)) {
			ContattoAvvioProcessoRmail avv = processiRmail.get(campagna);
			avv.avvia(ape);
		}
		ape.setAttivo(true);
		salvaAggiorna(ape);
	}
	public synchronized void removeCampagnaRmail(AutoProcessRmail ape)  {
		Campagna campagna = ape.getCampagna();
		if(processiRmail.containsKey(campagna)) {
			ContattoAvvioProcessoRmail avv = processiRmail.remove(campagna);
			avv.sospendi();
			avv.interrupt();
		}
		cancellaAggiorna(ape);
	}
	public Integer inviaRmail(Contatto contatto, RmailTemplate template, MailSupport ms) {
		if (template != null && ApplicationUtil.isTrue(template.getAttivo())) {
			Rmail rmail = pannelloDao.creaRmailDaTemplate(template, contatto);
			rmail.setContatto(contatto);
			rmail.imposta();
			return comunicazioneDao.inviaRmail(rmail, null, ms);
		}
		return null;
	}
	public synchronized List<ProcessoReportRmail> getProcessiRmail() {
		List<ProcessoReportRmail> processiList = new ArrayList<ProcessoReportRmail>();
		if(ApplicationUtil.notEmpty(processiRmail)) {
			Set<Campagna> lngList = processiRmail.keySet();
			if(ApplicationUtil.notEmpty(lngList)) {
				Iterator<Campagna> lnt = lngList.iterator();
				while (lnt.hasNext()) {
					ContattoAvvioProcessoRmail avv = processiRmail.get(lnt.next());
					ProcessoReportRmail pr = new ProcessoReportRmail();
					pr.setAuto(avv.getAutoProcessRmail());
					pr.setSospeso(avv.getCodaManagerContatti().isSospeso());
					pr.setCaricato(avv.getCodaManagerContatti().isCaricato());
					pr.setCodaContattiFalliti(avv.getCodaManagerContatti().getCodaContattiFalliti().getSize());
					pr.setCodaContattiTotali(avv.getCodaManagerContatti().getCodaContattiTotali().getSize());
					pr.setCodaContattiRiusciti(avv.getCodaManagerContatti().getCodaContattiRiusciti().getSize());
					processiList.add(pr);
				}
			}
		}
		return processiList;
	}
	@SuppressWarnings("unchecked")
	public List<AutoProcessRmail> getAutoProcessRmail() {
		DetachedCriteria critCE = DetachedCriteria.forClass(AutoProcessRmail.class);
		return getList(critCE);
	}
	public void rimuoviContattoTotali(AutoProcessRmail ape, Long id)  {
		ContattoAvvioProcessoRmail avv = processiRmail.get(ape.getCampagna());
		if(avv!=null) {
			avv.getCodaManagerContatti().getCodaContattiTotali().remove(id);
		}
	}
	public void rimuoviContattoFalliti(AutoProcessRmail ape, Long id)  {
		ContattoAvvioProcessoRmail avv = processiRmail.get(ape.getCampagna());
		if(avv!=null) {
			avv.getCodaManagerContatti().getCodaContattiFalliti().remove(id);
		}
	}
	public void rimuoviContattoRiusciti(AutoProcessRmail ape, Long id)  {
		ContattoAvvioProcessoRmail avv = processiRmail.get(ape.getCampagna());
		if(avv!=null) {
			avv.getCodaManagerContatti().getCodaContattiRiusciti().remove(id);
		}
	}
	public synchronized Contatto getContattoRmail(Campagna campagna) {
		DetachedCriteria critCE = DetachedCriteria.forClass(Rmail.class);
		DetachedCriteria critC = critCE.createCriteria("contatto");
		critC.setProjection(Projections.distinct(Projections.property("id")));
		DetachedCriteria criteria = getCriteriaContatto(campagna, critC, 4);
		Contatto out =  pannelloDao.getEstrazioneContatto(criteria);
		if (out != null) {
			Chiamate.getContattiAttiviRmail().put(out);
		} 
		return out;
	}
	
	public List<SelectItem> getTemplateRmailSI() {
		DetachedCriteria criteria = DetachedCriteria.forClass(RmailTemplate.class);
		List<GenericSI> lis = getGenericSI(criteria, "id", "nome");
		return impostazioneDao.getGeneric(lis, false);
	}
	public List<SelectItem> getCampagneRmailSI() {
		DetachedCriteria critCE = DetachedCriteria.forClass(AutoProcessRmail.class);
		DetachedCriteria critC = critCE.createCriteria("campagna");
		critC.setProjection(Projections.distinct(Projections.property("id")));
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		impostazioneDao.subPropertyNotInS(criteria, critC, "id");
		List<CampagnaSI> lis = getCampagnaSI(criteria);
		return impostazioneDao.getCampagna(lis, false, false, false, null);
	}
	
	public Rmail getRmail(Long id){
		return (Rmail) get(Rmail.class, id);
	}
	public void eliminaRmail(Rmail id) {
		cancellaAggiorna(id);
	}

	public void destroy() {
	}
	@SuppressWarnings("unchecked")
	public List<AutoProcessRmail> getAutoProcessRmailByCampagnaId(Long campagnaId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(AutoProcessRmail.class);
		criteria.add(Restrictions.eq("campagna.id",campagnaId));
		return getList(criteria);
	}
	
	@SuppressWarnings("unchecked")
	public List<AutoProcessRmail> getAutoProcessByRmailId(Long id) {
		DetachedCriteria criteria = DetachedCriteria.forClass(AutoProcessRmail.class);
		criteria.createCriteria("rmailTemplate").add(Restrictions.idEq(id));
		return getList(criteria);
	}

	public synchronized void rimuoviContatto(Contatto contatto) {
		Chiamate.getContattiAttiviEmail().remove(contatto.getId());
		Chiamate.getContattiAttiviRmail().remove(contatto.getId());
		Chiamate.getContattiAttiviTelecom().remove(contatto.getId());
		ContattoAvvioProcessoEmail avvEmail = processiEmail.get(contatto.getCampagna());
		if(avvEmail!=null) {
			avvEmail.getCodaManagerContatti().getCodaContattiTotali().remove(contatto.getId());
			avvEmail.getCodaManagerContatti().getCodaContattiFalliti().remove(contatto.getId());
			avvEmail.getCodaManagerContatti().getCodaContattiRiusciti().remove(contatto.getId());
		}
		ContattoAvvioProcessoRmail avvRmail = processiRmail.get(contatto.getCampagna());
		if(avvRmail!=null) {
			avvRmail.getCodaManagerContatti().getCodaContattiTotali().remove(contatto.getId());
			avvRmail.getCodaManagerContatti().getCodaContattiFalliti().remove(contatto.getId());
			avvRmail.getCodaManagerContatti().getCodaContattiRiusciti().remove(contatto.getId());
		}
		ContattoAvvioProcessoTelecom avvTelecom = processiTelecom.get(contatto.getCampagna());
		if(avvTelecom!=null) {
			avvTelecom.getCodaManagerContatti().getCodaContattiTotali().remove(contatto.getId());
			avvTelecom.getCodaManagerContatti().getCodaContattiFalliti().remove(contatto.getId());
			avvTelecom.getCodaManagerContatti().getCodaContattiRiusciti().remove(contatto.getId());
		}
	}
	public synchronized void rimuoviContatti(List<Contatto> contatti) {
		Chiamate.getContattiAttiviEmail().removeContatti(contatti);
		Chiamate.getContattiAttiviRmail().removeContatti(contatti);
		Chiamate.getContattiAttiviTelecom().removeContatti(contatti);
		if(ApplicationUtil.notEmpty(processiEmail)) {
			Set<Campagna> lngList = processiEmail.keySet();
			if(ApplicationUtil.notEmpty(lngList)) {
				Iterator<Campagna> lnt = lngList.iterator();
				while (lnt.hasNext()) {
					ContattoAvvioProcessoEmail avvEmail = processiEmail.get(lnt.next());
					if(avvEmail!=null) {
						avvEmail.getCodaManagerContatti().getCodaContattiTotali().removeContatti(contatti);
						avvEmail.getCodaManagerContatti().getCodaContattiFalliti().removeContatti(contatti);
						avvEmail.getCodaManagerContatti().getCodaContattiRiusciti().removeContatti(contatti);
					}
				}
			}
		}
		if(ApplicationUtil.notEmpty(processiRmail)) {
			Set<Campagna> lngList = processiRmail.keySet();
			if(ApplicationUtil.notEmpty(lngList)) {
				Iterator<Campagna> lnt = lngList.iterator();
				while (lnt.hasNext()) {
					ContattoAvvioProcessoRmail avvRmail = processiRmail.get(lnt.next());
					if(avvRmail!=null) {
						avvRmail.getCodaManagerContatti().getCodaContattiTotali().removeContatti(contatti);
						avvRmail.getCodaManagerContatti().getCodaContattiFalliti().removeContatti(contatti);
						avvRmail.getCodaManagerContatti().getCodaContattiRiusciti().removeContatti(contatti);
					}
				}
			}
		}
		if(ApplicationUtil.notEmpty(processiTelecom)) {
			Set<Campagna> lngList = processiTelecom.keySet();
			if(ApplicationUtil.notEmpty(lngList)) {
				Iterator<Campagna> lnt = lngList.iterator();
				while (lnt.hasNext()) {
					ContattoAvvioProcessoTelecom avvTelecom = processiTelecom.get(lnt.next());
					if(avvTelecom!=null) {
						avvTelecom.getCodaManagerContatti().getCodaContattiTotali().removeContatti(contatti);
						avvTelecom.getCodaManagerContatti().getCodaContattiFalliti().removeContatti(contatti);
						avvTelecom.getCodaManagerContatti().getCodaContattiRiusciti().removeContatti(contatti);
					}
				}
			}
		}
	}
	public void contattoCriteria(DetachedCriteria crit, Campagna campagna, DetachedCriteria critC, int tipo) {
		Boolean tutti = impostazioneDao.getEstrazioneCampagna();
		impostazioneDao.subPropertyNotInS(crit, critC, "id");
		crit.add(Restrictions.eq("campagna.id", campagna.getId()));
		crit.add(Restrictions.eq("tipo", ConstantsGlobal.inviato));
		if(tipo == 1) {
			Chiamate.getContattiAttiviEmail().criteria(tutti, campagna, crit);
		}
		if(tipo == 2) {
			Chiamate.getContattiAttiviRmail().criteria(tutti, campagna, crit);
		}
		if(tipo == 3) {
			Chiamate.getContattiAttiviTelecom().criteria(tutti, campagna, crit);
		}
	}
	public DetachedCriteria getCriteriaContatto(Campagna campagna, DetachedCriteria critC, int tipo) {
		Long idLista = null;
		Integer est = campagna.getEstrazione_lista();
		if(ApplicationUtil.gtZero(est)) {
			List<Long> ids = getListeCampagna(campagna, critC, tipo);
			if(ApplicationUtil.notEmpty(ids)) {
				if(ApplicationUtil.eq(est, 1)) {
					AtomicInteger seq = campagna.getSeq();
					int index = seq.get();
					if(index >= ids.size()) {
						int reset = 0;
						seq.set(reset);
						index = reset;
					}
					idLista = ids.get(index);
					seq.set(index+1);
				} 
				if(ApplicationUtil.eq(est, 2)) {
					int index = new Random(System.nanoTime()).nextInt(ids.size());
					idLista = ids.get(index);
				}
			}
		}
		DetachedCriteria crit = DetachedCriteria.forClass(Contatto.class);
		DetachedCriteria crita = null;
		DetachedCriteria critaL = null;
		DetachedCriteria critaO = null;
		String ordine = ApplicationUtil.trimToNull(campagna.getOrdine());
		String query = ApplicationUtil.trimToNull(campagna.getQuery_proprieta());
		String [] ords = null;
		String indirizzo = ConstantsMap.indirizzo;
		String legale = ConstantsMap.indirizzo_legale;
		String operativo = ConstantsMap.indirizzo_operativo;
		if( query != null || ordine != null) {
			crita = crit.createCriteria("anagrafica");
			if((query != null && query.startsWith(legale)) || (ords != null && ords[0].startsWith(legale))) {
				critaL = crita.createCriteria(legale, legale);
			}
			if((query != null && query.startsWith(operativo)) ||  (ords != null && ords[0].startsWith(operativo))) {
				critaO = crita.createCriteria(operativo, operativo);
			}
			if(query != null) {
				pannelloDao.ricercaCriteria(indirizzo, legale, operativo, query, crita, critaL, critaO, campagna);
			}
		}
		contattoCriteria(crit, campagna, critC, tipo);
		if(ApplicationUtil.gtZero(idLista)) {
			crit.add(Restrictions.eq("lista.id", idLista));
		} else {
			Boolean s = impostazioneDao.getEstrazioneListe();
			if(ApplicationUtil.isTrue(s)) {
				Boolean ts = impostazioneDao.getEstrazioneStato();
				DetachedCriteria listaC = crit.createCriteria("lista");
				ApplicationUtil.criteriaStato(listaC, true, ApplicationUtil.isTrue(ts));
			}
		}
		if(!ApplicationUtil.gtZero(idLista) && ApplicationUtil.eq(est, 0)) {
			crit.addOrder(Order.desc("priorita"));
		}
		pannelloDao.ordineCriteria(indirizzo, legale, operativo, ords, crita, critaL, critaO, campagna);
		return crit;
	}
	@SuppressWarnings("unchecked")
	public List<Long> getListeCampagna(Campagna campagna, DetachedCriteria critC, int tipo) {
		DetachedCriteria listaC = DetachedCriteria.forClass(Lista.class);
		listaC.createCriteria("campagna").add(Restrictions.eq("id", campagna.getId()));
		Boolean s = impostazioneDao.getEstrazioneListe();
		if(ApplicationUtil.isTrue(s)) {
			Boolean ts = impostazioneDao.getEstrazioneStato();
			ApplicationUtil.criteriaStato(listaC, true, ApplicationUtil.isTrue(ts));
		}
		DetachedCriteria crit = listaC.createCriteria("chiamate");
		DetachedCriteria crita = null;
		DetachedCriteria critaL = null;
		DetachedCriteria critaO = null;
		String query = ApplicationUtil.trimToNull(campagna.getQuery_proprieta());
		String indirizzo = ConstantsMap.indirizzo;
		String legale = ConstantsMap.indirizzo_legale;
		String operativo = ConstantsMap.indirizzo_operativo;
		if( query != null) {
			crita = crit.createCriteria("anagrafica");
			if((query != null && query.startsWith(legale))) {
				critaL = crita.createCriteria(legale, legale);
			}
			if((query != null && query.startsWith(operativo))) {
				critaO = crita.createCriteria(operativo, operativo);
			}
			if(query != null) {
				pannelloDao.ricercaCriteria(indirizzo, legale, operativo, query, crita, critaL, critaO, campagna);
			}
		}
		contattoCriteria(crit, campagna, critC, tipo);
		ProjectionList projection = Projections.projectionList();
		projection.add(Projections.distinct(Projections.property("id").as("id")));
		projection.add(Projections.property("priorita").as("priorita"));
		listaC.setProjection(projection);
		listaC.addOrder(Order.desc("priorita"));
		List<ListaRandom>  lst = getList(listaC, ListaRandom.class);
		List<Long> ids = new ArrayList<Long>();
		if(ApplicationUtil.notEmpty(lst)) {
			Iterator<ListaRandom> itR = lst.iterator();
			while (itR.hasNext()) {
				ListaRandom lr = (ListaRandom) itR.next();
				int pr = lr.getPriorita() != null ? lr.getPriorita() : 1;
				for(int i = 0; i < pr; i++ ) {
					ids.add(lr.getId());
				}
			}
		}
		return ids;
	}
	
	public Boolean sleepEmailSemaforo() {
		return comunicazioneDao.sleepEmailSemaforo();
	}
	public Boolean sleepRmailSemaforo() {
		return comunicazioneDao.sleepRmailSemaforo();
	}
	public Boolean sleepTelecomSemaforo() {
		return comunicazioneDao.sleepTelecomSemaforo();
	}
	
	public void rimuoviContattoTotali(AutoProcessTelecom ape)  {
		ContattoAvvioProcessoTelecom avv = processiTelecom.get(ape.getCampagna());
		if(avv!=null) {
			avv.getCodaManagerContatti().getCodaContattiTotali().clear();
		}
	}
	public void rimuoviContattoFalliti(AutoProcessTelecom ape)  {
		ContattoAvvioProcessoTelecom avv = processiTelecom.get(ape.getCampagna());
		if(avv!=null) {
			avv.getCodaManagerContatti().getCodaContattiFalliti().clear();
		}
	}
	public void rimuoviContattoRiusciti(AutoProcessTelecom ape)  {
		ContattoAvvioProcessoTelecom avv = processiTelecom.get(ape.getCampagna());
		if(avv!=null) {
			avv.getCodaManagerContatti().getCodaContattiRiusciti().clear();
		}
	}
	
	public synchronized void inserisciCampagnaTelecom(AutoProcessTelecom ape)  {
		Campagna campagna = ape.getCampagna();
		if(!processiTelecom.containsKey(campagna)) {
			ContattoAvvioProcessoTelecom avv = new ContattoAvvioProcessoTelecom(this, ape);
			if(ape.getAttivo())
				avv.avvia(null);
			else 
				avv.sospendi();
			avv.start();
			processiTelecom.put(campagna, avv);
		}
		salvaAggiorna(ape);
	}
	
	public synchronized void modificaCampagnaTelecom(AutoProcessTelecom ape)  {
		Campagna campagna = ape.getCampagna();
		if(processiTelecom.containsKey(campagna)) {
			ContattoAvvioProcessoTelecom avv = processiTelecom.get(campagna);
			avv.replace(ape);
			if(ape.getAttivo())
				avv.avvia(null);
			else 
				avv.sospendi();
			processiTelecom.put(campagna, avv);
		}
		salvaAggiorna(ape);
	}
	public synchronized void caricaCampagnaTelecom(AutoProcessTelecom ape, boolean stato)  {
		Campagna campagna = ape.getCampagna();
		if(!processiTelecom.containsKey(campagna)) {
			ContattoAvvioProcessoTelecom avv = new ContattoAvvioProcessoTelecom(this, ape);
			if(stato)
				avv.avvia(null);
			else
				avv.sospendi();
			avv.start();
			processiTelecom.put(campagna, avv);
		}
	}
	public TelecomTemplate getTelecomTemplate(Long id) {
		return (TelecomTemplate) get(TelecomTemplate.class, id);
	}
	public synchronized void rimuoviDaAttiveTelecom(Contatto contatto) {
		Chiamate.getContattiAttiviTelecom().remove(contatto.getId());
	}
	public synchronized AutoInvioReportList contaContattiList(AutoProcessTelecom ape)  {
		Campagna campagna = ape.getCampagna();
		AutoInvioReportList ad = new AutoInvioReportList();
		ContattoAvvioProcessoTelecom avv = processiTelecom.get(campagna);
		if(avv!=null) {
			ad.setCodaContattiRiusciti(avv.getCodaManagerContatti().getCodaContattiRiusciti().list());
			ad.setCodaContattiFalliti(avv.getCodaManagerContatti().getCodaContattiFalliti().list());
			ad.setCodaContattiTotali(avv.getCodaManagerContatti().getCodaContattiTotali().list());
		}
		return ad;
	}
	public synchronized void ricaricaCampagnaTelecom(AutoProcessTelecom apeO)  {
		AutoProcessTelecom ape = (AutoProcessTelecom) get(AutoProcessTelecom.class, apeO.getId());
		Campagna campagna = ape.getCampagna();
		if(processiTelecom.containsKey(campagna)) {
			ContattoAvvioProcessoTelecom avv = processiTelecom.get(campagna);
			avv.ricarica(ape);
		}
	}
	public synchronized void sospendiCampagnaTelecom(AutoProcessTelecom ape)  {
		Campagna campagna = ape.getCampagna();
		if(processiTelecom.containsKey(campagna)) {
			ContattoAvvioProcessoTelecom avv = processiTelecom.get(campagna);
			avv.sospendi();
		}
		ape.setAttivo(false);
		salvaAggiorna(ape);
	}
	public synchronized void avviaCampagnaTelecom(AutoProcessTelecom apeO)  {
		AutoProcessTelecom ape = (AutoProcessTelecom) get(AutoProcessTelecom.class, apeO.getId());
		Campagna campagna = ape.getCampagna();
		if(processiTelecom.containsKey(campagna)) {
			ContattoAvvioProcessoTelecom avv = processiTelecom.get(campagna);
			avv.avvia(ape);
		}
		ape.setAttivo(true);
		salvaAggiorna(ape);
	}
	public synchronized void removeCampagnaTelecom(AutoProcessTelecom ape)  {
		Campagna campagna = ape.getCampagna();
		if(processiTelecom.containsKey(campagna)) {
			ContattoAvvioProcessoTelecom avv = processiTelecom.remove(campagna);
			avv.sospendi();
			avv.interrupt();
		}
		cancellaAggiorna(ape);
	}
	public Integer inviaTelecom(Contatto contatto, TelecomTemplate template, Boolean background) {
		if (template != null && ApplicationUtil.isTrue(template.getAttivo())) {
			Telecom telecom = pannelloDao.creaTelecomDaTemplate(template, contatto);
			telecom.setContatto(contatto);
			return comunicazioneDao.inviaTelecom(telecom, null, background);
		}
		return null;
	}
	public synchronized List<ProcessoReportTelecom> getProcessiTelecom() {
		List<ProcessoReportTelecom> processiList = new ArrayList<ProcessoReportTelecom>();
		if(ApplicationUtil.notEmpty(processiTelecom)) {
			Set<Campagna> lngList = processiTelecom.keySet();
			if(ApplicationUtil.notEmpty(lngList)) {
				Iterator<Campagna> lnt = lngList.iterator();
				while (lnt.hasNext()) {
					ContattoAvvioProcessoTelecom avv = processiTelecom.get(lnt.next());
					ProcessoReportTelecom pr = new ProcessoReportTelecom();
					pr.setAuto(avv.getAutoProcessTelecom());
					pr.setSospeso(avv.getCodaManagerContatti().isSospeso());
					pr.setCaricato(avv.getCodaManagerContatti().isCaricato());
					pr.setCodaContattiFalliti(avv.getCodaManagerContatti().getCodaContattiFalliti().getSize());
					pr.setCodaContattiTotali(avv.getCodaManagerContatti().getCodaContattiTotali().getSize());
					pr.setCodaContattiRiusciti(avv.getCodaManagerContatti().getCodaContattiRiusciti().getSize());
					processiList.add(pr);
				}
			}
		}
		return processiList;
	}
	@SuppressWarnings("unchecked")
	public List<AutoProcessTelecom> getAutoProcessTelecom() {
		DetachedCriteria critCE = DetachedCriteria.forClass(AutoProcessTelecom.class);
		return getList(critCE);
	}
	public void rimuoviContattoTotali(AutoProcessTelecom ape, Long id)  {
		ContattoAvvioProcessoTelecom avv = processiTelecom.get(ape.getCampagna());
		if(avv!=null) {
			avv.getCodaManagerContatti().getCodaContattiTotali().remove(id);
		}
	}
	public void rimuoviContattoFalliti(AutoProcessTelecom ape, Long id)  {
		ContattoAvvioProcessoTelecom avv = processiTelecom.get(ape.getCampagna());
		if(avv!=null) {
			avv.getCodaManagerContatti().getCodaContattiFalliti().remove(id);
		}
	}
	public void rimuoviContattoRiusciti(AutoProcessTelecom ape, Long id)  {
		ContattoAvvioProcessoTelecom avv = processiTelecom.get(ape.getCampagna());
		if(avv!=null) {
			avv.getCodaManagerContatti().getCodaContattiRiusciti().remove(id);
		}
	}
	public synchronized Contatto getContattoTelecom(Campagna campagna) {
		DetachedCriteria critCE = DetachedCriteria.forClass(Telecom.class);
		DetachedCriteria critC = critCE.createCriteria("contatto");
		critC.setProjection(Projections.distinct(Projections.property("id")));
		DetachedCriteria criteria = getCriteriaContatto(campagna, critC, 7);
		Contatto out =  pannelloDao.getEstrazioneContatto(criteria);
		if (out != null) {
			Chiamate.getContattiAttiviTelecom().put(out);
		} 
		return out;
	}
	public List<SelectItem> getTemplateTelecomSI() {
		DetachedCriteria criteria = DetachedCriteria.forClass(TelecomTemplate.class);
		List<GenericSI> lis = getGenericSI(criteria, "id", "nome");
		return impostazioneDao.getGeneric(lis, false);
	}
	public Telecom getTelecom(Long id){
		return (Telecom) get(Telecom.class, id);
	}
	public void eliminaTelecom(Telecom id) {
		cancellaAggiorna(id);
	}
	private ExtendedCriteria trovaCriteriaTelecom(TelecomCriteria ema) {
		if(ema != null)  ema = (TelecomCriteria) ema.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Telecom.class);
		DetachedCriteria criteriaCo = criteria.createCriteria("contatto");
		if (ApplicationUtil.isNotBlank(ema.getTesto())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, ema.getTesto(), criteria, "body", "link", "headers", "parameters");
		}
		if (ApplicationUtil.isNotBlank(ema.getRicerca_anagrafica())) {
			DetachedCriteria criteriaA = criteriaCo.createCriteria("anagrafica");
			ApplicationUtil.anagrafica(ema.getRicerca_anagraficaV(), criteriaA, ema.getRicerca_anagrafica());
		}
		if (ApplicationUtil.gtZero(ema.getCampagna())) {
			criteriaCo.add(Restrictions.eq("campagna.id", ema.getCampagna()));
		}
		if (ApplicationUtil.gtZero(ema.getCodice())) {
			criteria.add(Restrictions.eq("codice", ema.getCodice()));
		}
		ema.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, ema);
	}
	@SuppressWarnings("unchecked")
	public List<Telecom> trovaTelecom(TelecomCriteria ema,int start, int size) {
		ExtendedCriteria criteria = trovaCriteriaTelecom(ema);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int trovaTelecomSize(TelecomCriteria ema) {
		ExtendedCriteria criteria = trovaCriteriaTelecom(ema);
		return getListSize(criteria);
	}
	public List<SelectItem> getCampagneTelecomSI() {
		DetachedCriteria critCE = DetachedCriteria.forClass(AutoProcessTelecom.class);
		DetachedCriteria critC = critCE.createCriteria("campagna");
		critC.setProjection(Projections.distinct(Projections.property("id")));
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		impostazioneDao.subPropertyNotInS(criteria, critC, "id");
		List<CampagnaSI> lis = getCampagnaSI(criteria);
		return impostazioneDao.getCampagna(lis, false, false, false, null);
	}
	@SuppressWarnings("unchecked")
	public List<AutoProcessTelecom> getAutoProcessTelecomByCampagnaId(Long campagnaId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(AutoProcessTelecom.class);
		criteria.add(Restrictions.eq("campagna.id",campagnaId));
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<AutoProcessTelecom> getAutoProcessByTelecomId(Long id) {
		DetachedCriteria criteria = DetachedCriteria.forClass(AutoProcessTelecom.class);
		criteria.createCriteria("telecomTemplate").add(Restrictions.idEq(id));
		return getList(criteria);
	}
}