package it.callite.dao;
import java.util.ArrayList;
import java.util.List;

import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.MatchMode;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Projections;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsMap;
import it.callite.dao.generic.GenericDao;
import it.callite.message.Message;
import it.callite.model.Anagrafica;
import it.callite.model.Campagna;
import it.callite.model.Comune;
import it.callite.model.Provincia;
import it.callite.model.Utente;
import it.callite.model.sip.Musiconhold;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.item.UtenteSI;
@SuppressWarnings("serial")
@Repository("autocompleteDao")
public class AutocompleteDao extends GenericDao {
	
	@Autowired
	private UserDao userDao;
	@Autowired
	private ImpostazioneDao impostazioneDao;
	@Autowired
	private AutodialerDao autodialerDao;
	
	@SuppressWarnings("unchecked")
	public List<String> autocompleteRicercaCampagnaSupervisore(Object q) {
		Utente user = ApplicationUtil.getUsernameLoggato();
		String query = ApplicationUtil.trimToNull((String) q);
		if(user != null && query != null) {
			DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
			criteria.createCriteria("team").createCriteria("supervisore").add(Restrictions.idEq(user.getId()));
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, query, criteria, "nome");
			criteria.setProjection(Projections.distinct(Projections.property("nome")));
			ApplicationUtil.orderCL(criteria);
			List<String> etichette = getList(criteria);
			return etichette;
		}
		return null;
	}
	@SuppressWarnings("unchecked")
	public List<String> autocompleteRicercaCampagnaOperatore(Object q) {
		Utente user = ApplicationUtil.getUsernameLoggato();
		String query = ApplicationUtil.trimToNull((String) q);
		if(user != null && query != null) {
			DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
			criteria.createCriteria("team").createCriteria("user").add(Restrictions.idEq(user.getId()));
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, query, criteria, "nome");
			criteria.setProjection(Projections.distinct(Projections.property("nome")));
			ApplicationUtil.orderCL(criteria);
			List<String> etichette = getList(criteria);
			return etichette;
		}
		return null;
	}
	@SuppressWarnings("unchecked")
	public List<String> autocompleteRicercaCampagnaPartner(Object q) {
		Utente user = ApplicationUtil.getUsernameLoggato();
		String query = ApplicationUtil.trimToNull((String) q);
		if(user != null && query != null) {
			DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
			criteria.createCriteria("partner").add(Restrictions.idEq(user.getId()));
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, query, criteria, "nome");
			criteria.setProjection(Projections.distinct(Projections.property("nome")));
			ApplicationUtil.orderCL(criteria);
			List<String> etichette = getList(criteria);
			return etichette;
		}
		return null;
	}
	@SuppressWarnings("unchecked")
	public List<String> autocompleteRicercaCampagnaCoordinatore(Object q) {
		Utente user = ApplicationUtil.getUsernameLoggato();
		String query = ApplicationUtil.trimToNull((String) q);
		if(user != null && query != null) {
			DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
			criteria.createCriteria("zone").createCriteria("supervisori").createCriteria("user").add(Restrictions.idEq(user.getId()));
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, query, criteria, "nome");
			criteria.setProjection(Projections.distinct(Projections.property("nome")));
			ApplicationUtil.orderCL(criteria);
			List<String> etichette = getList(criteria);
			return etichette;
		}
		return null;
	}
	@SuppressWarnings("unchecked")
	public List<String> autocompleteRicercaCampagnaRete(Object q) {
		Integer max = userDao.getMaxLivelloRete();
		Utente user = ApplicationUtil.getUsernameLoggato();
		String query = ApplicationUtil.trimToNull((String) q);
		if(user != null && query != null) {
			DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
			DetachedCriteria criteriaAd = criteria.createCriteria("zone").createCriteria("agente");
			impostazioneDao.subCriteriaAgentiSottoReteByUser(user.getId(), 0, max, criteriaAd, "id");
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, query, criteria, "nome");
			criteria.setProjection(Projections.distinct(Projections.property("nome")));
			ApplicationUtil.orderCL(criteria);
			List<String> etichette = getList(criteria);
			return etichette;
		}
		return null;
	}
	@SuppressWarnings("unchecked")
	public List<String> autocompleteRicercaCampagnaAgente(Object q) {
		Utente user = ApplicationUtil.getUsernameLoggato();
		String query = ApplicationUtil.trimToNull((String) q);
		if(user != null && query != null) {
			DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
			criteria.createCriteria("zone").createCriteria("agente").createCriteria("user").add(Restrictions.idEq(user.getId()));
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, query, criteria, "nome");
			criteria.setProjection(Projections.distinct(Projections.property("nome")));
			ApplicationUtil.orderCL(criteria);
			List<String> etichette = getList(criteria);
			return etichette;
		}
		return null;
	}
	@SuppressWarnings("unchecked")
	public List<String> autocompleteRicercaCampagna(Object q) {
		String query = ApplicationUtil.trimToNull((String) q);
		if(query != null) {
			DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, query, criteria, "nome");
			criteria.setProjection(Projections.distinct(Projections.property("nome")));
			ApplicationUtil.orderCL(criteria);
			List<String> etichette = getList(criteria);
			return etichette;
		}
		return null;
	}
	@SuppressWarnings("unchecked")
	public List<String> autocompleteRicercaIndirizzoProvincia(Object q) {
		String query = ApplicationUtil.trimToNull((String) q);
		if(query != null) {
			DetachedCriteria criteria = DetachedCriteria.forClass(Provincia.class);
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, query, criteria, "codice");
			criteria.setProjection(Projections.distinct(Projections.property("codice")));
			criteria.addOrder(Order.asc("codice"));
			return getList(criteria);
		}
		return null;
	}
	@SuppressWarnings("unchecked")
	public List<String> autocompleteRicercaIndirizzoRegione(Object q) {
		String query = ApplicationUtil.trimToNull((String) q);
		if(query != null) {
			DetachedCriteria criteria = DetachedCriteria.forClass(Provincia.class);
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, query, criteria, "regione");
			criteria.setProjection(Projections.distinct(Projections.property("regione")));
			criteria.addOrder(Order.asc("regione"));
			return getList(criteria);
		} 
		return null;
	}
	@SuppressWarnings("unchecked")
	public List<String> autocompleteRicercaIndirizzoComune(Object q) {
		String query = ApplicationUtil.trimToNull((String) q);
		if(query != null) {
			DetachedCriteria criteria = DetachedCriteria.forClass(Comune.class);
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, query, criteria, "nome");
			criteria.setProjection(Projections.distinct(Projections.property("nome")));
			criteria.addOrder(Order.asc("nome"));
			return getList(criteria);
		}
		return null;
	}
	
	@SuppressWarnings("unchecked")
	public List<String> autocompleteRicercaCategoria(Object q) {
		String query = ApplicationUtil.trimToNull((String) q);
		if(query != null) {
			DetachedCriteria criteria = DetachedCriteria.forClass(Anagrafica.class);
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, query, criteria, "settore");
			criteria.setProjection(Projections.distinct(Projections.property("settore")));
			criteria.addOrder(Order.asc("settore"));
			List<String> etichette = getList(criteria);
			return etichette;
		}
		return null;
	}
	public List<String> autocompleteInputStrada(Object q) {
		List<String> sis = new ArrayList<String>();
		for(Integer c : ConstantsMap.getTipoStrade()) {
			sis.add(Message.addCatalogText("TIPO_STRADA_" + c));
		}
		List<String> ss = new ArrayList<String>();
		for(String s : sis) {
			String query = ApplicationUtil.upperCaseNull((String) q);
			if(query != null && s.contains(query)) {
				ss.add(s);
			}
		}
		return ss;
	}
	
	@SuppressWarnings("unchecked")
	public List<Comune> autocompleteInputNascita(Object q) {
		String query = ApplicationUtil.trimToNull((String) q);
		if(query != null) {
			DetachedCriteria criteria = DetachedCriteria.forClass(Comune.class);
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, query, criteria, MatchMode.ANYWHERE, "nome");
			criteria.addOrder(Order.asc("nome"));
			return getList(criteria);
		}
		return null;
	}
	
	
	@SuppressWarnings("unchecked")
	public List<String> autocompleteInputCategoria(Object q) {
		String query = ApplicationUtil.trimToNull((String) q);
		if(query != null) {
			DetachedCriteria criteria = DetachedCriteria.forClass(Anagrafica.class);
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, query, criteria, MatchMode.ANYWHERE, "settore");
			criteria.setProjection(Projections.distinct(Projections.property("settore")));
			criteria.addOrder(Order.asc("settore"));
			List<String> etichette = getList(criteria);
			return etichette;
		}
		return null;
	}
	
	@SuppressWarnings("unchecked")
	public List<String> autocompleteMusic(Object q) {
		String query = ApplicationUtil.trimToNull((String) q);
		if(query != null) {
			DetachedCriteria criteria = DetachedCriteria.forClass(Musiconhold.class);
			ApplicationUtil.valori(query, criteria, MatchMode.ANYWHERE, "name");
			criteria.setProjection(Projections.distinct(Projections.property("name")));
			criteria.addOrder(Order.asc("name"));
			List<String> etichette = getList(criteria);
			return etichette;
		}
		return null;
	}
	
	public List<UtenteSI> autocompleteUser(Object q) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		String query = ApplicationUtil.trimToNull((String) q);
		if(query == null) {
			return null;
		}
		criteria.add(Restrictions.eq("enabled", Boolean.TRUE));
		criteria.add(Restrictions.ne("disattivaMessaggistica", 2));
		ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, query, criteria, MatchMode.ANYWHERE, "username", "cognome", "nome", "email");
		criteria.addOrder(Order.asc("username"));
		return getUtenteSI(criteria);
	}
	
	public List<String> autocompleteSuono(Object q) {
		String query = ApplicationUtil.trimToNull((String) q);
		List<String> etichette = autodialerDao.listaSuoni(query);
		return etichette;
	}
}