package it.callite.dao;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.faces.model.SelectItem;

import org.apache.commons.net.ftp.FTPClient;
import org.apache.commons.net.ftp.FTPFile;
import org.hibernate.FetchMode;
import org.hibernate.Session;
import org.hibernate.Transaction;
import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Projections;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.DisposableBean;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import it.asterisk.fastagi.AgiChannel;
import it.asterisk.fastagi.AgiException;
import it.asterisk.live.AsteriskServer;
import it.asterisk.live.AsteriskServerListener;
import it.asterisk.live.DefaultAsteriskServer;
import it.asterisk.manager.AuthenticationFailedException;
import it.asterisk.manager.ManagerConnection;
import it.asterisk.manager.ManagerConnectionFactory;
import it.asterisk.manager.TimeoutException;
import it.asterisk.manager.action.CommandAction;
import it.asterisk.manager.action.MessageSendAction;
import it.asterisk.manager.action.ModuleLoadAction;
import it.asterisk.manager.action.OriginateAction;
import it.asterisk.manager.event.AbstractQueueMemberEvent;
import it.asterisk.manager.event.BridgeEnterEvent;
import it.asterisk.manager.event.BridgeLeaveEvent;
import it.asterisk.manager.event.ChanSpyStartEvent;
import it.asterisk.manager.event.ChanSpyStopEvent;
import it.asterisk.manager.event.DialBeginEvent;
import it.asterisk.manager.event.DialEndEvent;
import it.asterisk.manager.event.HangupEvent;
import it.asterisk.manager.event.HangupRequestEvent;
import it.asterisk.manager.event.ManagerEvent;
import it.asterisk.manager.event.NewStateEvent;
import it.asterisk.manager.event.OriginateResponseEvent;
import it.asterisk.manager.event.PeerStatusEvent;
import it.asterisk.manager.event.QueueCallerJoinEvent;
import it.asterisk.manager.event.QueueEvent;
import it.asterisk.manager.event.QueueMemberAddedEvent;
import it.asterisk.manager.event.QueueMemberPauseEvent;
import it.asterisk.manager.event.QueueMemberStatusEvent;
import it.asterisk.manager.event.SoftHangupRequestEvent;
import it.asterisk.manager.response.CommandResponse;
import it.asterisk.manager.response.ManagerError;
import it.asterisk.manager.response.ManagerResponse;
import it.callite.action.DownloadBean;
import it.callite.autodialer.autodial.AgiServer;
import it.callite.autodialer.autodial.AnagraficaServer;
import it.callite.autodialer.autodial.IVRServer;
import it.callite.autodialer.autodial.InboundServer;
import it.callite.autodialer.autodial.RispostaServer;
import it.callite.autodialer.autodial.SondaggioServer;
import it.callite.autodialer.controllo.CodaManager;
import it.callite.autodialer.controllo.CodaStatic;
import it.callite.autodialer.pubblico.PubblicoAvvioProcesso;
import it.callite.autodialer.response.ManagerEventResponse;
import it.callite.autodialer.response.ServerEventResponse;
import it.callite.chiamate.Chiamate;
import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.constants.ConstantsMap;
import it.callite.constants.config.DefaultsAsterisk;
import it.callite.criteria.AutoContattoCriteria;
import it.callite.criteria.ConfigurazioneCriteria;
import it.callite.criteria.FileCriteria;
import it.callite.criteria.base.ExtendedCriteria;
import it.callite.dao.generic.GenericDao;
import it.callite.log.ErrorLog;
import it.callite.model.AutoContatto;
import it.callite.model.Autodialer;
import it.callite.model.Campagna;
import it.callite.model.Contatto;
import it.callite.model.DatoAggiuntivoAnagrafica;
import it.callite.model.EsitoChiamata;
import it.callite.model.Inbound;
import it.callite.model.Opposizione;
import it.callite.model.Risposta;
import it.callite.model.Utente;
import it.callite.model.config.ConfigAsterisk;
import it.callite.model.report.AutodialerContatti;
import it.callite.model.report.AutodialerReport;
import it.callite.model.report.AutodialerReportList;
import it.callite.model.report.AutodialerStatistiche;
import it.callite.model.report.MonitorOperatore;
import it.callite.model.sip.SipBuddies;
import it.callite.utility.AiutoCampagna;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.AutodialerList;
import it.callite.utility.AutodialerMap;
import it.callite.utility.CallUtil;
import it.callite.utility.DatoAggiuntivo;
import it.callite.utility.FileUpload;
import it.callite.utility.IVRUtil;
import it.callite.utility.InfoQueue;
import it.callite.utility.InfoSpia;
import it.callite.utility.InfoTelefono;
import it.callite.utility.InfoUtente;
import it.callite.utility.IntervalloInbound;
import it.callite.utility.LoggerContatto;
import it.callite.utility.MyFile;
import it.callite.utility.Parameter;
import it.callite.utility.ParameterAutodialer;
import it.callite.utility.QueueChiamata;
import it.callite.utility.QueueUtente;
import it.callite.utility.StatoServer;
import it.callite.utility.UtilAutodialer;
import it.callite.utility.UtilFTP;
import it.callite.utility.item.AutodialerSI;
import it.callite.utility.item.CampagnaSI;
import it.callite.utility.sincronizzazione.InfoContatto;
import it.callite.utility.thread.ApriContattoThread;
import it.callite.utility.thread.SalvaAutoContattoThread;
import it.callite.utility.thread.SalvaContattoThread;
@SuppressWarnings("serial")
@Repository("autodialerDao")
public class AutodialerDao extends GenericDao implements InitializingBean, DisposableBean {
	@Autowired
	private CampagnaDao campagnaDao;
	@Autowired
	private PannelloDao pannelloDao;
	@Autowired
	private OperazioneDao operazioneDao;
	@Autowired
	private ImpostazioneDao impostazioneDao;
	@Autowired
	private ContattoDao contattoDao;

	private Map<Campagna, PubblicoAvvioProcesso> processi = new HashMap<Campagna, PubblicoAvvioProcesso>();
	private ManagerConnection managerConnection;
	private AsteriskServer asteriskServer;
	private AgiServer agiServer;
	private InboundServer inbServer;
	private SondaggioServer sonServer;
	private RispostaServer ingServer;
	private IVRServer ivrServer;
	private AnagraficaServer angServer;
	private ManagerEventResponse manager;
	private ServerEventResponse server;


	public Boolean getSip() {
		return ApplicationUtil.isTrue(impostazioneDao.telefonoProtocol());
	}
	
	public String caller(String key) {
		if(key != null) {
			String caller = ApplicationUtil.channel(key);
			if(caller != null) {
				if(ApplicationUtil.isInterno(caller, impostazioneDao.validatore_interno())){
					return caller;
				}
			}
		}
		return null;
	}
	
	public Map<Campagna, CodaManager> getCodaManagerMap() {
		return Chiamate.getCodaProcessi().getData();
	}

	public void ricarica() {
		List<String> keyList = new ArrayList<String>(DefaultsAsterisk.DEFAULTS.keySet());
		if(ApplicationUtil.notEmpty(keyList)) {
			Iterator<String> iter = keyList.iterator();
			while (iter.hasNext()) {
				String key = (String) iter.next();
				Boolean valore = getValore(key);
				sincronizza(key, valore);
			}
		}
	}
	public synchronized void aggiornaCampagna(Campagna campagna) {
		if(!processi.containsKey(campagna)) {
			Integer numCanali = getNumCanali();
			PubblicoAvvioProcesso avv = new PubblicoAvvioProcesso(this, campagna, numCanali);
			avv.start();
			processi.put(campagna, avv);
		}
	}
	public synchronized void addUserCampagna(Utente user, Campagna campagna, Date data) {
		if(data!=null) {
			PubblicoAvvioProcesso avv = processi.get(campagna);
			if(avv == null) {
				Integer numCanali = getNumCanali();
				avv = new PubblicoAvvioProcesso(this, campagna, numCanali);
				avv.start();
				processi.put(campagna, avv);
			} 
			Integer priorita = Chiamate.getCodaOperatori().hasPrecedenza(user, campagna);
			avv.getCodaManager().setStato(true);
			data = ApplicationUtil.aggiustaDataT(data, false);
			InfoUtente utente = new InfoUtente(user, data.getTime(), true);
			//CodaStatic.getCodaTeam().rimuoviUtenteT(utente);
			CodaStatic.getCodaTeam().addUser(campagna, utente, priorita);
			Chiamate.getCodaInterni().stato(user.getInternoString(), false);
		}
	}
	public synchronized void addUserContatto(Utente user, Contatto out, Date data, Integer numCanali) {
		if(out != null && out.isPubblica() && !out.getArchiviato() && !Chiamate.getCodaAutodialer().contains(out.getId())) {
			if(data == null) {
				data = ApplicationUtil.date();
			}
			data = ApplicationUtil.aggiustaDataT(data, true);
			Campagna campagna = getCampagna(out.getCampagna());
			PubblicoAvvioProcesso avv = processi.get(campagna);
			if(avv == null) {
				avv = new PubblicoAvvioProcesso(this, campagna, numCanali);
				if(avv.getCodaManager()!=null && avv.getCodaManager().getCampagna().getAutodialer() != null) {
					avv.start();
					processi.put(campagna, avv);
				}
			}
			if(avv.getCodaManager()!=null && avv.getCodaManager().getCampagna().getAutodialer() != null) {
				LoggerContatto l = new LoggerContatto(impostazioneDao.getLog());
				long time = System.currentTimeMillis();
				Long diff = (data.getTime() - time) / (ConstantsGlobal.t_secondi);
				Chiamate.getCodaAutodialer().put(out);
				AutoContatto auto = new AutoContatto(out, ConstantsMap.CHIAMATA_SCHEDULATA, out.getTelefono_chiamata(), false);
				auto.impostaUserfield();
				out.setTelefono(auto.getTelefono());
				auto.incRichiamata(diff.intValue());
				out.setAutodialer(true);
				out.setInbound(false);
				auto.setAutodialer(avv.getCodaManager().getCampagna().getAutodialer());
				avv.getCodaManager().getChiamateSchedulate().put(auto);
				l.log("caricato autodialer", user, out);
			}
		}
	}
	public synchronized void removeUserCampagna(Utente user, Campagna campagna) {
		//InfoUtente utente = new InfoUtente(user);
		//CodaStatic.getCodaTeam().rimuoviUtente(campagna, utente);
		CodaStatic.getCodaTeam().remove(campagna);
		CodaManager codaManager = Chiamate.getCodaProcessi().get(campagna);
		if(codaManager != null) {
			codaManager.setStato(false);
		}
	}
	public synchronized void esitoOperatore(AutoContatto autoContatto, Integer stato, Integer cause) {	
		autoContatto.setStato(stato);
		autoContatto.addCause(cause);
		Autodialer auto = autoContatto.getAutodialer();
		if(auto!=null) {
			esitaContatto(autoContatto, auto.getEsitoHangup(), false);
		}
	}
	public synchronized void gestisciContatto(AutoContatto autoContatto, int frec, int max, Integer stato, EsitoChiamata esito, String causa) {	
		autoContatto.setStato(stato);
		autoContatto.addCause(causa);
		if(autoContatto.getTentativi() <= max) {
			autoContatto.incRichiamata(frec);
			//autoContatto.reset();
			Contatto contatto = autoContatto.getContatto();
			CodaManager codaManager = Chiamate.getCodaProcessi().get(contatto.getCampagna());
			if(codaManager != null) {
				codaManager.getChiamateSchedulate().put(autoContatto);
			}
		} else {
			esitaContatto(autoContatto, esito, false);
		}
	}
	public synchronized InfoUtente getInternoDaChiamare1(Contatto contatto) {
		Campagna c = contatto.getCampagna();
		if(c!= null) {
			Map<Utente, MonitorOperatore> operatori = Chiamate.getCodaOperatori().getData();
			List<InfoUtente> users = CodaStatic.getCodaTeam().get(c);
			if(ApplicationUtil.notEmpty(users) && ApplicationUtil.notEmpty(operatori)) {
				Iterator<InfoUtente> iter = users.iterator();
				while (iter.hasNext()) {
					InfoUtente iu = iter.next();
					Utente utente = iu.getOperatore();
					if(utente != null && utente.isLocale()) {
						MonitorOperatore to = operatori.get(utente);
						if(to !=null && ApplicationUtil.equals(c, to.getCampagna()) && to.isInAttesa() && ApplicationUtil.eq(to.getLibero(), 1) && utente.getLoggato() && utente.getPronto() && iu.isReady()) {
							iu.setReady(false);
							return iu;
						}
					}
				}
			}
		}
		return null;
	}
	public synchronized InfoUtente getInternoDaChiamare2(Contatto contatto) {
		Campagna c = contatto.getCampagna();
		if(c!= null) {
			Map<Utente, MonitorOperatore> operatori = Chiamate.getCodaOperatori().getData();
			List<InfoUtente> users = CodaStatic.getCodaTeam().get(c);
			if(ApplicationUtil.notEmpty(users) && ApplicationUtil.notEmpty(operatori)) {
				Iterator<InfoUtente> iter = users.iterator();
				while (iter.hasNext()) {
					InfoUtente iu = iter.next();
					Utente utente = iu.getOperatore();
					if(utente != null && utente.isLocale()) {
						MonitorOperatore to = operatori.get(utente);
						List<Long> ids = CodaStatic.getCodaLista().getDataL(contatto.getCampagna(), utente);
						if(ApplicationUtil.empty(ids) || (ApplicationUtil.notEmpty(ids) && ids.contains(contatto.getLista().getId()))) {
							if(to !=null && ApplicationUtil.equals(c, to.getCampagna()) && to.isInAttesa() && ApplicationUtil.eq(to.getLibero(), 1) && utente.getLoggato() && utente.getPronto() && iu.isReady()) {
								iu.setReady(false);
								return iu;
							}
						}
					}
				}
			}
		}
		return null;
	}
	public synchronized void esitaContatto(AutoContatto auto, EsitoChiamata esito, boolean remove) {
		Integer salva = auto.getAutodialer().getSave_autodialer();
		Contatto contatto = auto.getContatto();
		if(remove) {
			CodaManager codaManager = Chiamate.getCodaProcessi().get(contatto.getCampagna());
			if(codaManager != null) {
				codaManager.getChiamateInCorso().remove(contatto.getId());
				codaManager.getChiamateInAttesa().remove(contatto.getId());
			}
			CodaStatic.getChiamateInCorso().remove(contatto.getId());
		}
		if(esito != null && ApplicationUtil.eq(salva, 2)) {
			boolean conta = false;
			if(ApplicationUtil.isTrue(esito.getRichiamare()) && ApplicationUtil.gtZero(esito.getMax_tentativi())) {
				Integer count = countEsitoChiamataInAuto(contatto.getId(), esito.getConta_tentativi());
				if(!ApplicationUtil.minore(count, esito.getMax_tentativi())) {
					conta = true;
				}
			}
			if(conta) {
				EsitoChiamata e = pannelloDao.getFullEsitoChiamataById(esito.getId());
				salvaContatto(auto, e.getEsito_collegato() != null ? e.getEsito_collegato() : e);
			} else {
				salvaContatto(auto, esito);
			}
		} else {
			salvaContatto(auto, esito);
		}
	}
	public synchronized void salvaContatto(AutoContatto auto, EsitoChiamata esito) {
		Contatto contatto = auto.getContatto();
		if (contatto != null && esito != null) {
			if(Chiamate.getCodaAutodialer().contains(contatto.getId())) {
				if(esito!=null) {
					pannelloDao.esitaAutodialer(true, auto.getAutodialer().getSave_storia(), contatto, null, esito);
				}
				Chiamate.getCodaAutodialer().remove(contatto.getId());
				Integer salva = auto.getAutodialer().getSave_autodialer();
				if(ApplicationUtil.eq(salva, 1) || ApplicationUtil.eq(salva, 2)){
					new SalvaAutoContattoThread(this, auto).start();
				}
				Chiamate.getCodaEsiti().add(null, contatto);
			}
		}
	}
	
	public String getHome() {
		return impostazioneDao.getHome();
	}
	private void createManager(){
		Integer ami_port = 0;
		UtilAutodialer ua = impostazioneDao.getUtilAutodialer();
		String ami_ip = ua.getHost();
		try {
			ami_port = ua.getPort();
		} catch (Exception e) {
		}
		if(ApplicationUtil.gtZero(ami_port) && ami_ip != null) {
			String ami_username = ua.getUsername();
			String ami_password = ua.getPassword();
			ManagerConnectionFactory factory = new ManagerConnectionFactory(ami_ip,ami_port,ami_username,ami_password);
			managerConnection = factory.createManagerConnection();
		}
	}
	private void createServer(){
		UtilAutodialer ua = impostazioneDao.getUtilAutodialer();
		String ami_ip = ua.getHost();
		if(ami_ip != null) {
			String ami_username = ua.getUsername();
			String ami_password = ua.getPassword();
			asteriskServer = new DefaultAsteriskServer(ami_ip, ami_username, ami_password);
		}
	}
	public String initialize() {
		createManager();
		if(ConstantsGlobal.agiServer) {
			createServer();
		}
		return login();
	}

	public void afterPropertiesSet() throws Exception {
		this.manager = new ManagerEventResponse(this);
		if(ConstantsGlobal.agiServer) {
			this.server = new ServerEventResponse(this);
		}
		initialize();
		agiStart();
		List<String> keyList = new ArrayList<String>(DefaultsAsterisk.DEFAULTS.keySet());
		eliminaNotInConfig(keyList);
		sincronizzaInterno();
	}
	
	public Map<String, InfoQueue> getCodaQueue() {
		return Chiamate.getCodaQueue().getData();
	}
	
	public List<InfoTelefono> getCodaTelefono() {
		return Chiamate.getCodaTelefono().list();
	}
	public void resetInterni() {
		Chiamate.getCodaTelefono().clear();
	}

	public void destroy() {
		agiFinalize();
		logoff();
	}

	public Boolean getAmiLogged() {
		return (managerConnection != null && managerConnection.getState().toString().equalsIgnoreCase("CONNECTED"));
	}
	public String logoff() {
		removeEventListener(manager);
		if(ConstantsGlobal.agiServer) {
			removeServerListener(server);
		}
		if(getAmiLogged()) {
			managerConnection.logoff();
		}
		return null;
	}
	public String login() {
		logoff();
		if(managerConnection != null) {
			try {
				managerConnection.login();
				addEventListener(manager);
				if(ConstantsGlobal.agiServer) {
					addServerListener(server);
				}
			} catch (IllegalStateException e) {
				return ConstantsLanguage.LOGIN_EXCEPTION.stampa();
			} catch (IOException e) {
				return ConstantsLanguage.LOGIN_EXCEPTION.stampa();
			} catch (AuthenticationFailedException e) {
				return ConstantsLanguage.LOGIN_FAILED.stampa();
			} catch (TimeoutException e) {
				return ConstantsLanguage.LOGIN_TIMEOUT.stampa();
			}
			ricarica();
			return ConstantsLanguage.LOGIN_OK.stampa();
		} else {
			return ConstantsLanguage.LOGIN_KO.stampa();
		}
	}
	public Boolean getAgiLogged() {
		try {
			return agiServer.isAlive() && inbServer.isAlive() && sonServer.isAlive() && ingServer.isAlive() && ivrServer.isAlive() && angServer.isAlive();
		} catch (Exception e) {
			return false;
		}
	}
	public void agiFinalize() {
		agiServer.spegni();
		inbServer.spegni();
		sonServer.spegni();
		ingServer.spegni();
		ivrServer.spegni();
		angServer.spegni();
	}
	
	public void agiStart() {
		agiServer = new AgiServer(this);
		inbServer = new InboundServer(this);
		sonServer = new SondaggioServer(this);
		ingServer = new RispostaServer(this);
		ivrServer = new IVRServer(this);
		angServer = new AnagraficaServer(this);
		Integer numCanali = getNumCanali();
		if(ApplicationUtil.gtZero(numCanali)) {
			agiServer.setPoolSize(numCanali);
			inbServer.setPoolSize(numCanali);
			sonServer.setPoolSize(numCanali);
			ingServer.setPoolSize(numCanali);
			ivrServer.setPoolSize(numCanali);
			angServer.setPoolSize(numCanali);
		}
		agiServer.start();
		inbServer.start();
		sonServer.start();
		ingServer.start();
		ivrServer.start();
		angServer.start();
	}
	public Contatto getContatto(Campagna campagna, Utente utente) {
		LoggerContatto l = new LoggerContatto(impostazioneDao.getLog());
		DetachedCriteria criteria = pannelloDao.getCriteriaContatto(campagna.getAutodialer().getTipologia(), campagna, campagna.getAutodialer().getNuovo(), utente);
		Contatto out = pannelloDao.getEstrazioneContatto(criteria);
		if(out != null) {
			Chiamate.getCodaAutodialer().put(out);
		}
		l.log("caricato autodialer", null, out);
		return out;
	}
	public List<Contatto> getContatto(Campagna campagna, int size, Utente utente) {
		LoggerContatto l = new LoggerContatto(impostazioneDao.getLog());
		DetachedCriteria criteria = pannelloDao.getCriteriaContatto(campagna.getAutodialer().getTipologia(), campagna, campagna.getAutodialer().getNuovo(), utente);
		List<Contatto> contatti = pannelloDao.getEstrazioneContatto(criteria, size);
		if(ApplicationUtil.notEmpty(contatti)) {
			Iterator<Contatto> cIt = contatti.iterator();
			while (cIt.hasNext()) {
				Contatto out = (Contatto) cIt.next();
				if(out != null) {
					Chiamate.getCodaAutodialer().put(out);
				}
				l.log("caricato autodialer", null, out);
			}
		}
		return contatti;
	}
	public Campagna getCampagna(Campagna campagna) {
		DetachedCriteria crit = DetachedCriteria.forClass(Campagna.class);
		crit.add(Restrictions.idEq(campagna.getId()));
		crit.setFetchMode("autodialer", FetchMode.JOIN);
		return (Campagna) getUniqueResult(crit);
	}

	public boolean isOpposizione(String opposizione) {
		boolean t = true;
		if(ApplicationUtil.isNotEmpty(opposizione)) {
			DetachedCriteria criteria = DetachedCriteria.forClass(Opposizione.class);
			criteria.add(Restrictions.isNull("campagna.id"));
			criteria.add(Restrictions.eq("opposizione", opposizione));
			if(t) {
				int c = getListSize(criteria);
				return c > 0;
			} else {
				criteria.setProjection(Projections.distinct(Projections.property("id")));
				return ApplicationUtil.gtZero((Long) getUniqueResult(criteria));
			}
		}
		return false;
	}
	
	public int countEsitoChiamataInAuto(Long contattoId, boolean conta_tentativi) {
		DetachedCriteria criteria = DetachedCriteria.forClass(AutoContatto.class);
		criteria.add(Restrictions.eq("contatto.id",contattoId));
		return getListSize(criteria);
	}
	public synchronized void azzeraCampagna(Campagna campagna) {
		CodaManager codaManager = Chiamate.getCodaProcessi().get(campagna);
		if(codaManager != null) {
			codaManager.reset();
		}
		Chiamate.getCodaAutodialer().remove(campagna);
		CodaStatic.getChiamateInCorso().remove(campagna);
	}
	public synchronized void azzeraCampagne() {
		Set<Campagna> lngList = Chiamate.getCodaProcessi().keySet();
		if(ApplicationUtil.notEmpty(lngList)) {
			Iterator<Campagna> lnt = lngList.iterator();
			while (lnt.hasNext()) {
				Campagna campagna = lnt.next();
				CodaManager codaManager = Chiamate.getCodaProcessi().get(campagna);
				if(codaManager != null) {
					codaManager.reset();
				}
			}
		}
		Chiamate.getCodaAutodialer().clear();
		CodaStatic.getChiamateInCorso().clear();
	}
	public synchronized void puliziaCampagne(boolean p) {
		Set<Campagna> codaList = Chiamate.getCodaProcessi().keySet();
		if(ApplicationUtil.notEmpty(codaList)) {
			Iterator<Campagna> lnt = codaList.iterator();
			while (lnt.hasNext()) {
				Campagna campagna = lnt.next();
				CodaManager codaManager = Chiamate.getCodaProcessi().get(campagna);
				if(codaManager != null) {
					codaManager.reset();
				}
			}
		}
		Chiamate.pulizia(p);
		CodaStatic.pulizia();
		if(p) {
			puliziaProcessi();
		}
	}


	private synchronized void puliziaProcessi() {
		Set<Campagna> prcList = processi.keySet();
		if(ApplicationUtil.notEmpty(prcList)) {
			Iterator<Campagna> lnt = prcList.iterator();
			while (lnt.hasNext()) {
				Campagna campagna = lnt.next();
				PubblicoAvvioProcesso avv = processi.get(campagna);
				if(avv != null) {
					avv.interrupt();
				}
			}
		}
		processi.clear();
	}

	public synchronized void sospendiCampagne() {
		Set<Campagna> lngList = Chiamate.getCodaProcessi().keySet();
		if(ApplicationUtil.notEmpty(lngList)) {
			Iterator<Campagna> lnt = lngList.iterator();
			while (lnt.hasNext()) {
				Campagna campagna = lnt.next();
				CodaManager codaManager = Chiamate.getCodaProcessi().get(campagna);
				if(codaManager != null) {
					codaManager.setAvviato(false);
				}
			}
		}
	}
	public synchronized void avviaCampagne() {
		Set<Campagna> lngList = Chiamate.getCodaProcessi().keySet();
		if(ApplicationUtil.notEmpty(lngList)) {
			Iterator<Campagna> lnt = lngList.iterator();
			while (lnt.hasNext()) {
				Campagna campagna = lnt.next();
				CodaManager codaManager = Chiamate.getCodaProcessi().get(campagna);
				if(codaManager != null) {
					codaManager.setAvviato(true);
				}
			}
		}
	}
	private synchronized void ricaricaCampagne(Utente user) {
		Set<Campagna> lngList = Chiamate.getCodaProcessi().keySet();
		if(ApplicationUtil.notEmpty(lngList)) {
			List<Long> camps = campagnaDao.prendiIdCampagnaUser(user);
			Iterator<Campagna> lnt = lngList.iterator();
			while (lnt.hasNext()) {
				Campagna campagna = lnt.next();
				if(camps.contains(campagna.getId())) {
					ricaricaCampagna(campagna);
				}
			}
		}
	}
	private synchronized void ricaricaCampagne(List<Campagna> campagne) {
		if(ApplicationUtil.notEmpty(campagne)) {
			Iterator<Campagna> lnt = campagne.iterator();
			while (lnt.hasNext()) {
				Campagna campagna = lnt.next();
				ricaricaCampagna(campagna);
			}
		}
	}
	private synchronized void ricaricaCampagne() {
		Set<Campagna> lngList = Chiamate.getCodaProcessi().keySet();
		if(ApplicationUtil.notEmpty(lngList)) {
			Iterator<Campagna> lnt = lngList.iterator();
			while (lnt.hasNext()) {
				Campagna campagna = lnt.next();
				ricaricaCampagna(campagna);
			}
		}
	}
	private synchronized void ricaricaCampagna(Campagna campagna) {
		CodaManager codaManager = Chiamate.getCodaProcessi().get(campagna);
		if(codaManager != null) {
			codaManager.setRicarica(true);
		}
	}
	public void ricaricaAutodialer() {
		Integer numCanali = getNumCanali();
		if(ApplicationUtil.gtZero(numCanali)) {
			agiServer.setPoolSize(numCanali);
		}
		ricaricaCampagne();
	}
	public void ricaricaAutodialer(Utente user) {
		Integer numCanali = getNumCanali();
		if(ApplicationUtil.gtZero(numCanali)) {
			agiServer.setPoolSize(numCanali);
		}
		ricaricaCampagne(user);
	}
	public void ricaricaAutodialer(SipBuddies trunk) {
		List<Campagna> campagne = getCampagneByTrunk(trunk.getId());
		Integer numCanali = getNumCanali();
		if(ApplicationUtil.gtZero(numCanali)) {
			agiServer.setPoolSize(numCanali);
		}
		ricaricaCampagne(campagne);
	}
	public void ricaricaAutodialer(Autodialer auto) {
		List<Campagna> campagne = getCampagneByAutoId(auto.getId());
		Integer numCanali = getNumCanali();
		if(ApplicationUtil.gtZero(numCanali)) {
			agiServer.setPoolSize(numCanali);
		}
		ricaricaCampagne(campagne);
	}
	public void ricaricaAutodialer(Campagna campagna) {
		Integer numCanali = getNumCanali();
		if(ApplicationUtil.gtZero(numCanali)) {
			agiServer.setPoolSize(numCanali);
		}
		ricaricaCampagna(campagna);
	}
	public synchronized void sospendi(Campagna campagna) {
		CodaManager codaManager = Chiamate.getCodaProcessi().get(campagna);
		if(codaManager != null) {
			codaManager.setAvviato(false);
		}
	}
	public synchronized AutodialerReport contaContatti(Campagna campagna) {
		AutodialerReport ad = new AutodialerReport();
		if(campagna != null) {
			CodaManager codaManager = Chiamate.getCodaProcessi().get(campagna);
			if(codaManager != null) {
				codaManager.updateReport(ad, codaManager);
				ad.setCampagna(campagna);
				ad.setSospeso(codaManager.isSospeso());
				ad.setAvviato(codaManager.isAvviato());
				ad.setCaricato(codaManager.isCaricato());
				ad.setEsaurito(codaManager.isEsaurito());
				ad.setStarted(codaManager.isStarted());
				ad.setFinito(codaManager.isFinito());

				ad.setCriterio(codaManager.getCriterio());
				ad.setCorrezione(codaManager.getCorrezione());
				ad.setChiamateAttive(codaManager.getChiamateAttive().getSize());
				ad.setChiamateAttiveMax(codaManager.getChiamateAttive().getCapacity());
				ad.setChiamateInCorso(codaManager.getChiamateInCorso().getSize());
				ad.setChiamateInCorsoMax(codaManager.getChiamateInCorso().getCapacity());
				ad.setChiamateInAttesa(codaManager.getChiamateInAttesa().getSize());
				ad.setChiamateSchedulate(codaManager.getChiamateSchedulate().getSize());
				ad.setChiamateAutodialer(Chiamate.getCodaAutodialer().get(campagna).size());
				ad.setCodaTeamCampagna(CodaStatic.getCodaTeam().get(campagna).size());
			}
		}
		return ad;
	}
	public synchronized List<AutodialerReport> getProcessiReport() {
		List<AutodialerReport> processiList = new ArrayList<AutodialerReport>();
		Set<Campagna> lngList = Chiamate.getCodaProcessi().keySet();
		if(ApplicationUtil.notEmpty(lngList)) {
			Iterator<Campagna> lnt = lngList.iterator();
			while (lnt.hasNext()) {
				Campagna campagna = lnt.next();
				if(campagna != null) {
					CodaManager codaManager = Chiamate.getCodaProcessi().get(campagna);
					if(codaManager != null) {
						AutodialerReport pr = new AutodialerReport();
						codaManager.updateReport(pr, codaManager);
						pr.setCampagna(campagna);
						pr.setSospeso(codaManager.isSospeso());
						pr.setAvviato(codaManager.isAvviato());
						pr.setCaricato(codaManager.isCaricato());
						pr.setEsaurito(codaManager.isEsaurito());
						pr.setStarted(codaManager.isStarted());
						pr.setFinito(codaManager.isFinito());
						pr.setCriterio(codaManager.getCriterio());
						pr.setCorrezione(codaManager.getCorrezione());
						processiList.add(pr);
					}
				}
			}
		}
		return processiList;
	}
	public synchronized void incProcesso(AutodialerReport a) {
		CodaManager codaManager = Chiamate.getCodaProcessi().get(a.getCampagna());
		if(codaManager != null) {
			codaManager.incCorrezione();
		}
	}
	public synchronized void decProcesso(AutodialerReport a) {
		CodaManager codaManager = Chiamate.getCodaProcessi().get(a.getCampagna());
		if(codaManager != null) {
			codaManager.decCorrezione();
		}
	}
	public synchronized void avviaCampagne(Utente user) {
		Set<Campagna> lngList = Chiamate.getCodaProcessi().keySet();
		if(ApplicationUtil.notEmpty(lngList)) {
			List<Long> camps = campagnaDao.prendiIdCampagnaUser(user);
			Iterator<Campagna> lnt = lngList.iterator();
			while (lnt.hasNext()) {
				Campagna campagna = lnt.next();
				CodaManager codaManager = Chiamate.getCodaProcessi().get(campagna);
				if(codaManager != null && camps.contains(campagna.getId())) {
					codaManager.setAvviato(true);
				}
			}
		}
	}
	public synchronized void sospendiCampagne(Utente user) {
		Set<Campagna> lngList = Chiamate.getCodaProcessi().keySet();
		if(ApplicationUtil.notEmpty(lngList)) {
			List<Long> camps = campagnaDao.prendiIdCampagnaUser(user);
			Iterator<Campagna> lnt = lngList.iterator();
			while (lnt.hasNext()) {
				Campagna campagna = lnt.next();
				CodaManager codaManager = Chiamate.getCodaProcessi().get(campagna);
				if(codaManager != null && camps.contains(campagna.getId())) {
					codaManager.setAvviato(false);
				}
			}
		}
	}
	public synchronized void azzeraCampagne(Utente user) {
		Set<Campagna> lngList = Chiamate.getCodaProcessi().keySet();
		if(ApplicationUtil.notEmpty(lngList)) {
			List<Long> camps = campagnaDao.prendiIdCampagnaUser(user);
			Iterator<Campagna> lnt = lngList.iterator();
			while (lnt.hasNext()) {
				Campagna campagna = lnt.next();
				CodaManager codaManager = Chiamate.getCodaProcessi().get(campagna);
				if(codaManager != null && camps.contains(campagna.getId())) {
					CodaStatic.getChiamateInCorso().remove(campagna);
					Chiamate.getCodaAutodialer().remove(campagna);
					codaManager.reset();
				}
			}
		}
	}
	public synchronized List<AutodialerReport> getProcessiReport(Utente user) {
		List<AutodialerReport> processiList = new ArrayList<AutodialerReport>();
		Set<Campagna> lngList = Chiamate.getCodaProcessi().keySet();
		if(ApplicationUtil.notEmpty(lngList)) {
			List<Long> camps = campagnaDao.prendiIdCampagnaUser(user);
			Iterator<Campagna> lnt = lngList.iterator();
			while (lnt.hasNext()) {
				Campagna campagna = lnt.next();
				if(campagna != null) {
					CodaManager codaManager = Chiamate.getCodaProcessi().get(campagna);
					if(codaManager != null && camps.contains(campagna.getId())) {
						AutodialerReport pr = new AutodialerReport();
						codaManager.updateReport(pr, codaManager);
						pr.setCampagna(campagna);
						pr.setSospeso(codaManager.isSospeso());
						pr.setAvviato(codaManager.isAvviato());
						pr.setCaricato(codaManager.isCaricato());
						pr.setEsaurito(codaManager.isEsaurito());
						pr.setStarted(codaManager.isStarted());
						pr.setFinito(codaManager.isFinito());
						pr.setCriterio(codaManager.getCriterio());
						pr.setCorrezione(codaManager.getCorrezione());
						processiList.add(pr);
					}
				}
			}
		}
		return processiList;
	}
	public synchronized void resetContattiStatistiche() {
		CodaStatic.getAutodialerStatistiche().resetAll();
		CodaStatic.getAutodialerContatti().resetAll();
		Set<Campagna> lngList = Chiamate.getCodaProcessi().keySet();
		if(ApplicationUtil.notEmpty(lngList)) {
			Iterator<Campagna> lnt = lngList.iterator();
			while (lnt.hasNext()) {
				Campagna campagna = lnt.next();
				CodaManager codaManager = Chiamate.getCodaProcessi().get(campagna);
				if(codaManager != null) {
					codaManager.getAutodialerStatistiche().resetAll();
					codaManager.getAutodialerContatti().resetAll();
				}
			}
		}
	}
	public synchronized void resetContattiStatistiche(Campagna campagna) {
		CodaManager codaManager = Chiamate.getCodaProcessi().get(campagna);
		if(codaManager != null) {
			codaManager.getAutodialerStatistiche().resetAll();
			codaManager.getAutodialerContatti().resetAll();
		}
	}
	public List<AutoContatto> getMappa(AutodialerContatti a, String chiave) {
		return ApplicationUtil.getMappa(a,chiave);
	}
	public synchronized AutodialerStatistiche contaContattiStatistiche1() {
		return new AutodialerStatistiche(CodaStatic.getAutodialerStatistiche());
	}
	public synchronized AutodialerContatti contaContattiStatistiche2() {
		return new AutodialerContatti(CodaStatic.getAutodialerContatti());
	}
	public synchronized AutodialerStatistiche contaContattiStatistiche1(Campagna campagna) {
		CodaManager codaManager = Chiamate.getCodaProcessi().get(campagna);
		if(codaManager != null) {
			return new AutodialerStatistiche(codaManager.getAutodialerStatistiche());
		}
		return new AutodialerStatistiche();
	}
	public synchronized AutodialerContatti contaContattiStatistiche2(Campagna campagna) {
		CodaManager codaManager = Chiamate.getCodaProcessi().get(campagna);
		if(codaManager != null) {
			return new AutodialerContatti(codaManager.getAutodialerContatti());
		}
		return new AutodialerContatti();
	}
	public synchronized AutodialerStatistiche contaContattiPannello(Integer a, Campagna campagna) {
		if(ApplicationUtil.gtZero(a)) {
			if(ApplicationUtil.eq(a, 1)) {
				CodaManager codaManager = Chiamate.getCodaProcessi().get(campagna);
				if(codaManager != null) {
					return new AutodialerStatistiche(codaManager.getAutodialerStatistiche());
				}
			}
			if(ApplicationUtil.eq(a, 2)) {
				return new AutodialerStatistiche(CodaStatic.getAutodialerStatistiche());
			}
		}
		return new AutodialerStatistiche();
	}
	public synchronized AutodialerReport contaContattiTotale() {
		Integer chiamateAttive = 0;
		Integer chiamateAttiveMax = 0;
		Integer chiamateInCorso = 0;
		Integer chiamateInCorsoMax = 0;
		Integer chiamateInAttesa = 0;
		Integer chiamateSchedulate = 0;
		Integer chiamateAutodialer = 0;
		Integer chiamateTeam = 0;
		Set<Campagna> lngList = Chiamate.getCodaProcessi().keySet();
		if(ApplicationUtil.notEmpty(lngList)) {
			Iterator<Campagna> lnt = lngList.iterator();
			while (lnt.hasNext()) {
				Campagna campagna = lnt.next();
				CodaManager codaManager = Chiamate.getCodaProcessi().get(campagna);
				if(codaManager != null) {
					chiamateAttive += codaManager.getChiamateAttive().getSize();
					chiamateAttiveMax += codaManager.getChiamateAttive().getCapacity();
					chiamateInCorso += codaManager.getChiamateInCorso().getSize();
					chiamateInCorsoMax += codaManager.getChiamateInCorso().getCapacity();
					chiamateInAttesa += codaManager.getChiamateInAttesa().getSize();
					chiamateSchedulate += codaManager.getChiamateSchedulate().getSize();
					chiamateAutodialer += Chiamate.getCodaAutodialer().get(campagna).size();
					chiamateTeam += CodaStatic.getCodaTeam().get(campagna).size();
				}
			}
		}
		AutodialerReport ad = new AutodialerReport();
		ad.setChiamateAttive(chiamateAttive);
		ad.setChiamateAttiveMax(chiamateAttiveMax);
		ad.setChiamateInCorso(chiamateInCorso);
		ad.setChiamateInCorsoMax(chiamateInCorsoMax);
		ad.setChiamateInAttesa(chiamateInAttesa);
		ad.setChiamateSchedulate(chiamateSchedulate);
		ad.setChiamateAutodialer(chiamateAutodialer);
		ad.setCodaTeamCampagna(chiamateTeam);
		return ad;
	}
	public synchronized AutodialerReportList contaContattiList(Campagna campagna) {
		AutodialerReportList ad = new AutodialerReportList();
		CodaManager codaManager = Chiamate.getCodaProcessi().get(campagna);
		if(codaManager != null) {
			ad.setChiamateInAttesa(codaManager.getChiamateInAttesa().list());
			ad.setChiamateAttive(codaManager.getChiamateAttive().list());
			ad.setChiamateSchedulate(codaManager.getChiamateSchedulate().list());
			ad.setChiamateInCorso(codaManager.getChiamateInCorso().list());
			ad.setChiamateAutodialer(Chiamate.getCodaAutodialer().get(campagna));
			ad.setCodaTeamCampagna(CodaStatic.getCodaTeam().get(campagna));
		}
		return ad;
	}
	public synchronized AutodialerReportList contaContattiTotaleList() {
		List<AutoContatto> chiamateInAttesa = new ArrayList<AutoContatto>();
		List<AutoContatto> chiamateInCorso = new ArrayList<AutoContatto>();
		List<AutoContatto> chiamateAttive = new ArrayList<AutoContatto>();
		List<AutoContatto> chiamateSchedulate = new ArrayList<AutoContatto>();
		List<Contatto> chiamateAutodialer = new ArrayList<Contatto>();
		List<InfoUtente> chiamateTeam = new ArrayList<InfoUtente>();
		Set<Campagna> lngList = Chiamate.getCodaProcessi().keySet();
		if(ApplicationUtil.notEmpty(lngList)) {
			Iterator<Campagna> lnt = lngList.iterator();
			while (lnt.hasNext()) {
				Campagna campagna = lnt.next();
				CodaManager codaManager = Chiamate.getCodaProcessi().get(campagna);
				if(codaManager != null) {
					chiamateInCorso.addAll(codaManager.getChiamateInCorso().list());
					chiamateInAttesa.addAll(codaManager.getChiamateInAttesa().list());
					chiamateAttive.addAll(codaManager.getChiamateAttive().list());
					chiamateSchedulate.addAll(codaManager.getChiamateSchedulate().list());
					chiamateAutodialer.addAll(Chiamate.getCodaAutodialer().get(campagna));
					chiamateTeam.addAll(CodaStatic.getCodaTeam().get(campagna));
				}
			}
		}
		AutodialerReportList ad = new AutodialerReportList();
		ad.setChiamateInAttesa(chiamateInAttesa);
		ad.setChiamateAttive(chiamateAttive);
		ad.setChiamateSchedulate(chiamateSchedulate);
		ad.setChiamateInCorso(chiamateInCorso);
		ad.setChiamateAutodialer(chiamateAutodialer);
		ad.setCodaTeamCampagna(chiamateTeam);
		return ad;
	}
	public boolean autodialerPrivata(Contatto contatto) {
		if(contatto==null)
			return false;
		else 
			return true;
	}

	public synchronized void removeUser(Utente user, Campagna campagna) {
		InfoUtente utente = new InfoUtente(user);
		CodaStatic.getCodaTeam().rimuoviUtente(campagna, utente);
		Chiamate.getCodaInterni().stato(user.getInternoString(), false);
	}

	public synchronized void rimuoviUtente(Campagna campagna, Utente user) {
		InfoUtente utente = new InfoUtente(user);
		CodaStatic.getCodaTeam().rimuoviUtente(campagna, utente);
	}
	public synchronized void rimuoviUtenteT(Utente user) {
		InfoUtente utente = new InfoUtente(user);
		CodaStatic.getCodaTeam().rimuoviUtenteT(utente);
		Chiamate.getCodaInterni().stato(user.getInternoString(), false);
	}

	public synchronized void rimuoviUtenteAll(Utente user) {
		InfoUtente utente = new InfoUtente(user);
		CodaStatic.getCodaTeam().rimuoviUtenteAll(utente);
	}
	public synchronized void rimuoviContatto(Contatto contatto) {
		CodaStatic.getChiamateInCorso().remove(contatto.getId());
		Chiamate.getCodaAutodialer().remove(contatto.getId());
		Chiamate.getCodaContatti().remove(contatto.getId());
		CodaManager codaManager = Chiamate.getCodaProcessi().get(contatto.getCampagna());
		if(codaManager != null) {
			codaManager.getChiamateInAttesa().remove(contatto.getId());
			codaManager.getChiamateInCorso().remove(contatto.getId());
			codaManager.getChiamateSchedulate().remove(contatto.getId());
			codaManager.getChiamateAttive().remove(contatto.getId());
		}
	}
	public synchronized void rimuoviContatti(List<Contatto> contatti) {
		CodaStatic.getChiamateInCorso().removeContatti(contatti);
		Chiamate.getCodaAutodialer().removeContatti(contatti);
		Chiamate.getCodaContatti().removeContatti(contatti);
		Set<Campagna> lngList = Chiamate.getCodaProcessi().keySet();
		if(ApplicationUtil.notEmpty(lngList)) {
			Iterator<Campagna> lnt = lngList.iterator();
			while (lnt.hasNext()) {
				Campagna campagna = lnt.next();
				CodaManager codaManager = Chiamate.getCodaProcessi().get(campagna);
				if(codaManager != null) {
					codaManager.getChiamateInAttesa().removeContatti(contatti);
					codaManager.getChiamateInCorso().removeContatti(contatti);
					codaManager.getChiamateSchedulate().removeContatti(contatti);
					codaManager.getChiamateAttive().removeContatti(contatti);
				}
			}
		}
	}
	public synchronized void rimuoviContattoInCorso(Contatto contatto) {
		CodaStatic.getChiamateInCorso().remove(contatto.getId());
		CodaManager codaManager = Chiamate.getCodaProcessi().get(contatto.getCampagna());
		if(codaManager != null) {
			codaManager.getChiamateInAttesa().remove(contatto.getId());
			codaManager.getChiamateInCorso().remove(contatto.getId());
		}
	}
	public synchronized void rimuoviContattoInAttesa(Contatto contatto) {
		CodaStatic.getChiamateInCorso().remove(contatto.getId());
		CodaManager codaManager = Chiamate.getCodaProcessi().get(contatto.getCampagna());
		if(codaManager != null) {
			codaManager.getChiamateInAttesa().remove(contatto.getId());
			codaManager.getChiamateInCorso().remove(contatto.getId());
		}
	}
	public synchronized void rimuoviContattoAttivo(Contatto contatto) {
		CodaManager codaManager = Chiamate.getCodaProcessi().get(contatto.getCampagna());
		if(codaManager != null) {
			codaManager.getChiamateAttive().remove(contatto.getId());
		}
	}
	public synchronized void rimuoviContattoSchedulato(Contatto contatto) {
		CodaManager codaManager = Chiamate.getCodaProcessi().get(contatto.getCampagna());
		if(codaManager != null) {
			codaManager.getChiamateSchedulate().remove(contatto.getId());
		}
	}
	public synchronized void rimuoviContattiInCorso(Campagna campagna) {
		CodaStatic.getChiamateInCorso().remove(campagna);
		CodaManager codaManager = Chiamate.getCodaProcessi().get(campagna);
		if(codaManager != null) {
			codaManager.getChiamateInAttesa().clear();
			codaManager.getChiamateInCorso().clear();
		}
	}
	public synchronized void rimuoviContattiInAttesa(Campagna campagna) {
		CodaManager codaManager = Chiamate.getCodaProcessi().get(campagna);
		if(codaManager != null) {
			List<AutoContatto> contatti = codaManager.getChiamateInAttesa().list();
			CodaStatic.getChiamateInCorso().remove(contatti);
			codaManager.getChiamateInCorso().remove(contatti);
			codaManager.getChiamateInAttesa().clear();
		}
	}
	public synchronized void rimuoviContattiAttivo(Campagna campagna) {
		CodaManager codaManager = Chiamate.getCodaProcessi().get(campagna);
		if(codaManager != null) {
			codaManager.getChiamateAttive().clear();
		}
	}
	public synchronized void rimuoviContattiSchedulato(Campagna campagna) {
		CodaManager codaManager = Chiamate.getCodaProcessi().get(campagna);
		if(codaManager != null) {
			codaManager.getChiamateSchedulate().clear();
		}
	}
	public synchronized void rimuoviContattiInCorso() {
		CodaStatic.getChiamateInCorso().clear();
		Set<Campagna> lngList = Chiamate.getCodaProcessi().keySet();
		if(ApplicationUtil.notEmpty(lngList)) {
			Iterator<Campagna> lnt = lngList.iterator();
			while (lnt.hasNext()) {
				Campagna campagna = lnt.next();
				CodaManager codaManager = Chiamate.getCodaProcessi().get(campagna);
				if(codaManager != null) {
					codaManager.getChiamateInAttesa().clear();
					codaManager.getChiamateInCorso().clear();
				}
			}
		}
	}
	public synchronized void rimuoviContattiInAttesa() {
		Set<Campagna> lngList = Chiamate.getCodaProcessi().keySet();
		if(ApplicationUtil.notEmpty(lngList)) {
			Iterator<Campagna> lnt = lngList.iterator();
			while (lnt.hasNext()) {
				Campagna campagna = lnt.next();
				CodaManager codaManager = Chiamate.getCodaProcessi().get(campagna);
				if(codaManager != null) {
					List<AutoContatto> contatti = codaManager.getChiamateInAttesa().list();
					CodaStatic.getChiamateInCorso().remove(contatti);
					codaManager.getChiamateInCorso().remove(contatti);
					codaManager.getChiamateInAttesa().clear();
				}
			}
		}
	}
	public synchronized void rimuoviContattiAttivo() {
		Set<Campagna> lngList = Chiamate.getCodaProcessi().keySet();
		if(ApplicationUtil.notEmpty(lngList)) {
			Iterator<Campagna> lnt = lngList.iterator();
			while (lnt.hasNext()) {
				Campagna campagna = lnt.next();
				CodaManager codaManager = Chiamate.getCodaProcessi().get(campagna);
				if(codaManager != null) {
					codaManager.getChiamateAttive().clear();
				}
			}
		}
	}
	public synchronized void rimuoviContattiSchedulato() {
		Set<Campagna> lngList = Chiamate.getCodaProcessi().keySet();
		if(ApplicationUtil.notEmpty(lngList)) {
			Iterator<Campagna> lnt = lngList.iterator();
			while (lnt.hasNext()) {
				Campagna campagna = lnt.next();
				CodaManager codaManager = Chiamate.getCodaProcessi().get(campagna);
				if(codaManager != null) {
					codaManager.getChiamateSchedulate().clear();
				}
			}
		}
	}
	public void addEventListener(ManagerEventResponse manager) {
		if(managerConnection != null) {
			managerConnection.addEventListener(manager);
		}
	}
	public void removeEventListener(ManagerEventResponse manager) {
		if(managerConnection != null) {
			managerConnection.removeEventListener(manager);
		}
	}
	public void addServerListener(AsteriskServerListener server) {
		if(asteriskServer != null) {
			asteriskServer.addAsteriskServerListener(server);
		}
	}
	public void removeServerListener(AsteriskServerListener server) {
		if(asteriskServer != null) {
			asteriskServer.removeAsteriskServerListener(server);
		}
	}
	public int getNumCanali() {
		DetachedCriteria c = DetachedCriteria.forClass(SipBuddies.class);
		c.add(Restrictions.eq("tipologia", ApplicationUtil.convertBS(false)));
		c.createCriteria("autodialer").add(Restrictions.ne("stato", ConstantsGlobal.ts_chiuso));
		c.setProjection(Projections.distinct(Projections.property("id")));
		DetachedCriteria criteria = DetachedCriteria.forClass(SipBuddies.class);
		impostazioneDao.subPropertyInS(criteria, c, "id");
		return getSommaIntero(criteria, "max_contacts");
	}
	public AutoContatto get(Long id){
		return (AutoContatto) get(AutoContatto.class, id);
	}
	public void elimina(AutoContatto auto) {
		cancellaAggiorna(auto);
	}
	private ExtendedCriteria trovaCriteria(AutoContattoCriteria ema) {
		if(ema != null) ema = (AutoContattoCriteria) ema.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(AutoContatto.class);
		DetachedCriteria criteriaCo = criteria.createCriteria("contatto");
		DetachedCriteria criteriaCa = criteriaCo.createCriteria("campagna");
		if (ApplicationUtil.isNotBlank(ema.getRicerca_anagrafica())) {
			DetachedCriteria criteriaA = criteriaCo.createCriteria("anagrafica");
			ApplicationUtil.anagrafica(ema.getRicerca_anagraficaV(), criteriaA, ema.getRicerca_anagrafica());
		}
		if(ApplicationUtil.gtZero(ema.getStato())) {
			criteria.add(Restrictions.eq("stato",ema.getStato()));
		}
		if(ema.getInbound() != null) {
			criteria.add(Restrictions.eq("inbound",ema.getInbound()));
		}
		if(ApplicationUtil.gtZero(ema.getAutodialer())) {
			criteriaCa.add(Restrictions.eq("autodialer.id",ema.getAutodialer()));
		}
		if(ApplicationUtil.gtZero(ema.getCampagna())) {
			criteriaCa.add(Restrictions.idEq(ema.getCampagna()));
		}
		if (ApplicationUtil.isNotBlank(ema.getUtente())) {
			DetachedCriteria criteriaU = criteria.createCriteria("utente");
			ApplicationUtil.operatore(criteriaU, ema.getUtente());
		}
		ema.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, ema);
	}
	private ExtendedCriteria trovaCriteria(Utente user, AutoContattoCriteria ema) {
		if(ema != null) ema = (AutoContattoCriteria) ema.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(AutoContatto.class);
		DetachedCriteria criteriaCo = criteria.createCriteria("contatto");
		DetachedCriteria criteriaCa = criteriaCo.createCriteria("campagna");
		impostazioneDao.subCriteriaCampagnaSupervisore(user.getId(), criteriaCa, "id");
		if (ApplicationUtil.isNotBlank(ema.getRicerca_anagrafica())) {
			DetachedCriteria criteriaA = criteriaCo.createCriteria("anagrafica");
			ApplicationUtil.anagrafica(ema.getRicerca_anagraficaV(), criteriaA, ema.getRicerca_anagrafica());
		}
		if(ApplicationUtil.gtZero(ema.getStato())) {
			criteria.add(Restrictions.eq("stato",ema.getStato()));
		}
		if(ema.getInbound() != null) {
			criteria.add(Restrictions.eq("inbound",ema.getInbound()));
		}
		if(ApplicationUtil.gtZero(ema.getAutodialer())) {
			criteriaCa.add(Restrictions.eq("autodialer.id",ema.getAutodialer()));
		}
		if(ApplicationUtil.gtZero(ema.getCampagna())) {
			criteriaCa.add(Restrictions.idEq(ema.getCampagna()));
		}
		if (ApplicationUtil.isNotBlank(ema.getUtente())) {
			DetachedCriteria criteriaU = criteria.createCriteria("utente");
			ApplicationUtil.operatore(criteriaU, ema.getUtente());
		}
		ema.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, ema);
	}
	@SuppressWarnings("unchecked")
	public List<AutoContatto> trova(AutoContattoCriteria auto, int start, int size) {
		ExtendedCriteria criteria = trovaCriteria(auto);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), auto.getO_tipo(), auto.getO_colonna(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int trovaSize(AutoContattoCriteria auto) {
		ExtendedCriteria criteria = trovaCriteria(auto);
		return getListSize(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<AutoContatto> trova(Utente user, AutoContattoCriteria auto, int start, int size) {
		ExtendedCriteria criteria = trovaCriteria(user, auto);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), auto.getO_tipo(), auto.getO_colonna(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int trovaSize(Utente user, AutoContattoCriteria auto) {
		ExtendedCriteria criteria = trovaCriteria(user, auto);
		return getListSize(criteria);
	}
	public List<SelectItem> getAutodialerSI() {
		DetachedCriteria criteria = DetachedCriteria.forClass(Autodialer.class);
		List<AutodialerSI> lis = getAutodialerSI(criteria);
		return impostazioneDao.getAutodialer(lis, true, null);
	}
	public List<SelectItem> getAutodialerSupervisoreSI(Long utenteId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Autodialer.class);
		impostazioneDao.subCriteriaCampagnaSupervisore(utenteId, criteria.createCriteria("campagne"), "id");
		List<AutodialerSI> lis = getAutodialerSI(criteria);
		return impostazioneDao.getAutodialer(lis, true, null);
	}
	public List<SelectItem> getAutodialerPartnerSI(Long utenteId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Autodialer.class);
		impostazioneDao.subCriteriaCampagnaPartner(utenteId, criteria.createCriteria("campagne"), "id");
		List<AutodialerSI> lis = getAutodialerSI(criteria);
		return impostazioneDao.getAutodialer(lis, true, null);
	}
	public List<SelectItem> getAutodialerManagerSI(Long utenteId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Autodialer.class);
		List<AutodialerSI> lis = getAutodialerSI(criteria);
		return impostazioneDao.getAutodialer(lis, true, null);
	}
	public List<SelectItem> getAutodialerBackofficeSI(Long utenteId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Autodialer.class);
		List<AutodialerSI> lis = getAutodialerSI(criteria);
		return impostazioneDao.getAutodialer(lis, true, null);
	}
	public List<SelectItem> getCampagneSupervisoreSI(Long utenteId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		criteria.createCriteria("team").createCriteria("supervisore").add(Restrictions.idEq(utenteId));
		List<CampagnaSI> lis = getCampagnaSI(criteria);
		return impostazioneDao.getCampagna(lis, true, false, false, null);
	} 
	public void salvaSuono(FileUpload fileUpload, Autodialer auto) {
		if(ApplicationUtil.isNotTrue(impostazioneDao.getFTPEnabled())) {
			String home = impostazioneDao.getHome();
			try {
				List<MyFile> fls = fileUpload.getFiles();
				if(ApplicationUtil.notEmpty(fls)) {
					Iterator<MyFile> flsIt = fls.iterator();
					while (flsIt.hasNext()) {
						MyFile ff = (MyFile) flsIt.next();
						ByteArrayOutputStream bos = new ByteArrayOutputStream();
						String fileN = ApplicationUtil.salvaAudio(ff, bos, false);
						byte [] ffB = bos.toByteArray();
						if(ffB != null && ffB.length > 0) {
							FileOutputStream fi = new FileOutputStream(home + ConstantsGlobal.path_sounds + "/" + fileN);
							fi.write(ffB);
							fi.flush();
							fi.close();
							auto.setSuono(fileN);
						}
					}
				}
			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		} else {
			UtilFTP ut = impostazioneDao.getUtilFTP();
			FTPClient vmftp = ApplicationUtil.createClient();
			if (ApplicationUtil.connect(vmftp, ut)) {
				try {
					List<MyFile> fls = fileUpload.getFiles();
					if(ApplicationUtil.notEmpty(fls)) {
						Iterator<MyFile> flsIt = fls.iterator();
						while (flsIt.hasNext()) {
							MyFile ff = (MyFile) flsIt.next();
							ByteArrayOutputStream bos = new ByteArrayOutputStream();
							String fileN = ApplicationUtil.salvaAudio(ff, bos, false);
							byte [] ffB = bos.toByteArray();
							if(ffB != null && ffB.length > 0) {
								InputStream fis = new ByteArrayInputStream(ffB);
								ApplicationUtil.storeFile(vmftp, ConstantsGlobal.path_sounds + "/" + fileN, fis);
								auto.setSuono(fileN);
								fis.close();
							}
						}
					}
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				} finally {
					ApplicationUtil.disconnect(vmftp);
				}
			}	
		}
	}
	public void salvaSuono(FileUpload fileUpload) {
		if(ApplicationUtil.isNotTrue(impostazioneDao.getFTPEnabled())) {
			String home = impostazioneDao.getHome();
			try {
				List<MyFile> fls = fileUpload.getFiles();
				if(ApplicationUtil.notEmpty(fls)) {
					Iterator<MyFile> flsIt = fls.iterator();
					while (flsIt.hasNext()) {
						MyFile ff = (MyFile) flsIt.next();
						ByteArrayOutputStream bos = new ByteArrayOutputStream();
						String fileN = ApplicationUtil.salvaAudio(ff, bos, false);
						byte [] ffB = bos.toByteArray();
						if(ffB != null && ffB.length > 0) {
							FileOutputStream fi = new FileOutputStream(home + ConstantsGlobal.path_sounds + "/" + fileN);
							fi.write(ffB);
							fi.flush();
							fi.close();
						}
					}
				}
			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		} else {
			UtilFTP ut = impostazioneDao.getUtilFTP();
			FTPClient vmftp = ApplicationUtil.createClient();
			if (ApplicationUtil.connect(vmftp, ut)) {
				try {
					List<MyFile> fls = fileUpload.getFiles();
					if(ApplicationUtil.notEmpty(fls)) {
						Iterator<MyFile> flsIt = fls.iterator();
						while (flsIt.hasNext()) {
							MyFile ff = (MyFile) flsIt.next();
							ByteArrayOutputStream bos = new ByteArrayOutputStream();
							String fileN = ApplicationUtil.salvaAudio(ff, bos, false);
							byte [] ffB = bos.toByteArray();
							if(ffB != null && ffB.length > 0) {
								InputStream fis = new ByteArrayInputStream(ffB);
								ApplicationUtil.storeFile(vmftp, ConstantsGlobal.path_sounds + "/" + fileN, fis);
								fis.close();
							}
						}
					}
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				} finally {
					ApplicationUtil.disconnect(vmftp);
				}
			}	
		}
	}
	public void eliminaSuonoFtp(String suono) {
		if(ApplicationUtil.isNotTrue(impostazioneDao.getFTPEnabled())) {
			String home = impostazioneDao.getHome();
			File perc = new File(home + ConstantsGlobal.path_sounds + "/" + suono);
			if(perc.exists()) {
				ApplicationUtil.deleteFile(perc);
			}
		} else {
			UtilFTP ut = impostazioneDao.getUtilFTP();
			FTPClient vmftp = ApplicationUtil.createClient();
			if (ApplicationUtil.connect(vmftp, ut)) {
				try {
					ApplicationUtil.deleteFile(vmftp, ConstantsGlobal.path_sounds + "/" + suono);
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				} finally {
					ApplicationUtil.disconnect(vmftp);
				}
			}	
		}
	}
	public void eliminaSuono(MyFile s) {
		eliminaSuonoFtp(s.getName());
		Session session = openSession();
		Transaction tx = session.beginTransaction();
		String hql1 = "update Autodialer set preambolo = false, suono = null where suono like :suono";
		String hql2 = "update Campagna set suono = null where suono like :suono";
		execute(false, session, hql1, new Parameter("suono", "%" + s.getName()));
		execute(false, session, hql2, new Parameter("suono", "%" + s.getName()));
		session.flush();
		session.clear();
		tx.commit();
		closeSession(session);
	}
	public void eliminaSuono(List<MyFile> suoni) {
		if(ApplicationUtil.notEmpty(suoni)) {
			Iterator<MyFile> eIt = suoni.iterator();
			while (eIt.hasNext()) {
				MyFile suono = (MyFile) eIt.next();
				eliminaSuono(suono);
			}
		}
	}
	public void scaricaSuono(MyFile suono, DownloadBean downloadBean) {
		if(ApplicationUtil.isNotTrue(impostazioneDao.getFTPEnabled())) {
			String home = impostazioneDao.getHome();
			try {
				OutputStream out = downloadBean.outputStream();
				downloadBean.filename(suono.getName());
				String nome = home + ConstantsGlobal.path_sounds +"/"+ suono.getName();
				InputStream istream = new FileInputStream(nome);
				ApplicationUtil.copy(istream, out);
				istream.close();
				downloadBean.close();
			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			} finally {
			}
		} else {
			UtilFTP ut = impostazioneDao.getUtilFTP();
			FTPClient vmftp = ApplicationUtil.createClient();
			if (ApplicationUtil.connect(vmftp, ut)) {
				try {
					OutputStream out = downloadBean.outputStream();
					downloadBean.filename(suono.getName());
					String path = ConstantsGlobal.path_sounds + "/" + suono.getName();
					ApplicationUtil.scaricaFile(vmftp, out, path);
					downloadBean.close();
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				} finally {
					ApplicationUtil.disconnect(vmftp);
				}
			}	
		}
	} 
	public List<MyFile> listaSuoni(FileCriteria mm) {
		if(mm != null) mm = (FileCriteria) mm.loadCriteria();
		List<MyFile> ff = new ArrayList<MyFile>();
		if(mm != null && mm.getCerca()) {
			if(ApplicationUtil.isNotTrue(impostazioneDao.getFTPEnabled())) {
				String home = impostazioneDao.getHome();
				File perc = new File(home + ConstantsGlobal.path_sounds);
				if(perc.exists() && perc.isDirectory()) {
					File [] files = perc.listFiles();
					if(ApplicationUtil.notEmpty(files)) {
						for(File file : files) {
							if(mm.isRicerca(file.getName(), file.length(), file.lastModified())) {
								MyFile f = new MyFile(file);
								f.setName(file.getName());
								f.setModified(file.lastModified());
								ff.add(f);
							}
						}
					}
				}
			} else {
				UtilFTP ut = impostazioneDao.getUtilFTP();
				FTPClient vmftp = ApplicationUtil.createClient();
				if (ApplicationUtil.connect(vmftp, ut)) {
					try {
						FTPFile[] wavFiles = ApplicationUtil.listFiles(vmftp, ConstantsGlobal.path_sounds + "/");
						if (ApplicationUtil.notEmpty(wavFiles)) {
							for(FTPFile file : wavFiles) {
								Long time = ApplicationUtil.time(file.getTimestamp());
								if(mm.isRicerca(file.getName(), file.getSize(), time)) {
									MyFile f = new MyFile(file);
									f.setName(file.getName());
									f.setModified(time);
									ff.add(f);
								}
							}
						}
					} catch (Exception e) {
						ErrorLog.log(null, getClass(), e);
					} finally {
						ApplicationUtil.disconnect(vmftp);
					}
				}	
			}
		}
		return ff;
	}
	
	public List<String> listaSuoni(String nome) {
		List<String> ff = new ArrayList<String>();
			if(ApplicationUtil.isNotTrue(impostazioneDao.getFTPEnabled())) {
				String home = impostazioneDao.getHome();
				File perc = new File(home + ConstantsGlobal.path_sounds);
				if(perc.exists() && perc.isDirectory()) {
					File [] files = perc.listFiles();
					if(ApplicationUtil.notEmpty(files)) {
						for(File file : files) {
							String n = ApplicationUtil.getBaseNameN(file.getName());
							if(nome != null && n.startsWith(nome)) {
								ff.add(n);
							}
						}
					}
				}
			} else {
				UtilFTP ut = impostazioneDao.getUtilFTP();
				FTPClient vmftp = ApplicationUtil.createClient();
				if (ApplicationUtil.connect(vmftp, ut)) {
					try {
						FTPFile[] wavFiles = ApplicationUtil.listFiles(vmftp, ConstantsGlobal.path_sounds + "/");
						if (ApplicationUtil.notEmpty(wavFiles)) {
							for(FTPFile file : wavFiles) {
								String n = ApplicationUtil.getBaseNameN(file.getName());
								if(nome != null && n.startsWith(nome)) {
									ff.add(n);
								}
							}
						}
					} catch (Exception e) {
						ErrorLog.log(null, getClass(), e);
					} finally {
						ApplicationUtil.disconnect(vmftp);
					}
				}	
		}
		return ff;
	}
	
	@SuppressWarnings("unchecked")
	public List<Campagna> getCampagneByAutoId(Long autoId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		criteria.add(Restrictions.eq("autodialer.id", autoId));
		ApplicationUtil.orderCL(criteria);
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Autodialer> getAutodialerByTrunk(Long trunk) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Autodialer.class);
		criteria.createCriteria("trunk").add(Restrictions.idEq(trunk));
		criteria.addOrder(Order.asc("nome"));
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Campagna> getCampagneByTrunk(Long trunk) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		criteria.createCriteria("autodialer").createCriteria("trunk").add(Restrictions.idEq(trunk));
		ApplicationUtil.orderCL(criteria);
		return getList(criteria);
	}
	public synchronized void lasciaContatto(Utente utente, Contatto contatto) {
		if (contatto != null) {
			CodaStatic.getChiamateInCorso().remove(contatto.getId());
			Chiamate.getCodaAutodialer().remove(contatto.getId());
			CodaManager codaManager = Chiamate.getCodaProcessi().get(contatto.getCampagna());
			if(codaManager != null) {
				codaManager.getChiamateInAttesa().remove(contatto.getId());
				codaManager.getChiamateInCorso().remove(contatto.getId());
				codaManager.getChiamateSchedulate().remove(contatto.getId());
				codaManager.getChiamateAttive().remove(contatto.getId());
			}
		}
	}
	public synchronized void lasciaContattoTutti() {
		Chiamate.getCodaAutodialer().clear();
		CodaStatic.getChiamateInCorso().clear();
		Set<Campagna> lngList = Chiamate.getCodaProcessi().keySet();
		if(ApplicationUtil.notEmpty(lngList)) {
			Iterator<Campagna> lnt = lngList.iterator();
			while (lnt.hasNext()) {
				Campagna campagna = lnt.next();
				CodaManager codaManager = Chiamate.getCodaProcessi().get(campagna);
				if(codaManager != null) {
					codaManager.clear();
				}
			}
		}
	}
	public synchronized void avvia(Campagna campagna) {
		CodaManager codaManager = Chiamate.getCodaProcessi().get(campagna);
		if(codaManager != null) {
			codaManager.setAvviato(true);
		}
	}
	public synchronized void lasciaContattoCampagna(Campagna campagna) {	
		CodaManager codaManager = Chiamate.getCodaProcessi().get(campagna);
		if(codaManager != null) {
			codaManager.clear();
		}
		Chiamate.getCodaAutodialer().remove(campagna);
		CodaStatic.getChiamateInCorso().remove(campagna);
	}
	public synchronized void rimuoviContattiInCorso(Utente user) {
		Set<Campagna> lngList = Chiamate.getCodaProcessi().keySet();
		if(ApplicationUtil.notEmpty(lngList)) {
			List<Long> camps = campagnaDao.prendiIdCampagnaUser(user);
			Iterator<Campagna> lnt = lngList.iterator();
			while (lnt.hasNext()) {
				Campagna campagna = lnt.next();
				CodaManager codaManager = Chiamate.getCodaProcessi().get(campagna);
				if(codaManager != null && camps.contains(campagna.getId())) {
					CodaStatic.getChiamateInCorso().remove(campagna);
					codaManager.getChiamateInAttesa().clear();
					codaManager.getChiamateInCorso().clear();
				}
			}
		}
	}
	public synchronized void rimuoviContattiInAttesa(Utente user) {
		Set<Campagna> lngList = Chiamate.getCodaProcessi().keySet();
		if(ApplicationUtil.notEmpty(lngList)) {
			List<Long> camps = campagnaDao.prendiIdCampagnaUser(user);
			Iterator<Campagna> lnt = lngList.iterator();
			while (lnt.hasNext()) {
				Campagna campagna = lnt.next();
				CodaManager codaManager = Chiamate.getCodaProcessi().get(campagna);
				if(codaManager != null && camps.contains(campagna.getId())) {
					List<AutoContatto> contatti = codaManager.getChiamateInAttesa().list();
					CodaStatic.getChiamateInCorso().remove(contatti);
					codaManager.getChiamateInCorso().remove(contatti);
					codaManager.getChiamateInAttesa().clear();
				}
			}
		}
	}
	public synchronized void rimuoviContattiAttivo(Utente user) {
		Set<Campagna> lngList = Chiamate.getCodaProcessi().keySet();
		if(ApplicationUtil.notEmpty(lngList)) {
			List<Long> camps = campagnaDao.prendiIdCampagnaUser(user);
			Iterator<Campagna> lnt = lngList.iterator();
			while (lnt.hasNext()) {
				Campagna campagna = lnt.next();
				CodaManager codaManager = Chiamate.getCodaProcessi().get(campagna);
				if(codaManager != null && camps.contains(campagna.getId())) {
					codaManager.getChiamateAttive().clear();
				}
			}
		}
	}
	public synchronized void rimuoviContattiSchedulato(Utente user) {
		Set<Campagna> lngList = Chiamate.getCodaProcessi().keySet();
		if(ApplicationUtil.notEmpty(lngList)) {
			List<Long> camps = campagnaDao.prendiIdCampagnaUser(user);
			Iterator<Campagna> lnt = lngList.iterator();
			while (lnt.hasNext()) {
				Campagna campagna = lnt.next();
				CodaManager codaManager = Chiamate.getCodaProcessi().get(campagna);
				if(codaManager != null && camps.contains(campagna.getId())) {
					codaManager.getChiamateSchedulate().clear();
				}
			}
		}
	}
	public synchronized AutodialerReportList contaContattiList(Utente user) {
		List<AutoContatto> chiamateInAttesa = new ArrayList<AutoContatto>();
		List<AutoContatto> chiamateAttive = new ArrayList<AutoContatto>();
		List<AutoContatto> chiamateInCorso = new ArrayList<AutoContatto>();
		List<AutoContatto> chiamateSchedulate = new ArrayList<AutoContatto>();
		List<Contatto> chiamateAutodialer = new ArrayList<Contatto>();
		List<InfoUtente> chiamateTeam = new ArrayList<InfoUtente>();
		Set<Campagna> lngList = Chiamate.getCodaProcessi().keySet();
		if(ApplicationUtil.notEmpty(lngList)) {
			List<Long> camps = campagnaDao.prendiIdCampagnaUser(user);
			Iterator<Campagna> lnt = lngList.iterator();
			while (lnt.hasNext()) {
				Campagna campagna = lnt.next();
				CodaManager codaManager = Chiamate.getCodaProcessi().get(campagna);
				if(codaManager != null && camps.contains(campagna.getId())) {
					chiamateInAttesa.addAll(codaManager.getChiamateInAttesa().list());
					chiamateAttive.addAll(codaManager.getChiamateAttive().list());
					chiamateSchedulate.addAll(codaManager.getChiamateSchedulate().list());
					chiamateInCorso.addAll(codaManager.getChiamateInCorso().list());
					chiamateAutodialer.addAll(Chiamate.getCodaAutodialer().get(campagna));
					chiamateTeam.addAll(CodaStatic.getCodaTeam().get(campagna));
				}
			}
		}
		AutodialerReportList ad = new AutodialerReportList();
		ad.setChiamateInAttesa(chiamateInAttesa);
		ad.setChiamateAttive(chiamateAttive);
		ad.setChiamateSchedulate(chiamateSchedulate);
		ad.setChiamateInCorso(chiamateInCorso);
		ad.setChiamateAutodialer(chiamateAutodialer);
		ad.setCodaTeamCampagna(chiamateTeam);
		return ad;
	}
	public synchronized AutodialerReport contaContatti(Utente user) {
		Integer chiamateAttive = 0;
		Integer chiamateAttiveMax = 0;
		Integer chiamateInAttesa = 0;
		Integer chiamateSchedulate = 0;
		Integer chiamateInCorso = 0;
		Integer chiamateInCorsoMax = 0;
		Integer chiamateAutodialer = 0;
		Integer chiamateTeam = 0;
		Set<Campagna> lngList = Chiamate.getCodaProcessi().keySet();
		if(ApplicationUtil.notEmpty(lngList)) {
			List<Long> camps = campagnaDao.prendiIdCampagnaUser(user);
			Iterator<Campagna> lnt = lngList.iterator();
			while (lnt.hasNext()) {
				Campagna campagna = lnt.next();
				CodaManager codaManager = Chiamate.getCodaProcessi().get(campagna);
				if(codaManager != null && camps.contains(campagna.getId())) {
					chiamateInAttesa += codaManager.getChiamateInAttesa().getSize();
					chiamateAttive += codaManager.getChiamateAttive().getSize();
					chiamateAttiveMax += codaManager.getChiamateAttive().getCapacity();
					chiamateSchedulate += codaManager.getChiamateSchedulate().getSize();
					chiamateInCorso += codaManager.getChiamateInCorso().getSize();
					chiamateInCorsoMax += codaManager.getChiamateInCorso().getCapacity();
					chiamateAutodialer += Chiamate.getCodaAutodialer().get(campagna).size();
					chiamateTeam += CodaStatic.getCodaTeam().get(campagna).size();
				}
			}
		}
		AutodialerReport ad = new AutodialerReport();
		ad.setChiamateInAttesa(chiamateInAttesa);
		ad.setChiamateAttive(chiamateAttive);
		ad.setChiamateAttiveMax(chiamateAttiveMax);
		ad.setChiamateInCorso(chiamateInCorso);
		ad.setChiamateInCorsoMax(chiamateInCorsoMax);
		ad.setChiamateSchedulate(chiamateSchedulate);
		ad.setChiamateAutodialer(chiamateAutodialer);
		ad.setCodaTeamCampagna(chiamateTeam);
		return ad;
	}
	public synchronized AutodialerStatistiche contaContattiStatistiche1(Utente user) {
		AutodialerStatistiche as = new AutodialerStatistiche();
		Long timeAttesa = Long.valueOf(0);
		Long timeChiamata = Long.valueOf(0);
		Long timeGestione = Long.valueOf(0);
		Integer a_rimosse = 0;
		Integer a_saltate = 0;
		Integer a_fallite = 0;
		AutodialerMap statistiche = new AutodialerMap();
		Map<String, AutodialerMap> mappa = new TreeMap<String, AutodialerMap>();
		Set<Campagna> lngList = Chiamate.getCodaProcessi().keySet();
		if(ApplicationUtil.notEmpty(lngList)) {
			List<Long> camps = campagnaDao.prendiIdCampagnaUser(user);
			Iterator<Campagna> lnt = lngList.iterator();
			while (lnt.hasNext()) {
				Campagna campagna = lnt.next();
				CodaManager codaManager = Chiamate.getCodaProcessi().get(campagna);
				if(codaManager != null && camps.contains(campagna.getId())) {
					AutodialerStatistiche ad = codaManager.getAutodialerStatistiche();
					timeAttesa += ad.getTimeAttesa();
					timeChiamata += ad.getTimeChiamata();
					timeGestione += ad.getTimeGestione();
					a_rimosse += ad.getA_rimosse();
					a_saltate += ad.getA_saltate();
					a_fallite += ad.getA_fallite();
					statistiche.update(ad.getStatistiche());
					mappa.putAll(ad.getMappa());
				}
			}
		}
		as.setTimeAttesa(timeAttesa);
		as.setTimeChiamata(timeChiamata);
		as.setTimeGestione(timeGestione);
		as.setA_rimosse(a_rimosse);
		as.setA_saltate(a_saltate);
		as.setA_fallite(a_fallite);
		as.setStatistiche(statistiche);
		as.setMappa(mappa);
		return as;
	}
	public synchronized AutodialerContatti contaContattiStatistiche2(Utente user) {
		AutodialerContatti as = new AutodialerContatti();
		AutodialerList list = new AutodialerList();
		Set<Campagna> lngList = Chiamate.getCodaProcessi().keySet();
		if(ApplicationUtil.notEmpty(lngList)) {
			List<Long> camps = campagnaDao.prendiIdCampagnaUser(user);
			Iterator<Campagna> lnt = lngList.iterator();
			while (lnt.hasNext()) {
				Campagna campagna = lnt.next();
				CodaManager codaManager = Chiamate.getCodaProcessi().get(campagna);
				if(codaManager != null && camps.contains(campagna.getId())) {
					AutodialerContatti ad = codaManager.getAutodialerContatti();
					list.update(ad.getList());
				}
			}
		}
		as.setList(list);
		return as;
	}
	public synchronized void resetContattiStatistiche(Utente user) {
		Set<Campagna> lngList = Chiamate.getCodaProcessi().keySet();
		if(ApplicationUtil.notEmpty(lngList)) {
			List<Long> camps = campagnaDao.prendiIdCampagnaUser(user);
			Iterator<Campagna> lnt = lngList.iterator();
			while (lnt.hasNext()) {
				Campagna campagna = lnt.next();
				CodaManager codaManager = Chiamate.getCodaProcessi().get(campagna);
				if(codaManager != null && camps.contains(campagna.getId())) {
					codaManager.getAutodialerStatistiche().resetAll();
					codaManager.getAutodialerContatti().resetAll();
				}
			}
		}
	}
	public synchronized void lasciaContattoCampagna(Utente user) {	
		Set<Campagna> lngList = Chiamate.getCodaProcessi().keySet();
		if(ApplicationUtil.notEmpty(lngList)) {
			List<Long> camps = campagnaDao.prendiIdCampagnaUser(user);
			Iterator<Campagna> lnt = lngList.iterator();
			while (lnt.hasNext()) {
				Campagna campagna = lnt.next();
				CodaManager codaManager = Chiamate.getCodaProcessi().get(campagna);
				if(codaManager != null && camps.contains(campagna.getId())) {
					Chiamate.getCodaAutodialer().remove(campagna);
					CodaStatic.getChiamateInCorso().remove(campagna);
					codaManager.clear();
				}
			}
		}
	}
	public synchronized Contatto getContattoAutodialer(Utente user, Long id) {
		LoggerContatto l = new LoggerContatto(impostazioneDao.getLog());
		long time = System.currentTimeMillis();
		Contatto contatto = null;
		AutoContatto auto = CodaStatic.getChiamateInCorso().remove(id);
		if (auto != null) {
			contatto = auto.getContatto();
			contatto.setAutoContatto(auto);
			EsitoChiamata esito = auto.getAutodialer().getEsitoApertura();
			if(esito != null) {
				contatto.setAutodialerApertura(true);
				pannelloDao.esitaAutodialer(false, auto.getAutodialer().getSave_storia(), contatto, user, esito);
			} 
			Chiamate.getCodaContatti().put(contatto.getId(), new InfoContatto(time, user,contatto, false));
		}
		l.log("aperto autodialer", user, contatto);
		new ApriContattoThread(this, operazioneDao, auto, user, time).start();
		return contatto;
	}
	public synchronized void addUser(Utente user, Campagna campagna) {
		Integer priorita = Chiamate.getCodaOperatori().hasPrecedenza(user, campagna);
		Long attesa = System.currentTimeMillis();
		user.setAttesa(attesa);
		InfoUtente utente = new InfoUtente(user, attesa, false);
		CodaStatic.getCodaTeam().rimuoviUtenteT(utente);
		CodaStatic.getCodaTeam().addUser(campagna, utente, priorita);
		Chiamate.getCodaInterni().stato(user.getInternoString(), true);
	}

	public synchronized void addUser(Utente user, Campagna campagna, Contatto contatto, Boolean auto) {
		if(contatto == null || !auto) {
			Integer priorita = Chiamate.getCodaOperatori().hasPrecedenza(user, campagna);
			Long attesa = System.currentTimeMillis();
			user.setAttesa(attesa);
			InfoUtente utente = new InfoUtente(user, attesa, false);
			CodaStatic.getCodaTeam().addUser(campagna, utente, priorita);
			Chiamate.getCodaInterni().stato(user.getInternoString(), true);
		}
	}
	public synchronized void endorse(Utente user, Campagna campagna) {
		Integer priorita = Chiamate.getCodaOperatori().hasPrecedenza(user, campagna);
		long time = System.currentTimeMillis();
		InfoUtente utente = new InfoUtente(user, time, false);
		CodaStatic.getCodaTeam().addUser(campagna, utente, priorita);
	}

	public synchronized void update(Utente user, Campagna campagna) {
		if(campagna != null) {
			CodaStatic.getCodaTeam().update(campagna, user);
			CodaManager codaManager = Chiamate.getCodaProcessi().get(campagna);
			if(codaManager != null) {
				codaManager.setData(ApplicationUtil.date());
			}
		}
	}
	public synchronized Boolean updateReady(Utente user, Campagna campagna, Contatto contatto) {
		return CodaStatic.getCodaTeam().updateReady(contatto, campagna, user); 
	}
	public synchronized Boolean ready(Utente user, Campagna campagna, Contatto contatto) {
		return CodaStatic.getCodaTeam().ready(contatto, campagna, user); 
	}
	public Long getTempoMedio(Campagna campagna){
		if(campagna!=null && campagna.getAutodialer() !=null && campagna.getAutodialer().getCalcola_media()) {
			CodaManager codaManager = Chiamate.getCodaProcessi().get(campagna);
			if(codaManager != null) {
				Long tm = codaManager.getAutodialerStatistiche().getMediaGestione() - codaManager.getAutodialerStatistiche().getMediaChiamata();
				if(ApplicationUtil.gtZero(tm)) {
					return tm;
				} 
			}
		}
		return Long.valueOf(0);
	}
	public AutodialerReport contaCaricato(Campagna campagna) {
		AutodialerReport ad = new AutodialerReport();
		if(campagna != null) {
			CodaManager codaManager = Chiamate.getCodaProcessi().get(campagna);
			if(codaManager != null) {
				codaManager.updateReport(ad, codaManager);
				ad.setCampagna(campagna);
				ad.setSospeso(codaManager.isSospeso());
				ad.setAvviato(codaManager.isAvviato());
				ad.setCaricato(codaManager.isCaricato());
				ad.setEsaurito(codaManager.isEsaurito());
				ad.setStarted(codaManager.isStarted());
				ad.setFinito(codaManager.isFinito());
				ad.setCriterio(codaManager.getCriterio());
				ad.setCorrezione(codaManager.getCorrezione());
			}
		}
		return ad;
	}
	public void timeoutUser() {
		CodaStatic.getCodaTeam().timeout();
	}
	public void timeoutAuto() {
		Date data = ApplicationUtil.date();
		List<AutoContatto> contatti = CodaStatic.getChiamateInCorso().timeout(data);
		if(ApplicationUtil.notEmpty(contatti)) {
			Iterator<AutoContatto> lnt = contatti.iterator();
			while (lnt.hasNext()) {
				AutoContatto auto = lnt.next();
				if(auto!=null) {
					Contatto contatto = auto.getContatto();
					CodaStatic.getChiamateInCorso().remove(contatto.getId());
					auto.setStato(ConstantsMap.CHIAMATA_RIMOSSA);
					Autodialer autoD = auto.getAutodialer();
					esitaContatto(auto, autoD.getEsitoHangup(), false);
					CodaManager codaManager = Chiamate.getCodaProcessi().get(auto.getContatto().getCampagna());
					if(codaManager != null) {
						codaManager.getAutodialerStatistiche().incRimosse(data);
						codaManager.getAutodialerContatti().incRimosse(auto, data);
						codaManager.getChiamateInCorso().remove(contatto.getId());
						codaManager.getChiamateInAttesa().remove(contatto.getId());
					}
					CodaStatic.getAutodialerStatistiche().incRimosse(data);
					CodaStatic.getAutodialerContatti().incRimosse(auto,data);
				}
			}
		}
	}
	public Boolean moduleStatus(String modulo) {
		if(getAmiLogged()) {
			CommandResponse mr = null;
			try {
				CommandAction mm = new CommandAction("module show like " + modulo);
				Object a = managerConnection.sendAction(mm, getTimeoutAmi() * ConstantsGlobal.millis);
				if(a instanceof ManagerError) {
					//ManagerError m = (ManagerError) a;
				} else {
					mr = (CommandResponse) a;
				}
			} catch (IllegalArgumentException | IllegalStateException | IOException | TimeoutException e) {	}
			if(mr != null) {
				List<String> resp = mr.getResult();
				Object o = mr.getOutput();
				boolean a = ApplicationUtil.notEmpty(resp) && resp.toString().contains("Running");
				boolean b = o != null && o.toString().contains("Running");
				return a || b;
			}
		}

		return null;
	}
	
	public void sendMessage(List<SipBuddies> accounts, String from, String body) {
		for(SipBuddies account : accounts) {
			sendMessage(account.getUsername(), from, body);
		}
	}
	
	public void sendMessage(String to, String from, String body) {
		if(getAmiLogged()) {
			try {
				String link = impostazioneDao.getUrlBase();
				MessageSendAction mm = new MessageSendAction();
				boolean sip = true;
				if(sip) {
					String protocol = ApplicationUtil.formatMessages(getSip());
					mm.setTo(protocol+":" + to + "@" + link);
					mm.setFrom(protocol+":" + from);
				} else {
					String protocol = ConstantsGlobal.xmpp;
					mm.setTo(protocol+":" + to + "@" + link);
					mm.setFrom(protocol+":" + from);
				}
				mm.setBody(body);
			
				managerConnection.sendAction(mm, getTimeoutAmi() * ConstantsGlobal.millis);
			} catch (IllegalArgumentException | IllegalStateException | IOException | TimeoutException e) {	}
		}
	}
	
	public List<SelectItem> getTipoChat() {
		List<SelectItem> sisSI = new ArrayList<SelectItem>();
		if(ApplicationUtil.getService(ConstantsGlobal.service_prosody) || ApplicationUtil.getService(ConstantsGlobal.service_ejabberd) || ApplicationUtil.getService(ConstantsGlobal.service_openfire)) { 
			sisSI.add(new SelectItem(false, ConstantsGlobal.xmpp));
		}
		if(ApplicationUtil.getService(ConstantsGlobal.service_asterisk)) {
			sisSI.add(new SelectItem(true, ApplicationUtil.formatMessages(getSip())));
		}
		return sisSI;
	}
	
	public void loadModule(boolean load, String modulo) {
		if(getAmiLogged()) {
			try {
				ModuleLoadAction mm = new ModuleLoadAction(modulo, load ? ModuleLoadAction.LOAD_TYPE_LOAD : ModuleLoadAction.LOAD_TYPE_UNLOAD);
				managerConnection.sendAction(mm, getTimeoutAmi() * ConstantsGlobal.millis);
			} catch (IllegalArgumentException | IllegalStateException | IOException | TimeoutException e) {
			}
		}
	}

	public String reloadResponse() {
		reloadModule(null);
		return ConstantsLanguage.RICARICATO.stampa();
	}

	public void reloadModule(String modulo) {
		if(getAmiLogged()) {
			try {
				ModuleLoadAction mm = new ModuleLoadAction(modulo, ModuleLoadAction.LOAD_TYPE_RELOAD);
				managerConnection.sendAction(mm, getTimeoutAmi() * ConstantsGlobal.millis);
			} catch (IllegalArgumentException | IllegalStateException | IOException | TimeoutException e) {
			}
		}
	}
	public void sincronizza(ConfigAsterisk c) {
		sincronizza(c.getChiave(), c.getValore());
	}
	public void sincronizza(String key, Boolean v) {
		String modulo = getModulo(key);
		Boolean m = moduleStatus(modulo);
		if(m != null && !v.equals(m)) {
			loadModule(v, modulo);
		}
	}
	public String getModulo(String key) {
		return key.substring(key.lastIndexOf(".")+1, key.length());
	}
	public Boolean getValore(String key) {
		if(Chiamate.getCodaAsterisk().contains(key)) {
			return Chiamate.getCodaAsterisk().get(key);
		} else { 
			ConfigAsterisk conf = getValue(key);
			Boolean valore = conf.getValore();
			Chiamate.getCodaAsterisk().put(key, valore);
			return valore;
		}
	}
	public ConfigAsterisk salva(ConfigAsterisk config) {
		String chiave = config.getChiave();
		Boolean valore = config.getValore();
		sincronizza(chiave, valore);
		Chiamate.getCodaAsterisk().put(chiave, valore);
		return (ConfigAsterisk) salvaAggiorna(config);
	}
	public void elimina(ConfigAsterisk config) {
		String chiave = config.getChiave();
		if(Chiamate.getCodaAsterisk().contains(chiave)) {
			Chiamate.getCodaAsterisk().remove(chiave);
		}
		cancellaAggiorna(config);
	}
	public ConfigAsterisk getValue(String key) {
		ConfigAsterisk conf = getValoreByChiave(key);
		if (conf == null) {
			conf = DefaultsAsterisk.DEFAULTS.get(key);
			try {
				salva(conf);
			} catch (Exception e) {	}
		} 
		return conf;
	}
	private ConfigAsterisk getValoreByChiave(String chiave) {
		DetachedCriteria criteria = DetachedCriteria.forClass(ConfigAsterisk.class);
		criteria.add(Restrictions.eq("chiave", ApplicationUtil.trimToEmpty(chiave)));
		return (ConfigAsterisk) getUniqueResult(criteria);
	}
	@SuppressWarnings("unchecked")
	public void eliminaNotInConfig(List<String> chiavi) {
		if(ApplicationUtil.notEmpty(chiavi)) {
			DetachedCriteria criteria = DetachedCriteria.forClass(ConfigAsterisk.class);
			criteria.add(Restrictions.not(Restrictions.in("chiave", chiavi)));
			List<ConfigAsterisk> c = getList(criteria);
			cancellaLista(c);
		}
	}
	private ExtendedCriteria getConfigByCriteria(ConfigurazioneCriteria conf) {
		if(conf != null) conf = (ConfigurazioneCriteria) conf.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(ConfigAsterisk.class);
		if(ApplicationUtil.isNotBlank(conf.getNome())) {
			ApplicationUtil.chiave(ConstantsGlobal.colonSeparator, conf.getNome(), criteria, "chiave");
		} 
		criteria.add(Restrictions.eq("visibile", Boolean.TRUE));
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, conf);
	}
	@SuppressWarnings("unchecked")
	public List<ConfigAsterisk> trova(ConfigurazioneCriteria conf, int start, int size) {
		ExtendedCriteria criteria = getConfigByCriteria(conf);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("chiave")});
		List<ConfigAsterisk> lst = getList(criteria, start, size);
		if(ApplicationUtil.notEmpty(lst)) {
			Iterator<ConfigAsterisk> it = lst.iterator();
			while (it.hasNext()) {
				ConfigAsterisk ca = (ConfigAsterisk) it.next();
				String modulo = getModulo(ca.getChiave());
				Boolean m = moduleStatus(modulo);
				ca.setStato(m);
			}
		}
		return lst;
	}
	public int trovaSize(ConfigurazioneCriteria conf) {
		ExtendedCriteria criteria = getConfigByCriteria(conf);
		return getListSize(criteria);
	}
	
	public boolean lanciaChiamata(CodaManager codaManager) throws Exception {
		AutoContatto autoContatto = codaManager.getChiamateAttive().remove();
		if(autoContatto != null) {
			Date data = ApplicationUtil.date();
			autoContatto.incTentativi();
			autoContatto.setData_chiamata(data);
			boolean ch = avviaChiamata(autoContatto);
			if(ch) {
				autoContatto.setStato(ConstantsMap.CHIAMATA_INCORSO);
				CodaStatic.getChiamateInCorso().put(autoContatto);
				codaManager.getChiamateInCorso().put(autoContatto);
			} else {
				autoContatto.setStato(ConstantsMap.CHIAMATA_SALTATA);
				codaManager.getAutodialerStatistiche().incSaltate(data);
				codaManager.getAutodialerContatti().incSaltate(autoContatto, data);
				CodaStatic.getAutodialerStatistiche().incSaltate(data);
				CodaStatic.getAutodialerContatti().incSaltate(autoContatto, data);
				new SalvaContattoThread(this, autoContatto, autoContatto.getAutodialer().getEsitoSaltata()).start();
			}
			codaManager.getAutodialerStatistiche().incTentate(data);
			codaManager.getAutodialerContatti().incTentate(autoContatto, data);
			CodaStatic.getAutodialerStatistiche().incTentate(data);
			CodaStatic.getAutodialerContatti().incTentate(autoContatto, data);
			return true;
		}
		return false;
	}  

	public boolean avviaChiamata(AutoContatto auto) throws Exception {
		Contatto c = auto.getContatto();
		Autodialer autod = auto.getAutodialer();
		SipBuddies sp = autod.getTrunk();
		String telefono = ApplicationUtil.compila(auto.getTelefono(), autod.getAppend_prefisso());
		if(ApplicationUtil.isNotBlank(telefono)) {
			String tel = ApplicationUtil.uniforma(telefono, autod.getRem_prefisso(), autod.getAdd_prefisso());
			ParameterAutodialer pa = new ParameterAutodialer(sp, c.getCallers(), c.getMusicclass());
			String validatore = (pa.getInterurbane() ? impostazioneDao.validatore_interurbana1() : impostazioneDao.validatore_interurbana2());
			Pattern p = Pattern.compile(validatore);
			Matcher m = p.matcher(telefono);
			if(m.matches()){
				boolean opp = pa.getOpposizione() && isOpposizione(telefono);
				if(!opp) {
					boolean a = getSip();
					String contatto = c.getId().toString();
					String cmp = c.getCampagna().getId().toString();
					String userfield = c.getUserfield();
					String caller = pa.getAntecall() + pa.getCallerId();
					String telE = ApplicationUtil.uniformaF(tel, pa.getPrefisso());
					String mc = pa.getMusicclass();
					String vtx = pa.getVtx().toString(); 
					String vrx = pa.getVrx().toString(); 
					String utente = null;
					OriginateAction originateAction = new OriginateAction();
					originateAction.setChannel(ApplicationUtil.formatCallee(a, pa.getTrunk(), telE));
					originateAction.setContext(autod.getContext() != null ? autod.getContext() : ConstantsGlobal.autocall);
					originateAction.setCallerId(caller);
					//originateAction.setApplication(ConstantsGlobal.appData);
					if(ApplicationUtil.notEmpty(pa.getCodecs())) {
						originateAction.setCodecs(pa.getCodecs());
					}
					originateAction.setEarlyMedia(false);
					originateAction.setPriority(ConstantsGlobal.priority);
					originateAction.setExten(ConstantsGlobal.extensionS);
					ApplicationUtil.setVariable(true, originateAction, ConstantsGlobal.CLIENTE, telefono);
					ApplicationUtil.setVariable(true, originateAction, ConstantsGlobal.CAMPAGNA, cmp);
					ApplicationUtil.setVariable(true, originateAction, ConstantsGlobal.ESTENSIONE, caller);
					ApplicationUtil.setVariable(true, originateAction, ConstantsGlobal.UTENTE, ApplicationUtil.toZero(utente));
					ApplicationUtil.setVariable(true, originateAction, ConstantsGlobal.VTX, vtx);
					ApplicationUtil.setVariable(true, originateAction, ConstantsGlobal.VRX, vrx);
					ApplicationUtil.setVariable(true, originateAction, ConstantsGlobal.MONITOR, ApplicationUtil.evalZero(pa.getRegistrazione()));
					ApplicationUtil.setVariable(true, originateAction, ConstantsGlobal.OPPOSIZIONE, ApplicationUtil.evalZero(pa.getOpposizione()));
					ApplicationUtil.setVariable(true, originateAction, ConstantsGlobal.TRUNKS, ApplicationUtil.formatTrunk(a, pa.getTrunk(), ""));
					ApplicationUtil.setVariable(true, originateAction, ConstantsGlobal.PREFISSO, pa.getPrefisso());
					ApplicationUtil.setVariable(true, originateAction, ConstantsGlobal.CONTATTO, contatto);
					ApplicationUtil.setVariable(true, originateAction, ConstantsGlobal.MUSICA, mc);
					ApplicationUtil.setVariable(true, originateAction, ConstantsGlobal.USERFIELD, userfield);
					ApplicationUtil.setVariable(true, originateAction, ConstantsGlobal.CALLERS, caller);
					ApplicationUtil.setVariable(true, originateAction, ConstantsGlobal.NUMERO, "0");
					originateAction.setAccount(contatto);
					auto.impostaCD1(caller, telefono);
					ApplicationUtil.impostaCDR(originateAction, mc, vtx, vrx, caller, telefono, telefono, utente, contatto, userfield, false);
					originateAction.setActionId(contatto);
					originateAction.setTimeout(autod.getTimeoutAmi() * ConstantsGlobal.millis);
					originateAction.setAsync(true);
					ManagerResponse mn = managerConnection.sendAction(originateAction, autod.getTimeoutAmi() * ConstantsGlobal.millis);
					if(mn != null && ApplicationUtil.trimToEmpty(mn.getResponse()).equalsIgnoreCase(ConstantsGlobal.SUCCESS)) {
						return true;
					} else {
						auto.addCause(ConstantsLanguage.ERRORE.stampa());
					}
				} else {
					auto.addCause(ConstantsLanguage.OPPOSIZIONE.stampa());
				}
			} else {
				auto.addCause(ConstantsLanguage.NON_CONSENTITO.stampa());
			}
		} else {
			auto.addCause(ConstantsLanguage.VUOTO.stampa());
		}
		return false;
	}

	public ManagerResponse originateCommand(Utente user, String telefono, String richiama, String c, String ingresso, String context) {
		if(user.isLocale() && ApplicationUtil.isNotEmpty(telefono)) {
			if(getAmiLogged()) {
				Long id = ApplicationUtil.asLong(c);
				Contatto cont = ApplicationUtil.gtZero(id) ? getContatto(id) : null;
				String contatto = cont != null ? ApplicationUtil.asString(cont.getId()) : "";
				String campagna = cont != null ? ApplicationUtil.asString(cont.getCampagna().getId()) : "";
				String userfield = cont != null ? ApplicationUtil.asString(cont.getUserfield()) : "";
				OriginateAction originateAction = new OriginateAction();
				boolean a = getSip();
				String caller = "";
				String utente = user.getId().toString();
				List<String> codecs = null;
				SipBuddies interno = user.getInterno();
				ParameterAutodialer ra = new ParameterAutodialer(user.getTrunk(), interno.getCallers(), null);
				boolean opp = ra.getOpposizione() && (isOpposizione(telefono) || isOpposizione(richiama));
				if(opp) {
					return null;
				}
				String v = impostazioneDao.validatore_interno();
				if(ApplicationUtil.isInterno(telefono, v)) {
					caller = impostaCaller(ingresso, ApplicationUtil.isNotEmpty(richiama) ? richiama : user.getInternoString(), contatto, campagna);
					originateAction.setChannel(ApplicationUtil.formatCallee(a, "", telefono));
					codecs = ApplicationUtil.token(interno.getAllow(), ConstantsGlobal.colonSeparator);
				} else {
					caller = ra.getAntecall() + ra.getCallerId();
					originateAction.setChannel(ApplicationUtil.formatCallee(a, ra.getTrunk(), ApplicationUtil.uniformaF(telefono, ra.getPrefisso())));
					codecs = ra.getCodecs();
				} 
				if(ApplicationUtil.notEmpty(codecs)) {
					originateAction.setCodecs(codecs);
				}
				originateAction.setCallerId(caller);
				originateAction.setContext(ApplicationUtil.isNotBlank(context) ? context : ConstantsGlobal.manualcall);
				//originateAction.setApplication(ConstantsGlobal.appData);
				String n = "";
				String d = "";
				if(ApplicationUtil.isNotEmpty(richiama)) {
					if(ApplicationUtil.isInterno(richiama, v)) {
						d = impostaCaller(ingresso, telefono, contatto, campagna);
						n = richiama;
					} else {
						d = ra.getAntecall() + ra.getCallerId();
						n = ApplicationUtil.formatTrunk(a, ra.getTrunk(), ApplicationUtil.uniformaF(richiama, ra.getPrefisso()));
					}
				} else {
					d = impostaCaller(ingresso, telefono, contatto, campagna);
					n = user.getInternoString();
				}
				String mc = ra.getMusicclass();
				String vtx = ra.getVtx().toString(); 
				String vrx = ra.getVrx().toString(); 
				originateAction.setEarlyMedia(false);
				originateAction.setPriority(ConstantsGlobal.priority);
				originateAction.setExten(ConstantsGlobal.extensionS);
				ApplicationUtil.setVariable(true, originateAction, ConstantsGlobal.CLIENTE, telefono);
				ApplicationUtil.setVariable(true, originateAction, ConstantsGlobal.CAMPAGNA, ApplicationUtil.toZero(campagna));
				ApplicationUtil.setVariable(true, originateAction, ConstantsGlobal.ESTENSIONE, caller);
				ApplicationUtil.setVariable(true, originateAction, ConstantsGlobal.UTENTE, utente);
				ApplicationUtil.setVariable(true, originateAction, ConstantsGlobal.VTX, vtx);
				ApplicationUtil.setVariable(true, originateAction, ConstantsGlobal.VRX, vrx);
				ApplicationUtil.setVariable(true, originateAction, ConstantsGlobal.MONITOR, ApplicationUtil.evalZero(ra.getRegistrazione() || user.getRegistrazione()));
				ApplicationUtil.setVariable(true, originateAction, ConstantsGlobal.OPPOSIZIONE, ApplicationUtil.evalZero(ra.getOpposizione() || user.getOpposizione()));
				ApplicationUtil.setVariable(true, originateAction, ConstantsGlobal.TRUNKS, ApplicationUtil.formatTrunk(a, ra.getTrunk(), ""));
				ApplicationUtil.setVariable(true, originateAction, ConstantsGlobal.PREFISSO, user.getPrefisso() + ra.getPrefisso());
				ApplicationUtil.setVariable(true, originateAction, ConstantsGlobal.CONTATTO, ApplicationUtil.toZero(contatto));
				ApplicationUtil.setVariable(true, originateAction, ConstantsGlobal.MUSICA, mc);
				ApplicationUtil.setVariable(true, originateAction, ConstantsGlobal.USERFIELD, ApplicationUtil.toZero(userfield));
				ApplicationUtil.setVariable(true, originateAction, ConstantsGlobal.CALLERS, d);
				ApplicationUtil.setVariable(true, originateAction, ConstantsGlobal.NUMERO, n);
				ApplicationUtil.impostaCDR(originateAction, mc, vrx, vrx, caller, telefono, telefono, utente, contatto, userfield, true);
				originateAction.setActionId(telefono);
				originateAction.setTimeout(getTimeoutAmi() * ConstantsGlobal.millis);
				originateAction.setAsync(true);
				try {
					return (ManagerResponse) managerConnection.sendAction(originateAction, getTimeoutAmi() * ConstantsGlobal.millis);
				} catch (IllegalArgumentException | IllegalStateException | IOException | TimeoutException e) {
				}
			}
		}
		return null;
	}

	private String impostaCaller(String ingresso, String telefono, String campagna, String contatto) {
		if(ApplicationUtil.isBlank(ingresso)) {
			return ConstantsGlobal.remotaSMB + telefono;
		} else {
			return ConstantsGlobal.ingressoSMB + (ApplicationUtil.isNotEmpty(contatto) ? (contatto) : (telefono + "["+ingresso+"]" + campagna));
		}
	}
	
	public void sendOpposizione(Opposizione opposizione, boolean ins) {
		if(opposizione.getCampagna() == null) {
			String valore = opposizione.getOpposizione();
			send("opposizione", valore, valore, ins);
		}
	}

	public void sincronizzaOpposizione(List<String> valori) {
		clear("opposizione");
		if(ApplicationUtil.notEmpty(valori)) {
			Iterator<String> valIt = valori.iterator();
			while (valIt.hasNext()) {
				String opposizione = (String) valIt.next();
				send("opposizione", opposizione, opposizione, true);
			}
		}
	}

	public void sendInterno(SipBuddies interno, boolean ins) {
		send("interno", interno.getName(), interno.getName(), ins);
	}


	@SuppressWarnings("unchecked")
	public void sincronizzaInterno() {
		clear("interno");
		DetachedCriteria criteria = DetachedCriteria.forClass(SipBuddies.class);
		criteria.add(Restrictions.eq("tipologia", ApplicationUtil.convertBS(true)));
		criteria.setProjection(Projections.distinct(Projections.property("name")));
		List<String> lis = getList(criteria);
		if(ApplicationUtil.notEmpty(lis)) {
			Iterator<String> itCont = lis.iterator();
			while (itCont.hasNext()) {
				String sip = (String) itCont.next();
				send("interno", sip, sip, true);
			}
		}
	}

	public void sendInbound(Inbound inbound, boolean ins) {
		send("extensionI", inbound.getNome(), inbound.getMusicclass() != null ? inbound.getMusicclass() : ConstantsGlobal.musicclass, ins);
	}

	public void sincronizzaInbound(List<Inbound> valori) {
		clear("extensionI");
		if(ApplicationUtil.notEmpty(valori)) {
			Iterator<Inbound> valIt = valori.iterator();
			while (valIt.hasNext()) {
				Inbound inbound = (Inbound) valIt.next();
				send("extensionI", inbound.getNome(), inbound.getMusicclass() != null ? inbound.getMusicclass() : ConstantsGlobal.musicclass, true);
			}
		}
	}


	public void sendRisposta(Risposta risposta, boolean ins) {
		send("extensionR", risposta.getNome(), risposta.getMusicclass() != null ? risposta.getMusicclass() : ConstantsGlobal.musicclass, ins);
	}

	public void sincronizzaRisposta(List<Risposta> valori) {
		clear("extensionR");
		if(ApplicationUtil.notEmpty(valori)) {
			Iterator<Risposta> valIt = valori.iterator();
			while (valIt.hasNext()) {
				Risposta risposta = (Risposta) valIt.next();
				send("extensionR", risposta.getNome(), risposta.getMusicclass() != null ? risposta.getMusicclass() : ConstantsGlobal.musicclass, true);
			}
		}
	}

	public void chiudiContatto(AutoContatto auto, Integer stato, EsitoChiamata esito) {
		auto.setStato(stato);
		esitaContatto(auto, esito, true);
	}


	@SuppressWarnings("unchecked")
	public Map<String, DatoAggiuntivo> loadValori(Contatto contatto) {
		if(contatto != null) {
			DetachedCriteria criteria = DetachedCriteria.forClass(DatoAggiuntivoAnagrafica.class);
			criteria.add(Restrictions.eq("anagrafica.id",contatto.getAnagrafica().getId()));
			List<DatoAggiuntivoAnagrafica> dettagli = getList(criteria);
			List <String> valoriValidi = new ArrayList<String>(ConstantsMap.anagraficaMap);
			Map<String, DatoAggiuntivo> valori = ApplicationUtil.convertiAnagrafica(contatto.getAnagrafica(), valoriValidi);
			if(ApplicationUtil.notEmpty(dettagli)) {
				valori.putAll(ApplicationUtil.convertiDatoAggiuntivoA(dettagli));
			}

			return valori;
		}
		return new HashMap<String, DatoAggiuntivo>();
	}

	public void intervalloInbound(AgiChannel chanel, List<IntervalloInbound> rules, Map<String, DatoAggiuntivo> valori, AutoContatto auto, String chS, String chD) throws AgiException {
		String home = getHome();
		for(IntervalloInbound aa : rules) {
			if(ApplicationUtil.isTrue(aa.getAttivo()) && ApplicationUtil.isTrue(aa.getStatoAttiva())) {
				ApplicationUtil.riproduciSuono(chanel, home, aa.getMusica_iniziale());
				int ripetizioni = aa.getRipetizioni();
				String premuto = aa.getValide();
				String exit = null;
				for(int i = 0; i < ripetizioni; i++) {
					Character e = ApplicationUtil.riproduciSuono(chanel, home, aa.getMusica_ripetuta(), premuto);
					exit = ApplicationUtil.convertC(e);
					if(ApplicationUtil.isNotBlank(exit)) {
						if(premuto.contains(exit)) {
							chanel.streamFile(ConstantsGlobal.path_sondaggio + ConstantsGlobal.path_audio_1);
							break;
						} else {
							chanel.streamFile(ConstantsGlobal.path_sondaggio + ConstantsGlobal.path_audio_2);
							IVRUtil ivr = aa.getIVR("i"); 
							boolean b = ApplicationUtil.executeIVR(chanel, home, ivr);
							if(b) {
								break;
							} else {
								ApplicationUtil.sleep(sleepSuono());
							}
						}
					}
					if(ApplicationUtil.isBlank(exit)) {
						ApplicationUtil.sleep(sleepSuono());
					}
				}
				if(ApplicationUtil.isNotBlank(exit) && premuto.contains(exit)) {
					IVRUtil ivr = aa.getIVR(exit);
					ApplicationUtil.executeIVR(chanel, home, ivr);
				}
				String app = aa.getApp();
				String callers = ApplicationUtil.replaceValori(chanel, aa.getCallers(), valori);
				String pre = ApplicationUtil.replaceValori(chanel, aa.getPre_caller(), valori);
				String post = ApplicationUtil.replaceValori(chanel, aa.getPost_caller(), valori);
				String option = ApplicationUtil.replaceValori(chanel, aa.getNumero(), valori);
				CallUtil ca = new CallUtil(app, callers, pre, post, option, chS);
				if(auto != null) {
					auto.impostaCD2(ca.getChiamanteN(), ca.getChiamatoN());
				}
				chanel.setCallerId(ca.getChiamanteN());
				ApplicationUtil.riproduciSuono(chanel, home, aa.getMusica_finale());
				if(ca.getAppN() != null) {
					if(ca.getOptionN() != null) {
						chanel.exec(ca.getAppN(), ca.getOptionN());
					} else {
						chanel.exec(ca.getAppN());
					}
				}
			}
		}
	}
	
	private void send(String proprieta, String chiave, Object valore, boolean ins) {
		if(ApplicationUtil.isNotBlank(chiave)) {
			if(getAmiLogged()) {
				String cmd = null;
				if(ins) {
					cmd = "database put" + " " + proprieta + " " + chiave + " " + (valore != null ? valore.toString() : "");
				} else {
					cmd = "database del" + " " + proprieta + " " + chiave;
				}
				CommandAction mm = new CommandAction(cmd);
				try {
					managerConnection.sendAction(mm, getTimeoutAmi() * ConstantsGlobal.millis);
				} catch (IllegalArgumentException | IllegalStateException | IOException | TimeoutException e) {
				}
			}
		}
	}

	private void clear(String proprieta) {
		if(getAmiLogged()) {
			try {
				String cmd = "database deltree" + " " + proprieta;
				CommandAction mm = new CommandAction(cmd);
				managerConnection.sendAction(mm, getTimeoutAmi() * ConstantsGlobal.millis);
			} catch (IllegalArgumentException | IllegalStateException | IOException | TimeoutException e) {
			}
		}
	}
	
	public boolean isVariabile() {
		return impostazioneDao.autodialeVariabile();
	}

	public boolean isStatico() {
		return impostazioneDao.autodialeStatico();
	}
	public Integer sleepSuono() {
		return impostazioneDao.sleepSuono();
	}
	public Integer sleepAbbattuta() {
		return impostazioneDao.sleepAbbattuta();
	}
	public Integer sleepReady() {
		return impostazioneDao.sleepReady();
	}
	public Integer sleepConsumatore() {
		return impostazioneDao.sleepConsumatore();
	}
	public Integer sleepProduttore() {
		return impostazioneDao.sleepProduttore();
	}
	public Integer sleepSchedulato() {
		return impostazioneDao.sleepSchedulato();
	}
	public Integer sleepSospendi() {
		return impostazioneDao.sleepSospendi();
	}

	public boolean isAsterisk() {
		if(impostazioneDao != null) {
			String s = impostazioneDao.autodialerCentralino();
			if(s != null && ApplicationUtil.getService(ConstantsGlobal.service_asterisk)) {
				return true;
			}
		}
		return false;
	}
	public void service() {
		if(impostazioneDao != null) {
			boolean start = impostazioneDao.autodialerControlla();
			if(start) {
				List<StatoServer> fws = impostazioneDao.getService(ConstantsGlobal.service_asterisk);
				if(ApplicationUtil.notEmpty(fws)) {
					for(StatoServer st : fws) {
						if(ApplicationUtil.getService(st.getService()) && !st.getOk()) {
							ApplicationUtil.executeRuntime(new String[]{ConstantsGlobal.path_shell, "-c", "systemctl" + " " + "restart" + " " + st.getService()});
						}
					}
				}
			}
		}
	}

	public Contatto salvaContatto(String tipo, boolean nuovaAnagrafica, String telefono, Long camp, EsitoChiamata esito, String url) {
		return contattoDao.salvaContatto(tipo, nuovaAnagrafica, telefono, camp, esito, url);
	}

	public Contatto updateContatto(Contatto contatto, Utente utente, EsitoChiamata esito) {
		pannelloDao.salvaContattoC(false, null, contatto, utente, esito, false);
		return contatto;
	}
	public Contatto getContatto(String telefono, Long camp, Boolean pubbliche, Boolean soloAperte, String tipo) {
		return contattoDao.getContatto(telefono, camp, pubbliche, soloAperte, tipo);
	}
	public Contatto getContatto(Long id) {
		return contattoDao.getContatto(id);
	}
	public List<Campagna> getProcessiInbound1(String telefono) {
		List<Campagna> cl = new ArrayList<Campagna>();
		Set<Campagna> lngList = Chiamate.getCodaProcessi().keySet();
		if(ApplicationUtil.notEmpty(lngList)) {
			Iterator<Campagna> lnt = lngList.iterator();
			while (lnt.hasNext()) {
				Campagna campagna = lnt.next();
				if(campagna != null) {
					CodaManager codaManager = Chiamate.getCodaProcessi().get(campagna);
					if(codaManager != null) {
						Date date2 = ApplicationUtil.date();
						Date date1 = codaManager.getData();
						if(date1 != null) {
							Long secondi = (date2.getTime()-date1.getTime())/ConstantsGlobal.millis;
							if(campagna.getAutodialer().getEsitoInbound() != null && !codaManager.isSospeso() && codaManager.isAvviato() && secondi.intValue() < 20) {
								boolean b = telefono != null && campagna.getInbound() != null && ApplicationUtil.token(campagna.getInbound(), ConstantsGlobal.colonSeparator).contains(telefono);
								if(telefono == null || campagna.getInbound() == null || b) {
									cl.add(codaManager.getCampagna());
								}
							}
						}
					}
				}
			}
		}
		return cl;
	}
	public List<Campagna> getProcessiInbound2(String telefono) {
		List<Campagna> cl = new ArrayList<Campagna>();
		Set<Campagna> lngList = Chiamate.getCodaProcessi().keySet();
		if(ApplicationUtil.notEmpty(lngList)) {
			Iterator<Campagna> lnt = lngList.iterator();
			while (lnt.hasNext()) {
				Campagna campagna = lnt.next();
				if(campagna != null) {
					CodaManager codaManager = Chiamate.getCodaProcessi().get(campagna);
					if(codaManager != null) {
						if(campagna.getAutodialer().getEsitoInbound() != null) {
							boolean b = telefono != null && campagna.getInbound() != null && ApplicationUtil.token(campagna.getInbound(), ConstantsGlobal.colonSeparator).contains(telefono);
							if(telefono == null || campagna.getInbound() == null || b) {
								cl.add(codaManager.getCampagna());
							}
						}
					}
				}
			}
		}
		return cl;
	}

	
	public List<InfoSpia> getListaSpia() {
		return Chiamate.getCodaSpia().list();
	}

	public Inbound getInbound(String nome) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Inbound.class);
		criteria.add(Restrictions.eq("nome", nome));
		return (Inbound) getUniqueResult(criteria);
	}

	public Risposta getRisposta(String nome, boolean c) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Risposta.class);
		criteria.add(Restrictions.eq("nome", nome));
		if(c) {
			criteria.setFetchMode("campagne", FetchMode.JOIN);
		}
		return (Risposta) getUniqueResult(criteria);
	}

	public long getTimeoutAmi(){
		String s = impostazioneDao.timeoutCentralino();
		if(s != null) {
			return Long.valueOf(s);
		} else {
			return ConstantsGlobal.timeoutAMI;
		}
	}

	public void sondaggio(String contattoId, String utenteId, String votoS, String userfield){
		contattoDao.sondaggio(contattoId, utenteId, votoS, userfield);
	}

	public void hangupCause(Long contattoId, Integer cause) {
		AutoContatto autoContatto = CodaStatic.getChiamateInCorso().get(contattoId);
		if(autoContatto!=null) {
			autoContatto.addCause(cause);
		}
	}
	
	public void peerStatus(PeerStatusEvent evento) {
		String stato = evento.getPeerStatus();
		String key = evento.getPeer();
		String caller = caller(key);
		Chiamate.getCodaTelefono().put(caller, stato);
		if(stato != null && caller != null) {
			if(stato.equalsIgnoreCase(ConstantsGlobal.REGISTERED) || stato.equalsIgnoreCase(ConstantsGlobal.REACHABLE)) {
				Chiamate.getCodaInterni().accesso(caller, true);
			} else {
				Chiamate.getCodaInterni().accesso(caller, false);
			}
		}
	}

	public void spyStart(ChanSpyStartEvent evento) {
		String uniqueId = evento.getSpyerUniqueId();
		InfoSpia is = new InfoSpia(uniqueId, evento.getSpyerCallerIdNum(), evento.getSpyerCallerIdName(), evento.getSpyerExten());
		Chiamate.getCodaSpia().put(uniqueId, is);
	}

	public void spyStop(ChanSpyStopEvent evento) {
		String uniqueId = evento.getSpyerUniqueId();
		Chiamate.getCodaSpia().remove(uniqueId);
	}

	public void aggiungiCodaU(ManagerEvent event) {
		QueueUtente utente = null;
		String coda = null;
		String interno = null;
		if (event instanceof QueueMemberStatusEvent) {
			QueueMemberStatusEvent evento = (QueueMemberStatusEvent) event;
			coda = evento.getQueue();
			interno = ApplicationUtil.channel(evento.getInterface());
			utente = new QueueUtente(coda, interno, evento.getCallsTaken(), evento.getMembership(),	evento.getPaused(), evento.getRinginuse(), evento.getStatus());
		}
		if (event instanceof QueueMemberPauseEvent) {
			QueueMemberPauseEvent evento = (QueueMemberPauseEvent) event;
			coda = evento.getQueue();
			interno = ApplicationUtil.channel(evento.getInterface());
			utente = new QueueUtente(coda, interno, evento.getCallsTaken(), evento.getMembership(),	evento.getPaused(), evento.getRinginuse(), evento.getStatus());
		}
		if (event instanceof QueueMemberAddedEvent) {
			QueueMemberAddedEvent evento = (QueueMemberAddedEvent) event;
			coda = evento.getQueue();
			interno = ApplicationUtil.channel(evento.getInterface());
			utente = new QueueUtente(coda, interno, evento.getCallsTaken(), evento.getMembership(),	evento.getPaused(), evento.getRinginuse(), evento.getStatus());
		}
		if(coda != null && interno != null && interno != null) {
			Chiamate.getCodaQueue().putU(coda, interno, utente);
		}
	}

	public void aggiungiCodaC(ManagerEvent event) {
		QueueChiamata chiamata = null;
		String coda = null;
		String uniqueId = null;
		if (event instanceof QueueCallerJoinEvent) {
			QueueCallerJoinEvent evento = (QueueCallerJoinEvent) event;
			coda = evento.getQueue();
			uniqueId = evento.getUniqueId();
			chiamata = new QueueChiamata(coda, uniqueId, evento.getCallerIdNum(), evento.getCount(), evento.getConnectedLineNum(), evento.getPosition(), evento.getChannelStateDesc());
		}
		if(coda != null && uniqueId != null && chiamata != null) {
			Chiamate.getCodaQueue().putC(coda, uniqueId, chiamata);
		}
	}
	public void rimuoviCodaC(ManagerEvent event) {
		String coda = null;
		String uniqueId = null;
		if (event instanceof QueueEvent) {
			QueueEvent evento = (QueueEvent) event;
			coda = evento.getQueue();
			uniqueId = evento.getUniqueId();
		}
		if(coda != null && uniqueId != null) {
			Chiamate.getCodaQueue().removeC(coda, uniqueId);
		}
	}
	
	public void rimuoviCodaU(ManagerEvent event) {
		String coda = null;
		String interno = null;
		if (event instanceof AbstractQueueMemberEvent) {
			AbstractQueueMemberEvent evento = (AbstractQueueMemberEvent) event;
			coda = evento.getQueue();
			interno = ApplicationUtil.channel(evento.getInterface());
		}
		if(coda != null && interno != null) {
			Chiamate.getCodaQueue().removeU(coda, interno);
		}
	}

	public void eventDialBegin(DialBeginEvent evento) {
		String line = ApplicationUtil.trimToEmpty(evento.getCallerIdNum());
		if(!line.startsWith(ConstantsGlobal.remotaSMB)) {
			if(!(line.startsWith(ConstantsGlobal.autodialerSMB) || line.startsWith(ConstantsGlobal.inboundSMB))) {
				String uniqueId = evento.getDestUniqueId();
				Long contattoId = ApplicationUtil.contatto(evento.getDestAccountCode());
				if(contattoId != null) {
					AutoContatto autoContatto = CodaStatic.getChiamateInCorso().get(contattoId);
					if(autoContatto!=null) {
						autoContatto.impostaCU(uniqueId, null);
						autoContatto.setDial(true);
					}
				}
			}
		}
	}

	public void eventDialEnd(DialEndEvent evento) {
		String uniqueId = evento.getDestUniqueId();
		String reason = evento.getDialStatus();
		String line = ApplicationUtil.trimToEmpty(evento.getCallerIdNum());
		if(!line.startsWith(ConstantsGlobal.remotaSMB)) {
			if(!(line.startsWith(ConstantsGlobal.autodialerSMB) || line.startsWith(ConstantsGlobal.inboundSMB))) {
				if(uniqueId != null && reason != null && !(reason.equals(ConstantsGlobal.ANSWER) || reason.equals(ConstantsGlobal.CONTINUE) || reason.equals(ConstantsGlobal.GOTO))) {
					Long contattoId = ApplicationUtil.contatto(evento.getDestAccountCode());
					if(contattoId != null) {
						if(CodaStatic.getChiamateInCorso().dialStatus(contattoId)) {
							AutoContatto autoContatto = CodaStatic.getChiamateInCorso().remove(contattoId);
							if(autoContatto!=null) {
								autoContatto.impostaCU(uniqueId, null);
								Date data = ApplicationUtil.date();
								autoContatto.setData_rifiuto(data);
								Contatto contatto = autoContatto.getContatto();
								CodaManager codaManager = Chiamate.getCodaProcessi().get(contatto.getCampagna());
								if(ApplicationUtil.isTrue(autoContatto.getRinging())) {
									gestisciRinging(autoContatto, codaManager, contattoId, reason, data);
								} else {
									if(ApplicationUtil.equals(reason, ConstantsGlobal.CANCEL)) {
										gestisciRifiutata(autoContatto, codaManager, contattoId, reason, data);
									} else if(ApplicationUtil.equals(reason, ConstantsGlobal.BUSY)) {
										gestisciOccupato(autoContatto, codaManager, contattoId, reason, data);
									} else if(ApplicationUtil.equals(reason, ConstantsGlobal.CONGESTION)) {
										gestisciCongestion(autoContatto, codaManager, contattoId, reason, data);
									} else if(ApplicationUtil.equals(reason, ConstantsGlobal.NOANSWER)) {
										gestisciNonRisposto(autoContatto, codaManager, contattoId, reason, data);
									} else {
										gestisciFallita(autoContatto, codaManager, contattoId, reason, data);
									}
								}
							}
						}
					}
				}
			}
		}
	}


	public void eventHangupRequest(HangupRequestEvent evento) {
		Long contattoId = ApplicationUtil.contatto(evento.getAccountCode());
		if(contattoId != null) {
			if(CodaStatic.getChiamateInCorso().hangupStatus(contattoId)) {
				AutoContatto autoContatto = CodaStatic.getChiamateInCorso().remove(contattoId);
				if(autoContatto!=null) {
					Date data = ApplicationUtil.date();
					autoContatto.setData_aggancio(data);
					Contatto contatto = autoContatto.getContatto();
					esitoOperatore(autoContatto, ConstantsMap.CHIAMATA_HANGUP, evento.getCause());
					CodaManager codaManager = Chiamate.getCodaProcessi().get(contatto.getCampagna());
					if(codaManager != null) {
						codaManager.getAutodialerStatistiche().incHangup(data);
						codaManager.getAutodialerContatti().incHangup(autoContatto, data);
						codaManager.getChiamateInCorso().remove(contatto.getId());
						codaManager.getChiamateInAttesa().remove(contatto.getId());
					}
					CodaStatic.getAutodialerStatistiche().incHangup(data);
					CodaStatic.getAutodialerContatti().incHangup(autoContatto, data);
				}
			} else {
				hangupCause(contattoId, evento.getCause());
			}
		}
	}

	public void eventSoftHangupRequest(SoftHangupRequestEvent evento) {
		Long contattoId = ApplicationUtil.contatto(evento.getAccountCode());
		if(contattoId != null) {
			if(CodaStatic.getChiamateInCorso().hangupStatus(contattoId)) {
				AutoContatto autoContatto = CodaStatic.getChiamateInCorso().remove(contattoId);
				if(autoContatto!=null) {
					Date data = ApplicationUtil.date();
					autoContatto.setData_aggancio(data);
					Contatto contatto = autoContatto.getContatto();
					esitoOperatore(autoContatto, ConstantsMap.CHIAMATA_CHIUSURA, evento.getCause());
					CodaManager codaManager = Chiamate.getCodaProcessi().get(contatto.getCampagna());
					if(codaManager != null) {
						codaManager.getAutodialerStatistiche().incScartate(data);
						codaManager.getAutodialerContatti().incScartate(autoContatto, data);
						codaManager.getChiamateInCorso().remove(contatto.getId());
						codaManager.getChiamateInAttesa().remove(contatto.getId());
					}
					CodaStatic.getAutodialerStatistiche().incScartate(data);
					CodaStatic.getAutodialerContatti().incScartate(autoContatto, data);
				}
			} else {
				hangupCause(contattoId, evento.getCause());
			}
		}
	}


	public void eventHangup(HangupEvent evento) {
		Long contattoId = ApplicationUtil.contatto(evento.getAccountCode());
		if(contattoId != null) {
			if(CodaStatic.getChiamateInCorso().hangupStatus(contattoId)) {
				AutoContatto autoContatto = CodaStatic.getChiamateInCorso().remove(contattoId);
				if(autoContatto!=null) {
					Date data = ApplicationUtil.date();
					autoContatto.setData_aggancio(data);
					Contatto contatto = autoContatto.getContatto();
					esitoOperatore(autoContatto, ConstantsMap.CHIAMATA_AGGANCIATA, evento.getCause());
					CodaManager codaManager = Chiamate.getCodaProcessi().get(contatto.getCampagna());
					if(codaManager != null) {
						codaManager.getAutodialerStatistiche().incAgganciate(data);
						codaManager.getAutodialerContatti().incAgganciate(autoContatto, data);
						codaManager.getChiamateInCorso().remove(contatto.getId());
						codaManager.getChiamateInAttesa().remove(contatto.getId());
					}
					CodaStatic.getAutodialerStatistiche().incAgganciate(data);
					CodaStatic.getAutodialerContatti().incAgganciate(autoContatto, data);
				}
			} else {
				hangupCause(contattoId, evento.getCause());
			}
		}
	} 


	public void eventNewState(NewStateEvent evento) {
		String stato = ApplicationUtil.trimToNull(evento.getChannelStateDesc());
		String line = ApplicationUtil.trimToEmpty(evento.getConnectedLineNum());
		String channel = evento.getChannel();
		if(stato != null && !line.startsWith(ConstantsGlobal.remotaSMB)){
			if(stato.equalsIgnoreCase(ConstantsGlobal.STATUS_RINGING) && !(line.startsWith(ConstantsGlobal.autodialerSMB) || line.startsWith(ConstantsGlobal.inboundSMB))) {
				Long contattoId = ApplicationUtil.contatto(evento.getAccountCode());
				if(contattoId != null) {
					AutoContatto autoContatto = CodaStatic.getChiamateInCorso().get(contattoId);
					if(autoContatto!=null) {
						autoContatto.setRinging(true);
						Contatto contatto = autoContatto.getContatto();
						CodaManager codaManager = Chiamate.getCodaProcessi().get(contatto.getCampagna());
						if(codaManager != null) {
							codaManager.getChiamateInCorso().put(autoContatto);
						}
					}
				}
			}
			if(stato.equalsIgnoreCase(ConstantsGlobal.STATUS_UP) && (line.startsWith(ConstantsGlobal.autodialerSMB) || line.startsWith(ConstantsGlobal.inboundSMB))) {
				Long contattoId = ApplicationUtil.contatto(evento.getAccountCode());
				if(contattoId != null) {
					String uniqueId = evento.getUniqueId();
					String caller = caller(channel);
					if(caller != null){
						AutoContatto autoContatto = CodaStatic.getChiamateInCorso().get(contattoId);
						if(autoContatto!=null){
							autoContatto.impostaCU(uniqueId, evento.getLinkedId());
							autoContatto.setData_link(ApplicationUtil.date());
							autoContatto.setStato(ConstantsMap.CHIAMATA_UP);
							Chiamate.getCodaInterni().stato(caller, false);
						}
					} 
				}
			}
		}
	}


	public void eventOriginateResponseFailure(OriginateResponseEvent evento) {
		Long id = ApplicationUtil.contatto(evento.getActionId());
		if(id != null) {
			String uniqueId = evento.getUniqueId();
			AutoContatto autoContatto = CodaStatic.getChiamateInCorso().remove(id);
			if(autoContatto!=null) {
				int reason = ApplicationUtil.asZero(evento.getReason());
				autoContatto.impostaCU(uniqueId, null);
				Date data = ApplicationUtil.date();
				autoContatto.setData_rifiuto(data);
				Contatto contatto = autoContatto.getContatto();
				CodaManager codaManager = Chiamate.getCodaProcessi().get(contatto.getCampagna());
				String cause = evento.getResponse() + " " + "(" + reason + ")";
				if(ApplicationUtil.isTrue(autoContatto.getRinging())) {
					gestisciRinging(autoContatto, codaManager, id, cause, data);
				} else {
					if(ApplicationUtil.eq(reason, 1)) {
						gestisciRifiutata(autoContatto, codaManager, id, cause, data);
					} else if(ApplicationUtil.eq(reason, 5)) {
						gestisciOccupato(autoContatto, codaManager, id, cause, data);
					} else if(ApplicationUtil.eq(reason, 8)) {
						gestisciCongestion(autoContatto, codaManager, id, cause, data);
					} else if(ApplicationUtil.eq(reason, 3)) {
						gestisciNonRisposto(autoContatto, codaManager, id, cause, data);
					} else {
						gestisciFallita(autoContatto, codaManager, id, cause, data);
					}
				}
			}
		}
	}
	
	
	private void gestisciRifiutata(AutoContatto autoContatto, CodaManager codaManager, Long contattoId, String cause, Date data) {
		Autodialer auto = autoContatto.getAutodialer();
		gestisciContatto(autoContatto, auto.getFrecRifiutata(), auto.getMaxRifiutata(), ConstantsMap.CHIAMATA_RIFIUTATA, auto.getEsitoRifiutata(), cause);
		if(codaManager != null) {
			codaManager.getAutodialerStatistiche().incRifiutato(data);
			codaManager.getAutodialerContatti().incRifiutato(autoContatto, data);
			codaManager.getChiamateInCorso().remove(contattoId);
		}
		CodaStatic.getAutodialerStatistiche().incRifiutato(data);
		CodaStatic.getAutodialerContatti().incRifiutato(autoContatto, data);
	}
	
	private void gestisciOccupato(AutoContatto autoContatto, CodaManager codaManager, Long contattoId, String cause, Date data) {
		Autodialer auto = autoContatto.getAutodialer();
		gestisciContatto(autoContatto, auto.getFrecOccupato(), auto.getMaxOccupato(), ConstantsMap.CHIAMATA_OCCUPATA, auto.getEsitoOccupato(), cause);
		if(codaManager != null) {
			codaManager.getAutodialerStatistiche().incOccupato(data);
			codaManager.getAutodialerContatti().incOccupato(autoContatto, data);
			codaManager.getChiamateInCorso().remove(contattoId);
		}
		CodaStatic.getAutodialerStatistiche().incOccupato(data);
		CodaStatic.getAutodialerContatti().incOccupato(autoContatto, data);
	}
	
	private void gestisciCongestion(AutoContatto autoContatto, CodaManager codaManager, Long contattoId, String cause, Date data) {
		Autodialer auto = autoContatto.getAutodialer();
		gestisciContatto(autoContatto, auto.getFrecCongestion(), auto.getMaxCongestion(), ConstantsMap.CHIAMATA_CONGESTIONE, auto.getEsitoCongestion(), cause);
		if(codaManager != null) {
			codaManager.getAutodialerStatistiche().incCongestion(data);
			codaManager.getAutodialerContatti().incCongestion(autoContatto, data);
			codaManager.getChiamateInCorso().remove(contattoId);
		}
		CodaStatic.getAutodialerStatistiche().incCongestion(data);
		CodaStatic.getAutodialerContatti().incCongestion(autoContatto, data);
	}
	
	private void gestisciNonRisposto(AutoContatto autoContatto, CodaManager codaManager, Long contattoId, String cause, Date data) {
		Autodialer auto = autoContatto.getAutodialer();
		gestisciContatto(autoContatto, auto.getFrecNonRisposto(), auto.getMaxNonRisposto(), ConstantsMap.CHIAMATA_NONRISPOSTA, auto.getEsitoNonRisposto(), cause);
		if(codaManager != null) {
			codaManager.getAutodialerStatistiche().incNonRisposto(data);
			codaManager.getAutodialerContatti().incNonRisposto(autoContatto, data);
			codaManager.getChiamateInCorso().remove(contattoId);
		}
		CodaStatic.getAutodialerStatistiche().incNonRisposto(data);
		CodaStatic.getAutodialerContatti().incNonRisposto(autoContatto, data);
	}
	
	private void gestisciFallita(AutoContatto autoContatto, CodaManager codaManager, Long contattoId, String cause, Date data) {
		Autodialer auto = autoContatto.getAutodialer();
		gestisciContatto(autoContatto, auto.getFrecFallita(), auto.getMaxFallita(), ConstantsMap.CHIAMATA_FALLITA, auto.getEsitoFallita(), cause);
		if(codaManager != null) {
			codaManager.getAutodialerStatistiche().incFallito(data);
			codaManager.getAutodialerContatti().incFallito(autoContatto, data);
			codaManager.getChiamateInCorso().remove(contattoId);
		}
		CodaStatic.getAutodialerStatistiche().incFallito(data);
		CodaStatic.getAutodialerContatti().incFallito(autoContatto, data);
	}
	
	private void gestisciRinging(AutoContatto autoContatto, CodaManager codaManager, Long contattoId, String cause, Date data) {
		Autodialer auto = autoContatto.getAutodialer();
		gestisciContatto(autoContatto, auto.getFrecRinging(), auto.getMaxRinging(), ConstantsMap.CHIAMATA_RINGING, auto.getEsitoRinging(), cause);
		if(codaManager != null) {
			codaManager.getAutodialerStatistiche().incRinging(data);
			codaManager.getAutodialerContatti().incRinging(autoContatto, data);
			codaManager.getChiamateInCorso().remove(contattoId);
		}
		CodaStatic.getAutodialerStatistiche().incRinging(data);
		CodaStatic.getAutodialerContatti().incRinging(autoContatto, data);
	}
	

	public void eventOriginateResponseSuccess(OriginateResponseEvent evento) {
		Long id = ApplicationUtil.contatto(evento.getActionId());
		if(id != null) {
			String uniqueId = evento.getUniqueId();
			AutoContatto autoContatto = CodaStatic.getChiamateInCorso().get(id);
			if(autoContatto!=null) {
				autoContatto.setHangup(true);
				autoContatto.impostaCU(uniqueId, null);
				Date data = ApplicationUtil.date();
				autoContatto.setData_risposta(data);
				autoContatto.setStato(ConstantsMap.CHIAMATA_RISPOSTA);
				Contatto contatto = autoContatto.getContatto();
				CodaManager codaManager = Chiamate.getCodaProcessi().get(contatto.getCampagna());
				if(codaManager != null) {
					codaManager.getChiamateInCorso().put(autoContatto);
					codaManager.getAutodialerStatistiche().incRisposte(data);
					codaManager.getAutodialerStatistiche().addTimeChiamata(autoContatto.getData_chiamata(), autoContatto.getData_risposta());
					codaManager.getAutodialerContatti().incRisposte(autoContatto, data);
				}
				CodaStatic.getAutodialerStatistiche().incRisposte(data);
				CodaStatic.getAutodialerStatistiche().addTimeChiamata(autoContatto.getData_chiamata(), autoContatto.getData_risposta());
				CodaStatic.getAutodialerContatti().incRisposte(autoContatto, data);
			}
		}
	}
	
	public void bridge(BridgeEnterEvent evento) {
	}

	public void bridge(BridgeLeaveEvent evento) {
		if(ApplicationUtil.eq(evento.getBridgeNumChannels(), 1)) {
			Integer sleep = impostazioneDao.sleepChiusura();
			if(ApplicationUtil.gtZero(sleep)) {
				String close = ApplicationUtil.channel(evento.getChannel());
				if(close != null) {
					String uniqueid = evento.getLinkedId();
					ApplicationUtil.sleep(sleep);
					String hql = "update Cdr set close = :close where uniqueid = :uniqueid";
					execute(false, hql, new Parameter("close", close), new Parameter("uniqueid", uniqueid));
				}
			}
		}
	}

	public void preparaContatto(Contatto contatto, String chS, String chD, String recall_callers, String recall_musicclass, AiutoCampagna script) {
		pannelloDao.preparaContatto(contatto, chS, chD, recall_callers, recall_musicclass, script);
	}

}