package it.callite.dao;
import java.io.ByteArrayOutputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

import javax.faces.model.SelectItem;

import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import it.callite.action.DownloadBean;
import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.constants.ConstantsMap;
import it.callite.criteria.CalendarioCriteria;
import it.callite.dao.generic.GenericDao;
import it.callite.log.ErrorLog;
import it.callite.model.Agente;
import it.callite.model.Appuntamento;
import it.callite.model.Campagna;
import it.callite.model.Nota;
import it.callite.model.Utente;
import it.callite.model.Zona;
import it.callite.utility.Allegato;
import it.callite.utility.Annotazione;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.CalendarioAgenteVO;
import it.callite.utility.GiornoCalendarioAgenteVO;
import it.callite.utility.Orario;
import it.callite.utility.item.CampagnaSI;
import it.callite.utility.item.ZonaSI;
@SuppressWarnings("serial")
@Repository("calendarioDao")
public class CalendarioDao extends GenericDao {
	@Autowired
	private AppuntamentoDao appuntamentoDao;
	@Autowired
	private DownloadDao downloadDao;
	@Autowired
	private UserDao userDao;
	@Autowired
	private CampagnaDao campagnaDao;
	@Autowired
	private PannelloDao pannelloDao;
	@Autowired
	private ImpostazioneDao impostazioneDao;
	@Autowired
	private OperazioneDao operazioneDao;
	public void salvaAppuntamento(Appuntamento app, Utente utente) {
		pannelloDao.salvaAppuntamento(app, utente, false);
	}
	public void salvaNota(Nota nota, Utente utente) {
		pannelloDao.salvaNota(nota, utente);
	}
	public List<GiornoCalendarioAgenteVO> ricercaCalendarioManager(Utente utente, CalendarioCriteria appCr) {
		List<String> sis = new ArrayList<String>();
		for(int i = appCr.getInizioA(); i <= appCr.getFineA(); i++) {
			sis.add(String.format("%02d", i, 0));
		}
		Date attuale = ApplicationUtil.date();
		Agente agente = campagnaDao.getAgente(appCr.getAgenteId());
		List<GiornoCalendarioAgenteVO> calendario = new ArrayList<GiornoCalendarioAgenteVO>();
		if (agente != null) {
			Date inizio = ApplicationUtil.inizioGiornata(appCr.getGiornoPartenza());
			Date fine = ApplicationUtil.fineGiornata(ApplicationUtil.addDays(inizio, appCr.getGiorni()));
			for(int i = 0; i < appCr.getGiorni(); i++) {
				Date giorno_calendario = ApplicationUtil.addDays(inizio, i);
				GiornoCalendarioAgenteVO gc = new GiornoCalendarioAgenteVO();
				gc.setAppuntamenti(new ArrayList<Appuntamento>());
				gc.setNote(new ArrayList<Annotazione>());
				gc.setData(giorno_calendario);
				gc.setAgente(agente);
				gc.setOrari(sis);
				gc.setGiorniCalendario(appCr.getGiorni());
				calendario.add(gc);
			}
			List<Appuntamento> appuntamenti = ricercaCalendarioListAppuntamenti(inizio, fine, agente.getId(), appCr);
			List<Nota> note = trovaListaNote(inizio, fine, agente.getId());
			List<Orario> orari = new ArrayList<Orario>();
			Orario orario = new Orario();
			orario.setData(attuale);
			orari.add(orario);
			appuntamentoDao.completaCalendario(calendario, appuntamenti, note, orari, appCr.getInizioA(), appCr.getFineA(), attuale);
			if(ApplicationUtil.notEmpty(appuntamenti))  {
				String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.APPUNTAMENTO.stampa() +": " + ApplicationUtil.size(appuntamenti), appuntamenti);
				operazioneDao.creaOperazione(ConstantsMap.LOG4, p ,utente, ConstantsMap.OPERAZIONE_VISUALIZZA_APPUNTAMENTO);
			}
		}
		return calendario;
	}
	public List<GiornoCalendarioAgenteVO> ricercaCalendarioCoordinatore(Utente utente, CalendarioCriteria appCr) {
		List<String> sis = new ArrayList<String>();
		for(int i = appCr.getInizioA(); i <= appCr.getFineA(); i++) {
			sis.add(String.format("%02d", i, 0));
		}
		Date attuale = ApplicationUtil.date();
		Agente agente = campagnaDao.getAgente(appCr.getAgenteId());
		List<GiornoCalendarioAgenteVO> calendario = new ArrayList<GiornoCalendarioAgenteVO>();
		if (agente != null) {
			Date inizio = ApplicationUtil.inizioGiornata(appCr.getGiornoPartenza());
			Date fine = ApplicationUtil.fineGiornata(ApplicationUtil.addDays(inizio, appCr.getGiorni()));
			for(int i = 0; i < appCr.getGiorni(); i++) {
				Date giorno_calendario = ApplicationUtil.addDays(inizio, i);
				GiornoCalendarioAgenteVO gc = new GiornoCalendarioAgenteVO();
				gc.setAppuntamenti(new ArrayList<Appuntamento>());
				gc.setNote(new ArrayList<Annotazione>());
				gc.setData(giorno_calendario);
				gc.setAgente(agente);
				gc.setOrari(sis);
				gc.setGiorniCalendario(appCr.getGiorni());
				calendario.add(gc);
			}
			List<Appuntamento> appuntamenti = ricercaCalendarioCoordinatoreAppuntamenti(utente, inizio, fine, agente.getId(), appCr);
			List<Nota> note = trovaListaNote(inizio, fine, agente.getId());
			List<Orario> orari = new ArrayList<Orario>();
			Orario orario = new Orario();
			orario.setData(attuale);
			orari.add(orario);
			appuntamentoDao.completaCalendario(calendario, appuntamenti, note, orari, appCr.getInizioA(), appCr.getFineA(), attuale);
			if(ApplicationUtil.notEmpty(appuntamenti))  {
				String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.APPUNTAMENTO.stampa() +": " + ApplicationUtil.size(appuntamenti), appuntamenti);
				operazioneDao.creaOperazione(ConstantsMap.LOG4, p ,utente, ConstantsMap.OPERAZIONE_VISUALIZZA_APPUNTAMENTO);
			}
		}
		return calendario;
	}
	public void getRiepilogoAgenteAgente(Utente utente, CalendarioCriteria app, Date ora, DownloadBean downloadBean) {
		String html = impostazioneDao.getFormatoDocumento();
		String s = ConstantsGlobal.secretL;
		String p = impostazioneDao.getPasswordZIP();
		List<Agente> agenti = new ArrayList<Agente>();
		agenti.add(userDao.getAgente(app.getAgenteId()));
		List<Appuntamento> appuntamenti = new ArrayList<Appuntamento>();
		Iterator<Agente> objIter = agenti.iterator();
		while(objIter.hasNext()) {
			Agente agente = objIter.next();
			Date now = ApplicationUtil.inizioGiornata(ora);
			appuntamenti.addAll(ricercaCalendarioAgenteAppuntamenti(now, ApplicationUtil.addDays(now, 1), agente.getId(), app));
		}
		List<Allegato> files = new ArrayList<Allegato>();
		Iterator<Appuntamento> objAppIter = appuntamenti.iterator();
		if (ApplicationUtil.notEmpty(appuntamenti)) {
			String extM = (s == null || (html.equalsIgnoreCase(ConstantsMap.pdf))) ? html : ConstantsMap.zip;
			while(objAppIter.hasNext()) {
				Appuntamento appuntamento = objAppIter.next();
				try {
					String nomeFile = ApplicationUtil.reportFile(appuntamento.getData_inizio(), ConstantsLanguage.APPUNTAMENTO.stampa(), appuntamento.getId(), appuntamento.getContatto().getAnagrafica().getDenominazione())  + "." + extM;
					boolean vs = appuntamento.getAgente() != null && ApplicationUtil.isTrue(appuntamento.getAgente().getVisualizzaStoria());
					ByteArrayOutputStream result = downloadDao.appuntamentoToHTMLString(utente, appuntamento,  vs, true, html, true, false, s);
					result.close();
					ApplicationUtil.addFile(files, nomeFile, result.toByteArray(), appuntamento.getData_creazione().getTime());
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				}
			}
			String pg = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.APPUNTAMENTO.stampa() +": " + ApplicationUtil.size(appuntamenti), appuntamenti);
			operazioneDao.creaOperazione(ConstantsMap.LOG4, pg ,utente, ConstantsMap.OPERAZIONE_EXPORT_APPUNTAMENTO);
		}
		try {
			if (ApplicationUtil.empty(files)) {
				ApplicationUtil.fileWriter(files, ConstantsLanguage.NO_APPUNTAMENTO.stampa(), ConstantsLanguage.NO_APPUNTAMENTI.stampa());
			}
			String fileName = ApplicationUtil.reportFile(ora, ConstantsLanguage.APPUNTAMENTI.stampa()) + "." + ConstantsMap.zip;
			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(fileName, files);
			ApplicationUtil.zip(null,files, out, p);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		} 
	}
	public void getRiepilogoAgenteRete(Utente utente, CalendarioCriteria app, Date ora, DownloadBean downloadBean) {
		String html = impostazioneDao.getFormatoDocumento();
		String s = ConstantsGlobal.secretL;
		String p = impostazioneDao.getPasswordZIP();
		List<Agente> agenti = new ArrayList<Agente>();
		agenti.add(userDao.getAgente(app.getAgenteId()));
		List<Appuntamento> appuntamenti = new ArrayList<Appuntamento>();
		Iterator<Agente> objIter = agenti.iterator();
		while(objIter.hasNext()) {
			Agente agente = objIter.next();
			Date now = ApplicationUtil.inizioGiornata(ora);
			appuntamenti.addAll(ricercaCalendarioAgenteAppuntamenti(now, ApplicationUtil.addDays(now, 1), agente.getId(), app));
		}
		List<Allegato> files = new ArrayList<Allegato>();
		Iterator<Appuntamento> objAppIter = appuntamenti.iterator();
		if (ApplicationUtil.notEmpty(appuntamenti)) {
			String extM = (s == null || (html.equalsIgnoreCase(ConstantsMap.pdf))) ? html : ConstantsMap.zip;
			while(objAppIter.hasNext()) {
				Appuntamento appuntamento = objAppIter.next();
				try {
					String nomeFile = ApplicationUtil.reportFile(appuntamento.getData_inizio(), ConstantsLanguage.APPUNTAMENTO.stampa(), appuntamento.getId(), appuntamento.getContatto().getAnagrafica().getDenominazione())  + "." + extM;
					boolean vs = appuntamento.getAgente() != null && ApplicationUtil.isTrue(appuntamento.getAgente().getVisualizzaStoria());
					ByteArrayOutputStream result = downloadDao.appuntamentoToHTMLString(utente, appuntamento,  vs, true, html, true, false, s);
					result.close();
					ApplicationUtil.addFile(files, nomeFile, result.toByteArray(), appuntamento.getData_creazione().getTime());
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				}
			}
			String pg = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.APPUNTAMENTO.stampa() +": " + ApplicationUtil.size(appuntamenti), appuntamenti);
			operazioneDao.creaOperazione(ConstantsMap.LOG4, pg ,utente, ConstantsMap.OPERAZIONE_EXPORT_APPUNTAMENTO);
		}
		try {
			if (ApplicationUtil.empty(files)) {
				ApplicationUtil.fileWriter(files, ConstantsLanguage.NO_APPUNTAMENTO.stampa(), ConstantsLanguage.NO_APPUNTAMENTI.stampa());
			}
			String fileName = ApplicationUtil.reportFile(ora, ConstantsLanguage.APPUNTAMENTI.stampa()) + "." + ConstantsMap.zip;
			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(fileName, files);
			ApplicationUtil.zip(null,files, out, p);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		} 
	}
	public List<GiornoCalendarioAgenteVO> ricercaCalendarioPartner(Utente utente, CalendarioCriteria appCr) {
		List<String> sis = new ArrayList<String>();
		for(int i = appCr.getInizioA(); i <= appCr.getFineA(); i++) {
			sis.add(String.format("%02d", i, 0));
		}
		Date attuale = ApplicationUtil.date();
		Agente agente = campagnaDao.getAgente(appCr.getAgenteId());
		List<GiornoCalendarioAgenteVO> calendario = new ArrayList<GiornoCalendarioAgenteVO>();
		if (agente != null) {
			Date inizio = ApplicationUtil.inizioGiornata(appCr.getGiornoPartenza());
			Date fine = ApplicationUtil.fineGiornata(ApplicationUtil.addDays(inizio, appCr.getGiorni()));
			for(int i = 0; i < appCr.getGiorni(); i++) {
				Date giorno_calendario = ApplicationUtil.addDays(inizio, i);
				GiornoCalendarioAgenteVO gc = new GiornoCalendarioAgenteVO();
				gc.setAppuntamenti(new ArrayList<Appuntamento>());
				gc.setNote(new ArrayList<Annotazione>());
				gc.setData(giorno_calendario);
				gc.setAgente(agente);
				gc.setOrari(sis);
				gc.setGiorniCalendario(appCr.getGiorni());
				calendario.add(gc);
			}
			List<Appuntamento> appuntamenti = ricercaCalendarioListAppuntamentiPartner(utente.getId(), inizio, fine, agente.getId(), appCr);
			List<Nota> note = trovaListaNote(inizio, fine, agente.getId());
			List<Orario> orari = new ArrayList<Orario>();
			Orario orario = new Orario();
			orario.setData(attuale);
			orari.add(orario);
			appuntamentoDao.completaCalendario(calendario, appuntamenti, note, orari, appCr.getInizioA(), appCr.getFineA(), attuale);
			if(ApplicationUtil.notEmpty(appuntamenti))  {
				String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.APPUNTAMENTO.stampa() +": " + ApplicationUtil.size(appuntamenti), appuntamenti);
				operazioneDao.creaOperazione(ConstantsMap.LOG4, p ,utente, ConstantsMap.OPERAZIONE_VISUALIZZA_APPUNTAMENTO);
			}
		}
		return calendario;
	}
	public List<GiornoCalendarioAgenteVO> ricercaCalendarioSupervisore(Utente utente, CalendarioCriteria appCr) {
		List<String> sis = new ArrayList<String>();
		for(int i = appCr.getInizioA(); i <= appCr.getFineA(); i++) {
			sis.add(String.format("%02d", i, 0));
		}
		Date attuale = ApplicationUtil.date();
		Agente agente = campagnaDao.getAgente(appCr.getAgenteId());
		List<GiornoCalendarioAgenteVO> calendario = new ArrayList<GiornoCalendarioAgenteVO>();
		if (agente != null) {
			Date inizio = ApplicationUtil.inizioGiornata(appCr.getGiornoPartenza());
			Date fine = ApplicationUtil.fineGiornata(ApplicationUtil.addDays(inizio, appCr.getGiorni()));
			for(int i = 0; i < appCr.getGiorni(); i++) {
				Date giorno_calendario = ApplicationUtil.addDays(inizio, i);
				GiornoCalendarioAgenteVO gc = new GiornoCalendarioAgenteVO();
				gc.setAppuntamenti(new ArrayList<Appuntamento>());
				gc.setNote(new ArrayList<Annotazione>());
				gc.setData(giorno_calendario);
				gc.setAgente(agente);
				gc.setOrari(sis);
				gc.setGiorniCalendario(appCr.getGiorni());
				calendario.add(gc);
			}
			List<Appuntamento> appuntamenti = ricercaCalendarioSupervisoreAppuntamenti(utente, inizio, fine, agente.getId(), appCr);
			List<Nota> note = trovaListaNote(inizio, fine, agente.getId());
			List<Orario> orari = new ArrayList<Orario>();
			Orario orario = new Orario();
			orario.setData(attuale);
			orari.add(orario);
			appuntamentoDao.completaCalendario(calendario, appuntamenti, note, orari, appCr.getInizioA(), appCr.getFineA(), attuale);
			if(ApplicationUtil.notEmpty(appuntamenti))  {
				String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.APPUNTAMENTO.stampa() +": " + ApplicationUtil.size(appuntamenti), appuntamenti);
				operazioneDao.creaOperazione(ConstantsMap.LOG4, p ,utente, ConstantsMap.OPERAZIONE_VISUALIZZA_APPUNTAMENTO);
			}
		}
		return calendario;
	}
	public List<GiornoCalendarioAgenteVO> ricercaCalendarioOperatore(Utente utente, CalendarioCriteria appCr) {
		List<String> sis = new ArrayList<String>();
		for(int i = appCr.getInizioA(); i <= appCr.getFineA(); i++) {
			sis.add(String.format("%02d", i, 0));
		}
		Date attuale = ApplicationUtil.date();
		Agente agente = campagnaDao.getAgente(appCr.getAgenteId());
		List<GiornoCalendarioAgenteVO> calendario = new ArrayList<GiornoCalendarioAgenteVO>();
		if (agente != null) {
			Date inizio = ApplicationUtil.inizioGiornata(appCr.getGiornoPartenza());
			Date fine = ApplicationUtil.fineGiornata(ApplicationUtil.addDays(inizio, appCr.getGiorni()));
			for(int i = 0; i < appCr.getGiorni(); i++) {
				Date giorno_calendario = ApplicationUtil.addDays(inizio, i);
				GiornoCalendarioAgenteVO gc = new GiornoCalendarioAgenteVO();
				gc.setAppuntamenti(new ArrayList<Appuntamento>());
				gc.setNote(new ArrayList<Annotazione>());
				gc.setData(giorno_calendario);
				gc.setAgente(agente);
				gc.setOrari(sis);
				gc.setGiorniCalendario(appCr.getGiorni());
				calendario.add(gc);
			}
			List<Appuntamento> appuntamenti = ricercaCalendarioOperatoreAppuntamenti(utente.getId(), inizio, fine, agente.getId(), appCr);
			List<Nota> note = trovaListaNote(inizio, fine, agente.getId());
			List<Orario> orari = new ArrayList<Orario>();
			Orario orario = new Orario();
			orario.setData(attuale);
			orari.add(orario);
			appuntamentoDao.completaCalendario(calendario, appuntamenti, note, orari, appCr.getInizioA(), appCr.getFineA(), attuale);
			if(ApplicationUtil.notEmpty(appuntamenti))  {
				String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.APPUNTAMENTO.stampa() +": " + ApplicationUtil.size(appuntamenti), appuntamenti);
				operazioneDao.creaOperazione(ConstantsMap.LOG4, p ,utente, ConstantsMap.OPERAZIONE_VISUALIZZA_APPUNTAMENTO);
			}
		}
		return calendario;
	}
	public void getRiepilogoAgenteManager(Utente utente, CalendarioCriteria app, Date ora,  DownloadBean downloadBean) {
		String html = impostazioneDao.getFormatoDocumento();
		String s = ConstantsGlobal.secretL;
		String p = impostazioneDao.getPasswordZIP();
		List<Agente> agenti = new ArrayList<Agente>();
		agenti.add(userDao.getAgente(app.getAgenteId()));
		List<Appuntamento> appuntamenti = new ArrayList<Appuntamento>();
		Iterator<Agente> objIter = agenti.iterator();
		while(objIter.hasNext()) {
			Agente agente = objIter.next();
			Date now = ApplicationUtil.inizioGiornata(ora);
			appuntamenti.addAll(ricercaCalendarioListAppuntamenti(now, ApplicationUtil.addDays(now, 1), agente.getId(), app));
		}
		List<Allegato> files = new ArrayList<Allegato>();
		Iterator<Appuntamento> objAppIter = appuntamenti.iterator();
		if (ApplicationUtil.notEmpty(appuntamenti)) {
			String extM = (s == null || (html.equalsIgnoreCase(ConstantsMap.pdf))) ? html : ConstantsMap.zip;
			while(objAppIter.hasNext()) {
				Appuntamento appuntamento = objAppIter.next();
				try {
					String nomeFile = ApplicationUtil.reportFile(appuntamento.getData_inizio(), ConstantsLanguage.APPUNTAMENTO.stampa(), appuntamento.getId(), appuntamento.getContatto().getAnagrafica().getDenominazione())  + "." + extM;
					ByteArrayOutputStream result = downloadDao.appuntamentoToHTMLString(utente, appuntamento,  true, true, html, null, false, s);
					result.close();
					ApplicationUtil.addFile(files, nomeFile, result.toByteArray(), appuntamento.getData_creazione().getTime());
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				}
			}
			String pg = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.APPUNTAMENTO.stampa() +": " + ApplicationUtil.size(appuntamenti), appuntamenti);
			operazioneDao.creaOperazione(ConstantsMap.LOG4, pg ,utente, ConstantsMap.OPERAZIONE_EXPORT_APPUNTAMENTO);
		}
		try {
			if (ApplicationUtil.empty(files)) {
				ApplicationUtil.fileWriter(files, ConstantsLanguage.NO_APPUNTAMENTO.stampa(), ConstantsLanguage.NO_APPUNTAMENTI.stampa());
			}
			String fileName = ApplicationUtil.reportFile(ora, ConstantsLanguage.APPUNTAMENTI.stampa()) + "." + ConstantsMap.zip;
			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(fileName, files);
			ApplicationUtil.zip(null,files, out, p);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		} 
	}
	public void getRiepilogoAgenteCoordinatore(Utente utente, CalendarioCriteria app, Date ora,  DownloadBean downloadBean) {
		String html = impostazioneDao.getFormatoDocumento();
		String s = ConstantsGlobal.secretL;
		String p = impostazioneDao.getPasswordZIP();
		List<Agente> agenti = new ArrayList<Agente>();
		agenti.add(userDao.getAgente(app.getAgenteId()));
		List<Appuntamento> appuntamenti = new ArrayList<Appuntamento>();
		Iterator<Agente> objIter = agenti.iterator();
		while(objIter.hasNext()) {
			Agente agente = objIter.next();
			Date now = ApplicationUtil.inizioGiornata(ora);
			appuntamenti.addAll(ricercaCalendarioCoordinatoreAppuntamenti(utente, now, ApplicationUtil.addDays(now, 1), agente.getId(), app));
		}
		List<Allegato> files = new ArrayList<Allegato>();
		Iterator<Appuntamento> objAppIter = appuntamenti.iterator();
		if (ApplicationUtil.notEmpty(appuntamenti)) {
			String extM = (s == null || (html.equalsIgnoreCase(ConstantsMap.pdf))) ? html : ConstantsMap.zip;
			while(objAppIter.hasNext()) {
				Appuntamento appuntamento = objAppIter.next();
				try {
					String nomeFile = ApplicationUtil.reportFile(appuntamento.getData_inizio(), ConstantsLanguage.APPUNTAMENTO.stampa(), appuntamento.getId(), appuntamento.getContatto().getAnagrafica().getDenominazione())  + "." + extM;
					ByteArrayOutputStream result = downloadDao.appuntamentoToHTMLString(utente, appuntamento,  true, true, html, null, false, s);
					result.close();
					ApplicationUtil.addFile(files, nomeFile, result.toByteArray(), appuntamento.getData_creazione().getTime());
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				}
			}
			String pg = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.APPUNTAMENTO.stampa() +": " + ApplicationUtil.size(appuntamenti), appuntamenti);
			operazioneDao.creaOperazione(ConstantsMap.LOG4, pg ,utente, ConstantsMap.OPERAZIONE_EXPORT_APPUNTAMENTO);
		}
		try {
			if (ApplicationUtil.empty(files)) {
				ApplicationUtil.fileWriter(files, ConstantsLanguage.NO_APPUNTAMENTO.stampa(), ConstantsLanguage.NO_APPUNTAMENTI.stampa());
			}
			String fileName = ApplicationUtil.reportFile(ora, ConstantsLanguage.APPUNTAMENTI.stampa()) + "." + ConstantsMap.zip;
			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(fileName, files);
			ApplicationUtil.zip(null,files, out, p);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		} 
	}
	public void getRiepilogoAgenteSupervisore(Utente utente, CalendarioCriteria app, Date ora,  DownloadBean downloadBean) {
		String html = impostazioneDao.getFormatoDocumento();
		String p = impostazioneDao.getPasswordZIP();
		String s = ConstantsGlobal.secretL;
		List<Agente> agenti = new ArrayList<Agente>();
		agenti.add(userDao.getAgente(app.getAgenteId()));
		List<Appuntamento> appuntamenti = new ArrayList<Appuntamento>();
		Iterator<Agente> objIter = agenti.iterator();
		while(objIter.hasNext()) {
			Agente agente = objIter.next();
			Date now = ApplicationUtil.inizioGiornata(ora);
			appuntamenti.addAll(ricercaCalendarioSupervisoreAppuntamenti(utente, now, ApplicationUtil.addDays(now, 1), agente.getId(), app));
		}
		List<Allegato> files = new ArrayList<Allegato>();
		Iterator<Appuntamento> objAppIter = appuntamenti.iterator();
		if (ApplicationUtil.notEmpty(appuntamenti)) {
			String extM = (s == null || (html.equalsIgnoreCase(ConstantsMap.pdf))) ? html : ConstantsMap.zip;
			while(objAppIter.hasNext()) {
				Appuntamento appuntamento = objAppIter.next();
				try {
					String nomeFile = ApplicationUtil.reportFile(appuntamento.getData_inizio(), ConstantsLanguage.APPUNTAMENTO.stampa(), appuntamento.getId(), appuntamento.getContatto().getAnagrafica().getDenominazione())  + "." + extM;
					ByteArrayOutputStream result = downloadDao.appuntamentoToHTMLString(utente, appuntamento,  true, true, html, null, false, s);
					result.close();
					ApplicationUtil.addFile(files, nomeFile, result.toByteArray(), appuntamento.getData_creazione().getTime());
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				}
			}
			String pg = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.APPUNTAMENTO.stampa() +": " + ApplicationUtil.size(appuntamenti), appuntamenti);
			operazioneDao.creaOperazione(ConstantsMap.LOG4, pg ,utente, ConstantsMap.OPERAZIONE_EXPORT_APPUNTAMENTO);
		}
		try {
			if (ApplicationUtil.empty(files)) {
				ApplicationUtil.fileWriter(files, ConstantsLanguage.NO_APPUNTAMENTO.stampa(), ConstantsLanguage.NO_APPUNTAMENTI.stampa());
			}
			String fileName = ApplicationUtil.reportFile(ora, ConstantsLanguage.APPUNTAMENTI.stampa()) + "." + ConstantsMap.zip;
			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(fileName, files);
			ApplicationUtil.zip(null,files, out, p);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		} 
	}
	public void getRiepilogoAgentePartner(Utente utente, CalendarioCriteria app, Date ora,  DownloadBean downloadBean) {
		String html = impostazioneDao.getFormatoDocumento();
		String s = ConstantsGlobal.secretL;
		String p = impostazioneDao.getPasswordZIP();
		List<Agente> agenti = new ArrayList<Agente>();
		agenti.add(userDao.getAgente(app.getAgenteId()));
		List<Appuntamento> appuntamenti = new ArrayList<Appuntamento>();
		Iterator<Agente> objIter = agenti.iterator();
		while(objIter.hasNext()) {
			Agente agente = objIter.next();
			Date now = ApplicationUtil.inizioGiornata(ora);
			appuntamenti.addAll(ricercaCalendarioListAppuntamentiPartner(utente.getId(), now, ApplicationUtil.addDays(now, 1), agente.getId(), app));
		}
		List<Allegato> files = new ArrayList<Allegato>();
		Iterator<Appuntamento> objAppIter = appuntamenti.iterator();
		if (ApplicationUtil.notEmpty(appuntamenti)) {
			String extM = (s == null || (html.equalsIgnoreCase(ConstantsMap.pdf))) ? html : ConstantsMap.zip;
			while(objAppIter.hasNext()) {
				Appuntamento appuntamento = objAppIter.next();
				try {
					String nomeFile = ApplicationUtil.reportFile(appuntamento.getData_inizio(), ConstantsLanguage.APPUNTAMENTO.stampa(), appuntamento.getId(), appuntamento.getContatto().getAnagrafica().getDenominazione())  + "." + extM;
					ByteArrayOutputStream result = downloadDao.appuntamentoToHTMLString(utente, appuntamento,  true, true, html, null, false, s);
					result.close();
					ApplicationUtil.addFile(files, nomeFile, result.toByteArray(), appuntamento.getData_creazione().getTime());
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				}
			}
			String pg = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.APPUNTAMENTO.stampa() +": " + ApplicationUtil.size(appuntamenti), appuntamenti);
			operazioneDao.creaOperazione(ConstantsMap.LOG4, pg ,utente, ConstantsMap.OPERAZIONE_EXPORT_APPUNTAMENTO);
		}
		try {
			if (ApplicationUtil.empty(files)) {
				ApplicationUtil.fileWriter(files, ConstantsLanguage.NO_APPUNTAMENTO.stampa(), ConstantsLanguage.NO_APPUNTAMENTI.stampa());
			}
			String fileName = ApplicationUtil.reportFile(ora, ConstantsLanguage.APPUNTAMENTI.stampa()) + "." + ConstantsMap.zip;
			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(fileName, files);
			ApplicationUtil.zip(null,files, out, p);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		} 
	}
	public void getRiepilogoAgenteOperatore(Utente utente, CalendarioCriteria app, Date ora, DownloadBean downloadBean) {
		String html = impostazioneDao.getFormatoDocumento();
		String s = ConstantsGlobal.secretL;
		String p = impostazioneDao.getPasswordZIP();
		List<Agente> agenti = new ArrayList<Agente>();
		agenti.add(userDao.getAgente(app.getAgenteId()));
		List<Appuntamento> appuntamenti = new ArrayList<Appuntamento>();
		Iterator<Agente> objIter = agenti.iterator();
		while(objIter.hasNext()) {
			Agente agente = objIter.next();
			Date now = ApplicationUtil.inizioGiornata(ora);
			appuntamenti.addAll(ricercaCalendarioOperatoreAppuntamenti(utente.getId(), now, ApplicationUtil.addDays(now, 1), agente.getId(), app));
		}
		List<Allegato> files = new ArrayList<Allegato>();
		Iterator<Appuntamento> objAppIter = appuntamenti.iterator();
		if (ApplicationUtil.notEmpty(appuntamenti)) {
			String extM = (s == null || (html.equalsIgnoreCase(ConstantsMap.pdf))) ? html : ConstantsMap.zip;
			while(objAppIter.hasNext()) {
				Appuntamento appuntamento = objAppIter.next();
				if(appuntamento.getOperatore().equals(utente)) {
					try {
						String nomeFile = ApplicationUtil.reportFile(appuntamento.getData_inizio(), ConstantsLanguage.APPUNTAMENTO.stampa(), appuntamento.getId(), appuntamento.getContatto().getAnagrafica().getDenominazione())  + "." + extM;
						ByteArrayOutputStream result = downloadDao.appuntamentoToHTMLString(utente, appuntamento,  true, false, html, false, false, s);
						result.close();
						ApplicationUtil.addFile(files, nomeFile, result.toByteArray(), appuntamento.getData_creazione().getTime());
					} catch (Exception e) {
						ErrorLog.log(null, getClass(), e);
					}
				}
			}
			String pg = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.APPUNTAMENTO.stampa() +": " + ApplicationUtil.size(appuntamenti), appuntamenti);
			operazioneDao.creaOperazione(ConstantsMap.LOG4, pg ,utente, ConstantsMap.OPERAZIONE_EXPORT_APPUNTAMENTO);
		}
		try {
			if (ApplicationUtil.empty(files)) {
				ApplicationUtil.fileWriter(files, ConstantsLanguage.NO_APPUNTAMENTO.stampa(), ConstantsLanguage.NO_APPUNTAMENTI.stampa());
			}
			String fileName = ApplicationUtil.reportFile(ora, ConstantsLanguage.APPUNTAMENTI.stampa()) + "." + ConstantsMap.zip;
			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(fileName, files);
			ApplicationUtil.zip(null,files, out, p);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		} 
	}
	public void getRiepilogoAgenteBackoffice(Utente utente, CalendarioCriteria app, Date ora, DownloadBean downloadBean) {
		String html = impostazioneDao.getFormatoDocumento();
		String s = ConstantsGlobal.secretL;
		String p = impostazioneDao.getPasswordZIP();
		List<Agente> agenti = new ArrayList<Agente>();
		agenti.add(userDao.getAgente(app.getAgenteId()));
		List<Appuntamento> appuntamenti = new ArrayList<Appuntamento>();
		Iterator<Agente> objIter = agenti.iterator();
		while(objIter.hasNext()) {
			Agente agente = objIter.next();
			Date now = ApplicationUtil.inizioGiornata(ora);
			appuntamenti.addAll(ricercaCalendarioBackofficeAppuntamenti(now, ApplicationUtil.addDays(now, 1), agente.getId(), app));
		}
		List<Allegato> files = new ArrayList<Allegato>();
		Iterator<Appuntamento> objAppIter = appuntamenti.iterator();
		if (ApplicationUtil.notEmpty(appuntamenti)) {
			String extM = (s == null || (html.equalsIgnoreCase(ConstantsMap.pdf))) ? html : ConstantsMap.zip;
			while(objAppIter.hasNext()) {
				Appuntamento appuntamento = objAppIter.next();
				try {
					String nomeFile = ApplicationUtil.reportFile(appuntamento.getData_inizio(), ConstantsLanguage.APPUNTAMENTO.stampa(), appuntamento.getId(), appuntamento.getContatto().getAnagrafica().getDenominazione())  + "." + extM;
					ByteArrayOutputStream result = downloadDao.appuntamentoToHTMLString(utente, appuntamento,  true, true, html, null, false, s);
					result.close();
					ApplicationUtil.addFile(files, nomeFile, result.toByteArray(), appuntamento.getData_creazione().getTime());
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				}
			}
			String pg = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.APPUNTAMENTO.stampa() +": " + ApplicationUtil.size(appuntamenti), appuntamenti);
			operazioneDao.creaOperazione(ConstantsMap.LOG4, pg ,utente, ConstantsMap.OPERAZIONE_EXPORT_APPUNTAMENTO);
		}
		try {
			if (ApplicationUtil.empty(files)) {
				ApplicationUtil.fileWriter(files, ConstantsLanguage.NO_APPUNTAMENTO.stampa(), ConstantsLanguage.NO_APPUNTAMENTI.stampa());
			}
			String fileName = ApplicationUtil.reportFile(ora, ConstantsLanguage.APPUNTAMENTI.stampa()) + "." + ConstantsMap.zip;
			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(fileName, files);
			ApplicationUtil.zip(null,files, out, p);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		} 
	}
	public List<GiornoCalendarioAgenteVO> ricercaCalendarioBackoffice(Utente utente, CalendarioCriteria appCr) {
		List<String> sis = new ArrayList<String>();
		for(int i = appCr.getInizioA(); i <= appCr.getFineA(); i++) {
			sis.add(String.format("%02d", i, 0));
		}
		Date attuale = ApplicationUtil.date();
		Agente agente = campagnaDao.getAgente(appCr.getAgenteId());
		List<GiornoCalendarioAgenteVO> calendario = new ArrayList<GiornoCalendarioAgenteVO>();
		if (agente != null) {
			Date inizio = ApplicationUtil.inizioGiornata(appCr.getGiornoPartenza());
			Date fine = ApplicationUtil.fineGiornata(ApplicationUtil.addDays(inizio, appCr.getGiorni()));
			for(int i = 0; i < appCr.getGiorni(); i++) {
				Date giorno_calendario = ApplicationUtil.addDays(inizio, i);
				GiornoCalendarioAgenteVO gc = new GiornoCalendarioAgenteVO();
				gc.setAppuntamenti(new ArrayList<Appuntamento>());
				gc.setNote(new ArrayList<Annotazione>());
				gc.setData(giorno_calendario);
				gc.setAgente(agente);
				gc.setOrari(sis);
				gc.setGiorniCalendario(appCr.getGiorni());
				calendario.add(gc);
			}
			List<Appuntamento> appuntamenti = ricercaCalendarioBackofficeAppuntamenti(inizio, fine, agente.getId(), appCr);
			List<Nota> note = trovaListaNote(inizio, fine, agente.getId());
			List<Orario> orari = new ArrayList<Orario>();
			Orario orario = new Orario();
			orario.setData(attuale);
			orari.add(orario);
			appuntamentoDao.completaCalendario(calendario, appuntamenti, note, orari, appCr.getInizioA(), appCr.getFineA(), attuale);
			if(ApplicationUtil.notEmpty(appuntamenti))  {
				String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.APPUNTAMENTO.stampa() +": " + ApplicationUtil.size(appuntamenti), appuntamenti);
				operazioneDao.creaOperazione(ConstantsMap.LOG4, p ,utente, ConstantsMap.OPERAZIONE_VISUALIZZA_APPUNTAMENTO);
			}
		}
		return calendario;
	}
	public List<GiornoCalendarioAgenteVO> ricercaCalendarioAgente(Utente utente, CalendarioCriteria appCr) {
		List<String> sis = new ArrayList<String>();
		for(int i = appCr.getInizioA(); i <= appCr.getFineA(); i++) {
			sis.add(String.format("%02d", i, 0));
		}
		Date attuale = ApplicationUtil.date();
		Agente agente = campagnaDao.getAgente(appCr.getAgenteId());
		List<GiornoCalendarioAgenteVO> calendario = new ArrayList<GiornoCalendarioAgenteVO>();
		if (agente != null) {
			Date inizio = ApplicationUtil.inizioGiornata(appCr.getGiornoPartenza());
			Date fine = ApplicationUtil.fineGiornata(ApplicationUtil.addDays(inizio, appCr.getGiorni()));
			for(int i = 0; i < appCr.getGiorni(); i++) {
				Date giorno_calendario = ApplicationUtil.addDays(inizio, i);
				GiornoCalendarioAgenteVO gc = new GiornoCalendarioAgenteVO();
				gc.setAppuntamenti(new ArrayList<Appuntamento>());
				gc.setNote(new ArrayList<Annotazione>());
				gc.setData(giorno_calendario);
				gc.setAgente(agente);
				gc.setOrari(sis);
				gc.setGiorniCalendario(appCr.getGiorni());
				calendario.add(gc);
			}
			List<Appuntamento> appuntamenti = ricercaCalendarioAgenteAppuntamenti(inizio, fine, agente.getId(), appCr);
			List<Nota> note = trovaListaNote(inizio, fine, agente.getId());
			List<Orario> orari = new ArrayList<Orario>();
			Orario orario = new Orario();
			orario.setData(attuale);
			orari.add(orario);
			appuntamentoDao.completaCalendario(calendario, appuntamenti, note, orari, appCr.getInizioA(), appCr.getFineA(), attuale);
			if(ApplicationUtil.notEmpty(appuntamenti))  {
				String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.APPUNTAMENTO.stampa() +": " + ApplicationUtil.size(appuntamenti), appuntamenti);
				operazioneDao.creaOperazione(ConstantsMap.LOG4, p ,utente, ConstantsMap.OPERAZIONE_VISUALIZZA_APPUNTAMENTO);
			}
		}
		return calendario;
	}
	public List<GiornoCalendarioAgenteVO> ricercaCalendarioRete(Utente utente, CalendarioCriteria appCr) {
		List<String> sis = new ArrayList<String>();
		for(int i = appCr.getInizioA(); i <= appCr.getFineA(); i++) {
			sis.add(String.format("%02d", i, 0));
		}
		Date attuale = ApplicationUtil.date();
		Agente agente = campagnaDao.getAgente(appCr.getAgenteId());
		List<GiornoCalendarioAgenteVO> calendario = new ArrayList<GiornoCalendarioAgenteVO>();
		if (agente != null) {
			Date inizio = ApplicationUtil.inizioGiornata(appCr.getGiornoPartenza());
			Date fine = ApplicationUtil.fineGiornata(ApplicationUtil.addDays(inizio, appCr.getGiorni()));
			for(int i = 0; i < appCr.getGiorni(); i++) {
				Date giorno_calendario = ApplicationUtil.addDays(inizio, i);
				GiornoCalendarioAgenteVO gc = new GiornoCalendarioAgenteVO();
				gc.setAppuntamenti(new ArrayList<Appuntamento>());
				gc.setNote(new ArrayList<Annotazione>());
				gc.setData(giorno_calendario);
				gc.setAgente(agente);
				gc.setOrari(sis);
				gc.setGiorniCalendario(appCr.getGiorni());
				calendario.add(gc);
			}
			List<Appuntamento> appuntamenti = ricercaCalendarioAgenteAppuntamenti(inizio, fine, agente.getId(), appCr);
			List<Nota> note = trovaListaNote(inizio, fine, agente.getId());
			List<Orario> orari = new ArrayList<Orario>();
			Orario orario = new Orario();
			orario.setData(attuale);
			orari.add(orario);
			appuntamentoDao.completaCalendario(calendario, appuntamenti, note, orari, appCr.getInizioA(), appCr.getFineA(), attuale);
			if(ApplicationUtil.notEmpty(appuntamenti))  {
				String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.APPUNTAMENTO.stampa() +": " + ApplicationUtil.size(appuntamenti), appuntamenti);
				operazioneDao.creaOperazione(ConstantsMap.LOG4, p ,utente, ConstantsMap.OPERAZIONE_VISUALIZZA_APPUNTAMENTO);
			}
		}
		return calendario;
	}
	@SuppressWarnings("unchecked")
	public List<Appuntamento> ricercaCalendarioListAppuntamenti(Date inizio, Date fine, Long agenteId, CalendarioCriteria appCr) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Appuntamento.class);
		DetachedCriteria criteriaC = criteria.createCriteria("esito", ApplicationUtil.leftJoin());
		criteria.add(Restrictions.eq("agente.id", agenteId));
		if (ApplicationUtil.gtZero(appCr.getStato())) {
			criteria.add(Restrictions.eq("stato", appCr.getStato()));
		}
		if (ApplicationUtil.gtZero(appCr.getConfermato())){
			if(ApplicationUtil.eq(appCr.getConfermato(), 1))
				criteriaC.add(Restrictions.eq("confermato", Boolean.TRUE));
			else 
				criteriaC.add(Restrictions.eq("confermato", Boolean.FALSE));
		}
		if (ApplicationUtil.gtZero(appCr.getEsito())) {
			criteriaC.add(Restrictions.idEq(appCr.getEsito()));
		} 
		ApplicationUtil.dataDate(inizio, fine, "data_inizio", criteria);
		criteria.addOrder(Order.asc("data_inizio"));
		return  getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Appuntamento> ricercaCalendarioOperatoreAppuntamenti(Long utenteId, Date inizio, Date fine, Long agenteId, CalendarioCriteria appCr) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Appuntamento.class);
		DetachedCriteria criteriaC = criteria.createCriteria("esito", ApplicationUtil.leftJoin());
		criteria.add(Restrictions.eq("agente.id", agenteId));
		if (ApplicationUtil.gtZero(appCr.getConfermato())){
			if(ApplicationUtil.eq(appCr.getConfermato(), 1))
				criteriaC.add(Restrictions.eq("confermato", Boolean.TRUE));
			else 
				criteriaC.add(Restrictions.eq("confermato", Boolean.FALSE));
		}
		if (ApplicationUtil.gtZero(appCr.getStato())) {
			criteria.add(Restrictions.eq("stato", appCr.getStato()));
		}
		if (ApplicationUtil.gtZero(appCr.getEsito())) {
			criteriaC.add(Restrictions.idEq(appCr.getEsito()));
		} 
		DetachedCriteria c = criteria.createCriteria("operatore");
		impostazioneDao.subCriteriaOperatore(utenteId, c, "id");
		ApplicationUtil.dataDate(inizio, fine, "data_inizio", criteria);
		criteria.addOrder(Order.asc("data_inizio"));
		return  getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Appuntamento> ricercaCalendarioListAppuntamentiPartner(Long userid, Date inizio, Date fine, Long agenteId, CalendarioCriteria appCr) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Appuntamento.class);
		DetachedCriteria criteriaCa = criteria.createCriteria("contatto").createCriteria("campagna");
		DetachedCriteria criteriaC = criteria.createCriteria("esito", ApplicationUtil.leftJoin());
		criteria.add(Restrictions.eq("agente.id", agenteId));
		impostazioneDao.subCriteriaCampagnaPartner(userid, criteriaCa, "id");
		if (ApplicationUtil.gtZero(appCr.getStato())) {
			criteria.add(Restrictions.eq("stato", appCr.getStato()));
		}
		if (ApplicationUtil.gtZero(appCr.getConfermato())){
			if(ApplicationUtil.eq(appCr.getConfermato(), 1))
				criteriaC.add(Restrictions.eq("confermato", Boolean.TRUE));
			else 
				criteriaC.add(Restrictions.eq("confermato", Boolean.FALSE));
		}
		if (ApplicationUtil.gtZero(appCr.getEsito())) {
			criteriaC.add(Restrictions.idEq(appCr.getEsito()));
		} 
		ApplicationUtil.dataDate(inizio, fine, "data_inizio", criteria);
		criteria.addOrder(Order.asc("data_inizio"));
		return  getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Appuntamento> ricercaCalendarioBackofficeAppuntamenti(Date inizio, Date fine, Long agenteId, CalendarioCriteria appCr) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Appuntamento.class);
		DetachedCriteria criteriaCo = criteria.createCriteria("contatto");
		DetachedCriteria criteriaC = criteria.createCriteria("esito", ApplicationUtil.leftJoin());
		criteriaCo.createCriteria("esitoArchiviazione").add(Restrictions.eq("backoffice", Boolean.TRUE));
		criteria.add(Restrictions.eq("agente.id", agenteId));
		if (ApplicationUtil.gtZero(appCr.getStato())) {
			criteria.add(Restrictions.eq("stato", appCr.getStato()));
		}
		if (ApplicationUtil.gtZero(appCr.getConfermato())){
			if(ApplicationUtil.eq(appCr.getConfermato(), 1))
				criteriaC.add(Restrictions.eq("confermato", Boolean.TRUE));
			else 
				criteriaC.add(Restrictions.eq("confermato", Boolean.FALSE));
		}
		if (ApplicationUtil.gtZero(appCr.getEsito())) {
			criteriaC.add(Restrictions.idEq(appCr.getEsito()));
		} 
		ApplicationUtil.dataDate(inizio, fine, "data_inizio", criteria);
		criteria.addOrder(Order.asc("data_inizio"));
		return  getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Appuntamento> ricercaCalendarioAgenteAppuntamenti(Date inizio, Date fine, Long agenteId, CalendarioCriteria appCr) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Appuntamento.class);
		DetachedCriteria criteriaC = criteria.createCriteria("esito");
		criteria.add(Restrictions.eq("agente.id", agenteId));
		criteriaC.add(Restrictions.eq("confermato", Boolean.TRUE));
		if (ApplicationUtil.gtZero(appCr.getEsito())) {
			criteriaC.add(Restrictions.idEq(appCr.getEsito()));
		} 
		if (ApplicationUtil.gtZero(appCr.getStato())) {
			criteria.add(Restrictions.eq("stato", appCr.getStato()));
		}
		ApplicationUtil.dataDate(inizio, fine, "data_inizio", criteria);
		criteria.addOrder(Order.asc("data_inizio"));
		return  getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Appuntamento> ricercaCalendarioSupervisoreAppuntamenti(Utente utente, Date inizio, Date fine, Long agenteId, CalendarioCriteria appCr) {
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(Appuntamento.class);
		criteria.add(Restrictions.eq("agente.id", agenteId));
		DetachedCriteria criteriaC = criteria.createCriteria("esito", ApplicationUtil.leftJoin());
		DetachedCriteria criteriaCA = criteria.createCriteria("contatto").createCriteria("campagna", "campagna");
		DetachedCriteria c = criteria.createCriteria("operatore", "operatore");
		impostazioneDao.subCriteriaRoleSupervisore(utente.getId(), criteria, "campagna", "operatore");
		impostazioneDao.subCriteriaSupervisore(utente.getId(), c, "id");
		if(ApplicationUtil.isTrue(sd)) {
			impostazioneDao.subCriteriaCampagnaSupervisore(utente.getId(), criteriaCA, "id");
		}
		if (ApplicationUtil.gtZero(appCr.getConfermato())){
			if(ApplicationUtil.eq(appCr.getConfermato(), 1))
				criteriaC.add(Restrictions.eq("confermato", Boolean.TRUE));
			else 
				criteriaC.add(Restrictions.eq("confermato", Boolean.FALSE));
		}
		if (ApplicationUtil.gtZero(appCr.getStato())) {
			criteria.add(Restrictions.eq("stato", appCr.getStato()));
		}
		if (ApplicationUtil.gtZero(appCr.getEsito())) {
			criteriaC.add(Restrictions.idEq(appCr.getEsito()));
		} 
		ApplicationUtil.dataDate(inizio, fine, "data_inizio", criteria);
		criteria.addOrder(Order.asc("data_inizio"));
		return  getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Appuntamento> ricercaCalendarioCoordinatoreAppuntamenti(Utente utente, Date inizio, Date fine, Long agenteId, CalendarioCriteria appCr) {
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(Appuntamento.class);
		DetachedCriteria criteriaAg = criteria.createCriteria("agente", "agente");
		DetachedCriteria criteriaCA = criteria.createCriteria("contatto").createCriteria("campagna", "campagna");
		criteriaAg.add(Restrictions.idEq(agenteId));
		impostazioneDao.subCriteriaRoleCoordinatore(utente.getId(), criteria, "campagna", "agente");
		if(ApplicationUtil.isTrue(sd)) {
			impostazioneDao.subCriteriaCoordinatoreCampagna(utente.getId(), criteriaCA, "id");
		}
		DetachedCriteria criteriaC = criteria.createCriteria("esito", ApplicationUtil.leftJoin());
		if (ApplicationUtil.gtZero(appCr.getConfermato())){
			if(ApplicationUtil.eq(appCr.getConfermato(), 1))
				criteriaC.add(Restrictions.eq("confermato", Boolean.TRUE));
			else 
				criteriaC.add(Restrictions.eq("confermato", Boolean.FALSE));
		}
		if (ApplicationUtil.gtZero(appCr.getStato())) {
			criteria.add(Restrictions.eq("stato", appCr.getStato()));
		}
		if (ApplicationUtil.gtZero(appCr.getEsito())) {
			criteriaC.add(Restrictions.idEq(appCr.getEsito()));
		} 
		ApplicationUtil.dataDate(inizio, fine, "data_inizio", criteria);
		criteria.addOrder(Order.asc("data_inizio"));
		return  getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Nota> trovaListaNote(Date inizio, Date fine, Long agenteId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Nota.class);
		criteria.add(Restrictions.eq("agente.id", agenteId));
		ApplicationUtil.dataNota(inizio, fine, "data_inizio", criteria);
		criteria.addOrder(Order.asc("data_inizio"));
		return getList(criteria);
	}
	public List<CalendarioAgenteVO> getAppuntamentiListManager(List<Agente> agentiSel, CalendarioCriteria appCr, Utente utente) {
		List<String> sis = new ArrayList<String>();
		for(int i = appCr.getInizioA(); i <= appCr.getFineA(); i++) {
			sis.add(String.format("%02d", i, 0));
		}
		List<Agente> agenti = new ArrayList<Agente>();
		if (ApplicationUtil.gtZero(appCr.getAgenteId())) {
			agenti = new ArrayList<Agente>();
			agenti.add(campagnaDao.getAgente(appCr.getAgenteId()));
		} else {
			agenti = agentiSel;
		}
		List<Appuntamento> appuntamenti = new ArrayList<Appuntamento>();
		List<CalendarioAgenteVO> calendari = new ArrayList<CalendarioAgenteVO>();
		if(ApplicationUtil.gtZero(appCr.getCampagna_id()) || ApplicationUtil.gtZero(appCr.getZona_id()) || ApplicationUtil.gtZero(appCr.getAgenteId())) {
			Iterator<Agente> objIter = agenti.iterator();
			while(objIter.hasNext()) {
				Agente agente = objIter.next();
				List<GiornoCalendarioAgenteVO> calendario = getAppuntamentiListManager(appuntamenti, appCr.getGiornoPartenza(), agente, appCr.getGiorni(), appCr, utente.getId(), appCr.getInizioA(), appCr.getFineA());
				CalendarioAgenteVO cvo = new CalendarioAgenteVO();
				cvo.setAgente(agente);
				cvo.setGiorni(calendario);
				cvo.setOrari(sis);
				cvo.setGiorniCalendario(appCr.getGiorni());
				calendari.add(cvo);
			}
		}
		if(ApplicationUtil.notEmpty(appuntamenti))  {
			String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.APPUNTAMENTO.stampa() +": " + ApplicationUtil.size(appuntamenti), appuntamenti);
			operazioneDao.creaOperazione(ConstantsMap.LOG4, p ,utente, ConstantsMap.OPERAZIONE_VISUALIZZA_APPUNTAMENTO);
		}
		return calendari;
	}
	private List<GiornoCalendarioAgenteVO> getAppuntamentiListManager(List<Appuntamento> appuntamentiTot, Date giornoPartenza, Agente agente, int giorni, CalendarioCriteria appCr, Long userId, int oramin, int oramax) {
		List<GiornoCalendarioAgenteVO> calendario = new ArrayList<GiornoCalendarioAgenteVO>();
		Date attuale = ApplicationUtil.date();
		if (agente != null) {
			Date inizio = ApplicationUtil.inizioGiornata(giornoPartenza);
			Date fine = ApplicationUtil.fineGiornata(ApplicationUtil.addDays(inizio, giorni));
			for(int i = 0; i < giorni; i++) {
				Date giorno_calendario = ApplicationUtil.addDays(inizio, i);
				GiornoCalendarioAgenteVO gc = new GiornoCalendarioAgenteVO();
				gc.setAppuntamenti(new ArrayList<Appuntamento>());
				gc.setNote(new ArrayList<Annotazione>());
				gc.setData(giorno_calendario);
				gc.setAgente(agente);
				calendario.add(gc);
			}
			List<Appuntamento> appuntamenti = trovaListaAppuntamentiManager(inizio, fine, agente.getId(),  appCr, userId);
			List<Nota> note = trovaListaNote(inizio, fine, agente.getId());
			List<Orario> orari = new ArrayList<Orario>();
			Orario orario = new Orario();
			orario.setData(attuale);
			orari.add(orario);
			appuntamentoDao.completaCalendario(calendario, appuntamenti, note, orari, oramin, oramax, attuale);
			appuntamentiTot.addAll(appuntamenti);
		}
		return calendario;
	}
	@SuppressWarnings("unchecked")
	public List<Appuntamento> trovaListaAppuntamentiManager(Date inizio, Date fine, Long agenteId, CalendarioCriteria appCr, Long userId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Appuntamento.class);
		DetachedCriteria criteriaC = criteria.createCriteria("esito", ApplicationUtil.leftJoin());
		criteria.add(Restrictions.eq("agente.id", agenteId));
		if (ApplicationUtil.gtZero(appCr.getStato())) {
			criteria.add(Restrictions.eq("stato", appCr.getStato()));
		}
		if (ApplicationUtil.gtZero(appCr.getConfermato())){
			if(ApplicationUtil.eq(appCr.getConfermato(), 1))
				criteriaC.add(Restrictions.eq("confermato", Boolean.TRUE));
			else 
				criteriaC.add(Restrictions.eq("confermato", Boolean.FALSE));
		}
		if (ApplicationUtil.gtZero(appCr.getEsito())) {
			criteriaC.add(Restrictions.idEq(appCr.getEsito()));
		} 
		ApplicationUtil.dataDate(inizio, fine, "data_inizio", criteria);
		criteria.addOrder(Order.asc("data_inizio"));
		return getList(criteria);
	}
	public void getRiepilogoAgenteManager(Utente utente, CalendarioCriteria appCr, Agente agente, Date ora,  DownloadBean downloadBean) {
		String html = impostazioneDao.getFormatoDocumento();
		String s = ConstantsGlobal.secretL;
		String p = impostazioneDao.getPasswordZIP();
		List<Appuntamento> appuntamenti = new ArrayList<Appuntamento>();
		Date now = ApplicationUtil.inizioGiornata(ora);
		appuntamenti.addAll(trovaListaAppuntamentiManager(now, ApplicationUtil.addDays(now, 1), agente.getId(), appCr, utente.getId()));
		List<Allegato> files = new ArrayList<Allegato>();
		Iterator<Appuntamento> objAppIter = appuntamenti.iterator();
		if (ApplicationUtil.notEmpty(appuntamenti)) {
			String extM = (s == null || (html.equalsIgnoreCase(ConstantsMap.pdf))) ? html : ConstantsMap.zip;
			while(objAppIter.hasNext()) {
				Appuntamento appuntamento = objAppIter.next();
				try {
					String nomeFile = ApplicationUtil.reportFile(appuntamento.getData_inizio(), ConstantsLanguage.APPUNTAMENTO.stampa(), appuntamento.getId(), appuntamento.getContatto().getAnagrafica().getDenominazione())  + "." + extM;
					ByteArrayOutputStream result = downloadDao.appuntamentoToHTMLString(utente, appuntamento,  true, true, html, null, false, s);
					result.close();
					ApplicationUtil.addFile(files, nomeFile, result.toByteArray(), appuntamento.getData_creazione().getTime());
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				}
			}
			String pg = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.APPUNTAMENTO.stampa() +": " + ApplicationUtil.size(appuntamenti), appuntamenti);
			operazioneDao.creaOperazione(ConstantsMap.LOG4, pg ,utente, ConstantsMap.OPERAZIONE_EXPORT_APPUNTAMENTO);
		}
		try {
			if (ApplicationUtil.empty(files)) {
				ApplicationUtil.fileWriter(files, ConstantsLanguage.NO_APPUNTAMENTO.stampa(), ConstantsLanguage.NO_APPUNTAMENTI.stampa());
			}
			String fileName = ApplicationUtil.reportFile(ora, ConstantsLanguage.APPUNTAMENTI.stampa()) + "." + ConstantsMap.zip;
			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(fileName, files);
			ApplicationUtil.zip(null,files, out, p);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		} 
	}
	@SuppressWarnings("unchecked")
	public List<Agente> getAgenteByCampagnaId(CalendarioCriteria appCr) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
		DetachedCriteria criteriaZ = criteria.createCriteria("zone");
		if(ApplicationUtil.gtZero(appCr.getZona_id())) {
			criteriaZ.add(Restrictions.idEq(appCr.getZona_id()));
		}
		if(ApplicationUtil.gtZero(appCr.getCampagna_id())) {
			criteriaZ.createCriteria("campagne").add(Restrictions.idEq(appCr.getCampagna_id()));
		}
		ApplicationUtil.orderA(criteria);
		List<Agente> agenti = getList(criteria);
		return agenti;
	}
	public List<SelectItem> getZonaByCampagnaId(CalendarioCriteria appCr) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Zona.class);
		if(ApplicationUtil.gtZero(appCr.getAgenteId())) {
			criteria.createCriteria("agente").add(Restrictions.idEq(appCr.getAgenteId()));
		}
		if(ApplicationUtil.gtZero(appCr.getCampagna_id())) {
			criteria.createCriteria("campagne").add(Restrictions.idEq(appCr.getCampagna_id()));
		}
		List<ZonaSI> lis = getZonaSI(criteria);
		return impostazioneDao.getZona(lis, false, false, null);
	}
	public List<SelectItem> getCampagneSI(CalendarioCriteria appCr) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		DetachedCriteria criteriaC = criteria.createCriteria("zone");
		if(ApplicationUtil.gtZero(appCr.getAgenteId())) {
			criteriaC.createCriteria("agente").add(Restrictions.idEq(appCr.getAgenteId()));
		}
		if(ApplicationUtil.gtZero(appCr.getZona_id())) {
			criteriaC.add(Restrictions.idEq(appCr.getZona_id()));
		}
		List<CampagnaSI> lis = getCampagnaSI(criteria);
		return impostazioneDao.getCampagna(lis, true, false, false, null);
	}
	public List<CalendarioAgenteVO> getAppuntamentiListBackoffice(List<Agente> agentiSel, CalendarioCriteria appCr, Utente utente) {
		List<String> sis = new ArrayList<String>();
		for(int i = appCr.getInizioA(); i <= appCr.getFineA(); i++) {
			sis.add(String.format("%02d", i, 0));
		}
		List<Agente> agenti = new ArrayList<Agente>();
		if (ApplicationUtil.gtZero(appCr.getAgenteId())) {
			agenti = new ArrayList<Agente>();
			agenti.add(campagnaDao.getAgente(appCr.getAgenteId()));
		} else {
			agenti = agentiSel;
		}
		List<Appuntamento> appuntamenti = new ArrayList<Appuntamento>();
		List<CalendarioAgenteVO> calendari = new ArrayList<CalendarioAgenteVO>();
		if(ApplicationUtil.gtZero(appCr.getCampagna_id()) || ApplicationUtil.gtZero(appCr.getZona_id()) || ApplicationUtil.gtZero(appCr.getAgenteId())) {
			Iterator<Agente> objIter = agenti.iterator();
			while(objIter.hasNext()) {
				Agente agente = objIter.next();
				List<GiornoCalendarioAgenteVO> calendario = getAppuntamentiListBackoffice(appuntamenti, appCr.getGiornoPartenza(), agente, appCr.getGiorni(), appCr, utente.getId(), appCr.getInizioA(), appCr.getFineA());
				CalendarioAgenteVO cvo = new CalendarioAgenteVO();
				cvo.setAgente(agente);
				cvo.setGiorni(calendario);
				cvo.setOrari(sis);
				cvo.setGiorniCalendario(appCr.getGiorni());
				calendari.add(cvo);
			}
		}
		if(ApplicationUtil.notEmpty(appuntamenti))  {
			String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.APPUNTAMENTO.stampa() +": " + ApplicationUtil.size(appuntamenti), appuntamenti);
			operazioneDao.creaOperazione(ConstantsMap.LOG4, p ,utente, ConstantsMap.OPERAZIONE_VISUALIZZA_APPUNTAMENTO);
		}
		return calendari;
	}
	private List<GiornoCalendarioAgenteVO> getAppuntamentiListBackoffice(List<Appuntamento> appuntamentiTot, Date giornoPartenza, Agente agente, int giorni, CalendarioCriteria appCr, Long userId, int oramin, int oramax) {
		List<GiornoCalendarioAgenteVO> calendario = new ArrayList<GiornoCalendarioAgenteVO>();
		Date attuale = ApplicationUtil.date();
		if (agente != null) {
			Date inizio = ApplicationUtil.inizioGiornata(giornoPartenza);
			Date fine = ApplicationUtil.fineGiornata(ApplicationUtil.addDays(inizio, giorni));
			for(int i = 0; i < giorni; i++) {
				Date giorno_calendario = ApplicationUtil.addDays(inizio, i);
				GiornoCalendarioAgenteVO gc = new GiornoCalendarioAgenteVO();
				gc.setAppuntamenti(new ArrayList<Appuntamento>());
				gc.setNote(new ArrayList<Annotazione>());
				gc.setData(giorno_calendario);
				gc.setAgente(agente);
				calendario.add(gc);
			}
			List<Appuntamento> appuntamenti = trovaListaAppuntamentiBackoffice(inizio, fine, agente.getId(),  appCr, userId);
			List<Nota> note = trovaListaNote(inizio, fine, agente.getId());
			List<Orario> orari = new ArrayList<Orario>();
			Orario orario = new Orario();
			orario.setData(attuale);
			orari.add(orario);
			appuntamentoDao.completaCalendario(calendario, appuntamenti, note, orari, oramin, oramax, attuale);
			appuntamentiTot.addAll(appuntamenti);
		}
		return calendario;
	}
	@SuppressWarnings("unchecked")
	public List<Appuntamento> trovaListaAppuntamentiBackoffice(Date inizio, Date fine, Long agenteId, CalendarioCriteria appCr, Long userId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Appuntamento.class);
		DetachedCriteria criteriaCo = criteria.createCriteria("contatto");
		DetachedCriteria criteriaC = criteria.createCriteria("esito", ApplicationUtil.leftJoin());
		criteriaCo.createCriteria("esitoArchiviazione").add(Restrictions.eq("backoffice", Boolean.TRUE));
		criteria.add(Restrictions.eq("agente.id", agenteId));
		if (ApplicationUtil.gtZero(appCr.getStato())) {
			criteria.add(Restrictions.eq("stato", appCr.getStato()));
		}
		if (ApplicationUtil.gtZero(appCr.getConfermato())){
			if(ApplicationUtil.eq(appCr.getConfermato(), 1))
				criteriaC.add(Restrictions.eq("confermato", Boolean.TRUE));
			else 
				criteriaC.add(Restrictions.eq("confermato", Boolean.FALSE));
		}
		if (ApplicationUtil.gtZero(appCr.getEsito())) {
			criteriaC.add(Restrictions.idEq(appCr.getEsito()));
		} 
		ApplicationUtil.dataDate(inizio, fine, "data_inizio", criteria);
		criteria.addOrder(Order.asc("data_inizio"));
		return getList(criteria);
	}
	public void getRiepilogoAgenteBackoffice(Utente utente, CalendarioCriteria appCr, Agente agente, Date ora,  DownloadBean downloadBean) {
		String html = impostazioneDao.getFormatoDocumento();
		String s = ConstantsGlobal.secretL;
		String p = impostazioneDao.getPasswordZIP();
		List<Appuntamento> appuntamenti = new ArrayList<Appuntamento>();
		Date now = ApplicationUtil.inizioGiornata(ora);
		appuntamenti.addAll(trovaListaAppuntamentiBackoffice(now, ApplicationUtil.addDays(now, 1), agente.getId(), appCr, utente.getId()));
		List<Allegato> files = new ArrayList<Allegato>();
		Iterator<Appuntamento> objAppIter = appuntamenti.iterator();
		if (ApplicationUtil.notEmpty(appuntamenti)) {
			String extM = (s == null || (html.equalsIgnoreCase(ConstantsMap.pdf))) ? html : ConstantsMap.zip;
			while(objAppIter.hasNext()) {
				Appuntamento appuntamento = objAppIter.next();
				try {
					String nomeFile = ApplicationUtil.reportFile(appuntamento.getData_inizio(), ConstantsLanguage.APPUNTAMENTO.stampa(), appuntamento.getId(), appuntamento.getContatto().getAnagrafica().getDenominazione())  + "." + extM;
					ByteArrayOutputStream result = downloadDao.appuntamentoToHTMLString(utente, appuntamento,  true, true, html, null, false, s);
					result.close();
					ApplicationUtil.addFile(files, nomeFile, result.toByteArray(), appuntamento.getData_creazione().getTime());
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				}
			}
			String pg = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.APPUNTAMENTO.stampa() +": " + ApplicationUtil.size(appuntamenti), appuntamenti);
			operazioneDao.creaOperazione(ConstantsMap.LOG4, pg ,utente, ConstantsMap.OPERAZIONE_EXPORT_APPUNTAMENTO);
		}
		try {
			if (ApplicationUtil.empty(files)) {
				ApplicationUtil.fileWriter(files, ConstantsLanguage.NO_APPUNTAMENTO.stampa(), ConstantsLanguage.NO_APPUNTAMENTI.stampa());
			}
			String fileName = ApplicationUtil.reportFile(ora, ConstantsLanguage.APPUNTAMENTI.stampa()) + "." + ConstantsMap.zip;
			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(fileName, files);
			ApplicationUtil.zip(null,files, out, p);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public List<CalendarioAgenteVO> getAppuntamentiListSupervisore(List<Agente> agentiSel, CalendarioCriteria appCr, Utente utente) {
		List<String> sis = new ArrayList<String>();
		for(int i = appCr.getInizioA(); i <= appCr.getFineA(); i++) {
			sis.add(String.format("%02d", i, 0));
		}
		List<Agente> agenti = new ArrayList<Agente>();
		if (ApplicationUtil.gtZero(appCr.getAgenteId())) {
			agenti = new ArrayList<Agente>();
			agenti.add(campagnaDao.getAgente(appCr.getAgenteId()));
		} else {
			agenti = agentiSel;
		}
		List<Appuntamento> appuntamenti = new ArrayList<Appuntamento>();
		List<CalendarioAgenteVO> calendari = new ArrayList<CalendarioAgenteVO>();
		if(ApplicationUtil.gtZero(appCr.getCampagna_id()) || ApplicationUtil.gtZero(appCr.getZona_id()) || ApplicationUtil.gtZero(appCr.getAgenteId())) {
			Iterator<Agente> objIter = agenti.iterator();
			while(objIter.hasNext()) {
				Agente agente = objIter.next();
				List<GiornoCalendarioAgenteVO> calendario = getAppuntamentiListSupervisore(appuntamenti, appCr.getGiornoPartenza(), agente, appCr.getGiorni(), appCr, utente.getId(), appCr.getInizioA(), appCr.getFineA());
				CalendarioAgenteVO cvo = new CalendarioAgenteVO();
				cvo.setAgente(agente);
				cvo.setGiorni(calendario);
				cvo.setOrari(sis);
				cvo.setGiorniCalendario(appCr.getGiorni());
				calendari.add(cvo);
			}
		}
		if(ApplicationUtil.notEmpty(appuntamenti))  {
			String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.APPUNTAMENTO.stampa() +": " + ApplicationUtil.size(appuntamenti), appuntamenti);
			operazioneDao.creaOperazione(ConstantsMap.LOG4, p ,utente, ConstantsMap.OPERAZIONE_VISUALIZZA_APPUNTAMENTO);
		}
		return calendari;
	}
	public List<CalendarioAgenteVO> getAppuntamentiListCoordinatore(List<Agente> agentiSel, CalendarioCriteria appCr, Utente utente) {
		List<String> sis = new ArrayList<String>();
		for(int i = appCr.getInizioA(); i <= appCr.getFineA(); i++) {
			sis.add(String.format("%02d", i, 0));
		}
		List<Agente> agenti = new ArrayList<Agente>();
		if (ApplicationUtil.gtZero(appCr.getAgenteId())) {
			agenti = new ArrayList<Agente>();
			agenti.add(campagnaDao.getAgente(appCr.getAgenteId()));
		} else {
			agenti = agentiSel;
		}
		List<Appuntamento> appuntamenti = new ArrayList<Appuntamento>();
		List<CalendarioAgenteVO> calendari = new ArrayList<CalendarioAgenteVO>();
		if(ApplicationUtil.gtZero(appCr.getCampagna_id()) || ApplicationUtil.gtZero(appCr.getZona_id()) || ApplicationUtil.gtZero(appCr.getAgenteId())) {
			Iterator<Agente> objIter = agenti.iterator();
			while(objIter.hasNext()) {
				Agente agente = objIter.next();
				List<GiornoCalendarioAgenteVO> calendario = getAppuntamentiListCoordinatore(appuntamenti, appCr.getGiornoPartenza(), agente, appCr.getGiorni(), appCr, utente.getId(), appCr.getInizioA(), appCr.getFineA());
				CalendarioAgenteVO cvo = new CalendarioAgenteVO();
				cvo.setAgente(agente);
				cvo.setGiorni(calendario);
				cvo.setOrari(sis);
				cvo.setGiorniCalendario(appCr.getGiorni());
				calendari.add(cvo);
			}
		}
		if(ApplicationUtil.notEmpty(appuntamenti))  {
			String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.APPUNTAMENTO.stampa() +": " + ApplicationUtil.size(appuntamenti), appuntamenti);
			operazioneDao.creaOperazione(ConstantsMap.LOG4, p ,utente, ConstantsMap.OPERAZIONE_VISUALIZZA_APPUNTAMENTO);
		}
		return calendari;
	}
	private List<GiornoCalendarioAgenteVO> getAppuntamentiListSupervisore(List<Appuntamento> appuntamentiTot, Date giornoPartenza, Agente agente, int giorni, CalendarioCriteria appCr, Long userId, int oramin, int oramax) {
		List<GiornoCalendarioAgenteVO> calendario = new ArrayList<GiornoCalendarioAgenteVO>();
		Date attuale = ApplicationUtil.date();
		if (agente != null) {
			Date inizio = ApplicationUtil.inizioGiornata(giornoPartenza);
			Date fine = ApplicationUtil.fineGiornata(ApplicationUtil.addDays(inizio, giorni));
			for(int i = 0; i < giorni; i++) {
				Date giorno_calendario = ApplicationUtil.addDays(inizio, i);
				GiornoCalendarioAgenteVO gc = new GiornoCalendarioAgenteVO();
				gc.setAppuntamenti(new ArrayList<Appuntamento>());
				gc.setNote(new ArrayList<Annotazione>());
				gc.setData(giorno_calendario);
				gc.setAgente(agente);
				calendario.add(gc);
			}
			List<Appuntamento> appuntamenti = trovaListaAppuntamentiSupervisore(inizio, fine, agente.getId(),  appCr, userId);
			List<Nota> note = trovaListaNote(inizio, fine, agente.getId());
			List<Orario> orari = new ArrayList<Orario>();
			Orario orario = new Orario();
			orario.setData(attuale);
			orari.add(orario);
			appuntamentoDao.completaCalendario(calendario, appuntamenti, note, orari, oramin, oramax, attuale);
			appuntamentiTot.addAll(appuntamenti);
		}
		return calendario;
	}
	private List<GiornoCalendarioAgenteVO> getAppuntamentiListCoordinatore(List<Appuntamento> appuntamentiTot, Date giornoPartenza, Agente agente, int giorni, CalendarioCriteria appCr, Long userId, int oramin, int oramax) {
		List<GiornoCalendarioAgenteVO> calendario = new ArrayList<GiornoCalendarioAgenteVO>();
		Date attuale = ApplicationUtil.date();
		if (agente != null) {
			Date inizio = ApplicationUtil.inizioGiornata(giornoPartenza);
			Date fine = ApplicationUtil.fineGiornata(ApplicationUtil.addDays(inizio, giorni));
			for(int i = 0; i < giorni; i++) {
				Date giorno_calendario = ApplicationUtil.addDays(inizio, i);
				GiornoCalendarioAgenteVO gc = new GiornoCalendarioAgenteVO();
				gc.setAppuntamenti(new ArrayList<Appuntamento>());
				gc.setNote(new ArrayList<Annotazione>());
				gc.setData(giorno_calendario);
				gc.setAgente(agente);
				calendario.add(gc);
			}
			List<Appuntamento> appuntamenti = trovaListaAppuntamentiCoordinatore(inizio, fine, agente.getId(),  appCr, userId);
			List<Nota> note = trovaListaNote(inizio, fine, agente.getId());
			List<Orario> orari = new ArrayList<Orario>();
			Orario orario = new Orario();
			orario.setData(attuale);
			orari.add(orario);
			appuntamentoDao.completaCalendario(calendario, appuntamenti, note, orari, oramin, oramax, attuale);
			appuntamentiTot.addAll(appuntamenti);
		}
		return calendario;
	}
	@SuppressWarnings("unchecked")
	public List<Appuntamento> trovaListaAppuntamentiSupervisore(Date inizio, Date fine, Long agenteId, CalendarioCriteria appCr, Long userId) {
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(Appuntamento.class);
		DetachedCriteria criteriaC = criteria.createCriteria("esito", ApplicationUtil.leftJoin());
		DetachedCriteria criteriaCA = criteria.createCriteria("contatto").createCriteria("campagna", "campagna");
		DetachedCriteria c = criteria.createCriteria("operatore", "operatore");
		criteria.add(Restrictions.eq("agente.id", agenteId));
		impostazioneDao.subCriteriaRoleSupervisore(userId, criteria, "campagna", "operatore");
		impostazioneDao.subCriteriaSupervisore(userId, c, "id");
		if(ApplicationUtil.isTrue(sd)) {
			impostazioneDao.subCriteriaCampagnaSupervisore(userId, criteriaCA, "id");
		}
		if (ApplicationUtil.gtZero(appCr.getStato())) {
			criteria.add(Restrictions.eq("stato", appCr.getStato()));
		}
		if (ApplicationUtil.gtZero(appCr.getConfermato())){
			if(ApplicationUtil.eq(appCr.getConfermato(), 1))
				criteriaC.add(Restrictions.eq("confermato", Boolean.TRUE));
			else 
				criteriaC.add(Restrictions.eq("confermato", Boolean.FALSE));
		}
		if (ApplicationUtil.gtZero(appCr.getEsito())) {
			criteriaC.add(Restrictions.idEq(appCr.getEsito()));
		} 
		ApplicationUtil.dataDate(inizio, fine, "data_inizio", criteria);
		criteria.addOrder(Order.asc("data_inizio"));
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Appuntamento> trovaListaAppuntamentiCoordinatore(Date inizio, Date fine, Long agenteId, CalendarioCriteria appCr, Long userId) {
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(Appuntamento.class);
		DetachedCriteria criteriaAg = criteria.createCriteria("agente", "agente");
		criteriaAg.add(Restrictions.idEq(agenteId));
		DetachedCriteria criteriaCA = criteria.createCriteria("contatto").createCriteria("campagna", "campagna");
		impostazioneDao.subCriteriaRoleCoordinatore(userId, criteria, "campagna", "agente");
		if(ApplicationUtil.isTrue(sd)) {
			impostazioneDao.subCriteriaCoordinatoreCampagna(userId, criteriaCA, "id");
		}
		DetachedCriteria criteriaC = criteria.createCriteria("esito", ApplicationUtil.leftJoin());
		if (ApplicationUtil.gtZero(appCr.getStato())) {
			criteria.add(Restrictions.eq("stato", appCr.getStato()));
		}
		if (ApplicationUtil.gtZero(appCr.getConfermato())){
			if(ApplicationUtil.eq(appCr.getConfermato(), 1))
				criteriaC.add(Restrictions.eq("confermato", Boolean.TRUE));
			else 
				criteriaC.add(Restrictions.eq("confermato", Boolean.FALSE));
		}
		if (ApplicationUtil.gtZero(appCr.getEsito())) {
			criteriaC.add(Restrictions.idEq(appCr.getEsito()));
		} 
		ApplicationUtil.dataDate(inizio, fine, "data_inizio", criteria);
		criteria.addOrder(Order.asc("data_inizio"));
		return getList(criteria);
	}
	public void getRiepilogoAgenteSupervisore(Utente utente, CalendarioCriteria appCr, Agente agente, Date ora,  DownloadBean downloadBean) {
		String html = impostazioneDao.getFormatoDocumento();
		String s = ConstantsGlobal.secretL;	
		String p = impostazioneDao.getPasswordZIP();
		List<Appuntamento> appuntamenti = new ArrayList<Appuntamento>();
		Date now = ApplicationUtil.inizioGiornata(ora);
		appuntamenti.addAll(trovaListaAppuntamentiSupervisore(now, ApplicationUtil.addDays(now, 1), agente.getId(), appCr, utente.getId()));
		List<Allegato> files = new ArrayList<Allegato>();
		Iterator<Appuntamento> objAppIter = appuntamenti.iterator();
		if (ApplicationUtil.notEmpty(appuntamenti)) {
			String extM = (s == null || (html.equalsIgnoreCase(ConstantsMap.pdf))) ? html : ConstantsMap.zip;
			while(objAppIter.hasNext()) {
				Appuntamento appuntamento = objAppIter.next();
				try {
					String nomeFile = ApplicationUtil.reportFile(appuntamento.getData_inizio(), ConstantsLanguage.APPUNTAMENTO.stampa(), appuntamento.getId(), appuntamento.getContatto().getAnagrafica().getDenominazione())  + "." + extM;
					ByteArrayOutputStream result = downloadDao.appuntamentoToHTMLString(utente, appuntamento,  true, true, html, null, false, s);
					result.close();
					ApplicationUtil.addFile(files, nomeFile, result.toByteArray(), appuntamento.getData_creazione().getTime());
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				}
			}
			String pg = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.APPUNTAMENTO.stampa() +": " + ApplicationUtil.size(appuntamenti), appuntamenti);
			operazioneDao.creaOperazione(ConstantsMap.LOG4, pg ,utente, ConstantsMap.OPERAZIONE_EXPORT_APPUNTAMENTO);
		}
		try {
			if (ApplicationUtil.empty(files)) {
				ApplicationUtil.fileWriter(files, ConstantsLanguage.NO_APPUNTAMENTO.stampa(), ConstantsLanguage.NO_APPUNTAMENTI.stampa());
			}
			String fileName = ApplicationUtil.reportFile(ora, ConstantsLanguage.APPUNTAMENTI.stampa()) + "." + ConstantsMap.zip;
			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(fileName, files);
			ApplicationUtil.zip(null,files, out, p);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		} 
	}
	public void getRiepilogoAgenteCoordinatore(Utente utente, CalendarioCriteria appCr, Agente agente, Date ora,  DownloadBean downloadBean) {
		String html = impostazioneDao.getFormatoDocumento();
		String s = ConstantsGlobal.secretL;
		String p = impostazioneDao.getPasswordZIP();
		List<Appuntamento> appuntamenti = new ArrayList<Appuntamento>();
		Date now = ApplicationUtil.inizioGiornata(ora);
		appuntamenti.addAll(trovaListaAppuntamentiCoordinatore(now, ApplicationUtil.addDays(now, 1), agente.getId(), appCr, utente.getId()));
		List<Allegato> files = new ArrayList<Allegato>();
		Iterator<Appuntamento> objAppIter = appuntamenti.iterator();
		if (ApplicationUtil.notEmpty(appuntamenti)) {
			String extM = (s == null || (html.equalsIgnoreCase(ConstantsMap.pdf))) ? html : ConstantsMap.zip;
			while(objAppIter.hasNext()) {
				Appuntamento appuntamento = objAppIter.next();
				try {
					String nomeFile = ApplicationUtil.reportFile(appuntamento.getData_inizio(), ConstantsLanguage.APPUNTAMENTO.stampa(), appuntamento.getId(), appuntamento.getContatto().getAnagrafica().getDenominazione())  + "." + extM;
					ByteArrayOutputStream result = downloadDao.appuntamentoToHTMLString(utente, appuntamento,  true, true, html, null, false, s);
					result.close();
					ApplicationUtil.addFile(files, nomeFile, result.toByteArray(), appuntamento.getData_creazione().getTime());
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				}
			}
			String pg = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.APPUNTAMENTO.stampa() +": " + ApplicationUtil.size(appuntamenti), appuntamenti);
			operazioneDao.creaOperazione(ConstantsMap.LOG4, pg ,utente, ConstantsMap.OPERAZIONE_EXPORT_APPUNTAMENTO);
		}
		try {
			if (ApplicationUtil.empty(files)) {
				ApplicationUtil.fileWriter(files, ConstantsLanguage.NO_APPUNTAMENTO.stampa(), ConstantsLanguage.NO_APPUNTAMENTI.stampa());
			}
			String fileName = ApplicationUtil.reportFile(ora, ConstantsLanguage.APPUNTAMENTI.stampa()) + "." + ConstantsMap.zip;
			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(fileName, files);
			ApplicationUtil.zip(null,files, out, p);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		} 
	}
	@SuppressWarnings("unchecked")
	public List<Agente> getAgenteByCampagnaIdSupervisore(Long userid, CalendarioCriteria appCr) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
		DetachedCriteria criteriaZ = criteria.createCriteria("zone");  
		DetachedCriteria criteriaC = criteriaZ.createCriteria("campagne");
		criteriaC.createCriteria("team").createCriteria("supervisore").add(Restrictions.idEq(userid));
		if(ApplicationUtil.gtZero(appCr.getZona_id())) {
			criteriaZ.add(Restrictions.idEq(appCr.getZona_id()));
		}
		if(ApplicationUtil.gtZero(appCr.getCampagna_id())) {
			criteriaC.add(Restrictions.idEq(appCr.getCampagna_id()));
		}
		ApplicationUtil.orderA(criteria);
		List<Agente> agenti = getList(criteria);
		return agenti;
	}
	public List<SelectItem> getZonaByCampagnaIdSupervisore(Long userid, CalendarioCriteria appCr) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Zona.class);
		DetachedCriteria criteriaC = criteria.createCriteria("campagne");
		criteriaC.createCriteria("team").createCriteria("supervisore").add(Restrictions.idEq(userid));
		if(ApplicationUtil.gtZero(appCr.getAgenteId())) {
			criteria.createCriteria("agente").add(Restrictions.idEq(appCr.getAgenteId()));
		}
		if(ApplicationUtil.gtZero(appCr.getCampagna_id())) {
			criteriaC.add(Restrictions.idEq(appCr.getCampagna_id()));
		}
		List<ZonaSI> lis = getZonaSI(criteria);
		return impostazioneDao.getZona(lis, false, false, null);
	}
	public List<SelectItem> getCampagneSISupervisore(Long userid, CalendarioCriteria appCr) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		criteria.createCriteria("team").createCriteria("supervisore").add(Restrictions.idEq(userid));
		DetachedCriteria criteriaC = criteria.createCriteria("zone");
		if(ApplicationUtil.gtZero(appCr.getAgenteId())) {
			criteriaC.createCriteria("agente").add(Restrictions.idEq(appCr.getAgenteId()));
		}
		if(ApplicationUtil.gtZero(appCr.getZona_id())) {
			criteriaC.add(Restrictions.idEq(appCr.getZona_id()));
		}
		List<CampagnaSI> lis = getCampagnaSI(criteria);
		return impostazioneDao.getCampagna(lis, true, false, false, null);
	}
	@SuppressWarnings("unchecked")
	public List<Agente> getAgenteByCampagnaIdCoordinatore(Long userid, CalendarioCriteria appCr) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
		DetachedCriteria criteriaZ = criteria.createCriteria("zone");
		criteriaZ.createCriteria("supervisori").add(Restrictions.idEq(userid));
		if(ApplicationUtil.gtZero(appCr.getZona_id())) {
			criteriaZ.add(Restrictions.idEq(appCr.getZona_id()));
		}
		if(ApplicationUtil.gtZero(appCr.getCampagna_id())) {
			criteriaZ.createCriteria("campagne").add(Restrictions.idEq(appCr.getCampagna_id()));
		}
		ApplicationUtil.orderA(criteria);
		List<Agente> agenti = getList(criteria);
		return agenti;
	}
	public List<SelectItem> getZonaByCampagnaIdCoordinatore(Long userid, CalendarioCriteria appCr) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Zona.class);
		criteria.createCriteria("supervisori").add(Restrictions.idEq(userid));
		if(ApplicationUtil.gtZero(appCr.getAgenteId())) {
			criteria.createCriteria("agente").add(Restrictions.idEq(appCr.getAgenteId()));
		}
		if(ApplicationUtil.gtZero(appCr.getCampagna_id())) {
			criteria.createCriteria("campagne").add(Restrictions.idEq(appCr.getCampagna_id()));
		}
		List<ZonaSI> lis = getZonaSI(criteria);
		return impostazioneDao.getZona(lis, false, false, null);
	}
	public List<SelectItem> getCampagneSICoordinatore(Long userid, CalendarioCriteria appCr) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		DetachedCriteria criteriaC = criteria.createCriteria("zone");
		criteriaC.createCriteria("supervisori").add(Restrictions.idEq(userid));
		if(ApplicationUtil.gtZero(appCr.getAgenteId())) {
			criteriaC.createCriteria("agente").add(Restrictions.idEq(appCr.getAgenteId()));
		}
		if(ApplicationUtil.gtZero(appCr.getZona_id())) {
			criteriaC.add(Restrictions.idEq(appCr.getZona_id()));
		}
		List<CampagnaSI> lis = getCampagnaSI(criteria);
		return impostazioneDao.getCampagna(lis, true, false, false, null);
	}
	public List<CalendarioAgenteVO> getAppuntamentiListPartner(List<Agente> agentiSel, CalendarioCriteria appCr, Utente utente) {
		List<String> sis = new ArrayList<String>();
		for(int i = appCr.getInizioA(); i <= appCr.getFineA(); i++) {
			sis.add(String.format("%02d", i, 0));
		}
		List<Agente> agenti = new ArrayList<Agente>();
		if (ApplicationUtil.gtZero(appCr.getAgenteId())) {
			agenti = new ArrayList<Agente>();
			agenti.add(campagnaDao.getAgente(appCr.getAgenteId()));
		} else {
			agenti = agentiSel;
		}
		List<Appuntamento> appuntamenti = new ArrayList<Appuntamento>();
		List<CalendarioAgenteVO> calendari = new ArrayList<CalendarioAgenteVO>();
		if(ApplicationUtil.gtZero(appCr.getCampagna_id()) || ApplicationUtil.gtZero(appCr.getZona_id()) || ApplicationUtil.gtZero(appCr.getAgenteId())) {
			Iterator<Agente> objIter = agenti.iterator();
			while(objIter.hasNext()) {
				Agente agente = objIter.next();
				List<GiornoCalendarioAgenteVO> calendario = getAppuntamentiListPartner(appuntamenti, appCr.getGiornoPartenza(), agente, appCr.getGiorni(), appCr, utente.getId(), appCr.getInizioA(), appCr.getFineA());
				CalendarioAgenteVO cvo = new CalendarioAgenteVO();
				cvo.setAgente(agente);
				cvo.setGiorni(calendario);
				cvo.setOrari(sis);
				cvo.setGiorniCalendario(appCr.getGiorni());
				calendari.add(cvo);
			}
		}
		if(ApplicationUtil.notEmpty(appuntamenti))  {
			String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.APPUNTAMENTO.stampa() +": " + ApplicationUtil.size(appuntamenti), appuntamenti);
			operazioneDao.creaOperazione(ConstantsMap.LOG4, p ,utente, ConstantsMap.OPERAZIONE_VISUALIZZA_APPUNTAMENTO);
		}
		return calendari;
	}
	private List<GiornoCalendarioAgenteVO> getAppuntamentiListPartner(List<Appuntamento> appuntamentiTot, Date giornoPartenza, Agente agente, int giorni, CalendarioCriteria appCr, Long userId, int oramin, int oramax) {
		List<GiornoCalendarioAgenteVO> calendario = new ArrayList<GiornoCalendarioAgenteVO>();
		Date attuale = ApplicationUtil.date();
		if (agente != null) {
			Date inizio = ApplicationUtil.inizioGiornata(giornoPartenza);
			Date fine = ApplicationUtil.fineGiornata(ApplicationUtil.addDays(inizio, giorni));
			for(int i = 0; i < giorni; i++) {
				Date giorno_calendario = ApplicationUtil.addDays(inizio, i);
				GiornoCalendarioAgenteVO gc = new GiornoCalendarioAgenteVO();
				gc.setAppuntamenti(new ArrayList<Appuntamento>());
				gc.setNote(new ArrayList<Annotazione>());
				gc.setData(giorno_calendario);
				gc.setAgente(agente);
				calendario.add(gc);
			}
			List<Appuntamento> appuntamenti = trovaListaAppuntamentiPartner(inizio, fine, agente.getId(),  appCr, userId);
			List<Nota> note = trovaListaNote(inizio, fine, agente.getId());
			List<Orario> orari = new ArrayList<Orario>();
			Orario orario = new Orario();
			orario.setData(attuale);
			orari.add(orario);
			appuntamentoDao.completaCalendario(calendario, appuntamenti, note, orari, oramin, oramax, attuale);
			appuntamentiTot.addAll(appuntamenti);
		}
		return calendario;
	}
	@SuppressWarnings("unchecked")
	public List<Appuntamento> trovaListaAppuntamentiPartner(Date inizio, Date fine, Long agenteId, CalendarioCriteria appCr, Long userId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Appuntamento.class);
		DetachedCriteria criteriaCa = criteria.createCriteria("contatto").createCriteria("campagna");
		impostazioneDao.subCriteriaCampagnaPartner(userId, criteriaCa, "id");
		DetachedCriteria criteriaC = criteria.createCriteria("esito", ApplicationUtil.leftJoin());
		criteria.add(Restrictions.eq("agente.id", agenteId));
		if (ApplicationUtil.gtZero(appCr.getStato())) {
			criteria.add(Restrictions.eq("stato", appCr.getStato()));
		}
		if (ApplicationUtil.gtZero(appCr.getConfermato())){
			if(ApplicationUtil.eq(appCr.getConfermato(), 1))
				criteriaC.add(Restrictions.eq("confermato", Boolean.TRUE));
			else 
				criteriaC.add(Restrictions.eq("confermato", Boolean.FALSE));
		}
		if (ApplicationUtil.gtZero(appCr.getEsito())) {
			criteriaC.add(Restrictions.idEq(appCr.getEsito()));
		} 
		ApplicationUtil.dataDate(inizio, fine, "data_inizio", criteria);
		criteria.addOrder(Order.asc("data_inizio"));
		return getList(criteria);
	}
	public void getRiepilogoAgentePartner(Utente utente, CalendarioCriteria appCr, Agente agente, Date ora,  DownloadBean downloadBean) {
		String html = impostazioneDao.getFormatoDocumento();
		String s = ConstantsGlobal.secretL;
		String p = impostazioneDao.getPasswordZIP();
		List<Appuntamento> appuntamenti = new ArrayList<Appuntamento>();
		Date now = ApplicationUtil.inizioGiornata(ora);
		appuntamenti.addAll(trovaListaAppuntamentiPartner(now, ApplicationUtil.addDays(now, 1), agente.getId(), appCr, utente.getId()));
		List<Allegato> files = new ArrayList<Allegato>();
		Iterator<Appuntamento> objAppIter = appuntamenti.iterator();
		if (ApplicationUtil.notEmpty(appuntamenti)) {
			String extM = (s == null || (html.equalsIgnoreCase(ConstantsMap.pdf))) ? html : ConstantsMap.zip;
			while(objAppIter.hasNext()) {
				Appuntamento appuntamento = objAppIter.next();
				try {
					String nomeFile = ApplicationUtil.reportFile(appuntamento.getData_inizio(), ConstantsLanguage.APPUNTAMENTO.stampa(), appuntamento.getId(), appuntamento.getContatto().getAnagrafica().getDenominazione())  + "." + extM;
					ByteArrayOutputStream result = downloadDao.appuntamentoToHTMLString(utente, appuntamento,  true, true, html, null, false, s);
					result.close();
					ApplicationUtil.addFile(files, nomeFile, result.toByteArray(), appuntamento.getData_creazione().getTime());
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				}
			}
			String pg = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.APPUNTAMENTO.stampa() +": " + ApplicationUtil.size(appuntamenti), appuntamenti);
			operazioneDao.creaOperazione(ConstantsMap.LOG4, pg ,utente, ConstantsMap.OPERAZIONE_EXPORT_APPUNTAMENTO);
		}
		try {
			if (ApplicationUtil.empty(files)) {
				ApplicationUtil.fileWriter(files, ConstantsLanguage.NO_APPUNTAMENTO.stampa(), ConstantsLanguage.NO_APPUNTAMENTI.stampa());
			}
			String fileName = ApplicationUtil.reportFile(ora, ConstantsLanguage.APPUNTAMENTI.stampa()) + "." + ConstantsMap.zip;
			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(fileName, files);
			ApplicationUtil.zip(null,files, out, p);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		} 
	}
	@SuppressWarnings("unchecked")
	public List<Agente> getAgenteByCampagnaIdPartner(Long userid, CalendarioCriteria appCr) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
		DetachedCriteria criteriaZ = criteria.createCriteria("zone");  
		DetachedCriteria criteriaC = criteriaZ.createCriteria("campagne");
		criteriaC.createCriteria("partner").add(Restrictions.idEq(userid));
		if(ApplicationUtil.gtZero(appCr.getZona_id())) {
			criteriaZ.add(Restrictions.idEq(appCr.getZona_id()));
		}
		if(ApplicationUtil.gtZero(appCr.getCampagna_id())) {
			criteriaC.add(Restrictions.idEq(appCr.getCampagna_id()));
		}
		ApplicationUtil.orderA(criteria);
		List<Agente> agenti = getList(criteria);
		return agenti;
	}
	public List<SelectItem> getZonaByCampagnaIdPartner(Long userid, CalendarioCriteria appCr) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Zona.class);
		DetachedCriteria criteriaC = criteria.createCriteria("campagne");
		criteriaC.createCriteria("partner").add(Restrictions.idEq(userid));
		if(ApplicationUtil.gtZero(appCr.getAgenteId())) {
			criteria.createCriteria("agente").add(Restrictions.idEq(appCr.getAgenteId()));
		}
		if(ApplicationUtil.gtZero(appCr.getCampagna_id())) {
			criteriaC.add(Restrictions.idEq(appCr.getCampagna_id()));
		}
		List<ZonaSI> lis = getZonaSI(criteria);
		return impostazioneDao.getZona(lis, false, false, null);
	}
	public List<SelectItem> getCampagneSIPartner(Long userid, CalendarioCriteria appCr) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		criteria.createCriteria("partner").add(Restrictions.idEq(userid));
		DetachedCriteria criteriaC = criteria.createCriteria("zone");
		if(ApplicationUtil.gtZero(appCr.getAgenteId())) {
			criteriaC.createCriteria("agente").add(Restrictions.idEq(appCr.getAgenteId()));
		}
		if(ApplicationUtil.gtZero(appCr.getZona_id())) {
			criteriaC.add(Restrictions.idEq(appCr.getZona_id()));
		}
		List<CampagnaSI> lis = getCampagnaSI(criteria);
		return impostazioneDao.getCampagna(lis, true, false, false, null);
	}
	public List<CalendarioAgenteVO> getAppuntamentiListOperatore(List<Agente> agentiSel, CalendarioCriteria appCr, Utente utente) {
		List<String> sis = new ArrayList<String>();
		for(int i = appCr.getInizioA(); i <= appCr.getFineA(); i++) {
			sis.add(String.format("%02d", i, 0));
		}
		List<Agente> agenti = new ArrayList<Agente>();
		if (ApplicationUtil.gtZero(appCr.getAgenteId())) {
			agenti = new ArrayList<Agente>();
			agenti.add(campagnaDao.getAgente(appCr.getAgenteId()));
		} else {
			agenti = agentiSel;
		}
		List<Appuntamento> appuntamenti = new ArrayList<Appuntamento>();
		List<CalendarioAgenteVO> calendari = new ArrayList<CalendarioAgenteVO>();
		if(ApplicationUtil.gtZero(appCr.getCampagna_id()) || ApplicationUtil.gtZero(appCr.getZona_id()) || ApplicationUtil.gtZero(appCr.getAgenteId())) {
			Iterator<Agente> objIter = agenti.iterator();
			while(objIter.hasNext()) {
				Agente agente = objIter.next();
				List<GiornoCalendarioAgenteVO> calendario = getAppuntamentiListOperatore(appuntamenti, appCr.getGiornoPartenza(), agente, appCr.getGiorni(), appCr, utente.getId(), appCr.getInizioA(), appCr.getFineA());
				CalendarioAgenteVO cvo = new CalendarioAgenteVO();
				cvo.setAgente(agente);
				cvo.setGiorni(calendario);
				cvo.setOrari(sis);
				cvo.setGiorniCalendario(appCr.getGiorni());
				calendari.add(cvo);
			}
		}
		if(ApplicationUtil.notEmpty(appuntamenti))  {
			String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.APPUNTAMENTO.stampa() +": " + ApplicationUtil.size(appuntamenti), appuntamenti);
			operazioneDao.creaOperazione(ConstantsMap.LOG4, p ,utente, ConstantsMap.OPERAZIONE_VISUALIZZA_APPUNTAMENTO);
		}
		return calendari;
	}
	private List<GiornoCalendarioAgenteVO> getAppuntamentiListOperatore(List<Appuntamento> appuntamentiTot, Date giornoPartenza, Agente agente, int giorni, CalendarioCriteria appCr, Long userId, int oramin, int oramax) {
		List<GiornoCalendarioAgenteVO> calendario = new ArrayList<GiornoCalendarioAgenteVO>();
		Date attuale = ApplicationUtil.date();
		if (agente != null) {
			Date inizio = ApplicationUtil.inizioGiornata(giornoPartenza);
			Date fine = ApplicationUtil.fineGiornata(ApplicationUtil.addDays(inizio, giorni));
			for(int i = 0; i < giorni; i++) {
				Date giorno_calendario = ApplicationUtil.addDays(inizio, i);
				GiornoCalendarioAgenteVO gc = new GiornoCalendarioAgenteVO();
				gc.setAppuntamenti(new ArrayList<Appuntamento>());
				gc.setNote(new ArrayList<Annotazione>());
				gc.setData(giorno_calendario);
				gc.setAgente(agente);
				calendario.add(gc);
			}
			List<Appuntamento> appuntamenti = trovaListaAppuntamentiOperatore(inizio, fine, agente.getId(),  appCr, userId);
			List<Nota> note = trovaListaNote(inizio, fine, agente.getId());
			List<Orario> orari = new ArrayList<Orario>();
			Orario orario = new Orario();
			orario.setData(attuale);
			orari.add(orario);
			appuntamentoDao.completaCalendario(calendario, appuntamenti, note, orari, oramin, oramax, attuale);
			appuntamentiTot.addAll(appuntamenti);
		}
		return calendario;
	}
	@SuppressWarnings("unchecked")
	public List<Appuntamento> trovaListaAppuntamentiOperatore(Date inizio, Date fine, Long agenteId, CalendarioCriteria appCr, Long userId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Appuntamento.class);
		DetachedCriteria criteriaC = criteria.createCriteria("esito", ApplicationUtil.leftJoin());
		criteria.add(Restrictions.eq("agente.id", agenteId));
		if (ApplicationUtil.gtZero(appCr.getStato())) {
			criteria.add(Restrictions.eq("stato", appCr.getStato()));
		}
		if (ApplicationUtil.gtZero(appCr.getConfermato())){
			if(ApplicationUtil.eq(appCr.getConfermato(), 1))
				criteriaC.add(Restrictions.eq("confermato", Boolean.TRUE));
			else 
				criteriaC.add(Restrictions.eq("confermato", Boolean.FALSE));
		}
		if (ApplicationUtil.gtZero(appCr.getEsito())) {
			criteriaC.add(Restrictions.idEq(appCr.getEsito()));
		} 
		DetachedCriteria c = criteria.createCriteria("operatore");
		impostazioneDao.subCriteriaOperatore(userId, c, "id");
		ApplicationUtil.dataDate(inizio, fine, "data_inizio", criteria);
		criteria.addOrder(Order.asc("data_inizio"));
		return getList(criteria);
	}
	public void getRiepilogoAgenteOperatore(Utente utente, CalendarioCriteria appCr, Agente agente, Date ora,  DownloadBean downloadBean) {
		String html = impostazioneDao.getFormatoDocumento();
		String s = ConstantsGlobal.secretL;
		String p = impostazioneDao.getPasswordZIP();
		List<Appuntamento> appuntamenti = new ArrayList<Appuntamento>();
		Date now = ApplicationUtil.inizioGiornata(ora);
		appuntamenti.addAll(trovaListaAppuntamentiOperatore(now, ApplicationUtil.addDays(now, 1), agente.getId(), appCr, utente.getId()));
		List<Allegato> files = new ArrayList<Allegato>();
		Iterator<Appuntamento> objAppIter = appuntamenti.iterator();
		if (ApplicationUtil.notEmpty(appuntamenti)) {
			String extM = (s == null || (html.equalsIgnoreCase(ConstantsMap.pdf))) ? html : ConstantsMap.zip;
			while(objAppIter.hasNext()) {
				Appuntamento appuntamento = objAppIter.next();
				try {
					String nomeFile = ApplicationUtil.reportFile(appuntamento.getData_inizio(), ConstantsLanguage.APPUNTAMENTO.stampa(), appuntamento.getId(), appuntamento.getContatto().getAnagrafica().getDenominazione())  + "." + extM;
					ByteArrayOutputStream result = downloadDao.appuntamentoToHTMLString(utente, appuntamento,  true, false, html, false, false, s);
					result.close();
					ApplicationUtil.addFile(files, nomeFile, result.toByteArray(), appuntamento.getData_creazione().getTime());
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				}
			}
			String pg = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.APPUNTAMENTO.stampa() +": " + ApplicationUtil.size(appuntamenti), appuntamenti);
			operazioneDao.creaOperazione(ConstantsMap.LOG4, pg ,utente, ConstantsMap.OPERAZIONE_EXPORT_APPUNTAMENTO);
		}
		try {
			if (ApplicationUtil.empty(files)) {
				ApplicationUtil.fileWriter(files, ConstantsLanguage.NO_APPUNTAMENTO.stampa(), ConstantsLanguage.NO_APPUNTAMENTI.stampa());
			}
			String fileName = ApplicationUtil.reportFile(ora, ConstantsLanguage.APPUNTAMENTI.stampa()) + "." + ConstantsMap.zip;
			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(fileName, files);
			ApplicationUtil.zip(null,files, out, p);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		} 
	}
	@SuppressWarnings("unchecked")
	public List<Agente> getAgenteByCampagnaIdOperatore(Long userid, CalendarioCriteria appCr) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
		DetachedCriteria criteriaZ = criteria.createCriteria("zone");  
		DetachedCriteria criteriaC = criteriaZ.createCriteria("campagne");
		criteriaC.createCriteria("team").createCriteria("user").add(Restrictions.idEq(userid));
		if(ApplicationUtil.gtZero(appCr.getZona_id())) {
			criteriaZ.add(Restrictions.idEq(appCr.getZona_id()));
		}
		if(ApplicationUtil.gtZero(appCr.getCampagna_id())) {
			criteriaC.add(Restrictions.idEq(appCr.getCampagna_id()));
		}
		ApplicationUtil.orderA(criteria);
		List<Agente> agenti = getList(criteria);
		return agenti;
	}
	public List<SelectItem> getZonaByCampagnaIdOperatore(Long userid, CalendarioCriteria appCr) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Zona.class);
		DetachedCriteria criteriaC = criteria.createCriteria("campagne");
		criteriaC.createCriteria("team").createCriteria("user").add(Restrictions.idEq(userid));
		if(ApplicationUtil.gtZero(appCr.getAgenteId())) {
			criteria.createCriteria("agente").add(Restrictions.idEq(appCr.getAgenteId()));
		}
		if(ApplicationUtil.gtZero(appCr.getCampagna_id())) {
			criteriaC.add(Restrictions.idEq(appCr.getCampagna_id()));
		}
		List<ZonaSI> lis = getZonaSI(criteria);
		return impostazioneDao.getZona(lis, false, false, null);
	}
	public List<SelectItem> getCampagneSIOperatore(Long userid, CalendarioCriteria appCr) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		criteria.createCriteria("team").createCriteria("user").add(Restrictions.idEq(userid));
		DetachedCriteria criteriaC = criteria.createCriteria("zone");
		if(ApplicationUtil.gtZero(appCr.getAgenteId())) {
			criteriaC.createCriteria("agente").add(Restrictions.idEq(appCr.getAgenteId()));
		}
		if(ApplicationUtil.gtZero(appCr.getZona_id())) {
			criteriaC.add(Restrictions.idEq(appCr.getZona_id()));
		}
		List<CampagnaSI> lis = getCampagnaSI(criteria);
		return impostazioneDao.getCampagna(lis, true, false, false, null);
	}
	public List<CalendarioAgenteVO> getAppuntamentiListAgente(List<Agente> agentiSel, CalendarioCriteria appCr, Utente utente) {
		List<String> sis = new ArrayList<String>();
		for(int i = appCr.getInizioA(); i <= appCr.getFineA(); i++) {
			sis.add(String.format("%02d", i, 0));
		}
		List<Agente> agenti = new ArrayList<Agente>();
		if (ApplicationUtil.gtZero(appCr.getAgenteId())) {
			agenti = new ArrayList<Agente>();
			agenti.add(campagnaDao.getAgente(appCr.getAgenteId()));
		} else {
			agenti = agentiSel;
		}
		List<Appuntamento> appuntamenti = new ArrayList<Appuntamento>();
		List<CalendarioAgenteVO> calendari = new ArrayList<CalendarioAgenteVO>();
		if(ApplicationUtil.gtZero(appCr.getCampagna_id()) || ApplicationUtil.gtZero(appCr.getZona_id()) || ApplicationUtil.gtZero(appCr.getAgenteId())) {
			Iterator<Agente> objIter = agenti.iterator();
			while(objIter.hasNext()) {
				Agente agente = objIter.next();
				List<GiornoCalendarioAgenteVO> calendario = getAppuntamentiListAgente(appuntamenti, appCr.getGiornoPartenza(), agente, appCr.getGiorni(), appCr, utente.getId(), appCr.getInizioA(), appCr.getFineA());
				CalendarioAgenteVO cvo = new CalendarioAgenteVO();
				cvo.setAgente(agente);
				cvo.setGiorni(calendario);
				cvo.setOrari(sis);
				cvo.setGiorniCalendario(appCr.getGiorni());
				calendari.add(cvo);
			}
		}
		if(ApplicationUtil.notEmpty(appuntamenti))  {
			String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.APPUNTAMENTO.stampa() +": " + ApplicationUtil.size(appuntamenti), appuntamenti);
			operazioneDao.creaOperazione(ConstantsMap.LOG4, p ,utente, ConstantsMap.OPERAZIONE_VISUALIZZA_APPUNTAMENTO);
		}
		return calendari;
	}
	private List<GiornoCalendarioAgenteVO> getAppuntamentiListAgente(List<Appuntamento> appuntamentiTot, Date giornoPartenza, Agente agente, int giorni, CalendarioCriteria appCr, Long userId, int oramin, int oramax) {
		List<GiornoCalendarioAgenteVO> calendario = new ArrayList<GiornoCalendarioAgenteVO>();
		Date attuale = ApplicationUtil.date();
		if (agente != null) {
			Date inizio = ApplicationUtil.inizioGiornata(giornoPartenza);
			Date fine = ApplicationUtil.fineGiornata(ApplicationUtil.addDays(inizio, giorni));
			for(int i = 0; i < giorni; i++) {
				Date giorno_calendario = ApplicationUtil.addDays(inizio, i);
				GiornoCalendarioAgenteVO gc = new GiornoCalendarioAgenteVO();
				gc.setAppuntamenti(new ArrayList<Appuntamento>());
				gc.setNote(new ArrayList<Annotazione>());
				gc.setData(giorno_calendario);
				gc.setAgente(agente);
				calendario.add(gc);
			}
			List<Appuntamento> appuntamenti = trovaListaAppuntamentiAgente(inizio, fine, agente.getId(),  appCr, userId);
			List<Nota> note = trovaListaNote(inizio, fine, agente.getId());
			List<Orario> orari = new ArrayList<Orario>();
			Orario orario = new Orario();
			orario.setData(attuale);
			orari.add(orario);
			appuntamentoDao.completaCalendario(calendario, appuntamenti, note, orari, oramin, oramax, attuale);
			appuntamentiTot.addAll(appuntamenti);
		}
		return calendario;
	}
	@SuppressWarnings("unchecked")
	public List<Appuntamento> trovaListaAppuntamentiAgente(Date inizio, Date fine, Long agenteId, CalendarioCriteria appCr, Long userId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Appuntamento.class);
		DetachedCriteria criteriaC = criteria.createCriteria("esito");
		criteria.add(Restrictions.eq("agente.id", agenteId));
		criteriaC.add(Restrictions.eq("confermato", Boolean.TRUE));
		if (ApplicationUtil.gtZero(appCr.getEsito())) {
			criteriaC.add(Restrictions.idEq(appCr.getEsito()));
		} 
		if (ApplicationUtil.gtZero(appCr.getStato())) {
			criteria.add(Restrictions.eq("stato", appCr.getStato()));
		}
		ApplicationUtil.dataDate(inizio, fine, "data_inizio", criteria);
		criteria.addOrder(Order.asc("data_inizio"));
		return  getList(criteria);
	}
	public void getRiepilogoAgenteAgente(Utente utente, CalendarioCriteria appCr, Agente agente, Date ora,  DownloadBean downloadBean) {
		String html = impostazioneDao.getFormatoDocumento();
		String s = ConstantsGlobal.secretL;
		String p = impostazioneDao.getPasswordZIP();
		List<Appuntamento> appuntamenti = new ArrayList<Appuntamento>();
		Date now = ApplicationUtil.inizioGiornata(ora);
		appuntamenti.addAll(trovaListaAppuntamentiAgente(now, ApplicationUtil.addDays(now, 1), agente.getId(), appCr, utente.getId()));
		List<Allegato> files = new ArrayList<Allegato>();
		Iterator<Appuntamento> objAppIter = appuntamenti.iterator();
		if (ApplicationUtil.notEmpty(appuntamenti)) {
			String extM = (s == null || (html.equalsIgnoreCase(ConstantsMap.pdf))) ? html : ConstantsMap.zip;
			while(objAppIter.hasNext()) {
				Appuntamento appuntamento = objAppIter.next();
				try {
					String nomeFile = ApplicationUtil.reportFile(appuntamento.getData_inizio(), ConstantsLanguage.APPUNTAMENTO.stampa(), appuntamento.getId(), appuntamento.getContatto().getAnagrafica().getDenominazione())  + "." + extM;
					boolean vs = appuntamento.getAgente() != null && ApplicationUtil.isTrue(appuntamento.getAgente().getVisualizzaStoria());
					ByteArrayOutputStream result = downloadDao.appuntamentoToHTMLString(utente, appuntamento,  vs, true, html, true, false, s);
					result.close();
					ApplicationUtil.addFile(files, nomeFile, result.toByteArray(), appuntamento.getData_creazione().getTime());
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				}
			}
			String pg = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.APPUNTAMENTO.stampa() +": " + ApplicationUtil.size(appuntamenti), appuntamenti);
			operazioneDao.creaOperazione(ConstantsMap.LOG4, pg ,utente, ConstantsMap.OPERAZIONE_EXPORT_APPUNTAMENTO);
		}
		try {
			if (ApplicationUtil.empty(files)) {
				ApplicationUtil.fileWriter(files, ConstantsLanguage.NO_APPUNTAMENTO.stampa(), ConstantsLanguage.NO_APPUNTAMENTI.stampa());
			}
			String fileName = ApplicationUtil.reportFile(ora, ConstantsLanguage.APPUNTAMENTI.stampa()) + "." + ConstantsMap.zip;
			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(fileName, files);
			ApplicationUtil.zip(null,files, out, p);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		} 
	}
	public void getRiepilogoAgenteRete(Utente utente, CalendarioCriteria appCr, Agente agente, Date ora,  DownloadBean downloadBean) {
		String html = impostazioneDao.getFormatoDocumento();
		String s = ConstantsGlobal.secretL;
		String p = impostazioneDao.getPasswordZIP();
		List<Appuntamento> appuntamenti = new ArrayList<Appuntamento>();
		Date now = ApplicationUtil.inizioGiornata(ora);
		appuntamenti.addAll(trovaListaAppuntamentiAgente(now, ApplicationUtil.addDays(now, 1), agente.getId(), appCr, utente.getId()));
		List<Allegato> files = new ArrayList<Allegato>();
		Iterator<Appuntamento> objAppIter = appuntamenti.iterator();
		if (ApplicationUtil.notEmpty(appuntamenti)) {
			String extM = (s == null || (html.equalsIgnoreCase(ConstantsMap.pdf))) ? html : ConstantsMap.zip;
			while(objAppIter.hasNext()) {
				Appuntamento appuntamento = objAppIter.next();
				try {
					String nomeFile = ApplicationUtil.reportFile(appuntamento.getData_inizio(), ConstantsLanguage.APPUNTAMENTO.stampa(), appuntamento.getId(), appuntamento.getContatto().getAnagrafica().getDenominazione())  + "." + extM;
					boolean vs = appuntamento.getAgente() != null && ApplicationUtil.isTrue(appuntamento.getAgente().getVisualizzaStoria());
					ByteArrayOutputStream result = downloadDao.appuntamentoToHTMLString(utente, appuntamento,  vs, true, html, true, false, s);
					result.close();
					ApplicationUtil.addFile(files, nomeFile, result.toByteArray(), appuntamento.getData_creazione().getTime());
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				}
			}
			String pg = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.APPUNTAMENTO.stampa() +": " + ApplicationUtil.size(appuntamenti), appuntamenti);
			operazioneDao.creaOperazione(ConstantsMap.LOG4, pg ,utente, ConstantsMap.OPERAZIONE_EXPORT_APPUNTAMENTO);
		}
		try {
			if (ApplicationUtil.empty(files)) {
				ApplicationUtil.fileWriter(files, ConstantsLanguage.NO_APPUNTAMENTO.stampa(), ConstantsLanguage.NO_APPUNTAMENTI.stampa());
			}
			String fileName = ApplicationUtil.reportFile(ora, ConstantsLanguage.APPUNTAMENTI.stampa()) + "." + ConstantsMap.zip;
			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(fileName, files);
			ApplicationUtil.zip(null,files, out, p);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		} 
	}
	@SuppressWarnings("unchecked")
	public List<Agente> getAgenteByCampagnaIdAgente(Long userid, CalendarioCriteria appCr) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
		DetachedCriteria criteriaZ = criteria.createCriteria("zone");  
		criteria.createCriteria("user").add(Restrictions.idEq(userid));
		if(ApplicationUtil.gtZero(appCr.getZona_id())) {
			criteriaZ.add(Restrictions.idEq(appCr.getZona_id()));
		}
		if(ApplicationUtil.gtZero(appCr.getCampagna_id())) {
			criteriaZ.createCriteria("campagne").add(Restrictions.idEq(appCr.getCampagna_id()));
		}
		ApplicationUtil.orderA(criteria);
		List<Agente> agenti = getList(criteria);
		return agenti;
	}
	public List<SelectItem> getZonaByCampagnaIdAgente(Long userid, CalendarioCriteria appCr) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Zona.class);
		DetachedCriteria criteriaA = criteria.createCriteria("agente");
		criteriaA.createCriteria("user").add(Restrictions.idEq(userid));
		if(ApplicationUtil.gtZero(appCr.getAgenteId())) {
			criteriaA.add(Restrictions.idEq(appCr.getAgenteId()));
		}
		if(ApplicationUtil.gtZero(appCr.getCampagna_id())) {
			criteria.createCriteria("campagne").add(Restrictions.idEq(appCr.getCampagna_id()));
		}
		List<ZonaSI> lis = getZonaSI(criteria);
		return impostazioneDao.getZona(lis, false, false, null);
	}
	public List<SelectItem> getCampagneSIAgente(Long userid, CalendarioCriteria appCr) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		DetachedCriteria criteriaC = criteria.createCriteria("zone");
		DetachedCriteria criteriaA = criteriaC.createCriteria("agente");
		criteriaA.createCriteria("user").add(Restrictions.idEq(userid));
		if(ApplicationUtil.gtZero(appCr.getAgenteId())) {
			criteriaA.add(Restrictions.idEq(appCr.getAgenteId()));
		}
		if(ApplicationUtil.gtZero(appCr.getZona_id())) {
			criteriaC.add(Restrictions.idEq(appCr.getZona_id()));
		}
		List<CampagnaSI> lis = getCampagnaSI(criteria);
		return impostazioneDao.getCampagna(lis, true, false, false, null);
	}
	public List<CalendarioAgenteVO> getAppuntamentiListRete(List<Agente> agentiSel, CalendarioCriteria appCr, Utente utente) {
		List<String> sis = new ArrayList<String>();
		for(int i = appCr.getInizioA(); i <= appCr.getFineA(); i++) {
			sis.add(String.format("%02d", i, 0));
		}
		List<Agente> agenti = new ArrayList<Agente>();
		if (ApplicationUtil.gtZero(appCr.getAgenteId())) {
			agenti = new ArrayList<Agente>();
			agenti.add(campagnaDao.getAgente(appCr.getAgenteId()));
		} else {
			agenti = agentiSel;
		}
		List<Appuntamento> appuntamenti = new ArrayList<Appuntamento>();
		List<CalendarioAgenteVO> calendari = new ArrayList<CalendarioAgenteVO>();
		if(ApplicationUtil.gtZero(appCr.getCampagna_id()) || ApplicationUtil.gtZero(appCr.getZona_id()) || ApplicationUtil.gtZero(appCr.getAgenteId())) {
			Iterator<Agente> objIter = agenti.iterator();
			while(objIter.hasNext()) {
				Agente agente = objIter.next();
				List<GiornoCalendarioAgenteVO> calendario = getAppuntamentiListRete(appuntamenti, appCr.getGiornoPartenza(), agente, appCr.getGiorni(), appCr, utente.getId(), appCr.getInizioA(), appCr.getFineA());
				CalendarioAgenteVO cvo = new CalendarioAgenteVO();
				cvo.setAgente(agente);
				cvo.setGiorni(calendario);
				cvo.setOrari(sis);
				cvo.setGiorniCalendario(appCr.getGiorni());
				calendari.add(cvo);
			}
		}
		if(ApplicationUtil.notEmpty(appuntamenti))  {
			String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.APPUNTAMENTO.stampa() +": " + ApplicationUtil.size(appuntamenti), appuntamenti);
			operazioneDao.creaOperazione(ConstantsMap.LOG4, p ,utente, ConstantsMap.OPERAZIONE_VISUALIZZA_APPUNTAMENTO);
		}
		return calendari;
	}
	private List<GiornoCalendarioAgenteVO> getAppuntamentiListRete(List<Appuntamento> appuntamentiTot, Date giornoPartenza, Agente agente, int giorni, CalendarioCriteria appCr, Long userId, int oramin, int oramax) {
		List<GiornoCalendarioAgenteVO> calendario = new ArrayList<GiornoCalendarioAgenteVO>();
		Date attuale = ApplicationUtil.date();
		if (agente != null) {
			Date inizio = ApplicationUtil.inizioGiornata(giornoPartenza);
			Date fine = ApplicationUtil.fineGiornata(ApplicationUtil.addDays(inizio, giorni));
			for(int i = 0; i < giorni; i++) {
				Date giorno_calendario = ApplicationUtil.addDays(inizio, i);
				GiornoCalendarioAgenteVO gc = new GiornoCalendarioAgenteVO();
				gc.setAppuntamenti(new ArrayList<Appuntamento>());
				gc.setNote(new ArrayList<Annotazione>());
				gc.setData(giorno_calendario);
				gc.setAgente(agente);
				calendario.add(gc);
			}
			List<Appuntamento> appuntamenti = trovaListaAppuntamentiRete(inizio, fine, agente.getId(),  appCr, userId);
			List<Nota> note = trovaListaNote(inizio, fine, agente.getId());
			List<Orario> orari = new ArrayList<Orario>();
			Orario orario = new Orario();
			orario.setData(attuale);
			orari.add(orario);
			appuntamentoDao.completaCalendario(calendario, appuntamenti, note, orari, oramin, oramax, attuale);
			appuntamentiTot.addAll(appuntamenti);
		}
		return calendario;
	}
	@SuppressWarnings("unchecked")
	public List<Appuntamento> trovaListaAppuntamentiRete(Date inizio, Date fine, Long agenteId, CalendarioCriteria appCr, Long userId) {
		Integer max = userDao.getMaxLivelloRete();
		DetachedCriteria criteria = DetachedCriteria.forClass(Appuntamento.class);
		DetachedCriteria criteriaC = criteria.createCriteria("esito");
		criteriaC.add(Restrictions.eq("confermato", Boolean.TRUE));
		DetachedCriteria criteriaA = criteria.createCriteria("agente");
		impostazioneDao.subCriteriaAgentiSottoReteByUser(userId, 0, max, criteriaA, "id");
		if (ApplicationUtil.gtZero(appCr.getEsito())) {
			criteriaC.add(Restrictions.idEq(appCr.getEsito()));
		} 
		if (ApplicationUtil.gtZero(appCr.getStato())) {
			criteria.add(Restrictions.eq("stato", appCr.getStato()));
		}
		ApplicationUtil.dataDate(inizio, fine, "data_inizio", criteria);
		criteria.addOrder(Order.asc("data_inizio"));
		return  getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Agente> getAgenteByCampagnaIdRete(Long userid, CalendarioCriteria appCr) {
		Integer max = userDao.getMaxLivelloRete();
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
		DetachedCriteria criteriaZ = criteria.createCriteria("zone");  
		impostazioneDao.subCriteriaAgentiSottoReteByUser(userid, 0, max, criteria, "id");
		if(ApplicationUtil.gtZero(appCr.getZona_id())) {
			criteriaZ.add(Restrictions.idEq(appCr.getZona_id()));
		}
		if(ApplicationUtil.gtZero(appCr.getCampagna_id())) {
			criteriaZ.createCriteria("campagne").add(Restrictions.idEq(appCr.getCampagna_id()));
		}
		ApplicationUtil.orderA(criteria);
		List<Agente> agenti = getList(criteria);
		return agenti;
	}
	public List<SelectItem> getZonaByCampagnaIdRete(Long userid, CalendarioCriteria appCr) {
		Integer max = userDao.getMaxLivelloRete();
		DetachedCriteria criteria = DetachedCriteria.forClass(Zona.class);
		DetachedCriteria criteriaA = criteria.createCriteria("agente");
		impostazioneDao.subCriteriaAgentiSottoReteByUser(userid, 0, max, criteriaA, "id");
		if(ApplicationUtil.gtZero(appCr.getAgenteId())) {
			criteriaA.add(Restrictions.idEq(appCr.getAgenteId()));
		}
		if(ApplicationUtil.gtZero(appCr.getCampagna_id())) {
			criteria.createCriteria("campagne").add(Restrictions.idEq(appCr.getCampagna_id()));
		}
		List<ZonaSI> lis = getZonaSI(criteria);
		return impostazioneDao.getZona(lis, false, false, null);
	}
	public List<SelectItem> getCampagneSIRete(Long userid, CalendarioCriteria appCr) {
		Integer max = userDao.getMaxLivelloRete();
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		DetachedCriteria criteriaC = criteria.createCriteria("zone");
		DetachedCriteria criteriaA = criteriaC.createCriteria("agente");
		impostazioneDao.subCriteriaAgentiSottoReteByUser(userid, 0, max, criteriaA, "id");
		if(ApplicationUtil.gtZero(appCr.getAgenteId())) {
			criteriaA.add(Restrictions.idEq(appCr.getAgenteId()));
		}
		if(ApplicationUtil.gtZero(appCr.getZona_id())) {
			criteriaC.add(Restrictions.idEq(appCr.getZona_id()));
		}
		List<CampagnaSI> lis = getCampagnaSI(criteria);
		return impostazioneDao.getCampagna(lis, true, false, false, null);
	}
}