package it.callite.dao;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Scanner;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import it.callite.action.DownloadBean;
import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.constants.ConstantsMap;
import it.callite.criteria.FileCriteria;
import it.callite.criteria.LoggerCriteria;
import it.callite.dao.generic.GenericDao;
import it.callite.log.ErrorLog;
import it.callite.utility.Allegato;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.LoggerUtils;
import it.callite.utility.MyFile;
import it.callite.utility.Ricerca;
@SuppressWarnings("serial")
@Repository("captureDao")
public class CaptureDao extends GenericDao {
	@Autowired
	private ImpostazioneDao impostazioneDao;


	public void eliminaFile(MyFile att) {
		String home = impostazioneDao.getHome();
		File perc = new File(home + ConstantsGlobal.path_capture + "/" + att.getName());
		ApplicationUtil.deleteAll(perc);
	}
	public void eliminaFile(List<MyFile> atts) {
		if(ApplicationUtil.notEmpty(atts)) {
			String home = impostazioneDao.getHome();
			Iterator<MyFile> aIt = atts.iterator();
			while (aIt.hasNext()) {
				MyFile att = (MyFile) aIt.next();
				File perc = new File(home + ConstantsGlobal.path_capture + "/" + att.getName());
				ApplicationUtil.deleteAll(perc);
			}
		}
	}

	public void scaricaFile(List<MyFile> fils, DownloadBean downloadBean) {
		List<Allegato> files = new ArrayList<Allegato>();
		if (ApplicationUtil.notEmpty(fils)) {
			String home = impostazioneDao.getHome();
			Iterator<MyFile> itS = fils.iterator();
			while (itS.hasNext()) {
				MyFile file = (MyFile) itS.next();
				String root = home + ConstantsGlobal.path_capture;
				String base = home + ConstantsGlobal.path_capture;
				ApplicationUtil.generateMyFileList(files, new File(base + "/" + file.getName()), root);
			}
		}
		try {
			if (ApplicationUtil.notEmpty(files)) {
				String file_header = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.CAPTURE.stampa()) + "." + ConstantsMap.zip;
				OutputStream out = downloadBean.outputStream();
				downloadBean.filename(file_header, files);
				String g = ConstantsGlobal.secretZ;
				ApplicationUtil.zip(null,files, out, g);
				downloadBean.close();
			}
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}

	public void scaricaFile(MyFile att, DownloadBean downloadBean) {
		try {
			String home = impostazioneDao.getHome();
			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(att.getName());
			String nome = home + ConstantsGlobal.path_capture +"/"+ att.getName();
			InputStream istream = new FileInputStream(nome);
			ApplicationUtil.copy(istream, out);
			istream.close();
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}

	}

	public List<MyFile> listaFile(FileCriteria mm) {
		if(mm != null)  mm = (FileCriteria) mm.loadCriteria();
		List<MyFile> ff = new ArrayList<MyFile>();
		if(mm != null && mm.getCerca()) {
			String home = impostazioneDao.getHome();
			File perc = new File(home + ConstantsGlobal.path_capture);
			ApplicationUtil.creaFile(true, perc);
			File [] files = perc.listFiles();
			if(ApplicationUtil.notEmpty(files)) {
				for(File file : files) {
					if(mm.isRicerca(file.getName(), file.length(), file.lastModified())) {
						MyFile f = new MyFile(file);
						f.setParent(ApplicationUtil.removeStart(ConstantsGlobal.path_capture, ConstantsGlobal.path_capture));
						f.setName(file.getName());
						f.setModified(file.lastModified());
						ff.add(f);
					}
				}
			}
		}
		return ff;
	}

	public List<LoggerUtils> listaLog(LoggerCriteria mm){
		if(mm != null)  mm = (LoggerCriteria) mm.loadCriteria();
		List<LoggerUtils> log = new LinkedList<LoggerUtils>();
		if(mm != null && mm.getCerca()) {
			boolean builder = true;
			if(ApplicationUtil.isNotBlank(mm.getFullText())) {
				List<File> fileList = new ArrayList<File>();
				ApplicationUtil.generateFileList(fileList, new File(ConstantsGlobal.path_log));
				if(ApplicationUtil.notEmpty(fileList)) {
					List<String> rs = ApplicationUtil.token(ApplicationUtil.cleanText(mm.getFullText()), ConstantsGlobal.colonSeparator);
					if(ApplicationUtil.notEmpty(rs)) {
						for(File dd : fileList) {
							if(mm.isRicerca(dd.getName(), dd.length(), dd.lastModified())) {
								List<String> lines = new ArrayList<String>();
								String g = ApplicationUtil.getExtensionLower(dd.getAbsolutePath()).equalsIgnoreCase(ConstantsMap.gz) ? "zgrep" : "grep";
								String file = ApplicationUtil.normalizeNoEndSeparator(dd.getAbsolutePath());
								for(String ricerca : rs) {
									String[] commandS = null;
									Ricerca m = ApplicationUtil.match(ricerca);
									if(m.getValore() == 1) {
										if(m.isInverti()) {
											if(m.getRicerca() != null) {
												commandS = new String[] {ConstantsGlobal.path_shell, "-c", g + " " + "-i -v -s -B "+mm.getBefore()+" -A "+mm.getAfter()+"" + " '^" + m.getRicerca() + "$' " + file};
											}
										} else {
											if(m.getRicerca() != null) {
												commandS = new String[] {ConstantsGlobal.path_shell, "-c", g + " " + "-i -s -B "+mm.getBefore()+" -A "+mm.getAfter()+"" + " '^" + m.getRicerca() + "$' " + file};
											}
										}
									}
									if(m.getValore() == 2) {
										if(m.isInverti()) {
											if(m.getRicerca() != null) {
												commandS = new String[] {ConstantsGlobal.path_shell, "-c", g + " " + "-i -v -s -B "+mm.getBefore()+" -A "+mm.getAfter()+"" + " '^" + m.getRicerca() + "' " + file};
											}
										} else {
											if(m.getRicerca() != null) {
												commandS = new String[] {ConstantsGlobal.path_shell, "-c", g + " " + "-i -s -B "+mm.getBefore()+" -A "+mm.getAfter()+"" + " '^" + m.getRicerca() + "' " + file};
											}
										}
									}
									if(m.getValore() == 3) {
										if(m.isInverti()) {
											if(m.getRicerca() != null) {
												commandS = new String[] {ConstantsGlobal.path_shell, "-c", g + " " + "-i -v -s -B "+mm.getBefore()+" -A "+mm.getAfter()+"" + " '" + m.getRicerca() + "$' " + file};
											}
										} else {
											if(m.getRicerca() != null) {
												commandS = new String[] {ConstantsGlobal.path_shell, "-c", g + " " + "-i -s -B "+mm.getBefore()+" -A "+mm.getAfter()+"" + " '" + m.getRicerca() + "$' " + file};
											}
										}
									}
									if(m.getValore() == 4) {
										if(m.isInverti()) {
											if(m.getRicerca() != null) {
												commandS = new String[] {ConstantsGlobal.path_shell, "-c", g + " " + "-i -v -s -B "+mm.getBefore()+" -A "+mm.getAfter()+"" + " '" + m.getRicerca() + "' " + file};
											}
										} else {
											if(m.getRicerca() != null) {
												commandS = new String[] {ConstantsGlobal.path_shell, "-c", g + " " + "-i -s -B "+mm.getBefore()+" -A "+mm.getAfter()+"" + " '" + m.getRicerca() + "' " + file};
											}
										}
									}
									if(ApplicationUtil.notEmpty(commandS)) {
										try {
											Process processo = null;
											if(builder) {
												ProcessBuilder procBuilder = new ProcessBuilder(commandS);
												procBuilder.redirectErrorStream(true);
												processo = procBuilder.start();
											} else {
												Runtime rt = Runtime.getRuntime();
												processo = rt.exec(commandS);
											}
											if(processo != null) {
												Scanner stdInput = new Scanner(new InputStreamReader(processo.getInputStream()));
												Scanner stdError = new Scanner(new InputStreamReader(processo.getErrorStream()));
												OutputStream toProc = processo.getOutputStream();
												while (stdInput.hasNextLine()) {
													lines.add(stdInput.nextLine());
												}
												while (stdError.hasNextLine()) {
													lines.add(stdError.nextLine());
												}
												stdInput.close();
												stdError.close();
												toProc.close();
											}
										} catch (IOException e) {
											lines.add(e.getMessage());
										}
									}
								}
								if(ApplicationUtil.notEmpty(lines)) {
									LoggerUtils l = new LoggerUtils();
									l.setLines(lines);
									l.setNome(file);
									log.add(l);
								}
							}
						}
					}
				}
			}
		}
		return log;
	}

	public void scaricaLog(String att, DownloadBean downloadBean) {
		try {
			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(ApplicationUtil.getNameN(att));
			InputStream istream = new FileInputStream(ApplicationUtil.normalizeNoEndSeparator(att));
			ApplicationUtil.copy(istream, out);
			istream.close();
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void eliminaLog(String att) {
		File perc = new File(ApplicationUtil.normalizeNoEndSeparator(att));
		ApplicationUtil.deleteAll(perc);
	}

}