package it.callite.dao;
import java.util.Iterator;
import java.util.List;

import javax.faces.model.SelectItem;

import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Projections;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import it.callite.constants.ConstantsGlobal;
import it.callite.criteria.CategoriaCriteria;
import it.callite.criteria.base.ExtendedCriteria;
import it.callite.dao.generic.GenericDao;
import it.callite.model.Categoria;
import it.callite.model.Prodotto;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.item.GenericSI;
@SuppressWarnings("serial")
@Repository("categoriaDao")
public class CategoriaDao  extends GenericDao {
	
	@Autowired
	private ImpostazioneDao impostazioneDao;
	
	public Categoria salva(Categoria categoria) {
		if(categoria != null) {
			categoria = (Categoria) salvaAggiorna(categoria);
		}
		return categoria;
	}
	public Categoria get(Long id) {
		return (Categoria) get(Categoria.class, id);
	}
	public void elimina(Categoria categoria) {
		cancellaAggiorna(categoria);
	}
	@SuppressWarnings("unchecked")
	public List<Prodotto> getProdottiByIdCategoria(Long idCategoria) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Prodotto.class);
		criteria.add(Restrictions.eq("categoria.id",idCategoria));
		criteria.addOrder(Order.asc("nome"));
		return getList(criteria);
	}
	public ExtendedCriteria getCategoriaCriteria(CategoriaCriteria prd) {
		if(prd != null)  prd = (CategoriaCriteria) prd.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Categoria.class);
		if (ApplicationUtil.isNotBlank(prd.getNome())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, prd.getNome(),criteria, "nome");
		}
		if (ApplicationUtil.isNotBlank(prd.getDescrizione())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true,  prd.getDescrizione(), criteria, "descrizione");
		}
		prd.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, prd);
	}
	@SuppressWarnings("unchecked")
	public List<Categoria> trova(CategoriaCriteria prd, int start, int size) {
		ExtendedCriteria criteria = getCategoriaCriteria(prd);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int trovaSize(CategoriaCriteria prd) {
		ExtendedCriteria criteria = getCategoriaCriteria(prd);
		return  getListSize(criteria);
	}
	public List<SelectItem> getCategorieSI() {
		DetachedCriteria criteria = DetachedCriteria.forClass(Categoria.class);
		List<GenericSI> lis = getGenericSI(criteria, "id", "nome");
		return impostazioneDao.getGeneric(lis, true);
	}
	public List<SelectItem> getAllCategorieSI() {
		DetachedCriteria criteria = DetachedCriteria.forClass(Categoria.class);
		List<GenericSI> lis = getGenericSI(criteria, "id", "nome");
		return impostazioneDao.getGeneric(lis, false);
	}
	@SuppressWarnings("unchecked")
	public List<Prodotto> getProdottoByCategoriaId(Long categoriaId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Prodotto.class);
		criteria.createCriteria("categoria").add(Restrictions.idEq(categoriaId));
		criteria.addOrder(Order.asc("nome"));
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Prodotto> getProdottoNotInCategoria(Long categoriaId, Integer tutti) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Prodotto.class);
		if(ApplicationUtil.gtZero(tutti)) {
			if(ApplicationUtil.eq(tutti, 1)) {
				criteria.createCriteria("categoria", ApplicationUtil.leftJoin()).add(Restrictions.isNull("id"));
			} else {
				DetachedCriteria criteriaU = DetachedCriteria.forClass(Prodotto.class);
				criteriaU.createCriteria("categoria").add(Restrictions.idEq(categoriaId));
				criteriaU.setProjection(Projections.distinct(Projections.property("id")));
				impostazioneDao.subPropertyNotInS(criteria, criteriaU, "id");
				criteria.createCriteria("categoria", ApplicationUtil.leftJoin()).add(Restrictions.isNotNull("id"));
			}
		} else {
			DetachedCriteria criteriaU = DetachedCriteria.forClass(Prodotto.class);
			criteriaU.createCriteria("categoria").add(Restrictions.idEq(categoriaId));
			criteriaU.setProjection(Projections.distinct(Projections.property("id")));
			impostazioneDao.subPropertyNotInS(criteria, criteriaU, "id");
		}
		criteria.addOrder(Order.asc("nome"));
		return getList(criteria);
	}
	public void aggiungiProdottoACategoria(Long categoriaId, Long prodottoId) {
		Categoria categoria = get(categoriaId);
		Prodotto prodotto = getProdotto(prodottoId);
		if (categoria != null && prodotto != null) {
			prodotto.setCategoria(categoria);
			salvaAggiorna(prodotto);
		}
	}
	public void rimuoviProdottoDaCategoria(Long categoriaId, Long prodottoId) {
		Categoria categoria = get(categoriaId);
		Prodotto prodotto = getProdotto(prodottoId);
		if (categoria != null && prodotto != null) {
			prodotto.setCategoria(null);
			salvaAggiorna(prodotto);
		}
	}
	public void aggiungiProdottoTuttiACategoria(Long categoriaId, Integer tutti) {
		Categoria categoria = get(categoriaId);
		List<Prodotto> prodotti = getProdottoNotInCategoria(categoriaId, tutti);
		if (categoria != null && ApplicationUtil.notEmpty(prodotti)) {
			Iterator<Prodotto> it = prodotti.iterator();
			while (it.hasNext()) {
				Prodotto prodotto = (Prodotto) it.next();
				prodotto.setCategoria(categoria);
				salvaAggiorna(prodotto);
			}
		}
	}
	public void rimuoviProdottoTuttiDaCategoria(Long categoriaId) {
		Categoria categoria = get(categoriaId);
		List<Prodotto> prodotti = getProdottoByCategoriaId(categoriaId);
		if (categoria != null && ApplicationUtil.notEmpty(prodotti)) {
			Iterator<Prodotto> it = prodotti.iterator();
			while (it.hasNext()) {
				Prodotto prodotto = (Prodotto) it.next();
				prodotto.setCategoria(null);
				salvaAggiorna(prodotto);
			}
		}
	}
	public Prodotto getProdotto(Long id){
		return (Prodotto) get(Prodotto.class, id);
	}
	
}