package it.callite.dao;

import java.io.OutputStream;
import java.util.Iterator;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Restrictions;
import org.springframework.stereotype.Repository;

import it.callite.action.DownloadBean;
import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsMap;
import it.callite.criteria.CommandCriteria;
import it.callite.dao.generic.GenericDao;
import it.callite.log.ErrorLog;
import it.callite.model.Command;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.FileUpload;
import it.callite.utility.MyFile;
@SuppressWarnings("serial")
@Repository("commandDao")
public class CommandDao extends GenericDao {
	

	public void lanciaCommand(String nome) {
		try {
			Command sk = getValue(nome);
			lancia(sk);
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}


	public Command get(Long id) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Command.class);
		criteria.add(Restrictions.eq("id", id));
		return (Command) getUniqueResult(criteria);
	}

	public Command getValue(String value) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Command.class);
		criteria.add(Restrictions.eq("nome", value));
		Command sk = (Command) getUniqueResult(criteria);
		if(sk == null) {
			return new Command();
		} else {
			return sk;
		}
	}

	public Command getByName(String value) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Command.class);
		criteria.add(Restrictions.eq("nome", value));
		return (Command) getUniqueResult(criteria);
	}

	public Command salva(Command oggetto) {
		return (Command) salvaAggiorna(oggetto);
	}

	public void elimina(Command oggetto) {
		cancellaAggiorna(oggetto);
	}

	private DetachedCriteria getCommandByCriteria(CommandCriteria crit) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Command.class);
		if(StringUtils.isNotBlank(crit.getNome())) {
			criteria.add(Restrictions.in("nome", crit.getNome()));
		}
		if(crit.getAvviato() != null) {
			criteria.add(Restrictions.eq("started", crit.getAvviato()));
		} 		
		return criteria;
	}

	@SuppressWarnings("unchecked")
	public List<Command> trova(CommandCriteria crit, int start, int size) {
		DetachedCriteria criteria = getCommandByCriteria(crit);
		criteria.addOrder(Order.asc("nome"));
		return getList(criteria, start, size);
	}

	public int trovaSize(CommandCriteria crit) {
		DetachedCriteria criteria = getCommandByCriteria(crit);
		return getListSize(criteria);
	}


	@SuppressWarnings("unchecked")
	public List<Command> trovaTutti() {
		DetachedCriteria criteria = DetachedCriteria.forClass(Command.class);
		return getList(criteria);
	}

	public void lancia(Command sk) throws Exception {
		ApplicationUtil.lanciaCommand(sk.getNome(), sk.getFiles(), sk.getWrites());
	}

	
	public void salvaAllegato(FileUpload fileUpload, Command command) {
		try {
			List<MyFile> fls = fileUpload.getFiles();
			if(ApplicationUtil.notEmpty(fls)) {
				Iterator<MyFile> flsIt = fls.iterator();
				while (flsIt.hasNext()) {
					MyFile ff = (MyFile) flsIt.next();
					byte [] ffB = ff.getData();
					command.setFiles(new String (ffB, ConstantsGlobal.CHARSET));
					//salvaAggiorna(command);
				}
			}
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	
	public void getAllegato(Command command, DownloadBean downloadBean) {
		try {
			String file = command.getNome() + "." + ConstantsMap.sh;
				OutputStream out = downloadBean.outputStream();
				downloadBean.filename(file);
				out.write(command.getFiles().getBytes(ConstantsGlobal.CHARSET));
				downloadBean.close();
		} catch(Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	
}