package it.callite.dao;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.faces.model.SelectItem;

import org.hibernate.FetchMode;
import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Projections;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import it.callite.action.DownloadBean;
import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.constants.ConstantsMap;
import it.callite.constants.config.DefaultsGeneric;
import it.callite.constants.config.DefaultsSleep;
import it.callite.criteria.BustaCriteria;
import it.callite.criteria.ComunicazioneCriteria;
import it.callite.criteria.DocumentoComunicazioneCriteria;
import it.callite.criteria.EmailCriteria;
import it.callite.criteria.EmailTemplateCriteria;
import it.callite.criteria.InvioCriteria;
import it.callite.criteria.NotificaCriteria;
import it.callite.criteria.PDACriteria;
import it.callite.criteria.RicezioneCriteria;
import it.callite.criteria.RmailCriteria;
import it.callite.criteria.RmailTemplateCriteria;
import it.callite.criteria.SmtpCriteria;
import it.callite.criteria.SondaggioCriteria;
import it.callite.criteria.TelecomCriteria;
import it.callite.criteria.TelecomTemplateCriteria;
import it.callite.criteria.TemplateCriteria;
import it.callite.criteria.WebhookCriteria;
import it.callite.criteria.WebhookTemplateCriteria;
import it.callite.criteria.base.ExtendedCriteria;
import it.callite.dao.generic.GenericDao;
import it.callite.log.ErrorLog;
import it.callite.model.Busta;
import it.callite.model.BustaStore;
import it.callite.model.Comunicazione;
import it.callite.model.Contatto;
import it.callite.model.DatoAggiuntivoAnagrafica;
import it.callite.model.Documento;
import it.callite.model.Email;
import it.callite.model.EmailTemplate;
import it.callite.model.EsitoArchiviazione;
import it.callite.model.Indirizzo;
import it.callite.model.Invio;
import it.callite.model.Notifica;
import it.callite.model.Ricezione;
import it.callite.model.Rmail;
import it.callite.model.RmailTemplate;
import it.callite.model.Schedulatore;
import it.callite.model.Smtp;
import it.callite.model.Sondaggio;
import it.callite.model.Telecom;
import it.callite.model.TelecomTemplate;
import it.callite.model.Template;
import it.callite.model.TemplateProdotto;
import it.callite.model.TemplateStore;
import it.callite.model.Utente;
import it.callite.model.Webhook;
import it.callite.model.WebhookTemplate;
import it.callite.model.pk.TemplateProdottoPK;
import it.callite.service.ComunicazioneService;
import it.callite.service.SchedulerService;
import it.callite.service.SmtpService;
import it.callite.service.config.ConfigurazioneGenericService;
import it.callite.service.config.ConfigurazioneSleepService;
import it.callite.utility.Allegato;
import it.callite.utility.AllegatoUtils;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.DatoAggiuntivo;
import it.callite.utility.DocumentiCheckbox;
import it.callite.utility.FileUpload;
import it.callite.utility.MailSupport;
import it.callite.utility.MyFile;
import it.callite.utility.Parameter;
import it.callite.utility.UtilOffice;
import it.callite.utility.item.GenericSI;
import it.callite.utility.thread.ReInvioComunicazioneThread;
import it.callite.utility.thread.ReInvioNotificaThread;
import it.callite.utility.thread.ReInvioPostaThread;
import it.callite.utility.thread.SincronizzaThread;
import jakarta.mail.Flags;
import jodd.mail.EmailFilter;
import jodd.mail.ImapServer;
import jodd.mail.MailServer;
import jodd.mail.Pop3Server;
import jodd.mail.ReceiveMailSession;
import jodd.mail.ReceivedEmail;
@SuppressWarnings("serial")
@Repository("comunicazioneDao")
public class ComunicazioneDao extends GenericDao {
	
	@Autowired
	private ConfigurazioneGenericService configurazioneGenericService;
	@Autowired
	private ComunicazioneService comunicazioneService;
	@Autowired
	private SmtpService smtpService;
	@Autowired
	private ProdottoDao prodottoDao;
	@Autowired
	private UserDao userDao;
	@Autowired
	private CampagnaDao campagnaDao;
	@Autowired
	private AnagraficaDao anagraficaDao;
	@Autowired
	private ImpostazioneDao impostazioneDao;
	@Autowired
	private DownloadDao downloadDao;
	
	@Autowired
	private SchedulerService schedulerService;
	
	@Autowired
	private ConfigurazioneSleepService configurazioneSleepService;
	
	public Boolean sleepEmailSemaforo() {
		return ApplicationUtil.gt(sleepEmailTime(), 0);
	}
	public Boolean sleepRmailSemaforo() {
		return ApplicationUtil.gt(sleepRmailTime(), 0);
	}
	public Boolean sleepTelecomSemaforo() {
		return ApplicationUtil.gt(sleepTelecomTime(), 0);
	}
	
	public Integer sleepEmailTime() {
		return configurazioneSleepService != null ? Integer.valueOf(configurazioneSleepService.getValore(DefaultsSleep.KEY_SLEEP_EMAIL_TIME)) : ConstantsGlobal.sleep;
	}
	public Integer sleepRmailTime() {
		return configurazioneSleepService != null ? Integer.valueOf(configurazioneSleepService.getValore(DefaultsSleep.KEY_SLEEP_RMAIL_TIME)) : ConstantsGlobal.sleep;
	}
	public Integer sleepTelecomTime() {
		return configurazioneSleepService != null ? Integer.valueOf(configurazioneSleepService.getValore(DefaultsSleep.KEY_SLEEP_TELECOM_TIME)) : ConstantsGlobal.sleep;
	}
	
	public Email getEmail(Long id){
		return (Email) get(Email.class, id);
	}
	public void eliminaEmail(Email id) {
		cancellaAggiorna(id);
	}
	public void eliminaComunicazione(Comunicazione id) {
		cancellaAggiorna(id);
	}
	public void eliminaNotifica(Notifica id) {
		cancellaAggiorna(id);
	}
	@SuppressWarnings("unchecked")
	public List<EsitoArchiviazione> getEsitoArchiviazioneByEmailId(Long emailId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoArchiviazione.class);
		criteria.createCriteria("emails").add(Restrictions.idEq(emailId));
		criteria.setFetchMode("esito_collegato", FetchMode.JOIN);
		ApplicationUtil.orderEsito(criteria);
		return getList(criteria);
	}

	
	@SuppressWarnings("unchecked")
	public List<EsitoArchiviazione> getEsitoArchiviazioneByRmailId(Long rmailId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoArchiviazione.class);
		criteria.createCriteria("rmails").add(Restrictions.idEq(rmailId));
		criteria.setFetchMode("esito_collegato", FetchMode.JOIN);
		ApplicationUtil.orderEsito(criteria);
		return getList(criteria);
	}
	
	@SuppressWarnings("unchecked")
	public List<EsitoArchiviazione> getEsitoArchiviazioneByTelecomId(Long telecomId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoArchiviazione.class);
		criteria.createCriteria("telecoms").add(Restrictions.idEq(telecomId));
		criteria.setFetchMode("esito_collegato", FetchMode.JOIN);
		ApplicationUtil.orderEsito(criteria);
		return getList(criteria);
	}
	private ExtendedCriteria trovaCriteriaEmail(EmailCriteria ema) {
		if(ema != null)  ema = (EmailCriteria) ema.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Email.class);
		DetachedCriteria criteriaCo = criteria.createCriteria("contatto");
		if (ApplicationUtil.isNotBlank(ema.getTesto())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true,  ema.getTesto(), criteria, "oggetto", "testo");
		}
		if (ApplicationUtil.isNotBlank(ema.getOperatore())) {
			DetachedCriteria cOp = criteria.createCriteria("utente");
			ApplicationUtil.operatore(cOp, ema.getOperatore());
		}
		if (ApplicationUtil.isNotBlank(ema.getRicerca_anagrafica())) {
			DetachedCriteria criteriaA = criteriaCo.createCriteria("anagrafica");
			ApplicationUtil.anagrafica(ema.getRicerca_anagraficaV(), criteriaA, ema.getRicerca_anagrafica());
		}
		if (ApplicationUtil.gtZero(ema.getCampagna())) {
			criteriaCo.add(Restrictions.eq("campagna.id", ema.getCampagna()));
		}
		if (ApplicationUtil.gtZero(ema.getCodice())) {
			criteria.add(Restrictions.eq("codice", ema.getCodice()));
		}
		ema.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, ema);
	}
	@SuppressWarnings("unchecked")
	public List<Email> trovaEmail(EmailCriteria ema,int start, int size) {
		ExtendedCriteria criteria = trovaCriteriaEmail(ema);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int trovaEmailSize(EmailCriteria ema) {
		ExtendedCriteria criteria = trovaCriteriaEmail(ema);
		return getListSize(criteria);
	}
	private ExtendedCriteria trovaCriteriaNotifica(NotificaCriteria ema) {
		if(ema != null)  ema = (NotificaCriteria) ema.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Notifica.class);
		DetachedCriteria criteriaCo = criteria.createCriteria("contatto");
		if (ApplicationUtil.isNotBlank(ema.getTesto())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true,  ema.getTesto(), criteria, "testo", "oggetto");
		}
		if (ApplicationUtil.isNotBlank(ema.getOperatore())) {
			DetachedCriteria cOp = criteria.createCriteria("utente");
			ApplicationUtil.operatore(cOp, ema.getOperatore());
		}
		if (ApplicationUtil.isNotBlank(ema.getRicerca_anagrafica())) {
			DetachedCriteria criteriaA = criteriaCo.createCriteria("anagrafica");
			ApplicationUtil.anagrafica(ema.getRicerca_anagraficaV(), criteriaA, ema.getRicerca_anagrafica());
		}
		if (ApplicationUtil.gtZero(ema.getCampagna())) {
			criteriaCo.add(Restrictions.eq("campagna.id", ema.getCampagna()));
		}
		if (ApplicationUtil.gtZero(ema.getCodice())) {
			criteria.add(Restrictions.eq("codice", ema.getCodice()));
		}
		ema.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, ema);
	}
	@SuppressWarnings("unchecked")
	public List<Notifica> trovaNotifica(NotificaCriteria ema,int start, int size) {
		ExtendedCriteria criteria = trovaCriteriaNotifica(ema);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int trovaNotificaSize(NotificaCriteria ema) {
		ExtendedCriteria criteria = trovaCriteriaNotifica(ema);
		return getListSize(criteria);
	}
	public ExtendedCriteria trovaDocumentoCriteria(DocumentoComunicazioneCriteria ema) {
		if(ema != null)  ema = (DocumentoComunicazioneCriteria) ema.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Documento.class);
		if(ApplicationUtil.isNotBlank(ema.getNomeOggetto())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true,  ema.getNomeOggetto(), criteria, "nome");
		}
		ema.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, ema);
	}
	@SuppressWarnings("unchecked")
	public List<Documento> trovaDocumento(DocumentoComunicazioneCriteria ema, int start, int size) {
		ExtendedCriteria criteria = trovaDocumentoCriteria(ema);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int trovaDocumentoSize(DocumentoComunicazioneCriteria ema) {
		ExtendedCriteria criteria = trovaDocumentoCriteria(ema);
		return getListSize(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Documento> trovaDocumentiNonPresenti(DocumentoComunicazioneCriteria ema, List<Documento> documenti, int start, int size) {
		ExtendedCriteria criteria = trovaDocumentoComunicazioneCriteria(ema);
		List<Long> ids = ApplicationUtil.prendiIdDocumento(documenti);
		ApplicationUtil.removeCampi(criteria.getCriteria(), ids);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public void eliminaDocumento(Documento documento) {
		cancellaAggiorna(documento);
	}
	public void eliminaTemplate(Template documento) {
		cancellaAggiorna(documento);
	}
	public void eliminaTemplate(Busta busta) {
		cancellaAggiorna(busta);
	}
	public void eliminaBusta(Busta busta) {
		cancellaAggiorna(busta);
	}
	public void eliminaSmtp(Smtp smtp) {
		cancellaAggiorna(smtp);
	}
	public void eliminaInvio(Invio ti) {
		cancellaAggiorna(ti);
	}
	public int trovaDocumentiNonPresentiSize(DocumentoComunicazioneCriteria ema, List<Documento> documenti) {
		ExtendedCriteria criteria = trovaDocumentoComunicazioneCriteria(ema);
		List<Long> ids = ApplicationUtil.prendiIdDocumento(documenti);
		ApplicationUtil.removeCampi(criteria.getCriteria(), ids);
		return getListSize(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<EmailTemplate> trovaEmailTemplate(EmailTemplateCriteria ema, int start, int size){
		ExtendedCriteria criteria = getCriteriaEmail(ema);
		return getList(criteria, EmailTemplate.class, "documenti",  new Order [] {Order.asc("id")}, start, size);
	}
	public int trovaEmailTemplateSize(EmailTemplateCriteria ema) {
		ExtendedCriteria criteria = getCriteriaEmail(ema);
		return getListSize(criteria);
	}
	
	public int trovaTemplateDocumentoSize(PDACriteria ema) {
		ExtendedCriteria criteria = getCriteria(ema);
		return getListSize(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<TemplateProdotto> trovaTemplateDocumento(PDACriteria ema, int start, int size){
		ExtendedCriteria criteria = getCriteria(ema);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	private ExtendedCriteria getCriteria(PDACriteria ema) {
		if(ema != null)  ema = (PDACriteria) ema.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(TemplateProdotto.class);
		if(ApplicationUtil.gtZero(ema.getProdotto()))  {
			criteria.add(Restrictions.eq("id.prodotto.id", ema.getProdotto()));
		}
		ema.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, ema);
	}
	private ExtendedCriteria getCriteriaEmail(EmailTemplateCriteria ema) {
		if(ema != null)  ema = (EmailTemplateCriteria) ema.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(EmailTemplate.class);
		if (ApplicationUtil.isNotBlank(ema.getNomeOggetto())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true,  ema.getNomeOggetto(), criteria, "nome", "testo");
		}
		ema.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, ema); 
	}
	
	public EmailTemplate getEmailTemplateById(Long emailTemplateId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(EmailTemplate.class);
		criteria.add(Restrictions.idEq(emailTemplateId));
		criteria.setFetchMode("documenti", FetchMode.JOIN);
		return (EmailTemplate) getUniqueResult(criteria);
	}
	public EmailTemplate getEmailTemplate(Long id) {
		return (EmailTemplate) get(EmailTemplate.class, id);
	}
	
	public Documento getDocumentoById(Long documentoId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Documento.class);
		criteria.add(Restrictions.idEq(documentoId));
		return (Documento) getUniqueResult(criteria);
	}
	public void eliminaTemplate(TemplateProdotto template){
		cancellaAggiorna(template);
	}
	private ExtendedCriteria trovaDocumentoComunicazioneCriteria(DocumentoComunicazioneCriteria ema) {
		if(ema != null)  ema = (DocumentoComunicazioneCriteria) ema.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Documento.class);
		if (ApplicationUtil.isNotBlank(ema.getNomeOggetto())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true,  ema.getNomeOggetto(), criteria, "nome");
		}
		ema.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, ema); 
	}
	public Comunicazione salvaComunicazione(Comunicazione email) {
		return (Comunicazione) salvaAggiorna(email);
	}
	public Notifica salvaNotifica(Notifica email) {
		return (Notifica) salvaAggiorna(email);
	}
	public Email salvaEmail(Email email) {
		return (Email) salvaAggiorna(email);
	}
	
	
	public Email getEmailById(Long emailId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Email.class);
		criteria.add(Restrictions.idEq(emailId));
		return (Email) getUniqueResult(criteria);
	}

	
	public List<SelectItem> getTemplateEmailSI() {
		DetachedCriteria criteria = DetachedCriteria.forClass(EmailTemplate.class);
		List<GenericSI> lis = getGenericSI(criteria, "id", "nome");
		return impostazioneDao.getGeneric(lis, true);
	}
	public List<SelectItem> getBuste() {
		DetachedCriteria criteria = DetachedCriteria.forClass(Busta.class);
		List<GenericSI> lis = getGenericSI(criteria, "id", "nome");
		return impostazioneDao.getGeneric(lis, false);
	}
	public List<SelectItem> getSmtp() {
		DetachedCriteria criteria = DetachedCriteria.forClass(Smtp.class);
		List<GenericSI> lis = getGenericSI(criteria, "id", "smtp_mittente");
		return impostazioneDao.getGeneric(lis, false);
	}
	public List<SelectItem> getInvio() {
		DetachedCriteria criteria = DetachedCriteria.forClass(Invio.class);
		List<GenericSI> lis = getGenericSI(criteria, "id", "nome");
		return impostazioneDao.getGeneric(lis, false);
	}
	
	
	public List<SelectItem> getTemplateDocumentoSI() {
		DetachedCriteria criteria = DetachedCriteria.forClass(Template.class);
		List<GenericSI> lis = getGenericSI(criteria, "id", "nome");
		return impostazioneDao.getGeneric(lis, false);
	}
	public List<SelectItem> getTemplateDocumentoSI(Long prodottoID) {
		if(ApplicationUtil.gtZero(prodottoID))  {
			DetachedCriteria criteriaU = DetachedCriteria.forClass(Template.class);
			criteriaU.createCriteria("templates").add(Restrictions.eq("id.prodotto.id",prodottoID));
			criteriaU.setProjection(Projections.distinct(Projections.property("id")));
			DetachedCriteria criteria = DetachedCriteria.forClass(Template.class);
			impostazioneDao.subPropertyNotInS(criteria, criteriaU, "id");
			List<GenericSI> lis = getGenericSI(criteria, "id", "nome");
			return impostazioneDao.getGeneric(lis, false);
		}
		return new ArrayList<SelectItem>();
	}
	public Template getTemplateDocumentoById(Long templateId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Template.class);
		criteria.add(Restrictions.idEq(templateId));
		return (Template) getUniqueResult(criteria);
	}
	public TemplateProdotto getTemplateProdottoById(TemplateProdottoPK tmp) {
		DetachedCriteria criteria = DetachedCriteria.forClass(TemplateProdotto.class);
		criteria.add(Restrictions.eq("id.prodotto.id",tmp.getProdotto().getId()));
		criteria.add(Restrictions.eq("id.documento.id",tmp.getDocumento().getId()));
		return (TemplateProdotto) getUniqueResult(criteria);
	}

	@SuppressWarnings("unchecked")
	public List<EmailTemplate> getAllEmailTemplate(Long campagnaId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(EmailTemplate.class);
		criteria.add(Restrictions.eq("attivo", Boolean.TRUE));
		if(ApplicationUtil.gtZero(campagnaId)) {
			criteria.createCriteria("campagne").add(Restrictions.idEq(campagnaId));
		}
		criteria.setFetchMode("documenti", FetchMode.JOIN);
		criteria.addOrder(Order.asc("nome"));
		return getList(criteria);
	}


	@SuppressWarnings("unchecked")
	public List<Email> getEmailByEmailTemplate(EmailTemplate emailTemplate) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Email.class);
		criteria.add(Restrictions.eq("emailTemplate.id", emailTemplate));
		return getList(criteria);
	}
	
	@SuppressWarnings("unchecked")
	public List<DocumentiCheckbox> getCheckboxDocumentiEmail(Email email) {
		List<Documento> documenti = getList(DetachedCriteria.forClass(Documento.class));
		List<DocumentiCheckbox> documentiCheckbox = new ArrayList<DocumentiCheckbox>();
		if(email.getEmailTemplate() != null && email.getEmailTemplate().getDocumenti() != null) {
			if(ApplicationUtil.empty(email.getAllegati())) { 
				Iterator<Documento> objIter = documenti.iterator();
				while (objIter.hasNext()) {
					Documento documento = objIter.next();
					boolean selected = email.getEmailTemplate().getDocumenti().contains(documento);
					documentiCheckbox.add(new DocumentiCheckbox(documento, selected, selected));
				}
			} else {
				Iterator<Documento> objIter = documenti.iterator();
				while (objIter.hasNext()) {
					Documento documento = objIter.next();
					boolean selected = email.getEmailTemplate().getDocumenti().contains(documento);
					Allegato a = new Allegato(documento.getNomeFileS(), documento.getContent());
					a.setData_modifica(documento.getData_creazione());
					documentiCheckbox.add(new DocumentiCheckbox(documento, selected || email.getAllegati().get(a) != null, selected));
				}
			}
		}
		else {
			if(ApplicationUtil.empty(email.getAllegati())) {
				Iterator<Documento> objIter = documenti.iterator();
				while (objIter.hasNext()) {
					Documento documento = objIter.next();
					documentiCheckbox.add(new DocumentiCheckbox(documento));
				}
			} else {
				Iterator<Documento> objIter = documenti.iterator();
				while (objIter.hasNext()) {
					Documento documento = objIter.next();
					Allegato a = new Allegato(documento.getNomeFileS(), documento.getContent());
					a.setData_modifica(documento.getData_creazione());
					documentiCheckbox.add(new DocumentiCheckbox(documento, email.getAllegati().get(a) != null, Boolean.FALSE));
				}
			}
		}
		return documentiCheckbox;
	}
	
	public Documento salvaDocumento(Documento documento) {
		if (documento != null) {
			documento = (Documento) salvaAggiorna(documento);
		}
		return documento;
	}
	public void inserisciFileInDocumento(FileUpload fileUpload, Documento documento) {
		List<MyFile> fls = fileUpload.getFiles();
		if(ApplicationUtil.notEmpty(fls)) {
			try { 
				Iterator<MyFile> flsIt = fls.iterator();
				while (flsIt.hasNext()) {
					MyFile ff = (MyFile) flsIt.next();
					documento.setNomeFile(ApplicationUtil.getNameN(ff.getName()));
					documento.setContent(ff.getData());
				}
			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		}
	}
	public EmailTemplate salvaEmailTemplate(EmailTemplate emailTemplate) {
		if (emailTemplate != null) {
			emailTemplate = (EmailTemplate) salvaAggiorna(emailTemplate);
		}
		return emailTemplate;
	}
	
	public TemplateProdotto salvaTemplate(TemplateProdotto template) {
		return (TemplateProdotto) salvaAggiorna(template);
	}
	public TemplateProdotto salvaTemplate(TemplateProdotto template, Long documento, Long prodotto) {
		if(ApplicationUtil.gtZero(documento) && ApplicationUtil.gtZero(prodotto) ) {
			TemplateProdottoPK id = new TemplateProdottoPK();
			id.setDocumento(getTemplateDocumentoById(documento));
			id.setProdotto(prodottoDao.get(prodotto));
			template.setId(id);
		}
		return (TemplateProdotto) salvaAggiorna(template);
	}
	public void addDocumentoToEmailTemplate(EmailTemplate emailTemplate, Documento documento) {
		if (!emailTemplate.getDocumenti().contains(documento)) {
			emailTemplate.getDocumenti().add(documento);
		}
	}
	public void addDocumentiToEmailTemplate(EmailTemplate emailTemplate, List<Documento> documenti) {
		if(ApplicationUtil.notEmpty(documenti)) {
			Iterator<Documento> it = documenti.iterator();
			while (it.hasNext()) {
				Documento documento = (Documento) it.next();
				addDocumentoToEmailTemplate(emailTemplate, documento);
			}
		}
	}
	public void addDocumentiToRmailTemplate(RmailTemplate rmailTemplate, List<Documento> documenti) {
		if(ApplicationUtil.notEmpty(documenti)) {
			Iterator<Documento> it = documenti.iterator();
			while (it.hasNext()) {
				Documento documento = (Documento) it.next();
				addDocumentoToRmailTemplate(rmailTemplate, documento);
			}
		}
	}
	
	
	public void getDocumento(Documento documento, DownloadBean downloadBean) {
		OutputStream out = downloadBean.outputStream();
		downloadBean.filename(documento.getNomeFileS());
		try {
			InputStream bais = new ByteArrayInputStream(documento.getContent());
			ApplicationUtil.copy(bais, out);
			downloadBean.close();
			bais.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	
	public List<Allegato> allegatoEmail(Email email) {
		List<Allegato> allegati = new ArrayList<Allegato>();
		if(email.getEmailTemplate() != null) {
			EmailTemplate emt  = getEmailTemplateById(email.getEmailTemplate().getId());
			List<Documento> documenti = emt.getDocumentiList();
			for(int i = 0; i <  documenti.size(); i++){
				Documento doc = documenti.get(i);
				ApplicationUtil.addFile(allegati, doc.getNomeFileS(), doc.getContent(), doc.getData_creazione().getTime());
			}
			if(ApplicationUtil.notEmpty(email.getEmailTemplate().getAllegati())) {
				List<Allegato> altri_allegati = email.getEmailTemplate().getAllegati();
				for(int i = 0; i < altri_allegati.size(); i++){
					ApplicationUtil.addFile(allegati, altri_allegati.get(i));
				}
			}
			if(ApplicationUtil.notEmpty(email.getAllegati())) {
				List<Allegato> ancora_allegati = new ArrayList<Allegato>(email.getAllegati().keySet());
				for(int i = 0; i < ancora_allegati.size(); i++) {
					ApplicationUtil.addFile(allegati, ancora_allegati.get(i));
				}
			}
		}
		return allegati;
	}

	public Integer inviaEmail(Email email, Utente user, MailSupport ms) {
		email.setUtente(user);
		email.setData_invio(ApplicationUtil.date());
		email.setDestinatario(replace(email.getContatto(), email.getDestinatario()));
		email.setOggetto(replace(email.getContatto(), email.getOggetto()));
		email.setTesto(replace(email.getContatto(), email.getTesto()));
		email.setAllegatiTemplateList(allegatoEmail(email));
		email.impostaUserfield();
		if(!email.inviabile()) {
			email.addMotivo(ConstantsGlobal.NON_INVIATA, ConstantsLanguage.ERRORE_INVIO.stampa());
			email = salvaEmail(email);
			return email.getCodice();
		} 
		if(!email.valido()) {
			email.addMotivo(ConstantsGlobal.NON_INVIATA, ConstantsLanguage.DESTINATARIO_VUOTO.stampa());
			email = salvaEmail(email);
			return email.getCodice();
		} 
		if(!email.getContatto().getAnagrafica().getConsenso_emailT()) {
			email.addMotivo(ConstantsGlobal.NON_INVIATA, ConstantsLanguage.CONSENSO_NEGATO.stampa());
			email = salvaEmail(email);
			return email.getCodice();
		} 
		if(ApplicationUtil.gtOra(email.getDataSchedulata())) {
			email.setData_invio(email.getDataSchedulata());
			email.addMotivo(ConstantsGlobal.SCHEDULATA, null);
			Schedulatore sa = schedulerService.lancia(email.getId(), email.getDataSchedulata(), ConstantsGlobal.tipo_email);
			email.setScheduler(sa);
			email = salvaEmail(email);
			return email.getCodice();
		}
		email.addMotivo(ConstantsGlobal.IN_INVIO, null);
		email = salvaEmail(email);
		if(ms != null) {
			comunicazioneService.email(ms, email);
		} else {
			comunicazioneService.email(email);
		}
		return email.getCodice();
	}

	public Integer reInvia(Email email, MailSupport ms) {
		email.setData_invio(ApplicationUtil.date());
		email.setScheduler(null);
		if(!email.inviabile()) {
			email.addMotivo(ConstantsGlobal.NON_INVIATA, ConstantsLanguage.ERRORE_INVIO.stampa());
			email = salvaEmail(email);
			return email.getCodice();
		} 
		if(!email.valido()) {
			email.addMotivo(ConstantsGlobal.NON_INVIATA, ConstantsLanguage.DESTINATARIO_VUOTO.stampa());
			email = salvaEmail(email);
			return email.getCodice();
		} 
		if(!email.getContatto().getAnagrafica().getConsenso_emailT()) {
			email.addMotivo(ConstantsGlobal.NON_INVIATA, ConstantsLanguage.CONSENSO_NEGATO.stampa());
			email = salvaEmail(email);
			return email.getCodice();
		} 
		email.addMotivo(ConstantsGlobal.IN_INVIO, null);
		email = salvaEmail(email);
		if(ms != null) {
			comunicazioneService.email(ms, email);
		} else {
			comunicazioneService.email(email);
		}
		return email.getCodice();
	}
	
	public Email creaEmailDaTemplate(EmailTemplate emailTemplate) {
		Email email = new Email();
		email.setOggetto(emailTemplate.getOggetto());
		email.setTesto(emailTemplate.getTesto());
		email.setEmailTemplate(emailTemplate);
		return email;
	}
	
	public Email creaEmailDaTemplate(EmailTemplate emailTemplate, Map<String, DatoAggiuntivo> valori) {
		Email email = new Email();
		email.setOggetto(ApplicationUtil.replaceInString(emailTemplate.getOggetto(), valori));
		email.setTesto(ApplicationUtil.replaceInString(emailTemplate.getTesto(), valori));
		email.setEmailTemplate(emailTemplate);
		return email;
	}
	
	
	
	public void inserisciAllegatiCDEmail(Email email, List<DocumentiCheckbox> documentiCheckbox) {
		HashMap<Allegato, String> map = new HashMap<Allegato, String>();
		Iterator<DocumentiCheckbox> objIter = documentiCheckbox.iterator();
		while (objIter.hasNext()) {
			DocumentiCheckbox dc = objIter.next();
			if(ApplicationUtil.isTrue(dc.getSelezionato()) && ApplicationUtil.isFalse(dc.getDisabled())) {
				ApplicationUtil.addFile(map, dc);
			}
		}
		email.setAllegati(map);
	}
	
	public void eliminaEmailTemplate(EmailTemplate emailTemplate) {
		cancellaAggiorna(emailTemplate);
	}
		
	public List<Allegato> allegatoRmail(Rmail rmail) {
		List<Allegato> allegati = new ArrayList<Allegato>();
		if(rmail.getRmailTemplate() != null) {
			RmailTemplate fat  = getRmailTemplateById(rmail.getRmailTemplate().getId());
			List<Documento> documenti = fat.getDocumentiList();
			for(int i = 0; i <  documenti.size(); i++){
				Documento doc = documenti.get(i);
				ApplicationUtil.addFile(allegati, doc.getNomeFileS(), doc.getContent(), doc.getData_creazione().getTime());
			}
			if(ApplicationUtil.notEmpty(rmail.getRmailTemplate().getAllegati())) {
				List<Allegato> altri_allegati = rmail.getRmailTemplate().getAllegati();
				for(int i = 0; i < altri_allegati.size(); i++){
					ApplicationUtil.addFile(allegati, altri_allegati.get(i));
				}
			}
			if(ApplicationUtil.notEmpty(rmail.getAllegati())) {
				List<Allegato> ancora_allegati = new ArrayList<Allegato>(rmail.getAllegati().keySet());
				for(int i = 0; i < ancora_allegati.size(); i++) {
					ApplicationUtil.addFile(allegati, ancora_allegati.get(i));
				}
			}
		}
		String w = downloadDao.getWatermark();
		if(ApplicationUtil.isNotBlank(rmail.getTesto())) {
			try {
				ByteArrayOutputStream ostream = new ByteArrayOutputStream();
				ApplicationUtil.convert(replace(rmail.getContatto(), rmail.getTesto()), ostream);
				ostream.flush();
				ostream.close();
				ApplicationUtil.addFile(allegati, ApplicationUtil.getCover(w, ostream.toByteArray()), ConstantsLanguage.FRONTESPIZIO.stampa() + "." + ConstantsMap.pdf);
			}
			catch(Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		}
		if(rmail.getRmailTemplate().isLettera()) {
			Busta busta = rmail.getRmailTemplate().getBusta();
			if(busta != null) {
				Map<String, DatoAggiuntivo> valori = replaceByBustaId(busta, rmail.getDestinatario(), rmail.getContatto());
				byte [] b = ApplicationUtil.getBusta(w, busta, valori);
				ApplicationUtil.addFile(allegati, b, ConstantsLanguage.BUSTA.stampa() + "." + ConstantsMap.pdf);
			}
		}
		if (ApplicationUtil.isTrue(rmail.getRmailTemplate().getGenera_allegati())) {
			UtilOffice uo = impostazioneDao.getUtilOffice();
			int pagine = ApplicationUtil.convertPDF(w, allegati, uo);
			rmail.setPagine(pagine);
		}
		return allegati;

	}

	public ExtendedCriteria trovaTemplateCriteria(TemplateCriteria ema) {
		if(ema != null)  ema = (TemplateCriteria) ema.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Template.class);
		if(ApplicationUtil.isNotBlank(ema.getNomeOggetto())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true,  ema.getNomeOggetto(), criteria, "nome");
		}
		ema.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, ema);
	}
	@SuppressWarnings("unchecked")
	public List<Template> trovaTemplate(TemplateCriteria ema, int start, int size) {
		ExtendedCriteria criteria = trovaTemplateCriteria(ema);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int trovaTemplateSize(TemplateCriteria ema) {
		ExtendedCriteria criteria = trovaTemplateCriteria(ema);
		return getListSize(criteria);
	}
	public ExtendedCriteria trovaBustaCriteria(BustaCriteria ema) {
		if(ema != null)  ema = (BustaCriteria) ema.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Busta.class);
		if(ApplicationUtil.isNotBlank(ema.getNomeOggetto())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true,  ema.getNomeOggetto(), criteria, "nome");
		}
		ema.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, ema);
	}
	@SuppressWarnings("unchecked")
	public List<Template> trovaBusta(BustaCriteria ema, int start, int size) {
		ExtendedCriteria criteria = trovaBustaCriteria(ema);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int trovaBustaSize(BustaCriteria ema) {
		ExtendedCriteria criteria = trovaBustaCriteria(ema);
		return getListSize(criteria);
	}
	public ExtendedCriteria trovaSmtpCriteria(SmtpCriteria ema) {
		if(ema != null)  ema = (SmtpCriteria) ema.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Smtp.class);
		if(ApplicationUtil.isNotBlank(ema.getNomeOggetto())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true,  ema.getNomeOggetto(), criteria, "smtp_mittente");
		}
		ema.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, ema);
	}
	@SuppressWarnings("unchecked")
	public List<Template> trovaSmtp(SmtpCriteria ema, int start, int size) {
		ExtendedCriteria criteria = trovaSmtpCriteria(ema);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int trovaSmtpSize(SmtpCriteria ema) {
		ExtendedCriteria criteria = trovaSmtpCriteria(ema);
		return getListSize(criteria);
	}
	public ExtendedCriteria trovaInvioCriteria(InvioCriteria ema) {
		if(ema != null)  ema = (InvioCriteria) ema.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Invio.class);
		if(ApplicationUtil.isNotBlank(ema.getNomeOggetto())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true,  ema.getNomeOggetto(), criteria, "nome");
		}
		ema.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, ema);
	}
	@SuppressWarnings("unchecked")
	public List<Template> trovaInvio(InvioCriteria ema, int start, int size) {
		ExtendedCriteria criteria = trovaInvioCriteria(ema);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int trovaInvioSize(InvioCriteria ema) {
		ExtendedCriteria criteria = trovaInvioCriteria(ema);
		return getListSize(criteria);
	}
	public void getTemplate(Template documento, DownloadBean downloadBean) {
		OutputStream out = downloadBean.outputStream();
		downloadBean.filename(documento.getNomeFileS());
		try {
			InputStream bais = new ByteArrayInputStream(documento.getContent());
			ApplicationUtil.copy(bais, out);
			downloadBean.close();
			bais.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void getBusta(Busta busta, DownloadBean downloadBean) {
		OutputStream out = downloadBean.outputStream();
		downloadBean.filename(busta.getNomeFileS());
		try {
			InputStream bais = new ByteArrayInputStream(busta.getContent());
			ApplicationUtil.copy(bais, out);
			downloadBean.close();
			bais.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void inserisciFileInTemplate(FileUpload fileUpload, Template template) {
		try { 
			if(template.getId() != null) {
				String hql = "delete from TemplateStore where documento = :template";
				execute(false, hql, new Parameter("template", template));
			}
			List<MyFile> fls = fileUpload.getFiles();
			if(ApplicationUtil.notEmpty(fls)) {
				Iterator<MyFile> flsIt = fls.iterator();
				while (flsIt.hasNext()) {
					MyFile ff = (MyFile) flsIt.next();
					template.setNomeFile(ApplicationUtil.getNameN(ff.getName()));
					template.setContent(ff.getData());
				}
			}
			template.setApiKey(null);
			if(template.isEXT(ConstantsMap.a_pda1)) {
				template.setFirma(ApplicationUtil.verificaIntoTemplatePDF(template.getContent(), template.getNomeFileS()));
			} 
			if(template.isEXT(ConstantsMap.a_pda2)) {
				template.setFirma(ApplicationUtil.verificaIntoTemplateXLS(template.getContent(), template.getNomeFileS()));
			}
			if(!template.isEXT(ConstantsMap.a_pda1) && !template.isEXT(ConstantsMap.a_pda2)) {
				template.setFirma(0);
			}

		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void inserisciFileInBusta(FileUpload fileUpload, Busta busta) {
		try { 
			if(busta.getId() != null) {
				String hql = "delete from BustaStore where busta = :busta";
				execute(false, hql, new Parameter("busta", busta));
			}
			List<MyFile> fls = fileUpload.getFiles();
			if(ApplicationUtil.notEmpty(fls)) {
				Iterator<MyFile> flsIt = fls.iterator();
				while (flsIt.hasNext()) {
					MyFile ff = (MyFile) flsIt.next();
					busta.setNomeFile(ApplicationUtil.getNameN(ff.getName()));
					busta.setContent(ff.getData());
				}
			}
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public Map<String, DatoAggiuntivo> replaceByBustaId(Busta busta, Indirizzo indirizzo, Contatto contatto) {
		Map<String, DatoAggiuntivo> valoriNuovo = new HashMap<String, DatoAggiuntivo>();
		if(busta!= null) {
			Map<String, DatoAggiuntivo> valori  = new HashMap<String, DatoAggiuntivo>();
			List <String> valoriValidi = new ArrayList<String>(ConstantsMap.bustaMap);
			valori.putAll(ApplicationUtil.convertiAnagrafica(contatto.getAnagrafica(), valoriValidi));
			valori.putAll(ApplicationUtil.convertiIndirizzo(indirizzo, valoriValidi));
			valori.put(ConstantsMap.indirizzo_destinatario, new DatoAggiuntivo(indirizzo.getIndirizzo_completo()));
			List<BustaStore>  tmps = getBustaStoreByBustaId(busta);
			valoriNuovo.putAll(ApplicationUtil.replaceKeyBustainMap(valori, tmps));
		} 
		return valoriNuovo;
	}
	@SuppressWarnings("unchecked")
	public List<BustaStore> getBustaStoreByBustaId(Busta tmp) {
		DetachedCriteria criteria = DetachedCriteria.forClass(BustaStore.class);
		criteria.add(Restrictions.eq("busta.id", tmp.getId()));
		return  getList(criteria);
	}
	public Template salvaTemplateList(Template documento, List<TemplateStore> lst) {
		documento = (Template) salvaAggiorna(documento);
		if(ApplicationUtil.notEmpty(lst)) {
			for(TemplateStore tmp : lst) {
				tmp.setDocumento(documento);
				salvaAggiorna(tmp);
			}
		}
		return documento;
	}
	public Busta salvaBustaList(Busta busta, List<BustaStore> lst) {
		busta = (Busta) salvaAggiorna(busta);
		if(ApplicationUtil.notEmpty(lst)) {
			for(BustaStore tmp : lst) {
				tmp.setBusta(busta);
				salvaAggiorna(tmp);
			}
		}
		return busta;
	}
	public Smtp salvaSmtp(Smtp smtp) {
		if(ApplicationUtil.ltZero(smtp.getRicezione_type())) {
			smtp.setRicezione_host(null);
			smtp.setRicezione_port(null);
			smtp.setRicezione_email(null);
			smtp.setRicezione_link(null);
		}
		return (Smtp) salvaAggiorna(smtp);
	}
	public Invio salvaInvio(Invio invio) {
		return (Invio) salvaAggiorna(invio);
	}
	public Smtp copiaSmtp(Smtp smtp) {
		return impostazioneDao.copiaSmtp(smtp);
	}
	
	@SuppressWarnings("unchecked")
	public List<EmailTemplate> getTemplateByIdEmail(Long docId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(EmailTemplate.class);
		criteria.createCriteria("documenti").add(Restrictions.idEq(docId));
		return getList(criteria);
	}
	public Rmail getRmail(Long id){
		return (Rmail) get(Rmail.class, id);
	}
	public void eliminaRmail(Rmail id) {
		cancellaAggiorna(id);
	}

	private ExtendedCriteria trovaCriteriaRmail(RmailCriteria ema) {
		if(ema != null)  ema = (RmailCriteria) ema.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Rmail.class);
		DetachedCriteria criteriaCo = criteria.createCriteria("contatto");
		if (ApplicationUtil.isNotBlank(ema.getTesto())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true,  ema.getTesto(), criteria, "testoTemplate", "oggettoTemplate");
		}
		if (ApplicationUtil.isNotBlank(ema.getOperatore())) {
			DetachedCriteria cOp = criteria.createCriteria("utente");
			ApplicationUtil.operatore(cOp, ema.getOperatore());
		}
		if (ApplicationUtil.isNotBlank(ema.getRicerca_anagrafica())) {
			DetachedCriteria criteriaA = criteriaCo.createCriteria("anagrafica");
			ApplicationUtil.anagrafica(ema.getRicerca_anagraficaV(), criteriaA, ema.getRicerca_anagrafica());
		}
		if (ApplicationUtil.gtZero(ema.getCampagna())) {
			criteriaCo.add(Restrictions.eq("campagna.id", ema.getCampagna()));
		}
		if (ApplicationUtil.gtZero(ema.getCodice())) {
			criteria.add(Restrictions.eq("codice", ema.getCodice()));
		}
		if (ApplicationUtil.gtZero(ema.getTipo())) {
			criteria.createCriteria("rmailTemplate").add(Restrictions.eq("tipo", ema.getTipo()));
		}
		ema.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, ema);
	}
	@SuppressWarnings("unchecked")
	public List<Rmail> trovaRmail(RmailCriteria ema,int start, int size) {
		ExtendedCriteria criteria = trovaCriteriaRmail(ema);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int trovaRmailSize(RmailCriteria ema) {
		ExtendedCriteria criteria = trovaCriteriaRmail(ema);
		return getListSize(criteria);
	}
	
	@SuppressWarnings("unchecked")
	public List<RmailTemplate> trovaRmailTemplate(RmailTemplateCriteria ema, int start, int size){
		ExtendedCriteria criteria = getCriteriaRmail(ema);
		return getList(criteria, RmailTemplate.class, "documenti", new Order [] {Order.asc("id")}, start, size);
	}
	public int trovaRmailTemplateSize(RmailTemplateCriteria ema) {
		ExtendedCriteria criteria = getCriteriaRmail(ema);
		return getListSize(criteria);
	}
	private ExtendedCriteria getCriteriaRmail(RmailTemplateCriteria ema) {
		if(ema != null)  ema = (RmailTemplateCriteria) ema.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(RmailTemplate.class);
		if (ApplicationUtil.isNotBlank(ema.getNomeOggetto())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true,  ema.getNomeOggetto(), criteria, "nome", "testo");
		}
		if (ApplicationUtil.gtZero(ema.getTipo())) {
			criteria.add(Restrictions.eq("tipo", ema.getTipo()));
		}
		ema.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, ema); 
	}
	public RmailTemplate getRmailTemplateById(Long rmailTemplateId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(RmailTemplate.class);
		criteria.add(Restrictions.idEq(rmailTemplateId));
		criteria.setFetchMode("documenti", FetchMode.JOIN);
		return (RmailTemplate) getUniqueResult(criteria);
	}
	public RmailTemplate getRmailTemplate(Long id) {
		return (RmailTemplate) get(RmailTemplate.class, id);
	}
	
	public Rmail salvaRmail(Rmail rmail) {
		return (Rmail) salvaAggiorna(rmail);
	}
	
	public Rmail getRmailById(Long rmailId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Rmail.class);
		criteria.add(Restrictions.idEq(rmailId));
		return (Rmail) getUniqueResult(criteria);
	}
	
	public List<SelectItem> getTemplateRmailSI() {
		DetachedCriteria criteria = DetachedCriteria.forClass(RmailTemplate.class);
		List<GenericSI> lis = getGenericSI(criteria, "id", "nome");
		return impostazioneDao.getGeneric(lis, true);
	}
	
	@SuppressWarnings("unchecked")
	public List<RmailTemplate> getAllRmailTemplate(Long campagnaId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(RmailTemplate.class);
		criteria.add(Restrictions.eq("attivo", Boolean.TRUE));
		if(ApplicationUtil.gtZero(campagnaId)) {
			criteria.createCriteria("campagne").add(Restrictions.idEq(campagnaId));
		}
		criteria.setFetchMode("documenti", FetchMode.JOIN);
		criteria.addOrder(Order.asc("nome"));
		return getList(criteria);
	}
	

	@SuppressWarnings("unchecked")
	public List<Rmail> getRmailByRmailTemplate(RmailTemplate rmailTemplate) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Rmail.class);
		criteria.add(Restrictions.eq("rmailTemplate.id", rmailTemplate));
		return getList(criteria);
	}
	
	@SuppressWarnings("unchecked")
	public List<DocumentiCheckbox> getCheckboxDocumentiRmail(Rmail rmail) {
		List<Documento> documenti = getList(DetachedCriteria.forClass(Documento.class));
		List<DocumentiCheckbox> documentiCheckbox = new ArrayList<DocumentiCheckbox>();
		if(rmail.getRmailTemplate() != null && rmail.getRmailTemplate().getDocumenti() != null) {
			if(ApplicationUtil.empty(rmail.getAllegati())) { 
				Iterator<Documento> objIter = documenti.iterator();
				while (objIter.hasNext()) {
					Documento documento = objIter.next();
					boolean selected = rmail.getRmailTemplate().getDocumenti().contains(documento);
					documentiCheckbox.add(new DocumentiCheckbox(documento, selected, selected));
				}
			} else {
				Iterator<Documento> objIter = documenti.iterator();
				while (objIter.hasNext()) {
					Documento documento = objIter.next();
					boolean selected = rmail.getRmailTemplate().getDocumenti().contains(documento);
					Allegato a = new Allegato(documento.getNomeFileS(), documento.getContent());
					a.setData_modifica(documento.getData_creazione());
					documentiCheckbox.add(new DocumentiCheckbox(documento, selected || rmail.getAllegati().get(a) != null, selected));
				}
			}
		}
		else {
			if(ApplicationUtil.empty(rmail.getAllegati())) {
				Iterator<Documento> objIter = documenti.iterator();
				while (objIter.hasNext()) {
					Documento documento = objIter.next();
					documentiCheckbox.add(new DocumentiCheckbox(documento));
				}
			} else {
				Iterator<Documento> objIter = documenti.iterator();
				while (objIter.hasNext()) {
					Documento documento = objIter.next();
					Allegato a = new Allegato(documento.getNomeFileS(), documento.getContent());
					a.setData_modifica(documento.getData_creazione());
					documentiCheckbox.add(new DocumentiCheckbox(documento, rmail.getAllegati().get(a) != null, Boolean.FALSE));
				}
			}
		}
		return documentiCheckbox;
	}

	public RmailTemplate salvaRmailTemplate(RmailTemplate rmailTemplate) {
		if (rmailTemplate != null) {
			if(rmailTemplate.isFax() || rmailTemplate.isSms() || rmailTemplate.isTelegramma()) {
				rmailTemplate.setBusta(null);
			}
			if(rmailTemplate.isFax() || rmailTemplate.isSms()) {
				rmailTemplate.setResidenza(true);
			}
			if(rmailTemplate.isTelegramma() || rmailTemplate.isSms()) {
				rmailTemplate.setInvia_allegati(false);
				rmailTemplate.setGenera_allegati(false);
				rmailTemplate.setDocumenti(null);
			}
			rmailTemplate = (RmailTemplate) salvaAggiorna(rmailTemplate);
		}
		return rmailTemplate;
	}
	public Busta getBusta(Long id) {
		return (Busta) get(Busta.class, id);
	}
	public Smtp getSmtp(Long id) {
		return (Smtp) get(Smtp.class, id);
	}
	public Invio getInvio(Long id) {
		return (Invio) get(Invio.class, id);
	}
	public void addDocumentoToRmailTemplate(RmailTemplate rmailTemplate, Documento documento) {
		if (!rmailTemplate.getDocumenti().contains(documento)) {
			rmailTemplate.getDocumenti().add(documento);
		} 
	}
	public Rmail creaRmailDaTemplate(RmailTemplate rmailTemplate) {
		Rmail rmail = new Rmail();
		rmail.setTesto(rmailTemplate.getTesto());
		rmail.setRmailTemplate(rmailTemplate);
		return rmail;
	}
	
	public Integer inviaRmail(Rmail rmail, Utente user, MailSupport ms) {
		rmail.setUtente(user);
		rmail.setData_invio(ApplicationUtil.date());
		rmail.setDestinatarioTemplate(replace(rmail.getContatto(), downloadDao.mergeVelocityMessage(rmail.getRmailTemplate().getInvio().getDestinatarioTemplate(), rmail)));
		rmail.setOggettoTemplate(replace(rmail.getContatto(), downloadDao.mergeVelocityMessage(rmail.getRmailTemplate().getInvio().getOggettoTemplate(), rmail)));
		rmail.setTestoTemplate(replace(rmail.getContatto(), downloadDao.mergeVelocityMessage(rmail.getRmailTemplate().getInvio().getTestoTemplate(), rmail)));
		rmail.setAllegatiTemplateList(allegatoRmail(rmail));
		rmail.impostaUserfield();
		if(!rmail.inviabile()) {
			rmail.addMotivo(ConstantsGlobal.NON_INVIATA, ConstantsLanguage.ERRORE_INVIO.stampa());
			rmail = salvaRmail(rmail);
			return rmail.getCodice();
		} 
		if(!rmail.valido()) {
			rmail.addMotivo(ConstantsGlobal.NON_INVIATA, ConstantsLanguage.DESTINATARIO_VUOTO.stampa());
			rmail = salvaRmail(rmail);
			return rmail.getCodice();
		} 
		if(!rmail.getContatto().getAnagrafica().getConsenso_rmailT()) {
			rmail.addMotivo(ConstantsGlobal.NON_INVIATA, ConstantsLanguage.CONSENSO_NEGATO.stampa());
			rmail = salvaRmail(rmail);
			return rmail.getCodice();
		} 
		if(ApplicationUtil.gtOra(rmail.getDataSchedulata())) {
			rmail.setData_invio(rmail.getDataSchedulata());
			rmail.addMotivo(ConstantsGlobal.SCHEDULATA, null);
			Schedulatore sa = schedulerService.lancia(rmail.getId(), rmail.getDataSchedulata(), ConstantsGlobal.tipo_rmail);
			rmail.setScheduler(sa);
			rmail = salvaRmail(rmail);
			return rmail.getCodice();
		}
		rmail.addMotivo(ConstantsGlobal.IN_INVIO, null);
		rmail = salvaRmail(rmail);
		if(ms != null) {
			comunicazioneService.rmail(ms, rmail);
		} else {
			comunicazioneService.rmail(rmail);
		}
		return rmail.getCodice();
	}

	public Integer reInvia(Rmail rmail, MailSupport ms) {
		rmail.setData_invio(ApplicationUtil.date());
		rmail.setScheduler(null);
		if(!rmail.inviabile()) {
			rmail.addMotivo(ConstantsGlobal.NON_INVIATA, ConstantsLanguage.ERRORE_INVIO.stampa());
			rmail = salvaRmail(rmail);
			return rmail.getCodice();
		} 
		if(!rmail.valido()) {
			rmail.addMotivo(ConstantsGlobal.NON_INVIATA, ConstantsLanguage.DESTINATARIO_VUOTO.stampa());
			rmail = salvaRmail(rmail);
			return rmail.getCodice();
		}
		if(!rmail.getContatto().getAnagrafica().getConsenso_rmailT()) {
			rmail.addMotivo(ConstantsGlobal.NON_INVIATA, ConstantsLanguage.CONSENSO_NEGATO.stampa());
			rmail = salvaRmail(rmail);
			return rmail.getCodice();
		} 
		rmail.addMotivo(ConstantsGlobal.IN_INVIO, null);
		rmail = salvaRmail(rmail);
		if(ms != null) {
			comunicazioneService.rmail(ms, rmail);
		} else {
			comunicazioneService.rmail(rmail);
		}
		return rmail.getCodice();
	}

	public Rmail creaRmailDaTemplate(RmailTemplate rmailTemplate, Map<String, DatoAggiuntivo> valori) {
		Rmail rmail = new Rmail();
		rmail.setTesto(ApplicationUtil.replaceInString(rmailTemplate.getTesto(), valori));
		rmail.setRmailTemplate(rmailTemplate);
		return rmail;
	}
	

	public void inserisciAllegatiCDRmail(Rmail rmail, List<DocumentiCheckbox> documentiCheckbox) {
		HashMap<Allegato, String> map = new HashMap<Allegato, String>();
		Iterator<DocumentiCheckbox> objIter = documentiCheckbox.iterator();
		while (objIter.hasNext()) {
			DocumentiCheckbox dc = objIter.next();
			if(ApplicationUtil.isTrue(dc.getSelezionato()) && ApplicationUtil.isFalse(dc.getDisabled())) {
				ApplicationUtil.addFile(map, dc);
			}
		}
		rmail.setAllegati(map);
	}
	
	public void eliminaRmailTemplate(RmailTemplate rmailTemplate) {
		cancellaAggiorna(rmailTemplate);
	}
	
	
	
	@SuppressWarnings("unchecked")
	public List<RmailTemplate> getTemplateByIdRmail(Long docId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(RmailTemplate.class);
		criteria.createCriteria("documenti").add(Restrictions.idEq(docId));
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<RmailTemplate> getBustaByIdRmail(Long docId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(RmailTemplate.class);
		criteria.createCriteria("busta").add(Restrictions.idEq(docId));
		criteria.setFetchMode("documenti", FetchMode.JOIN);
		return getList(criteria);
	}
	
	@SuppressWarnings("unchecked")
	public List<RmailTemplate> getSmtpByIdRmail(Long docId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(RmailTemplate.class);
		criteria.createCriteria("smtp").add(Restrictions.idEq(docId));
		criteria.setFetchMode("documenti", FetchMode.JOIN);
		return getList(criteria);
	}
	
	@SuppressWarnings("unchecked")
	public List<EmailTemplate> getSmtpByIdEmail(Long docId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(EmailTemplate.class);
		criteria.createCriteria("smtp").add(Restrictions.idEq(docId));
		criteria.setFetchMode("documenti", FetchMode.JOIN);
		return getList(criteria);
	}
	
	@SuppressWarnings("unchecked")
	public List<RmailTemplate> getInvioByIdRmail(Long docId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(RmailTemplate.class);
		criteria.createCriteria("invio").add(Restrictions.idEq(docId));
		criteria.setFetchMode("documenti", FetchMode.JOIN);
		return getList(criteria);
	}
	
	public void scaricaAllegato(AllegatoUtils al, DownloadBean downloadBean) {
		try {
			InputStream is = null;
			if(al.isFileE()) {
				is = new ByteArrayInputStream(al.getFile());
			}
			if(al.isPercorsoE()) {
				is = new FileInputStream(new File(al.getPercorso()));
			}
			if(is != null) {
				OutputStream out = downloadBean.outputStream();
				downloadBean.filename(al.getNome());
				ApplicationUtil.copy(is, out);
				downloadBean.close();
				is.close();
			}
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	
	@SuppressWarnings("unchecked")
	public List<Long> getDaInviarePost(Schedulatore sa, Class<?> classe, Integer codice []) {
		DetachedCriteria criteria = DetachedCriteria.forClass(classe);
		criteria.add(Restrictions.in("codice", new ArrayList<Integer>(Arrays.asList(codice))));
		if(sa.getId() != null) {
			criteria.add(Restrictions.eq("scheduler.id", sa.getId()));
		}
		if(sa.getDateTime() == null) {
			criteria.add(Restrictions.le("data_invio", ApplicationUtil.date()));
		} else {
			criteria.add(Restrictions.eq("data_invio", sa.getDateTime()));
		}
		criteria.setProjection(Projections.distinct(Projections.property("id")));
		return getList(criteria);
	}
	
	public List<?> getDaInviarePostList(Schedulatore sa, Class<?> classe, Integer codice []) {
		DetachedCriteria criteria = DetachedCriteria.forClass(classe);
		criteria.add(Restrictions.in("codice", new ArrayList<Integer>(Arrays.asList(codice))));
		if(sa.getId() != null) {
			criteria.add(Restrictions.eq("scheduler.id", sa.getId()));
		}
		if(sa.getDateTime() == null) {
			criteria.add(Restrictions.le("data_invio", ApplicationUtil.date()));
		} else {
			criteria.add(Restrictions.eq("data_invio", sa.getDateTime()));
		}
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Email> getEmailDaInviareList(List<Long> ids) {
		if(ApplicationUtil.notEmpty(ids)) {
			DetachedCriteria criteria = DetachedCriteria.forClass(Email.class);
			criteria.add(Restrictions.in("id", ids));
			return getList(criteria);
		}
		return null;
	}
	
	@SuppressWarnings("unchecked")
	public List<Rmail> getRmailDaInviareList(List<Long> ids) {
		if(ApplicationUtil.notEmpty(ids)) {
			DetachedCriteria criteria = DetachedCriteria.forClass(Rmail.class);
			criteria.add(Restrictions.in("id", ids));
			return getList(criteria);
		}
		return null;
	}
	
	public void aggiungiEsitoArchiviazioneAdEmailTemplate(Long esitoId, Long templateEmailId) {
		EsitoArchiviazione esito = getFullEsitoArchiviazioneEmailTemplate(esitoId);
		EmailTemplate templateEmail = getEmailTemplate(templateEmailId);
		if (esito != null && templateEmail != null) {
			if (!esito.getEmails().contains(templateEmail)) {
				esito.getEmails().add(templateEmail);
				salvaAggiorna(esito);
			}
		}
	}
	public void rimuoviEsitoArchiviazioneAdEmailTemplate(Long esitoId, Long templateEmailId) {
		EsitoArchiviazione esito = getFullEsitoArchiviazioneEmailTemplate(esitoId);
		EmailTemplate templateEmail = getEmailTemplate(templateEmailId);
		if (esito != null && templateEmail != null) {
			if (esito.getEmails().contains(templateEmail)) {
				esito.getEmails().remove(templateEmail);
				salvaAggiorna(esito);
			}
		}
	}
	public void aggiungiEmailTemplateTuttiAdEsitoArchiviazione(Long esitoId) {
		EsitoArchiviazione esito  = getFullEsitoArchiviazioneEmailTemplate(esitoId);
		List<EmailTemplate> templateEmail = getEmailTemplateNotInEsitoArchiviazione(esitoId);
		if (ApplicationUtil.notEmpty(templateEmail) && esito != null) {
			esito.getEmails().addAll(templateEmail);
			salvaAggiorna(esito);
		}
	}
	public void rimuoviEmailTemplateTuttiAdEsitoArchiviazione(Long esitoId) {
		EsitoArchiviazione esito  = getFullEsitoArchiviazioneEmailTemplate(esitoId);
		List<EmailTemplate> templateEmail = getEmailTemplateInEsitoArchiviazione(esitoId);
		if (ApplicationUtil.notEmpty(templateEmail) && esito != null) {
			esito.getEmails().removeAll(templateEmail);
			salvaAggiorna(esito);
		}
	}
	public void aggiungiEmailTemplateAdEsitoArchiviazione(Long templateEmailId, Long esitoId) {
		EmailTemplate templateEmail = getFullEmailTemplateEsitoArchiviazione(templateEmailId);
		EsitoArchiviazione esito = campagnaDao.getEsitoArchiviazione(esitoId);
		if (templateEmail != null && esito != null) {
			if (!templateEmail.getEsiti().contains(esito)) {
				templateEmail.getEsiti().add(esito);
				salvaAggiorna(templateEmail);
			}
		}
	}
	public void rimuoviEmailTemplateAdEsitoArchiviazione(Long templateEmailId, Long esitoId) {
		EmailTemplate templateEmail = getFullEmailTemplateEsitoArchiviazione(templateEmailId);
		EsitoArchiviazione esito = campagnaDao.getEsitoArchiviazione(esitoId);
		if (templateEmail != null && esito != null) {
			if (templateEmail.getEsiti().contains(esito)) {
				templateEmail.getEsiti().remove(esito);
				salvaAggiorna(templateEmail);
			}
		}
	}
	public void aggiungiEsitoArchiviazioneTuttiAdEmailTemplate(Long templateEmailId) {
		EmailTemplate templateEmail  = getFullEmailTemplateEsitoArchiviazione(templateEmailId);
		List<EsitoArchiviazione> esito = getEsitoArchiviazioneNotInEmailTemplate(templateEmailId);
		if (ApplicationUtil.notEmpty(esito) && templateEmail != null) {
			templateEmail.getEsiti().addAll(esito);
			salvaAggiorna(templateEmail);
		}
	}
	public void rimuoviEsitoArchiviazioneTuttiAdEmailTemplate(Long templateEmailId) {
		EmailTemplate templateEmail  = getFullEmailTemplateEsitoArchiviazione(templateEmailId);
		List<EsitoArchiviazione> esito = getEsitoArchiviazioneInEmailTemplate(templateEmailId);
		if (ApplicationUtil.notEmpty(esito) && templateEmail != null) {
			templateEmail.getEsiti().removeAll(esito);
			salvaAggiorna(templateEmail);
		}
	}
	public EmailTemplate getFullEmailTemplateEsitoArchiviazione(Long templateEmail) {
		DetachedCriteria criteria = DetachedCriteria.forClass(EmailTemplate.class);
		criteria.add(Restrictions.idEq(templateEmail));
		criteria.setFetchMode("esiti", FetchMode.JOIN);
		return  (EmailTemplate) getUniqueResult(criteria);
	}
	public EsitoArchiviazione getFullEsitoArchiviazioneEmailTemplate(Long esito) {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoArchiviazione.class);
		criteria.add(Restrictions.idEq(esito));
		criteria.setFetchMode("emails", FetchMode.JOIN);
		return (EsitoArchiviazione) getUniqueResult(criteria);
	}

	@SuppressWarnings("unchecked")
	public List<EsitoArchiviazione> getEsitoArchiviazioneNotInEmailTemplate(Long templateEmail) {
		DetachedCriteria criteriaU = DetachedCriteria.forClass(EsitoArchiviazione.class);
		criteriaU.createCriteria("emails").add(Restrictions.idEq(templateEmail));
		criteriaU.setProjection(Projections.distinct(Projections.property("id")));
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoArchiviazione.class);
		impostazioneDao.subPropertyNotInS(criteria, criteriaU, "id");
		ApplicationUtil.orderEsito(criteria);
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<EmailTemplate> getEmailTemplateNotInEsitoArchiviazione(Long esito) {
		DetachedCriteria criteriaU = DetachedCriteria.forClass(EmailTemplate.class);
		criteriaU.createCriteria("esiti").add(Restrictions.idEq(esito));
		criteriaU.setProjection(Projections.distinct(Projections.property("id")));
		DetachedCriteria criteria = DetachedCriteria.forClass(EmailTemplate.class);
		impostazioneDao.subPropertyNotInS(criteria, criteriaU, "id");
		criteria.addOrder(Order.asc("nome"));
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<EmailTemplate> getEmailTemplateInEsitoArchiviazione(Long esito) {
		if(esito != null ){
			DetachedCriteria criteria = DetachedCriteria.forClass(EmailTemplate.class);
			criteria.createCriteria("esiti").add(Restrictions.idEq(esito));
			criteria.setFetchMode("documenti", FetchMode.JOIN);
			criteria.addOrder(Order.asc("nome"));
			return getList(criteria);
		}	else {
			return new ArrayList<EmailTemplate>();
		}
	}

	public void aggiungiEsitoArchiviazioneAdRmailTemplate(Long esitoId, Long templateRmailId) {
		EsitoArchiviazione esito = getFullEsitoArchiviazioneRmailTemplate(esitoId);
		RmailTemplate templateRmail = getRmailTemplate(templateRmailId);
		if (esito != null && templateRmail != null) {
			if (!esito.getRmails().contains(templateRmail)) {
				esito.getRmails().add(templateRmail);
				salvaAggiorna(esito);
			}
		}
	}
	public void rimuoviEsitoArchiviazioneAdRmailTemplate(Long esitoId, Long templateRmailId) {
		EsitoArchiviazione esito = getFullEsitoArchiviazioneRmailTemplate(esitoId);
		RmailTemplate templateRmail = getRmailTemplate(templateRmailId);
		if (esito != null && templateRmail != null) {
			if (esito.getRmails().contains(templateRmail)) {
				esito.getRmails().remove(templateRmail);
				salvaAggiorna(esito);
			}
		}
	}
	public void aggiungiRmailTemplateTuttiAdEsitoArchiviazione(Long esitoId) {
		EsitoArchiviazione esito  = getFullEsitoArchiviazioneRmailTemplate(esitoId);
		List<RmailTemplate> templateRmail = getRmailTemplateNotInEsitoArchiviazione(esitoId);
		if (ApplicationUtil.notEmpty(templateRmail) && esito != null) {
			esito.getRmails().addAll(templateRmail);
			salvaAggiorna(esito);
		}
	}
	public void rimuoviRmailTemplateTuttiAdEsitoArchiviazione(Long esitoId) {
		EsitoArchiviazione esito  = getFullEsitoArchiviazioneRmailTemplate(esitoId);
		List<RmailTemplate> templateRmail = getRmailTemplateInEsitoArchiviazione(esitoId);
		if (ApplicationUtil.notEmpty(templateRmail) && esito != null) {
			esito.getRmails().removeAll(templateRmail);
			salvaAggiorna(esito);
		}
	}
	public void aggiungiRmailTemplateAdEsitoArchiviazione(Long templateRmailId, Long esitoId) {
		RmailTemplate templateRmail = getFullRmailTemplateEsitoArchiviazione(templateRmailId);
		EsitoArchiviazione esito = campagnaDao.getEsitoArchiviazione(esitoId);
		if (templateRmail != null && esito != null) {
			if (!templateRmail.getEsiti().contains(esito)) {
				templateRmail.getEsiti().add(esito);
				salvaAggiorna(templateRmail);
			}
		}
	}
	public void rimuoviRmailTemplateAdEsitoArchiviazione(Long templateRmailId, Long esitoId) {
		RmailTemplate templateRmail = getFullRmailTemplateEsitoArchiviazione(templateRmailId);
		EsitoArchiviazione esito = campagnaDao.getEsitoArchiviazione(esitoId);
		if (templateRmail != null && esito != null) {
			if (templateRmail.getEsiti().contains(esito)) {
				templateRmail.getEsiti().remove(esito);
				salvaAggiorna(templateRmail);
			}
		}
	}
	public void aggiungiEsitoArchiviazioneTuttiAdRmailTemplate(Long templateRmailId) {
		RmailTemplate templateRmail  = getFullRmailTemplateEsitoArchiviazione(templateRmailId);
		List<EsitoArchiviazione> esito = getEsitoArchiviazioneNotInRmailTemplate(templateRmailId);
		if (ApplicationUtil.notEmpty(esito) && templateRmail != null) {
			templateRmail.getEsiti().addAll(esito);
			salvaAggiorna(templateRmail);
		}
	}
	public void rimuoviEsitoArchiviazioneTuttiAdRmailTemplate(Long templateRmailId) {
		RmailTemplate templateRmail  = getFullRmailTemplateEsitoArchiviazione(templateRmailId);
		List<EsitoArchiviazione> esito = getEsitoArchiviazioneInRmailTemplate(templateRmailId);
		if (ApplicationUtil.notEmpty(esito) && templateRmail != null) {
			templateRmail.getEsiti().removeAll(esito);
			salvaAggiorna(templateRmail);
		}
	}
	public RmailTemplate getFullRmailTemplateEsitoArchiviazione(Long templateRmail) {
		DetachedCriteria criteria = DetachedCriteria.forClass(RmailTemplate.class);
		criteria.add(Restrictions.idEq(templateRmail));
		criteria.setFetchMode("esiti", FetchMode.JOIN);
		return  (RmailTemplate) getUniqueResult(criteria);
	}
	public EsitoArchiviazione getFullEsitoArchiviazioneRmailTemplate(Long esito) {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoArchiviazione.class);
		criteria.add(Restrictions.idEq(esito));
		criteria.setFetchMode("rmails", FetchMode.JOIN);
		return (EsitoArchiviazione) getUniqueResult(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<EsitoArchiviazione> getEsitoArchiviazioneInRmailTemplate(Long templateRmail) {
		if(templateRmail != null ){
			DetachedCriteria criteria = DetachedCriteria.forClass(EsitoArchiviazione.class);
			criteria.createCriteria("rmails").add(Restrictions.idEq(templateRmail));
			criteria.setFetchMode("esito_collegato", FetchMode.JOIN);
			ApplicationUtil.orderEsito(criteria);
			return getList(criteria);
		}	else {
			return new ArrayList<EsitoArchiviazione>();
		}
	}
	@SuppressWarnings("unchecked")
	public List<EsitoArchiviazione> getEsitoArchiviazioneNotInRmailTemplate(Long templateRmail) {
		DetachedCriteria criteriaU = DetachedCriteria.forClass(EsitoArchiviazione.class);
		criteriaU.createCriteria("rmails").add(Restrictions.idEq(templateRmail));
		criteriaU.setProjection(Projections.distinct(Projections.property("id")));
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoArchiviazione.class);
		impostazioneDao.subPropertyNotInS(criteria, criteriaU, "id");
		ApplicationUtil.orderEsito(criteria);
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<RmailTemplate> getRmailTemplateNotInEsitoArchiviazione(Long esito) {
		DetachedCriteria criteriaU = DetachedCriteria.forClass(RmailTemplate.class);
		criteriaU.createCriteria("esiti").add(Restrictions.idEq(esito));
		criteriaU.setProjection(Projections.distinct(Projections.property("id")));
		DetachedCriteria criteria = DetachedCriteria.forClass(RmailTemplate.class);
		impostazioneDao.subPropertyNotInS(criteria, criteriaU, "id");
		criteria.addOrder(Order.asc("nome"));
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<RmailTemplate> getRmailTemplateInEsitoArchiviazione(Long esito) {
		if(esito != null ){
			DetachedCriteria criteria = DetachedCriteria.forClass(RmailTemplate.class);
			criteria.createCriteria("esiti").add(Restrictions.idEq(esito));
			criteria.setFetchMode("documenti", FetchMode.JOIN);
			criteria.addOrder(Order.asc("nome"));
			return getList(criteria);
		}	else {
			return new ArrayList<RmailTemplate>();
		}
	}
	
	
	@SuppressWarnings("unchecked")
	public List<EsitoArchiviazione> getEsitoArchiviazioneInEmailTemplate(Long templateEmail) {
		if(templateEmail != null ){
			DetachedCriteria criteria = DetachedCriteria.forClass(EsitoArchiviazione.class);
			criteria.createCriteria("emails").add(Restrictions.idEq(templateEmail));
			criteria.setFetchMode("esito_collegato", FetchMode.JOIN);
			ApplicationUtil.orderEsito(criteria);
			return getList(criteria);
		}	else {
			return new ArrayList<EsitoArchiviazione>();
		}
	}
	
	@SuppressWarnings("unchecked")
	public List<EsitoArchiviazione> getEsitoArchiviazioneInTelecomTemplate(Long templateTelecom) {
		if(templateTelecom != null ){
			DetachedCriteria criteria = DetachedCriteria.forClass(EsitoArchiviazione.class);
			criteria.createCriteria("telecoms").add(Restrictions.idEq(templateTelecom));
			criteria.setFetchMode("esito_collegato", FetchMode.JOIN);
			ApplicationUtil.orderEsito(criteria);
			return getList(criteria);
		}	else {
			return new ArrayList<EsitoArchiviazione>();
		}
	}
	
	@SuppressWarnings("unchecked")
	public List<EmailTemplate> getAllEmailTemplate(Long campagnaId, EsitoArchiviazione esito) {
		DetachedCriteria criteria = DetachedCriteria.forClass(EmailTemplate.class);
		criteria.createCriteria("esiti").add(Restrictions.idEq(esito.getId()));
		criteria.add(Restrictions.eq("attivo", Boolean.TRUE));
		criteria.setFetchMode("documenti", FetchMode.JOIN);
		criteria.addOrder(Order.asc("nome"));
		List<EmailTemplate> ls = getList(criteria);
		if(ApplicationUtil.empty(ls)) {
			return getAllEmailTemplate(campagnaId);
		} else {
			return ls;
		}
	}
	
	@SuppressWarnings("unchecked")
	public List<RmailTemplate> getAllRmailTemplate(Long campagnaId, EsitoArchiviazione esito) {
		DetachedCriteria criteria = DetachedCriteria.forClass(RmailTemplate.class);
		criteria.createCriteria("esiti").add(Restrictions.idEq(esito.getId()));
		criteria.add(Restrictions.eq("attivo", Boolean.TRUE));
		criteria.setFetchMode("documenti", FetchMode.JOIN);
		criteria.addOrder(Order.asc("nome"));
		List<RmailTemplate> ls = getList(criteria);
		if(ApplicationUtil.empty(ls)) {
			return getAllRmailTemplate(campagnaId);
		} else {
			return ls;
		}
	}

	public void  inviaSelezionati(List<?> lst) {
		new ReInvioComunicazioneThread(lst, this).start();
	}
	public void  inviaNotificaSelezionati(List<Notifica> lst) {
		new ReInvioNotificaThread(lst, smtpService, this).start();
	}
	public void  inviaPostaSelezionati(List<Comunicazione> lst) {
		new ReInvioPostaThread(lst, smtpService, this).start();
	}
	public String comunicazioneTemplate(String type, String hash) {
		String ids = ApplicationUtil.base64Decoder(hash);
		Long id = ApplicationUtil.asLong(ids);
		if(ApplicationUtil.isNotBlank(type) && ApplicationUtil.gtZero(id)) {
			Map<String, Object> model = new HashMap<String, Object>();
			model.put("tipo", ApplicationUtil.capitalize(type));
			if(type.equals("email")) {
				Email o = getEmail(id);
				if(o != null) {
					o.addMotivo(ConstantsGlobal.APERTO, null);
					salvaEmail(o);
					model.put("oggetto", ApplicationUtil.trimToNull(o.getOggetto()));
					model.put("testo", ApplicationUtil.trimToNull(o.getTesto()));
					model.put("allegati", o.getAllegatiTemplateList());
				}
			}
			if(type.equals("rmail")) {
				Rmail o = getRmail(id);
				if(o != null) {
					o.addMotivo(ConstantsGlobal.APERTO, null);
					salvaRmail(o);
					model.put("oggetto", ApplicationUtil.trimToNull(o.getOggettoTemplate()));
					model.put("testo", ApplicationUtil.trimToNull(o.getTestoTemplate()));
					model.put("allegati", o.getAllegatiTemplateList());
				}
			}
			return downloadDao.mergeVelocityTemplate("message", model);
		}
		return "";
	}
	
	public void salvaComunicazione(String type, String hash) {
		String ids = ApplicationUtil.base64Decoder(hash);
		Long id = ApplicationUtil.asLong(ids);
		if(ApplicationUtil.isNotBlank(type) && ApplicationUtil.gtZero(id)) {
			if(type.equals("email")) {
				Email o = getEmail(id);
				if(o != null) {
					o.addMotivo(ConstantsGlobal.APERTO, null);
					salvaEmail(o);
				}
			}
			if(type.equals("rmail")) {
				Rmail o = getRmail(id);
				if(o != null) {
					o.addMotivo(ConstantsGlobal.APERTO, null);
					salvaRmail(o);
				}
			}
		}
	}
	
	@SuppressWarnings("unchecked")
	public List<TelecomTemplate> getAllTelecomTemplate(Long campagnaId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(TelecomTemplate.class);
		criteria.add(Restrictions.eq("attivo", Boolean.TRUE));
		if(ApplicationUtil.gtZero(campagnaId)) {
			criteria.createCriteria("campagne").add(Restrictions.idEq(campagnaId));
		}
		criteria.addOrder(Order.asc("nome"));
		return getList(criteria);
	}

	public Telecom getTelecom(Long id){
		return (Telecom) get(Telecom.class, id);
	}
	public void eliminaTelecom(Telecom id) {
		cancellaAggiorna(id);
	}
	private ExtendedCriteria trovaCriteriaTelecom(TelecomCriteria ema) {
		if(ema != null)  ema = (TelecomCriteria) ema.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Telecom.class);
		DetachedCriteria criteriaCo = criteria.createCriteria("contatto");
		if (ApplicationUtil.isNotBlank(ema.getTesto())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true,  ema.getTesto(), criteria, "body", "link", "headers", "parameters");
		}
		if (ApplicationUtil.isNotBlank(ema.getOperatore())) {
			DetachedCriteria cOp = criteria.createCriteria("utente");
			ApplicationUtil.operatore(cOp, ema.getOperatore());
		}
		if (ApplicationUtil.isNotBlank(ema.getRicerca_anagrafica())) {
			DetachedCriteria criteriaA = criteriaCo.createCriteria("anagrafica");
			ApplicationUtil.anagrafica(ema.getRicerca_anagraficaV(), criteriaA, ema.getRicerca_anagrafica());
		}
		if (ApplicationUtil.gtZero(ema.getCampagna())) {
			criteriaCo.add(Restrictions.eq("campagna.id", ema.getCampagna()));
		}
		if (ApplicationUtil.gtZero(ema.getCodice())) {
			criteria.add(Restrictions.eq("codice", ema.getCodice()));
		}
		ema.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, ema);
	}
	@SuppressWarnings("unchecked")
	public List<Telecom> trovaTelecom(TelecomCriteria ema,int start, int size) {
		ExtendedCriteria criteria = trovaCriteriaTelecom(ema);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int trovaTelecomSize(TelecomCriteria ema) {
		ExtendedCriteria criteria = trovaCriteriaTelecom(ema);
		return getListSize(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<TelecomTemplate> trovaTelecomTemplate(TelecomTemplateCriteria ema, int start, int size){
		ExtendedCriteria criteria = getCriteriaTelecom(ema);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int trovaTelecomTemplateSize(TelecomTemplateCriteria ema) {
		ExtendedCriteria criteria = getCriteriaTelecom(ema);
		return getListSize(criteria);
	}
	private ExtendedCriteria getCriteriaTelecom(TelecomTemplateCriteria ema) {
		if(ema != null)  ema = (TelecomTemplateCriteria) ema.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(TelecomTemplate.class);
		if (ApplicationUtil.isNotBlank(ema.getNomeOggetto())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true,  ema.getNomeOggetto(), criteria, "nome", "body", "link", "headers", "parameters");
		}
		ema.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, ema); 
	}
	public TelecomTemplate getTelecomTemplate(Long id) {
		return (TelecomTemplate) get(TelecomTemplate.class, id);
	}
	public TelecomTemplate getTelecomTemplateById(Long telecomTemplateId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(TelecomTemplate.class);
		criteria.add(Restrictions.idEq(telecomTemplateId));
		return (TelecomTemplate) getUniqueResult(criteria);
	}
	
	public List<SelectItem> getTemplateTelecomSI() {
		DetachedCriteria criteria = DetachedCriteria.forClass(TelecomTemplate.class);
		List<GenericSI> lis = getGenericSI(criteria, "id", "nome");
		return impostazioneDao.getGeneric(lis, true);
	}
	public Telecom salvaTelecom(Telecom telecom) {
		return (Telecom) salvaAggiorna(telecom);
	}
	public Telecom getTelecomById(Long telecomId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Telecom.class);
		criteria.add(Restrictions.idEq(telecomId));
		return (Telecom) getUniqueResult(criteria);
	}

	@SuppressWarnings("unchecked")
	public List<Telecom> getTelecomByTelecomTemplate(TelecomTemplate telecomTemplate) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Telecom.class);
		criteria.add(Restrictions.eq("telecomTemplate.id", telecomTemplate));
		return getList(criteria);
	}
	public TelecomTemplate salvaTelecomTemplate(TelecomTemplate telecomTemplate) {
		if (telecomTemplate != null) {
			telecomTemplate = (TelecomTemplate) salvaAggiorna(telecomTemplate);
		}
		return telecomTemplate;
	}
	public Telecom creaTelecomDaTemplate(TelecomTemplate telecomTemplate) {
		Telecom telecom = new Telecom();
		telecom.setTipo(telecomTemplate.getTipo());
		telecom.setSistema(telecomTemplate.getSistema());
		telecom.setDriver(telecomTemplate.getDriver());
		telecom.setCommands(telecomTemplate.getCommands());
		telecom.setBody(telecomTemplate.getBody());
		telecom.setUsername(telecomTemplate.getUsername());
		telecom.setPassword(telecomTemplate.getPassword());
		telecom.setHeaders(telecomTemplate.getHeaders());
		telecom.setFields(telecomTemplate.getFields());
		telecom.setOutputs(telecomTemplate.getOutputs());
		telecom.setParameters(telecomTemplate.getParameters());
		telecom.setLink(telecomTemplate.getLink());
		telecom.setInputs(telecomTemplate.getInputs());
		telecom.setLabels(ApplicationUtil.readText(telecom.getInputs()));
		telecom.setTelecomTemplate(telecomTemplate);
		return telecom;
	}
	public Telecom creaTelecomDaTemplate(TelecomTemplate telecomTemplate, Map<String, DatoAggiuntivo> valori) {
		Telecom telecom = new Telecom();
		telecom.setTipo(telecomTemplate.getTipo());
		telecom.setSistema(telecomTemplate.getSistema());
		telecom.setDriver(ApplicationUtil.replaceInString(telecomTemplate.getDriver(), valori));
		telecom.setCommands(ApplicationUtil.replaceInString(telecomTemplate.getCommands(), valori));
		telecom.setUsername(ApplicationUtil.replaceInString(telecomTemplate.getUsername(), valori));
		telecom.setPassword(ApplicationUtil.replaceInString(telecomTemplate.getPassword(), valori));
		telecom.setBody(ApplicationUtil.replaceInString(telecomTemplate.getBody(), valori));
		telecom.setHeaders(ApplicationUtil.replaceInString(telecomTemplate.getHeaders(), valori));
		telecom.setFields(ApplicationUtil.replaceInString(telecomTemplate.getFields(), valori));
		telecom.setOutputs(ApplicationUtil.replaceInString(telecomTemplate.getOutputs(), valori));
		telecom.setParameters(ApplicationUtil.replaceInString(telecomTemplate.getParameters(), valori));
		telecom.setLink(ApplicationUtil.replaceInString(telecomTemplate.getLink(), valori));
		telecom.setInputs(ApplicationUtil.replaceInString(telecomTemplate.getInputs(), valori));
		telecom.setLabels(ApplicationUtil.readText(telecom.getInputs()));
		telecom.setTelecomTemplate(telecomTemplate);
		return telecom;
	}
	
	public void eliminaTelecomTemplate(TelecomTemplate telecomTemplate) {
		cancellaAggiorna(telecomTemplate);
	}
	public Integer inviaTelecom(Telecom telecom, Utente user, Boolean background) {
		telecom.setUtente(user);
		telecom.setData_invio(ApplicationUtil.date());
		telecom.impostaUserfield();
		if(!telecom.inviabile()) {
			telecom.addMotivo(ConstantsGlobal.NON_INVIATA, ConstantsLanguage.ERRORE_INVIO.stampa());
			telecom = salvaTelecom(telecom);
			return telecom.getCodice();
		} 
		if(!telecom.valido()) {
			telecom.addMotivo(ConstantsGlobal.NON_INVIATA, ConstantsLanguage.DESTINATARIO_VUOTO.stampa());
			telecom = salvaTelecom(telecom);
			return telecom.getCodice();
		} 
		if(!telecom.getContatto().getAnagrafica().getConsensoT()) {
			telecom.addMotivo(ConstantsGlobal.NON_INVIATA, ConstantsLanguage.CONSENSO_NEGATO.stampa());
			telecom = salvaTelecom(telecom);
			return telecom.getCodice();
		} 
		if(ApplicationUtil.gtOra(telecom.getDataSchedulata())) {
			telecom.setData_invio(telecom.getDataSchedulata());
			telecom.addMotivo(ConstantsGlobal.SCHEDULATA, null);
			Schedulatore sa = schedulerService.lancia(telecom.getId(), telecom.getDataSchedulata(), ConstantsGlobal.tipo_telecom);
			telecom.setScheduler(sa);
			telecom = salvaTelecom(telecom);
			return telecom.getCodice();
		}
		telecom.addMotivo(ConstantsGlobal.IN_INVIO, null);
		telecom = salvaTelecom(telecom);
		if(!ApplicationUtil.isTrue(background)) {
			comunicazioneService.telecom(null, telecom);
		} else {
			comunicazioneService.telecom(telecom);
		}
		return telecom.getCodice();
	}
	
	@SuppressWarnings("unchecked")
	public List<Telecom> getTelecomDaInviareList(List<Long> ids) {
		if(ApplicationUtil.notEmpty(ids)) {
			DetachedCriteria criteria = DetachedCriteria.forClass(Telecom.class);
			criteria.add(Restrictions.in("id", ids));
			return getList(criteria);
		}
		return null;
	}
	public void aggiungiEsitoArchiviazioneAdTelecomTemplate(Long esitoId, Long templateTelecomId) {
		EsitoArchiviazione esito = getFullEsitoArchiviazioneTelecomTemplate(esitoId);
		TelecomTemplate templateTelecom = getTelecomTemplate(templateTelecomId);
		if (esito != null && templateTelecom != null) {
			if (!esito.getTelecoms().contains(templateTelecom)) {
				esito.getTelecoms().add(templateTelecom);
				salvaAggiorna(esito);
			}
		}
	}
	public void rimuoviEsitoArchiviazioneAdTelecomTemplate(Long esitoId, Long templateTelecomId) {
		EsitoArchiviazione esito = getFullEsitoArchiviazioneTelecomTemplate(esitoId);
		TelecomTemplate templateTelecom = getTelecomTemplate(templateTelecomId);
		if (esito != null && templateTelecom != null) {
			if (esito.getTelecoms().contains(templateTelecom)) {
				esito.getTelecoms().remove(templateTelecom);
				salvaAggiorna(esito);
			}
		}
	}
	public void aggiungiTelecomTemplateTuttiAdEsitoArchiviazione(Long esitoId) {
		EsitoArchiviazione esito  = getFullEsitoArchiviazioneTelecomTemplate(esitoId);
		List<TelecomTemplate> templateTelecom = getTelecomTemplateNotInEsitoArchiviazione(esitoId);
		if (ApplicationUtil.notEmpty(templateTelecom) && esito != null) {
			esito.getTelecoms().addAll(templateTelecom);
			salvaAggiorna(esito);
		}
	}
	public void rimuoviTelecomTemplateTuttiAdEsitoArchiviazione(Long esitoId) {
		EsitoArchiviazione esito  = getFullEsitoArchiviazioneTelecomTemplate(esitoId);
		List<TelecomTemplate> templateTelecom = getTelecomTemplateInEsitoArchiviazione(esitoId);
		if (ApplicationUtil.notEmpty(templateTelecom) && esito != null) {
			esito.getTelecoms().removeAll(templateTelecom);
			salvaAggiorna(esito);
		}
	}
	public void aggiungiTelecomTemplateAdEsitoArchiviazione(Long templateTelecomId, Long esitoId) {
		TelecomTemplate templateTelecom = getFullTelecomTemplateEsitoArchiviazione(templateTelecomId);
		EsitoArchiviazione esito = campagnaDao.getEsitoArchiviazione(esitoId);
		if (templateTelecom != null && esito != null) {
			if (!templateTelecom.getEsiti().contains(esito)) {
				templateTelecom.getEsiti().add(esito);
				salvaAggiorna(templateTelecom);
			}
		}
	}
	public void rimuoviTelecomTemplateAdEsitoArchiviazione(Long templateTelecomId, Long esitoId) {
		TelecomTemplate templateTelecom = getFullTelecomTemplateEsitoArchiviazione(templateTelecomId);
		EsitoArchiviazione esito = campagnaDao.getEsitoArchiviazione(esitoId);
		if (templateTelecom != null && esito != null) {
			if (templateTelecom.getEsiti().contains(esito)) {
				templateTelecom.getEsiti().remove(esito);
				salvaAggiorna(templateTelecom);
			}
		}
	}
	public void aggiungiEsitoArchiviazioneTuttiAdTelecomTemplate(Long templateTelecomId) {
		TelecomTemplate templateTelecom  = getFullTelecomTemplateEsitoArchiviazione(templateTelecomId);
		List<EsitoArchiviazione> esito = getEsitoArchiviazioneNotInTelecomTemplate(templateTelecomId);
		if (ApplicationUtil.notEmpty(esito) && templateTelecom != null) {
			templateTelecom.getEsiti().addAll(esito);
			salvaAggiorna(templateTelecom);
		}
	}
	public void rimuoviEsitoArchiviazioneTuttiAdTelecomTemplate(Long templateTelecomId) {
		TelecomTemplate templateTelecom  = getFullTelecomTemplateEsitoArchiviazione(templateTelecomId);
		List<EsitoArchiviazione> esito = getEsitoArchiviazioneInTelecomTemplate(templateTelecomId);
		if (ApplicationUtil.notEmpty(esito) && templateTelecom != null) {
			templateTelecom.getEsiti().removeAll(esito);
			salvaAggiorna(templateTelecom);
		}
	}
	public TelecomTemplate getFullTelecomTemplateEsitoArchiviazione(Long templateTelecom) {
		DetachedCriteria criteria = DetachedCriteria.forClass(TelecomTemplate.class);
		criteria.add(Restrictions.idEq(templateTelecom));
		criteria.setFetchMode("esiti", FetchMode.JOIN);
		return  (TelecomTemplate) getUniqueResult(criteria);
	}
	public EsitoArchiviazione getFullEsitoArchiviazioneTelecomTemplate(Long esito) {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoArchiviazione.class);
		criteria.add(Restrictions.idEq(esito));
		criteria.setFetchMode("telecoms", FetchMode.JOIN);
		return (EsitoArchiviazione) getUniqueResult(criteria);
	}

	@SuppressWarnings("unchecked")
	public List<EsitoArchiviazione> getEsitoArchiviazioneNotInTelecomTemplate(Long templateTelecom) {
		DetachedCriteria criteriaU = DetachedCriteria.forClass(EsitoArchiviazione.class);
		criteriaU.createCriteria("telecoms").add(Restrictions.idEq(templateTelecom));
		criteriaU.setProjection(Projections.distinct(Projections.property("id")));
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoArchiviazione.class);
		impostazioneDao.subPropertyNotInS(criteria, criteriaU, "id");
		ApplicationUtil.orderEsito(criteria);
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<TelecomTemplate> getTelecomTemplateNotInEsitoArchiviazione(Long esito) {
		DetachedCriteria criteriaU = DetachedCriteria.forClass(TelecomTemplate.class);
		criteriaU.createCriteria("esiti").add(Restrictions.idEq(esito));
		criteriaU.setProjection(Projections.distinct(Projections.property("id")));
		DetachedCriteria criteria = DetachedCriteria.forClass(TelecomTemplate.class);
		impostazioneDao.subPropertyNotInS(criteria, criteriaU, "id");
		criteria.addOrder(Order.asc("nome"));
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<TelecomTemplate> getTelecomTemplateInEsitoArchiviazione(Long esito) {
		if(esito != null ){
			DetachedCriteria criteria = DetachedCriteria.forClass(TelecomTemplate.class);
			criteria.createCriteria("esiti").add(Restrictions.idEq(esito));
			criteria.addOrder(Order.asc("nome"));
			return getList(criteria);
		}	else {
			return new ArrayList<TelecomTemplate>();
		}
	}
	@SuppressWarnings("unchecked")
	public List<TelecomTemplate> getAllTelecomTemplate(Long campagnaId, EsitoArchiviazione esito) {
		DetachedCriteria criteria = DetachedCriteria.forClass(TelecomTemplate.class);
		criteria.createCriteria("esiti").add(Restrictions.idEq(esito.getId()));
		criteria.add(Restrictions.eq("attivo", Boolean.TRUE));
		criteria.addOrder(Order.asc("nome"));
		List<TelecomTemplate> ls = getList(criteria);
		if(ApplicationUtil.empty(ls)) {
			return getAllTelecomTemplate(campagnaId);
		} else {
			return ls;
		}
	}
	public Integer reInvia(Telecom telecom, boolean background) {
		telecom.setData_invio(ApplicationUtil.date());
		telecom.setScheduler(null);
		if(!telecom.inviabile()) {
			telecom.addMotivo(ConstantsGlobal.NON_INVIATA, ConstantsLanguage.ERRORE_INVIO.stampa());
			telecom = salvaTelecom(telecom);
			return telecom.getCodice();
		} 
		if(!telecom.valido()) {
			telecom.addMotivo(ConstantsGlobal.NON_INVIATA, ConstantsLanguage.DESTINATARIO_VUOTO.stampa());
			telecom = salvaTelecom(telecom);
			return telecom.getCodice();
		}
		if(!telecom.getContatto().getAnagrafica().getConsensoT()) {
			telecom.addMotivo(ConstantsGlobal.NON_INVIATA, ConstantsLanguage.CONSENSO_NEGATO.stampa());
			telecom = salvaTelecom(telecom);
			return telecom.getCodice();
		} 
		telecom.addMotivo(ConstantsGlobal.IN_INVIO, null);
		telecom = salvaTelecom(telecom);
		if(!ApplicationUtil.isTrue(background)) {
			comunicazioneService.telecom(null, telecom);
		} else {
			comunicazioneService.telecom(telecom);
		}
		return telecom.getCodice();
	}

	public String replace(Contatto contatto, String testo) {
		List <String> valoriValidi = new ArrayList<String>(ConstantsMap.anagraficaMap);
		Map<String, DatoAggiuntivo> valori =  ApplicationUtil.convertiAnagrafica(contatto.getAnagrafica(), valoriValidi);
		List<DatoAggiuntivoAnagrafica> dA = anagraficaDao.getDatiAggiuntiviAnagrafica(contatto.getAnagrafica().getId());
		if(ApplicationUtil.notEmpty(dA)) {
			valori.putAll(ApplicationUtil.convertiDatoAggiuntivoA(dA));
		}
		return ApplicationUtil.replaceInString(testo, valori);
	}
	private ExtendedCriteria trovaCriteriaComunicazione(ComunicazioneCriteria ema) {
		if(ema != null)  ema = (ComunicazioneCriteria) ema.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Comunicazione.class);
		if (ApplicationUtil.isNotBlank(ema.getDestinatario())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true,  ema.getDestinatario(), criteria, "destinatario");
		}
		if (ApplicationUtil.isNotBlank(ema.getTesto())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true,  ema.getTesto(), criteria, "testo", "oggetto");
		}
		if (ApplicationUtil.gtZero(ema.getCodice())) {
			criteria.add(Restrictions.eq("codice", ema.getCodice()));
		}
		ema.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, ema);
	}
	@SuppressWarnings("unchecked")
	public List<Comunicazione> trovaComunicazione(ComunicazioneCriteria ema,int start, int size) {
		ExtendedCriteria criteria = trovaCriteriaComunicazione(ema);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int trovaComunicazioneSize(ComunicazioneCriteria ema) {
		ExtendedCriteria criteria = trovaCriteriaComunicazione(ema);
		return getListSize(criteria);
	}

	private ExtendedCriteria trovaCriteriaSondaggio(SondaggioCriteria ema) {
		if(ema != null)  ema = (SondaggioCriteria) ema.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Sondaggio.class);
		DetachedCriteria criteriaCo = criteria.createCriteria("contatto");
		if (ApplicationUtil.isNotBlank(ema.getOperatore())) {
			DetachedCriteria cOp = criteria.createCriteria("utente");
			ApplicationUtil.operatore(cOp, ema.getOperatore());
		}
		if (ApplicationUtil.isNotBlank(ema.getRicerca_anagrafica())) {
			DetachedCriteria criteriaA = criteriaCo.createCriteria("anagrafica");
			ApplicationUtil.anagrafica(ema.getRicerca_anagraficaV(), criteriaA, ema.getRicerca_anagrafica());
		}
		if (ApplicationUtil.gtZero(ema.getCampagna())) {
			criteriaCo.add(Restrictions.eq("campagna.id", ema.getCampagna()));
		}
		ema.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, ema);
	}
	@SuppressWarnings("unchecked")
	public List<Sondaggio> trovaSondaggio(SondaggioCriteria ema,int start, int size) {
		ExtendedCriteria criteria = trovaCriteriaSondaggio(ema);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int trovaSondaggioSize(SondaggioCriteria ema) {
		ExtendedCriteria criteria = trovaCriteriaSondaggio(ema);
		return getListSize(criteria);
	}

	public void eliminaSondaggio(Sondaggio id) {
		cancellaAggiorna(id);
	}
	public Sondaggio salvaSondaggio(Sondaggio email) {
		return (Sondaggio) salvaAggiorna(email);
	}

	@SuppressWarnings("unchecked")
	public void sincronizza(){
		DetachedCriteria criteriaA = DetachedCriteria.forClass(Template.class);
		criteriaA.add(Restrictions.isNotNull("apiKey"));
		criteriaA.setProjection(Projections.distinct(Projections.property("apiKey")));
		List<String> ls = getList(criteriaA);
		if(ApplicationUtil.notEmpty(ls)) {
			String percorsoContratti = impostazioneDao.getPathBase(ConstantsGlobal.path_pda);
			new SincronizzaThread(percorsoContratti, ls).start();
		}
	}


	public void reInviaPosta(Comunicazione comunicazione, MailSupport ms) {
		comunicazione.setData_invio(ApplicationUtil.date());
		if(ms != null) {
			smtpService.email(ms, comunicazione);
		} else {
			smtpService.email(comunicazione);
		}
	}

	public void inviaNotifica(Notifica notifica, MailSupport ms) {
		if(!notifica.inviabile()) {
			notifica.addMotivo(ConstantsGlobal.NON_INVIATA, ConstantsLanguage.ERRORE_INVIO.stampa());
			notifica = salvaNotifica(notifica);
			return;
		} 
		if(!notifica.valido()) {
			notifica.addMotivo(ConstantsGlobal.NON_INVIATA, ConstantsLanguage.DESTINATARIO_VUOTO.stampa());
			notifica = salvaNotifica(notifica);
			return;
		} 
		notifica.addMotivo(ConstantsGlobal.IN_INVIO, null);
		notifica = salvaNotifica(notifica);
		if(ms != null) {
			smtpService.notifica(ms, notifica);
		} else {
			smtpService.notifica(notifica);
		}
	}
	
	
	public void reInviaNotifica(Notifica notifica, MailSupport ms) {
		notifica.setData_invio(ApplicationUtil.date());
		if(!notifica.inviabile()) {
			notifica.addMotivo(ConstantsGlobal.NON_INVIATA, ConstantsLanguage.ERRORE_INVIO.stampa());
			notifica = salvaNotifica(notifica);
			return;
		} 
		if(!notifica.valido()) {
			notifica.addMotivo(ConstantsGlobal.NON_INVIATA, ConstantsLanguage.DESTINATARIO_VUOTO.stampa());
			notifica = salvaNotifica(notifica);
			return;
		} 
		notifica.addMotivo(ConstantsGlobal.IN_INVIO, null);
		notifica = salvaNotifica(notifica);
		if(ms != null) {
			smtpService.notifica(ms, notifica);
		} else {
			smtpService.notifica(notifica);
		}
	}

	public void test(Smtp smtp){
		String s = ApplicationUtil.trimToNull(configurazioneGenericService.getValore(DefaultsGeneric.KEY_INVIO_MAIL_AMMINISTRAZIONE));
		if(s != null) {
			Date data = ApplicationUtil.date();
			String app = impostazioneDao.getApplicazione();
			String testo =  ConstantsLanguage.TEST.stampa() + ": " + app;
			String oggetto = ApplicationUtil.report(ConstantsLanguage.APPNAME.stampa() , ConstantsGlobal.no_periodo, data);
			MailSupport ms = ApplicationUtil.getMailSupport(true, smtp);
			smtpService.invia(ms, null, s, null, oggetto, testo, null, null);
			ms.close();
		}
	}
	
	public void reInvia(List<?> lst){
		Map<Smtp, Set<Object>> map = ApplicationUtil.creaMappa(lst);
		Iterator<Smtp> mapIter = map.keySet().iterator();
		while(mapIter.hasNext()) {
			Smtp smtp = mapIter.next();
			Set<Object> sobj = map.get(smtp);
			if(ApplicationUtil.notEmpty(sobj)) {
				MailSupport ms = ApplicationUtil.getMailSupport(false, smtp);
				Iterator<Object> sobjIter = sobj.iterator();
				while(sobjIter.hasNext()) {
					Object obj = sobjIter.next();
					if(obj instanceof Email) {
						reInvia((Email) obj, ms);
					}
					if(obj instanceof Rmail) {
						reInvia((Rmail) obj, ms);
					}
					if(obj instanceof Telecom) {
						reInvia((Telecom) obj, false);
					}
				}
				ms.close();
			}
		}
	}
	
	
	public Webhook getWebhook(Long id){
		return (Webhook) get(Webhook.class, id);
	}
	public void eliminaWebhook(Webhook id) {
		cancellaAggiorna(id);
	}
	public Webhook salvaWebhook(Webhook webhook) {
		return (Webhook) salvaAggiorna(webhook);
	}
	private ExtendedCriteria trovaCriteriaWebhook(WebhookCriteria ema) {
		if(ema != null)  ema = (WebhookCriteria) ema.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Webhook.class);
		if (ApplicationUtil.isNotBlank(ema.getAll())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true,  ema.getAll(), criteria, "response", "parameters");
		}
		if (ApplicationUtil.isNotBlank(ema.getName())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true,  ema.getName(), criteria, "name");
		}
		ema.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, ema);
	}
	@SuppressWarnings("unchecked")
	public List<Webhook> trovaWebhook(WebhookCriteria ema,int start, int size) {
		ExtendedCriteria criteria = trovaCriteriaWebhook(ema);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int trovaWebhookSize(WebhookCriteria ema) {
		ExtendedCriteria criteria = trovaCriteriaWebhook(ema);
		return getListSize(criteria);
	}
	
	
	@SuppressWarnings("unchecked")
	public List<WebhookTemplate> trovaWebhookTemplate(WebhookTemplateCriteria ema, int start, int size){
		ExtendedCriteria criteria = getCriteriaWebhookTemplate(ema);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int trovaWebhookTemplateSize(WebhookTemplateCriteria ema) {
		ExtendedCriteria criteria = getCriteriaWebhookTemplate(ema);
		return getListSize(criteria);
	}
	private ExtendedCriteria getCriteriaWebhookTemplate(WebhookTemplateCriteria ema) {
		if(ema != null)  ema = (WebhookTemplateCriteria) ema.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(WebhookTemplate.class);
		if (ApplicationUtil.isNotBlank(ema.getNome())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true,  ema.getNome(), criteria, "nome");
		}
		ema.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, ema); 
	}
	public WebhookTemplate getWebhookTemplate(Long id) {
		return (WebhookTemplate) get(WebhookTemplate.class, id);
	}
	public WebhookTemplate getWebhookTemplateByName(String name) {
		DetachedCriteria criteria = DetachedCriteria.forClass(WebhookTemplate.class);
		criteria.add(Restrictions.eq("nome", name));
		return (WebhookTemplate) getUniqueResult(criteria);
	}
	
	public void eliminaWebhookTemplate(WebhookTemplate id) {
		cancellaAggiorna(id);
	}
	public WebhookTemplate salvaWebhookTemplate(WebhookTemplate webhook) {
		return (WebhookTemplate) salvaAggiorna(webhook);
	}
	
	@SuppressWarnings("unchecked")
	public void sincronizzaRicezione() {
		DetachedCriteria criteria = DetachedCriteria.forClass(Smtp.class);
		criteria.add(Restrictions.gt("ricezione_type", 0));
		List<Smtp> ls =  getList(criteria);
		if(ApplicationUtil.notEmpty(ls)) {
			for (Smtp s : ls) { 
				boolean ssl = (ApplicationUtil.equals(s.getSmtp_encryption(), ConstantsMap.SSL) || ApplicationUtil.equals(s.getSmtp_encryption(), ConstantsMap.TLS));
				try {
					ReceiveMailSession session = null;
					if(ApplicationUtil.eq(s.getRicezione_type(), 1)) {
						ImapServer server  = MailServer.create()
								.host(s.getRicezione_host())
								.port(Integer.valueOf(s.getRicezione_port()))
								.ssl(ssl)
								.auth(s.getSmtp_user(), s.getSmtp_password())
								.buildImapMailServer();
						session = server.createSession();
					} 
					if(ApplicationUtil.eq(s.getRicezione_type(), 2)) {
						Pop3Server server  = MailServer.create()
								.host(s.getRicezione_host())
								.port(Integer.valueOf(s.getRicezione_port()))
								.ssl(ssl)
								.auth(s.getSmtp_user(), s.getSmtp_password())
								.buildPop3MailServer();
						session = server.createSession();
					}

					if(session != null) {
						session.open();
						ReceivedEmail[] emails = session.receiveEmailAndMarkSeen(new EmailFilter().flag(Flags.Flag.SEEN, false));
						if(ApplicationUtil.notEmpty(emails)) {
							for (ReceivedEmail email : emails) {
								Ricezione r = new Ricezione(s, email);
								salvaAggiorna(r);
								r.elabora();
								String rr = null;
								if(s.getRicezione_email() != null) {
									userDao.mailSender(r.getMittente(), r.getDestinatario(), r.getOggetto(), r.getTestoH(), r.getTestoT(), s.getRicezione_email());
								}
								if(s.getRicezione_link() != null) {
									rr = userDao.linkSender(s.getRicezione_link());
								}
								ApplicationUtil.ricezione(s.getRicezione_host(), r.getTestoT(), rr);
							}
						}
						session.close();
					}


				} catch(Exception e) {
					ErrorLog.log(null, getClass(), e);
				}
			}

		}
	}
	
	public Ricezione openRicezione(Long id){
		Ricezione r = (Ricezione) get(Ricezione.class, id);
		if(r != null) {
			r.elabora();
			r.setLetto(true);
			return salvaRicezione(r);
		}
		return new Ricezione();
		
	}
	
	public void closeRicezione(Ricezione r){
		if(r != null) {
			r.setLetto(false);
			salvaRicezione(r);
		}
	}
	
	public void archivioRicezione(Ricezione r, boolean archivio){
		if(r != null) {
			r.setArchiviato(archivio);
			salvaRicezione(r);
		}
	}
	
	public Ricezione getRicezione(Long id){
		return (Ricezione) get(Ricezione.class, id);
	}
	public void eliminaRicezione(Ricezione id) {
		cancellaAggiorna(id);
	}
	public Ricezione salvaRicezione(Ricezione ricezione) {
		return (Ricezione) salvaAggiorna(ricezione);
	}
	private ExtendedCriteria trovaCriteriaRicezione(RicezioneCriteria ema) {
		if(ema != null)  ema = (RicezioneCriteria) ema.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Ricezione.class);
		if (ApplicationUtil.isNotBlank(ema.getMittente())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true,  ema.getMittente(), criteria, "mittente");
		}
		if (ApplicationUtil.isNotBlank(ema.getOggetto())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true,  ema.getOggetto(), criteria, "oggetto");
		}
		
		if (ApplicationUtil.gtZero(ema.getLetto())) {
			if (ApplicationUtil.eq(ema.getLetto(), 1)) {
				criteria.add(Restrictions.eq("letto", Boolean.TRUE));
			} else {
				criteria.add(Restrictions.eq("letto", Boolean.FALSE));
			} 
		}
		
		if (ApplicationUtil.gtZero(ema.getArchiviato())) {
			if (ApplicationUtil.eq(ema.getArchiviato(), 1)) {
				criteria.add(Restrictions.eq("archiviato", Boolean.TRUE));
			} else {
				criteria.add(Restrictions.eq("archiviato", Boolean.FALSE));
			} 
		}
		
		ema.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, ema);
	}
	@SuppressWarnings("unchecked")
	public List<Ricezione> trovaRicezione(RicezioneCriteria ema,int start, int size) {
		ExtendedCriteria criteria = trovaCriteriaRicezione(ema);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int trovaRicezioneSize(RicezioneCriteria ema) {
		ExtendedCriteria criteria = trovaCriteriaRicezione(ema);
		return getListSize(criteria);
	}

}