package it.callite.dao;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.nio.file.DirectoryStream;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.faces.model.SelectItem;

import org.apache.commons.net.ftp.FTPClient;
import org.apache.commons.net.ftp.FTPFile;
import org.hibernate.FetchMode;
import org.hibernate.Hibernate;
import org.hibernate.Session;
import org.hibernate.Transaction;
import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Disjunction;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Projections;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import it.callite.action.DownloadBean;
import it.callite.column.generic.GenericColumn;
import it.callite.constants.ConstantsColumn;
import it.callite.constants.ConstantsEsito;
import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.constants.ConstantsMap;
import it.callite.criteria.ContattoCriteria;
import it.callite.criteria.EsitoChiamataCriteria;
import it.callite.criteria.FileCriteria;
import it.callite.criteria.PDAReportCriteria;
import it.callite.criteria.RegistrazioneCriteria;
import it.callite.criteria.RegistrazioneReportCriteria;
import it.callite.criteria.base.ExtendedCriteria;
import it.callite.dao.generic.GenericDao;
import it.callite.log.ErrorLog;
import it.callite.model.Agente;
import it.callite.model.Anagrafica;
import it.callite.model.AutoContatto;
import it.callite.model.Autodialer;
import it.callite.model.Campagna;
import it.callite.model.Contatto;
import it.callite.model.Contratto;
import it.callite.model.DatoAggiuntivoAnagrafica;
import it.callite.model.Email;
import it.callite.model.EmailTemplate;
import it.callite.model.EsitoArchiviazione;
import it.callite.model.EsitoChiamata;
import it.callite.model.Lista;
import it.callite.model.Notifica;
import it.callite.model.Prenotazione;
import it.callite.model.Rmail;
import it.callite.model.RmailTemplate;
import it.callite.model.Sondaggio;
import it.callite.model.StoriaContatto;
import it.callite.model.Telecom;
import it.callite.model.TelecomTemplate;
import it.callite.model.TemplateImportContatto;
import it.callite.model.Utente;
import it.callite.model.allegato.AllegatoContatto;
import it.callite.model.anagrafica.AnagraficaXML;
import it.callite.model.report.ColonnaModel;
import it.callite.model.report.ColonnaQuery;
import it.callite.model.sip.Cdr;
import it.callite.utility.Allegato;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.ContattoVO;
import it.callite.utility.EsitiMappa;
import it.callite.utility.FileUpload;
import it.callite.utility.LoggerColonna;
import it.callite.utility.MyFile;
import it.callite.utility.Parameter;
import it.callite.utility.StatoIntervallo;
import it.callite.utility.UtilFTP;
import it.callite.utility.importa.ImportModelContatto;
import it.callite.utility.item.CampagnaSI;
import it.callite.utility.item.EsitoArchiviazioneSI;
import it.callite.utility.item.EsitoChiamataSI;
import it.callite.utility.thread.SalvaStoriaContattoThread;
@SuppressWarnings("serial")
@Repository("contattoDao")
public class ContattoDao extends GenericDao {
	@Autowired
	private UserDao userDao;
	@Autowired
	private PannelloDao pannelloDao;
	@Autowired
	private AnagraficaDao anagraficaDao;
	@Autowired
	private CampagnaDao campagnaDao;
	@Autowired
	private ImpostazioneDao impostazioneDao;
	@Autowired
	private OperazioneDao operazioneDao;
	@Autowired
	private PrenotazioneDao prenotazioneDao;
	@Autowired
	private AutodialerDao autodialerDao;
	@Autowired
	private DataDao dataDao;
	
	public void scaricaAllegato(Contatto contatto, DownloadBean downloadBean) {
		List<AllegatoContatto> allegati = getListaAllegati(contatto);
		if(ApplicationUtil.notEmpty(allegati)) {
			String p = ConstantsGlobal.secretZ;
			try {
				String s = impostazioneDao.getPathBaseR(ConstantsGlobal.path_contatto);
				String file_header = ApplicationUtil.reportFile(contatto.getData_creazione(), ConstantsLanguage.CONTATTO.stampa(), contatto.getId(), contatto.getAnagrafica().getDenominazione()) +"." + ConstantsMap.zip;
				OutputStream out = downloadBean.outputStream();
				downloadBean.filename(file_header, allegati);
				List<Allegato> all = new ArrayList<Allegato>();
				Iterator<AllegatoContatto> itA = allegati.iterator();
				while (itA.hasNext()) {
					AllegatoContatto allegato = (AllegatoContatto) itA.next();
					ApplicationUtil.addFile(all, allegato.getId() + "_" +allegato.getNome(), allegato.getContentFile(s), allegato.getData().getTime());
				}
				ApplicationUtil.zip(null, all, out, p);
				downloadBean.close();
			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		}
	}
	
	public void scaricaAllegati(List<Contatto> contatti, Utente utente, DownloadBean downloadBean) {
		List<AllegatoContatto> allegati = getListaAllegati(contatti);
		String p = impostazioneDao.getPasswordZIP();
		try {
			String pc = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.CONTATTO.stampa() +": " + ApplicationUtil.size(contatti), contatti);
			operazioneDao.creaOperazione(ConstantsMap.LOG4,pc ,utente, ConstantsMap.OPERAZIONE_EXPORT_CONTATTO);
			String s = impostazioneDao.getPathBaseR(ConstantsGlobal.path_contatto);
			String file_header =  ApplicationUtil.reportFile(ApplicationUtil.date(), ApplicationUtil.lowerCase(ConstantsLanguage.ALLEGATI.stampa() + "_" + ConstantsLanguage.CONTATTI.stampa())) + "." + ConstantsMap.zip;
			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file_header, contatti, allegati);
			List<Allegato> all = new ArrayList<Allegato>();
			if(ApplicationUtil.notEmpty(allegati)) {
				Iterator<AllegatoContatto> itA = allegati.iterator();
				while (itA.hasNext()) {
					AllegatoContatto allegato = (AllegatoContatto) itA.next();
					ApplicationUtil.addFile(all, allegato.getContatto().getId() + "_" + allegato.getId() + "_" +allegato.getNome(), allegato.getContentFile(s), allegato.getData().getTime());
				}
			}
			if (ApplicationUtil.empty(all)) {
				ApplicationUtil.fileWriter(all, ConstantsLanguage.NO_CONTATTO.stampa(), ConstantsLanguage.NO_CONTATTI.stampa());
			}
			ApplicationUtil.zip(null, all, out, p);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void scaricaAllegato(AllegatoContatto allegato, DownloadBean downloadBean) {
		try {
			String s = impostazioneDao.getPathBaseR(ConstantsGlobal.path_contatto);
			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(allegato.getNome());
			InputStream is = new ByteArrayInputStream(allegato.getContentFile(s));
			ApplicationUtil.copy(is, out);
			downloadBean.close();
			is.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void salvaAllegato(FileUpload fileUpload, Contatto contatto) {
		try {
			List<MyFile> fls = fileUpload.getFiles();
			if(ApplicationUtil.notEmpty(fls)) {
				String s = impostazioneDao.getPathBaseW(ConstantsGlobal.path_contatto);
				Iterator<MyFile> flsIt = fls.iterator();
				while (flsIt.hasNext()) {
					MyFile ff = (MyFile) flsIt.next();
					byte [] ffB = ff.getData();
					AllegatoContatto allegato = new AllegatoContatto();
					allegato.setContatto(contatto);
					allegato.setDimensione(ffB.length);
					allegato.setNome(ApplicationUtil.getNameN(ff.getName()));
					allegato.setContentFile(ffB, s);
					salvaAggiorna(allegato);
				}
			}
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	@SuppressWarnings("unchecked")
	public List<AllegatoContatto> getListaAllegati(Contatto contatto) {
		if(contatto==null)
			return null;
		DetachedCriteria criteria = DetachedCriteria.forClass(AllegatoContatto.class);
		criteria.add(Restrictions.eq("contatto.id", contatto.getId()));
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<AllegatoContatto> getListaAllegati(List<Contatto> contatti) {
		if(ApplicationUtil.empty(contatti))
			return null;
		DetachedCriteria criteria = DetachedCriteria.forClass(AllegatoContatto.class);
		criteria.add(Restrictions.in("contatto.id", ApplicationUtil.prendiIdContatto(contatti)));
		return getList(criteria);
	}
	public void deleteAllegato(AllegatoContatto allegato)  {
		String s = impostazioneDao.getPathBaseR(ConstantsGlobal.path_contatto);
		allegato.delete(s);
		cancellaAggiorna(allegato);
	}
	public void deleteAllegati(Contatto contatto)  {
		List<AllegatoContatto> allegati = getListaAllegati(contatto);
		Iterator<AllegatoContatto> itAn = allegati.iterator();
		while (itAn.hasNext()) {
			AllegatoContatto allegato = (AllegatoContatto) itAn.next();
			deleteAllegato(allegato);
		}
	}
	public int getNumeroContattiByCampagna(Long campagnaId) {
		if(campagnaId == null) return 0;
		DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class);
		criteria.add(Restrictions.eq("campagna.id", campagnaId));
		return getListSize(criteria);
	}
	

	public List<Contatto> impostaConteggioContatto(Utente utente, GenericColumn c, List<Contatto> contatti) {
		if(ApplicationUtil.notEmpty(contatti))  {
			LoggerColonna l = new LoggerColonna(impostazioneDao.getLoggerColonne());
			String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.CONTATTO.stampa() +": " + ApplicationUtil.size(contatti), contatti);
			operazioneDao.creaOperazione(ConstantsMap.LOG4, p ,utente, ConstantsMap.OPERAZIONE_VISUALIZZA_CONTATTO);
			if(impostazioneDao.getTabellaDatiAggiuntivi() && c.getAggiuntiviA())  {
				Iterator<Contatto> objIter = contatti.iterator();
				Session session = openSession();
				while(objIter.hasNext()) {
					Contatto contatto = objIter.next();
					Contatto a = (Contatto) session.get(Contatto.class, contatto.getId());
					Hibernate.initialize(a.getAnagrafica().getDettagli());
					Iterator<DatoAggiuntivoAnagrafica> objd = a.getAnagrafica().getDettagli().iterator();
					while(objd.hasNext()) {
						DatoAggiuntivoAnagrafica da = objd.next();
						contatto.getAnagrafica().addDatoM(da.getCampo_aggiuntivo().getId(), da);
					}
				}
				closeSession(session);
			}
			List<Long> ll = ApplicationUtil.prendiIdContatto(contatti);
			Map<Long, ColonnaModel> map = dataDao.getConteggioColonna(c, ll, Contatto.class, new ColonnaQuery(ConstantsColumn.allegati, "allegati"), new ColonnaQuery(ConstantsColumn.storia, "storia"), new ColonnaQuery(ConstantsColumn.appuntamenti, "appuntamenti"), new ColonnaQuery(ConstantsColumn.contratti, "contratti"));
			Iterator<Contatto> it = contatti.iterator();
			while (it.hasNext()) {
				Contatto o = it.next();
				ColonnaModel lmap = map.get(o.getId());
				if(lmap != null) {
					o.setModel(lmap);
				}
			}
			l.log(c);
		}
		return contatti;
	}
	@SuppressWarnings("unchecked")
	public List<Contatto> ricercaContattoPartner(Utente user, ContattoCriteria cont,  int start, int size) {
		ExtendedCriteria criteria = ricercaContattoPartnerCriteria(user, cont);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), cont.getO_tipo(), cont.getO_colonna(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int ricercaContattoPartnerSize(Utente user, ContattoCriteria cont) {
		ExtendedCriteria criteria = ricercaContattoPartnerCriteria(user, cont);
		return getListSize(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Contatto> ricercaContattoSupervisore(Utente user, ContattoCriteria cont,  int start, int size) {
		ExtendedCriteria criteria = ricercaContattoSupervisoreCriteria(user, cont);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), cont.getO_tipo(), cont.getO_colonna(),  new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int ricercaContattoSupervisoreSize(Utente user, ContattoCriteria cont) {
		ExtendedCriteria criteria = ricercaContattoSupervisoreCriteria(user, cont);
		return getListSize(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Contatto> ricercaContattoOperatore(Utente user, ContattoCriteria cont,  int start, int size) {
		ExtendedCriteria criteria = ricercaContattoOperatoreCriteria(user, cont);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), cont.getO_tipo(), cont.getO_colonna(),  new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int ricercaContattoOperatoreSize(Utente user, ContattoCriteria cont) {
		ExtendedCriteria criteria = ricercaContattoOperatoreCriteria(user, cont);
		return getListSize(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Contatto> ricercaContattoCoordinatore(Utente user, ContattoCriteria cont,  int start, int size) {
		ExtendedCriteria criteria = ricercaContattoCoordinatoreCriteria(user, cont);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), cont.getO_tipo(), cont.getO_colonna(),  new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int ricercaContattoCoordinatoreSize(Utente user, ContattoCriteria cont) {
		ExtendedCriteria criteria = ricercaContattoCoordinatoreCriteria(user, cont);
		return getListSize(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Contatto> ricercaContattoAgente(Utente user, ContattoCriteria cont,  int start, int size) {
		ExtendedCriteria criteria = ricercaContattoAgenteCriteria(user, cont);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), cont.getO_tipo(), cont.getO_colonna(),  new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int ricercaContattoAgenteSize(Utente user, ContattoCriteria cont) {
		ExtendedCriteria criteria = ricercaContattoAgenteCriteria(user, cont);
		return getListSize(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Contatto> ricercaContattoRete(Utente user, ContattoCriteria cont,  int start, int size) {
		ExtendedCriteria criteria = ricercaContattoReteCriteria(user, cont);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), cont.getO_tipo(), cont.getO_colonna(),  new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int ricercaContattoReteSize(Utente user, ContattoCriteria cont) {
		ExtendedCriteria criteria = ricercaContattoReteCriteria(user, cont);
		return getListSize(criteria);
	}
	private ExtendedCriteria ricercaContattoPartnerCriteria(Utente user, ContattoCriteria cont) {
		if(cont != null)  cont = (ContattoCriteria) cont.loadCriteria();
		Boolean filtro = impostazioneDao.getFiltroMultiplo();
		String indirizzo = impostazioneDao.getRicercaIndirizzo();
		DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class);
		DetachedCriteria criteriaA = criteria.createCriteria("anagrafica");
		DetachedCriteria criteriaC = criteria.createCriteria("campagna");
		DetachedCriteria criteriaLI = criteria.createCriteria("lista");
		if(ApplicationUtil.gtZero(cont.getCampagnaId())) {
			criteriaC.add(Restrictions.idEq(cont.getCampagnaId()));
		} else { 
			impostazioneDao.subCriteriaCampagnaPartner(user.getId(), criteriaC, "id");
		}
		if(ApplicationUtil.gtZero(cont.getProprietarioId())) {
			criteria.add(Restrictions.eq("proprietario.id",cont.getProprietarioId()));
		} 
		impostazioneDao.ricercaConsenso(cont, criteriaA);
		impostazioneDao.ricercaIndirizzo(criteriaA, cont, indirizzo);
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaMultipla())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaMultipla(cont, criteriaA);
		} 
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaSingola())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaSingola(cont, criteriaA);
		}
		impostazioneDao.sottoCriteriaStoriaContattoI(false, null, cont, criteria);
		ApplicationUtil.proprieta(filtro, criteria, "tipo", cont.getTipo(), cont.getTipos());
		if (ApplicationUtil.isNotBlank(cont.getRicerca_anagrafica())) {
			ApplicationUtil.anagrafica(cont.getRicerca_anagraficaV(), criteriaA,cont.getRicerca_anagrafica());
		}
		if(ApplicationUtil.gtZero(cont.getAgenteId())) {
			criteria.add(Restrictions.eq("agente.id",cont.getAgenteId()));
		}
		ApplicationUtil.proprieta(false, filtro, criteria, "esitoChiamata.id", cont.getEsitoChiamataId(), cont.getEsitoChiamataIds());
		ApplicationUtil.proprieta(false, filtro, criteria, "esitoArchiviazione.id", cont.getEsitoArchiviazioneId(), cont.getEsitoArchiviazioneIds());
		
		ApplicationUtil.proprieta(false, filtro, criteriaLI, "id", cont.getLista(), cont.getListas());
		
		ApplicationUtil.statoLC(cont.getStatoIntervallo(), criteriaC, criteriaLI);
		if(ApplicationUtil.gtZero(cont.getProprietarioId())) {
			criteria.add(Restrictions.eq("proprietario.id",cont.getProprietarioId()));
		}
		if(ApplicationUtil.isNotBlank(cont.getPositivo()) || ApplicationUtil.isNotBlank(cont.getUtile())) {
			DetachedCriteria archiviazione = criteria.createCriteria("esitoArchiviazione");
			if(ApplicationUtil.isNotBlank(cont.getPositivo())) {
				if(ApplicationUtil.equalsIgnoreCase(cont.getPositivo(), ConstantsLanguage.SI.stampa())) {
					archiviazione.add(Restrictions.eq("positivo", Boolean.TRUE));
				} 
				if(ApplicationUtil.equalsIgnoreCase(cont.getPositivo(), ConstantsLanguage.NO.stampa())) {
					archiviazione.add(Restrictions.eq("positivo", Boolean.FALSE));
				} 
			}
			if(ApplicationUtil.isNotBlank(cont.getUtile())) {
				if(ApplicationUtil.equalsIgnoreCase(cont.getUtile(), ConstantsLanguage.SI.stampa())) {
					archiviazione.add(Restrictions.eq("utile", Boolean.TRUE));
				} 
				if(ApplicationUtil.equalsIgnoreCase(cont.getUtile(), ConstantsLanguage.NO.stampa())) {
					archiviazione.add(Restrictions.eq("utile", Boolean.FALSE));
				} 
			}
		}
		cont.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, cont);
	}
	private ExtendedCriteria ricercaContattoSupervisoreCriteria(Utente user, ContattoCriteria cont) {
		if(cont != null)  cont = (ContattoCriteria) cont.loadCriteria();
		Boolean sd = impostazioneDao.separaDati();
		Boolean filtro = impostazioneDao.getFiltroMultiplo();
		String indirizzo = impostazioneDao.getRicercaIndirizzo();
		DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class);
		DetachedCriteria criteriaA = criteria.createCriteria("anagrafica");
		DetachedCriteria criteriaC = criteria.createCriteria("campagna");
		DetachedCriteria criteriaLI = criteria.createCriteria("lista");
		if(ApplicationUtil.isTrue(sd)) {
			DetachedCriteria criteriaU = criteria.createCriteria("utente", ApplicationUtil.leftJoin());
			impostazioneDao.subCriteriaOperatori(true, user.getId(), criteriaU, "id");
		}
		if(ApplicationUtil.gtZero(cont.getCampagnaId())) {
			criteriaC.add(Restrictions.idEq(cont.getCampagnaId()));
		} else {
			impostazioneDao.subCriteriaCampagnaSupervisore(user.getId(), criteriaC, "id");
		}
		if(ApplicationUtil.gtZero(cont.getProprietarioId())) {
			criteria.add(Restrictions.eq("proprietario.id",cont.getProprietarioId()));
		} else {
			if(ApplicationUtil.isTrue(sd)) {
				DetachedCriteria criteriaP = criteria.createCriteria("proprietario", ApplicationUtil.leftJoin());
				impostazioneDao.subCriteriaOperatori(true, user.getId(), criteriaP, "id");
			}
		}
		impostazioneDao.ricercaConsenso(cont, criteriaA);
		impostazioneDao.ricercaIndirizzo(criteriaA, cont, indirizzo);
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaMultipla())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaMultipla(cont, criteriaA);
		} 
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaSingola())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaSingola(cont, criteriaA);
		}
		impostazioneDao.sottoCriteriaStoriaContattoI(false, cont.getStorico_operatoreId(), cont, criteria);
		ApplicationUtil.proprieta(filtro, criteria, "tipo", cont.getTipo(), cont.getTipos());
		if (ApplicationUtil.isNotBlank(cont.getRicerca_anagrafica())) {
			ApplicationUtil.anagrafica(cont.getRicerca_anagraficaV(), criteriaA,cont.getRicerca_anagrafica());
		}
		if(ApplicationUtil.gtZero(cont.getAgenteId())) {
			criteria.add(Restrictions.eq("agente.id",cont.getAgenteId()));
		}
		ApplicationUtil.proprieta(false, filtro, criteria, "esitoChiamata.id", cont.getEsitoChiamataId(), cont.getEsitoChiamataIds());
		ApplicationUtil.proprieta(false, filtro, criteria, "esitoArchiviazione.id", cont.getEsitoArchiviazioneId(), cont.getEsitoArchiviazioneIds());
		ApplicationUtil.proprieta(false, filtro, criteriaLI, "id", cont.getLista(), cont.getListas());
		ApplicationUtil.statoLC(cont.getStatoIntervallo(), criteriaC, criteriaLI);
		if(ApplicationUtil.isNotBlank(cont.getPositivo()) || ApplicationUtil.isNotBlank(cont.getUtile())) {
			DetachedCriteria archiviazione = criteria.createCriteria("esitoArchiviazione");
			if(ApplicationUtil.isNotBlank(cont.getPositivo())) {
				if(ApplicationUtil.equalsIgnoreCase(cont.getPositivo(), ConstantsLanguage.SI.stampa())) {
					archiviazione.add(Restrictions.eq("positivo", Boolean.TRUE));
				} 
				if(ApplicationUtil.equalsIgnoreCase(cont.getPositivo(), ConstantsLanguage.NO.stampa())) {
					archiviazione.add(Restrictions.eq("positivo", Boolean.FALSE));
				} 
			}
			if(ApplicationUtil.isNotBlank(cont.getUtile())) {
				if(ApplicationUtil.equalsIgnoreCase(cont.getUtile(), ConstantsLanguage.SI.stampa())) {
					archiviazione.add(Restrictions.eq("utile", Boolean.TRUE));
				} 
				if(ApplicationUtil.equalsIgnoreCase(cont.getUtile(), ConstantsLanguage.NO.stampa())) {
					archiviazione.add(Restrictions.eq("utile", Boolean.FALSE));
				} 
			}
		}
		cont.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, cont);
	}
	private ExtendedCriteria ricercaContattoOperatoreCriteria(Utente user, ContattoCriteria cont) {
		if(cont != null)  cont = (ContattoCriteria) cont.loadCriteria();
		Boolean sd = impostazioneDao.separaDati();
		Boolean filtro = impostazioneDao.getFiltroMultiplo();
		String indirizzo = impostazioneDao.getRicercaIndirizzo();
		DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class);
		DetachedCriteria criteriaA = criteria.createCriteria("anagrafica");
		DetachedCriteria criteriaC = criteria.createCriteria("campagna", "campagna");
		DetachedCriteria criteriaLI = criteria.createCriteria("lista");
		DetachedCriteria criteriaOp = criteria.createCriteria("proprietario", "operatore");
		if(ApplicationUtil.isTrue(cont.getAzione())) {
			impostazioneDao.subCriteriaRoleOperatore(user.getId(), criteria, "campagna", "operatore");
			impostazioneDao.subCriteriaOperatore(user.getId(), criteriaOp, "id");
			if(ApplicationUtil.isTrue(sd)) {
				impostazioneDao.subCriteriaCampagnaOperatore(user.getId(), criteriaC, "id");
			}
			if (ApplicationUtil.gtZero(cont.getOperatoreId())) {
				criteriaOp.add(Restrictions.idEq(cont.getOperatoreId()));
			}
		} else {
			criteriaOp.add(Restrictions.idEq(user.getId()));
		}
		impostazioneDao.sottoCriteriaStoriaContattoI(true, null, cont, criteria);
		impostazioneDao.ricercaConsenso(cont, criteriaA);
		impostazioneDao.ricercaIndirizzo(criteriaA, cont, indirizzo);
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaMultipla())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaMultipla(cont, criteriaA);
		} 
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaSingola())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaSingola(cont, criteriaA);
		}
		ApplicationUtil.proprieta(filtro, criteria, "tipo", cont.getTipo(), cont.getTipos());
		if(ApplicationUtil.gtZero(cont.getCampagnaId())) {
			criteriaC.add(Restrictions.idEq(cont.getCampagnaId()));
		} 
		if (ApplicationUtil.isNotBlank(cont.getRicerca_anagrafica())) {
			ApplicationUtil.anagrafica(cont.getRicerca_anagraficaV(), criteriaA,cont.getRicerca_anagrafica());
		}
		if(ApplicationUtil.gtZero(cont.getAgenteId())) {
			criteria.add(Restrictions.eq("agente.id",cont.getAgenteId()));
		}
		ApplicationUtil.proprieta(false, filtro, criteria, "esitoChiamata.id", cont.getEsitoChiamataId(), cont.getEsitoChiamataIds());
		ApplicationUtil.proprieta(false, filtro, criteria, "esitoArchiviazione.id", cont.getEsitoArchiviazioneId(), cont.getEsitoArchiviazioneIds());
		ApplicationUtil.proprieta(false, filtro, criteriaLI, "id", cont.getLista(), cont.getListas());
		ApplicationUtil.statoLC(cont.getStatoIntervallo(), criteriaC, criteriaLI);
		if(ApplicationUtil.isNotBlank(cont.getPositivo()) || ApplicationUtil.isNotBlank(cont.getUtile())) {
			DetachedCriteria archiviazione = criteria.createCriteria("esitoArchiviazione");
			if(ApplicationUtil.isNotBlank(cont.getPositivo())) {
				if(ApplicationUtil.equalsIgnoreCase(cont.getPositivo(), ConstantsLanguage.SI.stampa())) {
					archiviazione.add(Restrictions.eq("positivo", Boolean.TRUE));
				} 
				if(ApplicationUtil.equalsIgnoreCase(cont.getPositivo(), ConstantsLanguage.NO.stampa())) {
					archiviazione.add(Restrictions.eq("positivo", Boolean.FALSE));
				} 
			}
			if(ApplicationUtil.isNotBlank(cont.getUtile())) {
				if(ApplicationUtil.equalsIgnoreCase(cont.getUtile(), ConstantsLanguage.SI.stampa())) {
					archiviazione.add(Restrictions.eq("utile", Boolean.TRUE));
				} 
				if(ApplicationUtil.equalsIgnoreCase(cont.getUtile(), ConstantsLanguage.NO.stampa())) {
					archiviazione.add(Restrictions.eq("utile", Boolean.FALSE));
				} 
			}
		}
		cont.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, cont);
	}
	private ExtendedCriteria ricercaContattoCoordinatoreCriteria(Utente user, ContattoCriteria cont) {
		if(cont != null)  cont = (ContattoCriteria) cont.loadCriteria();
		Boolean sd = impostazioneDao.separaDati();
		Boolean filtro = impostazioneDao.getFiltroMultiplo();
		String indirizzo = impostazioneDao.getRicercaIndirizzo();
		DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class);
		DetachedCriteria criteriaA = criteria.createCriteria("anagrafica");
		DetachedCriteria criteriaC = criteria.createCriteria("campagna", "campagna");
		DetachedCriteria criteriaAg = criteria.createCriteria("agente", "agente");
		DetachedCriteria criteriaLI = criteria.createCriteria("lista");
		impostazioneDao.subCriteriaRoleCoordinatore(user.getId(), criteria, "campagna", "agente");
		if(ApplicationUtil.gtZero(cont.getAgenteId())) {
			criteriaAg.add(Restrictions.idEq(cont.getAgenteId()));
		} else {
			impostazioneDao.subCriteriaCoordinatore(user.getId(), criteriaAg, "id");
		}
		if(ApplicationUtil.gtZero(cont.getCampagnaId())) {
			criteriaC.add(Restrictions.idEq(cont.getCampagnaId()));
		} else {
			if(ApplicationUtil.isTrue(sd)) {
				impostazioneDao.subCriteriaCoordinatoreCampagna(user.getId(), criteriaC, "id");
			}
		}	
		impostazioneDao.ricercaConsenso(cont, criteriaA);
		impostazioneDao.ricercaIndirizzo(criteriaA, cont, indirizzo);
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaMultipla())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaMultipla(cont, criteriaA);
		} 
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaSingola())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaSingola(cont, criteriaA);
		}
		impostazioneDao.sottoCriteriaStoriaContattoS(false, cont.getStorico_operatore(), cont, criteria);
		ApplicationUtil.proprieta(filtro, criteria, "tipo", cont.getTipo(), cont.getTipos());
		if (ApplicationUtil.isNotBlank(cont.getRicerca_anagrafica())) {
			ApplicationUtil.anagrafica(cont.getRicerca_anagraficaV(), criteriaA,cont.getRicerca_anagrafica());
		}
		ApplicationUtil.proprieta(false, filtro, criteria, "esitoChiamata.id", cont.getEsitoChiamataId(), cont.getEsitoChiamataIds());
		ApplicationUtil.proprieta(false, filtro, criteria, "esitoArchiviazione.id", cont.getEsitoArchiviazioneId(), cont.getEsitoArchiviazioneIds());
		ApplicationUtil.proprieta(false, filtro, criteriaLI, "id", cont.getLista(), cont.getListas());
		ApplicationUtil.statoLC(cont.getStatoIntervallo(), criteriaC, criteriaLI);
		if(ApplicationUtil.isNotBlank(cont.getPositivo()) || ApplicationUtil.isNotBlank(cont.getUtile())) {
			DetachedCriteria archiviazione = criteria.createCriteria("esitoArchiviazione");
			if(ApplicationUtil.isNotBlank(cont.getPositivo())) {
				if(ApplicationUtil.equalsIgnoreCase(cont.getPositivo(), ConstantsLanguage.SI.stampa())) {
					archiviazione.add(Restrictions.eq("positivo", Boolean.TRUE));
				} 
				if(ApplicationUtil.equalsIgnoreCase(cont.getPositivo(), ConstantsLanguage.NO.stampa())) {
					archiviazione.add(Restrictions.eq("positivo", Boolean.FALSE));
				} 
			}
			if(ApplicationUtil.isNotBlank(cont.getUtile())) {
				if(ApplicationUtil.equalsIgnoreCase(cont.getUtile(), ConstantsLanguage.SI.stampa())) {
					archiviazione.add(Restrictions.eq("utile", Boolean.TRUE));
				} 
				if(ApplicationUtil.equalsIgnoreCase(cont.getUtile(), ConstantsLanguage.NO.stampa())) {
					archiviazione.add(Restrictions.eq("utile", Boolean.FALSE));
				} 
			}
		}
		if (ApplicationUtil.isNotBlank(cont.getProprietario())) {
			DetachedCriteria cOp = criteria.createCriteria("proprietario");
			ApplicationUtil.operatore(cOp, cont.getProprietario());
		}
		cont.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, cont);
	}
	private ExtendedCriteria ricercaContattoAgenteCriteria(Utente user, ContattoCriteria cont) {
		if(cont != null)  cont = (ContattoCriteria) cont.loadCriteria();
		Boolean filtro = impostazioneDao.getFiltroMultiplo();
		String indirizzo = impostazioneDao.getRicercaIndirizzo();
		DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class);
		DetachedCriteria criteriaA = criteria.createCriteria("anagrafica");
		DetachedCriteria criteriaC = criteria.createCriteria("campagna");
		DetachedCriteria criteriaLI = criteria.createCriteria("lista");
		if(ApplicationUtil.gtZero(cont.getAgenteId())) {
			criteria.add(Restrictions.eq("agente.id",cont.getAgenteId()));
		} else {
			impostazioneDao.subCriteriaAgente(user.getId(), criteria, "agente.id");
		}
		impostazioneDao.ricercaConsenso(cont, criteriaA);
		impostazioneDao.ricercaIndirizzo(criteriaA, cont, indirizzo);
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaMultipla())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaMultipla(cont, criteriaA);
		} 
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaSingola())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaSingola(cont, criteriaA);
		}
		ApplicationUtil.proprieta(filtro, criteria, "tipo", cont.getTipo(), cont.getTipos());
		if(ApplicationUtil.gtZero(cont.getCampagnaId())) {
			criteriaC.add(Restrictions.idEq(cont.getCampagnaId()));
		}		
		if (ApplicationUtil.isNotBlank(cont.getRicerca_anagrafica())) {
			ApplicationUtil.anagrafica(cont.getRicerca_anagraficaV(), criteriaA,cont.getRicerca_anagrafica());
		}
		ApplicationUtil.proprieta(false, filtro, criteria, "esitoChiamata.id", cont.getEsitoChiamataId(), cont.getEsitoChiamataIds());
		ApplicationUtil.proprieta(false, filtro, criteria, "esitoArchiviazione.id", cont.getEsitoArchiviazioneId(), cont.getEsitoArchiviazioneIds());
		ApplicationUtil.proprieta(false, filtro, criteriaLI, "id", cont.getLista(), cont.getListas());
		ApplicationUtil.statoLC(cont.getStatoIntervallo(), criteriaC, criteriaLI);
		if(ApplicationUtil.gtZero(cont.getProprietarioId())) {
			criteria.add(Restrictions.eq("proprietario.id",cont.getProprietarioId()));
		}
		if(ApplicationUtil.isNotBlank(cont.getPositivo()) || ApplicationUtil.isNotBlank(cont.getUtile())) {
			DetachedCriteria archiviazione = criteria.createCriteria("esitoArchiviazione");
			if(ApplicationUtil.isNotBlank(cont.getPositivo())) {
				if(ApplicationUtil.equalsIgnoreCase(cont.getPositivo(), ConstantsLanguage.SI.stampa())) {
					archiviazione.add(Restrictions.eq("positivo", Boolean.TRUE));
				} 
				if(ApplicationUtil.equalsIgnoreCase(cont.getPositivo(), ConstantsLanguage.NO.stampa())) {
					archiviazione.add(Restrictions.eq("positivo", Boolean.FALSE));
				} 
			}
			if(ApplicationUtil.isNotBlank(cont.getUtile())) {
				if(ApplicationUtil.equalsIgnoreCase(cont.getUtile(), ConstantsLanguage.SI.stampa())) {
					archiviazione.add(Restrictions.eq("utile", Boolean.TRUE));
				} 
				if(ApplicationUtil.equalsIgnoreCase(cont.getUtile(), ConstantsLanguage.NO.stampa())) {
					archiviazione.add(Restrictions.eq("utile", Boolean.FALSE));
				} 
			}
		}
		cont.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, cont);
	}
	private ExtendedCriteria ricercaContattoReteCriteria(Utente user, ContattoCriteria cont) {
		if(cont != null)  cont = (ContattoCriteria) cont.loadCriteria();
		Integer max = userDao.getMaxLivelloRete();
		Boolean filtro = impostazioneDao.getFiltroMultiplo();
		String indirizzo = impostazioneDao.getRicercaIndirizzo();
		DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class);
		DetachedCriteria criteriaA = criteria.createCriteria("anagrafica");
		DetachedCriteria criteriaC = criteria.createCriteria("campagna");
		DetachedCriteria criteriaLI = criteria.createCriteria("lista");
		DetachedCriteria criteriaAd = criteria.createCriteria("agente");
		if (ApplicationUtil.gtZero(cont.getRete())) {
			if(ApplicationUtil.eq(cont.getRete(), 1)) {
				if (ApplicationUtil.gtZero(cont.getAgenteId())) {
					criteriaAd.createCriteria("agente_padre").add(Restrictions.idEq(cont.getAgenteId()));
				}
			}
			if(ApplicationUtil.eq(cont.getRete(), 1)) {
				if (ApplicationUtil.gtZero(cont.getAgenteId())) {
					impostazioneDao.subCriteriaAgentiSottoRete(cont.getAgenteId(), true, max, criteriaAd, "id");
				}
			}
			if(ApplicationUtil.eq(cont.getRete(), 1)) {
				if (ApplicationUtil.gtZero(cont.getAgenteId())) {
					impostazioneDao.subCriteriaAgentiSottoRete(cont.getAgenteId(), true, max, criteriaAd, "id");
					criteriaAd.add(Restrictions.ne("id", cont.getAgenteId()));
				}
			}
			if(ApplicationUtil.eq(cont.getRete(), 1)) {
				if (ApplicationUtil.gtZero(cont.getAgenteId())) {
					criteriaAd.add(Restrictions.idEq(cont.getAgenteId()));
				}
			}
		} else {
			impostazioneDao.subCriteriaAgentiSottoReteByUser(user.getId(),0,  max, criteriaAd, "id");
		}
		impostazioneDao.ricercaConsenso(cont, criteriaA);
		impostazioneDao.ricercaIndirizzo(criteriaA, cont, indirizzo);
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaMultipla())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaMultipla(cont, criteriaA);
		} 
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaSingola())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaSingola(cont, criteriaA);
		}
		if(ApplicationUtil.gtZero(cont.getCampagnaId())) {
			criteriaC.add(Restrictions.idEq(cont.getCampagnaId()));
		}		
		if (ApplicationUtil.isNotBlank(cont.getRicerca_anagrafica())) {
			ApplicationUtil.anagrafica(cont.getRicerca_anagraficaV(), criteriaA,cont.getRicerca_anagrafica());
		}
		ApplicationUtil.proprieta(filtro, criteria, "tipo", cont.getTipo(), cont.getTipos());
		ApplicationUtil.proprieta(false, filtro, criteria, "esitoChiamata.id", cont.getEsitoChiamataId(), cont.getEsitoChiamataIds());
		ApplicationUtil.proprieta(false, filtro, criteria, "esitoArchiviazione.id", cont.getEsitoArchiviazioneId(), cont.getEsitoArchiviazioneIds());
		ApplicationUtil.proprieta(false, filtro, criteriaLI, "id", cont.getLista(), cont.getListas());
		ApplicationUtil.statoLC(cont.getStatoIntervallo(), criteriaC, criteriaLI);
		if(ApplicationUtil.gtZero(cont.getProprietarioId())) {
			criteria.add(Restrictions.eq("proprietario.id",cont.getProprietarioId()));
		}
		if(ApplicationUtil.isNotBlank(cont.getPositivo()) || ApplicationUtil.isNotBlank(cont.getUtile())) {
			DetachedCriteria archiviazione = criteria.createCriteria("esitoArchiviazione");
			if(ApplicationUtil.isNotBlank(cont.getPositivo())) {
				if(ApplicationUtil.equalsIgnoreCase(cont.getPositivo(), ConstantsLanguage.SI.stampa())) {
					archiviazione.add(Restrictions.eq("positivo", Boolean.TRUE));
				} 
				if(ApplicationUtil.equalsIgnoreCase(cont.getPositivo(), ConstantsLanguage.NO.stampa())) {
					archiviazione.add(Restrictions.eq("positivo", Boolean.FALSE));
				} 
			}
			if(ApplicationUtil.isNotBlank(cont.getUtile())) {
				if(ApplicationUtil.equalsIgnoreCase(cont.getUtile(), ConstantsLanguage.SI.stampa())) {
					archiviazione.add(Restrictions.eq("utile", Boolean.TRUE));
				} 
				if(ApplicationUtil.equalsIgnoreCase(cont.getUtile(), ConstantsLanguage.NO.stampa())) {
					archiviazione.add(Restrictions.eq("utile", Boolean.FALSE));
				} 
			}
		}
		cont.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, cont);
	}
	private ExtendedCriteria ricercaContattoManagerCriteria(ContattoCriteria cont) {
		if(cont != null)  cont = (ContattoCriteria) cont.loadCriteria();
		Boolean filtro = impostazioneDao.getFiltroMultiplo();
		String indirizzo = impostazioneDao.getRicercaIndirizzo();
		DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class);
		DetachedCriteria criteriaA = criteria.createCriteria("anagrafica");
		DetachedCriteria criteriaC = criteria.createCriteria("campagna");
		DetachedCriteria criteriaLI = criteria.createCriteria("lista");
		impostazioneDao.ricercaConsenso(cont, criteriaA);
		impostazioneDao.ricercaIndirizzo(criteriaA, cont, indirizzo);
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaMultipla())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaMultipla(cont, criteriaA);
		} 
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaSingola())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaSingola(cont, criteriaA);
		}
		impostazioneDao.sottoCriteriaStoriaContattoI(false, cont.getStorico_operatoreId(), cont, criteria);
		ApplicationUtil.proprieta(filtro, criteria, "tipo", cont.getTipo(), cont.getTipos());
		if(ApplicationUtil.gtZero(cont.getCampagnaId())) {
			criteriaC.add(Restrictions.idEq(cont.getCampagnaId()));
		}
		if (ApplicationUtil.isNotBlank(cont.getRicerca_anagrafica())) {
			ApplicationUtil.anagrafica(cont.getRicerca_anagraficaV(), criteriaA,cont.getRicerca_anagrafica());
		}
		if(ApplicationUtil.gtZero(cont.getAgenteId())) {
			criteria.add(Restrictions.eq("agente.id",cont.getAgenteId()));
		}
		ApplicationUtil.proprieta(false, filtro, criteria, "esitoChiamata.id", cont.getEsitoChiamataId(), cont.getEsitoChiamataIds());
		ApplicationUtil.proprieta(false, filtro, criteria, "esitoArchiviazione.id", cont.getEsitoArchiviazioneId(), cont.getEsitoArchiviazioneIds());
		ApplicationUtil.proprieta(false, filtro, criteriaLI, "id", cont.getLista(), cont.getListas());
		ApplicationUtil.statoLC(cont.getStatoIntervallo(), criteriaC, criteriaLI);
		if(ApplicationUtil.gtZero(cont.getProprietarioId())) {
			criteria.add(Restrictions.eq("proprietario.id",cont.getProprietarioId()));
		}
		if (ApplicationUtil.gtZero(cont.getOperatoreId())) {
			criteria.add(Restrictions.eq("utente.id", cont.getOperatoreId()));
		}
		if(ApplicationUtil.isNotBlank(cont.getPositivo()) || ApplicationUtil.isNotBlank(cont.getUtile())) {
			DetachedCriteria archiviazione = criteria.createCriteria("esitoArchiviazione");
			if(ApplicationUtil.isNotBlank(cont.getPositivo())) {
				if(ApplicationUtil.equalsIgnoreCase(cont.getPositivo(), ConstantsLanguage.SI.stampa())) {
					archiviazione.add(Restrictions.eq("positivo", Boolean.TRUE));
				} 
				if(ApplicationUtil.equalsIgnoreCase(cont.getPositivo(), ConstantsLanguage.NO.stampa())) {
					archiviazione.add(Restrictions.eq("positivo", Boolean.FALSE));
				} 
			}
			if(ApplicationUtil.isNotBlank(cont.getUtile())) {
				if(ApplicationUtil.equalsIgnoreCase(cont.getUtile(), ConstantsLanguage.SI.stampa())) {
					archiviazione.add(Restrictions.eq("utile", Boolean.TRUE));
				} 
				if(ApplicationUtil.equalsIgnoreCase(cont.getUtile(), ConstantsLanguage.NO.stampa())) {
					archiviazione.add(Restrictions.eq("utile", Boolean.FALSE));
				} 
			}
		}
		cont.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, cont);
	}
	@SuppressWarnings("unchecked")
	public List<EsitoArchiviazione> getAllEsitiArchiviazione() {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoArchiviazione.class);
		ApplicationUtil.orderEsito(criteria);
		return getList(criteria);
	}
	public void cancella(EsitoChiamata esito) {
		cancellaAggiorna(esito);
	}
	@SuppressWarnings("unchecked")
	public EsitiMappa getAllEsitiChiamata(Contatto contatto) {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoChiamata.class);
		criteria.add(Restrictions.eq("esitoNormale",  Boolean.TRUE));
		criteria.add(Restrictions.isNotNull("richiamare"));
		List<Long> keys = new ArrayList<Long>(ConstantsEsito.DEFAULTS.keySet());
		criteria.add(Restrictions.not(Restrictions.in("id", keys)));
		ApplicationUtil.orderEsito(criteria);
		List<EsitoChiamata> ls = getList(criteria);
		return new EsitiMappa(ls, contatto, true);
	}
	@SuppressWarnings("unchecked")
	public List<EsitoChiamata> getAllEsitiChiamataA() {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoChiamata.class);
		//criteria.add(Restrictions.eq("esitoAutodialer",  Boolean.TRUE));
		criteria.add(Restrictions.isNotNull("richiamare"));
		List<Long> keys = new ArrayList<Long>(ConstantsEsito.DEFAULTS.keySet());
		criteria.add(Restrictions.not(Restrictions.in("id", keys)));
		ApplicationUtil.orderEsito(criteria);
		return getList(criteria);
	}
	public EsitiMappa getEsitiAutodialer(Contatto contatto, List<EsitoChiamata> esiti, Boolean normale) {
		return new EsitiMappa(esiti, contatto, ApplicationUtil.isTrue(normale));
	}
	public List<SelectItem>  getAllEsitiArchiviazioneProvvisorioSI() {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoArchiviazione.class);
		List<EsitoArchiviazioneSI> lis = getEsitoArchiviazioneSI(criteria);
		return impostazioneDao.getEsitoArchiviazione(lis, false, null);
	}
	public List<SelectItem>  getAllEsitiChiamataProvvisorioSI() {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoChiamata.class);
		List<Long> keys = new ArrayList<Long>(ConstantsEsito.DEFAULTS.keySet());
		criteria.add(Restrictions.not(Restrictions.in("id", keys)));
		List<EsitoChiamataSI> lis = getEsitoChiamataSI(criteria);
		return impostazioneDao.getEsitoChiamata(lis, false, null);
	}
	@SuppressWarnings("unchecked")
	public List<Campagna> getAllCampagne() {
		DetachedCriteria criteria =  DetachedCriteria.forClass(Campagna.class);
		ApplicationUtil.orderCL(criteria);
		return getList(criteria);
	}
	public void elimina(Contatto contatto) {
		cancellaAggiorna(contatto);
	}
	public void eliminaContatto(Utente utente, Contatto contatto) {
		String p = ApplicationUtil.asString( ConstantsLanguage.CONTATTO.stampa(), contatto);
		operazioneDao.creaOperazione(ConstantsMap.LOG3, p, utente, ConstantsMap.OPERAZIONE_ELIMINA_CONTATTO);
		cancellaAggiorna(contatto);
	}
	public List<SelectItem> getAllCampagneMasterSI() {
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		criteria.add(Restrictions.eq("prenotabile_interno", Boolean.TRUE));
		List<CampagnaSI> lis = getCampagnaSI(criteria);
		return impostazioneDao.getCampagna(lis, false, false, false, null);
	}
	public List<SelectItem> getAllCampagneSI() {
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		List<CampagnaSI> lis = getCampagnaSI(criteria);
		return impostazioneDao.getCampagna(lis, false, false, false, null);
	}
	public List<SelectItem> getAllCampagneSI(Lista lista, Campagna campagna) {
		List<CampagnaSI> lis = new ArrayList<CampagnaSI>();
		DetachedCriteria criteria1 = DetachedCriteria.forClass(Campagna.class);
		lis.addAll(getCampagnaSI(criteria1));
		DetachedCriteria criteria2 = DetachedCriteria.forClass(Campagna.class);
		if(ApplicationUtil.gtZero(lista.getId())) {
			criteria2.createCriteria("liste").add(Restrictions.eq("id", lista.getId()));
		}
		lis.removeAll(getCampagnaSI(criteria2));
		if(ApplicationUtil.gtZero(campagna.getId())) {
			lis.remove(new CampagnaSI(campagna));
		}
		return impostazioneDao.getCampagna(lis, false, false, false, null);
	}
	public List<SelectItem> getCampagnePartnerSI(Long utenteId, Lista lista, Campagna campagna) {
		List<CampagnaSI> lis = new ArrayList<CampagnaSI>();
		DetachedCriteria criteria1 = DetachedCriteria.forClass(Campagna.class);
		criteria1.createCriteria("partner").add(Restrictions.idEq(utenteId));
		lis.addAll(getCampagnaSI(criteria1));
		DetachedCriteria criteria2 = DetachedCriteria.forClass(Campagna.class);
		if(ApplicationUtil.gtZero(lista.getId())) {
			criteria2.createCriteria("liste").add(Restrictions.eq("id", lista.getId()));
		}
		lis.removeAll(getCampagnaSI(criteria2));
		if(ApplicationUtil.gtZero(campagna.getId())) {
			lis.remove(new CampagnaSI(campagna));
		}
		return impostazioneDao.getCampagna(lis, false, false, false, null);
	}
	public List<SelectItem> getCampagneSupervisoreSI(Long utenteId, Lista lista, Campagna campagna) {
		List<CampagnaSI> lis = new ArrayList<CampagnaSI>();
		DetachedCriteria criteria1 = DetachedCriteria.forClass(Campagna.class);
		criteria1.createCriteria("team").createCriteria("supervisore").add(Restrictions.idEq(utenteId));
		lis.addAll(getCampagnaSI(criteria1));
		DetachedCriteria criteria2 = DetachedCriteria.forClass(Campagna.class);
		if(ApplicationUtil.gtZero(lista.getId())) {
			criteria2.createCriteria("liste").add(Restrictions.eq("id", lista.getId()));
		}
		lis.removeAll(getCampagnaSI(criteria2));
		if(ApplicationUtil.gtZero(campagna.getId())) {
			lis.remove(new CampagnaSI(campagna));
		}
		return impostazioneDao.getCampagna(lis, false, false, false, null);
	} 
	public List<SelectItem> getCampagneSupervisoreSI(Long utenteId, StatoIntervallo stato) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		ApplicationUtil.statoLC(stato, criteria, null);
		criteria.createCriteria("team").createCriteria("supervisore").add(Restrictions.idEq(utenteId));
		List<CampagnaSI> lis = getCampagnaSI(criteria);
		return impostazioneDao.getCampagna(lis, true, false, false, null);
	}
	public List<SelectItem> getCampagneSupervisoreSI(Long utenteId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		criteria.createCriteria("team").createCriteria("supervisore").add(Restrictions.idEq(utenteId));
		List<CampagnaSI> lis = getCampagnaSI(criteria);
		return impostazioneDao.getCampagna(lis, false, false, false, null);
	} 
	public List<SelectItem> getCampagneSupervisoreMasterSI(Long utenteId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		criteria.add(Restrictions.eq("prenotabile_interno", Boolean.TRUE));
		criteria.createCriteria("team").createCriteria("supervisore").add(Restrictions.idEq(utenteId));
		List<CampagnaSI> lis = getCampagnaSI(criteria);
		return impostazioneDao.getCampagna(lis, false, false, false, null);
	}
	public List<SelectItem> getCampagneSupervisoreMasterSI(Long utenteId, boolean semaforo) {
		DetachedCriteria criteria = campagnaDao.trovaCampagneAttiveCriteria(semaforo, utenteId);
		criteria.add(Restrictions.eq("prenotabile_interno", Boolean.TRUE));
		if(!semaforo) {
			criteria.add(Restrictions.eq("inserisci_dati", Boolean.TRUE));
		}
		List<CampagnaSI> lis = getCampagnaSI(criteria);
		return impostazioneDao.getCampagna(lis, true, false, false, null);
	}
	public List<SelectItem> getCampagneOperatoriRicercaSI(Long userId, boolean ricerca) {
		if(ricerca) {
			Boolean ts = impostazioneDao.getEstrazioneStato();
			DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
			criteria.createCriteria("team").createCriteria("user").add(Restrictions.idEq(userId));
			ApplicationUtil.criteriaStato(criteria, false, ApplicationUtil.isTrue(ts));
			List<CampagnaSI> lis = getCampagnaSI(criteria);
			return impostazioneDao.getCampagna(lis, true, false, false, null);
		}
		return  new ArrayList<SelectItem>();
	}
	public List<SelectItem> getCampagneCoordinatoreMasterSI(Long utenteId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		criteria.add(Restrictions.eq("prenotabile_esterno", Boolean.TRUE));
		criteria.createCriteria("zone").createCriteria("supervisori").add(Restrictions.idEq(utenteId));
		List<CampagnaSI> lis = getCampagnaSI(criteria);
		return impostazioneDao.getCampagna(lis, false, false, false, null);
	}
	public List<SelectItem> getCampagneCoordinatoreSI(Long utenteId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		criteria.createCriteria("zone").createCriteria("supervisori").add(Restrictions.idEq(utenteId));
		List<CampagnaSI> lis = getCampagnaSI(criteria);
		return impostazioneDao.getCampagna(lis, false, false, false, null);
	}
	public List<SelectItem> getCampagneOperatorePrenotazioneSI(Long utenteId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		criteria.add(Restrictions.eq("prenotabile_interno", Boolean.TRUE));
		criteria.createCriteria("team").createCriteria("user").add(Restrictions.idEq(utenteId));
		List<CampagnaSI> lis = getCampagnaSI(criteria);
		return impostazioneDao.getCampagna(lis, false, false, false, null);
	}
	public List<SelectItem> getCampagneOperatoreContattoSI(Long utenteId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		criteria.add(Restrictions.eq("prenotabile_interno", Boolean.TRUE));
		criteria.add(Restrictions.eq("inserisci_dati", Boolean.TRUE));
		criteria.createCriteria("team").createCriteria("user").add(Restrictions.idEq(utenteId));
		List<CampagnaSI> lis = getCampagnaSI(criteria);
		return impostazioneDao.getCampagna(lis, false, false, false, null);
	}
	public List<SelectItem> getCampagneAgenteMasterSI(Long utenteId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		criteria.add(Restrictions.eq("prenotabile_esterno", Boolean.TRUE));
		criteria.createCriteria("zone").createCriteria("agente").createCriteria("user").add(Restrictions.idEq(utenteId));
		List<CampagnaSI> lis = getCampagnaSI(criteria);
		return impostazioneDao.getCampagna(lis, false, false, false, null);
	}
	public List<SelectItem> getCampagneReteMasterSI(Long utenteId) {
		Integer max = userDao.getMaxLivelloRete();
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		criteria.add(Restrictions.eq("prenotabile_esterno", Boolean.TRUE));
		DetachedCriteria criteriaAd = criteria.createCriteria("zone").createCriteria("agente");
		impostazioneDao.subCriteriaAgentiSottoReteByUser(utenteId, 0, max, criteriaAd, "id");
		List<CampagnaSI> lis = getCampagnaSI(criteria);
		return impostazioneDao.getCampagna(lis, false, false, false, null);
	}
	public List<SelectItem> getCampagneCoordinatoreSI(Long utenteId, StatoIntervallo stato) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		ApplicationUtil.statoLC(stato, criteria, null);
		criteria.createCriteria("zone").createCriteria("supervisori").add(Restrictions.idEq(utenteId));
		List<CampagnaSI> lis = getCampagnaSI(criteria);
		return impostazioneDao.getCampagna(lis, true, false, false, null);
	}
	public List<SelectItem> getCampagneOperatoreSI(Long utenteId, StatoIntervallo stato) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		ApplicationUtil.statoLC(stato, criteria, null);
		criteria.createCriteria("team").createCriteria("user").add(Restrictions.idEq(utenteId));
		List<CampagnaSI> lis = getCampagnaSI(criteria);
		return impostazioneDao.getCampagna(lis, true, false, false, null);
	}
	public List<SelectItem> getCampagneOperatoreSI(Long utenteId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		criteria.createCriteria("team").createCriteria("user").add(Restrictions.idEq(utenteId));
		List<CampagnaSI> lis = getCampagnaSI(criteria);
		return impostazioneDao.getCampagna(lis, true, false, false, null);
	}
	public List<SelectItem> getCampagneReteSI(Long utenteId, StatoIntervallo stato) {
		Integer max = userDao.getMaxLivelloRete();
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		ApplicationUtil.statoLC(stato, criteria, null);
		DetachedCriteria c = criteria.createCriteria("zone").createCriteria("agente");
		impostazioneDao.subCriteriaAgentiSottoReteByUser(utenteId, 0, max, c, "id");
		List<CampagnaSI> lis = getCampagnaSI(criteria);
		return impostazioneDao.getCampagna(lis, true, false, false, null);
	}
	public List<SelectItem> getCampagneReteSI(Long utenteId) {
		Integer max = userDao.getMaxLivelloRete();
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		DetachedCriteria c = criteria.createCriteria("zone").createCriteria("agente");
		impostazioneDao.subCriteriaAgentiSottoReteByUser(utenteId, 0, max, c, "id");
		List<CampagnaSI> lis = getCampagnaSI(criteria);
		return impostazioneDao.getCampagna(lis, true, false, false, null);
	}
	public List<SelectItem> getCampagneAgenteSI(Long utenteId, StatoIntervallo stato) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		ApplicationUtil.statoLC(stato, criteria, null);
		criteria.createCriteria("zone").createCriteria("agente").createCriteria("user").add(Restrictions.idEq(utenteId));
		List<CampagnaSI> lis = getCampagnaSI(criteria);
		return impostazioneDao.getCampagna(lis, true, false, false, null);
	}
	public List<SelectItem> getCampagneAgenteSI(Long utenteId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		criteria.createCriteria("zone").createCriteria("agente").createCriteria("user").add(Restrictions.idEq(utenteId));
		List<CampagnaSI> lis = getCampagnaSI(criteria);
		return impostazioneDao.getCampagna(lis, true, false, false, null);
	}
	@SuppressWarnings("unchecked")
	public List<Contatto> ricercaContattoManager(ContattoCriteria cont, int start, int size) {
		ExtendedCriteria criteria = ricercaContattoManagerCriteria(cont);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), cont.getO_tipo(), cont.getO_colonna(),  new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int ricercaContattoManagerSize(ContattoCriteria cont) {
		ExtendedCriteria criteria = ricercaContattoManagerCriteria(cont);
		return getListSize(criteria);
	}
	public void eliminaTutti(List<Contatto> duplicati) {
		if(ApplicationUtil.notEmpty(duplicati)) {
			eliminaContatti(duplicati);
		}
	}
	private ExtendedCriteria ricercaContattoBackofficeCriteria(ContattoCriteria cont) {
		if(cont != null)  cont = (ContattoCriteria) cont.loadCriteria();
		Boolean filtro = impostazioneDao.getFiltroMultiplo();
		String indirizzo = impostazioneDao.getRicercaIndirizzo();
		DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class);
		DetachedCriteria criteriaA = criteria.createCriteria("anagrafica");
		DetachedCriteria criteriaC = criteria.createCriteria("campagna");
		DetachedCriteria criteriaLI = criteria.createCriteria("lista");
		DetachedCriteria archiviazione = criteria.createCriteria("esitoArchiviazione");
		archiviazione.add(Restrictions.eq("backoffice", Boolean.TRUE));
		impostazioneDao.ricercaConsenso(cont, criteriaA);
		impostazioneDao.ricercaIndirizzo(criteriaA, cont, indirizzo);
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaMultipla())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaMultipla(cont, criteriaA);
		} 
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaSingola())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaSingola(cont, criteriaA);
		}
		impostazioneDao.sottoCriteriaStoriaContattoI(false, cont.getStorico_operatoreId(), cont, criteria);
		if(ApplicationUtil.gtZero(cont.getTipo())) {
			criteria.add(Restrictions.eq("tipo",  cont.getTipo()));
		}
		if(ApplicationUtil.gtZero(cont.getCampagnaId())) {
			criteriaC.add(Restrictions.idEq(cont.getCampagnaId()));
		}
		ApplicationUtil.proprieta(false, filtro, criteria, "esitoArchiviazione.id", cont.getEsitoArchiviazioneId(), cont.getEsitoArchiviazioneIds());
		if (ApplicationUtil.isNotBlank(cont.getRicerca_anagrafica())) {
			ApplicationUtil.anagrafica(cont.getRicerca_anagraficaV(), criteriaA,cont.getRicerca_anagrafica());
		}
		if(ApplicationUtil.gtZero(cont.getAgenteId())) {
			criteria.add(Restrictions.eq("agente.id",cont.getAgenteId()));
		}
		ApplicationUtil.statoLC(cont.getStatoIntervallo(), criteriaC, criteriaLI);
		if(ApplicationUtil.gtZero(cont.getProprietarioId())) {
			criteria.add(Restrictions.eq("proprietario.id",cont.getProprietarioId()));
		}
		if(ApplicationUtil.isNotBlank(cont.getPositivo()) || ApplicationUtil.isNotBlank(cont.getUtile())) {
			if(ApplicationUtil.isNotBlank(cont.getPositivo())) {
				if(ApplicationUtil.equalsIgnoreCase(cont.getPositivo(), ConstantsLanguage.SI.stampa())) {
					archiviazione.add(Restrictions.eq("positivo", Boolean.TRUE));
				} 
				if(ApplicationUtil.equalsIgnoreCase(cont.getPositivo(), ConstantsLanguage.NO.stampa())) {
					archiviazione.add(Restrictions.eq("positivo", Boolean.FALSE));
				} 
			}
			if(ApplicationUtil.isNotBlank(cont.getUtile())) {
				if(ApplicationUtil.equalsIgnoreCase(cont.getUtile(), ConstantsLanguage.SI.stampa())) {
					archiviazione.add(Restrictions.eq("utile", Boolean.TRUE));
				} 
				if(ApplicationUtil.equalsIgnoreCase(cont.getUtile(), ConstantsLanguage.NO.stampa())) {
					archiviazione.add(Restrictions.eq("utile", Boolean.FALSE));
				} 
			}
		}
		cont.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, cont);
	}
	@SuppressWarnings("unchecked")
	public List<Contatto> ricercaContattoBackoffice(ContattoCriteria cont,  int start, int size) {
		ExtendedCriteria criteria = ricercaContattoBackofficeCriteria(cont);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), cont.getO_tipo(), cont.getO_colonna(),  new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int ricercaContattoBackofficeSize(ContattoCriteria cont) {
		ExtendedCriteria criteria = ricercaContattoBackofficeCriteria(cont);
		return getListSize(criteria);
	}
	public void eliminaContattiSelezionati(Utente utente, List<Contatto> contatti) {
		String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.CONTATTO.stampa() +": " + ApplicationUtil.size(contatti), contatti);
		operazioneDao.creaOperazione(ConstantsMap.LOG3, p, utente, ConstantsMap.OPERAZIONE_ELIMINA_CONTATTO);
		cancellaLista(contatti);
	}
	public void eliminaContatti(List<Contatto> contatti) {
		cancellaLista(contatti);
	}
	public EsitoChiamata salvaEsito(EsitoChiamata esito) {
		if(!ConstantsEsito.DEFAULTS.keySet().contains(esito.getId())) {
			if(ApplicationUtil.isNotTrue(esito.getRichiamare())) {
				esito.setDaNonRichiamare();
			} else {
				esito.setDaRichiamare();
			}
		} else {
			ConstantsEsito.DEFAULTS.put(esito.getId(), esito);
		}
		return (EsitoChiamata) salvaAggiorna(esito);
	}
	public EsitoChiamata salvaOrdineEsito(EsitoChiamata esito) {
		return (EsitoChiamata) salvaAggiorna(esito);
	}
	private ExtendedCriteria getCriteriaEsitiChiamata(EsitoChiamataCriteria esitoc) {
		if(esitoc != null)  esitoc = (EsitoChiamataCriteria) esitoc.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoChiamata.class);
		if (ApplicationUtil.isNotBlank(esitoc.getEtichetta())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, esitoc.getEtichetta(), criteria, "etichetta", "descrizione");
		}
		//List<Long> keys = new ArrayList<Long>(ConstantsEsito.DEFAULTS.keySet());
		//criteria.add(Restrictions.not(Restrictions.in("id", keys)));
		esitoc.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, esitoc);
	}
	@SuppressWarnings("unchecked")
	public List<EsitoChiamata> trova(EsitoChiamataCriteria esitoc,  int start, int size) {
		ExtendedCriteria criteria =  getCriteriaEsitiChiamata(esitoc);
		ApplicationUtil.orderEsito(criteria.getCriteria());
		criteria.getCriteria().setFetchMode("esito_collegato", FetchMode.JOIN);
		return getList(criteria, start, size);
	}
	public int trovaSize(EsitoChiamataCriteria esitoc) {
		ExtendedCriteria criteria =  getCriteriaEsitiChiamata(esitoc);
		return getListSize(criteria);
	}
	public void  eliminaEsitoChiamataSelezionati(List<EsitoChiamata> lst) {
		if (ApplicationUtil.notEmpty(lst)) {
			Iterator<EsitoChiamata> itCont = lst.iterator();
			while (itCont.hasNext()) {
				EsitoChiamata esito = (EsitoChiamata) itCont.next();
				if(esito.getSelezionabile()) {
					cancellaAggiorna(esito);
				}
			}
		}
	}
	public boolean hasOk() {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoArchiviazione.class);
		criteria.add(Restrictions.or(Restrictions.eq("tipologiaS", 2), Restrictions.eq("tipologiaS", 3)));
		return ApplicationUtil.gtZero(getListSize(criteria));
	}
	@SuppressWarnings("unchecked")
	public List<EsitoChiamata> trovaTutti() {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoChiamata.class);
		if(!hasOk()) {
			criteria.add(Restrictions.ne("id", ConstantsEsito.ID_OK));
		}
		criteria.add(Restrictions.ne("id", ConstantsEsito.ID_RICHIAMO_PRIVATO));
		criteria.add(Restrictions.or(Restrictions.eq("richiamare", Boolean.TRUE), Restrictions.isNull("richiamare")));
		criteria.setFetchMode("esito_collegato", FetchMode.JOIN);
		criteria.addOrder(Order.asc("ordine")).addOrder(Order.asc("etichetta"));
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Autodialer> getAutodialerInEsitoChiamata(Long esitoChiamata) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Autodialer.class);
		Disjunction disjunction =  Restrictions.disjunction();
		disjunction.add(Restrictions.eq("esitoInbound.id", esitoChiamata));
		disjunction.add(Restrictions.eq("esitoApertura.id", esitoChiamata));
		disjunction.add(Restrictions.eq("esitoAbbattuta.id", esitoChiamata)); 
		disjunction.add(Restrictions.eq("esitoAgganciata.id", esitoChiamata));
		disjunction.add(Restrictions.eq("esitoCongestion.id", esitoChiamata));
		disjunction.add(Restrictions.eq("esitoFallita.id", esitoChiamata));
		disjunction.add(Restrictions.eq("esitoHangup.id", esitoChiamata));
		disjunction.add(Restrictions.eq("esitoMacchina.id", esitoChiamata));	
		disjunction.add(Restrictions.eq("esitoNonRisposto.id", esitoChiamata));
		disjunction.add(Restrictions.eq("esitoRinging.id", esitoChiamata));
		disjunction.add(Restrictions.eq("esitoOccupato.id", esitoChiamata));
		disjunction.add(Restrictions.eq("esitoPrenotazione.id", esitoChiamata));
		disjunction.add(Restrictions.eq("esitoUscita.id", esitoChiamata));
		disjunction.add(Restrictions.eq("esitoRifiutata.id", esitoChiamata));
		disjunction.add(Restrictions.eq("esitoSaltata.id", esitoChiamata));
		criteria.add(disjunction);
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<EsitoChiamata> getCollegatiInEsitoChiamata(Long esitoChiamata) {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoChiamata.class);
		criteria.add(Restrictions.eq("esito_collegato.id", esitoChiamata));
		criteria.setFetchMode("esito_collegato", FetchMode.JOIN);
		return getList(criteria);
	}

	@SuppressWarnings("unchecked")
	public List<EsitoArchiviazione> getCollegatiInEsitoArchiviazione(Long esitoArchiviazione) {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoArchiviazione.class);
		criteria.add(Restrictions.eq("esito_collegato.id", esitoArchiviazione));
		criteria.setFetchMode("esito_collegato", FetchMode.JOIN);
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Campagna> getCampagneByEsitoId(Long esitoChiamata) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		criteria.add(Restrictions.eq("esito.id", esitoChiamata));
		return getList(criteria);
	}
	public void nascondiStoria(Boolean elimina_storia, Contatto contatto) {
		if(ApplicationUtil.isTrue(elimina_storia) && contatto != null) {
			String hql = "update StoriaContatto set invisibile = true where contatto = :contatto and invisibile is not true";
			execute(false, hql, new Parameter("contatto", contatto));
		}
	} 
	public void nascondiStoria(Boolean elimina_storia, List<Contatto> contatti) {
		if(ApplicationUtil.isTrue(elimina_storia) && ApplicationUtil.notEmpty(contatti)) {
			String hql = "update StoriaContatto set invisibile = true where contatto in (:contatti) and invisibile is not true";
			execute(false, hql, new Parameter("contatti", contatti));
		}
	} 
	@SuppressWarnings("unchecked")
	public List<StoriaContatto> getStoriaVisibileByContattoId(Long contatto, boolean semaforo, Long userid) {
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContatto.class);
		DetachedCriteria criteriaCO = criteria.createCriteria("contatto");
		DetachedCriteria criteriaCA = criteriaCO.createCriteria("campagna");
		impostazioneDao.subCriteriaCampagnaUser(semaforo, userid, criteriaCA, "id");
		criteriaCO.add(Restrictions.idEq(contatto));
		criteria.setFetchMode("contatto", FetchMode.JOIN);
		criteria.addOrder(Order.asc("data"));
		return  getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<AutoContatto> getAutoContattoVisibileByContattoId(Long contatto, boolean semaforo, Long userid) {
		DetachedCriteria criteria = DetachedCriteria.forClass(AutoContatto.class);
		DetachedCriteria criteriaCO = criteria.createCriteria("contatto");
		DetachedCriteria criteriaCA = criteriaCO.createCriteria("campagna");
		impostazioneDao.subCriteriaCampagnaUser(semaforo, userid, criteriaCA, "id");
		criteriaCO.add(Restrictions.idEq(contatto));
		criteria.addOrder(Order.asc("data_chiamata"));
		return  getList(criteria);
	}

	@SuppressWarnings("unchecked")
	public List<Email> getEmailVisibileByContattoId(Long contatto, boolean semaforo, Long userid) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Email.class);
		DetachedCriteria criteriaCO = criteria.createCriteria("contatto");
		DetachedCriteria criteriaCA = criteriaCO.createCriteria("campagna");
		impostazioneDao.subCriteriaCampagnaUser(semaforo, userid, criteriaCA, "id");
		criteriaCO.add(Restrictions.idEq(contatto));
		criteria.addOrder(Order.asc("data_invio"));
		return  getList(criteria);
	}
	
	@SuppressWarnings("unchecked")
	public List<StoriaContatto> getStoriaByContattoId(Long contatto) {
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContatto.class);
		criteria.add(Restrictions.eq("contatto.id", contatto));
		criteria.setFetchMode("contatto", FetchMode.JOIN);
		criteria.addOrder(Order.asc("data"));
		return  getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<AutoContatto> getAutoContattoByContattoId(Long contatto) {
		DetachedCriteria criteria = DetachedCriteria.forClass(AutoContatto.class);
		criteria.add(Restrictions.eq("contatto.id", contatto));
		criteria.addOrder(Order.asc("data_chiamata"));
		return  getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Email> getEmailByContattoId(Long contatto) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Email.class);
		criteria.add(Restrictions.eq("contatto.id", contatto));
		criteria.addOrder(Order.asc("data_invio"));
		return  getList(criteria);
	}
	
	public void riprogrammaPubblico(Boolean elimina_storia, Boolean elimina_nota, Utente utente, List<Contatto> contatti, Date data) {
		nascondiStoria(elimina_storia, contatti);
		EsitoChiamata richiamoPubblico = ConstantsEsito.DEFAULTS.get(ConstantsEsito.ID_RICHIAMO_PUBBLICO);
		Iterator<Contatto> objIter = contatti.iterator();
		while(objIter.hasNext()) {
			Contatto contatto = objIter.next();
			if(contatto != null) {
				contatto.imposta(data, richiamoPubblico, null);
				if(ApplicationUtil.isTrue(elimina_nota)) {
					contatto.setNota(null);
				}
				if(!ConstantsGlobal.massivo) {
					pannelloDao.salvaContatto(true,contatto, utente);
				}
			}
		}
		if(ConstantsGlobal.massivo) {
			salvaContatti(true, contatti, utente, false);
		}
		String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.CONTATTO.stampa() +": " + ApplicationUtil.size(contatti), contatti);
		operazioneDao.creaOperazione(ConstantsMap.LOG5, p, utente, ConstantsMap.OPERAZIONE_RIPROGRAMMA_PUBBLICO);
	}
	public void salvaContatto(boolean w, Contatto contatto, Utente utente, Date data) {
		contatto.setData_lavorazione(data);
		contatto.setUtente(utente);
		if(w) {
			anagraficaDao.evaluateWorkflow(utente, contatto);
		}
		salvaAggiorna(contatto);
		if(ApplicationUtil.isDisponibile(contatto.getData_presentazione()) && ApplicationUtil.isTrue(impostazioneDao.storiaImmediata())) {
			StoriaContatto storia = pannelloDao.creaStoriaContatto(contatto, utente, data, false, false);
			pannelloDao.salvaAggiorna(storia);
		} else {
			new SalvaStoriaContattoThread(pannelloDao, contatto, utente, data, false, false).start();
		}
	}
	public void inviaContatto(Boolean esterno, Contatto contatto, Agente agente) {
		Boolean diretto =  (esterno && ApplicationUtil.eq(impostazioneDao.getPermettiContattoEsterno(), 2)) || (!esterno && ApplicationUtil.eq(impostazioneDao.getPermettiContattoInterno(), 2));
		if(diretto) {
			contatto.setTipo(ConstantsGlobal.inviato);
		} else {
			contatto.setTipo(ConstantsGlobal.inserito);
		}
	}
	public void inviaPrenotazione(Boolean esterno, Contatto contatto, Agente agente) {
		Boolean diretto =  (esterno && ApplicationUtil.eq(impostazioneDao.getPermettiPrenotazioneEsterno(), 2)) || (!esterno && ApplicationUtil.eq(impostazioneDao.getPermettiPrenotazioneInterno(), 2));
		if(diretto) {
			contatto.setTipo(ConstantsGlobal.inviato);
		} else {
			contatto.setTipo(ConstantsGlobal.inserito);
		}
	}
	public Contatto salvaContattoOffline(Boolean esterno, Long listaId, Anagrafica anagrafica, Campagna campagnaP, Long proprietarioID, Date data, Integer tipoContatto, Utente utente, Long agenteId) throws Exception {
		Campagna campagna = campagnaDao.getFullCampagna(campagnaP.getId());
		String nomeLista =  ConstantsLanguage.OFFLINE.stampa() + ": " + campagna.getNome();
		Lista lista = null;
		if(ApplicationUtil.gtZero(listaId)) {
			lista = campagnaDao.getLista(listaId);
		}
		if(lista == null ) {
			lista =  campagnaDao.getByCampagnaNome(nomeLista, campagnaP.getId());
		}
		if(lista == null ) {
			lista = new Lista();
			lista.setNome(nomeLista);
			lista.setDescrizione(nomeLista);
			salvaAggiorna(lista);
		}
		campagnaDao.addListaToCampagnaSalva(campagna, lista);
		List<Lista> liste = new ArrayList<Lista>();
		liste.add(lista);
		anagrafica.setLista(liste);
		anagraficaDao.salva(anagrafica);
		Contatto contatto = new Contatto();
		contatto.setAnagrafica(anagrafica);
		Agente agente = userDao.getAgente(agenteId);
		contatto.setAgente(agente);
		inviaContatto(esterno, contatto, agente);
		contatto.setCampagna(campagna);
		contatto.setLista(lista);
		if(ApplicationUtil.eq(tipoContatto, 0)) {
			EsitoChiamata da_chiamare = ConstantsEsito.DEFAULTS.get(ConstantsEsito.ID_DA_CHIAMARE);
			contatto.imposta(data, da_chiamare, null);
		}
		if(ApplicationUtil.eq(tipoContatto, 1)) {
			EsitoChiamata riprogrammatoPubblico = ConstantsEsito.DEFAULTS.get(ConstantsEsito.ID_RICHIAMO_PUBBLICO);
			contatto.imposta(data, riprogrammatoPubblico, null);
		} 
		if(ApplicationUtil.eq(tipoContatto, 2)) {
			EsitoChiamata richiamoPrivato = ConstantsEsito.DEFAULTS.get(ConstantsEsito.ID_RICHIAMO_PRIVATO);
			if(ApplicationUtil.gtZero(proprietarioID)) {
				Utente proprietario = userDao.getUser(proprietarioID);
				contatto.imposta(data, richiamoPrivato, proprietario);
			} else {
				contatto.imposta(data, richiamoPrivato, utente);
			}
		} 
		salvaContatto(false, contatto, utente, ApplicationUtil.date());
		String p = ApplicationUtil.asString( ConstantsLanguage.CONTATTO.stampa(), contatto);
		operazioneDao.creaOperazione(ConstantsMap.LOG3, p, utente, ConstantsMap.OPERAZIONE_SALVA_CONTATTO);
		salvaAllegati(anagrafica, contatto, false);
		return contatto;
	}
	public Contatto salvaContattoFiglio(Boolean esterno, Anagrafica anagrafica, Long agenteId, Contatto contattoPadre, Long proprietarioID, Date data, Integer tipoContatto, Utente utente){
		Campagna campagnaP =  contattoPadre.getCampagna();
		Campagna campagna = campagnaDao.getFullCampagna(campagnaP.getId());
		Lista lista =  contattoPadre.getLista();
		campagnaDao.addListaToCampagnaSalva(campagna, lista);
		anagrafica.setAnagrafica_padre(contattoPadre.getAnagrafica());
		List<Lista> liste = new ArrayList<Lista>();
		liste.add(lista);
		anagrafica.setLista(liste);
		anagraficaDao.salva(anagrafica);
		Contatto contatto = new Contatto();
		Agente agente = userDao.getAgente(agenteId);
		contatto.setAgente(agente);
		inviaContatto(esterno, contatto, agente);
		contatto.setAnagrafica(anagrafica);
		contatto.setCampagna(campagna);
		contatto.setLista(lista);
		if(ApplicationUtil.eq(tipoContatto, 0)) {
			EsitoChiamata da_chiamare = ConstantsEsito.DEFAULTS.get(ConstantsEsito.ID_DA_CHIAMARE);
			contatto.imposta(data, da_chiamare, null);
		}
		if(ApplicationUtil.eq(tipoContatto, 1)) {
			EsitoChiamata riprogrammatoPubblico = ConstantsEsito.DEFAULTS.get(ConstantsEsito.ID_RICHIAMO_PUBBLICO);
			contatto.imposta(data, riprogrammatoPubblico, null);
		} 
		if(ApplicationUtil.eq(tipoContatto, 2)) {
			EsitoChiamata richiamoPrivato = ConstantsEsito.DEFAULTS.get(ConstantsEsito.ID_RICHIAMO_PRIVATO);
			if(ApplicationUtil.gtZero(proprietarioID)) {
				Utente proprietario = userDao.getUser(proprietarioID);
				contatto.imposta(data, richiamoPrivato, proprietario);
			} else {
				contatto.imposta(data, richiamoPrivato, utente);
			}
		} 
		salvaContatto(false, contatto, utente, ApplicationUtil.date());
		String p = ApplicationUtil.asString( ConstantsLanguage.CONTATTO.stampa(), contatto);
		operazioneDao.creaOperazione(ConstantsMap.LOG3, p, utente, ConstantsMap.OPERAZIONE_SALVA_CONTATTO);
		salvaAllegati(anagrafica, contatto, false);
		return contatto;
	}
	private void salvaAllegati(Anagrafica anagrafica, Contatto contatto, boolean clear) {
		try {
			List<Allegato> allegati = anagrafica.getAllegato();
			if(ApplicationUtil.notEmpty(allegati)) {
				String s = impostazioneDao.getPathBaseW(ConstantsGlobal.path_contatto);
				Iterator<Allegato> allIt = allegati.iterator();
				while (allIt.hasNext()) {
					Allegato all = (Allegato) allIt.next();
					AllegatoContatto allegato = new AllegatoContatto();
					allegato.setContatto(contatto);
					allegato.setDimensione(all.getDimensione());
					allegato.setNome(ApplicationUtil.getNameN(all.getNome()));
					allegato.setContentFile(all.getFile(), s);
					salvaAggiorna(allegato);
				}
			}
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
		if(clear) {
			if(ApplicationUtil.notEmpty(anagrafica.getAllegato())) {
				anagrafica.getAllegato().clear();
			}
		}
	}
	public Contatto salvaContattoNuovo(Boolean esterno, Anagrafica anagrafica, Long campagnaId, Long listaId, Long agenteId, Long proprietarioID, Date data, Integer tipoContatto, Utente utente) throws Exception  {
		Campagna campagna = campagnaDao.getFullCampagna(campagnaId);
		String nomeLista =  ConstantsLanguage.OFFLINE.stampa() + ": " + campagna.getNome();
		Lista lista = null;
		if(ApplicationUtil.gtZero(listaId)) {
			lista = campagnaDao.getLista(listaId);
		}
		if(lista == null ) {
			lista =  campagnaDao.getByCampagnaNome(nomeLista, campagna.getId());
		}
		if(lista == null ) {
			lista = new Lista();
			lista.setNome(nomeLista);
			lista.setDescrizione(nomeLista);
			salvaAggiorna(lista);
		}
		campagnaDao.addListaToCampagnaSalva(campagna, lista);
		List<Lista> liste = new ArrayList<Lista>();
		liste.add(lista);
		anagrafica.setLista(liste);
		anagraficaDao.salva(anagrafica);
		Contatto contatto = new Contatto();
		Agente agente = userDao.getAgente(agenteId);
		contatto.setAgente(agente);
		inviaContatto(esterno, contatto, agente);
		contatto.setAnagrafica(anagrafica);
		contatto.setCampagna(campagna);
		contatto.setLista(lista);
		if(ApplicationUtil.eq(tipoContatto, 0)) {
			EsitoChiamata da_chiamare = ConstantsEsito.DEFAULTS.get(ConstantsEsito.ID_DA_CHIAMARE);
			contatto.imposta(data, da_chiamare, null);
		}
		if(ApplicationUtil.eq(tipoContatto, 1)) {
			EsitoChiamata riprogrammatoPubblico = ConstantsEsito.DEFAULTS.get(ConstantsEsito.ID_RICHIAMO_PUBBLICO);
			contatto.imposta(data, riprogrammatoPubblico, null);
		}
		if(ApplicationUtil.eq(tipoContatto, 2)) {
			EsitoChiamata richiamoPrivato = ConstantsEsito.DEFAULTS.get(ConstantsEsito.ID_RICHIAMO_PRIVATO);
			if(ApplicationUtil.gtZero(proprietarioID)) {
				Utente proprietario = userDao.getUser(proprietarioID);
				contatto.imposta(data, richiamoPrivato, proprietario);
			} else {
				contatto.imposta(data, richiamoPrivato, utente);
			}
		} 
		salvaContatto(false, contatto, utente, ApplicationUtil.date());
		String p = ApplicationUtil.asString( ConstantsLanguage.CONTATTO.stampa(), contatto);
		operazioneDao.creaOperazione(ConstantsMap.LOG3, p, utente, ConstantsMap.OPERAZIONE_SALVA_CONTATTO);
		salvaAllegati(anagrafica, contatto, false);
		return contatto;
	}
	public int salvaPrenotazione(Boolean esterno, String  telefono, Integer tipoTelefono, Long campagnaId, Long listaId, Long agenteId, Utente utente)  {
		int successo = 0;
		Campagna campagna = campagnaDao.getFullCampagna(campagnaId);
		String nomeLista =  ConstantsLanguage.OFFLINE.stampa() + ": " + campagna.getNome();
		Lista lista = null;
		if(ApplicationUtil.gtZero(listaId)) {
			lista = campagnaDao.getLista(listaId);
		}
		if(lista == null ) {
			lista =  campagnaDao.getByCampagnaNome(nomeLista, campagna.getId());
		}
		if(lista == null ) {
			lista = new Lista();
			lista.setNome(nomeLista);
			lista.setDescrizione(nomeLista);
			salvaAggiorna(lista);
		}
		campagnaDao.addListaToCampagnaSalva(campagna, lista);
		List<Lista> liste = new ArrayList<Lista>();
		liste.add(lista);
		Agente agente = userDao.getAgente(agenteId);
		String tipo = ApplicationUtil.soloNumero(tipoTelefono);
		Long idA = campagnaDao.getAnagraficaByTelefono(ApplicationUtil.trimToEmpty(telefono), tipo);
		if(idA == null) {
			Anagrafica anagrafica = new Anagrafica();
			anagrafica.setLista(liste);
			if(tipo.equals(ConstantsMap.telefono_fisso)) {
				anagrafica.setTelefono_fisso(telefono);
			}
			if(tipo.equals(ConstantsMap.telefono_mobile)) {
				anagrafica.setTelefono_mobile(telefono);
			}
			if(tipo.equals(ConstantsMap.telefono_fax)) {
				anagrafica.setTelefono_fax(telefono);
			}
			if(tipo.equals(ConstantsMap.telefono_altro)) {
				anagrafica.setTelefono_altro(telefono);
			}
			anagraficaDao.salva(anagrafica);
			Date data = ApplicationUtil.date();
			Contatto contatto = new Contatto();
			contatto.setAgente(agente);
			inviaPrenotazione(esterno, contatto, agente);
			contatto.setAnagrafica(anagrafica);
			contatto.setCampagna(campagna);
			contatto.setLista(lista);
			EsitoChiamata da_chiamare = ConstantsEsito.DEFAULTS.get(ConstantsEsito.ID_DA_CHIAMARE);
			contatto.imposta(data, da_chiamare, null);
			salvaContatto(false, contatto, utente, data);
			String p = ApplicationUtil.asString( ConstantsLanguage.CONTATTO.stampa(), contatto);
			operazioneDao.creaOperazione(ConstantsMap.LOG3, p, utente, ConstantsMap.OPERAZIONE_SALVA_CONTATTO);
			successo = 1;
		} else {
			Long idP = campagnaDao.getPrenotazioneByTelefono(ApplicationUtil.trimToEmpty(telefono), campagnaId);
			if(idP == null) {
				Prenotazione prenotazione = new Prenotazione();
				prenotazione.setAgente(agente);
				prenotazione.setTelefono(ApplicationUtil.trimToEmpty(telefono));
				prenotazione.setTipo_telefono(tipoTelefono);
				prenotazione.setOperatore(utente);
				prenotazione.setCampagna(campagna);
				prenotazione.setLista(lista);
				prenotazione.setStato(ConstantsGlobal.prenotazione_duplicata);
				prenotazioneDao.salvaPrenotazione(prenotazione, utente);
				String p = ApplicationUtil.asString( ConstantsLanguage.PRENOTAZIONE.stampa(), prenotazione);
				operazioneDao.creaOperazione(ConstantsMap.LOG3, p, utente, ConstantsMap.OPERAZIONE_SALVA_PRENOTAZIONE);
				successo = 2;
			} else {
				successo = 3;
			}
		}
		return successo;
	}
	public void inviaPrenotazione(Boolean esterno, Prenotazione prenotazione,  Utente utente) {
		Campagna campagna = campagnaDao.getFullCampagna(prenotazione.getCampagna().getId());
		Lista lista = prenotazione.getLista();
		campagnaDao.addListaToCampagnaSalva(campagna, lista);
		List<Lista> liste = new ArrayList<Lista>();
		liste.add(lista);
		Agente agente = prenotazione.getAgente();
		Anagrafica anagrafica = new Anagrafica();
		anagrafica.setLista(liste);
		String tipo = ApplicationUtil.soloNumero(prenotazione.getTipo_telefono());
		if(tipo.equals(ConstantsMap.telefono_fisso)) {
			anagrafica.setTelefono_fisso(prenotazione.getTelefono());
		}
		if(tipo.equals(ConstantsMap.telefono_mobile)) {
			anagrafica.setTelefono_mobile(prenotazione.getTelefono());
		}
		if(tipo.equals(ConstantsMap.telefono_fax)) {
			anagrafica.setTelefono_fax(prenotazione.getTelefono());
		}
		if(tipo.equals(ConstantsMap.telefono_altro)) {
			anagrafica.setTelefono_altro(prenotazione.getTelefono());
		}
		anagraficaDao.salva(anagrafica);
		Contatto contatto = new Contatto();
		Date data = ApplicationUtil.date();
		inviaPrenotazione(esterno, contatto, agente);
		contatto.setAgente(agente);
		contatto.setAnagrafica(anagrafica);
		contatto.setCampagna(campagna);
		contatto.setLista(lista);
		EsitoChiamata da_chiamare = ConstantsEsito.DEFAULTS.get(ConstantsEsito.ID_DA_CHIAMARE);
		contatto.imposta(data, da_chiamare, null);
		salvaContatto(false, contatto, utente, data);
		String p = ApplicationUtil.asString( ConstantsLanguage.CONTATTO.stampa(), contatto);
		operazioneDao.creaOperazione(ConstantsMap.LOG3, p,  utente, ConstantsMap.OPERAZIONE_SALVA_CONTATTO);
		prenotazione.setStato(ConstantsGlobal.prenotazione_inviata);
		prenotazioneDao.salvaPrenotazione(prenotazione, utente);
	}
	public List<SelectItem> getEsitiChiamataSI() {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoChiamata.class);
		List<EsitoChiamataSI> lis = getEsitoChiamataSI(criteria);
		return impostazioneDao.getEsitoChiamata(lis, false, null);
	}
	public List<SelectItem> getEsitiChiamataAutodialerSI() {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoChiamata.class);
		List<Long> keys = new ArrayList<Long>(ConstantsEsito.DEFAULTS.keySet());
		criteria.add(Restrictions.not(Restrictions.in("id", keys)));
		List<EsitoChiamataSI> lis = getEsitoChiamataSI(criteria);
		return impostazioneDao.getEsitoChiamata(lis, false, null);
	}
	public List<SelectItem> getEsitiChiamataOperatoreSI() {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoChiamata.class);
		List<EsitoChiamataSI> lis = getEsitoChiamataSI(criteria);
		return impostazioneDao.getEsitoChiamata(lis, false, null);
	}
	public List<SelectItem> getCampagneSI() {
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		List<CampagnaSI> lis = getCampagnaSI(criteria);
		return impostazioneDao.getCampagna(lis, true, false, false, null);
	}
	public List<SelectItem> getCampagneSI(StatoIntervallo stato) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		ApplicationUtil.statoLC(stato, criteria, null);
		List<CampagnaSI> lis = getCampagnaSI(criteria);
		return impostazioneDao.getCampagna(lis, true, false, false, null);
	}
	public void riprogrammaPrivato(Boolean elimina_storia, Boolean elimina_nota, Utente utente, List<Contatto> contatti,Date data, Long proprietarioID) {
		Utente proprietario = userDao.getUser(proprietarioID);
		if (proprietario != null) {
			nascondiStoria(elimina_storia, contatti);
			EsitoChiamata richiamoPrivato = ConstantsEsito.DEFAULTS.get(ConstantsEsito.ID_RICHIAMO_PRIVATO);
			Iterator<Contatto> objIter = contatti.iterator();
			while(objIter.hasNext()) {
				Contatto contatto = objIter.next();
				if(contatto != null) {
					contatto.imposta(data, richiamoPrivato, proprietario);
					if(ApplicationUtil.isTrue(elimina_nota)) {
						contatto.setNota(null);
					}
					if(!ConstantsGlobal.massivo) {
						pannelloDao.salvaContatto(true, contatto, utente);
					}
				}
			}
			if(ConstantsGlobal.massivo) {
				salvaContatti(true, contatti, utente, false);
			}
			String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.CONTATTO.stampa() +": " + ApplicationUtil.size(contatti), contatti);
			operazioneDao.creaOperazione(ConstantsMap.LOG5, p, utente, ConstantsMap.OPERAZIONE_RIPROGRAMMA_PRIVATO);
		}
	}
	
	public void inviaEmailScelti(Utente utente, List<Contatto> contatti, EmailTemplate emailTemplate, Email email) {
		pannelloDao.inviaEmail(contatti, utente, emailTemplate, email.getTesto(), email.getOggetto(), email.getAllegati(), email.getDataSchedulata());
	}
	
	
	@SuppressWarnings("unchecked")
	public List<Rmail> getRmailVisibileByContattoId(Long contatto, boolean semaforo, Long userid) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Rmail.class);
		DetachedCriteria criteriaCO = criteria.createCriteria("contatto");
		DetachedCriteria criteriaCA = criteriaCO.createCriteria("campagna");
		impostazioneDao.subCriteriaCampagnaUser(semaforo, userid, criteriaCA, "id");
		criteriaCO.add(Restrictions.idEq(contatto));
		criteria.addOrder(Order.asc("data_invio"));
		return  getList(criteria);
	}
	
	
	@SuppressWarnings("unchecked")
	public List<Rmail> getRmailByContattoId(Long contatto) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Rmail.class);
		criteria.add(Restrictions.eq("contatto.id", contatto));
		criteria.addOrder(Order.asc("data_invio"));
		return  getList(criteria);
	}
	
	public void inviaRmailScelti(Utente utente, List<Contatto> contatti, RmailTemplate rmailTemplate, Rmail rmail) {
		pannelloDao.inviaRmail(contatti, utente, rmailTemplate, rmail.getTesto(), rmail.getAllegati(), rmail.getDataSchedulata());
	}
	
	public void riprogrammaPrivatoSingolo(Boolean elimina_storia, Utente utente,Contatto contatto, Date data, Long proprietarioId) {
		Utente proprietario = userDao.getUser(proprietarioId);
		if (proprietario != null) {
			nascondiStoria(elimina_storia, contatto);
			EsitoChiamata richiamoPrivato = ConstantsEsito.DEFAULTS.get(ConstantsEsito.ID_RICHIAMO_PRIVATO);
			contatto.imposta(data, richiamoPrivato, proprietario);
			pannelloDao.salvaContatto(true,contatto, utente);
			String p = ApplicationUtil.asString( ConstantsLanguage.CONTATTO.stampa(), contatto);
			operazioneDao.creaOperazione(ConstantsMap.LOG5, p,  utente, ConstantsMap.OPERAZIONE_RIPROGRAMMA_PRIVATO);
		}
	}
	public void riprogrammaPrivatoSingolo(Boolean elimina_storia, Utente utente,Contatto contatto, Date data) {
		if (utente != null) {
			nascondiStoria(elimina_storia, contatto);
			EsitoChiamata richiamoPrivato = ConstantsEsito.DEFAULTS.get(ConstantsEsito.ID_RICHIAMO_PRIVATO);
			contatto.imposta(data, richiamoPrivato, utente);
			pannelloDao.salvaContatto(true,contatto, utente);
			String p = ApplicationUtil.asString( ConstantsLanguage.CONTATTO.stampa(), contatto);
			operazioneDao.creaOperazione(ConstantsMap.LOG5, p,  utente, ConstantsMap.OPERAZIONE_RIPROGRAMMA_PRIVATO);
		}
	}
	public void riprogrammaArchiviatoSingolo(Boolean elimina_storia, Utente utente,Contatto contatto, Date data, Long proprietarioId) {
		nascondiStoria(elimina_storia, contatto);
		if (proprietarioId != null) {
			Utente proprietario =  userDao.getUser(proprietarioId);
			contatto.proprietario(data, proprietario);
		} else {
			contatto.proprietario(data, null);
		}
		pannelloDao.salvaContatto(true,contatto, utente);
		String p = ApplicationUtil.asString( ConstantsLanguage.CONTATTO.stampa(), contatto);
		operazioneDao.creaOperazione(ConstantsMap.LOG5, p, utente, ConstantsMap.OPERAZIONE_RIPROGRAMMA_ARCHIVIATO);
	}
	public List<SelectItem> getAllEsitiChiamataDefinitiviSI() {
		boolean all = true;
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoChiamata.class);
		if(all) {
			List<Long> keys = new ArrayList<Long>(ConstantsEsito.DEFAULTS.keySet());
			criteria.add(Restrictions.not(Restrictions.in("id", keys)));
		} else {
			criteria.add(Restrictions.eq("richiamare", Boolean.FALSE));
		}
		List<EsitoChiamataSI> lis = getEsitoChiamataSI(criteria);
		return impostazioneDao.getEsitoChiamata(lis, false, null);
	}

	public List<SelectItem> getAllEsitiArchiviazioneDefinitiviSI() {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoArchiviazione.class);
		criteria.add(Restrictions.or(Restrictions.eq("tipologiaS", 0), Restrictions.eq("tipologiaS", 3)));
		List<EsitoArchiviazioneSI> lis = getEsitoArchiviazioneSI(criteria);
		return impostazioneDao.getEsitoArchiviazione(lis, false, null);
	}

	public void riprogrammaPrivato(Utente utente,Contatto contatto, Date data, Utente proprietario) {
		EsitoChiamata richiamoPrivato = ConstantsEsito.DEFAULTS.get(ConstantsEsito.ID_RICHIAMO_PRIVATO);
		contatto.imposta(data, richiamoPrivato, proprietario);
		pannelloDao.salvaContatto(true,contatto, utente);
		String p = ApplicationUtil.asString( ConstantsLanguage.CONTATTO.stampa(), contatto);
		operazioneDao.creaOperazione(ConstantsMap.LOG5, p, utente, ConstantsMap.OPERAZIONE_RIPROGRAMMA_PRIVATO);
	}
	public void riprogrammaPubblico(Utente utente,Contatto contatto, Date data) {
		EsitoChiamata richiamoPubblico = ConstantsEsito.DEFAULTS.get(ConstantsEsito.ID_RICHIAMO_PUBBLICO);
		contatto.imposta(data, richiamoPubblico, null);
		pannelloDao.salvaContatto(true,contatto, utente);
		String p = ApplicationUtil.asString( ConstantsLanguage.CONTATTO.stampa(), contatto);
		operazioneDao.creaOperazione(ConstantsMap.LOG5, p, utente, ConstantsMap.OPERAZIONE_RIPROGRAMMA_PUBBLICO);
	}
	public void riprogrammaPrivatoAll(Utente utente, Contatto contatto, Date data, Long proprietarioId) {
		EsitoChiamata richiamoPrivato = ConstantsEsito.DEFAULTS.get(ConstantsEsito.ID_RICHIAMO_PRIVATO);
		Utente proprietario = userDao.getUser(proprietarioId);
		contatto.imposta(data, richiamoPrivato, proprietario);
		pannelloDao.salvaContatto(true,contatto, utente);
		String p = ApplicationUtil.asString( ConstantsLanguage.CONTATTO.stampa(), contatto);
		operazioneDao.creaOperazione(ConstantsMap.LOG5, p, utente, ConstantsMap.OPERAZIONE_RIPROGRAMMA_PRIVATO);
	}
	public void riprogrammaPubblicoSingolo(Boolean elimina_storia, Utente utente, Contatto contatto, Date data) {
		nascondiStoria(elimina_storia, contatto);
		EsitoChiamata richiamoPubblico = ConstantsEsito.DEFAULTS.get(ConstantsEsito.ID_RICHIAMO_PUBBLICO);
		contatto.imposta(data, richiamoPubblico, null);
		pannelloDao.salvaContatto(true,contatto, utente);
		String p = ApplicationUtil.asString( ConstantsLanguage.CONTATTO.stampa(), contatto);
		operazioneDao.creaOperazione(ConstantsMap.LOG5, p,  utente, ConstantsMap.OPERAZIONE_RIPROGRAMMA_PUBBLICO);
	}
	public Contatto get(Long id){
		return (Contatto) get(Contatto.class, id);
	}

	public void eliminaRegistrazioneFtp(Contatto contatto) {
		String monitor = ConstantsGlobal.path_monitor;
		if(ApplicationUtil.isNotTrue(impostazioneDao.getFTPEnabled())) {
			String home = impostazioneDao.getHome();
			File perc = new File(home + monitor);
			if(perc.exists()) {
				Path folder = Paths.get(perc.getAbsolutePath());
				DirectoryStream<Path> stream = ApplicationUtil.streamFile(folder, "_" + contatto.getId() + "." + ConstantsMap.wav);
				if(stream != null) {
					for(Path entry : stream) {
						File file = entry.toFile();
						ApplicationUtil.deleteFile(file);
					}
				}
			}
		} else {
			UtilFTP ut = impostazioneDao.getUtilFTP();
			FTPClient vmftp = ApplicationUtil.createClient();
			if (ApplicationUtil.connect(vmftp, ut)) {
				try {
					FTPFile[] wavFiles = ApplicationUtil.listMonitor(vmftp, monitor + "/", contatto.getId(), ConstantsMap.wav);
					if (ApplicationUtil.notEmpty(wavFiles)) {
						List<FTPFile> tmp = new ArrayList<FTPFile>(Arrays.asList(wavFiles));
						Iterator<FTPFile> it = tmp.iterator();
						while (it.hasNext()) {
							FTPFile fil = (FTPFile) it.next();
							ApplicationUtil.deleteFile(vmftp, monitor + "/" + ApplicationUtil.getNameN(fil.getName()));
						}
					}
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				} finally {
					ApplicationUtil.disconnect(vmftp);
				}
			}	
		}
	}

	public List<MyFile> listaRegistrazioni(Contatto contatto) {
		List<MyFile> ff = new ArrayList<MyFile>();
		String monitor = ConstantsGlobal.path_monitor;
		if(ApplicationUtil.isNotTrue(impostazioneDao.getFTPEnabled())) {
			String home = impostazioneDao.getHome();
			File perc = new File(home + monitor);
			if(perc.exists()) {
				Path folder = Paths.get(perc.getAbsolutePath());
				DirectoryStream<Path> stream = ApplicationUtil.streamFile(folder, "_" + contatto.getId() + "." + ConstantsMap.wav);
				if(stream != null) {
					for(Path entry : stream) {
						File file = entry.toFile();
						MyFile f = new MyFile(file);
						f.setName(file.getName());
						f.setModified(file.lastModified());
						f.setId(contatto.getId());
						ff.add(f);
					}
				}
			}
		} else {
			UtilFTP ut = impostazioneDao.getUtilFTP();
			FTPClient vmftp = ApplicationUtil.createClient();
			if (ApplicationUtil.connect(vmftp, ut)) {
				try {
					FTPFile[] wavFiles = ApplicationUtil.listMonitor(vmftp, monitor + "/", contatto.getId(), ConstantsMap.wav);
					if (ApplicationUtil.notEmpty(wavFiles)) {
						for(FTPFile file : wavFiles) {
							Long time = ApplicationUtil.time(file.getTimestamp());
							MyFile f = new MyFile(file);
							f.setName(ApplicationUtil.getNameN(file.getName()));
							f.setModified(time);
							f.setId(contatto.getId());
							ff.add(f);
						}
					}
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				} finally {
					ApplicationUtil.disconnect(vmftp);
				}
			}	
		}
		return ff;
	}

	public List<MyFile> listaRegistrazioni(RegistrazioneCriteria mm) {
		if(mm != null)  mm = (RegistrazioneCriteria) mm.loadCriteria();
		List<MyFile> ff = new ArrayList<MyFile>();
		String monitor = ConstantsGlobal.path_monitor;
		if(mm != null && mm.getCerca()) {
			if(ApplicationUtil.isNotTrue(impostazioneDao.getFTPEnabled())) {
				String home = impostazioneDao.getHome();
				File perc = new File(home + monitor);
				if(perc.exists()) {
					Path folder = Paths.get(perc.getAbsolutePath());
					DirectoryStream<Path> stream = ApplicationUtil.streamFile(folder, "." + ConstantsMap.wav, mm);
					if(stream != null) {
						for(Path entry : stream) {
							File file = entry.toFile();
							if(mm.getUtente() == null || (mm.getUtente()!= null && ApplicationUtil.fitraUtente(file.getName(), mm.getUtente()))) {
								MyFile f = new MyFile(file);
								f.setName(file.getName());
								f.setModified(file.lastModified());
								String a = ApplicationUtil.substringAfterLast(file.getName(), "_");
								String b = ApplicationUtil.trimToNull(ApplicationUtil.substringBeforeLast(a, "." + ConstantsMap.wav));
								if(b != null) {
									Long id = null;
									try {
										id = Long.parseLong(b);
									} catch (Exception nfe) {
									}
									if(id != null) {
										f.setId(id);
									}
								}
								ff.add(f);
							}
						}
					}
				}
			} else {
				UtilFTP ut = impostazioneDao.getUtilFTP();
				FTPClient vmftp = ApplicationUtil.createClient();
				if (ApplicationUtil.connect(vmftp, ut)) {
					try {
						FTPFile[] wavFiles = ApplicationUtil.listMonitor(vmftp, monitor + "/", null, ConstantsMap.wav);
						if (ApplicationUtil.notEmpty(wavFiles)) {
							for(FTPFile file : wavFiles) {
								Long time = ApplicationUtil.time(file.getTimestamp());
								if(mm.isRicerca(file.getName(), file.getSize(), time)) {
									MyFile f = new MyFile(file);
									f.setName(file.getName());
									f.setModified(time);
									String a = ApplicationUtil.substringAfterLast(file.getName(), "_");
									String b = ApplicationUtil.trimToNull(ApplicationUtil.substringBeforeLast(a, "." + ConstantsMap.wav));
									if(b != null) {
										Long id = null;
										try {
											id = Long.parseLong(b);
										} catch (Exception nfe) {
										}
										if(id != null) {
											f.setId(id);
										}
									}
									ff.add(f);
								}
							}
						}
					} catch (Exception e) {
						ErrorLog.log(null, getClass(), e);
					} finally {
						ApplicationUtil.disconnect(vmftp);
					}
				}	
			}
		}
		return ff;
	}
	public void eliminaRegistrazioneFtp(MyFile rec) {
		String monitor = ConstantsGlobal.path_monitor;
		if(ApplicationUtil.isNotTrue(impostazioneDao.getFTPEnabled())) {
			String home = impostazioneDao.getHome();
			File perc = new File(home + monitor + "/" + rec.getName());
			if(perc.exists()) {
				ApplicationUtil.deleteFile(perc);
			}
		} else {
			UtilFTP ut = impostazioneDao.getUtilFTP();
			FTPClient vmftp = ApplicationUtil.createClient();
			if (ApplicationUtil.connect(vmftp, ut)) {
				try {
					ApplicationUtil.deleteFile(vmftp, monitor + "/" + rec.getName());
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				} finally {
					ApplicationUtil.disconnect(vmftp);
				}
			}	
		}
	}
	public void eliminaRegistrazioneFtp(List<MyFile> recs) {
		String monitor = ConstantsGlobal.path_monitor;
		if(ApplicationUtil.isNotTrue(impostazioneDao.getFTPEnabled())) {
			if(ApplicationUtil.notEmpty(recs)) {
				String home = impostazioneDao.getHome();
				Iterator<MyFile> rIt = recs.iterator();
				while (rIt.hasNext()) {
					MyFile rec= (MyFile) rIt.next();
					File perc = new File(home + monitor + "/" + rec.getName());
					if(perc.exists()) {
						ApplicationUtil.deleteFile(perc);
					}
				}
			}
		} else {
			UtilFTP ut = impostazioneDao.getUtilFTP();
			FTPClient vmftp = ApplicationUtil.createClient();
			if (ApplicationUtil.connect(vmftp, ut)) {
				try {
					if(ApplicationUtil.notEmpty(recs)) {
						Iterator<MyFile> rIt = recs.iterator();
						while (rIt.hasNext()) {
							MyFile rec= (MyFile) rIt.next();
							ApplicationUtil.deleteFile(vmftp, monitor + "/" + rec.getName());
						}
					}
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				} finally {
					ApplicationUtil.disconnect(vmftp);
				}
			}	
		}
	}
	public void scaricaRegistrazione(Contatto contatto, MyFile rec, DownloadBean downloadBean) {
		String b = impostazioneDao.audioSupportato();
		String monitor = ConstantsGlobal.path_monitor;
		if(ApplicationUtil.isNotTrue(impostazioneDao.getFTPEnabled())) {
			try {
				String home = impostazioneDao.getHome();
				OutputStream out = downloadBean.outputStream();
				String n_name = ApplicationUtil.outFileRCodec(contatto != null ? contatto.getId() : rec.getId(), out, home + monitor +"/"+ rec.getName(), b);
				downloadBean.filename(n_name);
				downloadBean.close();
			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		} else {
			UtilFTP ut = impostazioneDao.getUtilFTP();
			FTPClient vmftp = ApplicationUtil.createClient();
			if (ApplicationUtil.connect(vmftp, ut)) {
				try {
					OutputStream out = downloadBean.outputStream();
					String n_name = ApplicationUtil.outFileRCodec(vmftp, contatto != null ? contatto.getId() : rec.getId(), out, monitor + "/" + rec.getName(), b);
					downloadBean.filename(n_name);
					downloadBean.close();
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				} finally {
					ApplicationUtil.disconnect(vmftp);
				}
			}	
		}
	}
	public void modificaAgenteSelezionati( Utente utente,  List<Contatto> contatti,  Long agenteNuovoId) {
		Agente agenteNuovo = campagnaDao.getAgente(agenteNuovoId);
		modificaAgente(utente, contatti, agenteNuovo);
	}
	public void modificaAgente(Utente utente, List<Contatto> contatti, Agente agenteNuovo) {
		Iterator<Contatto> objIter = contatti.iterator();
		while(objIter.hasNext()) {
			Contatto contatto = objIter.next();
			contatto.setAgente(agenteNuovo);
			if(!ConstantsGlobal.massivo) {
				pannelloDao.salvaContatto(true,contatto, utente);
			}
		}
		if(ConstantsGlobal.massivo) {
			salvaContatti(true, contatti, utente, false);
		}
	}
	public List<MyFile> listaAttese(FileCriteria mm) {
		if(mm != null)  mm = (FileCriteria) mm.loadCriteria();
		List<MyFile> ff = new ArrayList<MyFile>();
		if(mm != null && mm.getCerca()) {
			if(ApplicationUtil.isNotTrue(impostazioneDao.getFTPEnabled())) {
				String home = impostazioneDao.getHome();
				File perc = new File(home + ConstantsGlobal.path_attesa);
				if(perc.exists() && perc.isDirectory()) {
					File [] files = perc.listFiles();
					if(ApplicationUtil.notEmpty(files)) {
						for(File file : files) {
							if(mm.isRicerca(file.getName(), file.length(), file.lastModified())) {
								MyFile f = new MyFile(file);
								f.setName(file.getName());
								f.setModified(file.lastModified());
								ff.add(f);
							}
						}
					}
				}
			} else {
				UtilFTP ut = impostazioneDao.getUtilFTP();
				FTPClient vmftp = ApplicationUtil.createClient();
				if (ApplicationUtil.connect(vmftp, ut)) {
					try {
						FTPFile[] wavFiles = ApplicationUtil.listFiles(vmftp, ConstantsGlobal.path_attesa + "/");
						if (ApplicationUtil.notEmpty(wavFiles)) {
							for(FTPFile file : wavFiles) {
								Long time = ApplicationUtil.time(file.getTimestamp());
								if(mm.isRicerca(file.getName(), file.getSize(), time)) {
									MyFile f = new MyFile(file);
									f.setName(file.getName());
									f.setModified(time);
									ff.add(f);
								}
							}
						}
					} catch (Exception e) {
						ErrorLog.log(null, getClass(), e);
					} finally {
						ApplicationUtil.disconnect(vmftp);
					}
				}	
			}
		}
		return ff;
	}
	public void eliminaAttesaFtp(MyFile att) {
		if(ApplicationUtil.isNotTrue(impostazioneDao.getFTPEnabled())) {
			String home = impostazioneDao.getHome();
			File perc = new File(home + ConstantsGlobal.path_attesa + "/" + att.getName());
			ApplicationUtil.deleteFile(perc);
		} else {
			UtilFTP ut = impostazioneDao.getUtilFTP();
			FTPClient vmftp = ApplicationUtil.createClient();
			if (ApplicationUtil.connect(vmftp, ut)) {
				try {
					ApplicationUtil.deleteFile(vmftp, ConstantsGlobal.path_attesa + "/" + att.getName());
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				} finally {
					ApplicationUtil.disconnect(vmftp);
				}
			}	
		}
		autodialerDao.reloadModule("res_musiconhold");
	}
	public void eliminaAttesaFtp(List<MyFile> atts) {
		if(ApplicationUtil.isNotTrue(impostazioneDao.getFTPEnabled())) {
			if(ApplicationUtil.notEmpty(atts)) {
				String home = impostazioneDao.getHome();
				Iterator<MyFile> aIt = atts.iterator();
				while (aIt.hasNext()) {
					MyFile att = (MyFile) aIt.next();
					File perc = new File(home + ConstantsGlobal.path_attesa + "/" + att.getName());
					if(perc.exists()) {
						ApplicationUtil.deleteFile(perc);
					}
				}
			}
		} else {
			UtilFTP ut = impostazioneDao.getUtilFTP();
			FTPClient vmftp = ApplicationUtil.createClient();
			if (ApplicationUtil.connect(vmftp, ut)) {
				try {
					if(ApplicationUtil.notEmpty(atts)) {
						Iterator<MyFile> aIt = atts.iterator();
						while (aIt.hasNext()) {
							MyFile att = (MyFile) aIt.next();
							ApplicationUtil.deleteFile(vmftp, ConstantsGlobal.path_attesa + "/" + att.getName());
						}
					}
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				} finally {
					ApplicationUtil.disconnect(vmftp);
				}
			}	
		}
		autodialerDao.reloadModule("res_musiconhold");
	}
	public void scaricaAttesa(MyFile att, DownloadBean downloadBean) {
		if(ApplicationUtil.isNotTrue(impostazioneDao.getFTPEnabled())) {
			try {
				String home = impostazioneDao.getHome();
				OutputStream out = downloadBean.outputStream();
				downloadBean.filename(att.getName());
				String nome = home + ConstantsGlobal.path_attesa +"/"+ att.getName();
				InputStream istream = new FileInputStream(nome);
				ApplicationUtil.copy(istream, out);
				istream.close();
				downloadBean.close();
			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		} else {
			UtilFTP ut = impostazioneDao.getUtilFTP();
			FTPClient vmftp = ApplicationUtil.createClient();
			if (ApplicationUtil.connect(vmftp, ut)) {
				try {
					OutputStream out = downloadBean.outputStream();
					downloadBean.filename(att.getName());
					String path = ConstantsGlobal.path_attesa + "/" + att.getName();
					ApplicationUtil.scaricaFile(vmftp, out, path);
					downloadBean.close();
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				} finally {
					ApplicationUtil.disconnect(vmftp);
				}
			}	
		}
	}
	public void salvaAttesa(FileUpload fileUpload) {
		if(ApplicationUtil.isNotTrue(impostazioneDao.getFTPEnabled())) {
			String home = impostazioneDao.getHome();
			try {
				List<MyFile> fls = fileUpload.getFiles();
				if(ApplicationUtil.notEmpty(fls)) {
					Iterator<MyFile> flsIt = fls.iterator();
					while (flsIt.hasNext()) {
						MyFile ff = (MyFile) flsIt.next();
						ByteArrayOutputStream bos = new ByteArrayOutputStream();
						String fileN = ApplicationUtil.salvaAudio(ff, bos, false);
						byte [] ffB = bos.toByteArray();
						if(ffB != null  && ffB.length > 0) {
							FileOutputStream fi = new FileOutputStream(home + ConstantsGlobal.path_attesa + "/" + fileN);
							fi.write(ffB);
							fi.flush();
							fi.close();
						}
					}
				}
			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		} else {
			UtilFTP ut = impostazioneDao.getUtilFTP();
			FTPClient vmftp = ApplicationUtil.createClient();
			if (ApplicationUtil.connect(vmftp, ut)) {
				try {
					List<MyFile> fls = fileUpload.getFiles();
					if(ApplicationUtil.notEmpty(fls)) {
						Iterator<MyFile> flsIt = fls.iterator();
						while (flsIt.hasNext()) {
							MyFile ff = (MyFile) flsIt.next();
							ByteArrayOutputStream bos = new ByteArrayOutputStream();
							String fileN = ApplicationUtil.salvaAudio(ff, bos, false);
							byte [] ffB = bos.toByteArray();
							if(ffB != null  && ffB.length > 0) {
								InputStream  fis = new ByteArrayInputStream(ffB);
								ApplicationUtil.storeFile(vmftp, ConstantsGlobal.path_attesa + "/" + fileN, fis);
								fis.close();
							}
						}
					}
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				} finally {
					ApplicationUtil.disconnect(vmftp);
				}
			}	
		}
		autodialerDao.reloadModule("res_musiconhold");
	}
	public void salvaAllegatoContatto(FileUpload fileUpload, Anagrafica anagrafica) {
		try {
			List<MyFile> fls = fileUpload.getFiles();
			if(ApplicationUtil.notEmpty(fls)) {
				Iterator<MyFile> flsIt = fls.iterator();
				while (flsIt.hasNext()) {
					MyFile ff = (MyFile) flsIt.next();
					byte [] ffB = ff.getData();
					Allegato all = new Allegato(ff.getName(), ffB);
					all.setData_modifica(ApplicationUtil.date());
					all.setDimensione(ffB.length);
					if(!anagrafica.getAllegato().contains(all)){
						anagrafica.getAllegato().add(all);
					}
				}
			}
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void deleteAllegatoContatto(Allegato allegato, Anagrafica anagrafica) {
		anagrafica.getAllegato().remove(allegato);
	}
	public void scaricaAllegatoContatto(Allegato allegato, DownloadBean downloadBean) {
		try {
			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(allegato.getNome());
			InputStream is = new ByteArrayInputStream(allegato.getFile());
			ApplicationUtil.copy(is, out);
			downloadBean.close();
			is.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void deleteAllegatoContatto(Anagrafica anagrafica) {
		anagrafica.getAllegato().clear();
	}
	public void scaricaAllegatoContatto(Anagrafica anagrafica, DownloadBean downloadBean) {
		List<Allegato> allegati = anagrafica.getAllegato();
		if(ApplicationUtil.notEmpty(allegati)) {
			String p = ConstantsGlobal.secretZ;
			try {
				String file_header = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.CONTATTO.stampa()) + "." + ConstantsMap.zip;
				OutputStream out = downloadBean.outputStream();
				downloadBean.filename(file_header, allegati);
				ApplicationUtil.zip(null,allegati, out, p);
				downloadBean.close();
			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		}
	}
		
	@SuppressWarnings("unchecked")
	public List<Telecom> getTelecomVisibileByContattoId(Long contatto, boolean semaforo, Long userid) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Telecom.class);
		DetachedCriteria criteriaCO = criteria.createCriteria("contatto");
		DetachedCriteria criteriaCA = criteriaCO.createCriteria("campagna");
		impostazioneDao.subCriteriaCampagnaUser(semaforo, userid, criteriaCA, "id");
		criteriaCO.add(Restrictions.idEq(contatto));
		criteria.addOrder(Order.asc("data_invio"));
		return  getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Notifica> getNotificaVisibileByContattoId(Long contatto, boolean semaforo, Long userid) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Notifica.class);
		DetachedCriteria criteriaCO = criteria.createCriteria("contatto");
		DetachedCriteria criteriaCA = criteriaCO.createCriteria("campagna");
		impostazioneDao.subCriteriaCampagnaUser(semaforo, userid, criteriaCA, "id");
		criteriaCO.add(Restrictions.idEq(contatto));
		criteria.addOrder(Order.asc("data_invio"));
		return  getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Sondaggio> getSondaggioVisibileByContattoId(Long contatto, boolean semaforo, Long userid) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Sondaggio.class);
		DetachedCriteria criteriaCO = criteria.createCriteria("contatto");
		DetachedCriteria criteriaCA = criteriaCO.createCriteria("campagna");
		impostazioneDao.subCriteriaCampagnaUser(semaforo, userid, criteriaCA, "id");
		criteriaCO.add(Restrictions.idEq(contatto));
		criteria.addOrder(Order.asc("data_creazione"));
		return  getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Cdr> getCdrVisibileByContattoId(Long contatto, boolean semaforo, Long userid) {
		DetachedCriteria criteriaP = DetachedCriteria.forClass(Cdr.class);
		criteriaP.add(Restrictions.eq("contatto.id", contatto));
		criteriaP.setProjection(Projections.distinct(Projections.property("linkedid")));
		DetachedCriteria criteria = DetachedCriteria.forClass(Cdr.class);
		ApplicationUtil.lastapp(criteria);
		impostazioneDao.subPropertyInS(criteria, criteriaP, "linkedid");
		criteria.addOrder(Order.asc("calldate"));
		return  getList(criteria);
	}
	
	
	@SuppressWarnings("unchecked")
	public List<Telecom> getTelecomByContattoId(Long contatto) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Telecom.class);
		criteria.add(Restrictions.eq("contatto.id", contatto));
		criteria.addOrder(Order.asc("data_invio"));
		return  getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Notifica> getNotificaByContattoId(Long contatto) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Notifica.class);
		criteria.add(Restrictions.eq("contatto.id", contatto));
		criteria.addOrder(Order.asc("data_invio"));
		return  getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Sondaggio> getSondaggioByContattoId(Long contatto) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Sondaggio.class);
		criteria.add(Restrictions.eq("contatto.id", contatto));
		criteria.addOrder(Order.asc("data_creazione"));
		return  getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Cdr> getCdrByContattoId(Long contatto) {
		DetachedCriteria criteriaP = DetachedCriteria.forClass(Cdr.class);
		criteriaP.add(Restrictions.eq("contatto.id", contatto));
		criteriaP.setProjection(Projections.distinct(Projections.property("linkedid")));
		DetachedCriteria criteria = DetachedCriteria.forClass(Cdr.class);
		ApplicationUtil.lastapp(criteria);
		impostazioneDao.subPropertyInS(criteria, criteriaP, "linkedid");
		criteria.addOrder(Order.asc("calldate"));
		return  getList(criteria);
	}
	
	public void inviaTelecomScelti(Utente utente, List<Contatto> contatti, TelecomTemplate telecomTemplate,  Telecom telecom) {
		pannelloDao.inviaTelecom(contatti, utente, telecomTemplate, telecom);
	}
	public void inviaChiamataScelti(Utente utente, List<Contatto> contatti,  Date dataSchedulata) {
		pannelloDao.inviaChiamata(contatti, utente, dataSchedulata);
	}
	public List<SelectItem> getCampagnePartnerSI(Long utenteId, StatoIntervallo stato) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		ApplicationUtil.statoLC(stato, criteria, null);
		criteria.createCriteria("partner").add(Restrictions.idEq(utenteId));
		List<CampagnaSI> lis = getCampagnaSI(criteria);
		return impostazioneDao.getCampagna(lis, true, false, false, null);
	}
	public List<SelectItem> getCampagnePartnerSI(Long utenteId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		criteria.createCriteria("partner").add(Restrictions.idEq(utenteId));
		List<CampagnaSI> lis = getCampagnaSI(criteria);
		return impostazioneDao.getCampagna(lis, false, false, false, null);
	}
	public List<SelectItem> getCampagnePartnerMasterSI(Long utenteId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		criteria.add(Restrictions.eq("prenotabile_interno", Boolean.TRUE));
		criteria.createCriteria("partner").add(Restrictions.idEq(utenteId));
		List<CampagnaSI> lis = getCampagnaSI(criteria);
		return impostazioneDao.getCampagna(lis, false, false, false, null);
	}
	public void resetContatto(Boolean elimina_storia, Contatto contatto,  Utente utente, Date data) {
		EsitoChiamata esito = ConstantsEsito.DEFAULTS.get(ConstantsEsito.ID_DA_CHIAMARE);
		if(esito!=null && contatto != null) {
			nascondiStoria(elimina_storia, contatto);
			contatto.imposta(data);
			pannelloDao.salvaContattoC(true, null, contatto, utente, esito, false);
			String p = ApplicationUtil.asString( ConstantsLanguage.CONTATTO.stampa(), contatto);
			operazioneDao.creaOperazione(ConstantsMap.LOG5, p,  utente, ConstantsMap.OPERAZIONE_RIPROGRAMMA_RESET);
		}
	}
	public void esitaContatto(Boolean elimina_storia, Contatto contatto, Long esitoId, Utente utente) {
		EsitoChiamata esito = pannelloDao.getEsitoChiamataById(esitoId);
		if(esito!=null && contatto != null) {
			nascondiStoria(elimina_storia, contatto);
			pannelloDao.salvaContattoC(true, null, contatto, utente, esito, false);
			String p = ApplicationUtil.asString( ConstantsLanguage.CONTATTO.stampa(), contatto);
			operazioneDao.creaOperazione(ConstantsMap.LOG5, p, utente, ConstantsMap.OPERAZIONE_RIPROGRAMMA_GENERICO);
		}
	}
	public void archiviazioneContatto(Boolean elimina_storia, Contatto contatto, Long esitoId, Long operatore, Date data, Utente utente) {
		EsitoChiamata ok = ConstantsEsito.DEFAULTS.get(ConstantsEsito.ID_OK);
		EsitoArchiviazione esito = pannelloDao.getEsitoArchiviazioneById(esitoId);
		Utente op = null;
		if(ApplicationUtil.gtZero(operatore)) {
			op = campagnaDao.getOperatore(operatore);
		}
		if(esito != null && contatto != null) {
			nascondiStoria(elimina_storia, contatto);
			contatto.setEsitoChiamata(ok);
			contatto.setEsitoArchiviazione(esito);
			if (ApplicationUtil.gtZero(esito.getTipologiaS())) {
				if (ApplicationUtil.eq(esito.getTipologiaS(), 1)) {
					contatto.proprietario(data, op);
					contatto.archivia(true);
				}
				if (ApplicationUtil.eq(esito.getTipologiaS(), 2)) {
					contatto.proprietario(data, null);
					contatto.pubblico();
				}
				if (ApplicationUtil.eq(esito.getTipologiaS(), 3)) {
					data = ApplicationUtil.date();
					contatto.proprietario(ApplicationUtil.addMinutes(data, esito.getAttesa()), null);
					contatto.pubblico();
				}
			} else {
				contatto.archivia(false);
			}
			pannelloDao.salvaContatto(true, contatto, utente);
			String p = ApplicationUtil.asString( ConstantsLanguage.CONTATTO.stampa(), contatto);
			operazioneDao.creaOperazione(ConstantsMap.LOG5, p, utente, ConstantsMap.OPERAZIONE_RIPROGRAMMA_GENERICO);
		}
	}
	public void revertStoriaContatto(Contatto  contatto, StoriaContatto storia) {
		Campagna c = storia.getCampagna();
		if(c != null) {
			contatto.setCampagna(c);
		}
		Lista l = storia.getLista();
		if(l != null) {
			contatto.setLista(l);
		}
		contatto.setSequenziale(storia.getSequenziale());
		contatto.setBloccato(storia.getBloccato());
		contatto.setData_lavorazione(storia.getData());
		contatto.setUtente(storia.getOperatore());
		contatto.setNota(storia.getNota());
		contatto.setTipo(storia.getTipo());
		contatto.setEsitoChiamata(storia.getEsito());
		contatto.setEsitoArchiviazione(storia.getArchiviazione());
		contatto.setArchiviato(storia.getArchiviato());
		contatto.setData_archiviazione(storia.getData_archiviazione());
		contatto.setData_presentazione(storia.getData_presentazione());
		contatto.setProprietario(storia.getProprietario());
		contatto.setAgente(storia.getAgente());
	}
	public void salvaContatti(boolean w, List<Contatto> contatti, Utente utente, boolean a) {
		if(ApplicationUtil.notEmpty(contatti)) {
			Date data = ApplicationUtil.date();
			Session session = openSession();
			Transaction tx = session.beginTransaction();
			int i = 0;
			Iterator<Contatto> objIter = contatti.iterator();
			while(objIter.hasNext()) {
				Contatto contatto = objIter.next();
				contatto.setData_lavorazione(data);
				contatto.setUtente(utente);
				if(w) {
					anagraficaDao.evaluateWorkflow(session, utente, contatto);
				}
				session.update(contatto);
				if(a) {
					session.update(contatto.getAnagrafica());
				}
				StoriaContatto storia = pannelloDao.creaStoriaContatto(contatto, utente, data, false, false);
				session.save(storia);
				if (i++ % ConstantsGlobal.flushSize == 0) {
					session.flush();
					session.clear();
				}
			}
			tx.commit();
			closeSession(session);

		}
	}
	public void revertContatto(boolean w, List<Contatto> contatti, Utente utente, Date data, Integer step, Boolean modo) {
		if(ApplicationUtil.notEmpty(contatti)) {
			List<Contatto> contattiList = new ArrayList<Contatto>();
			Session session = openSession();
			Transaction tx = session.beginTransaction();
			int i = 0;
			Iterator<Contatto> objIter = contatti.iterator();
			while(objIter.hasNext()) {
				Contatto old = objIter.next();
				Contatto contatto = (Contatto) session.get(Contatto.class, old.getId());
				Hibernate.initialize(contatto.getStoria());
				List<StoriaContatto> storia = contatto.getStoria(); 
				if(ApplicationUtil.notEmpty(storia)) {
					List<StoriaContatto> removeList = new ArrayList<StoriaContatto>();
					Iterator<StoriaContatto> objSt = storia.iterator();
					int j = 0;
					while(objSt.hasNext()) {
						StoriaContatto st = objSt.next();
						if(ApplicationUtil.isTrue(modo)) {
							if(j+step > storia.size()-1) {
								removeList.add(st);
							}
							j++;
						} else {
							if(st.getData().after(data)) {
								removeList.add(st);
							}
						}
					}
					if(ApplicationUtil.notEmpty(removeList)){
						boolean b = storia.size() == removeList.size();
						if(b) {
							removeList.remove(0);
						} 
						storia.removeAll(removeList);
					}
					if(ApplicationUtil.notEmpty(storia)) {
						if(ApplicationUtil.notEmpty(removeList)){
							Iterator<StoriaContatto> objRm = removeList.iterator();
							while(objRm.hasNext()) {
								session.delete(objRm.next());
							}
						}
						StoriaContatto el1 = storia.remove(storia.size()-1);
						anagraficaDao.spostaAnagrafica(session, true, contatto, el1);
						revertStoriaContatto(contatto, el1);
						if(w) {
							anagraficaDao.evaluateWorkflow(session, utente, contatto);
						}
						session.update(contatto);
						contattiList.add(contatto);
						if (i++ % ConstantsGlobal.flushSize == 0) {
							session.flush();
							session.clear();
						}
					}
				}
			}
			tx.commit();
			closeSession(session);

			String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.CONTATTO.stampa() +": " + ApplicationUtil.size(contattiList), contattiList);
			operazioneDao.creaOperazione(ConstantsMap.LOG5, p, utente, ConstantsMap.OPERAZIONE_REVERT_CONTATTO);
		}
	}
	public void revertContatto(boolean w, Contatto contatto, Utente utente, Date data, Integer step, Boolean modo) {
		if(contatto != null) {
			List<Contatto> contatti = new ArrayList<Contatto>();
			contatti.add(contatto);
			revertContatto(w, contatti, utente, data, step, modo);
		}
	}
	public List<ContattoVO>  nuovaListaContattoVO() {
		return new ArrayList<ContattoVO>();
	}
	public List<ContattoVO>  salvaContattiVO(TemplateImportContatto ti, Long campagnaR, Boolean esistente, Lista listaM, Boolean sposta, Long campagnaId, Long listaId, ImportModelContatto tiM, Utente utente, Long esitoChiamata,Long esitoArchiviazione, Long operatore, Integer modifica, Boolean elimina_storia,  Boolean elimina_nota, Date data, Integer step, Boolean modo) {
		List<ContattoVO> nonTrovati = new ArrayList<ContattoVO>();
		if(ti != null) {
			boolean a = false;
			List<Contatto> contattiNew = tiM.getElementi();
			if(ApplicationUtil.eq(modifica, 1) || ApplicationUtil.eq(modifica, 3)) {
				if (ApplicationUtil.notEmpty(contattiNew)) {
					Utente op = null;
					EsitoChiamata esC = null;
					EsitoArchiviazione esA = null;
					if(ApplicationUtil.eq(modifica, 3)) {
						esC = campagnaDao.getEsitoChiamata(ConstantsEsito.ID_DA_CHIAMARE);
						esA = null;
						op = null;
						campagnaId = null;
						listaId = null;
					} else {
						if(ApplicationUtil.gtZero(esitoChiamata)) {
							esC = campagnaDao.getEsitoChiamata(esitoChiamata);
						}
						if(ApplicationUtil.gtZero(esitoArchiviazione)) {
							esA = campagnaDao.getEsitoArchiviazione(esitoArchiviazione);
						}
						if(ApplicationUtil.gtZero(operatore)) {
							op = campagnaDao.getOperatore(operatore);
						}
					}
					List<String> valori = ti.getValore_multiplo();
					List<Contatto>  contr = prendiEsistenti(contattiNew, tiM.getTipo(), campagnaR);
					List<Contatto> ll = new ArrayList<Contatto>();
					for (Contatto appNew : contattiNew) {
						List<Contatto> ctrs = trovaContatto(contr, appNew.getChiaveL(tiM.getTipo()), tiM.getTipo());
						if (ApplicationUtil.notEmpty(ctrs)) {
							for (Contatto ctr : ctrs) {
								if (!appNew.getDuplicato() && !ll.contains(ctr)) {
									ll.add(ctr);
									if(esC!= null) {
										if(esC.getId().equals(ConstantsEsito.ID_DA_CHIAMARE)) {
											ctr.imposta(data, esC, null);
										} 					
										if(esC.getId().equals(ConstantsEsito.ID_RICHIAMO_PUBBLICO)) {
											ctr.imposta(data, esC, null);
										}
										if(esC.getId().equals(ConstantsEsito.ID_RICHIAMO_PRIVATO)) {
											ctr.imposta(data, esC, op);
										}
										if(esC.getId().equals(ConstantsEsito.ID_OK)) {
											ctr.setEsitoChiamata(esC);
											ctr.setEsitoArchiviazione(esA);
											if (esA != null && ApplicationUtil.gtZero(esA.getTipologiaS())) {
												if (ApplicationUtil.eq(esA.getTipologiaS(), 1)) {
													ctr.proprietario(data, op);
													ctr.archivia(true);
												}
												if (ApplicationUtil.eq(esA.getTipologiaS(), 2)) {
													ctr.proprietario(data, null);
													ctr.pubblico();
												}
												if (ApplicationUtil.eq(esA.getTipologiaS(), 3)) {
													data = ApplicationUtil.date();
													ctr.proprietario(ApplicationUtil.addMinutes(data, esA.getAttesa()), null);
													ctr.pubblico();
												}
											} else {
												ctr.archivia(false);
											}
										}
										if(!esC.getId().equals(ConstantsEsito.ID_RICHIAMO_PUBBLICO) && !esC.getId().equals(ConstantsEsito.ID_DA_CHIAMARE) && !esC.getId().equals(ConstantsEsito.ID_OK) && !esC.getId().equals(ConstantsEsito.ID_RICHIAMO_PRIVATO)) {
											pannelloDao.esitaContattoC(ctr, esC, utente);
										}
									}
									if(ApplicationUtil.isTrue(elimina_nota)) {
										ctr.setNota(null);
									}
									Anagrafica anagraficaO = ctr.getAnagrafica();
									Anagrafica anagraficaN = appNew.getAnagrafica();
									if(valori.indexOf(ConstantsMap.contatto_nota) > -1) {
										ctr.setNota(appNew.getNota());
									}
									if(valori.indexOf(ConstantsMap.consenso_tfissoS) > -1) {
										a = true;
										anagraficaO.setConsenso_tfisso(anagraficaN.getConsenso_tfisso());
									}
									if(valori.indexOf(ConstantsMap.consenso_tmobileS) > -1) {
										a = true;
										anagraficaO.setConsenso_tmobile(anagraficaN.getConsenso_tmobile());
									}
									if(valori.indexOf(ConstantsMap.consenso_tfaxS) > -1) {
										a = true;
										anagraficaO.setConsenso_tfax(anagraficaN.getConsenso_tfax());
									}
									if(valori.indexOf(ConstantsMap.consenso_taltroS) > -1) {
										a = true;
										anagraficaO.setConsenso_taltro(anagraficaN.getConsenso_taltro());
									}
									if(valori.indexOf(ConstantsMap.consenso_emailS) > -1) {
										a = true;
										anagraficaO.setConsenso_email(anagraficaN.getConsenso_email());
									}
									
									if(valori.indexOf(ConstantsMap.consenso_rmailS) > -1) {
										a = true;
										anagraficaO.setConsenso_rmail(anagraficaN.getConsenso_rmail());
									}
									
								} else {
									nonTrovati.add(new ContattoVO(appNew.getChiave(), ApplicationUtil.upperCaseEmpty(ConstantsLanguage.DUPLICATO.stampa()),esC,esA, op, appNew.getNota()));
								}
							}
						} else {
							nonTrovati.add(new ContattoVO(appNew.getChiave(), ApplicationUtil.upperCaseEmpty(ConstantsLanguage.NON_TROVATO.stampa()),esC,esA, op, appNew.getNota()));
						}
					}
					if(ApplicationUtil.notEmpty(ll)) {
						nascondiStoria(elimina_storia, ll);
						anagraficaDao.cambiaAnagrafiche(esistente, listaM, sposta, utente, ll, listaId, campagnaId);
						salvaContatti(false, ll, utente, a);
						if(esC!= null) {
							String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.CONTATTO.stampa() +": " + ApplicationUtil.size(ll), ll);
							if(esC.getId().equals(ConstantsEsito.ID_DA_CHIAMARE)) {
								operazioneDao.creaOperazione(ConstantsMap.LOG5,p,  utente, ConstantsMap.OPERAZIONE_RIPROGRAMMA_RESET);
							} 					
							if(esC.getId().equals(ConstantsEsito.ID_RICHIAMO_PUBBLICO)) {
								operazioneDao.creaOperazione(ConstantsMap.LOG5, p,  utente, ConstantsMap.OPERAZIONE_RIPROGRAMMA_PUBBLICO);
							}
							if(esC.getId().equals(ConstantsEsito.ID_RICHIAMO_PRIVATO)) {
								operazioneDao.creaOperazione(ConstantsMap.LOG5, p,  utente, ConstantsMap.OPERAZIONE_RIPROGRAMMA_PRIVATO);
							}
							if(esC.getId().equals(ConstantsEsito.ID_OK)) {
								operazioneDao.creaOperazione(ConstantsMap.LOG5, p,  utente, ConstantsMap.OPERAZIONE_RIPROGRAMMA_GENERICO);
							}
							if(!esC.getId().equals(ConstantsEsito.ID_RICHIAMO_PUBBLICO) && !esC.getId().equals(ConstantsEsito.ID_DA_CHIAMARE) && !esC.getId().equals(ConstantsEsito.ID_OK) && !esC.getId().equals(ConstantsEsito.ID_RICHIAMO_PRIVATO)) {
								operazioneDao.creaOperazione(ConstantsMap.LOG5, p,  utente, ConstantsMap.OPERAZIONE_RIPROGRAMMA_GENERICO);
							}
						}
					}
				}
			} 
			if(ApplicationUtil.eq(modifica, 2)) {
				if (ApplicationUtil.notEmpty(contattiNew)) {
					List<Contatto>  contr = prendiEsistenti(contattiNew, tiM.getTipo(), campagnaR);
					List<Contatto> ll = new ArrayList<Contatto>();
					for (Contatto appNew : contattiNew) {
						List<Contatto> ctrs = trovaContatto(contr, appNew.getChiaveL(tiM.getTipo()), tiM.getTipo());
						if (ApplicationUtil.notEmpty(ctrs)) {
							for (Contatto ctr : ctrs) {
								if (!appNew.getDuplicato() && !ll.contains(ctr)) {
									ll.add(ctr);
								} else {
									nonTrovati.add(new ContattoVO(appNew.getChiave(), ApplicationUtil.upperCaseEmpty(ConstantsLanguage.DUPLICATO.stampa()), null,  null,  null, null));
								}
							}
						} else {
							nonTrovati.add(new ContattoVO(appNew.getChiave(), ApplicationUtil.upperCaseEmpty(ConstantsLanguage.NON_TROVATO.stampa()), null, null, null,  null));
						}
					}
					if(ApplicationUtil.notEmpty(ll)) {
						String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.CONTATTO.stampa() +": " + ApplicationUtil.size(ll), ll);
						operazioneDao.creaOperazione(ConstantsMap.LOG3, p, utente, ConstantsMap.OPERAZIONE_ELIMINA_CONTATTO);
						eliminaContatti(ll);
					}
				}
			}
			if(ApplicationUtil.eq(modifica, 4)) {
				if (ApplicationUtil.notEmpty(contattiNew)) {
					List<Contatto>  contr = prendiEsistenti(contattiNew, tiM.getTipo(), campagnaR);
					List<Contatto> ll = new ArrayList<Contatto>();
					for (Contatto appNew : contattiNew) {
						List<Contatto> ctrs = trovaContatto(contr, appNew.getChiaveL(tiM.getTipo()), tiM.getTipo());
						if (ApplicationUtil.notEmpty(ctrs)) {
							for (Contatto ctr : ctrs) {
								if (!appNew.getDuplicato() && !ll.contains(ctr)) {
									ll.add(ctr);
								} else {
									nonTrovati.add(new ContattoVO(appNew.getChiave(), ApplicationUtil.upperCaseEmpty(ConstantsLanguage.DUPLICATO.stampa()), null,  null,  null, null));
								}
							}
						} else {
							nonTrovati.add(new ContattoVO(appNew.getChiave(), ApplicationUtil.upperCaseEmpty(ConstantsLanguage.NON_TROVATO.stampa()), null, null, null,  null));
						}
					}
					if(ApplicationUtil.notEmpty(ll)) {
						revertContatto(false, ll, utente, data, step, modo);
					}
				}
			}
			if(ApplicationUtil.eq(modifica, 5)) {
				if (ApplicationUtil.notEmpty(contattiNew)) {
					List<Contatto>  contr = prendiEsistenti(contattiNew, tiM.getTipo(), campagnaR);
					List<Contatto> ll = new ArrayList<Contatto>();
					for (Contatto appNew : contattiNew) {
						List<Contatto> ctrs = trovaContatto(contr, appNew.getChiaveL(tiM.getTipo()), tiM.getTipo());
						if (ApplicationUtil.notEmpty(ctrs)) {
							for (Contatto ctr : ctrs) {
								if (!appNew.getDuplicato() && !ll.contains(ctr)) {
									ll.add(ctr);
								} else {
									nonTrovati.add(new ContattoVO(appNew.getChiave(), ApplicationUtil.upperCaseEmpty(ConstantsLanguage.DUPLICATO.stampa()), null,  null,  null, null));
								}
							}
						} else {
							nonTrovati.add(new ContattoVO(appNew.getChiave(), ApplicationUtil.upperCaseEmpty(ConstantsLanguage.NON_TROVATO.stampa()), null, null, null,  null));
						}
					}
					if(ApplicationUtil.notEmpty(ll)) {
						anagraficaDao.updateAnagrafiche(esistente, listaM, sposta, utente, ll, listaId, campagnaId);
					}
				}
			}
		}
		return nonTrovati;
	}
	private List<Contatto> trovaContatto( List<Contatto> contatti, Object seriale, String tipo) {
		List<Contatto> o = new ArrayList<Contatto>();
		if(ApplicationUtil.notEmpty(contatti)) {
			for (Contatto contatto: contatti) {
				Object serial = contatto.getChiaveO(tipo);
				if (serial != null && seriale != null && serial.equals(seriale)) {
					o.add(contatto);
				}
			}
		}
		return o;
	}
	@SuppressWarnings("unchecked")
	public List<Contatto> prendiEsistenti(List<Contatto> conts, String tipo, Long campagna) {
		List<Object> contattiL = new ArrayList<Object>();
		if(ApplicationUtil.notEmpty(conts)) {
			for (Contatto cont : conts) {
				Object seriale = cont.getChiaveL(tipo);
				if(seriale!= null && !contattiL.contains(seriale) && ApplicationUtil.isTrue(cont.getTrovato())) {
					contattiL.add(seriale);
				}
			}
		}
		if(ApplicationUtil.notEmpty(contattiL))		{
			DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class);
			if(tipo != null && !ConstantsMap.contatto_id.equals(tipo)) {
				String tipoS = tipo.substring((ConstantsMap.anagrafica + "_").length());
				criteria.createCriteria("anagrafica").add(Restrictions.in(tipoS, contattiL)); 
			} else {
				criteria.add(Restrictions.in("id", contattiL)); 
			}
			if(ApplicationUtil.gtZero(campagna)) {
				criteria.add(Restrictions.eq("campagna.id", campagna));
			}
			return getList(criteria);
		}
		return null;
	}
	public List<SelectItem> getEsitiContatto() {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoChiamata.class);
		List<EsitoChiamataSI> lis = getEsitoChiamataSI(criteria);
		return impostazioneDao.getEsitoChiamata(lis, false, null);
	}
	public List<SelectItem> getEsitiChiamata() {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoChiamata.class);
		if(!impostazioneDao.getResetContatto()) {
			criteria.add(Restrictions.ne("id", ConstantsEsito.ID_DA_CHIAMARE));
		}
		List<EsitoChiamataSI> lis = getEsitoChiamataSI(criteria);
		return impostazioneDao.getEsitoChiamata(lis, false, null);
	}
	public List<SelectItem> getEsitiArchiviazione() {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoArchiviazione.class);
		List<EsitoArchiviazioneSI> lis = getEsitoArchiviazioneSI(criteria);
		return impostazioneDao.getEsitoArchiviazione(lis, false, null);
	}
	public Contatto getContatto(Utente utente, MyFile rec) {
		Contatto contatto = (Contatto) get(Contatto.class, rec.getId());
		String pg = ApplicationUtil.asString(ConstantsLanguage.CONTATTO.stampa(), contatto);
		operazioneDao.creaOperazione(ConstantsMap.LOG7, pg ,utente, ConstantsMap.OPERAZIONE_ELEMENTO_CONTATTO);
		return contatto;
	}
	@SuppressWarnings("unchecked")
	public List<MyFile> ricercaContattoRegistrazioni(Utente user, RegistrazioneReportCriteria cont) {
		if(cont != null)  cont = (RegistrazioneReportCriteria) cont.loadCriteria();
		if(cont != null && cont.getCerca()) {
			List<Long> ids = new ArrayList<Long>();
			List<MyFile> mf = listaRegistrazioni(cont, ids);
			if(ApplicationUtil.notEmpty(ids)) {
				DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class);
				criteria.add(Restrictions.in("id", ids));
				DetachedCriteria criteriaC = criteria.createCriteria("campagna");
				if(user!= null) {
					impostazioneDao.subCriteriaCampagnaSupervisore(user.getId(), criteriaC, "id");
				}
				if(ApplicationUtil.gtZero(cont.getCampagnaId())) {
					criteriaC.add(Restrictions.idEq(cont.getCampagnaId()));
				}
				if (ApplicationUtil.isNotBlank(cont.getRicerca_anagrafica())) {
					DetachedCriteria criteriaA = criteria.createCriteria("anagrafica");
					ApplicationUtil.anagrafica(cont.getRicerca_anagraficaV(), criteriaA, cont.getRicerca_anagrafica());
				}
				if(ApplicationUtil.gtZero(cont.getEsitoChiamataId())) {
					criteria.add(Restrictions.eq("esitoChiamata.id",cont.getEsitoChiamataId()));
				}
				criteria.setProjection((Projections.distinct(Projections.property("id"))));
				return prendiRegistrazioniEsistenti(mf, getList(criteria));
			}
		}
		return null;
	}
	private List<MyFile> prendiRegistrazioniEsistenti(List<MyFile> mfl, List<Long> idsEsitenti) {
		List<MyFile> ff = new ArrayList<MyFile>();
		if(ApplicationUtil.notEmpty(mfl) && ApplicationUtil.notEmpty(idsEsitenti)) {
			Iterator<MyFile> it = mfl.iterator();
			while (it.hasNext()) {
				MyFile mf = (MyFile) it.next();
				if(mf.getId() != null && idsEsitenti.contains(mf.getId()) && !ff.contains(mf)) {
					ff.add(mf);
				}
			}
		}
		return ff;
	}
	private List<MyFile> listaRegistrazioni(RegistrazioneReportCriteria mm, List<Long> ids) {
		List<MyFile> ff = new ArrayList<MyFile>();
		String monitor = ConstantsGlobal.path_monitor;
		if(ApplicationUtil.isNotTrue(impostazioneDao.getFTPEnabled())) {
			String home = impostazioneDao.getHome();
			File perc = new File(home + monitor);
			if(perc.exists()) {
				Path folder = Paths.get(perc.getAbsolutePath());
				DirectoryStream<Path> stream = ApplicationUtil.streamFile(folder, "." + ConstantsMap.wav, mm);
				if(stream != null) {
					for(Path entry : stream) {
						File file = entry.toFile();
						String a = ApplicationUtil.substringAfterLast(file.getName(), "_");
						String b = ApplicationUtil.trimToNull(ApplicationUtil.substringBeforeLast(a, "." + ConstantsMap.wav));
						if(b != null) {
							MyFile f = new MyFile(file);
							f.setName(file.getName());
							f.setModified(file.lastModified());
							Long id = null;
							try {
								id = Long.parseLong(b);
							} catch (Exception nfe) {
							}
							if(id != null) {
								if(!ff.contains(f)) {
									f.setId(id);
									ff.add(f);
								}
								if(!ids.contains(id)) {
									ids.add(id);
								}
							}
						}
					}
				}
			}
		} else {
			UtilFTP ut = impostazioneDao.getUtilFTP();
			FTPClient vmftp = ApplicationUtil.createClient();
			if (ApplicationUtil.connect(vmftp, ut)) {
				try {
					FTPFile[] wavFiles = ApplicationUtil.listMonitor(vmftp, monitor + "/", null, ConstantsMap.wav);
					if (ApplicationUtil.notEmpty(wavFiles)) {
						for(FTPFile file : wavFiles) {
							Long time = ApplicationUtil.time(file.getTimestamp());
							if(mm.isRicerca(file.getName(), file.getSize(), time)) {
								String a = ApplicationUtil.substringAfterLast(file.getName(), "_");
								String b = ApplicationUtil.trimToNull(ApplicationUtil.substringBeforeLast(a, "." + ConstantsMap.wav));
								if(b != null) {
									MyFile f = new MyFile(file);
									f.setName(file.getName());
									f.setModified(time);
									Long id = null;
									try {
										id = Long.parseLong(b);
									} catch (Exception nfe) {
									}
									if(id != null) {
										if(!ff.contains(f)) {
											f.setId(id);
											ff.add(f);
										}
										if(!ids.contains(id)) {
											ids.add(id);
										}
									}
								}
							}
						}
					}
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				} finally {
					ApplicationUtil.disconnect(vmftp);
				}
			}	
		}
		return ff;
	}
	@SuppressWarnings("unchecked")
	public List<MyFile> ricercaContattoPDA(Utente user, PDAReportCriteria cont) {
		if(cont != null)  cont = (PDAReportCriteria) cont.loadCriteria();
		if(cont != null && cont.getCerca()) {
			List<Long> ids = new ArrayList<Long>();
			List<MyFile> mf = listaPDA(cont, ids);
			if(ApplicationUtil.notEmpty(ids)) {
				DetachedCriteria criteria = DetachedCriteria.forClass(Contratto.class);
				DetachedCriteria criteriaCC = criteria.createCriteria("contatto");
				criteria.add(Restrictions.in("id", ids));
				DetachedCriteria criteriaC = criteriaCC.createCriteria("campagna");
				if(user!= null) {
					impostazioneDao.subCriteriaCampagnaSupervisore(user.getId(), criteriaC, "id");
				}
				if(ApplicationUtil.gtZero(cont.getCampagnaId())) {
					criteriaC.add(Restrictions.idEq(cont.getCampagnaId()));
				}
				if (ApplicationUtil.isNotBlank(cont.getRicerca_anagrafica())) {
					DetachedCriteria criteriaA = criteriaCC.createCriteria("anagrafica");
					ApplicationUtil.anagrafica(cont.getRicerca_anagraficaV(), criteriaA, cont.getRicerca_anagrafica());
				}
				if(ApplicationUtil.gtZero(cont.getEsitoChiamataId())) {
					criteriaCC.add(Restrictions.eq("esitoChiamata.id",cont.getEsitoChiamataId()));
				}
				criteria.setProjection((Projections.distinct(Projections.property("id"))));
				return prendiPDAEsistenti(mf, getList(criteria));
			}
		}
		return null;
	}
	private List<MyFile> prendiPDAEsistenti(List<MyFile> mfl, List<Long> idsEsitenti) {
		List<MyFile> ff = new ArrayList<MyFile>();
		if(ApplicationUtil.notEmpty(mfl) && ApplicationUtil.notEmpty(idsEsitenti)) {
			Iterator<MyFile> it = mfl.iterator();
			while (it.hasNext()) {
				MyFile mf = (MyFile) it.next();
				if(mf.getId() != null && idsEsitenti.contains(mf.getId()) && !ff.contains(mf)) {
					ff.add(mf);
				}
			}
		}
		return ff;
	}
	private List<MyFile> listaPDA(PDAReportCriteria mm, List<Long> ids) {
		List<MyFile> ff = new ArrayList<MyFile>();
		File perc = new File(impostazioneDao.getPathBase(ConstantsGlobal.path_pda));
		if(perc.exists() && perc.isDirectory()) {
			File [] files = perc.listFiles();
			if(ApplicationUtil.notEmpty(files)) {
				List<String> rs = ApplicationUtil.token(mm.getFullText(), ConstantsGlobal.colonSeparator);
				for(File file : files) {
					if(mm.isRicerca(file.getName(), file.length(), file.lastModified()) && (ApplicationUtil.textInPDF(file, rs) || ApplicationUtil.textInXLS(file, rs))) {
						String a = ApplicationUtil.substringAfter(file.getName(), "_");
						String b = ApplicationUtil.trimToNull(ApplicationUtil.substringBefore(a, "_"));
						if(b != null) {
							MyFile f = new MyFile(file);
							f.setName(file.getName());
							f.setModified(file.lastModified());
							Long id = null;
							try {
								id = Long.parseLong(b);
							} catch (Exception nfe) {
							}
							if(id != null) {
								if(!ff.contains(f)) {
									f.setId(id);
									ff.add(f);
								}
								if(!ids.contains(id)) {
									ids.add(id);
								}
							}
						}
					}
				}
			}
		}
		return ff;
	}
	public void bloccaContattiSelezionati(Utente utente, List<Contatto> contatti, boolean blocca) {
		if (ApplicationUtil.notEmpty(contatti)) {
			Iterator<Contatto> it = contatti.iterator();
			while (it.hasNext()) {
				Contatto contatto = (Contatto) it.next();
				bloccaContatto(utente, contatto, blocca);
			}
		} 
	}
	public void bloccaContatto(Utente utente, Contatto contatto, boolean blocca) {
		if (contatto != null ) {
			contatto.setBloccato(blocca);
			salvaContatto(false, contatto, utente, ApplicationUtil.date());
		} 
	}
	public List<MyFile> listaRegistrazioni(Utente utente, RegistrazioneCriteria mm) {
		if(mm != null)  mm = (RegistrazioneCriteria) mm.loadCriteria();
		List<MyFile> ff = new ArrayList<MyFile>();
		String monitor = ConstantsGlobal.path_monitor;
		if(mm != null && mm.getCerca()) {
			if(ApplicationUtil.isNotTrue(impostazioneDao.getFTPEnabled())) {
				String home = impostazioneDao.getHome();
				File perc = new File(home + monitor);
				if(perc.exists()) {
					Path folder = Paths.get(perc.getAbsolutePath());
					DirectoryStream<Path> stream = ApplicationUtil.streamFile(folder, "." + ConstantsMap.wav, mm);
					if(stream != null) {
						for(Path entry : stream) {
							File file = entry.toFile();
							if(ApplicationUtil.fitraUtente(file.getName(), utente.getId())) {
								MyFile f = new MyFile(file);
								f.setName(file.getName());
								f.setModified(file.lastModified());
								String a = ApplicationUtil.substringAfterLast(file.getName(), "_");
								String b = ApplicationUtil.trimToNull(ApplicationUtil.substringBeforeLast(a, "." + ConstantsMap.wav));
								if(b != null) {
									Long id = null;
									try {
										id = Long.parseLong(b);
									} catch (Exception nfe) {
									}
									if(id != null) {
										f.setId(id);
									}
								}
								ff.add(f);
							}
						}
					}
				}
			} else {
				UtilFTP ut = impostazioneDao.getUtilFTP();
				FTPClient vmftp = ApplicationUtil.createClient();
				if (ApplicationUtil.connect(vmftp, ut)) {
					try {
						FTPFile[] wavFiles = ApplicationUtil.listMonitor(vmftp, monitor + "/", null, ConstantsMap.wav);
						if (ApplicationUtil.notEmpty(wavFiles)) {
							for(FTPFile file : wavFiles) {
								Long time = ApplicationUtil.time(file.getTimestamp());
								if(mm.isRicerca(file.getName(), file.getSize(), time) && ApplicationUtil.fitraUtente(file.getName(), utente.getId())) {
									MyFile f = new MyFile(file);
									f.setName(file.getName());
									f.setModified(time);
									String a = ApplicationUtil.substringAfterLast(file.getName(), "_");
									String b = ApplicationUtil.trimToNull(ApplicationUtil.substringBeforeLast(a, "." + ConstantsMap.wav));
									if(b != null) {
										Long id = null;
										try {
											id = Long.parseLong(b);
										} catch (Exception nfe) {
										}
										if(id != null) {
											f.setId(id);
										}
									}
									ff.add(f);
								}
							}
						}
					} catch (Exception e) {
						ErrorLog.log(null, getClass(), e);
					} finally {
						ApplicationUtil.disconnect(vmftp);
					}
				}	
			}
		}
		return ff;
	}


	public void sondaggio(String contattoId,  String utenteId, String votoS, String userfield){
		try {
			Long c = Long.parseLong(contattoId);
			Long u = Long.parseLong(utenteId);
			Integer voto = Integer.parseInt(votoS);
			Contatto contatto = (Contatto) get(Contatto.class, c);
			Utente utente = (Utente) get(Utente.class, u);
			if(contatto != null && utente != null) {
				Sondaggio sondaggio = new Sondaggio();
				sondaggio.setUserfield(userfield);
				sondaggio.setContatto(contatto);
				sondaggio.setUtente(utente);
				sondaggio.setVoto(voto);
				salvaAggiorna(sondaggio);
			}
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}


	public Contatto updateContatto(Contatto contatto,  Anagrafica anagrafica){
		if(!ApplicationUtil.equals(contatto.getAnagrafica().getTelefonoC(), anagrafica.getTelefonoC())) {
			contatto.setTelefono(null);
		}
		contatto.setAnagrafica(anagrafica);
		return contatto;
	}
	
	public Contatto getContatto(String telefono, Long camp, Boolean pubbliche, Boolean soloAperte, String tipo) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class);
		if(ApplicationUtil.isTrue(pubbliche)) {
			criteria.add(Restrictions.isNull("proprietario.id"));
		}
		if(ApplicationUtil.isTrue(soloAperte)) {
			criteria.add(Restrictions.eq("archiviato", Boolean.FALSE));
		}
		DetachedCriteria criteriaA = criteria.createCriteria("anagrafica");
		if(tipo != null && ConstantsMap.telefonoFMap.contains(tipo)) {
			ApplicationUtil.telefonoE(criteriaA, telefono, tipo);
		} else {
			ApplicationUtil.telefonoE(criteriaA, telefono);
		}
		criteria.add(Restrictions.eq("campagna.id", camp));
		criteria.addOrder(Order.desc("id"));
		return (Contatto) getUniqueResult(criteria);
	}
	
	public Contatto getContatto(Long id) {
		Contatto c = null;
		if(id != null) {
			c = get(id);
		}
		return c;
	}
		
	public Contatto salvaContatto(String tipo, boolean nuovaAnagrafica, String telefono, Long camp, EsitoChiamata esito, String url)  {
		Anagrafica anagrafica = nuovaAnagrafica ? null : anagraficaDao.getAnagrafica(telefono, tipo);
		String text = null;
		if(anagrafica == null) {
			anagrafica =  new Anagrafica();
			if(url != null) {
				text = ApplicationUtil.downloadLink(url);
				if(text != null) {
					InputStream stream = new ByteArrayInputStream(text.getBytes(ConstantsGlobal.CHARSET));
					List<AnagraficaXML> anaList = new ArrayList<AnagraficaXML>();
					try {
						if(url.endsWith(ConstantsMap.xml)) {
							ApplicationUtil.xmlDecoder(anaList, stream);
						} 
						if(url.endsWith(ConstantsMap.json)) {
							ApplicationUtil.jsonDecoder(anaList, stream);
						}
						stream.close();
					} catch (Exception e) {
						ErrorLog.log(null, getClass(), e);
					}
					anagrafica.update(ApplicationUtil.merge(null, 0, anaList.toArray(new AnagraficaXML[anaList.size()])));
				}
			}
			if(tipo != null && ConstantsMap.telefonoFMap.contains(tipo)) {
				if(tipo.equals(ConstantsMap.telefono_fisso)) {
					anagrafica.setTelefono_fisso(telefono);
				}
				if(tipo.equals(ConstantsMap.telefono_mobile)) {
					anagrafica.setTelefono_mobile(telefono);
				}
				if(tipo.equals(ConstantsMap.telefono_fax)) {
					anagrafica.setTelefono_fax(telefono);
				}
				if(tipo.equals(ConstantsMap.telefono_altro)) {
					anagrafica.setTelefono_altro(telefono);
				}
			}
		}
		Campagna campagna = campagnaDao.getFullCampagna(camp);
		String nomeLista =  ConstantsLanguage.INBOUND.stampa() + ": " + campagna.getNome();
		Lista lista = campagnaDao.getByCampagnaNome(nomeLista, campagna.getId());
		if(lista == null ) {
			lista = new Lista();
			lista.setNome(nomeLista);
			lista.setDescrizione(nomeLista);
			salvaAggiorna(lista);
		}
		campagnaDao.addListaToCampagnaSalva(campagna, lista);
		if(ApplicationUtil.notEmpty(anagrafica.getLista())) {
			if(!anagrafica.getLista().contains(lista)) {
				anagrafica.getLista().add(lista);
			}
		} else {
			List<Lista> ls = new ArrayList<Lista>();
			ls.add(lista);
			anagrafica.setLista(ls);
		}
		anagraficaDao.salva(anagrafica);
		if(text != null) {
			anagraficaDao.salvaCampiAggiuntivi(anagrafica);
		}
		Contatto contatto = new Contatto();
		contatto.setAnagrafica(anagrafica);
		contatto.setCampagna(campagna);
		contatto.setLista(lista);
		pannelloDao.salvaContattoC(false, null, contatto, null, esito, false);
		return contatto;
	}

}