package it.callite.dao;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import it.callite.constants.ConstantsEsito;
import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.constants.ConstantsMap;
import it.callite.dao.generic.GenericDao;
import it.callite.message.Message;
import it.callite.model.Agente;
import it.callite.model.Anagrafica;
import it.callite.model.Campagna;
import it.callite.model.Contatto;
import it.callite.model.EsitoChiamata;
import it.callite.model.Lista;
import it.callite.model.anagrafica.AnagraficaXML;
import it.callite.utility.ApplicationUtil;
@SuppressWarnings("serial")
@Repository("controllerDao")
public class ControllerDao extends GenericDao {
	@Autowired
	private AnagraficaDao anagraficaDao;
	@Autowired
	private CampagnaDao campagnaDao;
	@Autowired
	private ImpostazioneDao impostazioneDao;

	public List<String> inviaContatto(AnagraficaXML anagraficaXML, Long campagnaId, Long listaId, Long agenteId, Date date) {
		List<Contatto> contatti = campagnaDao.getFullContatti(anagraficaXML.getId(), campagnaId, listaId);
		if(ApplicationUtil.notEmpty(contatti)) {
			return updateContatto(contatti, anagraficaXML, campagnaId, agenteId, date);
		} else {
			return salvaContatto(anagraficaXML, campagnaId, listaId, agenteId, date);
		}
	}

	private List<String> updateContatto(List<Contatto> contatti, AnagraficaXML anagraficaXML, Long campagnaId, Long agenteId, Date date) {
		List<String> messageS = new ArrayList<String>();
		boolean a = false;
		Integer oC = impostazioneDao.getOpposizioneForm();
		boolean rfisso = ApplicationUtil.isTrue(impostazioneDao.getRichiestoFisso());
		boolean rmobile = ApplicationUtil.isTrue(impostazioneDao.getRichiestoMobile());
		boolean rfax = ApplicationUtil.isTrue(impostazioneDao.getRichiestoFax());
		boolean raltro = ApplicationUtil.isTrue(impostazioneDao.getRichiestoAltro());
		boolean remail = ApplicationUtil.isTrue(impostazioneDao.getRichiestoEmail());
		boolean rlink = ApplicationUtil.isTrue(impostazioneDao.getRichiestoLink());
		boolean rpartita = ApplicationUtil.isTrue(impostazioneDao.getRichiestoPartita());
		boolean rcodice = ApplicationUtil.isTrue(impostazioneDao.getRichiestoCodice());

		boolean gfisso = ApplicationUtil.isTrue(impostazioneDao.getRegulareFissoForm());
		boolean gmobile = ApplicationUtil.isTrue(impostazioneDao.getRegulareMobileForm());
		boolean gfax = ApplicationUtil.isTrue(impostazioneDao.getRegulareFaxForm());
		boolean galtro = ApplicationUtil.isTrue(impostazioneDao.getRegulareAltroForm());
		boolean gemail = ApplicationUtil.isTrue(impostazioneDao.getRegulareEmailForm());
		boolean glink = ApplicationUtil.isTrue(impostazioneDao.getRegulareLinkForm());
		boolean gpartita = ApplicationUtil.isTrue(impostazioneDao.getRegularePartitaForm());
		boolean gcodice = ApplicationUtil.isTrue(impostazioneDao.getRegulareCodiceForm());

		String vfisso = impostazioneDao.validatore_fisso();
		String vmobile = impostazioneDao.validatore_mobile();
		String vfax = impostazioneDao.validatore_fax();
		String valtro = impostazioneDao.validatore_altro();
		String vemail = impostazioneDao.validatore_email();
		String vlink = impostazioneDao.validatore_link();
		String vpartita = impostazioneDao.validatore_partita();
		String vcodice = impostazioneDao.validatore_codice();

		if(ApplicationUtil.eq(oC, 1)) {
			boolean fisso = invalido(messageS, anagraficaXML.getTelefono_fisso(), campagnaId, "telefono_fisso", rfisso, gfisso, "", vfisso);
			boolean mobile = invalido(messageS, anagraficaXML.getTelefono_mobile(), campagnaId, "telefono_mobile", rmobile, gmobile, "", vmobile);
			boolean fax = invalido(messageS, anagraficaXML.getTelefono_fax(), campagnaId, "telefono_fax", rfax, gfax, "", vfax);
			boolean altro = invalido(messageS, anagraficaXML.getTelefono_altro(), campagnaId, "telefono_altro", raltro, galtro, "", valtro);
			boolean email = invalido(messageS, anagraficaXML.getEmail(), campagnaId, "email", remail, gemail, "", vemail);
			boolean link = invalido(messageS, anagraficaXML.getLink(), campagnaId, "link", rlink, glink, "", vlink);
			boolean partita = invalido(messageS, anagraficaXML.getPartita_iva(), campagnaId, "partita_iva", rpartita, gpartita, "", vpartita);
			boolean codice = invalido(messageS, anagraficaXML.getCodice_fiscale(), campagnaId, "codice_fiscale", rcodice, gcodice, "", vcodice);
			a = fisso || mobile || fax || altro || email || link || partita || codice;
		}
		if(ApplicationUtil.eq(oC, 2)) {
			boolean fisso = invalido(messageS, anagraficaXML.getTelefono_fisso(), campagnaId, "telefono_fisso", rfisso, gfisso, "", vfisso);
			boolean mobile = invalido(messageS, anagraficaXML.getTelefono_mobile(), campagnaId, "telefono_mobile", rmobile, gmobile, "", vmobile);
			boolean fax = invalido(messageS, anagraficaXML.getTelefono_fax(), campagnaId, "telefono_fax", rfax, gfax, "", vfax);
			boolean altro = invalido(messageS, anagraficaXML.getTelefono_altro(), campagnaId, "telefono_altro", raltro, galtro, "", valtro);
			boolean email = invalido(messageS, anagraficaXML.getEmail(), campagnaId, "email", remail, gemail, "", vemail);
			a = fisso || mobile || fax || altro || email;
		}
		if(ApplicationUtil.eq(oC, 3)) {
			boolean fisso = invalido(messageS, anagraficaXML.getTelefono_fisso(), campagnaId, "telefono_fisso", rfisso, gfisso, "", vfisso);
			boolean mobile = invalido(messageS, anagraficaXML.getTelefono_mobile(), campagnaId, "telefono_mobile", rmobile, gmobile, "", vmobile);
			boolean fax = invalido(messageS, anagraficaXML.getTelefono_fax(), campagnaId, "telefono_fax", rfax, gfax, "", vfax);
			boolean altro = invalido(messageS, anagraficaXML.getTelefono_altro(), campagnaId, "telefono_altro", raltro, galtro, "", valtro);
			a = fisso || mobile || fax || altro;
		}
		boolean i = invalido(messageS, anagraficaXML);
		if(!a && !i) {
			List<Long> ln = ApplicationUtil.prendiIdAnagraficaByContatto(contatti);
			String id = ApplicationUtil.detoken(ln, ConstantsGlobal.commaSeparator); 
			String hql = "delete from DatoAggiuntivoAnagrafica where anagrafica.id in ("+id+")";
			execute(false, hql);
			for(Contatto contatto : contatti) {
				Anagrafica anagrafica = contatto.getAnagrafica();
				anagrafica.update(anagraficaXML);
				anagraficaDao.salva(anagrafica);
				anagraficaDao.salvaCampiAggiuntivi(anagrafica);
				contatto.setTipo(ConstantsGlobal.inviato);
				if(agenteId != null) {
					Agente agente = campagnaDao.getAgente(agenteId);
					contatto.setAgente(agente);
				}
				EsitoChiamata da_chiamare = ConstantsEsito.DEFAULTS.get(ConstantsEsito.ID_DA_CHIAMARE);
				contatto.imposta(date == null ? ApplicationUtil.date() : date, da_chiamare, null);
				salvaAggiorna(contatto);
				
				anagraficaXML.setId(anagrafica.getId());
			}
		}
		return messageS;
	}

	private List<String> salvaContatto(AnagraficaXML anagraficaXML, Long campagnaId, Long listaId, Long agenteId, Date date) {

		List<String> messageS = new ArrayList<String>();
		if(ApplicationUtil.gtZero(campagnaId)) {
			Campagna campagna = campagnaDao.getFullCampagna(campagnaId);
			if(campagna != null) {
				boolean a = false;
				Integer oC = impostazioneDao.getOpposizioneForm();
				boolean rfisso = ApplicationUtil.isTrue(impostazioneDao.getRichiestoFisso());
				boolean rmobile = ApplicationUtil.isTrue(impostazioneDao.getRichiestoMobile());
				boolean rfax = ApplicationUtil.isTrue(impostazioneDao.getRichiestoFax());
				boolean raltro = ApplicationUtil.isTrue(impostazioneDao.getRichiestoAltro());
				boolean remail = ApplicationUtil.isTrue(impostazioneDao.getRichiestoEmail());
				boolean rlink = ApplicationUtil.isTrue(impostazioneDao.getRichiestoLink());
				boolean rpartita = ApplicationUtil.isTrue(impostazioneDao.getRichiestoPartita());
				boolean rcodice = ApplicationUtil.isTrue(impostazioneDao.getRichiestoCodice());

				boolean gfisso = ApplicationUtil.isTrue(impostazioneDao.getRegulareFissoForm());
				boolean gmobile = ApplicationUtil.isTrue(impostazioneDao.getRegulareMobileForm());
				boolean gfax = ApplicationUtil.isTrue(impostazioneDao.getRegulareFaxForm());
				boolean galtro = ApplicationUtil.isTrue(impostazioneDao.getRegulareAltroForm());
				boolean gemail = ApplicationUtil.isTrue(impostazioneDao.getRegulareEmailForm());
				boolean glink = ApplicationUtil.isTrue(impostazioneDao.getRegulareLinkForm());
				boolean gpartita = ApplicationUtil.isTrue(impostazioneDao.getRegularePartitaForm());
				boolean gcodice = ApplicationUtil.isTrue(impostazioneDao.getRegulareCodiceForm());

				String dfisso = impostazioneDao.getDuplicatoFissoForm();
				String dmobile = impostazioneDao.getDuplicatoMobileForm();
				String dfax = impostazioneDao.getDuplicatoFaxForm();
				String daltro = impostazioneDao.getDuplicatoAltroForm();
				String demail = impostazioneDao.getDuplicatoEmailForm();
				String dlink = impostazioneDao.getDuplicatoLinkForm();
				String dpartita = impostazioneDao.getDuplicatoPartitaForm();
				String dcodice = impostazioneDao.getDuplicatoCodiceForm();

				String vfisso = impostazioneDao.validatore_fisso();
				String vmobile = impostazioneDao.validatore_mobile();
				String vfax = impostazioneDao.validatore_fax();
				String valtro = impostazioneDao.validatore_altro();
				String vemail = impostazioneDao.validatore_email();
				String vlink = impostazioneDao.validatore_link();
				String vpartita = impostazioneDao.validatore_partita();
				String vcodice = impostazioneDao.validatore_codice();

				if(ApplicationUtil.eq(oC, 1)) {
					boolean fisso = invalido(messageS, anagraficaXML.getTelefono_fisso(), campagnaId, "telefono_fisso", rfisso, gfisso, dfisso, vfisso);
					boolean mobile = invalido(messageS, anagraficaXML.getTelefono_mobile(), campagnaId, "telefono_mobile", rmobile, gmobile, dmobile, vmobile);
					boolean fax = invalido(messageS, anagraficaXML.getTelefono_fax(), campagnaId, "telefono_fax", rfax, gfax, dfax, vfax);
					boolean altro = invalido(messageS, anagraficaXML.getTelefono_altro(), campagnaId, "telefono_altro", raltro, galtro, daltro, valtro);
					boolean email = invalido(messageS, anagraficaXML.getEmail(), campagnaId, "email", remail, gemail, demail, vemail);
					boolean link = invalido(messageS, anagraficaXML.getLink(), campagnaId, "link", rlink, glink, dlink, vlink);
					boolean partita = invalido(messageS, anagraficaXML.getPartita_iva(), campagnaId, "partita_iva", rpartita, gpartita, dpartita, vpartita);
					boolean codice = invalido(messageS, anagraficaXML.getCodice_fiscale(), campagnaId, "codice_fiscale", rcodice, gcodice, dcodice, vcodice);
					a = fisso || mobile || fax || altro || email || link || partita || codice;
				}
				if(ApplicationUtil.eq(oC, 2)) {
					boolean fisso = invalido(messageS, anagraficaXML.getTelefono_fisso(), campagnaId, "telefono_fisso", rfisso, gfisso, dfisso, vfisso);
					boolean mobile = invalido(messageS, anagraficaXML.getTelefono_mobile(), campagnaId, "telefono_mobile", rmobile, gmobile, dmobile, vmobile);
					boolean fax = invalido(messageS, anagraficaXML.getTelefono_fax(), campagnaId, "telefono_fax", rfax, gfax, dfax, vfax);
					boolean altro = invalido(messageS, anagraficaXML.getTelefono_altro(), campagnaId, "telefono_altro", raltro, galtro, daltro, valtro);
					boolean email = invalido(messageS, anagraficaXML.getEmail(), campagnaId, "email", remail, gemail, demail, vemail);
					a = fisso || mobile || fax || altro || email;
				}
				if(ApplicationUtil.eq(oC, 3)) {
					boolean fisso = invalido(messageS, anagraficaXML.getTelefono_fisso(), campagnaId, "telefono_fisso", rfisso, gfisso, dfisso, vfisso);
					boolean mobile = invalido(messageS, anagraficaXML.getTelefono_mobile(), campagnaId, "telefono_mobile", rmobile, gmobile, dmobile, vmobile);
					boolean fax = invalido(messageS, anagraficaXML.getTelefono_fax(), campagnaId, "telefono_fax", rfax, gfax, dfax, vfax);
					boolean altro = invalido(messageS, anagraficaXML.getTelefono_altro(), campagnaId, "telefono_altro", raltro, galtro, daltro, valtro);
					a = fisso || mobile || fax || altro;
				}
				boolean i = invalido(messageS, anagraficaXML);
				if(!a && !i) {
					String nomeLista = ConstantsLanguage.OFFLINE.stampa() + ": " + campagna.getNome();
					Lista lista = null;
					if(ApplicationUtil.gtZero(listaId)) {
						lista = campagnaDao.getLista(listaId);
					}
					if(lista == null ) {
						lista = campagnaDao.getByCampagnaNome(nomeLista, campagna.getId());
					}
					if(lista == null ) {
						lista = new Lista();
						lista.setNome(nomeLista);
						lista.setDescrizione(nomeLista);
						salvaAggiorna(lista);
					}
					campagnaDao.addListaToCampagnaSalva(campagna, lista);
					Anagrafica anagrafica = new Anagrafica(anagraficaXML);
					List<Lista> liste = new ArrayList<Lista>();
					liste.add(lista);
					anagrafica.setLista(liste);
					anagraficaDao.salva(anagrafica);
					anagraficaDao.salvaCampiAggiuntivi(anagrafica);
					Contatto contatto = new Contatto();
					contatto.setTipo(ConstantsGlobal.inviato);
					if(agenteId != null) {
						Agente agente = campagnaDao.getAgente(agenteId);
						contatto.setAgente(agente);
					}
					contatto.setAnagrafica(anagrafica);
					contatto.setCampagna(campagna);
					contatto.setLista(lista);
					EsitoChiamata da_chiamare = ConstantsEsito.DEFAULTS.get(ConstantsEsito.ID_DA_CHIAMARE);
					contatto.imposta(date == null ? ApplicationUtil.date() : date, da_chiamare, null);
					salvaAggiorna(contatto);
					
					anagraficaXML.setId(anagrafica.getId());
				}
			} else {
				messageS.add(Message.addCatalogText("campagna") + ": " + Message.addCatalogText("inesistente"));
			}
		} 
		return messageS;
	}


	private boolean invalido(List<String> messageS, String t, Long campagna, String campo, boolean richiesto, boolean valido, String duplicato, String regulare) {
		String telefono = ApplicationUtil.trimToNull(t);
		if(telefono != null) {
			if(valido) {
				if(ApplicationUtil.isNotBlank(regulare)) {
					Pattern p = Pattern.compile(regulare);
					Matcher m = p.matcher(telefono);
					boolean b = m.matches();	 		 
					if(!b){
						messageS.add(Message.addCatalogText(campo) + ": " + Message.addCatalogText("formato.errato"));
						return true;
					}
				}
			}
			Integer oC = impostazioneDao.getOpposizioneForm();
			if(ApplicationUtil.gt(oC, 0)) {
				int ctrs1 = impostazioneDao.getByTelefono(null, telefono);
				if (ctrs1 > 0) {
					messageS.add(Message.addCatalogText(campo) + ": " + Message.addCatalogText("opposizione"));
					return true;
				}
				if(ApplicationUtil.gtZero(campagna)) {
					int ctrs2 = impostazioneDao.getByTelefono(campagna, telefono);
					if (ctrs2 > 0) {
						messageS.add(Message.addCatalogText(campo) + ": " + Message.addCatalogText("opposizione"));
						return true;
					}
				}
			}
			if(ConstantsMap.getDuplicatoList().contains(duplicato)) {
				List<Long> ctrs = impostazioneDao.getIdAnagraficaByCampo(campo, telefono, duplicato, campagna, null);
				if(ApplicationUtil.notEmpty(ctrs)) {
					messageS.add(Message.addCatalogText(campo) + ": " + Message.addCatalogText("duplicato"));
					return true;
				}
			} 
		} else {
			if(richiesto) {
				messageS.add(Message.addCatalogText(campo) + ": " + Message.addCatalogText("richiesto"));
				return true;
			}
		}
		return false;
	}
	private boolean invalido(List<String> messageS, AnagraficaXML anagrafica) {
		if(ApplicationUtil.eq(impostazioneDao.getFormImportCondition(), 1)) {
			boolean b_fisso = ApplicationUtil.isBlank(anagrafica.getTelefono_fisso());
			boolean b_mobile = ApplicationUtil.isBlank(anagrafica.getTelefono_mobile());
			boolean b_fax = ApplicationUtil.isBlank(anagrafica.getTelefono_fax());
			boolean b_altro = ApplicationUtil.isBlank(anagrafica.getTelefono_altro());
			boolean b_email = ApplicationUtil.isBlank(anagrafica.getEmail());
			boolean b_link = ApplicationUtil.isBlank(anagrafica.getLink());
			boolean b_codice = ApplicationUtil.isBlank(anagrafica.getCodice_fiscale());
			boolean b_partita = ApplicationUtil.isBlank(anagrafica.getPartita_iva());
			if (b_fisso && b_mobile && b_altro && b_fax && b_email && b_link && b_codice && b_partita) {
				messageS.add(Message.addCatalogText("inserisci_telefono1"));
				return true;
			}
		}
		if(ApplicationUtil.eq(impostazioneDao.getFormImportCondition(), 2)) {
			boolean b_fisso = ApplicationUtil.isBlank(anagrafica.getTelefono_fisso());
			boolean b_mobile = ApplicationUtil.isBlank(anagrafica.getTelefono_mobile());
			boolean b_fax = ApplicationUtil.isBlank(anagrafica.getTelefono_fax());
			boolean b_altro = ApplicationUtil.isBlank(anagrafica.getTelefono_altro());
			boolean b_email = ApplicationUtil.isBlank(anagrafica.getEmail());
			if (b_fisso && b_mobile && b_altro && b_fax && b_email) {
				messageS.add(Message.addCatalogText("inserisci_telefono2"));
				return true;
			}
		}
		if(ApplicationUtil.eq(impostazioneDao.getFormImportCondition(), 3)) {
			boolean b_fisso = ApplicationUtil.isBlank(anagrafica.getTelefono_fisso());
			boolean b_mobile = ApplicationUtil.isBlank(anagrafica.getTelefono_mobile());
			boolean b_fax = ApplicationUtil.isBlank(anagrafica.getTelefono_fax());
			boolean b_altro = ApplicationUtil.isBlank(anagrafica.getTelefono_altro());
			if (b_fisso && b_mobile && b_altro && b_fax) {
				messageS.add(Message.addCatalogText("inserisci_telefono3"));
				return true;
			}
		}
		return false;
	}
}