package it.callite.dao;
import java.util.ArrayList;
import java.util.List;

import javax.faces.model.SelectItem;

import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.criteria.ContestCriteria;
import it.callite.criteria.GettoneCriteria;
import it.callite.criteria.base.ExtendedCriteria;
import it.callite.dao.generic.GenericDao;
import it.callite.model.Contest;
import it.callite.model.Gettone;
import it.callite.utility.ApplicationUtil;
@SuppressWarnings("serial")
@Repository("gettoneDao")
public class GettoneDao extends GenericDao {
	
	@Autowired
	private ImpostazioneDao impostazioneDao;
	
	public Gettone salvaGettone(Gettone gettone) {
		gettone.reset();
		return (Gettone) salvaAggiorna(gettone);
	}
	
	public void eliminaGettone(Gettone gettone) {
		cancellaAggiorna(gettone);
	}
	private ExtendedCriteria trovaCriteria(GettoneCriteria ana) {
		if(ana != null)  ana = (GettoneCriteria) ana.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Gettone.class);
		if (ApplicationUtil.gtZero(ana.getUtente())) {
			criteria.add(Restrictions.eq("utente", ana.getUtente()));
		}
		if (ApplicationUtil.gtZero(ana.getTipo())) {
			if(ApplicationUtil.eq(ana.getTipo(),1)) {
				criteria.add(Restrictions.eq("contratto", Boolean.TRUE));
			} else {
				criteria.add(Restrictions.eq("contratto", Boolean.FALSE));
			}
		}
		if (ApplicationUtil.gtZero(ana.getAttivo())) {
			if(ApplicationUtil.eq(ana.getAttivo(),1)) {
				criteria.add(Restrictions.eq("attivo", Boolean.TRUE));
			} else {
				criteria.add(Restrictions.eq("attivo", Boolean.FALSE));
			}
		}
		ana.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, ana);
	}
	@SuppressWarnings("unchecked")
	public List<Gettone> trova(GettoneCriteria ana, int start, int size) {
		ExtendedCriteria criteria = trovaCriteria(ana);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int trovaSize(GettoneCriteria ana) {
		ExtendedCriteria criteria = trovaCriteria(ana);
		return getListSize(criteria);
	}
	public Gettone getGettone(Long id) {
		return (Gettone) get(Gettone.class, id);
	}
	public Contest salvaContest(Contest contest) {
		contest.reset();
		return (Contest) salvaAggiorna(contest);
	}
	
	public void eliminaContest(Contest contest) {
		cancellaAggiorna(contest);
	}
	private ExtendedCriteria trovaCriteria(ContestCriteria ana) {
		if(ana != null)  ana = (ContestCriteria) ana.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Contest.class);
		if (ApplicationUtil.gtZero(ana.getUtente())) {
			criteria.add(Restrictions.eq("utente", ana.getUtente()));
		}
		if (ApplicationUtil.gtZero(ana.getTipo())) {
			if(ApplicationUtil.eq(ana.getTipo(),1)) {
				criteria.add(Restrictions.eq("contratto", Boolean.TRUE));
			} else {
				criteria.add(Restrictions.eq("contratto", Boolean.FALSE));
			}
		}
		if (ApplicationUtil.gtZero(ana.getGenere())) {
			criteria.add(Restrictions.eq("moltiplicatore", ana.getGenere()));
		}
		if (ApplicationUtil.gtZero(ana.getAttivo())) {
			if(ApplicationUtil.eq(ana.getAttivo(),1)) {
				criteria.add(Restrictions.eq("attivo", Boolean.TRUE));
			} else {
				criteria.add(Restrictions.eq("attivo", Boolean.FALSE));
			}
		}
		ana.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, ana);
	}
	@SuppressWarnings("unchecked")
	public List<Contest> trova(ContestCriteria ana, int start, int size) {
		ExtendedCriteria criteria = trovaCriteria(ana);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int trovaSize(ContestCriteria ana) {
		ExtendedCriteria criteria = trovaCriteria(ana);
		return getListSize(criteria);
	}
	public Contest getContest(Long id) {
		return (Contest) get(Contest.class, id);
	}
	public List<SelectItem> getUtenti() {
		List<SelectItem> sis = new ArrayList<SelectItem>();
		sis.add(new SelectItem(ConstantsGlobal.operatore, ConstantsLanguage.OPERATORE.stampa()));
		sis.add(new SelectItem(ConstantsGlobal.supervisore, ConstantsLanguage.SUPERVISORE.stampa()));
		sis.add(new SelectItem(ConstantsGlobal.coordinatore, ConstantsLanguage.COORDINATORE.stampa()));
		sis.add(new SelectItem(ConstantsGlobal.agente, ConstantsLanguage.DIRETTO_AGENTE.stampa()));
		sis.add(new SelectItem(ConstantsGlobal.rete, ConstantsLanguage.RETE_AGENTE.stampa()));
		sis.add(new SelectItem(ConstantsGlobal.sotto, ConstantsLanguage.SOTTO_AGENTE.stampa()));
		sis.add(new SelectItem(ConstantsGlobal.prima, ConstantsLanguage.PRIMA_AGENTE.stampa()));
		sis.add(new SelectItem(ConstantsGlobal.orete, ConstantsLanguage.RETE_OAGENTE.stampa()));
		sis.add(new SelectItem(ConstantsGlobal.osotto, ConstantsLanguage.SOTTO_OAGENTE.stampa()));
		sis.add(new SelectItem(ConstantsGlobal.oprima, ConstantsLanguage.PRIMA_OAGENTE.stampa()));
		return sis;
	}
	public List<SelectItem> getUtentiRicerca() {
		List<SelectItem> sis = new ArrayList<SelectItem>();
		sis.add(new SelectItem(null, ConstantsLanguage.TRATTINI.stampa()));
		sis.addAll(getUtenti());
		return sis;
	}
	@SuppressWarnings("unchecked")
	public List<Contest> trovaContestByProdottoId(Long prodottoId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Contest.class);
		criteria.add(Restrictions.eq("prodotto.id", prodottoId));
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Gettone> trovaGettoneByProdottoId(Long prodottoId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Gettone.class);
		criteria.add(Restrictions.eq("prodotto.id", prodottoId));
		return getList(criteria);
	}
	
	@SuppressWarnings("unchecked")
	public List<Contest> trovaContestByCategoriaId(Long categoriaId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Contest.class);
		criteria.add(Restrictions.eq("categoria.id", categoriaId));
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Gettone> trovaGettoneByCategoriaId(Long categoriaId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Gettone.class);
		criteria.add(Restrictions.eq("categoria.id", categoriaId));
		return getList(criteria);
	}
}