package it.callite.dao;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URL;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import it.callite.constants.ConstantsFormat;
import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.constants.ConstantsMap;
import it.callite.criteria.InvoiceCriteria;
import it.callite.dao.generic.GenericDao;
import it.callite.log.ErrorLog;
import it.callite.model.Invoice;
import it.callite.service.SmtpService;
import it.callite.utility.Allegato;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.CSVExport;
import it.callite.utility.ExcelExport;
import it.callite.utility.PDFWriter;
import it.callite.utility.QueryElement;
import it.callite.utility.ZipPassword;
import it.callite.utility.utils.ModuloFileFilter;
@SuppressWarnings("serial")
@Repository("invoiceDao")
public class InvoiceDao extends GenericDao {

	@Autowired
	private DownloadDao downloadDao;
	@Autowired
	private ImpostazioneDao impostazioneDao;
	@Autowired
	private SmtpService smtpService;

	private String ok_invio = "";
	private String ko_invio = "_";

	public void lanciaInvoice(String nome) {
		try {
			Invoice sk = getValue(nome);
			lancia(sk);
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}


	public Invoice get(Long id) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Invoice.class);
		criteria.add(Restrictions.eq("id", id));
		return (Invoice) getUniqueResult(criteria);
	}

	public Invoice getValue(String value) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Invoice.class);
		criteria.add(Restrictions.eq("nome", value));
		Invoice sk = (Invoice) getUniqueResult(criteria);
		if(sk == null) {
			return new Invoice();
		} else {
			return sk;
		}
	}

	public Invoice getByName(String value) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Invoice.class);
		criteria.add(Restrictions.eq("nome", value));
		return (Invoice) getUniqueResult(criteria);
	}

	public Invoice salva(Invoice oggetto) {
		return (Invoice) salvaAggiorna(oggetto);
	}

	public void elimina(Invoice oggetto) {
		cancellaAggiorna(oggetto);
	}

	private DetachedCriteria getInvoiceByCriteria(InvoiceCriteria crit) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Invoice.class);
		if(StringUtils.isNotBlank(crit.getNome())) {
			criteria.add(Restrictions.in("nome", crit.getNome()));
		}
		if(crit.getAvviato() != null) {
			criteria.add(Restrictions.eq("started", crit.getAvviato()));
		} 		
		return criteria;
	}

	@SuppressWarnings("unchecked")
	public List<Invoice> trova(InvoiceCriteria crit, int start, int size) {
		DetachedCriteria criteria = getInvoiceByCriteria(crit);
		criteria.addOrder(Order.asc("nome"));
		return getList(criteria, start, size);
	}

	public int trovaSize(InvoiceCriteria crit) {
		DetachedCriteria criteria = getInvoiceByCriteria(crit);
		return getListSize(criteria);
	}


	@SuppressWarnings("unchecked")
	public List<Invoice> trovaTutti() {
		DetachedCriteria criteria = DetachedCriteria.forClass(Invoice.class);
		return getList(criteria);
	}

	public void lancia(Invoice sk) throws Exception {
		String home = impostazioneDao.getHome();
		String path =  home + ConstantsGlobal.path_file;
		File f = new File(path + "/");
		ApplicationUtil.creaFile(true, f);
		Date now = ApplicationUtil.date();
		query(sk, path, now);
		invoca(sk, path, now);
	}

	private void query(Invoice sk, String path, Date date)  {
		String servizio = sk.getNome();
		String queries = sk.getQueries();
		if (servizio != null) {
			List<String> ql = ApplicationUtil.splitV(queries, ConstantsGlobal.qsp);
			if(ApplicationUtil.notEmpty(ql)) {
				try {
					Connection conn = getJdbcTemplate().getDataSource().getConnection();
					conn.setAutoCommit(true);
					Statement st = conn.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
					for(String q : ql) {
						String test = ApplicationUtil.lowerCase(q);
						List<List<QueryElement>> ls = new ArrayList<List<QueryElement>>();
						List<String> lh = new ArrayList<String>();
						if(test.startsWith(ConstantsGlobal.select) || test.startsWith(ConstantsGlobal.show)) {
							ResultSet rs = st.executeQuery(q);
							ResultSetMetaData rsMetaData = rs.getMetaData();
							int numberOfColumns = rsMetaData.getColumnCount();
							for (int i = 1; i <= numberOfColumns; i++) {
								String h = rsMetaData.getColumnLabel(i);
								lh.add(h);
							}
							while(rs.next()) {
								List<QueryElement> ld = new ArrayList<QueryElement>();
								for (int i = 1; i <= numberOfColumns; i++) {
									QueryElement a = new QueryElement(rs.getObject(i));
									ld.add(a);
								}
								ls.add(ld);
							}
						} else {
							st.executeUpdate(q);
						}
						write(ls, lh, path, sk, date);
					}
					st.close();
					conn.close();
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				}
			}
		}
	}

	private void write(List<List<QueryElement>> ls, List<String> lh, String path, Invoice sk, Date date) throws Exception {
		String servizio = sk.getNome();
		Boolean header = sk.getHeader();
		Boolean p = sk.getPassword();
		String ext = sk.getType();
		if(ApplicationUtil.notEmpty(ls)) {
			String base = servizio + "_" + ApplicationUtil.formatDate(ConstantsFormat.invoiceDate.stampa(), date);
			ApplicationUtil.logger(base + " - " + "[" + ls.size() + "]");
			if(ext.equals(ConstantsMap.csv)) {
				String password = ApplicationUtil.isTrue(p) ? impostazioneDao.getPasswordTXT() : null;
				if(password != null) {
					String filename = base + "." + ConstantsMap.zip;
					FileOutputStream fs = new FileOutputStream(path + "/" + ko_invio + filename);
					ByteArrayOutputStream bos = new ByteArrayOutputStream();
					CSVExport.esportaCSV(header, sk, bos, lh, ls);
					ZipPassword.zip(null, bos.toByteArray(), fs, ConstantsMap.csv, password);
					bos.close();
					fs.close();
				} else {
					String filename = base + "." + ConstantsMap.csv;
					FileOutputStream fs = new FileOutputStream(path + "/" + ko_invio + filename);
					CSVExport.esportaCSV(header, sk, fs, lh, ls);
					fs.close();
				}
			}
			if(ext.equals(ConstantsMap.pdf)) {
				String password = ApplicationUtil.isTrue(p) ? impostazioneDao.getPasswordPDF() : null;
				String filename = base + "." + ConstantsMap.pdf;
				FileOutputStream fs = new FileOutputStream(path + "/" + ko_invio + filename);
				PDFWriter.stampaPDF(header, lh, ls, fs, password);
				fs.close();
			}
			if(ext.equals(ConstantsMap.xlsx)) {
				String password = ApplicationUtil.isTrue(p) ? impostazioneDao.getPasswordXLS() : null;
				String filename = base + "." + ConstantsMap.xlsx;
				FileOutputStream fs = new FileOutputStream(path + "/" + ko_invio + filename);
				ExcelExport.esportaExcel(header, lh, ls, fs, password);
				fs.close();
			}
		}
	}

	private void invoca(Invoice sk, String path, Date date) {
		String servizio = sk.getNome();
		if (servizio != null) {
			File libDir = new File(path);
			if(libDir.exists() && libDir.isDirectory()) {
				File[] files = libDir.listFiles(new ModuloFileFilter(ko_invio + servizio +  "_", true));
				if (ApplicationUtil.notEmpty(files)) {
					for(File f : files) {
						boolean success = invoice(f, sk, date);
						if (success) {	
							String nfile = StringUtils.substringAfter(f.getName(), ko_invio);
							f.renameTo(new File(path + "/" + ok_invio + nfile));
						}
					}
				}
			}
		}
	}

	private boolean invoice(File f, Invoice sk, Date date) {
		String link = sk.getLink();
		if(link != null) {
			String protocol = ApplicationUtil.protocol(link);
			if(ApplicationUtil.isHTTP(protocol) ) {
				try {
					String lineEnd = "\r\n";
					String twoHyphens = "--";
					String boundary = "*****";

					int maxBufferSize = ConstantsGlobal.kb* ConstantsGlobal.kb;
					URL url = new URL(link);
					HttpURLConnection conn = (HttpURLConnection) url.openConnection();
					conn.setDoInput(true);
					conn.setDoOutput(true);
					conn.setUseCaches(false);
					conn.setRequestMethod("POST");
					conn.setRequestProperty("Connection", "Keep-Alive");
					conn.setRequestProperty("Content-Type", "multipart/form-data;boundary=" + boundary);

					DataOutputStream dos = new DataOutputStream(conn.getOutputStream());

					dos.writeBytes(twoHyphens + boundary + lineEnd);

					dos.writeBytes("Content-Disposition: form-data; name=\"file\";" + " filename=\"" + f.getName() +"\"" + lineEnd);
					dos.writeBytes(lineEnd);

					FileInputStream fileInputStream = new FileInputStream(f);

					int bytesAvailable = fileInputStream.available();
					int bufferSize = Math.min(bytesAvailable, maxBufferSize);
					byte[] buffer = new byte[bufferSize];

					int bytesRead = fileInputStream.read(buffer, 0, bufferSize);
					while(bytesRead > 0) {
						dos.write(buffer, 0, bufferSize);
						bytesAvailable = fileInputStream.available();
						bufferSize = Math.min(bytesAvailable, maxBufferSize);
						bytesRead = fileInputStream.read(buffer, 0, bufferSize);
					}

					fileInputStream.close();
					dos.writeBytes(lineEnd);
					dos.writeBytes(twoHyphens + boundary + twoHyphens + lineEnd);

					InputStreamReader  inStream = new InputStreamReader(conn.getInputStream());
					BufferedReader input = new BufferedReader(inStream);
					StringBuffer b = new StringBuffer();
					String text;
					while ((text = input.readLine()) != null) {
						b.append(text + System.lineSeparator());
					}
					input.close();
					inStream.close();
					ApplicationUtil.logger("INVOICE: " + b.toString());
					return true;
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
					return false;
				}
			}
			if(ApplicationUtil.isMAILTO(protocol) ) {
				try {
					String email = ApplicationUtil.substringAfter(link, protocol + ":");
					String testo = downloadDao.preparaTemplateEmail(ConstantsLanguage.REPORT_IN_ALLEGATO.stampa(), date);
					String oggetto = ApplicationUtil.report(ConstantsLanguage.INVOICE.stampa() , ConstantsGlobal.no_periodo, date);
					ByteArrayInputStream is = new ByteArrayInputStream(ApplicationUtil.readFileToByteArray(f));
					List<Allegato> allegati = ApplicationUtil.allegatiConvertI(is, f.getName());
					smtpService.invia(null, email, null, oggetto, testo, allegati, null);
					return true;
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
					return false;
				}
			}
		}
		return true;
	}

}