package it.callite.dao;
import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.hibernate.Session;
import org.hibernate.Transaction;
import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Projections;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import it.callite.action.DownloadBean;
import it.callite.chiamate.Chiamate;
import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.constants.ConstantsMap;
import it.callite.constants.config.DefaultsGeneric;
import it.callite.criteria.DocumentoCriteria;
import it.callite.criteria.MessaggioAdminCriteria;
import it.callite.criteria.MessaggioBozzaCriteria;
import it.callite.criteria.MessaggioCestinoInArrivoCriteria;
import it.callite.criteria.MessaggioCestinoInUscitaCriteria;
import it.callite.criteria.MessaggioInArrivoCriteria;
import it.callite.criteria.MessaggioInUscitaCriteria;
import it.callite.criteria.UserCriteria;
import it.callite.criteria.base.ExtendedCriteria;
import it.callite.dao.generic.GenericDao;
import it.callite.log.ErrorLog;
import it.callite.model.Utente;
import it.callite.model.messaggio.AllegatoMessaggio;
import it.callite.model.messaggio.Destinatario;
import it.callite.model.messaggio.DocumentoMessaggio;
import it.callite.model.messaggio.Messaggio;
import it.callite.model.report.ColonnaModel;
import it.callite.model.report.ColonnaQuery;
import it.callite.model.report.Messaggistica;
import it.callite.model.report.MonitorOperatore;
import it.callite.service.SmtpService;
import it.callite.service.config.ConfigurazioneGenericService;
import it.callite.utility.Allegato;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.FileUpload;
import it.callite.utility.MyFile;
import it.callite.utility.Parameter;
import it.callite.utility.thread.MessaggioThread;
@SuppressWarnings("serial")
@Repository("messaggioDao")
public class MessaggioDao extends GenericDao {
	@Autowired
	private UserDao userDao;
	@Autowired
	private DownloadDao downloadDao;
	@Autowired
	private ImpostazioneDao impostazioneDao;
	@Autowired
	private DataDao dataDao;
	@Autowired
	private SmtpService smtpService;
	@Autowired
	private ConfigurazioneGenericService configurazioneGenericService;
		
	public List<Messaggio> impostaConteggioMessaggio(List<Messaggio> messaggi) {
		if(ApplicationUtil.notEmpty(messaggi)) {
			List<Long> ll = ApplicationUtil.prendiIdMittenteMessaggi(messaggi);
			Map<Long, ColonnaModel> map = dataDao.getConteggioColonna(null, ll, Messaggio.class, new ColonnaQuery("destinatari", "destinatari"), new ColonnaQuery("allegati", "allegati"));
			Iterator<Messaggio> it = messaggi.iterator();
			while (it.hasNext()) {
				Messaggio o = it.next();
				ColonnaModel lmap = map.get(o.getId());
				if(lmap != null) {
					o.setModel(lmap);
				}
			}
		}
		return messaggi;
	}
	public List<Destinatario> impostaConteggioDestinatario(List<Destinatario> destinatari) {
		if(ApplicationUtil.notEmpty(destinatari)) {
			List<Long> ll = ApplicationUtil.prendiIdDestinatarioMessaggi(destinatari);
			Map<Long, ColonnaModel> map = dataDao.getConteggioColonna(null, ll, Messaggio.class, new ColonnaQuery("destinatari", "destinatari"), new ColonnaQuery("allegati", "allegati"));
			Iterator<Destinatario> it = destinatari.iterator();
			while (it.hasNext()) {
				Destinatario o = it.next();
				ColonnaModel lmap = map.get(o.getMessaggio().getId());
				if(lmap != null) {
					o.getMessaggio().setModel(lmap);
				}
			}
		}
		return destinatari;
	}
	public Messaggio salvaMessaggio(Messaggio messaggio) {
		return (Messaggio) salvaAggiorna(messaggio);
	}
	public void eliminaMessaggio(Messaggio messaggio) {
		cancellaAggiorna(messaggio);
	}
	public Messaggio getMessaggio(Long messaggioId) {
		return (Messaggio) get(Messaggio.class, messaggioId);
	}
	public void eliminaMessaggioInArrivo(Utente utente, Destinatario dest) {
		dest.setCestino(true);
		dest.setEliminato(false);
		salvaAggiorna(dest);
	}
	public void eliminaMessaggioInUscita(Utente utente, Messaggio mess) {
		mess.setCestino(true);
		mess.setEliminato(false);
		salvaAggiorna(mess);
	}
	public void eliminaMessaggioBozza(Utente utente, Messaggio mess) {
		mess.setCestino(true);
		mess.setEliminato(false);
		salvaAggiorna(mess);
	}
	private ExtendedCriteria trovaCriteriaMessaggioBozza(Utente utente, MessaggioBozzaCriteria criterion) {
		if(criterion != null) criterion = (MessaggioBozzaCriteria) criterion.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Messaggio.class);
		criteria.add(Restrictions.eq("stato", false));
		criteria.add(Restrictions.eq("cestino", false));
		criteria.add(Restrictions.eq("eliminato", false));
		if (ApplicationUtil.isNotBlank(criterion.getTesto())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, criterion.getTesto(), criteria, "oggetto", "testo");
		}
		if(ApplicationUtil.isNotBlank(criterion.getDestinatario())) {
			DetachedCriteria criteriaD = criteria.createCriteria("destinatari", ApplicationUtil.leftJoin());
			DetachedCriteria criteriaU = criteriaD.createCriteria("user", ApplicationUtil.leftJoin());
			ApplicationUtil.operatore(criteriaU, criterion.getDestinatario());
		}
		criteria.createCriteria("mittente").add(Restrictions.idEq(utente.getId()));
		criterion.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, criterion);
	}
	@SuppressWarnings("unchecked")
	public List<Messaggio> trovaMessaggioBozza(Utente utente, MessaggioBozzaCriteria ana, int start, int size) {
		ExtendedCriteria criteria = trovaCriteriaMessaggioBozza(utente, ana);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.desc("id")});
		return getList(criteria, start, size);
	}
	public int trovaMessaggioBozzaSize(Utente utente, MessaggioBozzaCriteria ana) {
		ExtendedCriteria criteria = trovaCriteriaMessaggioBozza(utente, ana);
		return getListSize(criteria);
	}
	private ExtendedCriteria trovaCriteriaMessaggioAdmin(MessaggioAdminCriteria criterion) {
		if(criterion != null) criterion = (MessaggioAdminCriteria) criterion.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Messaggio.class);
		if (ApplicationUtil.isNotBlank(criterion.getTesto())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, criterion.getTesto(), criteria, "oggetto", "testo");
		}
		if(ApplicationUtil.isNotBlank(criterion.getMittente())) {
			DetachedCriteria criteriaM = criteria.createCriteria("mittente", ApplicationUtil.leftJoin());
			ApplicationUtil.operatore(criteriaM, criterion.getMittente());
		}
		if(ApplicationUtil.isNotBlank(criterion.getDestinatario())) {
			DetachedCriteria criteriaD = criteria.createCriteria("destinatari", ApplicationUtil.leftJoin());
			DetachedCriteria criteriaU = criteriaD.createCriteria("user", ApplicationUtil.leftJoin());
			ApplicationUtil.operatore(criteriaU, criterion.getDestinatario());
		}
		criterion.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, criterion);
	}
	@SuppressWarnings("unchecked")
	public List<Messaggio> trovaMessaggioAdmin(MessaggioAdminCriteria ana, int start, int size) {
		ExtendedCriteria criteria = trovaCriteriaMessaggioAdmin(ana);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.desc("id")});
		return getList(criteria, start, size);
	}
	public int trovaMessaggioAdminSize(MessaggioAdminCriteria ana) {
		ExtendedCriteria criteria = trovaCriteriaMessaggioAdmin(ana);
		return getListSize(criteria);
	}
	public void eliminaMessaggioBozzaSelezionati(Utente utente, List<Messaggio> messaggi) {
		if (ApplicationUtil.notEmpty(messaggi)) {
			Iterator<Messaggio> itCont = messaggi.iterator();
			while (itCont.hasNext()) {
				Messaggio messaggio = (Messaggio) itCont.next();
				eliminaMessaggioBozza(utente, messaggio);
			}
		}
	}
	public void eliminaMessaggioSelezionati(List<Messaggio> messaggi) {
		if (ApplicationUtil.notEmpty(messaggi)) {
			Iterator<Messaggio> itCont = messaggi.iterator();
			while (itCont.hasNext()) {
				Messaggio messaggio = (Messaggio) itCont.next();
				eliminaMessaggio(messaggio);
			}
		}
	}
	private ExtendedCriteria trovaCriteriaMessaggioInUscita(Utente utente, MessaggioInUscitaCriteria criterion) {
		if(criterion != null) criterion = (MessaggioInUscitaCriteria) criterion.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Messaggio.class);
		criteria.add(Restrictions.eq("stato", true));
		criteria.add(Restrictions.eq("cestino", false));
		criteria.add(Restrictions.eq("eliminato", false));
		if (ApplicationUtil.isNotBlank(criterion.getTesto())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, criterion.getTesto(), criteria, "oggetto", "testo");
		}
		if(ApplicationUtil.isNotBlank(criterion.getDestinatario())) {
			DetachedCriteria criteriaD = criteria.createCriteria("destinatari", ApplicationUtil.leftJoin());
			DetachedCriteria criteriaU = criteriaD.createCriteria("user", ApplicationUtil.leftJoin());
			ApplicationUtil.operatore(criteriaU, criterion.getDestinatario());
		}
		criteria.createCriteria("mittente").add(Restrictions.idEq(utente.getId()));
		criterion.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, criterion);
	}
	@SuppressWarnings("unchecked")
	public List<Messaggio> trovaMessaggioInUscita(Utente utente, MessaggioInUscitaCriteria ana, int start, int size) {
		ExtendedCriteria criteria = trovaCriteriaMessaggioInUscita(utente, ana);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.desc("id")});
		return getList(criteria, start, size);
	}
	public int trovaMessaggioInUscitaSize(Utente utente, MessaggioInUscitaCriteria ana) {
		ExtendedCriteria criteria = trovaCriteriaMessaggioInUscita(utente, ana);
		return getListSize(criteria);
	}
	private ExtendedCriteria trovaCriteriaMessaggioInArrivo(Utente utente, MessaggioInArrivoCriteria criterion) {
		if(criterion != null) criterion = (MessaggioInArrivoCriteria) criterion.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Destinatario.class);
		criteria.createCriteria("user").add(Restrictions.idEq(utente.getId()));
		criteria.add(Restrictions.eq("cestino", false));
		criteria.add(Restrictions.eq("eliminato", false));
		DetachedCriteria crtMess = criteria.createCriteria("messaggio");
		crtMess.add(Restrictions.ne("stato", false));
		if (ApplicationUtil.isNotBlank(criterion.getTesto())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, criterion.getTesto(), crtMess, "oggetto", "testo");
		}
		if(ApplicationUtil.isNotBlank(criterion.getMittente())) {
			DetachedCriteria criteriaM = crtMess.createCriteria("mittente", ApplicationUtil.leftJoin());
			ApplicationUtil.operatore(criteriaM, criterion.getMittente());
		}
		criterion.impostaRicercaDate(crtMess);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, criterion);
	}
	@SuppressWarnings("unchecked")
	public List<Destinatario> trovaMessaggioInArrivo(Utente utente, MessaggioInArrivoCriteria ana, int start, int size) {
		ExtendedCriteria criteria = trovaCriteriaMessaggioInArrivo(utente, ana);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.desc("id")});
		return getList(criteria, start, size);
	}
	public int trovaMessaggioInArrivoSize(Utente utente, MessaggioInArrivoCriteria ana) {
		ExtendedCriteria criteria = trovaCriteriaMessaggioInArrivo(utente, ana);
		return getListSize(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Utente> trovaUtentiMessaggioTutte(List<Destinatario> destinatari) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		List<Long> ids = ApplicationUtil.prendiIdDestinatario(destinatari);
		ApplicationUtil.removeCampi(criteria, ids);
		ApplicationUtil.orderU(criteria);
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Utente> trovaSenzaMessaggio(Utente utente, List<Destinatario> destinatari, UserCriteria criteUser, int start, int size) {
		ExtendedCriteria criteria = trovaSenzaMessaggioCriteria(utente, destinatari, criteUser);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), criteUser.getO_tipo(), criteUser.getO_colonna(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int trovaSenzaMessaggioSize(Utente utente, List<Destinatario> destinatari, UserCriteria user) {
		ExtendedCriteria criteria = trovaSenzaMessaggioCriteria(utente, destinatari, user);
		return getListSize(criteria);
	}
	private ExtendedCriteria trovaSenzaMessaggioCriteria(Utente utente, List<Destinatario> destinatari, UserCriteria user) {
		if(user != null) user = (UserCriteria) user.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		criteria.add(Restrictions.ne("id", utente.getId()));
		List<Long> ids = ApplicationUtil.prendiIdDestinatario(destinatari);
		ApplicationUtil.removeCampi(criteria, ids);
		ApplicationUtil.utente(criteria, user);
		user.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, user);
	}
	public void eliminaMessaggioInArrivoSelezionati(Utente utente, List<Destinatario> destinatari) {
		if (ApplicationUtil.notEmpty(destinatari)) {
			Iterator<Destinatario> itCont = destinatari.iterator();
			while (itCont.hasNext()) {
				Destinatario destinatario = (Destinatario) itCont.next();
				eliminaMessaggioInArrivo(utente, destinatario);
			}
		}
	}
	public void eliminaMessaggioInUscitaSelezionati(Utente utente, List<Messaggio> messaggi) {
		if (ApplicationUtil.notEmpty(messaggi)) {
			Iterator<Messaggio> itCont = messaggi.iterator();
			while (itCont.hasNext()) {
				Messaggio messaggio = (Messaggio) itCont.next();
				eliminaMessaggioInUscita(utente, messaggio);
			}
		}
	}
	public void eliminaMessaggioCestinoInArrivo(Utente utente, Destinatario dest) {
		dest.setCestino(true);
		dest.setEliminato(true);
		salvaAggiorna(dest);
	}
	public void recuperaMessaggioCestinoInArrivo(Utente utente, Destinatario dest) {
		dest.setCestino(false);
		dest.setEliminato(false);
		salvaAggiorna(dest);
	}
	public void eliminaMessaggioCestinoInUscita(Utente utente, Messaggio mess) {
		mess.setCestino(true);
		mess.setEliminato(true);
		salvaAggiorna(mess);
	}
	public void recuperaMessaggioCestinoInUscita(Utente utente, Messaggio mess) {
		mess.setCestino(false);
		mess.setEliminato(false);
		salvaAggiorna(mess);
	}
	public void eliminaMessaggioCestinoInArrivoSelezionati(Utente utente, List<Destinatario> destinatari) {
		if (ApplicationUtil.notEmpty(destinatari)) {
			Iterator<Destinatario> itCont = destinatari.iterator();
			while (itCont.hasNext()) {
				Destinatario destinatario = (Destinatario) itCont.next();
				eliminaMessaggioCestinoInArrivo(utente, destinatario);
			}
		}
	}
	public void eliminaMessaggioCestinoInUscitaSelezionati(Utente utente, List<Messaggio> messaggi) {
		if (ApplicationUtil.notEmpty(messaggi)) {
			Iterator<Messaggio> itCont = messaggi.iterator();
			while (itCont.hasNext()) {
				Messaggio messaggio = (Messaggio) itCont.next();
				eliminaMessaggioCestinoInUscita(utente, messaggio);
			}
		}
	}
	public void recuperaMessaggioCestinoInArrivoSelezionati(Utente utente, List<Destinatario> destinatari) {
		Iterator<Destinatario> itCont = destinatari.iterator();
		while (itCont.hasNext()) {
			Destinatario destinatario = (Destinatario) itCont.next();
			recuperaMessaggioCestinoInArrivo(utente, destinatario);
		}
	}
	public void recuperaMessaggioCestinoInUscitaSelezionati(Utente utente, List<Messaggio> destinatari) {
		Iterator<Messaggio> itCont = destinatari.iterator();
		while (itCont.hasNext()) {
			Messaggio destinatario = (Messaggio) itCont.next();
			recuperaMessaggioCestinoInUscita(utente, destinatario);
		}
	}
	public List<Destinatario> aggiungiUtentiALista(List<Destinatario> destinatari, List<Utente> utenti) {
		if(destinatari==null){
			destinatari = new ArrayList<Destinatario>();
		}
		if(ApplicationUtil.notEmpty(utenti)) {
			for(Utente utente: utenti) {
				Destinatario dest = new Destinatario(utente);
				if(!destinatari.contains(dest)){
					destinatari.add(dest);
				} 
			}
		}
		return destinatari;
	}
	public List<Destinatario> rimuoviUtentiDaLista(List<Destinatario> destinatari) {
		destinatari.clear();
		return destinatari;
	}
	public List<Destinatario> aggiungiUtenteALista(List<Destinatario> destinatari, Utente utente) {
		if(destinatari==null){
			destinatari = new ArrayList<Destinatario>();
		}
		Destinatario dest = new Destinatario(utente);
		if(!destinatari.contains(dest)){
			destinatari.add(dest);
		} 
		return destinatari;
	}
	public List<Destinatario> rimuoviUtenteDaLista(List<Destinatario> destinatari, Destinatario dest) {
		if(destinatari==null){
			destinatari = new ArrayList<Destinatario>();
		}
		if(destinatari.contains(dest)){
			destinatari.remove(dest);
		}
		return destinatari;
	}
	public List<AllegatoMessaggio> rimuoviAllegatoDaLista(List<AllegatoMessaggio> allegati, AllegatoMessaggio allegato) {
		if(allegati==null){
			allegati = new ArrayList<AllegatoMessaggio>();
		}
		if(allegati.contains(allegato)){
			allegati.remove(allegato);
		}
		return allegati;
	}
	public List<AllegatoMessaggio> rimuoviAllegatiDaLista(List<AllegatoMessaggio> allegati) {
		if(allegati==null){
			allegati = new ArrayList<AllegatoMessaggio>();
		}
		allegati.clear();
		return allegati;
	}
	public List<AllegatoMessaggio> aggiungiAllegatoALista(List<AllegatoMessaggio> allegati, FileUpload fileUpload) {
		if(allegati==null){
			allegati = new ArrayList<AllegatoMessaggio>();
		}
		List<MyFile> fls = fileUpload.getFiles();
		if(ApplicationUtil.notEmpty(fls)) {
			Iterator<MyFile> flsIt = fls.iterator();
			while (flsIt.hasNext()) {
				MyFile ff = (MyFile) flsIt.next();
				AllegatoMessaggio all = new AllegatoMessaggio(ff.getLength(), ApplicationUtil.date(), ff.getData(), ff.getName());
				if(!allegati.contains(all)){
					allegati.add(all);
				} 
			}
		}
		return allegati;
	}
	@SuppressWarnings("unchecked")
	public List<AllegatoMessaggio> listaAllegati(Messaggio all) {
		DetachedCriteria criteria = DetachedCriteria.forClass(AllegatoMessaggio.class);
		criteria.createCriteria("messaggio").add(Restrictions.idEq(all.getId()));
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Destinatario> listaDestinatari(Messaggio all) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Destinatario.class);
		criteria.createCriteria("messaggio").add(Restrictions.idEq(all.getId()));
		return getList(criteria);
	}
	public void scaricaAllegato(AllegatoMessaggio allegato, DownloadBean downloadBean) {
		try {


			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(allegato.getNome());

			InputStream is = new ByteArrayInputStream(allegato.getFile());
			ApplicationUtil.copy(is, out);
			downloadBean.close();
			is.close();

		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void salvaMessaggioNuovoBozza(Messaggio messaggio) {
		messaggio.setStato(false);
		messaggio.setCestino(false);
		messaggio.setEliminato(false);
		Session session = openSession();
		Transaction tx = session.beginTransaction();
		session.save(messaggio);
		List<Destinatario> destinatari = messaggio.getDestinatari(); 
		if(ApplicationUtil.notEmpty(destinatari)){
			int i = 0;
			Iterator<Destinatario> objIter = destinatari.iterator();
			while(objIter.hasNext()) {
				Destinatario destinatario = objIter.next();
				destinatario.setMessaggio(messaggio);
				session.save(destinatario);
				if (i++ % ConstantsGlobal.flushSize == 0) {
					session.flush();
					session.clear();
				}
			}
		}
		List<AllegatoMessaggio> allegati = messaggio.getAllegati(); 
		if(ApplicationUtil.notEmpty(allegati)){
			int i = 0;
			Iterator<AllegatoMessaggio> objIter = allegati.iterator();
			while(objIter.hasNext()) {
				AllegatoMessaggio allegato = objIter.next();
				allegato.setMessaggio(messaggio);
				session.save(allegato);
				if (i++ % ConstantsGlobal.flushSize == 0) {
					session.flush();
					session.clear();
				}
			}
		}
		tx.commit();
		closeSession(session);
	}
	public void salvaMessaggioNuovoInvia(Messaggio messaggio) {
		messaggio.setData_invio(ApplicationUtil.date());
		messaggio.setStato(true);
		messaggio.setCestino(false);
		messaggio.setEliminato(false);
		Session session = openSession();
		Transaction tx = session.beginTransaction();
		session.save(messaggio);
		List<Destinatario> destinatari = messaggio.getDestinatari();
		if(ApplicationUtil.notEmpty(destinatari)){
			int i = 0;
			Iterator<Destinatario> objIter = destinatari.iterator();
			while(objIter.hasNext()) {
				Destinatario destinatario = objIter.next();
				destinatario.setMessaggio(messaggio);
				session.save(destinatario);
				if (i++ % ConstantsGlobal.flushSize == 0) {
					session.flush();
					session.clear();
				}
			}
		}
		List<AllegatoMessaggio> allegati = messaggio.getAllegati(); 
		if(ApplicationUtil.notEmpty(allegati)){
			int i = 0;
			Iterator<AllegatoMessaggio> objIter = allegati.iterator();
			while(objIter.hasNext()) {
				AllegatoMessaggio allegato = objIter.next();
				allegato.setMessaggio(messaggio);
				session.save(allegato);
				if (i++ % ConstantsGlobal.flushSize == 0) {
					session.flush();
					session.clear();
				}
			}
		}
		tx.commit();
		closeSession(session);
		emailUtenteNotificaMessaggio(messaggio, destinatari, allegati);
	}
	
	public void salvaMessaggioNuovoInvia(Messaggio messaggio, Utente user) {
		messaggio.setData_invio(ApplicationUtil.date());
		messaggio.setCestino(false);
		messaggio.setEliminato(false);
		messaggio.setStato(true);
		List<Destinatario> destinatari = new ArrayList<Destinatario>();
		List<AllegatoMessaggio> allegati = messaggio.getAllegati();
		String emailDestinatario = ApplicationUtil.trimToEmpty(messaggio.getDestinatariString());
		if (ApplicationUtil.isNotBlank(emailDestinatario)) {
			List<String> uns = ApplicationUtil.tokenU(emailDestinatario, ConstantsGlobal.colonSeparator);
			if(ApplicationUtil.notEmpty(uns)) {
				Iterator<String> uIt = uns.iterator();
				while (uIt.hasNext()) {
					String us = ApplicationUtil.subStringUtente((String) uIt.next());
					if(us != null) {
						Utente utente = userDao.getUserByUsername(us);
						if(utente != null && !utente.equals(user)) {
							Destinatario d = new Destinatario(utente);
							if(!destinatari.contains(d)) {
								destinatari.add(d);
							}
						}
					}
				}
			}
		}
		if(ApplicationUtil.empty(destinatari)){
			messaggio.setStato(false);
		}
		Session session = openSession();
		Transaction tx = session.beginTransaction();
		session.save(messaggio);
		if(ApplicationUtil.notEmpty(destinatari)){
			int i = 0;
			Iterator<Destinatario> objIter = destinatari.iterator();
			while(objIter.hasNext()) {
				Destinatario destinatario = objIter.next();
				destinatario.setMessaggio(messaggio);
				session.save(destinatario);
				if (i++ % ConstantsGlobal.flushSize == 0) {
					session.flush();
					session.clear();
				}
			}
		} 
		if(ApplicationUtil.notEmpty(allegati)){
			int i = 0;
			Iterator<AllegatoMessaggio> objIter = allegati.iterator();
			while(objIter.hasNext()) {
				AllegatoMessaggio allegato = objIter.next();
				allegato.setMessaggio(messaggio);
				session.save(allegato);
				if (i++ % ConstantsGlobal.flushSize == 0) {
					session.flush();
					session.clear();
				}
			}
		}
		tx.commit();
		closeSession(session);
		if(ApplicationUtil.notEmpty(destinatari)){
			emailUtenteNotificaMessaggio(messaggio, destinatari, allegati);
		}
	}
	public void salvaMessaggioBozza(Messaggio messaggio) {
		messaggio.setStato(false);
		messaggio.setCestino(false);
		messaggio.setEliminato(false);
		salvaAggiorna(messaggio);
	}
	public void salvaMessaggioInvia(Messaggio messaggio, List<Destinatario> destinatari, List<AllegatoMessaggio> allegati) {
		messaggio.setData_invio(ApplicationUtil.date());
		messaggio.setStato(true);
		messaggio.setCestino(false);
		messaggio.setEliminato(false);
		salvaAggiorna(messaggio);
		emailUtenteNotificaMessaggio(messaggio, destinatari, allegati);
	}
	private ExtendedCriteria trovaCriteriaMessaggioCestinoInUscita(Utente utente, MessaggioCestinoInUscitaCriteria criterion) {
		if(criterion != null) criterion = (MessaggioCestinoInUscitaCriteria) criterion.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Messaggio.class);
		criteria.add(Restrictions.eq("cestino", true));
		criteria.add(Restrictions.eq("eliminato", false));
		if (ApplicationUtil.isNotBlank(criterion.getTesto())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, criterion.getTesto(), criteria, "oggetto", "testo");
		}
		if(ApplicationUtil.isNotBlank(criterion.getDestinatario())) {
			DetachedCriteria criteriaD = criteria.createCriteria("destinatari", ApplicationUtil.leftJoin());
			DetachedCriteria criteriaU = criteriaD.createCriteria("user", ApplicationUtil.leftJoin());
			ApplicationUtil.operatore(criteriaU, criterion.getDestinatario());
		}
		criteria.createCriteria("mittente").add(Restrictions.idEq(utente.getId()));
		criterion.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, criterion);
	}
	@SuppressWarnings("unchecked")
	public List<Messaggio> trovaMessaggioCestinoInUscita(Utente utente, MessaggioCestinoInUscitaCriteria ana, int start, int size) {
		ExtendedCriteria criteria = trovaCriteriaMessaggioCestinoInUscita(utente, ana);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.desc("id")});
		return getList(criteria, start, size);
	}
	public int trovaMessaggioCestinoInUscitaSize(Utente utente, MessaggioCestinoInUscitaCriteria ana) {
		ExtendedCriteria criteria = trovaCriteriaMessaggioCestinoInUscita(utente, ana);
		return getListSize(criteria);
	}
	private ExtendedCriteria trovaCriteriaMessaggioCestinoInArrivo(Utente utente, MessaggioCestinoInArrivoCriteria criterion) {
		if(criterion != null) criterion = (MessaggioCestinoInArrivoCriteria) criterion.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Destinatario.class);
		criteria.createCriteria("user").add(Restrictions.idEq(utente.getId()));
		criteria.add(Restrictions.eq("cestino", true));
		criteria.add(Restrictions.eq("eliminato", false));
		DetachedCriteria crtMess = criteria.createCriteria("messaggio");
		crtMess.add(Restrictions.ne("stato", false));
		if (ApplicationUtil.isNotBlank(criterion.getTesto())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, criterion.getTesto(), crtMess, "oggetto", "testo");
		}
		if(ApplicationUtil.isNotBlank(criterion.getMittente())) {
			DetachedCriteria criteriaM = crtMess.createCriteria("mittente", ApplicationUtil.leftJoin());
			ApplicationUtil.operatore(criteriaM, criterion.getMittente());
		}
		criterion.impostaRicercaDate(crtMess);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, criterion);
	}
	@SuppressWarnings("unchecked")
	public List<Destinatario> trovaMessaggioCestinoInArrivo(Utente utente, MessaggioCestinoInArrivoCriteria ana, int start, int size) {
		ExtendedCriteria criteria = trovaCriteriaMessaggioCestinoInArrivo(utente, ana);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.desc("id")});
		return getList(criteria, start, size);
	}
	public int trovaMessaggioCestinoInArrivoSize(Utente utente, MessaggioCestinoInArrivoCriteria ana) {
		ExtendedCriteria criteria = trovaCriteriaMessaggioCestinoInArrivo(utente, ana);
		return getListSize(criteria);
	}
	public Messaggio messaggioInInvio(Utente utente) {
		Messaggio mm = new Messaggio();
		mm.setMittente(utente);
		mm.setDestinatari(new ArrayList<Destinatario>());
		mm.setAllegati(new ArrayList<AllegatoMessaggio>());
		return mm;
	}
	public Messaggio messaggioInoltro(Utente utente, Messaggio messaggio, List<Destinatario> destinatari, List<AllegatoMessaggio> allegati) {
		List<AllegatoMessaggio> all = new ArrayList<AllegatoMessaggio>();
		if(ApplicationUtil.notEmpty(allegati)) {
			for(AllegatoMessaggio dd: allegati){
				all.add(dd.getCopia());
			}
		}
		Messaggio mm = new Messaggio();
		mm.setCestino(false);
		mm.setStato(false);
		mm.setEliminato(false);
		mm.setMittente(utente);
		mm.setDestinatari(new ArrayList<Destinatario>());
		mm.setDestinatariString(ApplicationUtil.prendiDestinatarioMessaggi(null));
		mm.setAllegati(all);
		mm.setAllegatiString(ApplicationUtil.prendiAllegatoMessaggi(null));
		mm.setOggetto("Fwd: " + messaggio.getOggetto());
		mm.setTesto(downloadDao.getMessaggioTemplate(messaggio, destinatari, allegati));
		return mm;
	}
	public Messaggio messaggioRispondiSingolo(Utente utente, Messaggio messaggio, List<Destinatario> destinatari, List<AllegatoMessaggio> allegati) {
		List<Destinatario> dest = new ArrayList<Destinatario>();
		dest.add(new Destinatario(messaggio.getMittente()));
		Messaggio mm = new Messaggio();
		mm.setCestino(false);
		mm.setStato(false);
		mm.setEliminato(false);
		mm.setMittente(utente);
		mm.setDestinatari(dest);
		mm.setDestinatariString(ApplicationUtil.prendiDestinatarioMessaggi(null));
		mm.setAllegati(new ArrayList<AllegatoMessaggio>());
		mm.setAllegatiString(ApplicationUtil.prendiAllegatoMessaggi(null));
		mm.setOggetto("Re: " + messaggio.getOggetto());
		mm.setTesto(downloadDao.getMessaggioTemplate(messaggio, destinatari, allegati));
		return mm;
	}
	public Messaggio messaggioRispondiSingoloHome(Utente utente, Messaggio messaggio, List<Destinatario> destinatari, List<AllegatoMessaggio> allegati) {
		List<Destinatario> dest = new ArrayList<Destinatario>();
		dest.add(new Destinatario(messaggio.getMittente()));
		Messaggio mm = new Messaggio();
		mm.setCestino(false);
		mm.setStato(false);
		mm.setEliminato(false);
		mm.setMittente(utente);
		mm.setDestinatari(new ArrayList<Destinatario>());
		mm.setDestinatariString(ApplicationUtil.prendiDestinatarioMessaggi(dest));
		mm.setAllegati(new ArrayList<AllegatoMessaggio>());
		mm.setAllegatiString(ApplicationUtil.prendiAllegatoMessaggi(null));
		mm.setOggetto("Re: " + messaggio.getOggetto());
		mm.setTesto(downloadDao.getMessaggioTemplate(messaggio, destinatari, allegati));
		return mm;
	}
	
	public Messaggio messaggioRispondiTutti(Utente utente, Messaggio messaggio, List<Destinatario> destinatari, List<AllegatoMessaggio> allegati) {
		List<Destinatario> dest = new ArrayList<Destinatario>();
		dest.add(new Destinatario(messaggio.getMittente()));
		if(ApplicationUtil.notEmpty(destinatari)) {
			for(Destinatario dd: destinatari){
				if(!dd.getUser().equals(utente)) {
					dest.add(dd.getCopia());
				}
			}
		}
		Messaggio mm = new Messaggio();
		mm.setCestino(false);
		mm.setStato(false);
		mm.setEliminato(false);
		mm.setMittente(utente);
		mm.setDestinatari(dest);
		mm.setDestinatariString(ApplicationUtil.prendiDestinatarioMessaggi(null));
		mm.setAllegati(new ArrayList<AllegatoMessaggio>());
		mm.setAllegatiString(ApplicationUtil.prendiAllegatoMessaggi(null));
		mm.setOggetto("Re: " + messaggio.getOggetto());
		mm.setTesto(downloadDao.getMessaggioTemplate(messaggio, destinatari, allegati));
		return mm;
	}
	
	public Messaggio messaggioRispondiTuttiHome(Utente utente, Messaggio messaggio, List<Destinatario> destinatari, List<AllegatoMessaggio> allegati) {
		List<Destinatario> dest = new ArrayList<Destinatario>();
		dest.add(new Destinatario(messaggio.getMittente()));
		if(ApplicationUtil.notEmpty(destinatari)) {
			for(Destinatario dd: destinatari){
				if(!dd.getUser().equals(utente)) {
					dest.add(dd.getCopia());
				}
			}
		}
		Messaggio mm = new Messaggio();
		mm.setCestino(false);
		mm.setStato(false);
		mm.setEliminato(false);
		mm.setMittente(utente);
		mm.setDestinatari(new ArrayList<Destinatario>());
		mm.setDestinatariString(ApplicationUtil.prendiDestinatarioMessaggi(dest));
		mm.setAllegati(new ArrayList<AllegatoMessaggio>());
		mm.setAllegatiString(ApplicationUtil.prendiAllegatoMessaggi(null));
		mm.setOggetto("Re: " + messaggio.getOggetto());
		mm.setTesto(downloadDao.getMessaggioTemplate(messaggio, destinatari, allegati));
		return mm;
	}
	
	public Messaggio messaggioNuovo(Utente utente) {
		Messaggio mm = new Messaggio();
		mm.setCestino(false);
		mm.setStato(false);
		mm.setEliminato(false);
		mm.setDestinatari(new ArrayList<Destinatario>());
		mm.setDestinatariString(ApplicationUtil.prendiDestinatarioMessaggi(null));
		mm.setAllegati(new ArrayList<AllegatoMessaggio>());
		mm.setAllegatiString(ApplicationUtil.prendiAllegatoMessaggi(null));
		mm.setMittente(utente);
		mm.setOggetto("");
		mm.setTesto("");
		return mm;
	}
	
	public void segna(Utente utente, List<Destinatario> messaggi, boolean letto) {
		Iterator<Destinatario> itCont = messaggi.iterator();
		while (itCont.hasNext()) {
			Destinatario messaggio = (Destinatario) itCont.next();
			segna(utente, messaggio, letto);
		}
	}
	public void segna(Utente utente, Destinatario messaggio, boolean letto) {
		messaggio.setStato(letto);
		salvaAggiorna(messaggio);
	}
	public void scaricaAllegato(Messaggio messaggio, DownloadBean downloadBean) {
		List<AllegatoMessaggio> allegati = getListaAllegati(messaggio);
		if(ApplicationUtil.notEmpty(allegati)) {
			String p = ConstantsGlobal.secretZ;
			try {
				String file_header = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.MESSAGGIO.stampa(), messaggio.getId()) + "." + ConstantsMap.zip;
				OutputStream out = downloadBean.outputStream();
				downloadBean.filename(file_header, allegati);
				List<Allegato> all = new ArrayList<Allegato>();
				Iterator<AllegatoMessaggio> itA = allegati.iterator();
				while (itA.hasNext()) {
					AllegatoMessaggio al = (AllegatoMessaggio) itA.next();
					ApplicationUtil.addFile(all, al.getId() + "_" + al.getNome(), al.getFile(), al.getData_creazione().getTime());
				}

				ApplicationUtil.zip(null,all, out, p);
				downloadBean.close();

			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		}
	}
	public void scaricaAllegato(List<AllegatoMessaggio> allegati, DownloadBean downloadBean) {
		if(ApplicationUtil.notEmpty(allegati)) {
			String p = ConstantsGlobal.secretZ;
			try {
				String file_header = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.MESSAGGIO.stampa()) + "." + ConstantsMap.zip;
				OutputStream out = downloadBean.outputStream();
				downloadBean.filename(file_header, allegati);
				List<Allegato> all = new ArrayList<Allegato>();
				Iterator<AllegatoMessaggio> itA = allegati.iterator();
				while (itA.hasNext()) {
					AllegatoMessaggio al = (AllegatoMessaggio) itA.next();
					ApplicationUtil.addFile(all, al.getId() + "_" + al.getNome(), al.getFile(), al.getData_creazione().getTime());
				}

				ApplicationUtil.zip(null,all, out, p);
				downloadBean.close();

			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		}
	}
	public void salvaAllegato(FileUpload fileUpload, Messaggio messaggio) {
		try {
			List<MyFile> fls = fileUpload.getFiles();
			if(ApplicationUtil.notEmpty(fls)) {
				Iterator<MyFile> flsIt = fls.iterator();
				while (flsIt.hasNext()) {
					MyFile ff = (MyFile) flsIt.next();
					AllegatoMessaggio all = new AllegatoMessaggio(ff.getLength(), ApplicationUtil.date(), ff.getData(), ff.getName());
					all.setMessaggio(messaggio);
					salvaAggiorna(all);
				}
			}
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	@SuppressWarnings("unchecked")
	public List<AllegatoMessaggio> getListaAllegati(Messaggio messaggio) {
		if(messaggio==null)
			return null;
		DetachedCriteria criteria = DetachedCriteria.forClass(AllegatoMessaggio.class);
		criteria.add(Restrictions.eq("messaggio.id", messaggio.getId()));
		return getList(criteria);
	}
	public void eliminaAllegato(AllegatoMessaggio allegato) {
		cancellaAggiorna(allegato);
	}
	public void eliminaAllegati(Messaggio messaggio) {
		List<AllegatoMessaggio> allegati = getListaAllegati(messaggio);
		Iterator<AllegatoMessaggio> itAn = allegati.iterator();
		while (itAn.hasNext()) {
			AllegatoMessaggio allegato = (AllegatoMessaggio) itAn.next();
			eliminaAllegato(allegato);
		}
	}
	@SuppressWarnings("unchecked")
	public List<Destinatario> trovaUtentiMessaggioTutte(Long messaggio) {
		DetachedCriteria criteriaC = DetachedCriteria.forClass(Destinatario.class);
		criteriaC.createCriteria("messaggio").add(Restrictions.idEq(messaggio));
		return getList(criteriaC);
	}
	@SuppressWarnings("unchecked")
	public List<Utente> trovaSenzaMessaggio(Utente utente, Long agenteId, UserCriteria criteUser, int start, int size) {
		ExtendedCriteria criteria = trovaSenzaMessaggioCriteria(utente, agenteId, criteUser);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), criteUser.getO_tipo(), criteUser.getO_colonna(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int trovaSenzaMessaggioSize(Utente utente, Long agenteId, UserCriteria user) {
		ExtendedCriteria criteria = trovaSenzaMessaggioCriteria(utente, agenteId, user);
		return getListSize(criteria);
	}
	private ExtendedCriteria trovaSenzaMessaggioCriteria(Utente utente, Long messaggioId, UserCriteria user) {
		if(user != null) user = (UserCriteria) user.loadCriteria();
		DetachedCriteria criteriaA = DetachedCriteria.forClass(Utente.class);
		criteriaA.createCriteria("destinatari").createCriteria("messaggio").add(Restrictions.idEq(messaggioId));
		criteriaA.setProjection(Projections.distinct(Projections.property("id")));
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		impostazioneDao.subPropertyNotInS(criteria, criteriaA, "id");
		criteria.add(Restrictions.ne("id", utente.getId()));
		ApplicationUtil.utente(criteria, user);
		user.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, user);
	}
	public void aggiungiMessaggioAUtente(Utente utente, Long messaggioId) {
		Messaggio messaggio = getMessaggio(messaggioId);
		if (utente != null && messaggio != null) {
			Destinatario dest = new Destinatario(utente);
			dest.setMessaggio(messaggio);
			salvaAggiorna(dest);
		}
	}
	public void rimuoviMessaggioDaUtente(Destinatario dest, Long messaggioId) {
		if (dest != null) {
			cancellaAggiorna(dest);
		}
	}
	public void aggiungiMessaggioAUtenteTutti(List<Utente> utenti, Long messaggioId) {
		if(ApplicationUtil.notEmpty(utenti)) {
			Messaggio messaggio = getMessaggio(messaggioId);
			List<Destinatario> dests = new ArrayList<Destinatario>();
			Iterator<Utente> itDes = utenti.iterator();
			while (itDes.hasNext()) {
				Utente user = itDes.next();
				Destinatario dest = new Destinatario(user);
				dest.setMessaggio(messaggio);
				dests.add(dest);
			}
			salvaLista(dests);
		}
	}
	public void rimuoviMessaggioAUtenteTutti(List<Destinatario> dests, Long messaggioId) {
		cancellaLista(dests);
	}
	public void svuotaCestinoInUscita(Utente utente) {
		String hql = "update Messaggio set cestino = true, eliminato = true where mittente like :mittente";
		execute(false, hql, new Parameter("mittente", utente));
	}
	public void svuotaCestinoInArrivo(Utente utente) {
		String hql = "update Destinatario set cestino = true, eliminato = true where user like :user";
		execute(false, hql, new Parameter("user", utente));
	}
	
	public void emailUtenteNotificaMessaggio(Messaggio messaggio, List<Destinatario> destinatari, List<AllegatoMessaggio> allegati) {
		Boolean zip = Boolean.valueOf(configurazioneGenericService.getValore(DefaultsGeneric.KEY_MESSAGGISTICA_ALLEGATO_COMPRESSO));
		if(ApplicationUtil.notEmpty(destinatari)) {
			new MessaggioThread(messaggio, destinatari, allegati, ApplicationUtil.isTrue(zip), smtpService, downloadDao).start();
		}
	}
	
	private ExtendedCriteria trovaCriteriaDocumento(Utente utente, DocumentoCriteria criterion) {
		if(criterion != null) criterion = (DocumentoCriteria) criterion.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(DocumentoMessaggio.class);
		criteria.createCriteria("utente").add(Restrictions.idEq(utente.getId()));
		if (ApplicationUtil.isNotBlank(criterion.getTesto())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, criterion.getTesto(), criteria, "nome");
		}
		criterion.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, criterion);
	}
	@SuppressWarnings("unchecked")
	public List<Destinatario> trovaDocumento(Utente utente, DocumentoCriteria ana, int start, int size) {
		ExtendedCriteria criteria = trovaCriteriaDocumento(utente, ana);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.desc("id")});
		return getList(criteria, start, size);
	}
	public int trovaDocumentoSize(Utente utente, DocumentoCriteria ana) {
		ExtendedCriteria criteria = trovaCriteriaDocumento(utente, ana);
		return getListSize(criteria);
	}
	public void salvaDocumento(FileUpload fileUpload, Utente utente) {
		try {
			List<MyFile> fls = fileUpload.getFiles();
			if(ApplicationUtil.notEmpty(fls)) {
				Iterator<MyFile> flsIt = fls.iterator();
				while (flsIt.hasNext()) {
					MyFile ff = (MyFile) flsIt.next();
					DocumentoMessaggio documento = new DocumentoMessaggio(ff.getLength(), ApplicationUtil.date(), ff.getData(), ff.getName());
					documento.setUtente(utente);
					salvaAggiorna(documento);
				}
			}
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void eliminaDocumento(DocumentoMessaggio documento) {
		cancellaAggiorna(documento);
	}
	public void scaricaDocumento(DocumentoMessaggio documento, DownloadBean downloadBean) {
		try {

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(documento.getNome());

			InputStream is = new ByteArrayInputStream(documento.getFile());
			ApplicationUtil.copy(is, out);
			downloadBean.close();
			is.close();

		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void eliminaDocumentoSelezionati(List<DocumentoMessaggio> documenti) {
		if (ApplicationUtil.notEmpty(documenti)) {
			Iterator<DocumentoMessaggio> itCont = documenti.iterator();
			while (itCont.hasNext()) {
				DocumentoMessaggio documento = (DocumentoMessaggio) itCont.next();
				eliminaDocumento(documento);
			}
		}
	}
	public void scaricaDocumentoSelezionati(List<DocumentoMessaggio> documenti, DownloadBean downloadBean) {
		if(ApplicationUtil.notEmpty(documenti)) {
			String p = ConstantsGlobal.secretZ;
			try {

				String file_header = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.DOCUMENTO.stampa()) + "." + ConstantsMap.zip;
				OutputStream out = downloadBean.outputStream();
				downloadBean.filename(file_header, documenti);
				List<Allegato> all = new ArrayList<Allegato>();
				Iterator<DocumentoMessaggio> itA = documenti.iterator();
				while (itA.hasNext()) {
					DocumentoMessaggio al = (DocumentoMessaggio) itA.next();
					ApplicationUtil.addFile(all, al.getId() + "_" + al.getNome(), al.getFile(), al.getData_creazione().getTime());
				}

				ApplicationUtil.zip(null,all, out, p);
				downloadBean.close();

			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		}
	}
	@SuppressWarnings("unchecked")
	public List<Destinatario> trovaDocumento(DocumentoCriteria ana, int start, int size) {
		ExtendedCriteria criteria = trovaCriteriaDocumento(ana);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.desc("id")});
		return getList(criteria, start, size);
	}
	public int trovaDocumentoSize( DocumentoCriteria ana) {
		ExtendedCriteria criteria = trovaCriteriaDocumento(ana);
		return getListSize(criteria);
	}
	private ExtendedCriteria trovaCriteriaDocumento(DocumentoCriteria criterion) {
		if(criterion != null) criterion = (DocumentoCriteria) criterion.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(DocumentoMessaggio.class);
		DetachedCriteria criteriaU = criteria.createCriteria("utente", ApplicationUtil.leftJoin());
		if (ApplicationUtil.isNotBlank(criterion.getTesto())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, criterion.getTesto(), criteria, "nome");
		}
		if(ApplicationUtil.isNotBlank(criterion.getUtente())) {
			ApplicationUtil.operatore(criteriaU, criterion.getUtente());
		}
		criterion.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, criterion);
	}
	private ExtendedCriteria trovaCriteriaDocumentoPopup(Utente utente, List<AllegatoMessaggio> allegati, DocumentoCriteria criterion) {
		if(criterion != null) criterion = (DocumentoCriteria) criterion.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(DocumentoMessaggio.class);
		criteria.createCriteria("utente").add(Restrictions.idEq(utente.getId()));
		List<String> nomi = prendiNomiDocumenti(allegati);
		if(ApplicationUtil.notEmpty(nomi)) {
			criteria.add(Restrictions.not(Restrictions.in("nome", nomi)));
		}
		if (ApplicationUtil.isNotBlank(criterion.getTesto())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, criterion.getTesto(), criteria, "nome");
		}
		criterion.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, criterion);
	}
	@SuppressWarnings("unchecked")
	public List<Destinatario> trovaDocumentoPopup(Utente utente, List<AllegatoMessaggio> allegati, DocumentoCriteria ana, int start, int size) {
		ExtendedCriteria criteria = trovaCriteriaDocumentoPopup(utente, allegati, ana);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.desc("id")});
		return getList(criteria, start, size);
	}
	public int trovaDocumentoPopupSize(Utente utente, List<AllegatoMessaggio> allegati, DocumentoCriteria ana) {
		ExtendedCriteria criteria = trovaCriteriaDocumentoPopup(utente, allegati, ana);
		return getListSize(criteria);
	}
	public List<AllegatoMessaggio> aggiungiAllegatoALista(List<AllegatoMessaggio> allegati, DocumentoMessaggio documento) {
		if(allegati==null){
			allegati = new ArrayList<AllegatoMessaggio>();
		}
		AllegatoMessaggio all = new AllegatoMessaggio(documento.getDimensione(), documento.getData_modifica(), documento.getFile(), documento.getNome());
		if(!allegati.contains(all)){
			allegati.add(all);
		} 
		return allegati;
	}
	public boolean salvaAllegato(DocumentoMessaggio documento, Messaggio messaggio) {
		try {
			AllegatoMessaggio all = new AllegatoMessaggio(documento.getDimensione(), documento.getData_modifica(), documento.getFile(), documento.getNome());
			all.setMessaggio(messaggio);
			salvaAggiorna(all);
			return true;
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
		return false;
	}
	public List<String> prendiNomiDocumenti(List<AllegatoMessaggio> allegati) {
		List<String> allegatiId = new ArrayList<String>();
		if(ApplicationUtil.notEmpty(allegati)) {
			Iterator<AllegatoMessaggio> objIter = allegati.iterator();
			while(objIter.hasNext()) {
				AllegatoMessaggio allegato = objIter.next();
				allegatiId.add(allegato.getNome());
			}
		}
		return allegatiId;
	}
	public int trovaMessaggioInArrivoNuovi(Utente utente) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Destinatario.class);
		criteria.createCriteria("user").add(Restrictions.idEq(utente.getId()));
		criteria.add(Restrictions.eq("cestino", false));
		criteria.add(Restrictions.eq("eliminato", false));
		criteria.add(Restrictions.ne("stato", true));
		DetachedCriteria crtMess = criteria.createCriteria("messaggio");
		crtMess.add(Restrictions.ne("stato", false));
		return getListSize(criteria);
	}
	public int trovaMessaggioInArrivoTotali(Utente utente) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Destinatario.class);
		criteria.createCriteria("user").add(Restrictions.idEq(utente.getId()));
		criteria.add(Restrictions.eq("cestino", false));
		criteria.add(Restrictions.eq("eliminato", false));
		DetachedCriteria crtMess = criteria.createCriteria("messaggio");
		crtMess.add(Restrictions.ne("stato", false));
		return getListSize(criteria);
	}
	public int trovaMessaggioInUscita(Utente utente) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Messaggio.class);
		criteria.add(Restrictions.eq("stato", true));
		criteria.add(Restrictions.eq("cestino", false));
		criteria.add(Restrictions.eq("eliminato", false));
		criteria.createCriteria("mittente").add(Restrictions.idEq(utente.getId()));
		return getListSize(criteria);
	}
	public int trovaMessaggioBozza(Utente utente) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Messaggio.class);
		criteria.add(Restrictions.eq("stato", false));
		criteria.add(Restrictions.eq("cestino", false));
		criteria.add(Restrictions.eq("eliminato", false));
		criteria.createCriteria("mittente").add(Restrictions.idEq(utente.getId()));
		return getListSize(criteria);
	}
	public int trovaMessaggioCestinoInUscita(Utente utente) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Messaggio.class);
		criteria.add(Restrictions.eq("cestino", true));
		criteria.add(Restrictions.eq("eliminato", false));
		criteria.createCriteria("mittente").add(Restrictions.idEq(utente.getId()));
		return getListSize(criteria);
	}
	public int trovaMessaggioCestinoInArrivoTotali(Utente utente) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Destinatario.class);
		criteria.createCriteria("user").add(Restrictions.idEq(utente.getId()));
		criteria.add(Restrictions.eq("cestino", true));
		criteria.add(Restrictions.eq("eliminato", false));
		DetachedCriteria crtMess = criteria.createCriteria("messaggio");
		crtMess.add(Restrictions.ne("stato", false));
		return getListSize(criteria);
	}
	public int trovaMessaggioCestinoInArrivoNuovi(Utente utente) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Destinatario.class);
		criteria.createCriteria("user").add(Restrictions.idEq(utente.getId()));
		criteria.add(Restrictions.eq("cestino", true));
		criteria.add(Restrictions.eq("eliminato", false));
		criteria.add(Restrictions.ne("stato", true));
		DetachedCriteria crtMess = criteria.createCriteria("messaggio");
		crtMess.add(Restrictions.ne("stato", false));
		return getListSize(criteria);
	}
	public int trovaDocumenti(Utente utente) {
		DetachedCriteria criteria = DetachedCriteria.forClass(DocumentoMessaggio.class);
		criteria.createCriteria("utente").add(Restrictions.idEq(utente.getId()));
		return getListSize(criteria);
	}
	public Messaggistica trovaMessaggi(Utente utente) {
		Messaggistica mm = new Messaggistica();
		mm.setMessaggiBozza(trovaMessaggioBozza(utente));
		mm.setMessaggiCestinoInArrivoTotali(trovaMessaggioCestinoInArrivoTotali(utente));
		mm.setMessaggiCestinoInArrivoNuovi(trovaMessaggioCestinoInArrivoNuovi(utente));
		mm.setMessaggiCestinoInUscita(trovaMessaggioCestinoInUscita(utente));
		mm.setMessaggiInUscita(trovaMessaggioInUscita(utente));
		mm.setMessaggiInArrivoTotali(trovaMessaggioInArrivoTotali(utente));
		mm.setMessaggiInArrivoNuovi(trovaMessaggioInArrivoNuovi(utente));
		mm.setDocumenti(trovaDocumenti(utente));
		return mm;
	}
	public int trovaMessaggioInArrivoNuovi(Long id) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Destinatario.class);
		criteria.createCriteria("user").add(Restrictions.idEq(id));
		criteria.add(Restrictions.eq("cestino", false));
		criteria.add(Restrictions.eq("eliminato", false));
		criteria.add(Restrictions.ne("stato", true));
		DetachedCriteria crtMess = criteria.createCriteria("messaggio");
		crtMess.add(Restrictions.ne("stato", false));
		return getListSize(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Destinatario> trovaMessaggioInArrivoNuoviList(Long id, Long messId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Destinatario.class);
		criteria.createCriteria("user").add(Restrictions.idEq(id));
		criteria.add(Restrictions.eq("cestino", false));
		criteria.add(Restrictions.eq("eliminato", false));
		criteria.add(Restrictions.ne("stato", true));
		DetachedCriteria crtMess = criteria.createCriteria("messaggio");
		if(ApplicationUtil.gtZero(messId)) {
			crtMess.add(Restrictions.idEq(messId));
		}
		crtMess.add(Restrictions.ne("stato", false));
		criteria.addOrder(Order.desc("id"));
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Destinatario> trovaMessaggiPosta(Utente utente) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Destinatario.class);
		criteria.createCriteria("user").add(Restrictions.idEq(utente.getId()));
		criteria.add(Restrictions.eq("cestino", false));
		criteria.add(Restrictions.eq("eliminato", false));
		criteria.add(Restrictions.ne("stato", true));
		DetachedCriteria crtMess = criteria.createCriteria("messaggio");
		crtMess.add(Restrictions.ne("stato", false));
		criteria.addOrder(Order.desc("id"));
		return getList(criteria);
	}
	public List<Destinatario> impostaConteggioPosta(List<Destinatario> destinatari) {
		if(ApplicationUtil.notEmpty(destinatari)) {
			List<Long> ll = ApplicationUtil.prendiIdDestinatarioMessaggi(destinatari);
			Map<Long, ColonnaModel> map = dataDao.getConteggioColonna(null, ll, Messaggio.class, new ColonnaQuery("allegati", "allegati"));
			Iterator<Destinatario> it = destinatari.iterator();
			while (it.hasNext()) {
				Destinatario o = it.next();
				ColonnaModel lmap = map.get(o.getMessaggio().getId());
				if(lmap != null) {
					o.getMessaggio().setModel(lmap);
				}
			}
			
		}
		return destinatari;
	}
	public void rimuoviMessaggioDaUtente(Utente utente, Destinatario destinatario) {
		segna(utente, destinatario, true);
	}
	@SuppressWarnings("unchecked")
	public List<AllegatoMessaggio> getListaAllegati(Destinatario destinatario) {
		if(destinatario==null)
			return null;
		DetachedCriteria criteria = DetachedCriteria.forClass(AllegatoMessaggio.class);
		criteria.add(Restrictions.eq("messaggio.id", destinatario.getMessaggio().getId()));
		return getList(criteria);
	}
	public void scaricaAllegato(Destinatario destinatario, DownloadBean downloadBean) {
		List<AllegatoMessaggio> allegati = getListaAllegati(destinatario);
		if(ApplicationUtil.notEmpty(allegati)) {
			String p = ConstantsGlobal.secretZ;
			try {
				String file_header = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.MESSAGGIO.stampa(), destinatario.getMessaggio().getId()) + "." + ConstantsMap.zip;
				OutputStream out = downloadBean.outputStream();
				downloadBean.filename(file_header, allegati);
				List<Allegato> all = new ArrayList<Allegato>();
				Iterator<AllegatoMessaggio> itA = allegati.iterator();
				while (itA.hasNext()) {
					AllegatoMessaggio al = (AllegatoMessaggio) itA.next();
					ApplicationUtil.addFile(all, al.getId() + "_" + al.getNome(), al.getFile(), al.getData_creazione().getTime());
				}

				ApplicationUtil.zip(null,all, out, p);
				downloadBean.close();

			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		}
	}
	public Messaggio messaggioRispondiRapido(Boolean singolo, Utente utente, Long id) {
		Messaggio messaggio = getMessaggio(id);
		return messaggioRispondiHome(singolo, utente, messaggio);
	}
	
	public Messaggio messaggioRispondiHome(Boolean singolo, Utente utente, Messaggio messaggio) {
		List<Destinatario> destinatari = listaDestinatari(messaggio);
		 List<AllegatoMessaggio> allegati = listaAllegati(messaggio);
		if(ApplicationUtil.isTrue(singolo)) {
			return messaggioRispondiSingoloHome(utente, messaggio, destinatari, allegati);
		} else {
			return messaggioRispondiTuttiHome(utente, messaggio, destinatari, allegati);
		}
	}
	public void leggiMessaggi(Long id, Long messID) {
		List<Destinatario> messaggi = trovaMessaggioInArrivoNuoviList(id, messID);
		if(ApplicationUtil.notEmpty(messaggi)) {
			Iterator<Destinatario> it = messaggi.iterator();
			while (it.hasNext()) {
				Destinatario destinatario = (Destinatario) it.next();
				destinatario.setStato(true);
				salvaAggiorna(destinatario);
			}
		}
	}
	public Messaggio apriMessaggio(Utente mittente, Utente destinatario) {
		List<Destinatario> destinatari = new ArrayList<Destinatario>();
		if (destinatario != null ) {
			destinatari.add(new Destinatario(destinatario));
		}
		if(ApplicationUtil.notEmpty(destinatari)) {
			Messaggio messaggio = messaggioInInvio(mittente);
			messaggio.setDestinatari(new ArrayList<Destinatario>());
			messaggio.setDestinatariString(ApplicationUtil.prendiDestinatarioMessaggi(destinatari));
			messaggio.setAllegati(null);
			messaggio.setAllegatiString(ApplicationUtil.prendiAllegatoMessaggi(null));
			return messaggio;
		}
		return null;
	}

	public Messaggio apriMessaggio(Utente mittente) {
		List<Destinatario> destinatari = new ArrayList<Destinatario>();
		Map<Utente, MonitorOperatore> operatori = Chiamate.getCodaOperatori().getData();
		if(ApplicationUtil.notEmpty(operatori)) {
			Iterator<Utente> objRIter = operatori.keySet().iterator();
			while(objRIter.hasNext()) {
				Utente user = objRIter.next();
				if(!mittente.equals(user)) {
					destinatari.add(new Destinatario(user));
				}
			}
		}
		if(ApplicationUtil.notEmpty(destinatari)) {
			Messaggio messaggio = messaggioInInvio(mittente);
			messaggio.setDestinatari(new ArrayList<Destinatario>());
			messaggio.setDestinatariString(ApplicationUtil.prendiDestinatarioMessaggi(destinatari));
			messaggio.setAllegati(null);
			messaggio.setAllegatiString(ApplicationUtil.prendiAllegatoMessaggi(null));
			return messaggio;
		}
		return null;
	}
	
	public void scaricaAllegatiT(Long messaggioId, DownloadBean downloadBean) {
		try {
			Messaggio messaggio = getMessaggio(messaggioId);
			List<AllegatoMessaggio> allegati = getListaAllegati(messaggio);
			if(ApplicationUtil.notEmpty(allegati)) {
				String p = ConstantsGlobal.secretZ;
				OutputStream out = downloadBean.outputStream();
				String filename = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.MESSAGGIO.stampa(), messaggio.getId()) + "." + ConstantsMap.zip;
				downloadBean.filename(filename, allegati);
				List<Allegato> all = new ArrayList<Allegato>();
				Iterator<AllegatoMessaggio> itA = allegati.iterator();
				while (itA.hasNext()) {
					AllegatoMessaggio al = (AllegatoMessaggio) itA.next();
					ApplicationUtil.addFile(all, al.getId() + "_" + al.getNome(), al.getFile(), al.getData_creazione().getTime());
				}
				ApplicationUtil.zip(null,all, out, p);
				downloadBean.close();
			}

		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
}