package it.callite.dao;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.faces.model.SelectItem;

import org.apache.commons.net.ftp.FTPClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import it.callite.action.DownloadBean;
import it.callite.constants.ConstantsGlobal;
import it.callite.criteria.FileCriteria;
import it.callite.dao.generic.GenericDao;
import it.callite.log.ErrorLog;
import it.callite.model.sip.Musiconhold;
import it.callite.utility.Allegato;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.FileUpload;
import it.callite.utility.MyFile;
import it.callite.utility.PathUtility;
import it.callite.utility.UtilFTP;
@SuppressWarnings("serial")
@Repository("musicaDao")
public class MusicaDao extends GenericDao {
	@Autowired
	private ImpostazioneDao impostazioneDao;
	@Autowired
	private AutodialerDao autodialerDao;
	
	@Autowired
	private FileDao fileDao;
	public String getRoot() {
		return ConstantsGlobal.path_musica;
	}
	public Boolean getEnabledFTP() {
		return ApplicationUtil.isNotTrue(impostazioneDao.getFTPEnabled());
	}
	public String editFile(PathUtility p, MyFile att) {
		return fileDao.editFile(p, att);
	}
	public void copyFile(PathUtility p, MyFile att, String percorso, String rinomina) {
		fileDao.copyFile(p, att, percorso, rinomina);
		sincronizza(p);
	}
	public void moveFile(PathUtility p, MyFile att, String percorso, String rinomina) {
		fileDao.moveFile(p, att, percorso, rinomina);
		sincronizza(p);
	}
	public boolean renameFile(PathUtility p, MyFile att, String n) {
		boolean a = fileDao.renameFile(p, att, n);
		sincronizza(p);
		return a;
	}
	public boolean newFolder(PathUtility p, String m) {
		boolean a = fileDao.newFolder(p, m);
		sincronizza(p);
		return a;
	}
	public void closeRoot(PathUtility p) {
		fileDao.closeRoot(p);
	}
	public String openRoot(PathUtility p) {
		return fileDao.openRoot(p);
	}
	public String openRoot(PathUtility p, MyFile m) {
		return fileDao.openRoot(p, m);
	}
	public void openFolder(PathUtility p, MyFile m) {
		fileDao.openFolder(p, m);
	}
	public void closeFolder(PathUtility p) {
		fileDao.closeFolder(p);
	}
	public List<MyFile> listaFile(FileCriteria mm, PathUtility p) {
		return fileDao.listaFile(mm, p);
	}
	public void eliminaFile(MyFile att, PathUtility p) {
		fileDao.eliminaFile(att, p);
		sincronizza(p);
	}
	public void eliminaFile(List<MyFile> atts,  PathUtility p) {
		fileDao.eliminaFile(atts, p);
		sincronizza(p);
	}
	public void scaricaFile(List<MyFile> fils,  PathUtility p, DownloadBean downloadBean) {
		fileDao.scaricaFile(fils, p, downloadBean);
	}
	public void scaricaFile(MyFile att,  PathUtility p, DownloadBean downloadBean) {
		fileDao.scaricaFile(att, p, downloadBean);
	}
	public void salvaFile(FileUpload fileUpload,  PathUtility p) {
		if(getEnabledFTP()) {
	
			try {
				List<MyFile> fls = fileUpload.getFiles();
				if(ApplicationUtil.notEmpty(fls)) {
					String home = impostazioneDao.getHome();
					Iterator<MyFile> flsIt = fls.iterator();
					while (flsIt.hasNext()) {
						MyFile ff = (MyFile) flsIt.next();
						ByteArrayOutputStream bos = new ByteArrayOutputStream();
						String fileN = ApplicationUtil.salvaAudio(ff, bos, false);
						byte [] ffB = bos.toByteArray();
						if(ffB != null  && ffB.length > 0) {
							FileOutputStream fi = new FileOutputStream(home +  p.getPath() + "/" + fileN);
							fi.write(ffB);
							fi.flush();
							fi.close();
						}
					}
				}
			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		} else {
			UtilFTP ut = impostazioneDao.getUtilFTP();
			FTPClient vmftp = ApplicationUtil.createClient();
			if (ApplicationUtil.connect(vmftp, ut)) {
				try {
					List<MyFile> fls = fileUpload.getFiles();
					if(ApplicationUtil.notEmpty(fls)) {
						Iterator<MyFile> flsIt = fls.iterator();
						while (flsIt.hasNext()) {
							MyFile ff = (MyFile) flsIt.next();
							ByteArrayOutputStream bos = new ByteArrayOutputStream();
							String fileN = ApplicationUtil.salvaAudio(ff, bos, false);
							byte [] ffB = bos.toByteArray();
							if(ffB != null  && ffB.length > 0) {
								InputStream  fis = new ByteArrayInputStream(ffB);
								ApplicationUtil.storeFile(vmftp, p.getPath() + "/" + fileN, fis);
								fis.close();
							}
						}
					}
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				} finally {
					ApplicationUtil.disconnect(vmftp);
				}
			}	
		}
		sincronizza(p);
	}
	public void salvaZip(FileUpload fileUpload,  PathUtility p) {
		List<MyFile> scompattati = new ArrayList<MyFile>();
		try {
			List<MyFile> fls = fileUpload.getFiles();
			if(ApplicationUtil.notEmpty(fls)) {
				Iterator<MyFile> flsIt = fls.iterator();
				while (flsIt.hasNext()) {
					MyFile dd = (MyFile) flsIt.next();
					scompattati.addAll(ApplicationUtil.unzip(dd));
				}
			}
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
		if(ApplicationUtil.notEmpty(scompattati)) {
			if(getEnabledFTP()) {
				String home = impostazioneDao.getHome();
				try {
					Iterator<MyFile> flsIt = scompattati.iterator();
					while (flsIt.hasNext()) {
						MyFile ff = (MyFile) flsIt.next();
						ByteArrayOutputStream bos = new ByteArrayOutputStream();
						String fileN = ApplicationUtil.salvaAudio(ff, bos, true);
						byte [] ffB = bos.toByteArray();
						if(ffB != null  && ffB.length > 0) {
							String base = home +  p.getPath();
							File n = new File(base + "/" + ApplicationUtil.getPathN(fileN));
							ApplicationUtil.creaFile(true, n);
							File fileOut = new File(base + "/" + fileN);
							if(fileOut.isFile()) {
								FileOutputStream fi = new FileOutputStream(fileOut);
								fi.write(ff.getData());
								fi.flush();
								fi.close();
							}
						}
					}
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				}
			} else {
				UtilFTP ut = impostazioneDao.getUtilFTP();
				FTPClient vmftp = ApplicationUtil.createClient();
				if (ApplicationUtil.connect(vmftp, ut)) {
					try {
						Iterator<MyFile> flsIt = scompattati.iterator();
						while (flsIt.hasNext()) {
							MyFile ff = (MyFile) flsIt.next();
							ByteArrayOutputStream bos = new ByteArrayOutputStream();
							String fileN = ApplicationUtil.salvaAudio(ff, bos, true);
							byte [] ffB = bos.toByteArray();
							if(ffB != null  && ffB.length > 0) {
								String dir = ApplicationUtil.normalizeNoEndSeparator(p.getPath() + "/" + ApplicationUtil.getPathN(fileN));
								String file = ApplicationUtil.normalizeNoEndSeparator(p.getPath() + "/" + fileN);
								ApplicationUtil.makeDirectories(vmftp, dir);
								InputStream  fis = new ByteArrayInputStream(ffB);
								ApplicationUtil.storeFile(vmftp, file, fis);
								fis.close();
							}
						}
					} catch (Exception e) {
						ErrorLog.log(null, getClass(), e);
					} finally {
						ApplicationUtil.disconnect(vmftp);
					}
				}	
			}
			sincronizza(p);
		}
	}

	public void createFile(PathUtility p, String file, String testo, String language) throws Exception {
		if(getEnabledFTP()) {
			String home = impostazioneDao.getHome();
			ApplicationUtil.createAudio(home +  p.getPath(), file, testo, language);
			sincronizza(p);
		}
	}

	public boolean isFileCreabile() {
		if(getEnabledFTP()) {
			return ApplicationUtil.getPath(ConstantsGlobal.path_espeak, true);
		}
		return false;
	}

	public List<SelectItem> getLingua() {
		List<SelectItem> sis = new ArrayList<SelectItem>();
		for(String l : impostazioneDao.getLingue()) {
			sis.add(new SelectItem(ApplicationUtil.lingua(l)));
		}
		return sis;
	}
	
	public void sincronizza(PathUtility p) {
		String home = impostazioneDao.getHome();
		String base = home +  p.getRoot();
		File baseB = new File(base);
		List<Allegato> allegati = scaricaFile(p);
		execute(true, "truncate table " + ConstantsGlobal.ts + "musiconhold");
		List<Musiconhold> ms = new ArrayList<Musiconhold>();
		String baseC = ApplicationUtil.normalizeNoEndSeparator(baseB.getAbsolutePath());
		List<String> classi = new ArrayList<String>();
		if(ApplicationUtil.notEmpty(allegati)) {
			for(Allegato f : allegati) {
				String dir = ApplicationUtil.normalizeNoEndSeparator(f.getPercorso());
				String sname = ApplicationUtil.trimToNull(ApplicationUtil.removeStart(dir, baseC));
				if(sname != null && !classi.contains(sname)) {
					Musiconhold m = new Musiconhold();
					m.setName(sname);
					m.setDirectory(dir);
					ms.add(m);
					classi.add(sname);
				}
			}
		}
		if(ApplicationUtil.notEmpty(ms)) {
			salvaLista(ms);
		}
		autodialerDao.reloadModule("res_musiconhold");
	}

	
	public List<Allegato> scaricaFile(PathUtility p) {
		List<Allegato> files = new ArrayList<Allegato>();
		String home = impostazioneDao.getHome();
		if(getEnabledFTP()) {
			String root = home + p.getRoot();
			String base = home + p.getRoot();
			ApplicationUtil.generateDirectoryList(files, new File(base), root);
		} else {
			UtilFTP ut = impostazioneDao.getUtilFTP();
			FTPClient vmftp = ApplicationUtil.createClient();
			if (ApplicationUtil.connect(vmftp, ut)) {
				try {
					ApplicationUtil.scaricaDirectory(vmftp, files, p.getRoot() + "/", p.getRoot() + "/", home);
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				} finally {
					ApplicationUtil.disconnect(vmftp);
				}
			}
		}
		return files;
	}
}