package it.callite.dao;
import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.hibernate.FetchMode;
import org.hibernate.Session;
import org.hibernate.Transaction;
import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Projections;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import it.callite.action.DownloadBean;
import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.constants.ConstantsMap;
import it.callite.criteria.NotiziaCriteria;
import it.callite.criteria.UserCriteria;
import it.callite.criteria.base.ExtendedCriteria;
import it.callite.dao.generic.GenericDao;
import it.callite.log.ErrorLog;
import it.callite.model.Notizia;
import it.callite.model.Utente;
import it.callite.model.allegato.AllegatoNotizia;
import it.callite.model.report.ColonnaModel;
import it.callite.model.report.ColonnaQuery;
import it.callite.utility.Allegato;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.FileUpload;
import it.callite.utility.MyFile;
@SuppressWarnings("serial")
@Repository("notiziaDao")
public class NotiziaDao extends GenericDao {

	@Autowired
	private ImpostazioneDao impostazioneDao;
	@Autowired
	private DataDao dataDao;
	
	public Notizia salvaNotizia(Notizia notizia) {
		return (Notizia) salvaAggiorna(notizia);
	}
	public void eliminaNotizia(Notizia notizia) {
		cancellaAggiorna(notizia);
	}
	public void scaricaAllegato(Notizia notizia, DownloadBean downloadBean) {
		List<AllegatoNotizia> allegati = getListaAllegati(notizia);
		if(ApplicationUtil.notEmpty(allegati)) {
			String p = ConstantsGlobal.secretZ;
			try {
				String s = impostazioneDao.getPathBaseR(ConstantsGlobal.path_notizia);
				String file_header = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.NOTIZIA.stampa(), notizia.getId()) + "." + ConstantsMap.zip;
				OutputStream out = downloadBean.outputStream();
				downloadBean.filename(file_header, allegati);
				List<Allegato> all = new ArrayList<Allegato>();
				Iterator<AllegatoNotizia> itA = allegati.iterator();
				while (itA.hasNext()) {
					AllegatoNotizia allegato = (AllegatoNotizia) itA.next();
					ApplicationUtil.addFile(all, allegato.getId() + "_" +allegato.getNome(), allegato.getContentFile(s), allegato.getData().getTime());
				}

				ApplicationUtil.zip(null, all, out, p);
				downloadBean.close();

			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		}
	}
	public void scaricaAllegato(AllegatoNotizia allegato, DownloadBean downloadBean) {
		try {
			String s = impostazioneDao.getPathBaseR(ConstantsGlobal.path_notizia);


			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(allegato.getNome());

			InputStream is = new ByteArrayInputStream(allegato.getContentFile(s));
			ApplicationUtil.copy(is, out);
			downloadBean.close();
			is.close();

		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void salvaAllegato(FileUpload fileUpload, Notizia notizia) {
		try {
			List<MyFile> fls = fileUpload.getFiles();
			if(ApplicationUtil.notEmpty(fls)) {
				String s = impostazioneDao.getPathBaseW(ConstantsGlobal.path_notizia);
				Iterator<MyFile> flsIt = fls.iterator();
				while (flsIt.hasNext()) {
					MyFile ff = (MyFile) flsIt.next();
					byte [] ffB = ff.getData();
					AllegatoNotizia allegato = new AllegatoNotizia();
					allegato.setNotizia(notizia);
					allegato.setDimensione(ffB.length);
					allegato.setNome(ApplicationUtil.getNameN(ff.getName()));
					allegato.setContentFile(ffB, s);
					salvaAggiorna(allegato);
				}
			}
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	@SuppressWarnings("unchecked")
	public List<AllegatoNotizia> getListaAllegati(Notizia notizia) {
		if(notizia==null)
			return null;
		DetachedCriteria criteria = DetachedCriteria.forClass(AllegatoNotizia.class);
		criteria.add(Restrictions.eq("notizia.id", notizia.getId()));
		return getList(criteria);
	}
	public void deleteAllegato(AllegatoNotizia allegato)  {
		String s = impostazioneDao.getPathBaseR(ConstantsGlobal.path_notizia);
		allegato.delete(s);
		cancellaAggiorna(allegato);
	}
	public void deleteAllegati(Notizia notizia)  {
		List<AllegatoNotizia> allegati = getListaAllegati(notizia);
		Iterator<AllegatoNotizia> itAn = allegati.iterator();
		while (itAn.hasNext()) {
			AllegatoNotizia allegato = (AllegatoNotizia) itAn.next();
			deleteAllegato(allegato);
		}
	}
	private ExtendedCriteria trovaCriteria(NotiziaCriteria ana) {
		if(ana != null)  ana = (NotiziaCriteria) ana.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Notizia.class);
		if (ApplicationUtil.isNotBlank(ana.getOggetto())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, ana.getOggetto(), criteria, "oggetto", "testo");
		}
		if (ApplicationUtil.gtZero(ana.getAttivo())) {
			if (ApplicationUtil.eq(ana.getAttivo(), 1)) {
				criteria.add(Restrictions.eq("attivo", Boolean.TRUE));
			} else 	{
				criteria.add(Restrictions.eq("attivo", Boolean.FALSE));
			} 
		}
		ana.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, ana);
	}
	@SuppressWarnings("unchecked")
	public List<Notizia> trova(NotiziaCriteria ana, int start, int size) {
		ExtendedCriteria criteria = trovaCriteria(ana);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int trovaSize(NotiziaCriteria ana) {
		ExtendedCriteria criteria = trovaCriteria(ana);
		return getListSize(criteria);
	}
	
	public List<Notizia> impostaConteggioNotizia(List<Notizia> notizie) {
		if(ApplicationUtil.notEmpty(notizie))  {
			List<Long> ll = ApplicationUtil.prendiIdNotizia(notizie);
			Map<Long, ColonnaModel> map = dataDao.getConteggioColonna(null, ll, Notizia.class, new ColonnaQuery("allegati", "allegati"));
			Iterator<Notizia> it = notizie.iterator();
			while (it.hasNext()) {
				Notizia o = it.next();
				ColonnaModel lmap = map.get(o.getId());
				if(lmap != null) {
					o.setModel(lmap);
				}
			}
		}
		return notizie;
	}
	@SuppressWarnings("unchecked")
	public List<Notizia> trovaNotizieFeed(Long utente, String password) {
		if(utente != null) {
			DetachedCriteria criteria = DetachedCriteria.forClass(Notizia.class);
			criteria.add(Restrictions.eq("attivo", Boolean.TRUE));
			DetachedCriteria criteriaUA = criteria.createCriteria("utenti");
			criteriaUA.add(Restrictions.idEq(utente));
			criteriaUA.add(Restrictions.eq("password", password));
			ApplicationUtil.criteriaOrario(criteria, false);
			return getList(criteria);
		}
		return null;
	}
	@SuppressWarnings("unchecked")
	public List<Notizia> trovaNotizieAttive(Utente utente) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Notizia.class);
		criteria.add(Restrictions.eq("attivo", Boolean.TRUE));
		criteria.createCriteria("utenti").add(Restrictions.idEq(utente.getId()));
		ApplicationUtil.criteriaOrario(criteria, false);
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Utente> trovaUtentiNotiziaTutte(Long notizia) {
		DetachedCriteria criteriaC = DetachedCriteria.forClass(Utente.class);
		criteriaC.createCriteria("notizie").add(Restrictions.idEq(notizia));
		ApplicationUtil.orderU(criteriaC);
		return getList(criteriaC);
	}
	@SuppressWarnings("unchecked")
	public List<Utente> trovaSenzaNotizia(Long agenteId, UserCriteria criteUser, int start, int size) {
		ExtendedCriteria criteria = trovaSenzaNotiziaCriteria(agenteId, criteUser);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), criteUser.getO_tipo(), criteUser.getO_colonna(),  new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int trovaSenzaNotiziaSize(Long agenteId, UserCriteria user) {
		ExtendedCriteria criteria = trovaSenzaNotiziaCriteria(agenteId, user);
		return getListSize(criteria);
	}
	private ExtendedCriteria trovaSenzaNotiziaCriteria(Long notiziaId, UserCriteria user) {
		if(user != null)  user = (UserCriteria) user.loadCriteria();
		DetachedCriteria criteriaA = DetachedCriteria.forClass(Utente.class);
		criteriaA.createCriteria("notizie").add(Restrictions.idEq(notiziaId));
		criteriaA.setProjection(Projections.distinct(Projections.property("id")));
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		impostazioneDao.subPropertyNotInS(criteria, criteriaA, "id");
		ApplicationUtil.utente(criteria, user);
		user.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, user);
	}
	public void aggiungiNotiziaAUtente(Long utenteId, Long notiziaId) {
		Utente utente = getFullUtente(utenteId);
		Notizia notizia = getNotizia(notiziaId);
		if (utente != null && notizia != null) {
			if (!utente.getNotizie().contains(notizia)) {
				utente.getNotizie().add(notizia);
				salvaAggiorna(utente);
			}
		}
	}
	public void aggiungiNotiziaAUtenteTutti(List<Utente> utenti, Long notiziaId) {
		Notizia notizia = getFullNotizia(notiziaId);
		if (notizia != null && ApplicationUtil.notEmpty(utenti)) {
			notizia.getUtenti().addAll(utenti);
			salvaAggiorna(notizia);
		}
	}
	public void rimuoviNotiziaAUtenteTutti(List<Utente> utenti, Long notiziaId) {
		Notizia notizia = getFullNotizia(notiziaId);
		if (notizia != null && ApplicationUtil.notEmpty(utenti)) {
			notizia.getUtenti().removeAll(utenti);
			salvaAggiorna(notizia);
		}
	}
	public Notizia getNotizia(Long notiziaId) {
		return (Notizia) get(Notizia.class, notiziaId);
	}
	public void rimuoviNotiziaDaUtente(Long utenteId, Long notiziaId) {
		Utente utente = getFullUtente(utenteId);
		Notizia notizia = getNotizia(notiziaId);
		if (utente != null && notizia != null) {
			if (utente.getNotizie().contains(notizia)) {
				utente.getNotizie().remove(notizia);
				salvaAggiorna(utente);
			}
		}
	}
	public Utente getFullUtente(Long utenteId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		criteria.add(Restrictions.idEq(utenteId));
		criteria.setFetchMode("notizie", FetchMode.JOIN);
		return  (Utente) getUniqueResult(criteria);
	}
	public Notizia getFullNotizia(Long notiziaId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Notizia.class);
		criteria.add(Restrictions.idEq(notiziaId));
		criteria.setFetchMode("utenti", FetchMode.JOIN);
		return  (Notizia) getUniqueResult(criteria);
	}
	public void salvaNotiziaNuova(Notizia notizia) {
		Session session = openSession();
		Transaction tx = session.beginTransaction();
		session.save(notizia);
		List<AllegatoNotizia> allegati = notizia.getAllegati(); 
		if(ApplicationUtil.notEmpty(allegati)){
			int i = 0;
			Iterator<AllegatoNotizia> objIter = allegati.iterator();
			while(objIter.hasNext()) {
				AllegatoNotizia allegato = objIter.next();
				allegato.setNotizia(notizia);
				session.save(allegato);
				if (i++ % ConstantsGlobal.flushSize == 0) {
					session.flush();
					session.clear();
				}
			}
		}
		tx.commit();
		closeSession(session);

	}
}