package it.callite.dao;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.faces.model.SelectItem;

import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Projections;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContext;
import org.springframework.stereotype.Repository;
import org.springframework.webflow.core.collection.SharedAttributeMap;
import org.springframework.webflow.mvc.servlet.MvcExternalContext;

import it.callite.chiamate.Chiamate;
import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.constants.ConstantsMap;
import it.callite.constants.config.DefaultsPannello;
import it.callite.criteria.OnlineCriteria;
import it.callite.criteria.OperazioneCriteria;
import it.callite.criteria.PausaCriteria;
import it.callite.criteria.base.ExtendedCriteria;
import it.callite.criteria.report.MonitorCriteria;
import it.callite.dao.generic.GenericDao;
import it.callite.message.Message;
import it.callite.model.Campagna;
import it.callite.model.Contatto;
import it.callite.model.EsitoArchiviazione;
import it.callite.model.Operazione;
import it.callite.model.Pausa;
import it.callite.model.Rules;
import it.callite.model.Utente;
import it.callite.model.config.ConfigPausa;
import it.callite.model.report.MonitorOperatore;
import it.callite.model.sip.SipBuddies;
import it.callite.service.LoginAttemptService;
import it.callite.service.config.ConfigurazionePannelloService;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.Firewall;
import it.callite.utility.LoggerLogin;
import it.callite.utility.StatoLista;
import it.callite.utility.StatoOperatore;
import it.callite.utility.importa.ImportModelAnagrafica;
import it.callite.utility.item.GenericSI;
import it.callite.utility.thread.InviaLoginThreadA;
import it.callite.utility.thread.InviaLoginThreadU;
import it.callite.utility.thread.SalvaAccessoThread;
import it.callite.utility.thread.SalvaMarcaturaThread;
import it.callite.utility.thread.SalvaOperazioneThread;
@SuppressWarnings("serial")
@Repository("operazioneDao")
public class OperazioneDao extends GenericDao implements Serializable {
	@Autowired
	private UserDao userDao;
	@Autowired
	private ImpostazioneDao impostazioneDao;
	@Autowired
	private ConfigurazionePannelloService configurazionePannelloService;
	@Autowired
	private LoginAttemptService loginAttemptService;
	public Map<Utente, MonitorOperatore> getOperatori() {
		return Chiamate.getCodaOperatori().getData();
	}

	public List<StatoLista> getMonitorLista(List<Long> utentiId, MonitorCriteria m) {
		StatoOperatore monop = new StatoOperatore();
		Map<Utente, MonitorOperatore> operatori = Chiamate.getCodaOperatori().getData();
		if(ApplicationUtil.notEmpty(operatori)) {
			Iterator<Utente> objRIter = operatori.keySet().iterator();
			while(objRIter.hasNext()) {
				Utente user = objRIter.next();
				MonitorOperatore mo = operatori.get(user);
				if(mo != null && ApplicationUtil.notEmpty(utentiId) && utentiId.contains(user.getId())) {
					boolean b = ApplicationUtil.isLocale(mo, m.getTipo());
					boolean c = ApplicationUtil.isBrowser(mo, m.getOnline());
					boolean d = ApplicationUtil.isRegistrato(mo, m.getLoggato());
					if(b && c && d && ApplicationUtil.isTrue(mo.getVisibile())) {
						monop.add(mo);
					}
				}
			}
		}
		return monop.getList();
	}
	public List<MonitorOperatore> getMonitorOperatoriList(List<Long> utentiId, MonitorCriteria m) {
		List<MonitorOperatore> monop = new ArrayList<MonitorOperatore>();
		Map<Utente, MonitorOperatore> operatori = Chiamate.getCodaOperatori().getData();
		if(ApplicationUtil.notEmpty(operatori)) {
			Iterator<Utente> objRIter = operatori.keySet().iterator();
			while(objRIter.hasNext()) {
				Utente user = objRIter.next();
				MonitorOperatore mo = operatori.get(user);
				if(mo != null && ApplicationUtil.notEmpty(utentiId) && utentiId.contains(user.getId())) {
					boolean a = (m.getStato()==null || mo.getStato().equals(m.getStato()));
					boolean b = ApplicationUtil.isLocale(mo, m.getTipo());
					boolean c = ApplicationUtil.isBrowser(mo, m.getOnline());
					boolean d = ApplicationUtil.isRegistrato(mo, m.getLoggato());
					if(a && b && c && d && ApplicationUtil.isTrue(mo.getVisibile())) {
						monop.add(mo);
					}
				}
			}
		}
		return monop;
	}
	public void rimuoviMonitorOperatoriList(List<Long> utentiId, MonitorCriteria m) {
		Map<Utente, MonitorOperatore> operatori = Chiamate.getCodaOperatori().getData();
		if(ApplicationUtil.notEmpty(operatori)) {
			Iterator<Utente> objRIter = operatori.keySet().iterator();
			while(objRIter.hasNext()) {
				Utente user = objRIter.next();
				MonitorOperatore mo = operatori.get(user);
				if(mo != null && ApplicationUtil.notEmpty(utentiId) && utentiId.contains(user.getId())) {
					boolean a = (m.getStato()==null || mo.getStato().equals(m.getStato()));
					boolean b = ApplicationUtil.isLocale(mo, m.getTipo());
					boolean c = ApplicationUtil.isBrowser(mo, m.getOnline());
					boolean d = ApplicationUtil.isRegistrato(mo, m.getLoggato());
					if(a && b && c && d && ApplicationUtil.isTrue(mo.getVisibile())) {
						mo.setVisibile(false);
					}
				}
			}
		}
	}
	public ExtendedCriteria trovaCriteriaOperazione(OperazioneCriteria crit) {
		if(crit != null)  crit = (OperazioneCriteria) crit.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Operazione.class);
		DetachedCriteria criteriau =  criteria.createCriteria("user");
		
		if (ApplicationUtil.isNotBlank(crit.getDescrizione())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator, true, crit.getDescrizione(), criteria, "descrizione");
		}
		
		if (ApplicationUtil.isNotBlank(crit.getOperatore())) {
			ApplicationUtil.operatore(criteriau, crit.getOperatore());
		}
		if (ApplicationUtil.gtZero(crit.getStato())) {
			criteria.add(Restrictions.eq("operazione", crit.getStato()));
		}
		crit.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, crit);
	}
	public ExtendedCriteria trovaCriteriaOperazione(Utente utente, OperazioneCriteria crit) {
		if(crit != null)  crit = (OperazioneCriteria) crit.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Operazione.class);
		DetachedCriteria criteriau =  criteria.createCriteria("user");
		impostazioneDao.subCriteriaSupervisore(utente.getId(), criteriau, "id");
		
		if (ApplicationUtil.isNotBlank(crit.getDescrizione())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator, true, crit.getDescrizione(), criteria, "descrizione");
		}
		
		if (ApplicationUtil.isNotBlank(crit.getOperatore())) {
			ApplicationUtil.operatore(criteriau, crit.getOperatore());
		}
		if (ApplicationUtil.gtZero(crit.getStato())) {
			criteria.add(Restrictions.eq("operazione", crit.getStato()));
		}
		crit.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, crit);
	}
	@SuppressWarnings("unchecked")
	public List<Operazione> trovaOperazione(OperazioneCriteria crit, int start, int size) {
		ExtendedCriteria criteria = trovaCriteriaOperazione(crit);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int trovaOperazioneSize(OperazioneCriteria crit) {
		ExtendedCriteria criteria = trovaCriteriaOperazione(crit);
		return getListSize(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Operazione> trovaOperazione(Utente utente, OperazioneCriteria crit, int start, int size) {
		ExtendedCriteria criteria = trovaCriteriaOperazione(utente, crit);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int trovaOperazioneSize(Utente utente, OperazioneCriteria crit) {
		ExtendedCriteria criteria = trovaCriteriaOperazione(utente, crit);
		return getListSize(criteria);
	}
	public void apriContatto(Integer flag, Utente user, Contatto contatto) {
		if(contatto!=null) {
			if(contatto.getTimestamp_apertura() == null) {
				contatto.setTimestamp_apertura(ApplicationUtil.date());
			}
			settaMonitor(null, user, contatto,   null,   ConstantsMap.STATO_SCHEDA_CONTATTO, flag);
		}
		if (ApplicationUtil.eq(flag, ConstantsGlobal.ID_SEZIONE_PRIVATE)) {
			String p = ApplicationUtil.asString(ConstantsLanguage.CONTATTO.stampa(), contatto);
			settaOperazione(user,  p,   ConstantsMap.OPERAZIONE_APRI_PRIVATA, ConstantsMap.LOG7);
		}
		if (ApplicationUtil.eq(flag, ConstantsGlobal.ID_SEZIONE_ARCHIVIATE)) {
			String p = ApplicationUtil.asString(ConstantsLanguage.CONTATTO.stampa(), contatto);
			settaOperazione(user,  p,   ConstantsMap.OPERAZIONE_APRI_ARCHIVIATA, ConstantsMap.LOG7);
		}
		if (ApplicationUtil.eq(flag, ConstantsGlobal.ID_SEZIONE_PUBBLICHE)) {
			String p = ApplicationUtil.asString(ConstantsLanguage.CONTATTO.stampa(), contatto);
			settaOperazione(user, p,   ConstantsMap.OPERAZIONE_APRI_PUBBLICHE, ConstantsMap.LOG7);
		}
		settaPannello(user, null, null);
	}
	public void apriContatto(Utente user, Contatto contatto) {
		if(contatto!=null) {
			if(contatto.getTimestamp_apertura() == null) {
				contatto.setTimestamp_apertura(ApplicationUtil.date());
			}
			settaMonitor(null, user, contatto,   null,   ConstantsMap.STATO_SCHEDA_CONTATTO, ConstantsGlobal.ID_SEZIONE_PANNELLO);
		}
		String p = ApplicationUtil.asString(ConstantsLanguage.CONTATTO.stampa(), contatto);
		settaOperazione(user, p,   ConstantsMap.OPERAZIONE_APRI_PANNELLO, ConstantsMap.LOG7);
		settaPannello(user, null, null);
	}
	public void loginSuccess(Utente user, String ip) {
		apriFW(ip, user);
		notifica(user, ip);
		Chiamate.getCodaLogin().loginSuccess(user.getUsername(), ip);
		loginAttemptService.loginSuccess(ip, user.getUsername());
		if(ApplicationUtil.eq(impostazioneDao.getLoginMultiplo(), 4)) {
			user.setSessionId(ApplicationUtil.getSessionId());
		}
		marcatura(user, true);
		settaLogin(user, ConstantsMap.STATO_LOGIN, ip);
		String p = ApplicationUtil.asString(ConstantsLanguage.IP.stampa(), ip);
		settaOperazione(user, p, ConstantsMap.OPERAZIONE_LOGIN, ConstantsMap.LOG0);
		Integer logType = impostazioneDao.getAccessLogType();
		ApplicationUtil.access(logType, "login OK" + " - " + ConstantsLanguage.USERNAME.stampa() + ": " + user + " - " + ConstantsLanguage.IP.stampa() + ": " + ip);
	}

	public void logoutSuccess(Utente user, String ip, String tipo) {
		removeUser(user);
		marcatura(user, false);
		settaLogout(user, ConstantsMap.STATO_LOGOUT, ip);
		String p = ApplicationUtil.asString(ConstantsLanguage.IP.stampa(), ip);
		settaOperazione(user, p, ConstantsMap.OPERAZIONE_LOGOUT, ConstantsMap.LOG0);
		Integer logType = impostazioneDao.getAccessLogType();
		ApplicationUtil.access(logType, "logout " + tipo + " - " + ConstantsLanguage.USERNAME.stampa() + ": " + user + " - " + ConstantsLanguage.IP.stampa() + ": " + ip);
	}

	public void loginFailure(String lm, String ip) {
		Integer logType = impostazioneDao.getAccessLogType();
		ApplicationUtil.access(logType, "login" + " " + ConstantsGlobal.ERROR + " - " + lm + " - " + ConstantsLanguage.IP.stampa() + ": " + ip);
	}
	public void aggiungiPorta(String ip, String service) {
		List<Firewall> fws = impostazioneDao.getFirewall(service);
		if(ApplicationUtil.notEmpty(fws)) {
			for(Firewall fw : fws) {
				fw.setIp(ip);
				fw.setUtente(true);
				aggiungiPorta(fw);
			}
		}
	}
	public void aggiungiPorta(SipBuddies r) {
		if(ApplicationUtil.isNotBlank(r.getHost())) {
			aggiungiPorta(new Firewall(r.getHost(), r.getPort().toString(), null, ConstantsGlobal.udp, false));
			aggiungiPorta(new Firewall(r.getHost(), r.getRtp1() +ConstantsGlobal.portSeparator1+ r.getRtp2(), null, ConstantsGlobal.udp, false));
		}
	}
	
	public void aggiungiPorta(Rules r) {
		if(ApplicationUtil.isNotBlank(r.getService())) {
			List<Firewall> fws = impostazioneDao.getFirewall(r.getService());
			if(ApplicationUtil.notEmpty(fws)) {
				for(Firewall fw : fws) {
					fw.setIp(r.getIp());
					fw.setNegata(r.getNegata());
					aggiungiPorta(fw);
				}
			}
		} else {
			aggiungiPorta(new Firewall(r.getIp(), r.getPorta(), r.getService(), r.getProtocol(), r.getNegata()));
		}
	}
	public void aggiungiPorta(Firewall fw) {
		if(!Chiamate.getCodaIP().contains(fw) && ApplicationUtil.getPath(ConstantsGlobal.path_firewall, false)) {
			boolean ok = false;
			if((ApplicationUtil.isNotBlank(fw.getService()) && ApplicationUtil.getService(fw.getService())) || ApplicationUtil.isBlank(fw.getService())) {
				String protcols_S = ApplicationUtil.isNotBlank(fw.getProtocol())  ? (" "+ "-p" + " " + fw.getProtocol()) : "";
				String ip_S = ApplicationUtil.isNotBlank(fw.getIp())  ? (" "+ "-s" + " " + fw.getIp()) : "";
				String porta_S = fw.getPorta() != null ? (" " + "--dport" + " " +  fw.getPorta()) : "";
				if(ApplicationUtil.isNotBlank(protcols_S) || ApplicationUtil.isNotBlank(ip_S) || ApplicationUtil.isNotBlank(porta_S)) {
					String cmd = "iptables -A INPUT" + protcols_S + ip_S + porta_S + " " + "-j" + " " + (!fw.getNegata() ? "ACCEPT" : "DROP");
					ok = ApplicationUtil.executeRuntime(cmd, true);
				}
			}
			if(ok) {
				Chiamate.getCodaIP().put(fw);
			}
		}
	}
	public void eliminaPorta(Firewall fw) {
		if(Chiamate.getCodaIP().contains(fw) && ApplicationUtil.getPath(ConstantsGlobal.path_firewall, false)) {
			boolean ok = false;
			if((ApplicationUtil.isNotBlank(fw.getService()) && ApplicationUtil.getService(fw.getService())) || ApplicationUtil.isBlank(fw.getService())) {
				String protcols_S = ApplicationUtil.isNotBlank(fw.getProtocol())  ? (" "+ "-p" + " " + fw.getProtocol()) : "";
				String ip_S = ApplicationUtil.isNotBlank(fw.getIp())  ? (" "+ "-s" + " " + fw.getIp()) : "";
				String porta_S = fw.getPorta() != null ? (" " + "--dport" + " " +  fw.getPorta()) : "";
				if(ApplicationUtil.isNotBlank(protcols_S) || ApplicationUtil.isNotBlank(ip_S) || ApplicationUtil.isNotBlank(porta_S)) {
					String cmd = "iptables -D INPUT" + protcols_S + ip_S + porta_S + " " + "-j" + " " + (!fw.getNegata() ? "ACCEPT" : "DROP");
					ok = ApplicationUtil.executeRuntime(cmd, true);
				}
			}
			if(ok) {
				Chiamate.getCodaIP().remove(fw);
			}
		}
	}
	public void apriFW(String ip, Utente user) {
		if(user != null) {
			if(user.isShell()) {
				aggiungiPorta(ip, ConstantsGlobal.service_ssh);
				aggiungiPorta(ip, ConstantsGlobal.service_shell);
				aggiungiPorta(ip, ConstantsGlobal.service_db);
				aggiungiPorta(ip, ConstantsGlobal.service_sql);
				aggiungiPorta(ip, ConstantsGlobal.service_vpn);
				aggiungiPorta(ip, ConstantsGlobal.service_ftp);
				//aggiungiPorta(ip, ConstantsGlobal.service_office);
			}
			if(user.getBarra()) {
				aggiungiPorta(ip, ConstantsGlobal.service_asterisk);
			}
			if(ApplicationUtil.gtZero(impostazioneDao.chatAbilitata())) {
				aggiungiPorta(ip, ConstantsGlobal.service_prosody);
				aggiungiPorta(ip, ConstantsGlobal.service_ejabberd);
				aggiungiPorta(ip, ConstantsGlobal.service_openfire);
			}
			new SalvaAccessoThread(userDao, user, ip, user.getBrowser()).start();
		}
	}
	public List<Firewall> getRules() {
		return Chiamate.getCodaIP().list();
	}
	public void clearRules() {
		Chiamate.getCodaIP().clear();
	}
	
	public List<Firewall> utenteRules() {
		return Chiamate.getCodaIP().utente();
	}
	public void eliminaFirewall(Firewall fw) {
		eliminaPorta(fw);
	}
	public void eliminaFirewall() {
		List<Firewall> fwL = new ArrayList<Firewall>(getRules());
		Iterator<Firewall> itF = fwL.iterator();
		while (itF.hasNext()) {
			Firewall fw = (Firewall) itF.next();
			eliminaPorta(fw);
		}
		//clearRules();
	}
	public void marcatura(Utente user, boolean start) {
		if(user!= null && ApplicationUtil.ne(user.getDisattivaFunzioni(), 2)) {
			new SalvaMarcaturaThread(userDao, user, start, impostazioneDao.getMarcaturaMultipla()).start();
		}
	}
	public void notifica(Utente user, String ip) {
		Integer s = impostazioneDao.getNotificaLogin();
		boolean notifica = user != null && ApplicationUtil.isTrue(user.getNotificaLogin());
		if(ApplicationUtil.gtZero(s)) {
			if(ApplicationUtil.eq(s, 2) || (ApplicationUtil.eq(s, 1) && notifica)) {
				new InviaLoginThreadA(userDao, user, ip).start();
			}
		}
		if(notifica) {
			new InviaLoginThreadU(userDao, user, ip).start();
		}
	}
	public void pausa(Utente user, ConfigPausa pausa) {
		settaMonitor(pausa.getChiave(), user, null,  null, ConstantsMap.STATO_PAUSA, null);
		settaOperazione(user, pausa.getChiave(), ConstantsMap.OPERAZIONE_PAUSA, ConstantsMap.LOG1);
	}
	public void apriMessaggistica(Utente user) {
		settaMonitor(null, user, null,   null,   ConstantsMap.STATO_LST_MESSAGGISTICA, null);
		settaOperazione(user,  null,  ConstantsMap.OPERAZIONE_LST_MESSAGGISTICA, ConstantsMap.LOG2);
	}
	public void listaAppuntamenti(Utente user) {
		settaMonitor(null, user, null, null,   ConstantsMap.STATO_LST_APPUNTAMENTI, null);
		settaOperazione(user, null, ConstantsMap.OPERAZIONE_LST_APPUNTAMENTI, ConstantsMap.LOG2);
	}
	public void listaReportistca(Utente user) {
		settaMonitor(null, user, null, null,   ConstantsMap.STATO_LST_REPORTISTICA, null);
		settaOperazione(user, null, ConstantsMap.OPERAZIONE_LST_REPORTISTICA, ConstantsMap.LOG2);
	}
	public void listaContratti(Utente user) { 
		settaMonitor(null, user, null, null,   ConstantsMap.STATO_LST_CONTRATTI, null);
		settaOperazione(user, null, ConstantsMap.OPERAZIONE_LST_CONTRATTI, ConstantsMap.LOG2);
	}
	public void listaContatti(Utente user) {
		settaMonitor(null, user, null, null,   ConstantsMap.STATO_LST_CONTATTI, null);
		settaOperazione(user, null,  ConstantsMap.OPERAZIONE_LST_CONTATTI, ConstantsMap.LOG2);
	}
	public void listaPrenotazioni(Utente user) {
		settaMonitor(null, user, null, null,   ConstantsMap.STATO_LST_PRENOTAZIONI, null);
		settaOperazione(user, null,  ConstantsMap.OPERAZIONE_LST_PRENOTAZIONI, ConstantsMap.LOG2);
	}
	public void listaNote(Utente user) {
		settaMonitor(null, user, null, null,   ConstantsMap.STATO_LST_NOTE, null);
		settaOperazione(user, null,  ConstantsMap.OPERAZIONE_LST_NOTE, ConstantsMap.LOG2);
	}
	public void listaStorici(Utente user) {
		settaMonitor(null, user, null, null,   ConstantsMap.STATO_LST_STORICI, null);
		settaOperazione(user, null,  ConstantsMap.OPERAZIONE_LST_STORICI, ConstantsMap.LOG2);
	}
	public void listaEstratti(Utente user) {
		settaMonitor(null, user, null, null,   ConstantsMap.STATO_LST_ESTRATTI, null);
		settaOperazione(user,  null, ConstantsMap.OPERAZIONE_LST_ESTRATTI, ConstantsMap.LOG2);
	}
	public void calendarioAppuntamenti(Utente user) {
		settaMonitor(null, user, null, null,   ConstantsMap.STATO_LST_CALENDARIO, null);
		settaOperazione(user, null, ConstantsMap.OPERAZIONE_LST_CALENDARIO, ConstantsMap.LOG2);
	}
	public void listaCampagne(Utente user) {
		settaMonitor(null, user, null, null,   ConstantsMap.STATO_LST_CAMPAGNE, null);
		settaOperazione(user,  null, ConstantsMap.OPERAZIONE_LST_CAMPAGNE, ConstantsMap.LOG2);
	}
	public void listaCampagna(Utente user) {
		settaMonitor(null, user, null, null,   ConstantsMap.STATO_LST_CAMPAGNA, null);
		settaOperazione(user, null,   ConstantsMap.OPERAZIONE_LST_CAMPAGNA, ConstantsMap.LOG2);
	}
	public void listaRiprogramma(Utente user) {
		settaMonitor(null, user, null, null,   ConstantsMap.STATO_LST_RIPROGRAMMA, null);
		settaOperazione(user, null,  ConstantsMap.OPERAZIONE_LST_RIPROGRAMMA, ConstantsMap.LOG2);
	}
	public void excelImportContatto(Utente user) {
		settaMonitor(null, user, null, null,   ConstantsMap.STATO_IMPORT_CONTATTO, null); 
		settaOperazione(user,  null, ConstantsMap.OPERAZIONE_IMPORT_CONTATTO, ConstantsMap.LOG2);
	}
	public void excelImportContratto(Utente user) {
		settaMonitor(null, user, null, null,   ConstantsMap.STATO_IMPORT_CONTRATTO, null);
		settaOperazione(user, null,  ConstantsMap.OPERAZIONE_IMPORT_CONTRATTO, ConstantsMap.LOG2);
	}
	public void excelImportAppuntamento(Utente user) {
		settaMonitor(null, user, null, null,   ConstantsMap.STATO_IMPORT_APPUNTAMENTO, null);
		settaOperazione(user, null, ConstantsMap.OPERAZIONE_IMPORT_APPUNTAMENTO, ConstantsMap.LOG2);
	}
	public void listaPrivate(Utente user) {
		settaMonitor(null, user, null,  null,  ConstantsMap.STATO_LST_PRIVATE, null);
		settaOperazione(user, null, ConstantsMap.OPERAZIONE_LST_PRIVATE, ConstantsMap.LOG2);
	}
	public void listaArchiviate(Utente user) {
		settaMonitor(null, user, null, null, ConstantsMap.STATO_LST_ARCHIVIATE, null);
		settaOperazione(user, null, ConstantsMap.OPERAZIONE_LST_ARCHIVIATE, ConstantsMap.LOG2);
	}
	public void home(Utente user) {
		settaMonitor(null, user, null, null, ConstantsMap.STATO_HOME, null);
		settaOperazione(user, null,  ConstantsMap.OPERAZIONE_HOME, ConstantsMap.LOG2);
		settaPannello(user, null, null);
	}
	public void esita(Utente user, Contatto contatto) {
		if(contatto != null) {
			if(contatto.getTimestamp_apertura() != null) {
				contatto.setTimestamp_esito(ApplicationUtil.date());
			}
			settaMonitor(null, user, contatto,  null, ConstantsMap.STATO_ESITA_CONTATTO, null);
			String p = ApplicationUtil.asString(ConstantsLanguage.CONTATTO.stampa(), contatto);
			settaOperazione(user, p, ConstantsMap.OPERAZIONE_ESITA_NORMALE, ConstantsMap.LOG7);
			settaPannello(user, null, null);
		}
	}
	public void attesa(Utente user, Contatto contatto, Campagna campagna) {
		if(contatto == null) {
			settaMonitor(null, user, null,  campagna, ConstantsMap.STATO_ATTESA_CONTATTO, null);
			String p = ApplicationUtil.asString(ConstantsLanguage.CAMPAGNA.stampa(), campagna);
			settaOperazione(user, p, ConstantsMap.OPERAZIONE_ATTESA_CONTATTO, ConstantsMap.LOG7);
		}
	}
	public void fine(Utente user, Contatto contatto) {
		if(contatto != null) {
			if(contatto.getTimestamp_chiamata() != null) {
				contatto.setTimestamp_fine(ApplicationUtil.date());
			}
			settaMonitor(null, user, contatto,  null, ConstantsMap.STATO_POST_CHIAMATA, null);
			String p = ApplicationUtil.asString(ConstantsLanguage.CONTATTO.stampa(), contatto);
			settaOperazione(user, p, ConstantsMap.OPERAZIONE_FINE_CHIAMATA, ConstantsMap.LOG7);
		}
	}
	public void risponde(Utente user, Contatto contatto) {
		if(contatto != null) {
			if(contatto.getTimestamp_risposta() == null) {
				contatto.setTimestamp_risposta(ApplicationUtil.date());
			}
			settaMonitor(null, user, contatto, null, ConstantsMap.STATO_CONVERSAZIONE, null);
			String p = ApplicationUtil.asString(ConstantsLanguage.CONTATTO.stampa(), contatto);
			settaOperazione(user, p,  ConstantsMap.OPERAZIONE_CONVERSAZIONE, ConstantsMap.LOG7);
		}
	}
	public void chiama(Utente user, Contatto contatto) {
		if(contatto != null) {
			if(contatto.getTimestamp_chiamata() == null) {
				contatto.setTimestamp_chiamata(ApplicationUtil.date());
			}
			settaMonitor(null, user, contatto,  null, ConstantsMap.STATO_CHIAMATA, null);
			String p = ApplicationUtil.asString(ConstantsLanguage.CONTATTO.stampa(), contatto);
			settaOperazione(user, p,   ConstantsMap.OPERAZIONE_CHIAMA, ConstantsMap.LOG7);
		}
	}
	public void registraStart(Utente user, Contatto contatto) {
		if(contatto != null) {
			if(contatto.getTimestamp_rec_inizio() == null) {
				contatto.setTimestamp_rec_inizio(ApplicationUtil.date());
			}
			settaRegistrazione(user, true);
		}
	}
	public void registraStop(Utente user, Contatto contatto) {
		if(contatto != null) {
			if(contatto.getTimestamp_rec_inizio() != null) {
				contatto.setTimestamp_rec_fine(ApplicationUtil.date());
			}
			settaRegistrazione(user, false);
		}
	}
	public void fineChiamata(Utente user, Contatto contatto) {
		if(contatto != null) {
			if(ApplicationUtil.controllaTempo(contatto.getTimestamp_rec_inizio(), contatto.getTimestamp_rec_fine())) {
				registraStop(user, contatto);
			}
			if(ApplicationUtil.controllaTempo(contatto.getTimestamp_chiamata(), contatto.getTimestamp_fine())) {
				fine(user, contatto);
			}
		}
	}
	public void modificaContatto(Utente user, Object obj) {
		String p = ApplicationUtil.asString(ConstantsLanguage.CONTATTO.stampa(), obj);
		settaOperazione(user, p, ConstantsMap.OPERAZIONE_MODIFICA_CONTATTO, ConstantsMap.LOG3);
	}
	public void salvaContatti(Utente user, List<?> lista) {
		String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.CONTATTO.stampa() +": " + ApplicationUtil.size(lista), lista);
		settaOperazione(user, p, ConstantsMap.OPERAZIONE_SALVA_CONTATTO, ConstantsMap.LOG3);
	}
	public void salvaAnagrafiche(Utente user, ImportModelAnagrafica im) {
		String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.CONTATTO.stampa() +": " + ApplicationUtil.size(im != null ? im.getAnagrafiche() : null), im != null ? im.getAnagrafiche() : null);
		settaOperazione(user, p, ConstantsMap.OPERAZIONE_SALVA_CONTATTO, ConstantsMap.LOG3);
	}
	public void creaOperazione(String soglia, String descrizione, Utente user, Short stato) {
		settaOperazione(user, descrizione, stato, soglia);
	}
	public Operazione salva(Operazione operazione) {
		return (Operazione) salvaAggiorna(operazione);
	}
	public List<String> getRegistraLog() {
		String a = configurazionePannelloService.getValore(DefaultsPannello.KEY_PANNELLO_REGISTRA_LOG);
		return ApplicationUtil.token(a, ConstantsGlobal.colonSeparator);
	}
	private void settaOperazione(Utente user, String descrizione, Short oper, String soglia) {
		
		if(user != null) {
			if(ApplicationUtil.ne(user.getDisattivaFunzioni(), 2)) {
				List<String> regLog = getRegistraLog();
				if(ApplicationUtil.notEmpty(regLog) && regLog.contains(soglia)) {
					Operazione operazione = new Operazione(user);
					operazione.setOperazione(oper);
					operazione.setDescrizione(descrizione);
					new SalvaOperazioneThread(this, operazione).start();
				}
			}
		}
	}
	private void settaOperazionePannello(MonitorOperatore mop, Utente user) {
		if(mop != null && mop.isPannelloAperto() && user!= null && ApplicationUtil.ne(user.getDisattivaFunzioni(), 2) && mop.getStatoTesto() != null) {
			List<String> regLog = getRegistraLog();
			if(ApplicationUtil.notEmpty(regLog) && regLog.contains(ConstantsMap.LOG9)) {
				Operazione operazione = new Operazione(user);
				operazione.setOperazione(ConstantsMap.OPERAZIONE_APRI_POPUP);
				operazione.setDescrizione(Message.addCatalogMessage(mop.getStatoTesto() + ":" + " " + mop.getPannelloTesto()));
				new SalvaOperazioneThread(this, operazione).start();
			}
		}
	}

	public void forzaReload(Utente user) {
		if(user!= null) {
			MonitorOperatore mop = Chiamate.getCodaOperatori().get(user);
			if(mop != null) { 
				mop.setReload(true);
			}
		}
	}
	public void bloccaReload(Utente user) {
		if(user!= null) {
			MonitorOperatore mop = Chiamate.getCodaOperatori().get(user);
			if(mop != null) { 
				mop.setReload(false);
			}
		}
	}
	public void logga(Utente utente, String menu, String url) {
		String op = null;
		if(url == null) {
			op = menu;
		} else {
			op =  menu + " " + "(" + url + ")";
		}
		settaOperazione(utente, op, ConstantsMap.OPERAZIONE_NAVIGAZIONE, ConstantsMap.LOG8);
	}
	private void settaMonitor(String tipoPausa, Utente user,  Contatto contatto, Campagna campagna, Short stato, Integer flag) {
		if(user!= null) {
			MonitorOperatore mop = Chiamate.getCodaOperatori().get(user);
			if(mop == null) { 
				mop = new MonitorOperatore();
				Chiamate.getCodaOperatori().put(user, mop);
			} 
			mop.svuota();
			Date data = ApplicationUtil.date();
			mop.setOnline(true);
			mop.setAttivo(true);
			if(ApplicationUtil.gtZero(user.getLoginMultiplo())) {
				mop.setEndorse(data.getTime());
			} else {
				mop.setEndorse(null);
			}
			mop.setOperatore(user);
			mop.setContatto(contatto);
			if(contatto != null) {
				mop.setCampagna(contatto.getCampagna());
			} else {
				mop.setCampagna(campagna);
			}
			if(stato != null) {
				mop.setStato(stato);
			}
			if(flag != null) {
				mop.setFlag(flag);
			}
			mop.setTipoPausa(tipoPausa);
			mop.setData_azione(data);
			if(ApplicationUtil.ne(user.getDisattivaFunzioni(), 2)) {
				mop.setVisibile(true);
			} else {
				mop.setVisibile(false);
			}
		}
	}

	private void settaLogin(Utente user, Short stato, String ip) {
		if(user!= null) {
			MonitorOperatore mop = Chiamate.getCodaOperatori().get(user);
			if(mop == null) { 
				mop = new MonitorOperatore();
				Chiamate.getCodaOperatori().put(user, mop);
			} 
			mop.svuota();
			mop.addSecurityContext(ApplicationUtil.getSecurityContext());
			Date data = ApplicationUtil.date();
			mop.setLibero(0);
			mop.setOnline(true);
			mop.setAttivo(true);
			if(ApplicationUtil.gtZero(user.getLoginMultiplo())) {
				mop.setEndorse(data.getTime());
			} else {
				mop.setEndorse(null);
			}

			mop.setOperatore(user);
			mop.setContatto(null);
			mop.setCampagna(null);
			mop.setStato(stato);
			if(ip != null) {
				mop.setIp(ip);
			}
			mop.setTipoPausa(null);
			mop.setData_azione(data);
			if(ApplicationUtil.ne(user.getDisattivaFunzioni(), 2)) {
				mop.setVisibile(true);
			} else {
				mop.setVisibile(false);
			}
			mop.setReload(false);
			mop.setRegistrazione(false);
		}
	}

	private void settaLogout(Utente user,  Short stato, String ip) {
		if(user!= null) {
			MonitorOperatore mop = Chiamate.getCodaOperatori().get(user);
			if(mop == null) { 
				mop = new MonitorOperatore();
				Chiamate.getCodaOperatori().put(user, mop);
			} 
			mop.svuota();
			Date data = ApplicationUtil.date();
			mop.clearSession();
			mop.setLibero(0);
			mop.setOnline(false);
			mop.setAttivo(false);
			if(ApplicationUtil.gtZero(user.getLoginMultiplo())) {
				mop.setEndorse(data.getTime());
			} else {
				mop.setEndorse(null);
			}

			mop.setOperatore(user);
			mop.setContatto(null);
			mop.setCampagna(null);
			mop.setStato(stato);
			if(ip != null) {
				mop.setIp(ip);
			}
			mop.setTipoPausa(null);
			mop.setData_azione(data);
			if(ApplicationUtil.ne(user.getDisattivaFunzioni(), 2)) {
				mop.setVisibile(true);
			} else {
				mop.setVisibile(false);
			}
			mop.setReload(false);
			mop.setRegistrazione(false);
		}
	}

	public void settaRegistrazione(Utente user, Boolean registrazione) {
		if(user!= null) {
			MonitorOperatore mop = Chiamate.getCodaOperatori().get(user);
			if(mop != null) { 
				mop.setRegistrazione(registrazione);
			} 
		}
	}
	public void settaPannello(Utente user, EsitoArchiviazione esito, String pannello) {
		if(user!= null) {
			MonitorOperatore mop = Chiamate.getCodaOperatori().get(user);
			if(mop != null) { 
				if(pannello != null) {
					mop.setData_pannello(ApplicationUtil.date());
					mop.setEsito(esito);
				} else {
					mop.setData_pannello(null);
					mop.setEsito(null);
				}
				mop.setPannello(pannello);
				settaOperazionePannello(mop, user);
			} 
		}
	}

	public void rimuoviMonitor(Utente user) {
		MonitorOperatore mop = Chiamate.getCodaOperatori().get(user);
		if(mop != null) { 
			mop.setVisibile(false);
		} 
	}
	public void aggiungiMonitor(Utente user) {
		MonitorOperatore mop = Chiamate.getCodaOperatori().get(user);
		if(mop != null) { 
			mop.setVisibile(true);
		} 
	}
	private ExtendedCriteria getCriteria(PausaCriteria ema) {
		if(ema != null)  ema = (PausaCriteria) ema.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Pausa.class);
		DetachedCriteria criteriaU = criteria.createCriteria("user");
		if(ApplicationUtil.gtZero(ema.getOperatore())) {
			criteriaU.add(Restrictions.idEq(ema.getOperatore()));
		}
		if(ApplicationUtil.gtZero(ema.getTeam())) {
			impostazioneDao.subCriteriaOperatoreTeam(ema.getTeam(), criteriaU, "id");
		}
		if(ApplicationUtil.gtZero(ema.getPositivo())) {
			if(ApplicationUtil.eq(ema.getPositivo(), 1)) {
				criteria.add(Restrictions.eq("positivo", Boolean.TRUE));
			}
			if(ApplicationUtil.eq(ema.getPositivo(), 2)) {
				criteria.add(Restrictions.eq("positivo", Boolean.FALSE));
			}
		}
		if (ApplicationUtil.isNotBlank(ema.getDescrizione())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, ema.getDescrizione(), criteria, "chiave");
		}
		ema.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, ema);
	}
	@SuppressWarnings("unchecked")
	public List<Pausa> trova(PausaCriteria ema, int start, int size) {
		ExtendedCriteria criteria = getCriteria(ema);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int trovaSize(PausaCriteria ema) {
		ExtendedCriteria criteria = getCriteria(ema);
		return getListSize(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Pausa> trova(Utente utente, PausaCriteria ema, int start, int size) {
		ExtendedCriteria criteria = getCriteria(utente, ema);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int trovaSize(Utente utente, PausaCriteria ema) {
		ExtendedCriteria criteria = getCriteria(utente, ema);
		return getListSize(criteria);
	}
	public void elimina(Operazione oggetto) {
		cancellaAggiorna(oggetto);
	}
	public void elimina(Pausa oggetto) {
		cancellaAggiorna(oggetto);
	}
	private ExtendedCriteria getCriteria(Utente user, PausaCriteria ema) {
		if(ema != null)  ema = (PausaCriteria) ema.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Pausa.class);
		DetachedCriteria criteriaU = criteria.createCriteria("user");
		if(ApplicationUtil.gtZero(ema.getOperatore())) {
			criteriaU.add(Restrictions.idEq(ema.getOperatore()));
		}
		if(ApplicationUtil.gtZero(ema.getTeam())) {
			impostazioneDao.subCriteriaOperatoreTeam(ema.getTeam(), criteriaU, "id");
		}
		if(ApplicationUtil.gtZero(ema.getPositivo())) {
			if(ApplicationUtil.eq(ema.getPositivo(), 1)) {
				criteria.add(Restrictions.eq("positivo", Boolean.TRUE));
			}
			if(ApplicationUtil.eq(ema.getPositivo(), 2)) {
				criteria.add(Restrictions.eq("positivo", Boolean.FALSE));
			}
		}
		if (ApplicationUtil.isNotBlank(ema.getDescrizione())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, ema.getDescrizione(), criteria, "chiave");
		}
		impostazioneDao.subCriteriaSupervisore(user.getId(), criteriaU, "id");
		ema.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, ema);
	}
	public int contaTentativi(String ip, String username) {
		return loginAttemptService.get(ip, username);
	}
	public void loginFallito(String ip, String username) {
		loginAttemptService.loginFailed(ip, username);
	}
	public List<LoggerLogin> getTentativiLogin() {
		return loginAttemptService.table();
	}
	public void removeTentativoLogin(LoggerLogin l) {
		loginAttemptService.remove(l);
	}
	public void removeTentativiLogin() {
		loginAttemptService.clear();
	}
	public int getMassimoLogin1() {
		Integer m = impostazioneDao.getMassimoLogin1();
		if(ApplicationUtil.gtZero(m)) {
			return m.intValue();
		}
		return 0;
	}
	public int getMassimoLogin2() {
		Integer m = impostazioneDao.getMassimoLogin2();
		if(ApplicationUtil.gtZero(m)) {
			return m.intValue();
		}
		return 0;
	}
	public List<MonitorOperatore> getMonitorOperatoriList(OnlineCriteria m) {
		List<MonitorOperatore> monop = new ArrayList<MonitorOperatore>();
		Map<Utente, MonitorOperatore> operatori = Chiamate.getCodaOperatori().getData();
		if(ApplicationUtil.notEmpty(operatori)) {
			Iterator<Utente> objRIter = operatori.keySet().iterator();
			while(objRIter.hasNext()) {
				Utente user = objRIter.next();
				MonitorOperatore mo = operatori.get(user);
				if(mo != null) {
					boolean a = (m.getStato()==null || mo.getStato().equals(m.getStato()));
					boolean b = ApplicationUtil.isLocale(mo, m.getTipo());
					boolean c = ApplicationUtil.isBrowser(mo, m.getOnline());
					boolean d = ApplicationUtil.isRegistrato(mo, m.getLoggato());
					boolean e = ApplicationUtil.isVisibile(mo, m.getVisibile());
					if(a && b && c && d && e) {
						monop.add(mo);
					}
				}
			}
		}
		return monop;
	}
	public void rimuoviMonitorOperatoriList(OnlineCriteria m) {
		Map<Utente, MonitorOperatore> operatori = Chiamate.getCodaOperatori().getData();
		if(ApplicationUtil.notEmpty(operatori)) {
			Iterator<Utente> objRIter = operatori.keySet().iterator();
			while(objRIter.hasNext()) {
				Utente user = objRIter.next();
				MonitorOperatore mo = operatori.get(user);
				if(mo != null) {
					boolean a = (m.getStato()==null || mo.getStato().equals(m.getStato()));
					boolean b = ApplicationUtil.isLocale(mo, m.getTipo());
					boolean c = ApplicationUtil.isBrowser(mo, m.getOnline());
					boolean d = ApplicationUtil.isRegistrato(mo, m.getLoggato());
					boolean e = ApplicationUtil.isVisibile(mo, m.getVisibile());
					if(a && b && c && d && e) {
						mo.setVisibile(false);
					}
				}
			}
		}
	}
	public void aggiungiMonitorOperatoriList(OnlineCriteria m) {
		Map<Utente, MonitorOperatore> operatori = Chiamate.getCodaOperatori().getData();
		if(ApplicationUtil.notEmpty(operatori)) {
			Iterator<Utente> objRIter = operatori.keySet().iterator();
			while(objRIter.hasNext()) {
				Utente user = objRIter.next();
				MonitorOperatore mo = operatori.get(user);
				if(mo != null) {
					boolean a = (m.getStato()==null || mo.getStato().equals(m.getStato()));
					boolean b = ApplicationUtil.isLocale(mo, m.getTipo());
					boolean c = ApplicationUtil.isBrowser(mo, m.getOnline());
					boolean d = ApplicationUtil.isRegistrato(mo, m.getLoggato());
					boolean e = ApplicationUtil.isVisibile(mo, m.getVisibile());
					if(a && b && c && d && e) {
						mo.setVisibile(true);
					}
				}
			}
		}
	}
	public void forzaLogout(OnlineCriteria m) {
		Map<Utente, MonitorOperatore> operatori = Chiamate.getCodaOperatori().getData();
		if(ApplicationUtil.notEmpty(operatori)) {
			Iterator<Utente> objRIter = operatori.keySet().iterator();
			while(objRIter.hasNext()) {
				Utente user = objRIter.next();
				MonitorOperatore mo = operatori.get(user);
				if(mo != null && mo.isDaLogout()) {
					boolean a = (m.getStato()==null || mo.getStato().equals(m.getStato()));
					boolean b = ApplicationUtil.isLocale(mo, m.getTipo());
					boolean c = ApplicationUtil.isBrowser(mo, m.getOnline());
					boolean d = ApplicationUtil.isRegistrato(mo, m.getLoggato());
					boolean e = ApplicationUtil.isVisibile(mo, m.getVisibile());
					if(a && b && c && d && e) {
						forzaLogout(user);
					}
				}
			}
		}
	}

	public void forzaReload(OnlineCriteria m) {
		Map<Utente, MonitorOperatore> operatori = Chiamate.getCodaOperatori().getData();
		if(ApplicationUtil.notEmpty(operatori)) {
			Iterator<Utente> objRIter = operatori.keySet().iterator();
			while(objRIter.hasNext()) {
				Utente user = objRIter.next();
				MonitorOperatore mo = operatori.get(user);
				if(mo != null && mo.isDaReload()) {
					boolean a = (m.getStato()==null || mo.getStato().equals(m.getStato()));
					boolean b = ApplicationUtil.isLocale(mo, m.getTipo());
					boolean c = ApplicationUtil.isBrowser(mo, m.getOnline());
					boolean d = ApplicationUtil.isRegistrato(mo, m.getLoggato());
					boolean e = ApplicationUtil.isVisibile(mo, m.getVisibile());
					if(a && b && c && d && e) {
						forzaReload(user);
					}
				}
			}
		}
	}
	public void dashboard(MvcExternalContext mec, Utente user) {
		SharedAttributeMap map = mec.getSessionMap();
		SecurityContext sci = (SecurityContext) map.get(ConstantsGlobal.EVENT_KEY);	
		if(sci != null){
			Authentication a = sci.getAuthentication();
			if(a != null){
				//WebAuthenticationDetails wad = (WebAuthenticationDetails) a.getDetails();
				//map.put(user.getUsername(), wad.getRemoteAddress());		
			}
		}
	}

	public void inviaNotifica(String messaggio, Utente utente, Integer time) {
		String message = ApplicationUtil.trimToNull(messaggio);
		if (utente != null && message != null && time != null) {
			MonitorOperatore mop = Chiamate.getCodaOperatori().get(utente);
			if(mop != null) {
				mop.addMessaggio(time + message);
			}
		}
	}

	public void inviaNotifica(String messaggio, Integer time) {
		String message = ApplicationUtil.trimToNull(messaggio);
		if (message != null && time != null) {
			Map<Utente, MonitorOperatore> operatori = Chiamate.getCodaOperatori().getData();
			if(ApplicationUtil.notEmpty(operatori)) {
				Iterator<Utente> objRIter = operatori.keySet().iterator();
				while(objRIter.hasNext()) {
					Utente user = objRIter.next();
					MonitorOperatore mo = operatori.get(user);
					if(mo != null) {
						mo.addMessaggio(time + message);
					}
				}
			}
		}
	}
	public void removeUser(Utente user) {
		if(user.isLocale()) {
			Chiamate.getCodaInterni().accesso(user.getInternoString(), false);
			Chiamate.getCodaInterni().stato(user.getInternoString(), false);
			Chiamate.getCodaSpia().rimuovi(user.getInternoString());
			Chiamate.getCodaBuddy().remove(user.getInternoString());
		}
	}
	
	public void logoutUser() {
		Integer a = impostazioneDao.getLoginMultiplo();
		if(ApplicationUtil.gt(a, 1)) {
			Set<Utente> utenti = new HashSet<Utente>();
			Map<Utente, MonitorOperatore> operatori = Chiamate.getCodaOperatori().getData();
			if(ApplicationUtil.notEmpty(operatori)) {
				Iterator<Utente> objRIter = operatori.keySet().iterator();
				while(objRIter.hasNext()) {
					Utente user = objRIter.next();
					MonitorOperatore mop = operatori.get(user);
					if(mop != null) {
						if(!mop.isBrowser()) {
							if(mop.isLoggato()) {
								logoutSuccess(user, mop.getIp(), ConstantsGlobal.logoutR);
							} else {
								removeUser(user);
							}
							utenti.add(user);
						}
					}
				}
				if(ApplicationUtil.eq(a, 3) && ApplicationUtil.notEmpty(utenti)) {
					Chiamate.getCodaOperatori().removeList(utenti);
				}
			}
		}
	}

	public void logoutOrario() {
		Boolean a = impostazioneDao.getControlloUtente();
		if(ApplicationUtil.isTrue(a)) {
			Map<Utente, MonitorOperatore> operatori = Chiamate.getCodaOperatori().getData();
			if(ApplicationUtil.notEmpty(operatori)) {
				Iterator<Utente> objRIter = operatori.keySet().iterator();
				while(objRIter.hasNext()) {
					Utente user = objRIter.next();
					MonitorOperatore mop = operatori.get(user);
					if(mop != null) {
						if(mop.isLoggato() && !(user.isEnabled() && user.getStatoAttiva())) {
							logoutSuccess(user, mop.getIp(), ConstantsGlobal.logoutR);
						}
					}
				}
			}
		}
	}

	public void puliziaUtenti() {
		Map<Utente, MonitorOperatore> operatori = Chiamate.getCodaOperatori().getData();
		if(ApplicationUtil.notEmpty(operatori)) {
			Iterator<Utente> objRIter = operatori.keySet().iterator();
			while(objRIter.hasNext()) {
				Utente user = objRIter.next();
				MonitorOperatore mop = operatori.get(user);
				if(mop != null) {
					if(mop.isLoggato()) {
						logoutSuccess(user, mop.getIp(), ConstantsGlobal.logoutR);
					} else {
						removeUser(user);
					}
				}
			}
		}
	}

	public void forzaLogout(Utente user) {
		MonitorOperatore mop = Chiamate.getCodaOperatori().get(user);
		if(mop != null) { 
			logoutSuccessR(user, mop.getIp(), ConstantsGlobal.logoutF);
		}
	}

	public void logoutSuccessR(Utente user, String ip, String tipo) {
		logoutSuccess(user, ip, tipo);
		if(ApplicationUtil.eq(impostazioneDao.getLoginMultiplo(), 4)) {
			Chiamate.getCodaOperatori().remove(user);
		}
	}

	public List<SelectItem> getCampagnePrecedenzaSI() {
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		criteria.createCriteria("autodialer").add(Restrictions.eq("precedenza", Boolean.TRUE));
		List<GenericSI> lis = getGenericSI(criteria, "id", "nome");
		return impostazioneDao.getGeneric(lis, true);
	}
	
	@SuppressWarnings("unchecked")
	public List<Long> trovaOperatoriPrecedenza(Long campagnaId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		if(ApplicationUtil.gtZero(campagnaId)) {
			criteria.createCriteria("team").createCriteria("campagne").add(Restrictions.idEq(campagnaId));
		} else {
			criteria.createCriteria("team").createCriteria("campagne").createCriteria("autodialer").add(Restrictions.eq("precedenza", Boolean.TRUE));
		}
		criteria.setProjection(Projections.distinct(Projections.property("id")));
		return getList(criteria);
	}
	
	public List<MonitorOperatore> getMonitorPrecedenzaList(List<Long> utentiId, MonitorCriteria m) {
		List<MonitorOperatore> monop = new ArrayList<MonitorOperatore>();
		Map<Utente, MonitorOperatore> operatori = Chiamate.getCodaOperatori().getData();
		if(ApplicationUtil.notEmpty(operatori)) {
			Boolean al = ApplicationUtil.gtZero(impostazioneDao.getLoginMultiplo());
			Iterator<Utente> objRIter = operatori.keySet().iterator();
			while(objRIter.hasNext()) {
				Utente user = objRIter.next();
				MonitorOperatore mo = operatori.get(user);
				if(mo != null && ApplicationUtil.notEmpty(utentiId) && utentiId.contains(user.getId()) && mo.getCampagna() != null) {
					boolean a = ApplicationUtil.isTrue(mo.getVisibile());
					boolean b = ApplicationUtil.isLocale(mo, 1);
					boolean c = ApplicationUtil.isBrowser(mo, al ? 1 : 0);
					boolean d = ApplicationUtil.isRegistrato(mo, 1);
					if(a && b && c && d && Chiamate.getCodaProcessi().hasPrecedenza(mo.getCampagna())) {
						monop.add(mo);
					}
				}
			}
		}
		return monop;
	}

	public void rimuoviMonitorPrecedenzaList(List<Long> utentiId, MonitorCriteria m) {
		Map<Utente, MonitorOperatore> operatori = Chiamate.getCodaOperatori().getData();
		if(ApplicationUtil.notEmpty(operatori)) {
			Boolean al = ApplicationUtil.gtZero(impostazioneDao.getLoginMultiplo());
			Iterator<Utente> objRIter = operatori.keySet().iterator();
			while(objRIter.hasNext()) {
				Utente user = objRIter.next();
				MonitorOperatore mo = operatori.get(user);
				if(mo != null && ApplicationUtil.notEmpty(utentiId) && utentiId.contains(user.getId()) && mo.getCampagna() != null) {
					boolean a = ApplicationUtil.isTrue(mo.getVisibile());
					boolean b = ApplicationUtil.isLocale(mo, 1);
					boolean c = ApplicationUtil.isBrowser(mo, al ? 1 : 0);
					boolean d = ApplicationUtil.isRegistrato(mo, 1);
					if(a && b && c && d && Chiamate.getCodaProcessi().hasPrecedenza(mo.getCampagna())) {
						mo.setPrecedenza(0);
					}
				}
			}
		}
	}
}
