package it.callite.dao;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.faces.model.SelectItem;

import org.hibernate.Session;
import org.hibernate.Transaction;
import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Projections;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.DisposableBean;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import it.callite.constants.ConstantsGlobal;
import it.callite.criteria.OpposizioneCriteria;
import it.callite.criteria.base.ExtendedCriteria;
import it.callite.dao.generic.GenericDao;
import it.callite.log.ErrorLog;
import it.callite.model.Anagrafica;
import it.callite.model.Campagna;
import it.callite.model.Contatto;
import it.callite.model.Opposizione;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.Parameter;
import it.callite.utility.item.CampagnaSI;
@SuppressWarnings("serial")
@Repository("opposizioneDao")
public class OpposizioneDao extends GenericDao implements InitializingBean, DisposableBean {
	@Autowired
	private AutodialerDao autodialerDao;
	@Autowired
	private ImpostazioneDao impostazioneDao;
	
	public void insertOpposizione(List<Opposizione> valori) {
		if(ApplicationUtil.notEmpty(valori)) {
			Iterator<Opposizione> valIt = valori.iterator();
			while (valIt.hasNext()) {
				Opposizione opposizione = (Opposizione) valIt.next();
				autodialerDao.sendOpposizione(opposizione, true);
			}
		}
	}
	public void deleteOpposizione(List<Opposizione> valori) {
		if(ApplicationUtil.notEmpty(valori)) {
			Iterator<Opposizione> valIt = valori.iterator();
			while (valIt.hasNext()) {
				Opposizione opposizione = (Opposizione) valIt.next();
				autodialerDao.sendOpposizione(opposizione, false);
			}
		}
	}
	public void salva(Opposizione opposizione) {
		salvaAggiorna(opposizione);
		autodialerDao.sendOpposizione(opposizione, true);
	}
	public void modifica(Opposizione nuovo) {
		Opposizione vecchio = get(nuovo.getId());
		salvaAggiorna(nuovo);
		if(!ApplicationUtil.equals(nuovo.getOpposizione(), vecchio.getOpposizione()) || !ApplicationUtil.equals(nuovo.getCampagna(), vecchio.getCampagna())) {
			autodialerDao.sendOpposizione(vecchio, false);
			autodialerDao.sendOpposizione(nuovo, true);
		}
	}
	public Opposizione get(Long id){
		return (Opposizione) get(Opposizione.class, id);
	}
	public void elimina(Opposizione opposizione) {
		cancellaAggiorna(opposizione);
		autodialerDao.sendOpposizione(opposizione, false);
	}
	private ExtendedCriteria trovaCriteria(OpposizioneCriteria ana) {
		if(ana != null)  ana = (OpposizioneCriteria) ana.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Opposizione.class);
		if (ApplicationUtil.isNotBlank(ana.getRicerca_denominazione())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, ana.getRicerca_denominazione(), criteria,  "descrizione");
		}
		if (ApplicationUtil.isNotBlank(ana.getNumero())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, ana.getNumero(), criteria, "opposizione");
		}
		if (ApplicationUtil.gtZero(ana.getUtente())) {
			if (ApplicationUtil.eq(ana.getUtente(), 1)) {
				criteria.add(Restrictions.isNotNull("utente.id"));
			} else {
				criteria.add(Restrictions.isNull("utente.id"));
			}
		}
		if (ApplicationUtil.gtZero(ana.getCampagna())) {
			criteria.add(Restrictions.eq("campagna.id", ana.getCampagna()));
		} else {
			if (ApplicationUtil.eq(ana.getCampagna(), ConstantsGlobal.ALTRO)) {
				criteria.add(Restrictions.isNull("campagna.id"));
			}
		}
		ana.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, ana);
	}
	@SuppressWarnings("unchecked")
	public List<Opposizione> trova(OpposizioneCriteria ana, int start, int size) {
		ExtendedCriteria criteria = trovaCriteria(ana);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int trovaSize(OpposizioneCriteria ana) {
		ExtendedCriteria criteria = trovaCriteria(ana);
		return getListSize(criteria);
	}
	public void salvaOpposizioni(List<Opposizione> anagrafiche, Long campagna) {
		if (ApplicationUtil.notEmpty(anagrafiche)) {
			salvaListaOpposizione(anagrafiche, campagna);
		}
	}
	public void eliminaOpposizioni(List<Opposizione> anagrafiche, Long campagna) {
		if (ApplicationUtil.notEmpty(anagrafiche)) {
			eliminaListaOpposizione(anagrafiche, campagna);
		}
	}
	public void salvaListaOpposizione(List<Opposizione> objs, Long campagna) {
		if(ApplicationUtil.notEmpty(objs)){
			Session session = openSession();
			Transaction tx = session.beginTransaction();
			int i = 0;
			Iterator<Opposizione> objIter = objs.iterator();
			while(objIter.hasNext()) {
				Opposizione opposizione = objIter.next();
				if(!opposizione.getDuplicato()) {
					session.save(opposizione);
					if (i++ % ConstantsGlobal.flushSize == 0) {
						session.flush();
						session.clear();
					}
				}
			}
			tx.commit();
			closeSession(session);
		}
		insertOpposizione(objs);
	}
	public void eliminaListaOpposizione(List<Opposizione> objs, Long campagna) {
		try {
			List<String> opp = prendiOpposizioni(objs);
			if(ApplicationUtil.notEmpty(opp)) {
				if(ApplicationUtil.gtZero(campagna)) {
					String hql = "delete from Opposizione where opposizione in (:opposizione) and campagna.id = :campagna";
					execute(false, hql, new Parameter("opposizione", opp), new Parameter("campagna", campagna));
				} else {
					String hql = "delete from Opposizione where opposizione in (:opposizione)";
					execute(false, hql, new Parameter("opposizione", opp));
				}
			}
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
		deleteOpposizione(objs);
	}
	public List<String> prendiOpposizioni(List<Opposizione> c) {
		List<String> ln = new ArrayList<String>();
		if(ApplicationUtil.notEmpty(c)) {
			Iterator<Opposizione> objIter = c.iterator();
			while(objIter.hasNext()) {
				Opposizione an = objIter.next();
				if(!an.getDuplicato() && an.getTrovato()) {
					if(!ln.contains(an.getOpposizione())) {
						ln.add(an.getOpposizione());
					}
				}
			}
		}
		return ln;
	}
	public void pulisciOpposizioni() {
		List<String> anc =  getOpposizioneList(null);
		if(ApplicationUtil.notEmpty(anc)) {
			List<Anagrafica> ana = getAnagraficaNonContattabileList(anc);
			if(ApplicationUtil.notEmpty(ana)) {
				cancellaLista(ana);
			}
		}
		List<Long> campagne = getCampagne();
		if(ApplicationUtil.notEmpty(campagne)) {
			Iterator<Long> valIt = campagne.iterator();
			while (valIt.hasNext()) {
				Long campagna = (Long) valIt.next();
				List<String> ancC =  getOpposizioneList(campagna);
				if(ApplicationUtil.notEmpty(ancC)) {
					List<Contatto> cno = getContattoNonContattabileList(campagna, ancC);
					if(ApplicationUtil.notEmpty(cno)) {
						cancellaLista(cno);
					}
				}
			}
		}
	}
	public void pulisciOpposizione(Opposizione opposizione) {
		if(opposizione.getCampagna() != null) {
			List<Contatto> cno = getContattoNonContattabile(opposizione.getCampagna().getId(), opposizione.getOpposizione());
			if(ApplicationUtil.notEmpty(cno)) {
				cancellaLista(cno);
			}
		} else {
			List<Anagrafica> ana = getAnagraficaNonContattabile(opposizione.getOpposizione());
			if(ApplicationUtil.notEmpty(ana)) {
				cancellaLista(ana);
			}
		}
	}
	public void pulisciOpposizioneSelezionati(List<Opposizione> opposizioni) {
		if(ApplicationUtil.notEmpty(opposizioni)) {
			Iterator<Opposizione> valIt = opposizioni.iterator();
			while (valIt.hasNext()) {
				Opposizione opposizione = (Opposizione) valIt.next();
				pulisciOpposizione(opposizione);
			}
		}
	}
	public void sincronizzaOpposizione() {
		List<String> opps = getOpposizioneList(null);
		sincronizzaOpposizione(opps);
	}
	public void sincronizzaOpposizione(List<String> valori) {
		autodialerDao.sincronizzaOpposizione(valori);
	}
	
	public List<SelectItem> getCampagneSI() {
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		List<CampagnaSI> lis = getCampagnaSI(criteria);
		return impostazioneDao.getCampagna(lis, false, false, false, null);
	}
	public List<SelectItem> getCampagneRicercaSI() {
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		List<CampagnaSI> lis = getCampagnaSI(criteria);
		return impostazioneDao.getCampagna(lis, true, false, true, null);
	}
	@SuppressWarnings("unchecked")
	public List<Anagrafica> getAnagraficaNonContattabileList(List<String> telefono) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Anagrafica.class);
		criteria.add(Restrictions.or(Restrictions.in("telefono_fisso", telefono),Restrictions.in("telefono_mobile", telefono),Restrictions.in("telefono_fax", telefono),Restrictions.in("telefono_altro", telefono),Restrictions.in("email", telefono),Restrictions.in("link", telefono),Restrictions.in("codice_fiscale", telefono),Restrictions.in("partita_iva", telefono)));
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Anagrafica> getAnagraficaNonContattabile(String telefono) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Anagrafica.class);
		criteria.add(Restrictions.or(Restrictions.eq("telefono_fisso", telefono),Restrictions.eq("telefono_mobile", telefono),Restrictions.eq("telefono_fax", telefono),Restrictions.eq("telefono_altro", telefono),Restrictions.eq("email", telefono),Restrictions.eq("link", telefono),Restrictions.eq("codice_fiscale", telefono),Restrictions.eq("partita_iva", telefono)));
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Long> getCampagne() {
		DetachedCriteria criteria = DetachedCriteria.forClass(Opposizione.class);
		criteria.add(Restrictions.isNotNull("campagna.id"));
		criteria.setProjection(Projections.distinct(Projections.property("campagna.id")));
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Contatto> getContattoNonContattabileList(Long campagnaId, List<String> telefono) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class);
		criteria.add(Restrictions.eq("campagna.id", campagnaId));
		DetachedCriteria criteriaA = criteria.createCriteria("anagrafica");
		criteriaA.add(Restrictions.or(Restrictions.in("telefono_fisso", telefono),Restrictions.in("telefono_mobile", telefono),Restrictions.in("telefono_fax", telefono),Restrictions.in("telefono_altro", telefono),Restrictions.in("email", telefono),Restrictions.in("link", telefono),Restrictions.in("codice_fiscale", telefono),Restrictions.in("partita_iva", telefono)));
		
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Contatto> getContattoNonContattabile(Long campagnaId, String telefono) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class);
		criteria.add(Restrictions.eq("campagna.id", campagnaId));
		DetachedCriteria criteriaA = criteria.createCriteria("anagrafica");
		criteriaA.add(Restrictions.or(Restrictions.eq("telefono_fisso", telefono),Restrictions.eq("telefono_mobile", telefono),Restrictions.eq("telefono_fax", telefono),Restrictions.eq("telefono_altro", telefono),Restrictions.eq("email", telefono),Restrictions.eq("link", telefono),Restrictions.eq("codice_fiscale", telefono),Restrictions.eq("partita_iva", telefono)));
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<String> getOpposizioneList(Long campagnaId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Opposizione.class);
		if(ApplicationUtil.gtZero(campagnaId)) {
			criteria.add(Restrictions.eq("campagna.id", campagnaId));
		} else {
			criteria.add(Restrictions.isNull("campagna.id"));
		}
		criteria.setProjection(Projections.distinct(Projections.property("opposizione")));
		return getList(criteria);
	}
	public void eliminaOpposizioneSelezionati(List<Opposizione> objs) {
		cancellaLista(objs);
		deleteOpposizione(objs);
	}
	public Long getByOpposizione(Long campagnaId, String opposizione) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Opposizione.class);
		if(ApplicationUtil.gtZero(campagnaId)) {
			criteria.add(Restrictions.eq("campagna.id", campagnaId));
		} else {
			criteria.add(Restrictions.isNull("campagna.id"));
		}
		criteria.add(Restrictions.eq("opposizione", opposizione));
		criteria.setProjection(Projections.distinct(Projections.property("id")));
		return (Long) getUniqueResult(criteria);
	}
	public Opposizione getByFullOpposizione(Campagna campagna, String opposizione) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Opposizione.class);
		if(campagna != null) {
			criteria.add(Restrictions.eq("campagna.id", campagna.getId()));
		} else {
			criteria.add(Restrictions.isNull("campagna.id"));
		}
		criteria.add(Restrictions.eq("opposizione", opposizione));
		return (Opposizione) getUniqueResult(criteria);
	}
	public void destroy() throws Exception {
		//sincronizzaOpposizione();
	}
	public void afterPropertiesSet() throws Exception {
		//sincronizzaOpposizione();
	}
}