package it.callite.dao;
import java.io.ByteArrayInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Random;
import java.util.concurrent.atomic.AtomicInteger;

import javax.faces.model.SelectItem;

import org.hibernate.FetchMode;
import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.ProjectionList;
import org.hibernate.criterion.Projections;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import it.callite.autodialer.controllo.CodaManager;
import it.callite.autodialer.controllo.CodaStatic;
import it.callite.chiamate.Chiamate;
import it.callite.constants.ConstantsEsito;
import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.constants.ConstantsMap;
import it.callite.criteria.ContattoPannelloCriteria;
import it.callite.criteria.base.ExtendedOrder;
import it.callite.dao.generic.GenericDao;
import it.callite.log.ErrorLog;
import it.callite.message.Message;
import it.callite.model.Agente;
import it.callite.model.Anagrafica;
import it.callite.model.Appuntamento;
import it.callite.model.AutoContatto;
import it.callite.model.Busta;
import it.callite.model.BustaStore;
import it.callite.model.Campagna;
import it.callite.model.Contatto;
import it.callite.model.Contratto;
import it.callite.model.DatoAggiuntivoAnagrafica;
import it.callite.model.DatoAggiuntivoContratto;
import it.callite.model.Email;
import it.callite.model.EmailTemplate;
import it.callite.model.EsitoAppuntamento;
import it.callite.model.EsitoArchiviazione;
import it.callite.model.EsitoChiamata;
import it.callite.model.EsitoContratto;
import it.callite.model.Ingresso;
import it.callite.model.Rmail;
import it.callite.model.RmailTemplate;
import it.callite.model.Lista;
import it.callite.model.Nota;
import it.callite.model.Opposizione;
import it.callite.model.Pausa;
import it.callite.model.Prodotto;
import it.callite.model.StoriaAppuntamento;
import it.callite.model.StoriaContatto;
import it.callite.model.StoriaNota;
import it.callite.model.Telecom;
import it.callite.model.TelecomTemplate;
import it.callite.model.Template;
import it.callite.model.TemplateProdotto;
import it.callite.model.TemplateStore;
import it.callite.model.Utente;
import it.callite.model.allegato.AllegatoAppuntamento;
import it.callite.model.allegato.AllegatoContatto;
import it.callite.model.allegato.AllegatoContratto;
import it.callite.model.config.ConfigPausa;
import it.callite.model.report.ChiamateAttive;
import it.callite.service.config.ConfigurazioneGenericService;
import it.callite.utility.AiutoCampagna;
import it.callite.utility.Allegato;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.CallerUtil;
import it.callite.utility.ContrattoProdotto;
import it.callite.utility.DatoAggiuntivo;
import it.callite.utility.DatoSondaggioAnagrafica;
import it.callite.utility.Label;
import it.callite.utility.ListaRandom;
import it.callite.utility.LoggerContatto;
import it.callite.utility.ProdottoCheckbox;
import it.callite.utility.ProdottoSelezionato;
import it.callite.utility.Transizione;
import it.callite.utility.item.EsitoAppuntamentoSI;
import it.callite.utility.sincronizzazione.InfoContatto;
import it.callite.utility.thread.AgenteMessaggioThread;
import it.callite.utility.thread.AgenteNotiziaThread;
import it.callite.utility.thread.AppuntamentoMessaggioThread;
import it.callite.utility.thread.AppuntamentoNotiziaThread;
import it.callite.utility.thread.ApriIngressoThread;
import it.callite.utility.thread.ContattoMessaggioThread;
import it.callite.utility.thread.ContattoNotiziaThread;
import it.callite.utility.thread.EmailNotificaThread;
import it.callite.utility.thread.EsitaPannelloThread;
import it.callite.utility.thread.FirmaThread;
import it.callite.utility.thread.PannelloEmailSenderThread;
import it.callite.utility.thread.PannelloRmailSenderThread;
import it.callite.utility.thread.PannelloTelecomSenderThread;
import it.callite.utility.thread.ProprietarioMessaggioThread;
import it.callite.utility.thread.ProprietarioNotiziaThread;
import it.callite.utility.thread.SalvaStoriaAppuntamentoThread;
import it.callite.utility.thread.SalvaStoriaContattoThread;
import it.callite.utility.thread.SalvaStoriaNotaThread;
import it.callite.utility.thread.SettaOpposizioneThread;
import it.callite.utility.thread.invio.MultiploEmailSenderThread;
import it.callite.utility.thread.invio.MultiploRmailSenderThread;
import it.callite.utility.thread.invio.MultiploTelecomSenderThread;
@SuppressWarnings("serial")
@Repository("pannelloDao")
public class PannelloDao extends GenericDao {
	@Autowired
	private OperazioneDao operazioneDao;
	@Autowired
	private UserDao userDao;
	@Autowired
	private ImpostazioneDao impostazioneDao;
	@Autowired
	private CampagnaDao campagnaDao;
	@Autowired
	private ContattoDao contattoDao;
	@Autowired
	private ProdottoDao prodottoDao;
	@Autowired
	private ComunicazioneDao comunicazioneDao;
	@Autowired
	private AutodialerDao autodialerDao;
	@Autowired
	private SondaggioDao sondaggioDao;
	@Autowired
	private AutoInvioDao autoInvioDao;
	@Autowired
	private ContrattoDao contrattoDao;
	@Autowired
	private DownloadDao downloadDao;
	@Autowired
	private OpposizioneDao opposizioneDao;
	@Autowired
	private AnagraficaDao anagraficaDao;
	@Autowired
	private NotiziaDao notiziaDao;
	@Autowired
	private MessaggioDao messaggioDao;
	@Autowired
	private ConfigurazioneGenericService configurazioneGenericService;
	
	public void impostaEsito(EsitoArchiviazione esito) {
		if(ApplicationUtil.gtZero(esito.getContratto_imposta())) {
			esito.setContratto_tipo(2);
		} else {
			esito.setContratto_tipo(0);
		}
		esito.setContratto_pda(false);
		esito.setContratto_aiuto(false);
		esito.setContratto_selezione(0);
		esito.setContratto_ricerca(false);
		esito.setContratto_filtro_prodotto(null);
		esito.setContratto_filtro_categoria(null);
	}
	private List<ChiamateAttive> prendiChiamateAttive(List<Long> contatti, Integer privata, List<Long> utentiId) {	
		return Chiamate.getCodaContatti().get(contatti, privata, utentiId);
	}
	private List<ChiamateAttive> prendiChiamateAttive(Integer privata) {	
		return Chiamate.getCodaContatti().get(privata);
	}
	public Contatto getEstrazioneContatto(DetachedCriteria criteria) {
		Contatto out = null;
		String s = impostazioneDao.getEstrazioneContatti();
		if(ApplicationUtil.equals(s, ConstantsMap.diretta)) {
			out =  (Contatto) getFirstResult(criteria);
		}
		if(ApplicationUtil.equals(s, ConstantsMap.indiretta)) {
			criteria.setProjection((Projections.distinct(Projections.property("id"))));
			Long cnt =  (Long) getFirstResult(criteria);
			if(ApplicationUtil.gtZero(cnt)) {
				DetachedCriteria crit = DetachedCriteria.forClass(Contatto.class);
				crit.add(Restrictions.idEq(cnt));
				out = (Contatto) getUniqueResult(crit);
			}
		}
		if(out != null) {
			Integer tel = out.getTipoTelefono();
			if(ApplicationUtil.eq(tel, 6) || ApplicationUtil.eq(tel, 8)) {
				List<String> tels = getCampiTelefoniciS(out);
				out.setTelefoniS(tels);
			}
		}
		return out;
	}
	@SuppressWarnings("unchecked")
	public List<Contatto> getEstrazioneContatto(DetachedCriteria criteria, int size) {
		List<Contatto> outs = new ArrayList<Contatto>();
		String s = impostazioneDao.getEstrazioneContatti();
		if(ApplicationUtil.equals(s, ConstantsMap.diretta)) {
			outs.addAll(getFirstResult(criteria, size));
		}
		if(ApplicationUtil.equals(s, ConstantsMap.indiretta)) {
			criteria.setProjection((Projections.distinct(Projections.property("id"))));
			List<Long> cnt =  getFirstResult(criteria, size);
			if(ApplicationUtil.notEmpty(cnt)) {
				DetachedCriteria crit = DetachedCriteria.forClass(Contatto.class);
				crit.add(Restrictions.in("id", cnt));
				outs.addAll(getList(crit));
			}
		}
		for(Contatto out : outs) {
			Integer tel = out.getTipoTelefono();
			if(ApplicationUtil.eq(tel, 6) || ApplicationUtil.eq(tel, 8)) {
				List<String> tels = getCampiTelefoniciS(out);
				out.setTelefoniS(tels);
			}
		}
		return outs;
	}
	@SuppressWarnings("unchecked")
	public List<Long> getListeCampagna(String tipo, Campagna campagna, Boolean nuovo, Utente utente) {
		DetachedCriteria listaC = DetachedCriteria.forClass(Lista.class);
		if(ApplicationUtil.isTrue(campagna.getSeleziona_liste())) {	
			List<Long> ids = null;
			if(utente != null) {
				ids = CodaStatic.getCodaLista().getDataL(campagna, utente);
			} else {
				ids = CodaStatic.getCodaLista().getDataL(campagna);
			}
			if(ApplicationUtil.notEmpty(ids)) {
				listaC.add(Restrictions.in("id",ids));
			}
		}
		listaC.createCriteria("campagna").add(Restrictions.idEq(campagna.getId()));
		Boolean s = impostazioneDao.getEstrazioneListe();
		if(ApplicationUtil.isTrue(s)) {
			Boolean ts = impostazioneDao.getEstrazioneStato();
			ApplicationUtil.criteriaStato(listaC, true, ApplicationUtil.isTrue(ts));
		}
		DetachedCriteria crit = listaC.createCriteria("chiamate");
		String proprieta = ApplicationUtil.trimToNull(campagna.getProprieta());
		String query = ApplicationUtil.trimToNull(campagna.getQuery_proprieta());
		String indirizzo = ConstantsMap.indirizzo;
		String legale = ConstantsMap.indirizzo_legale;
		String operativo = ConstantsMap.indirizzo_operativo;
		if(ApplicationUtil.isNotBlank(tipo) || proprieta != null || query != null) {
			DetachedCriteria crita = crit.createCriteria("anagrafica", "a");
			DetachedCriteria critaL = null;
			DetachedCriteria critaO = null;
			if(ApplicationUtil.isNotBlank(tipo)) { 
				crita.add(ApplicationUtil.isNotNull(tipo));
			}
			if(proprieta != null || query != null) {
				if((proprieta != null && proprieta.startsWith(legale)) || (query != null && query.startsWith(legale))) {
					critaL = crita.createCriteria(legale, legale);
				}
				if((proprieta != null && proprieta.startsWith(operativo)) || (query != null && query.startsWith(operativo))) {
					critaO = crita.createCriteria(operativo, operativo);
				}
			}
			if(proprieta != null) {
				estrazioneCriteria(indirizzo, legale, operativo, proprieta, crita, critaL, critaO, campagna);
			}
			if(query != null) {
				ricercaCriteria(indirizzo, legale, operativo, query, crita,critaL, critaO, campagna);
			}
		}
		contattoCriteria(crit, campagna, nuovo);
		ProjectionList projection = Projections.projectionList();
		projection.add(Projections.distinct(Projections.property("id").as("id")));
		projection.add(Projections.property("priorita").as("priorita"));
		listaC.setProjection(projection);
		listaC.addOrder(Order.desc("priorita"));
		List<ListaRandom>  lst = getList(listaC, ListaRandom.class);
		List<Long> ids = new ArrayList<Long>();
		if(ApplicationUtil.notEmpty(lst)) {
			Iterator<ListaRandom> itR = lst.iterator();
			while (itR.hasNext()) {
				ListaRandom lr = (ListaRandom) itR.next();
				int pr = lr.getPriorita() != null ? lr.getPriorita() : 1;
				for(int i = 0; i < pr; i++ ) {
					ids.add(lr.getId());
				}
			}
		}
		return ids;
	}
	public DetachedCriteria getCriteriaContatto(String tipo, Campagna campagna, Boolean nuovo, Utente utente) {
		Long idLista = null;
		Integer est = campagna.getEstrazione_lista();
		if(ApplicationUtil.gtZero(est)) {
			List<Long> ids = getListeCampagna(tipo, campagna, nuovo, utente);
			if(ApplicationUtil.notEmpty(ids)) {
				if(ApplicationUtil.eq(est,1)) {
					AtomicInteger seq = campagna.getSeq();
					int index = seq.get();
					if(index >= ids.size()) {
						int reset = 0;
						seq.set(reset);
						index = reset;
					}
					idLista = ids.get(index);
					seq.set(index+1);
				} 
				if(ApplicationUtil.eq(est,2)) {
					int index = new Random(System.nanoTime()).nextInt(ids.size());
					idLista = ids.get(index);
				}
			}
		}
		DetachedCriteria crit = DetachedCriteria.forClass(Contatto.class);
		if(ApplicationUtil.gtZero(idLista)) {
			crit.add(Restrictions.eq("lista.id", idLista));
		} else {
			Boolean s = impostazioneDao.getEstrazioneListe();
			if(ApplicationUtil.isTrue(s)) {
				Boolean ts = impostazioneDao.getEstrazioneStato();
				DetachedCriteria listaC = crit.createCriteria("lista");
				ApplicationUtil.criteriaStato(listaC, true, ApplicationUtil.isTrue(ts));
			}
		}
		DetachedCriteria crita = null;
		DetachedCriteria critaL = null;
		DetachedCriteria critaO = null;
		String ordine = ApplicationUtil.trimToNull(campagna.getOrdine());
		String proprieta = ApplicationUtil.trimToNull(campagna.getProprieta());
		String query = ApplicationUtil.trimToNull(campagna.getQuery_proprieta());
		String [] ords = null;
		String indirizzo = ConstantsMap.indirizzo;
		String legale = ConstantsMap.indirizzo_legale;
		String operativo = ConstantsMap.indirizzo_operativo;
		if(ApplicationUtil.isNotBlank(tipo) || proprieta != null || query != null || ordine != null) {
			crita = crit.createCriteria("anagrafica", "a");
			if(ApplicationUtil.isNotBlank(tipo)) { 
				crita.add(ApplicationUtil.isNotNull(tipo));
			}
			if(proprieta != null || query != null || ordine != null) {
				if(ordine != null) {
					ords = ordine.split(ConstantsGlobal.colonSeparator);
				}
				if((proprieta != null && proprieta.startsWith(legale)) || (query != null && query.startsWith(legale)) || (ords != null && ords[0].startsWith(legale))) {
					critaL = crita.createCriteria(legale, legale);
				}
				if((proprieta != null && proprieta.startsWith(operativo)) || (query != null && query.startsWith(operativo)) ||  (ords != null && ords[0].startsWith(operativo))) {
					critaO = crita.createCriteria(operativo, operativo);
				}
			}
			if(proprieta != null) {
				estrazioneCriteria(indirizzo, legale, operativo, proprieta, crita, critaL, critaO, campagna);
			}
			if(query != null) {
				ricercaCriteria(indirizzo, legale, operativo, query, crita, critaL, critaO, campagna);
			}
		}
		contattoCriteria(crit, campagna, nuovo);
		if(!ApplicationUtil.gtZero(idLista) && ApplicationUtil.eq(est, 0)) {
			crit.addOrder(Order.desc("priorita"));
		}
		Boolean p = impostazioneDao.getEstrazionePrioritaria();
		if(ApplicationUtil.isFalse(nuovo) && ApplicationUtil.isTrue(p)) {
			crit.addOrder(Order.asc("ordine"));
		}
		crit.addOrder(Order.asc("data_presentazione"));
		ordineCriteria(indirizzo, legale, operativo, ords, crita,  critaL, critaO, campagna);
		return crit;
	}
	public void estrazioneCriteria(String indirizzo, String legale, String operativo, String proprieta, DetachedCriteria crita, DetachedCriteria critaL, DetachedCriteria critaO, Campagna campagna) {
		if(proprieta != null) {
			List<String> ls = Chiamate.getCodaCampagne().get(campagna);
			if(ApplicationUtil.notEmpty(ls)) {
				if(crita != null && !proprieta.startsWith(indirizzo)) {
					if(proprieta.startsWith(ConstantsMap.anagrafica)) {
						String ord = proprieta.substring((ConstantsMap.anagrafica + "_").length());
						crita.add(Restrictions.not(Restrictions.in(ord, ls)));
					} else {
						crita.add(Restrictions.not(Restrictions.in(proprieta, ls)));
					}
				} else {
					if(critaL != null && proprieta.startsWith(legale)) {
						String ord = proprieta.substring((legale + "_").length());
						critaL.add(Restrictions.not(Restrictions.in(ord, ls)));
					}
					if(critaO != null && proprieta.startsWith(operativo)) {
						String ord = proprieta.substring((operativo + "_").length());
						critaO.add(Restrictions.not(Restrictions.in(ord, ls)));
					}
				}
			}
		}
	}
	public void ordineCriteria(String indirizzo, String legale, String operativo, String [] ords, DetachedCriteria crita, DetachedCriteria critL, DetachedCriteria critO, Campagna campagna) {
		if(ApplicationUtil.notEmpty(ords)) {
			if(crita != null && !ords[0].startsWith(indirizzo)) {
				if(ords[0].startsWith(ConstantsMap.anagrafica)) {
					String ord = ords[0].substring((ConstantsMap.anagrafica + "_").length());
					if(ords[1].equals(ConstantsGlobal.desc)) {
						crita.addOrder(Order.desc(ord));
					} else {
						crita.addOrder(Order.asc(ord));
					}
				} else {
					if(ords[1].equals(ConstantsGlobal.desc)) {
						crita.addOrder(Order.desc(ords[0]));
					} else {
						crita.addOrder(Order.asc(ords[0]));
					}
				}
			} else {
				if(critL != null && ords[0].startsWith(legale)) {
					String ord = ords[0].substring((legale + "_").length());
					if(!ord.equals(ConstantsMap.geocodifica)) {
						if(ords[1].equals(ConstantsGlobal.desc)) {
							critL.addOrder(Order.desc(ord));
						} else {
							critL.addOrder(Order.asc(ord));
						}
					} else {
						String lat = campagna.getLatitudine();
						String lon = campagna.getLongitudine();
						if(lat != null && lon !=null) {
							if(ords[1].equals(ConstantsGlobal.desc)) {
								critL.addOrder(ExtendedOrder.desc(ApplicationUtil.ordineCriteria("", lat, lon)));
							} else {
								critL.addOrder(ExtendedOrder.asc(ApplicationUtil.ordineCriteria("", lat, lon)));
							}
						}
					}
				}
				if(critO != null &&  ords[0].startsWith(operativo)) {
					String ord = ords[0].substring((operativo + "_").length());
					if(!ord.equals(ConstantsMap.geocodifica)) {
						if(ords[1].equals(ConstantsGlobal.desc)) {
							critO.addOrder(Order.desc(ord));
						} else {
							critO.addOrder(Order.asc(ord));
						}
					} else {
						String lat = campagna.getLatitudine();
						String lon = campagna.getLongitudine();
						if(lat != null && lon !=null) {
							if(ords[1].equals(ConstantsGlobal.desc)) {
								critO.addOrder(ExtendedOrder.desc(ApplicationUtil.ordineCriteria("", lat, lon)));
							} else {
								critO.addOrder(ExtendedOrder.asc(ApplicationUtil.ordineCriteria("", lat, lon)));
							}
						}
					}
				}
			}
		}
	}
	public void ricercaCriteria(String indirizzo, String legale, String operativo, String proprieta, DetachedCriteria crita, DetachedCriteria critaL, DetachedCriteria critaO,  Campagna campagna) {
		if(proprieta != null) {
			String valore = campagna.getQuery_valore();
			if(valore != null) {
				if(crita != null  && !proprieta.startsWith(indirizzo)) {
					if(proprieta.startsWith(ConstantsMap.anagrafica)) {
						String ord = proprieta.substring((ConstantsMap.anagrafica + "_").length());
						crita.add(Restrictions.sqlRestriction("{alias}." + ord + " " + valore));
					} else {
						crita.add(Restrictions.sqlRestriction("{alias}." + proprieta + " " + valore));
					}
				} else {
					if(critaL != null && proprieta.startsWith(legale)) {
						String ord = proprieta.substring((legale + "_").length());
						critaL.add(Restrictions.sqlRestriction("{alias}." + ord + " " + valore));
					}
					if(critaO != null && proprieta.startsWith(operativo)) {
						String ord = proprieta.substring((operativo + "_").length());
						critaO.add(Restrictions.sqlRestriction("{alias}." + ord + " " + valore));
					}
				}
			}
		}
	}
	public void contattoCriteria(DetachedCriteria crit, Campagna campagna, Boolean nuovo) {
		Boolean tutti = impostazioneDao.getEstrazioneCampagna();
		crit.add(Restrictions.eq("tipo", ConstantsGlobal.inviato));
		crit.add(Restrictions.eq("campagna.id", campagna.getId()));
		ApplicationUtil.archiviazione(crit, ApplicationUtil.date(), false);
		crit.add(Restrictions.isNull("proprietario.id"));
		if(ApplicationUtil.isTrue(nuovo)) {
			crit.add(Restrictions.eq("esitoChiamata.id", ConstantsEsito.ID_DA_CHIAMARE));
		} 
		if(ApplicationUtil.isNotTrue(nuovo)) {
			crit.add(Restrictions.ne("esitoChiamata.id", ConstantsEsito.ID_DA_CHIAMARE));
		}
		Chiamate.getCodaContatti().criteria(tutti, campagna, crit);
		Chiamate.getCodaAutodialer().criteria(tutti, campagna, crit);
	}
	private synchronized Contatto getContattoPubblico(Integer flag, String tipo, Utente user, Campagna campagna, Boolean nuovo) {
		LoggerContatto l = new LoggerContatto(impostazioneDao.getLog());
		long time = System.currentTimeMillis();
		DetachedCriteria criteria = getCriteriaContatto(tipo, campagna, nuovo, user);
		Contatto out =  getEstrazioneContatto(criteria);
		if (out != null) {
			Chiamate.getCodaContatti().put(out.getId(), new InfoContatto(time, user, out, false));
			operazioneDao.apriContatto(flag, user, out);
		} else {
			if(campagna != null) {
				campagna.criterio(tipo, campagna.getProprieta(), campagna.getLimite(), campagna.getQuery_proprieta(), campagna.getQuery_valore(), nuovo, true);
			}
		}
		l.log("aperto pubblico", user, out);
		return out;
	}
	public void updateContatto(Utente utente, Contatto contatto) {
		if(contatto!=null) {
			Chiamate.getCodaContatti().update(contatto.getId());
		}
	}
	
	public void lasciaContatto(Utente utente, Contatto contatto) {
		LoggerContatto l = new LoggerContatto(impostazioneDao.getLog());
		Chiamate.getCodaContatti().remove(contatto.getId());
		l.log("rilascio contatto", utente, contatto);
	}
	
	public void timeoutContatto() {
		List<InfoContatto> contatti = Chiamate.getCodaContatti().timeout(ApplicationUtil.date());
		if(ApplicationUtil.notEmpty(contatti)) {
			Iterator<InfoContatto> lnt = contatti.iterator();
			while (lnt.hasNext()) {
				InfoContatto info = lnt.next();
				if(info!=null) {
					Contatto contatto = info.getItem();
					if(contatto != null) {
						esitaCampagna(contatto, info.getUtente());
					}
				}
			}
		}
	}
	
	public void timeoutRisposta() {
		List<Contatto> contatti = Chiamate.getCodaRisposta().timeout(ApplicationUtil.date());
		if(ApplicationUtil.notEmpty(contatti)) {
			Iterator<Contatto> lnt = contatti.iterator();
			while (lnt.hasNext()) {
				Contatto info = lnt.next();
				if(info!=null) {
					Chiamate.getCodaRisposta().remove(info.getId());
				}
			}
		}
	}
	
	private Contatto getContattoPrivato(String tipo, Integer flag, Utente user, Contatto in) {
		LoggerContatto l = new LoggerContatto(impostazioneDao.getLog());
		long time = System.currentTimeMillis();
		Contatto out = getContatto(false, tipo, user, in);
		if (out != null) {
			Integer tel = out.getTipoTelefono();
			if(ApplicationUtil.eq(tel, 6) || ApplicationUtil.eq(tel, 8)) {
				List<String> tels = getCampiTelefoniciS(out);
				out.setTelefoniS(tels);
			}
			Chiamate.getCodaContatti().put(out.getId(), new InfoContatto(time, user, out, true));
			operazioneDao.apriContatto(flag, user, out);
		} 
		l.log("aperto privato", user, out);
		return out;
	}
	public Contatto loadContatto(Contatto in) {
		Contatto out = null;
		if(ApplicationUtil.isTrue(impostazioneDao.getSincronia())) {
			DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class);
			criteria.add(Restrictions.idEq(in.getId()));
			out =  (Contatto) getUniqueResult(criteria);
		}
		if(out == null) {
			out = in;
		}
		if(out != null) {
			Integer tel = out.getTipoTelefono();
			if(ApplicationUtil.eq(tel, 6) || ApplicationUtil.eq(tel, 8)) {
				List<String> tels = getCampiTelefoniciS(out);
				out.setTelefoniS(tels);
			}
		}
		return out;
	}
	private Contatto getContatto(boolean archiviato, String tipo, Utente user, Contatto in) {
		if(ApplicationUtil.isTrue(impostazioneDao.getSincronia())) {
			DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class);
			criteria.add(Restrictions.idEq(in.getId()));
			Contatto out =  (Contatto) getUniqueResult(criteria);
			if(out != null) {
				return out;
			} 
		}
		return in;
	}
	private Contatto getContattoArchiviato(String tipo, Integer flag, Utente user, Contatto in) {
		LoggerContatto l = new LoggerContatto(impostazioneDao.getLog());
		long time = System.currentTimeMillis();
		Contatto out = getContatto(true, tipo, user, in);
		if (out != null) {
			Integer tel = out.getTipoTelefono();
			if(ApplicationUtil.eq(tel, 6) || ApplicationUtil.eq(tel, 8)) {
				List<String> tels = getCampiTelefoniciS(out);
				out.setTelefoniS(tels);
			}
			Chiamate.getCodaContatti().put(out.getId(), new InfoContatto(time, user, out, true));
			operazioneDao.apriContatto(flag, user, out);
		} 
		l.log("aperto archiviato", user, out);
		return out;
	}
	private Contatto getContattoPrivatoNext(Integer precedenza, Integer flag, String tipo, Utente user, Campagna campagna) {
		Boolean ts = impostazioneDao.getEstrazioneStato();
		LoggerContatto l = new LoggerContatto(impostazioneDao.getLog());
		long time = System.currentTimeMillis();
		DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class);
		DetachedCriteria campagnaC = criteria.createCriteria("campagna");
		if(campagna != null && ApplicationUtil.eq(precedenza, 1)) {
			campagnaC.add(Restrictions.idEq(campagna.getId()));
		}
		if(ApplicationUtil.isNotBlank(tipo)) {
			criteria.createCriteria("anagrafica").add(ApplicationUtil.isNotNull(tipo));
		}
		criteria.add(Restrictions.eq("tipo", ConstantsGlobal.inviato));
		ApplicationUtil.archiviazione(criteria, ApplicationUtil.date(), false);
		criteria.add(Restrictions.eq("proprietario.id",user.getId()));
		ApplicationUtil.criteriaStato(campagnaC, false, ApplicationUtil.isTrue(ts));
		Boolean s = impostazioneDao.getEstrazioneListe();
		if(ApplicationUtil.isTrue(s)) {
			DetachedCriteria listaC = criteria.createCriteria("lista");
			ApplicationUtil.criteriaStato(listaC, false, ApplicationUtil.isTrue(ts));
		}
		Contatto out = getEstrazioneContatto(criteria);
		if (out != null) {
			Chiamate.getCodaContatti().put(out.getId(), new InfoContatto(time, user, out, true));
			operazioneDao.apriContatto(flag, user, out);
		} else {
			if(campagna != null) {
				campagna.criterio(tipo);
			}
		}
		l.log("aperto prossimo privato", user, out);
		return out;
	}
	private Contatto getContattoArchiviatoNext(Integer precedenza, Integer flag, String tipo, Utente user, Campagna campagna) {
		LoggerContatto l = new LoggerContatto(impostazioneDao.getLog());
		long time = System.currentTimeMillis();
		Boolean ts = impostazioneDao.getEstrazioneStato();
		DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class);
		DetachedCriteria campagnaC = criteria.createCriteria("campagna");
		if(campagna != null && ApplicationUtil.eq(precedenza, 1)) {
			campagnaC.add(Restrictions.idEq(campagna.getId()));
		}
		if(ApplicationUtil.isNotBlank(tipo)) {
			criteria.createCriteria("anagrafica").add(ApplicationUtil.isNotNull(tipo));
		}
		criteria.add(Restrictions.eq("tipo", ConstantsGlobal.inviato));
		ApplicationUtil.archiviazione(criteria, ApplicationUtil.date(), true);
		criteria.add(Restrictions.eq("proprietario.id",user.getId()));
		ApplicationUtil.criteriaStato(campagnaC, false, ApplicationUtil.isTrue(ts));
		Boolean s = impostazioneDao.getEstrazioneListe();
		if(ApplicationUtil.isTrue(s)) {
			DetachedCriteria listaC = criteria.createCriteria("lista");
			ApplicationUtil.criteriaStato(listaC, false, ApplicationUtil.isTrue(ts));
		}
		Contatto out = getEstrazioneContatto(criteria);
		if (out != null) {
			Chiamate.getCodaContatti().put(out.getId(), new InfoContatto(time, user, out, true));
			operazioneDao.apriContatto(flag, user, out);
		} else {
			if(campagna != null) {
				campagna.criterio(tipo);
			}
		}
		l.log("aperto prossimo archiviato", user, out);
		return out;
	}
	public void salvaContattoC(boolean w, Integer tipologia, Contatto contatto, Utente utente, EsitoChiamata esitoChiamata, Boolean pannello) {
		esitaContattoC(contatto, esitoChiamata, utente);
		salvaContatto(w, tipologia, contatto, utente, pannello, true);
	}
	public void salvaContattoA(boolean w, Integer tipologia, Contatto contatto, Utente utente, EsitoChiamata esitoChiamata, EsitoArchiviazione esitoArchiviazione, Boolean pannello, Boolean storia) {
		esitaContattoA(contatto, utente, esitoChiamata, esitoArchiviazione);
		salvaContatto(w, tipologia, contatto, utente, pannello, storia);
	}
	
	private void esitaProvvisorioA(Boolean controllo,  boolean w, Integer tipologia, Contatto contatto, Utente utente, EsitoChiamata esitoChiamata, EsitoArchiviazione esitoArchiviazione, Boolean pannello, Boolean storia) {
		if(esitoArchiviazione != null) {
			if(controllo) {
				boolean conta = false;
				if(ApplicationUtil.eq(esitoArchiviazione.getTipologiaS(), 3) && ApplicationUtil.gtZero(esitoArchiviazione.getMax_tentativi())) {
					Integer count = countEsitoArchiviazioneInStoria(contatto.getId(), esitoArchiviazione.getId(), esitoArchiviazione.getConta_tentativi(), utente);
					if(!ApplicationUtil.minore(count, esitoArchiviazione.getMax_tentativi())) {
						conta = true;
					}
				}
				if(conta) {
					EsitoArchiviazione e = campagnaDao.getFullEsitoArchiviazioneById(esitoArchiviazione.getId());
					salvaContattoA(true, tipologia, contatto, utente, esitoChiamata, e.getEsito_collegato() != null ? e.getEsito_collegato() : e, pannello, storia);
					String p = ApplicationUtil.asString(ConstantsLanguage.CONTATTO.stampa() + " " + "[" + ConstantsLanguage.ESITO_ARCHIVIAZIONE.stampa() + ":" + " " + esitoArchiviazione.getId() + "]", contatto);
					operazioneDao.creaOperazione(ConstantsMap.LOG7, p, utente, ConstantsMap.OPERAZIONE_ESITA_IRREPERIBILE);
				} else {
					salvaContattoA(true, tipologia, contatto, utente, esitoChiamata, esitoArchiviazione, pannello, storia);
				}
			} else {
				salvaContattoA(true, tipologia, contatto, utente, esitoChiamata, esitoArchiviazione, pannello, storia);
			}
		}
	}
	
	public void salvaContatto(boolean w, Contatto contatto, Utente utente) {
		if(contatto != null) {
			saveContatto(w, contatto, utente, false, true);
		}
	}
	public void salvaContatto(boolean w, Integer tipologia, Contatto contatto, Utente utente, Boolean pannello, Boolean storia) {
		if(contatto != null) {
			if(tipologia == null) {
				saveContatto(w, contatto, utente, pannello, storia);
			} else {
				LoggerContatto l = new LoggerContatto(impostazioneDao.getLog());
				saveContatto(w, contatto, utente, pannello, storia);
				Chiamate.getCodaContatti().remove(contatto.getId());
				l.log("salvo contatto", utente, contatto);
			}
		}
	}
	private void saveContatto(boolean w, Contatto contatto, Utente utente, Boolean pannello, Boolean s) {
		if(pannello) {
			operazioneDao.esita(utente, contatto);
		}
		Date data = ApplicationUtil.date();
		contatto.setData_lavorazione(data);
		contatto.setUtente(utente);
		try {
			if(w) {
				anagraficaDao.evaluateWorkflow(utente, contatto);
			}
			salvaAggiorna(contatto);
			if(s) {
				if(ApplicationUtil.isDisponibile(contatto.getData_presentazione()) && ApplicationUtil.isTrue(impostazioneDao.storiaImmediata())) {
					StoriaContatto storia = creaStoriaContatto(contatto, utente, data, pannello, false);
					salvaAggiorna(storia);
				} else {
					new SalvaStoriaContattoThread(this, contatto, utente, data, pannello, false).start();
				}
			}
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	@SuppressWarnings("unchecked")
	public  List<DatoAggiuntivoAnagrafica>  getDatiAggiuntiviAnagrafica(Long anagraficaId, Boolean p) {
		DetachedCriteria criteria = DetachedCriteria.forClass(DatoAggiuntivoAnagrafica.class);
		if(ApplicationUtil.isTrue(p)) {
			criteria.createCriteria("campo_aggiuntivo").add(Restrictions.eq("visibile_agente", Boolean.TRUE));
		}
		if(ApplicationUtil.isNotTrue(p)) {
			criteria.createCriteria("campo_aggiuntivo").add(Restrictions.eq("visibile_operatore", Boolean.TRUE));
		}
		criteria.add(Restrictions.eq("anagrafica.id", anagraficaId));
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<DatoAggiuntivoContratto> getDatiAggiuntiviContratto(Long contrattoId, Boolean p) {
		DetachedCriteria criteria = DetachedCriteria.forClass(DatoAggiuntivoContratto.class);
		if(ApplicationUtil.isTrue(p)) {
			criteria.createCriteria("campo_aggiuntivo").add(Restrictions.eq("visibile_agente", Boolean.TRUE));
		}
		if(ApplicationUtil.isNotTrue(p)) {
			criteria.createCriteria("campo_aggiuntivo").add(Restrictions.eq("visibile_operatore", Boolean.TRUE));
		}
		criteria.add(Restrictions.eq("contratto.id",contrattoId));
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<DatoAggiuntivoAnagrafica> getCampiTelefonici(Boolean inline, Boolean p, Contatto contatto) {
		if(contatto != null && ApplicationUtil.isTrue(inline)) {
			DetachedCriteria criteria = DetachedCriteria.forClass(DatoAggiuntivoAnagrafica.class);
			if(ApplicationUtil.isNotTrue(p)) {
				criteria.createCriteria("campo_aggiuntivo").add(Restrictions.eq("visibile_operatore", Boolean.TRUE));
			}
			criteria.add(Restrictions.eq("anagrafica.id",contatto.getAnagrafica().getId()));
			List<DatoAggiuntivoAnagrafica> dati = getList(criteria);
			contatto.setDettagli(dati);
			return dati;
		} 
		return null;
	}
	
	public void preparaContatto(Contatto c, String chS, String chD, String recall_callers, String recall_musicclass, AiutoCampagna script) {
		c.setInbound(true);
		c.setAutodialer(false);
		c.setDestinazione(chD);
		c.setTelefono(chS);
		c.setRecall_callers(recall_callers);
		c.setRecall_musicclass(recall_musicclass);
		c.setScript(script);
		Integer tel = c.getTipoTelefono();
		if(ApplicationUtil.eq(tel, 6) || ApplicationUtil.eq(tel, 8)) {
			List<String> tels = getCampiTelefoniciS(c);
			c.setTelefoniS(tels);
		}
		
	}
	
	@SuppressWarnings("unchecked")
	public List<String> getCampiTelefoniciS(Contatto c) {
		if(c == null) {
			return new ArrayList<String>();
		}
		DetachedCriteria criteria1 = DetachedCriteria.forClass(DatoAggiuntivoAnagrafica.class);
		criteria1.add(Restrictions.eq("anagrafica.id", c.getAnagrafica().getId()));
		criteria1.createCriteria("campo_aggiuntivo").add(Restrictions.eq("tipo", ConstantsMap.TIPO_TELEFONICO));
		criteria1.setProjection(Projections.distinct(Projections.property("valore")));
		return getList(criteria1);
	}
	public int countEsitoChiamataInStoria(Long contattoId, Long esitoChiamata, Boolean conta_tentativi, Utente utente) {
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContatto.class);
		criteria.add(Restrictions.eq("contatto.id",contattoId));
		criteria.add(Restrictions.eq("invisibile", Boolean.FALSE));
		//if(utente != null) {
		//	criteria.add(Restrictions.isNotNull("date_apertura"));
		//}
		if(ApplicationUtil.isTrue(conta_tentativi) && ApplicationUtil.gtZero(esitoChiamata)) {
			criteria.createCriteria("esito").add(Restrictions.idEq(esitoChiamata));
		} else {
			criteria.createCriteria("esito").add(Restrictions.eq("richiamare", Boolean.TRUE));
		}
		return getListSize(criteria);
	}
	
	public int countEsitoArchiviazioneInStoria(Long contattoId, Long esitoArchiviazione, Boolean conta_tentativi, Utente utente) {
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContatto.class);
		criteria.add(Restrictions.eq("contatto.id",contattoId));
		criteria.add(Restrictions.eq("invisibile", Boolean.FALSE));
		//if(utente != null) {
		//	criteria.add(Restrictions.isNotNull("date_apertura"));
		//}
		if(ApplicationUtil.isTrue(conta_tentativi) && ApplicationUtil.gtZero(esitoArchiviazione)) {
			criteria.createCriteria("archiviazione").add(Restrictions.idEq(esitoArchiviazione));
		} else {
			criteria.createCriteria("archiviazione").add(Restrictions.eq("tipologiaS", 3));
		}
		return getListSize(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<EsitoAppuntamento> getAllEsitiAppuntamento() {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoAppuntamento.class);
		ApplicationUtil.orderEsito(criteria);
		return getList(criteria);
	}
	public List<SelectItem> getEsitiAppuntamentoSI(List<EsitoAppuntamento> appuntamenti) {
		List<EsitoAppuntamentoSI> lis = getEsitoAppuntamentoModel(appuntamenti, true);
		return impostazioneDao.getEsitoAppuntamento(lis, false, null);
	}
	public List<SelectItem> getEsitiAppuntamentoSI(Boolean supervisore, List<EsitoAppuntamento> appuntamenti) {
		List<EsitoAppuntamentoSI> lis = getEsitoAppuntamentoModel(appuntamenti, supervisore);
		return impostazioneDao.getEsitoAppuntamento(lis, false, null);
	}
	
	public List<EsitoAppuntamentoSI> getEsitoAppuntamentoModel(List<EsitoAppuntamento> lis, boolean supervisore) {
		List<EsitoAppuntamentoSI> sis = new ArrayList<EsitoAppuntamentoSI>();
		if(ApplicationUtil.notEmpty(lis)) {
			Iterator<EsitoAppuntamento> objIter = lis.iterator();
			while(objIter.hasNext()) {
				EsitoAppuntamento c = objIter.next();
				if(ApplicationUtil.isTrue(supervisore) || ApplicationUtil.isTrue(c.getVisibile())) {
					sis.add(new EsitoAppuntamentoSI(c));
				}
			}
		}
		return sis;
	}
	
	@SuppressWarnings("unchecked")
	public List<EsitoAppuntamento> getEsitiAppuntamentoOperatore() {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoAppuntamento.class);
		//criteria.add(Restrictions.ne("predefinito", Boolean.TRUE));
		criteria.add(Restrictions.eq("impostabile_operatore", Boolean.TRUE));
		ApplicationUtil.orderEsito(criteria);
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<EsitoAppuntamento> getEsitiAppuntamentoOperatoreP() {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoAppuntamento.class);
		criteria.add(Restrictions.eq("visibile", Boolean.TRUE));
		ApplicationUtil.orderEsito(criteria);
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<EsitoAppuntamento> getEsitiAppuntamentoAgente() {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoAppuntamento.class);
		//criteria.add(Restrictions.ne("predefinito", Boolean.TRUE));
		criteria.add(Restrictions.eq("impostabile_agente", Boolean.TRUE));
		ApplicationUtil.orderEsito(criteria);
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<EsitoAppuntamento> getEsitiAppuntamentoAgenteP() {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoAppuntamento.class);
		criteria.add(Restrictions.eq("confermato", Boolean.TRUE));
		ApplicationUtil.orderEsito(criteria);
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<EsitoAppuntamento> getAllEsitiAppuntamento(Boolean supervisore) {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoAppuntamento.class);
		//criteria.add(Restrictions.ne("predefinito", Boolean.TRUE));
		if(!supervisore) {
			criteria.add(Restrictions.eq("impostabile_operatore", Boolean.TRUE));
		}
		ApplicationUtil.orderEsito(criteria);
		return getList(criteria);
	}
	public EsitoAppuntamento getEsitoAppuntamentoPredefinito() {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoAppuntamento.class);
		criteria.add(Restrictions.eq("predefinito", Boolean.TRUE));
		return (EsitoAppuntamento) getUniqueResult(criteria);
	}
	public EsitoAppuntamento getEsitoAppuntamentoByCodice(Long codice) {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoAppuntamento.class);
		criteria.add(Restrictions.idEq(codice));
		return (EsitoAppuntamento) getUniqueResult(criteria);
	}
	@SuppressWarnings("unchecked")
	public  List<ChiamateAttive> getListaChiamateAttive(Long userId, Integer privata) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class);
		criteria.setProjection(Projections.distinct(Projections.property("id")));
		criteria.createCriteria("campagna").createCriteria("team").createCriteria("supervisore").add(Restrictions.idEq(userId));
		List<Long> contatti = getList(criteria);
		List<Long> utentiId = campagnaDao.getIdUtentiBySupervisore(userId);
		return prendiChiamateAttive(contatti, privata, utentiId);
	}
	public  List<ChiamateAttive> getListaChiamateAttive(Integer privata) {
		return prendiChiamateAttive(privata);
	}
	@SuppressWarnings("unchecked")
	public  List<ChiamateAttive> getListaChiamateAttiveCampagna(Utente user, Campagna campagna, Integer privata) {
		List<Long> contattiA = new ArrayList<Long>();
		if(campagna!=null) {
			DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class);
			criteria.setProjection(Projections.distinct(Projections.property("id")));
			DetachedCriteria criteriaC = criteria.createCriteria("campagna");
			criteriaC.add(Restrictions.idEq(campagna.getId()));
			criteriaC.createCriteria("team").createCriteria("supervisore").add(Restrictions.idEq(user.getId()));
			contattiA.addAll(getList(criteria));
		} 
		List<Long> utentiId = campagnaDao.getIdUtentiBySupervisore(user.getId());
		return prendiChiamateAttive(contattiA, privata, utentiId);
	}
	public EsitoChiamata getEsitoChiamataById(Long id) {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoChiamata.class);
		criteria.add(Restrictions.idEq(id));
		return (EsitoChiamata) getUniqueResult(criteria);
	}
	
	public EsitoChiamata getFullEsitoChiamataById(Long id) {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoChiamata.class);
		criteria.add(Restrictions.idEq(id));
		criteria.setFetchMode("esito_collegato", FetchMode.JOIN);
		return (EsitoChiamata) getUniqueResult(criteria);
	}
	public Utente getOperatore(Long id) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		criteria.add(Restrictions.idEq(id));
		return (Utente) getUniqueResult(criteria);
	}
	private DetachedCriteria getRichiamiPrivatiCriteria(String tipo, Long userId) {
		Boolean ts = impostazioneDao.getEstrazioneStato();
		DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class);
		criteria.add(Restrictions.eq("tipo", ConstantsGlobal.inviato));
		criteria.add(Restrictions.eq("proprietario.id",userId));
		DetachedCriteria campagnaC = criteria.createCriteria("campagna");
		ApplicationUtil.criteriaStato(campagnaC, false, ApplicationUtil.isTrue(ts));
		if(ApplicationUtil.isNotBlank(tipo)) {
			criteria.createCriteria("anagrafica").add(ApplicationUtil.isNotNull(tipo));
		}
		Boolean s = impostazioneDao.getEstrazioneListe();
		if(ApplicationUtil.isTrue(s)) {
			DetachedCriteria listaC = criteria.createCriteria("lista");
			ApplicationUtil.criteriaStato(listaC, false, ApplicationUtil.isTrue(ts));
		}
		ApplicationUtil.archiviazione(criteria, ApplicationUtil.date(), false);
		return criteria;
	}
	private DetachedCriteria getRichiamiPrivatiCriteria(ContattoPannelloCriteria cont, String tipo, Long userId) {
		Boolean ts = impostazioneDao.getEstrazioneStato();
		DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class);
		criteria.add(Restrictions.eq("tipo", ConstantsGlobal.inviato));
		criteria.add(Restrictions.eq("proprietario.id",userId));
		DetachedCriteria campagnaC = criteria.createCriteria("campagna");
		if(ApplicationUtil.gtZero(cont.getCampagnaId())) {
			campagnaC.add(Restrictions.idEq(cont.getCampagnaId()));
		} 
		ApplicationUtil.criteriaStato(campagnaC, false, ApplicationUtil.isTrue(ts));
		if(ApplicationUtil.isNotBlank(tipo)) {
			criteria.createCriteria("anagrafica").add(ApplicationUtil.isNotNull(tipo));
		}
		Boolean s = impostazioneDao.getEstrazioneListe();
		if(ApplicationUtil.isTrue(s)) {
			DetachedCriteria listaC = criteria.createCriteria("lista");
			ApplicationUtil.criteriaStato(listaC, false, ApplicationUtil.isTrue(ts));
		}
		ApplicationUtil.archiviazione(criteria, ApplicationUtil.date(), false);
		return criteria;
	}
	@SuppressWarnings("unchecked")
	public List<Contatto> getRichiamiPrivati(ContattoPannelloCriteria cont, String tipo,Long userId,  int start, int size) {
		DetachedCriteria criteria = getRichiamiPrivatiCriteria(cont, tipo, userId);
		ApplicationUtil.setOrdinamento(criteria, cont.getO_tipo(), cont.getO_colonna(), new Order [] {Order.asc("data_presentazione")});
		return getList(criteria, start, size);
	}
	public int getRichiamiPrivatiSize(ContattoPannelloCriteria cont, String tipo,Long userId) {
		DetachedCriteria criteria = getRichiamiPrivatiCriteria(cont, tipo, userId);
		return getListSize(criteria);
	}
	public int getRichiamiPrivatiSize(String tipo,Long userId) {
		DetachedCriteria criteria = getRichiamiPrivatiCriteria(tipo, userId);
		return getListSize(criteria);
	}
	public EsitoArchiviazione getEsitoArchiviazioneById(Long esitoArchiviazioneId) {
		if(esitoArchiviazioneId==null)
			return null;
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoArchiviazione.class);
		criteria.add(Restrictions.idEq(esitoArchiviazioneId));
		return (EsitoArchiviazione) getUniqueResult(criteria);
	}
	public Appuntamento getAppuntamento(Long appuntamentoId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Appuntamento.class);
		criteria.add(Restrictions.idEq(appuntamentoId));
		return (Appuntamento) getUniqueResult(criteria);
	}
	public Appuntamento getAppuntamentoById(Long id) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Appuntamento.class);
		criteria.add(Restrictions.idEq(id));
		return (Appuntamento) getUniqueResult(criteria);
	}
	public void programmaRichiamoPubblicoDaPersonali(Contatto contatto, Utente utente, Date data) {
		EsitoChiamata riprogrammatoPubblico = ConstantsEsito.DEFAULTS.get(ConstantsEsito.ID_RICHIAMO_PUBBLICO);
		contatto.imposta(data, riprogrammatoPubblico, null);
		salvaContatto(true, contatto, utente);
		String p = ApplicationUtil.asString( ConstantsLanguage.CONTATTO.stampa(), contatto);
		operazioneDao.creaOperazione(ConstantsMap.LOG5, p,  utente, ConstantsMap.OPERAZIONE_RIPROGRAMMA_PUBBLICO);
	}
	public void programmaRichiamoPrivatoDaPersonali(Long nuovoUtente, Contatto contatto, Utente utente, Date data) {
		EsitoChiamata riprogrammatoPrivato = ConstantsEsito.DEFAULTS.get(ConstantsEsito.ID_RICHIAMO_PRIVATO);
		Utente proprietario = userDao.getUser(nuovoUtente);
		contatto.imposta(data, riprogrammatoPrivato, proprietario);
		salvaContatto(true, contatto, utente);
		String p = ApplicationUtil.asString( ConstantsLanguage.CONTATTO.stampa(), contatto);
		operazioneDao.creaOperazione(ConstantsMap.LOG5,  p, utente, ConstantsMap.OPERAZIONE_RIPROGRAMMA_PRIVATO);
	}
	public void programmaRichiamoPrivatoDaPersonali(Contatto contatto, Utente utente, Date data) {
		EsitoChiamata riprogrammatoPrivato = ConstantsEsito.DEFAULTS.get(ConstantsEsito.ID_RICHIAMO_PRIVATO);
		contatto.imposta(data, riprogrammatoPrivato, utente);
		salvaContatto(true, contatto, utente);
		String p = ApplicationUtil.asString( ConstantsLanguage.CONTATTO.stampa(), contatto);
		operazioneDao.creaOperazione(ConstantsMap.LOG5, p,  utente, ConstantsMap.OPERAZIONE_RIPROGRAMMA_PRIVATO);
	}
	public void programmaRichiamoArchiviatoDaPersonali(Long nuovoUtente, Contatto contatto, Utente utente, Date data) {
		Utente proprietario = userDao.getUser(nuovoUtente);
		contatto.proprietario(data, proprietario);
		salvaContatto(true, contatto, utente);
		String p = ApplicationUtil.asString( ConstantsLanguage.CONTATTO.stampa(), contatto);
		operazioneDao.creaOperazione(ConstantsMap.LOG5, p,  utente, ConstantsMap.OPERAZIONE_RIPROGRAMMA_ARCHIVIATO);
	}
	public void programmaRichiamoArchiviatoDaPersonali(Contatto contatto, Utente utente, Date data) {
		contatto.proprietario(data, utente);
		salvaContatto(true, contatto, utente);
		String p = ApplicationUtil.asString( ConstantsLanguage.CONTATTO.stampa(), contatto);
		operazioneDao.creaOperazione(ConstantsMap.LOG5,  p, utente, ConstantsMap.OPERAZIONE_RIPROGRAMMA_ARCHIVIATO);
	}
	public void programmaRichiamo(int a, Long nuovoUtente, Integer tipologia, Contatto contatto, Utente utente, Date data) {
		if(ApplicationUtil.eq(a, 1)) {
			programmaRichiamoPubblico(tipologia, contatto, utente, data);
		} else if(ApplicationUtil.eq(a, 2)) {
			programmaRichiamoPrivato(tipologia, nuovoUtente, contatto, utente, data);
		} else {
			programmaRichiamoArchiviato(tipologia, nuovoUtente, contatto, utente, data);
		}
	}
	private void programmaRichiamoPubblico(Integer tipologia, Contatto contatto, Utente utente, Date data) {
		EsitoChiamata esito = ConstantsEsito.DEFAULTS.get(ConstantsEsito.ID_RICHIAMO_PUBBLICO);
		contatto.imposta(data, esito, null);
		salvaContattoC(true, tipologia, contatto, utente, esito, true);
	}
	private void programmaRichiamoPrivato(Integer tipologia, Long nuovoUtente, Contatto contatto, Utente utente,  Date data) {
		EsitoChiamata esito = ConstantsEsito.DEFAULTS.get(ConstantsEsito.ID_RICHIAMO_PRIVATO);
		Utente proprietario = userDao.getUser(nuovoUtente);
		contatto.imposta(data, esito, proprietario);
		salvaContattoC(true, tipologia, contatto, utente, esito, true);
	}
	private void programmaRichiamoArchiviato(Integer tipologia, Long nuovoUtente, Contatto contatto, Utente utente,  Date data) {
		Utente proprietario = userDao.getUser(nuovoUtente);
		contatto.proprietario(data, proprietario);
		salvaContattoA(true, tipologia, contatto, utente, contatto.getEsitoChiamata(), contatto.getEsitoArchiviazione(), true, true);
	}
	public void esitaInviato(Integer tipologia, Contatto contatto, Utente utente) {
		salvaContatto(true, tipologia, contatto, utente, true, true);
	}
	public void esitaProvvisorio(Boolean controllo, Integer tipologia, Contatto contatto, Utente user, EsitoChiamata esito, boolean pannello) {
		if(esito != null) {
			if(controllo) {
				boolean conta = false;
				if(ApplicationUtil.isTrue(esito.getRichiamare()) && ApplicationUtil.gtZero(esito.getMax_tentativi())) {
					Integer count = countEsitoChiamataInStoria(contatto.getId(), esito.getId(), esito.getConta_tentativi(), user);
					if(!ApplicationUtil.minore(count, esito.getMax_tentativi())) {
						conta = true;
					}
				}
				if(conta) {
					EsitoChiamata e = getFullEsitoChiamataById(esito.getId());
					salvaContattoC(true, tipologia, contatto, user,  e.getEsito_collegato() != null ? e.getEsito_collegato() : e, pannello);
					String p = ApplicationUtil.asString(ConstantsLanguage.CONTATTO.stampa() + " " + "[" + ConstantsLanguage.ESITO_CHIAMATA.stampa() + ":" + " " + esito.getId() + "]", contatto);
					operazioneDao.creaOperazione(ConstantsMap.LOG7, p, user, ConstantsMap.OPERAZIONE_ESITA_IRREPERIBILE);
				} else {
					salvaContattoC(true, tipologia, contatto, user,  esito, pannello);
				}
				
			} else {
				salvaContattoC(true, tipologia, contatto, user, esito, pannello);
			}
		}
	}
	public void esitaAutomatico(Boolean controllo, Integer tipologia, Contatto contatto, Utente user, boolean pannello) {
		if(contatto != null && contatto.isPubblica()) {
			EsitoChiamata esito = contatto.getCampagna().getEsito();
			esitaProvvisorio(controllo, tipologia, contatto, user, esito, pannello);
		}
	}
	public void esitaCampagna(Contatto contatto, Utente user) {
		if(contatto != null) {
			EsitoChiamata esito = null;
			if(contatto.isPubblica()) {
				esito = contatto.getCampagna().getEsito();
				esitaProvvisorio(true, contatto.getTipologia(), contatto, user, esito, true);
			}
			if(esito == null) {
				lasciaContatto(user, contatto);
			}
		}
	}
	public void esitaAutodialer(boolean w, Integer save, Contatto contatto, Utente user, EsitoChiamata esito) {
		if(ApplicationUtil.eq(save, 2)) {
			boolean conta = false;
			if(ApplicationUtil.isTrue(esito.getRichiamare()) && ApplicationUtil.gtZero(esito.getMax_tentativi())) {
				Integer count = countEsitoChiamataInStoria(contatto.getId(), esito.getId(), esito.getConta_tentativi(), user);
				if(!ApplicationUtil.minore(count, esito.getMax_tentativi())) {
					conta = true;
				}
			}
			if(conta) {
				EsitoChiamata e = getFullEsitoChiamataById(esito.getId());
				salvaContattoAuto(w, contatto, user,  e.getEsito_collegato() != null ? e.getEsito_collegato() : e, save);
				String p = ApplicationUtil.asString(ConstantsLanguage.CONTATTO.stampa() + " " + "[" + ConstantsLanguage.ESITO_CHIAMATA.stampa() + ":" + " " + esito.getId() + "]", contatto);
				operazioneDao.creaOperazione(ConstantsMap.LOG7, p , user, ConstantsMap.OPERAZIONE_ESITA_IRREPERIBILE);
			} else {
				salvaContattoAuto(w, contatto, user,  esito, save);
			}
			
		} else {
			salvaContattoAuto(w, contatto, user, esito, save);
		}
	}
	public void salvaContattoAuto(boolean w, Contatto contatto, Utente utente, EsitoChiamata esitoChiamata, Integer save) {
		LoggerContatto l = new LoggerContatto(impostazioneDao.getLog());
		esitaContattoC(contatto, esitoChiamata, utente);
		Date data = ApplicationUtil.date();
		if(ApplicationUtil.eq(save, 1) || ApplicationUtil.eq(save, 2)) {
			contatto.setData_lavorazione(data);
			contatto.setUtente(utente);
		}
		try {
			if(w) {
				anagraficaDao.evaluateWorkflow(utente, contatto);
			}
			salvaAggiorna(contatto);
			if(ApplicationUtil.eq(save, 1) || ApplicationUtil.eq(save, 2)) {
				StoriaContatto storia = creaStoriaContatto(contatto, utente, data, false, false);
				salvaAggiorna(storia);
			}
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
		l.log("salvo autodialer", utente, contatto);
	}
	public StoriaContatto creaStoriaContatto(Contatto contatto, Utente utente, Date data, Boolean pannello, Boolean invisibile) {
		
		if(utente != null) {
			Campagna campagna = contatto.getCampagna();
			if (ApplicationUtil.isTrue(campagna.getProprietario_messaggio())) {
				new ProprietarioMessaggioThread(contatto, messaggioDao, downloadDao, utente).start();
			}
			if (ApplicationUtil.isTrue(campagna.getProprietario_notizia())) {
				new ProprietarioNotiziaThread(contatto, notiziaDao, downloadDao, utente).start();
			}
			if (ApplicationUtil.isTrue(campagna.getAgente_messaggio())) {
				new AgenteMessaggioThread(contatto, messaggioDao, downloadDao, userDao, utente).start();
			}
			if (ApplicationUtil.isTrue(campagna.getAgente_notizia())) {
				new AgenteNotiziaThread(contatto, notiziaDao, downloadDao, userDao, utente).start();
			}
		}
		StoriaContatto storia = new StoriaContatto();
		storia.setData(data);
		storia.setSequenziale(contatto.getSequenziale());
		storia.setBloccato(contatto.getBloccato());
		storia.setContatto(contatto);
		storia.setOperatore(utente);
		storia.setNota(contatto.getNota());
		storia.setCampagna(contatto.getCampagna());
		storia.setLista(contatto.getLista());
		storia.setTipo(contatto.getTipo());
		storia.setEsito(contatto.getEsitoChiamata());
		storia.setArchiviazione(contatto.getEsitoArchiviazione());
		storia.setArchiviato(contatto.getArchiviato());
		storia.setData_archiviazione(contatto.getData_archiviazione());
		storia.setData_presentazione(contatto.getData_presentazione());
		storia.setProprietario(contatto.getProprietario());
		storia.setAgente(contatto.getAgente());
		storia.setInvisibile(invisibile);
		storia.impostaUserfield();
		if(pannello) {
			Date ts_apertura = contatto.getTimestamp_apertura();
			Date ts_chiamata = contatto.getTimestamp_chiamata();
			Date ts_risposta = contatto.getTimestamp_risposta();
			Date ts_fine = contatto.getTimestamp_fine();
			Date ts_esito = contatto.getTimestamp_esito();
			Date ts_rec_inizio = contatto.getTimestamp_rec_inizio();
			Date ts_rec_fine = contatto.getTimestamp_rec_fine();
			Date ts_attesa_inizio = contatto.getTimestamp_attesa_inizio();
			Date ts_attesa_fine = contatto.getTimestamp_attesa_fine();
			storia.setSecondi_pre(ApplicationUtil.impostaTempo(ts_chiamata, ts_apertura));
			storia.setSecondi_conv_tot(ApplicationUtil.impostaTempo(ts_fine, ts_chiamata));
			storia.setSecondi_conv_eff(ApplicationUtil.impostaTempo(ts_fine, ts_risposta));
			storia.setSecondi_rec(ApplicationUtil.impostaTempo(ts_rec_fine, ts_rec_inizio));
			storia.setSecondi_post(ApplicationUtil.impostaTempo(ts_esito, ts_fine));
			storia.setSecondi_tot(ApplicationUtil.impostaTempo(ts_esito, ts_apertura));
			storia.setSecondi_attesa(ApplicationUtil.impostaTempo(ts_attesa_fine, ts_attesa_inizio));
			storia.setDate_apertura(ts_apertura);
			storia.setDate_chiamata(ts_chiamata);
			storia.setDate_risposta(ts_risposta);
			storia.setDate_rec_inizio(ts_rec_inizio);
			storia.setDate_rec_fine(ts_rec_fine);
			storia.setDate_fine(ts_fine);
			storia.setDate_esito(ts_esito);
			storia.setDate_attesa_inizio(ts_attesa_inizio);
			storia.setDate_attesa_fine(ts_attesa_fine);
			if(utente != null) {
				Chiamate.getCodaEsiti().add(utente, contatto);
				Chiamate.getCodaStatistiche().get(contatto.getCampagna()).incContatti();
				if(ts_risposta != null) {
					Chiamate.getCodaStatistiche().get(contatto.getCampagna()).incConversazioni();
				}
				if(ApplicationUtil.isTrue(contatto.getArchiviato())) {
					Chiamate.getCodaStatistiche().get(contatto.getCampagna()).incArchiviazioni();
				}
			}
			if(contatto.getAutodialer() || contatto.getInbound()) {
				AutoContatto auto = contatto.getAutoContatto(); 
				if(auto != null) {
					auto.setStato(ConstantsMap.CHIAMATA_ESITATA);
					CodaManager codaManager = Chiamate.getCodaProcessi().get(contatto.getCampagna());
					if(codaManager != null) {
						codaManager.getAutodialerStatistiche().incEsitate(data);
						codaManager.getAutodialerStatistiche().addTimeGestione(ts_apertura, ts_esito);
						codaManager.getAutodialerContatti().incEsitate(auto, data);
					}
					CodaStatic.getAutodialerStatistiche().incEsitate(data);
					CodaStatic.getAutodialerStatistiche().addTimeGestione(ts_apertura, ts_esito);
					CodaStatic.getAutodialerContatti().incEsitate(auto, data);
				}
			}
		}
		return storia;
	}
	public void esitaContattoC(Contatto contatto, EsitoChiamata esitoChiamata, Utente utente) {
		if(esitoChiamata != null) {
			Date data = ApplicationUtil.date();
			if(!esitoChiamata.getId().equals(ConstantsEsito.ID_DA_CHIAMARE)) {
				boolean isDaRichiamare = ApplicationUtil.isTrue(esitoChiamata.getRichiamare());
				if (isDaRichiamare) {
					contatto.imposta(ApplicationUtil.addMinutes(data, esitoChiamata.getAttesa()), esitoChiamata, null);
				} 
				boolean isNotDaRichiamare = ApplicationUtil.isNotTrue(esitoChiamata.getRichiamare());
				if (isNotDaRichiamare) {
					contatto.archivia(esitoChiamata);
					if(ApplicationUtil.gtZero(esitoChiamata.getInvia_opposizione())) {
						if (ApplicationUtil.eq(esitoChiamata.getInvia_opposizione(), 1)) {
							new SettaOpposizioneThread(false, this, contatto, utente).start();
						}
						if (ApplicationUtil.eq(esitoChiamata.getInvia_opposizione(), 2)) {
							new SettaOpposizioneThread(true, this, contatto, utente).start();
						}
					}
				}
			} else {
				contatto.imposta(data, esitoChiamata, null);
			}
		}
	}
	public void esitaContattoA(Contatto contatto, Utente utente, EsitoChiamata esitoChiamata, EsitoArchiviazione esitoArchiviazione) {
		if(esitoChiamata != null) {
			contatto.setEsitoChiamata(esitoChiamata);
			contatto.setEsitoArchiviazione(esitoArchiviazione);
			if (esitoArchiviazione != null && ApplicationUtil.gtZero(esitoArchiviazione.getTipologiaS())) {
				if (ApplicationUtil.eq(esitoArchiviazione.getTipologiaS(), 1)) {
					contatto.archivia(true);
				}
				if (ApplicationUtil.eq(esitoArchiviazione.getTipologiaS(), 2)) {
					contatto.pubblico();
				}
				if (ApplicationUtil.eq(esitoArchiviazione.getTipologiaS(), 3)) {
					Date data = ApplicationUtil.date(); 
					contatto.proprietario(ApplicationUtil.addMinutes(data, esitoArchiviazione.getAttesa()), null);
					contatto.pubblico();
				}
			} else {
				contatto.archivia(false);
				if(esitoArchiviazione != null && ApplicationUtil.gtZero(esitoArchiviazione.getInvia_opposizione())) {
					if (ApplicationUtil.eq(esitoArchiviazione.getInvia_opposizione(), 1)) {
						new SettaOpposizioneThread(false, this, contatto, utente).start();
					}
					if (ApplicationUtil.eq(esitoArchiviazione.getInvia_opposizione(), 2)) {
						new SettaOpposizioneThread(true, this, contatto, utente).start();
					}
				}
			}
		}
	}
	
	public void setCampagnaPannello(Contatto contatto, Campagna campagna) {
		if(contatto != null && campagna == null) {
			campagna = contatto.getCampagna();
		}
	}
	public Contatto getContattoPannello(String tipo, Integer flag, Utente utente, Campagna campagna, Contatto contatto, Boolean nuovo) {
		switch (flag) {
		case ConstantsGlobal.ID_SEZIONE_PRIVATE:
			return getContattoDaRichiamiPrivati(tipo, utente, contatto);
		case ConstantsGlobal.ID_SEZIONE_ARCHIVIATE:
			return getContattoDaRichiamiArchiviati(tipo, utente, contatto);
		default:
			return getContattoDaCampagna(tipo, utente, campagna, nuovo);
		}
	}
	
	public Contatto getProssimoContattoPannello(String tipo, Integer flag, Utente utente, Campagna campagna, Boolean nuovo) {
		Integer precedenza = impostazioneDao.getPrecedenza_personali();
		if(ApplicationUtil.gtZero(precedenza)) {
			Contatto out =  getProssimoContattoPrivato(precedenza, tipo, utente, campagna);
			if(out==null) {
				out = getProssimoContattoArchiviato(precedenza, tipo, utente, campagna);
			}
			if(out==null) {
				out = getContattoDaCampagna(tipo, utente, campagna, nuovo);
			}
			return out;
		} else {
			switch (flag) {
			case ConstantsGlobal.ID_SEZIONE_PRIVATE:
				return getProssimoContattoPrivato(null, tipo, utente, campagna);
			case ConstantsGlobal.ID_SEZIONE_ARCHIVIATE:
				return getProssimoContattoArchiviato(null, tipo, utente, campagna);
			default:
				return getContattoDaCampagna(tipo, utente, campagna, nuovo);
			}
		}
	}
	public Contatto getProssimoContattoPannello(String tipo, Utente utente, Boolean auto, Campagna campagna) {
		if(ApplicationUtil.isNotTrue(auto)) {
			return null;
		}
		Integer precedenza = impostazioneDao.getPrecedenza_personali();
		Contatto out =  getProssimoContattoPrivato(precedenza, tipo, utente, campagna);
		if(out==null) {
			out = getProssimoContattoArchiviato(precedenza, tipo, utente, campagna);
		}
		return out;
	}
	public Contatto getContattoDaCampagna(String tipo, Utente user, Campagna campagna, Boolean nuovo) {
		Contatto out = null;
		if(campagna!=null) {
			out = getContattoPubblico(ConstantsGlobal.ID_SEZIONE_PUBBLICHE, tipo, user, campagna, nuovo); 
			if(out!=null) {
				out.setTipologia(ConstantsGlobal.ID_SEZIONE_PUBBLICHE);
			}
		}
		return out;
	}
	public Contatto getProssimoContattoPrivato(Integer precedenza, String tipo, Utente user, Campagna campagna) {
		Contatto out = getContattoPrivatoNext(precedenza, ConstantsGlobal.ID_SEZIONE_PRIVATE, tipo, user, campagna); 
		if(out!=null) {
			out.setTipologia(ConstantsGlobal.ID_SEZIONE_PRIVATE);
		}
		return out;
	}
	public Contatto getProssimoContattoArchiviato(Integer precedenza,String tipo, Utente user, Campagna campagna) {
		Contatto out = getContattoArchiviatoNext(precedenza, ConstantsGlobal.ID_SEZIONE_ARCHIVIATE, tipo, user, campagna); 
		if(out!=null) {
			out.setTipologia(ConstantsGlobal.ID_SEZIONE_ARCHIVIATE);
		}
		return out;
	}
	public Contatto getContattoDaRichiamiPrivati(String tipo, Utente user, Contatto contatto) {
		Contatto out = null;
		if(contatto!=null) {
			out = getContattoPrivato(tipo, ConstantsGlobal.ID_SEZIONE_PRIVATE, user, contatto); 
			if(out!=null) {
				out.setTipologia(ConstantsGlobal.ID_SEZIONE_PRIVATE);
			}
		}
		return out;
	}
	public Contatto getContattoDaRichiamiArchiviati(String tipo, Utente user, Contatto contatto) {
		Contatto out = null;
		if(contatto!=null) {
			out = getContattoArchiviato(tipo, ConstantsGlobal.ID_SEZIONE_ARCHIVIATE, user, contatto); 
			if(out!=null) {
				out.setTipologia(ConstantsGlobal.ID_SEZIONE_ARCHIVIATE);
			}
		}
		return out;
	}
	public void eliminaNota(Utente utente, Nota nota) {
		String p = ApplicationUtil.asString( ConstantsLanguage.NOTA.stampa(), nota);
		operazioneDao.creaOperazione(ConstantsMap.LOG3, p, utente,ConstantsMap.OPERAZIONE_ELIMINA_NOTA);
		cancellaAggiorna(nota);
	}
	public void salvaNota(Nota nota, Utente utente) {
		Date data = ApplicationUtil.date();
		nota.setData_lavorazione(data);
		nota.setUtente(utente);
		salvaAggiorna(nota);
		new SalvaStoriaNotaThread(this, nota, utente, data).start();
	}
	@SuppressWarnings("unchecked")
	public List<StoriaNota> getStoriaNota(Nota nota) {
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaNota.class);
		criteria.add(Restrictions.eq("nota.id", nota.getId()));
		criteria.addOrder(Order.asc("data"));
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<StoriaAppuntamento> getStoriaAppuntamento(Appuntamento appuntamento) {
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaAppuntamento.class);
		criteria.add(Restrictions.eq("appuntamento.id", appuntamento.getId()));
		criteria.addOrder(Order.asc("data"));
		criteria.setFetchMode("appuntamento", FetchMode.JOIN);
		return getList(criteria);
	}
	public void salvaAppuntamento(Appuntamento app, Utente utente) {
		salvaAppuntamento(app, utente, true);
	}
	public void salvaAppuntamento(Appuntamento appuntamento, Utente utente, boolean f) {
		Date data = ApplicationUtil.date();
		boolean modificato = modificaEsitoStatoAppuntamento(appuntamento, data, f);
		appuntamento.setData_lavorazione(data);
		appuntamento.setUtente(utente);
		appuntamento.impostaUserfield();
		boolean inserito = (appuntamento.getId() == null);
		if(inserito) {
			appuntamento.setData_esito(data);
			appuntamento.setData_stato(data);
		}
		salvaAggiorna(appuntamento);
		new SalvaStoriaAppuntamentoThread(this, appuntamento, utente, data, inserito, modificato).start();
	}
	public void salvaStoriaAppuntamento(Appuntamento appuntamento, Utente utente, Date data, boolean inserito, boolean modificato) {
		StoriaAppuntamento storia = creaStoriaAppuntamento(appuntamento, utente, data);
		salvaAggiorna(storia);
		if((modificato || inserito) && appuntamento.getEsito() != null) {
			if (ApplicationUtil.isNotBlank(appuntamento.getEsito().getMessaggio_notifica()) || ApplicationUtil.isTrue(appuntamento.getEsito().getMessaggio_operatore()) || ApplicationUtil.isTrue(appuntamento.getEsito().getMessaggio_agente()) ) {
				new AppuntamentoMessaggioThread(appuntamento, messaggioDao, downloadDao, userDao, utente).start();
			}
			if (ApplicationUtil.isNotBlank(appuntamento.getEsito().getNotizia_notifica()) || ApplicationUtil.isTrue(appuntamento.getEsito().getNotizia_operatore()) || ApplicationUtil.isTrue(appuntamento.getEsito().getNotizia_agente()) ) {
				new AppuntamentoNotiziaThread(appuntamento, notiziaDao, downloadDao, userDao, utente).start();
			}
		}
	}
	public void salvaAppuntamentoE(Appuntamento appuntamento, Utente utente, boolean f) {
		Date data = ApplicationUtil.date();
		boolean modificato = modificaEsitoStatoAppuntamento(appuntamento, data, f);
		appuntamento.setData_lavorazione(data);
		appuntamento.setUtente(utente);
		appuntamento.impostaUserfield();
		boolean inserito = (appuntamento.getId() == null);
		if(inserito) {
			appuntamento.setData_esito(data);
			appuntamento.setData_stato(data);
		}
		salvaAggiorna(appuntamento);
		StoriaAppuntamento storia = creaStoriaAppuntamento(appuntamento, utente, data);
		salvaAggiorna(storia);
		if((modificato || inserito) && appuntamento.getEsito() != null) {
			if (ApplicationUtil.isNotBlank(appuntamento.getEsito().getMessaggio_notifica()) || ApplicationUtil.isTrue(appuntamento.getEsito().getMessaggio_operatore()) || ApplicationUtil.isTrue(appuntamento.getEsito().getMessaggio_agente()) ) {
				new AppuntamentoMessaggioThread(appuntamento, messaggioDao, downloadDao, userDao, utente).start();
			}
			if (ApplicationUtil.isNotBlank(appuntamento.getEsito().getNotizia_notifica()) || ApplicationUtil.isTrue(appuntamento.getEsito().getNotizia_operatore()) || ApplicationUtil.isTrue(appuntamento.getEsito().getNotizia_agente()) ) {
				new AppuntamentoNotiziaThread(appuntamento, notiziaDao, downloadDao, userDao, utente).start();
			}
		}
	}
	public boolean modificaEsitoStatoAppuntamento(Appuntamento  appuntamento, Date data, boolean f) {
		boolean modificato = false;
		if(appuntamento!=null && appuntamento.getId() != null && f) {
			Appuntamento crt = (Appuntamento) get(Appuntamento.class, appuntamento.getId());
			if(crt!=null) {
				int esito = ApplicationUtil.compareTo(appuntamento.getEsito() != null ? appuntamento.getEsito().getId() : null, crt.getEsito() != null ? crt.getEsito().getId() : null);
				if (esito != 0) {
					appuntamento.setData_esito(data);
					modificato = true;
				}
				int stato = ApplicationUtil.compareTo(appuntamento.getStato(), crt.getStato());
				if (stato != 0) {
					appuntamento.setData_stato(data);
					modificato = true;
				}
			}
		}
		return modificato;
	}
	public void salvaAppuntamentoNuovo(Appuntamento app, Utente utente)  {
		if(app != null) {
			if(ApplicationUtil.isToSave(app.getId())) {
				salvaAppuntamento(app, utente, true);
				salvaAllegati(app);
			}
		}
	}
	public void salvaAllegati(Appuntamento app)  {
		try {
			List<Allegato> allegati = app.getAllegato();
			if(ApplicationUtil.notEmpty(allegati)) {
				String s = impostazioneDao.getPathBaseW(ConstantsGlobal.path_appuntamento);
				Iterator<Allegato> allIt = allegati.iterator();
				while (allIt.hasNext()) {
					Allegato all = (Allegato) allIt.next();
					AllegatoAppuntamento allegato = new AllegatoAppuntamento();
					allegato.setAppuntamento(app);
					allegato.setDimensione(all.getDimensione());
					allegato.setNome(ApplicationUtil.getNameN(all.getNome()));
					allegato.setContentFile(all.getFile(), s);
					salvaAggiorna(allegato);
				}
			}
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void salva(Pausa pausa, Utente user, ConfigPausa p) {
		if(user!= null && ApplicationUtil.ne(user.getDisattivaFunzioni(), 2)) {
			Date adesso = ApplicationUtil.date();
			pausa.setChiave(p.getChiave());
			pausa.setPositivo(p.getPositivo());
			pausa.setData_creazione(ApplicationUtil.inizioGiornata(adesso));
			pausa.setUser(user);
			pausa.setData_fine(adesso);
			salvaAggiorna(pausa);
		}
	}
	public Appuntamento nuovoAppuntamento(Boolean diretto, String ora, Long angenteId, Date data, Contatto contatto) {
		Appuntamento app = new Appuntamento();
		app.setMinTime(impostazioneDao.getMinTimeAppuntamento());
		app.setData_appuntamento(data);
		app.setOraInizio(ora);
		Agente agente = campagnaDao.getAgente(angenteId);
		app.setAgente(agente);
		if(ApplicationUtil.isFalse(diretto)) {
			app.setId(contatto.getMappaId());
		}
		return app;
	}
	public Appuntamento nuovoAppuntamento(Contatto contatto) {
		Integer giorni = impostazioneDao.getGiorniAppuntamenti();
		Appuntamento app = contatto.getAppuntamento(Long.valueOf (0));
		if(app != null) {
			return app;
		}
		app = new Appuntamento();
		app.setMinTime(impostazioneDao.getMinTimeAppuntamento());
		if(giorni != null) {
			app.setData_appuntamento(ApplicationUtil.addDays(ApplicationUtil.date(), giorni));
		} else {
			app.setData_appuntamento(ApplicationUtil.date());
		}
		app.setEsito(getEsitoAppuntamentoPredefinito());
		app.setOraInizio(null);
		app.setId(Long.valueOf(0));
		return app;
	}
	public Nota nuovoNota(String ora, Long angenteId, Date data) {
		Nota nota = new Nota();
		nota.setMinTime(impostazioneDao.getMinTimeNota());
		nota.setData_nota(data);
		nota.setOraInizio(ora);
		Agente agente = campagnaDao.getAgente(angenteId);
		nota.setAgente(agente);
		return nota;
	}
	public void eliminaAppuntamento(Utente utente, Appuntamento appuntamento) {
		if (appuntamento != null ) {
			String p = ApplicationUtil.asString( ConstantsLanguage.APPUNTAMENTO.stampa(), appuntamento);
			operazioneDao.creaOperazione(ConstantsMap.LOG3, p, utente, ConstantsMap.OPERAZIONE_ELIMINA_APPUNTAMENTO);
			cancellaAggiorna(appuntamento);
		} 
	}
	public void bloccaAppuntamento(Utente utente, Appuntamento appuntamento, boolean blocca) {
		if (appuntamento != null ) {
			appuntamento.setBloccato(blocca);
			salvaAppuntamento(appuntamento, utente, false);
		} 
	}
	public void bloccaAppuntamentiSelezionati(Utente utente, List<Appuntamento> appuntamenti, boolean blocca) {
		if (ApplicationUtil.notEmpty(appuntamenti)) {
			Iterator<Appuntamento> it = appuntamenti.iterator();
			while (it.hasNext()) {
				Appuntamento appuntamento = (Appuntamento) it.next();
				bloccaAppuntamento(utente, appuntamento, blocca);
			}
		} 
	}
	public void bloccaNoteSelezionati(Utente utente, List<Nota> note, boolean blocca) {
		if (ApplicationUtil.notEmpty(note)) {
			Iterator<Nota> it = note.iterator();
			while (it.hasNext()) {
				Nota nota = (Nota) it.next();
				bloccaNota(utente, nota, blocca);
			}
		} 
	}
	public void bloccaNota(Utente utente, Nota nota, boolean blocca) {
		if (nota != null ) {
			nota.setBloccato(blocca);
			salvaNota(nota, utente);
		} 
	}
	public String azione(Utente utente, String transizione) {
		if(ApplicationUtil.isNotBlank(transizione) && ConstantsMap.POPUP.contains(transizione)) {
			operazioneDao.settaPannello(utente, null, transizione);
		} else {
			operazioneDao.settaPannello(utente, null, null);
		}
		return transizione;
	}
	public Transizione prossima(Utente utente, EsitoArchiviazione esito, String transizione) {
		if(ApplicationUtil.isNotBlank(transizione) && ConstantsMap.POPUP.contains(transizione)) {
			operazioneDao.settaPannello(utente, esito, transizione);
		} else {
			operazioneDao.settaPannello(utente, null, null);
		}
		return new Transizione(transizione);
	}
	public String precedente(Utente utente, EsitoArchiviazione esito, String transizione) {
		if(ApplicationUtil.isNotBlank(transizione) && ConstantsMap.POPUP.contains(transizione)) {
			operazioneDao.settaPannello(utente, esito, transizione);
		} else {
			operazioneDao.settaPannello(utente, null, null);
		}
		return transizione;
	}
	public Transizione prossimaTransizione(Contatto contatto, Utente utente, EsitoArchiviazione esito, String pannelloModale) {
		String prossima = ConstantsMap.sceltaFinale;
		boolean b = ApplicationUtil.equals(pannelloModale, prossima);
		if (esito != null && ApplicationUtil.isNotBlank(pannelloModale) && !b) {
			List<String> t = contatto.getCampagna().getTransizioniList();
			int index = t.indexOf(pannelloModale); 
			int ni = index+1;
			if(ni >= 0 && ni < t.size()) {
				prossima = t.get(ni);
				if(ApplicationUtil.getMethod(esito, prossima)) {
					return prossima(utente, esito, prossima);
				} else {
					return prossimaTransizione(contatto, utente, esito, prossima);
				}
			}
		} 
		return prossima(utente, esito, prossima);
	}
	public String tornaIndietro(Contatto contatto, Utente utente, EsitoArchiviazione esito, String pannelloModale) {
		String precedente = ConstantsMap.sceltaChiudi;
		if (esito != null && ApplicationUtil.isNotBlank(pannelloModale)) {
			if (ApplicationUtil.equals(pannelloModale, ConstantsMap.sceltaData)) {
				if(!impostazioneDao.getEsitiInline()) {
					return precedente(utente, esito, ConstantsMap.sceltaOk);
				}
			} else {
				List<String> t = contatto.getCampagna().getTransizioniList();
				int index = t.indexOf(pannelloModale); 
				int ni = index-1;
				if(ni >= 0 && ni < t.size()) {
					precedente = t.get(ni);
					if(ApplicationUtil.getMethod(esito, precedente)) {
						return precedente(utente, esito, precedente);
					} else {
						return tornaIndietro(contatto, utente, esito, precedente);
					}
				}
			}
		}
		return precedente(utente, esito, precedente);
	}
	public Boolean validaContratto(Map<ProdottoSelezionato, ContrattoProdotto> lp) {
		List<String> duplicato = new ArrayList<String>(); 
		if(ApplicationUtil.notEmpty(lp)) {
			Iterator<ProdottoSelezionato> it = lp.keySet().iterator();
			while (it.hasNext()) {
				ProdottoSelezionato p = (ProdottoSelezionato) it.next();
				ContrattoProdotto c = lp.get(p);
				String seriale = ApplicationUtil.trimToNull(c.getSeriale());
				if(seriale != null) {
					if(!duplicato.contains(seriale)) {
						duplicato.add(seriale);
					} else {
						return true;
					}
				}
			}
		}
		return false;
	}
	public List<ProdottoCheckbox> getProdottiInVendita(EsitoArchiviazione esito, Contatto contatto) {
		List<ProdottoCheckbox> prodottic = new ArrayList<ProdottoCheckbox>();
		if(ApplicationUtil.eq(esito.getContratto_imposta(), 3) || ApplicationUtil.eq(esito.getContratto_imposta(), 4) || ApplicationUtil.eq(esito.getContratto_imposta(), 5)) {
			List<Contratto> contratti = getContrattiByContatto(contatto.getId());
			Iterator<Contratto> objIter = contratti.iterator();
			while(objIter.hasNext()) {
				Contratto contratto = objIter.next();
				ProdottoCheckbox prd = new ProdottoCheckbox(contratto);
				prodottic.add(prd);
			}
		} else {
			List<Prodotto> prodotti = getProdottiByCampagna(contatto.getCampagna().getId());
			Iterator<Prodotto> objIter = prodotti.iterator();
			while(objIter.hasNext()) {
				Prodotto prodotto = objIter.next();
				ProdottoCheckbox prd = new ProdottoCheckbox(prodotto);
				prodottic.add(prd);
			}
		}
		return prodottic;
	}
	@SuppressWarnings("unchecked")
	public List<Contratto> getContrattiByContatto(Long contattoId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Contratto.class);
		criteria.add(Restrictions.eq("contatto.id", contattoId));
		List<Contratto> contratti =  getList(criteria);
		if(contratti==null)
			contratti = new ArrayList<Contratto>();
		return contratti;
	}
	@SuppressWarnings("unchecked")
	public List<Prodotto> getProdottiByCampagna(Long campagnaId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Prodotto.class);
		criteria.add(Restrictions.eq("attivo", Boolean.TRUE));
		DetachedCriteria criteriaV = criteria.createCriteria("prodotti_in_vendita");
		criteriaV.add(Restrictions.eq("id.campagna.id", campagnaId));
		criteria.createCriteria("categoria", "categoria", ApplicationUtil.leftJoin());
		ApplicationUtil.orderP(criteria);
		List<Prodotto> prodotti =  getList(criteria);
		if(prodotti==null)
			prodotti = new ArrayList<Prodotto>();
		return prodotti;
	}
	public Email creaEmailDaTemplate(EmailTemplate emailTemplate, Contatto contatto, List<DatoAggiuntivoAnagrafica> dA, List<DatoSondaggioAnagrafica> dS) {
		List<String> valoriValidi = new ArrayList<String>(ConstantsMap.anagraficaMap);
		Map<String, DatoAggiuntivo> valori =  ApplicationUtil.convertiAnagrafica(contatto.getAnagrafica(), valoriValidi);
		if(ApplicationUtil.notEmpty(dA)) {
			valori.putAll(ApplicationUtil.convertiDatoAggiuntivoA(dA));
		}
		if(ApplicationUtil.notEmpty(dS)) {
			valori.putAll(ApplicationUtil.convertiSondaggioA(dS));
		}
		return comunicazioneDao.creaEmailDaTemplate(emailTemplate, valori);
	}
	
	
	public Email creaEmailDaTemplate(EmailTemplate emailTemplate, Contatto contatto, Map<String, DatoAggiuntivo> valoriA) {
		List<String> valoriValidi = new ArrayList<String>(ConstantsMap.anagraficaMap);
		Map<String, DatoAggiuntivo> valori =  ApplicationUtil.convertiAnagrafica(contatto.getAnagrafica(), valoriValidi);
		if(ApplicationUtil.notEmpty(valoriA)) {
			valori.putAll(valoriA);
		}
		return comunicazioneDao.creaEmailDaTemplate(emailTemplate, valori);
	}
	
	
	public Rmail creaRmailDaTemplate(RmailTemplate rmailTemplate, Contatto contatto, Map<String, DatoAggiuntivo> valoriA) {
		List<String> valoriValidi = new ArrayList<String>(ConstantsMap.anagraficaMap);
		Map<String, DatoAggiuntivo> valori =  ApplicationUtil.convertiAnagrafica(contatto.getAnagrafica(), valoriValidi);
		if(ApplicationUtil.notEmpty(valoriA)) {
			valori.putAll(valoriA);
		}
		return comunicazioneDao.creaRmailDaTemplate(rmailTemplate, valori);
	}
	
	
	public Telecom creaTelecomDaTemplate(TelecomTemplate telecomTemplate, Contatto contatto, Map<String, DatoAggiuntivo> valoriA) {
		List<String> valoriValidi = new ArrayList<String>(ConstantsMap.anagraficaMap);
		Map<String, DatoAggiuntivo> valori =  ApplicationUtil.convertiAnagrafica(contatto.getAnagrafica(), valoriValidi);
		if(ApplicationUtil.notEmpty(valoriA)) {
			valori.putAll(valoriA);
		}
		return comunicazioneDao.creaTelecomDaTemplate(telecomTemplate, valori);
	}
	
	public Email creaEmailDaTemplate(EmailTemplate emailTemplate, Contatto contatto) {
		List<String> valoriValidi = new ArrayList<String>(ConstantsMap.anagraficaMap);
		Map<String, DatoAggiuntivo> valori =  ApplicationUtil.convertiAnagrafica(contatto.getAnagrafica(), valoriValidi);
		List<DatoAggiuntivoAnagrafica> dA = anagraficaDao.getDatiAggiuntiviAnagrafica(contatto.getAnagrafica().getId());
		if(ApplicationUtil.notEmpty(dA)) {
			valori.putAll(ApplicationUtil.convertiDatoAggiuntivoA(dA));
		}
		return comunicazioneDao.creaEmailDaTemplate(emailTemplate, valori);
	}
	

	public void  inviaEmail(List<Contatto> contatti, Utente user, EmailTemplate emailTemplate, String testo, String oggetto, HashMap<Allegato, String> allegati, Date dataSchedulata) {
		if(ApplicationUtil.notEmpty(contatti)) {
			new MultiploEmailSenderThread(contatti, user, emailTemplate, testo, oggetto, allegati, dataSchedulata, comunicazioneDao, this).start();
			String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.EMAIL.stampa() +": " + ApplicationUtil.size(contatti), contatti);
			operazioneDao.creaOperazione(ConstantsMap.LOG6, p, user, ConstantsMap.OPERAZIONE_INVIO_EMAIL_MASSIVO);
		}
	}
	public Email replaceInTemplateEmail(Contatto contatto, Email email, String testo, String oggetto) {
		List<String> valoriValidi = new ArrayList<String>(ConstantsMap.anagraficaMap);
		Map<String, DatoAggiuntivo> valori =  ApplicationUtil.convertiAnagrafica(contatto.getAnagrafica(), valoriValidi);
		List<DatoAggiuntivoAnagrafica> dA = anagraficaDao.getDatiAggiuntiviAnagrafica(contatto.getAnagrafica().getId());
		if(ApplicationUtil.notEmpty(dA)) {
			valori.putAll(ApplicationUtil.convertiDatoAggiuntivoA(dA));
		}
		email.setOggetto(ApplicationUtil.replaceInString(oggetto, valori));
		email.setTesto(ApplicationUtil.replaceInString(testo, valori));
		return email;
	}
	public Rmail creaRmailDaTemplate(RmailTemplate rmailTemplate, Contatto contatto, List<DatoAggiuntivoAnagrafica> dA, List<DatoSondaggioAnagrafica> dS) {
		List<String> valoriValidi = new ArrayList<String>(ConstantsMap.anagraficaMap);
		Map<String, DatoAggiuntivo> valori =  ApplicationUtil.convertiAnagrafica(contatto.getAnagrafica(), valoriValidi);
		if(ApplicationUtil.notEmpty(dA)) {
			valori.putAll(ApplicationUtil.convertiDatoAggiuntivoA(dA));
		}
		if(ApplicationUtil.notEmpty(dS)) {
			valori.putAll(ApplicationUtil.convertiSondaggioA(dS));
		}
		return comunicazioneDao.creaRmailDaTemplate(rmailTemplate, valori);
	}
	public Rmail creaRmailDaTemplate(RmailTemplate rmailTemplate, Contatto contatto) {
		List<String> valoriValidi = new ArrayList<String>(ConstantsMap.anagraficaMap);
		Map<String, DatoAggiuntivo> valori =  ApplicationUtil.convertiAnagrafica(contatto.getAnagrafica(), valoriValidi);
		List<DatoAggiuntivoAnagrafica> dA = anagraficaDao.getDatiAggiuntiviAnagrafica(contatto.getAnagrafica().getId());
		if(ApplicationUtil.notEmpty(dA)) {
			valori.putAll(ApplicationUtil.convertiDatoAggiuntivoA(dA));
		}
		return comunicazioneDao.creaRmailDaTemplate(rmailTemplate, valori);
	}
	public void  inviaRmail(List<Contatto> contatti, Utente user, RmailTemplate rmailTemplate, String testo, HashMap<Allegato, String> allegati, Date dataSchedulata) {
		if(ApplicationUtil.notEmpty(contatti)) {
			new MultiploRmailSenderThread(contatti, user, rmailTemplate, testo, allegati, dataSchedulata, comunicazioneDao, this).start();
			String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.RMAIL.stampa() +": " + ApplicationUtil.size(contatti), contatti);
			operazioneDao.creaOperazione(ConstantsMap.LOG6,  p, user, ConstantsMap.OPERAZIONE_INVIO_RMAIL_MASSIVO);
		}
	}
	public Rmail replaceInTemplateRmail(Contatto contatto, Rmail rmail, String testo) {
		List<String> valoriValidi = new ArrayList<String>(ConstantsMap.anagraficaMap);
		Map<String, DatoAggiuntivo> valori =  ApplicationUtil.convertiAnagrafica(contatto.getAnagrafica(), valoriValidi);
		List<DatoAggiuntivoAnagrafica> dA = anagraficaDao.getDatiAggiuntiviAnagrafica(contatto.getAnagrafica().getId());
		if(ApplicationUtil.notEmpty(dA)) {
			valori.putAll(ApplicationUtil.convertiDatoAggiuntivoA(dA));
		}
		rmail.setTesto(ApplicationUtil.replaceInString(testo, valori));
		return rmail;
	}
	public void esitoDefinitivo(Integer tipologia, Contatto contatto, Utente utente, EsitoArchiviazione esitoCliccato, List<DatoSondaggioAnagrafica> dati, List<DatoAggiuntivoAnagrafica> dA,  Map<ProdottoSelezionato, ContrattoProdotto> mapCr, Anagrafica nuovaAnagrafica, Long agenteId, Long proprietarioId, Date dataProgrammata,  Integer tipoContatto)  {
		EsitoChiamata esito = ConstantsEsito.DEFAULTS.get(ConstantsEsito.ID_OK);
		esitaProvvisorioA(true, true, tipologia, contatto, utente, esito, esitoCliccato, true, false);
		if(ApplicationUtil.isDisponibile(contatto.getData_presentazione()) && ApplicationUtil.isTrue(impostazioneDao.storiaImmediata())) {
			StoriaContatto storia = creaStoriaContatto(contatto, utente, contatto.getData_lavorazione(), true, false);
			salvaAggiorna(storia);
		}
		if(ApplicationUtil.isDisponibile(contatto.getData_presentazione()) && ApplicationUtil.isTrue(esitoCliccato.getModifica_contatto())) {
			salvaAggiorna(contatto.getAnagrafica());
		}
		if(ApplicationUtil.isDisponibile(contatto.getData_presentazione()) && ApplicationUtil.isTrue(impostazioneDao.getCampiInline())) {
			sondaggioDao.saveSondaggioToAnagrafica(esitoCliccato, dati, contatto.getAnagrafica());
		}
		if(ApplicationUtil.isDisponibile(dataProgrammata) && ApplicationUtil.isTrue(esitoCliccato.getInserisci_contatto()) && nuovaAnagrafica != null) {
			contattoDao.salvaContattoFiglio(false, nuovaAnagrafica, agenteId, contatto, proprietarioId, dataProgrammata, tipoContatto, utente);
		}
		new EsitaPannelloThread(this, nuovaAnagrafica, agenteId, proprietarioId, dataProgrammata, tipoContatto, contatto, esitoCliccato, dati, utente, dA, mapCr).start();
	}
	public void salvaContattoArchiviato(Anagrafica nuovaAnagrafica, Long agenteId, Long proprietarioId, Date dataProgrammata, Integer tipoContatto, Contatto contatto, EsitoArchiviazione esitoCliccato, List<DatoSondaggioAnagrafica> dS, Utente utente, List<DatoAggiuntivoAnagrafica> dA, Map<ProdottoSelezionato, ContrattoProdotto> mapCr) {
		if (esitoCliccato != null) {
			Chiamate.getCodaCampagne().add(contatto);
			List<Long> appId = new ArrayList<Long>();
			List<Long> ctrId = new ArrayList<Long>();
			if(!(ApplicationUtil.isDisponibile(contatto.getData_presentazione()) && ApplicationUtil.isTrue(impostazioneDao.storiaImmediata()))) {
				StoriaContatto storia = creaStoriaContatto(contatto, utente, contatto.getData_lavorazione(), true, false);
				salvaAggiorna(storia);
			}
			if(!ApplicationUtil.isDisponibile(contatto.getData_presentazione()) && ApplicationUtil.isTrue(esitoCliccato.getModifica_contatto())) {
				salvaAggiorna(contatto.getAnagrafica());
			}
			if(ApplicationUtil.isTrue(esitoCliccato.getModifica_contatto())) {
				operazioneDao.modificaContatto(utente, contatto);
			}
			if(!(ApplicationUtil.isDisponibile(contatto.getData_presentazione()) && ApplicationUtil.isTrue(impostazioneDao.getCampiInline()))) {
				sondaggioDao.saveSondaggioToAnagrafica(esitoCliccato, dS, contatto.getAnagrafica());
			}
			if(!ApplicationUtil.isDisponibile(dataProgrammata) && ApplicationUtil.isTrue(esitoCliccato.getInserisci_contatto()) && nuovaAnagrafica != null) {
				contattoDao.salvaContattoFiglio(false, nuovaAnagrafica, agenteId, contatto, proprietarioId, dataProgrammata, tipoContatto, utente);
			}
			if(ApplicationUtil.gtZero(esitoCliccato.getPresa_appuntamento())) {
				Map<Long, Appuntamento> map = contatto.getMappaAppuntamenti();
				if(ApplicationUtil.notEmpty(map)) {
					Iterator<Long> it = map.keySet().iterator();
					while (it.hasNext()) {
						Appuntamento appuntamento = map.get(it.next());
						if(ApplicationUtil.isToAdd(appuntamento.getId())) {
							appuntamento.setId(null);
							if(appuntamento != null) {
								if(ApplicationUtil.isToSave(appuntamento.getId())) {
									salvaAppuntamentoE(appuntamento, utente, true);
									salvaAllegati(appuntamento);
								}
							}
							appId.add(appuntamento.getId());
							Chiamate.getCodaStatistiche().get(contatto.getCampagna()).incAppuntamenti();
						}
					}
				}
			}
			List<Allegato> documentiGenerati = new ArrayList<Allegato>();
			String percorsoContratti = impostazioneDao.getPathBase(ConstantsGlobal.path_pda);
			Map<String, DatoAggiuntivo> valori_sondaggio = new HashMap<String, DatoAggiuntivo>();
			List<String> valoriValidi = new ArrayList<String>(ConstantsMap.anagraficaMap);
			if(ApplicationUtil.empty(dA)) {
				dA = new ArrayList<DatoAggiuntivoAnagrafica>();
			}
			if(mapCr == null) {
				mapCr =  new HashMap<ProdottoSelezionato, ContrattoProdotto>();
			}
			valori_sondaggio.putAll(ApplicationUtil.convertiAnagrafica(contatto.getAnagrafica(), valoriValidi));
			valori_sondaggio.putAll(ApplicationUtil.convertiDatoAggiuntivoA(dA));
			valori_sondaggio.putAll(ApplicationUtil.convertiSondaggioA(dS));
			String w = downloadDao.getWatermark();
			if (ApplicationUtil.notEmpty(mapCr)) {
				Iterator<ProdottoSelezionato> objIter = mapCr.keySet().iterator();
				while(objIter.hasNext()) {
					ProdottoSelezionato prd = objIter.next();
					ContrattoProdotto lsCrM = mapCr.get(prd);
					if(lsCrM != null){
						try {
							generaContratto(w, prd, ctrId, contatto, utente, percorsoContratti, contatto.getData_lavorazione(), documentiGenerati, valori_sondaggio, esitoCliccato, lsCrM);
						} catch (Exception e) {
						}
					}
				}
			}
			if (ApplicationUtil.isNotBlank(esitoCliccato.getEmail_notifica())) {
				new EmailNotificaThread(appId, ctrId, esitoCliccato, contatto, documentiGenerati, comunicazioneDao, downloadDao, impostazioneDao, configurazioneGenericService, utente, mapCr, dS).start();
			}
			if (ApplicationUtil.isNotBlank(esitoCliccato.getMessaggio_notifica())) {
				new ContattoMessaggioThread(esitoCliccato, contatto, messaggioDao, downloadDao, userDao, utente).start();
			}
			if (ApplicationUtil.isNotBlank(esitoCliccato.getNotizia_notifica())) {
				new ContattoNotiziaThread(esitoCliccato, contatto, notiziaDao, downloadDao, userDao, utente).start();
			}
			if (ApplicationUtil.isTrue(esitoCliccato.getInvia_email_cliente())) {
				new PannelloEmailSenderThread(utente, contatto, esitoCliccato, valori_sondaggio, documentiGenerati, comunicazioneDao).start();
			}
			if (ApplicationUtil.isTrue(esitoCliccato.getInvia_rmail_cliente())) {
				new PannelloRmailSenderThread(utente, contatto, esitoCliccato, valori_sondaggio, documentiGenerati, comunicazioneDao).start();
			}
			if (ApplicationUtil.isTrue(esitoCliccato.getInvia_telecom_cliente())) {
				new PannelloTelecomSenderThread(utente, contatto, esitoCliccato, valori_sondaggio, comunicazioneDao).start();
			}
		}
	}
	public void generaContratto(String w, ProdottoSelezionato prd, List<Long> ctrId, Contatto contatto, Utente operatore, String percorsoContratti, Date data, List<Allegato> documentiGenerati, Map<String, DatoAggiuntivo> valori_sondaggio, EsitoArchiviazione esitoCliccato, ContrattoProdotto daCr) throws Exception {
		Contratto contratto = daCr.getContratto();
		if(ApplicationUtil.eq(esitoCliccato.getContratto_imposta(), 1) || ApplicationUtil.eq(esitoCliccato.getContratto_imposta(), 2) ) {
			contratto.setContatto(contatto);
			contratto.setData_creazione(data);
		} 
		Long idOperatore = daCr.getOperatoreId();
		if(ApplicationUtil.gtZero(idOperatore)) {
			Utente o = campagnaDao.getOperatore(idOperatore);
			contratto.setOperatore(o);
		}
		if(ApplicationUtil.eq(esitoCliccato.getContratto_imposta(), 1) || ApplicationUtil.eq(esitoCliccato.getContratto_imposta(), 2) ) {
			if(contratto.getOperatore() == null) {
				contratto.setOperatore(operatore);
			}
		}
		contratto.setProdotto(daCr.getProdotto());
		contratto.setData_stipula(daCr.getData_stipula());
		contratto.setData_contabile(daCr.getData_contabile());
		contratto.setData_attivazione(daCr.getData_attivazione());
		contratto.setData_caricamento(daCr.getData_caricamento());
		contratto.setSeriale(daCr.getSeriale());
		contratto.setQuantita(daCr.getQuantita());
		contratto.setCosto_scontato(daCr.getCosto_scontato());
		contratto.setNota(daCr.getNota());
		Long idEsito = daCr.getStatoId();
		if(ApplicationUtil.gtZero(idEsito)) {
			EsitoContratto esito = campagnaDao.getEsitoContratto(idEsito);
			contratto.setEsito(esito);
		}
		Long idAgente = daCr.getAgenteId();
		if(ApplicationUtil.gtZero(idAgente)) {
			Agente agente = campagnaDao.getAgente(idAgente);
			contratto.setAgente(agente);
		}
		contrattoDao.salvaContrattoE(contratto, operatore, false);
		if(ApplicationUtil.eq(esitoCliccato.getContratto_imposta(), 1) || ApplicationUtil.eq(esitoCliccato.getContratto_imposta(), 2) ) {
			ctrId.add(contratto.getId());
		}
		Chiamate.getCodaStatistiche().get(contatto.getCampagna()).incContratti();
		if(ApplicationUtil.eq(esitoCliccato.getContratto_imposta(), 1) || ApplicationUtil.eq(esitoCliccato.getContratto_imposta(), 2) ) {
			List<Allegato> alls = daCr.getAllegato();
			if(ApplicationUtil.notEmpty(alls)) {
				String s = impostazioneDao.getPathBaseW(ConstantsGlobal.path_contratto);
				Iterator<Allegato> allIt = alls.iterator();
				while (allIt.hasNext()) {
					Allegato all = (Allegato) allIt.next();
					AllegatoContratto allegato = new AllegatoContratto();
					allegato.setContratto(contratto);
					allegato.setDimensione(all.getDimensione());
					allegato.setNome(ApplicationUtil.getNameN(all.getNome()));
					allegato.setContentFile(all.getFile(), s);
					salvaAggiorna(allegato);
				}
			}
		}
		contrattoDao.saveSondaggioToContratto(esitoCliccato, daCr.getDatoSondaggioContratto(), contratto);
		Map<String, DatoAggiuntivo> valori = new HashMap<String, DatoAggiuntivo>(valori_sondaggio);
		valori.putAll(ApplicationUtil.convertiSondaggioC(daCr.getDatoSondaggioContratto()));
		boolean nuovo_pda = false;
		List<TemplateProdotto> documenti = prodottoDao.getTemplateByProdottoId(daCr.getProdotto().getId());
		if (ApplicationUtil.notEmpty(documenti)) {
			Iterator<TemplateProdotto> objIter = documenti.iterator();
			while(objIter.hasNext()) {
				TemplateProdotto documento = objIter.next();
				Template t = documento.getId().getDocumento();
				String base = t.getNomeFileS();
				String ext = ApplicationUtil.getExtensionLower(base);
				Date date = nuovo_pda ? ApplicationUtil.date() : contratto.getData_creazione();
				String name = ConstantsGlobal.pda + "_" + ApplicationUtil.reportFile(date,  contratto.getId(), daCr.getProdotto().getId(), daCr.getProdotto().getNome(), ApplicationUtil.getBaseNameN(base)) + "." + ext;
				String percorso = ApplicationUtil.normalizeNoEndSeparator(percorsoContratti) + "/" + name;
				if(t.isPDA()) {
					List<TemplateStore>  tmps = getTemplateStoreByTemplateId(t);
					Map<String, DatoAggiuntivo> valori_sondaggio_nuovi = ApplicationUtil.replaceKeyinMap(valori, tmps);
					if(t.isPDA1()) {
						ApplicationUtil.mergeDataIntoTemplatePDF(w, percorso, valori_sondaggio_nuovi, documento.getId());
					} 
					if(t.isPDA2()) {
						ApplicationUtil.mergeDataIntoTemplateXLS(percorso, valori_sondaggio_nuovi, documento.getId());
					}
					if (ApplicationUtil.isTrue(esitoCliccato.getContratto_pda()) && prd.isValidPda() && t.isFirmabile()) {
						new FirmaThread(downloadDao, documento, name, prd, valori_sondaggio_nuovi, contratto).start();
					}
				} else {
					byte [] b = t.getContent();
					InputStream in = new ByteArrayInputStream(b);
					FileOutputStream out = new FileOutputStream(percorso); 
					ApplicationUtil.copy(in, out);
					out.close();
					in.close();
				}
				ApplicationUtil.addFile(documentiGenerati, ApplicationUtil.getNameN(percorso), percorso, date.getTime());
			}
		}
	}
	@SuppressWarnings("unchecked")
	public List<TemplateStore> getTemplateStoreByTemplateId(Template tmp) {
		DetachedCriteria criteria = DetachedCriteria.forClass(TemplateStore.class);
		criteria.add(Restrictions.eq("documento.id", tmp.getId()));
		return  getList(criteria);
	}
	public AllegatoContatto getDocumentiByAllegato(Long contatto, String documento) {
		DetachedCriteria criteria = DetachedCriteria.forClass(AllegatoContatto.class);
		criteria.add(Restrictions.eq("contatto.id", contatto));
		criteria.add(Restrictions.eq("nome", documento));
		return  (AllegatoContatto) getUniqueResult(criteria);
	}
	public List<TemplateStore> getDataFromTemplate(Template  ts) {
		List<TemplateStore> valori = new ArrayList<TemplateStore>();
		if(ts != null){ 
			try {
				if(ts.isPDA1()) {
					ApplicationUtil.pda1(ts, valori);
				}
				if(ts.isPDA2()) {
					ApplicationUtil.pda2(ts, valori);
				}
			}
			catch(Exception e) { 
				ErrorLog.log(null, getClass(), e);
			}
		}
		return valori;
	}
	
	@SuppressWarnings("unchecked")
	public List<TemplateStore> getDataFromTemplateList(Template template) {
		DetachedCriteria criteria = DetachedCriteria.forClass(TemplateStore.class);
		criteria.add(Restrictions.eq("documento.id",template.getId()));
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<BustaStore> getDataFromBustaList(Busta busta) {
		DetachedCriteria criteria = DetachedCriteria.forClass(BustaStore.class);
		criteria.add(Restrictions.eq("busta.id",busta.getId()));
		return getList(criteria);
	}
	public List<TemplateStore> getDataFromTemplateListMap(Template  template) {
		List<TemplateStore> tmp = getDataFromTemplate(template);
		try { 
			List<String> lstS = new ArrayList<String>(ConstantsMap.anagraficaMap);
			for(TemplateStore t : tmp) {
				if(lstS.contains(t.getChiave())) {
					t.setValore(t.getChiave());
				}
			}
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
		return tmp;
	}
	public List<SelectItem> getValori() {
		List<SelectItem> lstS = new ArrayList<SelectItem>();
		for(String data : ConstantsMap.destinatarioMap) {
			lstS.add(new SelectItem(data, Message.addCatalogText(data)));
		}
		return lstS;
	}
	public List<BustaStore> getDataFromBustaListMap(Busta busta) {
		List<BustaStore> lst = new ArrayList<BustaStore>();
		try { 
			List<String> tmp = ApplicationUtil.getDataFromBusta(busta);
			List<String> lstS = new ArrayList<String>(ConstantsMap.anagraficaMap);
			for(String chiave : tmp) {
				BustaStore tmps = new BustaStore();
				tmps.setChiave(chiave);
				if(lstS.contains(chiave)) {
					tmps.setValore(chiave);
				}
				lst.add(tmps);
			}
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
		return lst;
	}
	public void archiviaRichiamo(Contatto contatto, EsitoArchiviazione ea, Date data, Long operatore) {
		if (ApplicationUtil.eq(ea.getTipologiaS(), 1)) {
			Utente proprietario = userDao.getUser(operatore);
			contatto.proprietario(data, proprietario);
		}
		if (ApplicationUtil.eq(ea.getTipologiaS(), 2)) {
			contatto.proprietario(data, null);
		}
	}
	private DetachedCriteria getRichiamiArchiviatiCriteria(ContattoPannelloCriteria cont, String tipo, Long userId) {
		Boolean ts = impostazioneDao.getEstrazioneStato();
		DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class);
		criteria.add(Restrictions.eq("tipo", ConstantsGlobal.inviato));
		criteria.add(Restrictions.eq("proprietario.id",userId));
		DetachedCriteria campagnaC = criteria.createCriteria("campagna");
		if(ApplicationUtil.gtZero(cont.getCampagnaId())) {
			campagnaC.add(Restrictions.idEq(cont.getCampagnaId()));
		} 
		ApplicationUtil.criteriaStato(campagnaC, false, ApplicationUtil.isTrue(ts));
		
		if(ApplicationUtil.isNotBlank(tipo)) {
			criteria.createCriteria("anagrafica").add(ApplicationUtil.isNotNull(tipo));
		}
		Boolean s = impostazioneDao.getEstrazioneListe();
		if(ApplicationUtil.isTrue(s)) {
			DetachedCriteria listaC = criteria.createCriteria("lista");
			ApplicationUtil.criteriaStato(listaC, false, ApplicationUtil.isTrue(ts));
		}
		ApplicationUtil.archiviazione(criteria, ApplicationUtil.date(), true);
		return criteria;
	}
	private DetachedCriteria getRichiamiArchiviatiCriteria(String tipo, Long userId) {
		Boolean ts = impostazioneDao.getEstrazioneStato();
		DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class);
		criteria.add(Restrictions.eq("tipo", ConstantsGlobal.inviato));
		criteria.add(Restrictions.eq("proprietario.id",userId));
		DetachedCriteria campagnaC = criteria.createCriteria("campagna");
		ApplicationUtil.criteriaStato(campagnaC, false, ApplicationUtil.isTrue(ts));
		if(ApplicationUtil.isNotBlank(tipo)) {
			criteria.createCriteria("anagrafica").add(ApplicationUtil.isNotNull(tipo));
		}
		Boolean s = impostazioneDao.getEstrazioneListe();
		if(ApplicationUtil.isTrue(s)) {
			DetachedCriteria listaC = criteria.createCriteria("lista");
			ApplicationUtil.criteriaStato(listaC, false, ApplicationUtil.isTrue(ts));
		}
		ApplicationUtil.archiviazione(criteria, ApplicationUtil.date(), true);
		return criteria;
	}
	@SuppressWarnings("unchecked")
	public List<Contatto> getRichiamiArchiviati(ContattoPannelloCriteria cont, String tipo, Long userId,  int start, int size ) {
		DetachedCriteria criteria = getRichiamiArchiviatiCriteria(cont, tipo, userId);
		ApplicationUtil.setOrdinamento(criteria, cont.getO_tipo(), cont.getO_colonna(),  new Order [] {Order.asc("data_presentazione")});
		return getList(criteria, start, size);
	}
	public int getRichiamiArchiviatiSize(ContattoPannelloCriteria cont, String tipo, Long userId) {
		DetachedCriteria criteria = getRichiamiArchiviatiCriteria(cont, tipo, userId);
		return getListSize(criteria);
	}
	public int getRichiamiArchiviatiSize(String tipo, Long userId) {
		DetachedCriteria criteria = getRichiamiArchiviatiCriteria(tipo, userId);
		return getListSize(criteria);
	}
	public void aggiungiAppuntamentoAContatto(Contatto contatto, Appuntamento appuntamento) {
		if(ApplicationUtil.isToAdd(appuntamento.getId())) {
			contatto.addAppuntamento(appuntamento);
		} 
	}
	public void rimuoviAppuntamentoAContatto(Contatto contatto, Appuntamento appuntamento) {
		contatto.removeAppuntamento(appuntamento.getId());
	}
	public void esitaAppuntamentoAll(Appuntamento appuntamento, EsitoAppuntamento esito, Utente utente) {
		if(esito !=null && esito.getConfermato()) {
			if(appuntamento.getData_conferma() == null) {
				appuntamento.setData_conferma(ApplicationUtil.date());
			}
		} else {
			appuntamento.setData_conferma(null);
		}
		appuntamento.setEsito(esito);	
		if(ApplicationUtil.isToSave(appuntamento.getId())) {
			salvaAppuntamento(appuntamento, utente, true);
		}
	}
	
	
	public void esitaAppuntamento(Appuntamento appuntamento, Utente utente) {
		if(ApplicationUtil.isToSave(appuntamento.getId())) {
			salvaAppuntamento(appuntamento, utente, true);
		}
	}
	public void salvaAppuntamentoAgente(Appuntamento appuntamento,  Utente utente) {
		if(ApplicationUtil.isToSave(appuntamento.getId())) {
			salvaAppuntamento(appuntamento, utente, false);
		}
	}
	public void esitaAppuntamentoMassivo(List<Appuntamento> appuntamenti, EsitoAppuntamento esito, Utente utente) {
		if(ApplicationUtil.notEmpty(appuntamenti)) {
			Iterator<Appuntamento> objIter = appuntamenti.iterator();
			while(objIter.hasNext()) {
				Appuntamento appuntamento = objIter.next();
				appuntamento.setEsito(esito);	
				salvaAppuntamento(appuntamento, utente);
			}
		}
	}
	public synchronized void rimuoviInCodaContatti(List<Contatto> contatti) {
		autodialerDao.rimuoviContatti(contatti);
		autoInvioDao.rimuoviContatti(contatti);
	}
	public synchronized void rimuoviInCodaContatti(Contatto contatto) {
		autodialerDao.rimuoviContatto(contatto);
		autoInvioDao.rimuoviContatto(contatto);
	}
	public void esitaContattoMassivo(Boolean elimina_storia, Boolean elimina_nota, List<Contatto> contatti, Long esitoId, Utente utente) {
		if(ApplicationUtil.notEmpty(contatti)) {
			EsitoChiamata esito = getEsitoChiamataById(esitoId);
			if(esito!=null) {
				contattoDao.nascondiStoria(elimina_storia, contatti);
				Iterator<Contatto> objIter = contatti.iterator();
				while(objIter.hasNext()) {
					Contatto contatto = objIter.next();
					if(contatto != null) {
						if(ApplicationUtil.isTrue(elimina_nota)) {
							contatto.setNota(null);
						}
						if(!ConstantsGlobal.massivo) {
							salvaContattoC(true, null, contatto, utente, esito, false);
						} else {
							esitaContattoC(contatto, esito, utente);
						}
					}
				}
				if(ConstantsGlobal.massivo) {
					contattoDao.salvaContatti(true, contatti, utente, false);
				}
				String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.CONTATTO.stampa() +": " + ApplicationUtil.size(contatti), contatti);
				operazioneDao.creaOperazione(ConstantsMap.LOG5,  p, utente, ConstantsMap.OPERAZIONE_RIPROGRAMMA_GENERICO);
			}
		}
	}
	public void esitaArchiviazioneMassivo(Date data, Long operatore, Boolean elimina_storia, Boolean elimina_nota, List<Contatto> contatti, Long esitoId, Utente utente) {
		if(ApplicationUtil.notEmpty(contatti)) {
			EsitoChiamata ok = ConstantsEsito.DEFAULTS.get(ConstantsEsito.ID_OK);
			EsitoArchiviazione esito = getEsitoArchiviazioneById(esitoId);
			Utente op = null;
			if(ApplicationUtil.gtZero(operatore)) {
				op = campagnaDao.getOperatore(operatore);
			}
			if(esito!=null) {
				contattoDao.nascondiStoria(elimina_storia, contatti);
				Iterator<Contatto> objIter = contatti.iterator();
				while(objIter.hasNext()) {
					Contatto contatto = objIter.next();
					if(contatto != null) {
						if(ApplicationUtil.isTrue(elimina_nota)) {
							contatto.setNota(null);
						}
						contatto.setEsitoChiamata(ok);
						contatto.setEsitoArchiviazione(esito);
						if (ApplicationUtil.gtZero(esito.getTipologiaS())) {
							if (ApplicationUtil.eq(esito.getTipologiaS(), 1)) {
								contatto.proprietario(data, op);
								contatto.archivia(true);
							}
							if (ApplicationUtil.eq(esito.getTipologiaS(), 2)) {
								contatto.proprietario(data, null);
								contatto.pubblico();
							}
							if (ApplicationUtil.eq(esito.getTipologiaS(), 3)) {
								data = ApplicationUtil.date();
								contatto.proprietario(ApplicationUtil.addMinutes(data, esito.getAttesa()), null);
								contatto.pubblico();
							}
						} else {
							contatto.archivia(false);
						}
						if(!ConstantsGlobal.massivo) {
							salvaContatto(true, contatto, utente);
						}
					}
				}
				if(ConstantsGlobal.massivo) {
					contattoDao.salvaContatti(true, contatti, utente, false);
				}
				String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.CONTATTO.stampa() +": " + ApplicationUtil.size(contatti), contatti);
				operazioneDao.creaOperazione(ConstantsMap.LOG5, p,  utente, ConstantsMap.OPERAZIONE_RIPROGRAMMA_GENERICO);
			}
		}
	}
	public void resetContattoMassivo(Boolean elimina_storia, Boolean elimina_nota, List<Contatto> contatti,  Utente utente, Date data) {
		if(ApplicationUtil.notEmpty(contatti)) {
			EsitoChiamata esito = ConstantsEsito.DEFAULTS.get(ConstantsEsito.ID_DA_CHIAMARE);
			if(esito!=null) {
				contattoDao.nascondiStoria(elimina_storia, contatti);
				Iterator<Contatto> objIter = contatti.iterator();
				while(objIter.hasNext()) {
					Contatto contatto = objIter.next();
					if(contatto != null) {
						contatto.imposta(data);
						if(ApplicationUtil.isTrue(elimina_nota)) {
							contatto.setNota(null);
						}
						if(!ConstantsGlobal.massivo) {
							salvaContattoC(true, null, contatto, utente, esito, false);
						} else {
							esitaContattoC(contatto, esito, utente);
						}
					}
				}
				if(ConstantsGlobal.massivo) {
					contattoDao.salvaContatti(true, contatti, utente, false);
				}
				String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.CONTATTO.stampa() +": " + ApplicationUtil.size(contatti), contatti);
				operazioneDao.creaOperazione(ConstantsMap.LOG5, p, utente, ConstantsMap.OPERAZIONE_RIPROGRAMMA_RESET);
			}
		}
	}
	public void settaOpposizione(Long campagnaId, List<Contatto> contatti, Utente utente) {
		if(ApplicationUtil.notEmpty(contatti)) {
			if(ApplicationUtil.gtZero(campagnaId)) {
				Campagna campagna = campagnaDao.getCampagna(campagnaId);
				List<String> opp = impostazioneDao.trovaOpposizioni(campagnaId);
				Iterator<Contatto> objIter = contatti.iterator();
				while(objIter.hasNext()) {
					Contatto contatto = objIter.next();
					impostaOpposizione(campagna, contatto, opp, utente);
				}
			} else {
				if(ApplicationUtil.eq(campagnaId, ConstantsGlobal.ALTRO)) {
					Iterator<Contatto> objIter = contatti.iterator();
					Map<Long, List<String>> mc = new HashMap<Long, List<String>>();
					while(objIter.hasNext()) {
						Contatto contatto = objIter.next();
						List<String> opp = mc.get(contatto.getCampagna().getId());
						if(opp == null) {
							opp = new ArrayList<String>();
							opp.addAll(impostazioneDao.trovaOpposizioni(contatto.getCampagna().getId()));
							mc.put(contatto.getCampagna().getId(), opp);
						}
						impostaOpposizione(contatto.getCampagna(), contatto, opp, utente);
					}
				} else {
					List<String> opp = impostazioneDao.trovaOpposizioni(null);
					Iterator<Contatto> objIter = contatti.iterator();
					while(objIter.hasNext()) {
						Contatto contatto = objIter.next();
						impostaOpposizione(null, contatto, opp, utente);
					}
				}
			} 
		}
	}
	public void settaOpposizione(boolean all, Contatto contatto, Utente utente) {
		if(all) {
			List<String> opp = impostazioneDao.trovaOpposizioni(null);
			impostaOpposizione(null, contatto, opp, utente);
		} else {
			List<String> opp = impostazioneDao.trovaOpposizioni(contatto.getCampagna().getId());
			impostaOpposizione(contatto.getCampagna(), contatto, opp, utente);
		}
	}
	public void impostaOpposizione(Campagna campagna, Contatto contatto, List<String> opp, Utente utente) {
		Integer oC = impostazioneDao.getOpposizioneImportCondition();
		if(ApplicationUtil.eq(oC, 1)) {
			if(!opp.contains(contatto.getAnagrafica().getTelefono_fisso()) && ApplicationUtil.isNotBlank(contatto.getAnagrafica().getTelefono_fisso())) {
				opp.add(contatto.getAnagrafica().getTelefono_fisso());
				saveOpposizione(campagna, contatto, contatto.getAnagrafica().getTelefono_fisso(), utente);
			}
			if(!opp.contains(contatto.getAnagrafica().getTelefono_mobile()) && ApplicationUtil.isNotBlank(contatto.getAnagrafica().getTelefono_mobile())) {
				opp.add(contatto.getAnagrafica().getTelefono_mobile());
				saveOpposizione(campagna, contatto, contatto.getAnagrafica().getTelefono_mobile(), utente);
			}
			if(!opp.contains(contatto.getAnagrafica().getTelefono_altro()) && ApplicationUtil.isNotBlank(contatto.getAnagrafica().getTelefono_altro())) {
				opp.add(contatto.getAnagrafica().getTelefono_altro());
				saveOpposizione(campagna, contatto, contatto.getAnagrafica().getTelefono_altro(), utente);
			}
			if(!opp.contains(contatto.getAnagrafica().getTelefono_fax()) && ApplicationUtil.isNotBlank(contatto.getAnagrafica().getTelefono_fax())) {
				opp.add(contatto.getAnagrafica().getTelefono_fax());
				saveOpposizione(campagna, contatto, contatto.getAnagrafica().getTelefono_fax(), utente);
			}
			if(!opp.contains(contatto.getAnagrafica().getEmail()) && ApplicationUtil.isNotBlank(contatto.getAnagrafica().getEmail())) {
				opp.add(contatto.getAnagrafica().getEmail());
				saveOpposizione(campagna, contatto, contatto.getAnagrafica().getEmail(), utente);
			}
			if(!opp.contains(contatto.getAnagrafica().getLink()) && ApplicationUtil.isNotBlank(contatto.getAnagrafica().getLink())) {
				opp.add(contatto.getAnagrafica().getLink());
				saveOpposizione(campagna, contatto, contatto.getAnagrafica().getLink(), utente);
			}
			if(!opp.contains(contatto.getAnagrafica().getCodice_fiscale()) && ApplicationUtil.isNotBlank(contatto.getAnagrafica().getCodice_fiscale())) {
				opp.add(contatto.getAnagrafica().getCodice_fiscale());
				saveOpposizione(campagna, contatto, contatto.getAnagrafica().getCodice_fiscale(), utente);
			}
			if(!opp.contains(contatto.getAnagrafica().getPartita_iva()) && ApplicationUtil.isNotBlank(contatto.getAnagrafica().getPartita_iva())) {
				opp.add(contatto.getAnagrafica().getPartita_iva());
				saveOpposizione(campagna, contatto, contatto.getAnagrafica().getPartita_iva(), utente);
			}
		}
		if(ApplicationUtil.eq(oC, 2)) {
			if(!opp.contains(contatto.getAnagrafica().getTelefono_fisso()) && ApplicationUtil.isNotBlank(contatto.getAnagrafica().getTelefono_fisso())) {
				opp.add(contatto.getAnagrafica().getTelefono_fisso());
				saveOpposizione(campagna, contatto, contatto.getAnagrafica().getTelefono_fisso(), utente);
			}
			if(!opp.contains(contatto.getAnagrafica().getTelefono_mobile()) && ApplicationUtil.isNotBlank(contatto.getAnagrafica().getTelefono_mobile())) {
				opp.add(contatto.getAnagrafica().getTelefono_mobile());
				saveOpposizione(campagna, contatto, contatto.getAnagrafica().getTelefono_mobile(), utente);
			}
			if(!opp.contains(contatto.getAnagrafica().getTelefono_altro()) && ApplicationUtil.isNotBlank(contatto.getAnagrafica().getTelefono_altro())) {
				opp.add(contatto.getAnagrafica().getTelefono_altro());
				saveOpposizione(campagna, contatto, contatto.getAnagrafica().getTelefono_altro(), utente);
			}
			if(!opp.contains(contatto.getAnagrafica().getTelefono_fax()) && ApplicationUtil.isNotBlank(contatto.getAnagrafica().getTelefono_fax())) {
				opp.add(contatto.getAnagrafica().getTelefono_fax());
				saveOpposizione(campagna, contatto, contatto.getAnagrafica().getTelefono_fax(), utente);
			}
			if(!opp.contains(contatto.getAnagrafica().getEmail()) && ApplicationUtil.isNotBlank(contatto.getAnagrafica().getEmail())) {
				opp.add(contatto.getAnagrafica().getEmail());
				saveOpposizione(campagna, contatto, contatto.getAnagrafica().getEmail(), utente);
			}
		}
		if(ApplicationUtil.eq(oC, 3)) {
			if(!opp.contains(contatto.getAnagrafica().getTelefono_fisso()) && ApplicationUtil.isNotBlank(contatto.getAnagrafica().getTelefono_fisso())) {
				opp.add(contatto.getAnagrafica().getTelefono_fisso());
				saveOpposizione(campagna, contatto, contatto.getAnagrafica().getTelefono_fisso(), utente);
			}
			if(!opp.contains(contatto.getAnagrafica().getTelefono_mobile()) && ApplicationUtil.isNotBlank(contatto.getAnagrafica().getTelefono_mobile())) {
				opp.add(contatto.getAnagrafica().getTelefono_mobile());
				saveOpposizione(campagna, contatto, contatto.getAnagrafica().getTelefono_mobile(), utente);
			}
			if(!opp.contains(contatto.getAnagrafica().getTelefono_altro()) && ApplicationUtil.isNotBlank(contatto.getAnagrafica().getTelefono_altro())) {
				opp.add(contatto.getAnagrafica().getTelefono_altro());
				saveOpposizione(campagna, contatto, contatto.getAnagrafica().getTelefono_altro(), utente);
			}
			if(!opp.contains(contatto.getAnagrafica().getTelefono_fax()) && ApplicationUtil.isNotBlank(contatto.getAnagrafica().getTelefono_fax())) {
				opp.add(contatto.getAnagrafica().getTelefono_fax());
				saveOpposizione(campagna, contatto, contatto.getAnagrafica().getTelefono_fax(), utente);
			}
		}
	}
		
	private void saveOpposizione(Campagna campagna, Contatto contatto, String opp, Utente utente) {
		Opposizione opposizione = new Opposizione();
		opposizione.setDescrizione(contatto.getAnagrafica().getDenominazione());
		opposizione.setCampagna(campagna);
		opposizione.setOpposizione(opp);
		opposizione.setUtente(utente);
		opposizioneDao.salva(opposizione);
	}
	public StoriaAppuntamento salvaStoria(StoriaAppuntamento  storiaAppuntamento) {
		return (StoriaAppuntamento) salvaAggiorna(storiaAppuntamento);
	}
	public StoriaNota creaStoriaNota(Nota  nota, Utente utente, Date data) {
		StoriaNota storia = new StoriaNota();
		storia.setData(data);
		storia.setUtente(utente);
		storia.setBloccato(nota.getBloccato());
		storia.setOperatore(nota.getOperatore());
		storia.setAgente(nota.getAgente());
		storia.setNota(nota);
		storia.setColore(nota.getColore());
		storia.setRipetizioni(nota.getRipetizioni());
		storia.setData_termine(nota.getData_termine());
		storia.setData_inizio(nota.getData_inizio());
		storia.setData_fine(nota.getData_fine());
		storia.setNote(nota.getNote());
		return storia;
	}
	public StoriaAppuntamento creaStoriaAppuntamento(Appuntamento  appuntamento, Utente utente, Date data) {
		StoriaAppuntamento storia = new StoriaAppuntamento();
		storia.setData(data);
		storia.setUtente(utente);
		storia.setBloccato(appuntamento.getBloccato());
		storia.setOperatore(appuntamento.getOperatore());
		storia.setAgente(appuntamento.getAgente());
		storia.setAppuntamento(appuntamento);
		storia.setData_conferma(appuntamento.getData_conferma());
		storia.setData_esito(appuntamento.getData_esito());
		storia.setData_stato(appuntamento.getData_stato());
		storia.setLocalizzazione(appuntamento.getLocalizzazione());
		storia.setStato(appuntamento.getStato());
		storia.setData_inizio(appuntamento.getData_inizio());
		storia.setData_fine(appuntamento.getData_fine());
		storia.setNote(appuntamento.getNote());
		storia.setEsito(appuntamento.getEsito());
		return storia;
	}
	private DetachedCriteria getRichiamiPrivatoTeamCriteria(Boolean tutto, String tipo, Boolean supervisore, Long userId, Campagna campagna) {
		Boolean ts = impostazioneDao.getEstrazioneStato();
		DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class);
		DetachedCriteria campagnaC = criteria.createCriteria("campagna");
		if(campagna != null) {
			campagnaC.add(Restrictions.idEq(campagna.getId()));
		} 
		ApplicationUtil.criteriaStato(campagnaC, false, ApplicationUtil.isTrue(ts));
		if(ApplicationUtil.isFalse(tutto)) {
			if(ApplicationUtil.isNotBlank(tipo)) {
				criteria.createCriteria("anagrafica").add(ApplicationUtil.isNotNull(tipo));
			}
		}
		criteria.add(Restrictions.eq("tipo", ConstantsGlobal.inviato));
		Boolean s = impostazioneDao.getEstrazioneListe();
		if(ApplicationUtil.isTrue(s)) {
			DetachedCriteria listaC = criteria.createCriteria("lista");
			ApplicationUtil.criteriaStato(listaC, false, ApplicationUtil.isTrue(ts));
		}
		ApplicationUtil.archiviazione(criteria, ApplicationUtil.date(), false);
		
		impostazioneDao.subCriteriaUser(userId, supervisore, criteria, "proprietario.id");
		return criteria;
	}
	@SuppressWarnings("unchecked")
	public List<Contatto> getRichiamiPrivatiTeam(Boolean tutto, String tipo, Boolean supervisore, Long userId, Campagna campagna,  int start, int size) {
		DetachedCriteria criteria = getRichiamiPrivatoTeamCriteria(tutto, tipo, supervisore,userId, campagna);
		criteria.addOrder(Order.asc("data_presentazione"));
		return getList(criteria, start, size);
	}
	public int getRichiamiPrivatiTeamSize(Boolean tutto, String tipo, Boolean supervisore,Long userId, Campagna campagna) {
		DetachedCriteria criteria = getRichiamiPrivatoTeamCriteria(tutto, tipo, supervisore,userId, campagna);
		return getListSize(criteria);
	}
	private DetachedCriteria getRichiamiArchiviatoTeamCriteria(Boolean tutto, String tipo, Boolean supervisore, Long userId, Campagna campagna) {
		Boolean ts = impostazioneDao.getEstrazioneStato();
		DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class);
		DetachedCriteria campagnaC = criteria.createCriteria("campagna");
		if(campagna != null) {
			campagnaC.add(Restrictions.idEq(campagna.getId()));
		}
		ApplicationUtil.criteriaStato(campagnaC, false, ApplicationUtil.isTrue(ts));
		if(ApplicationUtil.isFalse(tutto)) {
			if(ApplicationUtil.isNotBlank(tipo)) {
				criteria.createCriteria("anagrafica").add(ApplicationUtil.isNotNull(tipo));
			}
		}
		criteria.add(Restrictions.eq("tipo", ConstantsGlobal.inviato));
		Boolean s = impostazioneDao.getEstrazioneListe();
		if(ApplicationUtil.isTrue(s)) {
			DetachedCriteria listaC = criteria.createCriteria("lista");
			ApplicationUtil.criteriaStato(listaC, false, ApplicationUtil.isTrue(ts));
		}
		ApplicationUtil.archiviazione(criteria, ApplicationUtil.date(), true);
		
		impostazioneDao.subCriteriaUser(userId, supervisore, criteria, "proprietario.id");
		return criteria;
	}
	@SuppressWarnings("unchecked")
	public List<Contatto> getRichiamiArchiviatiTeam(Boolean tutto, String tipo, Boolean supervisore, Long userId, Campagna campagna,  int start, int size ) {
		DetachedCriteria criteria = getRichiamiArchiviatoTeamCriteria(tutto, tipo, supervisore,userId, campagna);
		criteria.addOrder(Order.asc("data_presentazione"));
		return getList(criteria, start, size);
	}
	public int getRichiamiArchiviatiTeamSize(Boolean tutto, String tipo, Boolean supervisore, Long userId, Campagna campagna) {
		DetachedCriteria criteria = getRichiamiArchiviatoTeamCriteria(tutto, tipo, supervisore,userId, campagna);
		return getListSize(criteria);
	}
	public void  inviaChiamata(List<Contatto> contatti, Utente user, Date dataSchedulata) {
		if(ApplicationUtil.notEmpty(contatti)) {
			Integer numCanali = autodialerDao.getNumCanali();
			Iterator<Contatto> itCont = contatti.iterator();
			while (itCont.hasNext()) {
				Contatto contatto = (Contatto) itCont.next();
				autodialerDao.addUserContatto(user, contatto, dataSchedulata, numCanali);
			}
			String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.CONTATTO.stampa() +": " + ApplicationUtil.size(contatti), contatti);
			operazioneDao.creaOperazione(ConstantsMap.LOG6, p,  user, ConstantsMap.OPERAZIONE_INVIO_CONTATTO_MASSIVO);
		}
	}
	@SuppressWarnings("unchecked")
	public List<StoriaContatto> getStoriaContatto(Contatto contatto, Boolean supervisore) {
		if(contatto != null) {
			DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContatto.class);
			criteria.add(Restrictions.eq("contatto.id", contatto.getId()));
			if(ApplicationUtil.isFalse(supervisore)) {
				criteria.add(Restrictions.eq("invisibile", Boolean.FALSE));
			}
			criteria.addOrder(Order.asc("data"));
			criteria.setFetchMode("contatto", FetchMode.JOIN);
			return getList(criteria);
		} else return null;
	}
	@SuppressWarnings("unchecked")
	public List<StoriaContatto> getStoriaContatto(Contatto contatto, Integer mostra_storia, Boolean supervisore) {
		if(contatto != null && ApplicationUtil.eq(mostra_storia, 1)) {
			DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContatto.class);
			criteria.add(Restrictions.eq("contatto.id", contatto.getId()));
			if(ApplicationUtil.isFalse(supervisore)) {
				criteria.add(Restrictions.eq("invisibile", Boolean.FALSE));
			}
			criteria.addOrder(Order.asc("data"));
			criteria.setFetchMode("contatto", FetchMode.JOIN);
			return getList(criteria);
		} else return null;
	}
	public void operatoreContattoMassivo(Boolean elimina_storia, Boolean elimina_nota, List<Contatto> contattiLista, Long operatoreId, Utente utente) {
		List<Contatto> contatti = new ArrayList<Contatto>();
		if(ApplicationUtil.notEmpty(contattiLista)) {
			Iterator<Contatto> objCIter = contattiLista.iterator();
			while(objCIter.hasNext()) {
				Contatto contatto = objCIter.next();
				if(contatto != null && contatto.getProprietario() != null) {
					contatti.add(contatto);
				}
			}
			if(ApplicationUtil.notEmpty(contatti)) {
				contattoDao.nascondiStoria(elimina_storia, contatti);
				Utente proprietario = userDao.getUser(operatoreId);
				Iterator<Contatto> objIter = contatti.iterator();
				while(objIter.hasNext()) {
					Contatto contatto = objIter.next();
					contatto.setProprietario(proprietario);
					if(ApplicationUtil.isTrue(elimina_nota)) {
						contatto.setNota(null);
					}
					if(!ConstantsGlobal.massivo) {
						salvaContatto(true, contatto, utente);
					}
				}
				if(ConstantsGlobal.massivo) {
					contattoDao.salvaContatti(true, contatti, utente, false);
				}
				String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.CONTATTO.stampa() +": " + ApplicationUtil.size(contatti), contatti);
				operazioneDao.creaOperazione(ConstantsMap.LOG5,  p, utente, ConstantsMap.OPERAZIONE_MODIFICA_CONTATTO);
			}
		}
	}
	public void operatoreContatto(Boolean elimina_storia, Contatto contatto, Long operatoreId, Utente utente) {
		if(contatto != null && contatto.getProprietario() != null) {
			contattoDao.nascondiStoria(elimina_storia, contatto);
			Utente proprietario = userDao.getUser(operatoreId);
			contatto.setProprietario(proprietario);
			salvaContatto(true, contatto, utente);
			String p = ApplicationUtil.asString( ConstantsLanguage.CONTATTO.stampa(), contatto);
			operazioneDao.creaOperazione(ConstantsMap.LOG5, p,  utente, ConstantsMap.OPERAZIONE_MODIFICA_CONTATTO);
		}
	}
	public void settaOpposizione(String segnalazione) {
		List<String> opp = impostazioneDao.trovaOpposizioni(null);
		if(!opp.contains(segnalazione) && ApplicationUtil.isNotBlank(segnalazione)) {
			Opposizione opposizione = new Opposizione();
			opposizione.setOpposizione(segnalazione);
			opposizioneDao.salva(opposizione);
		}
	}
	public int getRichiamiPersonali(String tipo, boolean privato, Long userId) {
		Boolean ts = impostazioneDao.getEstrazioneStato();
		DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class);
		criteria.add(Restrictions.eq("tipo", ConstantsGlobal.inviato));
		criteria.add(Restrictions.eq("proprietario.id",userId));
		DetachedCriteria campagnaC = criteria.createCriteria("campagna");
		ApplicationUtil.criteriaStato(campagnaC, false, ApplicationUtil.isTrue(ts));
		if(ApplicationUtil.isNotBlank(tipo)) {
			criteria.createCriteria("anagrafica").add(ApplicationUtil.isNotNull(tipo));
		}
		Boolean s = impostazioneDao.getEstrazioneListe();
		if(ApplicationUtil.isTrue(s)) {
			DetachedCriteria listaC = criteria.createCriteria("lista");
			ApplicationUtil.criteriaStato(listaC, false, ApplicationUtil.isTrue(ts));
		}
		ApplicationUtil.archiviazione(criteria, ApplicationUtil.date(), !privato);
		return getListSize(criteria);
	}
	
	public Telecom creaTelecomDaTemplate(TelecomTemplate telecomTemplate, Contatto contatto, List<DatoAggiuntivoAnagrafica> dA, List<DatoSondaggioAnagrafica> dS) {
		List<String> valoriValidi = new ArrayList<String>(ConstantsMap.anagraficaMap);
		Map<String, DatoAggiuntivo> valori =  ApplicationUtil.convertiAnagrafica(contatto.getAnagrafica(), valoriValidi);
		if(ApplicationUtil.notEmpty(dA)) {
			valori.putAll(ApplicationUtil.convertiDatoAggiuntivoA(dA));
		}
		if(ApplicationUtil.notEmpty(dS)) {
			valori.putAll(ApplicationUtil.convertiSondaggioA(dS));
		}
		return comunicazioneDao.creaTelecomDaTemplate(telecomTemplate, valori);
	}
	public void  inviaTelecom(List<Contatto> contatti, Utente user, TelecomTemplate telecomTemplate, Telecom telecom) {
		if(ApplicationUtil.notEmpty(contatti)) {
			new MultiploTelecomSenderThread(contatti, user, telecomTemplate,  telecom, comunicazioneDao, this).start();
			String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.TELECOM.stampa() +": " + ApplicationUtil.size(contatti), contatti);
			operazioneDao.creaOperazione(ConstantsMap.LOG6, p,  user, ConstantsMap.OPERAZIONE_INVIO_TELECOM_MASSIVO);
		}
	}
	public Telecom replaceInTemplateTelecom(Contatto contatto, Telecom telecom, Telecom t) {
		List<String> valoriValidi = new ArrayList<String>(ConstantsMap.anagraficaMap);
		Map<String, DatoAggiuntivo> valori =  ApplicationUtil.convertiAnagrafica(contatto.getAnagrafica(), valoriValidi);
		List<DatoAggiuntivoAnagrafica> dA = anagraficaDao.getDatiAggiuntiviAnagrafica(contatto.getAnagrafica().getId());
		if(ApplicationUtil.notEmpty(dA)) {
			valori.putAll(ApplicationUtil.convertiDatoAggiuntivoA(dA));
		}
		telecom.setTipo(t.getTipo());
		telecom.setSistema(t.getSistema());
		telecom.setDriver(ApplicationUtil.replaceInString(t.getDriver(), valori));
		telecom.setCommands(ApplicationUtil.replaceInString(t.getCommands(), valori));
		telecom.setBody(ApplicationUtil.replaceInString(t.getBody(), valori));
		telecom.setHeaders(ApplicationUtil.replaceInString(t.getHeaders(), valori));
		telecom.setFields(ApplicationUtil.replaceInString(t.getFields(), valori));
		telecom.setOutputs(ApplicationUtil.replaceInString(t.getOutputs(), valori));
		telecom.setParameters(ApplicationUtil.replaceInString(t.getParameters(), valori));
		telecom.setLink(ApplicationUtil.replaceInString(t.getLink(), valori));
		telecom.setUsername(ApplicationUtil.replaceInString(t.getUsername(), valori));
		telecom.setPassword(ApplicationUtil.replaceInString(t.getPassword(), valori));
		telecom.setInputs(ApplicationUtil.replaceInString(t.getInputs(), valori));
		List<Label> labels = new ArrayList<Label>();
		for(Label l : t.getLabels()) {
			Label nl = new Label();
			nl.setLabel(ApplicationUtil.replaceInString(l.getLabel(), valori));
			nl.setValue(ApplicationUtil.replaceInString(l.getValue(), valori));
			if(!labels.contains(nl)) {
				labels.add(nl);
			}
		}
		telecom.setLabels(labels);
		return telecom;
	}
	public Telecom creaTelecomDaTemplate(TelecomTemplate telecomTemplate, Contatto contatto) {
		List<String> valoriValidi = new ArrayList<String>(ConstantsMap.anagraficaMap);
		Map<String, DatoAggiuntivo> valori =  ApplicationUtil.convertiAnagrafica(contatto.getAnagrafica(), valoriValidi);
		List<DatoAggiuntivoAnagrafica> dA = anagraficaDao.getDatiAggiuntiviAnagrafica(contatto.getAnagrafica().getId());
		if(ApplicationUtil.notEmpty(dA)) {
			valori.putAll(ApplicationUtil.convertiDatoAggiuntivoA(dA));
		}
		return comunicazioneDao.creaTelecomDaTemplate(telecomTemplate, valori);
	}
	
	
	public Contatto getContattoPannello(String ids, Utente utente, Boolean supervisore) {
		LoggerContatto l = new LoggerContatto(impostazioneDao.getLog());
		long time = System.currentTimeMillis();
		Contatto c = null;
		String id = ApplicationUtil.base64Decoder(ids);
		if(ApplicationUtil.isNotBlank(id)) {
			if(id.contains("[") && id.contains("]")) {
				c = getContattoByPannello(id, utente, supervisore);
			} else {
				c = getContattoByPannello(id);
			}
		}
		if(c != null) {
			Chiamate.getCodaContatti().put(c.getId(), new InfoContatto(time, utente, c, false));
		} 
		l.log("aperto ingresso", utente, c);
		new ApriIngressoThread(autodialerDao, operazioneDao, c, utente).start();
		return c;
	}
	
	@SuppressWarnings("unchecked")
	private Contatto getContattoByPannello(String id, Utente utente, Boolean supervisore) {
		//${INGRESSO}${CLIENTE}[${ESTENSIONE}]${CAMPAGNA}#url
		String chP = ApplicationUtil.trimToEmpty(ApplicationUtil.substringAfter(id, "]"));
		String chDC = ApplicationUtil.trimToEmpty(ApplicationUtil.substringBetween(id, "[", "]"));
		String chSC = ApplicationUtil.trimToEmpty(ApplicationUtil.substringBefore(id, "["));
		CallerUtil ca = new CallerUtil(chP);
		List<Long>	campagneIdList = ApplicationUtil.tokenL(ca.getPre(), ConstantsGlobal.commaSeparator);
		Contatto c = null;
		String chD = "";
		String chS = "";
		String recall_callers = null;
		String recall_musicclass = null;
		Integer soloNumero = null;
		Boolean soloAttive = null;
		Boolean soloPubbliche = null;
		Boolean soloUpdate = null;
		Boolean soloAperte = null;
		Boolean soloNuova = null;
		AiutoCampagna script = null;
		Ingresso inbound = getIngresso(chDC);
		if(inbound != null) {
			chD = ApplicationUtil.clearChiamante(inbound.getExtension());
			chS = ApplicationUtil.clearChiamante(ApplicationUtil.uniforma(chSC, inbound.getRem_prefisso(), inbound.getAdd_prefisso()));
			soloAttive = ApplicationUtil.isTrue(inbound.getSoloAttive());
			soloPubbliche = ApplicationUtil.isTrue(inbound.getSoloPubbliche());
			soloAperte = ApplicationUtil.isTrue(inbound.getSoloAperte());
			soloNuova = ApplicationUtil.isTrue(inbound.getSoloNuova());
			soloUpdate = ApplicationUtil.isTrue(inbound.getSoloUpdate());
			soloNumero = inbound.getSoloNumero();
			recall_callers = inbound.getRecall_callers();
			recall_musicclass = inbound.getRecall_musicclass();
			script = ApplicationUtil.isTrue(inbound.getEnabledScript()) ? new AiutoCampagna(inbound.getScript(), inbound.getScript_velocita(), inbound.getModulo(), inbound.getPdf()) : null;
		} else {
			chD = ApplicationUtil.clearChiamante(chDC);
			chS = ApplicationUtil.clearChiamante(chSC);
			soloAttive = Boolean.FALSE;
			soloPubbliche = Boolean.FALSE;
			soloAperte = Boolean.FALSE;
			soloNuova = Boolean.FALSE;
			soloUpdate = Boolean.FALSE;
			soloNumero = 1;
			recall_callers = null;
			recall_musicclass = null;
			script = null;
		}
		String tipo = ApplicationUtil.soloNumero(soloNumero);
		DetachedCriteria campagnaC = DetachedCriteria.forClass(Campagna.class);
		if(ApplicationUtil.notEmpty(campagneIdList)) {
			campagnaC.add(Restrictions.in("id", campagneIdList));
		} else {
			impostazioneDao.subCriteriaCampagnaUser(supervisore, utente.getId(), campagnaC, "id");
		}
		if(soloAttive) {
			Boolean ts = impostazioneDao.getEstrazioneStato();
			ApplicationUtil.criteriaStato(campagnaC, true, ApplicationUtil.isTrue(ts));
		}
		List<Campagna> ls =  getList(campagnaC);
		if(ApplicationUtil.notEmpty(ls)) {
			Campagna camp = ls.get(new Random().nextInt(ls.size()));
			if(camp != null) {
				c = autodialerDao.getContatto(chS, camp.getId(), soloPubbliche, soloAperte, tipo);
				EsitoChiamata esito = ApplicationUtil.esitoPredefinito(camp);
				if(c == null) {
					c = autodialerDao.salvaContatto(tipo, soloNuova, chS, camp.getId(), esito, ca.getUrl());
				} else {
					if(ApplicationUtil.isTrue(soloUpdate)) {
						c = autodialerDao.updateContatto(c, utente, esito);
					}
				}
			}
		}
		if(c != null) {
			preparaContatto(c, chS, chD, recall_callers, recall_musicclass, script);
		} 
		return c;
	}

	private Contatto getContattoByPannello(String ids) {
		Long id = ApplicationUtil.toLong(ids);
		Contatto c = Chiamate.getCodaRisposta().remove(id);
		if(c == null) {
			c = contattoDao.getContatto(id);
		}
		if(c != null) {
			preparaContatto(c, null, null, null, null, null);
		} 
		return c;
	}

	public Ingresso getIngresso(String nome) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Ingresso.class);
		criteria.add(Restrictions.eq("nome", nome));
		return  (Ingresso) getUniqueResult(criteria);
	}
	
	public void setDestinatario(boolean open, Rmail rmail, RmailTemplate template, Contatto contatto) {
		rmail.imposta(open, template, contatto);
	}
	
	public Contatto getContatto(Long id) {
		return contattoDao.getContatto(id);
	}
}