package it.callite.dao;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.hibernate.FetchMode;
import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import it.callite.column.generic.GenericColumn;
import it.callite.constants.ConstantsColumn;
import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.constants.ConstantsMap;
import it.callite.criteria.PrenotazioneCriteria;
import it.callite.criteria.base.ExtendedCriteria;
import it.callite.dao.generic.GenericDao;
import it.callite.model.Prenotazione;
import it.callite.model.StoriaPrenotazione;
import it.callite.model.Utente;
import it.callite.model.report.ColonnaModel;
import it.callite.model.report.ColonnaQuery;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.LoggerColonna;
import it.callite.utility.thread.SalvaStoriaPrenotazioneThread;
@SuppressWarnings("serial")
@Repository("prenotazioneDao")
public class PrenotazioneDao extends GenericDao {
	@Autowired
	private UserDao userDao;
	@Autowired
	private OperazioneDao operazioneDao;
	@Autowired
	private ImpostazioneDao impostazioneDao;
	@Autowired
	private DataDao dataDao;
	
	public void salvaPrenotazione(Prenotazione prenotazione, Utente utente) {
		Date data = ApplicationUtil.date();
		prenotazione.setData_lavorazione(data);
		prenotazione.setUtente(utente);
		salvaAggiorna(prenotazione);
		new SalvaStoriaPrenotazioneThread(this, prenotazione, utente, data).start();
	}
	public StoriaPrenotazione creaStoriaPrenotazione(Prenotazione  prenotazione, Utente utente, Date data) {
		StoriaPrenotazione storia = new StoriaPrenotazione();
		storia.setData(data);
		storia.setUtente(utente);
		storia.setBloccato(prenotazione.getBloccato());
		storia.setOperatore(prenotazione.getOperatore());
		storia.setAgente(prenotazione.getAgente());
		storia.setPrenotazione(prenotazione);
		storia.setTelefono(prenotazione.getTelefono());
		storia.setTipo_telefono(prenotazione.getTipo_telefono());
		storia.setStato(prenotazione.getStato());
		return storia;
	}
	@SuppressWarnings("unchecked")
	public List<StoriaPrenotazione> getStoriaPrenotazione(Prenotazione  prenotazione) {
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaPrenotazione.class);
		criteria.add(Restrictions.eq("prenotazione.id", prenotazione.getId()));
		criteria.setFetchMode("prenotazione", FetchMode.JOIN);
		criteria.addOrder(Order.asc("data"));
		return getList(criteria);
	}
	public void eliminaPrenotazione(Utente utente, Prenotazione prenotazione) {
		String p = ApplicationUtil.asString( ConstantsLanguage.PRENOTAZIONE.stampa(), prenotazione);
		operazioneDao.creaOperazione(ConstantsMap.LOG3, p, utente, ConstantsMap.OPERAZIONE_ELIMINA_PRENOTAZIONE);
		cancellaAggiorna(prenotazione);
	}
	public void eliminaPrenotazioneSelezionati(Utente utente, List<Prenotazione> lista) {
		String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.PRENOTAZIONE.stampa() +": " + ApplicationUtil.size(lista), lista);
		operazioneDao.creaOperazione(ConstantsMap.LOG3,p, utente, ConstantsMap.OPERAZIONE_ELIMINA_PRENOTAZIONE);
		cancellaLista(lista);
	}
	private ExtendedCriteria ricercaPrenotazioneAgenteCriteria(Utente utente, PrenotazioneCriteria ana) {
		if(ana != null)  ana = (PrenotazioneCriteria) ana.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Prenotazione.class);
		DetachedCriteria criteriaC = criteria.createCriteria("campagna");
		if (ApplicationUtil.gtZero(ana.getAgenteId())) {
			criteria.add(Restrictions.eq("agente.id", ana.getAgenteId()));
		} else {
			impostazioneDao.subCriteriaAgente(utente.getId(), criteria, "agente.id");
		}
		
		if(ApplicationUtil.isNotBlank(ana.getUtente())) {
			DetachedCriteria cOp = criteria.createCriteria("operatore");
			ApplicationUtil.operatore(cOp, ana.getUtente());
		}
		if (ApplicationUtil.isNotBlank(ana.getAgente())) {
			DetachedCriteria cAg = criteria.createCriteria("agente");
			ApplicationUtil.denominazione(cAg, ana.getAgente());
		}
		if(ApplicationUtil.isNotBlank(ana.getTelefono())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, ana.getTelefono(), criteria, "telefono");
		}
		if (ApplicationUtil.gtZero(ana.getStato())) {
			criteria.add(Restrictions.eq("stato", ana.getStato()));
		}
		if (ApplicationUtil.gtZero(ana.getCampagnaId())) {
			criteriaC.add(Restrictions.idEq(ana.getCampagnaId()));
		}
		ana.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, ana);
	}
	@SuppressWarnings("unchecked")
	public List<Prenotazione> ricercaPrenotazioneAgente(Utente utente, PrenotazioneCriteria ana, int start, int size) {
		ExtendedCriteria criteria = ricercaPrenotazioneAgenteCriteria(utente, ana);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), ana.getO_tipo(), ana.getO_colonna(), new Order [] {Order.desc("id")});
		return getList(criteria, start, size);
	}
	public int ricercaPrenotazioneAgenteSize(Utente utente, PrenotazioneCriteria ana) {
		ExtendedCriteria criteria = ricercaPrenotazioneAgenteCriteria(utente, ana);
		return getListSize(criteria);
	}
	private ExtendedCriteria ricercaPrenotazioneOperatoreCriteria(Utente utente, PrenotazioneCriteria ana) {
		if(ana != null)  ana = (PrenotazioneCriteria) ana.loadCriteria();
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(Prenotazione.class);
		DetachedCriteria criteriaCA = criteria.createCriteria("campagna", "campagna");
		DetachedCriteria criteriaOp = criteria.createCriteria("operatore", "operatore");
		if(ApplicationUtil.isTrue(ana.getAzione())) {
			impostazioneDao.subCriteriaRoleOperatore(utente.getId(), criteria, "campagna", "operatore");
			impostazioneDao.subCriteriaOperatore(utente.getId(), criteriaOp, "id");
			if(ApplicationUtil.isTrue(sd)) {
				impostazioneDao.subCriteriaCampagnaOperatore(utente.getId(), criteriaCA, "id");
			}
			if (ApplicationUtil.gtZero(ana.getOperatoreId())) {
				criteriaOp.add(Restrictions.idEq(ana.getOperatoreId()));
			}
		} else {
			criteriaOp.add(Restrictions.idEq(utente.getId()));
		}
		if (ApplicationUtil.isNotBlank(ana.getAgente())) {
			DetachedCriteria cAg = criteria.createCriteria("agente");
			ApplicationUtil.denominazione(cAg, ana.getAgente());
		}
		if(ApplicationUtil.isNotBlank(ana.getTelefono())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, ana.getTelefono(), criteria, "telefono");
		}
		if (ApplicationUtil.gtZero(ana.getStato())) {
			criteria.add(Restrictions.eq("stato", ana.getStato()));
		}
		if (ApplicationUtil.gtZero(ana.getCampagnaId())) {
			criteriaCA.add(Restrictions.idEq(ana.getCampagnaId()));
		}
		ana.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, ana);
	}
	@SuppressWarnings("unchecked")
	public List<Prenotazione> ricercaPrenotazioneOperatore(Utente utente, PrenotazioneCriteria ana, int start, int size) {
		ExtendedCriteria criteria = ricercaPrenotazioneOperatoreCriteria(utente, ana);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), ana.getO_tipo(), ana.getO_colonna(), new Order [] {Order.desc("id")});
		return getList(criteria, start, size);
	}
	public int ricercaPrenotazioneOperatoreSize(Utente utente, PrenotazioneCriteria ana) {
		ExtendedCriteria criteria = ricercaPrenotazioneOperatoreCriteria(utente, ana);
		return getListSize(criteria);
	}
	private ExtendedCriteria ricercaPrenotazioneSupervisoreCriteria(Utente utente, PrenotazioneCriteria ana) {
		if(ana != null)  ana = (PrenotazioneCriteria) ana.loadCriteria();
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(Prenotazione.class);
		DetachedCriteria criteriaC = criteria.createCriteria("campagna", "campagna");
		DetachedCriteria cOp = criteria.createCriteria("operatore", "operatore");
		impostazioneDao.subCriteriaRoleSupervisore(utente.getId(), criteria, "campagna", "operatore");
		impostazioneDao.subCriteriaSupervisore(utente.getId(), cOp, "id");
		if (ApplicationUtil.gtZero(ana.getCampagnaId())) {
			criteriaC.add(Restrictions.idEq(ana.getCampagnaId()));
		} else {
			if(ApplicationUtil.isTrue(sd)) {
				impostazioneDao.subCriteriaCampagnaSupervisore(utente.getId(), criteriaC, "id");
			}
		}
		if(ApplicationUtil.isNotBlank(ana.getUtente())) {
			ApplicationUtil.operatore(cOp, ana.getUtente());
		}
		if (ApplicationUtil.isNotBlank(ana.getAgente())) {
			DetachedCriteria cAg = criteria.createCriteria("agente");
			ApplicationUtil.denominazione(cAg, ana.getAgente());
		}
		if(ApplicationUtil.isNotBlank(ana.getTelefono())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, ana.getTelefono(), criteria, "telefono");
		}
		if (ApplicationUtil.gtZero(ana.getStato())) {
			criteria.add(Restrictions.eq("stato", ana.getStato()));
		}
		ana.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, ana);
	}
	private ExtendedCriteria ricercaPrenotazionePartnerCriteria(Utente utente, PrenotazioneCriteria ana) {
		if(ana != null)  ana = (PrenotazioneCriteria) ana.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Prenotazione.class);
		DetachedCriteria criteriaC = criteria.createCriteria("campagna");
		
		impostazioneDao.subCriteriaCampagnaPartner(utente.getId(), criteriaC, "id");
		
		if(ApplicationUtil.isNotBlank(ana.getUtente())) {
			DetachedCriteria cOp = criteria.createCriteria("operatore");
			ApplicationUtil.operatore(cOp, ana.getUtente());
		}
		if (ApplicationUtil.isNotBlank(ana.getAgente())) {
			DetachedCriteria cAg = criteria.createCriteria("agente");
			ApplicationUtil.denominazione(cAg, ana.getAgente());
		}
		if(ApplicationUtil.isNotBlank(ana.getTelefono())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, ana.getTelefono(), criteria, "telefono");
		}
		if (ApplicationUtil.gtZero(ana.getStato())) {
			criteria.add(Restrictions.eq("stato", ana.getStato()));
		}
		if (ApplicationUtil.gtZero(ana.getCampagnaId())) {
			criteriaC.add(Restrictions.idEq(ana.getCampagnaId()));
		}
		ana.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, ana);
	}
	@SuppressWarnings("unchecked")
	public List<Prenotazione> ricercaPrenotazioneSupervisore(Utente utente, PrenotazioneCriteria ana, int start, int size) {
		ExtendedCriteria criteria = ricercaPrenotazioneSupervisoreCriteria(utente, ana);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), ana.getO_tipo(), ana.getO_colonna(), new Order [] {Order.desc("id")});
		return getList(criteria, start, size);
	}
	public int ricercaPrenotazioneSupervisoreSize(Utente utente, PrenotazioneCriteria ana) {
		ExtendedCriteria criteria = ricercaPrenotazioneSupervisoreCriteria(utente, ana);
		return getListSize(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Prenotazione> ricercaPrenotazionePartner(Utente utente, PrenotazioneCriteria ana, int start, int size) {
		ExtendedCriteria criteria = ricercaPrenotazionePartnerCriteria(utente, ana);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), ana.getO_tipo(), ana.getO_colonna(), new Order [] {Order.desc("id")});
		return getList(criteria, start, size);
	}
	public int ricercaPrenotazionePartnerSize(Utente utente, PrenotazioneCriteria ana) {
		ExtendedCriteria criteria = ricercaPrenotazionePartnerCriteria(utente, ana);
		return getListSize(criteria);
	}
	private ExtendedCriteria ricercaPrenotazioneCoordinatoreCriteria(Utente utente, PrenotazioneCriteria ana) {
		if(ana != null)  ana = (PrenotazioneCriteria) ana.loadCriteria();
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(Prenotazione.class);
		DetachedCriteria criteriaC = criteria.createCriteria("campagna", "campagna");
		DetachedCriteria criteriaAg = criteria.createCriteria("agente", "agente");
		impostazioneDao.subCriteriaRoleCoordinatore(utente.getId(), criteria, "campagna", "agente");
		if (ApplicationUtil.gtZero(ana.getAgenteId())) {
			criteriaAg.add(Restrictions.idEq(ana.getAgenteId()));
		} else {
			impostazioneDao.subCriteriaCoordinatore(utente.getId(), criteriaAg, "id");
		}
		if (ApplicationUtil.gtZero(ana.getCampagnaId())) {
			criteriaC.add(Restrictions.idEq(ana.getCampagnaId()));
		} else {
			if(ApplicationUtil.isTrue(sd)) {
				impostazioneDao.subCriteriaCoordinatoreCampagna(utente.getId(), criteriaC, "id");
			}
		}
		if(ApplicationUtil.isNotBlank(ana.getUtente())) {
			DetachedCriteria cOp = criteria.createCriteria("operatore");
			ApplicationUtil.operatore(cOp, ana.getUtente());
		}
		if (ApplicationUtil.isNotBlank(ana.getAgente())) {
			ApplicationUtil.denominazione(criteriaAg, ana.getAgente());
		}
		if(ApplicationUtil.isNotBlank(ana.getTelefono())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, ana.getTelefono(), criteria, "telefono");
		}
		if (ApplicationUtil.gtZero(ana.getStato())) {
			criteria.add(Restrictions.eq("stato", ana.getStato()));
		}
		ana.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, ana);
	}
	@SuppressWarnings("unchecked")
	public List<Prenotazione> ricercaPrenotazioneCoordinatore(Utente utente, PrenotazioneCriteria ana, int start, int size) {
		ExtendedCriteria criteria = ricercaPrenotazioneCoordinatoreCriteria(utente, ana);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), ana.getO_tipo(), ana.getO_colonna(), new Order [] {Order.desc("id")});
		return getList(criteria, start, size);
	}
	public int ricercaPrenotazioneCoordinatoreSize(Utente utente, PrenotazioneCriteria ana) {
		ExtendedCriteria criteria = ricercaPrenotazioneCoordinatoreCriteria(utente, ana);
		return getListSize(criteria);
	}
	private ExtendedCriteria ricercaPrenotazioneManagerCriteria(Utente utente, PrenotazioneCriteria ana) {
		if(ana != null)  ana = (PrenotazioneCriteria) ana.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Prenotazione.class);
		DetachedCriteria criteriaC = criteria.createCriteria("campagna");
		if(ApplicationUtil.isNotBlank(ana.getUtente())) {
			DetachedCriteria cOp = criteria.createCriteria("operatore");
			ApplicationUtil.operatore(cOp, ana.getUtente());
		}
		if (ApplicationUtil.isNotBlank(ana.getAgente())) {
			DetachedCriteria cAg = criteria.createCriteria("agente");
			ApplicationUtil.denominazione(cAg, ana.getAgente());
		}
		if(ApplicationUtil.isNotBlank(ana.getTelefono())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, ana.getTelefono(), criteria, "telefono");
		}
		if (ApplicationUtil.gtZero(ana.getStato())) {
			criteria.add(Restrictions.eq("stato", ana.getStato()));
		}
		if (ApplicationUtil.gtZero(ana.getCampagnaId())) {
			criteriaC.add(Restrictions.idEq(ana.getCampagnaId()));
		}
		ana.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, ana);
	}
	@SuppressWarnings("unchecked")
	public List<Prenotazione> ricercaPrenotazioneManager(Utente utente, PrenotazioneCriteria ana, int start, int size) {
		ExtendedCriteria criteria = ricercaPrenotazioneManagerCriteria(utente, ana);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), ana.getO_tipo(), ana.getO_colonna(), new Order [] {Order.desc("id")});
		return getList(criteria, start, size);
	}
	public int ricercaPrenotazioneManagerSize(Utente utente, PrenotazioneCriteria ana) {
		ExtendedCriteria criteria = ricercaPrenotazioneManagerCriteria(utente, ana);
		return getListSize(criteria);
	}
	private ExtendedCriteria ricercaPrenotazioneBackofficeCriteria(Utente utente, PrenotazioneCriteria ana) {
		if(ana != null)  ana = (PrenotazioneCriteria) ana.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Prenotazione.class);
		DetachedCriteria criteriaC = criteria.createCriteria("campagna");
		if(ApplicationUtil.isNotBlank(ana.getUtente())) {
			DetachedCriteria cOp = criteria.createCriteria("operatore");
			ApplicationUtil.operatore(cOp, ana.getUtente());
		}
		if (ApplicationUtil.isNotBlank(ana.getAgente())) {
			DetachedCriteria cAg = criteria.createCriteria("agente");
			ApplicationUtil.denominazione(cAg, ana.getAgente());
		}
		if(ApplicationUtil.isNotBlank(ana.getTelefono())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, ana.getTelefono(), criteria, "telefono");
		}
		if (ApplicationUtil.gtZero(ana.getStato())) {
			criteria.add(Restrictions.eq("stato", ana.getStato()));
		}
		if (ApplicationUtil.gtZero(ana.getCampagnaId())) {
			criteriaC.add(Restrictions.idEq(ana.getCampagnaId()));
		}
		ana.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, ana);
	}
	@SuppressWarnings("unchecked")
	public List<Prenotazione> ricercaPrenotazioneBackoffice(Utente utente, PrenotazioneCriteria ana, int start, int size) {
		ExtendedCriteria criteria = ricercaPrenotazioneBackofficeCriteria(utente, ana);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), ana.getO_tipo(), ana.getO_colonna(), new Order [] {Order.desc("id")});
		return getList(criteria, start, size);
	}
	public int ricercaPrenotazioneBackofficeSize(Utente utente, PrenotazioneCriteria ana) {
		ExtendedCriteria criteria = ricercaPrenotazioneBackofficeCriteria(utente, ana);
		return getListSize(criteria);
	}
	public Prenotazione getPrenotazione(Long id) {
		return (Prenotazione) get(Prenotazione.class, id);
	}
	private ExtendedCriteria ricercaPrenotazioneReteCriteria(Utente utente, PrenotazioneCriteria ana) {
		if(ana != null)  ana = (PrenotazioneCriteria) ana.loadCriteria();
		Integer max = userDao.getMaxLivelloRete();
		DetachedCriteria criteria = DetachedCriteria.forClass(Prenotazione.class);
		DetachedCriteria criteriaC = criteria.createCriteria("campagna");
		DetachedCriteria critA = criteria.createCriteria("agente");
		if (ApplicationUtil.gtZero(ana.getRete())) {
			if(ApplicationUtil.eq(ana.getRete(), 1)) {
				if (ApplicationUtil.gtZero(ana.getAgenteId())) {
					critA.createCriteria("agente_padre").add(Restrictions.idEq(ana.getAgenteId()));
				}
			}
			if(ApplicationUtil.eq(ana.getRete(), 2)) {
				if (ApplicationUtil.gtZero(ana.getAgenteId())) {
					impostazioneDao.subCriteriaAgentiSottoRete(ana.getAgenteId(), true, max, critA, "id");
				}
			}
			if(ApplicationUtil.eq(ana.getRete(), 3)) {
				if (ApplicationUtil.gtZero(ana.getAgenteId())) {
					impostazioneDao.subCriteriaAgentiSottoRete(ana.getAgenteId(), true, max, critA, "id");
					critA.add(Restrictions.ne("id", ana.getAgenteId()));
				}
			}
			if(ApplicationUtil.eq(ana.getRete(), 4)) {
				if (ApplicationUtil.gtZero(ana.getAgenteId())) {
					critA.add(Restrictions.idEq(ana.getAgenteId()));
				}
			}
		} else {
			impostazioneDao.subCriteriaAgentiSottoReteByUser(utente.getId(), 0, max, critA, "id");
		}
		if(ApplicationUtil.isNotBlank(ana.getUtente())) {
			DetachedCriteria cOp = criteria.createCriteria("operatore");
			ApplicationUtil.operatore(cOp, ana.getUtente());
		}
		if(ApplicationUtil.isNotBlank(ana.getTelefono())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, ana.getTelefono(), criteria, "telefono");
		}
		if (ApplicationUtil.gtZero(ana.getStato())) {
			criteria.add(Restrictions.eq("stato", ana.getStato()));
		}
		if (ApplicationUtil.gtZero(ana.getCampagnaId())) {
			criteriaC.add(Restrictions.idEq(ana.getCampagnaId()));
		}
		ana.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, ana);
	}
	@SuppressWarnings("unchecked")
	public List<Prenotazione> ricercaPrenotazioneRete(Utente utente, PrenotazioneCriteria ana, int start, int size) {
		ExtendedCriteria criteria = ricercaPrenotazioneReteCriteria(utente, ana);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), ana.getO_tipo(), ana.getO_colonna(), new Order [] {Order.desc("id")});
		return getList(criteria, start, size);
	}
	public int ricercaPrenotazioneReteSize(Utente utente, PrenotazioneCriteria ana) {
		ExtendedCriteria criteria = ricercaPrenotazioneReteCriteria(utente, ana);
		return getListSize(criteria);
	}
	
	public List<Prenotazione> impostaConteggioPrenotazione(Utente utente, GenericColumn c, List<Prenotazione> prenotazioni) {
		
		if(ApplicationUtil.notEmpty(prenotazioni))  {
			LoggerColonna l = new LoggerColonna(impostazioneDao.getLoggerColonne());
			String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.PRENOTAZIONE.stampa() +": " + ApplicationUtil.size(prenotazioni), prenotazioni);
			operazioneDao.creaOperazione(ConstantsMap.LOG4, p ,utente, ConstantsMap.OPERAZIONE_VISUALIZZA_PRENOTAZIONE);
			List<Long> ll = ApplicationUtil.prendiIdPrenotazione(prenotazioni);
			Map<Long, ColonnaModel> map = dataDao.getConteggioColonna(c, ll, Prenotazione.class, new ColonnaQuery(ConstantsColumn.storia, "storia"));
			Iterator<Prenotazione> it = prenotazioni.iterator();
			while (it.hasNext()) {
				Prenotazione o = it.next();
				ColonnaModel lmap = map.get(o.getId());
				if(lmap != null) {
					o.setModel(lmap);
				}
			}
			l.log(c);
		}
		return prenotazioni;
	}
	
	public void bloccaPrenotazione(Utente utente, Prenotazione prenotazione, boolean blocca) {
		if (prenotazione != null ) {
			prenotazione.setBloccato(blocca);
			salvaPrenotazione(prenotazione, utente);
		} 
	}
	
	public void bloccaPrenotazioniSelezionati(Utente utente, List<Prenotazione> prenotazioni, boolean blocca) {
		if (ApplicationUtil.notEmpty(prenotazioni)) {
			Iterator<Prenotazione> it = prenotazioni.iterator();
			while (it.hasNext()) {
				Prenotazione prenotazione = (Prenotazione) it.next();
				bloccaPrenotazione(utente, prenotazione, blocca);
			}
		} 
	}
}