package it.callite.dao;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import it.callite.constants.ConstantsFormat;
import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsMap;
import it.callite.dao.generic.GenericDao;
import it.callite.log.ErrorLog;
import it.callite.model.Anagrafica;
import it.callite.model.Campagna;
import it.callite.model.CampoAggiuntivoAnagrafica;
import it.callite.model.DatoAggiuntivoAnagrafica;
import it.callite.model.DomandaAnagrafica;
import it.callite.model.anagrafica.AnagraficaXML;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.CampoSpeciale;
import it.callite.utility.DatoAggiuntivo;
import it.callite.utility.DatoSondaggioAnagrafica;
import it.callite.utility.ValoreMappa;
@SuppressWarnings("serial")
@Repository("privacyDao")
public class PrivacyDao extends GenericDao {

	@Autowired
	private AnagraficaDao anagraficaDao;
	@Autowired
	private SondaggioDao sondaggioDao;
	@Autowired
	private ImpostazioneDao impostazioneDao;
	@Autowired
	private DownloadDao downloadDao;
	@Autowired
	private DataDao dataDao;
	@Autowired
	private ControllerDao controllerDao;


	public void privacy(Anagrafica anagrafica, boolean accetta, List<DatoSondaggioAnagrafica> dati, Integer tipo) {
		if(anagrafica != null) {
			Date oggi = ApplicationUtil.date();
			Date nuova = accetta ? ApplicationUtil.privacy(oggi, impostazioneDao.privacy()) : ApplicationUtil.inizioGiornata(oggi);
			try {
				anagrafica.setConsenso_tfisso(nuova);
				anagrafica.setConsenso_tmobile(nuova);
				anagrafica.setConsenso_tfax(nuova);
				anagrafica.setConsenso_taltro(nuova);
				anagrafica.setConsenso_email(nuova);
				anagrafica.setConsenso_rmail(nuova);
				anagraficaDao.salva(anagrafica);
				saveSondaggioToAnagrafica(dati, anagrafica, tipo);
			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		}
	}

	public void privacy(Anagrafica a, Date date, Long campagna, Long lista, List<DatoSondaggioAnagrafica> dati) {
		if(a != null) {
			try {
				AnagraficaXML anagrafica = new AnagraficaXML(a, dati);
				controllerDao.inviaContatto(anagrafica, campagna, lista, null, date);
			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		}
	}


	public Anagrafica getPrivacyById(String a) {
		try {
			String p = ApplicationUtil.base64Decoder(a);
			Long ana = Long.parseLong(p);
			return (Anagrafica) anagraficaDao.getAnagraficaById(ana);
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
		return null;
	}

	public Anagrafica getInputById(String campagna) {
		if(campagnaExist(campagna)) {
			return (Anagrafica) new Anagrafica();
		}
		return null;
	}

	public boolean campagnaExist(String campagna) {
		Long id = ApplicationUtil.asLong(campagna);
		if(ApplicationUtil.gtZero(id)) {
			DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
			criteria.add(Restrictions.idEq(id));
			return getListSize(criteria) > 0;
		}
		return false;
	}


	public String privacy(Anagrafica anagrafica) {
		return downloadDao.privacy(anagrafica);
	}

	@SuppressWarnings("unchecked")
	public List<DatoAggiuntivoAnagrafica> getDati(Anagrafica anagrafica) {
		DetachedCriteria criteria = DetachedCriteria.forClass(DatoAggiuntivoAnagrafica.class);
		criteria.add(Restrictions.eq("anagrafica.id",anagrafica.getId()));
		return getList(criteria);
	}
	public List<DatoSondaggioAnagrafica> getSondaggioForm(Anagrafica anagrafica, Integer tipo, String sondaggio) {
		List<DatoSondaggioAnagrafica> dati = new ArrayList<DatoSondaggioAnagrafica>();
		Map<String, DatoAggiuntivo> valori = new HashMap<String, DatoAggiuntivo>();
		List<DatoAggiuntivoAnagrafica> dA = new ArrayList<DatoAggiuntivoAnagrafica>();
		if(anagrafica != null) {
			List <String> valoriValidi = new ArrayList<String>(ConstantsMap.anagraficaMap);
			valori.putAll(ApplicationUtil.convertiAnagrafica(anagrafica, valoriValidi));
			List<DatoAggiuntivoAnagrafica> datiV = getDati(anagrafica);
			if(ApplicationUtil.notEmpty(datiV)) {
				dA.addAll(datiV);
			}
		}
		if(ApplicationUtil.notEmpty(dA)) {
			valori.putAll(ApplicationUtil.convertiDatoAggiuntivoA(dA));

			List<DomandaAnagrafica> domandeR = getRisposteDomandeSondaggio(anagrafica);
			if(ApplicationUtil.eq(tipo, 1) || ApplicationUtil.eq(tipo, 2)) {
				Map<String,Object> valoriS = new HashMap<String,Object>();
				Map<String,List<String>> valoriM = new HashMap<String,List<String>>();
				List<DomandaAnagrafica> domandeT = getTutteDomandeSondaggio(sondaggio);
				if(ApplicationUtil.notEmpty(domandeR)) {
					Iterator<DomandaAnagrafica> objDocIter = domandeR.iterator();
					while(objDocIter.hasNext()) {
						DomandaAnagrafica domanda = objDocIter.next();
						if(ApplicationUtil.isNotBlank(domanda.getCampo_aggiuntivo().getGruppo())) {
							String risposta = domanda.getRisposta();
							if(domanda.getCampo_aggiuntivo().getManyCheckbox() || domanda.getCampo_aggiuntivo().getManyMenu() || domanda.getCampo_aggiuntivo().getPickList()) {
								valoriM.put(domanda.getCampo_aggiuntivo().getChiave(), ApplicationUtil.token(risposta, ConstantsGlobal.colonSeparator));
							} else if(domanda.getCampo_aggiuntivo().getData()) {
								valoriS.put(domanda.getCampo_aggiuntivo().getChiave(), ApplicationUtil.parseData(domanda.getCampo_aggiuntivo().getValoriT(), risposta));
							} else if(domanda.getCampo_aggiuntivo().getDataora()) {
								valoriS.put(domanda.getCampo_aggiuntivo().getChiave(), ApplicationUtil.parseData(domanda.getCampo_aggiuntivo().getValoriT(), risposta));
							} else if(domanda.getCampo_aggiuntivo().getOra()) {
								valoriS.put(domanda.getCampo_aggiuntivo().getChiave(), ApplicationUtil.parseData(ConstantsFormat.time.stampa(), risposta));
							} else {
								valoriS.put(domanda.getCampo_aggiuntivo().getChiave(), risposta);
							}
						}
					}
				}
				if(ApplicationUtil.notEmpty(domandeT)) {
					Iterator<DomandaAnagrafica> objDocIter = domandeT.iterator();
					while(objDocIter.hasNext()) {
						DomandaAnagrafica domanda = objDocIter.next();
						DatoSondaggioAnagrafica datoS = null;
						if(domandeR.contains(domanda)){
							int index = domandeR.indexOf(domanda);
							if(index > -1) {
								DomandaAnagrafica d = domandeR.get(index);
								String risposta = d.getRisposta();
								datoS = ApplicationUtil.creaDomanda(domanda, risposta);

							}
						} else {
							String risposta = ApplicationUtil.rispondi(domanda,valori);
							datoS = ApplicationUtil.creaDomanda(domanda, risposta);

						}
						if(datoS != null) {
							sondaggioDao.addDomanda(dati, datoS);
							if(domanda.getCampo_aggiuntivo().getForm()) {
								datoS.setValoriM(valoriM);
								datoS.setValoriS(valoriS);
							}
						}
					}
				}
			}
			if(ApplicationUtil.eq(tipo, 2) || ApplicationUtil.eq(tipo, 3)) {
				if(ApplicationUtil.notEmpty(domandeR)) {
					Iterator<DomandaAnagrafica> objDocIter = domandeR.iterator();
					while(objDocIter.hasNext()) {
						DomandaAnagrafica domanda = objDocIter.next();
						if((ApplicationUtil.eq(tipo, 2) && ApplicationUtil.isBlank(domanda.getCampo_aggiuntivo().getGruppo())) || ApplicationUtil.eq(tipo, 3)) {
							String risposta = domanda.getRisposta();
							sondaggioDao.addDomanda(dati, ApplicationUtil.creaDomanda(domanda, risposta));
						}
					}
				}
			}
		}
		return dati;
	}


	public List<DatoSondaggioAnagrafica> getSondaggioForm(Anagrafica anagrafica, String sondaggio) {
		List<DatoSondaggioAnagrafica> dati = new ArrayList<DatoSondaggioAnagrafica>();
		List<DomandaAnagrafica> domandeT = getTutteDomandeSondaggio(sondaggio);
		if(ApplicationUtil.notEmpty(domandeT)) {
			Iterator<DomandaAnagrafica> objDocIter = domandeT.iterator();
			while(objDocIter.hasNext()) {
				DomandaAnagrafica domanda = objDocIter.next();
				DatoSondaggioAnagrafica datoS = ApplicationUtil.creaDomanda(domanda, null);
				sondaggioDao.addDomanda(dati, datoS);
			}
		}
		return dati;
	}

	@SuppressWarnings("unchecked")
	public List<DomandaAnagrafica> getRisposteDomandeSondaggio(Anagrafica anagrafica) {
		List<DomandaAnagrafica> dms = new ArrayList<DomandaAnagrafica>();
		DetachedCriteria criteriaC = DetachedCriteria.forClass(DatoAggiuntivoAnagrafica.class);
		criteriaC.add(Restrictions.eq("anagrafica.id", anagrafica.getId()));
		List<DatoAggiuntivoAnagrafica> das = getList(criteriaC);
		if(ApplicationUtil.notEmpty(das)) {
			Iterator<DatoAggiuntivoAnagrafica> objIter = das.iterator();
			while(objIter.hasNext()) {
				DatoAggiuntivoAnagrafica da = objIter.next();
				DomandaAnagrafica dm = new DomandaAnagrafica();
				dm.setCampo_aggiuntivo(da.getCampo_aggiuntivo());
				dm.setRisposta(da.getValore());
				dm.setObbligatoria(da.getCampo_aggiuntivo().getObbligatorio());
				dm.setVisibile(true);
				dms.add(dm);
			}
		}
		return dms;
	}

	@SuppressWarnings("unchecked")
	public List<DomandaAnagrafica> getTutteDomandeSondaggio(String s) {
		List<Long> ls = ApplicationUtil.tokenL(s, ConstantsGlobal.colonSeparator);
		if(ApplicationUtil.notEmpty(ls)) {
			DetachedCriteria criteriaC = DetachedCriteria.forClass(DomandaAnagrafica.class);
			DetachedCriteria criteriaS = criteriaC.createCriteria("sondaggio");
			criteriaS.add(Restrictions.in("id", ls));
			criteriaC.addOrder(Order.asc("posizione"));
			return getList(criteriaC);
		}
		return new ArrayList<DomandaAnagrafica>();
	}


	public void saveSondaggioToAnagrafica(List<DatoSondaggioAnagrafica> dati, Anagrafica anagrafica, Integer tipo) {
		if(anagrafica != null && ApplicationUtil.gtZero(tipo) && ApplicationUtil.notEmpty(dati)) {
			List<DatoAggiuntivoAnagrafica> dr = new ArrayList<DatoAggiuntivoAnagrafica>();
			dr.addAll(getDomandeRisposte(anagrafica));
			Iterator<DatoSondaggioAnagrafica> objIter = dati.iterator();
			while(objIter.hasNext()) {
				DatoSondaggioAnagrafica dato = objIter.next();
				if (dato.getCampo() != null) {
					if (dato.getCampo().getForm()) {
						Map<CampoSpeciale,ValoreMappa> mappa = dato.getMappa();
						if (ApplicationUtil.notEmpty(mappa)) {
							Iterator<CampoSpeciale> ssIt = mappa.keySet().iterator();
							while(ssIt.hasNext()) {
								CampoSpeciale cc = ssIt.next();
								ValoreMappa val = mappa.get(cc);
								String chiave = ApplicationUtil.trimToEmpty(cc.getChiave()).replaceAll(ConstantsGlobal.colonSeparator, " ");
								if(val!=null) {
									if(ApplicationUtil.isBlank(cc.getEtichetta())) {
										cc.setEtichetta(chiave);
									}
									CampoAggiuntivoAnagrafica campo = dato.getCampo();
									String nuovaChiave = ApplicationUtil.trimToEmpty(ApplicationUtil.impostaCampoS(cc.getEredita(),campo.getChiave(), chiave, ConstantsGlobal.ereditaChiave));
									if(!ConstantsMap.anagraficaMap.contains(nuovaChiave)){
										CampoAggiuntivoAnagrafica caa = anagraficaDao.getCampoAggiuntivoByChiave(nuovaChiave);
										if (caa == null) {
											caa = new CampoAggiuntivoAnagrafica();
											caa.setChiave(nuovaChiave);
											String gruppo = ApplicationUtil.impostaGruppo(campo.getEtichetta(), campo.getColore());
											caa.setGruppo(gruppo);
											caa.setEtichetta(ApplicationUtil.impostaCampoS(cc.getEredita(),campo.getEtichetta(), cc.getEtichetta(), ConstantsGlobal.ereditaEtichetta));
											caa.setDescrizione(ApplicationUtil.impostaCampoC(cc.getEredita(),campo.getDescrizione(), cc.getDescrizione()));
											caa.setModificabile_operatore(ApplicationUtil.impostaCampoB(cc.getEredita(), campo.getModificabile_operatore(), cc.getModificabile_operatore()));
											caa.setVisibile_operatore(ApplicationUtil.impostaCampoB(cc.getEredita(), campo.getVisibile_operatore(), cc.getVisibile_operatore()));
											caa.setModificabile_agente(ApplicationUtil.impostaCampoB(cc.getEredita(), campo.getModificabile_agente(), cc.getModificabile_agente()));
											caa.setVisibile_agente(ApplicationUtil.impostaCampoB(cc.getEredita(), campo.getVisibile_agente(), cc.getVisibile_agente()));
											caa.setObbligatorio(ApplicationUtil.impostaCampoB(cc.getEredita(), campo.getObbligatorio(), cc.getObbligatorio()));
											caa.setColore(ApplicationUtil.impostaCampoC(cc.getEredita(), campo.getColore(), cc.getColore()));
											caa.setStile(ApplicationUtil.impostaCampoC(cc.getEredita(), campo.getStile(), cc.getStile()));
											caa.setTipo(ApplicationUtil.gtZero(caa.getTipo()) ? cc.getTipo() : ConstantsMap.TIPO_APERTO);
											caa.setDefaults(ApplicationUtil.impostaCampoC(cc.getEredita(), campo.getDefaults(), cc.getDefaults()));
											if(caa.getOneMenu() || caa.getInplaceSelect() || caa.getOneRadio() || caa.getSuggerimento() || caa.getPickList() || caa.getManyCheckbox() || caa.getManyMenu() || caa.getCombo()) {
												Set<String> valori = new HashSet<String>();
												valori.addAll(cc.getValuesList());
												if(val.getMultiplo() != null && (caa.getPickList() || caa.getManyCheckbox() || caa.getManyMenu())) {
													valori.addAll(val.getMultiplo());
												} 
												if(val.getSingolo() != null && (caa.getCombo() || caa.getOneMenu() || caa.getInplaceSelect() || caa.getOneRadio() || caa.getSuggerimento())) {
													valori.add(ApplicationUtil.trimToEmpty((String) val.getSingolo()));
												} 
												valori.removeAll(ApplicationUtil.asList("", null));
												caa.setValori(ApplicationUtil.detoken(new ArrayList<String>(valori), ConstantsGlobal.colonSeparator));
											}
											if(caa.getData() ||caa.getDataora() || caa.getRegulare() || caa.getUnivoco()) {
												caa.setValori(cc.getValues());
											} 
											caa = (CampoAggiuntivoAnagrafica) anagraficaDao.salvaCampoAggiuntivo(caa);
											dataDao.addCampo(caa);
										} else {
											boolean save = false;
											if(caa.getOneMenu() || caa.getInplaceSelect() || caa.getOneRadio() || caa.getSuggerimento() || caa.getPickList() || caa.getManyCheckbox() || caa.getManyMenu() || caa.getCombo()) {
												Set<String> vn = new HashSet<String>();
												vn.addAll(caa.getValoriList());
												vn.addAll(cc.getValuesList());
												if(val.getMultiplo() != null && (caa.getPickList() || caa.getManyCheckbox() || caa.getManyMenu())) {
													vn.addAll(val.getMultiplo());
												}
												if(val.getSingolo() != null && (caa.getCombo() || caa.getOneMenu() || caa.getInplaceSelect() || caa.getOneRadio() || caa.getSuggerimento())) {
													vn.add(ApplicationUtil.trimToEmpty((String) val.getSingolo()));
												}
												vn.removeAll(ApplicationUtil.asList("", null));
												Set<String> vl = new HashSet<String>(caa.getValoriList());
												if(!vl.containsAll(vn)) {
													vl.addAll(vn);
													caa.setValori(ApplicationUtil.detoken( new ArrayList<String>(vl), ConstantsGlobal.colonSeparator));
													save = true;
												}
											}
											if(caa.getData() || caa.getDataora() || caa.getRegulare() || caa.getUnivoco()) {
												String v = ApplicationUtil.trimToEmpty(cc.getValues());
												if(!ApplicationUtil.equals(caa.getValori(), v)) {
													caa.setValori(v);
													save = true;
												}
											} 
											if(!ApplicationUtil.equals(caa.getTipo(), cc.getTipo())) {
												caa.setTipo(cc.getTipo());
												save = true;
											}
											String gruppo = ApplicationUtil.impostaGruppo(campo.getEtichetta(), campo.getColore());
											if(!ApplicationUtil.equals(caa.getGruppo(), gruppo)) {
												caa.setGruppo(gruppo);
												save = true;
											}
											Boolean b1 = ApplicationUtil.impostaCampoB(cc.getEredita(), campo.getObbligatorio(), cc.getObbligatorio());
											if(!ApplicationUtil.equals(caa.getObbligatorio(), b1)) {
												caa.setObbligatorio(b1);
												save = true;
											}
											Boolean o1 = ApplicationUtil.impostaCampoB(cc.getEredita(), campo.getModificabile_operatore(), cc.getModificabile_operatore());
											if(!ApplicationUtil.equals(caa.getModificabile_operatore(), o1)) {
												caa.setModificabile_operatore(o1);
												save = true;
											}
											Boolean o2 = ApplicationUtil.impostaCampoB(cc.getEredita(), campo.getVisibile_operatore(), cc.getVisibile_operatore());
											if(!ApplicationUtil.equals(caa.getVisibile_operatore(), o2)) {
												caa.setVisibile_operatore(o2);
												save = true;
											}
											Boolean a1 = ApplicationUtil.impostaCampoB(cc.getEredita(), campo.getModificabile_agente(), cc.getModificabile_agente());
											if(!ApplicationUtil.equals(caa.getModificabile_agente(), a1)) {
												caa.setModificabile_agente(a1);
												save = true;
											}
											Boolean a2 = ApplicationUtil.impostaCampoB(cc.getEredita(), campo.getVisibile_agente(), cc.getVisibile_agente());
											if(!ApplicationUtil.equals(caa.getVisibile_agente(), a2)) {
												caa.setVisibile_agente(a2);
												save = true;
											}
											String e = ApplicationUtil.impostaCampoS(cc.getEredita(),campo.getEtichetta(), cc.getEtichetta(), ConstantsGlobal.ereditaEtichetta);
											if(!ApplicationUtil.equals(caa.getEtichetta(), e)) {
												caa.setEtichetta(e);
												save = true;
											}
											String h = ApplicationUtil.impostaCampoC(cc.getEredita(), campo.getDescrizione(), cc.getDescrizione());
											if(!ApplicationUtil.equals(caa.getDescrizione(), h)) {
												caa.setDescrizione(h);
												save = true;
											}
											String c = ApplicationUtil.impostaCampoC(cc.getEredita(), campo.getColore(), cc.getColore());
											if(!ApplicationUtil.equals(caa.getColore(), c)) {
												caa.setColore(c);
												save = true;
											}
											String s = ApplicationUtil.impostaCampoC(cc.getEredita(), campo.getStile(), cc.getStile());
											if(!ApplicationUtil.equals(caa.getStile(), s)) {
												caa.setStile(s);
												save = true;
											}
											String d = ApplicationUtil.impostaCampoC(cc.getEredita(), campo.getDefaults(), cc.getDefaults());
											if(!ApplicationUtil.equals(caa.getDefaults(), d)) {
												caa.setDefaults(d);
												save = true;
											}
											if(save) {
												caa = (CampoAggiuntivoAnagrafica) anagraficaDao.salvaCampoAggiuntivo(caa);
											}
										}
										DatoAggiuntivoAnagrafica daa = new DatoAggiuntivoAnagrafica();
										daa.setCampo_aggiuntivo(caa);
										if(!dr.contains(daa)){
											daa.setAnagrafica(anagrafica);
										} else {
											int index = dr.indexOf(daa);
											if(index > -1) {
												daa = dr.get(index);
											}
										}
										if(caa.getData()) {
											daa.setValore_data(ApplicationUtil.castDate(val.getSingolo()));
										} else if(caa.getDataora()) {
											daa.setValore_datatime(ApplicationUtil.castDate(val.getSingolo()));
										} else if(caa.getOra()) {
											daa.setValore_time(ApplicationUtil.castDate(val.getSingolo()));
										} else if(caa.getBooleano()) {
											daa.setValore(ApplicationUtil.castBoolean(val.getSingolo()));
										} else if(caa.getPickList() || caa.getManyCheckbox() || caa.getManyMenu()) {
											daa.setValore(ApplicationUtil.detoken(val.getMultiplo(), ConstantsGlobal.colonSeparator));
										} else {
											daa.setValore(ApplicationUtil.castString(val.getSingolo()));
										}
										anagraficaDao.salvaDatoAggiuntivo(daa);
									}
								}
							}
						}
					} else {
						DatoAggiuntivoAnagrafica daa = new DatoAggiuntivoAnagrafica();
						daa.setCampo_aggiuntivo(dato.getCampo());
						if(!dr.contains(daa)){
							daa.setAnagrafica(anagrafica);
						} else {
							int index = dr.indexOf(daa);
							if(index > -1) {
								daa = dr.get(index);
							}
						}
						if(dato.getCampo().getManyCheckbox() || dato.getCampo().getManyMenu() || dato.getCampo().getPickList()) {
							daa.setValore(ApplicationUtil.detoken(dato.getValore_multiplo(), ConstantsGlobal.colonSeparator));
						} else if(dato.getCampo().getData()) {
							daa.setValore_data(dato.getValore_data());
						} else if(dato.getCampo().getDataora()) {
							daa.setValore_datatime(dato.getValore_datatime());
						} else if(dato.getCampo().getOra()) {
							daa.setValore_time(dato.getValore_time());
						} else {
							daa.setValore(dato.getValore());
						}
						anagraficaDao.salvaDatoAggiuntivo(daa);
					}
				}
			}
		}
	}

	@SuppressWarnings("unchecked")
	public List<DatoAggiuntivoAnagrafica> getDomandeRisposte(Anagrafica anagrafica) {
		DetachedCriteria criteriaC = DetachedCriteria.forClass(DatoAggiuntivoAnagrafica.class);
		criteriaC.add(Restrictions.eq("anagrafica.id", anagrafica.getId()));
		criteriaC.createCriteria("campo_aggiuntivo").add(Restrictions.ne("tipo", ConstantsMap.TIPO_FORM));
		return getList(criteriaC);
	}

}