package it.callite.dao;
import java.io.File;
import java.io.FileInputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

import javax.faces.model.SelectItem;
import javax.servlet.ServletContext;

import org.hibernate.FetchMode;
import org.hibernate.Session;
import org.hibernate.Transaction;
import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Projections;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.FileSystemResource;
import org.springframework.stereotype.Repository;

import it.callite.action.DownloadBean;
import it.callite.comparator.FileComparator;
import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.constants.ConstantsMap;
import it.callite.criteria.FileCriteria;
import it.callite.criteria.ProdottoCriteria;
import it.callite.criteria.base.ExtendedCriteria;
import it.callite.dao.generic.GenericDao;
import it.callite.log.ErrorLog;
import it.callite.model.Campagna;
import it.callite.model.Categoria;
import it.callite.model.Prodotto;
import it.callite.model.SondaggioContratto;
import it.callite.model.Tema;
import it.callite.model.Template;
import it.callite.model.TemplateProdotto;
import it.callite.model.VocalOrder;
import it.callite.model.pk.TemplateProdottoPK;
import it.callite.utility.Allegato;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.FileUpload;
import it.callite.utility.MyFile;
import it.callite.utility.Parameter;
import it.callite.utility.item.GenericSI;
import it.callite.utility.item.ProdottoSI;
@SuppressWarnings("serial")
@Repository("prodottoDao")
public class ProdottoDao  extends GenericDao {
	@Autowired
	private ImpostazioneDao impostazioneDao;
	@Autowired
	private UserDao userDao;
	@Autowired
	private ContrattoDao contrattoDao;
	@Autowired
	private CampagnaDao campagnaDao;
	@Autowired
	private VocalOrderDao vocalOrderDao;
	@Autowired
	private ServletContext servletContext; 
	public Prodotto salva(Prodotto prodotto) {
		if(!prodotto.getVendita()) {
			prodotto.notVendita();
		}
		return (Prodotto) salvaAggiorna(prodotto);
	}
	public void scaricaImmagine(List<MyFile> mFiles, DownloadBean downloadBean) {
		String percorso_immagini = impostazioneDao.getPercorsoImmagini();
		try {
			List<Allegato> files = new ArrayList<Allegato>();
			if (ApplicationUtil.notEmpty(mFiles)) {
				String p = ConstantsGlobal.secretZ;
				Iterator<MyFile> itS = mFiles.iterator();
				while (itS.hasNext()) {
					MyFile file = (MyFile) itS.next();
					ApplicationUtil.addFile(files, file.getName(), percorso_immagini+"/"+file.getName(), file.getModified());
				}
				String fileName = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.IMMAGINI.stampa()) +"." + ConstantsMap.zip;
				OutputStream out = downloadBean.outputStream();
				downloadBean.filename(fileName, files);

				ApplicationUtil.zip(null, files, out, p);
				downloadBean.close();

			}
		} catch(Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void scaricaImmagine(MyFile d, DownloadBean downloadBean) {
		String percorso_immagini = impostazioneDao.getPercorsoImmagini();
		try {
			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(d.getName());

			FileInputStream fi = new FileInputStream(percorso_immagini+"/"+ d.getName());
			ApplicationUtil.copy(fi, out);
			fi.close();
			downloadBean.close();

		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void salvaImmagine(FileUpload fileUpload, Prodotto prodotto) {
		String file = getNuovaImmagine();
		String path = impostazioneDao.getPercorsoImmagini() + "/";
		boolean b = ApplicationUtil.salvaImmagine(fileUpload.getFiles(), path + file, ConstantsGlobal.imageW, ConstantsGlobal.imageH);
		if(b) {
			Boolean l = impostazioneDao.getContentLogo();
			prodotto.setContent(ApplicationUtil.content(l, path, file));
		}
	}
	public void salvaImmagine(FileUpload fileUpload) {
		String file = getNuovaImmagine();
		String path = impostazioneDao.getPercorsoImmagini() + "/";
		ApplicationUtil.salvaImmagine(fileUpload.getFiles(), path + file, ConstantsGlobal.imageW, ConstantsGlobal.imageH);
	}
	public void eliminaImmagine(List<MyFile> myFile) {
		if(ApplicationUtil.notEmpty(myFile)) {
			Iterator<MyFile> it= myFile.iterator();
			while (it.hasNext()) {
				MyFile a = (MyFile) it.next();
				eliminaImmagine(a);
			}
		}
	}
	
	public void salvaImmagine(Prodotto prodotto, File logo) {
		String path = impostazioneDao.getPercorsoImmagini() + "/";
		Boolean l = impostazioneDao.getContentLogo();
		prodotto.setContent(ApplicationUtil.content(l, path, ApplicationUtil.getNameN(logo.getAbsolutePath())));
	}
	public void eliminaImmagine(MyFile myFile) {
		Boolean l = impostazioneDao.getContentLogo();
		if(ApplicationUtil.isNotTrue(l)) {
			Session session = openSession();
			Transaction tx = session.beginTransaction();
			String hql1 = "update Campagna set content = null where content = :content";
			String hql2 = "update Prodotto set content = null where content = :content";
			String hql3 = "update Tema set content = null where content = :content";
			execute(false, session, hql1, new Parameter("content", myFile.getName().getBytes(ConstantsGlobal.CHARSET)));
			execute(false, session, hql2, new Parameter("content", myFile.getName().getBytes(ConstantsGlobal.CHARSET)));
			execute(false, session, hql3, new Parameter("content", myFile.getName().getBytes(ConstantsGlobal.CHARSET)));
			session.flush();
			session.clear();
			tx.commit();
			closeSession(session);
		}
		String percorso_immagini = impostazioneDao.getPercorsoImmagini();
		File file = new File(percorso_immagini + "/" + myFile.getName());
		ApplicationUtil.deleteFile(file);
	}
	@SuppressWarnings("unchecked")
	public List<Campagna> getListaCampagneImmagine(MyFile myFile) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		criteria.add(Restrictions.eq("content",myFile.getData()));
		ApplicationUtil.orderCL(criteria);
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Prodotto> getListaProdottiImmagine(MyFile myFile) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Prodotto.class);
		criteria.add(Restrictions.eq("content",myFile.getData()));
		criteria.createCriteria("categoria", "categoria", ApplicationUtil.leftJoin());
		ApplicationUtil.orderP(criteria);
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Campagna> getListaNotInCampagneImmagine(MyFile myFile) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		criteria.add(Restrictions.isNull("content"));
		ApplicationUtil.orderCL(criteria);
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Prodotto> getListaNotInProdottiImmagine(MyFile myFile) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Prodotto.class);
		criteria.add(Restrictions.isNull("content"));
		criteria.createCriteria("categoria", "categoria", ApplicationUtil.leftJoin());
		ApplicationUtil.orderP(criteria);
		return getList(criteria);
	}
	public boolean salvaImmagineInternet(String url, Prodotto prodotto) {
		String file = getNuovaImmagine();
		String path = impostazioneDao.getPercorsoImmagini() + "/";
		boolean b = ApplicationUtil.salvaImmagine(url, path + file, ConstantsGlobal.imageW, ConstantsGlobal.imageH);
		if(b) {
			Boolean l = impostazioneDao.getContentLogo();
			prodotto.setContent(ApplicationUtil.content(l, path, file));
		}
		return b;
	}
	public boolean salvaImmagineInternet(String url) {
		String file = getNuovaImmagine();
		String path = impostazioneDao.getPercorsoImmagini() + "/";
		return ApplicationUtil.salvaImmagine(url, path + file, ConstantsGlobal.imageW, ConstantsGlobal.imageH);
	}
	
	public String getNuovaImmagine() {
		return System.currentTimeMillis()  + "." + ConstantsMap.png;
	}
	public List<File> listaImmagini() {
		String percorso_immagini = impostazioneDao.getPercorsoImmagini();
		String[] estensioni_immagini = ConstantsMap.a_immagini;
		List<File> files = new ArrayList<File>();
		File dir = new File(percorso_immagini);
		if(dir.exists() && dir.isDirectory()) {
			for(File file : dir.listFiles()) {
				if(ApplicationUtil.isExtensionN(file.getName(), estensioni_immagini)) {
					files.add(file);
				}
			}
		}
		Collections.sort(files, new FileComparator());
		return files;
	}
	public List<MyFile> listaImmagini(FileCriteria mm) {
		if(mm != null)  mm = (FileCriteria) mm.loadCriteria();
		List<MyFile> ff = new ArrayList<MyFile>();
		if(mm != null && mm.getCerca()) {
			String path = impostazioneDao.getPercorsoImmagini() + "/";
			String[] estensioni_immagini = ConstantsMap.a_immagini;
			File dir = new File(path);
			if(dir.exists() && dir.isDirectory()) {
				Boolean l = impostazioneDao.getContentLogo();
				for(File file : dir.listFiles()) {
					if(ApplicationUtil.isExtensionN(file.getName(), estensioni_immagini)) {
						if(mm.isRicerca(file.getName(), file.length(), file.lastModified())) {
							MyFile f = new MyFile(file);
							f.setName(file.getName());
							f.setModified(file.lastModified());
							f.setData(ApplicationUtil.content(l, path , file.getName()));
							ff.add(f);
						}
					}
				}
			}
		}
		return ff;
	}
	public List<File> listaImmaginiEsito() {
		FileSystemResource resource = new FileSystemResource(servletContext.getRealPath("/images/esito/"));
		File perc =  resource.getFile();
		List<File> files = new ArrayList<File>(Arrays.asList(perc.listFiles()));
		Collections.sort(files, new FileComparator());
		return files;
	}
	public Prodotto get(Long id) {
		return (Prodotto) get(Prodotto.class, id);
	}
	public void elimina(Prodotto prodotto) {
		cancellaAggiorna(prodotto);
	}
	@SuppressWarnings("unchecked")
	public List<Campagna> getCampagneByIdProdotto(Long idProdotto) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		criteria.createCriteria("prodottiInVendita").add(Restrictions.eq("id.prodotto.id",idProdotto));
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<TemplateProdotto> getTemplateByProdottoId(Long prodottoId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(TemplateProdotto.class);
		criteria.add(Restrictions.eq("attivo", Boolean.TRUE));
		criteria.add(Restrictions.eq("id.prodotto.id", prodottoId));
		return  getList(criteria);
	}
	
	@SuppressWarnings("unchecked")
	public boolean getTemplateByProdottoS(Long prodottoId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(TemplateProdotto.class);
		criteria.add(Restrictions.eq("attivo", Boolean.TRUE));
		criteria.add(Restrictions.eq("id.prodotto.id", prodottoId));
		List<TemplateProdotto> tp  = getList(criteria);
		if(ApplicationUtil.notEmpty(tp))
			for (TemplateProdotto t : tp) {
				Template tt = t.getId().getDocumento();
				if(tt.isFirmabile()) {
					return true;
				}
			}
		return false;
	}
	
	public void removeAssociationDocumenti(Long prodottoId, List<Long> documentiAssociati) {
		if(ApplicationUtil.notEmpty(documentiAssociati)) {
			String doc = ApplicationUtil.detoken(documentiAssociati, ConstantsGlobal.commaSeparator);
			String hql = "delete from TemplateProdotto where id.documento.id in ("+doc+") and id.prodotto.id = :prodotto";
			execute(false, hql, new Parameter("prodotto", prodottoId));
		}
	}
	public ExtendedCriteria getProdottoCriteria(ProdottoCriteria prd) {
		if(prd != null)  prd = (ProdottoCriteria) prd.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Prodotto.class);
		if (ApplicationUtil.isNotBlank(prd.getNome())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, prd.getNome(), criteria, "nome");
		}
		if (ApplicationUtil.isNotBlank(prd.getDescrizione())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, prd.getDescrizione(), criteria, "descrizione");
		}
		if (ApplicationUtil.gtZero(prd.getCategoria())) {
			criteria.add(Restrictions.eq("categoria.id", prd.getCategoria()));
		}
		prd.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, prd);
	}
	@SuppressWarnings("unchecked")
	public List<Prodotto> trova(ProdottoCriteria prd, int start, int size) {
		ExtendedCriteria criteria = getProdottoCriteria(prd);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int trovaSize(ProdottoCriteria prd) {
		ExtendedCriteria criteria = getProdottoCriteria(prd);
		return  getListSize(criteria);
	}
	public List<SelectItem> getProdottiSI() {
		DetachedCriteria criteria = DetachedCriteria.forClass(Prodotto.class);
		criteria.createCriteria("categoria", "categoria", ApplicationUtil.leftJoin());
		List<ProdottoSI> lis = getProdottoSI(criteria);
		return impostazioneDao.getProdotto(lis, true, true, null);
	}
	public List<SelectItem> getAllProdottiSI() {
		DetachedCriteria criteria = DetachedCriteria.forClass(Prodotto.class);
		criteria.createCriteria("categoria", "categoria", ApplicationUtil.leftJoin());
		List<ProdottoSI> lis = getProdottoSI(criteria);
		return impostazioneDao.getProdotto(lis, false, true, null);
	}
	@SuppressWarnings("unchecked")
	public List<SondaggioContratto> getSondaggioByIdProdotto(Long id) {
		DetachedCriteria criteria = DetachedCriteria.forClass(SondaggioContratto.class);
		criteria.createCriteria("prodotti").add(Restrictions.eq("id",id));
		return  getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<TemplateProdotto> getTemplateByIdProdotto(Long id) {
		DetachedCriteria criteria = DetachedCriteria.forClass(TemplateProdotto.class);
		criteria.add(Restrictions.eq("id.prodotto.id", id));
		return  getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<TemplateProdotto> getTemplateByIdDocumento(Long id) {
		DetachedCriteria criteria = DetachedCriteria.forClass(TemplateProdotto.class);
		criteria.add(Restrictions.eq("id.documento.id", id));
		return  getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<VocalOrder> getVocalByIdProdotto(Long id) {
		DetachedCriteria criteria = DetachedCriteria.forClass(VocalOrder.class);
		criteria.createCriteria("prodotti").add(Restrictions.eq("id",id));
		return  getList(criteria);
	}
	public void aggiungiProdottoAImmagine(MyFile file, Long prodottoId) {
		Prodotto prodotto = get(prodottoId);
		if (prodotto != null) {
			String path = impostazioneDao.getPercorsoImmagini() + "/";
			Boolean l = impostazioneDao.getContentLogo();
			prodotto.setContent(ApplicationUtil.content(l, path , file.getName()));
			salvaAggiorna(prodotto);
		}
	}
	public void rimuoviProdottoDaImmagine(Long prodottoId) {
		Prodotto prodotto = get(prodottoId);
		if (prodotto != null) {
			prodotto.setContent(null);
			salvaAggiorna(prodotto);
		}
	}
	public void aggiungiProdottoTuttiAImmagine(MyFile file) {
		String path = impostazioneDao.getPercorsoImmagini() + "/";
		List<Prodotto> prodotti = getListaNotInProdottiImmagine(file);
		if (ApplicationUtil.notEmpty(prodotti)) {
			Boolean l = impostazioneDao.getContentLogo();
			Iterator<Prodotto> it = prodotti.iterator();
			while (it.hasNext()) {
				Prodotto prodotto = (Prodotto) it.next();
				prodotto.setContent(ApplicationUtil.content(l, path, file.getName()));
				salvaAggiorna(prodotto);
			}
		}
	}
	public void rimuoviProdottoTuttiDaImmagine(MyFile file) {
		List<Prodotto> prodotti = getListaProdottiImmagine(file);
		if (ApplicationUtil.notEmpty(prodotti)) {
			Iterator<Prodotto> it = prodotti.iterator();
			while (it.hasNext()) {
				Prodotto prodotto = (Prodotto) it.next();
				prodotto.setContent(null);
				salvaAggiorna(prodotto);
			}
		}
	}
	public void aggiungiCampagnaAImmagine(MyFile file, Long campagnaId) {
		Campagna campagna = campagnaDao.getCampagna(campagnaId);
		if (campagna != null) {
			String path = impostazioneDao.getPercorsoImmagini() + "/";
			Boolean l = impostazioneDao.getContentLogo();
			campagna.setContent(ApplicationUtil.content(l, path, file.getName()));
			salvaAggiorna(campagna);
		}
	}
	public void rimuoviCampagnaDaImmagine(Long campagnaId) {
		Campagna campagna = campagnaDao.getCampagna(campagnaId);
		if (campagna != null) {
			campagna.setContent(null);
			salvaAggiorna(campagna);
		}
	}
	public void aggiungiCampagnaTuttiAImmagine(MyFile file) {
		List<Campagna> campagne = getListaNotInCampagneImmagine(file);
		if (ApplicationUtil.notEmpty(campagne)) {
			String path = impostazioneDao.getPercorsoImmagini() + "/";
			Boolean l = impostazioneDao.getContentLogo();
			Iterator<Campagna> it = campagne.iterator();
			while (it.hasNext()) {
				Campagna campagna = (Campagna) it.next();
				campagna.setContent(ApplicationUtil.content(l, path, file.getName()));
				salvaAggiorna(campagna);
			}
		}
	}
	public void rimuoviCampagnaTuttiDaImmagine(MyFile file) {
		List<Campagna> campagne = getListaCampagneImmagine(file);
		if (ApplicationUtil.notEmpty(campagne)) {
			Iterator<Campagna> it = campagne.iterator();
			while (it.hasNext()) {
				Campagna campagna = (Campagna) it.next();
				campagna.setContent(null);
				salvaAggiorna(campagna);
			}
		}
	}
	
	public List<SelectItem> getProdottoReport(Long categoria, Long userId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Prodotto.class);
		DetachedCriteria catc = criteria.createCriteria("categoria", "categoria", ApplicationUtil.leftJoin());
		catc.add(Restrictions.eq("id", categoria));
		if(ApplicationUtil.gtZero(userId)) {
			DetachedCriteria c =  criteria.createCriteria("prodotti_in_vendita");
			impostazioneDao.sottoCriteriaCampagnaSupervisore(userId, c, "id.campagna.id");
		}
		List<ProdottoSI> lis = getProdottoSI(criteria);
		return impostazioneDao.getProdotto(lis, false, false, null);
	}

	public List<SelectItem> getCategoriaReport(Long userId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Categoria.class);
		DetachedCriteria c =  criteria.createCriteria("prodotti");
		if(ApplicationUtil.gtZero(userId)) {
			DetachedCriteria d = c.createCriteria("prodotti_in_vendita");
			impostazioneDao.sottoCriteriaCampagnaSupervisore(userId, d, "id.campagna.id");
		}
		List<GenericSI> lis = getGenericSI(criteria, "id", "nome");
		return impostazioneDao.getGeneric(lis, false);
	}

	public List<SelectItem> getCategoriaManager(Long userId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Categoria.class);
		criteria.createCriteria("prodotti");
		List<GenericSI> lis = getGenericSI(criteria, "id", "nome");
		return impostazioneDao.getGeneric(lis, false);
	}
	public List<SelectItem> getProdottoManager(Long singolo, List<String> multiplo, Long userId) {
		Boolean filtro = impostazioneDao.getFiltroMultiplo();
		DetachedCriteria criteria = DetachedCriteria.forClass(Prodotto.class);
		DetachedCriteria catc = criteria.createCriteria("categoria", "categoria", ApplicationUtil.leftJoin());
		ApplicationUtil.proprieta(false, filtro, catc, "id", singolo, multiplo);
		List<ProdottoSI> lis = getProdottoSI(criteria);
		return impostazioneDao.getProdotto(lis, false, false, null);
	}
	public List<SelectItem> getCategoriaSupervisore(Long userId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Categoria.class);
		DetachedCriteria c =  criteria.createCriteria("prodotti").createCriteria("prodotti_in_vendita");
		impostazioneDao.sottoCriteriaCampagnaSupervisore(userId, c, "id.campagna.id");
		List<GenericSI> lis = getGenericSI(criteria, "id", "nome");
		return impostazioneDao.getGeneric(lis, false);
	}
	
	public List<SelectItem> getProdottoSupervisore(Long singolo, List<String> multiplo, Long userId) {
		Boolean filtro = impostazioneDao.getFiltroMultiplo();
		DetachedCriteria criteria = DetachedCriteria.forClass(Prodotto.class);
		DetachedCriteria catc = criteria.createCriteria("categoria", "categoria", ApplicationUtil.leftJoin());
		ApplicationUtil.proprieta(false, filtro, catc, "id", singolo, multiplo);
			if(ApplicationUtil.gtZero(userId)) {
				DetachedCriteria c =  criteria.createCriteria("prodotti_in_vendita");
				impostazioneDao.sottoCriteriaCampagnaSupervisore(userId, c, "id.campagna.id");
			}
		
		List<ProdottoSI> lis = getProdottoSI(criteria);
		return impostazioneDao.getProdotto(lis, false, false, null);
	}
	public List<SelectItem> getCategoriaPartner(Long userId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Categoria.class);
		DetachedCriteria c =  criteria.createCriteria("prodotti").createCriteria("prodotti_in_vendita");
		impostazioneDao.subCriteriaCampagnaPartner(userId, c, "id.campagna.id");
		List<GenericSI> lis = getGenericSI(criteria, "id", "nome");
		return impostazioneDao.getGeneric(lis, false);
	}
	public List<SelectItem> getProdottoPartner(Long singolo, List<String> multiplo, Long userId) {
		Boolean filtro = impostazioneDao.getFiltroMultiplo();
		DetachedCriteria criteria = DetachedCriteria.forClass(Prodotto.class);
		DetachedCriteria catc = criteria.createCriteria("categoria", "categoria", ApplicationUtil.leftJoin());
		ApplicationUtil.proprieta(false, filtro, catc, "id", singolo, multiplo);
			if(ApplicationUtil.gtZero(userId)) {
				DetachedCriteria c =  criteria.createCriteria("prodotti_in_vendita");
				impostazioneDao.subCriteriaCampagnaPartner(userId, c, "id.campagna.id");
			}
		
		List<ProdottoSI> lis = getProdottoSI(criteria);
		return impostazioneDao.getProdotto(lis, false, false, null);
	}
	public List<SelectItem> getCategoriaCoordinatore(Long userId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Categoria.class);
		DetachedCriteria c = criteria.createCriteria("prodotti").createCriteria("prodotti_in_vendita");
		impostazioneDao.subCriteriaCoordinatoreCampagna(userId, c, "id.campagna.id");
		List<GenericSI> lis = getGenericSI(criteria, "id", "nome");
		return impostazioneDao.getGeneric(lis, false);
	}
	public List<SelectItem> getProdottoCoordinatore(Long singolo, List<String> multiplo, Long userId) {
		Boolean filtro = impostazioneDao.getFiltroMultiplo();
		DetachedCriteria criteria = DetachedCriteria.forClass(Prodotto.class);
		DetachedCriteria catc = criteria.createCriteria("categoria", "categoria", ApplicationUtil.leftJoin());
		ApplicationUtil.proprieta(false, filtro, catc, "id", singolo, multiplo);
			if(ApplicationUtil.gtZero(userId)) {
				DetachedCriteria c = criteria.createCriteria("prodotti_in_vendita");
				impostazioneDao.subCriteriaCoordinatoreCampagna(userId, c , "id.campagna.id");
			}
		
		List<ProdottoSI> lis = getProdottoSI(criteria);
		return impostazioneDao.getProdotto(lis, false, false, null);
	}
	public List<SelectItem> getCategoriaOperatore(Long userId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Categoria.class);
		DetachedCriteria c = criteria.createCriteria("prodotti").createCriteria("prodotti_in_vendita");
		impostazioneDao.subCriteriaCampagnaOperatore(userId, c, "id.campagna.id");
		List<GenericSI> lis = getGenericSI(criteria, "id", "nome");
		return impostazioneDao.getGeneric(lis, false);
	}
	public List<SelectItem> getProdottoOperatore(Long singolo, List<String> multiplo, Long userId) {
		Boolean filtro = impostazioneDao.getFiltroMultiplo();
		DetachedCriteria criteria = DetachedCriteria.forClass(Prodotto.class);
		DetachedCriteria catc = criteria.createCriteria("categoria", "categoria", ApplicationUtil.leftJoin());
		ApplicationUtil.proprieta(false, filtro, catc, "id", singolo, multiplo);
		if(ApplicationUtil.gtZero(userId)) {
			DetachedCriteria c = criteria.createCriteria("prodotti_in_vendita");
			impostazioneDao.subCriteriaCampagnaOperatore(userId, c, "id.campagna.id");
		}
		List<ProdottoSI> lis = getProdottoSI(criteria);
		return impostazioneDao.getProdotto(lis, false, false, null);
	}
	public List<SelectItem> getCategoriaAgente(Long userId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Categoria.class);
		DetachedCriteria c = criteria.createCriteria("prodotti").createCriteria("prodotti_in_vendita");
		impostazioneDao.subCriteriaCampagnaAgente(userId, c, "id.campagna.id");
		List<GenericSI> lis = getGenericSI(criteria, "id", "nome");
		return impostazioneDao.getGeneric(lis, false);
	}
	public List<SelectItem> getProdottoAgente(Long singolo, List<String> multiplo, Long userId) {
		Boolean filtro = impostazioneDao.getFiltroMultiplo();
		DetachedCriteria criteria = DetachedCriteria.forClass(Prodotto.class);
		DetachedCriteria catc = criteria.createCriteria("categoria", "categoria", ApplicationUtil.leftJoin());
		ApplicationUtil.proprieta(false, filtro, catc, "id", singolo, multiplo);
		if(ApplicationUtil.gtZero(userId)) {
			DetachedCriteria c = criteria.createCriteria("prodotti_in_vendita");
			impostazioneDao.subCriteriaCampagnaAgente(userId, c, "id.campagna.id");
		}
		List<ProdottoSI> lis = getProdottoSI(criteria);
		return impostazioneDao.getProdotto(lis, false, false, null);
	}
	public List<SelectItem> getCategoriaRete(Long userId) {
		Integer max = userDao.getMaxLivelloRete();
		DetachedCriteria criteria = DetachedCriteria.forClass(Categoria.class);
		DetachedCriteria c = criteria.createCriteria("prodotti").createCriteria("prodotti_in_vendita");
		impostazioneDao.sottoCriteriaCampagnaRete(userId, max, c, "id.campagna.id");
		List<GenericSI> lis = getGenericSI(criteria, "id", "nome");
		return impostazioneDao.getGeneric(lis, false);
	}
	public List<SelectItem> getProdottoRete(Long singolo, List<String> multiplo, Long userId) {
		Boolean filtro = impostazioneDao.getFiltroMultiplo();
		DetachedCriteria criteria = DetachedCriteria.forClass(Prodotto.class);
		DetachedCriteria catc = criteria.createCriteria("categoria", "categoria", ApplicationUtil.leftJoin());
		ApplicationUtil.proprieta(false, filtro, catc, "id", singolo, multiplo);
		if(ApplicationUtil.gtZero(userId)) {
			Integer max = userDao.getMaxLivelloRete();
			DetachedCriteria c = criteria.createCriteria("prodotti_in_vendita");
			impostazioneDao.sottoCriteriaCampagnaRete(userId, max, c, "id.campagna.id");
		}
		List<ProdottoSI> lis = getProdottoSI(criteria);
		return impostazioneDao.getProdotto(lis, false, false, null);
	}
	public void aggiungiSondaggioContrattoAdProdotto(Long sondaggioId, Long prodottoId) {
		SondaggioContratto sondaggio = getFullSondaggioContrattoProdotto(sondaggioId);
		Prodotto prodotto = get(prodottoId);
		if (sondaggio != null && prodotto != null) {
			if (!sondaggio.getProdotti().contains(prodotto)) {
				sondaggio.getProdotti().add(prodotto);
				salvaAggiorna(sondaggio);
			}
		}
	}
	public void rimuoviSondaggioContrattoAdProdotto(Long sondaggioId, Long prodottoId) {
		SondaggioContratto sondaggio = getFullSondaggioContrattoProdotto(sondaggioId);
		Prodotto prodotto = get(prodottoId);
		if (sondaggio != null && prodotto != null) {
			if (sondaggio.getProdotti().contains(prodotto)) {
				sondaggio.getProdotti().remove(prodotto);
				salvaAggiorna(sondaggio);
			}
		}
	}
	public void aggiungiProdottoTuttiAdSondaggioContratto(Long sondaggioId) {
		SondaggioContratto sondaggio  = getFullSondaggioContrattoProdotto(sondaggioId);
		List<Prodotto> prodotto = getProdottoNotInSondaggioContratto(sondaggioId);
		if (ApplicationUtil.notEmpty(prodotto) && sondaggio != null) {
			sondaggio.getProdotti().addAll(prodotto);
			salvaAggiorna(sondaggio);
		}
	}
	public void rimuoviProdottoTuttiAdSondaggioContratto(Long sondaggioId) {
		SondaggioContratto sondaggio  = getFullSondaggioContrattoProdotto(sondaggioId);
		List<Prodotto> prodotto = getProdottoInSondaggioContratto(sondaggioId);
		if (ApplicationUtil.notEmpty(prodotto) && sondaggio != null) {
			sondaggio.getProdotti().removeAll(prodotto);
			salvaAggiorna(sondaggio);
		}
	}
	public void aggiungiProdottoAdSondaggioContratto(Long prodottoId, Long sondaggioId) {
		Prodotto prodotto = getFullProdottoSondaggioContratto(prodottoId);
		SondaggioContratto sondaggio = contrattoDao.get(sondaggioId);
		if (prodotto != null && sondaggio != null) {
			if (!prodotto.getSondaggi().contains(sondaggio)) {
				prodotto.getSondaggi().add(sondaggio);
				salvaAggiorna(prodotto);
			}
		}
	}
	public void rimuoviProdottoAdSondaggioContratto(Long prodottoId, Long sondaggioId) {
		Prodotto prodotto = getFullProdottoSondaggioContratto(prodottoId);
		SondaggioContratto sondaggio = contrattoDao.get(sondaggioId);
		if (prodotto != null && sondaggio != null) {
			if (prodotto.getSondaggi().contains(sondaggio)) {
				prodotto.getSondaggi().remove(sondaggio);
				salvaAggiorna(prodotto);
			}
		}
	}
	public void aggiungiSondaggioContrattoTuttiAdProdotto(Long prodottoId) {
		Prodotto prodotto  = getFullProdottoSondaggioContratto(prodottoId);
		List<SondaggioContratto> sondaggio = getSondaggioContrattoNotInProdotto(prodottoId);
		if (ApplicationUtil.notEmpty(sondaggio) && prodotto != null) {
			prodotto.getSondaggi().addAll(sondaggio);
			salvaAggiorna(prodotto);
		}
	}
	public void rimuoviSondaggioContrattoTuttiAdProdotto(Long prodottoId) {
		Prodotto prodotto  = getFullProdottoSondaggioContratto(prodottoId);
		List<SondaggioContratto> sondaggio = getSondaggioContrattoInProdotto(prodottoId);
		if (ApplicationUtil.notEmpty(sondaggio) && prodotto != null) {
			prodotto.getSondaggi().removeAll(sondaggio);
			salvaAggiorna(prodotto);
		}
	}
	public Prodotto getFullProdottoSondaggioContratto(Long prodotto) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Prodotto.class);
		criteria.add(Restrictions.idEq(prodotto));
		criteria.setFetchMode("sondaggi", FetchMode.JOIN);
		return  (Prodotto) getUniqueResult(criteria);
	}
	public SondaggioContratto getFullSondaggioContrattoProdotto(Long sondaggio) {
		DetachedCriteria criteria = DetachedCriteria.forClass(SondaggioContratto.class);
		criteria.add(Restrictions.idEq(sondaggio));
		criteria.setFetchMode("prodotti", FetchMode.JOIN);
		return (SondaggioContratto) getUniqueResult(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<SondaggioContratto> getSondaggioContrattoInProdotto(Long prodotto) {
		if(prodotto != null ){
			DetachedCriteria criteria = DetachedCriteria.forClass(SondaggioContratto.class);
			criteria.createCriteria("prodotti").add(Restrictions.idEq(prodotto));
			criteria.addOrder(Order.asc("nome"));
			return getList(criteria);
		}	else {
			return new ArrayList<SondaggioContratto>();
		}
	}
	@SuppressWarnings("unchecked")
	public List<SondaggioContratto> getSondaggioContrattoNotInProdotto(Long prodotto) {
		DetachedCriteria criteriaU = DetachedCriteria.forClass(SondaggioContratto.class);
		criteriaU.createCriteria("prodotti").add(Restrictions.idEq(prodotto));
		criteriaU.setProjection(Projections.distinct(Projections.property("id")));
		DetachedCriteria criteria = DetachedCriteria.forClass(SondaggioContratto.class);
		impostazioneDao.subPropertyNotInS(criteria, criteriaU, "id");
		criteria.addOrder(Order.asc("nome"));
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Prodotto> getProdottoNotInSondaggioContratto(Long sondaggio) {
		DetachedCriteria criteriaU = DetachedCriteria.forClass(Prodotto.class);
		criteriaU.createCriteria("sondaggi").add(Restrictions.idEq(sondaggio));
		criteriaU.setProjection(Projections.distinct(Projections.property("id")));
		DetachedCriteria criteria = DetachedCriteria.forClass(Prodotto.class);
		impostazioneDao.subPropertyNotInS(criteria, criteriaU, "id");
		criteria.createCriteria("categoria", "categoria", ApplicationUtil.leftJoin());
		ApplicationUtil.orderP(criteria);
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Prodotto> getProdottoInSondaggioContratto(Long sondaggio) {
		if(sondaggio != null ){
			DetachedCriteria criteria = DetachedCriteria.forClass(Prodotto.class);
			criteria.createCriteria("sondaggi").add(Restrictions.idEq(sondaggio));
			criteria.createCriteria("categoria", "categoria", ApplicationUtil.leftJoin());
			ApplicationUtil.orderP(criteria);
			return getList(criteria);
		}	else {
			return new ArrayList<Prodotto>();
		}
	}
	public void aggiungiAiutoAdProdotto(Long aiutoId, Long prodottoId) {
		VocalOrder aiuto = getFullAiutoProdotto(aiutoId);
		Prodotto prodotto = get(prodottoId);
		if (aiuto != null && prodotto != null) {
			if (!aiuto.getProdotti().contains(prodotto)) {
				aiuto.getProdotti().add(prodotto);
				salvaAggiorna(aiuto);
			}
		}
	}
	public void rimuoviAiutoAdProdotto(Long aiutoId, Long prodottoId) {
		VocalOrder aiuto = getFullAiutoProdotto(aiutoId);
		Prodotto prodotto = get(prodottoId);
		if (aiuto != null && prodotto != null) {
			if (aiuto.getProdotti().contains(prodotto)) {
				aiuto.getProdotti().remove(prodotto);
				salvaAggiorna(aiuto);
			}
		}
	}
	public void aggiungiProdottoTuttiAdAiuto(Long aiutoId) {
		VocalOrder aiuto  = getFullAiutoProdotto(aiutoId);
		List<Prodotto> prodotto = getProdottoNotInAiuto(aiutoId);
		if (ApplicationUtil.notEmpty(prodotto) && aiuto != null) {
			aiuto.getProdotti().addAll(prodotto);
			salvaAggiorna(aiuto);
		}
	}
	public void rimuoviProdottoTuttiAdAiuto(Long aiutoId) {
		VocalOrder aiuto  = getFullAiutoProdotto(aiutoId);
		List<Prodotto> prodotto = getProdottoInAiuto(aiutoId);
		if (ApplicationUtil.notEmpty(prodotto) && aiuto != null) {
			aiuto.getProdotti().removeAll(prodotto);
			salvaAggiorna(aiuto);
		}
	}
	public void aggiungiProdottoAdAiuto(Long prodottoId, Long aiutoId) {
		Prodotto prodotto = getFullProdottoAiuto(prodottoId);
		VocalOrder aiuto = vocalOrderDao.get(aiutoId);
		if (prodotto != null && aiuto != null) {
			if (!prodotto.getVocalOrders().contains(aiuto)) {
				prodotto.getVocalOrders().add(aiuto);
				salvaAggiorna(prodotto);
			}
		}
	}
	public void rimuoviProdottoAdAiuto(Long prodottoId, Long aiutoId) {
		Prodotto prodotto = getFullProdottoAiuto(prodottoId);
		VocalOrder aiuto = vocalOrderDao.get(aiutoId);
		if (prodotto != null && aiuto != null) {
			if (prodotto.getVocalOrders().contains(aiuto)) {
				prodotto.getVocalOrders().remove(aiuto);
				salvaAggiorna(prodotto);
			}
		}
	}
	public void aggiungiAiutoTuttiAdProdotto(Long prodottoId) {
		Prodotto prodotto  = getFullProdottoAiuto(prodottoId);
		List<VocalOrder> aiuto = getAiutoNotInProdotto(prodottoId);
		if (ApplicationUtil.notEmpty(aiuto) && prodotto != null) {
			prodotto.getVocalOrders().addAll(aiuto);
			salvaAggiorna(prodotto);
		}
	}
	public void rimuoviAiutoTuttiAdProdotto(Long prodottoId) {
		Prodotto prodotto  = getFullProdottoAiuto(prodottoId);
		List<VocalOrder> aiuto = getAiutoInProdotto(prodottoId);
		if (ApplicationUtil.notEmpty(aiuto) && prodotto != null) {
			prodotto.getVocalOrders().removeAll(aiuto);
			salvaAggiorna(prodotto);
		}
	}
	public Prodotto getFullProdottoAiuto(Long prodotto) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Prodotto.class);
		criteria.add(Restrictions.idEq(prodotto));
		criteria.setFetchMode("vocalOrders", FetchMode.JOIN);
		return  (Prodotto) getUniqueResult(criteria);
	}
	public VocalOrder getFullAiutoProdotto(Long aiuto) {
		DetachedCriteria criteria = DetachedCriteria.forClass(VocalOrder.class);
		criteria.add(Restrictions.idEq(aiuto));
		criteria.setFetchMode("prodotti", FetchMode.JOIN);
		return (VocalOrder) getUniqueResult(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<VocalOrder> getAiutoInProdotto(Long prodotto) {
		if(prodotto != null ){
			DetachedCriteria criteria = DetachedCriteria.forClass(VocalOrder.class);
			criteria.createCriteria("prodotti").add(Restrictions.idEq(prodotto));
			criteria.addOrder(Order.asc("nome"));
			return getList(criteria);
		}	else {
			return new ArrayList<VocalOrder>();
		}
	}
	@SuppressWarnings("unchecked")
	public List<VocalOrder> getAiutoNotInProdotto(Long prodotto) {
		DetachedCriteria criteriaU = DetachedCriteria.forClass(VocalOrder.class);
		criteriaU.createCriteria("prodotti").add(Restrictions.idEq(prodotto));
		criteriaU.setProjection(Projections.distinct(Projections.property("id")));
		DetachedCriteria criteria = DetachedCriteria.forClass(VocalOrder.class);
		impostazioneDao.subPropertyNotInS(criteria, criteriaU, "id");
		criteria.addOrder(Order.asc("nome"));
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Prodotto> getProdottoNotInAiuto(Long aiuto) {
		DetachedCriteria criteriaU = DetachedCriteria.forClass(Prodotto.class);
		criteriaU.createCriteria("vocalOrders").add(Restrictions.idEq(aiuto));
		criteriaU.setProjection(Projections.distinct(Projections.property("id")));
		DetachedCriteria criteria = DetachedCriteria.forClass(Prodotto.class);
		impostazioneDao.subPropertyNotInS(criteria, criteriaU, "id");
		criteria.createCriteria("categoria", "categoria", ApplicationUtil.leftJoin());
		ApplicationUtil.orderP(criteria);
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Prodotto> getProdottoInAiuto(Long aiuto) {
		if(aiuto != null ){
			DetachedCriteria criteria = DetachedCriteria.forClass(Prodotto.class);
			criteria.createCriteria("vocalOrders").add(Restrictions.idEq(aiuto));
			criteria.createCriteria("categoria", "categoria", ApplicationUtil.leftJoin());
			ApplicationUtil.orderP(criteria);
			return getList(criteria);
		}	else {
			return new ArrayList<Prodotto>();
		}
	}
	public void aggiungiTemplateAdProdotto(Long templateId, Long prodottoId) {
		Template template = getTemplate(templateId);
		Prodotto prodotto = get(prodottoId);
		if (template != null && prodotto != null) {
			aggiungiTemplateAProdotto(template, prodotto);
		}
	}
	public void rimuoviTemplateAdProdotto(Long templateId, Long prodottoId) {
		if (prodottoId != null && templateId != null) {
			removeAssociationTemplate(templateId, prodottoId);
		}
	}
	public void aggiungiProdottoTuttiAdTemplate(Long templateId) {
		Template template  = getTemplate(templateId);
		List<Prodotto> prodottos = getProdottoNotInTemplate(templateId);
		if (ApplicationUtil.notEmpty(prodottos) && template != null) {
			Iterator<Prodotto> temIt = prodottos.iterator();
			while (temIt.hasNext()) {
				Prodotto prodotto = (Prodotto) temIt.next();
				aggiungiTemplateAProdotto(template, prodotto);
			}
		}
	}
	public void rimuoviProdottoTuttiAdTemplate(Long templateId) {
		List<Prodotto> prodottos = getProdottoInTemplate(templateId);
		if (ApplicationUtil.notEmpty(prodottos) && templateId != null) {
			Iterator<Prodotto> temIt = prodottos.iterator();
			while (temIt.hasNext()) {
				Prodotto prodotto = (Prodotto) temIt.next();
				removeAssociationTemplate(templateId, prodotto.getId());
			}
		}
	}
	public void aggiungiTemplateTuttiAdProdotto(Long prodottoId) {
		Prodotto prodotto  = get(prodottoId);
		List<Template> templates = getTemplateNotInProdotto(prodottoId);
		if (ApplicationUtil.notEmpty(templates) && prodotto != null) {
			Iterator<Template> temIt = templates.iterator();
			while (temIt.hasNext()) {
				Template template = (Template) temIt.next();
				aggiungiTemplateAProdotto(template, prodotto);
			}
		}
	}
	public void rimuoviTemplateTuttiAdProdotto(Long prodottoId) {
		List<Template> templates = getTemplateInProdotto(prodottoId);
		if (ApplicationUtil.notEmpty(templates) && prodottoId != null) {
			Iterator<Template> temIt = templates.iterator();
			while (temIt.hasNext()) {
				Template template = (Template) temIt.next();
				removeAssociationTemplate(template.getId(), prodottoId);
			}
		}
	}
	@SuppressWarnings("unchecked")
	public List<Template> getTemplateInProdotto(Long prodotto) {
		if(prodotto != null ){
			DetachedCriteria criteria = DetachedCriteria.forClass(Template.class);
			criteria.createCriteria("templates").add(Restrictions.eq("id.prodotto.id", prodotto));
			criteria.addOrder(Order.asc("nome"));
			return getList(criteria);
		}	else {
			return new ArrayList<Template>();
		}
	}
	@SuppressWarnings("unchecked")
	public List<Template> getTemplateNotInProdotto(Long prodotto) {
		DetachedCriteria criteriaU = DetachedCriteria.forClass(Template.class);
		criteriaU.createCriteria("templates").add(Restrictions.eq("id.prodotto.id", prodotto));
		criteriaU.setProjection(Projections.distinct(Projections.property("id")));
		DetachedCriteria criteria = DetachedCriteria.forClass(Template.class);
		impostazioneDao.subPropertyNotInS(criteria, criteriaU, "id");
		criteria.addOrder(Order.asc("nome"));
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Prodotto> getProdottoNotInTemplate(Long template) {
		DetachedCriteria criteriaU = DetachedCriteria.forClass(Prodotto.class);
		criteriaU.createCriteria("template").add(Restrictions.eq("id.documento.id", template));
		criteriaU.setProjection(Projections.distinct(Projections.property("id")));
		DetachedCriteria criteria = DetachedCriteria.forClass(Prodotto.class);
		impostazioneDao.subPropertyNotInS(criteria, criteriaU, "id");
		criteria.createCriteria("categoria", "categoria", ApplicationUtil.leftJoin());
		ApplicationUtil.orderP(criteria);
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Prodotto> getProdottoInTemplate(Long template) {
		if(template != null ){
			DetachedCriteria criteria = DetachedCriteria.forClass(Prodotto.class);
			criteria.createCriteria("template").add(Restrictions.eq("id.documento.id", template));
			criteria.createCriteria("categoria", "categoria", ApplicationUtil.leftJoin());
			ApplicationUtil.orderP(criteria);
			return getList(criteria);
		}	else {
			return new ArrayList<Prodotto>();
		}
	}
	public Template getTemplate(Long documento) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Template.class);
		criteria.add(Restrictions.idEq(documento));
		return (Template) getUniqueResult(criteria);
	}
	public void removeAssociationTemplate(Long documento, Long prodottoId) {
		String hql = "delete from TemplateProdotto where id.prodotto.id = :prodotto and id.documento.id = :documento";
		execute(false, hql, new Parameter("documento", documento), new Parameter("prodotto", prodottoId));
	}
	public void aggiungiTemplateAProdotto(Template documento, Prodotto  prodotto) {
		TemplateProdottoPK pivPK = new TemplateProdottoPK();
		pivPK.setDocumento(documento);
		pivPK.setProdotto(prodotto);
		TemplateProdotto piv = new TemplateProdotto(pivPK);
		if (documento != null && prodotto != null) {
			salvaAggiorna(piv);
		}
	}
	public boolean salvaLogoInternet(String url, Tema tema) {
		String file = getNuovaImmagine();
		String path = impostazioneDao.getPercorsoImmagini() + "/";
		boolean b = ApplicationUtil.salvaImmagine(url, path + file, ConstantsGlobal.imageW, ConstantsGlobal.imageH);
		if(b) {
			Boolean l = impostazioneDao.getContentLogo();
			tema.setContent(ApplicationUtil.content(l, path, file));
		}
		return b;
	}
	public void salvaLogo(FileUpload fileUpload, Tema tema) {
		String file = getNuovaImmagine();
		String path = impostazioneDao.getPercorsoImmagini() + "/";
		boolean b = ApplicationUtil.salvaImmagine(fileUpload.getFiles(), path + file, ConstantsGlobal.imageW, ConstantsGlobal.imageH);
		if(b) {
			Boolean l = impostazioneDao.getContentLogo();
			tema.setContent(ApplicationUtil.content(l, path, file));
		}
	}
	public void salvaLogo(Tema tema, File logo) {
		String path = impostazioneDao.getPercorsoImmagini() + "/";
		Boolean l = impostazioneDao.getContentLogo();
		tema.setContent(ApplicationUtil.content(l, path, ApplicationUtil.getNameN(logo.getAbsolutePath())));
	}
	@SuppressWarnings("unchecked")
	public List<Tema> getLogoAssociato(MyFile myFile) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Tema.class);
		criteria.add(Restrictions.eq("content",myFile.getData()));
		return getList(criteria);
	}
}