package it.callite.dao;
import java.util.List;

import javax.faces.model.SelectItem;

import org.hibernate.FetchMode;
import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Projections;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import it.callite.constants.ConstantsGlobal;
import it.callite.criteria.UfficioCriteria;
import it.callite.criteria.base.ExtendedCriteria;
import it.callite.dao.generic.GenericDao;
import it.callite.model.Agente;
import it.callite.model.Ufficio;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.item.GenericSI;
@SuppressWarnings("serial")
@Repository("ufficioDao")
public class UfficioDao extends GenericDao {
	
	@Autowired
	private ImpostazioneDao impostazioneDao;
	
	public Ufficio salvaUfficio(Ufficio ufficio) {
		return (Ufficio) salvaAggiorna(ufficio);
	}
	public void eliminaUfficio(Ufficio ufficio) {
		cancellaAggiorna(ufficio);
	}
	private ExtendedCriteria trovaCriteria(UfficioCriteria ana) {
		if(ana != null)  ana = (UfficioCriteria) ana.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Ufficio.class);
		if (ApplicationUtil.isNotBlank(ana.getTesto())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, ana.getTesto(), criteria, "nome", "descrizione");
		}
		if (ApplicationUtil.isNotBlank(ana.getZona())) {
			ApplicationUtil.zona(criteria,  ana.getZona());
		}
		ana.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, ana);
	}
	@SuppressWarnings("unchecked")
	public List<Ufficio> trova(UfficioCriteria ana, int start, int size) {
		ExtendedCriteria criteria = trovaCriteria(ana);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int trovaSize(UfficioCriteria ana) {
		ExtendedCriteria criteria = trovaCriteria(ana);
		return getListSize(criteria);
	}
	public Ufficio get(Long id) {
		return (Ufficio) get(Ufficio.class, id);
	}
	public Agente getAgente(Long id) {
		return (Agente) get(Agente.class, id);
	}
	public List<SelectItem> getUffici() {
		DetachedCriteria criteria = DetachedCriteria.forClass(Ufficio.class);
		List<GenericSI> lis = getGenericSI(criteria, "id", "nome");
		return impostazioneDao.getGeneric(lis, true);
	}
	@SuppressWarnings("unchecked")
	public List<Ufficio> getUfficiModel() {
		DetachedCriteria criteria = DetachedCriteria.forClass(Ufficio.class);
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Agente> getAgentiByUfficioId(Long ufficioId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
		criteria.createCriteria("ufficio").add(Restrictions.idEq(ufficioId));
		ApplicationUtil.orderA(criteria);
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Agente> getAgentiNotInUfficio(Long ufficio, Integer tutti) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
		if(ApplicationUtil.gtZero(tutti)) {
			if(ApplicationUtil.eq(tutti, 1)) {
				criteria.createCriteria("ufficio", ApplicationUtil.leftJoin()).add(Restrictions.isNull("id"));
			} else {
				DetachedCriteria criteriaU = DetachedCriteria.forClass(Agente.class);
				criteriaU.createCriteria("ufficio").add(Restrictions.idEq(ufficio));
				criteriaU.setProjection(Projections.distinct(Projections.property("id")));
				impostazioneDao.subPropertyNotInS(criteria, criteriaU, "id");
				criteria.createCriteria("ufficio", ApplicationUtil.leftJoin()).add(Restrictions.isNotNull("id"));
			}
		} else {
			DetachedCriteria criteriaU = DetachedCriteria.forClass(Agente.class);
			criteriaU.createCriteria("ufficio").add(Restrictions.idEq(ufficio));
			criteriaU.setProjection(Projections.distinct(Projections.property("id")));
			impostazioneDao.subPropertyNotInS(criteria, criteriaU, "id");
		}
		ApplicationUtil.orderA(criteria);
		return getList(criteria);
	}
	public void aggiungiAgenteAUfficio(Long ufficioId, Long agenteId) {
		Ufficio ufficio = getFullUfficioAgente(ufficioId);
		Agente agente = getAgente(agenteId);
		if (ufficio != null && agente != null) {
			if (!ufficio.getAgenti().contains(agente)) {
				ufficio.getAgenti().add(agente);
				salvaAggiorna(ufficio);
			}
		}
	}
	public void rimuoviAgenteDaUfficio(Long ufficioId, Long agenteId) {
		Ufficio ufficio = getFullUfficioAgente(ufficioId);
		Agente agente = getAgente(agenteId);
		if (ufficio != null && agente != null) {
			if (ufficio.getAgenti().contains(agente)) {
				ufficio.getAgenti().remove(agente);
				salvaAggiorna(ufficio);
			}
		}
	}
	public void aggiungiAgenteTuttiAUfficio(Long ufficioId, Integer tutti) {
		Ufficio ufficio = getFullUfficioAgente(ufficioId);
		List<Agente> agenti = getAgentiNotInUfficio(ufficioId, tutti);
		if (ufficio != null && ApplicationUtil.notEmpty(agenti)) {
			ufficio.getAgenti().addAll(agenti);
			salvaAggiorna(ufficio);
		}
	}
	public void rimuoviAgenteTuttiDaUfficio(Long ufficioId) {
		Ufficio ufficio = getFullUfficioAgente(ufficioId);
		List<Agente> agenti = getAgentiByUfficioId(ufficioId);
		if (ufficio != null && ApplicationUtil.notEmpty(agenti)) {
			ufficio.getAgenti().removeAll(agenti);
			salvaAggiorna(ufficio);
		}
	}
	public Ufficio getFullUfficioAgente(Long ufficioId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Ufficio.class);
		criteria.add(Restrictions.idEq(ufficioId));
		criteria.setFetchMode("agenti", FetchMode.JOIN);
		return  (Ufficio) getUniqueResult(criteria);
	}
}