package it.callite.dao;
import java.util.ArrayList;
import java.util.List;
import javax.faces.model.SelectItem;
import org.hibernate.FetchMode;
import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Disjunction;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;
import it.callite.constants.ConstantsEsito;
import it.callite.criteria.WorkflowCriteria;
import it.callite.criteria.base.ExtendedCriteria;
import it.callite.dao.generic.GenericDao;
import it.callite.model.Campagna;
import it.callite.model.EsitoArchiviazione;
import it.callite.model.EsitoChiamata;
import it.callite.model.Lista;
import it.callite.model.Workflow;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.item.CampagnaSI;
import it.callite.utility.item.EsitoArchiviazioneSI;
import it.callite.utility.item.EsitoChiamataSI;
import it.callite.utility.item.ListaSI;
@SuppressWarnings("serial")
@Repository("workflowDao")
public class WorkflowDao extends GenericDao {
	@Autowired
	private ImpostazioneDao impostazioneDao;
	public Workflow salva(Workflow workflow) {
		if(workflow != null && workflow.getEsitoChiamata() != null && workflow.getEsitoChiamata().getId() != null) {
			if(!workflow.getEsitoChiamata().getId().equals(ConstantsEsito.ID_OK)) {
				workflow.setEsitoArchiviazione(null);
			}
		}
		return (Workflow) salvaAggiorna(workflow);
	}
	public void elimina(Workflow workflow) {
		cancellaAggiorna(workflow);
	}
	private ExtendedCriteria trovaCriteria(WorkflowCriteria ana) {
		if(ana != null)  ana = (WorkflowCriteria) ana.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Workflow.class);
		if (ApplicationUtil.gtZero(ana.getAttivo())) {
			if (ApplicationUtil.eq(ana.getAttivo(), 1)) {
				criteria.add(Restrictions.eq("attivo", Boolean.TRUE));
			} else 	{
				criteria.add(Restrictions.eq("attivo", Boolean.FALSE));
			} 
		}
		if (ApplicationUtil.gtZero(ana.getSorgente())) {
			criteria.add(Restrictions.eq("sorgente.id", ana.getSorgente()));
		}
		if (ApplicationUtil.gtZero(ana.getDestinazione())) {
			criteria.add(Restrictions.eq("campagna.id", ana.getDestinazione()));
		}
		ana.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, ana);
	}
	@SuppressWarnings("unchecked")
	public List<Workflow> trova(WorkflowCriteria ana, int start, int size) {
		ExtendedCriteria criteria = trovaCriteria(ana);
		criteria.getCriteria().setFetchMode("sorgente", FetchMode.JOIN);
		criteria.getCriteria().setFetchMode("esitoChiamata", FetchMode.JOIN);
		criteria.getCriteria().setFetchMode("esitoArchiviazione", FetchMode.JOIN);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int trovaSize(WorkflowCriteria ana) {
		ExtendedCriteria criteria = trovaCriteria(ana);
		return getListSize(criteria);
	}
	public Workflow getWorkflow(Long id) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Workflow.class);
		criteria.add(Restrictions.idEq(id));
		criteria.setFetchMode("sorgente", FetchMode.JOIN);
		criteria.setFetchMode("esitoChiamata", FetchMode.JOIN);
		criteria.setFetchMode("esitoArchiviazione", FetchMode.JOIN);
		return (Workflow) getUniqueResult(criteria);
	}
	public Campagna getCampagna(Long id) {
		return (Campagna) get(Campagna.class, id);
	}
	public Lista getLista(Long id) {
		return (Lista) get(Lista.class, id);
	}
	public EsitoChiamata getEsitoChiamata(Long id) {
		return (EsitoChiamata) get(EsitoChiamata.class, id);
	}
	public EsitoArchiviazione getEsitoArchiviazione(Long id) {
		return (EsitoArchiviazione) get(EsitoArchiviazione.class, id);
	}
	public List<SelectItem> getCampagnaSI() {
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		List<CampagnaSI> lis = getCampagnaSI(criteria);
		return impostazioneDao.getCampagna(lis, false, false, false, null);
	}
	public List<SelectItem> getListaSI(Long campagna) {
		if(ApplicationUtil.gtZero(campagna)) {
			DetachedCriteria criteria = DetachedCriteria.forClass(Lista.class);
			criteria.createCriteria("campagna").add(Restrictions.idEq(campagna));
			List<ListaSI> lis = getListaSI(criteria);
			return impostazioneDao.getLista(lis, false, null);
		}
		return new ArrayList<SelectItem>();
	}
	public List<SelectItem> getEsitoChiamataSI() {
		List<Long> ids = new ArrayList<Long>();
		if(!impostazioneDao.getResetContatto()) {
			ids.add(ConstantsEsito.ID_DA_CHIAMARE);
		} 
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoChiamata.class);
		if(ApplicationUtil.notEmpty(ids)) {
			criteria.add(Restrictions.not(Restrictions.in("id",ids) ));
		}
		List<EsitoChiamataSI> lis = getEsitoChiamataSI(criteria);
		return impostazioneDao.getEsitoChiamata(lis, false, null);
	}
	public List<SelectItem> getEsitoArchiviazioneSI() {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoArchiviazione.class);
		List<EsitoArchiviazioneSI> lis = getEsitoArchiviazioneSI(criteria);
		return impostazioneDao.getEsitoArchiviazione(lis, false, null);
	}
	public Workflow getWorkflow(Long c, Long ec, Long ea ) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Workflow.class);
		criteria.add(Restrictions.eq("sorgente.id", c));
		criteria.add(Restrictions.eq("esitoChiamata.id", ec));
		if(ea != null) {
			criteria.add(Restrictions.eq("esitoArchiviazione.id", ea));
		} else {
			criteria.add(Restrictions.isNull("esitoArchiviazione"));
		}
		return (Workflow) getUniqueResult(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Workflow> getWorkflowByCampagnaId(Long campagna) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Workflow.class);
		Disjunction disjunction =  Restrictions.disjunction();
		disjunction.add(Restrictions.eq("sorgente.id", campagna));
		disjunction.add(Restrictions.eq("campagna.id", campagna));
		criteria.add(disjunction);
		criteria.setFetchMode("sorgente", FetchMode.JOIN);
		criteria.setFetchMode("esitoChiamata", FetchMode.JOIN);
		criteria.setFetchMode("esitoArchiviazione", FetchMode.JOIN);
		criteria.addOrder(Order.asc("id"));
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Workflow> getWorkflowByListaId(Long lista) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Workflow.class);
		criteria.add(Restrictions.eq("lista.id", lista));
		criteria.setFetchMode("sorgente", FetchMode.JOIN);
		criteria.setFetchMode("esitoChiamata", FetchMode.JOIN);
		criteria.setFetchMode("esitoArchiviazione", FetchMode.JOIN);
		criteria.addOrder(Order.asc("id"));
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Workflow> getWorkflowByEsitoChiamataId(Long esitoChiamata) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Workflow.class);
		criteria.add(Restrictions.eq("esitoChiamata.id", esitoChiamata));
		criteria.setFetchMode("sorgente", FetchMode.JOIN);
		criteria.setFetchMode("esitoChiamata", FetchMode.JOIN);
		criteria.setFetchMode("esitoArchiviazione", FetchMode.JOIN);
		criteria.addOrder(Order.asc("id"));
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Workflow> getWorkflowByEsitoArchiviazioneId(Long esitoArchiviazione) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Workflow.class);
		criteria.add(Restrictions.eq("esitoChiamata.id", ConstantsEsito.ID_OK));
		criteria.add(Restrictions.eq("esitoArchiviazione.id", esitoArchiviazione));
		criteria.setFetchMode("sorgente", FetchMode.JOIN);
		criteria.setFetchMode("esitoChiamata", FetchMode.JOIN);
		criteria.setFetchMode("esitoArchiviazione", FetchMode.JOIN);
		criteria.addOrder(Order.asc("id"));
		return getList(criteria);
	}
}