package it.callite.dao.config;
import java.util.Iterator;
import java.util.List;

import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import it.callite.constants.ConstantsGlobal;
import it.callite.constants.config.DefaultsData;
import it.callite.criteria.ConfigurazioneCriteria;
import it.callite.criteria.base.ExtendedCriteria;
import it.callite.dao.ImpostazioneDao;
import it.callite.dao.generic.GenericDao;
import it.callite.model.config.ConfigData;
import it.callite.utility.ApplicationUtil;
@SuppressWarnings("serial")
@Repository("configurazioneDataDao")
public class ConfigurazioneDataDao extends GenericDao {
	
	@Autowired
	private ImpostazioneDao impostazioneDao;
	
	public ConfigData getValue(String key) {
		ConfigData conf = getValoreByChiave(key);
		if (conf == null) {
			conf = DefaultsData.DEFAULTS.get(key);
			try {
				salva(conf);
			} catch (Exception e) {	}
		} 
		return conf;
	}
	private ConfigData getValoreByChiave(String chiave) {
		DetachedCriteria criteria = DetachedCriteria.forClass(ConfigData.class);
		criteria.add(Restrictions.eq("chiave", ApplicationUtil.trimToEmpty(chiave)));
		return (ConfigData) getUniqueResult(criteria);
	}
	public void elimina(ConfigData config) {
		cancellaAggiorna(config);
	}
	public ConfigData salva(ConfigData config) {
		return (ConfigData) salvaAggiorna(config);
	}
	@SuppressWarnings("unchecked")
	public void eliminaNotInConfig(List<String> chiavi) {
		if(ApplicationUtil.notEmpty(chiavi)) {
			DetachedCriteria criteria = DetachedCriteria.forClass(ConfigData.class);
			criteria.add(Restrictions.not(Restrictions.in("chiave", chiavi)));
			List<ConfigData> c = getList(criteria);
			cancellaLista(c);
		}
	}
	private ExtendedCriteria getConfigByCriteria(ConfigurazioneCriteria conf) {
		if(conf != null)  conf = (ConfigurazioneCriteria) conf.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(ConfigData.class);
		if(ApplicationUtil.isNotBlank(conf.getNome())) {
			ApplicationUtil.chiave(ConstantsGlobal.colonSeparator, conf.getNome(), criteria, "chiave");
		} 
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, conf);
	}
	@SuppressWarnings("unchecked")
	public List<ConfigData> trova(ConfigurazioneCriteria conf, int start, int size) {
		ExtendedCriteria criteria = getConfigByCriteria(conf);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("chiave")});
		List<ConfigData> lst = getList(criteria, start, size);
		return defaultS(lst);
	}
	public int trovaSize(ConfigurazioneCriteria conf) {
		ExtendedCriteria criteria = getConfigByCriteria(conf);
		return getListSize(criteria);
	}
	public List<ConfigData> defaultS(List<ConfigData> lst) {
		if(ApplicationUtil.notEmpty(lst)) {
			Iterator<ConfigData> it = lst.iterator();
			while (it.hasNext()) {
				ConfigData configData = (ConfigData) it.next();
				configData.setConfigure(DefaultsData.DEFAULTS.get(configData.getChiave()).getConfigure());
			}
		}
		return lst;
	}
}