package it.callite.dao.config;
import java.util.Iterator;
import java.util.List;

import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import it.callite.constants.ConstantsGlobal;
import it.callite.constants.config.DefaultsSmtp;
import it.callite.criteria.ConfigurazioneCriteria;
import it.callite.criteria.base.ExtendedCriteria;
import it.callite.dao.ImpostazioneDao;
import it.callite.dao.generic.GenericDao;
import it.callite.model.config.ConfigSmtp;
import it.callite.utility.ApplicationUtil;
@SuppressWarnings("serial")
@Repository("configurazioneSmtpDao")
public class ConfigurazioneSmtpDao extends GenericDao {
	
	@Autowired
	private ImpostazioneDao impostazioneDao;
	
	public ConfigSmtp getValue(String key) {
		ConfigSmtp conf = getValoreByChiave(key);
		if (conf == null) {
			conf = DefaultsSmtp.DEFAULTS.get(key);
			try {
				salva(conf);
			} catch (Exception e) {	}
		} 
		return conf;
	}
	private ConfigSmtp getValoreByChiave(String chiave) {
		DetachedCriteria criteria = DetachedCriteria.forClass(ConfigSmtp.class);
		criteria.add(Restrictions.eq("chiave", ApplicationUtil.trimToEmpty(chiave)));
		return (ConfigSmtp) getUniqueResult(criteria);
	}
	public void elimina(ConfigSmtp config) {
		cancellaAggiorna(config);
	}
	public ConfigSmtp salva(ConfigSmtp config) {
		return (ConfigSmtp) salvaAggiorna(config);
	}
	@SuppressWarnings("unchecked")
	public void eliminaNotInConfig(List<String> chiavi) {
		if(ApplicationUtil.notEmpty(chiavi)) {
			DetachedCriteria criteria = DetachedCriteria.forClass(ConfigSmtp.class);
			criteria.add(Restrictions.not(Restrictions.in("chiave", chiavi)));
			List<ConfigSmtp> c = getList(criteria);
			cancellaLista(c);
		}
	}
	private ExtendedCriteria getConfigByCriteria(ConfigurazioneCriteria conf) {
		if(conf != null)  conf = (ConfigurazioneCriteria) conf.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(ConfigSmtp.class);
		if(ApplicationUtil.isNotBlank(conf.getNome())) {
			ApplicationUtil.chiave(ConstantsGlobal.colonSeparator, conf.getNome(), criteria, "chiave");
		} 
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, conf);
	}
	@SuppressWarnings("unchecked")
	public List<ConfigSmtp> trova(ConfigurazioneCriteria conf, int start, int size) {
		ExtendedCriteria criteria = getConfigByCriteria(conf);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("chiave")});
		List<ConfigSmtp> lst = getList(criteria, start, size);
		return defaultS(lst);
	}
	public int trovaSize(ConfigurazioneCriteria conf) {
		ExtendedCriteria criteria = getConfigByCriteria(conf);
		return getListSize(criteria);
	}
	public List<ConfigSmtp> defaultS(List<ConfigSmtp> lst) {
		if(ApplicationUtil.notEmpty(lst)) {
			Iterator<ConfigSmtp> it = lst.iterator();
			while (it.hasNext()) {
				ConfigSmtp configSmtp = (ConfigSmtp) it.next();
				configSmtp.setConfigure(DefaultsSmtp.DEFAULTS.get(configSmtp.getChiave()).getConfigure());
			}
		}
		return lst;
	}
}