package it.callite.model;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Transient;
import org.hibernate.annotations.OnDelete;
import org.hibernate.annotations.OnDeleteAction;
import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.model.allegato.AllegatoAgente;
import it.callite.model.generic.SelezionatoAbstract;
import it.callite.model.report.ColonnaModel;
import it.callite.utility.Allegato;
import it.callite.utility.ApplicationUtil;
@SuppressWarnings("serial")
@Entity
//@Audited 
@Table(name = ConstantsGlobal.tp + "agente", indexes = {@Index(name= "agente", columnList = "data_creazione, codice, autonomo" ),	@Index(name= "dati_agente1", columnList = "cognome, nome" ), @Index(name= "dati_agente2", columnList = "nome_completo, ragione_sociale" ), @Index(name= "dati_agente3", columnList = "partita_iva, codice_fiscale" ), @Index(name= "dati_agente4", columnList = "telefono_fisso, telefono_mobile" ), @Index(name= "dati_agente5", columnList = "telefono_fax, telefono_altro" ), @Index(name= "dati_agente6", columnList = "email, link" )	})
public class Agente  extends SelezionatoAbstract implements Serializable, Comparable<Agente> {
	@Override
	public String toString() {
		return ApplicationUtil.toString(id, getDenominazione());
	}
	@OneToMany(mappedBy = "agente")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<StoriaPrenotazione> prenotazioni_storia;
	public List<StoriaPrenotazione> getPrenotazioni_storia() {
		return prenotazioni_storia;
	}
	public void setPrenotazioni_storia(List<StoriaPrenotazione> prenotazioni_storia) {
		this.prenotazioni_storia = prenotazioni_storia;
	}
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY) 
	private Long id;
	@ManyToMany(fetch = FetchType.LAZY)
	@JoinTable(name = ConstantsGlobal.tp + "agente_utente", joinColumns=@JoinColumn(name="agente_id", nullable=false, updatable=true, insertable=true), inverseJoinColumns=@JoinColumn(name="utente_id", nullable=false, updatable=true, insertable=true)    )
	private List<Utente> user;
	@OneToMany(mappedBy = "agente")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<AllegatoAgente> allegati;
	
	@Transient
	private ColonnaModel model;

	public ColonnaModel getModel() {
		if(model == null) {
			model = new ColonnaModel(id);
		}
		return model;
	}
	public void setModel(ColonnaModel model) {
		this.model = model;
	}
	
	
	@OneToMany
	@JoinColumn(name = "agente_id", nullable = true, updatable=true, insertable=true)
	private List<Contratto> contratti = new ArrayList<Contratto>();
	public List<Prenotazione> getPrenotazioni() {
		return prenotazioni;
	}
	public void setPrenotazioni(List<Prenotazione> prenotazioni) {
		this.prenotazioni = prenotazioni;
	}
	@OneToMany(mappedBy = "agente")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<Prenotazione> prenotazioni;
	@OneToMany(mappedBy = "agente")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<StoriaAppuntamento> storia_appuntamenti;
	@OneToMany(mappedBy = "agente")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<StoriaNota> storia_note;
	@OneToMany(mappedBy = "agente")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<StoriaContratto> storia_contratti;
	@OneToMany
	@JoinColumn(name = "agente_id", nullable = true, updatable=true, insertable=true)
	private List<Appuntamento> appuntamenti = new ArrayList<Appuntamento>();
	@OneToMany(mappedBy = "agente")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<Nota> note;
	public List<StoriaNota> getStoria_note() {
		return storia_note;
	}
	public void setStoria_note(List<StoriaNota> storia_note) {
		this.storia_note = storia_note;
	}
	public List<Nota> getNote() {
		return note;
	}
	public void setNote(List<Nota> note) {
		this.note = note;
	}
	@Column(columnDefinition = "text")
	private String nota;
	private String codice_fiscale;
	private String codice;
	public String getCodice() {
		return ApplicationUtil.trimToEmpty(codice);
	}
	public void setCodice(String codice) {
		this.codice = ApplicationUtil.normalizeToNull(codice);
	}
	@Column(name="persona_fisica")
	private Boolean pf;
	public Boolean getPf() {
		return pf;
	}
	public void setPf(Boolean pf) {
		this.pf = pf;
	}
	private Date data_creazione = ApplicationUtil.date();
	private Date date_nascita;
	private String email;
	@Column(columnDefinition = "text")
	private String emailReport;
	public String getEmailReport() {
		return ApplicationUtil.lowerCaseEmpty(emailReport);
	}
	public void setEmailReport(String emailReport) {
		this.emailReport = ApplicationUtil.lowerCase(ApplicationUtil.aggiustaLista(true, ConstantsGlobal.colonSeparator, emailReport));
	}
	@ManyToMany(fetch = FetchType.LAZY)
	@JoinTable(name = ConstantsGlobal.tp + "agente_zona", joinColumns=@JoinColumn(name="agente_id", nullable=false, updatable=true, insertable=true), inverseJoinColumns=@JoinColumn(name="zone_id", nullable=false, updatable=true, insertable=true)    )
	private List<Zona> zone;
	@ManyToOne(cascade = CascadeType.ALL)
	@OnDelete(action = OnDeleteAction.CASCADE)
	private Indirizzo indirizzo_legale;
	@ManyToOne(cascade = CascadeType.ALL)
	@OnDelete(action = OnDeleteAction.CASCADE)
	private Indirizzo indirizzo_operativo;
	private String luogo_nascita;
	private String link;
	private Boolean maschio;
	private String cognome;
	private String nome;
	private String nome_completo;
	private String partita_iva;
	private String ragione_sociale;
	private String telefono_fax;
	private String telefono_fisso;
	private String telefono_mobile;
	private String telefono_altro; 
	private Boolean visualizzaStoria = Boolean.FALSE;
	public Boolean getVisualizzaStoria() {
		return ApplicationUtil.isTrue(visualizzaStoria);
	}
	public void setVisualizzaStoria(Boolean visualizzaStoria) {
		this.visualizzaStoria = visualizzaStoria;
	}
	private Boolean autonomo = Boolean.TRUE;
	@OneToMany(mappedBy = "agente")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<Contatto> contatti;
	@OneToMany(mappedBy = "agente")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<StoriaContatto> storia_contatti;
	public List<StoriaContatto> getStoria_contatti() {
		return storia_contatti;
	}
	public void setStoria_contatti(List<StoriaContatto> storia_contatti) {
		this.storia_contatti = storia_contatti;
	}
	public Agente() {
		if (indirizzo_legale == null) {
			indirizzo_legale = new Indirizzo();
		}
		if (indirizzo_operativo == null) {
			indirizzo_operativo = new Indirizzo();
		}
	}
	public int compareTo(Agente o) {
		if(id != null && o.id != null) {
			return id.compareTo(o.id);
		} else if(id != null && o.id == null) {
			return -1;
		} else if(id == null && o.id != null) {
			return 1;
		} else {
			return 0;
		}
	}
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Agente other = (Agente) obj;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		return true;
	}
	public List<AllegatoAgente> getAllegati() {
		return allegati;
	} 
	public List<Appuntamento> getAppuntamenti() {
		return appuntamenti;
	} 
	public Boolean getAutonomo() {
		return autonomo;
	}
	public String getCodice_fiscale() {
		return ApplicationUtil.upperCaseEmpty(codice_fiscale);
	}
	public String getCognome() {
		return ApplicationUtil.trimToEmpty(cognome);
	} 
	public List<Contatto> getContatti() {
		return contatti;
	} 
	public List<Contratto> getContratti() {
		return contratti;
	} 
	public Date getData_creazione() {
		return data_creazione;
	}
	public Date getDate_nascita() {
		return ApplicationUtil.aggiustaDataD(date_nascita, true);
	}
	@ManyToOne
	private Ufficio ufficio;
	public Ufficio getUfficio() {
		return ufficio;
	}
	public void setUfficio(Ufficio ufficio) {
		this.ufficio = ufficio;
	}
	@ManyToOne
	private Qualifica qualifica;
	public Qualifica getQualifica() {
		return qualifica;
	}
	public void setQualifica(Qualifica qualifica) {
		this.qualifica = qualifica;
	}
	public String getAppellativo() {
		return ApplicationUtil.denominazione(pf, nome_completo, ragione_sociale, cognome, nome, null);
	}

	public String getDenominazione() {
		StringBuilder sb = new StringBuilder(getAppellativo());
		if(ApplicationUtil.isNotBlank(codice)) {
			sb.insert(0, ApplicationUtil.trimToEmpty(codice) + " - ");
		}
		if(qualifica != null) {
			sb.append(" " + "[" + ApplicationUtil.trimToEmpty(qualifica.getNome()) + "]");
		}
		if(ufficio != null) {
			sb.append(" " + "[" + ApplicationUtil.trimToEmpty(ufficio.getNome()) + "]");
		}
		return ApplicationUtil.trimToEmpty(sb.toString());
	}
	public String getTelefono() {
		StringBuilder sb = new StringBuilder();
		if(ApplicationUtil.isNotBlank(getTelefono_fisso())) {
			sb.append(ConstantsGlobal.B1 + ConstantsLanguage.FISSO.stampa() + ConstantsGlobal.B2 + ": "+ getTelefono_fisso());
			sb.append(ConstantsGlobal.RETURN);
		}
		if(ApplicationUtil.isNotBlank(getTelefono_mobile())) {
			sb.append(ConstantsGlobal.B1 + ConstantsLanguage.MOBILE.stampa() + ConstantsGlobal.B2 +": " + getTelefono_mobile());
			sb.append(ConstantsGlobal.RETURN);
		}
		if(ApplicationUtil.isNotBlank(getTelefono_fax())) {
			sb.append(ConstantsGlobal.B1 + ConstantsLanguage.FAX.stampa() + ConstantsGlobal.B2 +": "+ getTelefono_fax());
			sb.append(ConstantsGlobal.RETURN);
		}
		if(ApplicationUtil.isNotBlank(getTelefono_altro())) {
			sb.append(ConstantsGlobal.B1 + ConstantsLanguage.ALTRO.stampa() + ConstantsGlobal.B2 +": "+ getTelefono_altro());
			sb.append(ConstantsGlobal.RETURN);
		}
		if(ApplicationUtil.isNotBlank(getEmail())) {
			sb.append(ConstantsGlobal.B1 + ConstantsLanguage.EMAIL.stampa() + ConstantsGlobal.B2 +": "+ getEmail());
			sb.append(ConstantsGlobal.RETURN);
		}
		if(ApplicationUtil.isNotBlank(getLink())) {
			sb.append(ConstantsGlobal.B1 + ConstantsLanguage.LINK.stampa() + ConstantsGlobal.B2 +": "+ getLink());
			sb.append(ConstantsGlobal.RETURN);
		}
		if(ApplicationUtil.isNotBlank(getCodice_fiscale())) {
			sb.append(ConstantsGlobal.B1 + ConstantsLanguage.CODICEA.stampa() + ConstantsGlobal.B2 +": "+ getCodice_fiscale());
			sb.append(ConstantsGlobal.RETURN);
		}
		if(ApplicationUtil.isNotBlank(getPartita_iva())) {
			sb.append(ConstantsGlobal.B1 + ConstantsLanguage.PARTITAA.stampa() + ConstantsGlobal.B2 +": "+ getPartita_iva());
			sb.append(ConstantsGlobal.RETURN);
		}
		return ApplicationUtil.trimToEmpty(sb.toString());
	}
	public String getEmail() {
		return ApplicationUtil.lowerCaseEmpty(email);
	}
	public Long getId() {
		return id;
	}
	public Indirizzo getIndirizzo_legale() {
		if (indirizzo_legale == null) {
			indirizzo_legale = new Indirizzo();
		}
		return indirizzo_legale;
	}
	public Indirizzo getIndirizzo_operativo() {
		if (indirizzo_operativo == null) {
			indirizzo_operativo = new Indirizzo();
		}
		return indirizzo_operativo;
	}
	public String getLuogo_nascita() {
		return ApplicationUtil.upperCaseEmpty(luogo_nascita);
	}
	public Boolean getMaschio() {
		return maschio;
	}
	public String getNome() {
		return ApplicationUtil.trimToEmpty(nome);
	}
	public String getNome_completo() {
		return ApplicationUtil.trimToEmpty(nome_completo);
	}
	public String getNota() {
		return nota;
	}
	public String getNotaShort() {
		return ApplicationUtil.tronca(nota);
	}
	public String getPartita_iva() {
		return ApplicationUtil.upperCaseEmpty(partita_iva);
	}
	public String getRagione_sociale() {
		return ApplicationUtil.trimToEmpty(ragione_sociale);
	}
	public List<StoriaAppuntamento> getStoria_appuntamenti() {
		return storia_appuntamenti;
	}
	public List<StoriaContratto> getStoria_contratti() {
		return storia_contratti;
	}
	public String getTelefono_altro() {
		return ApplicationUtil.trimToEmpty(telefono_altro);
	}
	public String getTelefono_fax() {
		return ApplicationUtil.trimToEmpty(telefono_fax);
	}
	public String getTelefono_fisso() {
		return ApplicationUtil.trimToEmpty(telefono_fisso);
	}
	public String getTelefono_mobile() {
		return ApplicationUtil.trimToEmpty(telefono_mobile);
	}
	public List<Utente> getUser() {
		return user;
	}
	public List<Zona> getZone() {
		return zone;
	}
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		return result;
	}
	public void setAllegati(List<AllegatoAgente> allegati) {
		this.allegati = allegati;
	}
	public void setAppuntamenti(List<Appuntamento> appuntamenti) {
		this.appuntamenti = appuntamenti;
	}
	public void setAutonomo(Boolean autonomo) {
		this.autonomo = autonomo;
	}
	public void setCodice_fiscale(String codice_fiscale) {
		this.codice_fiscale = ApplicationUtil.normalizeToUpperNull(codice_fiscale);
	}
	public void setCognome(String cognome) {
		this.cognome = ApplicationUtil.normalizeToNull(cognome);
	}
	public void setContatti(List<Contatto> contatti) {
		this.contatti = contatti;
	}
	public void setContratti(List<Contratto> contratti) {
		this.contratti = contratti;
	}
	public void setData_creazione(Date data_creazione) {
		this.data_creazione = data_creazione;
	}
	public void setDate_nascita(Date date_nascita) {
		this.date_nascita = ApplicationUtil.aggiustaDataD(date_nascita, true);
	}
	public void setEmail(String email) {
		this.email = ApplicationUtil.normalizeToLowerNull(email);
	}
	public void setId(Long id) {
		this.id = id;
	}
	public void setIndirizzo_legale(Indirizzo indirizzoLegale) {
		this.indirizzo_legale = indirizzoLegale;
	}
	public void setIndirizzo_operativo(Indirizzo indirizzoOperativo) {
		this.indirizzo_operativo = indirizzoOperativo;
	}
	public void setLuogo_nascita(String luogo_nascita) {
		this.luogo_nascita = ApplicationUtil.normalizeToUpperNull(luogo_nascita);
	}
	public String getLink() {
		return ApplicationUtil.trimToEmpty(link);
	}
	public void setLink(String link) {
		this.link = ApplicationUtil.normalizeToNull(link);
	}
	public void setMaschio(Boolean maschio) {
		this.maschio = maschio;
	}
	public void setNome(String nome) {
		this.nome = ApplicationUtil.normalizeToNull(nome);
	}
	public void setNome_completo(String nome_completo) {
		this.nome_completo = ApplicationUtil.normalizeToNull(nome_completo);
	}
	public void setNota(String nota) {
		this.nota = nota;
	}
	public void setPartita_iva(String partita_iva) {
		this.partita_iva = ApplicationUtil.normalizeToUpperNull(partita_iva);
	}
	public void setRagione_sociale(String ragione_sociale) {
		this.ragione_sociale = ApplicationUtil.normalizeToNull(ragione_sociale);
	}
	public void setStoria_appuntamenti(List<StoriaAppuntamento> storia_appuntamenti) {
		this.storia_appuntamenti = storia_appuntamenti;
	}
	public void setStoria_contratti(List<StoriaContratto> storia_contratti) {
		this.storia_contratti = storia_contratti;
	}
	public void setTelefono_altro(String telefono_altro) {
		this.telefono_altro = ApplicationUtil.normalizeToNull(telefono_altro);
	}
	public void setTelefono_fax(String telefono_fax) {
		this.telefono_fax = ApplicationUtil.normalizeToNull(telefono_fax);
	}
	public void setTelefono_fisso(String telefono_fisso) {
		this.telefono_fisso = ApplicationUtil.normalizeToNull(telefono_fisso);
	}
	public void setTelefono_mobile(String telefono_mobile) {
		this.telefono_mobile = ApplicationUtil.normalizeToNull(telefono_mobile);
	}
	public void setUser(List<Utente> user) {
		this.user = user;
	}
	public void setZone(List<Zona> zone) {
		this.zone = zone;
	}
	@Transient
	private List<Allegato> allegato;
	public List<Allegato> getAllegato() {
		if(allegato==null)
			allegato = new ArrayList<Allegato>();
		return allegato;
	}
	public int getAllegatoSize() {
		return ApplicationUtil.size(allegato);
	}
	@OneToMany
	@JoinColumn(name = "agente_padre_id", nullable = true, updatable=true, insertable=true)
	private List<Agente> agente_figli = new ArrayList<Agente>();
	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "agente_padre_id", nullable = true)
	private Agente agente_padre;
	//@NotAudited
	@OneToMany(mappedBy = "agente")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<Closure> closure_agenti;
	//@NotAudited
	@OneToMany(mappedBy = "agente_padre")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<Closure> closure_agenti_padre;
	public List<Agente> getAgente_figli() {
		return agente_figli;
	}
	public void setAgente_figli(List<Agente> agente_figli) {
		this.agente_figli = agente_figli;
	}
	public Agente getAgente_padre() {
		return agente_padre;
	}
	public void setAgente_padre(Agente agente_padre) {
		this.agente_padre = agente_padre;
	}
	public List<Closure> getClosure_agenti() {
		return closure_agenti;
	}
	public void setClosure_agenti(List<Closure> closure_agenti) {
		this.closure_agenti = closure_agenti;
	}
	public List<Closure> getClosure_agenti_padre() {
		return closure_agenti_padre;
	}
	public void setClosure_agenti_padre(List<Closure> closure_agenti_padre) {
		this.closure_agenti_padre = closure_agenti_padre;
	}
	public String getUfficioNome() {
		return  getUfficio() != null ? getUfficio().getNome() : ""; 
	}
	

	public String getQualificaNome() {
		return  getQualifica() != null? getQualifica().getNome() : "";
	}
	public String getSesso() {
		if(this.maschio != null) {
			if(this.maschio) {
				return ConstantsLanguage.MASCHIOS.stampa();
			} else {
				return ConstantsLanguage.FEMMINAS.stampa();
			}
		} 
		return "";
	}
	public String getTipo() {
		if(this.pf != null) {
			if(this.pf) {
				return ConstantsLanguage.PERSONA_FISICA.stampa();
			} else {
				return ConstantsLanguage.PERSONA_GIURIDICA.stampa();
			}
		} 
		return "";
	}
	public Agente getCopia() {
		Agente pr = new Agente();
		pr.setQualifica(this.getQualifica());
		pr.setUfficio(this.getUfficio());
		pr.setAutonomo(this.getAutonomo());
		pr.setVisualizzaStoria(this.getVisualizzaStoria());
		return pr;
	}
	
	public boolean isLinkValid() {
		String protocol = ApplicationUtil.protocol(getLink());
		return protocol != null;
	}
	public boolean isEmailValid() {
		return ApplicationUtil.isNotBlank(getEmail());
	}
	
}