package it.callite.model;
import java.io.Serializable;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.hibernate.annotations.OnDelete;
import org.hibernate.annotations.OnDeleteAction;

import it.callite.constants.ConstantsFormat;
import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.model.allegato.AllegatoAnagrafica;
import it.callite.model.anagrafica.AnagraficaXML;
import it.callite.model.anagrafica.CampoAggiuntivoAnagraficaXML;
import it.callite.model.anagrafica.DatoAggiuntivoAnagraficaXML;
import it.callite.model.generic.SelezionatoAbstract;
import it.callite.model.report.ColonnaModel;
import it.callite.utility.Allegato;
import it.callite.utility.ApplicationUtil;
@SuppressWarnings("serial")
@Entity
//@Audited 
@Table(name = ConstantsGlobal.tp + "anagrafica", indexes = { @Index(name= "anagrafica", columnList = "data_creazione, settore, sorgente"), @Index(name= "consenso", columnList = "consenso_tfisso, consenso_tmobile, consenso_tfax, consenso_taltro, consenso_email, consenso_rmail" ), @Index(name= "dati_anagrafica1", columnList = "cognome, nome" ), @Index(name= "dati_anagrafica2", columnList = "nome_completo, ragione_sociale" ), @Index(name= "dati_anagrafica3", columnList = "partita_iva, codice_fiscale" ), @Index(name= "dati_anagrafica4", columnList = "telefono_fisso, telefono_mobile" ), @Index(name= "dati_anagrafica5", columnList = "telefono_fax, telefono_altro" ), @Index(name= "dati_anagrafica6", columnList = "email, link" )})
public class Anagrafica extends SelezionatoAbstract  implements Serializable, Comparable<Anagrafica> {

	private String sorgente;
	@Transient
	private Long seq = Long.valueOf(0);
	
	public Long getSeq() {
		return seq;
	}

	public void setSeq(Long seq) {
		this.seq = seq;
	}

	public String getPrivacy() {
		if(id != null) {
			return URLEncoder.encode(ApplicationUtil.base64Encoder(id.toString()), ConstantsGlobal.CHARSET);
		} 
		return null;
	}
		
	public String getSorgente() {
		return sorgente;
	}
	public void setSorgente(String sorgente) {
		this.sorgente = ApplicationUtil.normalizeToNull(sorgente);
	}
	@Transient
	private List<String> motivi = new ArrayList<String>();
	
	public void addMotivi(List<String> motivi) {
		this.motivi = motivi;
	}
	
	public String getMotivi() {
		return ApplicationUtil.detoken(motivi, ConstantsGlobal.colonSeparator);
	}
	@OneToMany(mappedBy = "anagrafica")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<AllegatoAnagrafica> allegati;
	@Transient
	private List<Allegato> allegato;
	//	@OneToMany(mappedBy = "anagrafica_padre", fetch = FetchType.LAZY, cascade =  CascadeType.ALL)
	//	@OnDelete(action = OnDeleteAction.CASCADE)
	@OneToMany
	@JoinColumn(name = "anagrafica_padre_id", nullable = true, updatable=true, insertable=true)
	private List<Anagrafica> anagrafica_figli = new ArrayList<Anagrafica>();
	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "anagrafica_padre_id", nullable = true)
	private Anagrafica anagrafica_padre;
	private String codice_fiscale;
	private String cognome;
	@Transient
	private Boolean consentito = Boolean.TRUE;
	@OneToMany(mappedBy = "anagrafica", fetch = FetchType.LAZY)
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<Contatto> contatti;
	private Date data_creazione = ApplicationUtil.date();
	@Column(name="data_nascita")
	private Date date_nascita;
	
	@Transient
	private Map<String, String> datoAggiuntivi = new HashMap<String, String>(); 
	
	@Transient
	private Map<Long, DatoAggiuntivoAnagrafica> datoM = new LinkedHashMap<Long, DatoAggiuntivoAnagrafica>(); 
	@Transient
	private Map<String, DatoAggiuntivoAnagrafica> dato = new LinkedHashMap<String, DatoAggiuntivoAnagrafica>(); 
	@Transient
	private Map<CampoAggiuntivoAnagraficaXML, String> datoXml = new LinkedHashMap<CampoAggiuntivoAnagraficaXML, String>(); 
	@OneToMany(mappedBy = "anagrafica", fetch = FetchType.LAZY)
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<DatoAggiuntivoAnagrafica> dettagli;
	@Transient
	private Boolean duplicato_altro = Boolean.FALSE;
	@Transient
	private Boolean duplicato_email = Boolean.FALSE;
	@Transient
	private Boolean duplicato_link = Boolean.FALSE;
	@Transient
	private Boolean duplicato_codice = Boolean.FALSE;
	@Transient
	private Boolean duplicato_partita = Boolean.FALSE;
	@Transient
	private Boolean duplicato_fax = Boolean.FALSE;
	@Transient
	private Boolean duplicato_fisso = Boolean.FALSE;
	@Transient
	private Boolean duplicato_mobile = Boolean.FALSE;
	private String email;
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY) 
	private Long id;
	@ManyToOne(cascade = CascadeType.ALL)
	@OnDelete(action = OnDeleteAction.CASCADE)
	private Indirizzo indirizzo_legale;
	@ManyToOne(cascade = CascadeType.ALL)
	@OnDelete(action = OnDeleteAction.CASCADE)
	private Indirizzo indirizzo_operativo;
	@ManyToMany(fetch = FetchType.LAZY)
	@JoinTable(name = ConstantsGlobal.tp + "lista_anagrafica", joinColumns=@JoinColumn(name="contatti_id", nullable=false, updatable=true, insertable=true), inverseJoinColumns=@JoinColumn(name="lista_id", nullable=false, updatable=true, insertable=true)    )
	private List<Lista> lista;
	
	
	private Date consenso_tfisso;
	private Date consenso_tmobile;
	private Date consenso_tfax;
	private Date consenso_taltro;
	
	private Date consenso_email;
	private Date consenso_rmail;
	
	private String luogo_nascita;
	@Column(name="sesso")
	private Boolean maschio = Boolean.TRUE;
	private String nome;
	private String nome_completo;
	@Column(columnDefinition = "text")
	private String nota;
	@Transient
	private Boolean opposizione = Boolean.FALSE;
	private String partita_iva;
	@Column(name="persona_fisica")
	private Boolean pf = Boolean.TRUE;
	@Transient
	private String prefisso_altro;
	@Transient
	private String prefisso_fax;
	@Transient
	private String prefisso_fisso; 
	@Transient
	private String prefisso_mobile;
	private String ragione_sociale;
	private String settore;
	private String link;
	private String telefono_altro;
	private String telefono_fax;
	private String telefono_fisso;
	private String telefono_mobile;
	public Anagrafica() {
		if (indirizzo_legale == null) {
			indirizzo_legale = new Indirizzo();
		}
		if (indirizzo_operativo == null) {
			indirizzo_operativo = new Indirizzo();
		}
	}
	public Anagrafica(AnagraficaXML anagrafica) {
		update(anagrafica);
	}
	
	public void update(AnagraficaXML anagrafica) {
		this.setData_creazione(ApplicationUtil.date());
		this.setConsenso_tfisso(anagrafica.getConsenso_tfisso());
		this.setConsenso_tmobile(anagrafica.getConsenso_tmobile());
		this.setConsenso_tfax(anagrafica.getConsenso_tfax());
		this.setConsenso_taltro(anagrafica.getConsenso_taltro());
		this.setConsenso_email(anagrafica.getConsenso_email());
		this.setConsenso_rmail(anagrafica.getConsenso_rmail());
		this.setNota(anagrafica.getNota());
		this.setCodice_fiscale(anagrafica.getCodice_fiscale());
		this.setCognome(anagrafica.getCognome());
		this.setRagione_sociale(anagrafica.getRagione_sociale());
		this.setLuogo_nascita(anagrafica.getLuogo_nascita());
		this.setSettore(anagrafica.getSettore());
		this.setDate_nascita(anagrafica.getData_nascita());
		this.setEmail(anagrafica.getEmail());
		this.setIndirizzo_legale(new Indirizzo(anagrafica.getIndirizzo_legale()));
		this.setIndirizzo_operativo(new Indirizzo(anagrafica.getIndirizzo_operativo()));
		this.setMaschio(anagrafica.getSesso());
		this.setNome(anagrafica.getNome());
		this.setNome_completo(anagrafica.getNome_completo());
		this.setPartita_iva(anagrafica.getPartita_iva());
		this.setPf(anagrafica.getTipo());
		this.setTelefono_fax(anagrafica.getTelefono_fax());
		this.setTelefono_fisso(anagrafica.getTelefono_fisso());
		this.setTelefono_altro(anagrafica.getTelefono_altro());
		this.setTelefono_mobile(anagrafica.getTelefono_mobile());
		this.setLink(anagrafica.getLink());
		this.datoXml = new HashMap<CampoAggiuntivoAnagraficaXML, String>();
		List<DatoAggiuntivoAnagraficaXML> dati =  anagrafica.getDettagli();
		if(ApplicationUtil.notEmpty(dati)) {
			Iterator<DatoAggiuntivoAnagraficaXML> iter = dati.iterator();
			while (iter.hasNext()) {
				DatoAggiuntivoAnagraficaXML datoAggiuntivoAnagrafica = (DatoAggiuntivoAnagraficaXML) iter.next();
				datoXml.put(datoAggiuntivoAnagrafica.getCampo_aggiuntivo(), datoAggiuntivoAnagrafica.getValore());
			}
		}
	}
	public void addDatoM(Long id, DatoAggiuntivoAnagrafica v) {
		if(id != null && !datoM.containsKey(id)) {
			datoM.put(id, v);
		}
	}
	public Map<Long, DatoAggiuntivoAnagrafica> getDatoM() {
		return datoM;
	}
	public void addDato(String chiave, DatoAggiuntivoAnagrafica v) {
		if(chiave != null && !dato.containsKey(chiave)) {
			dato.put(chiave, v);
		}
	}
	public Map<String, String> getDatoAggiuntivi() {
		return datoAggiuntivi;
	}
	public void setDatoAggiuntivi(Map<String, String> datoAggiuntivi) {
		this.datoAggiuntivi = datoAggiuntivi;
	}
	
	public void addDatoList(String chiave, String v) {
		if(chiave != null && v != null && !datoAggiuntivi.containsKey(chiave)) {
			datoAggiuntivi.put(chiave, v);
		}
	}
	
	public int compareTo(Anagrafica o) {
		if(id != null && o.id != null) {
			return id.compareTo(o.id);
		} else if(id != null && o.id == null) {
			return -1;
		} else if(id == null && o.id != null) {
			return 1;
		} else {
			return 0;
		}
	}
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Anagrafica other = (Anagrafica) obj;
		if (consentito == null) {
			if (other.consentito != null)
				return false;
		} else if (!consentito.equals(other.consentito))
			return false;
		if (data_creazione == null) {
			if (other.data_creazione != null)
				return false;
		} else if (!data_creazione.equals(other.data_creazione))
			return false;
		if (duplicato_altro == null) {
			if (other.duplicato_altro != null)
				return false;
		} else if (!duplicato_altro.equals(other.duplicato_altro))
			return false;
		if (duplicato_codice == null) {
			if (other.duplicato_codice != null)
				return false;
		} else if (!duplicato_codice.equals(other.duplicato_codice))
			return false;
		if (duplicato_email == null) {
			if (other.duplicato_email != null)
				return false;
		} else if (!duplicato_email.equals(other.duplicato_email))
			return false;
		if (duplicato_fax == null) {
			if (other.duplicato_fax != null)
				return false;
		} else if (!duplicato_fax.equals(other.duplicato_fax))
			return false;
		if (duplicato_fisso == null) {
			if (other.duplicato_fisso != null)
				return false;
		} else if (!duplicato_fisso.equals(other.duplicato_fisso))
			return false;
		if (duplicato_link == null) {
			if (other.duplicato_link != null)
				return false;
		} else if (!duplicato_link.equals(other.duplicato_link))
			return false;
		if (duplicato_mobile == null) {
			if (other.duplicato_mobile != null)
				return false;
		} else if (!duplicato_mobile.equals(other.duplicato_mobile))
			return false;
		if (duplicato_partita == null) {
			if (other.duplicato_partita != null)
				return false;
		} else if (!duplicato_partita.equals(other.duplicato_partita))
			return false;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		if (opposizione == null) {
			if (other.opposizione != null)
				return false;
		} else if (!opposizione.equals(other.opposizione))
			return false;
		if (seq == null) {
			if (other.seq != null)
				return false;
		} else if (!seq.equals(other.seq))
			return false;
		return true;
	}
	public List<AllegatoAnagrafica> getAllegati() {
		return allegati;
	}
	public void prepareDati() {
		if(ApplicationUtil.empty(dettagli)) {
			dettagli = new ArrayList<DatoAggiuntivoAnagrafica>();
		}
		if(ApplicationUtil.notEmpty(dato)) {
			Iterator<String> iter = dato.keySet().iterator();
			while (iter.hasNext()) {
				dettagli.add(dato.get(iter.next()));
			}
		}
	}
	public List<Allegato> getAllegato() {
		if(allegato==null)
			allegato = new ArrayList<Allegato>();
		return allegato;
	}
	public int getAllegatoSize() {
		return ApplicationUtil.size(allegato);
	}
	public List<Anagrafica> getAnagrafica_figli() {
		return anagrafica_figli;
	}
	public Anagrafica getAnagrafica_padre() {
		return anagrafica_padre;
	}
	public String getCodice_fiscale() {
		return ApplicationUtil.upperCaseEmpty(codice_fiscale);
	}
	public String getCognome() {
		return ApplicationUtil.trimToEmpty(cognome);
	}
	public Boolean getConsentito() {
		return consentito;
	}
	public List<Contatto> getContatti() {
		return contatti;
	}
	public Date getData_creazione() {
		return data_creazione;
	}
	public String getData_inserimento() {
		return ApplicationUtil.formatDate(ConstantsFormat.dateTime.stampa(), getData_creazione());
	}
	public String getData_nascita() {
		return ApplicationUtil.formatDate(ConstantsFormat.date.stampa(), getDate_nascita());
	}
	public Date getDate_nascita() {
		return ApplicationUtil.aggiustaDataD(date_nascita, true);
	}
	public String getDati_fiscali() {
		StringBuilder sb = new StringBuilder();
		String denominazione = getDenominazione();
		if(ApplicationUtil.isNotBlank(denominazione)) {
			sb.append(denominazione);
			sb.append(ConstantsGlobal.NEWLINE);
		}
		String strada = getIndirizzo_legale().getStrada_completo();
		if(ApplicationUtil.isNotBlank(strada)) {
			sb.append(strada);
			sb.append(ConstantsGlobal.NEWLINE);
		}
		String comune = getIndirizzo_legale().getComune_completo();
		if(ApplicationUtil.isNotBlank(comune)) {
			sb.append(comune);
			sb.append(ConstantsGlobal.NEWLINE);
		}
		String cf = getCodice_fiscale();
		if(ApplicationUtil.isNotBlank(cf)) {
			sb.append(cf);
			sb.append(ConstantsGlobal.NEWLINE);
		}
		String piva = getPartita_iva();
		if(ApplicationUtil.isNotBlank(piva)) {
			sb.append(piva);
			sb.append(ConstantsGlobal.NEWLINE);
		}
		return ApplicationUtil.trimToEmpty(sb.toString());
	}
	public Map<String, DatoAggiuntivoAnagrafica> getDato() {
		return dato;
	}
	public Map<CampoAggiuntivoAnagraficaXML, String> getDatoXml() {
		return datoXml;
	}
	public void setDenominazione(String denominazione) {
	}
	public String getDenominazione() {
		return ApplicationUtil.denominazione(pf, nome_completo, ragione_sociale, cognome, nome, null);
	}

	public List<DatoAggiuntivoAnagrafica> getDettagli() {
		return dettagli;
	}
	public Boolean getDuplicato_altro() {
		return duplicato_altro;
	}
	public Boolean getDuplicato_email() {
		return duplicato_email;
	}
	public Boolean getDuplicato_fax() {
		return duplicato_fax;
	}
	public Boolean getDuplicato_fisso() {
		return duplicato_fisso;
	}
	public Boolean getDuplicato_mobile() {
		return duplicato_mobile;
	}
	public String getEmail() {
		return ApplicationUtil.lowerCaseEmpty(email);
	}
	public Long getId() {
		return id;
	}
	public String getIdentificativo() {
		if(ApplicationUtil.gtZero(id)) {
			return id.toString();
		} else { 
			return null;
		}
	}
	public Indirizzo getIndirizzo_legale() {
		if (indirizzo_legale == null) {
			indirizzo_legale = new Indirizzo();
		}
		return indirizzo_legale;
	}
	public Indirizzo getIndirizzo_operativo() {
		if (indirizzo_operativo == null) {
			indirizzo_operativo = new Indirizzo();
		}
		return indirizzo_operativo;
	}
	public List<Lista> getLista() {
		return lista;
	}
	public String getLuogo_nascita() {
		return ApplicationUtil.upperCaseEmpty(luogo_nascita);
	}
	public Boolean getMaschio() {
		return maschio;
	}
	@Transient
	private ColonnaModel model;

	public ColonnaModel getModel() {
		if(model == null) {
			model = new ColonnaModel(id);
		}
		return model;
	}
	public void setModel(ColonnaModel model) {
		this.model = model;
	}
	public String getNome() {
		return ApplicationUtil.trimToEmpty(nome);
	}
	public String getNome_completo() {
		return ApplicationUtil.trimToEmpty(nome_completo);
	}
	public String getNota() {
		return nota;
	}
	public Boolean getOpposizione() {
		return opposizione;
	}
	public String getPartita_iva() {
		return ApplicationUtil.upperCaseEmpty(partita_iva);
	}
	public Boolean getPf() {
		return pf;
	}
	public String getPrefisso_altro() {
		return prefisso_altro;
	} 
	public String getPrefisso_fax() {
		return prefisso_fax;
	}
	public String getPrefisso_fisso() {
		return prefisso_fisso;
	} 
	public String getPrefisso_mobile() {
		return prefisso_mobile;
	} 
	public String getRagione_sociale() {
		return ApplicationUtil.trimToEmpty(ragione_sociale);
	}
	public String getSesso() {
		if(this.maschio != null) {
			if(this.maschio) {
				return ConstantsLanguage.MASCHIOS.stampa();
			} else {
				return ConstantsLanguage.FEMMINAS.stampa();
			}
		} 
		return "";
	}
	public String getTipo() {
		if(this.pf != null) {
			if(this.pf) {
				return ConstantsLanguage.PERSONA_FISICA.stampa();
			} else {
				return ConstantsLanguage.PERSONA_GIURIDICA.stampa();
			}
		} 
		return "";
	}
	public String getSettore() {
		return ApplicationUtil.upperCaseEmpty(settore);
	}
	public String getLink() {
		return ApplicationUtil.trimToEmpty(link);
	}
	public String getDatiFiscali() {
		StringBuilder sb = new StringBuilder();
		if(ApplicationUtil.isNotBlank(getCodice_fiscale())) {
			sb.append(ConstantsGlobal.B1 + ConstantsLanguage.CODICEA.stampa() + ConstantsGlobal.B2 +": "+ getCodice_fiscale());
			sb.append(ConstantsGlobal.RETURN);
		}
		if(ApplicationUtil.isNotBlank(getPartita_iva())) {
			sb.append(ConstantsGlobal.B1 + ConstantsLanguage.PARTITAA.stampa() + ConstantsGlobal.B2 +": " + getPartita_iva());
			sb.append(ConstantsGlobal.RETURN);
		}
		return ApplicationUtil.trimToEmpty(sb.toString());
	}
	public String getDatiBase() {
		StringBuilder sb = new StringBuilder();
		if(ApplicationUtil.isNotBlank(getCognome())) {
			sb.append(ConstantsGlobal.B1 + ConstantsLanguage.COGNOME.stampa() + ConstantsGlobal.B2 +": "+ getCognome());
			sb.append(ConstantsGlobal.RETURN);
		}
		if(ApplicationUtil.isNotBlank(getNome())) {
			sb.append(ConstantsGlobal.B1 + ConstantsLanguage.NOME.stampa() + ConstantsGlobal.B2 +": " + getNome());
			sb.append(ConstantsGlobal.RETURN);
		}
		if(ApplicationUtil.isNotBlank(getNome_completo())) {
			sb.append(ConstantsGlobal.B1 + ConstantsLanguage.NOME_COMPLETO.stampa() + ConstantsGlobal.B2 +": "+ getNome_completo());
			sb.append(ConstantsGlobal.RETURN);
		}
		if(ApplicationUtil.isNotBlank(getRagione_sociale())) {
			sb.append(ConstantsGlobal.B1 + ConstantsLanguage.RAGIONE_SOCIALE.stampa() + ConstantsGlobal.B2 + ": " + getRagione_sociale());
			sb.append(ConstantsGlobal.RETURN);
		}
		return ApplicationUtil.trimToEmpty(sb.toString());
	}
	public String getTelefono() {
		StringBuilder sb = new StringBuilder();
		if(ApplicationUtil.isNotBlank(getTelefono_fisso())) {
			sb.append(ConstantsGlobal.B1 + ConstantsLanguage.FISSO.stampa() + ConstantsGlobal.B2 +": "+ getTelefono_fisso());
			sb.append(ConstantsGlobal.RETURN);
		}
		if(ApplicationUtil.isNotBlank(getTelefono_mobile())) {
			sb.append(ConstantsGlobal.B1 + ConstantsLanguage.MOBILE.stampa() + ConstantsGlobal.B2 +": " + getTelefono_mobile());
			sb.append(ConstantsGlobal.RETURN);
		}
		if(ApplicationUtil.isNotBlank(getTelefono_fax())) {
			sb.append(ConstantsGlobal.B1 + ConstantsLanguage.FAX.stampa() + ConstantsGlobal.B2 +": "+ getTelefono_fax());
			sb.append(ConstantsGlobal.RETURN);
		}
		if(ApplicationUtil.isNotBlank(getTelefono_altro())) {
			sb.append(ConstantsGlobal.B1 + ConstantsLanguage.ALTRO.stampa() + ConstantsGlobal.B2 +": "+ getTelefono_altro());
			sb.append(ConstantsGlobal.RETURN);
		}
		if(ApplicationUtil.isNotBlank(getEmail())) {
			sb.append(ConstantsGlobal.B1 + ConstantsLanguage.EMAIL.stampa() + ConstantsGlobal.B2 + ": "+ getEmail());
			sb.append(ConstantsGlobal.RETURN);
		}
		if(ApplicationUtil.isNotBlank(getLink())) {
			sb.append(ConstantsGlobal.B1 + ConstantsLanguage.LINK.stampa() + ConstantsGlobal.B2 +": "+ getLink());
			sb.append(ConstantsGlobal.RETURN);
		}
		if(ApplicationUtil.isNotBlank(getCodice_fiscale())) {
			sb.append(ConstantsGlobal.B1 + ConstantsLanguage.CODICEA.stampa() + ConstantsGlobal.B2 +": "+ getCodice_fiscale());
			sb.append(ConstantsGlobal.RETURN);
		}
		if(ApplicationUtil.isNotBlank(getPartita_iva())) {
			sb.append(ConstantsGlobal.B1 + ConstantsLanguage.PARTITAA.stampa() + ConstantsGlobal.B2 +": "+ getPartita_iva());
			sb.append(ConstantsGlobal.RETURN);
		}
		return ApplicationUtil.trimToEmpty(sb.toString());
	}
	public String getTelefono_altro() {
		return ApplicationUtil.trimToEmpty(telefono_altro);
	}
	public String getTelefono_fax() {
		return ApplicationUtil.trimToEmpty(telefono_fax);
	}
	public String getTelefono_fisso() {
		return ApplicationUtil.trimToEmpty(telefono_fisso);
	}
	public String getTelefono_mobile() {
		return ApplicationUtil.trimToEmpty(telefono_mobile);
	}
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((consentito == null) ? 0 : consentito.hashCode());
		result = prime * result + ((data_creazione == null) ? 0 : data_creazione.hashCode());
		result = prime * result + ((duplicato_altro == null) ? 0 : duplicato_altro.hashCode());
		result = prime * result + ((duplicato_codice == null) ? 0 : duplicato_codice.hashCode());
		result = prime * result + ((duplicato_email == null) ? 0 : duplicato_email.hashCode());
		result = prime * result + ((duplicato_fax == null) ? 0 : duplicato_fax.hashCode());
		result = prime * result + ((duplicato_fisso == null) ? 0 : duplicato_fisso.hashCode());
		result = prime * result + ((duplicato_link == null) ? 0 : duplicato_link.hashCode());
		result = prime * result + ((duplicato_mobile == null) ? 0 : duplicato_mobile.hashCode());
		result = prime * result + ((duplicato_partita == null) ? 0 : duplicato_partita.hashCode());
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		result = prime * result + ((opposizione == null) ? 0 : opposizione.hashCode());
		result = prime * result + ((seq == null) ? 0 : seq.hashCode());
		return result;
	}
	public void setAllegati(List<AllegatoAnagrafica> allegati) {
		this.allegati = allegati;
	}
	public void setAnagrafica_figli(List<Anagrafica> anagrafica_figli) {
		this.anagrafica_figli = anagrafica_figli;
	}
	public void setAnagrafica_padre(Anagrafica anagrafica_padre) {
		this.anagrafica_padre = anagrafica_padre;
	}
	public void setCodice_fiscale(String codice_fiscale) {
		this.codice_fiscale = ApplicationUtil.normalizeToUpperNull(codice_fiscale);
	}
	public void setCognome(String cognome) {
		this.cognome = ApplicationUtil.normalizeToNull(cognome);
	}
	public void setConsentito(Boolean consentito) {
		this.consentito = consentito;
	}
	public void setContatti(List<Contatto> contatti) {
		this.contatti = contatti;
	}
	public void setData_creazione(Date data_creazione) {
		this.data_creazione = data_creazione;
	}
	public void setDate_nascita(Date date_nascita) {
		this.date_nascita = ApplicationUtil.aggiustaDataD(date_nascita, true);
	}
	public void setDato(Map<String, DatoAggiuntivoAnagrafica> dato) {
		this.dato = dato;
	}
	public void setDatoXml(Map<CampoAggiuntivoAnagraficaXML, String> datoXml) {
		this.datoXml = datoXml;
	}
	public void setDettagli(List<DatoAggiuntivoAnagrafica> dettagli) {
		this.dettagli = dettagli;
	}
	public void setDuplicato_altro(Boolean duplicato_altro) {
		this.duplicato_altro = duplicato_altro;
	}
	public void setDuplicato_email(Boolean duplicato_email) {
		this.duplicato_email = duplicato_email;
	}
	public void setDuplicato_fax(Boolean duplicato_fax) {
		this.duplicato_fax = duplicato_fax;
	}
	public Boolean getDuplicato_codice() {
		return duplicato_codice;
	}
	public void setDuplicato_codice(Boolean duplicato_codice) {
		this.duplicato_codice = duplicato_codice;
	}
	public Boolean getDuplicato_partita() {
		return duplicato_partita;
	}
	public Boolean getDuplicato_link() {
		return duplicato_link;
	}
	public void setDuplicato_link(Boolean duplicato_link) {
		this.duplicato_link = duplicato_link;
	}
	public void setDuplicato_partita(Boolean duplicato_partita) {
		this.duplicato_partita = duplicato_partita;
	}
	public void setDuplicato_fisso(Boolean duplicato_fisso) {
		this.duplicato_fisso = duplicato_fisso;
	}
	public void setDuplicato_mobile(Boolean duplicato_mobile) {
		this.duplicato_mobile = duplicato_mobile;
	}
	public void setEmail(String email) {
		this.email = ApplicationUtil.normalizeToLowerNull(email);
	}
	public void setId(Long id) {
		this.id = id;
	}
	public void setIndirizzo_legale(Indirizzo indirizzoLegale) {
		this.indirizzo_legale = indirizzoLegale;
	}
	public void setIndirizzo_operativo(Indirizzo indirizzoOperativo) {
		this.indirizzo_operativo = indirizzoOperativo;
	}
	public void setLista(List<Lista> lista) {
		this.lista = lista;
	}
	public void setLuogo_nascita(String luogo_nascita) {
		this.luogo_nascita = ApplicationUtil.normalizeToUpperNull(luogo_nascita);
	}
	public void setMaschio(Boolean maschio) {
		this.maschio = maschio;
	}
	
	public void setNome(String nome) {
		this.nome = ApplicationUtil.normalizeToNull(nome);
	}
	public void setNome_completo(String nome_completo) {
		this.nome_completo = ApplicationUtil.normalizeToNull(nome_completo);
	}
	public void setNota(String nota) {
		this.nota = nota;
	}
	public void setOpposizione(Boolean opposizione) {
		this.opposizione = opposizione;
	}
	public void setPartita_iva(String partita_iva) {
		this.partita_iva = ApplicationUtil.normalizeToUpperNull(partita_iva);
	}
	public void setPf(Boolean pf) {
		this.pf = pf;
	}
	public void setPrefisso_altro(String prefisso_altro) {
		this.prefisso_altro = prefisso_altro;
	}
	public void setPrefisso_fax(String prefisso_fax) {
		this.prefisso_fax = prefisso_fax;
	}
	public void setPrefisso_fisso(String prefisso_fisso) {
		this.prefisso_fisso = prefisso_fisso;
	}
	public void setPrefisso_mobile(String prefisso_mobile) {
		this.prefisso_mobile = prefisso_mobile;
	}
	public void setRagione_sociale(String ragione_sociale) {
		this.ragione_sociale = ApplicationUtil.normalizeToNull(ragione_sociale);
	}
	public void setSettore(String settore) {
		this.settore = ApplicationUtil.normalizeToUpperNull(ApplicationUtil.aggiustaLista(true, ConstantsGlobal.colonSeparator, settore));
	}
	public void setLink(String link) {
		this.link = ApplicationUtil.normalizeToNull(link);
	}
	public void setTelefono_altro(String telefono_altro) {
		this.telefono_altro = ApplicationUtil.normalizeToNull(telefono_altro);
	}
	public void setTelefono_fax(String telefono_fax) {
		this.telefono_fax = ApplicationUtil.normalizeToNull(telefono_fax);
	}
	public void setTelefono_fisso(String telefono_fisso) {
		this.telefono_fisso = ApplicationUtil.normalizeToNull(telefono_fisso);
	}
	public void setTelefono_mobile(String telefono_mobile) {
		this.telefono_mobile = ApplicationUtil.normalizeToNull(telefono_mobile);
	}
	public void geocode(String key) {
		this.getIndirizzo_legale().geocode(key);
		this.getIndirizzo_operativo().geocode(key);
	}
	@Override
	public String toString() {
		return ApplicationUtil.toString(id, getDenominazione());
	}
	public String getNotaShort() {
		return ApplicationUtil.tronca(nota);
	}
	
	public Date getConsenso_tfisso() {
		return consenso_tfisso;
	}
	public void setConsenso_tfisso(Date consenso_tfisso) {
		this.consenso_tfisso = consenso_tfisso;
	}
	public Date getConsenso_tmobile() {
		return consenso_tmobile;
	}
	public void setConsenso_tmobile(Date consenso_tmobile) {
		this.consenso_tmobile = consenso_tmobile;
	}
	public Date getConsenso_tfax() {
		return consenso_tfax;
	}
	public void setConsenso_tfax(Date consenso_tfax) {
		this.consenso_tfax = consenso_tfax;
	}
	public Date getConsenso_taltro() {
		return consenso_taltro;
	}
	public void setConsenso_taltro(Date consenso_taltro) {
		this.consenso_taltro = consenso_taltro;
	}
	public Date getConsenso_email() {
		return consenso_email;
	}
	public void setConsenso_email(Date consenso_email) {
		this.consenso_email = consenso_email;
	}

	public Date getConsenso_rmail() {
		return consenso_rmail;
	}
	public void setConsenso_rmail(Date consenso_rmail) {
		this.consenso_rmail = consenso_rmail;
	}
	
	
	public String getConsenso_tfissoS() {
		return ApplicationUtil.formatDate(ConstantsFormat.dateTime.stampa(), consenso_tfisso);
	}
	
	public String getConsenso_tmobileS() {
		return ApplicationUtil.formatDate(ConstantsFormat.dateTime.stampa(), consenso_tmobile);
	}
	
	public String getConsenso_tfaxS() {
		return ApplicationUtil.formatDate(ConstantsFormat.dateTime.stampa(), consenso_tfax);
	}
	
	public String getConsenso_taltroS() {
		return ApplicationUtil.formatDate(ConstantsFormat.dateTime.stampa(), consenso_taltro);
	}
	
	public String getConsenso_emailS() {
		return ApplicationUtil.formatDate(ConstantsFormat.dateTime.stampa(), consenso_email);
	}
	

	public String getConsenso_rmailS() {
		return ApplicationUtil.formatDate(ConstantsFormat.dateTime.stampa(), consenso_rmail);
	}
	
	public Boolean getConsenso() {
		Date now = ApplicationUtil.date();
		boolean t_1 = consenso_tfisso == null || (consenso_tfisso != null && consenso_tfisso.after(now));
		boolean t_2 = consenso_tmobile == null || (consenso_tmobile != null && consenso_tmobile.after(now));
		boolean t_3 = consenso_tfax == null || (consenso_tfax != null && consenso_tfax.after(now));
		boolean t_4 = consenso_taltro == null || (consenso_taltro != null && consenso_taltro.after(now));
		boolean c_1 = consenso_email == null || (consenso_email != null && consenso_email.after(now));
		boolean c_2 = consenso_rmail == null || (consenso_rmail != null && consenso_rmail.after(now));
		return !(t_1 && t_2 && t_3 && t_4 && c_1 && c_2);
	}
	
	public Boolean getConsensoT() {
		Date now = ApplicationUtil.date();
		boolean c_1 = consenso_email == null || (consenso_email != null && consenso_email.after(now));
		boolean c_2 = consenso_rmail == null || (consenso_rmail != null && consenso_rmail.after(now));
		return (c_1 && c_2);
	}
	
	
	public Boolean getConsenso_tfissoT() {
		return consenso_tfisso == null || (consenso_tfisso != null && consenso_tfisso.after(ApplicationUtil.date()));
	}
	public Boolean getConsenso_tmobileT() {
		return consenso_tmobile == null || (consenso_tmobile != null && consenso_tmobile.after(ApplicationUtil.date()));
	}
	public Boolean getConsenso_tfaxT() {
		return consenso_tfax == null || (consenso_tfax != null && consenso_tfax.after(ApplicationUtil.date()));
	}
	public Boolean getConsenso_taltroT() {
		return consenso_taltro == null || (consenso_taltro != null && consenso_taltro.after(ApplicationUtil.date()));
	}
	
	public Boolean getConsenso_emailT() {
		return consenso_email == null || (consenso_email != null && consenso_email.after(ApplicationUtil.date()));
	}
		
	public Boolean getConsenso_rmailT() {
		return consenso_rmail == null || (consenso_rmail != null && consenso_rmail.after(ApplicationUtil.date()));
	}
	
	public boolean isLinkValid() {
		String protocol = ApplicationUtil.protocol(getLink());
		return protocol != null;
	}
	public boolean isEmailValid() {
		return ApplicationUtil.isNotBlank(getEmail());
	}
	
	public String getTelefonoC() {
		StringBuilder sb = new StringBuilder();
		if(ApplicationUtil.isNotBlank(getTelefono_fisso())) {
			sb.append(ConstantsLanguage.FISSO.stampa() + ": "+ getTelefono_fisso() + ConstantsGlobal.colonSeparator);
		}
		if(ApplicationUtil.isNotBlank(getTelefono_mobile())) {
			sb.append(ConstantsLanguage.MOBILE.stampa() + ": "+ getTelefono_mobile() + ConstantsGlobal.colonSeparator);
		}
		if(ApplicationUtil.isNotBlank(getTelefono_fax())) {
			sb.append(ConstantsLanguage.FAX.stampa() + ": "+ getTelefono_fax() + ConstantsGlobal.colonSeparator);
		}
		if(ApplicationUtil.isNotBlank(getTelefono_altro())) {
			sb.append(ConstantsLanguage.ALTRO.stampa() + ": "+ getTelefono_altro() + ConstantsGlobal.colonSeparator);
		}
		return ApplicationUtil.trimToEmpty(sb.toString());
	}
		
}