package it.callite.model;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import javax.persistence.Transient;

import it.asterisk.live.HangupCause;
import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.model.generic.SelezionatoAbstract;
import it.callite.utility.ApplicationUtil;
@SuppressWarnings("serial")
@Entity
//@Audited 
@Table(name = ConstantsGlobal.tp + "auto_contatto", indexes = { @Index(name= "auto_contatto", columnList = "data_creazione, telefono, data_avvio , data_chiamata, data_risposta, data_attesa, data_gestione, data_apertura, data_aggancio, data_rifiuto, secondi_chiamata, secondi_conversazione, secondi_avvio, secondi_totale, stato"), @Index(name = "id", columnList = "userfield") })
public class AutoContatto  extends SelezionatoAbstract implements Serializable, Comparable<AutoContatto> {
	private String userfield;
	public String getUserfield() {
		return userfield;
	}
	public void setUserfield(String userfield) {
		this.userfield = userfield;
	}
	public void impostaUserfield() {
		if(contatto != null && userfield == null) {
			userfield = contatto.getUserfield();
		}
	}
	@Transient
	private Autodialer autodialer;
	@ManyToOne
	private Utente utente;
	private String  caller1;
	private String  caller2;
	private String  callee1;
	private String  callee2;
	@ManyToOne
	private Contatto contatto;
	private Boolean inbound = Boolean.FALSE;
	private Boolean ringing = Boolean.FALSE;

	@Transient
	private boolean dial = false;

	@Transient
	private boolean hangup = false;

	public Boolean getRinging() {
		return ringing;
	}
	public void setRinging(Boolean ringing) {
		this.ringing = ringing;
	}

	public boolean isDial() {
		return dial;
	}
	public void setDial(boolean dial) {
		this.dial = dial;
	}
	public boolean isHangup() {
		return hangup;
	}
	public void setHangup(boolean hangup) {
		this.hangup = hangup;
	}

	@Column(columnDefinition = "text")
	private String causeT;
	
	@Transient
	private List<String> causeList = new ArrayList<String>(); 

	public String getCauseT() {
		return causeT;
	}
	public void setCauseT(String causeT) {
		this.causeT = causeT;
	}
	
	public String getCause() {
		return causeT;
	}

	public List<String> getCauseList() {
		return causeList;
	}
	public void setCauseList(List<String> causeList) {
		this.causeList = causeList;
	}
	
	public void addCause(Integer codice) {
		if(codice != null) {
			HangupCause cause = HangupCause.getByCode(codice);
			causeList.add((cause == null ? ConstantsGlobal.VOID : cause.toString()) + " " + "(" + codice + ")");
			causeT = ApplicationUtil.detoken(causeList, "; ");
		}
	}	

	public void addCause(String causaS) {
		String causa = ApplicationUtil.trimToNull(causaS);
		if(causa != null) {
			causeList.add(causa);
			causeT = ApplicationUtil.detoken(causeList, "; ");
		}
	}

	@Transient 
	private Date data;
	public Utente getUtente() {
		return utente;
	}
	public void setUtente(Utente utente) {
		this.utente = utente;
	}
	private Date data_aggancio;
	private Date data_apertura;
	private Date data_attesa;
	private Date data_avvio;
	private Date data_chiamata;
	private Date data_creazione = ApplicationUtil.date();
	private Date data_gestione;
	private Date data_link;
	private Date data_rifiuto;
	private Date data_risposta;
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY) 
	private Long id;
	@Transient
	private Date richiamata;
	private Integer secondi_chiamata = 0;
	private Integer secondi_conversazione = 0;
	private Integer secondi_avvio = 0;
	private Integer secondi_totale = 0;
	private Integer stato;
	private Integer tentativi;
	@Transient
	private String uniqueId1;
	@Transient
	private String uniqueId2;
	public AutoContatto() {
	}
	public AutoContatto(Contatto contatto, Integer stato, String telefono, Boolean inbound) {
		this.telefono = telefono;
		this.contatto = contatto;
		this.stato = stato;
		this.tentativi = 0;
		this.inbound = inbound;
		this.richiamata = ApplicationUtil.date();
		clear();
	}
	public void clear(){
		this.data = null;
		this.caller1 = null;
		this.caller2 = null;
		this.callee1 = null;
		this.callee2 = null;
		this.uniqueId1 = null;
		this.uniqueId2 = null;
		this.data_avvio = ApplicationUtil.date();
		this.data_chiamata = null;
		this.data_rifiuto = null;
		this.data_risposta = null;
		this.data_gestione = null;
		this.data_attesa = null;
		this.data_aggancio = null;
		this.data_apertura = null;
		this.data_link = null;
		this.secondi_chiamata = 0;
		this.secondi_conversazione = 0;
		this.secondi_avvio = 0;
		this.secondi_totale = 0;
	}
	public int compareTo(AutoContatto o) {
		if(id != null && o.id != null) {
			return id.compareTo(o.id);
		} else if(id != null && o.id == null) {
			return -1;
		} else if(id == null && o.id != null) {
			return 1;
		} else {
			return 0;
		}
	}
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		AutoContatto other = (AutoContatto) obj;
		if (contatto == null) {
			if (other.contatto != null)
				return false;
		} else if (!contatto.equals(other.contatto))
			return false;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		return true;
	}
	
	public boolean equalsCampagna(Campagna campagna) {
		if(this.contatto.getCampagna() != null && campagna != null) {
			return this.contatto.getCampagna().equals(campagna);
		} else {
			return false;
		}
	}
	public boolean equalsUniqueId1(String uniqueId1) {
		if(this.uniqueId1 != null && uniqueId1 != null) {
			return this.uniqueId1.equals(uniqueId1);
		} else {
			return false;
		}
	}
	public boolean equalsUniqueId2(String uniqueId2) {
		if(this.uniqueId2 != null && uniqueId2 != null) {
			return this.uniqueId2.equals(uniqueId2);
		} else {
			return false;
		}
	}
	public Autodialer getAutodialer() {
		return autodialer;
	}
	public String getCaller1() {
		return caller1;
	}
	public String getCaller2() {
		return caller2;
	}
	
	public String getCallee1() {
		return callee1;
	}
	public void setCallee1(String callee1) {
		this.callee1 = callee1;
	}
	public String getCallee2() {
		return callee2;
	}
	public void setCallee2(String callee2) {
		this.callee2 = callee2;
	}
	public Contatto getContatto() {
		return contatto;
	}
	public Date getData() {
		return data;
	}
	public Date getData_aggancio() {
		return data_aggancio;
	}
	public Date getData_apertura() {
		return data_apertura;
	}
	public Date getData_attesa() {
		return data_attesa;
	}
	public Date getData_avvio() {
		return data_avvio;
	}
	public Date getData_chiamata() {
		return data_chiamata;
	}
	public Date getData_creazione() {
		return data_creazione;
	}
	public Date getData_gestione() {
		return data_gestione;
	}
	public Date getData_link() {
		return data_link;
	}
	public Date getData_rifiuto() {
		return data_rifiuto;
	}
	private String telefono;
	public String getTelefono() {
		return ApplicationUtil.trimToEmpty(telefono);
	}
	public void setTelefono(String telefono) {
		this.telefono = ApplicationUtil.normalizeToNull(telefono);
	}
	public Date getData_risposta() {
		return data_risposta;
	}
	public String getDenominazione() {
		return ApplicationUtil.trimToEmpty(getContatto().getDenominazione());
	}
	public Long getId() {
		return id;
	}
	public Date getRichiamata() {
		return richiamata;
	}
	public Integer getSecondi_chiamata() {
		return secondi_chiamata;
	}
	public Integer getSecondi_conversazione() {
		return secondi_conversazione;
	}
	public Integer getStato() {
		return stato;
	}
	public String getStatoContatto() {
		return "stato.chiamata." + stato;
	}
	public String getRingingS() {
		return ApplicationUtil.isTrue(ringing) ?  ConstantsLanguage.SI.stampa() : ConstantsLanguage.NO.stampa();
	}
	public String getInboundS() {
		return ApplicationUtil.isTrue(inbound) ?  ConstantsLanguage.SI.stampa() : ConstantsLanguage.NO.stampa();
	}
	public Integer getTentativi() {
		return tentativi;
	}
	public String getUniqueId1() {
		return uniqueId1;
	}
	public String getUniqueId2() {
		return uniqueId2;
	}
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result
				+ ((contatto == null) ? 0 : contatto.hashCode());
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		return result;
	}
	public void imposta() {
		this.setSecondi_avvio(ApplicationUtil.impostaTempo(data_chiamata, data_avvio));
		this.setSecondi_chiamata(ApplicationUtil.impostaTempo(new Date[]{data_rifiuto, data_risposta}, data_chiamata));
		this.setSecondi_conversazione(ApplicationUtil.impostaTempo(new Date[]{data_attesa, data_aggancio, data_apertura}, data_gestione));
		this.setSecondi_totale(ApplicationUtil.impostaTempo(new Date[]{data_rifiuto, data_risposta, data_attesa, data_aggancio, data_apertura}, data_avvio));
	}
	public void impostaCD1(String caller1, String callee1) {
		if(ApplicationUtil.isNotBlank(caller1)) {
			setCaller1(caller1);
		}
		if(ApplicationUtil.isNotBlank(callee1)) {
			setCallee1(callee1);
		}
	}
	
	public void impostaCD2(String caller2, String callee2) {
		if(ApplicationUtil.isNotBlank(caller2)) {
			setCaller2(caller2);
		}
		if(ApplicationUtil.isNotBlank(callee2)) {
			setCallee2(callee2);
		}
	}

	public void impostaCU(String uniqueId1, String uniqueId2) {
		if(ApplicationUtil.isNotBlank(uniqueId1)) {
			setUniqueId1(uniqueId1);
		}
		if(ApplicationUtil.isNotBlank(uniqueId2)) {
			setUniqueId2(uniqueId2);
		}
	}

	public void incRichiamata(int min){
		Calendar c = ApplicationUtil.getCalendar();
		c.setTime(richiamata);
		c.add(Calendar.MINUTE, min); 
		richiamata = c.getTime();
	}
	public void incTentativi(){
		tentativi++;
	}

	public void setAutodialer(Autodialer autodialer) {
		this.autodialer = autodialer;
	}
	public void setCaller1(String caller1) {
		this.caller1 = caller1;
	}
	public void setCaller2(String caller2) {
		this.caller2 = caller2;
	}
	public void setContatto(Contatto contatto) {
		this.contatto = contatto;
	}
	public void setData(Date data) {
		this.data = data;
	}
	public void setData_aggancio(Date data_aggancio) {
		this.data_aggancio = data_aggancio;
	}
	public void setData_apertura(Date data_apertura) {
		this.data_apertura = data_apertura;
	}
	public void setData_attesa(Date data_attesa) {
		this.data_attesa = data_attesa;
	}
	public void setData_avvio(Date data_avvio) {
		this.data_avvio = data_avvio;
	}
	public void setData_chiamata(Date data_chiamata) {
		this.data_chiamata = data_chiamata;
	}
	public void setData_creazione(Date data_creazione) {
		this.data_creazione = data_creazione;
	}
	public void setData_gestione(Date data_gestione) {
		this.data_gestione = data_gestione;
	}
	public void setData_link(Date data_link) {
		this.data_link = data_link;
	}
	public void setData_rifiuto(Date data_rifiuto) {
		this.data_rifiuto = data_rifiuto;
	}
	public void setData_risposta(Date data_risposta) {
		this.data_risposta = data_risposta;
	}
	public void setId(Long id) {
		this.id = id;
	}
	public void setRichiamata(Date richiamata) {
		this.richiamata = richiamata;
	}
	public void setSecondi_chiamata(Integer secondi_chiamata) {
		this.secondi_chiamata = secondi_chiamata;
	}
	public void setSecondi_conversazione(Integer secondi_conversazione) {
		this.secondi_conversazione = secondi_conversazione;
	}
	public Integer getSecondi_avvio() {
		return secondi_avvio;
	}
	public void setSecondi_avvio(Integer secondi_avvio) {
		this.secondi_avvio = secondi_avvio;
	}
	public Integer getSecondi_totale() {
		return secondi_totale;
	}
	public void setSecondi_totale(Integer secondi_totale) {
		this.secondi_totale = secondi_totale;
	}
	public void setStato(Integer stato) {
		this.stato = stato;
	}
	public void setTentativi(Integer tentativi) {
		this.tentativi = tentativi;
	}
	public void setUniqueId1(String uniqueId1) {
		this.uniqueId1 = uniqueId1;
	}
	public void setUniqueId2(String uniqueId2) {
		this.uniqueId2 = uniqueId2;
	}
	@Override
	public String toString() {
		return ApplicationUtil.toString(id, getDenominazione());
	}
	public String getSecondi_avvioH() {
		return ApplicationUtil.format(getSecondi_avvio());
	}
	public String getSecondi_chiamataH() {
		return ApplicationUtil.format(getSecondi_chiamata());
	}
	public String getSecondi_conversazioneH() {
		return ApplicationUtil.format(getSecondi_conversazione());
	}
	public String getSecondi_totaleH() {
		return ApplicationUtil.format(getSecondi_totale());
	}
	public Boolean getInbound() {
		return inbound;
	}
	public void setInbound(Boolean inbound) {
		this.inbound = inbound;
	}

	public Long getContatto_id() {
		return getContatto().getId();
	}
	public String getCampagna_denominazione() {
		return getContatto().getCampagna_denominazione();
	}
	public String getLista_denominazione() {
		return getContatto().getLista_denominazione();
	}
	public String getAnagrafica_denominazione() {
		return getContatto().getAnagrafica().getDenominazione();
	}

}