package it.callite.model;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.faces.model.SelectItem;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.Table;
import javax.persistence.Transient;

import it.callite.constants.ConstantsGlobal;
import it.callite.model.generic.SelezionatoAbstract;
import it.callite.utility.ApplicationUtil;
@SuppressWarnings("serial")
@Entity
//@Audited
@Table(name = ConstantsGlobal.tp + "comando", indexes = { @Index(name= "comando", columnList = "data_creazione, nome" )})
public class Command  extends SelezionatoAbstract implements Serializable, Comparable<Command> {
	
	
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY) 
	private Long id;
	
	private Boolean writes = Boolean.TRUE;
	
	public Boolean getWrites() {
		if(writes == null) {
			writes = Boolean.TRUE;
		}
		return writes;
	}
	public void setWrites(Boolean writes) {
		this.writes = writes;
	}
	public Command() {
	}
	public int compareTo(Command o) {
		if(id != null && o.id != null) {
			return id.compareTo(o.id);
		} else if(id != null && o.id == null) {
			return -1;
		} else if(id == null && o.id != null) {
			return 1;
		} else {
			return 0;
		}
	}

	private Date data_creazione = ApplicationUtil.date();
	
	private String nome;
	private String orario = "20";
	@Column(columnDefinition = "text")
	private String files;
	private Boolean started = Boolean.TRUE;
	private Integer tipo = 0;
	
	public Integer getTipo() {
		if(tipo == null) {
			tipo = 0;
		}
		return tipo;
	}
	public void setTipo(Integer tipo) {
		this.tipo = tipo;
	}

	@Transient
	private List<Integer> valore_multiplo;
	
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Command other = (Command) obj;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		if (nome == null) {
			if (other.nome != null)
				return false;
		} else if (!nome.equals(other.nome))
			return false;
		if (started == null) {
			if (other.started != null)
				return false;
		} else if (!started.equals(other.started))
			return false;
		return true;
	}
	public Date getData_creazione() {
		return data_creazione;
	}
	public String getDenominazione() {
		return ApplicationUtil.trimToEmpty(getNome());
	}

	public Long getId() {
		return id;
	}
	
	public String getNome() {
		return ApplicationUtil.trimToNull(nome);
	}
	public String getOrario() {
		return orario;
	}
	
	public String getFiles() {
		return ApplicationUtil.trimToNull(files);
	}
	public String getOrarioString() {
		return ApplicationUtil.trimToEmpty(orario);
	}
	public Boolean getStarted() {
		return started;
	}
	public List<Integer> getValore_multiplo() {
		if(ApplicationUtil.eq(tipo, 0)) {
			return ApplicationUtil.tokenI(orario, ConstantsGlobal.commaSeparator);
		}
		return null;
	}
	public List<SelectItem> getValori() {
		List<SelectItem> list = new ArrayList<SelectItem>();
		for(int i=0; i<24; i++){
			Integer ora = Integer.valueOf(i); 
			list.add(new SelectItem(ora, ora.toString()));
		}
		return list;
	}
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		result = prime * result + ((nome == null) ? 0 : nome.hashCode());
		result = prime * result + ((started == null) ? 0 : started.hashCode());
		return result;
	}
	public void setData_creazione(Date data_creazione) {
		this.data_creazione = data_creazione;
	}
	
	
	public void setId(Long id) {
		this.id = id;
	}
	
	public void setNome(String nome) {
		this.nome = ApplicationUtil.trimToNull(nome);
	}
	public void setOrario(String orario) {
		this.orario = orario;
	}
	public void setFiles(String files) {
		this.files = ApplicationUtil.trimToNull(files);
	}
	public void setStarted(Boolean started) {
		this.started = started;
	}
	public void setValore_multiplo(List<Integer> valoreMultiplo) {
		orario = ApplicationUtil.detoken(valoreMultiplo, ConstantsGlobal.commaSeparator);
	}
	@Override
	public String toString() {
		return ApplicationUtil.toString(id, getDenominazione());
	}
	
	
}