package it.callite.model;
import java.io.Serializable;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Transient;
import org.hibernate.annotations.OnDelete;
import org.hibernate.annotations.OnDeleteAction;
import it.callite.chiamate.Chiamate;
import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.constants.ConstantsMap;
import it.callite.model.allegato.AllegatoContratto;
import it.callite.model.generic.SelezionatoAbstract;
import it.callite.model.report.ColonnaModel;
import it.callite.utility.Allegato;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.sincronizzazione.InfoContratto;
@SuppressWarnings("serial")
@Entity
//@Audited 
@Table(name = ConstantsGlobal.tp + "contratto", indexes = { @Index(name= "contratto", columnList = "data_creazione, data_stipula, data_contabile, data_attivazione, data_caricamento, data_esito, data_stato, data_lavorazione, seriale, stato, quantita, costo_scontato"), @Index(name = "id", columnList = "userfield") })
public class Contratto  extends SelezionatoAbstract implements Serializable , Comparable<Contratto> {
	@Transient
	private Long seq = Long.valueOf(0);
	
	public Long getSeq() {
		return seq;
	}

	public void setSeq(Long seq) {
		this.seq = seq;
	}
	private String userfield;
	public String getUserfield() {
		return userfield;
	}
	public void setUserfield(String userfield) {
		this.userfield = userfield;
	}
	public void impostaUserfield() {
		if(contatto != null && userfield == null) {
			userfield = contatto.getUserfield();
		}
	}
	@Transient
	private Map<Long, DatoAggiuntivoContratto> datoM = new LinkedHashMap<Long, DatoAggiuntivoContratto>(); 
	
	public void addDatoM(Long id, DatoAggiuntivoContratto v) {
		if(id != null && !datoM.containsKey(id)) {
			datoM.put(id, v);
		}
	}
	public Map<Long, DatoAggiuntivoContratto> getDatoM() {
		return datoM;
	}
	@Transient
	private List<String> motivi = new ArrayList<String>();
	public void addMotivi(List<String> motivi) {
		this.motivi = motivi;
	}
	public String getMotivi() {
		return ApplicationUtil.detoken(motivi, ConstantsGlobal.colonSeparator);
	}
	@Transient
	private Map<String, DatoAggiuntivoContratto> dato = new HashMap<String, DatoAggiuntivoContratto>(); 
	public void addDato(String chiave, DatoAggiuntivoContratto v) {
		dato.put(chiave, v);
	}
	public Map<String, DatoAggiuntivoContratto> getDato() {
		return dato;
	}
 
	public void setDato(Map<String, DatoAggiuntivoContratto> dato) {
		this.dato = dato;
	}
	@ManyToOne
	private Agente agente;
	@OneToMany(mappedBy = "contratto")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<AllegatoContratto> allegati;
	@Transient
	private List<Allegato> allegato;
	@ManyToOne
	private Contatto contatto;
	private BigDecimal costo_scontato;
	private Date data_esito;
	private Date data_creazione = ApplicationUtil.date();
	private Date data_stato;
	private Date data_lavorazione;
	private Date data_contabile;
	private Date data_stipula;
	private Date data_attivazione;
	private Date data_caricamento;
	@OneToMany(mappedBy = "contratto", fetch = FetchType.LAZY)
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<DatoAggiuntivoContratto> dettagli; 
	@Transient
	private Boolean duplicato = Boolean.FALSE;
	@ManyToOne
	private EsitoContratto esito;
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY) 
	private Long id;

	@Column(columnDefinition = "text")
	private String nota;
	@ManyToOne
	private Utente operatore;
	@ManyToOne
	private Prodotto prodotto;
	private Integer quantita;
	private String seriale;
	private Integer stato = ConstantsGlobal.in_attesa;
	@OneToMany(mappedBy = "contratto", fetch = FetchType.LAZY)
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<StoriaContratto> storia;
	@Transient
	private Boolean trovato = Boolean.FALSE;
	@ManyToOne
	private Utente utente;
	public int compareTo(Contratto o) {
		if(id != null && o.id != null) {
			return id.compareTo(o.id);
		} else if(id != null && o.id == null) {
			return -1;
		} else if(id == null && o.id != null) {
			return 1;
		} else {
			return 0;
		}
	}
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Contratto other = (Contratto) obj;
		if (chiave == null) {
			if (other.chiave != null)
				return false;
		} else if (!chiave.equals(other.chiave))
			return false;
		if (data_creazione == null) {
			if (other.data_creazione != null)
				return false;
		} else if (!data_creazione.equals(other.data_creazione))
			return false;
		if (duplicato == null) {
			if (other.duplicato != null)
				return false;
		} else if (!duplicato.equals(other.duplicato))
			return false;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		if (seq == null) {
			if (other.seq != null)
				return false;
		} else if (!seq.equals(other.seq))
			return false;
		if (trovato == null) {
			if (other.trovato != null)
				return false;
		} else if (!trovato.equals(other.trovato))
			return false;
		return true;
	}
	public Agente getAgente() {
		return agente;
	}
	public String getAgente_denominazione() {
		return getAgente() != null ? getAgente().getDenominazione() : "";
	}
	public String getAgente_codice() {
		return getAgente() != null ? getAgente().getCodice() : "";
	}
	public String getProdotto_denominazione() {
		return getProdotto().getNomeTipo();
	}
	public String getCategoria_denominazione() {
		return getProdotto().getCategoriaDenominazione();
	}
	public String getOperatore_denominazione() {
		return getOperatore() != null ? getOperatore().getDenominazione() : "";
	}
	public String getOperatore_codice() {
		return getOperatore() != null ? getOperatore().getUsername() : "";
	}
	public List<AllegatoContratto> getAllegati() {
		return allegati;
	}
	public List<Allegato> getAllegato() {
		if(allegato==null)
			allegato = new ArrayList<Allegato>();
		return allegato;
	}
	public int getAllegatoSize() {
		return ApplicationUtil.size(allegato);
	}
	public String getColore() {
		if(ApplicationUtil.equals(stato, ConstantsGlobal.in_attesa)) {
			return ConstantsGlobal.color_in_attesa;
		} else if(ApplicationUtil.equals(stato, ConstantsGlobal.in_pagamento)) {
			return ConstantsGlobal.color_in_pagamento;
		} else if(ApplicationUtil.equals(stato, ConstantsGlobal.pagato)) {
			return ConstantsGlobal.color_pagato;
		} else if(ApplicationUtil.equals(stato, ConstantsGlobal.in_storno)) {
			return ConstantsGlobal.color_in_storno;
		} else if(ApplicationUtil.equals(stato, ConstantsGlobal.stornato)) {
			return ConstantsGlobal.color_stornato;
		} else {
			return ConstantsGlobal.color_in_attesa;
		} 
	}
	public Contatto getContatto() {
		return contatto;
	}
	public BigDecimal getCosto_effettivo() {
		if(getVendita()) {
			BigDecimal cb = prodotto.getCosto_base();
			BigDecimal cf = prodotto.getCosto_fisso();
			BigDecimal ta = prodotto.getTasse();
			if(cb == null) {
				cb = new BigDecimal(0);
			}
			if(cf == null) {
				cf = new BigDecimal(0);
			}
			if(ta == null) {
				ta = new BigDecimal(0);
			}
			ta = ta.add(new BigDecimal(100)).divide(new BigDecimal(100));
			return new BigDecimal(getQuantita()).multiply(cb).add(cf).multiply(ta);
		} 
		return new BigDecimal(0);
	}
	public double getParziale() {
		BigDecimal i = getCosto_scontato();
		if(i == null)  {
			return 0;
		} else {
			return i.doubleValue();
		}
	}
	public BigDecimal getCosto_scontato() {
		if(costo_scontato != null && getVendita()) {
			return costo_scontato;
		} 
		return new BigDecimal(0);
	}
	public Integer getQuantita() {
		if(ApplicationUtil.gtZero(quantita) && getVendita()) {
			return quantita;
		} 
		return 1;
	}
	public BigDecimal getCosto_scontatoU() {
		return costo_scontato;
	}
	public Integer getQuantitaU() {
		return quantita;
	}
	public void impostaVendita() {
		if(!getVendita()) {
			setQuantita(1);
			setCosto_scontato(new BigDecimal(0));
		} 
	}
	public Boolean getVendita() {
		return prodotto !=null && prodotto.getVendita();
	}
	public Date getData_esito() {
		return data_esito;
	}
	public Date getData_lavorazione() {
		return data_lavorazione;
	}
	public Date getData_creazione() {
		return data_creazione;
	}
	public Date getData_stato() {
		return data_stato;
	}
	public Date getData_stipula() {
		return ApplicationUtil.aggiustaDataD(data_stipula, true);
	}
	
	public Date getData_contabile() {
		return ApplicationUtil.aggiustaDataD(data_contabile, true);
	}
	public String getDenominazione() {
		return ApplicationUtil.trimToEmpty(contatto.getDenominazione());
	}
	public List<DatoAggiuntivoContratto> getDettagli() {
		return dettagli;
	}
	public Boolean getDuplicato() {
		return duplicato;
	}
	public EsitoContratto getEsito() {
		return esito;
	}
	public String getEsito_contratto() {
		return  getEsitoEtichetta();
	}
	public String getEsitoEtichetta() {
		return  getEsito() != null ? getEsito().getEtichetta() : "";
	}
	public String getEsitoEtichettaCodice() {
		StringBuilder sb = new StringBuilder();
		sb.append(ApplicationUtil.trimToEmpty(getEsitoEtichetta()));
		if(ApplicationUtil.isNotBlank(getEsitoCodice())) {
			sb.append(" (" + getEsitoCodice() + ")");
		}
		return sb.toString();
	}
	public String getCodice_contratto() {
		return getEsitoCodice();
	}
	public String getEsitoCodice() {
		return getEsito() != null ? getEsito().getCodice() : "";
	}

	public Boolean getFatturabile() {
		boolean validTotale = costo_scontato != null;
		boolean validVendita = getVendita();
		boolean validData = data_stipula != null;
		return validTotale && validVendita && validData;
	}
	public Long getId() {
		return id;
	}
	
	public String getNota() {
		return nota;
	}
	public String getNotaShort() {
		return ApplicationUtil.tronca(nota);
	}
	public Utente getOperatore() {
		return operatore;
	}
	public Prodotto getProdotto() {
		return prodotto;
	}
	@Transient
	private ColonnaModel model;

	public ColonnaModel getModel() {
		if(model == null) {
			model = new ColonnaModel(id);
		}
		return model;
	}
	public void setModel(ColonnaModel model) {
		this.model = model;
	}
	
	public Categoria getCategoria() {
		return prodotto != null ? prodotto.getCategoria() : null;
	}
	public String getSeriale() {
		return  ApplicationUtil.trimToEmpty(seriale);
	}
	public Object getChiaveO(String tipo) {
		if(tipo != null && !tipo.equals(ConstantsMap.contratto_id)) {
			if(tipo.equals(ConstantsMap.contratto_seriale)) {
				return ApplicationUtil.trimToNull(getSeriale());
			} else {
				if(tipo.equals(ConstantsMap.anagrafica_telefono_fisso)) {
					return ApplicationUtil.trimToNull(getContatto().getAnagrafica().getTelefono_fisso());
				} else if(tipo.equals(ConstantsMap.anagrafica_telefono_mobile)) {
					return ApplicationUtil.trimToNull(getContatto().getAnagrafica().getTelefono_mobile());
				} else if(tipo.equals(ConstantsMap.anagrafica_telefono_fax)) {
					return ApplicationUtil.trimToNull(getContatto().getAnagrafica().getTelefono_fax());
				} else if(tipo.equals(ConstantsMap.anagrafica_telefono_altro)) {
					return ApplicationUtil.trimToNull(getContatto().getAnagrafica().getTelefono_altro());
				} else if(tipo.equals(ConstantsMap.anagrafica_email)) {
					return ApplicationUtil.trimToNull(getContatto().getAnagrafica().getEmail());
				} else if(tipo.equals(ConstantsMap.anagrafica_link)) {
					return ApplicationUtil.trimToNull(getContatto().getAnagrafica().getLink());
				} else if(tipo.equals(ConstantsMap.anagrafica_codice_fiscale)) {
					return ApplicationUtil.trimToNull(getContatto().getAnagrafica().getCodice_fiscale());
				} else if(tipo.equals(ConstantsMap.anagrafica_partita_iva)) {
					return ApplicationUtil.trimToNull(getContatto().getAnagrafica().getPartita_iva());
				}
			}
		} else {
			return getId();
		}
		return null;
	}
	public Object getChiaveL(String tipo) {
		String s = ApplicationUtil.trimToNull(getChiave());
		if((tipo == null || tipo.equals(ConstantsMap.contratto_id)) && s != null) {
			Long l = null;
			try {
				l = Long.parseLong(s);
			} catch (Exception nfe) {
			}
			return l;
		}
		return s;
	}
	@Transient
	private String chiave;
	public String getChiave() {
		return chiave;
	}
	public void setChiave(String chiave) {
		this.chiave = chiave;
	}
	public String getSerialeNome() {
		if(prodotto!=null && ApplicationUtil.isNotBlank(prodotto.getSeriale())) {
			return ApplicationUtil.trimToEmpty(prodotto.getSeriale()) + ": " + getSeriale();
		} else {
			return getSeriale();
		}
	}
	public String getTipo_seriale() {
		if(prodotto!=null && ApplicationUtil.isNotBlank(prodotto.getSeriale())) {
			return ApplicationUtil.trimToEmpty(prodotto.getSeriale());
		} else {
			return ConstantsLanguage.SERIALE.stampa();
		}
	}
	public Integer getStato() {
		return stato;
	}
	public String getStatoTesto() {
		if(ApplicationUtil.equals(stato, ConstantsGlobal.in_attesa)) {
			return ConstantsLanguage.STATO_IN_ATTESA.stampa();
		} else if(ApplicationUtil.equals(stato, ConstantsGlobal.in_pagamento)) {
			return ConstantsLanguage.STATO_IN_PAGAMENTO.stampa();
		} else if(ApplicationUtil.equals(stato, ConstantsGlobal.pagato)) {
			return ConstantsLanguage.STATO_PAGATO.stampa();
		} else if(ApplicationUtil.equals(stato, ConstantsGlobal.in_storno)) {
			return ConstantsLanguage.STATO_IN_STORNO.stampa();
		} else if(ApplicationUtil.equals(stato, ConstantsGlobal.stornato)) {
			return ConstantsLanguage.STATO_STORNATO.stampa();
		} else {
			return ConstantsLanguage.STATO_IN_ATTESA.stampa();
		} 
	}
	public List<StoriaContratto> getStoria() {
		return storia;
	}
	public Boolean getTrovato() {
		return trovato;
	}
	public Utente getUtente() {
		return utente;
	}
	public String getUtente_denominazione() {
		return getUtente() != null ? getUtente().getDenominazione() : "";
	}
	public String getUtente_codice() {
		return getUtente() != null ? getUtente().getUsername() : "";
	}
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((chiave == null) ? 0 : chiave.hashCode());
		result = prime * result + ((data_creazione == null) ? 0 : data_creazione.hashCode());
		result = prime * result + ((duplicato == null) ? 0 : duplicato.hashCode());
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		result = prime * result + ((seq == null) ? 0 : seq.hashCode());
		result = prime * result + ((trovato == null) ? 0 : trovato.hashCode());
		return result;
	}
	public void setAgente(Agente agente) {
		this.agente = agente;
	}
	public void setAllegati(List<AllegatoContratto> allegati) {
		this.allegati = allegati;
	}
	public void setContatto(Contatto contatto) {
		this.contatto = contatto;
	}
	public void setCosto_scontato(BigDecimal costo_scontato) {
		this.costo_scontato = costo_scontato;
	}
	public void setData_esito(Date data_esito) {
		this.data_esito = data_esito;
	}
	public void setData_lavorazione(Date data_lavorazione) {
		this.data_lavorazione = data_lavorazione;
	}
	public void setData_creazione(Date data_creazione) {
		this.data_creazione = data_creazione;
	}
	public void setData_stato(Date data_stato) {
		this.data_stato = data_stato;
	}
	public void setData_stipula(Date data_stipula) {
		this.data_stipula = ApplicationUtil.aggiustaDataD(data_stipula, true);
	}
	
	public void setData_contabile(Date data_contabile) {
		this.data_contabile = ApplicationUtil.aggiustaDataD(data_contabile, true);
	}
	public void setDettagli(List<DatoAggiuntivoContratto> dettagli) {
		this.dettagli = dettagli;
	}
	public void setDuplicato(Boolean duplicato) {
		this.duplicato = duplicato;
	}
	public void setEsito(EsitoContratto esito) {
		this.esito = esito;
	}

	public Date getData_attivazione() {
		return ApplicationUtil.aggiustaDataD(data_attivazione, true);
	}
	public void setData_attivazione(Date data_attivazione) {
		this.data_attivazione = ApplicationUtil.aggiustaDataD(data_attivazione, true);
	}
	
	public Date getData_caricamento() {
		return ApplicationUtil.aggiustaDataD(data_caricamento, true);
	}
	public void setData_caricamento(Date data_caricamento) {
		this.data_caricamento = ApplicationUtil.aggiustaDataD(data_caricamento, true);
	}
	public void setId(Long id) {
		this.id = id;
	}
	
	public void setNota(String nota) {
		this.nota = nota;
	}
	public void setOperatore(Utente operatore) {
		this.operatore = operatore;
	}
	public void setProdotto(Prodotto prodotto) {
		this.prodotto = prodotto;
	}
	public void setQuantita(Integer quantita) {
		this.quantita = quantita;
	}
	public void setSeriale(String seriale) {
		this.seriale =  ApplicationUtil.normalizeToNull(seriale);
	}
	public void setStato(Integer stato) {
		this.stato = stato;
	}
	public void setStoria(List<StoriaContratto> storia) {
		this.storia = storia;
	}
	public void setTrovato(Boolean trovato) {
		this.trovato = trovato;
	}
	public void setUtente(Utente utente) {
		this.utente = utente;
	}
	@Override
	public String toString() {
		return ApplicationUtil.toString(id, getDenominazione());
	}
	public InfoContratto getSincronia() {
		if(Chiamate.getCodaSincroniaContratti().contains(getId())) {
			return Chiamate.getCodaSincroniaContratti().get(getId());
		} 
		return null;
	}
	
	private Boolean bloccato = Boolean.FALSE;
	public String getBloccatoS() {
		return getBloccato() ? ConstantsLanguage.SI.stampa() : ConstantsLanguage.NO.stampa();
	}
	public Boolean getBloccato() {
		if(bloccato == null) {
			bloccato = Boolean.FALSE;
		}
		return bloccato;
	}
	public void setBloccato(Boolean bloccato) {
		this.bloccato = bloccato;
	}
}