package it.callite.model;

import java.io.Serializable;
import java.util.Date;
import java.util.HashSet;
import java.util.Set;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.Lob;
import javax.persistence.ManyToMany;
import javax.persistence.Table;

import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsMap;
import it.callite.model.generic.SelezionatoAbstract;
import it.callite.utility.ApplicationUtil;



@SuppressWarnings("serial")
@Entity
//@Audited 
@Table(name = ConstantsGlobal.tp + "documento", indexes = { @Index(name= "documento", columnList = "data_creazione, nome") })
public class Documento  extends SelezionatoAbstract implements Serializable, Comparable<Documento> {
	@Override
	public String toString() {
		return ApplicationUtil.toString(id, getDenominazione());
	}

	public String getDenominazione() {
		return ApplicationUtil.trimToEmpty(getNome());
	}
	
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY) 
	private Long id;

	private String nome;
	
	private Date data_creazione = ApplicationUtil.date();

	public Date getData_creazione() {
		return data_creazione;
	}

	public void setData_creazione(Date data_creazione) {
		this.data_creazione = data_creazione;
	}

	public Long getId() {
		return id;
	}

	public void setId(Long id) {
		this.id = id;
	}

	public String getNome() {
		return nome;
	}

	public void setNome(String nome) {
		this.nome = ApplicationUtil.normalizeToNull(nome);
	}

	
	public String getPercorsoString() {
		return ApplicationUtil.getNameN(nomeFile);
	}
	
	public String getTagString() {
		String name  = ApplicationUtil.getNameN(nomeFile);
		if(ApplicationUtil.asList(ConstantsMap.a_immagini).contains(ApplicationUtil.getExtensionLower(name))) {
			return "<img src=\"cid:" + nome + "." + ApplicationUtil.getExtensionLower(name) + "\" />";
		}
		return null;
	}


	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Documento other = (Documento) obj;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		return true;
	}

	
	public Boolean getAccettabile() {
		return ConstantsMap.getAccettabili().contains(ApplicationUtil.getExtensionLower(nomeFile));
	}
	
	@Lob
	private byte[] content;

	public byte[] getContent() {
		return content;
	}

	public void setContent(byte[] content) {
		this.content = content;
	}
	
	@Column(columnDefinition = "text")
	private String nomeFile;
	
	
	public String getNomeFileS() {
		String n = ApplicationUtil.trimToNull(nome);
		if(n != null) {
			return n + "." + ApplicationUtil.getExtensionN(nomeFile);
		} else {
			return nomeFile;
		}
	}

	public String getNomeFile() {
		return nomeFile;
	}

	public void setNomeFile(String nomeFile) {
		this.nomeFile = nomeFile;
	}

	public int compareTo(Documento o) {
		if(id != null && o.id != null) {
			return id.compareTo(o.id);
		} else if(id != null && o.id == null) {
			return -1;
		} else if(id == null && o.id != null) {
			return 1;
		} else {
			return 0;
		}
	}


	@ManyToMany(fetch = FetchType.LAZY)
	@JoinTable(name = ConstantsGlobal.tp + "emailTemplate_documento", joinColumns=@JoinColumn(name="documenti_id", nullable=false, updatable=true, insertable=true), inverseJoinColumns=@JoinColumn(name="emailTemplate_id", nullable=false, updatable=true, insertable=true)    )
	private Set<EmailTemplate> emailTemplate = new HashSet<EmailTemplate>();

	public Set<EmailTemplate> getEmailTemplate() {
		return emailTemplate;
	}

	public void setEmailTemplate(Set<EmailTemplate> emailTemplate) {
		this.emailTemplate = emailTemplate;
	}

	public Set<RmailTemplate> getRmailTemplate() {
		return rmailTemplate;
	}

	public void setRmailTemplate(Set<RmailTemplate> rmailTemplate) {
		this.rmailTemplate = rmailTemplate;
	}



	@ManyToMany(fetch = FetchType.LAZY)
	@JoinTable(name = ConstantsGlobal.tp + "rmailTemplate_documento", joinColumns=@JoinColumn(name="documenti_id", nullable=false, updatable=true, insertable=true), inverseJoinColumns=@JoinColumn(name="rmailTemplate_id", nullable=false, updatable=true, insertable=true)    )
	private Set<RmailTemplate> rmailTemplate = new HashSet<RmailTemplate>();


}