package it.callite.model;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.Lob;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import javax.persistence.Transient;

import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.model.generic.SelezionatoAbstract;
import it.callite.utility.Allegato;
import it.callite.utility.AllegatoUtils;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.thread.Posta;



@SuppressWarnings("serial")
@Entity
//@Audited 
@Table(name = ConstantsGlobal.tp + "email", indexes = { @Index(name= "email", columnList = "data_creazione, data_invio, codice"), @Index(name = "id", columnList = "userfield") })
public class Email extends SelezionatoAbstract implements Serializable, Posta, Comparable<Email> {
	
	private String userfield;
	public String getUserfield() {
		return userfield;
	}
	public void setUserfield(String userfield) {
		this.userfield = userfield;
	}
	public void impostaUserfield() {
		if(contatto != null && userfield == null) {
			userfield = contatto.getUserfield();
		}
	}
	@Override
	public String toString() {
		return ApplicationUtil.toString(id, getDenominazione());
	}

	public String getDenominazione() {
		return ApplicationUtil.trimToEmpty(contatto.getDenominazione());
	}


	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Email other = (Email) obj;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		return true;
	}

	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY) 
	private Long id;

	@Column(columnDefinition = "text")
	private String destinatario;
	@Column(columnDefinition = "text")
	private String oggetto;
	@Column(columnDefinition = "text")
	private String cc;
	@Column(columnDefinition = "text")
	private String sender;

	public String getCc() {
		return cc;
	}
	public void setCc(String cc) {
		this.cc = cc;
	}
	public String getSender() {
		return sender;
	}
	public void setSender(String sender) {
		this.sender = sender;
	}

	@Transient
	private Date dataSchedulata;

	public Date getDataSchedulata() {
		return ApplicationUtil.aggiustaDataT(dataSchedulata, true);
	}

	public void setDataSchedulata(Date dataSchedulata) {
		this.dataSchedulata = ApplicationUtil.aggiustaDataT(dataSchedulata, true);
	}
	@Column(columnDefinition = "text")
	private String testo;

	@ManyToOne
	private EmailTemplate emailTemplate;

	@ManyToOne
	private Contatto contatto;

	private Date data_invio;
	@Column(columnDefinition="datetime NOT NULL COMMENT 'data'")
	private Date data_creazione = ApplicationUtil.date();

	public Date getData_creazione() {
		return data_creazione;
	}

	public void setData_creazione(Date data_creazione) {
		this.data_creazione = data_creazione;
	}

	@Column(columnDefinition = "text")
	private String stato;
	private Integer codice;

	public Integer getCodice() {
		return codice;
	}

	public void setCodice(Integer codice) {
		this.codice = codice;
	}

	public void addMotivo(Integer codice, String ragione) {
		setCodice(codice);
		String s = "";
		if(ApplicationUtil.equals(codice, ConstantsGlobal.IN_INVIO)) {
			s = ConstantsLanguage.TESTO_IN_INVIO.stampa();
		} else if(ApplicationUtil.equals(codice, ConstantsGlobal.INVIATA)) {
			s = ConstantsLanguage.TESTO_INVIATA.stampa();
		} else if(ApplicationUtil.equals(codice, ConstantsGlobal.NON_INVIATA)) {
			s = ConstantsLanguage.TESTO_NON_INVIATA.stampa();
		} else if(ApplicationUtil.equals(codice, ConstantsGlobal.SCHEDULATA)) {
			s = ConstantsLanguage.TESTO_SCHEDULATA.stampa();
		} else if(ApplicationUtil.equals(codice, ConstantsGlobal.PAUSE)) {
			s = ConstantsLanguage.TESTO_PAUSE.stampa();
		} else if(ApplicationUtil.equals(codice, ConstantsGlobal.ANNULLATA)) {
			s = ConstantsLanguage.TESTO_ANNULLATA.stampa();
		} else if(ApplicationUtil.equals(codice, ConstantsGlobal.APERTO)) {
			s = ConstantsLanguage.TESTO_APERTO.stampa();
		} else {
			s = ConstantsLanguage.TESTO_IN_INVIO.stampa();
		}
		if(ApplicationUtil.isNotBlank(ragione)) {
			setStato(s + ": " + ragione);
		} else {
			setStato(s);
		}
	}

	@Transient
	private HashMap<Allegato, String> allegati;

	@ManyToOne
	private Utente utente;

	public Utente getUtente() {
		return utente;
	}

	public void setUtente(Utente utente) {
		this.utente = utente;
	}

	public Long getId() {
		return id;
	}

	public void setId(Long id) {
		this.id = id;
	}


	public String getDestinatario() {
		return destinatario;
	}

	public String getDestinatarioTemplate() {
		return destinatario;
	}

	public void setDestinatario(String destinatario) {
		this.destinatario = ApplicationUtil.trimToNull(destinatario);
	}

	public String getOggetto() {
		return oggetto;
	}

	public String getOggettoTemplate() {
		return oggetto;
	}

	public void setOggetto(String oggetto) {
		this.oggetto = oggetto;
	}

	public String getTesto() {
		return testo;
	}

	public String getTestoTemplate() {
		return testo;
	}

	public void setTesto(String testo) {
		this.testo = testo;
	}

	public EmailTemplate getEmailTemplate() {
		return emailTemplate;
	}

	public void setEmailTemplate(EmailTemplate emailTemplate) {
		this.emailTemplate = emailTemplate;
	}

	public Contatto getContatto() {
		return contatto;
	}

	public void setContatto(Contatto contatto) {
		this.contatto = contatto;
	}

	public Date getData_invio() {
		return data_invio;
	}

	public void setData_invio(Date dataInvio) {
		data_invio = dataInvio;
	}

	public String getStato() {
		return stato;
	}

	public void setStato(String stato) {
		this.stato = stato;
	}

	@Transient
	public HashMap<Allegato, String> getAllegati() {
		return allegati;
	}

	public void setAllegati(HashMap<Allegato, String> allegati) {
		this.allegati = allegati;
	}	

	@Transient
	public String getNomiAllegati() {
		List<String> nomi = new ArrayList<String>();
		if(emailTemplate != null && ApplicationUtil.isNotBlank(emailTemplate.getDocumentiNome())) {
			if(!nomi.contains(emailTemplate.getDocumentiNome())) {
				nomi.add(emailTemplate.getDocumentiNome());
			}
		}
		if(allegati != null) {
			Set<Allegato> ss = allegati.keySet();
			if (ApplicationUtil.notEmpty(ss)) {
				Iterator<Allegato> iter = ss.iterator();
				while (iter.hasNext()) {
					Allegato doc = (Allegato) iter.next();
					if(!nomi.contains(allegati.get(doc))) {
						nomi.add(allegati.get(doc));
					}
				}
			}
		}
		return ApplicationUtil.detoken(nomi, ConstantsGlobal.colonSeparator);
	}

	public int compareTo(Email o) {
		if(id != null && o.id != null) {
			return id.compareTo(o.id);
		} else if(id != null && o.id == null) {
			return -1;
		} else if(id == null && o.id != null) {
			return 1;
		} else {
			return 0;
		}
	}

	@Lob
	private byte [] allegatiTemplates;

	public byte[] getAllegatiTemplates() {
		return allegatiTemplates;
	}

	public void setAllegatiTemplates(byte[] allegatiTemplates) {
		this.allegatiTemplates = allegatiTemplates;
	}

	public void setAllegatiTemplateList(List<Allegato> allegati) {
		this.allegatiTemplates = ApplicationUtil.setAllegatiTemplateList(allegati);
	}
	public List<AllegatoUtils> getAllegatiTemplateList() {
		return ApplicationUtil.getAllegatiTemplateList(allegatiTemplates);
	}

	public List<Allegato> getAttachments() {
		return ApplicationUtil.getAllegatiReinvioList(allegatiTemplates);
	}


	public boolean valido() {
		return ApplicationUtil.isNotBlank(this.getDestinatario());
	}

	public boolean inviabile() {
		return emailTemplate != null && this.emailTemplate.getSmtp() != null;
	}

	@ManyToOne
	private Schedulatore scheduler;
	public Schedulatore getScheduler() {
		return scheduler;
	}
	public void setScheduler(Schedulatore scheduler) {
		this.scheduler = scheduler;
	}

	public String getUtente_denominazione() {
		return getUtente() != null ? getUtente().getDenominazione() : "";
	}

	public Boolean getLink() {
		if(emailTemplate != null) {
			return emailTemplate.getLink();

		}
		return false;
	}

	
	public String getTemplate() {
		return null;
	}
	
	public Boolean getHtml() {
		return true;
	}

	public String getTo() {
		return destinatario;
	}

	public String getSubject() {
		return oggetto;
	}

	public String getText() {
		return testo;
	}
}