package it.callite.model;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;

import org.hibernate.annotations.Cache;
import org.hibernate.annotations.CacheConcurrencyStrategy;
import org.hibernate.annotations.OnDelete;
import org.hibernate.annotations.OnDeleteAction;

import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.model.generic.SelezionatoAbstract;
import it.callite.utility.ApplicationUtil;
@SuppressWarnings("serial")
@Entity
//@Audited 
@Table(name = ConstantsGlobal.tp + "esito_archiviazione", indexes = { @Index(name= "esito_archiviazione", columnList = "data_creazione, etichetta, ordinamento, codice, colore, utile, positivo, tipologiaS, backoffice, solo_operatore, solo_supervisore, solo_backoffice, solo_archiviati") })
@Cache(usage = CacheConcurrencyStrategy.READ_WRITE)
public class EsitoArchiviazione  extends SelezionatoAbstract implements Serializable {
	private Integer tipologiaS = 0;
	private Integer presa_appuntamento = 0;
	private Integer invia_opposizione = 0;
	private Boolean riepilogo_registrazioni = Boolean.FALSE;
	private Boolean backoffice = Boolean.FALSE;
	@ManyToMany(fetch = FetchType.LAZY)
	@JoinTable(name = ConstantsGlobal.tp + "esiti_campagna", joinColumns=@JoinColumn(name="esito_id", nullable=false, updatable=true, insertable=true), inverseJoinColumns=@JoinColumn(name="campagna_id", nullable=false, updatable=true, insertable=true)    )
	private List<Campagna> campagne;
	private String codice;
	private Long ordinamento = Long.valueOf(0);
	@Column(columnDefinition = "text")
	private String contratto_filtro_prodotto;
	@Column(columnDefinition = "text")
	private String contratto_filtro_categoria;
	private String colore = ConstantsGlobal.colore;
	@OneToMany(mappedBy = "esitoArchiviazione")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<Contatto> contatti;
	private Boolean contratto_aiuto = Boolean.FALSE;
	private Boolean contratto_pda = Boolean.FALSE;
	private Integer contratto_imposta = 0;
	private Boolean contratto_ricerca = Boolean.FALSE;
	private Integer contratto_selezione = 0;		
	private Integer contratto_tipo = 0;	 
	private Date data_creazione = ApplicationUtil.date();
	@Column(columnDefinition = "text")
	private String descrizione;
	@ManyToMany(fetch = FetchType.LAZY)
	@JoinTable(name = ConstantsGlobal.tp + "esito_email", joinColumns=@JoinColumn(name="esito_id", nullable=false, updatable=true, insertable=true), inverseJoinColumns=@JoinColumn(name="email_id", nullable=false, updatable=true, insertable=true)    )
	private List<EmailTemplate> emails;
	private String etichetta;	
	
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY) 
	private Long id;	
	private Boolean inserisci_contatto = Boolean.FALSE;
	private Boolean invia_email_cliente = Boolean.FALSE;		
	private Boolean invia_rmail_cliente = Boolean.FALSE;	
	@Column(columnDefinition = "text")
	private String email_notifica;		
	@Column(columnDefinition = "text")
	private String messaggio_notifica;	
	@Column(columnDefinition = "text")
	private String notizia_notifica;	
	private Boolean invia_telecom_cliente = Boolean.FALSE;		
	@ManyToMany(fetch = FetchType.LAZY)
	@JoinTable(name = ConstantsGlobal.tp + "esito_rmail", joinColumns=@JoinColumn(name="esito_id", nullable=false, updatable=true, insertable=true), inverseJoinColumns=@JoinColumn(name="rmail_id", nullable=false, updatable=true, insertable=true)    )
	private List<RmailTemplate> rmails;
	private Boolean modifica_contatto = Boolean.FALSE;
	private Boolean positivo = Boolean.FALSE;	
	
	private Boolean solo_archiviati = Boolean.FALSE;	
	private Boolean solo_backoffice = Boolean.TRUE;
	private Boolean solo_operatore = Boolean.TRUE;
	private Boolean solo_supervisore = Boolean.TRUE;
	@ManyToMany(fetch = FetchType.LAZY)
	@JoinTable(name = ConstantsGlobal.tp + "esito_sondaggio", joinColumns=@JoinColumn(name="esito_id", nullable=false, updatable=true, insertable=true), inverseJoinColumns=@JoinColumn(name="sondaggio_id", nullable=false, updatable=true, insertable=true)    )
	private List<SondaggioAnagrafica> sondaggi;
	private Integer anagrafica_sondaggio = 0;
	@OneToMany(mappedBy = "archiviazione")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<StoriaContatto> storia_contatto;
	
	@ManyToMany(fetch = FetchType.LAZY)
	@JoinTable(name = ConstantsGlobal.tp + "esito_telecom", joinColumns=@JoinColumn(name="esito_id", nullable=false, updatable=true, insertable=true), inverseJoinColumns=@JoinColumn(name="telecom_id", nullable=false, updatable=true, insertable=true)    )
	private List<TelecomTemplate> telecoms;
	private Boolean template_email = Boolean.FALSE;
	private Boolean template_rmail = Boolean.FALSE;				
	private Boolean template_telecom = Boolean.FALSE;
	private Boolean utile = Boolean.FALSE;
	private Boolean workflow = Boolean.FALSE;
	
	private Integer attesa = 360;
	private Integer max_tentativi = 0;
	private Boolean conta_tentativi = Boolean.TRUE;	
	
	public void setDaNonRichiamare(){
		attesa = 360;
		max_tentativi=0;
		esito_collegato = null;
		conta_tentativi = Boolean.TRUE;
	}
	
	public void setDaRichiamare(){
		invia_opposizione = 0;
	}
	public Integer getInvia_opposizione() {
		if(ApplicationUtil.ltZero(invia_opposizione)) {
			invia_opposizione = 0;
		}
		return invia_opposizione;
	}
	public void setInvia_opposizione(Integer invia_opposizione) {
		this.invia_opposizione = invia_opposizione;
	}
	public Boolean getConta_tentativi() {
		if(conta_tentativi == null) {
			conta_tentativi = Boolean.TRUE;
		}
		return conta_tentativi;
	}

	public Integer getAttesa() {
		if(attesa == null) {
			attesa = 360;
		}
		return attesa;
	}

	public void setAttesa(Integer attesa) {
		this.attesa = attesa;
	}

	public Integer getMax_tentativi() {
		if(max_tentativi == null) {
			max_tentativi = 0;
		}
		return max_tentativi;
	}

	public void setMax_tentativi(Integer max_tentativi) {
		this.max_tentativi = max_tentativi;
	}

	public List<EsitoArchiviazione> getEsiti_collegati() {
		return esiti_collegati;
	}

	public void setEsiti_collegati(List<EsitoArchiviazione> esiti_collegati) {
		this.esiti_collegati = esiti_collegati;
	}

	public EsitoArchiviazione getEsito_collegato() {
		return esito_collegato;
	}

	public void setEsito_collegato(EsitoArchiviazione esito_collegato) {
		this.esito_collegato = esito_collegato;
	}

	public void setConta_tentativi(Boolean conta_tentativi) {
		this.conta_tentativi = conta_tentativi;
	}

	@OneToMany
	@JoinColumn(name = "esito_collegato_id", nullable = true, updatable=true, insertable=true)
	private List<EsitoArchiviazione> esiti_collegati = new ArrayList<EsitoArchiviazione>();

	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "esito_collegato_id", nullable = true)
	private EsitoArchiviazione esito_collegato;
	
	
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		EsitoArchiviazione other = (EsitoArchiviazione) obj;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		return true;
	}
	public Boolean getBackoffice() {
		return backoffice;
	}
	public List<Campagna> getCampagne() {
		return campagne;
	}
	public String getCodice() {
		return ApplicationUtil.trimToEmpty(codice);
	}
	public void setOrdinamento(Long ordinamento) {
		this.ordinamento = ordinamento;
	}
	public Long getOrdinamento() {
		if(ApplicationUtil.ltZero(ordinamento)) {
			ordinamento = Long.valueOf(0);
		}
		return ordinamento;
	}
	public String getColore() {
		return colore;
	}
	public List<Contatto> getContatti() {
		return contatti;
	}
	public Boolean getContratto_aiuto() {
		return contratto_aiuto;
	}
	public Integer getContratto_imposta() {
		return contratto_imposta;
	}
	public Boolean getContratto_ricerca() {
		return contratto_ricerca;
	}
	
	public Integer getContratto_selezione() {
		if(ApplicationUtil.ltZero(contratto_selezione)) {
			contratto_selezione = 0;
		}
		return contratto_selezione;
	}
	public EsitoArchiviazione getCopia() {
		EsitoArchiviazione copia = new EsitoArchiviazione();
		copia.setCodice(this.getCodice());
		copia.setOrdinamento(this.getOrdinamento());
		copia.setEtichetta(this.getEtichetta() + " " +  ConstantsLanguage.COPY.stampa());
		copia.setDescrizione(this.getDescrizione());
		copia.setPositivo(this.getPositivo());			
		copia.setUtile(this.getUtile());
		copia.setInvia_opposizione(this.getInvia_opposizione());
		copia.setTipologiaS(this.getTipologiaS());		
		copia.setBackoffice(this.getBackoffice());
		copia.setContratto_aiuto(this.getContratto_aiuto());
		copia.setContratto_pda(this.getContratto_pda());
		copia.setTemplate_email(this.getTemplate_email());
		copia.setTemplate_rmail(this.getTemplate_rmail());
		copia.setTemplate_telecom(this.getTemplate_telecom());
		copia.setSolo_archiviati(this.getSolo_archiviati());
		copia.setSolo_backoffice(this.getSolo_backoffice());
		copia.setSolo_operatore(this.getSolo_operatore());
		copia.setSolo_supervisore(this.getSolo_supervisore());
		copia.setInserisci_contatto(this.getInserisci_contatto());
		copia.setModifica_contatto(this.getModifica_contatto());
		copia.setContratto_imposta(this.getContratto_imposta());
		copia.setInvia_email_cliente(this.getInvia_email_cliente());
		copia.setEmail_notifica(this.getEmail_notifica());	
		copia.setNotizia_notifica(this.getNotizia_notifica());
		copia.setMessaggio_notifica(this.getMessaggio_notifica());
		copia.setContratto_selezione(this.getContratto_selezione());
		copia.setContratto_ricerca(this.getContratto_ricerca());
		copia.setRiepilogo_registrazioni(this.getRiepilogo_registrazioni());
		copia.setInvia_telecom_cliente(this.getInvia_telecom_cliente());
		copia.setInvia_rmail_cliente(this.getInvia_rmail_cliente());
		copia.setAnagrafica_sondaggio(this.getAnagrafica_sondaggio());	
		copia.setContratto_tipo(this.getContratto_tipo());
		copia.setPresa_appuntamento(this.getPresa_appuntamento());
		copia.setColore(this.getColore());
		copia.setContratto_filtro_categoria(this.getContratto_filtro_categoria());
		copia.setContratto_filtro_prodotto(this.getContratto_filtro_prodotto());
		copia.setWorkflow(this.getWorkflow());
		copia.setMax_tentativi(this.getMax_tentativi());
		copia.setConta_tentativi(this.getConta_tentativi());
		copia.setEsito_collegato(this.getEsito_collegato());
		copia.setAttesa(this.getAttesa());
		return copia;
	}
	public Boolean getRiepilogo_registrazioni() {
		return riepilogo_registrazioni;
	}
	public void setRiepilogo_registrazioni(Boolean riepilogo_registrazioni) {
		this.riepilogo_registrazioni = riepilogo_registrazioni;
	}
	public Date getData_creazione() {
		return data_creazione;
	}
	public String getDenominazione() {
		return ApplicationUtil.trimToEmpty(getEtichettaCodice());
	}
	public String getDescrizione() {
		return ApplicationUtil.trimToEmpty(descrizione);
	}
	public List<EmailTemplate> getEmails() {
		return emails;
	}
	public String getEtichetta() {
		return etichetta;
	} 
	public String getEtichettaCodice() {
		StringBuilder sb = new StringBuilder();
		sb.append(ApplicationUtil.trimToEmpty(etichetta));
		if(ApplicationUtil.isNotBlank(codice)) {
			sb.append(" (" + codice + ")");
		}
		return sb.toString();
	}
		
	public Long getId() {
		return id;
	} 
	public Boolean getInserisci_contatto() {
		return inserisci_contatto;
	}
	public Boolean getInvia_email_cliente() {
		return invia_email_cliente;
	}		

	public Boolean getInvia_rmail_cliente() {
		return invia_rmail_cliente;
	}
	
	
	public List<RmailTemplate> getRmails() {
		return rmails;
	}
	public Boolean getModifica_contatto() {
		return modifica_contatto;
	}
	public Boolean getPositivo() {
		return positivo;
	}
	
	public Boolean getSolo_archiviati() {
		return solo_archiviati;
	}
	public Boolean getSolo_backoffice() {
		return solo_backoffice;
	}
	public Boolean getSolo_operatore() {
		return solo_operatore;
	}
	public Boolean getSolo_supervisore() {
		return solo_supervisore;
	}
	public List<SondaggioAnagrafica> getSondaggi() {
		return sondaggi;
	}
	public List<StoriaContatto> getStoria_contatto() {
		return storia_contatto;
	}
	
	public Boolean getTemplate_email() {
		return template_email;
	}
	
	public Boolean getTemplate_rmail() {
		return template_rmail;
	}
	
	public Boolean getUtile() {
		return utile;
	}
	public boolean has_sceltaCalendario(){
		return ApplicationUtil.eq(this.presa_appuntamento, 2) || ApplicationUtil.eq(this.presa_appuntamento, 3);
	}
	public boolean has_sceltaCalendarioForm(){
		return ApplicationUtil.eq(this.presa_appuntamento, 1) || ApplicationUtil.eq(this.presa_appuntamento, 3);
	}
	public boolean has_sceltaRegistrazioni(){
		return ApplicationUtil.isTrue(this.riepilogo_registrazioni);
	}
	public boolean has_sceltaCampi(){
		return ApplicationUtil.gtZero(this.contratto_tipo);
	}
	public boolean has_sceltaData(){
		return ApplicationUtil.eq(this.tipologiaS, 1) || ApplicationUtil.eq(this.tipologiaS, 2);
	} 
	public boolean has_sceltaInserisciContatto(){
		return ApplicationUtil.isTrue(this.inserisci_contatto);
	}
	public boolean has_sceltaModificaContatto(){
		return ApplicationUtil.isTrue(this.modifica_contatto);
	}
	public boolean has_sceltaProdotti(){
		return ApplicationUtil.gtZero(this.contratto_imposta);
	}
	public boolean has_sceltaSondaggio(){
		return ApplicationUtil.gtZero(this.anagrafica_sondaggio);
	}
	public boolean has_sceltaTemplateEmail(){
		return ApplicationUtil.isTrue(this.template_email);
	}
	
	public boolean has_sceltaTemplateRmail(){
		return ApplicationUtil.isTrue(this.template_rmail);
	}
	
	
	public boolean has_sceltaVocalOrder(){
		return ApplicationUtil.isTrue(this.contratto_aiuto);
	}
	public boolean has_sceltaContrattoPda(){
		return ApplicationUtil.isTrue(this.contratto_pda);
	}
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		return result;
	}
	public Integer getPresa_appuntamento() {
		return presa_appuntamento;
	}
	public void setPresa_appuntamento(Integer presa_appuntamento) {
		this.presa_appuntamento = presa_appuntamento;
	}
	public void setBackoffice(Boolean backoffice) {
		this.backoffice = backoffice;
	}
	public void setCampagne(List<Campagna> campagne) {
		this.campagne = campagne;
	}
	public void setCodice(String codice) {
		this.codice = ApplicationUtil.normalizeToEmpty(codice);
	}
	public void setColore(String colore) {
		this.colore = colore;
	}
	public void setContatti(List<Contatto> contatti) {
		this.contatti = contatti;
	}
	public void setContratto_aiuto(Boolean contratto_aiuto) {
		this.contratto_aiuto = contratto_aiuto;
	}
	public void setContratto_imposta(Integer sceltaProdotti) {
		contratto_imposta = sceltaProdotti;
	}
	public void setContratto_ricerca(Boolean contratto_ricerca) {
		this.contratto_ricerca = contratto_ricerca;
	}
	
	public void setContratto_selezione(Integer contratto_selezione) {
		this.contratto_selezione = contratto_selezione;
	}
	public Integer getContratto_tipo() {
		return contratto_tipo;
	}
	public void setContratto_tipo(Integer contratto_tipo) {
		this.contratto_tipo = contratto_tipo;
	}
	public void setData_creazione(Date data_creazione) {
		this.data_creazione = data_creazione;
	}
	public void setDescrizione(String descrizione) {
		this.descrizione = ApplicationUtil.trimToNull(descrizione);
	}
	public void setEmails(List<EmailTemplate> emails) {
		this.emails = emails;
	}
	public void setEtichetta(String etichetta) {
		this.etichetta = ApplicationUtil.normalizeToNull(etichetta);
	}
	
	public void setId(Long id) {
		this.id = id;
	}
	public void setInserisci_contatto(Boolean inserisci_contatto) {
		this.inserisci_contatto = inserisci_contatto;
	}
	public void setInvia_email_cliente(Boolean inviaMailCliente) {
		invia_email_cliente = inviaMailCliente;
	}
	
	public void setInvia_rmail_cliente(Boolean invia_rmail_cliente) {
		this.invia_rmail_cliente = invia_rmail_cliente;
	}
	public String getEmail_notifica() {
		return ApplicationUtil.lowerCaseEmpty(email_notifica);
	}
	public void setEmail_notifica(String email_notifica) {
		this.email_notifica = ApplicationUtil.lowerCaseNull(ApplicationUtil.aggiustaLista(true, ConstantsGlobal.colonSeparator, email_notifica));
	}
	public String getMessaggio_notifica() {
		return ApplicationUtil.trimToEmpty(messaggio_notifica);
	}
	public void setMessaggio_notifica(String messaggio_notifica) {
		this.messaggio_notifica = ApplicationUtil.aggiustaLista(true, ConstantsGlobal.colonSeparator, messaggio_notifica);
	}

	public void setNotizia_notifica(String notizia_notifica) {
		this.notizia_notifica = ApplicationUtil.aggiustaLista(true, ConstantsGlobal.colonSeparator, notizia_notifica);
	}
	public String getNotizia_notifica() {
		return ApplicationUtil.trimToEmpty(notizia_notifica);
	}

	public void setRmails(List<RmailTemplate> rmails) {
		this.rmails = rmails;
	}
	public void setModifica_contatto(Boolean modifica_contatto) {
		this.modifica_contatto = modifica_contatto;
	}
	public void setPositivo(Boolean positivo) {
		this.positivo = positivo;
	}	
	public Integer getTipologiaS() {
		if(tipologiaS == null) {
			tipologiaS = 0;
		} 
		return tipologiaS;
	}
	public void setTipologiaS(Integer tipologiaS) {
		this.tipologiaS = tipologiaS;
	}
	
	public void setSolo_archiviati(Boolean soloArchiviati) {
		solo_archiviati = soloArchiviati;
	}	
	public void setSolo_backoffice(Boolean soloBackoffice) {
		solo_backoffice = soloBackoffice;
	}
	public void setSolo_operatore(Boolean solo_operatore) {
		this.solo_operatore = solo_operatore;
	}
	public void setSolo_supervisore(Boolean solo_supervisore) {
		this.solo_supervisore = solo_supervisore;
	}
	public void setSondaggi(List<SondaggioAnagrafica> sondaggi) {
		this.sondaggi = sondaggi;
	}
	public Integer getAnagrafica_sondaggio() {
		return anagrafica_sondaggio;
	}
	public void setAnagrafica_sondaggio(Integer anagrafica_sondaggio) {
		this.anagrafica_sondaggio = anagrafica_sondaggio;
	}
	public void setStoria_contatto(List<StoriaContatto> storia_contatto) {
		this.storia_contatto = storia_contatto;
	}

	public void setTemplate_email(Boolean template_email) {
		this.template_email = template_email;
	}
	
	public void setTemplate_rmail(Boolean template_rmail) {
		this.template_rmail = template_rmail;
	}
	
	public void setUtile(Boolean utile) {
		this.utile = utile;
	}
	public String getContratto_filtro_prodotto() {
		return ApplicationUtil.trimToEmpty(contratto_filtro_prodotto);
	}
	public void setContratto_filtro_prodotto(String contratto_filtro_prodotto) {
		this.contratto_filtro_prodotto = ApplicationUtil.normalizeToNull(contratto_filtro_prodotto);
	}
	public String getContratto_filtro_categoria() {
		return ApplicationUtil.trimToEmpty(contratto_filtro_categoria);
	}
	public void setContratto_filtro_categoria(String contratto_filtro_categoria) {
		this.contratto_filtro_categoria = ApplicationUtil.normalizeToNull(contratto_filtro_categoria);
	}
	@Override
	public String toString() {
		return ApplicationUtil.toString(id, getDenominazione());
	}
	@OneToMany(mappedBy="esitoArchiviazione")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<Workflow> esiti; 
	public List<Workflow> getEsiti() {
		return esiti;
	}
	public void setEsiti(List<Workflow> esiti) {
		this.esiti = esiti;
	}
	public Boolean getWorkflow() {
		return workflow;
	}
	public void setWorkflow(Boolean workflow) {
		this.workflow = workflow;
	}
	public Boolean getInvia_telecom_cliente() {
		return invia_telecom_cliente;
	}
	public Boolean getTemplate_telecom() {
		return template_telecom;
	}
	public boolean has_sceltaTemplateTelecom(){
		return ApplicationUtil.isTrue(this.template_telecom);
	}
	public void setTemplate_telecom(Boolean template_telecom) {
		this.template_telecom = template_telecom;
	}
	public void setTelecoms(List<TelecomTemplate> telecoms) {
		this.telecoms = telecoms;
	}
	public void setInvia_telecom_cliente(Boolean invia_telecom_cliente) {
		this.invia_telecom_cliente = invia_telecom_cliente;
	}
	public List<TelecomTemplate> getTelecoms() {
		return telecoms;
	}

	public Boolean getContratto_pda() {
		return contratto_pda;
	}

	public void setContratto_pda(Boolean contratto_pda) {
		this.contratto_pda = contratto_pda;
	}
	
}