package it.callite.model;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Transient;
import org.hibernate.annotations.Cache;
import org.hibernate.annotations.CacheConcurrencyStrategy;
import org.hibernate.annotations.OnDelete;
import org.hibernate.annotations.OnDeleteAction;
import it.callite.constants.ConstantsEsito;
import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.model.generic.SelezionatoAbstract;
import it.callite.utility.ApplicationUtil;
@SuppressWarnings("serial")
@Entity
//@Audited 
@Table(name = ConstantsGlobal.tp + "esito_chiamata", indexes = { @Index(name= "esito_chiamata", columnList = "data_creazione, etichetta, ordinamento, codice, colore, ordine, ragione_pubblica, ragione_privata") })
@Cache(usage = CacheConcurrencyStrategy.READ_WRITE)
public class EsitoChiamata extends SelezionatoAbstract implements Serializable {
	
	@Transient
	private String identificativo;
	private Integer attesa = 360;
	private String codice;
	private Long ordinamento = Long.valueOf(0);
	
	private Boolean workflow = Boolean.FALSE;
	
	private Long ordine = ConstantsGlobal.defaultOrdine;
	private String colore = ConstantsGlobal.colore;
	@OneToMany(mappedBy = "esitoChiamata")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<Contatto> contatti;
	
	private Boolean conta_tentativi = Boolean.TRUE;
	private Date data_creazione = ApplicationUtil.date();
	@Column(columnDefinition = "text")
	private String descrizione;
	@OneToMany
	@JoinColumn(name = "esito_collegato_id", nullable = true, updatable=true, insertable=true)
	private List<EsitoChiamata> esiti_collegati = new ArrayList<EsitoChiamata>();
	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "esito_collegato_id", nullable = true)
	private EsitoChiamata esito_collegato;
	
	@OneToMany
	@JoinColumn(name = "esitoAbbattuta_id", nullable = true, updatable=true, insertable=true)
	private List<Autodialer> esitoAbbattuta = new ArrayList<Autodialer>();
	
	@OneToMany
	@JoinColumn(name = "esitoInbound_id", nullable = true, updatable=true, insertable=true)
	private List<Autodialer> esitoInbound = new ArrayList<Autodialer>();
	
	public List<Autodialer> getEsitoInbound() {
		return esitoInbound;
	}
	public void setEsitoInbound(List<Autodialer> esitoInbound) {
		this.esitoInbound = esitoInbound;
	}
	@OneToMany
	@JoinColumn(name = "esitoAgganciata_id", nullable = true, updatable=true, insertable=true)
	private List<Autodialer> esitoAgganciata = new ArrayList<Autodialer>(); 
	@OneToMany
	@JoinColumn(name = "esitoApertura_id", nullable = true, updatable=true, insertable=true)
	private List<Autodialer> esitoApertura = new ArrayList<Autodialer>();
	private Boolean esitoAutodialer = Boolean.TRUE;
	@OneToMany
	@JoinColumn(name = "esitoCongestion_id", nullable = true, updatable=true, insertable=true)
	private List<Autodialer> esitoCongestion = new ArrayList<Autodialer>();
	@OneToMany
	@JoinColumn(name = "esitoFallita_id", nullable = true, updatable=true, insertable=true)
	private List<Autodialer> esitoFallita = new ArrayList<Autodialer>();
	@OneToMany
	@JoinColumn(name = "esitoHangup_id", nullable = true, updatable=true, insertable=true)
	private List<Autodialer> esitoHangup = new ArrayList<Autodialer>();
	@OneToMany
	@JoinColumn(name = "esitoMacchina_id", nullable = true, updatable=true, insertable=true)
	private List<Autodialer> esitoMacchina = new ArrayList<Autodialer>();
	@OneToMany
	@JoinColumn(name = "esitoNonRisposto_id", nullable = true, updatable=true, insertable=true)
	private List<Autodialer> esitoNonRisposto = new ArrayList<Autodialer>();
	
	@OneToMany
	@JoinColumn(name = "esitoRinging_id", nullable = true, updatable=true, insertable=true)
	private List<Autodialer> esitoRinging = new ArrayList<Autodialer>();
	
	@OneToMany
	@JoinColumn(name = "esito_id", nullable = true, updatable=true, insertable=true)
	private List<Campagna> campagne = new ArrayList<Campagna>();
	public List<Campagna> getCampagne() {
		return campagne;
	}
	public void setCampagne(List<Campagna> campagne) {
		this.campagne = campagne;
	}
	private Boolean esitoNormale = Boolean.TRUE;
	@OneToMany
	@JoinColumn(name = "esitoOccupato_id", nullable = true, updatable=true, insertable=true)
	private List<Autodialer> esitoOccupato = new ArrayList<Autodialer>();
	@OneToMany
	@JoinColumn(name = "esitoPrenotazione_id", nullable = true, updatable=true, insertable=true)
	private List<Autodialer> esitoPrenotazione = new ArrayList<Autodialer>();
	@OneToMany
	@JoinColumn(name = "esitoRifiutata_id", nullable = true, updatable=true, insertable=true)
	private List<Autodialer> esitoRifiutata = new ArrayList<Autodialer>();
	@OneToMany
	@JoinColumn(name = "esitoSaltata_id", nullable = true, updatable=true, insertable=true)
	private List<Autodialer> esitoSaltata = new ArrayList<Autodialer>();
	@OneToMany
	@JoinColumn(name = "esitoUscita_id", nullable = true, updatable=true, insertable=true)
	private List<Autodialer> esitoUscita = new ArrayList<Autodialer>();
	private Boolean esitoVisibile = Boolean.TRUE;
	private String etichetta;
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY) 
	private Long id;
	private String immagine;
	private Integer max_tentativi = 0;
	private Integer invia_opposizione = 0;
	private String ragione_privata = null;
	private String ragione_pubblica = null;
	private Boolean richiamare = Boolean.FALSE;
	@OneToMany(mappedBy = "esito")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<StoriaContatto> storia_contatto;
	public EsitoChiamata() { }
	public EsitoChiamata(String etichetta, String descrizione) {
		this.setEtichetta(etichetta);
		this.setDescrizione(descrizione);
	}
	public Boolean getConta_tentativi() {
		if(conta_tentativi == null) {
			conta_tentativi = Boolean.TRUE;
		}
		return conta_tentativi;
	}
	public void setConta_tentativi(Boolean conta_tentativi) {
		this.conta_tentativi = conta_tentativi;
	}
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		EsitoChiamata other = (EsitoChiamata) obj;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		return true;
	}
	public Integer getAttesa() {
		return attesa;
	}
	public String getCodice() {
		return ApplicationUtil.trimToEmpty(codice);
	}
	public void setOrdinamento(Long ordinamento) {
		this.ordinamento = ordinamento;
	}
	public Long getOrdinamento() {
		if(ApplicationUtil.ltZero(ordinamento)) {
			ordinamento = Long.valueOf(0);
		}
		return ordinamento;
	}
	public String getColore() {
		return colore;
	}
	public List<Contatto> getContatti() {
		return contatti;
	}
	public EsitoChiamata getCopia() {
		EsitoChiamata copia = new EsitoChiamata();
		copia.setOrdine(this.getOrdine());
		copia.setCodice(this.getCodice());
		copia.setOrdinamento(this.getOrdinamento());
		copia.setOrdinamento(this.getOrdinamento());
		copia.setRagione_pubblica(this.getRagione_pubblica());
		copia.setRagione_privata(this.getRagione_privata());
		copia.setEtichetta(this.getEtichetta() + " " + ConstantsLanguage.COPY.stampa());
		copia.setDescrizione(this.getDescrizione());
		copia.setAttesa(this.getAttesa());
		copia.setImmagine(this.getImmagine());
		copia.setMax_tentativi(this.getMax_tentativi());
		copia.setConta_tentativi(this.getConta_tentativi());
		copia.setRichiamare(this.getRichiamare());
		copia.setEsitoAutodialer(this.getEsitoAutodialer());
		copia.setEsitoNormale(this.getEsitoNormale());
		copia.setEsitoVisibile(this.getEsitoVisibile());
		copia.setEsito_collegato(this.getEsito_collegato());
		copia.setColore(this.getColore());
		copia.setInvia_opposizione(this.getInvia_opposizione());
		copia.setWorkflow(this.getWorkflow());
		return copia;
	}
	
	public Integer getInvia_opposizione() {
		if(ApplicationUtil.ltZero(invia_opposizione)) {
			invia_opposizione = 0;
		}
		return invia_opposizione;
	}
	public void setInvia_opposizione(Integer invia_opposizione) {
		this.invia_opposizione = invia_opposizione;
	}
	public Date getData_creazione() {
		return data_creazione;
	}
	public String getDenominazione() {
		return ApplicationUtil.trimToEmpty(getEtichettaCodice());
	}
	public String getDescrizione() {
		return ApplicationUtil.trimToEmpty(descrizione);
	}
	public Boolean getEditabile() {
		List<Long> keys = new ArrayList<Long>(ConstantsEsito.DEFAULTS.keySet());
		return !keys.contains(getId());
	}
	
	
	public Boolean getNuovo() {
		return id != null && id.longValue() == ConstantsEsito.ID_DA_CHIAMARE;
	}
	
	public Boolean getPrivato() {
		return id != null && id.longValue() == ConstantsEsito.ID_RICHIAMO_PRIVATO;
	}
	
	public Boolean getPubblico() {
		return id != null && id.longValue() == ConstantsEsito.ID_RICHIAMO_PUBBLICO;
	}
	
	public Boolean getOk() {
		return id != null && id.longValue() == ConstantsEsito.ID_OK;
	}
	public List<EsitoChiamata> getEsiti_collegati() {
		return esiti_collegati;
	}
	
	public EsitoChiamata getEsito_collegato() {
		return esito_collegato;
	}
	public List<Autodialer> getEsitoAbbattuta() {
		return esitoAbbattuta;
	}
	public List<Autodialer> getEsitoAgganciata() {
		return esitoAgganciata;
	}
	
	public List<Autodialer> getEsitoApertura() {
		return esitoApertura;
	}
	public Boolean getEsitoAutodialer() {
		return esitoAutodialer;
	}
	public List<Autodialer> getEsitoCongestion() {
		return esitoCongestion;
	}
	public List<Autodialer> getEsitoFallita() {
		return esitoFallita;
	}
	public List<Autodialer> getEsitoHangup() {
		return esitoHangup;
	}
	public List<Autodialer> getEsitoMacchina() {
		return esitoMacchina;
	}
	public List<Autodialer> getEsitoNonRisposto() {
		return esitoNonRisposto;
	}
	public Boolean getEsitoNormale() {
		return esitoNormale;
	}
	public List<Autodialer> getEsitoOccupato() {
		return esitoOccupato;
	}
	public List<Autodialer> getEsitoPrenotazione() {
		return esitoPrenotazione;
	}
	public List<Autodialer> getEsitoRifiutata() {
		return esitoRifiutata;
	}
	public List<Autodialer> getEsitoSaltata() {
		return esitoSaltata;
	}
	public List<Autodialer> getEsitoUscita() {
		return esitoUscita;
	}
	public Boolean getEsitoVisibile() {
		if(esitoVisibile == null)
			esitoVisibile = Boolean.TRUE;
		return esitoVisibile;
	}
	public String getEtichetta() {
		return etichetta;
	}
	public String getEtichettaCodice() {
		StringBuilder sb = new StringBuilder();
		sb.append(ApplicationUtil.trimToEmpty(etichetta));
		if(ApplicationUtil.isNotBlank(codice)) {
			sb.append(" (" + codice + ")");
		}
		return sb.toString();
	}
	public Long getId() {
		return id;
	}
	public String getImmagine() {
		return immagine;
	}
	public Integer getMax_tentativi() {
		return max_tentativi;
	}
	public Long getOrdine() {
		if(ApplicationUtil.ltZero(ordine)){
			ordine = ConstantsGlobal.defaultOrdine;
		}
		return ordine;
	}
	public String getRagione_privata() {
		return ApplicationUtil.trimToNull(ragione_privata);
	}
	public String getRagione_pubblica() {
		return ApplicationUtil.trimToNull(ragione_pubblica);
	}
	
	public Boolean getRichiamare() {
		return richiamare;
	}
	
	public Boolean getSelezionabile() {
		return getEditabile();
	}
	public List<StoriaContatto> getStoria_contatto() {
		return storia_contatto;
	}
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		return result;
	}
	public void setAttesa(Integer attesa) {
		this.attesa = attesa;
	}
	public void setCodice(String codice) {
		this.codice = ApplicationUtil.normalizeToEmpty(codice);
	}
	public void setColore(String colore) {
		this.colore = colore;
	}
	public void setContatti(List<Contatto> contatti) {
		this.contatti = contatti;
	}
	public void setData_creazione(Date data_creazione) {
		this.data_creazione = data_creazione;
	}
	public void setDescrizione(String descrizione) {
		this.descrizione = ApplicationUtil.trimToNull(descrizione);
	}
	public void setEsiti_collegati(List<EsitoChiamata> esiti_collegati) {
		this.esiti_collegati = esiti_collegati;
	}
	public void setEsito_collegato(EsitoChiamata esito_collegato) {
		this.esito_collegato = esito_collegato;
	}
	public void setEsitoAbbattuta(List<Autodialer> esitoAbbattuta) {
		this.esitoAbbattuta = esitoAbbattuta;
	}
	public void setEsitoAgganciata(List<Autodialer> esitoAgganciata) {
		this.esitoAgganciata = esitoAgganciata;
	}
	public void setEsitoApertura(List<Autodialer> esitoApertura) {
		this.esitoApertura = esitoApertura;
	}
	public void setEsitoAutodialer(Boolean esitoAutodialer) {
		this.esitoAutodialer = esitoAutodialer;
	}
	public void setEsitoCongestion(List<Autodialer> esitoCongestion) {
		this.esitoCongestion = esitoCongestion;
	}
	public void setEsitoFallita(List<Autodialer> esitoFallita) {
		this.esitoFallita = esitoFallita;
	}
	public void setEsitoHangup(List<Autodialer> esitoHangup) {
		this.esitoHangup = esitoHangup;
	}
	public void setEsitoMacchina(List<Autodialer> esitoMacchina) {
		this.esitoMacchina = esitoMacchina;
	}
	public void setEsitoNonRisposto(List<Autodialer> esitoNonRisposto) {
		this.esitoNonRisposto = esitoNonRisposto;
	}
	
	public List<Autodialer> getEsitoRinging() {
		return esitoRinging;
	}
	public void setEsitoRinging(List<Autodialer> esitoRinging) {
		this.esitoRinging = esitoRinging;
	}
	public void setEsitoNormale(Boolean esitoNormale) {
		this.esitoNormale = esitoNormale;
	}
	public void setEsitoOccupato(List<Autodialer> esitoOccupato) {
		this.esitoOccupato = esitoOccupato;
	}
	public void setEsitoPrenotazione(List<Autodialer> esitoPrenotazione) {
		this.esitoPrenotazione = esitoPrenotazione;
	}
	public void setEsitoRifiutata(List<Autodialer> esitoRifiutata) {
		this.esitoRifiutata = esitoRifiutata;
	}
	public void setEsitoSaltata(List<Autodialer> esitoSaltata) {
		this.esitoSaltata = esitoSaltata;
	}
	public void setEsitoUscita(List<Autodialer> esitoUscita) {
		this.esitoUscita = esitoUscita;
	}
	public void setEsitoVisibile(Boolean esitoVisibile) {
		this.esitoVisibile = esitoVisibile;
	}
	public void setEtichetta(String etichetta) {
		this.etichetta = ApplicationUtil.normalizeToNull(etichetta);
	}
	public void setId(Long id) {
		this.id = id;
	}
	public void setImmagine(String immagine) {
		this.immagine = immagine;
	}
	public void setMax_tentativi(Integer max_tentativi) {
		this.max_tentativi = max_tentativi;
	}
	public void setDaNonRichiamare(){
		attesa = 360;
		max_tentativi=0;
		esito_collegato = null;
		conta_tentativi = Boolean.TRUE;
	}
	public void setDaRichiamare(){
		invia_opposizione = 0;
	}
	public void setOrdine(Long ordine) {
		this.ordine = ordine;
	}
	public void setRagione_privata(String ragione_privata) {
		this.ragione_privata = ApplicationUtil.trimToNull(ragione_privata);
	}
	public void setRagione_pubblica(String ragione_pubblica) {
		this.ragione_pubblica = ApplicationUtil.trimToNull(ragione_pubblica);
	}
	public void setRichiamare(Boolean richiamare) {
		this.richiamare = richiamare;
	}
	public void setStoria_contatto(List<StoriaContatto> storia_contatto) {
		this.storia_contatto = storia_contatto;
	}
	public String getIdentificativo() {
		return identificativo;
	}
	public void setIdentificativo(String identificativo) {
		this.identificativo = identificativo;
	}
	@OneToMany(mappedBy="esitoChiamata")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<Workflow> esiti; 
	public List<Workflow> getEsiti() {
		return esiti;
	}
	public void setEsiti(List<Workflow> esiti) {
		this.esiti = esiti;
	}
	@Override
	public String toString() {
		return ApplicationUtil.toString(id, getDenominazione());
	}
	public Boolean getWorkflow() {
		return workflow;
	}
	public void setWorkflow(Boolean workflow) {
		this.workflow = workflow;
	}
	
	
}