package it.callite.model;
import java.io.Serializable;
import java.util.Date;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.Lob;
import javax.persistence.Table;

import it.callite.constants.ConstantsGlobal;
import it.callite.model.generic.SelezionatoAbstract;
import it.callite.utility.ApplicationUtil;
@SuppressWarnings("serial")
@Entity
//@Audited 
@Table(name = ConstantsGlobal.tp + "ingresso", indexes = { @Index(name= "ingresso", columnList = "data_creazione, nome") })
public class Ingresso extends SelezionatoAbstract implements Serializable, Comparable<Ingresso> {
	
	@Column(columnDefinition = "text")
	private String modulo;
	@Lob
	private byte[] pdf;

	public byte[] getPdf() {
		return pdf;
	}
	public void setPdf(byte[] pdf) {
		this.pdf = pdf;
	}

	public Byte[] getFileP() {
		if(isFile()) {
			return ApplicationUtil.toObject(pdf);
		}
		return null;
	}

	public boolean isFile() {
		return pdf != null && pdf.length > 0;
	}

	public String getModulo() {
		return ApplicationUtil.trimToNull(modulo);
	}
	public void setModulo(String modulo) {
		this.modulo = ApplicationUtil.trimToNull(modulo);
	}

	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY) 
	private Long id;
	private Date data_creazione = ApplicationUtil.date();

	private String nome;
	private String extension;
	private String add_prefisso;
	private String rem_prefisso;

	@Column(columnDefinition = "text")
	private String script;
	private Short script_velocita;


	private Boolean enabledScript = Boolean.TRUE;

	private Boolean soloAttive = Boolean.FALSE;
	private Boolean soloPubbliche = Boolean.FALSE;
	private Boolean soloAperte = Boolean.FALSE;
	private Boolean soloNuova = Boolean.FALSE;
	private Boolean soloUpdate = Boolean.FALSE;
	private Integer soloNumero = 1;
	
	@Column(columnDefinition = "text")
	private String recall_callers;
	@Column(columnDefinition = "text")
	private String recall_musicclass;

	public int compareTo(Ingresso o) {
		if(id != null && o.id != null) {
			return id.compareTo(o.id);
		} else if(id != null && o.id == null) {
			return -1;
		} else if(id == null && o.id != null) {
			return 1;
		} else {
			return 0;
		}
	}
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Ingresso other = (Ingresso) obj;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		return true;
	}
	public String getAdd_prefisso() {
		return ApplicationUtil.trimToEmpty(add_prefisso);
	}
	public Ingresso getCopia() {
		Ingresso pr = new Ingresso();
		pr.setNome(this.getNome());
		pr.setExtension(this.getExtension());
		pr.setAdd_prefisso(this.getAdd_prefisso());
		pr.setRem_prefisso(this.getRem_prefisso());
		pr.setRecall_callers(this.getRecall_callers());
		pr.setRecall_musicclass(this.getRecall_musicclass());
		pr.setSoloAperte(this.getSoloAperte());
		pr.setSoloAttive(this.getSoloAttive());
		pr.setSoloNuova(this.getSoloNuova());
		pr.setSoloNumero(this.getSoloNumero());
		pr.setSoloPubbliche(this.getSoloPubbliche());
		pr.setScript(this.getScript());
		pr.setScript_velocita(this.getScript_velocita());
		pr.setPdf(this.getPdf());
		pr.setModulo(this.getModulo());
		pr.setEnabledScript(this.getEnabledScript());
		pr.setSoloUpdate(this.getSoloUpdate());
		return pr;
	}
	public Date getData_creazione() {
		return data_creazione;
	}
	public String getDenominazione() {
		return nome;
	}
	public String getExtension() {
		return ApplicationUtil.trimToEmpty(extension);
	}
	public Long getId() {
		return id;
	}


	public String getNome() {
		return ApplicationUtil.trimToEmpty(nome);
	}

	public String getRem_prefisso() {
		return ApplicationUtil.trimToEmpty(rem_prefisso);
	}
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		return result;
	}
	public void setAdd_prefisso(String add_prefisso) {
		this.add_prefisso = ApplicationUtil.normalize(ApplicationUtil.aggiustaLista(true, ConstantsGlobal.colonSeparator, add_prefisso));
	}

	public void setData_creazione(Date data_creazione) {
		this.data_creazione = data_creazione;
	}

	public void setExtension(String extension) {
		this.extension = ApplicationUtil.normalizeToNull(extension);
	}
	public void setId(Long id) {
		this.id = id;
	}

	public void setNome(String nome) {
		this.nome = ApplicationUtil.normalizeToNull(nome);
	}

	public void setRem_prefisso(String rem_prefisso) {
		this.rem_prefisso = ApplicationUtil.normalize(ApplicationUtil.aggiustaLista(true, ConstantsGlobal.colonSeparator, rem_prefisso));
	}

	@Override
	public String toString() {
		return ApplicationUtil.toString(id, getDenominazione());
	}
	public Boolean getSoloAttive() {
		return soloAttive;
	}
	public void setSoloAttive(Boolean soloAttive) {
		this.soloAttive = soloAttive;
	}
	public Boolean getSoloPubbliche() {
		return soloPubbliche;
	}
	public void setSoloPubbliche(Boolean soloPubbliche) {
		this.soloPubbliche = soloPubbliche;
	}
	public Boolean getSoloAperte() {
		return soloAperte;
	}
	public void setSoloAperte(Boolean soloAperte) {
		this.soloAperte = soloAperte;
	}
	public Boolean getSoloNuova() {
		return soloNuova;
	}
	public void setSoloNuova(Boolean soloNuova) {
		this.soloNuova = soloNuova;
	}
	
	public Boolean getSoloUpdate() {
		return soloUpdate;
	}
	public void setSoloUpdate(Boolean soloUpdate) {
		this.soloUpdate = soloUpdate;
	}
	public void setSoloNumero(Integer soloNumero) {
		this.soloNumero = soloNumero;
	}
	public Integer getSoloNumero() {
		if(soloNumero == null) {
			soloNumero = 1;
		}
		return soloNumero;
	}
	public String getRecall_callers() {
		return ApplicationUtil.trimToNull(recall_callers);
	}
	public void setRecall_callers(String recall_callers) {
		this.recall_callers = ApplicationUtil.aggiustaLista(false, ConstantsGlobal.colonSeparator, recall_callers);
	}
	public String getRecall_musicclass() {
		return ApplicationUtil.trimToNull(recall_musicclass);
	}
	public void setRecall_musicclass(String recall_musicclass) {
		this.recall_musicclass = ApplicationUtil.trimToNull(recall_musicclass);
	}
	
	public String getScript() {
		return ApplicationUtil.trimToNull(script);
	}
	public void setScript(String script) {
		this.script = ApplicationUtil.trimToNull(script);
	}
	public Short getScript_velocita() {
		return script_velocita;
	}
	public void setScript_velocita(Short script_velocita) {
		this.script_velocita = script_velocita;
	}
	public Boolean getEnabledScript() {
		return enabledScript;
	}
	public void setEnabledScript(Boolean enabledScript) {
		this.enabledScript = enabledScript;
	}


}