package it.callite.model;

import java.io.Serializable;
import java.util.Date;
import java.util.List;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.JoinColumn;
import javax.persistence.OneToMany;
import javax.persistence.Table;

import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.model.generic.SelezionatoAbstract;
import it.callite.utility.ApplicationUtil;


@SuppressWarnings("serial")
@Entity
//@Audited 
@Table(name= ConstantsGlobal.tp + "invio", indexes = { @Index(name= "invio", columnList="data_creazione, nome") })
public class Invio extends SelezionatoAbstract implements Serializable, Comparable<Invio> {

	@Override
	public String toString() {
		return ApplicationUtil.toString(id, getDenominazione());
	}

	public String getDenominazione() {
		return ApplicationUtil.trimToEmpty(getNome());
	}

	private Date data_creazione = ApplicationUtil.date();

	public Date getData_creazione() {
		return data_creazione;
	}

	public void setData_creazione(Date data_creazione) {
		this.data_creazione = data_creazione;
	}
	private String nome;

	public String getNome() {
		return nome;
	}

	public void setNome(String nome) {
		this.nome = ApplicationUtil.normalizeToNull(nome);
	}



	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY) 
	private Long id;
	@OneToMany
	@JoinColumn(name = "invio_id", nullable = true, updatable=true, insertable=true)
	private List<RmailTemplate> rmail;
	
	public int compareTo(Invio o) {
		if(id != null && o.id != null) {
			return id.compareTo(o.id);
		} else if(id != null && o.id == null) {
			return -1;
		} else if(id == null && o.id != null) {
			return 1;
		} else {
			return 0;
		}
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Invio other = (Invio) obj;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		return true;
	}


	
	public Long getId() {
		return id;
	}

	public List<RmailTemplate> getRmail() {
		return rmail;
	}


	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		return result;
	}


	public void setId(Long id) {
		this.id = id;
	}

	public void setRmail(List<RmailTemplate> rmail) {
		this.rmail = rmail;
	}

	@Column(columnDefinition = "text")
	private String oggettoTemplate;

	public String getOggettoTemplate() {
		return oggettoTemplate;
	}

	public void setOggettoTemplate(String oggettoTemplate) {
		this.oggettoTemplate = oggettoTemplate;
	}

	public String getTestoTemplate() {
		return testoTemplate;
	}

	public void setTestoTemplate(String testoTemplate) {
		this.testoTemplate = testoTemplate;
	}

	public String getDestinatarioTemplate() {
		return destinatarioTemplate;
	}

	public void setDestinatarioTemplate(String destinatarioTemplate) {
		this.destinatarioTemplate = destinatarioTemplate;
	}


	@Column(columnDefinition = "text")
	private String testoTemplate;


	@Column(columnDefinition = "text")
	private String destinatarioTemplate;
	
	public Invio getCopia() {
		Invio copia = new Invio();
		copia.setNome(this.getNome() + " " + ConstantsLanguage.COPY.stampa());
		copia.setDestinatarioTemplate(this.getDestinatarioTemplate());
		copia.setOggettoTemplate(this.getOggettoTemplate());
		copia.setTestoTemplate(this.getTestoTemplate());
		copia.setHtml(this.getHtml());
		return copia;
	}



	private Boolean html = Boolean.TRUE;

	public Boolean getHtml() {
		return html;
	}

	public void setHtml(Boolean html) {
		this.html = html;
	}
}