package it.callite.model;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;

import javax.faces.model.SelectItem;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.hibernate.annotations.OnDelete;
import org.hibernate.annotations.OnDeleteAction;

import it.callite.chiamate.Chiamate;
import it.callite.constants.ConstantsFormat;
import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.constants.ConstantsMap;
import it.callite.model.generic.SelezionatoAbstract;
import it.callite.model.report.ColonnaModel;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.sincronizzazione.InfoNota;
@SuppressWarnings("serial")
@Entity
//@Audited 
@Table(name = ConstantsGlobal.tp + "nota", indexes = { @Index(name= "nota", columnList = "data_creazione, data_inizio, data_fine, data_termine") })
public class Nota  extends SelezionatoAbstract implements Serializable, Comparable<Nota> {
	@ManyToOne 
	private Agente agente;
	private String colore = ConstantsGlobal.sfondo;
	private Date data_creazione = ApplicationUtil.date();
	@Transient
	private ColonnaModel model;

	public ColonnaModel getModel() {
		if(model == null) {
			model = new ColonnaModel(id);
		}
		return model;
	}
	public void setModel(ColonnaModel model) {
		this.model = model;
	}
	private Date data_fine;
	private Date data_inizio;
	@Transient
	private Date data_nota;
	private Date data_termine;
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY) 
	private Long id;
	@Transient
	private Integer minTime;
	@Column(columnDefinition = "text")
	private String note;
	@ManyToOne
	private Utente operatore;
	@ManyToOne
	private Utente utente;
	private Date data_lavorazione;
	@Transient
	private int oraMax;
	@Transient
	private Boolean dotted = Boolean.FALSE;
	@Transient
	private int oraMin;
	private Integer ripetizioni = ConstantsMap.ripeti_mai;
	@OneToMany(mappedBy = "nota", fetch = FetchType.LAZY)
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<StoriaNota> storia;
	public int compareTo(Nota o) {
		if(id != null && o.id != null) {
			return id.compareTo(o.id);
		} else if(id != null && o.id == null) {
			return -1;
		} else if(id == null && o.id != null) {
			return 1;
		} else {
			return 0;
		}
	}
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Nota other = (Nota) obj;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		return true;
	}
	public Agente getAgente() {
		return agente;
	}
	public Boolean getCentrato() {
		GregorianCalendar gc_a = ApplicationUtil.getGregorianCalendar();
		gc_a.setTime(getData_inizio());
		GregorianCalendar gc_b = ApplicationUtil.getGregorianCalendar();
		gc_b.setTime(getData_fine());
		int oraInizio = gc_a.get(GregorianCalendar.HOUR_OF_DAY);
		int oraFine = gc_b.get(GregorianCalendar.HOUR_OF_DAY);
		boolean inizio_ok = oraInizio <=  oraMax;
		boolean fine_ok = oraFine >=  oraMin;
		return inizio_ok && fine_ok;
	}
	public String getColore() {
		return colore;
	}
	public Date getData_creazione() {
		return data_creazione;
	}
	public Date getData_fine() {
		return ApplicationUtil.aggiustaDataT(data_fine, false);
	}
	public Date getData_inizio() {
		return ApplicationUtil.aggiustaDataT(data_inizio, true);
	}
	public Date getData_nota() {
		if(data_nota==null)
			data_nota = getData_inizio();
		return data_nota;
	}
	public void imposta(Date data) {
		data = ApplicationUtil.aggiustaDataT(data, true);
		int diff = ApplicationUtil.timeDiff(getData_inizio(), getData_fine());
		data_nota = data;
		data_inizio = data;
		Date f1 = ApplicationUtil.fineGiornata(data);
		Date f2 = ApplicationUtil.addMinutes(data, diff);
		if(f2.after(f1)) {
			data_fine = f1;
		} else {
			data_fine = f2;
		}
	}
	public Date getData_termine() {
		return ApplicationUtil.aggiustaDataD(data_termine, false);
	}
	public String getDenominazione() {
		return ApplicationUtil.trimToEmpty(getAgente().getDenominazione());
	}
	public Long getId() {
		return id;
	}
	public Integer getMinDurata() {
		if(ApplicationUtil.ltZero(minTime))
			return ApplicationUtil.timeDiff(getData_inizio(), getData_fine());
		return minTime;
	}
	public Integer getMinTime() {
		return minTime;
	}
	public String getNote() {
		return note;
	}
	public Utente getOperatore() {
		return operatore;
	}
	public int getOraMax() {
		return oraMax;
	}
	public int getOraMin() {
		return oraMin;
	}
	public Integer getRipetizioni() {
		return ripetizioni;
	}
	public List<StoriaNota> getStoria() {
		return storia;
	}
	public String getTime() {
		return ApplicationUtil.formatDate(ConstantsFormat.time.stampa(), getData_inizio()) + "-"  + ApplicationUtil.formatDate(ConstantsFormat.time.stampa(), getData_fine());
	}
	public String getTop() {
		GregorianCalendar gc_a = ApplicationUtil.getGregorianCalendar();
		gc_a.setTime(getData_inizio());
		GregorianCalendar gc_b = ApplicationUtil.getGregorianCalendar();
		gc_b.setTime(getData_fine());
		int ooI = gc_a.get(GregorianCalendar.HOUR_OF_DAY);
		int omI = gc_a.get(GregorianCalendar.MINUTE);
		int ore = (ooI >= oraMin ? ooI : oraMin) -  oraMin;
		int minuti = (ooI >= oraMin ? omI : 0);
		int minuti_totali = (ore * 60) + minuti;
		int top = Math.round(Integer.valueOf(minuti_totali).floatValue() / (60 / ConstantsGlobal.calendarSize));
		return String.valueOf(top);
	}
	public String getWidth() {
		GregorianCalendar gc_a = ApplicationUtil.getGregorianCalendar();
		gc_a.setTime(getData_inizio());
		GregorianCalendar gc_b = ApplicationUtil.getGregorianCalendar();
		gc_b.setTime(getData_fine());
		int ooI = gc_a.get(GregorianCalendar.HOUR_OF_DAY);
		int omI = gc_a.get(GregorianCalendar.MINUTE);
		int ooF = gc_b.get(GregorianCalendar.HOUR_OF_DAY);
		int omF = gc_b.get(GregorianCalendar.MINUTE);
		int ore_a = (ooI > oraMin ? ooI : oraMin);
		int minuti_a = (ooI > oraMin ? omI : 0);
		int ore_b = (ooF > oraMax ?  oraMax : ooF);
		int minuti_b = (ooF > oraMax ? 60 : omF);
		int minuti_totali_a = (ore_a * 60) + minuti_a;
		int minuti_totali_b = (ore_b * 60) + minuti_b;
		int top = Math.round(Integer.valueOf(minuti_totali_b - minuti_totali_a).floatValue() / (60 / ConstantsGlobal.calendarSize));
		if(top < (ConstantsGlobal.calendarSize/2)) {
			top = ConstantsGlobal.calendarSize/2;
			dotted = Boolean.TRUE;
		}
		return String.valueOf(top);
	}
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		return result;
	}
	public void setAgente(Agente agente) {
		this.agente = agente;
	}
	public void setColore(String colore) {
		this.colore = colore;
	}
	public void setData_creazione(Date data_creazione) {
		this.data_creazione = data_creazione;
	}
	
	public void setData_fine(Date dataFine) {
		data_fine = ApplicationUtil.settaOrarioAData(getData_nota(), dataFine);
	}
	public void setData_inizio(Date dataInizio) {
		data_inizio = ApplicationUtil.settaOrarioAData(getData_nota(), dataInizio);
	}
	public void setData_nota(Date data_nota) {
		this.data_nota = data_nota;
	}
	public void setData_termine(Date data_termine) {
		this.data_termine = ApplicationUtil.aggiustaDataD(data_termine, false);
	}
	public void setId(Long id) {
		this.id = id;
	}
	public void setMinDurata(Integer minTime) {
		data_fine = ApplicationUtil.addMinutes(getData_inizio(), minTime);
	}
	public void setMinTime(Integer minTime) {
		this.minTime = minTime;
	}
	public void setNote(String note) {
		this.note = note;
	}
	public void setOperatore(Utente operatore) {
		this.operatore = operatore;
	}
	public void setOraInizio(String ora) {
		GregorianCalendar cal = ApplicationUtil.getGregorianCalendar();
		cal.setTime(getData_nota());
		if(ora != null) {
			cal.set(GregorianCalendar.HOUR_OF_DAY, Integer.parseInt(ora));
		} 
		cal.set(GregorianCalendar.MINUTE, 0);
		cal.set(GregorianCalendar.SECOND, 0);
		cal.set(GregorianCalendar.MILLISECOND, 0);
		data_inizio = cal.getTime();
		data_fine = ApplicationUtil.addMinutes(cal.getTime(), getMinDurata());
	}
	public void setOraMax(int oraMax) {
		this.oraMax = oraMax;
	}
	public void setOraMin(int oraMin) {
		this.oraMin = oraMin;
	}
	public void setRipetizioni(Integer ripetizioni) {
		this.ripetizioni = ripetizioni;
	}
	public void setStoria(List<StoriaNota> storia) {
		this.storia = storia;
	}
	@Override
	public String toString() {
		return ApplicationUtil.toString(id, getDenominazione());
	}
	public List<SelectItem> getTempi() {
		List<SelectItem> sis = new ArrayList<SelectItem>();
		List<Integer> tempi = ConstantsMap.getTimeNota();
		Integer min = getMinDurata();
		if(!tempi.contains(min)) {
			tempi.add(min);
		}
		Collections.sort(tempi);
		for(Integer c : tempi) {
			sis.add(new SelectItem(c, c + ""));
		}
		return sis;
	}
	public Boolean getDotted() {
		return dotted;
	}
	public void setDotted(Boolean dotted) {
		this.dotted = dotted;
	}
	public String getNotaShort() {
		return ApplicationUtil.tronca(note);
	}
	public String getNota() {
		return note;
	}
	public InfoNota getSincronia() {
		if(Chiamate.getCodaSincroniaNote().contains(getId())) {
			return Chiamate.getCodaSincroniaNote().get(getId());
		} 
		return null;
	}
	private Boolean bloccato = Boolean.FALSE;
	public String getBloccatoS() {
		return getBloccato() ? ConstantsLanguage.SI.stampa() : ConstantsLanguage.NO.stampa();
	}
	public Boolean getBloccato() {
		if(bloccato == null) {
			bloccato = Boolean.FALSE;
		}
		return bloccato;
	}
	public void setBloccato(Boolean bloccato) {
		this.bloccato = bloccato;
	}
	public Utente getUtente() {
		return utente;
	}
	public void setUtente(Utente utente) {
		this.utente = utente;
	}
	public Date getData_lavorazione() {
		return data_lavorazione;
	}
	public void setData_lavorazione(Date data_lavorazione) {
		this.data_lavorazione = data_lavorazione;
	}
	
	
}