package it.callite.model;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import javax.persistence.Transient;
import org.hibernate.annotations.Cache;
import org.hibernate.annotations.CacheConcurrencyStrategy;
import it.callite.constants.ConstantsGlobal;
import it.callite.model.generic.SelezionatoAbstract;
import it.callite.utility.ApplicationUtil;
@SuppressWarnings("serial")
@Entity
//@Audited 
@Table(name = ConstantsGlobal.tp + "opposizione", indexes = { @Index(name= "opposizione", columnList = "data_creazione, opposizione, descrizione") })
@Cache(usage = CacheConcurrencyStrategy.READ_WRITE)
public class Opposizione extends SelezionatoAbstract implements Serializable, Comparable<Opposizione> {
	@Transient
	private Long seq = Long.valueOf(0);
	
	public Long getSeq() {
		return seq;
	}

	public void setSeq(Long seq) {
		this.seq = seq;
	}
	@Transient
	private List<String> motivi = new ArrayList<String>();
	
	public void addMotivi(List<String> motivi) {
		this.motivi = motivi;
	}
	public String getMotivi() {
		return ApplicationUtil.detoken(motivi, ConstantsGlobal.colonSeparator);
	}
	@Column(columnDefinition="datetime NOT NULL COMMENT 'data'")
	private Date data_creazione = ApplicationUtil.date();
	
	@ManyToOne
	private Utente utente;
	public Utente getUtente() {
		return utente;
	}
	public void setUtente(Utente utente) {
		this.utente = utente;
	}
	@ManyToOne
	private Campagna campagna;
	public Campagna getCampagna() {
		return campagna;
	}
	public void setCampagna(Campagna campagna) {
		this.campagna = campagna;
	}
	
	
	public String getCampagna_denominazione() {
		return getCampagna() != null ? getCampagna().getDenominazione() : "";
	} 
	
	public String getUtente_denominazione() {
		return getUtente() != null ? getUtente().getDenominazione() : "";
	} 
	@Transient
	private Boolean trovato = Boolean.FALSE; 
	@Transient
	private Boolean duplicato = Boolean.FALSE; 
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY) 
	private Long id;
	private String descrizione;
	private String opposizione; 
	public int compareTo(Opposizione o) {
		if(id != null && o.id != null) {
			return id.compareTo(o.id);
		} else if(id != null && o.id == null) {
			return -1;
		} else if(id == null && o.id != null) {
			return 1;
		} else {
			return 0;
		}
	}
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Opposizione other = (Opposizione) obj;
		if (data_creazione == null) {
			if (other.data_creazione != null)
				return false;
		} else if (!data_creazione.equals(other.data_creazione))
			return false;
		if (duplicato == null) {
			if (other.duplicato != null)
				return false;
		} else if (!duplicato.equals(other.duplicato))
			return false;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		if (opposizione == null) {
			if (other.opposizione != null)
				return false;
		} else if (!opposizione.equals(other.opposizione))
			return false;
		if (seq == null) {
			if (other.seq != null)
				return false;
		} else if (!seq.equals(other.seq))
			return false;
		if (trovato == null) {
			if (other.trovato != null)
				return false;
		} else if (!trovato.equals(other.trovato))
			return false;
		return true;
	}
	public Boolean getTrovato() {
		return trovato;
	}
	public void setTrovato(Boolean trovato) {
		this.trovato = trovato;
	}
	public Date getData_creazione() {
		return data_creazione;
	}
	public String getDenominazione() {
		return ApplicationUtil.trimToEmpty(getDescrizione() + " " + getOpposizione());
	}
	public Boolean getDuplicato() {
		return duplicato;
	}
	public Long getId() {
		return id;
	}
	public String getDescrizione() {
		return ApplicationUtil.trimToEmpty(descrizione);
	}
	public String getOpposizione() {
		return ApplicationUtil.clearOpposizione(ApplicationUtil.trimToEmpty(opposizione));
	}
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((data_creazione == null) ? 0 : data_creazione.hashCode());
		result = prime * result + ((duplicato == null) ? 0 : duplicato.hashCode());
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		result = prime * result + ((opposizione == null) ? 0 : opposizione.hashCode());
		result = prime * result + ((seq == null) ? 0 : seq.hashCode());
		result = prime * result + ((trovato == null) ? 0 : trovato.hashCode());
		return result;
	}
	
	public void setData_creazione(Date data_creazione) {
		this.data_creazione = data_creazione;
	}
	public void setDuplicato(Boolean duplicato) {
		this.duplicato = duplicato;
	}
	public void setId(Long id) {
		this.id = id;
	}
	public void setDescrizione(String descrizione) {
		this.descrizione = ApplicationUtil.normalizeToNull(descrizione);
	}
	public void setOpposizione(String opposizione) {
		this.opposizione = ApplicationUtil.normalizeToNull(ApplicationUtil.clearOpposizione(opposizione));
	}
	@Override
	public String toString() {
		return ApplicationUtil.toString(id, getDenominazione());
	}
}