package it.callite.model;
import java.io.Serializable;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.validator.ValidatorException;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.Lob;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import org.hibernate.annotations.OnDelete;
import org.hibernate.annotations.OnDeleteAction;
import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.message.Message;
import it.callite.model.generic.SelezionatoAbstract;
import it.callite.utility.Allegato;
import it.callite.utility.ApplicationUtil;
@SuppressWarnings("serial")
@Entity
//@Audited 
@Table(name = ConstantsGlobal.tp + "prodotto", indexes = { @Index(name= "prodotto", columnList = "data_creazione, tipo, nome, vendita, imposta_seriale, tipologia") })
public class Prodotto extends SelezionatoAbstract implements Serializable {
	@Override
	public String toString() {
		return ApplicationUtil.toString(id, getDenominazione());
	}
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		return result;
	}
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Prodotto other = (Prodotto) obj;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		return true;
	}
	private Date data_creazione = ApplicationUtil.date();
	@Column(columnDefinition = "text")
	private String descrizione;
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY) 
	private Long id;
	private String nome;
	@Column(columnDefinition = "text")
	private String sito;
	private BigDecimal costo_base = new BigDecimal(0);
	private BigDecimal costo_fisso = new BigDecimal(0);
	private BigDecimal tasse = new BigDecimal(0);
	private Boolean vendita = Boolean.FALSE;
	private Boolean costo_abilitato = Boolean.TRUE;
	private Boolean quantita_abilitato = Boolean.TRUE;
	public Boolean getCosto_abilitato() {
		if(costo_abilitato == null) {
			costo_abilitato = Boolean.TRUE;
		}
		return costo_abilitato;
	}
	public void setCosto_abilitato(Boolean costo_abilitato) {
		this.costo_abilitato = costo_abilitato;
	}
	public Boolean getQuantita_abilitato() {
		if(quantita_abilitato == null) {
			quantita_abilitato = Boolean.TRUE;
		}
		return quantita_abilitato;
	}
	public void setQuantita_abilitato(Boolean quantita_abilitato) {
		this.quantita_abilitato = quantita_abilitato;
	}
	private String seriale;
	public String getSeriale() {
		return seriale;
	}
	public void setSeriale(String seriale) {
		this.seriale = ApplicationUtil.normalizeToNull(seriale);
	}
	public void notVendita()  {
		costo_base = new BigDecimal(0);
		costo_fisso = new BigDecimal(0);
		tasse = new BigDecimal(0);
		tipologia = ConstantsGlobal.gconteggio;
		costo_abilitato = Boolean.TRUE;
		quantita_abilitato = Boolean.TRUE;
	}
	private Integer tipo;
	public String getDescrizioneTipo() {
		if(ApplicationUtil.equals(tipo, ConstantsGlobal.tipo_residenziale)) {
			return ConstantsLanguage.TIPO_RESIDENZIALE.stampa();
		} else if(ApplicationUtil.equals(tipo, ConstantsGlobal.tipo_business)) {
			return ConstantsLanguage.TIPO_BUSINESS.stampa();
		} else if(ApplicationUtil.equals(tipo, ConstantsGlobal.tipo_misto)) {
			return ConstantsLanguage.TIPO_MISTO.stampa();
		} else {
			return "";
		} 
	}
	public Integer getTipo() {
		return tipo;
	}
	public void setTipo(Integer tipo) {
		this.tipo = tipo;
	}
	private Integer tipologia = ConstantsGlobal.gconteggio;
	public Integer getTipologia() {
		if(tipologia == null) {
			tipologia = ConstantsGlobal.gconteggio;
		}
		return tipologia;
	}
	public void setTipologia(Integer tipologia) {
		this.tipologia = tipologia;
	}
	public Boolean getVendita() {
		if(vendita == null) {
			vendita = Boolean.FALSE;
		}
		return vendita;
	}
	public void setVendita(Boolean vendita) {
		this.vendita = vendita;
	}
	private String codice;
	private String validatore_seriale;
	private Boolean richiesto_seriale = Boolean.FALSE;
	private Boolean richiesto_agente = Boolean.FALSE;
	private Boolean richiesto_esito = Boolean.FALSE;
	private Integer richiesto_data = 3;
	private Boolean imposta_seriale = Boolean.TRUE;
	public Boolean getImposta_seriale() {
		if(imposta_seriale == null) {
			imposta_seriale = Boolean.TRUE;
		}
		return imposta_seriale;
	}
	public void setImposta_seriale(Boolean imposta_seriale) {
		this.imposta_seriale = imposta_seriale;
	}
	@OneToMany(mappedBy = "prodotto")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<Gettone> gettoni = new ArrayList<Gettone>();
	@OneToMany(mappedBy = "prodotto")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<Contest> contest = new ArrayList<Contest>();
	private Boolean attivo = Boolean.TRUE;
	@ManyToOne
	private Categoria categoria;
	@OneToMany(mappedBy = "prodotto")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<Contratto> contratti;
	@OneToMany(mappedBy = "prodotto")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<StoriaContratto> storia_contratti;
	@ManyToMany(fetch = FetchType.LAZY)
	@JoinTable(name = ConstantsGlobal.tp + "prodotto_aiuto", joinColumns=@JoinColumn(name="prodotto_id", nullable=false, updatable=true, insertable=true), inverseJoinColumns=@JoinColumn(name="aiuto_id", nullable=false, updatable=true, insertable=true)    )
	private List<VocalOrder> vocalOrders;
	//@NotAudited
	@OneToMany(mappedBy = "id.prodotto")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<ProdottiInVendita> prodotti_in_vendita;
	//@NotAudited
	@OneToMany(mappedBy = "id.prodotto")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<TemplateProdotto> template;
	@ManyToMany(fetch = FetchType.LAZY)
	@JoinTable(name = ConstantsGlobal.tp + "prodotto_sondaggio", joinColumns=@JoinColumn(name="prodotto_id", nullable=false, updatable=true, insertable=true), inverseJoinColumns=@JoinColumn(name="sondaggio_id", nullable=false, updatable=true, insertable=true)    )
	private List<SondaggioContratto> sondaggi;
	public Boolean getAttivo() {
		return attivo;
	}
	public Categoria getCategoria() {
		return categoria;
	}
	public String getCodice() {
		return codice;
	}
	public List<Contest> getContest() {
		return contest;
	}
	public List<Contratto> getContratti() {
		return contratti;
	}
	public Prodotto getCopia() {
		Prodotto pr = new Prodotto();
		pr.setNome(this.getNome() + " " +  ConstantsLanguage.COPY.stampa());
		pr.setDescrizione(this.getDescrizione());
		pr.setContent(this.getContent());
		pr.setSito(this.getSito());
		pr.setCosto_base(this.getCosto_base());
		pr.setCosto_fisso(this.getCosto_fisso());
		pr.setVendita(this.getVendita());
		pr.setTasse(this.getTasse());
		pr.setSeriale(this.getSeriale());
		pr.setCodice(this.getCodice());
		pr.setTipo(this.getTipo());
		pr.setAttivo(this.getAttivo());
		pr.setCategoria(this.getCategoria());
		pr.setTipologia(this.getTipologia());
		pr.setValidatore_seriale(this.getValidatore_seriale());
		pr.setRichiesto_seriale(this.getRichiesto_seriale());
		pr.setRichiesto_agente(this.getRichiesto_agente());
		pr.setRichiesto_esito(this.getRichiesto_esito());
		pr.setRichiesto_data(this.getRichiesto_data());
		pr.setImposta_seriale(this.getImposta_seriale());
		return pr;
	}
	public Boolean getRichiesto_agente() {
		return ApplicationUtil.isTrue(richiesto_agente);
	}
	public void setRichiesto_agente(Boolean richiesto_agente) {
		this.richiesto_agente = richiesto_agente;
	}
	public Boolean getRichiesto_esito() {
		return ApplicationUtil.isTrue(richiesto_esito);
	}
	public void setRichiesto_esito(Boolean richiesto_esito) {
		this.richiesto_esito = richiesto_esito;
	}
	public BigDecimal getCosto_base() {
		return costo_base;
	}
	public BigDecimal getCosto_fisso() {
		return costo_fisso;
	}
	public Date getData_creazione() {
		return data_creazione;
	}
	public String getDenominazione() {
		StringBuilder sb = new StringBuilder();
		if(categoria != null) {
			sb.append(categoria.getNome());
			sb.append(": ");
		} 
		sb.append(this.getNomeTipo());
		return ApplicationUtil.trimToEmpty(sb.toString());
	}
	public String getNomeTipo() {
		StringBuilder sb = new StringBuilder();
		sb.append(this.getNome());
		if(ApplicationUtil.isNotBlank(codice)) {
			sb.append(" (" +codice + ")");
		}
		String tipo = getDescrizioneTipo();
		if(!ApplicationUtil.isBlank(tipo)) {
			sb.append(" - " + tipo);
		}
		return ApplicationUtil.trimToEmpty(sb.toString());
	}
	public String getDescrizione() {
		return ApplicationUtil.trimToEmpty(descrizione);
	}
	public List<Gettone> getGettoni() {
		return gettoni;
	}
	public Long getId() {
		return id;
	}
	public String getNome() {
		return nome;
	}
	public List<ProdottiInVendita> getProdotti_in_vendita() {
		return prodotti_in_vendita;
	}
	public Boolean getRichiesto_seriale() {
		return richiesto_seriale;
	}
	public Long getCategoriaId() {
		return this.categoria != null ? this.categoria.getId() : null;
	}
	public String getSito() {
		return sito;
	}
	public List<SondaggioContratto> getSondaggi() {
		return sondaggi;
	}
	public List<StoriaContratto> getStoria_contratti() {
		return storia_contratti;
	}
	public BigDecimal getTasse() {
		return tasse;
	}
	public List<TemplateProdotto> getTemplate() {
		return template;
	}
	public String getValidatore_seriale() {
		return validatore_seriale;
	}
	public List<VocalOrder> getVocalOrders() {
		return vocalOrders;
	}
	public void setAttivo(Boolean attivo) {
		this.attivo = attivo;
	}
	public void setCategoria(Categoria categoria) {
		this.categoria = categoria;
	}
	public void setCodice(String codice) {
		this.codice = ApplicationUtil.normalizeToNull(codice);
	}
	public void setContest(List<Contest> contest) {
		this.contest = contest;
	}
	public void setContratti(List<Contratto> contratti) {
		this.contratti = contratti;
	}
	public void setCosto_base(BigDecimal costo_base) {
		this.costo_base = costo_base;
	}
	public void setCosto_fisso(BigDecimal costo_fisso) {
		this.costo_fisso = costo_fisso;
	}
	public void setData_creazione(Date data_creazione) {
		this.data_creazione = data_creazione;
	}
	public void setDescrizione(String descrizione) {
		this.descrizione = ApplicationUtil.trimToNull(descrizione);
	}
	public void setGettoni(List<Gettone> gettoni) {
		this.gettoni = gettoni;
	}
	public void setId(Long id) {
		this.id = id;
	}
	public boolean isLogo() {
		return content != null && content.length > 0;
	}
	@Lob
	private byte[] content;
	public Allegato getAllegato() {
		return isLogo() ? new Allegato("", content) : null;
	}
	public byte[] getContent() {
		return content;
	}
	public void setContent(byte[] content) {
		this.content = content;
	}
	public void setNome(String nome) {
		this.nome = ApplicationUtil.normalizeToNull(nome);
	}
	public void setProdotti_in_vendita(List<ProdottiInVendita> prodotti_in_vendita) {
		this.prodotti_in_vendita = prodotti_in_vendita;
	}
	public void setRichiesto_seriale(Boolean richiesto_seriale) {
		this.richiesto_seriale = richiesto_seriale;
	}
	public void setSito(String sito) {
		this.sito = sito;
	}
	public void setSondaggi(List<SondaggioContratto> sondaggi) {
		this.sondaggi = sondaggi;
	}
	public void setStoria_contratti(List<StoriaContratto> storia_contratti) {
		this.storia_contratti = storia_contratti;
	}
	public void setTasse(BigDecimal tasse) {
		this.tasse = tasse;
	}
	public void setTemplate(List<TemplateProdotto> template) {
		this.template = template;
	}
	public void setValidatore_seriale(String validatore_seriale) {
		this.validatore_seriale = ApplicationUtil.normalizeToNull(validatore_seriale);
	}
	public void setVocalOrders(List<VocalOrder> vocalOrders) {
		this.vocalOrders = vocalOrders;
	}
	public String getCategoriaDenominazione() {
		return  getCategoria() != null ? getCategoria().getDenominazione() : "";
	}
	
	public Integer getRichiesto_data() {
		if(richiesto_data == null) {
			richiesto_data = 3;
		}
		return richiesto_data;
	}
	public void setRichiesto_data(Integer richiesto_data) {
		this.richiesto_data = richiesto_data;
	}
	
	public void validate(FacesContext context, UIComponent component, Object input)  {
		String var = (String) input;
		if(ApplicationUtil.isNotBlank(validatore_seriale)) {
			Pattern p = Pattern.compile(validatore_seriale);
			Matcher m = p.matcher(var);
			boolean b = m.matches();	  		 
			if(!b){
				throw new ValidatorException(Message.addValidatorMessage("formato.errato" , validatore_seriale));
			}
		}
	} 
}