package it.callite.model;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.Lob;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import javax.persistence.Transient;

import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.model.generic.SelezionatoAbstract;
import it.callite.utility.Allegato;
import it.callite.utility.AllegatoUtils;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.thread.Posta;



@SuppressWarnings("serial")
@Entity
//@Audited 
@Table(name = ConstantsGlobal.tp + "rmail", indexes = { @Index(name= "rmail", columnList = "data_creazione, data_invio, codice"), @Index(name = "id", columnList = "userfield") })
public class Rmail extends SelezionatoAbstract implements Serializable, Posta, Comparable<Rmail> {
	
	private String userfield;
	public String getUserfield() {
		return userfield;
	}
	public void setUserfield(String userfield) {
		this.userfield = userfield;
	}
	public void impostaUserfield() {
		if(contatto != null && userfield == null) {
			userfield = contatto.getUserfield();
		}
	}
	@Column(columnDefinition = "text")
	private String cc;
	@Column(columnDefinition = "text")
	private String sender;

	public String getCc() {
		return cc;
	}
	public void setCc(String cc) {
		this.cc = cc;
	}
	public String getSender() {
		return sender;
	}
	public void setSender(String sender) {
		this.sender = sender;
	}

	@Override
	public String toString() {
		return ApplicationUtil.toString(id, getDenominazione());
	}

	public String getDenominazione() {
		return ApplicationUtil.trimToEmpty(getContatto().getDenominazione());
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Rmail other = (Rmail) obj;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		return true;
	}

	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY) 
	private Long id;


	@ManyToOne
	private Indirizzo destinatario;

	private String ricevente;


	public String getRicevente() {
		return ricevente;
	}
	public void setRicevente(String ricevente) {
		this.ricevente = ricevente;
	}
	public Indirizzo getDestinatario() {
		return destinatario;
	}

	public void setDestinatario(Indirizzo destinatario) {
		this.destinatario = destinatario;
	}

	@Transient
	private Date dataSchedulata;

	public Date getDataSchedulata() {
		return ApplicationUtil.aggiustaDataT(dataSchedulata, true);
	}

	public void setDataSchedulata(Date dataSchedulata) {
		this.dataSchedulata = ApplicationUtil.aggiustaDataT(dataSchedulata, true);
	}

	@Column(columnDefinition = "text")
	private String testo;

	@ManyToOne
	private RmailTemplate rmailTemplate;

	@ManyToOne
	private Contatto contatto;

	private Date data_invio;
	@Column(columnDefinition="datetime NOT NULL COMMENT 'data'")
	private Date data_creazione = ApplicationUtil.date();

	public Date getData_creazione() {
		return data_creazione;
	}

	public void setData_creazione(Date data_creazione) {
		this.data_creazione = data_creazione;
	}

	@Column(columnDefinition = "text")
	private String stato;
	private Integer codice;

	public Integer getCodice() {
		return codice;
	}

	public void setCodice(Integer codice) {
		this.codice = codice;
	}

	public void addMotivo(Integer codice, String ragione) {
		setCodice(codice);
		String s = "";
		if(ApplicationUtil.equals(codice, ConstantsGlobal.IN_INVIO)) {
			s = ConstantsLanguage.TESTO_IN_INVIO.stampa();
		} else if(ApplicationUtil.equals(codice, ConstantsGlobal.INVIATA)) {
			s = ConstantsLanguage.TESTO_INVIATA.stampa();
		} else if(ApplicationUtil.equals(codice, ConstantsGlobal.NON_INVIATA)) {
			s = ConstantsLanguage.TESTO_NON_INVIATA.stampa();
		} else if(ApplicationUtil.equals(codice, ConstantsGlobal.SCHEDULATA)) {
			s = ConstantsLanguage.TESTO_SCHEDULATA.stampa();
		} else if(ApplicationUtil.equals(codice, ConstantsGlobal.PAUSE)) {
			s = ConstantsLanguage.TESTO_PAUSE.stampa();
		} else if(ApplicationUtil.equals(codice, ConstantsGlobal.ANNULLATA)) {
			s = ConstantsLanguage.TESTO_ANNULLATA.stampa();
		} else if(ApplicationUtil.equals(codice, ConstantsGlobal.APERTO)) {
			s = ConstantsLanguage.TESTO_APERTO.stampa();
		} else {
			s = ConstantsLanguage.TESTO_IN_INVIO.stampa();
		}
		if(ApplicationUtil.isNotBlank(ragione)) {
			setStato(s + ": " + ragione);
		} else {
			setStato(s);
		}
	}

	@Transient
	private HashMap<Allegato, String> allegati;

	@ManyToOne
	private Utente utente;

	public Utente getUtente() {
		return utente;
	}

	public void setUtente(Utente utente) {
		this.utente = utente;
	}

	public Long getId() {
		return id;
	}

	public void setId(Long id) {
		this.id = id;
	}


	public String getTesto() {
		return testo;
	}

	public void setTesto(String testo) {
		this.testo = testo;
	}

	public RmailTemplate getRmailTemplate() {
		return rmailTemplate;
	}

	public void setRmailTemplate(RmailTemplate rmailTemplate) {
		this.rmailTemplate = rmailTemplate;
	}

	public Contatto getContatto() {
		return contatto;
	}

	public void setContatto(Contatto contatto) {
		this.contatto = contatto;
	}

	public Date getData_invio() {
		return data_invio;
	}

	public void setData_invio(Date dataInvio) {
		data_invio = dataInvio;
	}

	public String getStato() {
		return stato;
	}

	public void setStato(String stato) {
		this.stato = stato;
	}

	@Transient
	public HashMap<Allegato, String> getAllegati() {
		return allegati;
	}

	public void setAllegati(HashMap<Allegato, String> allegati) {
		this.allegati = allegati;
	}	

	@Transient
	public String getNomiAllegati() {
		List<String> nomi = new ArrayList<String>();
		if(rmailTemplate != null && ApplicationUtil.isNotBlank(rmailTemplate.getDocumentiNome())) {
			if(!nomi.contains(rmailTemplate.getDocumentiNome())) {
				nomi.add(rmailTemplate.getDocumentiNome());
			}
		}
		if(allegati != null) {
			Set<Allegato> ss = allegati.keySet();
			if (ApplicationUtil.notEmpty(ss)) {
				Iterator<Allegato> iter = ss.iterator();
				while (iter.hasNext()) {
					Allegato doc = (Allegato) iter.next();
					if(!nomi.contains(allegati.get(doc))) {
						nomi.add(allegati.get(doc));
					}
				}
			}
		}
		return ApplicationUtil.detoken(nomi, ConstantsGlobal.colonSeparator);
	}

	public int compareTo(Rmail o) {
		if(id != null && o.id != null) {
			return id.compareTo(o.id);
		} else if(id != null && o.id == null) {
			return -1;
		} else if(id == null && o.id != null) {
			return 1;
		} else {
			return 0;
		}
	}

	@Column(columnDefinition = "text")
	private String destinatarioTemplate;

	public String getDestinatarioTemplate() {
		return destinatarioTemplate;
	}

	public void setDestinatarioTemplate(String destinatarioTemplate) {
		this.destinatarioTemplate = destinatarioTemplate;
	}

	public String getOggettoTemplate() {
		return oggettoTemplate;
	}

	public void setOggettoTemplate(String oggettoTemplate) {
		this.oggettoTemplate = oggettoTemplate;
	}

	public String getTestoTemplate() {
		return testoTemplate;
	}

	public void setTestoTemplate(String testoTemplate) {
		this.testoTemplate = testoTemplate;
	}

	@Column(columnDefinition = "text")
	private String oggettoTemplate;

	@Column(columnDefinition = "text")
	private String testoTemplate;




	private Integer pagine = 0;

	public Integer getPagine() {
		if(pagine == null) {
			pagine = 0;
		}
		return pagine;
	}


	public void setPagine(Integer pagine) {
		this.pagine = pagine;
	}


	@Lob
	private byte [] allegatiTemplates;

	public byte[] getAllegatiTemplates() {
		return allegatiTemplates;
	}

	public void setAllegatiTemplates(byte[] allegatiTemplates) {
		this.allegatiTemplates = allegatiTemplates;
	}

	public void setAllegatiTemplateList(List<Allegato> allegati) {
		this.allegatiTemplates = ApplicationUtil.setAllegatiTemplateList(allegati);
	}

	public List<AllegatoUtils> getAllegatiTemplateList() {
		return ApplicationUtil.getAllegatiTemplateList(allegatiTemplates);
	}

	public List<Allegato> getAttachments() {
		return ApplicationUtil.getAllegatiReinvioList(allegatiTemplates);
	}


	@Transient
	private Boolean residenza = Boolean.TRUE;

	public Boolean getResidenza() {
		if(residenza==null)
			return false;
		return residenza;
	}

	public void setResidenza(Boolean residenza) {
		this.residenza = residenza;
	}


	public boolean valido() {
		if(rmailTemplate.isSms() || rmailTemplate.isFax()) {
			return ApplicationUtil.isNotBlank(this.getRicevente());
		}
		if(rmailTemplate.isLettera() || rmailTemplate.isTelegramma()) {
			return ApplicationUtil.isNotBlank(this.getDestinatario().getIndirizzo_completo());
		}
		return false;
	}

	public boolean inviabile() {
		return rmailTemplate != null && this.rmailTemplate.getSmtp() != null && this.rmailTemplate.getInvio() != null;
	}
	
	public String getDestinatarioS() {
		if(rmailTemplate.isSms() || rmailTemplate.isFax()) {
			return this.getRicevente();
		}
		if(rmailTemplate.isLettera() || rmailTemplate.isTelegramma()) {
			return this.getDestinatario().getIndirizzo_completo();
		}
		return null;
	}

	@ManyToOne
	private Schedulatore scheduler;
	public Schedulatore getScheduler() {
		return scheduler;
	}
	public void setScheduler(Schedulatore scheduler) {
		this.scheduler = scheduler;
	}

	public String getUtente_denominazione() {
		return getUtente() != null ? getUtente().getDenominazione() : "";
	}


	public Boolean getHtml() {
		if(rmailTemplate != null) {
			if(rmailTemplate.getInvio() != null) {
				return rmailTemplate.getInvio().getHtml();

			}
		}
		return false;
	}
	
	public Boolean getLink() {
		if(rmailTemplate != null) {
			return rmailTemplate.getLink();

		}
		return false;
	}

	public String getTemplate() {
		return null;
	}

	public String getTo() {
		return destinatarioTemplate;
	}

	public String getSubject() {
		return oggettoTemplate;
	}

	public String getText() {
		return testoTemplate;
	}
	
	public void imposta() {
		if(rmailTemplate != null && contatto != null) {
			if(rmailTemplate.isFax()) {
				setRicevente(contatto.getAnagrafica().getTelefono_fax());
				setDestinatario(null);
			}
			if(rmailTemplate.isSms()) {
				setRicevente(contatto.getAnagrafica().getTelefono_mobile());
				setDestinatario(null);
			}
			if(rmailTemplate.isLettera() || rmailTemplate.isTelegramma()) {
				setRicevente(null);
				if(rmailTemplate.getResidenza()) {
					setDestinatario(contatto.getAnagrafica().getIndirizzo_legale());
				} else {
					setDestinatario(contatto.getAnagrafica().getIndirizzo_operativo());
				}
			}
		}
	}
	public void imposta(boolean open, RmailTemplate template, Contatto contatto) {
		if(template.isFax()) {
			setRicevente(contatto.getAnagrafica().getTelefono_fax());
			setDestinatario(null);
		}
		if(template.isSms()) {
			setRicevente(contatto.getAnagrafica().getTelefono_mobile());
			setDestinatario(null);
		}
		if(template.isLettera() || template.isTelegramma()) {
			if(open) {
				setResidenza(template.getResidenza());
			}
			setRicevente(null);
			if(getResidenza()) {
				setDestinatario(contatto.getAnagrafica().getIndirizzo_legale());
			} else {
				setDestinatario(contatto.getAnagrafica().getIndirizzo_operativo());
			}
		}
	}
}