package it.callite.model;

import java.io.Serializable;
import java.util.Date;

import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.ManyToOne;
import javax.persistence.Table;

import it.callite.constants.ConstantsGlobal;
import it.callite.model.generic.SelezionatoAbstract;
import it.callite.utility.ApplicationUtil;



@SuppressWarnings("serial")
@Entity
//@Audited 
@Table(name = ConstantsGlobal.tp + "sondaggio", indexes = { @Index(name= "sondaggio", columnList = "data_creazione, voto"), @Index(name = "id", columnList = "userfield") })
public class Sondaggio  extends SelezionatoAbstract implements Serializable, Comparable<Sondaggio> {


	private String userfield;
	public String getUserfield() {
		return userfield;
	}
	public void setUserfield(String userfield) {
		this.userfield = userfield;
	}
	public void impostaUserfield() {
		if(contatto != null && userfield == null) {
			userfield = contatto.getUserfield();
		}
	}
	private Integer voto;

	@ManyToOne
	private Contatto contatto;

	private Date data_creazione = ApplicationUtil.date();

	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY) 
	private Long id;

	@ManyToOne
	private Utente utente;

	public int compareTo(Sondaggio o) {
		if(id != null && o.id != null) {
			return id.compareTo(o.id);
		} else if(id != null && o.id == null) {
			return -1;
		} else if(id == null && o.id != null) {
			return 1;
		} else {
			return 0;
		}
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Sondaggio other = (Sondaggio) obj;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		return true;
	}


	public Contatto getContatto() {
		return contatto;
	}

	public Date getData_creazione() {
		return data_creazione;
	}



	public String getDenominazione() {
		return ApplicationUtil.trimToEmpty(contatto.getDenominazione());
	}


	public Long getId() {
		return id;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		return result;
	}



	public void setContatto(Contatto contatto) {
		this.contatto = contatto;
	}

	public void setData_creazione(Date data_creazione) {
		this.data_creazione = data_creazione;
	}

	public void setId(Long id) {
		this.id = id;
	}


	@Override
	public String toString() {
		return ApplicationUtil.toString(id, getDenominazione());
	}

	public Integer getVoto() {
		return voto;
	}

	public void setVoto(Integer voto) {
		this.voto = voto;
	}

	public Utente getUtente() {
		return utente;
	}

	public void setUtente(Utente utente) {
		this.utente = utente;
	}
	public String getUtente_denominazione() {
		return getUtente() != null ? getUtente().getDenominazione() : "";
	}

}