package it.callite.model;
import java.io.Serializable;
import java.util.Date;
import java.util.List;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import org.hibernate.annotations.OnDelete;
import org.hibernate.annotations.OnDeleteAction;
import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.model.generic.SelezionatoAbstract;
import it.callite.utility.ApplicationUtil;
@SuppressWarnings("serial")
@Entity
//@Audited 
@Table(name = ConstantsGlobal.tp + "sondaggio_anagrafica", indexes = { @Index(name= "sondaggio_anagrafica", columnList = "data_creazione, nome, solo_operatore, solo_supervisore, solo_backoffice, solo_archiviati") })
public class SondaggioAnagrafica extends SelezionatoAbstract implements Serializable, Comparable<SondaggioAnagrafica> {
	
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY) 
	private Long id;
	private String nome;
	@Column(columnDefinition = "text")
	private String descrizione;
	private Date data_creazione = ApplicationUtil.date();
	private Boolean attivo= Boolean.TRUE;
	@OneToMany(mappedBy = "sondaggio")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<DomandaAnagrafica> domande;
	private Boolean solo_archiviati = Boolean.FALSE;	
	private Boolean solo_backoffice = Boolean.TRUE;
	private Boolean solo_operatore = Boolean.TRUE;
	private Boolean solo_supervisore = Boolean.TRUE;
	
	@ManyToMany(fetch = FetchType.LAZY)
	@JoinTable(name = ConstantsGlobal.tp + "esito_sondaggio", joinColumns=@JoinColumn(name="sondaggio_id", nullable=false, updatable=true, insertable=true), inverseJoinColumns=@JoinColumn(name="esito_id", nullable=false, updatable=true, insertable=true)    )
	private List<EsitoArchiviazione> esiti;
	
	public SondaggioAnagrafica getCopia() {
		SondaggioAnagrafica pr = new SondaggioAnagrafica();
		pr.setNome(this.getNome() + " " +  ConstantsLanguage.COPY.stampa());
		pr.setDescrizione(this.getDescrizione());
		pr.setAttivo(this.getAttivo());
		pr.setSolo_archiviati(this.getSolo_archiviati());
		pr.setSolo_backoffice(this.getSolo_backoffice());
		pr.setSolo_operatore(this.getSolo_operatore());
		pr.setSolo_supervisore(this.getSolo_supervisore());
		return pr;
	}
	
	public Boolean getSolo_archiviati() {
		return solo_archiviati;
	}
	public void setSolo_archiviati(Boolean solo_archiviati) {
		this.solo_archiviati = solo_archiviati;
	}
	public Boolean getSolo_backoffice() {
		return solo_backoffice;
	}
	public void setSolo_backoffice(Boolean solo_backoffice) {
		this.solo_backoffice = solo_backoffice;
	}
	public Boolean getSolo_operatore() {
		return solo_operatore;
	}
	public void setSolo_operatore(Boolean solo_operatore) {
		this.solo_operatore = solo_operatore;
	}
	public Boolean getSolo_supervisore() {
		return solo_supervisore;
	}
	public void setSolo_supervisore(Boolean solo_supervisore) {
		this.solo_supervisore = solo_supervisore;
	}
	public int compareTo(SondaggioAnagrafica o) {
		if(id != null && o.id != null) {
			return id.compareTo(o.id);
		} else if(id != null && o.id == null) {
			return -1;
		} else if(id == null && o.id != null) {
			return 1;
		} else {
			return 0;
		}
	}
	
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		SondaggioAnagrafica other = (SondaggioAnagrafica) obj;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		return true;
	}
	
	
	public Boolean getAttivo() {
		return attivo;
	}
	public void setAttivo(Boolean attivo) {
		this.attivo = attivo;
	}
	public Date getData_creazione() {
		return data_creazione;
	}
	public String getDenominazione() {
		return nome;
	}
	public String getDescrizione() {
		return ApplicationUtil.trimToEmpty(descrizione);
	}
	public List<DomandaAnagrafica> getDomande() {
		return domande;
	}
	public List<EsitoArchiviazione> getEsiti() {
		return esiti;
	}
	public Long getId() {
		return id;
	}
	public String getNome() {
		return nome;
	}
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		return result;
	}
	public void setData_creazione(Date data_creazione) {
		this.data_creazione = data_creazione;
	}
	public void setDescrizione(String descrizione) {
		this.descrizione = ApplicationUtil.trimToNull(descrizione);
	}
	public void setDomande(List<DomandaAnagrafica> domande) {
		this.domande = domande;
	}
	public void setEsiti(List<EsitoArchiviazione> esiti) {
		this.esiti = esiti;
	}
	public void setId(Long id) {
		this.id = id;
	}
	public void setNome(String nome) {
		this.nome = ApplicationUtil.normalizeToNull(nome);
	}
	@Override
	public String toString() {
		return ApplicationUtil.toString(id, getDenominazione());
	}
	
}