package it.callite.model;

import java.io.Serializable;
import java.sql.Time;
import java.util.Date;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.Table;

import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.model.generic.SelezionatoAbstract;
import it.callite.utility.ApplicationUtil;





@SuppressWarnings("serial")
@Entity
//@Audited 
@Table(name = ConstantsGlobal.tp + "storia_appuntamento", indexes = { @Index(name= "storia_appuntamento", columnList = "data, data_inizio, data_fine, data_conferma, data_esito, data_stato, stato") })
public class StoriaAppuntamento extends SelezionatoAbstract implements Serializable, Comparable<StoriaAppuntamento>  {

	@ManyToOne
	private Agente agente;

	@ManyToOne(fetch = FetchType.LAZY)
	private Appuntamento appuntamento;

	@Column(columnDefinition="datetime NOT NULL COMMENT 'data'")
	private Date data = ApplicationUtil.date();
	private Date data_conferma;
	private Date data_esito;

	private Date data_fine;

	private Date data_inizio ;

	private Date data_stato;

	@ManyToOne
	private EsitoAppuntamento esito;

	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY) 
	private Long id;

	private Boolean localizzazione;
	@Column(columnDefinition = "text")
	private String note;


	@ManyToOne
	@JoinColumn(name = "operatoreId")
	private Utente operatore;


	private Integer stato;

	@ManyToOne
	private Utente utente;

	public Agente getAgente() {
		return agente;
	}

	public Appuntamento getAppuntamento() {
		return appuntamento;
	}

	public String getColore() {
		if(ApplicationUtil.equals(stato, ConstantsGlobal.in_attesa)) {
			return ConstantsGlobal.color_in_attesa;
		} else if(ApplicationUtil.equals(stato, ConstantsGlobal.in_pagamento)) {
			return ConstantsGlobal.color_in_pagamento;
		} else if(ApplicationUtil.equals(stato, ConstantsGlobal.pagato)) {
			return ConstantsGlobal.color_pagato;
		} else if(ApplicationUtil.equals(stato, ConstantsGlobal.in_storno)) {
			return ConstantsGlobal.color_in_storno;
		} else if(ApplicationUtil.equals(stato, ConstantsGlobal.stornato)) {
			return ConstantsGlobal.color_stornato;
		} else {
			return ConstantsGlobal.color_in_attesa;
		} 
	}

	public Date getData() {
		return data;
	}

	public Date getData_conferma() {
		return data_conferma;
	}

	public Date getData_esito() {
		return data_esito;
	}


	public Date getData_fine() {
		return data_fine;
	}

	public Date getData_inizio() {
		return data_inizio;
	}

	public Date getData_stato() {
		return data_stato;
	}

	public EsitoAppuntamento getEsito() {
		return esito;
	}

	public Long getId() {
		return id;
	}

	public Boolean getLocalizzazione() {
		if(localizzazione==null)
			return false;
		return localizzazione;
	}

	public String getLocalizzazioneString() {
		if(getLocalizzazione()) {
			return ConstantsLanguage.INDIRIZZO_OPERATIVO.stampa();
		} else {
			return ConstantsLanguage.INDIRIZZO_LEGALE.stampa();
		}

	}

	public String getNote() {
		return note;
	}

	public String getNota() {
		return note;
	}


	public Utente getOperatore() {
		return operatore;
	}

	public Integer getStato() {
		return stato;
	}

	public String getStatoTesto() {
		if(ApplicationUtil.equals(stato, ConstantsGlobal.in_attesa)) {
			return ConstantsLanguage.STATO_IN_ATTESA.stampa();
		} else if(ApplicationUtil.equals(stato, ConstantsGlobal.in_pagamento)) {
			return ConstantsLanguage.STATO_IN_PAGAMENTO.stampa();
		} else if(ApplicationUtil.equals(stato, ConstantsGlobal.pagato)) {
			return ConstantsLanguage.STATO_PAGATO.stampa();
		} else if(ApplicationUtil.equals(stato, ConstantsGlobal.in_storno)) {
			return ConstantsLanguage.STATO_IN_STORNO.stampa();
		} else if(ApplicationUtil.equals(stato, ConstantsGlobal.stornato)) {
			return ConstantsLanguage.STATO_STORNATO.stampa();
		} else {
			return ConstantsLanguage.STATO_IN_ATTESA.stampa();
		} 
	}

	public Utente getUtente() {
		return utente;
	}

	public void setAgente(Agente agente) {
		this.agente = agente;
	}

	public void setAppuntamento(Appuntamento appuntamento) {
		this.appuntamento = appuntamento;
	}

	public void setData(Date data) {
		this.data = data;
	}

	public void setData_conferma(Date data_conferma) {
		this.data_conferma = data_conferma;
	}

	public void setData_esito(Date data_esito) {
		this.data_esito = data_esito;
	}

	public void setData_fine(Date data_fine) {
		this.data_fine = data_fine;
	}

	public void setData_inizio(Date data_inizio) {
		this.data_inizio = data_inizio;
	}

	public void setData_stato(Date data_stato) {
		this.data_stato = data_stato;
	}

	public void setEsito(EsitoAppuntamento esito) {
		this.esito = esito;
	}

	public void setId(Long id) {
		this.id = id;
	}

	public void setLocalizzazione(Boolean localizzazione) {
		this.localizzazione = localizzazione;
	}
	public void setNote(String note) {
		this.note = note;
	}

	public void setOperatore(Utente operatore) {
		this.operatore = operatore;
	}


	public void setStato(Integer stato) {
		this.stato = stato;
	}

	public void setUtente(Utente utente) {
		this.utente = utente;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((data == null) ? 0 : data.hashCode());
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		StoriaAppuntamento other = (StoriaAppuntamento) obj;
		if (data == null) {
			if (other.data != null)
				return false;
		} else if (!data.equals(other.data))
			return false;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		return true;
	}

	public int compareTo(StoriaAppuntamento o) {
		if(id != null && o.id != null) {
			return id.compareTo(o.id);
		} else if(id != null && o.id == null) {
			return -1;
		} else if(id == null && o.id != null) {
			return 1;
		} else {
			return 0;
		}
	}
	@Override
	public String toString() {
		return ApplicationUtil.toString(id, appuntamento.getDenominazione());
	}

	public String getAgente_denominazione() {
		return getAgente() != null ? getAgente().getDenominazione() : "";
	}
	public String getAgente_codice() {
		return getAgente() != null ? getAgente().getCodice() : "";
	}
	public String getUtente_denominazione() {
		return getUtente() != null ? getUtente().getDenominazione() : "";
	}

	public String getUtente_codice() {
		return getUtente() != null ? getUtente().getUsername() : "";
	}

	public String getOperatore_denominazione() {
		return getOperatore() != null ? getOperatore().getDenominazione() : "";
	}

	public String getOperatore_codice() {
		return getOperatore() != null ? getOperatore().getUsername() : "";
	}

	public String getEsito_appuntamento() {
		return  getEsitoEtichetta();
	}

	public String getEsitoEtichetta() {
		return getEsito() != null ? getEsito().getEtichetta() : "";
	}

	public String getEsitoEtichettaCodice() {
		StringBuilder sb = new StringBuilder();
		sb.append(ApplicationUtil.trimToEmpty(getEsitoEtichetta()));
		if(ApplicationUtil.isNotBlank(getEsitoCodice())) {
			sb.append(" (" + getEsitoCodice() + ")");
		}
		return sb.toString();
	}
	public String getCodice_appuntamento() {
		return getEsitoCodice();
	}
	public String getEsitoCodice() {
		return getEsito() != null ? getEsito().getCodice() : "";
	}

	public String getNotaShort() {
		return ApplicationUtil.tronca(note);
	}
	
	public Time getOra_inizio() {
		return ApplicationUtil.dateToTime(getData_inizio());
	}
	public Time getOra_fine() {
		return ApplicationUtil.dateToTime(getData_fine());
	}
	
	private Boolean bloccato = Boolean.FALSE;
	public String getBloccatoS() {
		return getBloccato() ? ConstantsLanguage.SI.stampa() : ConstantsLanguage.NO.stampa();
	}
	public Boolean getBloccato() {
		if(bloccato == null) {
			bloccato = Boolean.FALSE;
		}
		return bloccato;
	}
	public void setBloccato(Boolean bloccato) {
		this.bloccato = bloccato;
	}
}
