package it.callite.model;
import java.io.Serializable;
import java.util.Date;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.model.generic.SelezionatoAbstract;
import it.callite.utility.ApplicationUtil;
@SuppressWarnings("serial")
@Entity
//@Audited 
@Table(name = ConstantsGlobal.tp + "storia_contatto", indexes = { @Index(name= "storia_contatto", columnList = "data, archiviato, data_archiviazione, data_presentazione, invisibile, tipo"), @Index(name = "id", columnList = "userfield") })
public class StoriaContatto extends SelezionatoAbstract implements Serializable, Comparable<StoriaContatto>  {
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY) 
	private Long id;
	@Column(columnDefinition="datetime NOT NULL COMMENT 'data'")
	private Date data = ApplicationUtil.date();
	@ManyToOne
	private Agente agente;
	@ManyToOne
	private Utente proprietario;
	private Boolean archiviato;
	private Date data_archiviazione;
	private Date data_presentazione;
	private Integer sequenziale;
	private String userfield;
	public String getUserfield() {
		return userfield;
	}
	public void setUserfield(String userfield) {
		this.userfield = userfield;
	}
	public void impostaUserfield() {
		if(contatto != null && userfield == null) {
			userfield = contatto.getUserfield();
		}
	}
	public Integer getSequenziale() {
		return sequenziale;
	}
	public void setSequenziale(Integer sequenziale) {
		this.sequenziale = sequenziale;
	}
	public String getStato() {
		return getTipoTesto();
	} 
	public String getPositivo() {
		if(getArchiviazione()!=null && ApplicationUtil.isTrue(getArchiviazione().getPositivo()))
			return ConstantsLanguage.SI.stampa();
		else
			return ConstantsLanguage.NO.stampa();
	}
	public String getUtile() {
		if(getArchiviazione()!=null && ApplicationUtil.isTrue(getArchiviazione().getUtile()))
			return ConstantsLanguage.SI.stampa();
		else
			return ConstantsLanguage.NO.stampa();
	} 
	@ManyToOne
	@JoinColumn(name = "operatoreId")
	private Utente operatore;
	@ManyToOne
	private EsitoChiamata esito;
	@ManyToOne
	private EsitoArchiviazione archiviazione;
	@Column(columnDefinition = "text")
	private String nota;
	@ManyToOne(fetch = FetchType.LAZY)
	private Contatto contatto;
	private Integer tipo;
	public Integer getTipo() {
		return tipo;
	}
	public void setTipo(Integer tipo) {
		this.tipo = tipo;
	}
	private Integer secondi_pre = 0;
	private Integer secondi_conv_tot = 0;
	private Integer secondi_attesa = 0;
	private Integer secondi_conv_eff = 0;
	private Integer secondi_post = 0;
	private Integer secondi_tot = 0;
	private Integer secondi_rec = 0;
	private Boolean invisibile = false;
	private Date date_apertura;
	private Date date_chiamata;
	private Date date_esito;
	private Date date_fine;
	private Date date_risposta;
	private Date date_rec_inizio;
	private Date date_rec_fine;
	private Date date_attesa_inizio;
	private Date date_attesa_fine;
	public Integer getSecondi_conv_eff() {
		if(secondi_conv_eff == null)
			secondi_conv_eff = Integer.valueOf(0);
		return secondi_conv_eff;
	}
	public Integer getSecondi_conv_tot() {
		if(secondi_conv_tot == null)
			secondi_conv_tot = Integer.valueOf(0);
		return secondi_conv_tot;
	}
	public Integer getSecondi_post() {
		if(secondi_post == null)
			secondi_post = Integer.valueOf(0);
		return secondi_post;
	}
	public Integer getSecondi_pre() {
		if(secondi_pre == null)
			secondi_pre = Integer.valueOf(0);
		return secondi_pre;
	}
	public Integer getSecondi_tot() {
		if(secondi_tot == null)
			secondi_tot = Integer.valueOf(0);
		return secondi_tot;
	}
	public Integer getSecondi_rec() {
		if(secondi_rec == null)
			secondi_rec = Integer.valueOf(0);
		return secondi_rec;
	}
	public void setSecondi_rec(Integer secondi_rec) {
		this.secondi_rec = secondi_rec;
	}
	public Date getDate_rec_inizio() {
		return date_rec_inizio;
	}
	public void setDate_rec_inizio(Date date_rec_inizio) {
		this.date_rec_inizio = date_rec_inizio;
	}
	public Date getDate_rec_fine() {
		return date_rec_fine;
	}
	public void setDate_rec_fine(Date date_rec_fine) {
		this.date_rec_fine = date_rec_fine;
	}
	public EsitoArchiviazione getArchiviazione() {
		return archiviazione;
	}
	public Date getDate_attesa_inizio() {
		return date_attesa_inizio;
	}
	public void setDate_attesa_inizio(Date date_attesa_inizio) {
		this.date_attesa_inizio = date_attesa_inizio;
	}
	public Date getDate_attesa_fine() {
		return date_attesa_fine;
	}
	public void setDate_attesa_fine(Date date_attesa_fine) {
		this.date_attesa_fine = date_attesa_fine;
	}
	public Contatto getContatto() {
		return contatto;
	}
	public Date getData() {
		return data;
	}
	public EsitoChiamata getEsito() {
		return esito;
	}
	public Long getId() {
		return id;
	}
	public Boolean getInvisibile() {
		return invisibile;
	}
	public String getNota() {
		return nota;
	}
	public Utente getOperatore() {
		return operatore;
	}
	public String getNotaShort() {
		return ApplicationUtil.tronca(nota);
	}
	public Date getDate_apertura() {
		return date_apertura;
	}
	public Date getDate_chiamata() {
		return date_chiamata;
	}
	public Date getDate_esito() {
		return date_esito;
	}
	public Date getDate_fine() {
		return date_fine;
	}
	public Date getDate_risposta() {
		return date_risposta;
	}
	public void setArchiviazione(EsitoArchiviazione archiviazione) {
		this.archiviazione = archiviazione;
	}
	public void setContatto(Contatto contatto) {
		this.contatto = contatto;
	}
	public void setData(Date data) {
		this.data = data;
	}
	public void setEsito(EsitoChiamata esito) {
		this.esito = esito;
	}
	public void setId(Long id) {
		this.id = id;
	}
	public void setInvisibile(Boolean invisibile) {
		this.invisibile = invisibile;
	}
	public void setNota(String nota) {
		this.nota = nota;
	}
	public void setOperatore(Utente operatore) {
		this.operatore = operatore;
	}
	public void setSecondi_conv_eff(Integer secondiConvEff) {
		secondi_conv_eff = secondiConvEff;
	}
	public void setSecondi_conv_tot(Integer secondiConvTot) {
		secondi_conv_tot = secondiConvTot;
	}
	public void setSecondi_post(Integer secondiPost) {
		secondi_post = secondiPost;
	}
	public void setSecondi_pre(Integer secondiPre) {
		secondi_pre = secondiPre;
	}
	public void setSecondi_tot(Integer secondiTot) {
		secondi_tot = secondiTot;
	}
	public void setDate_apertura(Date date_apertura) {
		this.date_apertura = date_apertura;
	}
	public void setDate_chiamata(Date date_chiamata) {
		this.date_chiamata = date_chiamata;
	}
	public void setDate_esito(Date date_esito) {
		this.date_esito = date_esito;
	}
	public void setDate_fine(Date date_fine) {
		this.date_fine = date_fine;
	}
	public void setDate_risposta(Date date_risposta) {
		this.date_risposta = date_risposta;
	}
	public String getTipoTesto() {
		if(ApplicationUtil.equals(tipo, ConstantsGlobal.inserito)) {
			return ConstantsLanguage.INSERITO.stampa();
		} else if(ApplicationUtil.equals(tipo, ConstantsGlobal.inviato)) {
			return ConstantsLanguage.INVIATO.stampa();
		} else if(ApplicationUtil.equals(tipo, ConstantsGlobal.respinto)) {
			return ConstantsLanguage.RESPINTO.stampa();
		} else if(ApplicationUtil.equals(tipo, ConstantsGlobal.bloccato)) {
			return ConstantsLanguage.BLOCCATO.stampa();
		} else {
			return ConstantsLanguage.INVIATO.stampa();
		} 
	}
	public String getColore() {
		if(ApplicationUtil.equals(tipo, ConstantsGlobal.inserito)) {
			return ConstantsGlobal.color_inserito;
		} else if(ApplicationUtil.equals(tipo, ConstantsGlobal.inviato)) {
			return ConstantsGlobal.color_inviato;
		} else if(ApplicationUtil.equals(tipo, ConstantsGlobal.respinto)) {
			return ConstantsGlobal.color_respinto;
		} else if(ApplicationUtil.equals(tipo, ConstantsGlobal.bloccato)) {
			return ConstantsGlobal.color_bloccato;
		} else {
			return ConstantsGlobal.color_inviato;
		} 
	}
	public Boolean getVisibile() {
		return ApplicationUtil.isFalse(getInvisibile());
	}
	public Boolean getPannello() {
		return date_apertura != null;
	}
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((data == null) ? 0 : data.hashCode());
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		return result;
	}
	public String getEsito_generale() {
		if(archiviazione != null) {
			return getEsito_archiviazione();
		} else {
			return getEsito_chiamata();
		}
	}
	public String getCodice_chiamata() {
		return getEsito().getCodice();
	}
	public String getCodice_archiviazione() {
		return getArchiviazione() != null ? getArchiviazione().getCodice() : "";
	}
	public String getCodice_generale() {
		if(archiviazione != null) {
			return getCodice_archiviazione();
		} else {
			return getCodice_chiamata();
		}
	}
	public String getEsitoGeneraleColore() {
		if(archiviazione != null) {
			return getArchiviazione().getColore();
		} else {
			return getEsito().getColore();
		}
	}
	public String getEsitoGeneraleCodice() {
		if(archiviazione != null) {
			return getEsito_archiviazione_etichettaCodice();
		} else {
			return getEsito_chiamata_etichettaCodice();
		}
	}
	public String getEsito_archiviazione_etichettaCodice() {
		StringBuilder sb = new StringBuilder();
		sb.append(ApplicationUtil.trimToEmpty(getEsito_archiviazione()));
		if(ApplicationUtil.isNotBlank(getCodice_archiviazione())) {
			sb.append(" (" + getCodice_archiviazione() + ")");
		}
		return sb.toString();
	}
	public String getEsito_chiamata_etichettaCodice() {
		StringBuilder sb = new StringBuilder();
		sb.append(ApplicationUtil.trimToEmpty(getEsito_chiamata()));
		if(ApplicationUtil.isNotBlank(getCodice_chiamata())) {
			sb.append(" (" + getCodice_chiamata() + ")");
		}
		return sb.toString();
	}
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		StoriaContatto other = (StoriaContatto) obj;
		if (data == null) {
			if (other.data != null)
				return false;
		} else if (!data.equals(other.data))
			return false;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		return true;
	}
	public int compareTo(StoriaContatto o) {
		if(id != null && o.id != null) {
			return id.compareTo(o.id);
		} else if(id != null && o.id == null) {
			return -1;
		} else if(id == null && o.id != null) {
			return 1;
		} else {
			return 0;
		}
	}
	public String getEsito_archiviazione() {
		return getArchiviazione() != null ? getArchiviazione().getEtichetta() : "";
	}
	public String getEsito_chiamata() {
		return getEsito().getEtichetta();
	}
	public String getChiuso() {
		if(ApplicationUtil.isTrue(getArchiviato()))
			return ConstantsLanguage.SI.stampa();
		else
			return ConstantsLanguage.NO.stampa();
	}
	public String getOperatore_denominazione() {
		return getOperatore() != null ? getOperatore().getDenominazione() : "";
	}
	public String getOperatore_codice() {
		return getOperatore() != null ? getOperatore().getUsername() : "";
	}
	public String getProprietario_denominazione() {
		return getProprietario() != null ? getProprietario().getDenominazione() : "";
	}
	public String getProprietario_codice() {
		return getProprietario() != null ? getProprietario().getUsername() : "";
	}
	public String getAgente_denominazione() {
		return getAgente() != null ? getAgente().getDenominazione() : "";
	}
	public String getAgente_codice() {
		return getAgente() != null ? getAgente().getCodice() : "";
	}
	public Agente getAgente() {
		return agente;
	}
	public void setAgente(Agente agente) {
		this.agente = agente;
	}
	public Utente getProprietario() {
		return proprietario;
	}
	public void setProprietario(Utente proprietario) {
		this.proprietario = proprietario;
	}
	public Boolean getArchiviato() {
		return archiviato;
	}
	public void setArchiviato(Boolean archiviato) {
		this.archiviato = archiviato;
	}
	public Date getData_archiviazione() {
		return data_archiviazione;
	}
	public void setData_archiviazione(Date data_archiviazione) {
		this.data_archiviazione = data_archiviazione;
	}
	public Date getData_presentazione() {
		return data_presentazione;
	}
	public void setData_presentazione(Date data_presentazione) {
		this.data_presentazione = data_presentazione;
	}
	@Override
	public String toString() {
		return ApplicationUtil.toString(id, contatto.getDenominazione());
	}
	public String getSecondi_conv_effH() {
		return ApplicationUtil.format(getSecondi_conv_eff());
	}
	public String getSecondi_conv_totH() {
		return ApplicationUtil.format(getSecondi_conv_tot());
	}
	public String getSecondi_postH() {
		return ApplicationUtil.format(getSecondi_post());
	}
	public String getSecondi_preH() {
		return ApplicationUtil.format(getSecondi_pre());
	}
	public String getSecondi_totH() {
		return ApplicationUtil.format(getSecondi_tot());
	}
	public String getSecondi_recH() {
		return ApplicationUtil.format(getSecondi_rec());
	}
	@ManyToOne
	private Campagna campagna;
	@ManyToOne
	private Lista lista;
	public Campagna getCampagna() {
		return campagna;
	}
	public void setCampagna(Campagna campagna) {
		this.campagna = campagna;
	}
	public Lista getLista() {
		return lista;
	}
	public String getLista_denominazione() {
		return getLista() != null ? getLista().getDenominazione() : "";
	} 
	public String getCampagna_denominazione() {
		return getCampagna() != null ? getCampagna().getDenominazione() : "";
	} 
	
	public String getLista_codice() {
		return getLista() != null ? getLista().getCodice() : "";
	} 
	
	public String getCampagna_codice() {
		return getCampagna() != null ? getCampagna().getCodice() : "";
	} 
	public String getBloccatoS() {
		return getBloccato() ? ConstantsLanguage.SI.stampa() : ConstantsLanguage.NO.stampa();
	}
	
	public void setLista(Lista lista) {
		this.lista = lista;
	}
	public Integer getSecondi_attesa() {
		if(secondi_attesa == null)
			secondi_attesa = Integer.valueOf(0);
		return secondi_attesa;
	}
	public void setSecondi_attesa(Integer secondiAttesa) {
		secondi_attesa = secondiAttesa;
	}
	public String getSecondi_attesaH() {
		return ApplicationUtil.format(getSecondi_attesa());
	}
	
	private Boolean bloccato = Boolean.FALSE;
	public Boolean getBloccato() {
		if(bloccato == null) {
			bloccato = Boolean.FALSE;
		}
		return bloccato;
	}
	public void setBloccato(Boolean bloccato) {
		this.bloccato = bloccato;
	}
	
}