package it.callite.model;
import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Date;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.model.generic.SelezionatoAbstract;
import it.callite.utility.ApplicationUtil;
@SuppressWarnings("serial")
@Entity
//@Audited 
@Table(name = ConstantsGlobal.tp + "storia_contratto", indexes = { @Index(name= "storia_contratto", columnList = "data, data_stipula, data_contabile, data_attivazione, data_caricamento, data_esito, data_stato, seriale, stato, quantita, costo_scontato") })
public class StoriaContratto extends SelezionatoAbstract implements Serializable, Comparable<StoriaContratto>  {
	@ManyToOne
	private Agente agente;
	@ManyToOne(fetch = FetchType.LAZY)
	private Contratto contratto;
	private BigDecimal costo_scontato;
	@Column(columnDefinition="datetime NOT NULL COMMENT 'data'")
	private Date data = ApplicationUtil.date();
	private Date data_esito;
	private Date data_stato;
	private Date data_stipula;
	private Date data_contabile;
	private Date data_attivazione;
	private Date data_caricamento;
	@ManyToOne
	private EsitoContratto esito;
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY) 
	private Long id;
	@Column(columnDefinition = "text")
	private String nota;
	@ManyToOne
	private Utente operatore;
	@ManyToOne
	private Prodotto prodotto;
	private Integer quantita;
	private String seriale;
	private Integer stato;
	@ManyToOne
	private Utente utente;
	public int compareTo(StoriaContratto o) {
		if(id != null && o.id != null) {
			return id.compareTo(o.id);
		} else if(id != null && o.id == null) {
			return -1;
		} else if(id == null && o.id != null) {
			return 1;
		} else {
			return 0;
		}
	}
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		StoriaContratto other = (StoriaContratto) obj;
		if (data == null) {
			if (other.data != null)
				return false;
		} else if (!data.equals(other.data))
			return false;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		return true;
	}
	public Agente getAgente() {
		return agente;
	}
	public String getColore() {
		if(ApplicationUtil.equals(stato, ConstantsGlobal.in_attesa)) {
			return ConstantsGlobal.color_in_attesa;
		} else if(ApplicationUtil.equals(stato, ConstantsGlobal.in_pagamento)) {
			return ConstantsGlobal.color_in_pagamento;
		} else if(ApplicationUtil.equals(stato, ConstantsGlobal.pagato)) {
			return ConstantsGlobal.color_pagato;
		} else if(ApplicationUtil.equals(stato, ConstantsGlobal.in_storno)) {
			return ConstantsGlobal.color_in_storno;
		} else if(ApplicationUtil.equals(stato, ConstantsGlobal.stornato)) {
			return ConstantsGlobal.color_stornato;
		} else {
			return ConstantsGlobal.color_in_attesa;
		} 
	}
	public Contratto getContratto() {
		return contratto;
	}
	public Date getData() {
		return data;
	}
	public Boolean getVendita() {
		return prodotto !=null && prodotto.getVendita();
	}
	public Date getData_esito() {
		return data_esito;
	}
	public Date getData_stato() {
		return data_stato;
	}
	public Date getData_stipula() {
		return data_stipula;
	}
	public Date getData_contabile() {
		return data_contabile;
	}
	public void setData_contabile(Date data_contabile) {
		this.data_contabile = data_contabile;
	}
	public EsitoContratto getEsito() {
		return esito;
	}
	public Long getId() {
		return id;
	}
	public String getNota() {
		return nota;
	}
	public String getNotaShort() {
		return ApplicationUtil.tronca(nota);
	}
	public Utente getOperatore() {
		return operatore;
	}
	public Prodotto getProdotto() {
		return prodotto;
	}
	public Categoria getCategoria() {
		return prodotto != null ? prodotto.getCategoria() : null;
	}
	public BigDecimal getCosto_scontato() {
		if(costo_scontato != null && getVendita()) {
			return costo_scontato;
		} 
		return new BigDecimal(0);
	}
	public Integer getQuantita() {
		if(ApplicationUtil.gtZero(quantita) && getVendita()) {
			return quantita;
		} 
		return 1;
	}
	public String getSeriale() {
		return  ApplicationUtil.trimToEmpty(seriale);
	}
	public String getSerialeNome() {
		if(prodotto!=null && ApplicationUtil.isNotBlank(prodotto.getSeriale())) {
			return ApplicationUtil.trimToEmpty(prodotto.getSeriale()) + ": " + getSeriale();
		} else {
			return getSeriale();
		}
	}
	public String getTipo_seriale() {
		if(prodotto!=null && ApplicationUtil.isNotBlank(prodotto.getSeriale())) {
			return ApplicationUtil.trimToEmpty(prodotto.getSeriale());
		} else {
			return ConstantsLanguage.SERIALE.stampa();
		}
	}
	public Integer getStato() {
		return stato;
	}
	public String getStatoTesto() {
		if(ApplicationUtil.equals(stato, ConstantsGlobal.in_attesa)) {
			return ConstantsLanguage.STATO_IN_ATTESA.stampa();
		} else if(ApplicationUtil.equals(stato, ConstantsGlobal.in_pagamento)) {
			return ConstantsLanguage.STATO_IN_PAGAMENTO.stampa();
		} else if(ApplicationUtil.equals(stato, ConstantsGlobal.pagato)) {
			return ConstantsLanguage.STATO_PAGATO.stampa();
		} else if(ApplicationUtil.equals(stato, ConstantsGlobal.in_storno)) {
			return ConstantsLanguage.STATO_IN_STORNO.stampa();
		} else if(ApplicationUtil.equals(stato, ConstantsGlobal.stornato)) {
			return ConstantsLanguage.STATO_STORNATO.stampa();
		} else {
			return ConstantsLanguage.STATO_IN_ATTESA.stampa();
		} 
	}
	public Utente getUtente() {
		return utente;
	}
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((data == null) ? 0 : data.hashCode());
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		return result;
	}
	public void setAgente(Agente agente) {
		this.agente = agente;
	}
	public void setContratto(Contratto contratto) {
		this.contratto = contratto;
	}
	public void setCosto_scontato(BigDecimal costo_scontato) {
		this.costo_scontato = costo_scontato;
	}
	public void setData(Date data) {
		this.data = data;
	}
	public void setData_esito(Date data_esito) {
		this.data_esito = data_esito;
	}
	public void setData_stato(Date data_stato) {
		this.data_stato = data_stato;
	}
	public void setData_stipula(Date data_stipula) {
		this.data_stipula = data_stipula;
	}
	public void setEsito(EsitoContratto esito) {
		this.esito = esito;
	}
	public void setId(Long id) {
		this.id = id;
	}
	public void setNota(String nota) {
		this.nota = nota;
	}
	public Date getData_attivazione() {
		return data_attivazione;
	}
	public void setData_attivazione(Date data_attivazione) {
		this.data_attivazione = data_attivazione;
	}
	public void setOperatore(Utente operatore) {
		this.operatore = operatore;
	}
	public void setProdotto(Prodotto prodotto) {
		this.prodotto = prodotto;
	}
	public void setQuantita(Integer quantita) {
		this.quantita = quantita;
	}
	public void setSeriale(String seriale) {
		this.seriale = seriale;
	}
	public void setStato(Integer stato) {
		this.stato = stato;
	}
	public void setUtente(Utente utente) {
		this.utente = utente;
	}
	@Override
	public String toString() {
		return ApplicationUtil.toString(id, contratto.getDenominazione());
	}
	public String getAgente_denominazione() {
		return getAgente() != null ? getAgente().getDenominazione() : "";
	}
	public String getAgente_codice() {
		return getAgente() != null ? getAgente().getCodice() : "";
	}
	public String getUtente_denominazione() {
		return getUtente() != null ? getUtente().getDenominazione() : "";
	}
	public String getUtente_codice() {
		return getUtente() != null ? getUtente().getUsername() : "";
	}
	public String getOperatore_denominazione() {
		return getOperatore() != null ? getOperatore().getDenominazione() : "";
	}
	public String getOperatore_codice() {
		return getOperatore() != null ? getOperatore().getUsername() : "";
	}
	public String getProdotto_denominazione() {
		return getProdotto().getNomeTipo();
	}
	public String getCategoria_denominazione() {
		return getProdotto().getCategoriaDenominazione();
	}
	public String getEsito_contratto() {
		return  getEsitoEtichetta();
	}
	public String getEsitoEtichetta() {
		return getEsito() != null ? getEsito().getEtichetta() : "";
	}
	public String getEsitoEtichettaCodice() {
		StringBuilder sb = new StringBuilder();
		sb.append(ApplicationUtil.trimToEmpty(getEsitoEtichetta()));
		if(ApplicationUtil.isNotBlank(getEsitoCodice())) {
			sb.append(" (" + getEsitoCodice() + ")");
		}
		return sb.toString();
	}
	public String getCodice_contratto() {
		return getEsitoCodice();
	}
	public String getEsitoCodice() {
		return getEsito() != null ? getEsito().getCodice() : "";
	}
	
	private Boolean bloccato = Boolean.FALSE;
	public String getBloccatoS() {
		return getBloccato() ? ConstantsLanguage.SI.stampa() : ConstantsLanguage.NO.stampa();
	}
	public Boolean getBloccato() {
		if(bloccato == null) {
			bloccato = Boolean.FALSE;
		}
		return bloccato;
	}
	public void setBloccato(Boolean bloccato) {
		this.bloccato = bloccato;
	}
	public Date getData_caricamento() {
		return data_caricamento;
	}
	public void setData_caricamento(Date data_caricamento) {
		this.data_caricamento = data_caricamento;
	}
	
}