package it.callite.model;
import java.io.Serializable;
import java.util.Date;
import java.util.List;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.Table;
import javax.persistence.Transient;

import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.model.generic.SelezionatoAbstract;
import it.callite.model.report.ColonnaModel;
import it.callite.utility.ApplicationUtil;
@SuppressWarnings("serial")
@Entity
//@Audited 
@Table(name = ConstantsGlobal.tp + "team", indexes = { @Index(name= "team", columnList = "data_creazione, nome") })
public class Team extends SelezionatoAbstract implements Serializable, Comparable<Team> {
	
	public Team getCopia() {
		Team pr = new Team();
		pr.setNome(this.getNome() + " " +  ConstantsLanguage.COPY.stampa());
		pr.setTurno(this.getTurno());
		return pr;
	}
	
	@Override
	public String toString() {
		return ApplicationUtil.toString(id, getDenominazione());
	}
	private Date data_creazione = ApplicationUtil.date();
	public Date getData_creazione() {
		return data_creazione;
	}
	public void setData_creazione(Date data_creazione) {
		this.data_creazione = data_creazione;
	}
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY) 
	private Long id;
	private String nome;
	@Column(columnDefinition = "text")
	private String turno;
	@ManyToMany(fetch = FetchType.LAZY)
	@JoinTable(name = ConstantsGlobal.tp + "supervisore_team", joinColumns=@JoinColumn(name="team_id", nullable=false, updatable=true, insertable=true), inverseJoinColumns=@JoinColumn(name="supervisore_id", nullable=false, updatable=true, insertable=true)    )
	private List<Utente> supervisore;
	@ManyToMany(fetch = FetchType.LAZY)
	@JoinTable(name = ConstantsGlobal.tp + "user_team", joinColumns=@JoinColumn(name="team_id", nullable=false, updatable=true, insertable=true), inverseJoinColumns=@JoinColumn(name="user_id", nullable=false, updatable=true, insertable=true)    )
	private List<Utente> user;
	public List<Utente> getUser() {
		return user;
	}
	public String getDenominazione() {
		return nome;
	}
	public void setUser(List<Utente> user) {
		this.user = user;
	}
	public List<Utente> getSupervisore() {
		return supervisore;
	}
	public void setSupervisore(List<Utente> supervisore) {
		this.supervisore = supervisore;
	}
	@ManyToMany(fetch = FetchType.LAZY)
	@JoinTable(name = ConstantsGlobal.tp + "team_campagna", joinColumns=@JoinColumn(name="team_id", nullable=false, updatable=true, insertable=true), inverseJoinColumns=@JoinColumn(name="campagne_id", nullable=false, updatable=true, insertable=true)    )
	private List<Campagna> campagne;
	public Long getId() {
		return id;
	}
	public void setId(Long id) {
		this.id = id;
	}
	public String getNome() {
		return nome;
	}
	public void setNome(String nome) {
		this.nome = ApplicationUtil.normalizeToNull(nome);
	}
	public String getTurno() {
		return turno;
	}
	public void setTurno(String turno) {
		this.turno = turno;
	}
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		result = prime * result + ((nome == null) ? 0 : nome.hashCode());
		return result;
	}
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Team other = (Team) obj;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		if (nome == null) {
			if (other.nome != null)
				return false;
		} else if (!nome.equals(other.nome))
			return false;
		return true;
	}
	public int compareTo(Team o) {
		if(id != null && o.id != null) {
			return id.compareTo(o.id);
		} else if(id != null && o.id == null) {
			return -1;
		} else if(id == null && o.id != null) {
			return 1;
		} else {
			return 0;
		}
	}
	public List<Campagna> getCampagne() {
		return campagne;
	}
	public void setCampagne(List<Campagna> campagne) {
		this.campagne = campagne;
	}

	@Transient
	private ColonnaModel model;

	public ColonnaModel getModel() {
		if(model == null) {
			model = new ColonnaModel(id);
		}
		return model;
	}
	public void setModel(ColonnaModel model) {
		this.model = model;
	}
}