package it.callite.model;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import javax.persistence.Transient;
import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.constants.ConstantsMap;
import it.callite.model.generic.SelezionatoAbstract;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.Label;
@SuppressWarnings("serial")
@Entity
//@Audited 
@Table(name = ConstantsGlobal.tp + "telecom", indexes = { @Index(name= "telecom", columnList = "data_creazione, data_invio, codice"), @Index(name = "id", columnList = "userfield") })
public class Telecom   extends SelezionatoAbstract implements Serializable, Comparable<Telecom> {
	private String userfield;
	public String getUserfield() {
		return userfield;
	}
	public void setUserfield(String userfield) {
		this.userfield = userfield;
	}
	public void impostaUserfield() {
		if(contatto != null && userfield == null) {
			userfield = contatto.getUserfield();
		}
	}
	@Column(columnDefinition = "text")
	private String body;
	private Integer codice;
	@ManyToOne
	private Contatto contatto;
	@Column(columnDefinition="datetime NOT NULL COMMENT 'data'")
	private Date data_creazione = ApplicationUtil.date();
	
	private Date data_invio;
	@Transient
	private Date dataSchedulata;
	@Column(columnDefinition = "text")
	private String fields;
	@Column(columnDefinition = "text")
	private String outputs;
	@Column(columnDefinition = "text")
	private String headers;
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY) 
	private Long id;
	
	@Transient
	private List<Label> labels = new ArrayList<Label>();
	
	@Column(columnDefinition = "text")
	private String link;
	@Column(columnDefinition = "text")
	private String parameters;
	@Column(columnDefinition = "text")
	private String password;
	
	@Column(columnDefinition = "text")
	private String response;
	
	@Column(columnDefinition = "text")
	private String stato;
	
	@ManyToOne
	private TelecomTemplate telecomTemplate;
	private String tipo = ConstantsMap.r_type1;
	
	@Column(columnDefinition = "text")
	private String username;
	@ManyToOne
	private Utente utente;
	public void addMotivo(Integer codice, String ragione) {
		setCodice(codice);
		String s = "";
		if(ApplicationUtil.equals(codice, ConstantsGlobal.IN_INVIO)) {
			s = ConstantsLanguage.TESTO_IN_INVIO.stampa();
		} else if(ApplicationUtil.equals(codice, ConstantsGlobal.INVIATA)) {
			s = ConstantsLanguage.TESTO_INVIATA.stampa();
		} else if(ApplicationUtil.equals(codice, ConstantsGlobal.NON_INVIATA)) {
			s = ConstantsLanguage.TESTO_NON_INVIATA.stampa();
		} else if(ApplicationUtil.equals(codice, ConstantsGlobal.SCHEDULATA)) {
			s = ConstantsLanguage.TESTO_SCHEDULATA.stampa();
		} else if(ApplicationUtil.equals(codice, ConstantsGlobal.PAUSE)) {
			s = ConstantsLanguage.TESTO_PAUSE.stampa();
		} else if(ApplicationUtil.equals(codice, ConstantsGlobal.ANNULLATA)) {
			s = ConstantsLanguage.TESTO_ANNULLATA.stampa();
		} else {
			s = ConstantsLanguage.TESTO_IN_INVIO.stampa();
		}
		if(ApplicationUtil.isNotBlank(ragione)) {
			setStato(s + ": " + ragione);
		} else {
			setStato(s);
		}
	}
	public int compareTo(Telecom o) {
		if(id != null && o.id != null) {
			return id.compareTo(o.id);
		} else if(id != null && o.id == null) {
			return -1;
		} else if(id == null && o.id != null) {
			return 1;
		} else {
			return 0;
		}
	}
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Telecom other = (Telecom) obj;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		return true;
	}
	public String getBody() {
		return ApplicationUtil.trimToNull(body);
	}
	public Integer getCodice() {
		return codice;
	}
	
	public Contatto getContatto() {
		return contatto;
	}
	
	public Date getData_creazione() {
		return data_creazione;
	}
	public Date getData_invio() {
		return data_invio;
	}
	
	
	public Date getDataSchedulata() {
		return ApplicationUtil.aggiustaDataT(dataSchedulata, true);
	}
	public String getDenominazione() {
		return ApplicationUtil.trimToEmpty(getContatto().getDenominazione());
	}
	
	public String getOutputs() {
		return ApplicationUtil.trimToNull(outputs);
	}
	public void setOutputs(String outputs) {
		this.outputs = ApplicationUtil.trimToNull(outputs);
	}
	public String getFields() {
		return ApplicationUtil.trimToNull(fields);
	}
	public String getHeaders() {
		return ApplicationUtil.trimToNull(headers);
	}
	public Long getId() {
		return id;
	}
	public List<Label> getLabels() {
		return labels;
	}
	
	public String getLink() {
		return ApplicationUtil.trimToNull(link);
	}
	
	public String getParameters() {
		return ApplicationUtil.trimToNull(parameters);
	}
	
	public String getPassword() {
		return password;
	}
	public String getResponse() {
		return ApplicationUtil.trimToNull(response);
	}
	public String getStato() {
		return stato;
	}
	public TelecomTemplate getTelecomTemplate() {
		return telecomTemplate;
	}
	public String getTipo() {
		if(tipo == null) {
			tipo = ConstantsMap.r_type1;
		}
		return tipo;
	}
	public String getUsername() {
		return ApplicationUtil.trimToNull(username);
	}
	
	public Utente getUtente() {
		return utente;
	}
	
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		return result;
	}
	
	public void setBody(String body) {
		this.body = ApplicationUtil.trimToNull(body);
	}
	
	public void setCodice(Integer codice) {
		this.codice = codice;
	}
	public Boolean getType() {
		String t = getTipo();
		return   t != null && ( t.equals(ConstantsMap.r_type6) || t.equals(ConstantsMap.r_type7) );
	}
	public void setContatto(Contatto contatto) {
		this.contatto = contatto;
	}
	public void setData_creazione(Date data_creazione) {
		this.data_creazione = data_creazione;
	}
	public void setData_invio(Date dataInvio) {
		data_invio = dataInvio;
	}
	
	public void setDataSchedulata(Date dataSchedulata) {
		this.dataSchedulata = ApplicationUtil.aggiustaDataT(dataSchedulata, true);
	}
	
	public void setFields(String fields) {
		this.fields = ApplicationUtil.trimToNull(fields);
	}
	public void setHeaders(String headers) {
		this.headers = ApplicationUtil.trimToNull(headers);
	}
	public void setId(Long id) {
		this.id = id;
	}
	public void setLabels(List<Label> labels) {
		this.labels = labels;
	}
	
	public void setLink(String link) {
		this.link = ApplicationUtil.trimToNull(link);
	}
	public void setParameters(String parameters) {
		this.parameters = ApplicationUtil.trimToNull(parameters);
	}
	public void setPassword(String password) {
		this.password = password;
	}
	public void setResponse(String response) {
		this.response = ApplicationUtil.trimToNull(response);
	}
	public void setStato(String stato) {
		this.stato = stato;
	}
	public void setTelecomTemplate(TelecomTemplate telecomTemplate) {
		this.telecomTemplate = telecomTemplate;
	}
	public void setTipo(String tipo) {
		this.tipo = tipo;
	}
	public void setUsername(String username) {
		this.username = ApplicationUtil.trimToNull(username);
	}
	
	public void setUtente(Utente utente) {
		this.utente = utente;
	}
	@Override
	public String toString() {
		return ApplicationUtil.toString(id, getDenominazione());
	}
	public boolean valido() {
		return ApplicationUtil.isNotBlank(this.getLink());
	}
	
	public boolean inviabile() {
		return telecomTemplate != null;
	}
	
	private String sistema = ConstantsMap.r_system1;
	public String getSistema() {
		if(sistema == null) {
			sistema = ConstantsMap.r_system1;
		}
		return sistema;
	}
	public void setSistema(String sistema) {
		this.sistema = sistema;
	}
	
	public Boolean getSystem1() {
		String t = getSistema();
		return   t != null &&  t.equals(ConstantsMap.r_system1);
	}
	public Boolean getSystem2() {
		String t = getSistema();
		return   t != null &&  t.equals(ConstantsMap.r_system2);
	}
	public Boolean getSystem3() {
		String t = getSistema();
		return   t != null &&  t.equals(ConstantsMap.r_system3);
	}
	public Boolean getSystem4() {
		String t = getSistema();
		return   t != null &&  t.equals(ConstantsMap.r_system4);
	}
	
	private String driver;
	@Column(columnDefinition = "text")
	private String commands;
	
	public String getDriver() {
		return ApplicationUtil.normalizeToNull(driver);
	}
	public void setDriver(String driver) {
		this.driver = ApplicationUtil.normalizeToNull(driver);
	}
	public String getCommands() {
		return ApplicationUtil.trimToNull(commands);
	}
	public void setCommands(String commands) {
		this.commands = ApplicationUtil.trimToNull(commands);
	}
	
	@Column(columnDefinition = "text")
	private String inputs;
	
	public void setInputs(String inputs) {
		this.inputs = ApplicationUtil.trimToNull(inputs);
	}
	public String getInputs() {
		return ApplicationUtil.trimToNull(inputs);
	}
	
	@ManyToOne
	private Schedulatore scheduler;
	public Schedulatore getScheduler() {
		return scheduler;
	}
	public void setScheduler(Schedulatore scheduler) {
		this.scheduler = scheduler;
	}
	public String getUtente_denominazione() {
		return getUtente() != null ? getUtente().getDenominazione() : "";
	}
}