package it.callite.model;

import java.io.Serializable;
import java.util.Date;
import java.util.List;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.OneToMany;
import javax.persistence.Table;

import org.hibernate.annotations.OnDelete;
import org.hibernate.annotations.OnDeleteAction;

import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.constants.ConstantsMap;
import it.callite.model.generic.SelezionatoAbstract;
import it.callite.utility.ApplicationUtil;


@SuppressWarnings("serial")
@Entity
//@Audited 
@Table(name = ConstantsGlobal.tp + "telecom_template", indexes = { @Index(name= "telecom_template", columnList = "data_creazione, nome") })
public class TelecomTemplate extends SelezionatoAbstract implements Serializable {
	
	private Boolean attivo = Boolean.TRUE;
	

	@OneToMany(mappedBy = "telecomTemplate")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<AutoProcessTelecom> autoProcessTelecoms;
	
	@Column(columnDefinition = "text")
	private String body;

	@ManyToMany(fetch = FetchType.LAZY)
	@JoinTable(name = ConstantsGlobal.tp + "telecom_campagna", joinColumns=@JoinColumn(name="telecom_id", nullable=false, updatable=true, insertable=true), inverseJoinColumns=@JoinColumn(name="campagne_id", nullable=false, updatable=true, insertable=true)    )
	private List<Campagna> campagne;

	private Date data_creazione = ApplicationUtil.date();
	
	
	@ManyToMany(fetch = FetchType.LAZY)
	@JoinTable(name = ConstantsGlobal.tp + "esito_telecom", joinColumns=@JoinColumn(name="telecom_id", nullable=false, updatable=true, insertable=true), inverseJoinColumns=@JoinColumn(name="esito_id", nullable=false, updatable=true, insertable=true)    )
	private List<EsitoArchiviazione> esiti;
	@Column(columnDefinition = "text")
	private String fields;
	@Column(columnDefinition = "text")
	private String outputs;
	@Column(columnDefinition = "text")
	private String headers;

	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY) 
	private Long id;
	
	@Column(columnDefinition = "text")
	private String inputs;
	
	@Column(columnDefinition = "text")
	private String link;
	
	private String nome;
	
	@Column(columnDefinition = "text")
	private String parameters;

	@Column(columnDefinition = "text")
	private String password;
	
	@OneToMany(mappedBy = "telecomTemplate")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<Telecom> telecoms;

	private String tipo = ConstantsMap.r_type1;

	@Column(columnDefinition = "text")
	private String username;

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		TelecomTemplate other = (TelecomTemplate) obj;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		return true;
	}

	public Boolean getAttivo() {
		return attivo;
	}
	
	public List<AutoProcessTelecom> getAutoProcessTelecoms() {
		return autoProcessTelecoms;
	}

	public String getBody() {
		return ApplicationUtil.trimToNull(body);
	}

	public List<Campagna> getCampagne() {
		return campagne;
	}

	public TelecomTemplate getCopia() {
		TelecomTemplate copia = new TelecomTemplate();
		copia.setNome(this.getNome() + " " + ConstantsLanguage.COPY.stampa());
		copia.setAttivo(this.getAttivo());
		copia.setBody(this.getBody());
		copia.setLink(this.getLink());
		copia.setHeaders(this.getHeaders());
		copia.setFields(this.getFields());
		copia.setOutputs(this.getOutputs());
		copia.setParameters(this.getParameters());
		copia.setInputs(this.getInputs());
		copia.setTipo(this.getTipo());
		copia.setSistema(this.getSistema());
		copia.setUsername(this.getUsername());
		copia.setPassword(this.getPassword());
		copia.setCommands(this.getCommands());
		copia.setDriver(this.getDriver());
		return copia;
	}

	public Date getData_creazione() {
		return data_creazione;
	}

	public String getDenominazione() {
		return nome;
	}


	public List<EsitoArchiviazione> getEsiti() {
		return esiti;
	}

	public String getOutputs() {
		return ApplicationUtil.trimToNull(outputs);
	}
	public void setOutputs(String outputs) {
		this.outputs = ApplicationUtil.trimToNull(outputs);
	}
	
	public String getFields() {
		return ApplicationUtil.trimToNull(fields);
	}


	public String getHeaders() {
		return ApplicationUtil.trimToNull(headers);
	}

	public Long getId() {
		return id;
	}
	
	public String getInputs() {
		return ApplicationUtil.trimToNull(inputs);
	}

	public String getLink() {
		return ApplicationUtil.trimToNull(link);
	}

	public String getNome() {
		return ApplicationUtil.normalizeToNull(nome);
	}
	
	public String getParameters() {
		return ApplicationUtil.trimToNull(parameters);
	}

	public String getPassword() {
		return password;
	}

	
	public List<Telecom> getTelecoms() {
		return telecoms;
	}
	
	public String getTipo() {
		if(tipo == null) {
			tipo = ConstantsMap.r_type1;
		}
		return tipo;
	}

	public String getUsername() {
		return ApplicationUtil.trimToNull(username);
	}
	
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		return result;
	}
	public void setAttivo(Boolean attivo) {
		this.attivo = attivo;
	}
	public void setAutoProcessTelecoms(List<AutoProcessTelecom> autoProcessTelecoms) {
		this.autoProcessTelecoms = autoProcessTelecoms;
	}

	public void setBody(String body) {
		this.body = ApplicationUtil.trimToNull(body);
	}
	
	public void setCampagne(List<Campagna> campagne) {
		this.campagne = campagne;
	}
	
	public void setData_creazione(Date data_creazione) {
		this.data_creazione = data_creazione;
	}

	public void setEsiti(List<EsitoArchiviazione> esiti) {
		this.esiti = esiti;
	}

	public void setFields(String fields) {
		this.fields = ApplicationUtil.trimToNull(fields);
	}

	public void setHeaders(String headers) {
		this.headers = ApplicationUtil.trimToNull(headers);
	}
	public void setId(Long id) {
		this.id = id;
	}
	
	public Boolean getType() {
		String t = getTipo();
		return   t != null && ( t.equals(ConstantsMap.r_type6) || t.equals(ConstantsMap.r_type7) );
	}

	public void setInputs(String inputs) {
		this.inputs = ApplicationUtil.trimToNull(inputs);
	}

	public void setLink(String link) {
		this.link = ApplicationUtil.trimToNull(link);
	}

	public void setNome(String nome) {
		this.nome = ApplicationUtil.normalizeToNull(nome);
	}

	public void setParameters(String parameters) {
		this.parameters = ApplicationUtil.trimToNull(parameters);
	}

	public void setPassword(String password) {
		this.password = password;
	}

	public void setTelecoms(List<Telecom> telecoms) {
		this.telecoms = telecoms;
	}

	public void setTipo(String tipo) {
		this.tipo = tipo;
	}

	public void setUsername(String username) {
		this.username = ApplicationUtil.trimToNull(username);
	}

	@Override
	public String toString() {
		return ApplicationUtil.toString(id, getDenominazione());
	}
	
	private String sistema = ConstantsMap.r_system1;

	public String getSistema() {
		if(sistema == null) {
			sistema = ConstantsMap.r_system1;
		}
		return sistema;
	}

	public void setSistema(String sistema) {
		this.sistema = sistema;
	}
	

	public Boolean getSystem1() {
		String t = getSistema();
		return   t != null &&  t.equals(ConstantsMap.r_system1);
	}
	public Boolean getSystem2() {
		String t = getSistema();
		return   t != null &&  t.equals(ConstantsMap.r_system2);
	}
	public Boolean getSystem3() {
		String t = getSistema();
		return   t != null &&  t.equals(ConstantsMap.r_system3);
	}
	public Boolean getSystem4() {
		String t = getSistema();
		return   t != null &&  t.equals(ConstantsMap.r_system4);
	}

	private String driver;
	
	@Column(columnDefinition = "text")
	private String commands;


	public String getDriver() {
		return ApplicationUtil.normalizeToNull(driver);
	}

	public void setDriver(String driver) {
		this.driver = ApplicationUtil.normalizeToNull(driver);
	}

	public String getCommands() {
		return ApplicationUtil.trimToNull(commands);
	}

	public void setCommands(String commands) {
		this.commands = ApplicationUtil.trimToNull(commands);
	}
}