package it.callite.model;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.faces.model.SelectItem;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.apache.commons.collections4.ListUtils;

import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.constants.ConstantsMap;
import it.callite.message.Message;
import it.callite.model.generic.SelezionatoAbstract;
import it.callite.utility.ApplicationUtil;
@SuppressWarnings("serial")
@Entity
//@Audited 
@Table(name = ConstantsGlobal.tp + "template_export_anagrafica", indexes = { @Index(name= "template_export_anagrafica", columnList = "data_creazione, attivo, predefinito, nome") })
public class TemplateExportAnagrafica extends SelezionatoAbstract implements Serializable {
	private Boolean attivo = Boolean.TRUE;
	private Boolean predefinito = Boolean.FALSE;
	public Boolean getPredefinito() {
		return predefinito;
	}
	public void setPredefinito(Boolean predefinito) {
		this.predefinito = predefinito;
	}
	@Transient
	private List<String> campo_multiplo;
	private Date data_creazione = ApplicationUtil.date();
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY) 
	private Long id;
	private Boolean intestazione = Boolean.TRUE;
	private String nome;
	private String ordine = "";
	private String quote;
	private String separatore;
	private Integer tipo = ConstantsGlobal.random;
	@Transient
	private List<String> valore_multiplo;
	@Transient
	private List<String> valore_totale;
	@Transient
	private Map<String, Boolean> valoreTT = new HashMap<String, Boolean>();
	@Column(columnDefinition = "text")
	private String valori;
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		TemplateExportAnagrafica other = (TemplateExportAnagrafica) obj;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		return true;
	}
	public Boolean getAttivo() {
		return attivo;
	}
	public List<String> getCampo_multiplo() {
		String sep = ConstantsMap.separatore;
		List<String> valore = new ArrayList<String>();
		valore.addAll(ConstantsMap.anagraficaMap);
		List<String> st = ListUtils.subtract(getValore_multiplo(), valore);
		st.removeAll(Collections.singleton(sep));
		return st;
	}
	public TemplateExportAnagrafica getCopia() {
		TemplateExportAnagrafica pr = new TemplateExportAnagrafica();
		pr.setNome(this.getNome() + " " +  ConstantsLanguage.COPY.stampa());
		pr.setOrdine(this.getOrdine());
		pr.setTipo(this.getTipo());
		pr.setValori(this.getValori());
		pr.setAttivo(this.getAttivo());
		pr.setPredefinito(false);
		pr.setIntestazione(this.getIntestazione());
		pr.setQuote(this.getQuote());
		pr.setSeparatore(this.getSeparatore());
		pr.setRem_fisso(this.getRem_fisso());
		pr.setAdd_fisso(this.getAdd_fisso());
		pr.setRem_fax(this.getRem_fax());
		pr.setAdd_fax(this.getAdd_fax());
		pr.setRem_mobile(this.getRem_mobile());
		pr.setAdd_mobile(this.getAdd_mobile());
		pr.setRem_altro(this.getRem_altro());
		pr.setAdd_altro(this.getAdd_altro());
		return pr;
	}
	public Character getQuoteChar() {
		return ApplicationUtil.getChar(quote);
	}
	public Character getSeparatoreChar() {
		return ApplicationUtil.getChar(separatore);
	}
	public Boolean getTelefono_altro() {
		return getValore_multiplo().contains(ConstantsMap.anagrafica_telefono_altro);
	}
	public Boolean getTelefono_fax() {
		return getValore_multiplo().contains(ConstantsMap.anagrafica_telefono_fax);
	}
	public Boolean getTelefono_fisso() {
		return getValore_multiplo().contains(ConstantsMap.anagrafica_telefono_fisso);
	}
	public Boolean getTelefono_mobile() {
		return getValore_multiplo().contains(ConstantsMap.anagrafica_telefono_mobile);
	}
	public Date getData_creazione() {
		return data_creazione;
	}
	public String getDenominazione() {
		return ApplicationUtil.trimToEmpty(getNome());
	}
	public Long getId() {
		return id;
	}
	public Boolean getIntestazione() {
		return ApplicationUtil.isTrue(intestazione);
	}
	public String getNome() {
		return nome;
	}
	public List<SelectItem> getOrdinamento() {
		List<SelectItem> sis = new ArrayList<SelectItem>();
		sis.add(new SelectItem("", ConstantsLanguage.DEFAULT.stampa()));
		for(String c : getValore_multiplo()) {
			if(ConstantsMap.anagraficaMap.contains(c)) {
				sis.add(new SelectItem(c,  Message.addCatalogText(c)));
			}
		}
		return sis;
	}
	public List<SelectItem> getOrdinamentoTipo() {
		List<SelectItem> sis = new ArrayList<SelectItem>();
		sis.add(new SelectItem(ConstantsGlobal.random, ConstantsLanguage.RANDOM.stampa()));
		sis.add(new SelectItem(ConstantsGlobal.ascendente, ConstantsLanguage.ASCENDENTE.stampa()));
		sis.add(new SelectItem(ConstantsGlobal.discendente, ConstantsLanguage.DISCENDENTE.stampa()));
		return sis;
	}
	public String getOrdine() {
		return ordine;
	}
	public Integer getPosizione(String campo) {
		List<String> valori = getValore_multiplo(); 
		return valori.indexOf(campo);
	}
	public String getQuote() {
		return quote;
	}
	public String getSeparatore() {
		return separatore;
	}
	public List<SelectItem> getQuoteList() {
		return ConstantsMap.getQuoteList();
	}
	
	public List<SelectItem> getSeparatoreList() {
		return ConstantsMap.getSeparatoreList();
	}
	public Integer getTipo() {
		if(tipo == null)
			tipo = ConstantsGlobal.random;
		return tipo;
	}
	public String getTipoString() {
		StringBuilder sb = new StringBuilder();
		Integer tipo = getTipo();
		if(ApplicationUtil.equals(tipo, ConstantsGlobal.random)) {
			sb.append(ConstantsLanguage.RANDOM.stampa());
		} else if(ApplicationUtil.equals(tipo, ConstantsGlobal.ascendente)) {
			sb.append(ConstantsLanguage.ASCENDENTE.stampa());
		} else if(ApplicationUtil.equals(tipo, ConstantsGlobal.discendente)) {
			sb.append(ConstantsLanguage.DISCENDENTE.stampa());
		} else {
			sb.append(ConstantsLanguage.RANDOM.stampa());
		} 
		if(!tipo.equals(ConstantsGlobal.random)) {
			if(ApplicationUtil.isNotBlank(getOrdine())) {
				sb.append(" - " +  Message.addCatalogText(getOrdine()));
			} else {
				sb.append(" - " + ConstantsLanguage.DEFAULT.stampa());
			}
		}
		return sb.toString();
	}
	public boolean getValidatore() {
		String sep = ConstantsMap.separatore;
		List<String> st = getValore_multiplo();
		st.removeAll(Collections.singleton(sep));
		return st.isEmpty();
	}
	public List<String> getValore_multiplo() {
		return ApplicationUtil.token(valori, ConstantsGlobal.colonSeparator);
	}
	
	public List<String> getValore_totale() {
		String sep = ConstantsMap.separatore;
		//while(valore_totale.remove(sep));
		valore_totale.removeAll(Collections.singleton(sep));
		//Collections.sort(valore_totale);
		valore_totale.add(0, sep);
		return valore_totale;
	}
	public List<String> getValoreAnagrafica() {
		List<String> valore = new ArrayList<String>();
		valore.addAll(ConstantsMap.anagraficaMap);
		valore.removeAll(getValore_multiplo());
		return valore;
	}
	public Map<String, Boolean> getValoreMap() {
		if(ApplicationUtil.empty(valoreTT)) {
			List<String> valore = new ArrayList<String>(ConstantsMap.anagraficaMap);
			List<String> listaDaInt = getValore_multiplo();
			List<String> lista = ListUtils.intersection(listaDaInt, valore) ;
			Iterator<String> vvIt = valore.iterator();
			while (vvIt.hasNext()) {
				String token = (String) vvIt.next();
				if(lista.contains(token)) {
					valoreTT.put(token, true);
				} else {
					valoreTT.put(token, false);
				}
			}
		}
		return valoreTT;
	}
	public String getValori() {
		return valori;
	}
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		return result;
	}
	public void setAttivo(Boolean attivo) {
		this.attivo = attivo;
	}
	public void setData_creazione(Date data_creazione) {
		this.data_creazione = data_creazione;
	}
	public void setId(Long id) {
		this.id = id;
	}
	public void setIntestazione(Boolean intestazione) {
		this.intestazione = intestazione;
	}
	public void setNome(String nome) {
		this.nome = ApplicationUtil.normalizeToNull(nome);
	}
	public void setOrdine(String ordine) {
		this.ordine = ordine;
	}
	public void setQuote(String quote) {
		this.quote = quote;
	}
	public void setSeparatore(String separatore) {
		this.separatore = separatore;
	}
	public void setTipo(Integer tipo) {
		this.tipo = tipo;
	}
	public void setValore_multiplo(List<String> valore_multiplo) {
		valori = ApplicationUtil.detoken(valore_multiplo, ConstantsGlobal.colonSeparator);
	}
	public void setValore_totale(List<String> valore_totale) {
		this.valore_totale = valore_totale;
	}
	public void setValori(String valori) {
		this.valori = valori;
	}
	@Override
	public String toString() {
		return ApplicationUtil.toString(id, getDenominazione());
	}
	
	
	private String rem_fisso;
	private String add_fisso;
	private String rem_mobile;
	private String add_mobile;
	private String rem_fax;
	private String add_fax;
	private String rem_altro;
	private String add_altro;
	public String getRem_fisso() {
		return ApplicationUtil.trimToEmpty(rem_fisso);
	}
	public String getAdd_fisso() {
		return ApplicationUtil.trimToEmpty(add_fisso);
	}
	public String getRem_mobile() {
		return ApplicationUtil.trimToEmpty(rem_mobile);
	}
	public String getAdd_mobile() {
		return ApplicationUtil.trimToEmpty(add_mobile);
	}
	public String getRem_fax() {
		return ApplicationUtil.trimToEmpty(rem_fax);
	}
	public String getAdd_fax() {
		return ApplicationUtil.trimToEmpty(add_fax);
	}
	public String getRem_altro() {
		return ApplicationUtil.trimToEmpty(rem_altro);
	}
	public String getAdd_altro() {
		return ApplicationUtil.trimToEmpty(add_altro);
	}
	public void setRem_fisso(String rem_fisso) {
		this.rem_fisso =  ApplicationUtil.normalize(ApplicationUtil.aggiustaLista(true, ConstantsGlobal.colonSeparator, rem_fisso));
	}
	public void setAdd_fisso(String add_fisso) {
		this.add_fisso =  ApplicationUtil.normalize(ApplicationUtil.aggiustaLista(true, ConstantsGlobal.colonSeparator, add_fisso));
	}
	public void setRem_mobile(String rem_mobile) {
		this.rem_mobile =  ApplicationUtil.normalize(ApplicationUtil.aggiustaLista(true, ConstantsGlobal.colonSeparator, rem_mobile));
	}
	public void setAdd_mobile(String add_mobile) {
		this.add_mobile =  ApplicationUtil.normalize(ApplicationUtil.aggiustaLista(true, ConstantsGlobal.colonSeparator, add_mobile));
	}
	public void setRem_fax(String rem_fax) {
		this.rem_fax =  ApplicationUtil.normalize(ApplicationUtil.aggiustaLista(true, ConstantsGlobal.colonSeparator, rem_fax));
	}
	public void setAdd_fax(String add_fax) {
		this.add_fax =  ApplicationUtil.normalize(ApplicationUtil.aggiustaLista(true, ConstantsGlobal.colonSeparator, add_fax));
	}
	public void setRem_altro(String rem_altro) {
		this.rem_altro =  ApplicationUtil.normalize(ApplicationUtil.aggiustaLista(true, ConstantsGlobal.colonSeparator, rem_altro));
	}
	public void setAdd_altro(String add_altro) {
		this.add_altro =  ApplicationUtil.normalize(ApplicationUtil.aggiustaLista(true, ConstantsGlobal.colonSeparator, add_altro));
	}
}