package it.callite.model;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.JoinColumn;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.model.generic.SelezionatoAbstract;
import it.callite.utility.ApplicationUtil;
@SuppressWarnings("serial")
@Entity
//@Audited 
@Table(name = ConstantsGlobal.tp + "ufficio", indexes = { @Index(name= "ufficio", columnList = "data_creazione, nome") })
public class Ufficio extends SelezionatoAbstract  implements Serializable, Comparable<Ufficio> {
	
	public Ufficio getCopia() {
		Ufficio pr = new Ufficio();
		pr.setNome(this.getNome() + " " +  ConstantsLanguage.COPY.stampa());
		pr.setDescrizione(this.getDescrizione());
		pr.setRegione(this.getRegione());
		pr.setProvincia(this.getProvincia());
		pr.setComune(this.getComune());
		pr.setCap(this.getCap());
		return pr;
	}
	
	@Override
	public String toString() {
		return ApplicationUtil.toString(id, getDenominazione());
	}
	public String getDenominazione() {
		return ApplicationUtil.trimToEmpty(getNome());
	}
 
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY) 
	private Long id;
	private String nome;
	@Column(columnDefinition = "text")
	private String descrizione;
	private Date data_creazione = ApplicationUtil.date();
	private String regione;
	private String provincia;
	private String cap;
	private String comune;
	public String getRegione() {
		return regione;
	}
	public void setRegione(String regione) {
		this.regione = ApplicationUtil.normalizeToUpperNull(regione);
	}
	public String getProvincia() {
		return provincia;
	}
	public void setProvincia(String provincia) {
		this.provincia = ApplicationUtil.normalizeToUpperNull(provincia);
	}
	public String getCap() {
		return cap;
	}
	public void setCap(String cap) {
		this.cap = ApplicationUtil.normalizeToUpperNull(cap);
	}
	public String getComune() {
		return comune;
	}
	public void setComune(String comune) {
		this.comune = ApplicationUtil.normalizeToUpperNull(comune);
	}
	public Ufficio() {
	}
	public int compareTo(Ufficio o) {
		if(id != null && o.id != null) {
			return id.compareTo(o.id);
		} else if(id != null && o.id == null) {
			return -1;
		} else if(id == null && o.id != null) {
			return 1;
		} else {
			return 0;
		}
	}
	@OneToMany
	@JoinColumn(name = "ufficio_id", nullable = true, updatable=true, insertable=true)
	private List<Agente> agenti = new ArrayList<Agente>();
	public List<Agente> getAgenti() {
		return agenti;
	}
	public void setAgenti(List<Agente> agenti) {
		this.agenti = agenti;
	}
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Ufficio other = (Ufficio) obj;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		return true;
	}
	public Date getData_creazione() {
		return data_creazione;
	}
	public String getDescrizione() {
		return ApplicationUtil.trimToEmpty(descrizione);
	}
	public Long getId() {
		return id;
	}
	public String getNome() {
		return nome;
	}
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		return result;
	}
	public void setData_creazione(Date data_creazione) {
		this.data_creazione = data_creazione;
	}
	public void setDescrizione(String descrizione) {
		this.descrizione = ApplicationUtil.trimToNull(descrizione);
	}
	public void setId(Long id) {
		this.id = id;
	}
	public void setNome(String nome) {
		this.nome = ApplicationUtil.normalizeToNull(nome);
	}
}