package it.callite.model;
import java.io.Serializable;
import java.sql.Time;
import java.util.ArrayList;
import java.util.Date;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Set;
import java.util.concurrent.TimeUnit;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.validator.ValidatorException;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.hibernate.annotations.OnDelete;
import org.hibernate.annotations.OnDeleteAction;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.userdetails.UserDetails;

import it.callite.chiamate.Chiamate;
import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.constants.ConstantsMap;
import it.callite.constants.ConstantsRole;
import it.callite.message.Message;
import it.callite.model.generic.SelezionatoAbstract;
import it.callite.model.messaggio.Destinatario;
import it.callite.model.messaggio.DocumentoMessaggio;
import it.callite.model.messaggio.Messaggio;
import it.callite.model.report.ColonnaModel;
import it.callite.model.sip.Cdr;
import it.callite.model.sip.SipBuddies;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.ParameterTrunk;
@SuppressWarnings("serial")
@Entity
//@Audited 
@Table(name = ConstantsGlobal.tp + "utente", indexes = { @Index(name= "utente", columnList = "username, email, password"), @Index(name= "dati_utente", columnList = "cognome, nome"), @Index(name= "date", columnList = "data_creazione, data_login, data_logout, data_password, inverti, data_inizio, data_fine, week_inizio, week_fine, ora_inizio, ora_fine") })
public class Utente extends SelezionatoAbstract implements Serializable, UserDetails, Comparable<Utente> {

	public Utente getCopia() {
		Utente pr = new Utente();
		pr.setEnabled(this.isEnabled());
		//pr.setInterno(this.getInterno());
		pr.setTrunk(this.getTrunk());
		pr.setFattura(this.getFattura());
		pr.setFolder(this.getFolder());
		pr.setPromemoria(this.getPromemoria());
		pr.setGeneraEstratto(this.getGeneraEstratto());
		pr.setDati_fiscali(this.getDati_fiscali());
		pr.setIp_bloccati(this.getIp_bloccati());
		pr.setIp_consentiti(this.getIp_consentiti());
		pr.setInverti(this.getInverti());
		pr.setOra_inizio(this.getOra_inizio());
		pr.setOra_fine(this.getOra_fine());
		pr.setData_fine(this.getData_fine());
		pr.setData_inizio(this.getData_inizio());
		pr.setWeek_fine(this.getWeek_fine());
		pr.setWeek_inizio(this.getWeek_inizio());
		pr.setCambioPassword(this.getCambioPassword());
		pr.setNotificaMessaggi(this.getNotificaMessaggi());
		pr.setNotificaLogin(this.getNotificaLogin());
		pr.setDisattivaMessaggistica(this.getDisattivaMessaggistica());
		pr.setVisualizzaStorico(this.getVisualizzaStorico());
		pr.setDisattivaFunzioni(this.getDisattivaFunzioni());
		pr.setRuoli(this.getRuoli());
		return pr;
	}

	@OneToMany(mappedBy = "utente", fetch = FetchType.LAZY)
	@OnDelete(action = OnDeleteAction.CASCADE)
	private Set<Accesso> accesso;
	@OneToMany(mappedBy = "user", fetch = FetchType.LAZY)
	@OnDelete(action = OnDeleteAction.CASCADE)
	private Set<Acconto> acconti;
	@ManyToMany(fetch = FetchType.LAZY)
	@JoinTable(name = ConstantsGlobal.tp + "agente_utente", joinColumns=@JoinColumn(name="utente_id", nullable=false, updatable=true, insertable=true), inverseJoinColumns=@JoinColumn(name="agente_id", nullable=false, updatable=true, insertable=true)  )
	private List<Agente> agenti;
	@OneToMany(mappedBy = "operatore")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<Appuntamento> appuntamenti;
	@Transient
	private Long attesa;
	
	@Transient
	private String phone;
	
	public String getPhone() {
		return phone;
	}
	public void setPhone(String phone) {
		this.phone = phone;
	}

	@OneToMany(mappedBy = "utente")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<AutoContatto> autoContatti;
	@Transient
	private Boolean blocca = Boolean.FALSE;
	private Boolean cambioPassword = Boolean.FALSE;
	@ManyToMany(fetch = FetchType.LAZY)
	@JoinTable(name = ConstantsGlobal.tp + "campagne_utente", joinColumns=@JoinColumn(name="utente_id", nullable=false, updatable=true, insertable=true), inverseJoinColumns=@JoinColumn(name="campagna_id", nullable=false, updatable=true, insertable=true)  )
	private List<Campagna> campagne;
	//@OneToMany
	//@JoinColumn(name = "utente", nullable = true, updatable=true, insertable=true)
	@OneToMany(mappedBy = "utente")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<Cdr> cdr = new ArrayList<Cdr>();
	private String cognome;
	@OneToMany(mappedBy = "utente")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<Colonna> colonne;
	@OneToMany(mappedBy = "proprietario")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<Contatto> contatti;
	@OneToMany(mappedBy = "operatore")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<Contratto> contratti;
	@OneToMany(mappedBy = "utente")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<Criterio> criteri;
	private Date data_creazione = ApplicationUtil.date();
	private Date data_fine;
	private Date data_inizio;
	private Date data_login;
	private Date data_logout;
	private Date data_password;
	@Column(columnDefinition = "text")
	private String dati_fiscali;
	@OneToMany(mappedBy = "user")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<Destinatario> destinatari;
	private Integer disattivaFunzioni = 0;
	private Integer disattivaMessaggistica = 0;
	@OneToMany(mappedBy = "utente")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<DocumentoMessaggio> documenti;
	private String email;
	@OneToMany(mappedBy = "utente")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<Email> emails;
	private boolean enabled = true;
	@OneToMany(mappedBy = "user", fetch = FetchType.LAZY)
	@OnDelete(action = OnDeleteAction.CASCADE)
	private Set<Estratto> estratti;
	@ManyToOne
	private FatturaEstratto fattura;
	
	@Column(columnDefinition = "text")
	private String folder;
	private Boolean promemoria = Boolean.FALSE;
	private Boolean generaEstratto = Boolean.FALSE;
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY) 
	private Long id;
	@ManyToOne
	private SipBuddies interno;
	private Boolean inverti = false;
	private String ip_bloccati;
	private String ip_consentiti;
	@OneToMany(mappedBy = "utente")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<Rmail> rmails;
	private Boolean limitaAzioniAgente = Boolean.FALSE;
	private Boolean limitaAzioniCoordinatore = Boolean.FALSE;
	private Boolean limitaAzioniOperatore = Boolean.FALSE;
	private Boolean limitaAzioniPartner = Boolean.FALSE;
	private Boolean limitaAzioniSupervisore = Boolean.FALSE;

	private Boolean limitaRicercaAgente = Boolean.FALSE;
	private Boolean limitaRicercaCoordinatore = Boolean.FALSE;
	private Boolean limitaRicercaOperatore = Boolean.FALSE;
	private Boolean limitaRicercaPartner = Boolean.FALSE;
	private Boolean limitaRicercaSupervisore = Boolean.FALSE;
	private String limitaCampagnaPartner = ConstantsMap.campagna0;
	private String limitaCampagnaSupervisore = ConstantsMap.campagna0;

	private String limitaPermessiOperatore = ConstantsMap.team0;
	private Boolean limitaDatiAgente = Boolean.FALSE;
	private Boolean limitaDatiCoordinatore = Boolean.FALSE;
	private Boolean limitaDatiOperatore = Boolean.FALSE;
	private Boolean limitaDatiPartner = Boolean.FALSE;
	private Boolean limitaDatiSupervisore = Boolean.FALSE;

	private Boolean limitaModificaCoordinatore = Boolean.FALSE;
	private Boolean limitaModificaSupervisore = Boolean.FALSE;
	private Boolean limitaModificaPartner = Boolean.FALSE;
	private Boolean limitaModificaAgente = Boolean.FALSE;
	private Boolean limitaModificaOperatore = Boolean.FALSE;
	@Transient
	private Boolean forceReload = false;
	@Transient
	private Integer attivoControl = 0;
	@Transient
	private Integer loginMultiplo = 0;
	@Transient
	private Boolean internoMultiplo = false;
	@Transient
	private Integer massimoUtenti = 0;
	@Transient
	private String browser;

	public Boolean getForceReload() {
		return forceReload;
	}
	public void setForceReload(Boolean forceReload) {
		this.forceReload = forceReload;
	}
	public Integer getAttivoControl() {
		return attivoControl;
	}
	public void setAttivoControl(Integer attivoControl) {
		this.attivoControl = attivoControl;
	}
	public String getBrowser() {
		return browser;
	}
	public void setBrowser(String browser) {
		this.browser = browser;
	}
	public Boolean getInternoMultiplo() {
		return internoMultiplo;
	}
	public void setInternoMultiplo(Boolean internoMultiplo) {
		this.internoMultiplo = internoMultiplo;
	}
	@OneToMany(mappedBy = "user", fetch = FetchType.LAZY)
	@OnDelete(action = OnDeleteAction.CASCADE)
	private Set<Marcatura> marcature;
	@OneToMany(mappedBy = "mittente")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<Messaggio> messaggi;
	private String nome;
	@Transient
	private String sessionId;
	@OneToMany(mappedBy = "operatore")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<Nota> note;
	private Boolean notificaLogin = Boolean.FALSE;
	private Boolean notificaMessaggi = Boolean.FALSE;
	@OneToMany(mappedBy = "utente")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<Notifica> notifiche;
	
	@OneToMany(mappedBy = "utente")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<Sondaggio> sondaggi;
	
	@ManyToMany(fetch = FetchType.LAZY)
	@JoinTable(name = ConstantsGlobal.tp + "notizia_utente", joinColumns=@JoinColumn(name="utente_id", nullable=false, updatable=true, insertable=true), inverseJoinColumns=@JoinColumn(name="notizia_id", nullable=false, updatable=true, insertable=true)  )
	private List<Notizia> notizie;
	
	@OneToMany(mappedBy = "user")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<Operazione> operazioni;
	@OneToMany
	@JoinColumn(name = "utente_id", nullable = true, updatable=true, insertable=true)
	private List<Opposizione> opposizioni = new ArrayList<Opposizione>();
	private Time ora_fine;
	private Time ora_inizio;
	private String password;
	@OneToMany(mappedBy = "user")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<Pausa> pause;
	private String permessiAgente = ApplicationUtil.ruoliValoriS1(false, true, true);
	private String permessiCoordinatore = ApplicationUtil.ruoliValoriS1(true, true, true);
	private String permessiOperatore = ApplicationUtil.ruoliValoriS2(false, false, false, false);
	private String permessiPartner = ApplicationUtil.ruoliValoriS2(true, true, true, true);
	private String permessiSupervisore = ApplicationUtil.ruoliValoriS2(true, true, true, true);
	@OneToMany(mappedBy = "operatore")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<Prenotazione> prenotazioni;
	@OneToMany(mappedBy = "operatore")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<StoriaPrenotazione> prenotazioni_storia;
	@OneToMany(mappedBy = "utente")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<StoriaPrenotazione> prenotazioni_storia_utente;
	
	@ManyToMany(fetch = FetchType.LAZY)
	@JoinTable(name = ConstantsGlobal.tp + "user_ruolo", joinColumns=@JoinColumn(name="user_id", nullable=false, updatable=true, insertable=true), inverseJoinColumns=@JoinColumn(name="ruoli_id", nullable=false, updatable=true, insertable=true)  )
	private List<Ruolo> ruoli = new ArrayList<Ruolo>();
	@Transient
	private Boolean ruoloAgente;
	@Transient
	private Boolean ruoloCoordinatore;
	@Transient
	private Boolean ruoloEstratto;
	@Transient
	private Boolean ruoloOperatore;
	@Transient
	private Boolean ruoloPannello;
	@Transient
	private Boolean ruoloPartner;
	@Transient
	private Boolean ruoloRete;
	@Transient
	private Boolean ruoloSitter;
	@Transient
	private Boolean ruoloSupervisore;
	
	private Boolean pannelloOperatore = Boolean.TRUE;
	private Boolean pannelloSupervisore = Boolean.TRUE;
	
	@Transient
	private Boolean ruoloChat;
	
	private String secret;
	@ManyToMany(fetch = FetchType.LAZY)
	@JoinTable(name = ConstantsGlobal.tp + "sitter_utente", joinColumns=@JoinColumn(name="utente_id", nullable=false, updatable=true, insertable=true), inverseJoinColumns=@JoinColumn(name="sitter_id", nullable=false, updatable=true, insertable=true)  )
	private List<Utente> sitterMaster;
	@ManyToMany(fetch = FetchType.LAZY)
	@JoinTable(name = ConstantsGlobal.tp + "sitter_utente", joinColumns=@JoinColumn(name="sitter_id", nullable=false, updatable=true, insertable=true), inverseJoinColumns=@JoinColumn(name="utente_id", nullable=false, updatable=true, insertable=true)  )
	private List<Utente> sitterSlave;
	
	@OneToMany(mappedBy = "proprietario")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<StoriaContatto> storia_contatti;
	@OneToMany(mappedBy = "operatore")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<StoriaAppuntamento> storia_operatore_appuntamenti;
	@OneToMany(mappedBy = "operatore")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<StoriaContatto> storia_operatore_contatti;
	@OneToMany(mappedBy = "operatore")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<StoriaContratto> storia_operatore_contratti;
	@OneToMany(mappedBy = "operatore")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<StoriaNota> storia_operatore_note;
	@OneToMany(mappedBy = "utente")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<StoriaAppuntamento> storia_utente_appuntamenti;
	@OneToMany(mappedBy = "utente")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<StoriaContratto> storia_utente_contratti;
	@OneToMany(mappedBy = "utente")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<StoriaNota> storia_utente_note;
	@ManyToMany(fetch = FetchType.LAZY)
	@JoinTable(name = ConstantsGlobal.tp + "user_team", joinColumns=@JoinColumn(name="user_id", nullable=false, updatable=true, insertable=true), inverseJoinColumns=@JoinColumn(name="team_id", nullable=false, updatable=true, insertable=true)  )
	private List<Team> team;
	@ManyToMany(fetch = FetchType.LAZY)
	@JoinTable(name = ConstantsGlobal.tp + "supervisore_team", joinColumns=@JoinColumn(name="supervisore_id", nullable=false, updatable=true, insertable=true), inverseJoinColumns=@JoinColumn(name="team_id", nullable=false, updatable=true, insertable=true)  )
	private List<Team> teams;
	@OneToMany(mappedBy = "utente")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<Telecom> telecoms;
	
	@Transient
	private Integer tempoPassword; 
	@ManyToOne
	private SipBuddies trunk; 
	@OneToMany
	@JoinColumn(name = "utente_id", nullable = true, updatable=true, insertable=true)
	private List<Appuntamento> uappuntamenti = new ArrayList<Appuntamento>();
	@OneToMany
	@JoinColumn(name = "utente_id", nullable = true, updatable=true, insertable=true)
	private List<Contatto> ucontatti = new ArrayList<Contatto>();
	@OneToMany
	@JoinColumn(name = "utente_id", nullable = true, updatable=true, insertable=true)
	private List<Contratto> ucontratti = new ArrayList<Contratto>();

	@OneToMany
	@JoinColumn(name = "utente_id", nullable = true, updatable=true, insertable=true)
	private List<Nota> unote = new ArrayList<Nota>();

	@OneToMany
	@JoinColumn(name = "utente_id", nullable = true, updatable=true, insertable=true)
	private List<Prenotazione> uprenotazioni = new ArrayList<Prenotazione>();
	@Column(nullable=false, unique=true)
	private String username;
	private Boolean visualizzaStorico = Boolean.TRUE;
	private Long week_fine;
	private Long week_inizio;
	@ManyToMany(fetch = FetchType.LAZY)
	@JoinTable(name = ConstantsGlobal.tp + "supervisore_zona", joinColumns=@JoinColumn(name="supervisore_id", nullable=false, updatable=true, insertable=true), inverseJoinColumns=@JoinColumn(name="zone_id", nullable=false, updatable=true, insertable=true)  )
	private List<Zona> zone;

	public int compareTo(Utente o) {
		if(id != null && o.id != null) {
			return id.compareTo(o.id);
		} else if(id != null && o.id == null) {
			return -1;
		} else if(id == null && o.id != null) {
			return 1;
		} else {
			return 0;
		}
	}
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Utente other = (Utente) obj;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		if (sessionId == null) {
			if (other.sessionId != null)
				return false;
		} else if (!sessionId.equals(other.sessionId))
			return false;
		return true;
	}
	public void esterno(){
		this.setFolder(null);
		this.setTrunk(null);
	}
	public void estratto(){
		this.setFattura(null);
		this.setDati_fiscali(null);
	}
	public void fattura(){
		this.setDati_fiscali(null);
	}
	public Integer getAbilitaNumero() {
		return (isLocale() && interno.getTastierino() != null) ? interno.getTastierino() : ConstantsMap.t0;
	}
	public Set<Accesso> getAccesso() {
		return accesso;
	}
	public Set<Acconto> getAcconti() {
		return acconti;
	}
	public List<Agente> getAgenti() {
		return agenti;
	}
	public List<Appuntamento> getAppuntamenti() {
		return appuntamenti;
	}
	public Long getAttesa() {
		return attesa;
	}
	public Set<GrantedAuthority> getAuthorities() {
		Set<GrantedAuthority> authorities = new LinkedHashSet<GrantedAuthority>();
		authorities.addAll(getRuoliAll());
		return authorities;
	}
	public List<AutoContatto> getAutoContatti() {
		return autoContatti;
	}
	public Boolean getBlocca() {
		return blocca;
	}
	public Boolean getCambioPassword() {
		return ApplicationUtil.isTrue(cambioPassword);
	}
	public String getCambioPasswordString() {
		if(ApplicationUtil.isTrue(getCambioPassword()))
			return ConstantsLanguage.SI.stampa();
		else
			return ConstantsLanguage.NO.stampa();
	}
	public Boolean getCambioProgrammatoPassword() {
		if(ApplicationUtil.isTrue(getRuoloSitter())) {
			return false;
		} else {
			Integer a = getTempoPassword();
			boolean b = false;
			if(ApplicationUtil.gtZero(a)) {
				int g = getGiorniRimanentiPassword();
				if(g >= 0) {
					b = true;
				} else {
					b = false;
				}
			} else {
				b = false;
			}
			return cambioPassword || b;
		}
	}
	public List<Campagna> getCampagne() {
		return campagne;
	}
	public List<Cdr> getCdr() {
		return cdr;
	}
	public String getChiamante() {
		return getDenominazione().replaceAll("[<>]+", "");
	}
	public String getCognome() {
		return ApplicationUtil.trimToEmpty(cognome);
	}
	public List<Colonna> getColonne() {
		return colonne;
	}
	public List<Contatto> getContatti() {
		return contatti;
	}
	public List<Contratto> getContratti() {
		return contratti;
	}
	public List<Criterio> getCriteri() {
		return criteri;
	}
	public Date getData_creazione() {
		return data_creazione;
	}
	public Date getData_fine() {
		return ApplicationUtil.aggiustaDataT(data_fine, false);
	}
	public Date getData_inizio() {
		return ApplicationUtil.aggiustaDataT(data_inizio, true);
	}
	public Date getData_login() {
		return data_login;
	}
	public Date getData_logout() {
		return data_logout;
	}
	public Date getData_password() {
		return data_password;
	}
	public String getDati_fiscali() {
		return dati_fiscali;
	}
	public String getDenominazione() {
		return ConstantsGlobal.usernameSeparator1 + getUsername() + ConstantsGlobal.usernameSeparator2 + (getStatoAttiva() ? "" : "*") + " " + getNome_completo();
	}
	public List<Destinatario> getDestinatari() {
		return destinatari;
	}
	public Integer getDisattivaFunzioni() {
		if(disattivaFunzioni==null)
			disattivaFunzioni = 0;
		return disattivaFunzioni;
	}
	public Integer getDisattivaMessaggistica() {
		if(disattivaMessaggistica==null)
			disattivaMessaggistica = 0;
		return disattivaMessaggistica;
	}
	public List<DocumentoMessaggio> getDocumenti() {
		return documenti;
	}
	public String getEmail() {
		return ApplicationUtil.lowerCaseEmpty(email);
	}
	public List<Email> getEmails() {
		return emails;
	}
	public Boolean getEmailValida() {
		return ApplicationUtil.isNotBlank(getEmail());
	}
	public String getEnabledString() {
		if(isEnabled())
			return ConstantsLanguage.SI.stampa();
		else
			return ConstantsLanguage.NO.stampa();
	}
	public Set<Estratto> getEstratti() {
		return estratti;
	}
	public FatturaEstratto getFattura() {
		return fattura;
	}
	
	public Boolean getPromemoria() {
		return promemoria;
	}
	public void setPromemoria(Boolean promemoria) {
		this.promemoria = promemoria;
	}
	public String getFolder() {
		return ApplicationUtil.trimToEmpty(folder);
	}
	public Boolean getGeneraEstratto() {
		if(generaEstratto == null)
			return Boolean.FALSE;
		return generaEstratto;
	}
	public String getGeneraEstrattoString() {
		if(ApplicationUtil.isTrue(getGeneraEstratto()))
			return ConstantsLanguage.SI.stampa();
		else
			return ConstantsLanguage.NO.stampa();
	}
	public Integer getGiorniRimanentiPassword() {
		Integer a = getTempoPassword();
		Date d = getData_password();
		if(d == null) {
			return 0;
		} else {
			Date data = ApplicationUtil.addDays(d, a);
			Long diff = TimeUnit.MILLISECONDS.toDays(ApplicationUtil.date().getTime()-data.getTime());
			return diff.intValue();
		}
	}
	public Long getId() {
		return id;
	}
	public String getIdentificativo() {
		Long idP = getId();
		return (idP != null && !getCambioProgrammatoPassword()) ? idP.toString() :"";
	}
	public Boolean getInterne() {
		return isLocale() && ApplicationUtil.isTrue(interno.getInterne()) ;
	}
	public SipBuddies getInterno() {
		return interno;
	}
	public String getInternoString() {
		return isLocale() ? getInterno().getName() : "";
	}
	public Boolean getInterurbane() {
		return isLocale() && ApplicationUtil.isTrue(interno.getInterurbane()) ;
	}
	public Boolean getInverti() {
		if(inverti == null) {
			inverti = false;
		}
		return inverti;
	}
	public String getIp_bloccati() {
		return ApplicationUtil.trimToNull(ip_bloccati);
	}
	public String getIp_consentiti() {
		return ApplicationUtil.trimToNull(ip_consentiti);
	}
	public List<Rmail> getRmails() {
		return rmails;
	}
	public Boolean getLimitaAzioniAgente() {
		if(limitaAzioniAgente == null) {
			limitaAzioniAgente = Boolean.FALSE;
		}
		return limitaAzioniAgente;
	}
	public Boolean getLimitaAzioniCoordinatore() {
		if(limitaAzioniCoordinatore == null) {
			limitaAzioniCoordinatore = Boolean.FALSE;
		}
		return limitaAzioniCoordinatore;
	}
	public Boolean getLimitaAzioniOperatore() {
		if(limitaAzioniOperatore == null) {
			limitaAzioniOperatore = Boolean.FALSE;
		}
		return limitaAzioniOperatore;
	}
	public Boolean getLimitaAzioniPartner() {
		if(limitaAzioniPartner == null) {
			limitaAzioniPartner = Boolean.FALSE;
		}
		return limitaAzioniPartner;
	}
	public Boolean getLimitaAzioniSupervisore() {
		if(limitaAzioniSupervisore == null) {
			limitaAzioniSupervisore = Boolean.FALSE;
		}
		return limitaAzioniSupervisore;
	}
	public String getLimitaCampagnaPartner() {
		if(limitaCampagnaPartner == null) {
			limitaCampagnaPartner = ConstantsMap.campagna0;
		}
		return limitaCampagnaPartner;
	}
	public Integer getLimitaCampagnaPartnerS() {
		String s = getLimitaCampagnaPartner();
		if(s != null) {
			if(s.equals(ConstantsMap.campagna0)) return ConstantsMap.value0;
			if(s.equals(ConstantsMap.campagna1)) return ConstantsMap.value1;
			if(s.equals(ConstantsMap.campagna2)) return ConstantsMap.value2;
			if(s.equals(ConstantsMap.campagna3)) return ConstantsMap.value3;
		}
		return ConstantsMap.value0;
	}
	public String getLimitaCampagnaSupervisore() {
		if(limitaCampagnaSupervisore == null) {
			limitaCampagnaSupervisore = ConstantsMap.campagna0;
		}
		return limitaCampagnaSupervisore;
	}

	public String getLimitaPermessiOperatore() {
		if(limitaPermessiOperatore == null) {
			limitaPermessiOperatore = ConstantsMap.team0;
		}
		return limitaPermessiOperatore;
	}
	public Integer getLimitaPermessiOperatoreS() {
		String s = getLimitaPermessiOperatore();
		if(s != null) {
			if(s.equals(ConstantsMap.team0)) return ConstantsMap.value0;
			if(s.equals(ConstantsMap.team1)) return ConstantsMap.value1;
			if(s.equals(ConstantsMap.team2)) return ConstantsMap.value2;
		}
		return ConstantsMap.value0;
	}

	public Integer getLimitaCampagnaSupervisoreS() {
		String s = getLimitaCampagnaSupervisore();
		if(s != null) {
			if(s.equals(ConstantsMap.campagna0)) return ConstantsMap.value0;
			if(s.equals(ConstantsMap.campagna1)) return ConstantsMap.value1;
			if(s.equals(ConstantsMap.campagna2)) return ConstantsMap.value2;
			if(s.equals(ConstantsMap.campagna3)) return ConstantsMap.value3;
		}
		return ConstantsMap.value0;
	}
	public Boolean getLimitaDatiAgente() {
		if(limitaDatiAgente == null) {
			limitaDatiAgente = Boolean.FALSE;
		}
		return limitaDatiAgente;
	}
	public Boolean getLimitaDatiCoordinatore() {
		if(limitaDatiCoordinatore == null) {
			limitaDatiCoordinatore = Boolean.FALSE;
		}
		return limitaDatiCoordinatore;
	}
	public Boolean getLimitaDatiOperatore() {
		if(limitaDatiOperatore == null) {
			limitaDatiOperatore = Boolean.FALSE;
		}
		return limitaDatiOperatore;
	}
	public Boolean getLimitaDatiPartner() {
		if(limitaDatiPartner == null) {
			limitaDatiPartner = Boolean.FALSE;
		}
		return limitaDatiPartner;
	}
	public Boolean getLimitaDatiSupervisore() {
		if(limitaDatiSupervisore == null) {
			limitaDatiSupervisore = Boolean.FALSE;
		}
		return limitaDatiSupervisore;
	}
	public Boolean getLimitaModificaCoordinatore() {
		if(limitaModificaCoordinatore == null) {
			limitaModificaCoordinatore = Boolean.FALSE;
		}
		return limitaModificaCoordinatore;
	}
	public Boolean getLimitaModificaSupervisore() {
		if(limitaModificaSupervisore == null) {
			limitaModificaSupervisore = Boolean.FALSE;
		}
		return limitaModificaSupervisore;
	}

	public Boolean getLimitaModificaAgente() {
		if(limitaModificaAgente == null) {
			limitaModificaAgente = Boolean.FALSE;
		}
		return limitaModificaAgente;
	}

	public Boolean getLimitaModificaOperatore() {
		if(limitaModificaOperatore == null) {
			limitaModificaOperatore = Boolean.FALSE;
		}
		return limitaModificaOperatore;
	}

	public Boolean getLimitaModificaPartner() {
		if(limitaModificaPartner == null) {
			limitaModificaPartner = Boolean.FALSE;
		}
		return limitaModificaPartner;
	}

	public Integer getLoginMultiplo() {
		return loginMultiplo;
	}
	public Set<Marcatura> getMarcature() {
		return marcature;
	}
	public List<Messaggio> getMessaggi() {
		return messaggi;
	}

	
	public String getNome() {
		return ApplicationUtil.trimToEmpty(nome);
	}
	public String getNome_completo() {
		return ApplicationUtil.denominazione(cognome, nome);
	}
	
	public List<Nota> getNote() {
		return note;
	}
	public Boolean getNotificaLogin() {
		return notificaLogin;
	}
	public Boolean getNotificaMessaggi() {
		return notificaMessaggi;
	}
	public List<Notifica> getNotifiche() {
		return notifiche;
	}
	public List<Notizia> getNotizie() {
		return notizie;
	}
	
	public List<Operazione> getOperazioni() {
		return operazioni;
	}
	public Boolean getOpposizione() {
		return isEsterno() && ApplicationUtil.isTrue(interno.getOpposizione());
	}
	public List<Opposizione> getOpposizioni() {
		return opposizioni;
	}
	public Date getOra_fine() {
		return ApplicationUtil.timeToDate(ora_fine);
	}
	public Date getOra_inizio() {
		return ApplicationUtil.timeToDate(ora_inizio);
	}
	public Boolean getOtp() {
		return ApplicationUtil.isNotBlank(secret);
	}
	public ParameterTrunk getParameter() {
		return new ParameterTrunk(trunk);
	}
	
	public String getCallerid() {
		if(isLocale()) {
			return ApplicationUtil.trimToEmpty(interno.getCallers()); 
		}
		return "";
	}
	
	public String getPassword() {
		return password;
	}
	public String getPasswordString() {
		return isLocale() ? getInterno().getPassword() : "";
	}
	public List<Pausa> getPause() {
		return pause;
	}
	public String getPermessiAgente() {
		return permessiAgente;
	}
	public List<String> getPermessiAgenteS() {
		return ApplicationUtil.token(permessiAgente, ConstantsGlobal.colonSeparator);
	}
	public String getPermessiCoordinatore() {
		return permessiCoordinatore;
	}
	public List<String> getPermessiCoordinatoreS() {
		return ApplicationUtil.token(permessiCoordinatore, ConstantsGlobal.colonSeparator);
	}
	public String getPermessiOperatore() {
		return permessiOperatore;
	}
	public List<String> getPermessiOperatoreS() {
		return ApplicationUtil.token(permessiOperatore, ConstantsGlobal.colonSeparator);
	}
	public String getPermessiPartner() {
		return permessiPartner;
	}
	public List<String> getPermessiPartnerS() {
		return ApplicationUtil.token(permessiPartner, ConstantsGlobal.colonSeparator);
	}
	public String getPermessiSupervisore() {
		return permessiSupervisore;
	}
	public List<String> getPermessiSupervisoreS() {
		return ApplicationUtil.token(permessiSupervisore, ConstantsGlobal.colonSeparator);
	}
	public String getPrefisso() {
		return isLocale() ? ApplicationUtil.trimToEmpty(interno.getAdd_prefisso()) : "";
	}
	
	public List<Prenotazione> getPrenotazioni() {
		return prenotazioni;
	}
	public List<StoriaPrenotazione> getPrenotazioni_storia() {
		return prenotazioni_storia;
	}
	public List<StoriaPrenotazione> getPrenotazioni_storia_utente() {
		return prenotazioni_storia_utente;
	}
	public Boolean getLoggato() {
		return isLocale() ? interno.getLoggato() : false;
	}
	public Boolean getPronto() {
		return isLocale() ? interno.getPronto() : false;
	}
	
	public Boolean getRegistrazione() {
		return isLocale() && ApplicationUtil.isTrue(interno.getRegistrazione());
	}

	public String getPhoneS() {
		if(isLocale()) {
			String p = getInterno().getPhone();
			if(!p.equals(ConstantsMap.phone1))
				return p ;
		}
		return "";
	}
	public boolean getRiesitaOperatore() {
		List<String> s = ApplicationUtil.token(permessiOperatore, ConstantsGlobal.colonSeparator);
		if(ApplicationUtil.notEmpty(s) && s.contains(ConstantsGlobal.contatti)) {
			return true;
		}
		return false;
	}
	public boolean getRiesitaSupervisore() {
		List<String> s = ApplicationUtil.token(permessiSupervisore, ConstantsGlobal.colonSeparator);
		if(ApplicationUtil.notEmpty(s) && s.contains(ConstantsGlobal.contatti)) {
			return true;
		}
		return false;
	}
	public List<Ruolo> getRuoli() {
		return ruoli;
	}
	public List<Ruolo> getRuoliAll() {
		List<Ruolo> ruols = new ArrayList<Ruolo>();
		ruols.add(new Ruolo(ConstantsRole.ROLE_ANYBODY));
		if(isLocale()) {
			ruols.add(new Ruolo(ConstantsRole.ROLE_TELEFONICO));
		}
		if(!getCambioProgrammatoPassword()) {
			if(ApplicationUtil.eq(getDisattivaMessaggistica(), 0)) {
				ruols.add(new Ruolo(ConstantsRole.ROLE_MESSAGGISTICA));
			}
			if(ApplicationUtil.isTrue(getRuoloEstratto()) && ApplicationUtil.isTrue(getGeneraEstratto())) {
				ruols.add(new Ruolo(ConstantsRole.ROLE_ESTRATTO));
			}
			if(ApplicationUtil.isTrue(getRuoloPannello())) {
				if(isOperatore() || isSupervisore()) {
					ruols.add(new Ruolo(ConstantsRole.ROLE_PANNELLO));
				}
				if(ApplicationUtil.isTrue(getRuoloOperatore())) {
					List<String> psl = getPermessiOperatoreS();
					if(ApplicationUtil.notEmpty(psl)) {
						ruols.add(new Ruolo(ConstantsRole.ROLE_OPERATORE));
						for(String ps: psl) {
							ruols.add(new Ruolo(ConstantsRole.ROLE_OPERATORE+ ps));
						}
					}
				}
				if(ApplicationUtil.isTrue(getRuoloSupervisore())) {
					List<String> psl = getPermessiSupervisoreS();
					if(ApplicationUtil.notEmpty(psl)) {
						ruols.add(new Ruolo(ConstantsRole.ROLE_SUPERVISORE));
						for(String ps: psl) {
							ruols.add(new Ruolo(ConstantsRole.ROLE_SUPERVISORE+ ps));
						}
					}
				}
			}
			if(ApplicationUtil.isTrue(getRuoloAgente())) {
				List<String> psl = getPermessiAgenteS();
				if(ApplicationUtil.notEmpty(psl)) {
					ruols.add(new Ruolo(ConstantsRole.ROLE_AGENTE));
					for(String ps: psl) {
						ruols.add(new Ruolo(ConstantsRole.ROLE_AGENTE+ ps));
					}
				}
				if(ApplicationUtil.isTrue(getRuoloRete())) {
					if(ApplicationUtil.notEmpty(psl)) {
						ruols.add(new Ruolo(ConstantsRole.ROLE_RETE));
						for(String ps: psl) {
							ruols.add(new Ruolo(ConstantsRole.ROLE_RETE+ ps));
						}
					}
				}
			}
			if(ApplicationUtil.isTrue(getRuoloCoordinatore())) {
				List<String> psl = getPermessiCoordinatoreS();
				if(ApplicationUtil.notEmpty(psl)) {
					ruols.add(new Ruolo(ConstantsRole.ROLE_COORDINATORE));
					for(String ps: psl) {
						ruols.add(new Ruolo(ConstantsRole.ROLE_COORDINATORE+ ps));
					}
				}
			}
			if(ApplicationUtil.isTrue(getRuoloPartner())) {
				List<String> psl = getPermessiPartnerS();
				if(ApplicationUtil.notEmpty(psl)) {
					ruols.add(new Ruolo(ConstantsRole.ROLE_PARTNER));
					for(String ps: psl) {
						ruols.add(new Ruolo(ConstantsRole.ROLE_PARTNER+ ps));
					}
				}
			}
			List<Ruolo> statici = getRuoli();
			if(ApplicationUtil.notEmpty(statici)) {
				this.setRuoloChat(true);
			} else {
				this.setRuoloChat(false);
			}
			ruols.addAll(statici);
			if(ApplicationUtil.notEmpty(ruols)) {
				for (String r : ConstantsRole.ROLES.keySet()) {
					if(ruols.contains(ConstantsRole.ROLES.get(r)) && getVisualizzaStorico()) {
						ruols.add(new Ruolo(r + "_" + ConstantsRole.STORICO));
					}
				}
			}
		}
		return ruols;
	}
	public Boolean getRuoloAgente() {
		return ruoloAgente;
	}
	public Boolean getRuoloCoordinatore() {
		return ruoloCoordinatore;
	}
	public Boolean getRuoloChat() {
		return ruoloChat;
	}
	public Boolean getRuoloEstratto() {
		return ruoloEstratto;
	}
	public Boolean getRuoloOperatore() {
		return ruoloOperatore;
	}
	public Boolean getRuoloPannello() {
		return ruoloPannello;
	}
	public Boolean getRuoloPartner() {
		return ruoloPartner;
	}
	public Boolean getRuoloRete() {
		return ruoloRete;
	}
	public Boolean getRuoloSitter() {
		return ruoloSitter;
	}
	public Boolean getRuoloSupervisore() {
		return ruoloSupervisore;
	}
	public String getSecret() {
		return secret;
	}
	public List<Utente> getSitterMaster() {
		return sitterMaster;
	}
	public List<Utente> getSitterSlave() {
		return sitterSlave;
	}

	public Boolean getStatoAttiva() {
		return ApplicationUtil.calcolaIntervallo(getInverti(), getData_inizio(), getData_fine(), getWeek_inizio(), getWeek_fine(), getTime_inizio(), getTime_fine());
	}
	public String getStatoAttivaT() {
		return getStatoAttiva() ? ConstantsLanguage.SI.stampa() : ConstantsLanguage.NO.stampa();
	}
	public List<StoriaContatto> getStoria_contatti() {
		return storia_contatti;
	}
	public List<StoriaAppuntamento> getStoria_operatore_appuntamenti() {
		return storia_operatore_appuntamenti;
	}
	public List<StoriaContatto> getStoria_operatore_contatti() {
		return storia_operatore_contatti;
	}
	public List<StoriaContratto> getStoria_operatore_contratti() {
		return storia_operatore_contratti;
	}
	public List<StoriaNota> getStoria_operatore_note() {
		return storia_operatore_note;
	}
	public List<StoriaAppuntamento> getStoria_utente_appuntamenti() {
		return storia_utente_appuntamenti;
	}
	public List<StoriaContratto> getStoria_utente_contratti() {
		return storia_utente_contratti;
	}
	public List<StoriaNota> getStoria_utente_note() {
		return storia_utente_note;
	}
	public List<Team> getTeam() {
		return team;
	}
	public List<Team> getTeams() {
		return teams;
	}
	public List<Telecom> getTelecoms() {
		return telecoms;
	}
	
	public Integer getTempoPassword() {
		return tempoPassword;
	}
	public Time getTime_fine() {
		return ora_fine;
	}
	public Time getTime_inizio() {
		return ora_inizio;
	}
	public SipBuddies getTrunk() {
		return trunk;
	}
	public List<Appuntamento> getUappuntamenti() {
		return uappuntamenti;
	}
	public List<Contatto> getUcontatti() {
		return ucontatti;
	}
	public List<Contratto> getUcontratti() {
		return ucontratti;
	}
	public String getUsername() {
		return ApplicationUtil.trimToEmpty(username);
	}
	
	public Boolean getVisualizzaStorico() {
		if(visualizzaStorico == null) {
			visualizzaStorico = Boolean.TRUE;
		}
		return visualizzaStorico;
	}
	public Long getWeek_fine() {
		return week_fine;
	}
	public Long getWeek_inizio() {
		return week_inizio;
	}
	public List<Zona> getZone() {
		return zone;
	}
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		result = prime * result + ((sessionId == null) ? 0 : sessionId.hashCode());
		return result;
	}
	
	public boolean isAdmin() {
		return isLocale() && interno.getInterne();
	}
	public boolean isAccountNonExpired() {
		if(!superusers.contains(username)) {
			return !getLimite();
		}
		return true;
	}
	public boolean isAccountNonLocked() {
		return !getBlocca() && getStatoAttiva();
	}
	public boolean isCredentialsNonExpired() {
		if(!superusers.contains(username)) {
			boolean ok1 = true;
			boolean ok2 = true;
			if(ApplicationUtil.eq(getLoginMultiplo(), 3)) {
				ok1 = !getMonitor();
			}
			if(!ApplicationUtil.isTrue(getInternoMultiplo())) {
				ok2 = !getRegistrato();
			}
			return ok1 && ok2;
		}
		return true;
	}

	public Boolean getMonitor() {
		return Chiamate.getCodaOperatori().hasOperatore(this);
	}

	public Boolean getRegistrato() {
		return getLoggato() && getBarra();
	}

	public Boolean getLimite() {
		Integer utenti = getMassimoUtenti();
		return ApplicationUtil.gtZero(utenti) && getBarra() && Chiamate.getCodaInterni().registratiSize() > utenti;
	}
	
	public Boolean getBarra() {
		return isLocale() && ApplicationUtil.phone(getInterno().getPhoneS(), getBrowser());
	}

	public boolean isEnabled() {
		return enabled;
	}
	public boolean isEsterno() {
		return isLocale() && trunk != null;
	}
	public boolean isLocale() {
		return interno != null;
	}
	
	public boolean getSegreteria() {
		return isLocale() && ApplicationUtil.isTrue(interno.getSegreteria());
	}
	
	public void setAccesso(Set<Accesso> accesso) {
		this.accesso = accesso;
	}
	public void setAcconti(Set<Acconto> acconti) {
		this.acconti = acconti;
	}
	public void setAgenti(List<Agente> agenti) {
		this.agenti = agenti;
	}
	public void setAppuntamenti(List<Appuntamento> appuntamenti) {
		this.appuntamenti = appuntamenti;
	}
	public void setAttesa(Long attesa) {
		this.attesa = attesa;
	}
	public void setAutoContatti(List<AutoContatto> autoContatti) {
		this.autoContatti = autoContatti;
	}
	public void setBlocca(Boolean blocca) {
		this.blocca = blocca;
	}
	public boolean isShell() {
		boolean a1 = getIp_bloccati() != null || getIp_consentiti() != null;
		boolean a2 = ApplicationUtil.isNotBlank(secret);
		return superusers.contains(username) && (a1 || a2);
	}
	
	@Transient
	private List<String> superusers = new ArrayList<String>();
	
	public Boolean getSelezionabile() {
		return isRuoloA() || (!isRuoloA() && !isShell());
	}
	@Transient
	private boolean ruoloA = false;
	public boolean isRuoloA() {
		return ruoloA;
	}
	
	public List<String> getSuperusers() {
		return superusers;
	}
	public void setSuperusers(List<String> superusers) {
		this.superusers = superusers;
	}
	
	public void impostaRuoloA(boolean ruoloA, List<String> superusers) {
		this.ruoloA = ruoloA;
		this.superusers = superusers;
	}
	
	public void setRuoloA(boolean ruoloA) {
		this.ruoloA = ruoloA;
	}
	public void setCambioPassword(Boolean cambioPassword) {
		this.cambioPassword = cambioPassword;
	}
	public void setCampagne(List<Campagna> campagne) {
		this.campagne = campagne;
	}
	public void setCdr(List<Cdr> cdr) {
		this.cdr = cdr;
	}
	public void setCognome(String cognome) {
		this.cognome = ApplicationUtil.normalizeToNull(cognome);
	}
	public void setColonne(List<Colonna> colonne) {
		this.colonne = colonne;
	}
	public void setContatti(List<Contatto> contatti) {
		this.contatti = contatti;
	}
	public void setContratti(List<Contratto> contratti) {
		this.contratti = contratti;
	}
	public void setCriteri(List<Criterio> criteri) {
		this.criteri = criteri;
	}
	public void setData_creazione(Date data_creazione) {
		this.data_creazione = data_creazione;
	}
	public void setData_fine(Date data_fine) {
		this.data_fine = ApplicationUtil.aggiustaDataT(data_fine, false);
	}
	public void setData_inizio(Date data_inizio) {
		this.data_inizio = ApplicationUtil.aggiustaDataT(data_inizio, true);
	}
	public void setData_login(Date data_login) {
		this.data_login = data_login;
	}
	public void setData_logout(Date data_logout) {
		this.data_logout = data_logout;
	}
	public void setData_password(Date data_password) {
		this.data_password = data_password;
	}
	public void setDati_fiscali(String dati_fiscali) {
		this.dati_fiscali = dati_fiscali;
	}
	public void setDestinatari(List<Destinatario> destinatari) {
		this.destinatari = destinatari;
	}
	public void setDisattivaFunzioni(Integer disattivaFunzioni) {
		this.disattivaFunzioni = disattivaFunzioni;
	}
	public void setDisattivaMessaggistica(Integer disattivaMessaggistica) {
		this.disattivaMessaggistica = disattivaMessaggistica;
	}
	public void setDocumenti(List<DocumentoMessaggio> documenti) {
		this.documenti = documenti;
	}
	public void setEmail(String email) {
		this.email = ApplicationUtil.normalizeToLowerNull(email);
	}
	public void setEmails(List<Email> emails) {
		this.emails = emails;
	}
	public void setEnabled(boolean enabled) {
		this.enabled = enabled;
	}
	public void setEstratti(Set<Estratto> estratti) {
		this.estratti = estratti;
	}
	public void setFattura(FatturaEstratto fattura) {
		this.fattura = fattura;
	}
	
	public void setFolder(String folder) {
		this.folder = ApplicationUtil.trimToEmpty(folder);
	}
	public void setGeneraEstratto(Boolean generaEstratto) {
		this.generaEstratto = generaEstratto;
	}
	public void setId(Long id) {
		this.id = id;
	}
	public void setInterno(SipBuddies interno) {
		this.interno = interno;
	}
	public void setInverti(Boolean inverti) {
		this.inverti = inverti;
	}
	public void setIp_bloccati(String ip_bloccati) {
		this.ip_bloccati = ApplicationUtil.aggiustaLista(true, ConstantsGlobal.colonSeparator, ip_bloccati);
	}
	public void setIp_consentiti(String ip_consentiti) {
		this.ip_consentiti = ApplicationUtil.aggiustaLista(true, ConstantsGlobal.colonSeparator, ip_consentiti);
	}
	public void setRmails(List<Rmail> rmails) {
		this.rmails = rmails;
	}
	public void setLimitaAzioniAgente(Boolean limitaAzioniAgente) {
		this.limitaAzioniAgente = limitaAzioniAgente;
	}
	public void setLimitaAzioniCoordinatore(Boolean limitaAzioniCoordinatore) {
		this.limitaAzioniCoordinatore = limitaAzioniCoordinatore;
	}
	public void setLimitaAzioniOperatore(Boolean limitaAzioniOperatore) {
		this.limitaAzioniOperatore = limitaAzioniOperatore;
	}
	public void setLimitaAzioniPartner(Boolean limitaAzioniPartner) {
		this.limitaAzioniPartner = limitaAzioniPartner;
	}
	public void setLimitaAzioniSupervisore(Boolean limitaAzioniSupervisore) {
		this.limitaAzioniSupervisore = limitaAzioniSupervisore;
	}
	public void setLimitaCampagnaPartner(String limitaCampagnaPartner) {
		this.limitaCampagnaPartner = limitaCampagnaPartner;
	}
	public void setLimitaCampagnaSupervisore(String limitaCampagnaSupervisore) {
		this.limitaCampagnaSupervisore = limitaCampagnaSupervisore;
	}
	public void setLimitaPermessiOperatore(String limitaPermessiOperatore) {
		this.limitaPermessiOperatore = limitaPermessiOperatore;
	}
	public void setLimitaDatiAgente(Boolean limitaDatiAgente) {
		this.limitaDatiAgente = limitaDatiAgente;
	}
	public void setLimitaDatiCoordinatore(Boolean limitaDatiCoordinatore) {
		this.limitaDatiCoordinatore = limitaDatiCoordinatore;
	}
	public void setLimitaDatiOperatore(Boolean limitaDatiOperatore) {
		this.limitaDatiOperatore = limitaDatiOperatore;
	}
	public void setLimitaDatiPartner(Boolean limitaDatiPartner) {
		this.limitaDatiPartner = limitaDatiPartner;
	}
	public void setLimitaDatiSupervisore(Boolean limitaDatiSupervisore) {
		this.limitaDatiSupervisore = limitaDatiSupervisore;
	}
	public void setLimitaModificaCoordinatore(Boolean limitaModificaCoordinatore) {
		this.limitaModificaCoordinatore = limitaModificaCoordinatore;
	}
	public void setLimitaModificaSupervisore(Boolean limitaModificaSupervisore) {
		this.limitaModificaSupervisore = limitaModificaSupervisore;
	}

	public void setLimitaModificaAgente(Boolean limitaModificaAgente) {
		this.limitaModificaAgente = limitaModificaAgente;
	}
	public void setLimitaModificaOperatore(Boolean limitaModificaOperatore) {
		this.limitaModificaOperatore = limitaModificaOperatore;
	}
	public void setLimitaModificaPartner(Boolean limitaModificaPartner) {
		this.limitaModificaPartner = limitaModificaPartner;
	}
	public void setLoginMultiplo(Integer loginMultiplo) {
		this.loginMultiplo = loginMultiplo;
	}
	public void setMarcature(Set<Marcatura> marcature) {
		this.marcature = marcature;
	}
	public void setMessaggi(List<Messaggio> messaggi) {
		this.messaggi = messaggi;
	}
	
	public void setNome(String nome) {
		this.nome = ApplicationUtil.normalizeToNull(nome);
	}
	public void setNote(List<Nota> note) {
		this.note = note;
	}
	public void setNotificaLogin(Boolean notificaLogin) {
		this.notificaLogin = notificaLogin;
	}
	public void setNotificaMessaggi(Boolean notificaMessaggi) {
		this.notificaMessaggi = notificaMessaggi;
	}
	public void setNotifiche(List<Notifica> notifiche) {
		this.notifiche = notifiche;
	}
	public void setNotizie(List<Notizia> notizie) {
		this.notizie = notizie;
	}
	
	public void setOperazioni(List<Operazione> operazioni) {
		this.operazioni = operazioni;
	}
	public void setOpposizioni(List<Opposizione> opposizioni) {
		this.opposizioni = opposizioni;
	}
	public void setOra_fine(Date ora_fine) {
		this.ora_fine = ApplicationUtil.dateToTime(ora_fine);
	}
	public void setOra_inizio(Date ora_inizio) {
		this.ora_inizio = ApplicationUtil.dateToTime(ora_inizio);
	}
	public void setPassword(String password) {
		this.password = password;
	}
	public void setPause(List<Pausa> pause) {
		this.pause = pause;
	}
	public void setPermessiAgente(String permessiAgente) {
		this.permessiAgente = permessiAgente;
	}
	public void setPermessiAgenteS(List<String> valoreMultiplo) {
		permessiAgente = ApplicationUtil.detoken(valoreMultiplo, ConstantsGlobal.colonSeparator);
	}
	public void setPermessiCoordinatore(String permessiCoordinatore) {
		this.permessiCoordinatore = permessiCoordinatore;
	}
	public void setPermessiCoordinatoreS(List<String> valoreMultiplo) {
		permessiCoordinatore = ApplicationUtil.detoken(valoreMultiplo, ConstantsGlobal.colonSeparator);
	}
	public void setPermessiOperatore(String permessiOperatore) {
		this.permessiOperatore = permessiOperatore;
	}
	public void setPermessiOperatoreS(List<String> valoreMultiplo) {
		permessiOperatore = ApplicationUtil.detoken(valoreMultiplo, ConstantsGlobal.colonSeparator);
	}
	public void setPermessiPartner(String permessiPartner) {
		this.permessiPartner = permessiPartner;
	}
	public void setPermessiPartnerS(List<String> valoreMultiplo) {
		permessiPartner = ApplicationUtil.detoken(valoreMultiplo, ConstantsGlobal.colonSeparator);
	}
	public void setPermessiSupervisore(String permessiSupervisore) {
		this.permessiSupervisore = permessiSupervisore;
	}
	public void setPermessiSupervisoreS(List<String> valoreMultiplo) {
		permessiSupervisore = ApplicationUtil.detoken(valoreMultiplo, ConstantsGlobal.colonSeparator);
	}
	public void setPrenotazioni(List<Prenotazione> prenotazioni) {
		this.prenotazioni = prenotazioni;
	}
	public void setPrenotazioni_storia(List<StoriaPrenotazione> prenotazioni_storia) {
		this.prenotazioni_storia = prenotazioni_storia;
	}
	public void setPrenotazioni_storia_utente(List<StoriaPrenotazione> prenotazioni_storia_utente) {
		this.prenotazioni_storia_utente = prenotazioni_storia_utente;
	}

	public void setRuoli(List<Ruolo> ruoli) {
		this.ruoli = ruoli;
	}
	public void setRuoloAgente(Boolean ruoloAgente) {
		this.ruoloAgente = ruoloAgente;
	}
	public void setRuoloCoordinatore(Boolean ruoloCoordinatore) {
		this.ruoloCoordinatore = ruoloCoordinatore;
	}
	public void setRuoloEstratto(Boolean ruoloEstratto) {
		this.ruoloEstratto = ruoloEstratto;
	}
	public void setRuoloChat(Boolean ruoloChat) {
		this.ruoloChat = ruoloChat;
	}
	public void setRuoloOperatore(Boolean ruoloOperatore) {
		this.ruoloOperatore = ruoloOperatore;
	}
	public void setRuoloPannello(Boolean ruoloPannello) {
		this.ruoloPannello = ruoloPannello;
	}
	public void setRuoloPartner(Boolean ruoloPartner) {
		this.ruoloPartner = ruoloPartner;
	}
	public void setRuoloRete(Boolean ruoloRete) {
		this.ruoloRete = ruoloRete;
	}
	public void setRuoloSitter(Boolean ruoloSitter) {
		this.ruoloSitter = ruoloSitter;
	}
	public void setRuoloSupervisore(Boolean ruoloSupervisore) {
		this.ruoloSupervisore = ruoloSupervisore;
	}
	public void setSecret(String secret) {
		this.secret = secret;
	}
	public void setSitterMaster(List<Utente> sitterMaster) {
		this.sitterMaster = sitterMaster;
	}
	public void setSitterSlave(List<Utente> sitterSlave) {
		this.sitterSlave = sitterSlave;
	}
	
	public void setStoria_contatti(List<StoriaContatto> storia_contatti) {
		this.storia_contatti = storia_contatti;
	}
	public void setStoria_operatore_appuntamenti(List<StoriaAppuntamento> storia_operatore_appuntamenti) {
		this.storia_operatore_appuntamenti = storia_operatore_appuntamenti;
	}
	public void setStoria_operatore_contatti(List<StoriaContatto> storia_operatore_contatti) {
		this.storia_operatore_contatti = storia_operatore_contatti;
	}
	public void setStoria_operatore_contratti(List<StoriaContratto> storia_operatore_contratti) {
		this.storia_operatore_contratti = storia_operatore_contratti;
	}
	public void setStoria_operatore_note(List<StoriaNota> storia_operatore_note) {
		this.storia_operatore_note = storia_operatore_note;
	}
	public void setStoria_utente_appuntamenti(List<StoriaAppuntamento> storia_utente_appuntamenti) {
		this.storia_utente_appuntamenti = storia_utente_appuntamenti;
	}
	public void setStoria_utente_contratti(List<StoriaContratto> storia_utente_contratti) {
		this.storia_utente_contratti = storia_utente_contratti;
	}
	public void setStoria_utente_note(List<StoriaNota> storia_utente_note) {
		this.storia_utente_note = storia_utente_note;
	}
	public void setTeam(List<Team> team) {
		this.team = team;
	}
	public void setTeams(List<Team> teams) {
		this.teams = teams;
	}
	public void setTelecoms(List<Telecom> telecoms) {
		this.telecoms = telecoms;
	}
	
	public void setTempoPassword(Integer tempoPassword) {
		this.tempoPassword = tempoPassword;
	}
	public void setTrunk(SipBuddies trunk) {
		this.trunk = trunk;
	}
	public void setUappuntamenti(List<Appuntamento> uappuntamenti) {
		this.uappuntamenti = uappuntamenti;
	}
	public void setUcontatti(List<Contatto> ucontatti) {
		this.ucontatti = ucontatti;
	}
	public void setUcontratti(List<Contratto> ucontratti) {
		this.ucontratti = ucontratti;
	}

	public void setUsername(String username) {
		this.username = ApplicationUtil.normalizeToNull(username);
	}
	public void setVisualizzaStorico(Boolean visualizzaStorico) {
		this.visualizzaStorico = visualizzaStorico;
	}
	public void setWeek_fine(Long week_fine) {
		this.week_fine = week_fine;
	}
	public void setWeek_inizio(Long week_inizio) {
		this.week_inizio = week_inizio;
	}
	public void setZone(List<Zona> zone) {
		this.zone = zone;
	}
	@Override
	public String toString() {
		return ApplicationUtil.toString(id, getDenominazione());
	}
	public void validate_username(FacesContext context, UIComponent component, Object input) {
		String var = ApplicationUtil.trimToEmpty((String) input);
		if(var.contains(ConstantsGlobal.pipeSeparator) || var.contains(ConstantsGlobal.colonSeparator) || var.contains(ConstantsGlobal.usernameSeparator1) || var.contains(ConstantsGlobal.usernameSeparator2)){
			throw new ValidatorException(Message.addValidatorMessage("proibito" , ConstantsGlobal.pipeSeparator+ConstantsGlobal.colonSeparator+ConstantsGlobal.usernameSeparator1+ConstantsGlobal.usernameSeparator2));
		}
	}


	public Boolean getLimitaRicercaAgente() {
		if(limitaRicercaAgente == null) {
			limitaRicercaAgente = Boolean.FALSE;
		}
		return limitaRicercaAgente;
	}
	public Boolean getLimitaRicercaCoordinatore() {
		if(limitaRicercaCoordinatore == null) {
			limitaRicercaCoordinatore = Boolean.FALSE;
		}
		return limitaRicercaCoordinatore;
	}
	public Boolean getLimitaRicercaOperatore() {
		if(limitaRicercaOperatore == null) {
			limitaRicercaOperatore = Boolean.FALSE;
		}
		return limitaRicercaOperatore;
	}
	public Boolean getLimitaRicercaPartner() {
		if(limitaRicercaPartner == null) {
			limitaRicercaPartner = Boolean.FALSE;
		}
		return limitaRicercaPartner;
	}
	public Boolean getLimitaRicercaSupervisore() {
		if(limitaRicercaSupervisore == null) {
			limitaRicercaSupervisore = Boolean.FALSE;
		}
		return limitaRicercaSupervisore;
	}

	public void setLimitaRicercaAgente(Boolean limitaRicercaAgente) {
		this.limitaRicercaAgente = limitaRicercaAgente;
	}
	public void setLimitaRicercaCoordinatore(Boolean limitaRicercaCoordinatore) {
		this.limitaRicercaCoordinatore = limitaRicercaCoordinatore;
	}
	public void setLimitaRicercaOperatore(Boolean limitaRicercaOperatore) {
		this.limitaRicercaOperatore = limitaRicercaOperatore;
	}
	public void setLimitaRicercaPartner(Boolean limitaRicercaPartner) {
		this.limitaRicercaPartner = limitaRicercaPartner;
	}
	public void setLimitaRicercaSupervisore(Boolean limitaRicercaSupervisore) {
		this.limitaRicercaSupervisore = limitaRicercaSupervisore;
	}
	public List<Nota> getUnote() {
		return unote;
	}
	public void setUnote(List<Nota> unote) {
		this.unote = unote;
	}
	public List<Prenotazione> getUprenotazioni() {
		return uprenotazioni;
	}
	public void setUprenotazioni(List<Prenotazione> uprenotazioni) {
		this.uprenotazioni = uprenotazioni;
	}
	public String getSessionId() {
		return sessionId;
	}
	public void setSessionId(String sessionId) {
		this.sessionId = sessionId;
	}
	public Integer getMassimoUtenti() {
		return massimoUtenti;
	}
	public void setMassimoUtenti(Integer massimoUtenti) {
		this.massimoUtenti = massimoUtenti;
	}
	public List<Sondaggio> getSondaggi() {
		return sondaggi;
	}
	public void setSondaggi(List<Sondaggio> sondaggi) {
		this.sondaggi = sondaggi;
	}
	public Boolean getPannelloOperatore() {
		if(pannelloOperatore == null) {
			pannelloOperatore = Boolean.TRUE;
		}
		return pannelloOperatore;
	}
	public void setPannelloOperatore(Boolean pannelloOperatore) {
		this.pannelloOperatore = pannelloOperatore;
	}
	public Boolean getPannelloSupervisore() {
		if(pannelloSupervisore == null) {
			pannelloSupervisore = Boolean.TRUE;
		}
		return pannelloSupervisore;
	}
	public void setPannelloSupervisore(Boolean pannelloSupervisore) {
		this.pannelloSupervisore = pannelloSupervisore;
	}
	
	public boolean isAgente() {
		return ApplicationUtil.isTrue(getRuoloAgente());
	}

	public boolean isOperatore() {
		return ApplicationUtil.isTrue(getRuoloOperatore()) && ApplicationUtil.isTrue(getPannelloOperatore());
	}

	public boolean isSupervisore() {
		return ApplicationUtil.isTrue(getRuoloSupervisore()) && ApplicationUtil.isTrue(getPannelloSupervisore());
	}
	
	public boolean isChat() {
		return ApplicationUtil.isTrue(getRuoloChat());
	}
	
	@Transient
	private ColonnaModel model;

	public ColonnaModel getModel() {
		if(model == null) {
			model = new ColonnaModel(id);
		}
		return model;
	}
	public void setModel(ColonnaModel model) {
		this.model = model;
	}
	
	
}