package it.callite.model;
import java.io.Serializable;
import java.util.Date;
import java.util.List;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.Lob;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import javax.persistence.Transient;

import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.model.generic.SelezionatoAbstract;
import it.callite.model.sip.SipBuddies;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.ParameterTrunk;
@SuppressWarnings("serial")
@Entity
//@Audited 
@Table(name = ConstantsGlobal.tp + "vocal_order", indexes = { @Index(name= "vocal_order", columnList = "data_creazione, nome") })
public class VocalOrder extends SelezionatoAbstract implements Serializable, Comparable<VocalOrder> {
	private Boolean attivo = Boolean.TRUE;
	private Boolean conferenza = Boolean.FALSE;
	private Date data_creazione = ApplicationUtil.date();
	@Column(columnDefinition = "text")
	private String descrizione;
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY) 
	private Long id;
	private String nome;
	@ManyToMany(fetch = FetchType.LAZY)
	@JoinTable(name = ConstantsGlobal.tp + "prodotto_aiuto", joinColumns=@JoinColumn(name="aiuto_id", nullable=false, updatable=true, insertable=true), inverseJoinColumns=@JoinColumn(name="prodotto_id", nullable=false, updatable=true, insertable=true)    )
	private List<Prodotto> prodotti;
	private String telefono;
	@Column(columnDefinition = "text")
	private String testo;
	@Transient
	private String testoString;
	@ManyToOne
	private SipBuddies trunk;
	@Column(columnDefinition = "text")
	private String modulo;
	
	private Short velocita;
	public VocalOrder() {
	}
	public int compareTo(VocalOrder o) {
		if(id != null && o.id != null) {
			return id.compareTo(o.id);
		} else if(id != null && o.id == null) {
			return -1;
		} else if(id == null && o.id != null) {
			return 1;
		} else {
			return 0;
		}
	}
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		VocalOrder other = (VocalOrder) obj;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		return true;
	}
	public Boolean getAttivo() {
		return attivo;
	}
	public Boolean getCheckTelefono() {
		return ApplicationUtil.isNotBlank(telefono);
	}
	
	public ParameterTrunk getParameter() {
		return new ParameterTrunk(trunk);
	}
	public Boolean getConferenza() {
		return ApplicationUtil.isTrue(conferenza);
	}
	public VocalOrder getCopia() {
		VocalOrder pr = new VocalOrder();
		pr.setNome(this.getNome() + " " +  ConstantsLanguage.COPY.stampa());
		pr.setDescrizione(this.getDescrizione());
		pr.setTesto(this.getTesto());
		pr.setVelocita(this.getVelocita());
		pr.setAttivo(this.getAttivo());
		pr.setConferenza(this.getConferenza());
		pr.setTelefono(this.getTelefono());
		pr.setTrunk(this.getTrunk());
		pr.setModulo(this.getModulo());
		pr.setPdf(this.getPdf());
		return pr;
	}
	
	public Byte[] getFileP() {
		if(isFile()) {
			return ApplicationUtil.toObject(pdf);
		}
		return null;
	}
	
	public boolean isFile() {
		return pdf != null && pdf.length > 0;
	}
	
	@Lob
	private byte[] pdf;
	public byte[] getPdf() {
		return pdf;
	}
	public void setPdf(byte[] pdf) {
		this.pdf = pdf;
	}
	
	public String getModulo() {
		return ApplicationUtil.trimToNull(modulo);
	}
	public void setModulo(String modulo) {
		this.modulo = ApplicationUtil.trimToNull(modulo);
	}
	public Date getData_creazione() {
		return data_creazione;
	}
	public String getDenominazione() {
		return ApplicationUtil.trimToEmpty(getNome());
	}
	public String getDescrizione() {
		return ApplicationUtil.trimToEmpty(descrizione);
	}
	public Long getId() {
		return id;
	}
	public String getNome() {
		return nome;
	}
	public List<Prodotto> getProdotti() {
		return prodotti;
	}
	public String getTelefono() {
		return ApplicationUtil.normalizeToNull(telefono);
	}
	public String getTesto() {
		return testo;
	}
	public String getTestoString() {
		return testoString;
	}
	public SipBuddies getTrunk() {
		return trunk;
	}
	public Short getVelocita() {
		return velocita;
	}
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		return result;
	}
	public void setAttivo(Boolean attivo) {
		this.attivo = attivo;
	}
	public void setConferenza(Boolean conferenza) {
		this.conferenza = conferenza;
	}
	public void setData_creazione(Date data_creazione) {
		this.data_creazione = data_creazione;
	}
	public void setDescrizione(String descrizione) {
		this.descrizione = ApplicationUtil.trimToNull(descrizione);
	}
	public void setId(Long id) {
		this.id = id;
	}
	public void setNome(String nome) {
		this.nome = ApplicationUtil.normalizeToNull(nome);
	}
	public void setProdotti(List<Prodotto> prodotti) {
		this.prodotti = prodotti;
	}
	public void setTelefono(String telefono) {
		this.telefono = ApplicationUtil.normalizeToNull(telefono);
	}
	public void setTesto(String testo) {
		this.testo = testo;
	}
	public void setTestoString(String testoString) {
		this.testoString = testoString;
	}
	public void setTrunk(SipBuddies trunk) {
		this.trunk = trunk;
	}
	public void setVelocita(Short velocita) {
		this.velocita = velocita;
	}
	@Override
	public String toString() {
		return ApplicationUtil.toString(id, getDenominazione());
	}
}