package it.callite.model.config;
import java.io.Serializable;
import java.util.Date;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.Table;
import it.callite.constants.ConstantsGlobal;
import it.callite.model.generic.SelezionatoAbstract;
import it.callite.utility.ApplicationUtil;
@SuppressWarnings("serial")
@Entity
//@Audited
@Table(name = ConstantsGlobal.tp + "config_pausa", indexes = { @Index(name= "config_pausa", columnList = "chiave, data_creazione" )})
public class ConfigPausa  extends SelezionatoAbstract implements Serializable, Comparable<ConfigPausa> {
	private Boolean attivo = Boolean.TRUE;
	private String chiave;
	
	private Date data_creazione = ApplicationUtil.date();
	
	@Column(columnDefinition = "text")
	private String descrizione;
	private Integer durata = 15;
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY) 
	private Long id;
	
	private Boolean positivo = Boolean.FALSE;
	public ConfigPausa() {
	}
	
	public int compareTo(ConfigPausa o) {
		if(id != null && o.id != null) {
			return id.compareTo(o.id);
		} else if(id != null && o.id == null) {
			return -1;
		} else if(id == null && o.id != null) {
			return 1;
		} else {
			return 0;
		}
	}
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		ConfigPausa other = (ConfigPausa) obj;
		if (chiave == null) {
			if (other.chiave != null)
				return false;
		} else if (!chiave.equals(other.chiave))
			return false;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		return true;
	}
	public Boolean getAttivo() {
		return attivo;
	}
	public String getChiave() {
		return chiave;
	}
	public String getChiaveS() {
		String c = ApplicationUtil.trimToNull(chiave);
		if(c != null) {
			return c + " " + (ApplicationUtil.isTrue(positivo) ? "[+]" : "[-]");
		}
		return "";
	}
	public Date getData_creazione() {
		return data_creazione;
	}
	
	public String getDescrizione() {
		return descrizione;
	}
	public Integer getDurata() {
		return durata;
	}
	public Long getId() {
		return id;
	}
	public Boolean getPositivo() {
		return positivo;
	}
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((chiave == null) ? 0 : chiave.hashCode());
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		return result;
	}
	public void setAttivo(Boolean attivo) {
		this.attivo = attivo;
	}
	public void setChiave(String chiave) {
		this.chiave = ApplicationUtil.normalizeToNull(chiave);
	}
	public void setData_creazione(Date data_creazione) {
		this.data_creazione = data_creazione;
	}
	public void setDescrizione(String descrizione) {
		this.descrizione = descrizione;
	}
	public void setDurata(Integer durata) {
		this.durata = durata;
	}
	public void setId(Long id) {
		this.id = id;
	}
	public void setPositivo(Boolean positivo) {
		this.positivo = positivo;
	}
	@Override
	public String toString() {
		return chiave;
	}
	
	
}