package it.callite.model.config;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.model.SelectItem;
import javax.faces.validator.ValidatorException;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.Table;
import javax.persistence.Transient;
import it.callite.constants.ConstantsGlobal;
import it.callite.message.Message;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.ConfigUtility;
@SuppressWarnings("serial")
@Entity
//@Audited
@Table(name = ConstantsGlobal.tp + "config_validatore", indexes = { @Index(name= "config_validatore", columnList = "chiave" )})
public class ConfigValidatore implements Serializable {
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY) 
	private Long id;
	private String chiave;
	@Column(columnDefinition = "text")
	private String descrizione;
	@Column(columnDefinition = "text")
	private String valore;
	@Transient
	private ConfigUtility configure;
	public ConfigValidatore() {
		
	}
	public ConfigUtility getConfigure() {
		return configure;
	}
	public void setConfigure(ConfigUtility configure) {
		this.configure = configure;
	}
	public ConfigValidatore(String chiave, String valore, String valori, Integer tipo, Boolean richiesto, Boolean modificabile) {
		this.setChiave(chiave);
		this.setValore(valore);
		configure = new ConfigUtility(valori, tipo, richiesto, modificabile);
	}
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		ConfigValidatore other = (ConfigValidatore) obj;
		if (chiave == null) {
			if (other.chiave != null)
				return false;
		} else if (!chiave.equals(other.chiave))
			return false;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		return true;
	}
	public String getChiave() {
		return chiave;
	}
	public String getChiaveTesto() {
		return Message.addCatalogMessage(chiave);
	}
	public String getDescrizione() {
		return ApplicationUtil.trimToNull(descrizione);
	}
	public Long getId() {
		return id;
	}
	public Boolean getLista() {
		return configure.getLista();
	}
	public Boolean getModificabile() {
		return configure.getModificabile();
	}
	public List<String> getValoriItem() {
		return ApplicationUtil.token(valore, ConstantsGlobal.colonSeparator);
	}
	public void setValoriItem(List<String> valoriItem) {
		this.valore = ApplicationUtil.detoken(valoriItem, ConstantsGlobal.colonSeparator);
	}
	public Boolean getMultiplo() {
		return configure.getMultiplo();
	}
	public Boolean getOrdinabile() {
		return configure.getOrdinabile();
	}
	public Boolean getEditore() {
		return configure.getEditore();
	}
	public Boolean getPassword() {
		return configure.getPassword();
	}
	public Boolean getRichiesto() {
		return configure.getRichiesto();
	}
	public Boolean getScelta() {
		return configure.getScelta();
	}
	public Boolean getStringa() {
		return configure.getStringa();
	}
	public Boolean getTesto() {
		return configure.getTesto();
	}
	public Integer getTipo() {
		return configure.getTipo();
	}
	public String getValore() {
		return ApplicationUtil.trimToEmpty(valore);
	}
	public String getValori() {
		return configure.getValori();
	}
	public List<String> getValoriList() {
		return ApplicationUtil.token(getValori(), ConstantsGlobal.colonSeparator);
	}
	public List<String> getValoriListSO() {
		List<String> ls =  ApplicationUtil.token(getValori(), ConstantsGlobal.colonSeparator);
		ls.removeAll(getValoriItem());
		return ls;
	}
	public void setValoriListSO(List<String> valoriListSO) {
	}
	
	public List<SelectItem> getValoriListSI() {
		List<String> vs = getValoriList();
		List<SelectItem> sis = new ArrayList<SelectItem>();
		for (String v : vs) {
			sis.add(new SelectItem(v, Message.addCatalogText(v)));
		}
		return sis;
	}
	public Boolean getVisibile() {
		return configure.getVisibile();
	}
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((chiave == null) ? 0 : chiave.hashCode());
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		return result;
	}
	public void setChiave(String chiave) {
		this.chiave = ApplicationUtil.normalizeToNull(chiave);
	}
	public void setDescrizione(String descrizione) {
		this.descrizione = ApplicationUtil.trimToNull(descrizione);
	}
	public void setId(Long id) {
		this.id = id;
	}
	public void setValore(String valore) {
		this.valore = ApplicationUtil.trimToNull(valore);
	}
	public void validate(FacesContext context, UIComponent component, Object input)  {
		String v = getValori();
		if(v!=null) {
			String var = (String) input;
			Pattern p = Pattern.compile(v);
			Matcher m = p.matcher(var);
			boolean b = m.matches();
			if (!b) {
				throw new ValidatorException(Message.addValidatorMessage("formato.errato" , v));
			}
		}
	}
	@Override
	public String toString() {
		return id != null ? id.toString() : "";
	}
}