package it.callite.model.report;

import java.io.Serializable;
import java.util.Date;
import java.util.Map;
import java.util.TreeMap;

import it.callite.constants.ConstantsFormat;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.AutodialerMap;


@SuppressWarnings("serial")
public class AutodialerStatistiche implements Serializable {

	private AutodialerMap statistiche = new AutodialerMap();
	private Map<String, AutodialerMap> mappa = new TreeMap<String, AutodialerMap>();
	
	private Integer a_rimosse = 0;
	private Integer a_saltate = 0;
	private Integer a_fallite = 0;

	private Long timeAttesa = Long.valueOf(0);
	private Long timeChiamata = Long.valueOf(0);
	private Long timeGestione = Long.valueOf(0);

	public AutodialerMap getStatistiche() {
		return statistiche;
	}

	public void setStatistiche(AutodialerMap statistiche) {
		this.statistiche = statistiche;
	}
	

	public Map<String, AutodialerMap> getMappa() {
		return mappa;
	}

	public void setMappa(Map<String, AutodialerMap> mappa) {
		this.mappa = mappa;
	}


	public AutodialerStatistiche() {
	}

	public AutodialerStatistiche(AutodialerStatistiche s) {
		this.a_rimosse = s.getA_rimosse();
		this.a_saltate = s.getA_saltate();
		this.a_fallite = s.getA_fallite();
		this.timeAttesa = s.getTimeAttesa();
		this.timeChiamata = s.getTimeChiamata();
		this.timeGestione = s.getTimeGestione();
		this.statistiche = s.getStatistiche();
		this.mappa = s.getMappa();
	}
	
	
	public AutodialerMap getMap(Date data) {
		String key = ApplicationUtil.formatDate(ConstantsFormat.orarioT.stampa(), data);
		AutodialerMap a = mappa.get(key);
		if(a == null) {
			a = new AutodialerMap();
			mappa.put(key, a);
		}
		return a;
	}
	
	public String getCampiXML() {
		return ApplicationUtil.report6XML(mappa);
	}

	public void addTimeAttesa(Date time1 , Date time2) {
		Long time = ApplicationUtil.sottrai(time1, time2);
		this.timeAttesa += time;
	}

	public void addTimeChiamata(Date time1 , Date time2) {
		Long time = ApplicationUtil.sottrai(time1, time2);
		this.timeChiamata += time;
	}

	public void addTimeGestione(Date time1 , Date time2) {
		Long time = ApplicationUtil.sottrai(time1, time2);
		this.timeGestione += time;
	}

	public Integer getAbbattute() {
		return statistiche.getAbbattute();
	}
	public Double getAbbattuteP() {
		return statistiche.getAbbattuteP();
	}
	public Integer getAgganciate() {
		return statistiche.getAgganciate();
	}

	public Double getAgganciateP() {
		return statistiche.getAgganciateP();
	}

	public Integer getCongestion() {
		return statistiche.getCongestion();
	}
	public Double getCongestionP() {
		return statistiche.getCongestionP();
	}
	public Integer getEsitate() {
		return statistiche.getEsitate();
	}
	public Double getEsitateP() {
		return statistiche.getEsitateP();
	}

	public Integer getFallito() {
		return statistiche.getFallito();
	}

	public Double getFallitoP() {
		return statistiche.getFallitoP();
	}
	public Integer getGestite() {
		return statistiche.getGestite();
	}
	public Double getGestiteP() {
		return statistiche.getGestiteP();
	}
	public Integer getHangup() {
		return statistiche.getHangup();
	}

	public Double getHangupP() {
		return statistiche.getHangupP();
	}

	public Integer getMacchina() {
		return statistiche.getMacchina();
	}

	public Double getMacchinaP() {
		return statistiche.getMacchinaP();
	}
	public Double getInboundP() {
		return statistiche.getInboundP();
	}

	public Long getMediaAttesa() {
		if(ApplicationUtil.gtZero(timeAttesa) && ApplicationUtil.gtZero(statistiche.getGestite()))
			return Long.valueOf(timeAttesa/statistiche.getGestite());
		else return Long.valueOf(0);
	}

	public Long getMediaChiamata() {
		if(ApplicationUtil.gtZero(timeChiamata) && ApplicationUtil.gtZero(statistiche.getRisposte()))
			return Long.valueOf(timeChiamata/statistiche.getRisposte());
		else return Long.valueOf(0);
	}
	public Long getMediaGestione() {
		if(ApplicationUtil.gtZero(timeGestione) && ApplicationUtil.gtZero(statistiche.getEsitate()))
			return Long.valueOf(timeGestione/statistiche.getEsitate());
		else return Long.valueOf(0);
	}
	public Integer getNonRisposto() {
		return statistiche.getNonRisposto();
	}
	
	public Integer getRinging() {
		return statistiche.getRinging();
	}
	
	public Integer getPrenotato() {
		return statistiche.getPrenotato();
	}
	public Double getPrenotatoP() {
		return statistiche.getPrenotatoP();
	}
	public Double getNonRispostoP() {
		return statistiche.getNonRispostoP();
	}
	public Double getRingingP() {
		return statistiche.getRingingP();
	}
	public Integer getOccupato() {
		return statistiche.getOccupato();
	}
	public Double getOccupatoP() {
		return statistiche.getOccupatoP();
	}
	public Integer getRifiutato() {
		return statistiche.getRifiutato();
	}


	public Double getRifiutatoP() {
		return statistiche.getRifiutatoP();
	}

	public Integer getRimosse() {
		return statistiche.getRimosse();
	}

	public Double getRimosseP() {
		return statistiche.getRimosseP();
	}


	public Integer getRisposte() {
		return statistiche.getRisposte();
	}
	public Double getRisposteP() {
		return statistiche.getRisposteP();
	}

	public Integer getSaltate() {
		return statistiche.getSaltate();
	}

	public Double getSaltateP() {
		return statistiche.getSaltateP();
	}

	public Integer getScartate() {
		return statistiche.getScartate();
	}

	public Double getScartateP() {
		return statistiche.getScartateP();
	}
	public Integer getTentate() {
		return statistiche.getTentate();
	}
	public Long getTimeAttesa() {
		return timeAttesa;
	}

	public Long getTimeChiamata() {
		return timeChiamata;
	}


	public Long getTimeGestione() {
		return timeGestione;
	}

	public void incAbbattute(Date data) {
		statistiche.incAbbattute();
		getMap(data).incAbbattute();
		resetError();
	}

	public void incAgganciate(Date data) {
		statistiche.incAgganciate();
		getMap(data).incAgganciate();
		resetError();
	}

	public void incCongestion(Date data) {
		statistiche.incCongestion();
		getMap(data).incCongestion();
		a_fallite++;
	}

	public void incEsitate(Date data) {
		statistiche.incEsitate();
		getMap(data).incEsitate();
		resetError();
	}

	public void incFallito(Date data) {
		statistiche.incFallito();
		getMap(data).incFallito();
		a_fallite++;
	}
	public void incGestite(Date data) {
		statistiche.incGestite();
		getMap(data).incGestite();
		resetError();
	}

	public void incHangup(Date data) {
		statistiche.incHangup();
		getMap(data).incHangup();
		resetError();
	}
	public void incMacchina(Date data) {
		statistiche.incMacchina();
		getMap(data).incMacchina();
		resetError();
	}
	public void incInbound(Date data) {
		statistiche.incInbound();
	}
	public void incNonRisposto(Date data) {
		statistiche.incNonRisposto();
		getMap(data).incNonRisposto();
		a_fallite++;
	}
	public void incRinging(Date data) {
		statistiche.incRinging();
		getMap(data).incRinging();
		a_fallite++;
	}
	public void incPrenotato(Date data) {
		statistiche.incPrenotato();
		getMap(data).incPrenotato();
	}

	public void incOccupato(Date data) {
		statistiche.incOccupato();
		getMap(data).incOccupato();
		a_fallite++;
	}

	public void incRifiutato(Date data) {
		statistiche.incRifiutato();
		getMap(data).incRifiutato();
		a_fallite++;
	}
	public void incRimosse(Date data) {
		statistiche.incRimosse();
		getMap(data).incRimosse();
		a_rimosse++;
	}

	public void incRisposte(Date data) {
		statistiche.incRisposte();
		getMap(data).incRisposte();
		resetError();
	}

	public void incSaltate(Date data) {
		statistiche.incSaltate();
		getMap(data).incSaltate();
		a_saltate++;
	}

	public void incScartate(Date data) {
		statistiche.incScartate();
		getMap(data).incScartate();
		resetError();
	}

	public void incTentate(Date data) {
		statistiche.incTentate();
		getMap(data).incTentate();
	}
	public void resetError() {
		a_rimosse = 0;
		a_saltate = 0;
		a_fallite = 0;
	}

	public void resetAll() {
		a_rimosse = 0;
		a_saltate = 0;
		a_fallite = 0;
		
		statistiche.resetAll();
		mappa.clear();
		
		timeGestione = Long.valueOf(0);
		timeChiamata = Long.valueOf(0);
		timeAttesa = Long.valueOf(0);
	}

	public void setAbbattute(Integer abbattute) {
		this.statistiche.setAbbattute(abbattute);
	}

	public void setAgganciate(Integer agganciate) {
		this.statistiche.setAgganciate(agganciate);
	}

	public void setCongestion(Integer congestion) {
		this.statistiche.setCongestion(congestion);
	}

	public void setEsitate(Integer esitate) {
		this.statistiche.setEsitate(esitate);
	}

	public void setFallito(Integer fallito) {
		this.statistiche.setFallito(fallito);
	}

	public void setGestite(Integer gestite) {
		this.statistiche.setGestite(gestite);
	}

	public void setHangup(Integer hangup) {
		this.statistiche.setHangup(hangup);
	}
	public void setMacchina(Integer macchina) {
		this.statistiche.setMacchina(macchina);
	}
	public void setNonRisposto(Integer nonRisposto) {
		this.statistiche.setNonRisposto(nonRisposto);
	}
	public void setRinging(Integer ringing) {
		this.statistiche.setRinging(ringing);
	}
	public void setPrenotato(Integer prenotato) {
		this.statistiche.setPrenotato(prenotato);
	}
	public void setOccupato(Integer occupato) {
		this.statistiche.setOccupato(occupato);
	}
	public void setRifiutato(Integer rifiutato) {
		this.statistiche.setRifiutato(rifiutato);
	}
	public void setRimosse(Integer rimosse) {
		this.statistiche.setRimosse(rimosse);
	}
	public void setRisposte(Integer risposte) {
		this.statistiche.setRisposte(risposte);
	}
	public void setSaltate(Integer saltate) {
		this.statistiche.setSaltate(saltate);
	}
	public void setScartate(Integer scartate) {
		this.statistiche.setScartate(scartate);
	}
	public void setTentate(Integer tentate) {
		this.statistiche.setTentate(tentate);
	}
	public void setTimeAttesa(Long timeAttesa) {
		this.timeAttesa = timeAttesa;
	}

	public void setTimeChiamata(Long timeChiamata) {
		this.timeChiamata = timeChiamata;
	}

	public void setTimeGestione(Long timeGestione) {
		this.timeGestione = timeGestione;
	}


	public Integer getNonRisposte() {
		return statistiche.getNonRisposte();
	}
	public Double getNonRisposteP() {
		return statistiche.getNonRisposteP();
	}

	public Integer getNonGestite() {
		return statistiche.getNonGestite();
	}
	public Double getNonGestiteP() {
		return statistiche.getNonGestiteP();
	}

	public Integer getInbound() {
		return statistiche.getInbound();
	}

	public void setInbound(Integer inbound) {
		this.statistiche.setInbound(inbound);
	}

	public Integer getA_rimosse() {
		return a_rimosse;
	}

	public void setA_rimosse(Integer a_rimosse) {
		this.a_rimosse = a_rimosse;
	}


	public Integer getA_fallite() {
		return a_fallite;
	}

	public void setA_fallite(Integer a_fallite) {
		this.a_fallite = a_fallite;
	}

	public Integer getA_saltate() {
		return a_saltate;
	}

	public void setA_saltate(Integer a_saltate) {
		this.a_saltate = a_saltate;
	}

}