package it.callite.model.report;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContext;

import it.callite.autodialer.controllo.CodaStatic;
import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.constants.ConstantsMap;
import it.callite.message.Message;
import it.callite.model.Campagna;
import it.callite.model.Contatto;
import it.callite.model.EsitoArchiviazione;
import it.callite.model.Utente;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.Colore;
import it.callite.utility.InfoUtente;
@SuppressWarnings("serial")
public class MonitorOperatore implements Serializable, Comparable<MonitorOperatore> {
	private Boolean attivo = Boolean.TRUE;		
	private Boolean visibile = Boolean.TRUE;		
	private Boolean online = Boolean.FALSE;		
	private Integer libero = 0;
	private Boolean registrazione = Boolean.FALSE;
	private String pannello;
	private EsitoArchiviazione esito;
	private Boolean reload = Boolean.FALSE;
	private Campagna campagna;	
	private Contatto contatto;
	private Integer flag;
	private Map<Short, List<Colore>> colori = new LinkedHashMap<Short, List<Colore>>();
	private Date data_azione;
	private Date data_pannello;
	private Long endorse;
	private String ip;
	private String tipoPausa;
	private Utente operatore;
	private Short stato;
	private Integer precedenza = 0;
	private Set<SecurityContext> securityContextList = new HashSet<SecurityContext>();

	public Set<SecurityContext> getSecurityContextList() {
		return securityContextList;
	}

	public void addSecurityContext(SecurityContext securityContext) {
		if(securityContext != null && !securityContextList.contains(securityContext)) {
			this.securityContextList.add(securityContext);
		}
	}


	public String getSessions() {
		int size = securityContextList.size();
		if(ApplicationUtil.gt(size, 1)) {
			return " " + "[" + size + "]";
		}
		return "";
	}

	public void clearSession() {
		if(ApplicationUtil.notEmpty(securityContextList)) {
			for(SecurityContext sc : securityContextList) {
				if(sc !=  null) {
					Authentication au = sc.getAuthentication();
					if(au != null && au.isAuthenticated()) {
						au.setAuthenticated(false);
					}
				}
			}
			securityContextList.clear();
		}
	}

	private List<String> messaggi = new ArrayList<String>();

	public void addMessaggio(String messaggio) {
		messaggi.add(messaggio);
	}

	public List<String> getMessaggi() {
		return messaggi;
	}
	public void setMessaggi(List<String> messaggi) {
		this.messaggi = messaggi;
	}
	public Boolean getOnline() {
		return online;
	}
	public void setOnline(Boolean online) {
		this.online = online;
	}
	public EsitoArchiviazione getEsito() {
		return esito;
	}
	public void setEsito(EsitoArchiviazione esito) {
		this.esito = esito;
	}
	public String getPannello() {
		return pannello;
	}
	public void setPannello(String pannello) {
		this.pannello = pannello;
	}
	public Long getEndorse() {
		return endorse;
	}
	public void setEndorse(Long endorse) {
		this.endorse = endorse;
	}
	public Integer getFlag() {
		return flag;
	}
	public void setFlag(Integer flag) {
		this.flag = flag;
	}
	public MonitorOperatore() {
		colori.putAll(ConstantsMap.getColori());
	}
	public Date getData_pannello() {
		return data_pannello;
	}
	public void setData_pannello(Date data_pannello) {
		this.data_pannello = data_pannello;
	}
	public String getColore() {
		if(stato != null && ApplicationUtil.notEmpty(colori)) {
			List<Colore> cls = colori.get(stato);
			if(ApplicationUtil.notEmpty(cls)) {
				Long time =  getOrario();
				Iterator<Colore> cIt = cls.iterator();
				while (cIt.hasNext()) {
					Colore t = (Colore) cIt.next();
					if(t.getTempo() != null && ApplicationUtil.ge(time, t.getTempo())) {
						return t.getColore();
					}
				}
			}
		}
		return ConstantsGlobal.color_black;
	}
	public int compareTo(MonitorOperatore o) {
		if(operatore != null && o.operatore != null) {
			return operatore.compareTo(o.operatore);
		} else if(operatore != null && o.operatore == null) {
			return -1;
		} else if(operatore == null && o.operatore != null) {
			return 1;
		} else {
			return 0;
		}
	}
	public Boolean getAttivo() {
		return attivo;
	}
	public Campagna getCampagna() {
		return campagna;
	}
	public Boolean getVisibile() {
		return visibile;
	}
	public void setVisibile(Boolean visibile) {
		this.visibile = visibile;
	}
	public Contatto getContatto() {
		return contatto;
	}
	public Date getData_azione() {
		return data_azione;
	}
	public String getIp() {
		return ip;
	}
	public Integer getLibero() {
		return libero;
	}

	public Utente getOperatore() {
		return operatore;
	}
	public Boolean getRegistrazione() {
		return registrazione;
	}

	public boolean isLoggato() {
		return (stato != null && !ApplicationUtil.eq(stato, ConstantsMap.STATO_LOGOUT));
	}
	public boolean isInAttesa() {
		return (contatto == null && campagna != null && stato != null && ApplicationUtil.eq(stato, ConstantsMap.STATO_ATTESA_CONTATTO));
	}
	public String getPannelloString() {
		if(tipoPausa != null) {
			return tipoPausa;
		} else {
			return isPannelloAperto() ? (getPannelloTesto() + " " + "[" +  getTimer() + "]") : "";
		}
	}
	public String getPannelloTesto() {
		return Message.addCatalogText(pannello) + (esito != null ?  (" " + "[" + esito.getDenominazione() + "]") : "");
	}
	public boolean isPannelloAperto() {
		return (contatto != null && stato != null && ApplicationUtil.isNotBlank(pannello) && ConstantsMap.getPannello().contains(stato) );
	}
	public boolean isRegistrazioneAperto() {
		//boolean c = !(ApplicationUtil.isTrue(contatto.getCampagna().getAutostart_reg_remota()) || ApplicationUtil.isTrue(contatto.getCampagna().getAutostart_reg_locale()));
		boolean c = true;
		return (contatto != null && c && stato != null && ApplicationUtil.eq(stato, ConstantsMap.STATO_CONVERSAZIONE) && ApplicationUtil.isTrue(registrazione));
	}
	public String getColorPannello() {
		if(isRegistrazioneAperto()) {
			return "red";
		} else {
			return "black";
		}
	}
	public Boolean getReload() {
		return reload;
	}
	public Short getStato() {
		return stato;
	}	
	public String getStatoTesto() {
		if(stato != null) {
			return "text.stato."+stato;
		} else {
			return null;
		}
	}

	public boolean isBrowser() {
		return endorse == null || (endorse != null && ((System.currentTimeMillis()-endorse.longValue()) <= ConstantsGlobal.timeoutActive));
	}

	public Long getOrario() {
		return ApplicationUtil.time(data_azione, ApplicationUtil.date());
	}
	public String getTime() {
		Long time =  getOrario();
		return ApplicationUtil.format(time);
	}
	public String getTimer() {
		Long time = ApplicationUtil.time(data_pannello, ApplicationUtil.date());
		return ApplicationUtil.format(time);
	}
	public void setAttivo(Boolean attivo) {
		this.attivo = attivo;
	}
	public void setCampagna(Campagna campagna) {
		this.campagna = campagna;
	}
	public void setContatto(Contatto contatto) {
		this.contatto = contatto;
	}
	public void setData_azione(Date data_azione) {
		this.data_azione = data_azione;
	}
	public void setIp(String ip) {
		this.ip = ip;
	}
	public void setLibero(Integer libero) {
		this.libero = libero;
	}

	public void setOperatore(Utente operatore) {
		this.operatore = operatore;
	}
	public void setRegistrazione(Boolean registrazione) {
		this.registrazione = registrazione;
	}
	public void setReload(Boolean reload) {
		this.reload = reload;
	}
	public void setStato(Short stato) {
		this.stato = stato;
	}
	public void setStatoTesto(String statoTesto) {}
	public void svuota() {
		this.contatto = null;
		this.campagna = null;
		this.registrazione = null;
	}
	public String getTipo() {
		if(contatto != null) {
			return ApplicationUtil.testo(flag) + contatto.getSpecifica();
		} 
		return null;
	}
	public String getStatoTelefono() {
		if(operatore != null &&  operatore.isLocale() && ApplicationUtil.gtZero(libero)) {
			if(ApplicationUtil.eq(libero, 1)) {
				return ConstantsLanguage.SI.stampa();
			}
			if(ApplicationUtil.eq(libero, 2)) {
				return ConstantsLanguage.NO.stampa();
			}
			if(ApplicationUtil.eq(libero, 3)) {
				return ConstantsLanguage.ATTIVO.stampa();
			}
		}
		return ConstantsLanguage.TRATTINI.stampa();
	}
	public String getTipoPausa() {
		return tipoPausa;
	}
	public void setTipoPausa(String tipoPausa) {
		this.tipoPausa = tipoPausa;
	}

	public boolean isDaLogout() {
		if(isLoggato()) { 
			return true;
		}
		return false;
	}

	public boolean isDaReload() {
		if(isBrowser() && isLoggato()) { 
			return true;
		}
		return false;
	}

	public Integer getPrecedenza() {
		return precedenza;
	}

	public void setPrecedenza(Integer precedenza) {
		this.precedenza = precedenza;
	}
	
	public Boolean getTimeout() {
		if(stato != null && ApplicationUtil.eq(stato, ConstantsMap.STATO_ATTESA_CONTATTO) && campagna != null && operatore != null) {
			InfoUtente utente = new InfoUtente(operatore);
			List<InfoUtente> ls = CodaStatic.getCodaTeam().get(campagna);
			if(!ls.contains(utente)) {
				return true;
			}
		}
		return false;
	}

}