package it.callite.model.report;

import java.io.Serializable;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.sql.Time;
import java.util.Date;

import it.callite.constants.ConstantsLanguage;
import it.callite.message.Message;
import it.callite.utility.ApplicationUtil;
@SuppressWarnings("serial")
public class RiepilogoCampagnaList implements Serializable, Comparable<RiepilogoCampagnaList> {

	private PannelloReport report;

	public PannelloReport getReport() {
		return report;
	}


	public void setReport(PannelloReport report) {
		this.report = report;
	}


	private Long eid;
	private String esito;
	private String codice;
	private Integer stato;
	private Integer priorita;
	private Time ora_fine;
	private Time ora_inizio;
	private Date data_fine;
	private Date data_inizio;

	private Long archiviati = 0L;
	private Long richiami1 = 0L;	
	private Long richiami2 = 0L;	
	private Long richiami3 = 0L;	
	private Long richiami4 = 0L;


	public Long getTotali() {
		return getArchiviati() + getRichiami1() + getRichiami2() + getRichiami3() + getRichiami4();
	}


	public Integer getPriorita() {
		return priorita;
	}


	public void setPriorita(Integer priorita) {
		this.priorita = priorita;
	}


	public Time getOra_fine() {
		return ora_fine;
	}


	public void setOra_fine(Time ora_fine) {
		this.ora_fine = ora_fine;
	}


	public Time getOra_inizio() {
		return ora_inizio;
	}

	private Boolean inverti = false;

	public Boolean getInverti() {
		if(inverti == null) {
			inverti = false;
		}
		return inverti;
	}

	public void setInverti(Boolean inverti) {
		this.inverti = inverti;
	}
	public void setOra_inizio(Time ora_inizio) {
		this.ora_inizio = ora_inizio;
	}


	public Date getData_fine() {
		return data_fine;
	}


	public void setData_fine(Date data_fine) {
		this.data_fine = data_fine;
	}


	public Date getData_inizio() {
		return data_inizio;
	}


	public void setData_inizio(Date data_inizio) {
		this.data_inizio = data_inizio;
	}

	private Long week_fine;
	private Long week_inizio;
	
	public Long getWeek_fine() {
		return week_fine;
	}

	public void setWeek_fine(Long week_fine) {
		this.week_fine = week_fine;
	}

	public Long getWeek_inizio() {
		return week_inizio;
	}

	public void setWeek_inizio(Long week_inizio) {
		this.week_inizio = week_inizio;
	}


	public Boolean getStatoAttiva() {
		return ApplicationUtil.calcolaIntervallo(getInverti(), getData_inizio(), getData_fine(), getWeek_inizio(), getWeek_fine(), getOra_inizio(), getOra_fine());
	}

	public String getStatoAttivaT() {
		return getStatoAttiva() ? ConstantsLanguage.SI.stampa() : ConstantsLanguage.NO.stampa();
	}

	public RiepilogoCampagnaList() { }


	public int compareTo(RiepilogoCampagnaList o) {
		if(esito != null && o.esito != null) {
			return esito.compareTo(o.esito);
		} else if(esito != null && o.esito == null) {
			return -1;
		} else if(esito == null && o.esito != null) {
			return 1;
		} else {
			return 0;
		}
	}

	public Long getEid() {
		return eid;
	}
	public String getEsito() {
		return esito;
	}

	public String getDenominazione() {
		String codice = getCodice();
		return (ApplicationUtil.isNotBlank(codice) ? (codice + " - ") : "") + getEsito();
	}

	public BigDecimal getPercentuale() {
		BigDecimal num = new BigDecimal(0);
		if(ApplicationUtil.gtZero(getTotali()) && ApplicationUtil.gtZero(getArchiviati())) {
			Double archiviatiT = Double.valueOf(getArchiviati());
			Double totaliT = Double.valueOf(getTotali());
			num = new BigDecimal((archiviatiT / totaliT * 100));
		}
		return num.setScale(2, RoundingMode.HALF_UP) ;
	}


	public Integer getStato() {
		return stato;
	}
	public String getStatoTesto() {
		return Message.addCatalogText("ts_stato." + stato);
	}


	public void setEid(Long eid) {
		this.eid = eid;
	}
	public void setEsito(String esito) {
		this.esito = esito;
	}

	public void setStato(Integer stato) {
		this.stato = stato;
	}

	public String getCodice() {
		return codice;
	}


	public void setCodice(String codice) {
		this.codice = codice;
	}


	@Override
	public String toString() {
		return ApplicationUtil.trimToEmpty(esito);
	}

	public String getTotaliS() {
		return ApplicationUtil.formatNumber(getTotali());
	}

	public String getArchiviatiS() {
		return ApplicationUtil.formatNumber(getArchiviati());
	}

	public String getDaContattareS1() {
		if(ApplicationUtil.gtZero(richiami3 + richiami4)) {
			return getDaContattareS2() + " " + "("   +getDaContattareS3()+ ")";
		} else {
			return getDaContattareS2();
		}
	}

	public String getDaContattareS2() {
		return ApplicationUtil.formatNumber(getRichiami1()+getRichiami2());
	}

	public String getDaContattareS3() {
		return ApplicationUtil.formatNumber(getRichiami3()+getRichiami4());
	}

	public String getNuoviS1() {
		if(ApplicationUtil.gtZero(richiami3)) {
			return getNuoviS2() + " " + "("   +getNuoviS3()+ ")";
		} else {
			return getNuoviS2();
		}
	}

	public String getNuoviS2() {
		return ApplicationUtil.formatNumber(getRichiami1());
	}

	public String getNuoviS3() {
		return ApplicationUtil.formatNumber(getRichiami3());
	}

	public String getRichiamiS1() {
		if(ApplicationUtil.gtZero(richiami4)) {
			return getRichiamiS2() + " " + "("   + getRichiamiS3()+ ")";
		} else {
			return getRichiamiS2();
		}
	}

	public String getRichiamiS2() {
		return ApplicationUtil.formatNumber(getRichiami2());
	}
	public String getRichiamiS3() {
		return ApplicationUtil.formatNumber(getRichiami4());
	}


	public Long getRichiami1() {
		if(ApplicationUtil.ltZero(richiami1)) {
			richiami1 = Long.valueOf(0);	
		}
		return richiami1;
	}


	public void setRichiami1(Long richiami1) {
		this.richiami1 = richiami1;
	}


	public Long getRichiami2() {
		if(ApplicationUtil.ltZero(richiami2)) {
			richiami2 = Long.valueOf(0);	
		}
		return richiami2;
	}


	public void setRichiami2(Long richiami2) {
		this.richiami2 = richiami2;
	}



	public Long getRichiami3() {
		if(ApplicationUtil.ltZero(richiami3)) {
			richiami3 = Long.valueOf(0);	
		}
		return richiami3;
	}


	public void setRichiami3(Long richiami3) {
		this.richiami3 = richiami3;
	}


	public Long getRichiami4() {
		if(ApplicationUtil.ltZero(richiami4)) {
			richiami4 = Long.valueOf(0);	
		}
		return richiami4;
	}


	public void setRichiami4(Long richiami4) {
		this.richiami4 = richiami4;
	}

	public Long getArchiviati() {
		if(ApplicationUtil.ltZero(archiviati)) {
			archiviati = Long.valueOf(0);	
		}
		return archiviati;
	}


	public void setArchiviati(Long archiviati) {
		this.archiviati = archiviati;
	}
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((eid == null) ? 0 : eid.hashCode());
		return result;
	}


	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		RiepilogoCampagnaList other = (RiepilogoCampagnaList) obj;
		if (eid == null) {
			if (other.eid != null)
				return false;
		} else if (!eid.equals(other.eid))
			return false;
		return true;
	}



}