package it.callite.model.report;

import java.io.Serializable;

import it.callite.constants.ConstantsLanguage;
import it.callite.utility.ApplicationUtil;
@SuppressWarnings("serial")
public class RiepilogoEsito implements Serializable, Comparable<RiepilogoEsito> {

	private Long eid;
	private String esito;
	private String codice;	
	private String colore;	
	private Long numero;		
	private Long totale;
	private Boolean positivo;		


	public String getColore() {
		return colore;
	}

	public void setColore(String colore) {
		this.colore = colore;
	}
	
	public String getCodice() {
		return codice;
	}


	public void setCodice(String codice) {
		this.codice = codice;
	}


	public String getEtichettaCodice() {
		StringBuilder sb = new StringBuilder();
		sb.append(ApplicationUtil.trimToEmpty(esito));
		if(ApplicationUtil.isNotBlank(codice)) {
			sb.append(" (" + codice + ")");
		}
		return sb.toString();
	}

	
	public RiepilogoEsito() { }

	public RiepilogoEsito(String esito, String codice, String colore, Long numero, boolean positivo) {
		this.esito = esito;
		this.codice = codice;	
		this.colore = colore;	
		this.numero = numero;
		this.positivo = positivo;
	}


	public RiepilogoEsito(String esito, String codice, String colore, Long numero) {
		this.esito = esito;
		this.codice = codice;	
		this.colore = colore;	
		this.numero = numero;
	}
	
	public RiepilogoEsito(String esito, String codice, String colore) {
		this.esito = esito;
		this.codice = codice;	
		this.colore = colore;	
	}
	
	
	public void addNumero(Long numero) {
		if(this.numero==null) this.numero = Long.valueOf(0);
		if(numero==null) numero = Long.valueOf(0);
		this.numero += numero;
	}


	public int compareTo(RiepilogoEsito o) {
		if(esito != null && o.esito != null) {
			return esito.compareTo(o.esito);
		} else if(esito != null && o.esito == null) {
			return -1;
		} else if(esito == null && o.esito != null) {
			return 1;
		} else {
			return 0;
		}
	}
	
	public Long getEid() {
		return eid;
	}
	public String getEsito() {
		return esito;
	}
	
	public String getDenominazione() {
		return getEsito();
	}


	public Long getNumero() {
		if(numero == null)
			numero = Long.valueOf(0);
		return numero;
	}

	
	public Boolean getPositivo() {
		return positivo;
	}
	public String getPositivoString() {
		if(positivo!=null && positivo) {
			return "(" + ConstantsLanguage.POSITIVO.stampa() + ")";
		} else if(positivo!=null && !positivo) {
			return "(" + ConstantsLanguage.NEGATIVO.stampa() + ")";
		} else return "";

	}

	public Long getTotale() {
		return totale;
	}
	
	public void setEid(Long eid) {
		this.eid = eid;
	}
	public void setEsito(String esito) {
		this.esito = esito;
	}
	public void setNumero(Long numero) {
		this.numero = numero;
	}

	public void setPositivo(Boolean positivo) {
		this.positivo = positivo;
	}

	
	public void setTotale(Long totale) {
		this.totale = totale;
	}


	@Override
	public String toString() {
		return ApplicationUtil.trimToEmpty(esito);
	}

}