package it.callite.model.sip;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.Table;
import javax.persistence.Transient;

import it.callite.constants.ConstantsGlobal;
import it.callite.model.generic.SelezionatoAbstract;
import it.callite.model.report.ColonnaModel;
import it.callite.utility.ApplicationUtil;
@SuppressWarnings("serial")
@Entity
//@Audited 
@Table(name = ConstantsGlobal.ts + "coda", indexes = { @Index(name= "coda", columnList = "data_creazione, name" ) })
public class Coda extends SelezionatoAbstract implements Serializable, Comparable<Coda> {
	
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY) 
	private Long id;
	private Date data_creazione = ApplicationUtil.date();
	private String name;
	private String musiconhold = ConstantsGlobal.musicclass;
	
	private String announce;
	private Integer announce_frequency = 5;
	private Integer announce_round_seconds = 0;
	private String announce_holdtime = "no";
	private String periodic_announce;
	private Integer periodic_announce_frequency = 60;
	
	private String queue_youarenext;
	private String queue_thereare;
	private String queue_callswaiting;
	private String queue_holdtime;
	private String queue_minutes;
	private String queue_seconds;
	private String queue_lessthan;
	private String queue_thankyou;
	private String queue_reporthold;
	
	private String context = "";
	private String monitor_format = "";
	private String monitor_type = "";
	private String eventmemberstatus = "yes";
	private String eventwhencalled = "yes";
	private String joinempty = "yes";
	private String leavewhenempty = "no";
	private String reportholdtime = "no";
	private String setinterfacevar = "no";
	private String ringinuse = "no";
	private String timeoutrestart = "no";
	private String strategy = "ringall";
	private String callgroup;
	private String pickupgroup;

	private Integer maxlen = 0;
	private Integer memberdelay = 0;
	private Integer retry = 3;
	private Integer servicelevel = 60;
	private Integer timeout = 120;
	private Integer weight = 0;
	private Integer wrapuptime = 5;
	
	@ManyToMany(fetch=FetchType.LAZY)
	@JoinTable(name = ConstantsGlobal.tp + "interno_coda", joinColumns=@JoinColumn(name="coda_id", nullable=false, updatable=true, insertable=true), inverseJoinColumns=@JoinColumn(name="interno_id", nullable=false, updatable=true, insertable=true)    )
	private List<SipBuddies> interni = new ArrayList<SipBuddies>();
	
	
	@Transient
	private ColonnaModel model;

	public ColonnaModel getModel() {
		if(model == null) {
			model = new ColonnaModel(id);
		}
		return model;
	}
	public void setModel(ColonnaModel model) {
		this.model = model;
	}
	
	public int compareTo(Coda o) {
		if(id != null && o.id != null) {
			return id.compareTo(o.id);
		} else if(id != null && o.id == null) {
			return -1;
		} else if(id == null && o.id != null) {
			return 1;
		} else {
			return 0;
		}
	}
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Coda other = (Coda) obj;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		return true;
	}
	public Coda getCopia() {
		Coda pr = new Coda();
		pr.setMusiconhold(this.getMusiconhold());
		pr.setAnnounce(this.getAnnounce());
		pr.setAnnounce_frequency(this.getAnnounce_frequency());
		pr.setAnnounce_holdtime(this.getAnnounce_holdtime());
		pr.setAnnounce_round_seconds(this.getAnnounce_round_seconds());
		pr.setQueue_youarenext(this.getQueue_youarenext());
		pr.setQueue_thereare(this.getQueue_thereare());
		pr.setQueue_callswaiting(this.getQueue_callswaiting());
		pr.setQueue_holdtime(this.getQueue_holdtime());
		pr.setQueue_minutes(this.getQueue_minutes());
		pr.setQueue_seconds(this.getQueue_seconds());
		pr.setQueue_lessthan(this.getQueue_lessthan());
		pr.setQueue_thankyou(this.getQueue_thankyou());
		pr.setQueue_reporthold(this.getQueue_reporthold());
		pr.setPeriodic_announce(this.getPeriodic_announce());
		pr.setPeriodic_announce_frequency(this.getPeriodic_announce_frequency());
		pr.setContext(this.getContext());
		pr.setMonitor_format(this.getMonitor_format());
		pr.setMonitor_type(this.getMonitor_type());
		pr.setEventmemberstatus(this.getEventmemberstatus());
		pr.setEventwhencalled(this.getEventwhencalled());
		pr.setJoinempty(this.getJoinempty());
		pr.setLeavewhenempty(this.getLeavewhenempty());
		pr.setReportholdtime(this.getReportholdtime());
		pr.setSetinterfacevar(this.getSetinterfacevar());
		pr.setRinginuse(this.getRinginuse());
		pr.setTimeoutrestart(this.getTimeoutrestart());
		pr.setMaxlen(this.getMaxlen());
		pr.setMemberdelay(this.getMemberdelay());
		pr.setRetry(this.getRetry());
		pr.setServicelevel(this.getServicelevel());
		pr.setTimeout(this.getTimeout());
		pr.setWeight(this.getWeight());
		pr.setWrapuptime(this.getWrapuptime());
		pr.setStrategy(this.getStrategy());
		pr.setPickupgroup(this.getPickupgroup());
		pr.setCallgroup(this.getCallgroup());
		return pr;
	}
	public Date getData_creazione() {
		return data_creazione;
	}
	public Long getId() {
		return id;
	}
	public List<SipBuddies> getInterni() {
		return interni;
	}
	
	public String getMusiconhold() {
		return ApplicationUtil.trimToEmpty(musiconhold);
	}
	
	public String getName() {
		return ApplicationUtil.trimToEmpty(name);
	}
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		return result;
	}
	
	public void setData_creazione(Date data_creazione) {
		this.data_creazione = data_creazione;
	}
	public void setId(Long id) {
		this.id = id;
	}
	public void setInterni(List<SipBuddies> interni) {
		this.interni = interni;
	}
	public void setMusiconhold(String musiconhold) {
		this.musiconhold = ApplicationUtil.normalizeToEmpty(musiconhold);
	}
	public void setName(String name) {
		this.name = ApplicationUtil.normalizeToEmpty(name);
	}
	@Override
	public String toString() {
		return ApplicationUtil.toString(id, getDenominazione());
	}
	public String getDenominazione() {
		return ApplicationUtil.trimToEmpty(getName());
	}


	public String getAnnounce() {
		return announce;
	}


	public void setAnnounce(String announce) {
		this.announce = ApplicationUtil.normalizeToEmpty(announce);
	}


	public String getQueue_youarenext() {
		return ApplicationUtil.trimToEmpty(queue_youarenext);
	}


	public void setQueue_youarenext(String queue_youarenext) {
		this.queue_youarenext = ApplicationUtil.normalizeToEmpty(queue_youarenext);
	}


	public String getQueue_thereare() {
		return ApplicationUtil.trimToEmpty(queue_thereare);
	}


	public void setQueue_thereare(String queue_thereare) {
		this.queue_thereare = ApplicationUtil.normalizeToEmpty(queue_thereare);
	}


	public String getQueue_callswaiting() {
		return queue_callswaiting;
	}


	public void setQueue_callswaiting(String queue_callswaiting) {
		this.queue_callswaiting = ApplicationUtil.normalizeToEmpty(queue_callswaiting);
	}


	public String getQueue_holdtime() {
		return ApplicationUtil.trimToEmpty(queue_holdtime);
	}


	public void setQueue_holdtime(String queue_holdtime) {
		this.queue_holdtime = ApplicationUtil.normalizeToEmpty(queue_holdtime);
	}


	public String getQueue_minutes() {
		return ApplicationUtil.trimToEmpty(queue_minutes);
	}


	public void setQueue_minutes(String queue_minutes) {
		this.queue_minutes = ApplicationUtil.normalizeToEmpty(queue_minutes);
	}


	public String getQueue_seconds() {
		return ApplicationUtil.trimToEmpty(queue_seconds);
	}


	public void setQueue_seconds(String queue_seconds) {
		this.queue_seconds = ApplicationUtil.normalizeToEmpty(queue_seconds);
	}


	public String getQueue_lessthan() {
		return ApplicationUtil.trimToEmpty(queue_lessthan);
	}


	public void setQueue_lessthan(String queue_lessthan) {
		this.queue_lessthan = ApplicationUtil.normalizeToEmpty(queue_lessthan);
	}


	public String getQueue_thankyou() {
		return ApplicationUtil.trimToEmpty(queue_thankyou);
	}


	public void setQueue_thankyou(String queue_thankyou) {
		this.queue_thankyou = ApplicationUtil.normalizeToEmpty(queue_thankyou);
	}


	public String getQueue_reporthold() {
		return ApplicationUtil.trimToEmpty(queue_reporthold);
	}


	public void setQueue_reporthold(String queue_reporthold) {
		this.queue_reporthold = ApplicationUtil.normalizeToEmpty(queue_reporthold);
	}


	public String getPeriodic_announce() {
		return ApplicationUtil.trimToEmpty(periodic_announce);
	}


	public void setPeriodic_announce(String periodic_announce) {
		this.periodic_announce = ApplicationUtil.normalizeToEmpty(periodic_announce);
	}


	public Integer getPeriodic_announce_frequency() {
		return periodic_announce_frequency;
	}


	public void setPeriodic_announce_frequency(Integer periodic_announce_frequency) {
		this.periodic_announce_frequency = periodic_announce_frequency;
	}


	public Integer getAnnounce_frequency() {
		return announce_frequency;
	}


	public void setAnnounce_frequency(Integer announce_frequency) {
		this.announce_frequency = announce_frequency;
	}


	public Integer getAnnounce_round_seconds() {
		return announce_round_seconds;
	}


	public void setAnnounce_round_seconds(Integer announce_round_seconds) {
		this.announce_round_seconds = announce_round_seconds;
	}


	public Boolean getAnnounce_holdtime() {
		return ApplicationUtil.convertSB(announce_holdtime);
	}


	public void setAnnounce_holdtime(Boolean announce_holdtime) {
		this.announce_holdtime = ApplicationUtil.convertBS(announce_holdtime);
	}


	public String getContext() {
		return context;
	}


	public void setContext(String context) {
		this.context = context;
	}


	public String getMonitor_format() {
		return monitor_format;
	}


	public void setMonitor_format(String monitor_format) {
		this.monitor_format = monitor_format;
	}


	public String getMonitor_type() {
		return monitor_type;
	}


	public void setMonitor_type(String monitor_type) {
		this.monitor_type = monitor_type;
	}


	public String getEventmemberstatus() {
		return eventmemberstatus;
	}


	public void setEventmemberstatus(String eventmemberstatus) {
		this.eventmemberstatus = eventmemberstatus;
	}


	public String getEventwhencalled() {
		return eventwhencalled;
	}


	public void setEventwhencalled(String eventwhencalled) {
		this.eventwhencalled = eventwhencalled;
	}


	public String getJoinempty() {
		return joinempty;
	}


	public void setJoinempty(String joinempty) {
		this.joinempty = joinempty;
	}


	public String getLeavewhenempty() {
		return leavewhenempty;
	}


	public void setLeavewhenempty(String leavewhenempty) {
		this.leavewhenempty = leavewhenempty;
	}


	public Boolean getReportholdtime() {
		return ApplicationUtil.convertSB(reportholdtime);
	}


	public void setReportholdtime(Boolean reportholdtime) {
		this.reportholdtime = ApplicationUtil.convertBS(reportholdtime);
	}


	public String getSetinterfacevar() {
		return setinterfacevar;
	}


	public void setSetinterfacevar(String setinterfacevar) {
		this.setinterfacevar = setinterfacevar;
	}


	public Boolean getRinginuse() {
		return ApplicationUtil.convertSB(ringinuse);
	}


	public void setRinginuse(Boolean ringinuse) {
		this.ringinuse = ApplicationUtil.convertBS(ringinuse);
	}


	public String getTimeoutrestart() {
		return timeoutrestart;
	}


	public void setTimeoutrestart(String timeoutrestart) {
		this.timeoutrestart = timeoutrestart;
	}


	public Integer getMaxlen() {
		return maxlen;
	}


	public void setMaxlen(Integer maxlen) {
		this.maxlen = maxlen;
	}


	public Integer getMemberdelay() {
		return memberdelay;
	}


	public void setMemberdelay(Integer memberdelay) {
		this.memberdelay = memberdelay;
	}


	public Integer getRetry() {
		return retry;
	}


	public void setRetry(Integer retry) {
		this.retry = retry;
	}


	public Integer getServicelevel() {
		return servicelevel;
	}


	public void setServicelevel(Integer servicelevel) {
		this.servicelevel = servicelevel;
	}


	public Integer getTimeout() {
		return timeout;
	}


	public void setTimeout(Integer timeout) {
		this.timeout = timeout;
	}


	public Integer getWeight() {
		return weight;
	}


	public void setWeight(Integer weight) {
		this.weight = weight;
	}


	public Integer getWrapuptime() {
		return wrapuptime;
	}


	public void setWrapuptime(Integer wrapuptime) {
		this.wrapuptime = wrapuptime;
	}


	public String getStrategy() {
		return strategy;
	}


	public void setStrategy(String strategy) {
		this.strategy = strategy;
	}
	public String getCallgroup() {
		return ApplicationUtil.trimToEmpty(callgroup);
	}
	public void setCallgroup(String callgroup) {
		this.callgroup = ApplicationUtil.trimToEmpty(callgroup);
	}
	public String getPickupgroup() {
		return ApplicationUtil.trimToEmpty(pickupgroup);
	}
	public void setPickupgroup(String pickupgroup) {
		this.pickupgroup = ApplicationUtil.trimToEmpty(pickupgroup);
	}
	
	
	
}