package it.callite.service;
import java.io.Serializable;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.quartz.CronScheduleBuilder;
import org.quartz.CronTrigger;
import org.quartz.JobBuilder;
import org.quartz.JobDetail;
import org.quartz.JobKey;
import org.quartz.Scheduler;
import org.quartz.SchedulerException;
import org.quartz.Trigger;
import org.quartz.TriggerBuilder;
import org.quartz.TriggerKey;
import org.quartz.impl.StdSchedulerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.stereotype.Service;

import it.callite.constants.ConstantsMap;
import it.callite.dao.InvoiceDao;
import it.callite.log.ErrorLog;
import it.callite.model.Invoice;
import it.callite.scheduler.LanciaInvoice;
import it.callite.utility.ApplicationUtil;
@SuppressWarnings("serial")
@Service("invoiceService")
public class InvoiceService implements ApplicationContextAware, Serializable {

	@Autowired
	private InvoiceDao invoiceDao;
	private Scheduler schedulerAppl;
	private Map<String, TriggerKey> mapTrigger = new HashMap<String, TriggerKey>();
	private Map<String, JobKey> mapJob = new HashMap<String, JobKey>();

	public void setApplicationContext(ApplicationContext context)  {
		try {	
			schedulerAppl = StdSchedulerFactory.getDefaultScheduler();
			for(Invoice sa: invoiceDao.trovaTutti()) {
				avvia(sa);
			}
			schedulerAppl.start();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}

	public void avvia(Invoice sa)   {
		String trigger = sa.getNome();
		TriggerKey tk = new TriggerKey(trigger, ConstantsMap.group_invoice);
		JobKey jk = new JobKey(sa.getId().toString(), ConstantsMap.group_invoice);
		mapTrigger.put(trigger, tk);
		mapJob.put(trigger, jk);
		JobDetail job = creaJob(sa);
		schedulaTrigger(sa, job, tk);
		if(!sa.getStarted()) {
			pauseTrigger(tk);
		} else {
			resumeTrigger(tk);
		}
	}

	public void elimina(Invoice sa) {
		String trigger = sa.getNome();
		invoiceDao.elimina(sa);
		JobKey jk = mapJob.get(trigger);
		deleteJob(jk);
		TriggerKey tk =  mapTrigger.get(trigger);
		if(sa.getStarted()) {
			pauseTrigger(tk);
		}
	}

	public void salva(Invoice sa) {
		invoiceDao.salva(sa);
		String trigger = sa.getNome();
		TriggerKey tk = new TriggerKey(trigger, ConstantsMap.group_invoice);
		JobKey jk = new JobKey(sa.getId().toString(), ConstantsMap.group_invoice);
		mapTrigger.put(trigger, tk);
		mapJob.put(trigger, jk);
		JobDetail job = creaJob(sa);
		schedulaTrigger(sa, job, tk);
		if(!sa.getStarted()) {
			pauseTrigger(tk);
		} else {
			resumeTrigger(tk);
		}
	}

	public void modifica(Invoice sa) {
		String trigger = sa.getNome();
		invoiceDao.salva(sa);
		JobKey jk = mapJob.get(trigger);
		deleteJob(jk);
		JobDetail job = creaJob(sa);
		TriggerKey tk =  mapTrigger.get(trigger);
		schedulaTrigger(sa, job, tk);
		if(!sa.getStarted()) {
			pauseTrigger(tk);
		} else {
			resumeTrigger(tk);
		}
	}
	
	public void inviaTriggerData(Invoice sa, Date orario) throws SchedulerException   {
		if(orario != null) {
			String id = System.currentTimeMillis() + "";
			TriggerKey tk = new TriggerKey(sa.getNome(), id);
			JobKey jk = new JobKey(id, id);
			JobDetail job = JobBuilder.newJob(LanciaInvoice.class).withIdentity(jk).build();
			job.getJobDataMap().put("invoiceDao", invoiceDao);
			Trigger trigger = (Trigger) TriggerBuilder.newTrigger().withIdentity(tk).startAt(orario).forJob(job).build();
			schedulerAppl.scheduleJob(job, trigger);
		}
	}

	public void stop() {
		try {
			schedulerAppl.standby();
		} catch (SchedulerException e) {
		}
	}
	public void pauseTrigger(TriggerKey tk)   {
		try {
			if(tk != null) {
				schedulerAppl.pauseTrigger(tk);
				//schedulerAppl.unscheduleJob(tk);
			}
		} catch (SchedulerException e) {
		}
	}
	public void resumeTrigger(TriggerKey tk)   {
		try {
			if(tk != null) {
				schedulerAppl.resumeTrigger(tk);
				//schedulerAppl.unscheduleJob(tk);
			}
		} catch (SchedulerException e) {
		}
	}
	public void deleteJob(JobKey jk)   {
		try {
			if(jk != null) {
				schedulerAppl.deleteJob(jk);
			}
		} catch (SchedulerException e) {
		}
	}
	public void scheduleJob(JobDetail job, Trigger trigger)   {
		try {
			schedulerAppl.scheduleJob(job, trigger);
		} catch (SchedulerException e) {
		}
	}

	public JobDetail creaJob(Invoice sa)  {
		String trigger = sa.getNome();
		JobKey jk = mapJob.get(trigger);
		JobDetail job = JobBuilder.newJob(LanciaInvoice.class).withIdentity(jk).build();
		job.getJobDataMap().put("invoiceDao", invoiceDao);
		return job;
	}

	public void schedulaTrigger(Invoice sa, JobDetail job , TriggerKey tk)  {
		String orario = ApplicationUtil.eq(sa.getTipo(), 0) ? ("0 0 " +sa.getOrario() + " * * ?") : sa.getOrarioString();
		CronScheduleBuilder csb = CronScheduleBuilder.cronSchedule(orario);
		Trigger trigger = (CronTrigger) TriggerBuilder.newTrigger().withIdentity(tk).withSchedule(csb).forJob(job).build();
		scheduleJob(job, trigger);
	}

	public void start(Invoice sa) {
		String trigger = sa.getNome();
		sa.setStarted(Boolean.TRUE);
		invoiceDao.salva(sa);
		TriggerKey tk = mapTrigger.get(trigger);
		resumeTrigger(tk);

	}

	public void stop(Invoice sa) {
		String trigger = sa.getNome();
		sa.setStarted(Boolean.FALSE);
		invoiceDao.salva(sa);
		TriggerKey tk =  mapTrigger.get(trigger);
		pauseTrigger(tk);
	}
	

	public void invia(Invoice sa)   {
		String trigger = sa.getNome();
		invoiceDao.lanciaInvoice(trigger);
	}

	public void avviaSelezionati(List<Invoice> lst) {
		Iterator<Invoice> itCont = lst.iterator();
		while (itCont.hasNext()) {
			start(itCont.next());
		}
	}
	public void fermaSelezionati(List<Invoice> lst) {
		Iterator<Invoice> itCont = lst.iterator();
		while (itCont.hasNext()) {
			stop(itCont.next());
		}
	}
	
	public void inviaSelezionati(List<Invoice> lst) {
		Iterator<Invoice> itCont = lst.iterator();
		while (itCont.hasNext()) {
			invia(itCont.next());
		}
	}
	
	public void eliminaSelezionati(List<Invoice> lst) {
		Iterator<Invoice> itCont = lst.iterator();
		while (itCont.hasNext()) {
			elimina(itCont.next());
		}
	}

}