package it.callite.service;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.ExecutionException;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.google.common.cache.Cache;

import it.callite.constants.ConstantsGlobal;
import it.callite.dao.ImpostazioneDao;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.LoggerLogin;
import it.callite.utility.MyCallable;


@SuppressWarnings("serial")
@Service("loginAttemptService")
public class LoginAttemptService implements Serializable {

	
	@Autowired
	private ImpostazioneDao impostazioneDao;
	
	private Cache<LoggerLogin, Integer> attemptsCache;

	public LoginAttemptService() {
		super();
		attemptsCache = MyCallable.create(ConstantsGlobal.TIME_BLOCCO);
	}

	public boolean isBlocked(String ip, String username) {
		if(ip != null && username != null) {
			Integer maxLogin = impostazioneDao.getMassimoLogin2();
			if(ApplicationUtil.gtZero(maxLogin)) {
				LoggerLogin key = new LoggerLogin(username, ip);
				int attempts = get(key);
				return attempts >= maxLogin.intValue();
			}
			return false;
		}
		return true;
	}


	public void loginFailed(String ip, String username) {
		if(ip != null && username != null) {
			LoggerLogin key = new LoggerLogin(username, ip);
			int attempts = get(key);
			attempts++;
			attemptsCache.put(key, attempts);
		}
	}


	public void loginSuccess(String ip, String username) {
		if(ip != null && username != null) {
			LoggerLogin key = new LoggerLogin(username, ip);
			attemptsCache.invalidate(key);
		}
	}

	public int get(String ip, String username) {
		if(ip != null && username != null) {
			LoggerLogin key = new LoggerLogin(username, ip);
			return  get(key);
		}
		return 0;
	}

	private int get(LoggerLogin key) {
		Integer attempts = 0;
		try {
			attempts = attemptsCache.get(key, new MyCallable());
		} catch (ExecutionException e) {
		}
		if(ApplicationUtil.ltZero(attempts)) {
			attempts = 0;
		}
		return attempts.intValue();
	}
	
	public synchronized List<LoggerLogin> table() {
		List<LoggerLogin> table = new ArrayList<LoggerLogin>();
		Map<LoggerLogin, Integer> mappa = attemptsCache.asMap();
		if(ApplicationUtil.notEmpty(mappa)) {
			Set<LoggerLogin> dataList = mappa.keySet();
			Iterator<LoggerLogin> dataIt = dataList.iterator();
			while (dataIt.hasNext()) {
				LoggerLogin tmp = dataIt.next();
				int attempts = get(tmp);
				table.add(new LoggerLogin(tmp.getUsername(), tmp.getIp(), tmp.getMotivo(), tmp.getData(), attempts));
			}
		}
		return table;
	}
	
	public synchronized void clear() {
		attemptsCache.invalidateAll();
	}


	public synchronized void remove(LoggerLogin ld) {
		attemptsCache.invalidate(ld);
	}

}