package it.callite.service;
import java.io.Serializable;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.validator.ValidatorException;

import org.quartz.CalendarIntervalScheduleBuilder;
import org.quartz.CalendarIntervalTrigger;
import org.quartz.CronScheduleBuilder;
import org.quartz.CronTrigger;
import org.quartz.JobBuilder;
import org.quartz.JobDetail;
import org.quartz.JobKey;
import org.quartz.Scheduler;
import org.quartz.SchedulerException;
import org.quartz.Trigger;
import org.quartz.TriggerBuilder;
import org.quartz.TriggerKey;
import org.quartz.impl.StdSchedulerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.stereotype.Service;

import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsMap;
import it.callite.dao.ComunicazioneDao;
import it.callite.dao.SchedulerDao;
import it.callite.log.ErrorLog;
import it.callite.message.Message;
import it.callite.model.Schedulatore;
import it.callite.scheduler.LanciaScheduler;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.thread.PostInvioComunicazioneThread;
@SuppressWarnings("serial")
@Service("schedulerService")
public class SchedulerService implements ApplicationContextAware, Serializable {

	@Autowired
	private SchedulerDao schedulerDao;
	@Autowired
	private ComunicazioneDao comunicazioneDao;
	private Scheduler schedulerAppl;
	private List<String> triggerName = ConstantsMap.getSchedulatori();
	private Map<String, TriggerKey> mapTrigger = new HashMap<String, TriggerKey>();
	private Map<String, JobKey> mapJob = new HashMap<String, JobKey>();

	public void setApplicationContext(ApplicationContext context)  {
		try {	
			Scheduler st = (Scheduler) context.getBean("schedulatore");
			st.start();
			schedulerAppl = StdSchedulerFactory.getDefaultScheduler();
			for(String trigger: triggerName) {
				creaTriggerCron(trigger);
			}
			List<Schedulatore>  dateTrigger = schedulerDao.getDateTrigger();
			Date oggi = ApplicationUtil.date();
			for(Schedulatore sa: dateTrigger) {
				Date orario = sa.getDateTime();
				if(orario.after(oggi)) {
					creaTriggerData(sa, orario);
				} else {
					Integer codice [] = new Integer [] {ConstantsGlobal.SCHEDULATA};
					invia_comunicazioni(sa, codice);
				}
			}
			schedulerAppl.start();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
		schedulerDao.eliminaNotInScheduler(triggerName);
	}
	public void stop() {
		try {
			schedulerAppl.standby();
		} catch (SchedulerException e) {
		}
	}
	public void pauseTrigger(TriggerKey tk)   {
		try {
			if(tk != null) {
				schedulerAppl.pauseTrigger(tk);
				//schedulerAppl.unscheduleJob(tk);
			}
		} catch (SchedulerException e) {
		}
	}
	public void resumeTrigger(TriggerKey tk)   {
		try {
			if(tk != null) {
				schedulerAppl.resumeTrigger(tk);
				//schedulerAppl.unscheduleJob(tk);
			}
		} catch (SchedulerException e) {
		}
	}
	public void deleteJob(JobKey jk)   {
		try {
			if(jk != null) {
				schedulerAppl.deleteJob(jk);
			}
		} catch (SchedulerException e) {
		}
	}
	public void scheduleJob(JobDetail job, Trigger trigger)   {
		try {
			schedulerAppl.scheduleJob(job, trigger);
		} catch (SchedulerException e) {
		}
	}
	
	public void validate_cron(FacesContext context, UIComponent component, Object input)  {
		try {
			CronScheduleBuilder.cronSchedule((String) input);
		} catch (Exception e) {
			throw new ValidatorException(Message.addValidatorMessage("formato.errato" , null));
		}
	}
	public void creaTriggerCron(String trigger)   {
		Schedulatore sa = schedulerDao.get(trigger);
		if(sa.getId() == null) {
			sa.setNome(trigger);
			schedulerDao.salva(sa);
		}
		TriggerKey tk = new TriggerKey(trigger, ConstantsMap.group_scheduler);
		JobKey jk = new JobKey(sa.getId().toString(), ConstantsMap.group_scheduler);
		mapTrigger.put(trigger, tk);
		mapJob.put(trigger, jk);
		JobDetail job = creaJob(sa);
		schedulaTriggerCron(sa, job, tk);
		if(!sa.getStarted()) {
			pauseTrigger(tk);
		} else {
			resumeTrigger(tk);
		}
	}
	public void creaTriggerData(Schedulatore sa, Date orario)   {
		TriggerKey tk = new TriggerKey(sa.getNome(), ConstantsMap.group_scheduler);
		JobKey jk = new JobKey(sa.getId().toString(), ConstantsMap.group_scheduler);
		mapTrigger.put(sa.getNome(), tk);
		mapJob.put(sa.getNome(), jk);
		JobDetail job = creaJob(sa);
		schedulaTriggerData(sa, job, tk, orario);
		if(!sa.getStarted()) {
			pauseTrigger(tk);
		} else {
			resumeTrigger(tk);
		}
	}

	public void inviaTriggerData(Schedulatore sa, Date orario) throws SchedulerException   {
		if(sa != null && orario != null) {
			String id = System.currentTimeMillis() + "";
			TriggerKey tk = new TriggerKey(sa.getNome(), id);
			JobKey jk = new JobKey(id, id);
			JobDetail job = JobBuilder.newJob(LanciaScheduler.class).withIdentity(jk).build();
			job.getJobDataMap().put("schedulerDao", schedulerDao);
			Trigger trigger = (Trigger) TriggerBuilder.newTrigger().withIdentity(tk).startAt(orario).forJob(job).build();
			schedulerAppl.scheduleJob(job, trigger);
		}
	}

	public Schedulatore lancia(Long id, Date data, int tipo)  {
		String trigger = ConstantsMap.send_message + "_" + data.getTime() + "_" + tipo;
		Schedulatore sa = schedulerDao.get(trigger);
		if(sa.getId() == null) {
			sa.setNome(trigger);
			sa.setTipo(tipo);
			sa.setOrario("" + data.getTime());
			sa.setData_schedulata(data);
			sa.setStarted(true);
			schedulerDao.salva(sa);
			creaTriggerData(sa,data);
		}
		return sa;
	}

	public void elimina(Schedulatore sa, boolean elimina, boolean aggiorna)   {
		if(sa != null) {
			if(elimina) {
				schedulerDao.cancellaAggiorna(sa);
			}
			if(aggiorna) {
				Integer codice [] = new Integer [] {ConstantsGlobal.SCHEDULATA, ConstantsGlobal.PAUSE};
				schedulerDao.aggiorna_comunicazioni(sa, ConstantsGlobal.ANNULLATA, codice);
			}
			String trigger = sa.getNome();
			TriggerKey tk =  mapTrigger.remove(trigger);
			pauseTrigger(tk);
			JobKey jk = mapJob.remove(trigger);
			deleteJob(jk);
		}

	}
	public void avvia(Schedulatore sa, boolean aggiorna) {
		if(sa != null) {
			if(aggiorna) {
				Integer codice [] = new Integer [] {ConstantsGlobal.PAUSE};
				schedulerDao.aggiorna_comunicazioni(sa, ConstantsGlobal.SCHEDULATA, codice);
			}
			sa.setStarted(Boolean.TRUE);
			schedulerDao.salva(sa);
			String trigger = sa.getNome();
			TriggerKey tk = mapTrigger.get(trigger);
			resumeTrigger(tk);
		}
	}
	public void ferma(Schedulatore sa, boolean aggiorna) {
		if(sa != null) {
			if(aggiorna) {
				Integer codice [] = new Integer [] {ConstantsGlobal.SCHEDULATA};
				schedulerDao.aggiorna_comunicazioni(sa, ConstantsGlobal.PAUSE, codice);
			}
			sa.setStarted(Boolean.FALSE);
			schedulerDao.salva(sa);
			String trigger = sa.getNome();
			TriggerKey tk =  mapTrigger.get(trigger);
			pauseTrigger(tk);
		}
	}
	public void invia(Schedulatore sa)   {
		schedulerDao.lanciaScheduler(sa.getNome());
	}
	public JobDetail creaJob(Schedulatore sa)  {
		JobKey jk = mapJob.get(sa.getNome());
		JobDetail job = JobBuilder.newJob(LanciaScheduler.class).withIdentity(jk).build();
		job.getJobDataMap().put("schedulerDao", schedulerDao);
		return job;
	}
	public void salvaModificaCron(Schedulatore sa) {
		schedulerDao.salva(sa);
		JobKey jk = mapJob.get(sa.getNome());
		deleteJob(jk);
		JobDetail job = creaJob(sa);
		TriggerKey tk =  mapTrigger.get(sa.getNome());
		schedulaTriggerCron(sa, job, tk);
		if(!sa.getStarted()) {
			pauseTrigger(tk);
		} else {
			resumeTrigger(tk);
		}
	}
	public void salvaModificaData(Schedulatore sa, Date data) {
		if(sa != null && data != null) {
			Integer codice [] = new Integer [] {ConstantsGlobal.SCHEDULATA, ConstantsGlobal.PAUSE};
			schedulerDao.aggiorna_comunicazioni(sa, data, codice);
			sa.setOrario("" + data.getTime());
			sa.setData_schedulata(data);
			schedulerDao.salva(sa);
			String trigger = sa.getNome();
			JobKey jk = mapJob.get(trigger);
			deleteJob(jk);
			JobDetail job = creaJob(sa);
			TriggerKey tk =  mapTrigger.get(trigger);
			schedulaTriggerData(sa, job, tk, sa.getDateTime());
			if(!sa.getStarted()) {
				pauseTrigger(tk);
			} else {
				resumeTrigger(tk);
			}
		}
	}
	public void schedulaTriggerCron(Schedulatore sa, JobDetail job , TriggerKey tk)  {
		String orario = ApplicationUtil.eq(sa.getTipo(), 0) ? ("0 0 " +sa.getOrario() + " * * ?") : sa.getOrarioString();
		CronScheduleBuilder csb = CronScheduleBuilder.cronSchedule(orario);
		Trigger trigger = (CronTrigger) TriggerBuilder.newTrigger().withIdentity(tk).withSchedule(csb).forJob(job).build();
		scheduleJob(job, trigger);
	}
	public void schedulaTriggerData(Schedulatore sa, JobDetail job , TriggerKey tk, Date start) {
		if(start != null) {
			Trigger trigger = (Trigger) TriggerBuilder.newTrigger().withIdentity(tk).startAt(start).forJob(job).build();
			scheduleJob(job, trigger);
		}
	}
	public void schedulaTriggerData(Schedulatore sa, JobDetail job , TriggerKey tk, Date start, Date stop, Integer cron) {
		if(ApplicationUtil.gtZero(cron) && start != null){
			//Date oggi = ApplicationUtil.date();
			//Date data_inizio = oggi.before(start) ? start : oggi;
			//if(stop == null || (stop !=null && data_inizio.before(stop))) {
			int interval = 1;
			CalendarIntervalScheduleBuilder csb = CalendarIntervalScheduleBuilder.calendarIntervalSchedule();
			switch (cron.intValue()) {
			case 1: csb.withIntervalInYears(interval); 	break;
			case 2: csb.withIntervalInMonths(interval);	break;
			case 3: csb.withIntervalInWeeks(interval); 	break;
			case 4: csb.withIntervalInDays(interval); 	break;
			case 5: csb.withIntervalInHours(interval); 	break;
			}
			TriggerBuilder<CalendarIntervalTrigger> tb = TriggerBuilder.newTrigger().withIdentity(tk).startAt(start).withSchedule(csb);
			if(stop !=null) {
				tb.endAt(stop);
			}
			Trigger trigger = (Trigger) tb.forJob(job).build();
			scheduleJob(job, trigger);
			//}
		}
	}
	public void  eliminaSelezionati(List<Schedulatore> lst, boolean elimina, boolean aggiorna) {
		Iterator<Schedulatore> itCont = lst.iterator();
		while (itCont.hasNext()) {
			elimina(itCont.next(), elimina, aggiorna);
		}
		if(!elimina) {
			schedulerDao.cancellaLista(lst);
		}
	}
	public void  avviaSelezionati(List<Schedulatore> lst, boolean aggiorna) {
		Iterator<Schedulatore> itCont = lst.iterator();
		while (itCont.hasNext()) {
			avvia(itCont.next(), aggiorna);
		}
	}
	public void  fermaSelezionati(List<Schedulatore> lst, boolean aggiorna) {
		Iterator<Schedulatore> itCont = lst.iterator();
		while (itCont.hasNext()) {
			ferma(itCont.next(), aggiorna);
		}
	}
	public void  inviaSelezionati(List<Schedulatore> lst) {
		Iterator<Schedulatore> itCont = lst.iterator();
		while (itCont.hasNext()) {
			invia(itCont.next());
		}
	}
	
	public void invia_comunicazioni(Schedulatore sa, Integer codice [])  {
		new PostInvioComunicazioneThread(sa, codice, comunicazioneDao, this).start();
	}
}