package it.callite.utility;

import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.select.Elements;

import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.model.Anagrafica;
import it.callite.model.CampoAggiuntivoAnagrafica;
import it.callite.model.DatoAggiuntivoAnagrafica;
import it.callite.model.Indirizzo;
import it.callite.utility.importa.ImportModelAnagrafica;


public class AnagraficaPagineUtil {


	public static List<AnagraficaPagine> pagineBianche(boolean esegui, int tipo, String ricerca, String zona, int p) throws Exception  {
		int limit = 20;
		Integer pagine = null;
		int pagina = 0;
		List<AnagraficaPagine> lista = new ArrayList<AnagraficaPagine>();
		String urlRicerca = URLEncoder.encode(ricerca, ConstantsGlobal.CHARSET);
		do{ 
			pagina++;
			String url = getUrlBianche(tipo, urlRicerca, zona, pagina);
			Document doc = Jsoup.connect(url).userAgent(ConstantsGlobal.browser).timeout(ConstantsGlobal.timeout).ignoreContentType(true).ignoreHttpErrors(true).followRedirects(true).get();
			if(pagine == null ) {
				pagine = getNumero_pagine(p, limit);
			}
			Elements el1 = doc.getElementsByClass("list-element__content");
			Iterator<Element> it = el1.iterator();
			while (it.hasNext()) {
				Element figli = (Element) it.next();
				AnagraficaPagine an = new AnagraficaPagine();
				an.setPf(true);
				an.setLink("");

				Elements address = doc.select("div.list-element__address > span > span > span");
				Element a1 = address.size() > 0 ? address.get(0) : null;
				Element a2 = address.size() > 1 ? address.get(1) : null;
				Element a3 = address.size() > 2 ? address.get(2) : null;
				Element a4 = address.size() > 3 ? address.get(3) : null;
				an.setComune(ApplicationUtil.upperCaseEmpty(a3 != null ? a3.text() : ""));
				an.setProvincia(ApplicationUtil.upperCaseEmpty(ApplicationUtil.substringBetween(a4 != null ? a4.text() : "", "(", ")")));
				an.setIndirizzo(ApplicationUtil.upperCaseEmpty(ApplicationUtil.substringBefore(ApplicationUtil.upperCaseEmpty(a1 != null ? a1.text() : ""), "-")));
				an.setCap(ApplicationUtil.trimToEmpty(a2 != null ? a2.text() : "")); 
				an.setNome_completo(ApplicationUtil.trimToEmpty(figli.getElementsByClass("list-element__title").text()));
				an.setCategoria(ApplicationUtil.upperCaseEmpty(figli.getElementsByClass("list-element__category").text()));
				Elements tel = figli.select("div.tel");
				Iterator<Element> itEl =  tel.iterator();
				while (itEl.hasNext()) {
					Element element = (Element) itEl.next();
					String telefono = ApplicationUtil.trimToEmpty(element.text()).replaceAll(" ", "");
					if(telefono.startsWith(ConstantsGlobal.mobile)){
						an.addMobile(telefono);
					} else {
						an.addFisso(telefono);
					}
				}

				if(ApplicationUtil.notEmpty(an.getFisso()) || ApplicationUtil.notEmpty(an.getMobile())) {
					an.prepareTelefono(esegui);
					if(!lista.contains(an)) {
						lista.add(an);
					}
				}
			}
		} while (pagina < pagine) ;

		return lista;
	}


	public static List<AnagraficaPagine> pagineGialle(boolean esegui, int tipo, String ricerca, String zona, int p) throws Exception  {
		int limit = 50;
		Integer pagine = null;
		int pagina = 0;
		List<AnagraficaPagine> lista = new ArrayList<AnagraficaPagine>();
		String urlRicerca = URLEncoder.encode(ricerca, ConstantsGlobal.CHARSET);
		do{ 
			pagina++;
			String url = getUrlGialle(tipo, urlRicerca, zona, pagina);
			Document doc = Jsoup.connect(url).userAgent(ConstantsGlobal.browser).timeout(ConstantsGlobal.timeout).ignoreContentType(true).ignoreHttpErrors(true).followRedirects(true).get();
			if(pagine == null) {
				pagine = getNumero_pagine(p, limit);
			}
			Elements el1 = doc.getElementsByAttributeValue("data-tr", "listing-search-itm");
			Iterator<Element> it = el1.iterator();
			while (it.hasNext()) {
				Element figli = (Element) it.next();
				AnagraficaPagine an = new AnagraficaPagine();
				an.setPf(false);
				Elements lnk = figli.select("div.search-itm__info > div > a");
				if(lnk != null) {
					an.setLink(ApplicationUtil.trimToEmpty(lnk.attr("href")));
				}
				Elements adds = figli.getElementsByAttributeValue("data-tr", "listing-search-itm-adr");
				if(adds != null) {
					an.setIndirizzo(ApplicationUtil.upperCaseEmpty(adds.text()));
					an.setCap(ApplicationUtil.trimToEmpty(""));
					an.setComune(ApplicationUtil.upperCaseEmpty(""));
					an.setProvincia(ApplicationUtil.upperCaseEmpty(""));
				}
				String title = ApplicationUtil.trimToNull(figli.getElementsByAttributeValue("data-tr", "listing-search-itm-rag").text());
				if(title != null) {
					an.setRagione_sociale(title);
					String categoria = figli.getElementsByAttributeValue("data-tr", "listing-search-itm-cat").text();
					an.setCategoria(ApplicationUtil.upperCaseEmpty(categoria));
					Elements tel = figli.select("div.search-itm__phone > ul > li");
					Iterator<Element> itEl =  tel.iterator();
					while (itEl.hasNext()) {
						Element element = (Element) itEl.next();
						String telefoni = ApplicationUtil.trimToEmpty(element.text()).replaceAll(" ", "");
						List<String> tele = ApplicationUtil.token(telefoni, ",");
						for(String t : tele) {
							String telefono = ApplicationUtil.trimToEmpty(t);
							if(telefono.startsWith(ConstantsGlobal.mobile)){
								an.addMobile(telefono);
							} else {
								an.addFisso(telefono);
							}
						}
					}

					if(ApplicationUtil.notEmpty(an.getFisso()) || ApplicationUtil.notEmpty(an.getMobile())) {
						an.prepareTelefono(esegui);
						if(!lista.contains(an)) {
							lista.add(an);
						}
					}
				}
			}
		} while (pagina < pagine) ;
		return lista;
	}

	public static List<Anagrafica> aggiustaAnagrafica(Integer oC, String importaFile, List<AnagraficaPagine> anagraficheP, List<String> opposizioni, List<String> duplicati_fisso, List<String> duplicati_mobile, List<String> duplicati_fax, List<String> duplicati_altro,List<String> duplicati_email,List<String> duplicati_link,List<String> duplicati_codice,List<String> duplicati_partita, ImportModelAnagrafica importModel,  Map<String, String> province, Map<String, String> comuni) throws Exception {
		List<Anagrafica> anagrafiche = new ArrayList<Anagrafica>();
		Date dataC = ApplicationUtil.date();
		Iterator<AnagraficaPagine> it = anagraficheP.iterator();
		long i = 0;
		while (it.hasNext()) {
			AnagraficaPagine ap = (AnagraficaPagine) it.next();
			Anagrafica anagrafica = new Anagrafica();
			anagrafica.setPf(ap.getPf());
			anagrafica.setTelefono_fisso(ap.getTelefono_fisso());
			anagrafica.setTelefono_mobile(ap.getTelefono_mobile());
			anagrafica.setTelefono_fax(ap.getTelefono_fax());
			anagrafica.setTelefono_altro(ap.getTelefono_altro());
			anagrafica.setEmail(ap.getEmail());
			anagrafica.setLink(ap.getLink());
			anagrafica.setSettore(ap.getCategoria());
			if(ApplicationUtil.notEmpty(ap.getAltri())) {
				int j = 1;
				for(String a : ap.getAltri()) {
					String tagg = ApplicationUtil.trimToNull(a);
					if(tagg != null) {
						String chiave = ConstantsLanguage.TELEFONO.stampa();
						String c = chiave + (j++);
						CampoAggiuntivoAnagrafica ca = ApplicationUtil.nuovoCampo(c, true);
						DatoAggiuntivoAnagrafica da = new DatoAggiuntivoAnagrafica();
						da.setCampo_aggiuntivo(ca);
						da.setValore(tagg);
						anagrafica.addDato(c, da);
					}
				}
			}
			if(ap.getPf()) {
				anagrafica.setNome_completo(ap.getNome_completo());
			} else {
				anagrafica.setRagione_sociale(ap.getRagione_sociale());
			}
			Indirizzo indirizzo = new Indirizzo();
			indirizzo.setCap(ap.getCap());
			indirizzo.setComune(ap.getComune());
			indirizzo.setProvincia(ap.getProvincia());
			indirizzo.setStrada(ap.getIndirizzo());
			anagrafica.setIndirizzo_legale(indirizzo);
			anagrafica = ApplicationUtil.aggiustaNumeri(oC, null, null, anagrafica, importModel, opposizioni, duplicati_fisso, duplicati_mobile, duplicati_fax, duplicati_altro,duplicati_email,duplicati_link,duplicati_codice,duplicati_partita, province,  comuni, null, null, null);
			boolean iok = ApplicationUtil.importCondition(dataC, importaFile, anagrafica, importModel);
			if(iok) {
				anagrafica.setSeq(++i);
				anagrafiche.add(anagrafica);
			}
		}
		return anagrafiche;
	}

	private static String getUrlBianche(int tipo, String ricerca, String zonaS, int numero) throws Exception {
		String zona = URLEncoder.encode(zonaS, ConstantsGlobal.CHARSET);
		String url = "";
		if(tipo == 0) {
			url =  "https://www.paginebianche.it/ricerca-da-indirizzo?dv="+zona+"+"+ricerca+"&p="+numero;
		} 
		if(tipo == 1) {
			url =   "https://www.paginebianche.it/persone?qs="+ricerca+"&dv="+zona+"&p="+numero;
		}
		if(tipo == 2) {
			url =   "https://www.paginebianche.it/ricerca-da-numero?qs="+ricerca;
		}
		return url;
	}

	private static String getUrlGialle(int tipo, String ricerca, String zonaS, int numero) throws Exception{
		String zona = URLEncoder.encode(zonaS, ConstantsGlobal.CHARSET);
		String url = "";
		if(tipo == 0) {
			url =  "https://www.paginegialle.it/ricerca/indirizzo-"+ricerca+"+"+zona+"/p-"+numero + "?rk=1&mr=50";
		}
		if(tipo == 1) {
			url =  "https://www.paginegialle.it/ricerca/"+ ricerca + "/"+zona+"/p-"+numero + "?rk=1&mr=50";
		}
		if(tipo == 2) {
			url =  "https://www.paginegialle.it/ricerca/telefono-" + ricerca ;
		}
		return url;
	}

	private static int getNumero_pagine(int totale_risultati, int righePagina){
		if(ApplicationUtil.gtZero(totale_risultati)) {
			return (totale_risultati/righePagina) + ((totale_risultati % righePagina > 0) ? 1 : 0);
		}
		else return 0;
	}


}
