package it.callite.utility;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.apache.commons.collections4.ListUtils;

import au.com.bytecode.opencsv.CSVWriter;
import it.callite.constants.ConstantsFormat;
import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsMap;
import it.callite.dao.AnagraficaDao;
import it.callite.message.Message;
import it.callite.model.Anagrafica;
import it.callite.model.Appuntamento;
import it.callite.model.CampoAggiuntivoAnagrafica;
import it.callite.model.CampoAggiuntivoContratto;
import it.callite.model.Contatto;
import it.callite.model.Contratto;
import it.callite.model.DatoAggiuntivoAnagrafica;
import it.callite.model.DatoAggiuntivoContratto;
import it.callite.model.Invoice;
import it.callite.model.StoriaAppuntamento;
import it.callite.model.StoriaContatto;
import it.callite.model.StoriaContratto;
import it.callite.model.TemplateExportAnagrafica;
import it.callite.model.TemplateExportAppuntamento;
import it.callite.model.TemplateExportContatto;
import it.callite.model.TemplateExportContratto;
import it.callite.model.TemplateExportStoriaAppuntamento;
import it.callite.model.TemplateExportStoriaContatto;
import it.callite.model.TemplateExportStoriaContratto;
public class CSVExport {
	
	public static void esportaCSVAnagraficaTemplate(OutputStream out, List<Anagrafica> data, AnagraficaDao anagraficaDao, TemplateExportAnagrafica ti) throws Exception {
		List <String> titlesList = ti.getValore_multiplo();
		List <String> aggiuntivi = ListUtils.subtract(titlesList, new ArrayList<String>(ConstantsMap.anagraficaMap));
		if(ApplicationUtil.notEmpty(aggiuntivi)) {
			data = anagraficaDao.loadFullAnagrafica(data);
		}
		String[] titles = titlesList.toArray(new String[0]);
		CSVWriter writer = new CSVWriter(new OutputStreamWriter(out, ConstantsGlobal.CHARSET),ti.getSeparatoreChar(), ti.getQuoteChar());
		List<String> header = new ArrayList<String>();
		if(ti.getIntestazione()) {
			for (int i = 0; i < titles.length; i++) {
				if(ApplicationUtil.notEmpty(aggiuntivi) && aggiuntivi.contains(titles[i])) {
					String etichetta = anagraficaDao.getEtichettaByChiave(CampoAggiuntivoAnagrafica.class, titles[i]);
					header.add(etichetta);
				} else {
					header.add(Message.addCatalogText(titles[i]));
				}
			}
		} 
		if(ApplicationUtil.notEmpty(header)) {
			writer.writeNext(header.toArray(new String[0]));
		}
		Iterator<Anagrafica> objIter = data.iterator();
		while(objIter.hasNext()) {
			Anagrafica prd = objIter.next();
			List<String> stringa = new ArrayList<String>();
			for (int j = 0; j < titles.length; j++) {
				if(ti.getPosizione(ConstantsMap.anagrafica_cognome).equals(j)) {
					stringa.add(prd.getCognome());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_nome).equals(j)) {
					stringa.add(prd.getNome());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_nome_completo).equals(j)) {
					stringa.add(prd.getNome_completo());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_email).equals(j)) {
					stringa.add(prd.getEmail());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_codice_fiscale).equals(j)) {
					stringa.add(prd.getCodice_fiscale());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_sorgente).equals(j)) {
					stringa.add(prd.getSorgente());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_nota).equals(j)) {
					stringa.add(prd.getNota());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_luogo_nascita).equals(j)) {
					stringa.add(prd.getLuogo_nascita());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_data_nascita).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.date.stampa(), prd.getDate_nascita()));
				} else if(ti.getPosizione(ConstantsMap.anagrafica_link).equals(j)) {
					stringa.add(prd.getLink());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_data_inserimento).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateTime.stampa(), prd.getData_creazione()));
				} else if(ti.getPosizione(ConstantsMap.anagrafica_privacy).equals(j)) {
					stringa.add(prd.getPrivacy());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_tipo).equals(j)) {
					stringa.add(prd.getTipo());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_sesso).equals(j)) {
					stringa.add(prd.getSesso());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_partita_iva).equals(j)) {
					stringa.add(prd.getPartita_iva());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_ragione_sociale).equals(j)) {
					stringa.add(prd.getRagione_sociale());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_telefono_fisso).equals(j)) {
					stringa.add(ApplicationUtil.uniforma(prd.getTelefono_fisso(), ti.getRem_fisso(), ti.getAdd_fisso()));
				} else if(ti.getPosizione(ConstantsMap.anagrafica_telefono_altro).equals(j)) {
					stringa.add(ApplicationUtil.uniforma(prd.getTelefono_altro(), ti.getRem_altro(), ti.getAdd_altro()));
				} else if(ti.getPosizione(ConstantsMap.anagrafica_telefono_mobile).equals(j)) {
					stringa.add(ApplicationUtil.uniforma(prd.getTelefono_mobile(), ti.getRem_mobile(), ti.getAdd_mobile()));
				} else if(ti.getPosizione(ConstantsMap.anagrafica_telefono_fax).equals(j)) {
					stringa.add(ApplicationUtil.uniforma(prd.getTelefono_fax(), ti.getRem_fax(), ti.getAdd_fax()));
				} else if(ti.getPosizione(ConstantsMap.anagrafica_settore).equals(j)) {
					stringa.add(prd.getSettore());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_denominazione).equals(j)) {
					stringa.add(prd.getDenominazione());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_id).equals(j)) {
					stringa.add(prd.getIdentificativo());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_indirizzo_completo).equals(j)) {
					stringa.add(prd.getIndirizzo_legale().getIndirizzo_completo());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_strada_completo).equals(j)) {
					stringa.add(prd.getIndirizzo_legale().getStrada_completo());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_comune_completo).equals(j)) {
					stringa.add(prd.getIndirizzo_legale().getComune_completo());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_strada).equals(j)) {
					stringa.add(prd.getIndirizzo_legale().getStrada());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_civico).equals(j)) {
					stringa.add(prd.getIndirizzo_legale().getCivico());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_cap).equals(j)) {
					stringa.add(prd.getIndirizzo_legale().getCap());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_comune).equals(j)) {
					stringa.add(prd.getIndirizzo_legale().getComune());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_provincia).equals(j)) {
					stringa.add(prd.getIndirizzo_legale().getProvincia());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_regione).equals(j)) {
					stringa.add(prd.getIndirizzo_legale().getRegione());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_dettagli).equals(j)) {
					stringa.add(prd.getIndirizzo_legale().getDettagli());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_latitudine).equals(j)) {
					stringa.add(prd.getIndirizzo_legale().getLatitudine());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_longitudine).equals(j)) {
					stringa.add(prd.getIndirizzo_legale().getLongitudine());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_presso).equals(j)) {
					stringa.add(prd.getIndirizzo_legale().getPresso());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_indirizzo_completo).equals(j)) {
					stringa.add(prd.getIndirizzo_operativo().getIndirizzo_completo());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_strada_completo).equals(j)) {
					stringa.add(prd.getIndirizzo_operativo().getStrada_completo());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_comune_completo).equals(j)) {
					stringa.add(prd.getIndirizzo_operativo().getComune_completo());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_strada).equals(j)) {
					stringa.add(prd.getIndirizzo_operativo().getStrada());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_civico).equals(j)) {
					stringa.add(prd.getIndirizzo_operativo().getCivico());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_cap).equals(j)) {
					stringa.add(prd.getIndirizzo_operativo().getCap());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_comune).equals(j)) {
					stringa.add(prd.getIndirizzo_operativo().getComune());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_provincia).equals(j)) {
					stringa.add(prd.getIndirizzo_operativo().getProvincia());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_regione).equals(j)) {
					stringa.add(prd.getIndirizzo_operativo().getRegione());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_dettagli).equals(j)) {
					stringa.add(prd.getIndirizzo_operativo().getDettagli());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_latitudine).equals(j)) {
					stringa.add(prd.getIndirizzo_operativo().getLatitudine());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_longitudine).equals(j)) {
					stringa.add(prd.getIndirizzo_operativo().getLongitudine());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_presso).equals(j)) {
					stringa.add(prd.getIndirizzo_operativo().getPresso());
				} else if(ti.getPosizione(ConstantsMap.consenso_tfissoS).equals(j)) {
					stringa.add(prd.getConsenso_tfissoS());
				} else if(ti.getPosizione(ConstantsMap.consenso_tmobileS).equals(j)) {
					stringa.add(prd.getConsenso_tmobileS());
				} else if(ti.getPosizione(ConstantsMap.consenso_tfaxS).equals(j)) {
					stringa.add(prd.getConsenso_tfaxS());
				} else if(ti.getPosizione(ConstantsMap.consenso_taltroS).equals(j)) {
					stringa.add(prd.getConsenso_taltroS());
				} else if(ti.getPosizione(ConstantsMap.consenso_emailS).equals(j)) {
					stringa.add(prd.getConsenso_emailS());
				} else if(ti.getPosizione(ConstantsMap.consenso_rmailS).equals(j)) {
					stringa.add(prd.getConsenso_rmailS());
				} else {
					stringa.add(stampaDato(titles[j], prd.getDettagli(), null));
				}
			}
			writer.writeNext(stringa.toArray(new String[0]));
		}
		writer.close();
	}
	public static void esportaCSVContattoTemplate(OutputStream out, Boolean p, List<Contatto> data, AnagraficaDao anagraficaDao, TemplateExportContatto ti) throws Exception {
		List <String> titlesList = ti.getValore_multiplo();
		List <String> aggiuntivi = ListUtils.subtract(titlesList, new ArrayList<String>(ConstantsMap.contattoMap));
		if(p!=null && ApplicationUtil.notEmpty(aggiuntivi)) {
			List <String> invA = anagraficaDao.getEtichetteInvisibiliByChiave(p, CampoAggiuntivoAnagrafica.class);
			aggiuntivi.removeAll(invA);
			titlesList.removeAll(invA);
		}
		if(ApplicationUtil.notEmpty(aggiuntivi)) {
			data = anagraficaDao.loadFullContatto(data);
		}
		String[] titles = titlesList.toArray(new String[0]);
		CSVWriter writer = new CSVWriter(new OutputStreamWriter(out, ConstantsGlobal.CHARSET),ti.getSeparatoreChar(), ti.getQuoteChar());
		List<String> header = new ArrayList<String>();
		if(ti.getIntestazione()) {
			for (int i = 0; i < titles.length; i++) {
				if(ApplicationUtil.notEmpty(aggiuntivi) && aggiuntivi.contains(titles[i])) {
					String etichetta = anagraficaDao.getEtichettaByChiave(CampoAggiuntivoAnagrafica.class, titles[i]);
					header.add(etichetta);
				} else {
					header.add(Message.addCatalogText(titles[i]));
				}
			}
		} 
		if(ApplicationUtil.notEmpty(header)) {
			writer.writeNext(header.toArray(new String[0]));
		}
		Iterator<Contatto> objIter = data.iterator();
		while(objIter.hasNext()) {
			Contatto cnt = objIter.next();
			List<String> stringa = new ArrayList<String>();
			for (int j = 0; j < titles.length; j++) {
				if(ti.getPosizione(ConstantsMap.contatto_data_archiviazione).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateTime.stampa(), cnt.getData_archiviazione()));
				} else if(ti.getPosizione(ConstantsMap.contatto_esito_chiamata).equals(j)) {
					stringa.add(cnt.getEsito_chiamata());
				} else if(ti.getPosizione(ConstantsMap.contatto_esito_archiviazione).equals(j)) {
					stringa.add(cnt.getEsito_archiviazione());
				} else if(ti.getPosizione(ConstantsMap.contatto_esito_generale).equals(j)) {
					stringa.add(cnt.getEsito_generale());
				} else if(ti.getPosizione(ConstantsMap.contatto_codice_chiamata).equals(j)) {
					stringa.add(cnt.getCodice_chiamata());
				} else if(ti.getPosizione(ConstantsMap.contatto_codice_archiviazione).equals(j)) {
					stringa.add(cnt.getCodice_archiviazione());
				} else if(ti.getPosizione(ConstantsMap.contatto_codice_generale).equals(j)) {
					stringa.add(cnt.getCodice_generale());
				} else if(ti.getPosizione(ConstantsMap.contatto_data_lavorazione).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateTime.stampa(), cnt.getData_lavorazione()));
				} else if(ti.getPosizione(ConstantsMap.contatto_operatore_denominazione).equals(j)) {
					stringa.add(cnt.getOperatore_denominazione());
				} else if(ti.getPosizione(ConstantsMap.contatto_operatore_codice).equals(j)) {
					stringa.add(cnt.getOperatore_codice());
				} else if(ti.getPosizione(ConstantsMap.contatto_proprietario_denominazione).equals(j)) {
					stringa.add(cnt.getProprietario_denominazione());
				} else if(ti.getPosizione(ConstantsMap.contatto_proprietario_codice).equals(j)) {
					stringa.add(cnt.getProprietario_codice());
				} else if(ti.getPosizione(ConstantsMap.contatto_data_presentazione).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateTime.stampa(), cnt.getData_presentazione()));
				} else if(ti.getPosizione(ConstantsMap.contatto_nota).equals(j)) {
					stringa.add(cnt.getNota());
				} else if(ti.getPosizione(ConstantsMap.contatto_data_creazione).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateTime.stampa(), cnt.getData_creazione()));
				} else if(ti.getPosizione(ConstantsMap.contatto_agente_denominazione).equals(j)) {
					stringa.add(cnt.getAgente_denominazione());
				} else if(ti.getPosizione(ConstantsMap.contatto_agente_codice).equals(j)) {
					stringa.add(cnt.getAgente_codice());
				} else if(ti.getPosizione(ConstantsMap.contatto_stato).equals(j)) {
					stringa.add(cnt.getTipoTesto());
				} else if(ti.getPosizione(ConstantsMap.contatto_id).equals(j)) {
					stringa.add(cnt.getId().toString());
				} else if(ti.getPosizione(ConstantsMap.contatto_campagna_denominazione).equals(j)) {
					stringa.add(cnt.getCampagna_denominazione());
				} else if(ti.getPosizione(ConstantsMap.contatto_lista_denominazione).equals(j)) {
					stringa.add(cnt.getLista_denominazione());
				} else if(ti.getPosizione(ConstantsMap.contatto_campagna_codice).equals(j)) {
					stringa.add(cnt.getCampagna_codice());
				} else if(ti.getPosizione(ConstantsMap.contatto_bloccato).equals(j)) {
					stringa.add(cnt.getBloccatoS());
				} else if(ti.getPosizione(ConstantsMap.contatto_lista_codice).equals(j)) {
					stringa.add(cnt.getLista_codice());
				} else if(ti.getPosizione(ConstantsMap.contatto_positivo).equals(j)) {
					stringa.add(cnt.getPositivo());
				} else if(ti.getPosizione(ConstantsMap.contatto_telefono_predefinito).equals(j)) {
					stringa.add(ApplicationUtil.uniforma(cnt.getTelefono_predefinito(), ti.getRem_predefinito(), ti.getAdd_predefinito()));
				} else if(ti.getPosizione(ConstantsMap.contatto_utile).equals(j)) {
					stringa.add(cnt.getUtile());
				} else if(ti.getPosizione(ConstantsMap.contatto_chiuso).equals(j)) {
					stringa.add(cnt.getChiuso());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_cognome).equals(j)) {
					stringa.add(cnt.getAnagrafica().getCognome());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_nome).equals(j)) {
					stringa.add(cnt.getAnagrafica().getNome());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_nome_completo).equals(j)) {
					stringa.add(cnt.getAnagrafica().getNome_completo());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_email).equals(j)) {
					stringa.add(cnt.getAnagrafica().getEmail());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_codice_fiscale).equals(j)) {
					stringa.add(cnt.getAnagrafica().getCodice_fiscale());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_sorgente).equals(j)) {
					stringa.add(cnt.getAnagrafica().getSorgente());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_nota).equals(j)) {
					stringa.add(cnt.getAnagrafica().getNota());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_luogo_nascita).equals(j)) {
					stringa.add(cnt.getAnagrafica().getLuogo_nascita());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_data_nascita).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.date.stampa(), cnt.getAnagrafica().getDate_nascita()));
				} else if(ti.getPosizione(ConstantsMap.anagrafica_link).equals(j)) {
					stringa.add(cnt.getAnagrafica().getLink());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_data_inserimento).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateTime.stampa(), cnt.getAnagrafica().getData_creazione()));
				} else if(ti.getPosizione(ConstantsMap.anagrafica_privacy).equals(j)) {
					stringa.add(cnt.getAnagrafica().getPrivacy());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_tipo).equals(j)) {
					stringa.add(cnt.getAnagrafica().getTipo());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_sesso).equals(j)) {
					stringa.add(cnt.getAnagrafica().getSesso());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_partita_iva).equals(j)) {
					stringa.add(cnt.getAnagrafica().getPartita_iva());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_ragione_sociale).equals(j)) {
					stringa.add(cnt.getAnagrafica().getRagione_sociale());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_telefono_fisso).equals(j)) {
					stringa.add(ApplicationUtil.uniforma(cnt.getAnagrafica().getTelefono_fisso(), ti.getRem_fisso(), ti.getAdd_fisso()));
				} else if(ti.getPosizione(ConstantsMap.anagrafica_telefono_altro).equals(j)) {
					stringa.add(ApplicationUtil.uniforma(cnt.getAnagrafica().getTelefono_altro(), ti.getRem_altro(), ti.getAdd_altro()));
				} else if(ti.getPosizione(ConstantsMap.anagrafica_telefono_mobile).equals(j)) {
					stringa.add(ApplicationUtil.uniforma(cnt.getAnagrafica().getTelefono_mobile(), ti.getRem_mobile(), ti.getAdd_mobile()));
				} else if(ti.getPosizione(ConstantsMap.anagrafica_telefono_fax).equals(j)) {
					stringa.add(ApplicationUtil.uniforma(cnt.getAnagrafica().getTelefono_fax(), ti.getRem_fax(), ti.getAdd_fax()));
				} else if(ti.getPosizione(ConstantsMap.anagrafica_settore).equals(j)) {
					stringa.add(cnt.getAnagrafica().getSettore());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_denominazione).equals(j)) {
					stringa.add(cnt.getAnagrafica().getDenominazione());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_id).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIdentificativo());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_indirizzo_completo).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getIndirizzo_completo());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_strada_completo).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getStrada_completo());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_comune_completo).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getComune_completo());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_strada).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getStrada());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_civico).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getCivico());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_cap).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getCap());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_comune).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getComune());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_provincia).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getProvincia());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_regione).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getRegione());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_dettagli).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getDettagli());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_latitudine).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getLatitudine());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_longitudine).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getLongitudine());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_presso).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getPresso());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_indirizzo_completo).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getIndirizzo_completo());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_strada_completo).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getStrada_completo());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_comune_completo).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getComune_completo());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_strada).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getStrada());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_civico).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getCivico());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_cap).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getCap());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_comune).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getComune());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_provincia).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getProvincia());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_regione).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getRegione());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_dettagli).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getDettagli());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_latitudine).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getLatitudine());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_longitudine).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getLongitudine());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_presso).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getPresso());
				} else if(ti.getPosizione(ConstantsMap.consenso_tfissoS).equals(j)) {
					stringa.add(cnt.getAnagrafica().getConsenso_tfissoS());
				} else if(ti.getPosizione(ConstantsMap.consenso_tmobileS).equals(j)) {
					stringa.add(cnt.getAnagrafica().getConsenso_tmobileS());
				} else if(ti.getPosizione(ConstantsMap.consenso_tfaxS).equals(j)) {
					stringa.add(cnt.getAnagrafica().getConsenso_tfaxS());
				} else if(ti.getPosizione(ConstantsMap.consenso_taltroS).equals(j)) {
					stringa.add(cnt.getAnagrafica().getConsenso_taltroS());
				} else if(ti.getPosizione(ConstantsMap.consenso_emailS).equals(j)) {
					stringa.add(cnt.getAnagrafica().getConsenso_emailS());
				} else if(ti.getPosizione(ConstantsMap.consenso_rmailS).equals(j)) {
					stringa.add(cnt.getAnagrafica().getConsenso_rmailS());
				} else {
					stringa.add(stampaDato(titles[j], cnt.getAnagrafica().getDettagli(), null));
				}
			}
			writer.writeNext(stringa.toArray(new String[0]));
		}
		writer.close();
	}
	public static void esportaCSVAppuntamentoTemplate(OutputStream out, Boolean p, List<Appuntamento> data, AnagraficaDao anagraficaDao, TemplateExportAppuntamento ti) throws Exception {
		List <String> titlesList = ti.getValore_multiplo();
		List <String> aggiuntivi = ListUtils.subtract(titlesList, new ArrayList<String>(ConstantsMap.appuntamentoMap));
		if(p!=null && ApplicationUtil.notEmpty(aggiuntivi)) {
			List <String> invA = anagraficaDao.getEtichetteInvisibiliByChiave(p, CampoAggiuntivoAnagrafica.class);
			aggiuntivi.removeAll(invA);
			titlesList.removeAll(invA);
		}
		if(ApplicationUtil.notEmpty(aggiuntivi)) {
			data = anagraficaDao.loadFullAppuntamento(data);
		}
		String[] titles = titlesList.toArray(new String[0]);
		CSVWriter writer = new CSVWriter(new OutputStreamWriter(out, ConstantsGlobal.CHARSET),ti.getSeparatoreChar(), ti.getQuoteChar());
		List<String> header = new ArrayList<String>();
		if(ti.getIntestazione()) {
			for (int i = 0; i < titles.length; i++) {
				if(ApplicationUtil.notEmpty(aggiuntivi) && aggiuntivi.contains(titles[i])) {
					String etichetta = anagraficaDao.getEtichettaByChiave(CampoAggiuntivoAnagrafica.class, titles[i]);
					header.add(etichetta);
				} else {
					header.add(Message.addCatalogText(titles[i]));
				}
			}
		} 
		if(ApplicationUtil.notEmpty(header)) {
			writer.writeNext(header.toArray(new String[0]));
		}
		Iterator<Appuntamento> objIter = data.iterator();
		while(objIter.hasNext()) {
			Appuntamento app = objIter.next();
			Contatto cnt = app.getContatto();
			List<String> stringa = new ArrayList<String>();
			for (int j = 0; j < titles.length; j++) {
				if(ti.getPosizione(ConstantsMap.appuntamento_id).equals(j)) {
					stringa.add(app.getId().toString());
				} else if(ti.getPosizione(ConstantsMap.appuntamento_operatore_denominazione).equals(j)) {
					stringa.add(app.getOperatore_denominazione());
				} else if(ti.getPosizione(ConstantsMap.appuntamento_bloccato).equals(j)) {
					stringa.add(app.getBloccatoS());
				} else if(ti.getPosizione(ConstantsMap.appuntamento_operatore_codice).equals(j)) {
					stringa.add(app.getOperatore_codice());
				} else if(ti.getPosizione(ConstantsMap.appuntamento_agente_denominazione).equals(j)) {
					stringa.add(app.getAgente_denominazione());
				} else if(ti.getPosizione(ConstantsMap.appuntamento_agente_codice).equals(j)) {
					stringa.add(app.getAgente_codice());
				} else if(ti.getPosizione(ConstantsMap.appuntamento_nota).equals(j)) {
					stringa.add(app.getNota());
				} else if(ti.getPosizione(ConstantsMap.appuntamento_data_creazione).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateTime.stampa(), app.getData_creazione()));
				} else if(ti.getPosizione(ConstantsMap.appuntamento_data_appuntamento).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.date.stampa(), app.getData_inizio()));
				} else if(ti.getPosizione(ConstantsMap.appuntamento_data_ora).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateTime.stampa(), app.getData_inizio()));
				} else if(ti.getPosizione(ConstantsMap.appuntamento_ora_inizio).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.time.stampa(), app.getData_inizio()));
				} else if(ti.getPosizione(ConstantsMap.appuntamento_ora_fine).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.time.stampa(), app.getData_fine()));
				} else if(ti.getPosizione(ConstantsMap.appuntamento_data_conferma).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateTime.stampa(), app.getData_conferma()));
				} else if(ti.getPosizione(ConstantsMap.appuntamento_data_lavorazione).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateTime.stampa(), app.getData_lavorazione()));
				} else if(ti.getPosizione(ConstantsMap.appuntamento_localizzazione).equals(j)) {
					stringa.add(app.getLocalizzazioneString());
				} else if(ti.getPosizione(ConstantsMap.appuntamento_utente_denominazione).equals(j)) {
					stringa.add(app.getUtente_denominazione());
				} else if(ti.getPosizione(ConstantsMap.appuntamento_utente_codice).equals(j)) {
					stringa.add(app.getUtente_codice());
				} else if(ti.getPosizione(ConstantsMap.appuntamento_data_esito).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateTime.stampa(), app.getData_esito()));
				} else if(ti.getPosizione(ConstantsMap.appuntamento_data_stato).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateTime.stampa(), app.getData_stato()));
				} else if(ti.getPosizione(ConstantsMap.appuntamento_esito_appuntamento).equals(j)) {
					stringa.add(app.getEsitoEtichetta());
				} else if(ti.getPosizione(ConstantsMap.appuntamento_codice_appuntamento).equals(j)) {
					stringa.add(app.getEsitoCodice());
				} else if(ti.getPosizione(ConstantsMap.appuntamento_stato).equals(j)) {
					stringa.add(app.getStatoTesto());
				} else if(ti.getPosizione(ConstantsMap.contatto_data_lavorazione).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateTime.stampa(), cnt.getData_lavorazione()));
				} else if(ti.getPosizione(ConstantsMap.contatto_operatore_denominazione).equals(j)) {
					stringa.add(cnt.getOperatore_denominazione());
				} else if(ti.getPosizione(ConstantsMap.contatto_operatore_codice).equals(j)) {
					stringa.add(cnt.getOperatore_codice());
				} else if(ti.getPosizione(ConstantsMap.contatto_data_archiviazione).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateTime.stampa(), cnt.getData_archiviazione()));
				} else if(ti.getPosizione(ConstantsMap.contatto_esito_chiamata).equals(j)) {
					stringa.add(cnt.getEsito_chiamata());
				} else if(ti.getPosizione(ConstantsMap.contatto_esito_archiviazione).equals(j)) {
					stringa.add(cnt.getEsito_archiviazione());
				} else if(ti.getPosizione(ConstantsMap.contatto_esito_generale).equals(j)) {
					stringa.add(cnt.getEsito_generale());
				} else if(ti.getPosizione(ConstantsMap.contatto_codice_chiamata).equals(j)) {
					stringa.add(cnt.getCodice_chiamata());
				} else if(ti.getPosizione(ConstantsMap.contatto_codice_archiviazione).equals(j)) {
					stringa.add(cnt.getCodice_archiviazione());
				} else if(ti.getPosizione(ConstantsMap.contatto_codice_generale).equals(j)) {
					stringa.add(cnt.getCodice_generale());
				} else if(ti.getPosizione(ConstantsMap.contatto_proprietario_denominazione).equals(j)) {
					stringa.add(cnt.getProprietario_denominazione());
				} else if(ti.getPosizione(ConstantsMap.contatto_proprietario_codice).equals(j)) {
					stringa.add(cnt.getProprietario_codice());
				} else if(ti.getPosizione(ConstantsMap.contatto_data_presentazione).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateTime.stampa(), cnt.getData_presentazione()));
				} else if(ti.getPosizione(ConstantsMap.contatto_nota).equals(j)) {
					stringa.add(cnt.getNota());
				} else if(ti.getPosizione(ConstantsMap.contatto_data_creazione).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateTime.stampa(), cnt.getData_creazione()));
				} else if(ti.getPosizione(ConstantsMap.contatto_agente_denominazione).equals(j)) {
					stringa.add(cnt.getAgente_denominazione());
				} else if(ti.getPosizione(ConstantsMap.contatto_agente_codice).equals(j)) {
					stringa.add(cnt.getAgente_codice());
				} else if(ti.getPosizione(ConstantsMap.contatto_stato).equals(j)) {
					stringa.add(cnt.getTipoTesto());
				} else if(ti.getPosizione(ConstantsMap.contatto_id).equals(j)) {
					stringa.add(cnt.getId().toString());
				} else if(ti.getPosizione(ConstantsMap.contatto_campagna_denominazione).equals(j)) {
					stringa.add(cnt.getCampagna_denominazione());
				} else if(ti.getPosizione(ConstantsMap.contatto_lista_denominazione).equals(j)) {
					stringa.add(cnt.getLista_denominazione());
				} else if(ti.getPosizione(ConstantsMap.contatto_campagna_codice).equals(j)) {
					stringa.add(cnt.getCampagna_codice());
				} else if(ti.getPosizione(ConstantsMap.contatto_bloccato).equals(j)) {
					stringa.add(cnt.getBloccatoS());
				} else if(ti.getPosizione(ConstantsMap.contatto_lista_codice).equals(j)) {
					stringa.add(cnt.getLista_codice());
				} else if(ti.getPosizione(ConstantsMap.contatto_positivo).equals(j)) {
					stringa.add(cnt.getPositivo());
				} else if(ti.getPosizione(ConstantsMap.contatto_telefono_predefinito).equals(j)) {
					stringa.add(ApplicationUtil.uniforma(cnt.getTelefono_predefinito(), ti.getRem_predefinito(), ti.getAdd_predefinito()));
				} else if(ti.getPosizione(ConstantsMap.contatto_utile).equals(j)) {
					stringa.add(cnt.getUtile());
				} else if(ti.getPosizione(ConstantsMap.contatto_chiuso).equals(j)) {
					stringa.add(cnt.getChiuso());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_cognome).equals(j)) {
					stringa.add(cnt.getAnagrafica().getCognome());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_nome).equals(j)) {
					stringa.add(cnt.getAnagrafica().getNome());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_nome_completo).equals(j)) {
					stringa.add(cnt.getAnagrafica().getNome_completo());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_email).equals(j)) {
					stringa.add(cnt.getAnagrafica().getEmail());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_codice_fiscale).equals(j)) {
					stringa.add(cnt.getAnagrafica().getCodice_fiscale());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_sorgente).equals(j)) {
					stringa.add(cnt.getAnagrafica().getSorgente());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_nota).equals(j)) {
					stringa.add(cnt.getAnagrafica().getNota());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_luogo_nascita).equals(j)) {
					stringa.add(cnt.getAnagrafica().getLuogo_nascita());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_data_nascita).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.date.stampa(), cnt.getAnagrafica().getDate_nascita()));
				} else if(ti.getPosizione(ConstantsMap.anagrafica_link).equals(j)) {
					stringa.add(cnt.getAnagrafica().getLink());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_data_inserimento).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateTime.stampa(), cnt.getAnagrafica().getData_creazione()));
				} else if(ti.getPosizione(ConstantsMap.anagrafica_privacy).equals(j)) {
					stringa.add(cnt.getAnagrafica().getPrivacy());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_tipo).equals(j)) {
					stringa.add(cnt.getAnagrafica().getTipo());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_sesso).equals(j)) {
					stringa.add(cnt.getAnagrafica().getSesso());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_partita_iva).equals(j)) {
					stringa.add(cnt.getAnagrafica().getPartita_iva());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_ragione_sociale).equals(j)) {
					stringa.add(cnt.getAnagrafica().getRagione_sociale());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_telefono_fisso).equals(j)) {
					stringa.add(ApplicationUtil.uniforma(cnt.getAnagrafica().getTelefono_fisso(), ti.getRem_fisso(), ti.getAdd_fisso()));
				} else if(ti.getPosizione(ConstantsMap.anagrafica_telefono_altro).equals(j)) {
					stringa.add(ApplicationUtil.uniforma(cnt.getAnagrafica().getTelefono_altro(), ti.getRem_altro(), ti.getAdd_altro()));
				} else if(ti.getPosizione(ConstantsMap.anagrafica_telefono_mobile).equals(j)) {
					stringa.add(ApplicationUtil.uniforma(cnt.getAnagrafica().getTelefono_mobile(), ti.getRem_mobile(), ti.getAdd_mobile()));
				} else if(ti.getPosizione(ConstantsMap.anagrafica_telefono_fax).equals(j)) {
					stringa.add(ApplicationUtil.uniforma(cnt.getAnagrafica().getTelefono_fax(), ti.getRem_fax(), ti.getAdd_fax()));
				} else if(ti.getPosizione(ConstantsMap.anagrafica_settore).equals(j)) {
					stringa.add(cnt.getAnagrafica().getSettore());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_denominazione).equals(j)) {
					stringa.add(cnt.getAnagrafica().getDenominazione());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_id).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIdentificativo());
				}else if(ti.getPosizione(ConstantsMap.indirizzo_legale_indirizzo_completo).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getIndirizzo_completo());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_strada_completo).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getStrada_completo());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_comune_completo).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getComune_completo());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_strada).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getStrada());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_civico).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getCivico());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_cap).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getCap());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_comune).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getComune());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_provincia).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getProvincia());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_regione).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getRegione());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_dettagli).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getDettagli());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_latitudine).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getLatitudine());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_longitudine).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getLongitudine());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_presso).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getPresso());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_indirizzo_completo).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getIndirizzo_completo());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_strada_completo).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getStrada_completo());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_comune_completo).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getComune_completo());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_strada).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getStrada());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_civico).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getCivico());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_cap).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getCap());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_comune).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getComune());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_provincia).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getProvincia());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_regione).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getRegione());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_dettagli).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getDettagli());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_latitudine).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getLatitudine());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_longitudine).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getLongitudine());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_presso).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getPresso());
				} else if(ti.getPosizione(ConstantsMap.consenso_tfissoS).equals(j)) {
					stringa.add(cnt.getAnagrafica().getConsenso_tfissoS());
				} else if(ti.getPosizione(ConstantsMap.consenso_tmobileS).equals(j)) {
					stringa.add(cnt.getAnagrafica().getConsenso_tmobileS());
				} else if(ti.getPosizione(ConstantsMap.consenso_tfaxS).equals(j)) {
					stringa.add(cnt.getAnagrafica().getConsenso_tfaxS());
				} else if(ti.getPosizione(ConstantsMap.consenso_taltroS).equals(j)) {
					stringa.add(cnt.getAnagrafica().getConsenso_taltroS());
				} else if(ti.getPosizione(ConstantsMap.consenso_emailS).equals(j)) {
					stringa.add(cnt.getAnagrafica().getConsenso_emailS());
				} else if(ti.getPosizione(ConstantsMap.consenso_rmailS).equals(j)) {
					stringa.add(cnt.getAnagrafica().getConsenso_rmailS());
				} else {
					stringa.add(stampaDato(titles[j], cnt.getAnagrafica().getDettagli(), null));
				}
			}
			writer.writeNext(stringa.toArray(new String[0]));
		}
		writer.close();
	}
	public static void esportaCSVContrattoTemplate(OutputStream out, Boolean p, List<Contratto> data, AnagraficaDao anagraficaDao, TemplateExportContratto ti) throws Exception {
		List <String> titlesList = ti.getValore_multiplo();
		List <String> aggiuntivi = ListUtils.subtract(titlesList, new ArrayList<String>(ConstantsMap.contrattoMap));
		if(p!=null && ApplicationUtil.notEmpty(aggiuntivi)) {
			List <String> invA = anagraficaDao.getEtichetteInvisibiliByChiave(p, CampoAggiuntivoAnagrafica.class);
			List <String> invB = anagraficaDao.getEtichetteInvisibiliByChiave(p, CampoAggiuntivoContratto.class);
			aggiuntivi.removeAll(invA);
			aggiuntivi.removeAll(invB);
			titlesList.removeAll(invA);
			titlesList.removeAll(invB);
		}
		if(ApplicationUtil.notEmpty(aggiuntivi)) {
			data = anagraficaDao.loadFullContratto(data);
		}
		String[] titles = titlesList.toArray(new String[0]);
		CSVWriter writer = new CSVWriter(new OutputStreamWriter(out, ConstantsGlobal.CHARSET),ti.getSeparatoreChar(), ti.getQuoteChar());
		List<String> header = new ArrayList<String>();
		if(ti.getIntestazione()) {
			for (int i = 0; i < titles.length; i++) {
				if(ApplicationUtil.notEmpty(aggiuntivi) && aggiuntivi.contains(titles[i])) {
					String etichetta = anagraficaDao.getEtichettaByChiave(CampoAggiuntivoAnagrafica.class, titles[i]);
					if(ApplicationUtil.isBlank(etichetta)) {
						etichetta = anagraficaDao.getEtichettaByChiave(CampoAggiuntivoContratto.class, titles[i]);
					}
					header.add(etichetta);
				} else {
					header.add(Message.addCatalogText(titles[i]));
				}
			}
		} 
		if(ApplicationUtil.notEmpty(header)) {
			writer.writeNext(header.toArray(new String[0]));
		}
		Iterator<Contratto> objIter = data.iterator();
		while(objIter.hasNext()) {
			Contratto crt = objIter.next();
			Contatto cnt = crt.getContatto();
			List<String> stringa = new ArrayList<String>();
			for (int j = 0; j < titles.length; j++) {
				if(ti.getPosizione(ConstantsMap.contratto_data_creazione).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateTime.stampa(), crt.getData_creazione()));
				} else if(ti.getPosizione(ConstantsMap.contratto_id).equals(j)) {
					stringa.add(crt.getId().toString());
				} else if(ti.getPosizione(ConstantsMap.contratto_operatore_denominazione).equals(j)) {
					stringa.add(crt.getOperatore_denominazione());
				} else if(ti.getPosizione(ConstantsMap.contratto_operatore_codice).equals(j)) {
					stringa.add(crt.getOperatore_codice());
				} else if(ti.getPosizione(ConstantsMap.contratto_agente_denominazione).equals(j)) {
					stringa.add(crt.getAgente_denominazione());
				} else if(ti.getPosizione(ConstantsMap.contratto_agente_codice).equals(j)) {
					stringa.add(crt.getAgente_codice());
				} else if(ti.getPosizione(ConstantsMap.contratto_categoria_denominazione).equals(j)) {
					stringa.add(crt.getCategoria_denominazione());
				} else if(ti.getPosizione(ConstantsMap.contratto_prodotto_denominazione).equals(j)) {
					stringa.add(crt.getProdotto_denominazione());
				} else if(ti.getPosizione(ConstantsMap.contratto_esito_contratto).equals(j)) {
					stringa.add(crt.getEsitoEtichetta());
				} else if(ti.getPosizione(ConstantsMap.contratto_codice_contratto).equals(j)) {
					stringa.add(crt.getEsitoCodice());
				} else if(ti.getPosizione(ConstantsMap.contratto_data_stipula).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.date.stampa(),crt.getData_stipula()));
				} else if(ti.getPosizione(ConstantsMap.contratto_data_contabile).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.date.stampa(),crt.getData_contabile()));
				} else if(ti.getPosizione(ConstantsMap.contratto_data_attivazione).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.date.stampa(),crt.getData_attivazione()));
				} else if(ti.getPosizione(ConstantsMap.contratto_data_caricamento).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.date.stampa(),crt.getData_caricamento()));
				} else if(ti.getPosizione(ConstantsMap.contratto_data_lavorazione).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateTime.stampa(),crt.getData_lavorazione()));
				} else if(ti.getPosizione(ConstantsMap.contratto_utente_denominazione).equals(j)) {
					stringa.add(crt.getUtente_denominazione());
				} else if(ti.getPosizione(ConstantsMap.contratto_bloccato).equals(j)) {
					stringa.add(crt.getBloccatoS());
				} else if(ti.getPosizione(ConstantsMap.contratto_utente_codice).equals(j)) {
					stringa.add(crt.getUtente_codice());
				} else if(ti.getPosizione(ConstantsMap.contratto_data_esito).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateTime.stampa(), crt.getData_esito()));
				} else if(ti.getPosizione(ConstantsMap.contratto_data_stato).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateTime.stampa(), crt.getData_stato()));
				} else if(ti.getPosizione(ConstantsMap.contratto_nota).equals(j)) {
					stringa.add(crt.getNota());
				} else if(ti.getPosizione(ConstantsMap.contratto_tipo_seriale).equals(j)) {
					stringa.add(crt.getTipo_seriale());
				} else if(ti.getPosizione(ConstantsMap.contratto_seriale).equals(j)) {
					stringa.add(crt.getSeriale());
				} else if(ti.getPosizione(ConstantsMap.contratto_quantita).equals(j)) {
					stringa.add(ApplicationUtil.formatNumber(crt.getQuantita()));
				} else if(ti.getPosizione(ConstantsMap.contratto_costo_scontato).equals(j)) {
					stringa.add(ApplicationUtil.formatNumber(crt.getCosto_scontato()));
				} else if(ti.getPosizione(ConstantsMap.contratto_stato).equals(j)) {
					stringa.add(crt.getStatoTesto());
				} else if(ti.getPosizione(ConstantsMap.contatto_data_archiviazione).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateTime.stampa(), cnt.getData_archiviazione()));
				} else if(ti.getPosizione(ConstantsMap.contatto_data_lavorazione).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateTime.stampa(), cnt.getData_lavorazione()));
				} else if(ti.getPosizione(ConstantsMap.contatto_operatore_denominazione).equals(j)) {
					stringa.add(cnt.getOperatore_denominazione());
				} else if(ti.getPosizione(ConstantsMap.contatto_operatore_codice).equals(j)) {
					stringa.add(cnt.getOperatore_codice());
				} else if(ti.getPosizione(ConstantsMap.contatto_esito_chiamata).equals(j)) {
					stringa.add(cnt.getEsito_chiamata());
				} else if(ti.getPosizione(ConstantsMap.contatto_esito_archiviazione).equals(j)) {
					stringa.add(cnt.getEsito_archiviazione());
				} else if(ti.getPosizione(ConstantsMap.contatto_esito_generale).equals(j)) {
					stringa.add(cnt.getEsito_generale());
				} else if(ti.getPosizione(ConstantsMap.contatto_codice_chiamata).equals(j)) {
					stringa.add(cnt.getCodice_chiamata());
				} else if(ti.getPosizione(ConstantsMap.contatto_codice_archiviazione).equals(j)) {
					stringa.add(cnt.getCodice_archiviazione());
				} else if(ti.getPosizione(ConstantsMap.contatto_codice_generale).equals(j)) {
					stringa.add(cnt.getCodice_generale());
				} else if(ti.getPosizione(ConstantsMap.contatto_proprietario_denominazione).equals(j)) {
					stringa.add(cnt.getProprietario_denominazione());
				} else if(ti.getPosizione(ConstantsMap.contatto_proprietario_codice).equals(j)) {
					stringa.add(cnt.getProprietario_codice());
				} else if(ti.getPosizione(ConstantsMap.contatto_data_presentazione).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateTime.stampa(), cnt.getData_presentazione()));
				} else if(ti.getPosizione(ConstantsMap.contatto_nota).equals(j)) {
					stringa.add(cnt.getNota());
				} else if(ti.getPosizione(ConstantsMap.contatto_data_creazione).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateTime.stampa(), cnt.getData_creazione()));
				} else if(ti.getPosizione(ConstantsMap.contatto_agente_denominazione).equals(j)) {
					stringa.add(cnt.getAgente_denominazione());
				} else if(ti.getPosizione(ConstantsMap.contatto_agente_codice).equals(j)) {
					stringa.add(cnt.getAgente_codice());
				} else if(ti.getPosizione(ConstantsMap.contatto_stato).equals(j)) {
					stringa.add(cnt.getTipoTesto());
				} else if(ti.getPosizione(ConstantsMap.contatto_id).equals(j)) {
					stringa.add(cnt.getId().toString());
				} else if(ti.getPosizione(ConstantsMap.contatto_campagna_denominazione).equals(j)) {
					stringa.add(cnt.getCampagna_denominazione());
				} else if(ti.getPosizione(ConstantsMap.contatto_lista_denominazione).equals(j)) {
					stringa.add(cnt.getLista_denominazione());
				} else if(ti.getPosizione(ConstantsMap.contatto_campagna_codice).equals(j)) {
					stringa.add(cnt.getCampagna_codice());
				} else if(ti.getPosizione(ConstantsMap.contatto_bloccato).equals(j)) {
					stringa.add(cnt.getBloccatoS());
				} else if(ti.getPosizione(ConstantsMap.contatto_lista_codice).equals(j)) {
					stringa.add(cnt.getLista_codice());
				} else if(ti.getPosizione(ConstantsMap.contatto_positivo).equals(j)) {
					stringa.add(cnt.getPositivo());
				} else if(ti.getPosizione(ConstantsMap.contatto_telefono_predefinito).equals(j)) {
					stringa.add(ApplicationUtil.uniforma(cnt.getTelefono_predefinito(), ti.getRem_predefinito(), ti.getAdd_predefinito()));
				} else if(ti.getPosizione(ConstantsMap.contatto_utile).equals(j)) {
					stringa.add(cnt.getUtile());
				} else if(ti.getPosizione(ConstantsMap.contatto_chiuso).equals(j)) {
					stringa.add(cnt.getChiuso());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_cognome).equals(j)) {
					stringa.add(cnt.getAnagrafica().getCognome());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_nome).equals(j)) {
					stringa.add(cnt.getAnagrafica().getNome());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_nome_completo).equals(j)) {
					stringa.add(cnt.getAnagrafica().getNome_completo());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_email).equals(j)) {
					stringa.add(cnt.getAnagrafica().getEmail());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_codice_fiscale).equals(j)) {
					stringa.add(cnt.getAnagrafica().getCodice_fiscale());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_sorgente).equals(j)) {
					stringa.add(cnt.getAnagrafica().getSorgente());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_nota).equals(j)) {
					stringa.add(cnt.getAnagrafica().getNota());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_luogo_nascita).equals(j)) {
					stringa.add(cnt.getAnagrafica().getLuogo_nascita());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_data_nascita).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.date.stampa(), cnt.getAnagrafica().getDate_nascita()));
				} else if(ti.getPosizione(ConstantsMap.anagrafica_link).equals(j)) {
					stringa.add(cnt.getAnagrafica().getLink());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_data_inserimento).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateTime.stampa(), cnt.getAnagrafica().getData_creazione()));
				} else if(ti.getPosizione(ConstantsMap.anagrafica_privacy).equals(j)) {
					stringa.add(cnt.getAnagrafica().getPrivacy());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_tipo).equals(j)) {
					stringa.add(cnt.getAnagrafica().getTipo());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_sesso).equals(j)) {
					stringa.add(cnt.getAnagrafica().getSesso());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_partita_iva).equals(j)) {
					stringa.add(cnt.getAnagrafica().getPartita_iva());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_ragione_sociale).equals(j)) {
					stringa.add(cnt.getAnagrafica().getRagione_sociale());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_telefono_fisso).equals(j)) {
					stringa.add(ApplicationUtil.uniforma(cnt.getAnagrafica().getTelefono_fisso(), ti.getRem_fisso(), ti.getAdd_fisso()));
				} else if(ti.getPosizione(ConstantsMap.anagrafica_telefono_altro).equals(j)) {
					stringa.add(ApplicationUtil.uniforma(cnt.getAnagrafica().getTelefono_altro(), ti.getRem_altro(), ti.getAdd_altro()));
				} else if(ti.getPosizione(ConstantsMap.anagrafica_telefono_mobile).equals(j)) {
					stringa.add(ApplicationUtil.uniforma(cnt.getAnagrafica().getTelefono_mobile(), ti.getRem_mobile(), ti.getAdd_mobile()));
				} else if(ti.getPosizione(ConstantsMap.anagrafica_telefono_fax).equals(j)) {
					stringa.add(ApplicationUtil.uniforma(cnt.getAnagrafica().getTelefono_fax(), ti.getRem_fax(), ti.getAdd_fax()));
				} else if(ti.getPosizione(ConstantsMap.anagrafica_settore).equals(j)) {
					stringa.add(cnt.getAnagrafica().getSettore());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_denominazione).equals(j)) {
					stringa.add(cnt.getAnagrafica().getDenominazione());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_id).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIdentificativo());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_indirizzo_completo).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getIndirizzo_completo());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_strada_completo).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getStrada_completo());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_comune_completo).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getComune_completo());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_strada).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getStrada());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_civico).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getCivico());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_cap).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getCap());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_comune).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getComune());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_provincia).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getProvincia());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_regione).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getRegione());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_dettagli).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getDettagli());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_latitudine).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getLatitudine());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_longitudine).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getLongitudine());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_presso).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getPresso());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_indirizzo_completo).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getIndirizzo_completo());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_strada_completo).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getStrada_completo());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_comune_completo).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getComune_completo());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_strada).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getStrada());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_civico).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getCivico());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_cap).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getCap());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_comune).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getComune());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_provincia).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getProvincia());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_regione).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getRegione());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_dettagli).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getDettagli());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_latitudine).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getLatitudine());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_longitudine).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getLongitudine());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_presso).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getPresso());
				} else if(ti.getPosizione(ConstantsMap.consenso_tfissoS).equals(j)) {
					stringa.add(cnt.getAnagrafica().getConsenso_tfissoS());
				} else if(ti.getPosizione(ConstantsMap.consenso_tmobileS).equals(j)) {
					stringa.add(cnt.getAnagrafica().getConsenso_tmobileS());
				} else if(ti.getPosizione(ConstantsMap.consenso_tfaxS).equals(j)) {
					stringa.add(cnt.getAnagrafica().getConsenso_tfaxS());
				} else if(ti.getPosizione(ConstantsMap.consenso_taltroS).equals(j)) {
					stringa.add(cnt.getAnagrafica().getConsenso_taltroS());
				} else if(ti.getPosizione(ConstantsMap.consenso_emailS).equals(j)) {
					stringa.add(cnt.getAnagrafica().getConsenso_emailS());
				} else if(ti.getPosizione(ConstantsMap.consenso_rmailS).equals(j)) {
					stringa.add(cnt.getAnagrafica().getConsenso_rmailS());
				} else {
					stringa.add(stampaDato(titles[j], cnt.getAnagrafica().getDettagli(), crt.getDettagli()));
				}
			}
			writer.writeNext(stringa.toArray(new String[0]));
		}
		writer.close();
	}
	private static String stampaDato(String campo, List<DatoAggiuntivoAnagrafica> dati1, List<DatoAggiuntivoContratto> dati2) {
		boolean inserito = false;
		DatoAggiuntivoAnagrafica da1 = ApplicationUtil.getDatoAnagrafica(campo, dati1);
		if(da1!= null) {
			inserito = true;
			return da1.getValore_formattato();
		} 
		if(dati2 != null && !inserito) {
			DatoAggiuntivoContratto da2 = ApplicationUtil.getDatoContratto(campo, dati2);
			if(da2!= null) {
				inserito = true;
				return da2.getValore_formattato();
			} 
		}
		return "";
	}
	public static void esportaCSVStoriaContrattoTemplate(OutputStream out, Boolean p, List<StoriaContratto> data, AnagraficaDao anagraficaDao, TemplateExportStoriaContratto ti) throws Exception {
		List <String> titlesList = ti.getValore_multiplo();
		List <String> aggiuntivi = ListUtils.subtract(titlesList, new ArrayList<String>(ConstantsMap.storiaContrattoMap));
		if(p!=null && ApplicationUtil.notEmpty(aggiuntivi)) {
			List <String> invA = anagraficaDao.getEtichetteInvisibiliByChiave(p, CampoAggiuntivoAnagrafica.class);
			List <String> invB = anagraficaDao.getEtichetteInvisibiliByChiave(p, CampoAggiuntivoContratto.class);
			aggiuntivi.removeAll(invA);
			aggiuntivi.removeAll(invB);
			titlesList.removeAll(invA);
			titlesList.removeAll(invB);
		}
		if(ApplicationUtil.notEmpty(aggiuntivi)) {
			data = anagraficaDao.loadFullStoriaContratto(data);
		}
		String[] titles = titlesList.toArray(new String[0]);
		CSVWriter writer = new CSVWriter(new OutputStreamWriter(out, ConstantsGlobal.CHARSET),ti.getSeparatoreChar(), ti.getQuoteChar());
		List<String> header = new ArrayList<String>();
		if(ti.getIntestazione()) {
			for (int i = 0; i < titles.length; i++) {
				if(ApplicationUtil.notEmpty(aggiuntivi) && aggiuntivi.contains(titles[i])) {
					String etichetta = anagraficaDao.getEtichettaByChiave(CampoAggiuntivoAnagrafica.class, titles[i]);
					if(ApplicationUtil.isBlank(etichetta)) {
						etichetta = anagraficaDao.getEtichettaByChiave(CampoAggiuntivoContratto.class, titles[i]);
					}
					header.add(etichetta);
				} else {
					header.add(Message.addCatalogText(titles[i]));
				}
			}
		} 
		if(ApplicationUtil.notEmpty(header)) {
			writer.writeNext(header.toArray(new String[0]));
		}
		Iterator<StoriaContratto> objIter = data.iterator();
		while(objIter.hasNext()) {
			StoriaContratto scrt = objIter.next();
			Contratto crt = scrt.getContratto();
			Contatto cnt = crt.getContatto();
			List<String> stringa = new ArrayList<String>();
			for (int j = 0; j < titles.length; j++) {
				if(ti.getPosizione(ConstantsMap.storia_contratto_data).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateSeconds.stampa(), scrt.getData()));
				} else if(ti.getPosizione(ConstantsMap.storia_contratto_id).equals(j)) {
					stringa.add(scrt.getId().toString());
				} else if(ti.getPosizione(ConstantsMap.storia_contratto_operatore_denominazione).equals(j)) {
					stringa.add(scrt.getOperatore_denominazione());
				} else if(ti.getPosizione(ConstantsMap.storia_contratto_operatore_codice).equals(j)) {
					stringa.add(scrt.getOperatore_codice());
				} else if(ti.getPosizione(ConstantsMap.storia_contratto_agente_denominazione).equals(j)) {
					stringa.add(scrt.getAgente_denominazione());
				} else if(ti.getPosizione(ConstantsMap.storia_contratto_agente_codice).equals(j)) {
					stringa.add(scrt.getAgente_codice());
				} else if(ti.getPosizione(ConstantsMap.storia_contratto_categoria_denominazione).equals(j)) {
					stringa.add(scrt.getCategoria_denominazione());
				} else if(ti.getPosizione(ConstantsMap.storia_contratto_prodotto_denominazione).equals(j)) {
					stringa.add(scrt.getProdotto_denominazione());
				} else if(ti.getPosizione(ConstantsMap.storia_contratto_esito_contratto).equals(j)) {
					stringa.add(scrt.getEsitoEtichetta());
				} else if(ti.getPosizione(ConstantsMap.storia_contratto_codice_contratto).equals(j)) {
					stringa.add(scrt.getEsitoCodice());
				} else if(ti.getPosizione(ConstantsMap.storia_contratto_data_stipula).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.date.stampa(),scrt.getData_stipula()));
				} else if(ti.getPosizione(ConstantsMap.storia_contratto_data_contabile).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.date.stampa(),scrt.getData_contabile()));
				} else if(ti.getPosizione(ConstantsMap.storia_contratto_data_attivazione).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.date.stampa(),scrt.getData_attivazione()));
				} else if(ti.getPosizione(ConstantsMap.storia_contratto_data_caricamento).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.date.stampa(),scrt.getData_caricamento()));
				} else if(ti.getPosizione(ConstantsMap.storia_contratto_bloccato).equals(j)) {
					stringa.add(scrt.getBloccatoS());
				} else if(ti.getPosizione(ConstantsMap.storia_contratto_utente_denominazione).equals(j)) {
					stringa.add(scrt.getUtente_denominazione());
				} else if(ti.getPosizione(ConstantsMap.storia_contratto_utente_codice).equals(j)) {
					stringa.add(scrt.getUtente_codice());
				} else if(ti.getPosizione(ConstantsMap.storia_contratto_data_esito).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateTime.stampa(), scrt.getData_esito()));
				} else if(ti.getPosizione(ConstantsMap.storia_contratto_data_stato).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateTime.stampa(), scrt.getData_stato()));
				} else if(ti.getPosizione(ConstantsMap.storia_contratto_nota).equals(j)) {
					stringa.add(scrt.getNota());
				} else if(ti.getPosizione(ConstantsMap.storia_contratto_tipo_seriale).equals(j)) {
					stringa.add(scrt.getTipo_seriale());
				} else if(ti.getPosizione(ConstantsMap.storia_contratto_seriale).equals(j)) {
					stringa.add(scrt.getSeriale());
				} else if(ti.getPosizione(ConstantsMap.storia_contratto_quantita).equals(j)) {
					stringa.add(ApplicationUtil.formatNumber(scrt.getQuantita()));
				} else if(ti.getPosizione(ConstantsMap.storia_contratto_costo_scontato).equals(j)) {
					stringa.add(ApplicationUtil.formatNumber(scrt.getCosto_scontato()));
				} else if(ti.getPosizione(ConstantsMap.storia_contratto_stato).equals(j)) {
					stringa.add(scrt.getStatoTesto());
				} else if(ti.getPosizione(ConstantsMap.contratto_data_creazione).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateTime.stampa(), crt.getData_creazione()));
				} else if(ti.getPosizione(ConstantsMap.contratto_id).equals(j)) {
					stringa.add(crt.getId().toString());
				} else if(ti.getPosizione(ConstantsMap.contratto_operatore_denominazione).equals(j)) {
					stringa.add(crt.getOperatore_denominazione());
				} else if(ti.getPosizione(ConstantsMap.contratto_operatore_codice).equals(j)) {
					stringa.add(crt.getOperatore_codice());
				} else if(ti.getPosizione(ConstantsMap.contratto_agente_denominazione).equals(j)) {
					stringa.add(crt.getAgente_denominazione());
				} else if(ti.getPosizione(ConstantsMap.contratto_agente_codice).equals(j)) {
					stringa.add(crt.getAgente_codice());
				} else if(ti.getPosizione(ConstantsMap.contratto_categoria_denominazione).equals(j)) {
					stringa.add(crt.getCategoria_denominazione());
				} else if(ti.getPosizione(ConstantsMap.contratto_prodotto_denominazione).equals(j)) {
					stringa.add(crt.getProdotto_denominazione());
				} else if(ti.getPosizione(ConstantsMap.contratto_esito_contratto).equals(j)) {
					stringa.add(crt.getEsitoEtichetta());
				} else if(ti.getPosizione(ConstantsMap.contratto_codice_contratto).equals(j)) {
					stringa.add(crt.getEsitoCodice());
				} else if(ti.getPosizione(ConstantsMap.contratto_data_stipula).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.date.stampa(),crt.getData_stipula()));
				} else if(ti.getPosizione(ConstantsMap.contratto_data_contabile).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.date.stampa(),crt.getData_contabile()));
				} else if(ti.getPosizione(ConstantsMap.contratto_data_attivazione).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.date.stampa(),crt.getData_attivazione()));
				} else if(ti.getPosizione(ConstantsMap.contratto_data_caricamento).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.date.stampa(),crt.getData_caricamento()));
				} else if(ti.getPosizione(ConstantsMap.contratto_data_lavorazione).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateTime.stampa(),crt.getData_lavorazione()));
				} else if(ti.getPosizione(ConstantsMap.contratto_utente_denominazione).equals(j)) {
					stringa.add(crt.getUtente_denominazione());
				} else if(ti.getPosizione(ConstantsMap.contratto_bloccato).equals(j)) {
					stringa.add(crt.getBloccatoS());
				} else if(ti.getPosizione(ConstantsMap.contratto_utente_codice).equals(j)) {
					stringa.add(crt.getUtente_codice());
				} else if(ti.getPosizione(ConstantsMap.contratto_data_esito).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateTime.stampa(), crt.getData_esito()));
				} else if(ti.getPosizione(ConstantsMap.contratto_data_stato).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateTime.stampa(), crt.getData_stato()));
				} else if(ti.getPosizione(ConstantsMap.contratto_nota).equals(j)) {
					stringa.add(crt.getNota());
				} else if(ti.getPosizione(ConstantsMap.contratto_tipo_seriale).equals(j)) {
					stringa.add(crt.getTipo_seriale());
				} else if(ti.getPosizione(ConstantsMap.contratto_seriale).equals(j)) {
					stringa.add(crt.getSeriale());
				} else if(ti.getPosizione(ConstantsMap.contratto_quantita).equals(j)) {
					stringa.add(ApplicationUtil.formatNumber(crt.getQuantita()));
				} else if(ti.getPosizione(ConstantsMap.contratto_costo_scontato).equals(j)) {
					stringa.add(ApplicationUtil.formatNumber(crt.getCosto_scontato()));
				} else if(ti.getPosizione(ConstantsMap.contratto_stato).equals(j)) {
					stringa.add(crt.getStatoTesto());
				} else if(ti.getPosizione(ConstantsMap.contatto_data_archiviazione).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateTime.stampa(), cnt.getData_archiviazione()));
				} else if(ti.getPosizione(ConstantsMap.contatto_data_lavorazione).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateTime.stampa(), cnt.getData_lavorazione()));
				} else if(ti.getPosizione(ConstantsMap.contatto_operatore_denominazione).equals(j)) {
					stringa.add(cnt.getOperatore_denominazione());
				} else if(ti.getPosizione(ConstantsMap.contatto_operatore_codice).equals(j)) {
					stringa.add(cnt.getOperatore_codice());
				} else if(ti.getPosizione(ConstantsMap.contatto_esito_chiamata).equals(j)) {
					stringa.add(cnt.getEsito_chiamata());
				} else if(ti.getPosizione(ConstantsMap.contatto_esito_archiviazione).equals(j)) {
					stringa.add(cnt.getEsito_archiviazione());
				} else if(ti.getPosizione(ConstantsMap.contatto_esito_generale).equals(j)) {
					stringa.add(cnt.getEsito_generale());
				} else if(ti.getPosizione(ConstantsMap.contatto_codice_chiamata).equals(j)) {
					stringa.add(cnt.getCodice_chiamata());
				} else if(ti.getPosizione(ConstantsMap.contatto_codice_archiviazione).equals(j)) {
					stringa.add(cnt.getCodice_archiviazione());
				} else if(ti.getPosizione(ConstantsMap.contatto_codice_generale).equals(j)) {
					stringa.add(cnt.getCodice_generale());
				} else if(ti.getPosizione(ConstantsMap.contatto_proprietario_denominazione).equals(j)) {
					stringa.add(cnt.getProprietario_denominazione());
				} else if(ti.getPosizione(ConstantsMap.contatto_proprietario_codice).equals(j)) {
					stringa.add(cnt.getProprietario_codice());
				} else if(ti.getPosizione(ConstantsMap.contatto_data_presentazione).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateTime.stampa(), cnt.getData_presentazione()));
				} else if(ti.getPosizione(ConstantsMap.contatto_nota).equals(j)) {
					stringa.add(cnt.getNota());
				} else if(ti.getPosizione(ConstantsMap.contatto_data_creazione).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateTime.stampa(), cnt.getData_creazione()));
				} else if(ti.getPosizione(ConstantsMap.contatto_agente_denominazione).equals(j)) {
					stringa.add(cnt.getAgente_denominazione());
				} else if(ti.getPosizione(ConstantsMap.contatto_agente_codice).equals(j)) {
					stringa.add(cnt.getAgente_codice());
				} else if(ti.getPosizione(ConstantsMap.contatto_stato).equals(j)) {
					stringa.add(cnt.getTipoTesto());
				} else if(ti.getPosizione(ConstantsMap.contatto_id).equals(j)) {
					stringa.add(cnt.getId().toString());
				} else if(ti.getPosizione(ConstantsMap.contatto_campagna_denominazione).equals(j)) {
					stringa.add(cnt.getCampagna_denominazione());
				} else if(ti.getPosizione(ConstantsMap.contatto_lista_denominazione).equals(j)) {
					stringa.add(cnt.getLista_denominazione());
				} else if(ti.getPosizione(ConstantsMap.contatto_campagna_codice).equals(j)) {
					stringa.add(cnt.getCampagna_codice());
				} else if(ti.getPosizione(ConstantsMap.contatto_bloccato).equals(j)) {
					stringa.add(cnt.getBloccatoS());
				} else if(ti.getPosizione(ConstantsMap.contatto_lista_codice).equals(j)) {
					stringa.add(cnt.getLista_codice());
				} else if(ti.getPosizione(ConstantsMap.contatto_positivo).equals(j)) {
					stringa.add(cnt.getPositivo());
				} else if(ti.getPosizione(ConstantsMap.contatto_telefono_predefinito).equals(j)) {
					stringa.add(ApplicationUtil.uniforma(cnt.getTelefono_predefinito(), ti.getRem_predefinito(), ti.getAdd_predefinito()));
				} else if(ti.getPosizione(ConstantsMap.contatto_utile).equals(j)) {
					stringa.add(cnt.getUtile());
				} else if(ti.getPosizione(ConstantsMap.contatto_chiuso).equals(j)) {
					stringa.add(cnt.getChiuso());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_cognome).equals(j)) {
					stringa.add(cnt.getAnagrafica().getCognome());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_nome).equals(j)) {
					stringa.add(cnt.getAnagrafica().getNome());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_nome_completo).equals(j)) {
					stringa.add(cnt.getAnagrafica().getNome_completo());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_email).equals(j)) {
					stringa.add(cnt.getAnagrafica().getEmail());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_codice_fiscale).equals(j)) {
					stringa.add(cnt.getAnagrafica().getCodice_fiscale());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_sorgente).equals(j)) {
					stringa.add(cnt.getAnagrafica().getSorgente());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_nota).equals(j)) {
					stringa.add(cnt.getAnagrafica().getNota());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_luogo_nascita).equals(j)) {
					stringa.add(cnt.getAnagrafica().getLuogo_nascita());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_data_nascita).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.date.stampa(), cnt.getAnagrafica().getDate_nascita()));
				} else if(ti.getPosizione(ConstantsMap.anagrafica_link).equals(j)) {
					stringa.add(cnt.getAnagrafica().getLink());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_data_inserimento).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateTime.stampa(), cnt.getAnagrafica().getData_creazione()));
				} else if(ti.getPosizione(ConstantsMap.anagrafica_privacy).equals(j)) {
					stringa.add(cnt.getAnagrafica().getPrivacy());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_tipo).equals(j)) {
					stringa.add(cnt.getAnagrafica().getTipo());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_sesso).equals(j)) {
					stringa.add(cnt.getAnagrafica().getSesso());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_partita_iva).equals(j)) {
					stringa.add(cnt.getAnagrafica().getPartita_iva());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_ragione_sociale).equals(j)) {
					stringa.add(cnt.getAnagrafica().getRagione_sociale());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_telefono_fisso).equals(j)) {
					stringa.add(ApplicationUtil.uniforma(cnt.getAnagrafica().getTelefono_fisso(), ti.getRem_fisso(), ti.getAdd_fisso()));
				} else if(ti.getPosizione(ConstantsMap.anagrafica_telefono_altro).equals(j)) {
					stringa.add(ApplicationUtil.uniforma(cnt.getAnagrafica().getTelefono_altro(), ti.getRem_altro(), ti.getAdd_altro()));
				} else if(ti.getPosizione(ConstantsMap.anagrafica_telefono_mobile).equals(j)) {
					stringa.add(ApplicationUtil.uniforma(cnt.getAnagrafica().getTelefono_mobile(), ti.getRem_mobile(), ti.getAdd_mobile()));
				} else if(ti.getPosizione(ConstantsMap.anagrafica_telefono_fax).equals(j)) {
					stringa.add(ApplicationUtil.uniforma(cnt.getAnagrafica().getTelefono_fax(), ti.getRem_fax(), ti.getAdd_fax()));
				} else if(ti.getPosizione(ConstantsMap.anagrafica_settore).equals(j)) {
					stringa.add(cnt.getAnagrafica().getSettore());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_denominazione).equals(j)) {
					stringa.add(cnt.getAnagrafica().getDenominazione());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_id).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIdentificativo());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_indirizzo_completo).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getIndirizzo_completo());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_strada_completo).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getStrada_completo());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_comune_completo).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getComune_completo());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_strada).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getStrada());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_civico).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getCivico());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_cap).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getCap());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_comune).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getComune());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_provincia).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getProvincia());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_regione).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getRegione());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_dettagli).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getDettagli());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_latitudine).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getLatitudine());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_longitudine).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getLongitudine());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_presso).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getPresso());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_indirizzo_completo).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getIndirizzo_completo());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_strada_completo).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getStrada_completo());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_comune_completo).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getComune_completo());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_strada).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getStrada());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_civico).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getCivico());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_cap).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getCap());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_comune).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getComune());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_provincia).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getProvincia());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_regione).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getRegione());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_dettagli).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getDettagli());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_latitudine).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getLatitudine());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_longitudine).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getLongitudine());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_presso).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getPresso());
				} else if(ti.getPosizione(ConstantsMap.consenso_tfissoS).equals(j)) {
					stringa.add(cnt.getAnagrafica().getConsenso_tfissoS());
				} else if(ti.getPosizione(ConstantsMap.consenso_tmobileS).equals(j)) {
					stringa.add(cnt.getAnagrafica().getConsenso_tmobileS());
				} else if(ti.getPosizione(ConstantsMap.consenso_tfaxS).equals(j)) {
					stringa.add(cnt.getAnagrafica().getConsenso_tfaxS());
				} else if(ti.getPosizione(ConstantsMap.consenso_taltroS).equals(j)) {
					stringa.add(cnt.getAnagrafica().getConsenso_taltroS());
				} else if(ti.getPosizione(ConstantsMap.consenso_emailS).equals(j)) {
					stringa.add(cnt.getAnagrafica().getConsenso_emailS());
				} else if(ti.getPosizione(ConstantsMap.consenso_rmailS).equals(j)) {
					stringa.add(cnt.getAnagrafica().getConsenso_rmailS());
				} else {
					stringa.add(stampaDato(titles[j], cnt.getAnagrafica().getDettagli(), crt.getDettagli()));
				} 
			}
			writer.writeNext(stringa.toArray(new String[0]));
		}
		writer.close();
	}
	public static void esportaCSVStoriaContattoTemplate(OutputStream out, Boolean p, List<StoriaContatto> data, AnagraficaDao anagraficaDao, TemplateExportStoriaContatto ti) throws Exception {
		List <String> titlesList = ti.getValore_multiplo();
		List <String> aggiuntivi = ListUtils.subtract(titlesList, new ArrayList<String>(ConstantsMap.storiaContattoMap));
		if(p!=null && ApplicationUtil.notEmpty(aggiuntivi)) {
			List <String> invA = anagraficaDao.getEtichetteInvisibiliByChiave(p, CampoAggiuntivoAnagrafica.class);
			aggiuntivi.removeAll(invA);
			titlesList.removeAll(invA);
		}
		if(ApplicationUtil.notEmpty(aggiuntivi)) {
			data = anagraficaDao.loadFullStoriaContatto(data);
		}
		String[] titles = titlesList.toArray(new String[0]);
		CSVWriter writer = new CSVWriter(new OutputStreamWriter(out, ConstantsGlobal.CHARSET),ti.getSeparatoreChar(), ti.getQuoteChar());
		List<String> header = new ArrayList<String>();
		if(ti.getIntestazione()) {
			for (int i = 0; i < titles.length; i++) {
				if(ApplicationUtil.notEmpty(aggiuntivi) && aggiuntivi.contains(titles[i])) {
					String etichetta = anagraficaDao.getEtichettaByChiave(CampoAggiuntivoAnagrafica.class, titles[i]);
					header.add(etichetta);
				} else {
					header.add(Message.addCatalogText(titles[i]));
				}
			}
		} 
		if(ApplicationUtil.notEmpty(header)) {
			writer.writeNext(header.toArray(new String[0]));
		}
		Iterator<StoriaContatto> objIter = data.iterator();
		while(objIter.hasNext()) {
			StoriaContatto scnt = objIter.next();
			Contatto cnt = scnt.getContatto();
			List<String> stringa = new ArrayList<String>();
			for (int j = 0; j < titles.length; j++) {
				if(ti.getPosizione(ConstantsMap.storia_contatto_data_archiviazione).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateTime.stampa(), scnt.getData_archiviazione()));
				} else if(ti.getPosizione(ConstantsMap.storia_contatto_esito_chiamata).equals(j)) {
					stringa.add(scnt.getEsito_chiamata());
				} else if(ti.getPosizione(ConstantsMap.storia_contatto_esito_archiviazione).equals(j)) {
					stringa.add(scnt.getEsito_archiviazione());
				} else if(ti.getPosizione(ConstantsMap.storia_contatto_esito_generale).equals(j)) {
					stringa.add(scnt.getEsito_generale());
				} else if(ti.getPosizione(ConstantsMap.storia_contatto_codice_chiamata).equals(j)) {
					stringa.add(scnt.getCodice_chiamata());
				} else if(ti.getPosizione(ConstantsMap.storia_contatto_codice_archiviazione).equals(j)) {
					stringa.add(scnt.getCodice_archiviazione());
				} else if(ti.getPosizione(ConstantsMap.storia_contatto_codice_generale).equals(j)) {
					stringa.add(scnt.getCodice_generale());
				} else if(ti.getPosizione(ConstantsMap.storia_contatto_campagna_denominazione).equals(j)) {
					stringa.add(scnt.getCampagna_denominazione());
				} else if(ti.getPosizione(ConstantsMap.storia_contatto_lista_denominazione).equals(j)) {
					stringa.add(scnt.getLista_denominazione());
				} else if(ti.getPosizione(ConstantsMap.storia_contatto_campagna_codice).equals(j)) {
					stringa.add(scnt.getCampagna_codice());
				} else if(ti.getPosizione(ConstantsMap.storia_contatto_bloccato).equals(j)) {
					stringa.add(scnt.getBloccatoS());
				} else if(ti.getPosizione(ConstantsMap.storia_contatto_lista_codice).equals(j)) {
					stringa.add(scnt.getLista_codice());
				} else if(ti.getPosizione(ConstantsMap.storia_contatto_operatore_denominazione).equals(j)) {
					stringa.add(scnt.getOperatore_denominazione());
				} else if(ti.getPosizione(ConstantsMap.storia_contatto_operatore_codice).equals(j)) {
					stringa.add(scnt.getOperatore_codice());
				} else if(ti.getPosizione(ConstantsMap.storia_contatto_proprietario_denominazione).equals(j)) {
					stringa.add(scnt.getProprietario_denominazione());
				} else if(ti.getPosizione(ConstantsMap.storia_contatto_proprietario_codice).equals(j)) {
					stringa.add(scnt.getProprietario_codice());
				} else if(ti.getPosizione(ConstantsMap.storia_contatto_data_presentazione).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateTime.stampa(), scnt.getData_presentazione()));
				} else if(ti.getPosizione(ConstantsMap.storia_contatto_nota).equals(j)) {
					stringa.add(scnt.getNota());
				} else if(ti.getPosizione(ConstantsMap.storia_contatto_data).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateSeconds.stampa(), scnt.getData()));
				} else if(ti.getPosizione(ConstantsMap.storia_contatto_agente_denominazione).equals(j)) {
					stringa.add(scnt.getAgente_denominazione());
				} else if(ti.getPosizione(ConstantsMap.storia_contatto_agente_codice).equals(j)) {
					stringa.add(scnt.getAgente_codice());
				} else if(ti.getPosizione(ConstantsMap.storia_contatto_stato).equals(j)) {
					stringa.add(scnt.getTipoTesto());
				} else if(ti.getPosizione(ConstantsMap.storia_contatto_id).equals(j)) {
					stringa.add(scnt.getId().toString());
				} else if(ti.getPosizione(ConstantsMap.storia_contatto_positivo).equals(j)) {
					stringa.add(scnt.getPositivo());
				} else if(ti.getPosizione(ConstantsMap.storia_contatto_utile).equals(j)) {
					stringa.add(scnt.getUtile());
				} else if(ti.getPosizione(ConstantsMap.storia_contatto_chiuso).equals(j)) {
					stringa.add(scnt.getChiuso());
				} else if(ti.getPosizione(ConstantsMap.storia_contatto_secondi_tot).equals(j)) {
					stringa.add(ApplicationUtil.formatNumber(scnt.getSecondi_tot()));
				} else if(ti.getPosizione(ConstantsMap.storia_contatto_secondi_totH).equals(j)) {
					stringa.add(scnt.getSecondi_totH());
				} else if(ti.getPosizione(ConstantsMap.storia_contatto_secondi_conv_eff).equals(j)) {
					stringa.add(ApplicationUtil.formatNumber(scnt.getSecondi_conv_eff()));
				} else if(ti.getPosizione(ConstantsMap.storia_contatto_secondi_conv_effH).equals(j)) {
					stringa.add(scnt.getSecondi_conv_effH());
				} else if(ti.getPosizione(ConstantsMap.storia_contatto_secondi_conv_tot).equals(j)) {
					stringa.add(ApplicationUtil.formatNumber(scnt.getSecondi_conv_tot()));
				} else if(ti.getPosizione(ConstantsMap.storia_contatto_secondi_conv_totH).equals(j)) {
					stringa.add(scnt.getSecondi_conv_totH());
				} else if(ti.getPosizione(ConstantsMap.storia_contatto_secondi_post).equals(j)) {
					stringa.add(ApplicationUtil.formatNumber(scnt.getSecondi_post()));
				} else if(ti.getPosizione(ConstantsMap.storia_contatto_secondi_postH).equals(j)) {
					stringa.add(scnt.getSecondi_postH());
				} else if(ti.getPosizione(ConstantsMap.storia_contatto_secondi_pre).equals(j)) {
					stringa.add(ApplicationUtil.formatNumber(scnt.getSecondi_pre()));
				} else if(ti.getPosizione(ConstantsMap.storia_contatto_secondi_preH).equals(j)) {
					stringa.add(scnt.getSecondi_preH());
				} else if(ti.getPosizione(ConstantsMap.storia_contatto_secondi_rec).equals(j)) {
					stringa.add(ApplicationUtil.formatNumber(scnt.getSecondi_rec()));
				} else if(ti.getPosizione(ConstantsMap.storia_contatto_secondi_recH).equals(j)) {
					stringa.add(scnt.getSecondi_recH());
				} else if(ti.getPosizione(ConstantsMap.storia_contatto_secondi_attesa).equals(j)) {
					stringa.add(ApplicationUtil.formatNumber(scnt.getSecondi_attesa()));
				} else if(ti.getPosizione(ConstantsMap.storia_contatto_secondi_attesaH).equals(j)) {
					stringa.add(scnt.getSecondi_attesaH());
				} else if(ti.getPosizione(ConstantsMap.storia_contatto_date_apertura).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateSeconds.stampa(), scnt.getDate_apertura()));
				} else if(ti.getPosizione(ConstantsMap.storia_contatto_date_chiamata).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateSeconds.stampa(), scnt.getDate_chiamata()));
				} else if(ti.getPosizione(ConstantsMap.storia_contatto_date_risposta).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateSeconds.stampa(), scnt.getDate_risposta()));
				} else if(ti.getPosizione(ConstantsMap.storia_contatto_date_rec_inizio).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateSeconds.stampa(), scnt.getDate_rec_inizio()));
				} else if(ti.getPosizione(ConstantsMap.storia_contatto_date_rec_fine).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateSeconds.stampa(), scnt.getDate_rec_fine()));
				} else if(ti.getPosizione(ConstantsMap.storia_contatto_date_fine).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateSeconds.stampa(), scnt.getDate_fine()));
				} else if(ti.getPosizione(ConstantsMap.storia_contatto_date_esito).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateSeconds.stampa(), scnt.getDate_esito()));
				} else if(ti.getPosizione(ConstantsMap.storia_contatto_date_attesa_inizio).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateSeconds.stampa(), scnt.getDate_attesa_inizio()));
				} else if(ti.getPosizione(ConstantsMap.storia_contatto_date_attesa_fine).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateSeconds.stampa(), scnt.getDate_attesa_fine()));
				} else if(ti.getPosizione(ConstantsMap.contatto_data_archiviazione).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateTime.stampa(), cnt.getData_archiviazione()));
				} else if(ti.getPosizione(ConstantsMap.contatto_esito_chiamata).equals(j)) {
					stringa.add(cnt.getEsito_chiamata());
				} else if(ti.getPosizione(ConstantsMap.contatto_esito_archiviazione).equals(j)) {
					stringa.add(cnt.getEsito_archiviazione());
				} else if(ti.getPosizione(ConstantsMap.contatto_esito_generale).equals(j)) {
					stringa.add(cnt.getEsito_generale());
				} else if(ti.getPosizione(ConstantsMap.contatto_codice_chiamata).equals(j)) {
					stringa.add(cnt.getCodice_chiamata());
				} else if(ti.getPosizione(ConstantsMap.contatto_codice_archiviazione).equals(j)) {
					stringa.add(cnt.getCodice_archiviazione());
				} else if(ti.getPosizione(ConstantsMap.contatto_codice_generale).equals(j)) {
					stringa.add(cnt.getCodice_generale());
				} else if(ti.getPosizione(ConstantsMap.contatto_data_lavorazione).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateTime.stampa(), cnt.getData_lavorazione()));
				} else if(ti.getPosizione(ConstantsMap.contatto_operatore_denominazione).equals(j)) {
					stringa.add(cnt.getOperatore_denominazione());
				} else if(ti.getPosizione(ConstantsMap.contatto_operatore_codice).equals(j)) {
					stringa.add(cnt.getOperatore_codice());
				} else if(ti.getPosizione(ConstantsMap.contatto_proprietario_denominazione).equals(j)) {
					stringa.add(cnt.getProprietario_denominazione());
				} else if(ti.getPosizione(ConstantsMap.contatto_proprietario_codice).equals(j)) {
					stringa.add(cnt.getProprietario_codice());
				} else if(ti.getPosizione(ConstantsMap.contatto_data_presentazione).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateTime.stampa(), cnt.getData_presentazione()));
				} else if(ti.getPosizione(ConstantsMap.contatto_nota).equals(j)) {
					stringa.add(cnt.getNota());
				} else if(ti.getPosizione(ConstantsMap.contatto_data_creazione).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateTime.stampa(), cnt.getData_creazione()));
				} else if(ti.getPosizione(ConstantsMap.contatto_agente_denominazione).equals(j)) {
					stringa.add(cnt.getAgente_denominazione());
				} else if(ti.getPosizione(ConstantsMap.contatto_agente_codice).equals(j)) {
					stringa.add(cnt.getAgente_codice());
				} else if(ti.getPosizione(ConstantsMap.contatto_stato).equals(j)) {
					stringa.add(cnt.getTipoTesto());
				} else if(ti.getPosizione(ConstantsMap.contatto_id).equals(j)) {
					stringa.add(cnt.getId().toString());
				} else if(ti.getPosizione(ConstantsMap.contatto_campagna_denominazione).equals(j)) {
					stringa.add(cnt.getCampagna_denominazione());
				} else if(ti.getPosizione(ConstantsMap.contatto_lista_denominazione).equals(j)) {
					stringa.add(cnt.getLista_denominazione());
				} else if(ti.getPosizione(ConstantsMap.contatto_campagna_codice).equals(j)) {
					stringa.add(cnt.getCampagna_codice());
				} else if(ti.getPosizione(ConstantsMap.contatto_bloccato).equals(j)) {
					stringa.add(cnt.getBloccatoS());
				} else if(ti.getPosizione(ConstantsMap.contatto_lista_codice).equals(j)) {
					stringa.add(cnt.getLista_codice());
				} else if(ti.getPosizione(ConstantsMap.contatto_positivo).equals(j)) {
					stringa.add(cnt.getPositivo());
				} else if(ti.getPosizione(ConstantsMap.contatto_telefono_predefinito).equals(j)) {
					stringa.add(ApplicationUtil.uniforma(cnt.getTelefono_predefinito(), ti.getRem_predefinito(), ti.getAdd_predefinito()));
				} else if(ti.getPosizione(ConstantsMap.contatto_utile).equals(j)) {
					stringa.add(cnt.getUtile());
				} else if(ti.getPosizione(ConstantsMap.contatto_chiuso).equals(j)) {
					stringa.add(cnt.getChiuso());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_cognome).equals(j)) {
					stringa.add(cnt.getAnagrafica().getCognome());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_nome).equals(j)) {
					stringa.add(cnt.getAnagrafica().getNome());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_nome_completo).equals(j)) {
					stringa.add(cnt.getAnagrafica().getNome_completo());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_email).equals(j)) {
					stringa.add(cnt.getAnagrafica().getEmail());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_codice_fiscale).equals(j)) {
					stringa.add(cnt.getAnagrafica().getCodice_fiscale());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_sorgente).equals(j)) {
					stringa.add(cnt.getAnagrafica().getSorgente());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_nota).equals(j)) {
					stringa.add(cnt.getAnagrafica().getNota());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_luogo_nascita).equals(j)) {
					stringa.add(cnt.getAnagrafica().getLuogo_nascita());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_data_nascita).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.date.stampa(), cnt.getAnagrafica().getDate_nascita()));
				} else if(ti.getPosizione(ConstantsMap.anagrafica_link).equals(j)) {
					stringa.add(cnt.getAnagrafica().getLink());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_data_inserimento).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateTime.stampa(), cnt.getAnagrafica().getData_creazione()));
				} else if(ti.getPosizione(ConstantsMap.anagrafica_privacy).equals(j)) {
					stringa.add(cnt.getAnagrafica().getPrivacy());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_tipo).equals(j)) {
					stringa.add(cnt.getAnagrafica().getTipo());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_sesso).equals(j)) {
					stringa.add(cnt.getAnagrafica().getSesso());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_partita_iva).equals(j)) {
					stringa.add(cnt.getAnagrafica().getPartita_iva());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_ragione_sociale).equals(j)) {
					stringa.add(cnt.getAnagrafica().getRagione_sociale());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_telefono_fisso).equals(j)) {
					stringa.add(ApplicationUtil.uniforma(cnt.getAnagrafica().getTelefono_fisso(), ti.getRem_fisso(), ti.getAdd_fisso()));
				} else if(ti.getPosizione(ConstantsMap.anagrafica_telefono_altro).equals(j)) {
					stringa.add(ApplicationUtil.uniforma(cnt.getAnagrafica().getTelefono_altro(), ti.getRem_altro(), ti.getAdd_altro()));
				} else if(ti.getPosizione(ConstantsMap.anagrafica_telefono_mobile).equals(j)) {
					stringa.add(ApplicationUtil.uniforma(cnt.getAnagrafica().getTelefono_mobile(), ti.getRem_mobile(), ti.getAdd_mobile()));
				} else if(ti.getPosizione(ConstantsMap.anagrafica_telefono_fax).equals(j)) {
					stringa.add(ApplicationUtil.uniforma(cnt.getAnagrafica().getTelefono_fax(), ti.getRem_fax(), ti.getAdd_fax()));
				} else if(ti.getPosizione(ConstantsMap.anagrafica_settore).equals(j)) {
					stringa.add(cnt.getAnagrafica().getSettore());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_denominazione).equals(j)) {
					stringa.add(cnt.getAnagrafica().getDenominazione());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_id).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIdentificativo());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_indirizzo_completo).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getIndirizzo_completo());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_strada_completo).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getStrada_completo());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_comune_completo).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getComune_completo());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_strada).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getStrada());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_civico).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getCivico());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_cap).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getCap());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_comune).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getComune());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_provincia).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getProvincia());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_regione).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getRegione());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_dettagli).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getDettagli());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_latitudine).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getLatitudine());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_longitudine).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getLongitudine());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_presso).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getPresso());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_indirizzo_completo).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getIndirizzo_completo());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_strada_completo).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getStrada_completo());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_comune_completo).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getComune_completo());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_strada).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getStrada());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_civico).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getCivico());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_cap).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getCap());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_comune).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getComune());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_provincia).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getProvincia());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_regione).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getRegione());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_dettagli).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getDettagli());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_latitudine).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getLatitudine());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_longitudine).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getLongitudine());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_presso).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getPresso());
				} else if(ti.getPosizione(ConstantsMap.consenso_tfissoS).equals(j)) {
					stringa.add(cnt.getAnagrafica().getConsenso_tfissoS());
				} else if(ti.getPosizione(ConstantsMap.consenso_tmobileS).equals(j)) {
					stringa.add(cnt.getAnagrafica().getConsenso_tmobileS());
				} else if(ti.getPosizione(ConstantsMap.consenso_tfaxS).equals(j)) {
					stringa.add(cnt.getAnagrafica().getConsenso_tfaxS());
				} else if(ti.getPosizione(ConstantsMap.consenso_taltroS).equals(j)) {
					stringa.add(cnt.getAnagrafica().getConsenso_taltroS());
				} else if(ti.getPosizione(ConstantsMap.consenso_emailS).equals(j)) {
					stringa.add(cnt.getAnagrafica().getConsenso_emailS());
				} else if(ti.getPosizione(ConstantsMap.consenso_rmailS).equals(j)) {
					stringa.add(cnt.getAnagrafica().getConsenso_rmailS());
				} else {
					stringa.add(stampaDato(titles[j], cnt.getAnagrafica().getDettagli(), null));
				}
			}
			writer.writeNext(stringa.toArray(new String[0]));
		}
		writer.close();
	}
	public static void esportaCSVStoriaAppuntamentoTemplate(OutputStream out, Boolean p, List<StoriaAppuntamento> data, AnagraficaDao anagraficaDao, TemplateExportStoriaAppuntamento ti) throws Exception {
		List <String> titlesList = ti.getValore_multiplo();
		List <String> aggiuntivi = ListUtils.subtract(titlesList, new ArrayList<String>(ConstantsMap.storiaAppuntamentoMap));
		if(p!=null && ApplicationUtil.notEmpty(aggiuntivi)) {
			List <String> invA = anagraficaDao.getEtichetteInvisibiliByChiave(p, CampoAggiuntivoAnagrafica.class);
			aggiuntivi.removeAll(invA);
			titlesList.removeAll(invA);
		}
		if(ApplicationUtil.notEmpty(aggiuntivi)) {
			data = anagraficaDao.loadFullStoriaAppuntamento(data);
		}
		String[] titles = titlesList.toArray(new String[0]);
		CSVWriter writer = new CSVWriter(new OutputStreamWriter(out, ConstantsGlobal.CHARSET),ti.getSeparatoreChar(), ti.getQuoteChar());
		List<String> header = new ArrayList<String>();
		if(ti.getIntestazione()) {
			for (int i = 0; i < titles.length; i++) {
				if(ApplicationUtil.notEmpty(aggiuntivi) && aggiuntivi.contains(titles[i])) {
					String etichetta = anagraficaDao.getEtichettaByChiave(CampoAggiuntivoAnagrafica.class, titles[i]);
					header.add(etichetta);
				} else {
					header.add(Message.addCatalogText(titles[i]));
				}
			}
		} 
		if(ApplicationUtil.notEmpty(header)) {
			writer.writeNext(header.toArray(new String[0]));
		}
		Iterator<StoriaAppuntamento> objIter = data.iterator();
		while(objIter.hasNext()) {
			StoriaAppuntamento sapp = objIter.next();
			Appuntamento app = sapp.getAppuntamento();
			Contatto cnt = app.getContatto();
			List<String> stringa = new ArrayList<String>();
			for (int j = 0; j < titles.length; j++) {
				if(ti.getPosizione(ConstantsMap.storia_appuntamento_id).equals(j)) {
					stringa.add(sapp.getId().toString());
				} else if(ti.getPosizione(ConstantsMap.storia_appuntamento_operatore_denominazione).equals(j)) {
					stringa.add(sapp.getOperatore_denominazione());
				} else if(ti.getPosizione(ConstantsMap.storia_appuntamento_bloccato).equals(j)) {
					stringa.add(sapp.getBloccatoS());
				} else if(ti.getPosizione(ConstantsMap.storia_appuntamento_operatore_codice).equals(j)) {
					stringa.add(sapp.getOperatore_codice());
				} else if(ti.getPosizione(ConstantsMap.storia_appuntamento_agente_denominazione).equals(j)) {
					stringa.add(sapp.getAgente_denominazione());
				} else if(ti.getPosizione(ConstantsMap.storia_appuntamento_agente_codice).equals(j)) {
					stringa.add(sapp.getAgente_codice());
				} else if(ti.getPosizione(ConstantsMap.storia_appuntamento_nota).equals(j)) {
					stringa.add(sapp.getNota());
				} else if(ti.getPosizione(ConstantsMap.storia_appuntamento_data).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateSeconds.stampa(), sapp.getData()));
				} else if(ti.getPosizione(ConstantsMap.storia_appuntamento_data_inizio).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateTime.stampa(), sapp.getData_inizio()));
				} else if(ti.getPosizione(ConstantsMap.storia_appuntamento_data_fine).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateTime.stampa(), sapp.getData_fine()));
				} else if(ti.getPosizione(ConstantsMap.storia_appuntamento_data_conferma).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateTime.stampa(), sapp.getData_conferma()));
				} else if(ti.getPosizione(ConstantsMap.storia_appuntamento_localizzazione).equals(j)) {
					stringa.add(sapp.getLocalizzazioneString());
				} else if(ti.getPosizione(ConstantsMap.storia_appuntamento_utente_denominazione).equals(j)) {
					stringa.add(sapp.getUtente_denominazione());
				} else if(ti.getPosizione(ConstantsMap.storia_appuntamento_utente_codice).equals(j)) {
					stringa.add(sapp.getUtente_codice());
				} else if(ti.getPosizione(ConstantsMap.storia_appuntamento_data_esito).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateTime.stampa(), sapp.getData_esito()));
				} else if(ti.getPosizione(ConstantsMap.storia_appuntamento_data_stato).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateTime.stampa(), sapp.getData_stato()));
				} else if(ti.getPosizione(ConstantsMap.storia_appuntamento_esito_appuntamento).equals(j)) {
					stringa.add(sapp.getEsitoEtichetta());
				} else if(ti.getPosizione(ConstantsMap.storia_appuntamento_codice_appuntamento).equals(j)) {
					stringa.add(sapp.getEsitoCodice());
				} else if(ti.getPosizione(ConstantsMap.storia_appuntamento_stato).equals(j)) {
					stringa.add(sapp.getStatoTesto());
				} else if(ti.getPosizione(ConstantsMap.appuntamento_id).equals(j)) {
					stringa.add(app.getId().toString());
				} else if(ti.getPosizione(ConstantsMap.appuntamento_operatore_denominazione).equals(j)) {
					stringa.add(app.getOperatore_denominazione());
				} else if(ti.getPosizione(ConstantsMap.appuntamento_bloccato).equals(j)) {
					stringa.add(app.getBloccatoS());
				} else if(ti.getPosizione(ConstantsMap.appuntamento_operatore_codice).equals(j)) {
					stringa.add(app.getOperatore_codice());
				} else if(ti.getPosizione(ConstantsMap.appuntamento_agente_denominazione).equals(j)) {
					stringa.add(app.getAgente_denominazione());
				} else if(ti.getPosizione(ConstantsMap.appuntamento_agente_codice).equals(j)) {
					stringa.add(app.getAgente_codice());
				} else if(ti.getPosizione(ConstantsMap.appuntamento_nota).equals(j)) {
					stringa.add(app.getNota());
				} else if(ti.getPosizione(ConstantsMap.appuntamento_data_creazione).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateTime.stampa(), app.getData_creazione()));
				} else if(ti.getPosizione(ConstantsMap.appuntamento_data_appuntamento).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.date.stampa(), app.getData_inizio()));
				} else if(ti.getPosizione(ConstantsMap.appuntamento_data_ora).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateTime.stampa(), app.getData_inizio()));
				} else if(ti.getPosizione(ConstantsMap.appuntamento_ora_inizio).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.time.stampa(), app.getData_inizio()));
				} else if(ti.getPosizione(ConstantsMap.appuntamento_ora_fine).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.time.stampa(), app.getData_fine()));
				} else if(ti.getPosizione(ConstantsMap.appuntamento_data_conferma).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateTime.stampa(), app.getData_conferma()));
				} else if(ti.getPosizione(ConstantsMap.appuntamento_data_lavorazione).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateTime.stampa(), app.getData_lavorazione()));
				} else if(ti.getPosizione(ConstantsMap.appuntamento_localizzazione).equals(j)) {
					stringa.add(app.getLocalizzazioneString());
				} else if(ti.getPosizione(ConstantsMap.appuntamento_utente_denominazione).equals(j)) {
					stringa.add(app.getUtente_denominazione());
				} else if(ti.getPosizione(ConstantsMap.appuntamento_utente_codice).equals(j)) {
					stringa.add(app.getUtente_codice());
				} else if(ti.getPosizione(ConstantsMap.appuntamento_data_esito).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateTime.stampa(), app.getData_esito()));
				} else if(ti.getPosizione(ConstantsMap.appuntamento_data_stato).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateTime.stampa(), app.getData_stato()));
				} else if(ti.getPosizione(ConstantsMap.appuntamento_esito_appuntamento).equals(j)) {
					stringa.add(app.getEsitoEtichetta());
				} else if(ti.getPosizione(ConstantsMap.appuntamento_codice_appuntamento).equals(j)) {
					stringa.add(app.getEsitoCodice());
				} else if(ti.getPosizione(ConstantsMap.appuntamento_stato).equals(j)) {
					stringa.add(app.getStatoTesto());
				} else if(ti.getPosizione(ConstantsMap.contatto_data_lavorazione).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateTime.stampa(), cnt.getData_lavorazione()));
				} else if(ti.getPosizione(ConstantsMap.contatto_operatore_denominazione).equals(j)) {
					stringa.add(cnt.getOperatore_denominazione());
				} else if(ti.getPosizione(ConstantsMap.contatto_operatore_codice).equals(j)) {
					stringa.add(cnt.getOperatore_codice());
				} else if(ti.getPosizione(ConstantsMap.contatto_data_archiviazione).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateTime.stampa(), cnt.getData_archiviazione()));
				} else if(ti.getPosizione(ConstantsMap.contatto_esito_chiamata).equals(j)) {
					stringa.add(cnt.getEsito_chiamata());
				} else if(ti.getPosizione(ConstantsMap.contatto_esito_archiviazione).equals(j)) {
					stringa.add(cnt.getEsito_archiviazione());
				} else if(ti.getPosizione(ConstantsMap.contatto_esito_generale).equals(j)) {
					stringa.add(cnt.getEsito_generale());
				} else if(ti.getPosizione(ConstantsMap.contatto_codice_chiamata).equals(j)) {
					stringa.add(cnt.getCodice_chiamata());
				} else if(ti.getPosizione(ConstantsMap.contatto_codice_archiviazione).equals(j)) {
					stringa.add(cnt.getCodice_archiviazione());
				} else if(ti.getPosizione(ConstantsMap.contatto_codice_generale).equals(j)) {
					stringa.add(cnt.getCodice_generale());
				} else if(ti.getPosizione(ConstantsMap.contatto_proprietario_denominazione).equals(j)) {
					stringa.add(cnt.getProprietario_denominazione());
				} else if(ti.getPosizione(ConstantsMap.contatto_proprietario_codice).equals(j)) {
					stringa.add(cnt.getProprietario_codice());
				} else if(ti.getPosizione(ConstantsMap.contatto_data_presentazione).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateTime.stampa(), cnt.getData_presentazione()));
				} else if(ti.getPosizione(ConstantsMap.contatto_nota).equals(j)) {
					stringa.add(cnt.getNota());
				} else if(ti.getPosizione(ConstantsMap.contatto_data_creazione).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateTime.stampa(), cnt.getData_creazione()));
				} else if(ti.getPosizione(ConstantsMap.contatto_agente_denominazione).equals(j)) {
					stringa.add(cnt.getAgente_denominazione());
				} else if(ti.getPosizione(ConstantsMap.contatto_agente_codice).equals(j)) {
					stringa.add(cnt.getAgente_codice());
				} else if(ti.getPosizione(ConstantsMap.contatto_stato).equals(j)) {
					stringa.add(cnt.getTipoTesto());
				} else if(ti.getPosizione(ConstantsMap.contatto_id).equals(j)) {
					stringa.add(cnt.getId().toString());
				} else if(ti.getPosizione(ConstantsMap.contatto_campagna_denominazione).equals(j)) {
					stringa.add(cnt.getCampagna_denominazione());
				} else if(ti.getPosizione(ConstantsMap.contatto_lista_denominazione).equals(j)) {
					stringa.add(cnt.getLista_denominazione());
				} else if(ti.getPosizione(ConstantsMap.contatto_campagna_codice).equals(j)) {
					stringa.add(cnt.getCampagna_codice());
				} else if(ti.getPosizione(ConstantsMap.contatto_bloccato).equals(j)) {
					stringa.add(cnt.getBloccatoS());
				} else if(ti.getPosizione(ConstantsMap.contatto_lista_codice).equals(j)) {
					stringa.add(cnt.getLista_codice());
				} else if(ti.getPosizione(ConstantsMap.contatto_positivo).equals(j)) {
					stringa.add(cnt.getPositivo());
				} else if(ti.getPosizione(ConstantsMap.contatto_telefono_predefinito).equals(j)) {
					stringa.add(ApplicationUtil.uniforma(cnt.getTelefono_predefinito(), ti.getRem_predefinito(), ti.getAdd_predefinito()));
				} else if(ti.getPosizione(ConstantsMap.contatto_utile).equals(j)) {
					stringa.add(cnt.getUtile());
				} else if(ti.getPosizione(ConstantsMap.contatto_chiuso).equals(j)) {
					stringa.add(cnt.getChiuso());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_cognome).equals(j)) {
					stringa.add(cnt.getAnagrafica().getCognome());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_nome).equals(j)) {
					stringa.add(cnt.getAnagrafica().getNome());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_nome_completo).equals(j)) {
					stringa.add(cnt.getAnagrafica().getNome_completo());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_email).equals(j)) {
					stringa.add(cnt.getAnagrafica().getEmail());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_codice_fiscale).equals(j)) {
					stringa.add(cnt.getAnagrafica().getCodice_fiscale());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_sorgente).equals(j)) {
					stringa.add(cnt.getAnagrafica().getSorgente());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_nota).equals(j)) {
					stringa.add(cnt.getAnagrafica().getNota());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_luogo_nascita).equals(j)) {
					stringa.add(cnt.getAnagrafica().getLuogo_nascita());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_data_nascita).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.date.stampa(), cnt.getAnagrafica().getDate_nascita()));
				} else if(ti.getPosizione(ConstantsMap.anagrafica_link).equals(j)) {
					stringa.add(cnt.getAnagrafica().getLink());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_data_inserimento).equals(j)) {
					stringa.add(ApplicationUtil.formatDate(ConstantsFormat.dateTime.stampa(), cnt.getAnagrafica().getData_creazione()));
				} else if(ti.getPosizione(ConstantsMap.anagrafica_privacy).equals(j)) {
					stringa.add(cnt.getAnagrafica().getPrivacy());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_tipo).equals(j)) {
					stringa.add(cnt.getAnagrafica().getTipo());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_sesso).equals(j)) {
					stringa.add(cnt.getAnagrafica().getSesso());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_partita_iva).equals(j)) {
					stringa.add(cnt.getAnagrafica().getPartita_iva());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_ragione_sociale).equals(j)) {
					stringa.add(cnt.getAnagrafica().getRagione_sociale());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_telefono_fisso).equals(j)) {
					stringa.add(ApplicationUtil.uniforma(cnt.getAnagrafica().getTelefono_fisso(), ti.getRem_fisso(), ti.getAdd_fisso()));
				} else if(ti.getPosizione(ConstantsMap.anagrafica_telefono_altro).equals(j)) {
					stringa.add(ApplicationUtil.uniforma(cnt.getAnagrafica().getTelefono_altro(), ti.getRem_altro(), ti.getAdd_altro()));
				} else if(ti.getPosizione(ConstantsMap.anagrafica_telefono_mobile).equals(j)) {
					stringa.add(ApplicationUtil.uniforma(cnt.getAnagrafica().getTelefono_mobile(), ti.getRem_mobile(), ti.getAdd_mobile()));
				} else if(ti.getPosizione(ConstantsMap.anagrafica_telefono_fax).equals(j)) {
					stringa.add(ApplicationUtil.uniforma(cnt.getAnagrafica().getTelefono_fax(), ti.getRem_fax(), ti.getAdd_fax()));
				} else if(ti.getPosizione(ConstantsMap.anagrafica_settore).equals(j)) {
					stringa.add(cnt.getAnagrafica().getSettore());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_denominazione).equals(j)) {
					stringa.add(cnt.getAnagrafica().getDenominazione());
				} else if(ti.getPosizione(ConstantsMap.anagrafica_id).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIdentificativo());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_indirizzo_completo).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getIndirizzo_completo());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_strada_completo).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getStrada_completo());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_comune_completo).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getComune_completo());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_strada).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getStrada());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_civico).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getCivico());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_cap).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getCap());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_comune).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getComune());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_provincia).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getProvincia());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_regione).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getRegione());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_dettagli).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getDettagli());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_latitudine).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getLatitudine());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_longitudine).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getLongitudine());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_legale_presso).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_legale().getPresso());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_indirizzo_completo).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getIndirizzo_completo());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_strada_completo).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getStrada_completo());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_comune_completo).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getComune_completo());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_strada).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getStrada());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_civico).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getCivico());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_cap).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getCap());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_comune).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getComune());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_provincia).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getProvincia());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_regione).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getRegione());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_dettagli).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getDettagli());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_latitudine).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getLatitudine());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_longitudine).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getLongitudine());
				} else if(ti.getPosizione(ConstantsMap.indirizzo_operativo_presso).equals(j)) {
					stringa.add(cnt.getAnagrafica().getIndirizzo_operativo().getPresso());
				} else if(ti.getPosizione(ConstantsMap.consenso_tfissoS).equals(j)) {
					stringa.add(cnt.getAnagrafica().getConsenso_tfissoS());
				} else if(ti.getPosizione(ConstantsMap.consenso_tmobileS).equals(j)) {
					stringa.add(cnt.getAnagrafica().getConsenso_tmobileS());
				} else if(ti.getPosizione(ConstantsMap.consenso_tfaxS).equals(j)) {
					stringa.add(cnt.getAnagrafica().getConsenso_tfaxS());
				} else if(ti.getPosizione(ConstantsMap.consenso_taltroS).equals(j)) {
					stringa.add(cnt.getAnagrafica().getConsenso_taltroS());
				} else if(ti.getPosizione(ConstantsMap.consenso_emailS).equals(j)) {
					stringa.add(cnt.getAnagrafica().getConsenso_emailS());
				} else if(ti.getPosizione(ConstantsMap.consenso_rmailS).equals(j)) {
					stringa.add(cnt.getAnagrafica().getConsenso_rmailS());
				} else {
					stringa.add(stampaDato(titles[j], cnt.getAnagrafica().getDettagli(), null));
				}
			}
			writer.writeNext(stringa.toArray(new String[0]));
		}
		writer.close();
	}
	
	public static void esportaCSV(Boolean hd, Invoice sk, OutputStream out, List<String> titles, List<List<QueryElement>> ls) throws Exception {
		CSVWriter writer = new CSVWriter(new OutputStreamWriter(out, ConstantsGlobal.CHARSET), sk.getSeparatoreChar(), sk.getQuoteChar());
		if(ApplicationUtil.isTrue(hd)) {
			writer.writeNext(titles.toArray(new String[0]));
		}
		Iterator<List<QueryElement>> it = ls.iterator();
		while (it.hasNext()) {
			List<QueryElement> as = (List<QueryElement>) it.next();
			List<String> qs = new ArrayList<String>();
			for(int i = 0; i < as.size(); i++) {
				QueryElement o = as.get(i);
				qs.add(o != null ? o.toString() : "");
			}
			writer.writeNext(qs.toArray(new String[0]));
		}
		writer.close();
		out.close();
	}
}