package it.callite.utility;

import java.io.Serializable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.hibernate.criterion.DetachedCriteria;

import it.callite.column.generic.GenericColumn;
import it.callite.constants.ConstantsMap;
import it.callite.criteria.generic.RicercaAnagraficaCriteria;
import it.callite.dao.DataDao;
import it.callite.model.CampoAggiuntivoAnagrafica;
import it.callite.model.CampoAggiuntivoContratto;
import it.callite.model.Colonna;
import it.callite.model.Criterio;
import it.callite.model.TemplateExportAnagrafica;
import it.callite.model.TemplateExportAppuntamento;
import it.callite.model.TemplateExportContatto;
import it.callite.model.TemplateExportContratto;
import it.callite.model.TemplateExportStoriaAppuntamento;
import it.callite.model.TemplateExportStoriaContatto;
import it.callite.model.TemplateExportStoriaContratto;
import it.callite.model.TemplateImportAnagrafica;

public class Campo {

	private static Class<?> [] classi = new Class<?> [] { TemplateImportAnagrafica.class, TemplateExportAnagrafica.class, TemplateExportContatto.class, TemplateExportAppuntamento.class, TemplateExportContratto.class, TemplateExportStoriaContatto.class, TemplateExportStoriaAppuntamento.class, TemplateExportStoriaContratto.class };

	@SuppressWarnings({ "unchecked" })
	public static void updateCampo(Object t, DataDao dataDao, String ch){
		try {
			String chiaveN =  ApplicationUtil.trimToNull((String) t.getClass().getMethod("getChiave").invoke(t));
			if(chiaveN != null) {
				String chiaveV = ApplicationUtil.trimToNull(ch);
				boolean modifica = chiaveV != null && !chiaveV.equals(chiaveN);
				boolean elimina =  chiaveV == null;
				if(modifica || elimina) {
					for(Class<?> clazz : classi) { 
						DetachedCriteria criteria = DetachedCriteria.forClass(clazz);
						List<?> lst = dataDao.getList(criteria);
						if(ApplicationUtil.notEmpty(lst)) {
							Iterator<?> it = lst.iterator();
							while (it.hasNext()) {
								Object k = it.next();
								List<String> valori = (List<String>) k.getClass().getMethod("getValore_multiplo").invoke(k);
								if(ApplicationUtil.notEmpty(valori))  {
									if(modifica && valori.contains(chiaveV)) {
										int i = valori.indexOf(chiaveV);
										if(i>-1) {
											valori.set(i, chiaveN);
											k.getClass().getMethod("setValore_multiplo", List.class).invoke(k, valori);
											dataDao.salvaAggiorna(k);
										}
									}
									if(elimina && valori.contains(chiaveN)) {
										int i = valori.indexOf(chiaveN);
										if(i>-1) {
											valori.set(i, ConstantsMap.separatore);
											k.getClass().getMethod("setValore_multiplo", List.class).invoke(k, valori);
											dataDao.salvaAggiorna(k);
										}
									}
								}
							}
						}
					}
				}
			}
		} catch (Exception e) {
		}
	}

	public static void updateCampoCA(Object t, DataDao dataDao, boolean modifca){
		if(!modifca) {
			updateColonna(t, dataDao);
		}
		updateCriterio(t, dataDao, modifca);
	}


	@SuppressWarnings({ "unchecked" })
	private static void updateColonna(Object t, DataDao dataDao){
		DetachedCriteria criteria = DetachedCriteria.forClass(Colonna.class);
		List<Colonna> lst = dataDao.getList(criteria);
		if(ApplicationUtil.notEmpty(lst)) {
			Iterator<Colonna> it = lst.iterator();
			while (it.hasNext()) {
				Colonna k = it.next();
				String s = k.getSeriale();
				if(ApplicationUtil.isNotBlank(s))  {
					GenericColumn c = (GenericColumn) ApplicationUtil.deserialize(s);
					if(c != null) {
						Map<Long, Boolean> map = null;
						Long id =  null;
						if(t instanceof CampoAggiuntivoAnagrafica) {
							CampoAggiuntivoAnagrafica cc = (CampoAggiuntivoAnagrafica) t;
							map = c.getMappaA();
							id = cc.getId();

						}
						if(t instanceof CampoAggiuntivoContratto) {
							CampoAggiuntivoContratto cc = (CampoAggiuntivoContratto) t;
							map = c.getMappaC();
							id = cc.getId();
						}
						if(id != null && ApplicationUtil.notEmpty(map)) {
							if(map.containsKey(id)) {
								map.keySet().remove(id);
								k.setSeriale(ApplicationUtil.serialize(c));
								dataDao.salvaAggiorna(k);
							}

						}
					}
				}

			}
		}
	}

	@SuppressWarnings({ "unchecked" })
	private static void updateCriterio(Object t, DataDao dataDao, boolean modifica){
		DetachedCriteria criteria = DetachedCriteria.forClass(Criterio.class);
		List<Criterio> lst = dataDao.getList(criteria);
		if(ApplicationUtil.notEmpty(lst)) {
			Iterator<Criterio> it = lst.iterator();
			while (it.hasNext()) {
				Criterio k = it.next();
				String s = k.getSeriale();
				if(ApplicationUtil.isNotBlank(s))  {
					Serializable c = (Serializable) ApplicationUtil.deserialize(s);
					if(c != null) {
						if(t instanceof CampoAggiuntivoAnagrafica && c instanceof RicercaAnagraficaCriteria) {
							CampoAggiuntivoAnagrafica cc = (CampoAggiuntivoAnagrafica) t;
							RicercaAnagraficaCriteria r = (RicercaAnagraficaCriteria) c;
							List<DatoAggiuntivoAnagraficaRicerca> dAC = r.getDatiAggiuntivoAnagrafica();
							boolean save = false;
							if(ApplicationUtil.notEmpty(dAC)) {
								if(modifica) {
									save = ApplicationUtil.modificaCampo(dAC, cc);
								}
								if(!modifica) {
									save = ApplicationUtil.eliminaCampo(dAC, cc);
								}
								if(save) {
									k.setSeriale(ApplicationUtil.serialize(c));
									dataDao.salvaAggiorna(k);
								}
							}
						}
						if(t instanceof CampoAggiuntivoContratto && c instanceof RicercaAnagraficaCriteria) {
							CampoAggiuntivoContratto cc = (CampoAggiuntivoContratto) t;
							RicercaAnagraficaCriteria r = (RicercaAnagraficaCriteria) c;
							List<DatoAggiuntivoContrattoRicerca> dAC = r.getDatiAggiuntivoContratto();
							boolean save = false;
							if(ApplicationUtil.notEmpty(dAC)) {
								if(modifica) {
									save = ApplicationUtil.modificaCampo(dAC, cc);
								}
								if(!modifica) {
									save = ApplicationUtil.eliminaCampo(dAC, cc);
								}
								if(save) {
									k.setSeriale(ApplicationUtil.serialize(c));
									dataDao.salvaAggiorna(k);
								}
							}
						}
					}
				}

			}
		}
	}


}