package it.callite.utility;
import java.io.Serializable;

import it.callite.constants.ConstantsFormat;
import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.model.Contratto;
import it.callite.model.EsitoContratto;
@SuppressWarnings("serial")
public class ContrattoVO implements Serializable {
	
	private String chiave;
	private String motivo;
	
	public String getChiave() {
		return chiave;
	}
	public void setChiave(String chiave) {
		this.chiave = chiave;
	}
	public void setMotivo(String motivo) {
		this.motivo = motivo;
	}
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((motivo == null) ? 0 : motivo.hashCode());
		result = prime * result + ((chiave == null) ? 0 : chiave.hashCode());
		return result;
	}
	
	public ContrattoVO(Contratto a, Integer stato, String motivo, EsitoContratto es) {
		this.chiave = a.getChiave();
		StringBuilder sb = new StringBuilder(motivo);
		if(stato != null) {
			sb.append(" - " + ConstantsLanguage.STATO.stampa() + ": " + getStatoTesto(stato));
		}
		if(a.getData_stipula() != null) {
			sb.append(" - " + ConstantsLanguage.DATA_STIPULA.stampa() + ": " + ApplicationUtil.formatDate(ConstantsFormat.date.stampa(), a.getData_stipula()));
		}
		if(a.getData_contabile() != null) {
			sb.append(" - " + ConstantsLanguage.DATA_CONTABILE.stampa() + ": " + ApplicationUtil.formatDate(ConstantsFormat.date.stampa(), a.getData_contabile()));
		}
		if(a.getData_attivazione() != null) {
			sb.append(" - " + ConstantsLanguage.DATA_ATTIVAZIONE.stampa() + ": " + ApplicationUtil.formatDate(ConstantsFormat.date.stampa(), a.getData_attivazione()));
		}
		if(a.getData_caricamento() != null) {
			sb.append(" - " + ConstantsLanguage.DATA_CARICAMENTO.stampa() + ": " + ApplicationUtil.formatDate(ConstantsFormat.date.stampa(), a.getData_caricamento()));
		}
		if(a.getQuantita() != null) {
			sb.append(" - " + ConstantsLanguage.QUANTITA.stampa() + ": " + ApplicationUtil.formatNumber(a.getQuantita()));
		}
		if(a.getCosto_scontato() != null) {
			sb.append(" - " + ConstantsLanguage.IMPORTO.stampa() + ": " + ApplicationUtil.formatNumber(a.getCosto_scontato()));
		}
		if(es != null) {
			sb.append(" - " + ConstantsLanguage.ESITO_CONTRATTO.stampa() + ": " + es.getEtichettaCodice());
		}
		if(ApplicationUtil.isNotBlank(a.getNota())) {
			sb.append(" - " + ConstantsLanguage.NOTA.stampa() + ": " + a.getNota());
		}
		this.motivo = sb.toString();
	}
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		ContrattoVO other = (ContrattoVO) obj;
		if (motivo == null) {
			if (other.motivo != null)
				return false;
		} else if (!motivo.equals(other.motivo))
			return false;
		if (chiave == null) {
			if (other.chiave != null)
				return false;
		} else if (!chiave.equals(other.chiave))
			return false;
		return true;
	}
	public String getMotivo() {
		return motivo;
	}
	public String getStatoTesto(Integer stato) {
		if(ApplicationUtil.equals(stato, ConstantsGlobal.in_attesa)) {
			return ConstantsLanguage.STATO_IN_ATTESA.stampa();
		} else if(ApplicationUtil.equals(stato, ConstantsGlobal.in_pagamento)) {
			return ConstantsLanguage.STATO_IN_PAGAMENTO.stampa();
		} else if(ApplicationUtil.equals(stato, ConstantsGlobal.pagato)) {
			return ConstantsLanguage.STATO_PAGATO.stampa();
		} else if(ApplicationUtil.equals(stato, ConstantsGlobal.in_storno)) {
			return ConstantsLanguage.STATO_IN_STORNO.stampa();
		} else if(ApplicationUtil.equals(stato, ConstantsGlobal.stornato)) {
			return ConstantsLanguage.STATO_STORNATO.stampa();
		} else {
			return ConstantsLanguage.STATO_IN_ATTESA.stampa();
		} 
	}
}