package it.callite.utility;
import java.io.Serializable;

import it.callite.constants.ConstantsGlobal;
import it.callite.model.EsitoArchiviazione;
import it.callite.model.EsitoChiamata;
import it.callite.model.generic.SelezionatoAbstract;
public class EsitoGenerico extends SelezionatoAbstract implements Serializable {

	private static final long serialVersionUID = 1L;

	private String identificativo;
	private Long id;
	private String codice;
	private String colore;
	private String descrizione;
	private String etichetta;
	private Boolean tipo;
	private boolean chiamata = true;

	public EsitoGenerico(EsitoChiamata esitoC, EsitoArchiviazione esitoA) {
		if(esitoA !=null) {
			this.setIdentificativo(ConstantsGlobal.esito_archiviazione + esitoA.getId());
			this.setId(esitoA.getId());
			this.setEtichetta(esitoA.getEtichetta());
			this.setDescrizione(esitoA.getDescrizione());
			this.setCodice(esitoA.getCodice());
			this.setColore(esitoA.getColore());
			this.setChiamata(false);
			if(ApplicationUtil.gtZero(esitoA.getInvia_opposizione()) || ApplicationUtil.gt(esitoA.getTipologiaS(), 0)) {
				this.tipo = true;
			} else {
				this.tipo = false;
			}
		} else {
			this.setIdentificativo(ConstantsGlobal.esito_chiamata + esitoC.getId());
			this.setId(esitoC.getId());
			this.setEtichetta(esitoC.getEtichetta());
			this.setDescrizione(esitoC.getDescrizione());
			this.setCodice(esitoC.getCodice());
			this.setColore(esitoC.getColore());
			this.setChiamata(true);
			if(ApplicationUtil.gtZero(esitoC.getInvia_opposizione()) || ApplicationUtil.isTrue(esitoC.getRichiamare()) || !ApplicationUtil.isTrue(esitoC.getEditabile())) {
				this.tipo = true;
			} else {
				this.tipo = false;
			}
		}
	}

	public Boolean getTipo() {
		return tipo;
	}

	public void setTipo(Boolean tipo) {
		this.tipo = tipo;
	}


	public boolean isChiamata() {
		return chiamata;
	}
	public void setChiamata(boolean chiamata) {
		this.chiamata = chiamata;
	}
	public String getIdentificativo() {
		return identificativo;
	}
	public void setIdentificativo(String identificativo) {
		this.identificativo = identificativo;
	}
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + (chiamata ? 1231 : 1237);
		result = prime * result + ((codice == null) ? 0 : codice.hashCode());
		result = prime * result + ((colore == null) ? 0 : colore.hashCode());
		result = prime * result + ((descrizione == null) ? 0 : descrizione.hashCode());
		result = prime * result + ((etichetta == null) ? 0 : etichetta.hashCode());
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		result = prime * result + ((identificativo == null) ? 0 : identificativo.hashCode());
		return result;
	}
	@Override
	public String toString() {
		return identificativo;
	}
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		EsitoGenerico other = (EsitoGenerico) obj;
		if (chiamata != other.chiamata)
			return false;
		if (codice == null) {
			if (other.codice != null)
				return false;
		} else if (!codice.equals(other.codice))
			return false;
		if (colore == null) {
			if (other.colore != null)
				return false;
		} else if (!colore.equals(other.colore))
			return false;
		if (descrizione == null) {
			if (other.descrizione != null)
				return false;
		} else if (!descrizione.equals(other.descrizione))
			return false;
		if (etichetta == null) {
			if (other.etichetta != null)
				return false;
		} else if (!etichetta.equals(other.etichetta))
			return false;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		if (identificativo == null) {
			if (other.identificativo != null)
				return false;
		} else if (!identificativo.equals(other.identificativo))
			return false;
		return true;
	}
	public Long getId() {
		return id;
	}
	public void setId(Long id) {
		this.id = id;
	}
	public String getCodice() {
		return codice;
	}
	public String getColore() {
		return colore;
	}
	public String getEtichettaCodice() {
		StringBuilder sb = new StringBuilder();
		sb.append(ApplicationUtil.trimToEmpty(etichetta));
		if(ApplicationUtil.isNotBlank(codice)) {
			sb.append(" (" + codice + ")");
		}
		return sb.toString();
	}

	public String getDenominazione() {
		return getEtichettaCodice();
	}

	public String getDescrizione() {
		return descrizione;
	}
	public void setDescrizione(String descrizione) {
		this.descrizione = descrizione;
	}
	public String getEtichetta() {
		return etichetta;
	}
	public void setEtichetta(String etichetta) {
		this.etichetta = etichetta;
	}
	public void setCodice(String codice) {
		this.codice = codice;
	}
	public void setColore(String colore) {
		this.colore = colore;
	}
}