package it.callite.utility;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.SortedMap;
import java.util.TreeMap;

public class GroupBy<T,K> {

	private String method;
	private SortedMap<T, List<K>> mapList;
	
	public GroupBy(String method, List<K> list) {
		this.method = method;
		this.mapList = collectionToHash(list);
	}
	
	@SuppressWarnings("unchecked")
	private SortedMap<T, List<K>> collectionToHash(List<K> list){

		TreeMap<T,List<K>> hash = new TreeMap<T, List<K>>();
		for(K cmp : list){
			try {
				Class<?> klass = cmp.getClass();
				Method m = klass.getMethod(method);
				T resultAsKey = (T) m.invoke(cmp);
				List<K> arrayList = new ArrayList<K>();
				if(hash.containsKey(resultAsKey)){
					arrayList = hash.get(resultAsKey);
				} else {
					hash.put(resultAsKey, arrayList);
				}
				arrayList.add(cmp);

			}catch (Exception e) {}


		}
		return hash;
	}
	
	
	public List<K> get(T key){ 
		if (this.mapList.containsKey(key))
			return this.mapList.get(key);
		else 
			return new ArrayList<K>();
	}
	
	public int getSize(T key){ 
		if (this.mapList.containsKey(key))
			return this.mapList.get(key).size();
		else 
			return 0;
	}
	
	public List<K> remove(T key){ 	
			return this.mapList.remove(key);
	}
	
	public Set<T>  keySet() {
		return this.mapList.keySet();
	}

	public boolean containsKey(T key){ 
		return this.mapList.containsKey(key);
	}
	
	public boolean containsValue(List<K> value) {
		return this.mapList.containsValue(value);
	}
	
	public List<K> put(T key, List<K> value) {
		return this.mapList.put(key, value);
	}
	
}
