package it.callite.utility;
import java.io.Serializable;

import jakarta.mail.Session;
import jakarta.mail.Transport;

import it.callite.log.ErrorLog;
@SuppressWarnings("serial")
public class MailSupport implements Serializable {
	private boolean single;
	private String mail_host;
	private String mail_encryption;
	private String mail_protocol;
	private String mail_user;
	private String mail_password;
	private Integer mail_port;
	private Boolean mail_auth;
	private Boolean mail_pec;
	private Boolean mail_debug;
	private String mail_from;
	private String mail_mittente;
	private Session session;
	private Transport transport;
	public Session getSession() {
		return session;
	}
	public void setSession(Session session) {
		this.session = session;
	}
	public MailSupport(boolean single, Boolean mail_pec, String mail_protocol, String mail_host, String mail_encryption, String mail_user, String mail_password, Integer mail_port, Boolean mail_auth, Boolean mail_debug, String mail_from, String mail_mittente) {
		this.single = single;
		this.mail_mittente = mail_mittente;
		this.mail_host = mail_host;
		this.mail_encryption = mail_encryption;
		this.mail_pec = mail_pec;
		this.mail_user = mail_user;
		this.mail_password = mail_password;
		this.mail_port = mail_port;
		this.mail_auth = mail_auth;
		this.mail_debug = mail_debug;
		this.mail_from = mail_from;
		this.mail_protocol = ApplicationUtil.lowerCaseNull(mail_protocol);
		
		initialize(this.mail_pec, this.mail_protocol, this.mail_host, this.mail_encryption, this.mail_user, this.mail_password, this.mail_port, this.mail_auth, this.mail_debug, this.mail_from);
	}
	
	
	public void initialize(Boolean mail_pec, String mail_protocol, String mail_host, String mail_encryption, String mail_user, String mail_password, Integer mail_port, Boolean mail_auth, Boolean mail_debug, String mail_from) {
		if(mail_host != null && mail_port != null && mail_user != null && mail_password != null && mail_encryption != null) {
			session = EmailUtils.getSession(mail_pec, mail_protocol, mail_host, mail_port, mail_user, mail_password, mail_from, mail_auth, mail_encryption, mail_debug);
			if(isInviabile()) {
				try {
					transport = session.getTransport(mail_protocol);
					if(transport != null && !transport.isConnected()) {
						transport.connect(mail_host, mail_port, mail_user, mail_password);
					}
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				}
			}
		}
	}
	
	
	
	public boolean isConnected() {
		if(isInviabile()) {
			if(transport != null && transport.isConnected()) {
				return true;
			}
		}
		return false;
	}

	public void close() {
		if(isInviabile()) {
			if(transport != null && transport.isConnected()) {
				try {
					transport.close();
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				}
			}
		}
	}
	
	public boolean isInviabile() {
		return session != null && !single && mail_protocol != null;
	}
	public String getMail_protocol() {
		return mail_protocol;
	}
	public void setMail_protocol(String mail_protocol) {
		this.mail_protocol = mail_protocol;
	}
	public Transport getTransport() {
		return transport;
	}
	public void setTransport(Transport transport) {
		this.transport = transport;
	}
	public String getMail_host() {
		return mail_host;
	}
	public void setMail_host(String mail_host) {
		this.mail_host = mail_host;
	}
	public String getMail_encryption() {
		return mail_encryption;
	}
	public void setMail_encryption(String mail_encryption) {
		this.mail_encryption = mail_encryption;
	}
	public String getMail_user() {
		return mail_user;
	}
	public void setMail_user(String mail_user) {
		this.mail_user = mail_user;
	}
	public String getMail_password() {
		return mail_password;
	}
	public void setMail_password(String mail_password) {
		this.mail_password = mail_password;
	}
	public Integer getMail_port() {
		return mail_port;
	}
	public void setMail_port(Integer mail_port) {
		this.mail_port = mail_port;
	}
	public Boolean getMail_auth() {
		return mail_auth;
	}
	public void setMail_auth(Boolean mail_auth) {
		this.mail_auth = mail_auth;
	}
	public Boolean getMail_debug() {
		return mail_debug;
	}
	public void setMail_debug(Boolean mail_debug) {
		this.mail_debug = mail_debug;
	}
	public String getMail_from() {
		return mail_from;
	}
	public void setMail_from(String mail_from) {
		this.mail_from = mail_from;
	}
	public String getMail_mittente() {
		return mail_mittente;
	}
	public void setMail_mittente(String mail_mittente) {
		this.mail_mittente = mail_mittente;
	}
	public Boolean getMail_pec() {
		return mail_pec;
	}
	public void setMail_pec(Boolean mail_pec) {
		this.mail_pec = mail_pec;
	}
	
}