package it.callite.utility;

import java.io.File;
import java.io.Serializable;
import java.util.Arrays;

import org.apache.commons.net.ftp.FTPFile;

import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsMap;
import it.callite.model.generic.SelezionatoAbstract;
public class MyFile extends SelezionatoAbstract implements Serializable {
	
	private static final long serialVersionUID = 1L;
	
	

	@Override
	public String toString() {
		return ApplicationUtil.toString(null, getDenominazione());
	}

	public String getDenominazione() {
		return ApplicationUtil.trimToEmpty(getName());
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + Arrays.hashCode(data);
		result = prime * result + (int) (length ^ (length >>> 32));
		result = prime * result + (int) (modified ^ (modified >>> 32));
		result = prime * result + ((name == null) ? 0 : name.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		MyFile other = (MyFile) obj;
		if (!Arrays.equals(data, other.data))
			return false;
		if (length != other.length)
			return false;
		if (modified != other.modified)
			return false;
		if (name == null) {
			if (other.name != null)
				return false;
		} else if (!name.equals(other.name))
			return false;
		return true;
	}

	private byte[] data;
	private long length;
	private String name;
	private String parent;
	private String path;
	private long modified;
	private boolean folder = false;
	private String size;

	public boolean isFolder() {
		return folder;
	}

	public String getPath() {
		return path;
	}

	public void setPath(String path) {
		this.path = path;
	}

	public void setFolder(boolean folder) {
		this.folder = folder;
	}

	private Long id;

	public Long getId() {
		return id;
	}

	public void setId(Long id) {
		this.id = id;
	}

	public long getModified() {
		return modified;
	}

	public void setModified(long modified) {
		this.modified = modified;
	}

	public byte[] getData() {
		return data;
	}

	public long getLength() {
		return length;
	}

	public String getName() {
		return name;
	}

	public void setData(byte[] data) {
		this.data = data;
	}

	public void setLength(long length) {
		this.length = length;
	}

	public void setName(String name) {
		this.name = name;
	}


	public String getBaseName() {
		return ApplicationUtil.getBaseNameN(name);
	}

	public String getExtension() {
		return ApplicationUtil.getExtensionN(name);
	}

	public boolean isStream() {
		boolean all = ConstantsGlobal.streamAll;
		if(!all) {
			String ext = ApplicationUtil.lowerCaseNull(ApplicationUtil.getExtensionLower(name));
			if(ext != null) {
				return ApplicationUtil.asList(ConstantsMap.a_stream).contains(ext);
			}
			return false;
		} else {
			return true;
		}
	}
	
	public boolean isLogo() {
		String ext = ApplicationUtil.lowerCaseNull(ApplicationUtil.getExtensionLower(name));
		if(ext != null) {
			return ApplicationUtil.asList(ConstantsMap.a_immagini).contains(ext);
		}
		return false;
	}

	public boolean isPda() {
		String ext = ApplicationUtil.trimToNull(ApplicationUtil.getExtensionLower(name));
		if(ext != null) {
			return ApplicationUtil.asList(ConstantsMap.a_pda1).contains(ext);
		}
		return false;
	}

	public String getParent() {
		return parent;
	}

	public void setParent(String parent) {
		this.parent = parent;
	}

	public String getParentS() {
		StringBuilder sb = new StringBuilder();
		if(isFolder()) {
			sb.append(ConstantsGlobal.path_web_folder + "/");
		} else {
			sb.append(ConstantsGlobal.path_web_file + "/");
		}
		sb.append(getParentA());
		return sb.toString();
	}
	
	public String getParentA() {
		StringBuilder sb = new StringBuilder();
		String ext = isFolder() ? "" : ("." +  getExtension());
		String p = ApplicationUtil.trimToNull(parent);
		if(p != null) {
			sb.append(ApplicationUtil.base64Encoder(p + "/" + getBaseName()) + ext);
		} else {
			sb.append(ApplicationUtil.base64Encoder(getBaseName()) + ext);
		}
		return sb.toString();
	}

	public String getParentH() {
		String p = ApplicationUtil.trimToNull(parent);
		if(p != null) {
			return p + "/" + getBaseName();
		} else {
			return getBaseName();
		}
	}
	
	public String getSize() {
		return size;
	}

	public void setSize(String size) {
		this.size = size;
	}

	public MyFile() {
	}
	public MyFile(File f) {
		this.folder = f.isDirectory();
		this.length = ApplicationUtil.fileSize(f);
		this.size = ApplicationUtil.byteCount(this.length);
	}
	public MyFile(FTPFile f) {
		this.folder = f.isDirectory();
		this.length = f.getSize();
		this.size = ApplicationUtil.byteCount(this.length);
	}
}