package it.callite.utility;

import java.util.List;

import it.callite.constants.ConstantsGlobal;
import it.callite.model.sip.SipBuddies;

public class ParameterAutodialer {

	private String callerId; 
	private String musicclass; 
	private List<String> codecs; 
	private Boolean interurbane; 
	private Boolean opposizione; 
	private Boolean registrazione; 
	private String prefisso;
	private String antecall;
	private String trunk;
	private Integer vrx;
	private Integer vtx;

	public ParameterAutodialer(SipBuddies sp, String callers, String musicclass) {
		if(sp!=null) {
			this.vtx = sp.getVtx();
			this.vrx = sp.getVrx();
			this.prefisso = ApplicationUtil.trimToEmpty(sp.getAdd_prefisso());
			this.antecall = ApplicationUtil.trimToEmpty(sp.getAntecall());
			this.trunk = ApplicationUtil.trimToEmpty(sp.getName());
			this.opposizione = ApplicationUtil.isTrue(sp.getOpposizione());
			this.registrazione = ApplicationUtil.isTrue(sp.getRegistrazione());
			this.interurbane = ApplicationUtil.isTrue(sp.getInterurbane());
			this.codecs = sp.getValore_multiplo();
			this.musicclass = ApplicationUtil.trimToEmpty(sp.getMusicclass());
			this.callerId = ApplicationUtil.randomValue(ApplicationUtil.trimToEmpty(sp.getCallers()));
		}
		if(musicclass != null) {
			this.musicclass = ApplicationUtil.trimToEmpty(musicclass);
		} 
		if(callers != null) {
			this.callerId = ApplicationUtil.randomValue(ApplicationUtil.trimToEmpty(callers));
		} 
		initialize();
	}

	public void initialize() {
		if(ApplicationUtil.isBlank(this.getCallerId())) {
			this.setCallerId(ConstantsGlobal.callerid);
		}
		if(ApplicationUtil.isBlank(this.getMusicclass())) {
			this.setMusicclass(ConstantsGlobal.musicclass);
		}
		if(ApplicationUtil.isBlank(this.getTrunk())) {
			this.setTrunk("");
		}
		if(ApplicationUtil.isBlank(this.getPrefisso())) {
			this.setPrefisso("");
		}
		if(ApplicationUtil.isBlank(this.getAntecall())) {
			this.setAntecall("");
		}
		if(ApplicationUtil.ltZero(this.getVtx())) {
			this.setVtx(ConstantsGlobal.DEFAULT_VTX);
		}
		if(ApplicationUtil.ltZero(this.getVrx())) {
			this.setVrx(ConstantsGlobal.DEFAULT_VRX);
		}
		if(this.getOpposizione() == null) {
			this.setOpposizione(Boolean.FALSE);
		}
		if(this.getRegistrazione() == null) {
			this.setRegistrazione(Boolean.FALSE);
		}
		if(this.getInterurbane() == null) {
			this.setInterurbane(Boolean.TRUE);
		}
	}
	public String getCallerId() {
		return callerId;
	}
	public List<String> getCodecs() {
		return codecs;
	}

	public Boolean getInterurbane() {
		return interurbane;
	}
	public Boolean getOpposizione() {
		return opposizione;
	}

	public String getTrunk() {
		return trunk;
	}
	public Integer getVrx() {
		return vrx;
	}
	public Integer getVtx() {
		return vtx;
	}
	public void setCallerId(String callerId) {
		this.callerId = callerId;
	}
	public void setCodecs(List<String> codecs) {
		this.codecs = codecs;
	}
	public void setInterurbane(Boolean interurbane) {
		this.interurbane = interurbane;
	}
	public Boolean getRegistrazione() {
		return registrazione;
	}


	public void setRegistrazione(Boolean registrazione) {
		this.registrazione = registrazione;
	}


	public String getMusicclass() {
		return musicclass;
	}


	public void setMusicclass(String musicclass) {
		this.musicclass = musicclass;
	}


	public void setOpposizione(Boolean opposizione) {
		this.opposizione = opposizione;
	}

	public void setTrunk(String trunk) {
		this.trunk = trunk;
	}
	public void setVrx(Integer vrx) {
		this.vrx = vrx;
	}
	public void setVtx(Integer vtx) {
		this.vtx = vtx;
	}


	public String getPrefisso() {
		return prefisso;
	}


	public void setPrefisso(String prefisso) {
		this.prefisso = prefisso;
	}


	public String getAntecall() {
		return antecall;
	}


	public void setAntecall(String antecall) {
		this.antecall = antecall;
	}



}
