/*
 * Copyright (C) 2011 The Libphonenumber Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * @author Shaopeng Jia
 */

package it.callite.utility;

import java.util.Locale;

import com.google.i18n.phonenumbers.NumberParseException;
import com.google.i18n.phonenumbers.PhoneNumberToCarrierMapper;
import com.google.i18n.phonenumbers.PhoneNumberToTimeZonesMapper;
import com.google.i18n.phonenumbers.PhoneNumberUtil;
import com.google.i18n.phonenumbers.PhoneNumberUtil.PhoneNumberFormat;
import com.google.i18n.phonenumbers.PhoneNumberUtil.PhoneNumberType;
import com.google.i18n.phonenumbers.Phonenumber.PhoneNumber;
import com.google.i18n.phonenumbers.geocoding.PhoneNumberOfflineGeocoder;

public class PhoneNumberParser {

	public static void valida(String phoneNumber, String language, String country)  {
		//String language = "it";  
		//String country = "IT";
		parse(phoneNumber, language, country);

	}

	private static void stampa(String stampa) {
		ApplicationUtil.logger(stampa);
	}

	private static void parse(String phoneNumber, String language, String country) {
		try {
			PhoneNumberUtil phoneUtil = PhoneNumberUtil.getInstance();
			PhoneNumber number = phoneUtil.parseAndKeepRawInput(phoneNumber, country);

			stampa("national_number: " + Long.toString(number.getNationalNumber()));
			stampa("extension: " + number.getExtension());
			stampa("country_code_source: " + number.getCountryCodeSource().toString());
			stampa("italian_leading_zero: " + Boolean.toString(number.isItalianLeadingZero()));
			stampa("raw_input: " + number.getRawInput());

			boolean isPossible = phoneUtil.isPossibleNumber(number);
			boolean isNumberValid = phoneUtil.isValidNumber(number);
			PhoneNumberType numberType = phoneUtil.getNumberType(number);

			stampa("Result from isPossibleNumber(): " + Boolean.toString(isPossible));
			if (!isPossible) {
				stampa("Result from isPossibleNumberWithReason(): " + phoneUtil.isPossibleNumberWithReason(number).toString());
			} else {
				stampa("Result from isValidNumber(): " + Boolean.toString(isNumberValid));
				if (isNumberValid) {
					stampa("Result from isValidNumberForRegion(): " + Boolean.toString(phoneUtil.isValidNumberForRegion(number, country)));
				}
				String region = phoneUtil.getRegionCodeForNumber(number);
				stampa("Phone Number region: " + region);
				stampa("Result from getNumberType(): " + numberType.toString());
			}
			stampa("E164 format: " + (isNumberValid ? phoneUtil.format(number, PhoneNumberFormat.E164) : "invalid"));

			stampa("Original format: " + phoneUtil.formatInOriginalFormat(number, country));
			stampa("National format: " + phoneUtil.format(number, PhoneNumberFormat.NATIONAL));
			stampa("International format: " + (isNumberValid ? phoneUtil.format(number, PhoneNumberFormat.INTERNATIONAL) : "invalid"));
			stampa("Out-of-country format from US: " + (isNumberValid ? phoneUtil.formatOutOfCountryCallingNumber(number, country) : "invalid"));

			if (isNumberValid) {
				stampa("Location: " + PhoneNumberOfflineGeocoder.getInstance().getDescriptionForNumber(number, new Locale(language, country)));
				stampa("Time zone(s): " + PhoneNumberToTimeZonesMapper.getInstance().getTimeZonesForNumber(number).toString());
				if (numberType == PhoneNumberType.MOBILE ||	numberType == PhoneNumberType.FIXED_LINE_OR_MOBILE || numberType == PhoneNumberType.PAGER) {
					stampa("Carrier: " + PhoneNumberToCarrierMapper.getInstance().getNameForNumber(number, new Locale(language, country)));
				}
			}
		} catch (NumberParseException e) {

		}
	}
}
