package it.callite.utility;
import java.io.Serializable;
import java.math.BigDecimal;
import java.sql.Time;
import java.sql.Timestamp;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.util.Date;

import it.callite.constants.ConstantsFormat;
@SuppressWarnings("serial")
public class QueryElement implements Serializable {
	private Object element;
	public QueryElement(Object element) {
		this.element = element;
	}
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((element == null) ? 0 : element.hashCode());
		return result;
	}
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		QueryElement other = (QueryElement) obj;
		if (element == null) {
			if (other.element != null)
				return false;
		} else if (!element.equals(other.element))
			return false;
		return true;
	}
	@Override
	public String toString() {
		if(element != null) {
			if(element instanceof Timestamp) {
				Date d = ApplicationUtil.timestampToDate((Timestamp) element);
				return ApplicationUtil.formatDate(ConstantsFormat.dateSeconds.stampa(), d);
			} else if(element instanceof Time) {
				Date d = ApplicationUtil.timeToDate((Time) element);
				return ApplicationUtil.formatDate(ConstantsFormat.time.stampa(), d);
			} else if(element instanceof LocalDateTime) {
				Date d = ApplicationUtil.timestampToDate(Timestamp.valueOf((LocalDateTime) element));
				return ApplicationUtil.formatDate(ConstantsFormat.dateSeconds.stampa(), d);
			} else if(element instanceof LocalTime) {
				Date d = ApplicationUtil.timeToDate(Time.valueOf((LocalTime) element));
				return ApplicationUtil.formatDate(ConstantsFormat.time.stampa(), d);
			} else if(element instanceof BigDecimal) {
				return ApplicationUtil.formatNumber((BigDecimal) element);
			} else if(element instanceof Long) {
				return ApplicationUtil.formatNumber((Long) element);
			} else if(element instanceof Integer) {
				return ApplicationUtil.formatNumber((Integer) element);
			} else if(element instanceof byte[]) {
				return new String((byte[]) element);
			} else {
				return element.toString();
			}
		} else {
			return null;
		}
	}
	
	public ExcelFormat toExcel() {
		if(element != null) {
			if(element instanceof Timestamp) {
				return new ExcelFormat(ConstantsFormat.dateSeconds.stampa(), ApplicationUtil.timestampToDate((Timestamp) element));
			} else if(element instanceof Time) {
				return new ExcelFormat(ConstantsFormat.time.stampa(), ApplicationUtil.timeToDate((Time) element));
			} else if(element instanceof LocalDateTime) {
				return new ExcelFormat(ConstantsFormat.dateSeconds.stampa(), ApplicationUtil.timestampToDate(Timestamp.valueOf((LocalDateTime) element)));
			} else if(element instanceof LocalTime) {
				return new ExcelFormat(ConstantsFormat.time.stampa(), ApplicationUtil.timeToDate(Time.valueOf((LocalTime) element)));
			} else if(element instanceof BigDecimal) {
				return new ExcelFormat(null, (BigDecimal) element);
			} else if(element instanceof Long) {
				return new ExcelFormat(null, (Long) element);
			} else if(element instanceof Integer) {
				return new ExcelFormat(null, (Integer) element);
			} else if(element instanceof byte[]) {
				return new ExcelFormat(null, new String((byte[]) element));
			} else {
				return new ExcelFormat(null, element.toString());
			}
		} else {
			return new ExcelFormat(null, null);
		}
	}
}