package it.callite.utility.item;
import java.io.Serializable;
import java.util.List;

import it.callite.model.Agente;
import it.callite.model.generic.SelezionatoAbstract;
import it.callite.utility.ApplicationUtil;
public class AgenteSI extends SelezionatoAbstract implements Serializable {

	private static final long serialVersionUID = 1L;

	private Boolean autonomo;
	private String codice;
	private String cognome;
	private Long id;
	private List<AgenteSI> nodes;
	private String nome;
	private String qualifica;
	private String ufficio;
	private String nome_completo;
	private Boolean pf;
	private String ragione_sociale;

	public AgenteSI() {
	}

	public AgenteSI(Agente agente) {
		this.autonomo = agente.getAutonomo();
		this.codice = agente.getCodice();
		this.cognome = agente.getCognome();
		this.id = agente.getId();
		this.nome = agente.getNome();
		this.qualifica = agente.getQualifica() != null ? agente.getQualifica().getNome() : null;
		this.ufficio = agente.getUfficio() != null ? agente.getUfficio().getNome() : null;
		this.nome_completo = agente.getNome_completo();
		this.ragione_sociale = agente.getRagione_sociale();
		this.pf = agente.getPf();
	}

	public String getQualifica() {
		return qualifica;
	}
	public void setQualifica(String qualifica) {
		this.qualifica = qualifica;
	}
	public String getUfficio() {
		return ufficio;
	}
	public void setUfficio(String ufficio) {
		this.ufficio = ufficio;
	}
	public Boolean getAutonomo() {
		return autonomo;
	}
	public String getCodice() {
		return codice;
	}
	public String getCognome() {
		return cognome;
	}

	public String getAppellativo() {
		return ApplicationUtil.denominazione(pf, nome_completo, ragione_sociale, cognome, nome, null);
	}

	public String getDenominazione() {
		StringBuilder sb = new StringBuilder(getAppellativo());
		if(ApplicationUtil.isNotBlank(codice)) {
			sb.insert(0, ApplicationUtil.trimToEmpty(codice) + " - ");
		}
		if(ApplicationUtil.isNotBlank(qualifica)) {
			sb.append(" (" + ApplicationUtil.trimToEmpty(qualifica) + ")");
		}
		if(ApplicationUtil.isNotBlank(ufficio)) {
			sb.append(" (" + ApplicationUtil.trimToEmpty(ufficio) + ")");
		}
		return ApplicationUtil.trimToEmpty(sb.toString());
	}



	public Long getId() {
		return id;
	}
	public List<AgenteSI> getNodes() {
		return nodes;
	}
	public String getNome() {
		return nome;
	}
	public String getNome_completo() {
		return nome_completo;
	}
	public Boolean getPf() {
		return pf;
	}
	public String getRagione_sociale() {
		return ragione_sociale;
	}
	public void setAutonomo(Boolean autonomo) {
		this.autonomo = autonomo;
	}
	public void setCodice(String codice) {
		this.codice = codice;
	}
	public void setCognome(String cognome) {
		this.cognome = cognome;
	}
	public void setId(Long id) {
		this.id = id;
	}
	public void setNodes(List<AgenteSI> nodes) {
		this.nodes = nodes;
	}
	public void setNome(String nome) {
		this.nome = nome;
	}

	public void setNome_completo(String nome_completo) {
		this.nome_completo = nome_completo;
	}
	public void setPf(Boolean pf) {
		this.pf = pf;
	}
	public void setRagione_sociale(String ragione_sociale) {
		this.ragione_sociale = ragione_sociale;
	}

}