package it.callite.utility.item;
import java.io.Serializable;
import java.sql.Time;
import java.util.Date;
import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.message.Message;
import it.callite.model.Campagna;
import it.callite.model.generic.SelezionatoAbstract;
import it.callite.utility.ApplicationUtil;
public class CampagnaSI extends SelezionatoAbstract implements Serializable {

	private static final long serialVersionUID = 1L;
	
	public CampagnaSI() {
	}
		
	public CampagnaSI(ContattoSI contatto) {
		this.id = contatto.getCampagna_id();
		this.nome = contatto.getCampagna_nome();
		this.stato = contatto.getCampagna_stato();
		this.tipo = contatto.getCampagna_tipo();
		this.codice = contatto.getCampagna_codice();
		this.week_fine = contatto.getCampagna_week_fine();
		this.week_inizio = contatto.getCampagna_week_inizio();
		this.data_fine = contatto.getCampagna_data_fine();
		this.data_inizio = contatto.getCampagna_data_inizio();
		this.ora_fine = contatto.getCampagna_ora_fine();
		this.ora_inizio = contatto.getCampagna_ora_inizio();
		this.inverti = contatto.getCampagna_inverti();
	}
	
	public CampagnaSI(Campagna campagna) {
		this.id = campagna.getId();
		this.nome = campagna.getNome();
		this.stato = campagna.getStato();
		this.tipo = campagna.getTipo();
		this.codice = campagna.getCodice();
		this.week_fine = campagna.getWeek_fine();
		this.week_inizio = campagna.getWeek_inizio();
		this.data_fine = campagna.getData_fine();
		this.data_inizio = campagna.getData_inizio();
		this.ora_fine = campagna.getTime_fine();
		this.ora_inizio = campagna.getTime_inizio();
		this.inverti = campagna.getInverti();
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		result = prime * result + ((nome == null) ? 0 : nome.hashCode());
		result = prime * result + ((stato == null) ? 0 : stato.hashCode());
		result = prime * result + ((tipo == null) ? 0 : tipo.hashCode());
		return result;
	}
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		CampagnaSI other = (CampagnaSI) obj;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		if (nome == null) {
			if (other.nome != null)
				return false;
		} else if (!nome.equals(other.nome))
			return false;
		if (stato == null) {
			if (other.stato != null)
				return false;
		} else if (!stato.equals(other.stato))
			return false;
		if (tipo == null) {
			if (other.tipo != null)
				return false;
		} else if (!tipo.equals(other.tipo))
			return false;
		return true;
	}
	private Long id;
	private String nome;
	private String codice;
	private Integer stato;
	private Integer tipo;
	public Integer getTipo() {
		return tipo;
	}
	public void setTipo(Integer tipo) {
		this.tipo = tipo;
	}
	private Long week_fine;
	private Long week_inizio;
	
	public Long getWeek_fine() {
		return week_fine;
	}
	public void setWeek_fine(Long week_fine) {
		this.week_fine = week_fine;
	}
	public Long getWeek_inizio() {
		return week_inizio;
	}
	public void setWeek_inizio(Long week_inizio) {
		this.week_inizio = week_inizio;
	}
	public Boolean getStatoAttiva() {
		return ApplicationUtil.calcolaIntervallo(getInverti(), getData_inizio(), getData_fine(), getWeek_inizio(), getWeek_fine(), getOra_inizio(), getOra_fine());
	}
	public String getDenominazione() {
		return getDenominazioneTipo() + " (" + getDescrizioneStato() +(getStatoAttiva() ? "" : "*") + ")";
	}
	public String getDenominazioneTipo() {
		String codice = getCodice();
		String tipo = getDescrizioneTipo();
		if(ApplicationUtil.isBlank(tipo)) {
			return (ApplicationUtil.isNotBlank(codice) ? (codice + " - ") : "") + ApplicationUtil.trimToEmpty(nome);
		} else {
			return (ApplicationUtil.isNotBlank(codice) ? (codice + " - ") : "") + ApplicationUtil.trimToEmpty(nome) + " - " + tipo;
		}
	}
	public String getDescrizioneTipo() {
		if(ApplicationUtil.equals(tipo, ConstantsGlobal.tipo_residenziale)) {
			return ConstantsLanguage.TIPO_RESIDENZIALE.stampa();
		} else if(ApplicationUtil.equals(tipo, ConstantsGlobal.tipo_business)) {
			return ConstantsLanguage.TIPO_BUSINESS.stampa();
		} else if(ApplicationUtil.equals(tipo, ConstantsGlobal.tipo_misto)) {
			return ConstantsLanguage.TIPO_MISTO.stampa();
		} else {
			return "";
		} 
	}
	public String getDescrizioneStato() {
		return Message.addCatalogText("ts_stato." + stato);
	}
	private Date data_fine;
	private Date data_inizio;
	private Time ora_fine;
	private Time ora_inizio;
	
	
	public Date getData_fine() {
		return data_fine;
	}
	public void setData_fine(Date data_fine) {
		this.data_fine = data_fine;
	}
	public Date getData_inizio() {
		return data_inizio;
	}
	public void setData_inizio(Date data_inizio) {
		this.data_inizio = data_inizio;
	}
	public Time getOra_fine() {
		return ora_fine;
	}
	public void setOra_fine(Time ora_fine) {
		this.ora_fine = ora_fine;
	}
	public Time getOra_inizio() {
		return ora_inizio;
	}
	public void setOra_inizio(Time ora_inizio) {
		this.ora_inizio = ora_inizio;
	}
	private Boolean inverti = false;
	public Boolean getInverti() {
		if(inverti == null) {
			inverti = false;
		}
		return inverti;
	}
	public void setInverti(Boolean inverti) {
		this.inverti = inverti;
	}
	public Long getId() {
		return id;
	}
	public void setId(Long id) {
		this.id = id;
	}
	public String getNome() {
		return nome;
	}
	public void setNome(String nome) {
		this.nome = nome;
	}
	public Integer getStato() {
		return stato;
	}
	public void setStato(Integer stato) {
		this.stato = stato;
	}

	public String getCodice() {
		return codice;
	}

	public void setCodice(String codice) {
		this.codice = codice;
	}
	
}