package it.callite.utility.thread;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.nio.file.DirectoryStream;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.apache.commons.net.ftp.FTPClient;
import org.apache.commons.net.ftp.FTPFile;

import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.constants.ConstantsMap;
import it.callite.constants.config.DefaultsGeneric;
import it.callite.dao.ComunicazioneDao;
import it.callite.dao.DownloadDao;
import it.callite.dao.ImpostazioneDao;
import it.callite.log.ErrorLog;
import it.callite.model.Appuntamento;
import it.callite.model.Contatto;
import it.callite.model.EsitoArchiviazione;
import it.callite.model.Notifica;
import it.callite.model.Utente;
import it.callite.service.config.ConfigurazioneGenericService;
import it.callite.utility.Allegato;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.ContrattoProdotto;
import it.callite.utility.DatoSondaggioAnagrafica;
import it.callite.utility.ProdottoSelezionato;
import it.callite.utility.UtilFTP;
import it.callite.utility.UtilOffice;
import it.callite.utility.VCardUtils;
import it.callite.utility.ZipPassword;
public class EmailNotificaThread extends Thread {
	private Contatto contatto;
	private List<Allegato> documentiGenerati;
	private ComunicazioneDao comunicazioneDao;
	private ImpostazioneDao impostazioneDao;
	private DownloadDao downloadDao;
	private Map<ProdottoSelezionato, ContrattoProdotto> mapCr;
	private List<Long> appId;
	private List<Long> ctrId;
	private EsitoArchiviazione esitoCliccato;
	private Utente user;
	private List<DatoSondaggioAnagrafica> dS;
	private ConfigurazioneGenericService configurazioneGenericService;

	public EmailNotificaThread(List<Long> appId, List<Long> ctrId, EsitoArchiviazione esitoCliccato, Contatto contatto, List<Allegato> documentiGenerati, ComunicazioneDao comunicazioneDao, DownloadDao downloadDao, ImpostazioneDao impostazioneDao, ConfigurazioneGenericService configurazioneGenericService, Utente user, Map<ProdottoSelezionato, ContrattoProdotto> mapCr, List<DatoSondaggioAnagrafica> dS) {
		this.appId = appId;
		this.ctrId = ctrId;
		this.esitoCliccato = esitoCliccato;
		this.contatto = contatto;
		this.documentiGenerati = documentiGenerati;
		this.comunicazioneDao = comunicazioneDao;
		this.downloadDao = downloadDao;
		this.impostazioneDao = impostazioneDao;
		this.configurazioneGenericService = configurazioneGenericService;
		this.user = user;
		this.mapCr = mapCr;
		this.dS = dS;
	}

	public void run() {
		String emailDestinatario = ApplicationUtil.trimToEmpty(esitoCliccato.getEmail_notifica());
		if (ApplicationUtil.isNotBlank(emailDestinatario)) {
			String calendar = null;
			boolean soloNuovi = false;
			String w = downloadDao.getWatermark();
			boolean concatena = impostazioneDao.getConcatenaFile();
			String html = impostazioneDao.getFormatoDocumento();
			String secretL = ConstantsGlobal.secretL;
			String secretZ = ConstantsGlobal.secretZ;
			String secretT = ConstantsGlobal.secretT;
			UtilOffice uo = impostazioneDao.getUtilOffice();
			List<String> impostazioni = impostazioneDao.getDocumentoEmailNotifica();
			String appname = impostazioneDao.getApplicazione();
			String link = impostazioneDao.getUrlBase();
			String extM = (secretL == null || (html.equalsIgnoreCase(ConstantsMap.pdf))) ? html : ConstantsMap.zip;
			Object [] suffix = new Object [] { contatto.getId(), contatto.getAnagrafica().getDenominazione() };
			List<Allegato> allegati = new ArrayList<Allegato>();
			if(ApplicationUtil.isTrue(impostazioni.contains(ConstantsMap.scheda_sondaggio))) {
				if(ApplicationUtil.notEmpty(dS)) {
					try {
						ByteArrayOutputStream os = new ByteArrayOutputStream();
						String htmlPath = ApplicationUtil.reportFile(contatto.getData_lavorazione(), ConstantsLanguage.SONDAGGIO.stampa(), ConstantsLanguage.ANAGRAFICA.stampa(), suffix) + "." + extM;
						String result = downloadDao.sondaggioToHTMLString(contatto, dS);
						ApplicationUtil.convertM(result, os, html, uo, secretL);
						os.close();
						ApplicationUtil.addFile(allegati, htmlPath, os.toByteArray(), ApplicationUtil.date().getTime());
					} catch (Exception e) {
						ErrorLog.log(null, getClass(), e);
					}
				}
			}
			if(ApplicationUtil.isTrue(impostazioni.contains(ConstantsMap.scheda_prodotto))) {
				int i = 1;
				List<Allegato> fallegati = new ArrayList<Allegato>();
				if(ApplicationUtil.notEmpty(mapCr)) {
					Iterator<ProdottoSelezionato> objIter = mapCr.keySet().iterator();
					while(objIter.hasNext()) {
						ContrattoProdotto lsCrM = mapCr.get(objIter.next());
						try {
							if(lsCrM !=null) {
								ByteArrayOutputStream os = new ByteArrayOutputStream();
								String htmlPath = ApplicationUtil.reportFile(contatto.getData_lavorazione(), ConstantsLanguage.PRODOTTO.stampa(), (i++), lsCrM.getProdotto().getNome(), suffix) + "." + extM;
								String result = downloadDao.pdaToHTMLString(lsCrM);
								ApplicationUtil.convertM(result, os, concatena ? ConstantsMap.pdf : html, uo, secretL);
								os.close();
								ApplicationUtil.addFile(fallegati, htmlPath, os.toByteArray(), ApplicationUtil.date().getTime());
							}
						} catch (Exception e) {
							ErrorLog.log(null, getClass(), e);
						}
					}
				} 
				if(ApplicationUtil.notEmpty(fallegati)) {
					try {
						if(!concatena) {
							ByteArrayOutputStream os = new ByteArrayOutputStream();
							ApplicationUtil.zip(null, fallegati, os, secretZ);
							os.close();
							String nomeFile = ApplicationUtil.reportFile(contatto.getData_lavorazione(), ConstantsLanguage.PRODOTTO.stampa(), suffix) + "." + ConstantsMap.zip;
							ApplicationUtil.addFile(allegati, new Allegato(nomeFile, os.toByteArray()));
						} else {
							ByteArrayOutputStream os = new ByteArrayOutputStream();
							ApplicationUtil.concatenaPDF(w, fallegati, os, secretL);
							os.close();
							String nomeFile = ApplicationUtil.reportFile(contatto.getData_lavorazione(), ConstantsLanguage.PRODOTTO.stampa(), suffix) + "." + ConstantsMap.pdf;
							ApplicationUtil.addFile(allegati, new Allegato(nomeFile, os.toByteArray()));
						}
					} catch (Exception e) {
						ErrorLog.log(null, getClass(), e);
					}
				}
			}
			if(ApplicationUtil.isTrue(impostazioni.contains(ConstantsMap.scheda_anagrafica))) {
				try {
					String htmlPath = ApplicationUtil.reportFile(contatto.getData_lavorazione(), ConstantsLanguage.ANAGRAFICA.stampa(), suffix) + "." + extM;
					ByteArrayOutputStream result = downloadDao.anagraficaToHTMLString(contatto.getAnagrafica(), html, null, secretL);
					result.close();
					ApplicationUtil.addFile(allegati, htmlPath, result.toByteArray(), contatto.getAnagrafica().getData_creazione().getTime());
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				}
				String c = ApplicationUtil.trimToNull(VCardUtils.vcf(contatto.getAnagrafica(), appname));
				if(c != null) {
					Allegato p = new Allegato(ApplicationUtil.lowerCase(ConstantsLanguage.CONTATTO.stampa()) + "." + ConstantsMap.vcf, c.getBytes(ConstantsGlobal.CHARSET));
					ApplicationUtil.addFile(allegati, p);
				}
			}
			if(ApplicationUtil.isTrue(impostazioni.contains(ConstantsMap.scheda_contatto))) {
				try {
					ByteArrayOutputStream os = new ByteArrayOutputStream();
					String htmlPath = ApplicationUtil.reportFile(contatto.getData_lavorazione(), ConstantsLanguage.CONTATTO.stampa(), suffix) + "." + extM;
					String result = downloadDao.contattoToHTMLString(user, contatto, true, true, null);
					ApplicationUtil.convertM(result, os, html, uo, secretL);
					os.close();
					ApplicationUtil.addFile(allegati, htmlPath, os.toByteArray(), ApplicationUtil.date().getTime());
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				}
			}
			if(ApplicationUtil.isTrue(impostazioni.contains(ConstantsMap.scheda_appuntamento))) {
				List<Appuntamento> a = downloadDao.getAppuntamentiContatto(contatto);
				if(!concatena) {
					Allegato appuntamenti = downloadDao.allegaReportAppuntamentiZip(a, user, contatto, appId, secretZ, secretL, soloNuovi, suffix);
					ApplicationUtil.addFile(allegati, appuntamenti);
				} else {
					Allegato appuntamenti = downloadDao.allegaReportAppuntamentiConcatena(a, user, contatto, appId, secretL, soloNuovi, suffix);
					ApplicationUtil.addFile(allegati, appuntamenti);
				}
				Appuntamento n = downloadDao.firstAppuntamento(a, appId);
				if(n != null) {
					calendar = VCardUtils.ical(Arrays.asList(n), null, null, appname, link, true);
				}
			}
			if(ApplicationUtil.isTrue(impostazioni.contains(ConstantsMap.scheda_contratto))) {
				if(!concatena) {
					Allegato contratti = downloadDao.allegaReportContrattiZip(user, contatto, ctrId, secretZ, secretL, soloNuovi, suffix);
					ApplicationUtil.addFile(allegati, contratti);
				} else {
					Allegato contratti = downloadDao.allegaReportContrattiConcatena(user, contatto, ctrId, secretL, soloNuovi, suffix);
					ApplicationUtil.addFile(allegati, contratti);
				}
			}
			if(ApplicationUtil.isTrue(impostazioni.contains(ConstantsMap.file_pda))) {
				List<Allegato> fallegati = new ArrayList<Allegato>();
				if(ApplicationUtil.notEmpty(documentiGenerati)) {
					for(Allegato doc : documentiGenerati) {
						ApplicationUtil.addFile(fallegati, doc);
					}
				}
				if(ApplicationUtil.notEmpty(fallegati)) {
					try {
						if(!concatena) {
							ByteArrayOutputStream os = new ByteArrayOutputStream();
							ApplicationUtil.zip(null, fallegati, os, secretZ);
							os.close();
							String nomeFile = ApplicationUtil.reportFile(contatto.getData_lavorazione(), ConstantsLanguage.PDA.stampa(), suffix) + "." + ConstantsMap.zip;
							ApplicationUtil.addFile(allegati, new Allegato(nomeFile, os.toByteArray()));
						} else {
							ByteArrayOutputStream os = new ByteArrayOutputStream();
							ApplicationUtil.concatenaPDF(w, fallegati, os, secretL);
							os.close();
							String nomeFile = ApplicationUtil.reportFile(contatto.getData_lavorazione(), ConstantsLanguage.PDA.stampa(), suffix) + "." + ConstantsMap.pdf;
							ApplicationUtil.addFile(allegati, new Allegato(nomeFile, os.toByteArray()));
						}
					} catch (Exception e) {
						ErrorLog.log(null, getClass(), e);
					}
				}
			}
			if(ApplicationUtil.isTrue(impostazioni.contains(ConstantsMap.file_registrazione))) {
				List<Allegato> fallegati = new ArrayList<Allegato>();
				String b = impostazioneDao.audioSupportato();
				String monitor = ConstantsGlobal.path_monitor;
				if(ApplicationUtil.isNotTrue(impostazioneDao.getFTPEnabled())) {
					String home = impostazioneDao.getHome();
					File perc = new File(home + monitor);
					if(perc.exists()) {
						Path folder = Paths.get(perc.getAbsolutePath());
						DirectoryStream<Path> stream = ApplicationUtil.streamFile(folder, "_" + contatto.getId() + "." + ConstantsMap.wav);
						if(stream != null) {
							for(Path entry : stream) {
								File file = entry.toFile();
								try {
									ApplicationUtil.outFileRCodec(contatto.getId(), fallegati, home + monitor+"/"+file.getName(), b, file.lastModified());
								} catch (Exception e) { 
									ErrorLog.log(null, getClass(), e);
								} 
							}
						}
					} 
				} else {
					UtilFTP ut = impostazioneDao.getUtilFTP();
					FTPClient vmftp = ApplicationUtil.createClient();
					if (ApplicationUtil.connect(vmftp, ut)) {
						try {
							FTPFile[] wavFiles = ApplicationUtil.listMonitor(vmftp, monitor + "/", contatto.getId(), ConstantsMap.wav);
							if (ApplicationUtil.notEmpty(wavFiles)) {
								for(FTPFile file : wavFiles) {
									ApplicationUtil.outFileRCodec(vmftp, contatto.getId(), fallegati, monitor + "/" + file.getName(), b, ApplicationUtil.time(file.getTimestamp()));
								}
							}
						} catch (Exception e) { 
							ErrorLog.log(null, getClass(), e);
						} finally { 
							ApplicationUtil.disconnect(vmftp);
						}
					}
				}
				if(ApplicationUtil.notEmpty(fallegati)) {
					try {
						if(!concatena) {
							ByteArrayOutputStream os = new ByteArrayOutputStream();
							ApplicationUtil.zip(null,fallegati, os, secretZ);
							os.close();
							String nomeFile = ApplicationUtil.reportFile(contatto.getData_lavorazione(), ConstantsLanguage.REGISTRAZIONI.stampa(), suffix) + "." + ConstantsMap.zip;
							ApplicationUtil.addFile(allegati, new Allegato(nomeFile, os.toByteArray()));
						} else {
							String ext = b != null ? b : ConstantsMap.wav;
							String nomeFile = ApplicationUtil.reportFile(contatto.getData_lavorazione(), ConstantsLanguage.REGISTRAZIONI.stampa(), suffix) + "." + (secretT != null ? ConstantsMap.zip : ext);
							ByteArrayOutputStream sos = new ByteArrayOutputStream();
							if(secretT != null) {
								ByteArrayOutputStream bos = new ByteArrayOutputStream();
								ApplicationUtil.concatena(fallegati, bos, b);
								ZipPassword.zip(null, bos.toByteArray(), sos, ext, secretT);
								bos.close();
							} else {
								ApplicationUtil.concatena(fallegati, sos, b);
							}
							sos.close();
							ApplicationUtil.addFile(allegati, new Allegato(nomeFile, sos.toByteArray()));
						}
					} catch (Exception e) {
						ErrorLog.log(null, getClass(), e);
					}
				}
			}
			String testo = downloadDao.backofficeToHTMLString(contatto, user);
			String oggetto = ConstantsLanguage.ARCHIVIAZIONE.stampa() + " - " + ConstantsLanguage.CAMPAGNA.stampa() + ": "+ contatto.getCampagna_denominazione() +"; "+ ConstantsLanguage.CONTATTO.stampa() +": "+contatto.getId();
			Notifica email = new Notifica();
			email.setData_invio(ApplicationUtil.date());
			email.setSender(ApplicationUtil.trimToNull(configurazioneGenericService.getValore(DefaultsGeneric.KEY_INVIO_MAIL_NOTIFICA)));
			email.setDestinatario(emailDestinatario);
			email.setCc(null);
			email.setTemplate(calendar);
			email.setOggetto(oggetto);
			email.setTesto(testo);
			email.setAllegatiTemplateList(allegati);
			email.setContatto(contatto);
			email.setUtente(user);
			email.impostaUserfield();
			comunicazioneDao.inviaNotifica(email, null);
		}
	}
}