const favicons = new Map();

function addBadge(text){
	addBadgeToFavicon(text);
	if(text > 0) {
		if (navigator.setAppBadge) {
			navigator.setAppBadge(text);
		} else if (navigator.setExperimentalAppBadge) {
			navigator.setExperimentalAppBadge(text);
		} else if (window.ExperimentalBadge) {
			window.ExperimentalBadge.set(text);
		} else if (navigator.setClientBadge) {
			navigator.setClientBadge(text);
		} 
	} else {
		if (navigator.clearAppBadge) {
			navigator.clearAppBadge();
		} else if (navigator.clearExperimentalAppBadge) {
			navigator.clearExperimentalAppBadge();
		} else if (window.ExperimentalBadge) {
			window.ExperimentalBadge.clear();
		} else if (navigator.clearClientBadge) {
			navigator.clearClientBadge();
		} 
	}
}

function addBadgeToFavicon(text){
	const allFavicons = Array.from(document.querySelectorAll('link[rel="icon"]'));
	const faviconSize = 32;
	if(text > 0) {
		const padding = 6;
		const fontSize = 18;
		const fontFamily = 'Arial, sans-serif';
		const backgroundColor = '#B30000';
		const textColor = '#FFFFFF';
		const textAlign = 'center';
		const textBaseline = 'middle';
		allFavicons.forEach((favicon) => {
			const canvas = document.createElement('canvas');
			canvas.width = faviconSize;
			canvas.height = faviconSize;
			const context = canvas.getContext('2d');
			context.font = `${fontSize}px ${fontFamily}`;
			const textMetrics = context.measureText(text);
			const left = textMetrics.actualBoundingBoxLeft * -1;
			const top = textMetrics.actualBoundingBoxAscent * -1;
			const right = textMetrics.actualBoundingBoxRight;
			const bottom = textMetrics.actualBoundingBoxDescent;
			const width = right - left;
			const height = bottom - top;
			const drawBadge = (img) => {
				context.drawImage(img, 0, 0, faviconSize, faviconSize);
				context.beginPath();
				const badgeWidth = Math.max(width + (padding * 2), faviconSize / 2);
				const badgeHeight = height + (padding * 2);
				context.roundRect(faviconSize - (width + (padding * 2)),0,badgeWidth,badgeHeight,fontSize );
				context.fillStyle = backgroundColor;
				context.fill();
				context.textAlign = textAlign;
				context.textBaseline = textBaseline;
				context.fillStyle = textColor;
				context.fillText(text, faviconSize - (badgeWidth / 2), padding * 2);
				favicon.href = canvas.toDataURL('image/png');
			};
			const onImageLoaded = (event) => {
				if (favicons.has(favicon.dataset.originalHref) === false) {
					favicons.set(favicon.dataset.originalHref, event.target);
				}
				drawBadge(event.target);
			};
			if (favicons.has(favicon.dataset.originalHref)) {
				drawBadge(favicons.get(favicon.dataset.originalHref));
				return;
			}
			const img = document.createElement('img');
			img.addEventListener('load', onImageLoaded);
			img.src = favicon.href;
		});
	} else {
	    allFavicons.forEach((favicon) => {
			const canvas = document.createElement('canvas');
			const context = canvas.getContext('2d');
			canvas.height = faviconSize;
			canvas.width = faviconSize;
			context.drawImage(favicons.get(favicon.dataset.originalHref), 0, 0, faviconSize, faviconSize);
			favicon.href = canvas.toDataURL('image/png');
		});
	}
}