package it.callite.dao;
import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CopyOnWriteArrayList;

import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import it.callite.action.DownloadBean;
import it.callite.comparator.ListComparator;
import it.callite.constants.ConstantsFormat;
import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.constants.ConstantsMap;
import it.callite.constants.config.DefaultsGeneric;
import it.callite.criteria.FileCriteria;
import it.callite.dao.generic.GenericDao;
import it.callite.log.ErrorLog;
import it.callite.model.Accesso;
import it.callite.model.Acconto;
import it.callite.model.Agente;
import it.callite.model.Anagrafica;
import it.callite.model.Appuntamento;
import it.callite.model.AutoContatto;
import it.callite.model.Campagna;
import it.callite.model.Contatto;
import it.callite.model.Contratto;
import it.callite.model.DatoAggiuntivoAnagrafica;
import it.callite.model.DatoAggiuntivoContratto;
import it.callite.model.Email;
import it.callite.model.EsitoContratto;
import it.callite.model.Estratto;
import it.callite.model.EstrattoDettaglio;
import it.callite.model.FatturaContratto;
import it.callite.model.FatturaEstratto;
import it.callite.model.Marcatura;
import it.callite.model.Nota;
import it.callite.model.Notifica;
import it.callite.model.Operazione;
import it.callite.model.Opposizione;
import it.callite.model.Pausa;
import it.callite.model.Prenotazione;
import it.callite.model.Prodotto;
import it.callite.model.Ricezione;
import it.callite.model.Rmail;
import it.callite.model.Sondaggio;
import it.callite.model.StoriaAppuntamento;
import it.callite.model.StoriaContatto;
import it.callite.model.StoriaContratto;
import it.callite.model.Telecom;
import it.callite.model.TemplateExportAnagrafica;
import it.callite.model.TemplateExportAppuntamento;
import it.callite.model.TemplateExportContatto;
import it.callite.model.TemplateExportContratto;
import it.callite.model.TemplateExportStoriaAppuntamento;
import it.callite.model.TemplateExportStoriaContatto;
import it.callite.model.TemplateExportStoriaContratto;
import it.callite.model.TemplateImportAppuntamento;
import it.callite.model.TemplateImportContatto;
import it.callite.model.TemplateImportContratto;
import it.callite.model.Utente;
import it.callite.model.Webhook;
import it.callite.model.anagrafica.AnagraficaXML;
import it.callite.model.messaggio.AllegatoMessaggio;
import it.callite.model.messaggio.Destinatario;
import it.callite.model.messaggio.Messaggio;
import it.callite.model.report.ReportContattiOdierniTeam;
import it.callite.model.report.ReportContattiOdierniZona;
import it.callite.model.sip.Cdr;
import it.callite.model.sip.Cel;
import it.callite.model.sip.SipBuddies;
import it.callite.service.config.ConfigurazioneGenericService;
import it.callite.utility.Allegato;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.AppuntamentoVO;
import it.callite.utility.CSVExport;
import it.callite.utility.ContattoVO;
import it.callite.utility.ContrattoProdotto;
import it.callite.utility.ContrattoVO;
import it.callite.utility.DateFormatter;
import it.callite.utility.DatoAggiuntivo;
import it.callite.utility.DatoSondaggio;
import it.callite.utility.DatoSondaggioAnagrafica;
import it.callite.utility.ElementoScheda;
import it.callite.utility.ExcelExport;
import it.callite.utility.ExcelFatture;
import it.callite.utility.FileUpload;
import it.callite.utility.MyFile;
import it.callite.utility.PDFWriter;
import it.callite.utility.QueryKey;
import it.callite.utility.RicezioneUtil;
import it.callite.utility.StampaUtils;
import it.callite.utility.UtilOffice;
import it.callite.utility.VCardUtils;
import it.callite.utility.VelocityEngineUtil;
import it.callite.utility.VelocityUtils;
import it.callite.utility.ZipPassword;
import it.callite.utility.importa.ImportModelAnagrafica;
import it.callite.utility.importa.ImportModelConsentite;
import it.callite.utility.item.ContattoSI;
import it.callite.utility.utils.ModuloFileFilter;
@SuppressWarnings("serial")
@Repository("downloadDao")
public class DownloadDao extends GenericDao {
	@Autowired
	private PannelloDao pannelloDao;
	@Autowired
	private ImpostazioneDao impostazioneDao;
	@Autowired
	private AnagraficaDao anagraficaDao;
	@Autowired
	private EstrattoDao estrattoDao;
	@Autowired
	private ContrattoDao contrattoDao;
	@Autowired
	private OperazioneDao operazioneDao;
	@Autowired
	private CampagnaDao campagnaDao;
	@Autowired
	private ConfigurazioneGenericService configurazioneGenericService;
	//@Autowired
	//private Configuration freemarkerConfiguration;
	@Autowired
	private VelocityEngineUtil velocityEngineUtil;
	
	
	public String getEmailExportBackground() {
		return configurazioneGenericService.getValore(DefaultsGeneric.KEY_INVIO_MAIL_EXPORT_BACKGROUND);
	}
	
	public String getEmailImportBackground() {
		return configurazioneGenericService.getValore(DefaultsGeneric.KEY_INVIO_MAIL_IMPORT_BACKGROUND);
	}
	
	public String getPathExport() {
		return impostazioneDao.getPathBase(ConstantsGlobal.path_export);
	}
	
	public void esportaExcelContatto(Utente user, List<Contatto>  contatti, DownloadBean downloadBean) {
		try {
			String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.CONTATTO.stampa() +": " + ApplicationUtil.size(contatti), contatti);
			operazioneDao.creaOperazione(ConstantsMap.LOG4, p ,user, ConstantsMap.OPERAZIONE_EXPORT_CONTATTO);
			String s = impostazioneDao.getPasswordXLS();
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.CONTATTI.stampa()) + "." + ConstantsMap.xlsx;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, contatti);
			ExcelExport.esportaExcelContatto(contatti, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaExcelChiamate(Utente user, CopyOnWriteArrayList<AutoContatto>  c, DownloadBean downloadBean) {
		try { 
			String s = ConstantsGlobal.secretE;
			if(c == null) {
				c = new CopyOnWriteArrayList<AutoContatto>();
			}
			Collections.sort(c, new ListComparator());
			String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.CONTATTO.stampa() +": " + ApplicationUtil.size(c), c);
			operazioneDao.creaOperazione(ConstantsMap.LOG4, p ,user, ConstantsMap.OPERAZIONE_EXPORT_CHIAMATE);
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.AUTO_CONTATTI.stampa())+ "." + ConstantsMap.xlsx;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, c);
			ExcelExport.esportaExcelChiamate(c, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaPdfChiamate(Utente user, CopyOnWriteArrayList<AutoContatto>  c, DownloadBean downloadBean) {
		try {
			String s = ConstantsGlobal.secretP;
			if(c == null) {
				c = new CopyOnWriteArrayList<AutoContatto>();
			}
			Collections.sort(c, new ListComparator());
			String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.CONTATTO.stampa() +": " + ApplicationUtil.size(c), c);
			operazioneDao.creaOperazione(ConstantsMap.LOG4,  p,user,  ConstantsMap.OPERAZIONE_EXPORT_CHIAMATE);
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.AUTO_CONTATTI.stampa()) + "." +ConstantsMap.pdf;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, c);
			PDFWriter.stampaPDFChiamate(c, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaExcelAppuntamento(Utente user,List<Appuntamento> appuntamenti, List<Long> cc, DownloadBean downloadBean) {
		try {
			String s = impostazioneDao.getPasswordXLS();
			String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.APPUNTAMENTO.stampa() +": " + ApplicationUtil.size(appuntamenti), appuntamenti);
			operazioneDao.creaOperazione(ConstantsMap.LOG4,p ,user, ConstantsMap.OPERAZIONE_EXPORT_APPUNTAMENTO);
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.APPUNTAMENTI.stampa())+ "." + ConstantsMap.xlsx;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, appuntamenti);
			ExcelExport.esportaExcelAppuntamento(appuntamenti, anagraficaDao, cc, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaExcelContrattoVO(List<ContrattoVO> contratti, TemplateImportContratto ti, DownloadBean downloadBean) {
		try {
			String s = ConstantsGlobal.secretE;
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.CONTRATTI.stampa())+ "." + ConstantsMap.xlsx;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, contratti);
			ExcelExport.esportaExcelContrattoVO(contratti, ti, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaExcelAppuntamentoVO(List<AppuntamentoVO> contratti, TemplateImportAppuntamento ti, DownloadBean downloadBean) {
		try {
			String s = ConstantsGlobal.secretE;
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.APPUNTAMENTI.stampa())+ "." + ConstantsMap.xlsx;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, contratti);
			ExcelExport.esportaExcelAppuntamentoVO(contratti, ti, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaExcelContattoVO(List<ContattoVO> contratti, TemplateImportContatto ti, DownloadBean downloadBean) {
		try {
			String s = ConstantsGlobal.secretE;
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.CONTATTI.stampa())+ "." + ConstantsMap.xlsx;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, contratti);
			ExcelExport.esportaExcelContattoVO(contratti, ti, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaExcelAnagrafica(List<Anagrafica>  contatti, DownloadBean downloadBean) {
		try {
			String s = impostazioneDao.getPasswordXLS();
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.ANAGRAFICHE.stampa())+ "." + ConstantsMap.xlsx;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, contatti);
			ExcelExport.esportaExcelAnagrafica(contatti, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaPdfAnagrafica(List<Anagrafica>  contatti, DownloadBean downloadBean) {
		try {
			String s = impostazioneDao.getPasswordPDF();
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.ANAGRAFICHE.stampa())+ "." +ConstantsMap.pdf;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, contatti);
			PDFWriter.stampaPDFAnagrafica(contatti, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaExcelAnagrafica(List<Anagrafica> anagrafiche, List<Long> cc, DownloadBean downloadBean) {
		try {
			String s = impostazioneDao.getPasswordXLS();
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.ANAGRAFICHE.stampa())+ "." + ConstantsMap.xlsx;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, anagrafiche);
			ExcelExport.esportaExcelAnagrafica(anagrafiche, anagraficaDao, cc, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaPdfAnagrafica(List<Anagrafica> anagrafiche, List<Long> cc, DownloadBean downloadBean) {
		try {
			String s = impostazioneDao.getPasswordPDF();
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.ANAGRAFICHE.stampa()) + "." +ConstantsMap.pdf;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, anagrafiche);
			PDFWriter.stampaPDFAnagrafica(anagrafiche, out, cc, anagraficaDao, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaPdfTemplateContratto(Utente user, Boolean ps, List<Contratto>  contratti,  Long tid, DownloadBean downloadBean) {
		if(ApplicationUtil.gtZero(tid)) {
			try {
				String s = impostazioneDao.getPasswordPDF();
				String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.CONTRATTO.stampa() +": " + ApplicationUtil.size(contratti), contratti);
				operazioneDao.creaOperazione(ConstantsMap.LOG4,p ,user, ConstantsMap.OPERAZIONE_EXPORT_CONTRATTO);
				TemplateExportContratto ti = anagraficaDao.getTemplateExportContratto(tid);
				ApplicationUtil.comparatorCondition(contratti, ti);
				String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.CONTRATTI.stampa()) + "." +ConstantsMap.pdf;

				OutputStream out = downloadBean.outputStream();
				downloadBean.filename(file, contratti);
				PDFWriter.stampaPDFContrattoTemplate(ps, contratti, out, ti, anagraficaDao, s);
				downloadBean.close();
			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		}
	}
	public void esportaPdfTemplateAppuntamento(Utente user, Boolean ps, List<Appuntamento> appuntamenti,  Long tid, DownloadBean downloadBean) {
		if(ApplicationUtil.gtZero(tid)) {
			try {
				String s = impostazioneDao.getPasswordPDF();
				String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.APPUNTAMENTO.stampa() +": " + ApplicationUtil.size(appuntamenti), appuntamenti);
				operazioneDao.creaOperazione(ConstantsMap.LOG4,p ,user, ConstantsMap.OPERAZIONE_EXPORT_APPUNTAMENTO);
				TemplateExportAppuntamento ti = anagraficaDao.getTemplateExportAppuntamento(tid);
				ApplicationUtil.comparatorCondition(appuntamenti, ti);
				String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.APPUNTAMENTI.stampa()) + "." +ConstantsMap.pdf;

				OutputStream out = downloadBean.outputStream();
				downloadBean.filename(file, appuntamenti);
				PDFWriter.stampaPDFAppuntamentoTemplate(ps, appuntamenti, out, ti, anagraficaDao, s);
				downloadBean.close();
			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		}
	}
	public void esportaPdfTemplateContatto(Utente user, Boolean ps, List<Contatto>  contatti,  Long tid, DownloadBean downloadBean) {
		if(ApplicationUtil.gtZero(tid)) {
			try {
				String s = impostazioneDao.getPasswordPDF();
				String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.CONTATTO.stampa() +": " + ApplicationUtil.size(contatti), contatti);
				operazioneDao.creaOperazione(ConstantsMap.LOG4, p ,user, ConstantsMap.OPERAZIONE_EXPORT_CONTATTO);
				TemplateExportContatto ti = anagraficaDao.getTemplateExportContatto(tid);
				ApplicationUtil.comparatorCondition(contatti, ti);
				String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.CONTATTI.stampa()) + "." +ConstantsMap.pdf;

				OutputStream out = downloadBean.outputStream();
				downloadBean.filename(file, contatti);
				PDFWriter.stampaPDFContattoTemplate(ps, contatti, out, ti, anagraficaDao, s);
				downloadBean.close();
			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		}
	}
	public void esportaPdfTemplateAnagrafica(List<Anagrafica> anagrafiche,  Long tid, DownloadBean downloadBean) {
		if(ApplicationUtil.gtZero(tid)) {
			try {
				String s = impostazioneDao.getPasswordPDF();
				TemplateExportAnagrafica ti = anagraficaDao.getTemplateExportAnagrafica(tid);
				ApplicationUtil.comparatorCondition(anagrafiche, ti);
				String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.ANAGRAFICHE.stampa()) + "." +ConstantsMap.pdf;

				OutputStream out = downloadBean.outputStream();
				downloadBean.filename(file, anagrafiche);
				PDFWriter.stampaPDFAnagraficaTemplate(anagrafiche, out, ti, anagraficaDao, s);
				downloadBean.close();
			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		}
	}
	public void esportaExcelTemplateAppuntamento(Utente user, Boolean ps, List<Appuntamento> appuntamenti, Long tid, DownloadBean downloadBean) {
		if(ApplicationUtil.gtZero(tid)) {
			try {
				String s = impostazioneDao.getPasswordXLS();
				String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.APPUNTAMENTO.stampa() +": " + ApplicationUtil.size(appuntamenti), appuntamenti);
				operazioneDao.creaOperazione(ConstantsMap.LOG4,p ,user,  ConstantsMap.OPERAZIONE_EXPORT_APPUNTAMENTO);
				TemplateExportAppuntamento ti = anagraficaDao.getTemplateExportAppuntamento(tid);
				ApplicationUtil.comparatorCondition(appuntamenti, ti);
				String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.APPUNTAMENTI.stampa())+ "." + ConstantsMap.xlsx;

				OutputStream out = downloadBean.outputStream();
				downloadBean.filename(file, appuntamenti);
				ExcelExport.esportaExcelAppuntamentoTemplate(ps, appuntamenti, anagraficaDao, ti, out, s);
				downloadBean.close();
			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		}
	}
	public void esportaExcelTemplateContratto(Utente user, Boolean ps, List<Contratto>  contratti, Long tid, DownloadBean downloadBean) {
		if(ApplicationUtil.gtZero(tid)) {
			try {
				String s = impostazioneDao.getPasswordXLS();
				String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.CONTRATTO.stampa() +": " + ApplicationUtil.size(contratti), contratti);
				operazioneDao.creaOperazione(ConstantsMap.LOG4,p ,user, ConstantsMap.OPERAZIONE_EXPORT_CONTRATTO);
				TemplateExportContratto ti = anagraficaDao.getTemplateExportContratto(tid);
				ApplicationUtil.comparatorCondition(contratti, ti);
				String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.CONTRATTI.stampa())+ "." + ConstantsMap.xlsx;

				OutputStream out = downloadBean.outputStream();
				downloadBean.filename(file, contratti);
				ExcelExport.esportaExcelContrattoTemplate(ps, contratti, anagraficaDao, ti, out, s);
				downloadBean.close();
			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		}
	}
	public void esportaExcelTemplateAnagrafica(List<Anagrafica> anagrafiche, Long tid, DownloadBean downloadBean) {
		if(ApplicationUtil.gtZero(tid)) {
			try {
				String s = impostazioneDao.getPasswordXLS();
				TemplateExportAnagrafica ti = anagraficaDao.getTemplateExportAnagrafica(tid);
				ApplicationUtil.comparatorCondition(anagrafiche, ti);
				String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.ANAGRAFICHE.stampa())+ "." + ConstantsMap.xlsx;

				OutputStream out = downloadBean.outputStream();
				downloadBean.filename(file, anagrafiche);
				ExcelExport.esportaExcelAnagraficaTemplate(anagrafiche, anagraficaDao, ti, out, s);
				downloadBean.close();
			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		}
	}
	public void esportaExcelTemplateContatto(Utente user, Boolean ps, List<Contatto>  contatti, Long tid, DownloadBean downloadBean) {
		if(ApplicationUtil.gtZero(tid)) {
			try {
				String s = impostazioneDao.getPasswordXLS();
				String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.CONTATTO.stampa() +": " + ApplicationUtil.size(contatti), contatti);
				operazioneDao.creaOperazione(ConstantsMap.LOG4, p ,user, ConstantsMap.OPERAZIONE_EXPORT_CONTATTO);
				TemplateExportContatto ti = anagraficaDao.getTemplateExportContatto(tid);
				ApplicationUtil.comparatorCondition(contatti, ti);
				String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.CONTATTI.stampa())+ "." + ConstantsMap.xlsx;

				OutputStream out = downloadBean.outputStream();
				downloadBean.filename(file, contatti);
				ExcelExport.esportaExcelContattoTemplate(ps, contatti, anagraficaDao, ti, out, s);
				downloadBean.close();
			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		}
	}
	public void esportaExcelAppuntamento(Utente user, List<Appuntamento> appuntamenti, DownloadBean downloadBean) {
		try {
			String s = impostazioneDao.getPasswordXLS();
			String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.APPUNTAMENTO.stampa() +": " + ApplicationUtil.size(appuntamenti), appuntamenti);
			operazioneDao.creaOperazione(ConstantsMap.LOG4,p ,user, ConstantsMap.OPERAZIONE_EXPORT_APPUNTAMENTO);
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.APPUNTAMENTI.stampa())+ "." + ConstantsMap.xlsx;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, appuntamenti);
			ExcelExport.esportaExcelAppuntamento(appuntamenti, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaExcelAgente(List<Agente> agenti, DownloadBean downloadBean) {
		try {
			String s = ConstantsGlobal.secretE;
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.AGENTI.stampa())+ "." + ConstantsMap.xlsx;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, agenti);
			ExcelExport.esportaExcelAgente(agenti, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaExcelContatto(Utente user, List<Contatto>  contatti, List<Long> cc, DownloadBean downloadBean) {
		try {
			String s = impostazioneDao.getPasswordXLS();
			String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.CONTATTO.stampa() +": " + ApplicationUtil.size(contatti), contatti);
			operazioneDao.creaOperazione(ConstantsMap.LOG4, p ,user, ConstantsMap.OPERAZIONE_EXPORT_CONTATTO);
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.CONTATTI.stampa())+ "." + ConstantsMap.xlsx;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, contatti);
			ExcelExport.esportaExcelContatto(contatti, anagraficaDao, cc, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaExcelContratto(Utente user, List<Contratto>  contratti, List<Long> aa, List<Long> bb, DownloadBean downloadBean) {
		try {
			String s = impostazioneDao.getPasswordXLS();
			String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.CONTRATTO.stampa() +": " + ApplicationUtil.size(contratti), contratti);
			operazioneDao.creaOperazione(ConstantsMap.LOG4, p ,user, ConstantsMap.OPERAZIONE_EXPORT_CONTRATTO);
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.CONTRATTI.stampa())+ "." + ConstantsMap.xlsx;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, contratti);
			ExcelExport.esportaExcelContratto(contratti, anagraficaDao, aa, bb, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaExcelContratto(Utente user, List<Contratto>  contratti, DownloadBean downloadBean) {
		try {
			String s = impostazioneDao.getPasswordXLS();
			String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.CONTRATTO.stampa() +": " + ApplicationUtil.size(contratti), contratti);
			operazioneDao.creaOperazione(ConstantsMap.LOG4,p ,user, ConstantsMap.OPERAZIONE_EXPORT_CONTRATTO);
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.CONTRATTI.stampa())+ "." + ConstantsMap.xlsx;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, contratti);
			ExcelExport.esportaExcelContratto(contratti, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaExcelOpposizione(List<Opposizione>  opposizioni, DownloadBean downloadBean) {
		try {
			String s = ConstantsGlobal.secretE;
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.OPPOSIZIONI.stampa())+ "." + ConstantsMap.xlsx;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, opposizioni);
			ExcelExport.esportaExcelOpposizione(opposizioni, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaExcelPrenotazione(Utente user, List<Prenotazione>  prenotazioni, DownloadBean downloadBean) {
		try {
			String s = impostazioneDao.getPasswordXLS();
			String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.PRENOTAZIONE.stampa() +": " + ApplicationUtil.size(prenotazioni), prenotazioni);
			operazioneDao.creaOperazione(ConstantsMap.LOG4, p, user, ConstantsMap.OPERAZIONE_EXPORT_PRENOTAZIONE);
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.PRENOTAZIONI.stampa())+ "." + ConstantsMap.xlsx;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, prenotazioni);
			ExcelExport.esportaExcelPrenotazione(prenotazioni, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaExcelUtente(List<Utente>  contratti, DownloadBean downloadBean) {
		try {
			String s = ConstantsGlobal.secretE;
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.UTENTI.stampa())+ "." + ConstantsMap.xlsx;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, contratti);
			ExcelExport.esportaExcelUser(contratti, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaExcelEstratto(Utente user, List<Estratto>  estratti, DownloadBean downloadBean) {
		try {
			String s = ConstantsGlobal.secretE;
			if(user != null) {
				String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.ESTRATTO.stampa() +": " + ApplicationUtil.size(estratti), estratti);
				operazioneDao.creaOperazione(ConstantsMap.LOG4,p ,user, ConstantsMap.OPERAZIONE_EXPORT_ESTRATTO);
			}
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.ESTRATTI.stampa())+ "." + ConstantsMap.xlsx;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, estratti);
			ExcelExport.esportaExcelEstratto(estratti, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaExcelEstrattoAnteprima(Utente user, List<Estratto>  estratti, DownloadBean downloadBean) {
		try {
			String s = ConstantsGlobal.secretE;
			if(user != null) {
				String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.ESTRATTO.stampa() +": " + ApplicationUtil.size(estratti), estratti);
				operazioneDao.creaOperazione(ConstantsMap.LOG4,p ,user, ConstantsMap.OPERAZIONE_EXPORT_ESTRATTO);
			}
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.ESTRATTI.stampa())+ "." + ConstantsMap.xlsx;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, estratti);
			ExcelExport.esportaExcelEstrattoAnteprima(estratti, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaExcelAcconto(Utente user, List<Acconto> acconti, DownloadBean downloadBean) {
		try {
			String s = ConstantsGlobal.secretE;
			if(user != null) {
				String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.ACCONTO.stampa() +": " + ApplicationUtil.size(acconti), acconti);
				operazioneDao.creaOperazione(ConstantsMap.LOG4,p ,user, ConstantsMap.OPERAZIONE_EXPORT_ACCONTO);
			}
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.ACCONTI.stampa())+ "." + ConstantsMap.xlsx;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, acconti);
			ExcelExport.esportaExcelAcconto(acconti, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaExcelEstrattoDettagli(Estratto estratto, DownloadBean downloadBean) {
		List<EstrattoDettaglio>  dettagli = estrattoDao.trovaDettagli(estratto.getId());
		try {
			String s = ConstantsGlobal.secretE;
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.ESTRATTI_DETTAGLIO.stampa(), estratto.getUser().getDenominazione()) + "." + ConstantsMap.xlsx;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, dettagli);
			ExcelExport.esportaExcelDettagli(dettagli, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaExcelEstrattoDettagli(Estratto estratto, List<EstrattoDettaglio>  dettagli, DownloadBean downloadBean) {
		try {
			String s = ConstantsGlobal.secretE;
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.ESTRATTI_DETTAGLIO.stampa(), estratto.getUser().getDenominazione()) + "." + ConstantsMap.xlsx;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, dettagli);
			ExcelExport.esportaExcelDettagli(dettagli, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaExcelEstrattoDettagliAnteprima(Estratto estratto, List<EstrattoDettaglio>  dettagli, DownloadBean downloadBean) {
		try {
			String s = ConstantsGlobal.secretE;
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.ESTRATTI_DETTAGLIO.stampa(), estratto.getUser().getDenominazione()) + "." + ConstantsMap.xlsx;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, dettagli);
			ExcelExport.esportaExcelDettagliAnteprima(dettagli, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaExcelFatturaContratto(Contratto contratto, Long fattId, DownloadBean downloadBean) {
		int progressivo = estrattoDao.contaFatturePerContrattoAnno(contratto);
		FatturaContratto fatt = estrattoDao.getFatturaContratto(fattId);
		try {
			if(fatt!=null && contratto.getFatturabile()) {
				String ext = ApplicationUtil.getExtensionLower(fatt.getNomeFileS());
				String progressivoU = (progressivo+1) + impostazioneDao.getPreamboloFattura();
				String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.FATTURA.stampa(), contratto.getContatto().getAnagrafica().getDenominazione()) + "." + ext;

				OutputStream out = downloadBean.outputStream();
				downloadBean.filename(file);
				String s = ConstantsGlobal.secretE;
				ExcelFatture.stampaExcelFatturaContratto(contratto, fatt, progressivoU, out, s);
				downloadBean.close();
			}
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaExcelFatturaContrattoList(List<Contratto> contratti,  Long fattId, DownloadBean downloadBean) {
		FatturaContratto fatt = estrattoDao.getFatturaContratto(fattId);
		List<Allegato> files = new ArrayList<Allegato>();
		if(ApplicationUtil.notEmpty(contratti)) {
			String pc = impostazioneDao.getPreamboloFattura();
			Iterator<Contratto> objIter = contratti.iterator();
			String sE = ConstantsGlobal.secretE;
			String sZ = ConstantsGlobal.secretZ;
			while(objIter.hasNext()) {
				Contratto contratto = objIter.next();
				try {
					if(fatt!=null && contratto.getFatturabile()) {
						int progressivo = estrattoDao.contaFatturePerContrattoAnno(contratto);
						String ext = ApplicationUtil.getExtensionLower(fatt.getNomeFileS());
						String progressivoU = (progressivo+1) + pc;
						String file = ApplicationUtil.reportFile(contratto.getData_creazione(), ConstantsLanguage.FATTURA.stampa(), contratto.getContatto().getAnagrafica().getDenominazione(), contratto.getId(), progressivoU) + "." + ext;
						ByteArrayOutputStream fi = new ByteArrayOutputStream();
						ExcelFatture.stampaExcelFatturaContratto(contratto, fatt, progressivoU, fi, sE);
						fi.flush();
						fi.close();
						ApplicationUtil.addFile(files, file, fi.toByteArray(), contratto.getData_creazione().getTime());
					}
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				}
			}
			try {
				if (ApplicationUtil.empty(files)) {
					ApplicationUtil.fileWriter(files, ConstantsLanguage.NO_FATTURA.stampa(), ConstantsLanguage.FATTURA_NON_PRESENTE.stampa());
				}

				String fileName = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.FATTURE.stampa()) + "." + ConstantsMap.zip;
				OutputStream out = downloadBean.outputStream();
				downloadBean.filename(fileName, files);
				ApplicationUtil.zip(null, files, out, sZ);
				downloadBean.close();
			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			} 
		}
	}
	public void esportaPdfAppuntamento(Utente user, List<Appuntamento> appuntamenti, DownloadBean downloadBean) {
		try {
			String s = impostazioneDao.getPasswordPDF();
			String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.APPUNTAMENTO.stampa() +": " + ApplicationUtil.size(appuntamenti), appuntamenti);
			operazioneDao.creaOperazione(ConstantsMap.LOG4,p ,user,  ConstantsMap.OPERAZIONE_EXPORT_APPUNTAMENTO);
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.APPUNTAMENTI.stampa())+ "." +ConstantsMap.pdf;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, appuntamenti);
			PDFWriter.stampaPDFAppuntamento(appuntamenti, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaPdfAppuntamento(Utente user, List<Appuntamento> appuntamenti, List<Long> cc, DownloadBean downloadBean) {
		try {
			String s = impostazioneDao.getPasswordPDF();
			String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.APPUNTAMENTO.stampa() +": " + ApplicationUtil.size(appuntamenti), appuntamenti);
			operazioneDao.creaOperazione(ConstantsMap.LOG4,p ,user, ConstantsMap.OPERAZIONE_EXPORT_APPUNTAMENTO);
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.APPUNTAMENTI.stampa()) + "." + ConstantsMap.pdf;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, appuntamenti);
			PDFWriter.stampaPDFAppuntamento(appuntamenti, out, cc, anagraficaDao, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaPdfAgente(List<Agente> agenti, DownloadBean downloadBean) {
		try {
			String s = ConstantsGlobal.secretP;
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.AGENTI.stampa())+ "." +ConstantsMap.pdf;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, agenti);
			PDFWriter.stampaPDFAgente(agenti, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaPdfContattoLista(Utente user, List<Contatto>  contatti, List<Long> cc, DownloadBean downloadBean) {
		try {
			String s = impostazioneDao.getPasswordPDF();
			String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.CONTATTO.stampa() +": " + ApplicationUtil.size(contatti), contatti);
			operazioneDao.creaOperazione(ConstantsMap.LOG4, p ,user, ConstantsMap.OPERAZIONE_EXPORT_CONTATTO);
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.CONTATTI.stampa()) + "." +ConstantsMap.pdf;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, contatti);
			PDFWriter.stampaPDFContatto(contatti, out, cc, anagraficaDao, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaPdfContattoLista(Utente user, List<Contatto>  contatti, DownloadBean downloadBean) {
		try {
			String s = impostazioneDao.getPasswordPDF();
			String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.CONTATTO.stampa() +": " + ApplicationUtil.size(contatti), contatti);
			operazioneDao.creaOperazione(ConstantsMap.LOG4, p ,user, ConstantsMap.OPERAZIONE_EXPORT_CONTATTO);
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.CONTATTI.stampa())+ "." +ConstantsMap.pdf;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, contatti);
			PDFWriter.stampaPDFContatto(contatti, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaPdfContratto(Utente user, List<Contratto>  contratti, List<Long> aa, List<Long> bb, DownloadBean downloadBean) {
		try {
			String s = impostazioneDao.getPasswordPDF();
			String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.CONTRATTO.stampa() +": " + ApplicationUtil.size(contratti), contratti);
			operazioneDao.creaOperazione(ConstantsMap.LOG4,p ,user, ConstantsMap.OPERAZIONE_EXPORT_CONTRATTO);
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.CONTRATTI.stampa())+ "." +ConstantsMap.pdf;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, contratti);
			PDFWriter.stampaPDFContratto(contratti, out, aa, bb, anagraficaDao, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaPdfContratto(Utente user, List<Contratto>  contratti, DownloadBean downloadBean) {
		try {
			String s = impostazioneDao.getPasswordPDF();
			String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.CONTRATTO.stampa() +": " + ApplicationUtil.size(contratti), contratti);
			operazioneDao.creaOperazione(ConstantsMap.LOG4,p ,user, ConstantsMap.OPERAZIONE_EXPORT_CONTRATTO);
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.CONTRATTI.stampa())+ "." +ConstantsMap.pdf;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, contratti);
			PDFWriter.stampaPDFContratto(contratti, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaPdfEstratto(Utente user, List<Estratto>  estratti, DownloadBean downloadBean) {
		try {
			String s = ConstantsGlobal.secretP;
			if(user != null) {
				String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.ESTRATTO.stampa() +": " + ApplicationUtil.size(estratti), estratti);
				operazioneDao.creaOperazione(ConstantsMap.LOG4,p ,user, ConstantsMap.OPERAZIONE_EXPORT_ESTRATTO);
			}
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.ESTRATTI.stampa())+ "." +ConstantsMap.pdf;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, estratti);
			PDFWriter.stampaPDFEstratto(estratti, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaPdfEstrattoAnteprima(Utente user, List<Estratto>  estratti, DownloadBean downloadBean) {
		try {
			String s = ConstantsGlobal.secretP;
			if(user != null) {
				String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.ESTRATTO.stampa() +": " + ApplicationUtil.size(estratti), estratti);
				operazioneDao.creaOperazione(ConstantsMap.LOG4,p ,user, ConstantsMap.OPERAZIONE_EXPORT_ESTRATTO);
			}
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.ESTRATTI.stampa())+ "." +ConstantsMap.pdf;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, estratti);
			PDFWriter.stampaPDFEstrattoAnteprima(estratti, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaPdfAcconto(Utente user, List<Acconto> acconti, DownloadBean downloadBean) {
		try {
			String s = ConstantsGlobal.secretP;
			if(user != null) {
				String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.ACCONTO.stampa() +": " + ApplicationUtil.size(acconti), acconti);
				operazioneDao.creaOperazione(ConstantsMap.LOG4, p,user, ConstantsMap.OPERAZIONE_EXPORT_ACCONTO);
			}
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.ACCONTI.stampa())+ "." +ConstantsMap.pdf;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, acconti);
			PDFWriter.stampaPDFAcconto(acconti, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaExcelFatturaEstratto(Estratto estratto, DownloadBean downloadBean) {
		int progressivo = estrattoDao.contaFatturePerEstrattoAnno(estratto);
		try {
			FatturaEstratto fatt = estratto.getUser().getFattura();
			if(fatt!=null) {
				String s = ConstantsGlobal.secretE;
				String ext = ApplicationUtil.getExtensionLower(fatt.getNomeFileS());
				String progressivoU = (progressivo+1)+ impostazioneDao.getPreamboloEstratto();
				String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.FATTURA.stampa(), estratto.getUser().getDenominazione()) + "." + ext;

				OutputStream out = downloadBean.outputStream();
				downloadBean.filename(file);
				ExcelFatture.stampaExcelFatturaEstratto(estratto, progressivoU, out, s);
				downloadBean.close();
			}
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaExcelFatturaEstratto(List<Estratto> estratti, DownloadBean downloadBean) {
		List<Allegato> files = new ArrayList<Allegato>();
		if(ApplicationUtil.notEmpty(estratti)) {
			String pe = impostazioneDao.getPreamboloEstratto();
			Iterator<Estratto> objIter = estratti.iterator();
			String sE = ConstantsGlobal.secretE;
			String sZ = ConstantsGlobal.secretZ;
			while(objIter.hasNext()) {
				Estratto estratto = objIter.next();
				try {
					FatturaEstratto fatt = estratto.getUser().getFattura();
					if(fatt!=null) {
						int progressivo = estrattoDao.contaFatturePerEstrattoAnno(estratto);
						String ext = ApplicationUtil.getExtensionLower(fatt.getNomeFileS());
						String progressivoU = (progressivo+1) + pe;
						String file = ApplicationUtil.reportFile(estratto.getData_estratto(), ConstantsLanguage.FATTURA.stampa(), estratto.getUser().getDenominazione(), estratto.getId(), progressivoU) + "." + ext;
						ByteArrayOutputStream fi = new ByteArrayOutputStream();
						ExcelFatture.stampaExcelFatturaEstratto(estratto, progressivoU, fi, sE);
						fi.flush();
						fi.close();
						ApplicationUtil.addFile(files, file, fi.toByteArray(), estratto.getData_creazione().getTime());
					}
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				}
			}
			try {
				if (ApplicationUtil.empty(files)) {
					ApplicationUtil.fileWriter(files, ConstantsLanguage.NO_FATTURA.stampa(), ConstantsLanguage.FATTURA_NON_PRESENTE.stampa());
				}

				String fileName = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.FATTURE.stampa()) + "." + ConstantsMap.zip;
				OutputStream out = downloadBean.outputStream();
				downloadBean.filename(fileName, files);
				ApplicationUtil.zip(null, files, out, sZ);
				downloadBean.close();
			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			} 
		}
	}
	public void esportaPdfEstrattoDettagli(Estratto estratto,  DownloadBean downloadBean) {
		List<EstrattoDettaglio>  dettagli = estrattoDao.trovaDettagli(estratto.getId());
		try {
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.ESTRATTI_DETTAGLIO.stampa(), estratto.getUser().getDenominazione()) + "." +ConstantsMap.pdf;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, dettagli);
			String s = ConstantsGlobal.secretP;
			PDFWriter.stampaPDFDettagli(dettagli, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaPdfEstrattoDettagli(Estratto estratto, List<EstrattoDettaglio>  dettagli,  DownloadBean downloadBean) {
		try {
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.ESTRATTI_DETTAGLIO.stampa(), estratto.getUser().getDenominazione()) + "." +ConstantsMap.pdf;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, dettagli);
			String s = ConstantsGlobal.secretP;
			PDFWriter.stampaPDFDettagli(dettagli, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaPdfEstrattoDettagliAnteprima(Estratto estratto, List<EstrattoDettaglio>  dettagli,  DownloadBean downloadBean) {
		try {
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.ESTRATTI_DETTAGLIO.stampa(), estratto.getUser().getDenominazione()) + "." +ConstantsMap.pdf;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, dettagli);
			String s = ConstantsGlobal.secretP;
			PDFWriter.stampaPDFDettagliAnteprima(dettagli, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaPdfOpposizione(List<Opposizione>  opposizioni, DownloadBean downloadBean) {
		try {
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.OPPOSIZIONI.stampa())+ "." +ConstantsMap.pdf;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, opposizioni);
			String s = ConstantsGlobal.secretP;
			PDFWriter.stampaPDFOpposizione(opposizioni, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaPdfPrenotazione(Utente user, List<Prenotazione>  prenotazioni, DownloadBean downloadBean) {
		try {
			String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.PRENOTAZIONE.stampa() +": " + ApplicationUtil.size(prenotazioni), prenotazioni);
			operazioneDao.creaOperazione(ConstantsMap.LOG4,p ,user, ConstantsMap.OPERAZIONE_EXPORT_PRENOTAZIONE);
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.PRENOTAZIONI.stampa())+ "." +ConstantsMap.pdf;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, prenotazioni);
			String s = impostazioneDao.getPasswordPDF();
			PDFWriter.stampaPDFPrenotazione(prenotazioni, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaPdfUtente(List<Utente>  utenti, DownloadBean downloadBean) {
		try {
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.UTENTI.stampa())+ "." +ConstantsMap.pdf;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, utenti);
			String s = ConstantsGlobal.secretP;
			PDFWriter.stampaPDFUser(utenti, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void scaricaPdfAgenteAppuntamentoList(Boolean storia_operatore, Utente user, Boolean ps, Boolean supervisore, Boolean storia_agente, List<Appuntamento> appuntamenti, DownloadBean downloadBean) {
		String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.APPUNTAMENTO.stampa() +": " + ApplicationUtil.size(appuntamenti), appuntamenti);
		operazioneDao.creaOperazione(ConstantsMap.LOG4,p ,user,  ConstantsMap.OPERAZIONE_EXPORT_APPUNTAMENTO);
		List<Allegato> files = new ArrayList<Allegato>();
		if(ApplicationUtil.notEmpty(appuntamenti)) {
			String pp = ConstantsGlobal.secretL;
			String s = impostazioneDao.getPasswordZIP();
			String html = impostazioneDao.getFormatoDocumento();
			String extM = (pp == null || (html.equalsIgnoreCase(ConstantsMap.pdf))) ? html : ConstantsMap.zip;
			Iterator<Appuntamento> objAppIter = appuntamenti.iterator();
			while(objAppIter.hasNext()) {
				Appuntamento appuntamento = objAppIter.next();
				try {
					String nomeFile = ApplicationUtil.reportFile(appuntamento.getData_inizio(), ConstantsLanguage.APPUNTAMENTO.stampa(), appuntamento.getId(), appuntamento.getContatto().getAnagrafica().getDenominazione())  + "." + extM;
					ByteArrayOutputStream result;
					if(ApplicationUtil.isTrue(storia_agente)) {
						result = appuntamentoToHTMLString(user, appuntamento,  storia_operatore, supervisore,  html, ps, false, pp);
					} else {
						boolean vs = appuntamento.getAgente() != null && ApplicationUtil.isTrue(appuntamento.getAgente().getVisualizzaStoria());
						result = appuntamentoToHTMLString(user, appuntamento,  vs, supervisore, html, ps, false, pp);
					}
					result.close();
					ApplicationUtil.addFile(files, nomeFile, result.toByteArray(), appuntamento.getData_creazione().getTime());
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				}
			}
			if (ApplicationUtil.notEmpty(files)) {
				try {

					String fileName = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.APPUNTAMENTI.stampa()) + "." + ConstantsMap.zip;
					OutputStream out = downloadBean.outputStream();
					downloadBean.filename(fileName, files);
					ApplicationUtil.zip(null, files, out, s);
					downloadBean.close();
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				}
			} 
		}
	}
	public void scaricaPdfAppuntamentoConcatena(Boolean storia_operatore, Utente user, Boolean ps, Boolean supervisore, Boolean storia_agente, List<Appuntamento> appuntamenti, DownloadBean downloadBean) {
		String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.APPUNTAMENTO.stampa() +": " + ApplicationUtil.size(appuntamenti), appuntamenti);
		operazioneDao.creaOperazione(ConstantsMap.LOG4,p ,user, ConstantsMap.OPERAZIONE_EXPORT_APPUNTAMENTO);
		List<Allegato> files = new ArrayList<Allegato>();
		if(ApplicationUtil.notEmpty(appuntamenti)) {
			String html = ConstantsMap.pdf;
			String s = impostazioneDao.getPasswordLCK();
			String pp = null;
			Iterator<Appuntamento> objAppIter = appuntamenti.iterator();
			while(objAppIter.hasNext()) {
				Appuntamento appuntamento = objAppIter.next();
				try {
					String nomeFile = ApplicationUtil.reportFile(appuntamento.getData_inizio(), ConstantsLanguage.APPUNTAMENTO.stampa(), appuntamento.getId(), appuntamento.getContatto().getAnagrafica().getDenominazione())  + "." + html;
					ByteArrayOutputStream result;
					if(ApplicationUtil.isTrue(storia_agente)) {
						result = appuntamentoToHTMLString(user, appuntamento,  storia_operatore,  supervisore, html, ps, false, pp);
					} else {
						boolean vs = appuntamento.getAgente() != null && ApplicationUtil.isTrue(appuntamento.getAgente().getVisualizzaStoria());
						result = appuntamentoToHTMLString(user, appuntamento,  vs, supervisore,  html, ps, false, pp);
					}
					result.close();
					ApplicationUtil.addFile(files, nomeFile, result.toByteArray(), appuntamento.getData_creazione().getTime());
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				}
			}
			if (ApplicationUtil.notEmpty(files)) {
				try {

					String fileName = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.APPUNTAMENTI.stampa()) + "." + html;
					OutputStream out = downloadBean.outputStream();
					downloadBean.filename(fileName, files);
					String w = getWatermark();
					ApplicationUtil.concatenaPDF(w, files, out, s);
					downloadBean.close();
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				}
			} 
		}
	}
	
	public String getWatermark() {
		return ApplicationUtil.trimToNull(ConstantsGlobal.watermark);
	}
	public void esportaPdfAppuntamentoScheda(Utente utente, Boolean p, Appuntamento appuntamento, Boolean storia, Boolean supervisore, DownloadBean downloadBean) {
		String pg = ApplicationUtil.asString(ConstantsLanguage.APPUNTAMENTO.stampa(), appuntamento);
		operazioneDao.creaOperazione(ConstantsMap.LOG4, pg ,utente, ConstantsMap.OPERAZIONE_EXPORT_APPUNTAMENTO);
		String html = impostazioneDao.getFormatoDocumento();
		String s = ConstantsGlobal.secretL;
		String extM = (s == null || (html.equalsIgnoreCase(ConstantsMap.pdf))) ? html : ConstantsMap.zip;

		String nome = ApplicationUtil.reportFile(appuntamento.getData_inizio(), ConstantsLanguage.APPUNTAMENTO.stampa(), appuntamento.getContatto().getAnagrafica().getDenominazione())  + "." + extM;
		OutputStream out = downloadBean.outputStream();
		downloadBean.filename(nome);
		try {
			ByteArrayOutputStream result = appuntamentoToHTMLString(utente, appuntamento,  storia, supervisore, html, p, false, s);
			result.close();
			out.write(result.toByteArray());
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaPdfContratto(Utente utente, Boolean p, Contratto contratto, Boolean storia, Boolean supervisore, DownloadBean downloadBean) {
		String pg = ApplicationUtil.asString(ConstantsLanguage.CONTRATTO.stampa(), contratto);
		operazioneDao.creaOperazione(ConstantsMap.LOG4, pg ,utente, ConstantsMap.OPERAZIONE_EXPORT_CONTRATTO);
		String html = impostazioneDao.getFormatoDocumento();
		String s = ConstantsGlobal.secretL;
		String extM = (s == null || (html.equalsIgnoreCase(ConstantsMap.pdf))) ? html : ConstantsMap.zip;

		String nome = ApplicationUtil.reportFile(contratto.getData_creazione(), ConstantsLanguage.CONTRATTO.stampa(), contratto.getContatto().getAnagrafica().getDenominazione())  + "." + extM;
		OutputStream out = downloadBean.outputStream();
		downloadBean.filename(nome);
		try {
			ByteArrayOutputStream result = contrattoToHTMLString(utente, contratto,  storia, supervisore, html, p, false, s);
			result.close();
			out.write(result.toByteArray());
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaPdfAnagrafica(Anagrafica anagrafica, DownloadBean downloadBean) {
		String html = impostazioneDao.getFormatoDocumento();
		String s = ConstantsGlobal.secretL;
		String extM = (s == null || (html.equalsIgnoreCase(ConstantsMap.pdf))) ? html : ConstantsMap.zip;

		String nome = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.ANAGRAFICA.stampa(), anagrafica.getDenominazione())  + "." + extM;
		OutputStream out = downloadBean.outputStream();
		downloadBean.filename(nome);
		try {
			ByteArrayOutputStream result = anagraficaToHTMLString(anagrafica, html, null, s);
			result.close();
			out.write(result.toByteArray());
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaPdfContatto(Utente utente, Boolean p, Contatto contatto, Boolean storia, Boolean supervisore, DownloadBean downloadBean) {
		String pg = ApplicationUtil.asString(ConstantsLanguage.CONTATTO.stampa(), contatto);
		operazioneDao.creaOperazione(ConstantsMap.LOG4, pg ,utente, ConstantsMap.OPERAZIONE_EXPORT_CONTATTO);
		String html =  impostazioneDao.getFormatoDocumento();
		ByteArrayOutputStream os = new ByteArrayOutputStream();
		UtilOffice uo = impostazioneDao.getUtilOffice();
		String result = contattoToHTMLString(utente, contatto, storia, supervisore, p);
		String s = ConstantsGlobal.secretL;
		String extM = (s == null || (html.equalsIgnoreCase(ConstantsMap.pdf))) ? html : ConstantsMap.zip;
		try {
			ApplicationUtil.convertM(result, os, html, uo, s);

			String nomeFile = ApplicationUtil.reportFile(contatto.getData_creazione(), ConstantsLanguage.CONTATTO.stampa(), contatto.getAnagrafica().getDenominazione()) +  "." + extM;
			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(nomeFile);
			os.close();
			out.write(os.toByteArray());
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaVCFAppuntamento(Utente user, List<Appuntamento> appuntamenti, DownloadBean downloadBean) {
		String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.APPUNTAMENTO.stampa() +": " + ApplicationUtil.size(appuntamenti), appuntamenti);
		operazioneDao.creaOperazione(ConstantsMap.LOG4,p ,user, ConstantsMap.OPERAZIONE_EXPORT_APPUNTAMENTO);
		List<Anagrafica> anagrafiche = new ArrayList<Anagrafica>();
		Iterator<Appuntamento> itApp = appuntamenti.iterator();
		while (itApp.hasNext()) {
			Appuntamento appuntamento = (Appuntamento) itApp.next();
			anagrafiche.add(appuntamento.getContatto().getAnagrafica());
		}
		esportaVCFAnagrafica(anagrafiche, downloadBean);
	}
	public void esportaVCFContatto(Utente user, List<Contatto> contatti, DownloadBean downloadBean) {
		String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.CONTATTO.stampa() +": " + ApplicationUtil.size(contatti), contatti);
		operazioneDao.creaOperazione(ConstantsMap.LOG4,p,user, ConstantsMap.OPERAZIONE_EXPORT_CONTATTO);
		List<Anagrafica> anagrafiche = new ArrayList<Anagrafica>();
		Iterator<Contatto> itApp = contatti.iterator();
		while (itApp.hasNext()) {
			Contatto contatto = (Contatto) itApp.next();
			anagrafiche.add(contatto.getAnagrafica());
		}
		esportaVCFAnagrafica(anagrafiche, downloadBean);
	}
	public void esportaVCFContratto(Utente user, List<Contratto> contratti, DownloadBean downloadBean) {
		String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.CONTRATTO.stampa() +": " + ApplicationUtil.size(contratti), contratti);
		operazioneDao.creaOperazione(ConstantsMap.LOG4, p ,user, ConstantsMap.OPERAZIONE_EXPORT_CONTRATTO);
		List<Anagrafica> anagrafiche = new ArrayList<Anagrafica>();
		Iterator<Contratto> itApp = contratti.iterator();
		while (itApp.hasNext()) {
			Contratto contratto = (Contratto) itApp.next();
			anagrafiche.add(contratto.getContatto().getAnagrafica());
		}
		esportaVCFAnagrafica(anagrafiche, downloadBean);
	}
	public void esportaVCFAnagrafica(List<Anagrafica> anagrafiche, DownloadBean downloadBean) {
		
		try {
			String appname = impostazioneDao.getApplicazione();
			String result = contactList(anagrafiche, appname);
			String s = impostazioneDao.getPasswordTXT();
			ByteArrayInputStream istream = new ByteArrayInputStream(result.getBytes(ConstantsGlobal.CHARSET));

			String nomeFile = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.ANAGRAFICHE.stampa())+ "." + (s != null ? ConstantsMap.zip : ConstantsMap.vcf);
			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(nomeFile, anagrafiche);
			if(s != null) {
				ByteArrayOutputStream bos = new ByteArrayOutputStream();
				ApplicationUtil.copy(istream, bos);
				ZipPassword.zip(null, bos.toByteArray(), out, ConstantsMap.vcf, s);
				bos.close();
			} else {
				ApplicationUtil.copy(istream, out);
			}
			downloadBean.close();
			istream.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaXmlAnagrafica(List<Anagrafica> anagrafiche, DownloadBean downloadBean) {
		List<AnagraficaXML> anagraficheXML = anagraficaDao.convertToAnagrafiche(anagrafiche);
		try {
			String s = impostazioneDao.getPasswordTXT();
			ByteArrayOutputStream result = new ByteArrayOutputStream();
			ApplicationUtil.xmlEncoder(anagraficheXML, result);
			result.close();
			ByteArrayInputStream istream = new ByteArrayInputStream(result.toByteArray());

			String nomeFile = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.ANAGRAFICHE.stampa()) +  "." +  (s != null ? ConstantsMap.zip : ConstantsMap.xml);
			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(nomeFile, anagrafiche);
			if(s != null) {
				ByteArrayOutputStream bos = new ByteArrayOutputStream();
				ApplicationUtil.copy(istream, bos);
				ZipPassword.zip(null, bos.toByteArray(), out, ConstantsMap.xml, s);
				bos.close();
			} else {
				ApplicationUtil.copy(istream, out);
			}
			downloadBean.close();
			istream.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaJSONAnagrafica(List<Anagrafica> anagrafiche, DownloadBean downloadBean) {
		List<AnagraficaXML> anagraficheXML = anagraficaDao.convertToAnagrafiche(anagrafiche);
		try {
			String s = impostazioneDao.getPasswordTXT();
			ByteArrayOutputStream result = new ByteArrayOutputStream();
			ApplicationUtil.jsonEncoder(anagraficheXML, result);
			result.close();
			ByteArrayInputStream istream = new ByteArrayInputStream(result.toByteArray());

			String nomeFile = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.ANAGRAFICHE.stampa()) +  "." +  (s != null ? ConstantsMap.zip : ConstantsMap.json);
			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(nomeFile, anagrafiche);
			if(s != null) {
				ByteArrayOutputStream bos = new ByteArrayOutputStream();
				ApplicationUtil.copy(istream, bos);
				ZipPassword.zip(null, bos.toByteArray(), out, ConstantsMap.json, s);
				bos.close();
			} else {
				ApplicationUtil.copy(istream, out);
			}
			downloadBean.close();
			istream.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaQRContatto(Utente utente, Contatto contatto, DownloadBean downloadBean) {
		String pg = ApplicationUtil.asString(ConstantsLanguage.CONTATTO.stampa(), contatto);
		operazioneDao.creaOperazione(ConstantsMap.LOG4, pg ,utente, ConstantsMap.OPERAZIONE_EXPORT_CONTATTO);
		esportaQRAnagrafica(contatto.getAnagrafica(), downloadBean);
	}
	public void esportaQRContratto(Utente utente, Contratto contratto, DownloadBean downloadBean) {
		String pg = ApplicationUtil.asString(ConstantsLanguage.CONTRATTO.stampa(), contratto);
		operazioneDao.creaOperazione(ConstantsMap.LOG4, pg ,utente, ConstantsMap.OPERAZIONE_EXPORT_CONTRATTO);
		esportaQRAnagrafica(contratto.getContatto().getAnagrafica(), downloadBean);
	}
	public void esportaQRAnagrafica(Anagrafica anagrafica, DownloadBean downloadBean) {
		try {
			
			String appname = impostazioneDao.getApplicazione();
			String result = VCardUtils.vcf(anagrafica, appname);
			String nomeFile = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.ANAGRAFICA.stampa(), anagrafica.getDenominazione()) +  "." + ConstantsMap.pdf;
			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(nomeFile);
			String s = ConstantsGlobal.secretL;
			ApplicationUtil.createPdf(result, null,  out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaQRAppuntamento(Utente utente, Appuntamento appuntamento, DownloadBean downloadBean) {
		String pg = ApplicationUtil.asString(ConstantsLanguage.APPUNTAMENTO.stampa(), appuntamento);
		operazioneDao.creaOperazione(ConstantsMap.LOG4, pg ,utente, ConstantsMap.OPERAZIONE_EXPORT_APPUNTAMENTO);
		try {
			String appname = impostazioneDao.getApplicazione(); 
			String link = impostazioneDao.getUrlBase();
			String resultA = VCardUtils.vcf(appuntamento.getContatto().getAnagrafica(), appname);
			String resultB = VCardUtils.ical(Arrays.asList(appuntamento), null, null, appname, link, true);
			String nomeFile = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.APPUNTAMENTO.stampa(), appuntamento.getContatto().getAnagrafica().getDenominazione()) +  "." + ConstantsMap.pdf;
			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(nomeFile);
			String s = ConstantsGlobal.secretL;
			ApplicationUtil.createPdf(resultA, resultB,  out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	
	
	public String contactList(List<Anagrafica> anagrafiche, String appname) {
		StringBuilder sb = new StringBuilder();
		if(ApplicationUtil.notEmpty(anagrafiche)) {
			Iterator<Anagrafica> objIter = anagrafiche.iterator();
			while(objIter.hasNext()) {
				Anagrafica anagrafica = objIter.next();
				sb.append(ApplicationUtil.trimToEmpty(VCardUtils.vcf(anagrafica, appname)));
			}
		}
		return ApplicationUtil.trimToEmpty(sb.toString());
	}

	
	public void esportaICSAppuntamento(Utente utente, Appuntamento appuntamento, DownloadBean downloadBean) {
		String pg = ApplicationUtil.asString(ConstantsLanguage.APPUNTAMENTO.stampa(), appuntamento);
		operazioneDao.creaOperazione(ConstantsMap.LOG4, pg ,utente, ConstantsMap.OPERAZIONE_EXPORT_APPUNTAMENTO);
		String appname = impostazioneDao.getApplicazione();
		String link = impostazioneDao.getUrlBase();
		String result = VCardUtils.ical(Arrays.asList(appuntamento), null, null, appname, link, true);
		try {
			ByteArrayInputStream istream = new ByteArrayInputStream(result.getBytes(ConstantsGlobal.CHARSET));

			String nomeFile = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.APPUNTAMENTO.stampa(), appuntamento.getContatto().getAnagrafica().getDenominazione()) +  "." + ConstantsMap.ics;
			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(nomeFile);
			ApplicationUtil.copy(istream, out);
			downloadBean.close();
			istream.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaICSAppuntamentoList(Utente user, List<Appuntamento> appuntamenti, DownloadBean downloadBean) {
		String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.APPUNTAMENTO.stampa() +": " + ApplicationUtil.size(appuntamenti), appuntamenti);
		operazioneDao.creaOperazione(ConstantsMap.LOG4,p ,user,  ConstantsMap.OPERAZIONE_EXPORT_APPUNTAMENTO);
		String appname = impostazioneDao.getApplicazione();
		String link = impostazioneDao.getUrlBase();
		String result = VCardUtils.ical(appuntamenti, null, null, appname, link, true);
		try {
			String s = impostazioneDao.getPasswordTXT();
			ByteArrayInputStream istream = new ByteArrayInputStream(result.getBytes(ConstantsGlobal.CHARSET));

			String nomeFile = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.APPUNTAMENTI.stampa())+  "." + (s != null ? ConstantsMap.zip : ConstantsMap.ics);
			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(nomeFile, appuntamenti);
			if(s != null) {
				ByteArrayOutputStream bos = new ByteArrayOutputStream();
				ApplicationUtil.copy(istream, bos);
				ZipPassword.zip(null, bos.toByteArray(), out, ConstantsMap.ics, s);
				bos.close();
			} else {
				ApplicationUtil.copy(istream, out);
			}
			downloadBean.close();
			istream.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	
	@SuppressWarnings("unchecked")
	public List<Contatto> getContattiFuturiICS(Long id, String password) {
		Date ora = ApplicationUtil.date();
		Boolean ts = impostazioneDao.getEstrazioneStato();
		DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class);
		criteria.add(Restrictions.eq("tipo", ConstantsGlobal.inviato));
		DetachedCriteria campagnaC = criteria.createCriteria("campagna");
		ApplicationUtil.criteriaStato(campagnaC, false, ApplicationUtil.isTrue(ts));
		Boolean s = impostazioneDao.getEstrazioneListe();
		if(ApplicationUtil.isTrue(s)) {
			DetachedCriteria listaC = criteria.createCriteria("lista");
			ApplicationUtil.criteriaStato(listaC, false, ApplicationUtil.isTrue(ts));
		}
		criteria.add(Restrictions.ge("data_presentazione", ora));
		DetachedCriteria criteriaUA = criteria.createCriteria("proprietario");
		criteriaUA.add(Restrictions.idEq(id));
		criteriaUA.add(Restrictions.eq("password",password));
		criteriaUA.add(Restrictions.isNotNull("interno.id"));
		return getList(criteria);
		
	}
	
	@SuppressWarnings("unchecked")
	public List<Appuntamento> getAppuntamentiICS(Long id, String password) {
		DetachedCriteria criteriaA = DetachedCriteria.forClass(Appuntamento.class);
		criteriaA.add(Restrictions.gt("data_inizio",  ApplicationUtil.date()));
		DetachedCriteria criteriaC = criteriaA.createCriteria("esito");
		criteriaC.add(Restrictions.eq("confermato", Boolean.TRUE));
		DetachedCriteria criteriaUA = criteriaA.createCriteria("agente", ApplicationUtil.leftJoin()).createCriteria("user");
		criteriaUA.add(Restrictions.idEq(id));
		criteriaUA.add(Restrictions.eq("password",password));
		List<Appuntamento> appuntamenti = getList(criteriaA);
		return appuntamenti;
	}
	@SuppressWarnings("unchecked")
	public List<Nota> getNoteICS(Long id, String password) {
		DetachedCriteria criteriaN = DetachedCriteria.forClass(Nota.class);
		criteriaN.add(Restrictions.gt("data_inizio",  ApplicationUtil.date()));
		DetachedCriteria criteriaUN = criteriaN.createCriteria("agente").createCriteria("user");
		criteriaUN.add(Restrictions.idEq(id));
		criteriaUN.add(Restrictions.eq("password",password));
		List<Nota> note = getList(criteriaN);
		return note;
	}
	public String scaricaICSAppuntamentoList(Long id, String password, boolean publish) {
		List<Appuntamento> appuntamenti = getAppuntamentiICS(id, password);
		List<Nota> note = getNoteICS(id, password);
		String appname = impostazioneDao.getApplicazione();
		String link = impostazioneDao.getUrlBase();
		return  VCardUtils.ical(appuntamenti, note, null, appname, link, publish);
	}
	public String scaricaICSContattoList(Long id, String password, boolean publish) {
		List<Contatto> contatti = getContattiFuturiICS(id, password);
		String appname = impostazioneDao.getApplicazione();
		String link = impostazioneDao.getUrlBase();
		return  VCardUtils.ical(null, null, contatti, appname, link, publish);
	}
	
	
	public void esportaVCFContatto(Utente utente, Contatto contatto, DownloadBean downloadBean) {
		String pg = ApplicationUtil.asString(ConstantsLanguage.CONTATTO.stampa(), contatto);
		operazioneDao.creaOperazione(ConstantsMap.LOG4, pg ,utente, ConstantsMap.OPERAZIONE_EXPORT_CONTATTO);
		esportaVCFAnagrafica(contatto.getAnagrafica(), downloadBean);
	}
	public void esportaVCFContratto(Utente utente, Contratto contratto, DownloadBean downloadBean) {
		String pg = ApplicationUtil.asString(ConstantsLanguage.CONTRATTO.stampa(), contratto);
		operazioneDao.creaOperazione(ConstantsMap.LOG4, pg ,utente, ConstantsMap.OPERAZIONE_EXPORT_CONTRATTO);
		esportaVCFAnagrafica(contratto.getContatto().getAnagrafica(), downloadBean);
	}
	public void esportaVCFAppuntamento(Utente utente, Appuntamento appuntamento, DownloadBean downloadBean) {
		String pg = ApplicationUtil.asString(ConstantsLanguage.APPUNTAMENTO.stampa(), appuntamento);
		operazioneDao.creaOperazione(ConstantsMap.LOG4, pg ,utente, ConstantsMap.OPERAZIONE_EXPORT_APPUNTAMENTO);
		esportaVCFAnagrafica(appuntamento.getContatto().getAnagrafica(), downloadBean);
	}
	public void esportaVCFAnagrafica(Anagrafica anagrafica, DownloadBean downloadBean) {
		try {
			String appname = impostazioneDao.getApplicazione();
			String result = VCardUtils.vcf(anagrafica, appname);
			ByteArrayInputStream istream = new ByteArrayInputStream(result.getBytes(ConstantsGlobal.CHARSET));

			String nomeFile = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.ANAGRAFICA.stampa(), anagrafica.getDenominazione()) +  "." + ConstantsMap.vcf;
			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(nomeFile);
			ApplicationUtil.copy(istream, out);
			downloadBean.close();
			istream.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}

	public ByteArrayOutputStream appuntamentoToHTMLString(Utente utente, Appuntamento appuntamento, Boolean storia, Boolean supervisore, String format, Boolean p, boolean nuovo, String s) {
		ByteArrayOutputStream os = new ByteArrayOutputStream();
		try {
			UtilOffice uo = impostazioneDao.getUtilOffice();
			String result = appuntamentoToHTMLString(utente, appuntamento, storia, supervisore, p, nuovo);
			ApplicationUtil.convertM(result, os, format, uo, s);
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
		return os;
	}
	
	
	public String appuntamentoToHTMLString(Utente utente, Appuntamento appuntamento, Boolean storia, Boolean supervisore, Boolean p, boolean nuovo) {
		Map<String, Object> model = new HashMap<String, Object>();
		model.put("mappa", loadValori(appuntamento));
		model.put("schedaAppuntamento", impostazioneDao.getDocumentoDettagliAppuntamento());
		model.put("schedaContatto", impostazioneDao.getDocumentoDettagliContatto());
		model.put("schedaAnagrafica", impostazioneDao.getDocumentoDettagliAnagrafica());
		model.put("schedaIndirizzoLegale", impostazioneDao.getDocumentoDettagliIndirizzoLegale());
		model.put("schedaIndirizzoOperativo", impostazioneDao.getDocumentoDettagliIndirizzoOperativo());
		model.put("schedaConsenso", impostazioneDao.getDocumentoDettagliConsenso());
		model.put("visibile", new ElementoScheda(impostazioneDao.getDocumentoSchedaAppuntamento()));
		model.put("appuntamento", appuntamento);
		model.put("nuovo", nuovo);
		boolean storico = (utente == null) || (utente != null && utente.getVisualizzaStorico());
		model.put("dettagliAnagrafica",  pannelloDao.getDatiAggiuntiviAnagrafica(appuntamento.getContatto().getAnagrafica().getId(), p));
		model.put("storiaAppuntamento",  storico && storia ? pannelloDao.getStoriaAppuntamento(appuntamento) : null);
		model.put("storiaContatto",  storico && storia ? pannelloDao.getStoriaContatto(appuntamento.getContatto(), supervisore) : null);
		model.put("raggruppa", impostazioneDao.getRaggruppaCampoAggiuntivo());
		return mergeVelocityTemplate("appuntamento", model);
	}
	
	public ByteArrayOutputStream contrattoToHTMLString(Utente utente, Contratto contratto, Boolean storia, Boolean supervisore, String format, Boolean p, boolean nuovo, String s) {
		ByteArrayOutputStream os = new ByteArrayOutputStream();
		try {
			UtilOffice uo = impostazioneDao.getUtilOffice();
			String result = contrattoToHTMLString(utente, contratto, storia, supervisore, p, nuovo);
			ApplicationUtil.convertM(result, os, format, uo, s);
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
		return os;
	}
	
	public String contrattoToHTMLString(Utente utente, Contratto contratto, Boolean storia, Boolean supervisore, Boolean p, boolean nuovo) {
		Map<String, Object> model = new HashMap<String, Object>();
		model.put("mappa", loadValori(contratto));
		model.put("schedaContratto", impostazioneDao.getDocumentoDettagliContratto());
		model.put("schedaContatto", impostazioneDao.getDocumentoDettagliContatto());
		model.put("schedaAnagrafica", impostazioneDao.getDocumentoDettagliAnagrafica());
		model.put("schedaIndirizzoLegale", impostazioneDao.getDocumentoDettagliIndirizzoLegale());
		model.put("schedaIndirizzoOperativo", impostazioneDao.getDocumentoDettagliIndirizzoOperativo());
		model.put("schedaConsenso", impostazioneDao.getDocumentoDettagliConsenso());
		model.put("visibile", new ElementoScheda(impostazioneDao.getDocumentoSchedaContratto()));
		model.put("contratto", contratto);
		model.put("nuovo", nuovo);
		boolean storico = (utente == null) || (utente != null && utente.getVisualizzaStorico());
		model.put("dettagliAnagrafica",  pannelloDao.getDatiAggiuntiviAnagrafica(contratto.getContatto().getAnagrafica().getId(), p));
		model.put("dettagliContratto",  pannelloDao.getDatiAggiuntiviContratto(contratto.getId(), p));
		model.put("storiaContratto",  storico && storia ? contrattoDao.getStoriaContratto(contratto) : null);
		model.put("storiaContatto",  storico && storia ? pannelloDao.getStoriaContatto(contratto.getContatto(), supervisore) : null);
		model.put("raggruppa", impostazioneDao.getRaggruppaCampoAggiuntivo());
		return mergeVelocityTemplate("contratto", model);
	}
	
	public String contattoToHTMLString(Utente utente, Contatto contatto, Boolean storia, Boolean supervisore, Boolean p) {
		Map<String, Object> model = new HashMap<String, Object>();
		model.put("mappa", loadValori(contatto));
		model.put("schedaContatto", impostazioneDao.getDocumentoDettagliContatto());
		model.put("schedaAnagrafica", impostazioneDao.getDocumentoDettagliAnagrafica());
		model.put("schedaIndirizzoLegale", impostazioneDao.getDocumentoDettagliIndirizzoLegale());
		model.put("schedaIndirizzoOperativo", impostazioneDao.getDocumentoDettagliIndirizzoOperativo());
		model.put("schedaConsenso", impostazioneDao.getDocumentoDettagliConsenso());
		model.put("visibile", new ElementoScheda(impostazioneDao.getDocumentoSchedaContatto()));
		model.put("contatto", contatto);
		boolean storico = (utente == null) || (utente != null && utente.getVisualizzaStorico());
		model.put("dettagliAnagrafica",  pannelloDao.getDatiAggiuntiviAnagrafica(contatto.getAnagrafica().getId(), p));
		model.put("storiaContatto", storico && storia ? pannelloDao.getStoriaContatto(contatto, supervisore) : null);
		model.put("raggruppa", impostazioneDao.getRaggruppaCampoAggiuntivo());
		return mergeVelocityTemplate("contatto", model);
	}
	public ByteArrayOutputStream contattoToHTMLStream(Utente utente, Contatto contatto, Boolean storia, Boolean supervisore, String format, Boolean p, boolean nuovo, String s) {
		ByteArrayOutputStream os = new ByteArrayOutputStream();
		try {
			String result = contattoToHTMLString(utente, contatto, storia, supervisore, p);
			UtilOffice uo = impostazioneDao.getUtilOffice();
			ApplicationUtil.convertM(result, os, format, uo, s);
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
		return os;
	}
	public ByteArrayOutputStream anagraficaToHTMLString(Anagrafica anagrafica, String format, Boolean p, String s) {
		ByteArrayOutputStream os = new ByteArrayOutputStream();
		try {
			UtilOffice uo = impostazioneDao.getUtilOffice();
			String result = anagraficaToHTMLString(anagrafica, p, s);
			ApplicationUtil.convertM(result, os, format, uo, s);
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
		return os;
	}
	
	public String anagraficaToHTMLString(Anagrafica anagrafica, Boolean p, String s) {
		Map<String, Object> model = new HashMap<String, Object>();
		model.put("mappa", loadValori(anagrafica));
		model.put("schedaAnagrafica", impostazioneDao.getDocumentoDettagliAnagrafica());
		model.put("schedaIndirizzoLegale", impostazioneDao.getDocumentoDettagliIndirizzoLegale());
		model.put("schedaIndirizzoOperativo", impostazioneDao.getDocumentoDettagliIndirizzoOperativo());
		model.put("schedaConsenso", impostazioneDao.getDocumentoDettagliConsenso());
		model.put("visibile", new ElementoScheda(impostazioneDao.getDocumentoSchedaAnagrafica()));
		model.put("anagrafica", anagrafica);
		model.put("dettagliAnagrafica",  pannelloDao.getDatiAggiuntiviAnagrafica(anagrafica.getId(), p));
		model.put("raggruppa", impostazioneDao.getRaggruppaCampoAggiuntivo());
		return mergeVelocityTemplate("anagrafica", model);
	}
	
	public String sondaggioToHTMLString(Contatto contatto, List<DatoSondaggioAnagrafica> dati) {
		Map<String, Object> model = new HashMap<String, Object>();
		model.put("mappa", loadValori(contatto));
		model.put("dati", ApplicationUtil.convertiSondaggioEA(dati));
		model.put("etichetta", contatto.getAnagrafica().getDenominazione());
		model.put("raggruppa", impostazioneDao.getRaggruppaCampoAggiuntivo());
		return mergeVelocityTemplate("sondaggio", model);
	}
	public String pdaToHTMLString(ContrattoProdotto lsCrM) {
		Map<String, Object> model = new HashMap<String, Object>();
		model.put("mappa", loadValori(lsCrM.getContratto()));
		model.put("dati", ApplicationUtil.convertiSondaggioEC(lsCrM.getDatoSondaggioContratto()));
		model.put("contratto", lsCrM);
		EsitoContratto esito = null;
		Agente agente = null;
		if(ApplicationUtil.gtZero(lsCrM.getStatoId())) {
			esito = campagnaDao.getEsitoContratto(lsCrM.getStatoId());
		}
		if(ApplicationUtil.gtZero(lsCrM.getAgenteId())) {
			agente = campagnaDao.getAgente(lsCrM.getAgenteId());
		}
		model.put("esito", esito);
		model.put("agente", agente);
		model.put("raggruppa", impostazioneDao.getRaggruppaCampoAggiuntivo());
		return mergeVelocityTemplate("prodotto", model);
	}
	public String backofficeToHTMLString(Contatto contatto, Utente operatore) {
		Map<String, Object> model = new HashMap<String, Object>();
		model.put("mappa", loadValori(contatto));
		model.put("contatto", contatto);
		model.put("operatore", operatore);
		return mergeVelocityTemplate("backoffice", model);
	}
	public String notificaContattoToHTMLString(Contatto contatto, Utente operatore) {
		Map<String, Object> model = new HashMap<String, Object>();
		model.put("mappa", loadValori(contatto));
		model.put("contatto", contatto);
		model.put("operatore", operatore);
		return mergeVelocityTemplate("notifica_contatto", model);
	}
	
	public String notificaContrattoToHTMLString(Contratto contratto, Utente operatore) {
		Map<String, Object> model = new HashMap<String, Object>();
		model.put("mappa", loadValori(contratto));
		model.put("contratto", contratto);
		model.put("operatore", operatore);
		return mergeVelocityTemplate("notifica_contratto", model);
	}
	public String notificaAppuntamentoToHTMLString(Appuntamento appuntamento, Utente operatore) {
		Map<String, Object> model = new HashMap<String, Object>();
		model.put("mappa", loadValori(appuntamento));
		model.put("appuntamento", appuntamento);
		model.put("operatore", operatore);
		return mergeVelocityTemplate("notifica_appuntamento", model);
	}
	public String getMessaggioTemplate(Messaggio messaggio, List<Destinatario> destinatari, List<AllegatoMessaggio> allegati) {
		Map<String, Object> model = new HashMap<String, Object>();
		model.put("messaggio", messaggio);
		model.put("destinatari", ApplicationUtil.prendiDestinatarioMessaggi(destinatari));
		model.put("allegati", ApplicationUtil.prendiAllegatoMessaggi(allegati));
		return mergeVelocityTemplate("messaggio", model);
	}
	
	public String getMessaggioEmailTemplate(Messaggio messaggio, Utente user) {
		Map<String, Object> model = new HashMap<String, Object>();
		model.put("messaggio", messaggio);
		model.put("user", user);
		return mergeVelocityTemplate("invio", model);
	}
	public String reportToHTMLStringTeam(List<ReportContattiOdierniTeam> lsTeam) {
		Map<String, Object> model = new HashMap<String, Object>();
		model.put("team", lsTeam);
		return mergeVelocityTemplate("report_team", model);
	}
	public String reportToHTMLAnomalia(String anomalia) {
		Map<String, Object> model = new HashMap<String, Object>();
		model.put("anomalia", anomalia);
		return mergeVelocityTemplate("anomalia", model);
	}
	public String reportToHTMLStringZona(List<ReportContattiOdierniZona> lsZona) {
		Map<String, Object> model = new HashMap<String, Object>();
		model.put("zona", lsZona);
		return mergeVelocityTemplate("report_zona", model);
	}
	public String getRecupero(Utente user, String password) {
		Map<String, Object> model = new HashMap<String, Object>();
		model.put("user", user);
		model.put("password", password);
		model.put("tipo", password != null ? ConstantsLanguage.RECUPERO.stampa() : ConstantsLanguage.INVIO_OTP.stampa());
		return mergeVelocityTemplate("recupero", model);
	}
	public String getPrenotazione(Utente user, Date data, String password) {
		Map<String, Object> model = new HashMap<String, Object>();
		model.put("user", user);
		model.put("password", password);
		model.put("dataF", password);
		model.put("tipo", ConstantsLanguage.PRENOTAZIONE.stampa());
		return mergeVelocityTemplate("prenotazione", model);
	}
	public String getCodice(Utente user, String codice) {
		Map<String, Object> model = new HashMap<String, Object>();
		model.put("user", user);
		model.put("codice", codice);
		model.put("validita", ConstantsLanguage.VALIDITA.stampa() + ": " + ApplicationUtil.formatDate(ConstantsFormat.dateTime.stampa(), ApplicationUtil.addSeconds(ApplicationUtil.date(), ConstantsGlobal.secondi_otp)));
		return mergeVelocityTemplate("otp", model);
	}
	public String getRichiamo(String telefono, String destinazione) {
		Map<String, Object> model = new HashMap<String, Object>();
		model.put("telefono", telefono);
		model.put("destinazione", destinazione);
		return mergeVelocityTemplate("richiamo", model);
	}
	
	public String getWebhook(String nome, String body, String parameters) {
		Map<String, Object> model = new HashMap<String, Object>();
		model.put("nome", nome);
		model.put("body", body);
		model.put("parameters", parameters);
		return mergeVelocityTemplate("webhook", model);
	}
	
	public String getRicezione(String mittente, String destinatario, String oggetto, String testoH, String testoT) {
		Map<String, Object> model = new HashMap<String, Object>();
		model.put("mittente", mittente);
		model.put("destinatario", destinatario);
		model.put("oggetto", oggetto);
		model.put("html", testoH);
		model.put("text", testoT);
		return mergeVelocityTemplate("ricezione", model);
	}
	
	public String notificaLoginTemplate(Utente user, String ip, Date data) {
		Map<String, Object> model = new HashMap<String, Object>();
		model.put("user", user);
		model.put("ip", ip);
		model.put("orario", data);
		return mergeVelocityTemplate("login", model);
	}
	@SuppressWarnings("unchecked")
	public List<Appuntamento> getAppuntamentiContatto(Contatto contatto) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Appuntamento.class);
		DetachedCriteria criteriaCo = criteria.createCriteria("contatto");
		criteriaCo.add(Restrictions.idEq(contatto.getId()));
		criteria.addOrder(Order.asc("data_inizio"));
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Contratto> getContrattiContatto(Contatto contatto) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Contratto.class);
		DetachedCriteria criteriaCo = criteria.createCriteria("contatto");
		criteriaCo.add(Restrictions.idEq(contatto.getId()));
		criteria.addOrder(Order.asc("data_stipula"));
		return getList(criteria);
	}
	public Allegato allegaReportAppuntamentiZip(List<Appuntamento> appuntamenti,  Utente user, Contatto contatto, List<Long> appId, String secretZ, String secretL, boolean soloNuovi, Object [] suffix) {
		List<Allegato> files = new ArrayList<Allegato>();
		if(ApplicationUtil.notEmpty(appuntamenti)) {
			String html = impostazioneDao.getFormatoDocumento();
			String extM = (secretL == null || (html.equalsIgnoreCase(ConstantsMap.pdf))) ? html : ConstantsMap.zip;
			Iterator<Appuntamento> objIter = appuntamenti.iterator();
			while(objIter.hasNext()) {
				Appuntamento appuntamento = objIter.next();
				boolean nuovo = ApplicationUtil.notEmpty(appId) && appId.contains(appuntamento.getId());
				if((soloNuovi && nuovo) || !soloNuovi) {
					try {
						String n = ((!soloNuovi && nuovo) ? ConstantsLanguage.NUOVO.stampa() : "");
						String nomeFile = ApplicationUtil.reportFile(appuntamento.getData_inizio(), ConstantsLanguage.APPUNTAMENTO.stampa(), appuntamento.getId(),  contatto.getAnagrafica().getDenominazione(),  n) +  "." + extM;
						ByteArrayOutputStream result = appuntamentoToHTMLString(user, appuntamento,  true, true, html, null, !soloNuovi && nuovo, secretL);
						result.close();
						ApplicationUtil.addFile(files, nomeFile, result.toByteArray(), appuntamento.getData_creazione().getTime());
					} catch (Exception e) {
						ErrorLog.log(null, getClass(), e);
					}
				}
			}
			if (ApplicationUtil.notEmpty(files)) {
				try {
					String nomeFile = ApplicationUtil.reportFile(contatto.getData_lavorazione(), ConstantsLanguage.APPUNTAMENTI.stampa(), suffix) + "." + ConstantsMap.zip;
					ByteArrayOutputStream os = new ByteArrayOutputStream();
					boolean b = ApplicationUtil.zip(null,files, os, secretZ);
					os.close();
					Allegato p = new Allegato(nomeFile, os.toByteArray());
					if(b) {
						p.setData_modifica(ApplicationUtil.date());
					}
					return b ? p : null;
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				}
			}
		}
		return null;
	}
	public Allegato allegaReportContrattiZip(Utente utente, Contatto contatto, List<Long> crtId, String secretZ, String secretL, boolean soloNuovi, Object [] suffix) {
		List<Contratto> contratti = getContrattiContatto(contatto);
		List<Allegato> files = new ArrayList<Allegato>();
		if(ApplicationUtil.notEmpty(contratti)) {
			String html = impostazioneDao.getFormatoDocumento();
			String extM = (secretL == null || (html.equalsIgnoreCase(ConstantsMap.pdf))) ? html : ConstantsMap.zip;
			Iterator<Contratto> objIter = contratti.iterator();
			while(objIter.hasNext()) {
				Contratto contratto = objIter.next();
				boolean nuovo = ApplicationUtil.notEmpty(crtId) && crtId.contains(contratto.getId());
				if((soloNuovi && nuovo) || !soloNuovi) {
					try {
						String n = ((!soloNuovi && nuovo) ? ConstantsLanguage.NUOVO.stampa() : "");
						String nomeFile = ApplicationUtil.reportFile(contratto.getData_creazione(), ConstantsLanguage.CONTRATTO.stampa(), contratto.getId(), contatto.getAnagrafica().getDenominazione(), contratto.getProdotto().getNome(), n) +  "." + extM;
						ByteArrayOutputStream result = contrattoToHTMLString(utente, contratto,  true, true, html, null, !soloNuovi && nuovo, secretL);
						result.close();
						ApplicationUtil.addFile(files, nomeFile, result.toByteArray(), contratto.getData_creazione().getTime());
					} catch (Exception e) {
						ErrorLog.log(null, getClass(), e);
					}
				}
			}
			if (ApplicationUtil.notEmpty(files)) {
				try {
					String nomeFile = ApplicationUtil.reportFile(contatto.getData_lavorazione(), ConstantsLanguage.CONTRATTI.stampa(), suffix) + "." + ConstantsMap.zip;
					ByteArrayOutputStream os = new ByteArrayOutputStream();
					boolean b = ApplicationUtil.zip(null,files, os, secretZ);
					os.close();
					Allegato p = new Allegato(nomeFile, os.toByteArray());
					if(b) {
						p.setData_modifica(ApplicationUtil.date());
					}
					return b ? p : null;
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				}
			}
		}
		return null;
	}
	public void esportaExcelStoriaContatto(Utente user, List<StoriaContatto>  contatti, DownloadBean downloadBean) {
		try {
			String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.STORIA.stampa() + " " + ConstantsLanguage.CONTATTO.stampa() +": " + ApplicationUtil.size(contatti), contatti);
			operazioneDao.creaOperazione(ConstantsMap.LOG4, p ,user, ConstantsMap.OPERAZIONE_EXPORT_STORIA_CONTATTO);
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.STORIA.stampa(), ConstantsLanguage.CONTATTI.stampa())+ "." +ConstantsMap.xlsx;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, contatti);
			String s = impostazioneDao.getPasswordXLS();
			ExcelExport.esportaExcelStoriaContatto(contatti, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaPdfStoriaContatto(Utente user, List<StoriaContatto>  contatti, DownloadBean downloadBean) {
		try {
			String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.STORIA.stampa() + " " + ConstantsLanguage.CONTATTO.stampa() +": " + ApplicationUtil.size(contatti), contatti);
			operazioneDao.creaOperazione(ConstantsMap.LOG4, p ,user, ConstantsMap.OPERAZIONE_EXPORT_STORIA_CONTATTO);
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.STORIA.stampa(), ConstantsLanguage.CONTATTI.stampa())+ "." +ConstantsMap.pdf;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, contatti);
			String s = impostazioneDao.getPasswordPDF();
			PDFWriter.stampaPDFStoriaContatto(contatti, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaExcelStoriaAppuntamento(Utente user, List<StoriaAppuntamento> appuntamenti, DownloadBean downloadBean) {
		try {
			String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.STORIA.stampa() + " " + ConstantsLanguage.APPUNTAMENTO.stampa() +": " + ApplicationUtil.size(appuntamenti), appuntamenti);
			operazioneDao.creaOperazione(ConstantsMap.LOG4, p ,user, ConstantsMap.OPERAZIONE_EXPORT_STORIA_APPUNTAMENTO);
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.STORIA.stampa(), ConstantsLanguage.APPUNTAMENTI.stampa())+ "." +ConstantsMap.xlsx;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, appuntamenti);
			String s = impostazioneDao.getPasswordXLS();
			ExcelExport.esportaExcelStoriaAppuntamento(appuntamenti, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaPdfStoriaAppuntamento(Utente user, List<StoriaAppuntamento> appuntamenti, DownloadBean downloadBean) {
		try {
			String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.STORIA.stampa() + " " + ConstantsLanguage.APPUNTAMENTO.stampa() +": " + ApplicationUtil.size(appuntamenti), appuntamenti);
			operazioneDao.creaOperazione(ConstantsMap.LOG4, p ,user, ConstantsMap.OPERAZIONE_EXPORT_STORIA_APPUNTAMENTO);
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.STORIA.stampa(), ConstantsLanguage.APPUNTAMENTI.stampa())+ "." +ConstantsMap.pdf;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, appuntamenti);
			String s = impostazioneDao.getPasswordPDF();
			PDFWriter.stampaPDFStoriaAppuntamento(appuntamenti, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaExcelStoriaContratto(Utente user, List<StoriaContratto>  contratti, DownloadBean downloadBean) {
		try {
			String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.STORIA.stampa() + " " + ConstantsLanguage.CONTRATTO.stampa() +": " + ApplicationUtil.size(contratti), contratti);
			operazioneDao.creaOperazione(ConstantsMap.LOG4, p ,user, ConstantsMap.OPERAZIONE_EXPORT_STORIA_CONTRATTO);
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.STORIA.stampa(), ConstantsLanguage.CONTRATTI.stampa())+ "." +ConstantsMap.xlsx;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, contratti);
			String s = impostazioneDao.getPasswordXLS();
			ExcelExport.esportaExcelStoriaContratto(contratti, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaPdfStoriaContratto(Utente user, List<StoriaContratto>  contratti, DownloadBean downloadBean) {
		try {
			String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.STORIA.stampa() + " " + ConstantsLanguage.CONTRATTO.stampa() +": " + ApplicationUtil.size(contratti), contratti);
			operazioneDao.creaOperazione(ConstantsMap.LOG4, p ,user, ConstantsMap.OPERAZIONE_EXPORT_STORIA_CONTRATTO);
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.STORIA.stampa(), ConstantsLanguage.CONTRATTI.stampa())+ "." +ConstantsMap.pdf;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, contratti);
			String s = impostazioneDao.getPasswordPDF();
			PDFWriter.stampaPDFStoriaContratto(contratti, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	
	public void downloadClient(DownloadBean downloadBean) throws Exception  {
		OutputStream out = downloadBean.outputStream();
		String clientN = downloadClient(out);
		downloadBean.filename(clientN);
		downloadBean.close();
	}
		
	public String downloadClient(OutputStream out) throws Exception  {
		String nuo = impostazioneDao.getWebUrl();
		String home = impostazioneDao.getHome();
		return ApplicationUtil.downloadClient(out, nuo, home);
	}
	
	
	public String downloadInstaller(OutputStream out, String ext) throws Exception  {
		String nuo = impostazioneDao.getWebUrl();
		return ApplicationUtil.downloadInstaller(out, nuo, ext);
	}
	
	public void esportaPdfPausa(List<Pausa>  dati, DownloadBean downloadBean) {
		try {
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.PAUSE.stampa())+ "." +ConstantsMap.pdf;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, dati);
			String s = ConstantsGlobal.secretP;
			PDFWriter.stampaPDFPausa(dati, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaPdfMarcatura(List<Marcatura>  dati, DownloadBean downloadBean) {
		try {
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.MARCATURE.stampa())+ "." +ConstantsMap.pdf;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, dati);
			String s = ConstantsGlobal.secretP;
			PDFWriter.stampaPDFMarcatura(dati, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaPdfOperazione(List<Operazione>  dati, DownloadBean downloadBean) {
		try {
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.OPERAZIONI.stampa())+ "." +ConstantsMap.pdf;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, dati);
			String s = ConstantsGlobal.secretP;
			PDFWriter.stampaPDFOperazione(dati, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaExcelPausa(List<Pausa>  dati, DownloadBean downloadBean) {
		try {
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.PAUSE.stampa())+ "." + ConstantsMap.xlsx;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, dati);
			String s = ConstantsGlobal.secretE;
			ExcelExport.esportaExcelPausa(dati, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaExcelMarcatura(List<Marcatura>  dati, DownloadBean downloadBean) {
		try {
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.MARCATURE.stampa())+ "." + ConstantsMap.xlsx;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, dati);
			String s = ConstantsGlobal.secretE;
			ExcelExport.esportaExcelMarcatura(dati, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaExcelOperazione(List<Operazione>  dati, DownloadBean downloadBean) {
		try {
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.OPERAZIONI.stampa())+ "." + ConstantsMap.xlsx;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, dati);
			String s = ConstantsGlobal.secretE;
			ExcelExport.esportaExcelOperazione(dati, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaCSVTemplateAnagrafica(List<Anagrafica> anagrafiche, Long tid, DownloadBean downloadBean) {
		if(ApplicationUtil.gtZero(tid)) {
			try {
				String s = impostazioneDao.getPasswordTXT();
				TemplateExportAnagrafica ti = anagraficaDao.getTemplateExportAnagrafica(tid);
				ApplicationUtil.comparatorCondition(anagrafiche, ti);
				String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.ANAGRAFICHE.stampa())+ "." + (s != null ? ConstantsMap.zip : ConstantsMap.csv);

				OutputStream out = downloadBean.outputStream();
				downloadBean.filename(file, anagrafiche);
				if(s != null) {
					ByteArrayOutputStream bos = new ByteArrayOutputStream();
					CSVExport.esportaCSVAnagraficaTemplate(bos, anagrafiche, anagraficaDao, ti);
					ZipPassword.zip(null, bos.toByteArray(), out, ConstantsMap.csv, s);
					bos.close();
				} else {
					CSVExport.esportaCSVAnagraficaTemplate(out, anagrafiche, anagraficaDao, ti);
				}
				downloadBean.close();
			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		}
	}
	public void esportaCSVTemplateContatto(Utente user, Boolean ps, List<Contatto>  contatti, Long tid, DownloadBean downloadBean) {
		if(ApplicationUtil.gtZero(tid)) {
			try {
				String s = impostazioneDao.getPasswordTXT();
				String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.CONTATTO.stampa() +": " + ApplicationUtil.size(contatti), contatti);
				operazioneDao.creaOperazione(ConstantsMap.LOG4, p ,user, ConstantsMap.OPERAZIONE_EXPORT_CONTATTO);
				TemplateExportContatto ti = anagraficaDao.getTemplateExportContatto(tid);
				ApplicationUtil.comparatorCondition(contatti, ti);
				String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.CONTATTI.stampa())+ "." + (s != null ? ConstantsMap.zip : ConstantsMap.csv);

				OutputStream out = downloadBean.outputStream();
				downloadBean.filename(file, contatti);
				if(s != null) {
					ByteArrayOutputStream bos = new ByteArrayOutputStream();
					CSVExport.esportaCSVContattoTemplate(bos, ps, contatti, anagraficaDao, ti);
					ZipPassword.zip(null, bos.toByteArray(), out, ConstantsMap.csv, s);
					bos.close();
				} else {
					CSVExport.esportaCSVContattoTemplate(out, ps, contatti, anagraficaDao, ti);
				}
				downloadBean.close();
			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		}
	}
	public void esportaCSVTemplateAppuntamento(Utente user, Boolean ps, List<Appuntamento> appuntamenti, Long tid, DownloadBean downloadBean) {
		if(ApplicationUtil.gtZero(tid)) {
			try {
				String s = impostazioneDao.getPasswordTXT();
				String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.APPUNTAMENTO.stampa() +": " + ApplicationUtil.size(appuntamenti), appuntamenti);
				operazioneDao.creaOperazione(ConstantsMap.LOG4,p ,user, ConstantsMap.OPERAZIONE_EXPORT_APPUNTAMENTO);
				TemplateExportAppuntamento ti = anagraficaDao.getTemplateExportAppuntamento(tid);
				ApplicationUtil.comparatorCondition(appuntamenti, ti);
				String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.APPUNTAMENTI.stampa())+ "." + (s != null ? ConstantsMap.zip : ConstantsMap.csv);

				OutputStream out = downloadBean.outputStream();
				downloadBean.filename(file, appuntamenti);
				if(s != null) {
					ByteArrayOutputStream bos = new ByteArrayOutputStream();
					CSVExport.esportaCSVAppuntamentoTemplate(bos, ps, appuntamenti, anagraficaDao, ti);
					ZipPassword.zip(null, bos.toByteArray(), out, ConstantsMap.csv, s);
					bos.close();
				} else {
					CSVExport.esportaCSVAppuntamentoTemplate(out, ps, appuntamenti, anagraficaDao, ti);
				}
				downloadBean.close();
			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		}
	}
	public void esportaCSVTemplateContratto(Utente user, Boolean ps, List<Contratto>  contratti, Long tid, DownloadBean downloadBean) {
		if(ApplicationUtil.gtZero(tid)) {
			try {
				String s = impostazioneDao.getPasswordTXT();
				String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.CONTRATTO.stampa() +": " + ApplicationUtil.size(contratti), contratti);
				operazioneDao.creaOperazione(ConstantsMap.LOG4, p ,user,ConstantsMap.OPERAZIONE_EXPORT_CONTRATTO);
				TemplateExportContratto ti = anagraficaDao.getTemplateExportContratto(tid);
				ApplicationUtil.comparatorCondition(contratti, ti);
				String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.CONTRATTI.stampa())+ "." + (s != null ? ConstantsMap.zip : ConstantsMap.csv);

				OutputStream out = downloadBean.outputStream();
				downloadBean.filename(file, contratti);
				if(s != null) {
					ByteArrayOutputStream bos = new ByteArrayOutputStream();
					CSVExport.esportaCSVContrattoTemplate(bos, ps, contratti, anagraficaDao, ti);
					ZipPassword.zip(null, bos.toByteArray(), out, ConstantsMap.csv, s);
					bos.close();
				} else {
					CSVExport.esportaCSVContrattoTemplate(out, ps, contratti, anagraficaDao, ti);
				}
				downloadBean.close();
			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		}
	}
	public void esportaExcelAutoContatto(Utente user, List<AutoContatto>  contatti, DownloadBean downloadBean) {
		try {
			String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.CONTATTO.stampa() +": " + ApplicationUtil.size(contatti), contatti);
			operazioneDao.creaOperazione(ConstantsMap.LOG4, p ,user, ConstantsMap.OPERAZIONE_EXPORT_AUTO_CONTATTO);
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.AUTO_CONTATTI.stampa())+ "." + ConstantsMap.xlsx;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, contatti);
			String s = ConstantsGlobal.secretE;
			ExcelExport.esportaExcelAutoContatto(contatti, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaPdfAutoContatto(Utente user, List<AutoContatto>  contatti, DownloadBean downloadBean) {
		try {
			String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.CONTATTO.stampa() +": " + ApplicationUtil.size(contatti), contatti);
			operazioneDao.creaOperazione(ConstantsMap.LOG4,p ,user, ConstantsMap.OPERAZIONE_EXPORT_AUTO_CONTATTO);
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.AUTO_CONTATTI.stampa())+ "." +ConstantsMap.pdf;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, contatti);
			String s = ConstantsGlobal.secretP;
			PDFWriter.stampaPDFAutoContatto(contatti, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaExcelNota(Utente user, List<Nota>  contatti, DownloadBean downloadBean) {
		try {
			String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.NOTA.stampa() +": " + ApplicationUtil.size(contatti), contatti);
			operazioneDao.creaOperazione(ConstantsMap.LOG4,p ,user, ConstantsMap.OPERAZIONE_EXPORT_NOTA);
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.NOTE.stampa())+ "." + ConstantsMap.xlsx;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, contatti);
			String s = impostazioneDao.getPasswordXLS();
			ExcelExport.esportaExcelNota(contatti, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaPdfNota(Utente user, List<Nota>  contatti, DownloadBean downloadBean) {
		try {
			String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.NOTA.stampa() +": " + ApplicationUtil.size(contatti), contatti);
			operazioneDao.creaOperazione(ConstantsMap.LOG4,p ,user, ConstantsMap.OPERAZIONE_EXPORT_NOTA);
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.NOTE.stampa())+ "." +ConstantsMap.pdf;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, contatti);
			String s = impostazioneDao.getPasswordPDF();
			PDFWriter.stampaPDFNota(contatti, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}

	public void esportaExcelImportAnagrafica(Utente user, List<Anagrafica>  contatti, DownloadBean downloadBean) {
		try {
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.ESCLUSE.stampa(), ConstantsLanguage.ANAGRAFICHE.stampa())+ "." + ConstantsMap.xlsx;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, contatti);
			String s = ConstantsGlobal.secretE;
			ExcelExport.esportaExcelImportAnagrafica(contatti, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaExcelImportOpposizione(Utente user, List<Opposizione>  contatti, DownloadBean downloadBean) {
		try {
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.ESCLUSE.stampa(), ConstantsLanguage.OPPOSIZIONI.stampa())+ "." + ConstantsMap.xlsx;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, contatti);
			String s = ConstantsGlobal.secretE;
			ExcelExport.esportaExcelImportOpposizione(contatti, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaExcelImportContatto(Utente user, List<Contatto> a, TemplateImportContatto ti, DownloadBean downloadBean) {
		try {
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.ESCLUSE.stampa(), ConstantsLanguage.CONTATTI.stampa())+ "." + ConstantsMap.xlsx;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, a);
			String s = ConstantsGlobal.secretE;
			ExcelExport.esportaExcelImportContatto(a, ti, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaExcelImportContratto(Utente user, List<Contratto> a, TemplateImportContratto ti,DownloadBean downloadBean) {
		try {
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.ESCLUSE.stampa(), ConstantsLanguage.CONTRATTI.stampa())+ "." + ConstantsMap.xlsx;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, a);
			String s = ConstantsGlobal.secretE;
			ExcelExport.esportaExcelImportContratto(a, ti, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaExcelImportAppuntamento(Utente user, List<Appuntamento> a, TemplateImportAppuntamento ti,DownloadBean downloadBean) {
		try {
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.ESCLUSE.stampa(), ConstantsLanguage.APPUNTAMENTI.stampa())+ "." + ConstantsMap.xlsx;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, a);
			String s = ConstantsGlobal.secretE;
			ExcelExport.esportaExcelImportAppuntamento(a, ti, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaExcelContattoSI(List<ContattoSI>  contatti1, List<ContattoSI>  contatti2, DownloadBean downloadBean) {
		try {
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.CONTATTI.stampa())+ "." + ConstantsMap.xlsx;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, contatti1, contatti2);
			String s = ConstantsGlobal.secretE;
			ExcelExport.esportaExcelContattoSI(contatti1, contatti2, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaPdfContattoSI(List<ContattoSI>  contatti1, List<ContattoSI>  contatti2, DownloadBean downloadBean) {
		try {
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.CONTATTI.stampa())+ "." +ConstantsMap.pdf;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, contatti1, contatti2);
			String s = ConstantsGlobal.secretP;
			PDFWriter.stampaPDFContattoSI(contatti1, contatti2, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public String scadenzaToHTMLString(String oggetto) {
		Map<String, Object> model = new HashMap<String, Object>();
		model.put("oggetto", oggetto);
		return mergeVelocityTemplate("scadenza", model);
	}
	public void esportaExcelEmail(List<Email>  items, DownloadBean downloadBean) {
		try {
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.EMAIL.stampa())+ "." + ConstantsMap.xlsx;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, items);
			String s = ConstantsGlobal.secretE;
			ExcelExport.esportaExcelEmail(items, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaPdfEmail(List<Email>  items, DownloadBean downloadBean) {
		try {
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.EMAIL.stampa())+ "." +ConstantsMap.pdf;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, items);
			String s = ConstantsGlobal.secretP;
			PDFWriter.stampaPDFEmail(items, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	
	public void esportaExcelRmail(List<Rmail>  items, DownloadBean downloadBean) {
		try {
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.RMAIL.stampa())+ "." + ConstantsMap.xlsx;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, items);
			String s = ConstantsGlobal.secretE;
			ExcelExport.esportaExcelRmail(items, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaPdfRmail(List<Rmail>  items, DownloadBean downloadBean) {
		try {
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.RMAIL.stampa())+ "." +ConstantsMap.pdf;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, items);
			String s = ConstantsGlobal.secretP;
			PDFWriter.stampaPDFRmail(items, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	
	
	public void esportaExcelNotifica(List<Notifica>  items, DownloadBean downloadBean) {
		try {
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.NOTIFICA.stampa())+ "." + ConstantsMap.xlsx;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, items);
			String s = ConstantsGlobal.secretE;
			ExcelExport.esportaExcelNotifica(items, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaPdfNotifica(List<Notifica>  items, DownloadBean downloadBean) {
		try {
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.NOTIFICA.stampa())+ "." +ConstantsMap.pdf;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, items);
			String s = ConstantsGlobal.secretP;
			PDFWriter.stampaPDFNotifica(items, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaExcelCdr(List<Cdr>  items, DownloadBean downloadBean) {
		try {
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.CDR.stampa())+ "." + ConstantsMap.xlsx;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, items);
			String s = ConstantsGlobal.secretE;
			ExcelExport.esportaExcelCdr(items, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaPdfCdr(List<Cdr>  items, DownloadBean downloadBean) {
		try {
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.CDR.stampa())+ "." +ConstantsMap.pdf;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, items);
			String s = ConstantsGlobal.secretP;
			PDFWriter.stampaPDFCdr(items, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaExcelCel(List<Cel>  items, DownloadBean downloadBean) {
		try {
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.CEL.stampa())+ "." + ConstantsMap.xlsx;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, items);
			String s = ConstantsGlobal.secretE;
			ExcelExport.esportaExcelCel(items, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaPdfCel(List<Cel>  items, DownloadBean downloadBean) {
		try {
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.CEL.stampa())+ "." +ConstantsMap.pdf;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, items);
			String s = ConstantsGlobal.secretP;
			PDFWriter.stampaPDFCel(items, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaExcelQuery(List<QueryKey> qks, DownloadBean downloadBean) {
		try {
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.QUERY.stampa())+ "." + ConstantsMap.xlsx;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, qks);
			String s = ConstantsGlobal.secretE;
			ExcelExport.esportaExcelQuery(qks, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaPdfQuery(List<QueryKey> qks, DownloadBean downloadBean) {
		try {
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.QUERY.stampa())+ "." +ConstantsMap.pdf;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, qks);
			String s = ConstantsGlobal.secretP;
			PDFWriter.stampaPDFQuery(qks, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void scaricaPdfAgenteContrattoList(Boolean storia_operatore, Utente user, Boolean ps, Boolean supervisore, Boolean storia_agente, List<Contratto> contratti, DownloadBean downloadBean) {
		String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.CONTRATTO.stampa() +": " + ApplicationUtil.size(contratti), contratti);
		operazioneDao.creaOperazione(ConstantsMap.LOG4, p ,user, ConstantsMap.OPERAZIONE_EXPORT_CONTRATTO);
		List<Allegato> files = new ArrayList<Allegato>();
		if(ApplicationUtil.notEmpty(contratti)) {
			String s = ConstantsGlobal.secretL;
			String z = impostazioneDao.getPasswordZIP();
			String html = impostazioneDao.getFormatoDocumento();
			String extM = (s == null || (html.equalsIgnoreCase(ConstantsMap.pdf))) ? html : ConstantsMap.zip;
			Iterator<Contratto> objAppIter = contratti.iterator();
			while(objAppIter.hasNext()) {
				Contratto contratto = objAppIter.next();
				try {
					String nomeFile = ApplicationUtil.reportFile(contratto.getData_creazione(), ConstantsLanguage.CONTRATTO.stampa(), contratto.getId(), contratto.getContatto().getAnagrafica().getDenominazione())  + "." + extM;
					ByteArrayOutputStream result;
					if(ApplicationUtil.isTrue(storia_agente)) {
						result = contrattoToHTMLString(user, contratto,  storia_operatore, supervisore,  html, ps, false,s);
					} else {
						boolean vs = contratto.getAgente() != null && ApplicationUtil.isTrue(contratto.getAgente().getVisualizzaStoria());
						result = contrattoToHTMLString(user, contratto,  vs, supervisore, html, ps, false, s);
					}
					result.close();
					ApplicationUtil.addFile(files, nomeFile, result.toByteArray(), contratto.getData_creazione().getTime());
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				}
			}
			if (ApplicationUtil.notEmpty(files)) {
				try {

					String fileName = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.CONTRATTI.stampa()) + "." + ConstantsMap.zip;
					OutputStream out = downloadBean.outputStream();
					downloadBean.filename(fileName, files);
					ApplicationUtil.zip(null, files, out, z);
					downloadBean.close();
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				}
			} 
		}
	}
	public void scaricaPdfContrattoConcatena(Boolean storia_operatore, Utente user, Boolean ps, Boolean supervisore, Boolean storia_agente, List<Contratto> contratti, DownloadBean downloadBean) {
		String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.CONTRATTO.stampa() +": " + ApplicationUtil.size(contratti), contratti);
		operazioneDao.creaOperazione(ConstantsMap.LOG4, p,user, ConstantsMap.OPERAZIONE_EXPORT_CONTRATTO);
		List<Allegato> files = new ArrayList<Allegato>();
		if(ApplicationUtil.notEmpty(contratti)) {
			String html = ConstantsMap.pdf;
			String s = impostazioneDao.getPasswordLCK();
			String pp = null;
			Iterator<Contratto> objAppIter = contratti.iterator();
			while(objAppIter.hasNext()) {
				Contratto contratto = objAppIter.next();
				try {
					String nomeFile = ApplicationUtil.reportFile(contratto.getData_creazione(), ConstantsLanguage.CONTRATTO.stampa(), contratto.getId(), contratto.getContatto().getAnagrafica().getDenominazione())  + "." + html;
					ByteArrayOutputStream result;
					if(ApplicationUtil.isTrue(storia_agente)) {
						result = contrattoToHTMLString(user, contratto,  storia_operatore,  supervisore, html, ps, false, pp);
					} else {
						boolean vs = contratto.getAgente() != null && ApplicationUtil.isTrue(contratto.getAgente().getVisualizzaStoria());
						result = contrattoToHTMLString(user, contratto,  vs, supervisore,  html, ps, false, pp);
					}
					result.close();
					ApplicationUtil.addFile(files, nomeFile, result.toByteArray(), contratto.getData_creazione().getTime());
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				}
			}
			if (ApplicationUtil.notEmpty(files)) {
				try {

					String fileName = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.CONTRATTI.stampa()) + "." + html;
					OutputStream out = downloadBean.outputStream();
					downloadBean.filename(fileName, files);
					String w = getWatermark();
					ApplicationUtil.concatenaPDF(w, files, out, s);
					downloadBean.close();
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				}
			} 
		}
	}
	public void scaricaPdfAgenteContattoList(Boolean storia_operatore, Utente user, Boolean ps, Boolean supervisore, Boolean storia_agente, List<Contatto> contatti, DownloadBean downloadBean) {
		String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.CONTATTO.stampa() +": " + ApplicationUtil.size(contatti), contatti);
		operazioneDao.creaOperazione(ConstantsMap.LOG4,p ,user, ConstantsMap.OPERAZIONE_EXPORT_CONTATTO);
		List<Allegato> files = new ArrayList<Allegato>();
		if(ApplicationUtil.notEmpty(contatti)) {
			String s = impostazioneDao.getPasswordZIP();
			String pp = ConstantsGlobal.secretL;
			String html = impostazioneDao.getFormatoDocumento();
			String extM = (pp == null || (html.equalsIgnoreCase(ConstantsMap.pdf))) ? html : ConstantsMap.zip;
			Iterator<Contatto> objAppIter = contatti.iterator();
			while(objAppIter.hasNext()) {
				Contatto contatto = objAppIter.next();
				try {
					String nomeFile = ApplicationUtil.reportFile(contatto.getData_creazione(), ConstantsLanguage.CONTATTO.stampa(), contatto.getId(), contatto.getAnagrafica().getDenominazione())  + "." + extM;
					ByteArrayOutputStream result;
					if(ApplicationUtil.isTrue(storia_agente)) {
						result = contattoToHTMLStream(user, contatto,  storia_operatore, supervisore,  html, ps, false, pp);
					} else {
						boolean vs = contatto.getAgente() != null && ApplicationUtil.isTrue(contatto.getAgente().getVisualizzaStoria());
						result = contattoToHTMLStream(user, contatto,  vs, supervisore, html, ps, false, pp);
					}
					result.close();
					ApplicationUtil.addFile(files, nomeFile, result.toByteArray(), contatto.getData_creazione().getTime());
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				}
			}
			if (ApplicationUtil.notEmpty(files)) {
				try {

					String fileName = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.CONTATTI.stampa()) + "." + ConstantsMap.zip;
					OutputStream out = downloadBean.outputStream();
					downloadBean.filename(fileName, files);
					ApplicationUtil.zip(null, files, out, s);
					downloadBean.close();
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				}
			} 
		}
	}
	public void scaricaPdfContattoConcatena(Boolean storia_operatore, Utente user, Boolean ps, Boolean supervisore, Boolean storia_agente, List<Contatto> contatti, DownloadBean downloadBean) {
		String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.CONTATTO.stampa() +": " + ApplicationUtil.size(contatti), contatti);
		operazioneDao.creaOperazione(ConstantsMap.LOG4,p ,user, ConstantsMap.OPERAZIONE_EXPORT_CONTATTO);
		List<Allegato> files = new ArrayList<Allegato>();
		if(ApplicationUtil.notEmpty(contatti)) {
			String s = impostazioneDao.getPasswordLCK();
			String pp = null;
			String html = ConstantsMap.pdf;
			Iterator<Contatto> objAppIter = contatti.iterator();
			while(objAppIter.hasNext()) {
				Contatto contatto = objAppIter.next();
				try {
					String nomeFile = ApplicationUtil.reportFile(contatto.getData_creazione(), ConstantsLanguage.CONTATTO.stampa(), contatto.getId(), contatto.getAnagrafica().getDenominazione())  + "." + html;
					ByteArrayOutputStream result;
					if(ApplicationUtil.isTrue(storia_agente)) {
						result = contattoToHTMLStream(user, contatto,  storia_operatore,  supervisore, html, ps, false, pp);
					} else {
						boolean vs = contatto.getAgente() != null && ApplicationUtil.isTrue(contatto.getAgente().getVisualizzaStoria());
						result = contattoToHTMLStream(user, contatto,  vs, supervisore,  html, ps, false, pp);
					}
					result.close();
					ApplicationUtil.addFile(files, nomeFile, result.toByteArray(), contatto.getData_creazione().getTime());
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				}
			}
			if (ApplicationUtil.notEmpty(files)) {
				try {

					String fileName = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.CONTATTI.stampa()) + "." + html;
					OutputStream out = downloadBean.outputStream();
					downloadBean.filename(fileName, files);
					String w = getWatermark();
					ApplicationUtil.concatenaPDF(w, files, out, s);
					downloadBean.close();
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				}
			} 
		}
	}
	public void scaricaPdfAgenteAnagraficaList(Boolean p,List<Anagrafica> anagrafiche, DownloadBean downloadBean) {
		List<Allegato> files = new ArrayList<Allegato>();
		if(ApplicationUtil.notEmpty(anagrafiche)) {
			String s = impostazioneDao.getPasswordZIP();
			String pp = ConstantsGlobal.secretL;
			String html = impostazioneDao.getFormatoDocumento();
			String extM = (pp == null || (html.equalsIgnoreCase(ConstantsMap.pdf))) ? html : ConstantsMap.zip;
			Iterator<Anagrafica> objAppIter = anagrafiche.iterator();
			while(objAppIter.hasNext()) {
				Anagrafica anagrafica = objAppIter.next();
				try {
					String nomeFile = ApplicationUtil.reportFile(anagrafica.getData_creazione(), ConstantsLanguage.ANAGRAFICA.stampa(), anagrafica.getId(), anagrafica.getDenominazione())  + "." + extM;
					ByteArrayOutputStream result = anagraficaToHTMLString(anagrafica, html, p, pp);
					result.close();
					ApplicationUtil.addFile(files, nomeFile, result.toByteArray(), anagrafica.getData_creazione().getTime());
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				}
			}
			if (ApplicationUtil.notEmpty(files)) {
				try {

					String fileName = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.ANAGRAFICHE.stampa()) + "." + ConstantsMap.zip;
					OutputStream out = downloadBean.outputStream();
					downloadBean.filename(fileName, files);
					ApplicationUtil.zip(null, files, out, s);
					downloadBean.close();
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				}
			} 
		}
	}
	public void scaricaPdfAnagraficaConcatena(Boolean p, List<Anagrafica> anagrafiche, DownloadBean downloadBean) {
		List<Allegato> files = new ArrayList<Allegato>();
		if(ApplicationUtil.notEmpty(anagrafiche)) {
			String html = ConstantsMap.pdf;
			String s = impostazioneDao.getPasswordLCK();
			String pp = null;
			Iterator<Anagrafica> objAppIter = anagrafiche.iterator();
			while(objAppIter.hasNext()) {
				Anagrafica anagrafica = objAppIter.next();
				try {
					String nomeFile = ApplicationUtil.reportFile(anagrafica.getData_creazione(), ConstantsLanguage.ANAGRAFICA.stampa(), anagrafica.getId(), anagrafica.getDenominazione())  + "." + html;
					ByteArrayOutputStream result = anagraficaToHTMLString(anagrafica,  html, p, pp);
					result.close();
					ApplicationUtil.addFile(files, nomeFile, result.toByteArray(), anagrafica.getData_creazione().getTime());
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				}
			}
			if (ApplicationUtil.notEmpty(files)) {
				try {

					String fileName = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.ANAGRAFICHE.stampa()) + "." + html;
					OutputStream out = downloadBean.outputStream();
					downloadBean.filename(fileName, files);
					String w = getWatermark();
					ApplicationUtil.concatenaPDF(w, files, out, s);
					downloadBean.close();
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				}
			} 
		}
	}
	public void esportaPdfTemplateStoriaContratto(Utente user, Boolean ps, List<StoriaContratto>  contratti,  Long tid, DownloadBean downloadBean) {
		if(ApplicationUtil.gtZero(tid)) {
			try {
				String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.STORIA.stampa()+ " " + ConstantsLanguage.CONTRATTO.stampa() +": " + ApplicationUtil.size(contratti), contratti);
				operazioneDao.creaOperazione(ConstantsMap.LOG4,p ,user, ConstantsMap.OPERAZIONE_EXPORT_STORIA_CONTRATTO);
				TemplateExportStoriaContratto ti = anagraficaDao.getTemplateExportStoriaContratto(tid);
				ApplicationUtil.comparatorCondition(contratti, ti);
				String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.STORIA.stampa(), ConstantsLanguage.CONTRATTI.stampa()) + "." +ConstantsMap.pdf;

				OutputStream out = downloadBean.outputStream();
				downloadBean.filename(file, contratti);
				String s = impostazioneDao.getPasswordPDF();
				PDFWriter.stampaPDFStoriaContrattoTemplate(ps, contratti, out, ti, anagraficaDao, s);
				downloadBean.close();
			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		}
	}
	public void esportaPdfTemplateStoriaAppuntamento(Utente user, Boolean ps, List<StoriaAppuntamento> appuntamenti,  Long tid, DownloadBean downloadBean) {
		if(ApplicationUtil.gtZero(tid)) {
			try {
				String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.STORIA.stampa() + " " + ConstantsLanguage.APPUNTAMENTO.stampa() +": " + ApplicationUtil.size(appuntamenti), appuntamenti);
				operazioneDao.creaOperazione(ConstantsMap.LOG4,p ,user, ConstantsMap.OPERAZIONE_EXPORT_STORIA_APPUNTAMENTO);
				TemplateExportStoriaAppuntamento ti = anagraficaDao.getTemplateExportStoriaAppuntamento(tid);
				ApplicationUtil.comparatorCondition(appuntamenti, ti);
				String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.STORIA.stampa(), ConstantsLanguage.APPUNTAMENTI.stampa()) + "." +ConstantsMap.pdf;

				OutputStream out = downloadBean.outputStream();
				downloadBean.filename(file, appuntamenti);
				String s = impostazioneDao.getPasswordPDF();
				PDFWriter.stampaPDFStoriaAppuntamentoTemplate(ps, appuntamenti, out, ti, anagraficaDao, s);
				downloadBean.close();
			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		}
	}
	public void esportaPdfTemplateStoriaContatto(Utente user, Boolean ps, List<StoriaContatto>  contatti,  Long tid, DownloadBean downloadBean) {
		if(ApplicationUtil.gtZero(tid)) {
			try {
				String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.STORIA.stampa() + " " + ConstantsLanguage.CONTATTO.stampa() +": " + ApplicationUtil.size(contatti), contatti);
				operazioneDao.creaOperazione(ConstantsMap.LOG4, p ,user, ConstantsMap.OPERAZIONE_EXPORT_STORIA_CONTATTO);
				TemplateExportStoriaContatto ti = anagraficaDao.getTemplateExportStoriaContatto(tid);
				ApplicationUtil.comparatorCondition(contatti, ti);
				String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.STORIA.stampa(), ConstantsLanguage.CONTATTI.stampa()) + "." +ConstantsMap.pdf;

				OutputStream out = downloadBean.outputStream();
				downloadBean.filename(file, contatti);
				String s = impostazioneDao.getPasswordPDF();
				PDFWriter.stampaPDFStoriaContattoTemplate(ps, contatti, out, ti, anagraficaDao, s);
				downloadBean.close();
			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		}
	}
	public void esportaExcelTemplateStoriaAppuntamento(Utente user, Boolean ps, List<StoriaAppuntamento> appuntamenti, Long tid, DownloadBean downloadBean) {
		if(ApplicationUtil.gtZero(tid)) {
			try {
				String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.STORIA.stampa() + " " + ConstantsLanguage.APPUNTAMENTO.stampa() +": " + ApplicationUtil.size(appuntamenti), appuntamenti);
				operazioneDao.creaOperazione(ConstantsMap.LOG4,p ,user,  ConstantsMap.OPERAZIONE_EXPORT_STORIA_APPUNTAMENTO);
				TemplateExportStoriaAppuntamento ti = anagraficaDao.getTemplateExportStoriaAppuntamento(tid);
				ApplicationUtil.comparatorCondition(appuntamenti, ti);
				String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.STORIA.stampa(), ConstantsLanguage.APPUNTAMENTI.stampa())+ "." + ConstantsMap.xlsx;

				OutputStream out = downloadBean.outputStream();
				downloadBean.filename(file, appuntamenti);
				String s = impostazioneDao.getPasswordXLS();
				ExcelExport.esportaExcelStoriaAppuntamentoTemplate(ps, appuntamenti, anagraficaDao, ti, out, s);
				downloadBean.close();
			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		}
	}
	public void esportaExcelTemplateStoriaContratto(Utente user, Boolean ps, List<StoriaContratto>  contratti, Long tid, DownloadBean downloadBean) {
		if(ApplicationUtil.gtZero(tid)) {
			try {
				String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.STORIA.stampa() + " " + ConstantsLanguage.CONTRATTO.stampa() +": " + ApplicationUtil.size(contratti), contratti);
				operazioneDao.creaOperazione(ConstantsMap.LOG4,p ,user, ConstantsMap.OPERAZIONE_EXPORT_STORIA_CONTRATTO);
				TemplateExportStoriaContratto ti = anagraficaDao.getTemplateExportStoriaContratto(tid);
				ApplicationUtil.comparatorCondition(contratti, ti);
				String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.STORIA.stampa(), ConstantsLanguage.CONTRATTI.stampa())+ "." + ConstantsMap.xlsx;

				OutputStream out = downloadBean.outputStream();
				downloadBean.filename(file, contratti);
				String s = impostazioneDao.getPasswordXLS();
				ExcelExport.esportaExcelStoriaContrattoTemplate(ps, contratti, anagraficaDao, ti, out, s);
				downloadBean.close();
			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		}
	}
	public void esportaExcelTemplateStoriaContatto(Utente user, Boolean ps, List<StoriaContatto>  contatti, Long tid, DownloadBean downloadBean) {
		if(ApplicationUtil.gtZero(tid)) {
			try {
				String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.STORIA.stampa() + " " + ConstantsLanguage.CONTATTO.stampa() +": " + ApplicationUtil.size(contatti), contatti);
				operazioneDao.creaOperazione(ConstantsMap.LOG4, p ,user, ConstantsMap.OPERAZIONE_EXPORT_STORIA_CONTATTO);
				TemplateExportStoriaContatto ti = anagraficaDao.getTemplateExportStoriaContatto(tid);
				ApplicationUtil.comparatorCondition(contatti, ti);
				String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.STORIA.stampa(), ConstantsLanguage.CONTATTI.stampa())+ "." + ConstantsMap.xlsx;

				OutputStream out = downloadBean.outputStream();
				downloadBean.filename(file, contatti);
				String s = impostazioneDao.getPasswordXLS();
				ExcelExport.esportaExcelStoriaContattoTemplate(ps, contatti, anagraficaDao, ti, out, s);
				downloadBean.close();
			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		}
	}
	public void esportaCSVTemplateStoriaContatto(Utente user, Boolean ps, List<StoriaContatto>  contatti, Long tid, DownloadBean downloadBean) {
		if(ApplicationUtil.gtZero(tid)) {
			try {
				String s = impostazioneDao.getPasswordTXT();
				String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.STORIA.stampa() + " " + ConstantsLanguage.CONTATTO.stampa() +": " + ApplicationUtil.size(contatti), contatti);
				operazioneDao.creaOperazione(ConstantsMap.LOG4, p ,user, ConstantsMap.OPERAZIONE_EXPORT_STORIA_CONTATTO);
				TemplateExportStoriaContatto ti = anagraficaDao.getTemplateExportStoriaContatto(tid);
				ApplicationUtil.comparatorCondition(contatti, ti);
				String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.STORIA.stampa(), ConstantsLanguage.CONTATTI.stampa())+ "." + (s != null ? ConstantsMap.zip : ConstantsMap.csv);

				OutputStream out = downloadBean.outputStream();
				downloadBean.filename(file, contatti);
				if(s != null) {
					ByteArrayOutputStream bos = new ByteArrayOutputStream();
					CSVExport.esportaCSVStoriaContattoTemplate(bos, ps, contatti, anagraficaDao, ti);
					ZipPassword.zip(null, bos.toByteArray(), out, ConstantsMap.csv, s);
					bos.close();
				} else {
					CSVExport.esportaCSVStoriaContattoTemplate(out, ps, contatti, anagraficaDao, ti);
				}
				downloadBean.close();
			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		}
	}
	public void esportaCSVTemplateStoriaAppuntamento(Utente user, Boolean ps, List<StoriaAppuntamento> appuntamenti, Long tid, DownloadBean downloadBean) {
		if(ApplicationUtil.gtZero(tid)) {
			try {
				String s = impostazioneDao.getPasswordTXT();
				String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.STORIA.stampa() + " " + ConstantsLanguage.APPUNTAMENTO.stampa() +": " + ApplicationUtil.size(appuntamenti), appuntamenti);
				operazioneDao.creaOperazione(ConstantsMap.LOG4,p ,user, ConstantsMap.OPERAZIONE_EXPORT_STORIA_APPUNTAMENTO);
				TemplateExportStoriaAppuntamento ti = anagraficaDao.getTemplateExportStoriaAppuntamento(tid);
				ApplicationUtil.comparatorCondition(appuntamenti, ti);
				String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.STORIA.stampa(), ConstantsLanguage.APPUNTAMENTI.stampa())+ "." + (s != null ? ConstantsMap.zip : ConstantsMap.csv);

				OutputStream out = downloadBean.outputStream();
				downloadBean.filename(file, appuntamenti);
				if(s != null) {
					ByteArrayOutputStream bos = new ByteArrayOutputStream();
					CSVExport.esportaCSVStoriaAppuntamentoTemplate(bos, ps, appuntamenti, anagraficaDao, ti);
					ZipPassword.zip(null, bos.toByteArray(), out, ConstantsMap.csv, s);
					bos.close();
				} else {
					CSVExport.esportaCSVStoriaAppuntamentoTemplate(out, ps, appuntamenti, anagraficaDao, ti);
				}
				downloadBean.close();
			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		}
	}
	public void esportaCSVStoriaTemplateContratto(Utente user, Boolean ps, List<StoriaContratto>  contratti, Long tid, DownloadBean downloadBean) {
		if(ApplicationUtil.gtZero(tid)) {
			try {
				String s = impostazioneDao.getPasswordTXT();
				String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.STORIA.stampa() + " " + ConstantsLanguage.CONTRATTO.stampa() +": " + ApplicationUtil.size(contratti), contratti);
				operazioneDao.creaOperazione(ConstantsMap.LOG4, p ,user,ConstantsMap.OPERAZIONE_EXPORT_STORIA_CONTRATTO);
				TemplateExportStoriaContratto ti = anagraficaDao.getTemplateExportStoriaContratto(tid);
				ApplicationUtil.comparatorCondition(contratti, ti);
				String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.STORIA.stampa(), ConstantsLanguage.CONTRATTI.stampa())+ "." + (s != null ? ConstantsMap.zip : ConstantsMap.csv);

				OutputStream out = downloadBean.outputStream();
				downloadBean.filename(file, contratti);
				if(s != null) {
					ByteArrayOutputStream bos = new ByteArrayOutputStream();
					CSVExport.esportaCSVStoriaContrattoTemplate(bos, ps, contratti, anagraficaDao, ti);
					ZipPassword.zip(null, bos.toByteArray(), out, ConstantsMap.csv, s);
					bos.close();
				} else {
					CSVExport.esportaCSVStoriaContrattoTemplate(out, ps, contratti, anagraficaDao, ti);
				}
				downloadBean.close();
			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		}
	}
	public void esportaExcelStoriaAppuntamento(Utente user,List<StoriaAppuntamento> appuntamenti, List<Long> cc, DownloadBean downloadBean) {
		try {
			String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.STORIA.stampa() + " " + ConstantsLanguage.APPUNTAMENTO.stampa() +": " + ApplicationUtil.size(appuntamenti), appuntamenti);
			operazioneDao.creaOperazione(ConstantsMap.LOG4,p ,user, ConstantsMap.OPERAZIONE_EXPORT_STORIA_APPUNTAMENTO);
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.STORIA.stampa(), ConstantsLanguage.APPUNTAMENTI.stampa())+ "." + ConstantsMap.xlsx;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, appuntamenti);
			String s = impostazioneDao.getPasswordXLS();
			ExcelExport.esportaExcelStoriaAppuntamento(appuntamenti, anagraficaDao, cc, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaExcelStoriaContatto(Utente user, List<StoriaContatto>  contatti, List<Long> cc, DownloadBean downloadBean) {
		try {
			String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.STORIA.stampa() + " " + ConstantsLanguage.CONTATTO.stampa() +": " + ApplicationUtil.size(contatti), contatti);
			operazioneDao.creaOperazione(ConstantsMap.LOG4, p ,user, ConstantsMap.OPERAZIONE_EXPORT_STORIA_CONTATTO);
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.STORIA.stampa(), ConstantsLanguage.CONTATTI.stampa())+ "." + ConstantsMap.xlsx;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, contatti);
			String s = impostazioneDao.getPasswordXLS();
			ExcelExport.esportaExcelStoriaContatto(contatti, anagraficaDao, cc, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaExcelStoriaContratto(Utente user, List<StoriaContratto>  contratti, List<Long> aa, List<Long> bb, DownloadBean downloadBean) {
		try {
			String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.STORIA.stampa() + " " + ConstantsLanguage.CONTRATTO.stampa() +": " + ApplicationUtil.size(contratti), contratti);
			operazioneDao.creaOperazione(ConstantsMap.LOG4, p ,user, ConstantsMap.OPERAZIONE_EXPORT_STORIA_CONTRATTO);
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.STORIA.stampa(), ConstantsLanguage.CONTRATTI.stampa())+ "." + ConstantsMap.xlsx;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, contratti);
			String s = impostazioneDao.getPasswordXLS();
			ExcelExport.esportaExcelStoriaContratto(contratti, anagraficaDao, aa, bb, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaPdfStoriaAppuntamento(Utente user, List<StoriaAppuntamento> appuntamenti, List<Long> cc, DownloadBean downloadBean) {
		try {
			String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.STORIA.stampa() + " " + ConstantsLanguage.APPUNTAMENTO.stampa() +": " + ApplicationUtil.size(appuntamenti), appuntamenti);
			operazioneDao.creaOperazione(ConstantsMap.LOG4,p ,user, ConstantsMap.OPERAZIONE_EXPORT_STORIA_APPUNTAMENTO);
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.STORIA.stampa(), ConstantsLanguage.APPUNTAMENTI.stampa()) + "." + ConstantsMap.pdf;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, appuntamenti);
			String s = impostazioneDao.getPasswordPDF();
			PDFWriter.stampaPDFStoriaAppuntamento(appuntamenti, out, cc, anagraficaDao, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaPdfStoriaContattoLista(Utente user, List<StoriaContatto>  contatti, List<Long> cc, DownloadBean downloadBean) {
		try {
			String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.STORIA.stampa() + " " + ConstantsLanguage.CONTATTO.stampa() +": " + ApplicationUtil.size(contatti), contatti);
			operazioneDao.creaOperazione(ConstantsMap.LOG4, p ,user, ConstantsMap.OPERAZIONE_EXPORT_STORIA_CONTATTO);
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.STORIA.stampa(), ConstantsLanguage.CONTATTI.stampa()) + "." +ConstantsMap.pdf;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, contatti);
			String s = impostazioneDao.getPasswordPDF();
			PDFWriter.stampaPDFStoriaContatto(contatti, out, cc, anagraficaDao, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaPdfStoriaContratto(Utente user, List<StoriaContratto>  contratti, List<Long> aa, List<Long> bb, DownloadBean downloadBean) {
		try {
			String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() +" " + ConstantsLanguage.STORIA.stampa() + " " + ConstantsLanguage.CONTRATTO.stampa() +": " + ApplicationUtil.size(contratti), contratti);
			operazioneDao.creaOperazione(ConstantsMap.LOG4,p ,user, ConstantsMap.OPERAZIONE_EXPORT_STORIA_CONTRATTO);
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.STORIA.stampa(), ConstantsLanguage.CONTRATTI.stampa())+ "." +ConstantsMap.pdf;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, contratti);
			String s = impostazioneDao.getPasswordPDF();
			PDFWriter.stampaPDFStoriaContratto(contratti, out, aa, bb, anagraficaDao, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaDatiExcelAnagrafica(List<Anagrafica> data, OutputStream result) throws Exception {
		boolean ok = true;
		boolean stampa = false;
		String s = impostazioneDao.getPasswordXLS();
		if(ApplicationUtil.isTrue(ok) ) {
			Long til = anagraficaDao.getTemplatePredefinitoExportAnagrafica();
			if(ApplicationUtil.gtZero(til)) {
				TemplateExportAnagrafica ti = anagraficaDao.getTemplateExportAnagrafica(til);
				ApplicationUtil.comparatorCondition(data, ti);
				ExcelExport.esportaExcelAnagraficaTemplate(data, anagraficaDao, ti, result, s);
				stampa = true;
			} 
		}
		if(!stampa) {
			ExcelExport.esportaExcelAnagrafica(data, result, s);
		}
	}
	public String preparaTemplateEmail(String testo, Date data) {
		Map<String, Object> model = new HashMap<String, Object>();
		model.put("testo", testo  + " " + ConstantsLanguage.REPORT_DATA.stampa() +  " " + ApplicationUtil.formatDate(ConstantsFormat.dateTime.stampa(), data));
		return mergeVelocityTemplate("report", model);
	}
	
	public String preparaTemplateEscluse(String testo, Date data, ImportModelConsentite tc, ImportModelAnagrafica im, Campagna campagna) {
		Map<String, Object> model = new HashMap<String, Object>();
		model.put("testo", testo  + " " + ConstantsLanguage.REPORT_DATA.stampa() +  " " + ApplicationUtil.formatDate(ConstantsFormat.dateTime.stampa(), data));
		model.put("campagna", campagna);
		model.put("im", im);
		model.put("tc", tc);
		model.put("time", ApplicationUtil.format(ApplicationUtil.time(data, ApplicationUtil.date())));
		return mergeVelocityTemplate("escluse", model);
	}
	
	public String preparaTemplateEmailNA(Appuntamento appuntamento) {
		Map<String, Object> model = new HashMap<String, Object>();
		model.put("mappa", loadValori(appuntamento));
		model.put("appuntamento", appuntamento);
		return mergeVelocityTemplate("report_appuntamento", model);
	}
	
	public String preparaTemplateEmailNC(Contratto contratto) {
		Map<String, Object> model = new HashMap<String, Object>();
		model.put("mappa", loadValori(contratto));
		model.put("contratto", contratto);
		return mergeVelocityTemplate("report_contratto", model);
	}
	
	public void esportaDatiExcelAppuntamento(List<Appuntamento> data, OutputStream result) throws Exception {
		boolean ok = true; 
		boolean stampa = false;
		String s = impostazioneDao.getPasswordXLS();
		if(ApplicationUtil.isTrue(ok) ) {
			Long til = anagraficaDao.getTemplatePredefinitoExportAppuntamento();
			if(ApplicationUtil.gtZero(til)) {
				TemplateExportAppuntamento ti = anagraficaDao.getTemplateExportAppuntamento(til);
				ApplicationUtil.comparatorCondition(data, ti);
				ExcelExport.esportaExcelAppuntamentoTemplate(null, data, anagraficaDao, ti, result, s);
				stampa = true;
			} 
		}
		if(!stampa) {
			ExcelExport.esportaExcelAppuntamento(data, result, s);
		}
	}
	public void esportaDatiExcelContatto(List<Contatto> data, OutputStream result) throws Exception {
		boolean ok = true; 
		boolean stampa = false;
		String s = impostazioneDao.getPasswordXLS();
		if(ApplicationUtil.isTrue(ok) ) {
			Long til = anagraficaDao.getTemplatePredefinitoExportContatto();
			if(ApplicationUtil.gtZero(til)) {
				TemplateExportContatto ti = anagraficaDao.getTemplateExportContatto(til);
				ApplicationUtil.comparatorCondition(data, ti);
				ExcelExport.esportaExcelContattoTemplate(null, data, anagraficaDao, ti, result, s);
				stampa = true;
			} 
		}
		if(!stampa) {
			ExcelExport.esportaExcelContatto(data, result, s);
		}
	}
	public void esportaDatiExcelContratto(List<Contratto> data, OutputStream result) throws Exception {
		boolean ok = true; 
		boolean stampa = false;
		String s = impostazioneDao.getPasswordXLS();
		if(ApplicationUtil.isTrue(ok) ) {
			Long til = anagraficaDao.getTemplatePredefinitoExportContratto();
			if(ApplicationUtil.gtZero(til)) {
				TemplateExportContratto ti = anagraficaDao.getTemplateExportContratto(til);
				ApplicationUtil.comparatorCondition(data, ti);
				ExcelExport.esportaExcelContrattoTemplate(null, data, anagraficaDao, ti, result, s);
				stampa = true;
			} 
		}
		if(!stampa) {
			ExcelExport.esportaExcelContratto(data, result, s);
		}
	}
	public void esportaDatiExcelStoriaAppuntamento(List<StoriaAppuntamento> data, OutputStream result) throws Exception {
		boolean ok = true; 
		boolean stampa = false;
		String s = impostazioneDao.getPasswordXLS();
		if(ApplicationUtil.isTrue(ok) ) {
			Long til = anagraficaDao.getTemplatePredefinitoExportStoriaAppuntamento();
			if(ApplicationUtil.gtZero(til)) {
				TemplateExportStoriaAppuntamento ti = anagraficaDao.getTemplateExportStoriaAppuntamento(til);
				ApplicationUtil.comparatorCondition(data, ti);
				ExcelExport.esportaExcelStoriaAppuntamentoTemplate(null, data, anagraficaDao, ti, result, s);
				stampa = true;
			} 
		}
		if(!stampa) {
			ExcelExport.esportaExcelStoriaAppuntamento(data, result, s);
		}
	}
	public void esportaDatiExcelStoriaContatto(List<StoriaContatto> data, OutputStream result) throws Exception {
		boolean ok = true; 
		boolean stampa = false;
		String s = impostazioneDao.getPasswordXLS();
		if(ApplicationUtil.isTrue(ok) ) {
			Long til = anagraficaDao.getTemplatePredefinitoExportStoriaContatto();
			if(ApplicationUtil.gtZero(til)) {
				TemplateExportStoriaContatto ti = anagraficaDao.getTemplateExportStoriaContatto(til);
				ApplicationUtil.comparatorCondition(data, ti);
				ExcelExport.esportaExcelStoriaContattoTemplate(null, data, anagraficaDao, ti, result, s);
				stampa = true;
			} 
		}
		if(!stampa) {
			ExcelExport.esportaExcelStoriaContatto(data, result, s);
		}
	}
	public void esportaDatiExcelStoriaContratto(List<StoriaContratto> data, OutputStream result) throws Exception {
		boolean ok = true; 
		boolean stampa = false;
		String s = impostazioneDao.getPasswordXLS();
		if(ApplicationUtil.isTrue(ok) ) {
			Long til = anagraficaDao.getTemplatePredefinitoExportStoriaContratto();
			if(ApplicationUtil.gtZero(til)) {
				TemplateExportStoriaContratto ti = anagraficaDao.getTemplateExportStoriaContratto(til);
				ApplicationUtil.comparatorCondition(data, ti);
				ExcelExport.esportaExcelStoriaContrattoTemplate(null, data, anagraficaDao, ti, result, s);
				stampa = true;
			} 
		}
		if(!stampa) {
			ExcelExport.esportaExcelStoriaContratto(data, result, s);
		}
	}
	public void esportaDatiPdfAnagrafica(List<Anagrafica> data, OutputStream result) throws Exception {
		boolean ok = true; 
		boolean stampa = false;
		String s = impostazioneDao.getPasswordPDF();
		if(ApplicationUtil.isTrue(ok) ) {
			Long til = anagraficaDao.getTemplatePredefinitoExportAnagrafica();
			if(ApplicationUtil.gtZero(til)) {
				TemplateExportAnagrafica ti = anagraficaDao.getTemplateExportAnagrafica(til);
				ApplicationUtil.comparatorCondition(data, ti);
				PDFWriter.stampaPDFAnagraficaTemplate(data, result, ti, anagraficaDao, s);
				stampa = true;
			} 
		}
		if(!stampa) {
			PDFWriter.stampaPDFAnagrafica(data, result, s);
		}
	}
	public void esportaDatiPdfContatto(List<Contatto> data, OutputStream result) throws Exception {
		boolean ok = true; 
		boolean stampa = false;
		String s = impostazioneDao.getPasswordPDF();
		if(ApplicationUtil.isTrue(ok) ) {
			Long til = anagraficaDao.getTemplatePredefinitoExportContatto();
			if(ApplicationUtil.gtZero(til)) {
				TemplateExportContatto ti = anagraficaDao.getTemplateExportContatto(til);
				ApplicationUtil.comparatorCondition(data, ti);
				PDFWriter.stampaPDFContattoTemplate(null, data, result, ti, anagraficaDao, s);
				stampa = true;
			} 
		}
		if(!stampa) {
			PDFWriter.stampaPDFContatto(data, result, s);
		}
	}
	public void esportaDatiPdfContratto(List<Contratto> data, OutputStream result) throws Exception {
		boolean ok = true; 
		boolean stampa = false;
		String s = impostazioneDao.getPasswordPDF();
		if(ApplicationUtil.isTrue(ok) ) {
			Long til = anagraficaDao.getTemplatePredefinitoExportContratto();
			if(ApplicationUtil.gtZero(til)) {
				TemplateExportContratto ti = anagraficaDao.getTemplateExportContratto(til);
				ApplicationUtil.comparatorCondition(data, ti);
				PDFWriter.stampaPDFContrattoTemplate(null, data, result, ti, anagraficaDao, s);
				stampa = true;
			} 
		}
		if(!stampa) {
			PDFWriter.stampaPDFContratto(data, result, s);
		}
	}
	public void esportaDatiPdfAppuntamento(List<Appuntamento> data, OutputStream result) throws Exception {
		boolean ok = true; 
		boolean stampa = false;
		String s = impostazioneDao.getPasswordPDF();
		if(ApplicationUtil.isTrue(ok) ) {
			Long til = anagraficaDao.getTemplatePredefinitoExportAppuntamento();
			if(ApplicationUtil.gtZero(til)) {
				TemplateExportAppuntamento ti = anagraficaDao.getTemplateExportAppuntamento(til);
				ApplicationUtil.comparatorCondition(data, ti);
				PDFWriter.stampaPDFAppuntamentoTemplate(null, data, result, ti, anagraficaDao, s);
				stampa = true;
			} 
		}
		if(!stampa) {
			PDFWriter.stampaPDFAppuntamento(data, result, s);
		}
	}
	public void esportaDatiPdfStoriaContatto(List<StoriaContatto> data, OutputStream result) throws Exception {
		boolean ok = true; 
		boolean stampa = false;
		String s = impostazioneDao.getPasswordPDF();
		if(ApplicationUtil.isTrue(ok) ) {
			Long til = anagraficaDao.getTemplatePredefinitoExportStoriaContatto();
			if(ApplicationUtil.gtZero(til)) {
				TemplateExportStoriaContatto ti = anagraficaDao.getTemplateExportStoriaContatto(til);
				ApplicationUtil.comparatorCondition(data, ti);
				PDFWriter.stampaPDFStoriaContattoTemplate(null, data, result, ti, anagraficaDao, s);
				stampa = true;
			} 
		}
		if(!stampa) {
			PDFWriter.stampaPDFStoriaContatto(data, result, s);
		}
	}
	public void esportaDatiPdfStoriaContratto(List<StoriaContratto> data, OutputStream result) throws Exception {
		boolean ok = true; 
		boolean stampa = false;
		String s = impostazioneDao.getPasswordPDF();
		if(ApplicationUtil.isTrue(ok) ) {
			Long til = anagraficaDao.getTemplatePredefinitoExportStoriaContratto();
			if(ApplicationUtil.gtZero(til)) {
				TemplateExportStoriaContratto ti = anagraficaDao.getTemplateExportStoriaContratto(til);
				ApplicationUtil.comparatorCondition(data, ti);
				PDFWriter.stampaPDFStoriaContrattoTemplate(null, data, result, ti, anagraficaDao, s);
				stampa = true;
			} 
		}
		if(!stampa) {
			PDFWriter.stampaPDFStoriaContratto(data, result, s);
		}
	}
	public void esportaDatiPdfStoriaAppuntamento(List<StoriaAppuntamento> data, OutputStream result) throws Exception {
		boolean ok = true; 
		boolean stampa = false;
		String s = impostazioneDao.getPasswordPDF();
		if(ApplicationUtil.isTrue(ok) ) {
			Long til = anagraficaDao.getTemplatePredefinitoExportStoriaAppuntamento();
			if(ApplicationUtil.gtZero(til)) {
				TemplateExportStoriaAppuntamento ti = anagraficaDao.getTemplateExportStoriaAppuntamento(til);
				ApplicationUtil.comparatorCondition(data, ti);
				PDFWriter.stampaPDFStoriaAppuntamentoTemplate(null, data, result, ti, anagraficaDao, s);
				stampa = true;
			} 
		}
		if(!stampa) {
			PDFWriter.stampaPDFStoriaAppuntamento(data, result, s);
		}
	}
	public void esportaDatiPdfPrenotazione(List<Prenotazione> data, ByteArrayOutputStream result) {
		String s = impostazioneDao.getPasswordPDF();
		PDFWriter.stampaPDFPrenotazione(data, result, s);
	}
	public void esportaDatiPdfNota(List<Nota> data, ByteArrayOutputStream result) {
		String s = impostazioneDao.getPasswordPDF();
		PDFWriter.stampaPDFNota(data, result, s);
	}
	public void esportaDatiPdfOperazione(List<Operazione> data, ByteArrayOutputStream result) {
		String s = ConstantsGlobal.secretP;
		PDFWriter.stampaPDFOperazione(data, result, s);
	}
	public void esportaDatiPdfMarcatura(List<Marcatura> data, ByteArrayOutputStream result) {
		String s = ConstantsGlobal.secretP;
		PDFWriter.stampaPDFMarcatura(data, result, s);
	}
	public void esportaDatiPdfPausa(List<Pausa> data, ByteArrayOutputStream result) {
		String s = ConstantsGlobal.secretP;
		PDFWriter.stampaPDFPausa(data, result, s);
	}
	public void esportaDatiExcelPrenotazione(List<Prenotazione> data, ByteArrayOutputStream result) throws Exception {
		String s = impostazioneDao.getPasswordXLS();
		ExcelExport.esportaExcelPrenotazione(data, result, s);
	}
	public void esportaDatiExcelNota(List<Nota> data, ByteArrayOutputStream result) throws Exception {
		String s = impostazioneDao.getPasswordXLS();
		ExcelExport.esportaExcelNota(data, result, s);
	}
	public void esportaDatiExcelOperazione(List<Operazione> data, ByteArrayOutputStream result) throws Exception {
		String s = ConstantsGlobal.secretE;
		ExcelExport.esportaExcelOperazione(data, result, s);
	}
	public void esportaDatiExcelMarcatura(List<Marcatura> data, ByteArrayOutputStream result) throws Exception {
		String s = ConstantsGlobal.secretE;
		ExcelExport.esportaExcelMarcatura(data, result, s);
	}
	public void esportaDatiExcelPausa(List<Pausa> data, ByteArrayOutputStream result) throws Exception {
		String s = ConstantsGlobal.secretE;
		ExcelExport.esportaExcelPausa(data, result, s);
	}

	public void esportaExcelTelecom(List<Telecom>  items, DownloadBean downloadBean) {
		try {
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.TELECOM.stampa())+ "." + ConstantsMap.xlsx;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, items);
			String s = ConstantsGlobal.secretE;
			ExcelExport.esportaExcelTelecom(items, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaPdfTelecom(List<Telecom>  items, DownloadBean downloadBean) {
		try {
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.TELECOM.stampa())+ "." +ConstantsMap.pdf;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, items);
			String s = ConstantsGlobal.secretP;
			PDFWriter.stampaPDFTelecom(items, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	
	public void esportaExcelWebhook(List<Webhook>  items, DownloadBean downloadBean) {
		try {
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.WEBHOOK.stampa())+ "." + ConstantsMap.xlsx;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, items);
			String s = ConstantsGlobal.secretE;
			ExcelExport.esportaExcelWebhook(items, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaPdfWebhook(List<Webhook>  items, DownloadBean downloadBean) {
		try {
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.WEBHOOK.stampa())+ "." +ConstantsMap.pdf;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, items);
			String s = ConstantsGlobal.secretP;
			PDFWriter.stampaPDFWebhook(items, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	
	public void esportaExcelRicezione(List<Ricezione>  items, DownloadBean downloadBean) {
		try {
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.RICEZIONE.stampa())+ "." + ConstantsMap.xlsx;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, items);
			String s = ConstantsGlobal.secretE;
			ExcelExport.esportaExcelRicezione(items, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaPdfRicezione(List<Ricezione>  items, DownloadBean downloadBean) {
		try {
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.RICEZIONE.stampa())+ "." +ConstantsMap.pdf;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, items);
			String s = ConstantsGlobal.secretP;
			PDFWriter.stampaPDFRicezione(items, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaPdfAccesso(List<Accesso>  dati, DownloadBean downloadBean) {
		try {
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.RICEZIONE.stampa())+ "." +ConstantsMap.pdf;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, dati);
			String s = ConstantsGlobal.secretP;
			PDFWriter.stampaPDFAccesso(dati, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaExcelAccesso(List<Accesso>  dati, DownloadBean downloadBean) {
		try {
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.ACCESSI.stampa())+ "." + ConstantsMap.xlsx;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, dati);
			String s = ConstantsGlobal.secretE;
			ExcelExport.esportaExcelAccesso(dati, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaDatiPdfAccesso(List<Accesso> data, ByteArrayOutputStream result) {
		String s = ConstantsGlobal.secretP;
		PDFWriter.stampaPDFAccesso(data, result, s);
	}
	public void esportaDatiExcelAccesso(List<Accesso> data, ByteArrayOutputStream result) throws Exception {
		String s = ConstantsGlobal.secretE;
		ExcelExport.esportaExcelAccesso(data, result, s);
	}
	public Allegato allegaReportAppuntamentiConcatena(List<Appuntamento> appuntamenti, Utente utente, Contatto contatto, List<Long> ids, String secretL, boolean soloNuovi, Object [] suffix) {
		List<Allegato> files = new ArrayList<Allegato>();
		if(ApplicationUtil.notEmpty(appuntamenti)) {
			String html = ConstantsMap.pdf;
			Iterator<Appuntamento> objIter = appuntamenti.iterator();
			while(objIter.hasNext()) {
				Appuntamento appuntamento = objIter.next();
				boolean nuovo = ApplicationUtil.notEmpty(ids) && ids.contains(appuntamento.getId());
				if((soloNuovi && nuovo) || !soloNuovi) {
					try {
						ByteArrayOutputStream result = appuntamentoToHTMLString(utente, appuntamento,  true, true, html, null, !soloNuovi && nuovo, null);
						result.close();
						String nome = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.APPUNTAMENTO.stampa(), appuntamento.getContatto().getAnagrafica().getDenominazione()) +  "." + ConstantsMap.pdf;
						ApplicationUtil.addFile(files, result.toByteArray(), nome);
					} catch (Exception e) {
						ErrorLog.log(null, getClass(), e);
					}
				}
			}
			if (ApplicationUtil.notEmpty(files)) {
				try {
					String nomeFile = ApplicationUtil.reportFile(contatto.getData_lavorazione(), ConstantsLanguage.APPUNTAMENTI.stampa(), suffix) + "." + html;
					ByteArrayOutputStream os = new ByteArrayOutputStream();
					String w = getWatermark();
					ApplicationUtil.concatenaPDF(w, files, os, secretL);
					os.close();
					Allegato p = new Allegato(nomeFile, os.toByteArray());
					return p;
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				}
			}
		}
		return null;
	}
	public Allegato allegaReportContrattiConcatena(Utente utente, Contatto contatto, List<Long> ids, String secretL, boolean soloNuovi, Object [] suffix) {
		List<Contratto> contratti = getContrattiContatto(contatto);
		List<Allegato> files = new ArrayList<Allegato>();
		if(ApplicationUtil.notEmpty(contratti)) {
			String html = ConstantsMap.pdf;
			Iterator<Contratto> objIter = contratti.iterator();
			while(objIter.hasNext()) {
				Contratto contratto = objIter.next();
				boolean nuovo = ApplicationUtil.notEmpty(ids) && ids.contains(contratto.getId());
				if((soloNuovi && nuovo) || !soloNuovi) {
					try {
						ByteArrayOutputStream result = contrattoToHTMLString(utente, contratto,  true, true, html, null, !soloNuovi && nuovo, null);
						result.close();
						String nome = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.APPUNTAMENTO.stampa(), contratto.getContatto().getAnagrafica().getDenominazione()) +  "." + ConstantsMap.pdf;
						ApplicationUtil.addFile(files, result.toByteArray(), nome);
					} catch (Exception e) {
						ErrorLog.log(null, getClass(), e);
					}
				}
			}
			if (ApplicationUtil.notEmpty(files)) {
				try {
					String nomeFile = ApplicationUtil.reportFile(contatto.getData_lavorazione(), ConstantsLanguage.CONTRATTI.stampa(),  suffix) + "." + html;
					ByteArrayOutputStream os = new ByteArrayOutputStream();
					String w = getWatermark();
					ApplicationUtil.concatenaPDF(w, files, os, secretL);
					os.close();
					Allegato p = new Allegato(nomeFile, os.toByteArray());
					return p;
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				}
			}
		}
		return null;
	}
	
	public Appuntamento firstAppuntamento(List<Appuntamento> appuntamenti, List<Long> ids) {
		Appuntamento app = null;
		if(ApplicationUtil.notEmpty(ids) && ApplicationUtil.notEmpty(appuntamenti)) {
			Iterator<Appuntamento> objIter = appuntamenti.iterator();
			while(objIter.hasNext()) {
				Appuntamento appuntamento = objIter.next();
				if(ids.contains(appuntamento.getId()) && app == null) {
					app = appuntamento;
				}
			}
		}
		return app;
	}
	
	public void esportaExcelSondaggio(List<Sondaggio>  items, DownloadBean downloadBean) {
		try {
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.SONDAGGIO.stampa())+ "." + ConstantsMap.xlsx;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, items);
			String s = ConstantsGlobal.secretE;
			ExcelExport.esportaExcelSondaggio(items, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void esportaPdfSondaggio(List<Sondaggio>  items, DownloadBean downloadBean) {
		try {
			String file = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.SONDAGGIO.stampa())+ "." +ConstantsMap.pdf;

			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file, items);
			String s = ConstantsGlobal.secretP;
			PDFWriter.stampaPDFSondaggio(items, out, s);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	
	public void scaricaTemplate(MyFile file, DownloadBean downloadBean) {
		try {
			String home = impostazioneDao.getHome();
			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file.getName());
			InputStream stream = new FileInputStream(home + ConstantsGlobal.path_template + "/" + file.getName());  
			ApplicationUtil.copy(stream, out);
			stream.close();
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void salvaTemplate(String nome, FileUpload fileUpload) {
		List<MyFile> fls = fileUpload.getFiles();
		if(ApplicationUtil.notEmpty(fls)) {
			try {
				String home = impostazioneDao.getHome();
				Iterator<MyFile> flsIt = fls.iterator();
				while (flsIt.hasNext()) {
					MyFile ff = (MyFile) flsIt.next();
						if(nome != null) {
							File f = new File(home + ConstantsGlobal.path_template + "/" + nome + "." + ConstantsMap.html);
							FileOutputStream fis = new FileOutputStream(f);
							fis.write(ff.getData());
							fis.flush();
							fis.close();
						}
				}
			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		}
	}
	public void getTemplates(List<MyFile> mFiles, DownloadBean downloadBean) {
		try {
			String home = impostazioneDao.getHome();
			File path = new File(home + ConstantsGlobal.path_template);
			List<Allegato> files = new ArrayList<Allegato>();
			if (ApplicationUtil.notEmpty(mFiles)) {
				String p = ConstantsGlobal.secretZ;
				Iterator<MyFile> itS = mFiles.iterator();
				while (itS.hasNext()) {
					MyFile file = (MyFile) itS.next();
					ApplicationUtil.addFile(files, file.getName(), path+"/"+file.getName(), file.getModified());
				}
				String fileName = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.TEMPLATES.stampa()) +"." + ConstantsMap.zip;
				OutputStream out = downloadBean.outputStream();
				downloadBean.filename(fileName, files);
				ApplicationUtil.zip(null, files, out, p);
				downloadBean.close();
			}
		} catch(Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void salvaTemplate(MyFile ff, String content) throws Exception {
		String home = impostazioneDao.getHome();
		File file = new File(home + ConstantsGlobal.path_template + "/" + ff.getName());
		if (file.exists()) {
			FileOutputStream f = new FileOutputStream(file);
			PrintStream scrivi = new PrintStream(f);
			scrivi.println(content != null ? content.replaceAll(ConstantsGlobal.EOLINE, "") : content);
			scrivi.close();
		}
	}
	public void salvaTemplate(String nome, String content) throws Exception {
		String home = impostazioneDao.getHome();
		File file = new File(home + ConstantsGlobal.path_template + "/" + nome +  "." + ConstantsMap.html);
		//ApplicationUtil.creaFile(false, file);
		FileOutputStream f = new FileOutputStream(file);
		PrintStream scrivi = new PrintStream(f);
		scrivi.println(content != null ? content.replaceAll(ConstantsGlobal.EOLINE, "") : content);
		scrivi.close();
	}
	public void eliminaTemplates(List<MyFile> file) {
		if(ApplicationUtil.notEmpty(file)) {
			Iterator<MyFile> it = file.iterator();
			while (it.hasNext()) {
				MyFile myFile = (MyFile) it.next();
				eliminaTemplate(myFile);
			}
		}
	}
	public void eliminaTemplate(MyFile file) {
		String home = impostazioneDao.getHome();
		ApplicationUtil.deleteFile(home + ConstantsGlobal.path_template + "/" + file);
	}
	
	public List<MyFile> listaTemplates(FileCriteria mm) { 
		if(mm != null)  mm = (FileCriteria) mm.loadCriteria();
		List<MyFile> sis = new ArrayList<MyFile>();
		if(mm != null && mm.getCerca()) {
			String home = impostazioneDao.getHome();
			File perc = new File(home + ConstantsGlobal.path_template);
			ApplicationUtil.creaFile(true, perc);
			File [] ls = perc.listFiles(new ModuloFileFilter("." + ConstantsMap.html, false));
			if(ApplicationUtil.notEmpty(ls)) {
				List<String> rs = ApplicationUtil.token(mm.getFullText(), ConstantsGlobal.colonSeparator);
				for(File mod : ls) {
					String nameFile = ApplicationUtil.getBaseNameN(mod.getName()) + "." + ConstantsMap.html;
					if(mod.isFile() && mm.isRicerca(mod.getName(), mod.length(), mod.lastModified()) && ApplicationUtil.textInTXT(mod, rs)) {
						MyFile f = new MyFile(mod);
						f.setName(nameFile);
						f.setModified(mod.lastModified());
						sis.add(f);
					}
				}
			}
		}
		return sis;
	}

	
	public String readTemplate(MyFile ff) throws Exception {
		String home = impostazioneDao.getHome();
		File file = new File(home + ConstantsGlobal.path_template + "/" + ff.getName());
		if (file.exists()) {
			BufferedReader input = new BufferedReader(new FileReader(file));
			StringBuffer buffer = new StringBuffer();
			String text;
			while ((text = input.readLine()) != null)
				buffer.append(text + System.lineSeparator());
			input.close();
			return buffer.toString();
		}
		return null;
	}
	
	public String firmaToHTMLString(Contratto contratto) {
		Map<String, Object> model = new HashMap<String, Object>();
		model.put("mappa", loadValori(contratto));
		model.put("schedaContratto", impostazioneDao.getDocumentoDettagliContratto());
		model.put("schedaContatto", impostazioneDao.getDocumentoDettagliContatto());
		model.put("schedaAnagrafica", impostazioneDao.getDocumentoDettagliAnagrafica());
		model.put("schedaIndirizzoLegale", impostazioneDao.getDocumentoDettagliIndirizzoLegale());
		model.put("schedaIndirizzoOperativo", impostazioneDao.getDocumentoDettagliIndirizzoOperativo());
		model.put("schedaConsenso", impostazioneDao.getDocumentoDettagliConsenso());
		model.put("visibile", new ElementoScheda(impostazioneDao.getDocumentoSchedaContratto()));
		model.put("contratto", contratto);
		model.put("dettagliAnagrafica",  pannelloDao.getDatiAggiuntiviAnagrafica(contratto.getContatto().getAnagrafica().getId(), null));
		model.put("dettagliContratto",  pannelloDao.getDatiAggiuntiviContratto(contratto.getId(), null));
		model.put("raggruppa", impostazioneDao.getRaggruppaCampoAggiuntivo());
		return mergeVelocityTemplate("firma", model);
	}
	
	
	public String privacy(Anagrafica anagrafica) {
		if(anagrafica != null) {
			Map<String, Object> model = new HashMap<String, Object>();
			model.put("mappa", loadValori(anagrafica));
			model.put("schedaAnagrafica", impostazioneDao.getDocumentoDettagliAnagrafica());
			model.put("schedaIndirizzoLegale", impostazioneDao.getDocumentoDettagliIndirizzoLegale());
			model.put("schedaIndirizzoOperativo", impostazioneDao.getDocumentoDettagliIndirizzoOperativo());
			model.put("schedaConsenso", impostazioneDao.getDocumentoDettagliConsenso());
			model.put("visibile", new ElementoScheda(impostazioneDao.getDocumentoSchedaAnagrafica()));
			model.put("anagrafica", anagrafica);
			model.put("dettagliAnagrafica",  pannelloDao.getDatiAggiuntiviAnagrafica(anagrafica.getId(), null));
			model.put("raggruppa", impostazioneDao.getRaggruppaCampoAggiuntivo());
			return mergeVelocityTemplate("privacy", model);
		}
		return null;
	}
	
	
	public String pwaToHTMLString(SipBuddies c) {
		Map<String, Object> model = new HashMap<String, Object>();
		model.put("name", c.getName());
		model.put("password", c.getPassword());
		model.put("mailboxes", c.getMailboxes());
		model.put("phone", c.getPhone());
		model.put("call_group", c.getCall_group());
		model.put("pickup_group", c.getPickup_group());
		model.put("context", c.getContext());
		model.put("disallow", c.getDisallow());
		model.put("allow", c.getAllow());
		return mergeVelocityManifest("manifest.json",  model);
	}
	
	public String internoToHTMLString(SipBuddies c) {
		Map<String, Object> model = new HashMap<String, Object>();
		model.put("name", c.getName());
		model.put("password", c.getPassword());
		model.put("mailboxes", c.getMailboxes());
		model.put("phone", c.getPhone());
		model.put("call_group", c.getCall_group());
		model.put("pickup_group", c.getPickup_group());
		model.put("context", c.getContext());
		model.put("disallow", c.getDisallow());
		model.put("allow", c.getAllow());
		return mergeVelocityAsterisk("interni",  model);
	}
	
	public String trunkToHTMLString(SipBuddies c) {
		Map<String, Object> model = new HashMap<String, Object>();
		model.put("name", c.getName());
		model.put("host", c.getHost());
		model.put("port", c.getPort());
		model.put("max_contacts", c.getMax_contacts());
		model.put("username", c.getUsername());
		model.put("password", c.getPassword());
		model.put("contact_user", c.getContact_user());
		model.put("context", c.getContext());
		model.put("firewall", c.getFirewall());
		model.put("from_user", c.getFrom_user());
		model.put("call_group", c.getCall_group());
		model.put("pickup_group", c.getPickup_group());
		model.put("disallow", c.getDisallow());
		model.put("allow", c.getAllow());
		return mergeVelocityAsterisk("trunks",  model);
	}
	
	public String buddyToHTMLString(SipBuddies c, String server, Integer port) {
		Map<String, Object> model = new HashMap<String, Object>();
		model.put("name", c.getName());
		model.put("server", server);
		model.put("port", port);
		return mergeVelocityAsterisk("buddy",  model);
	}
	
	
	public String calendarToHTMLString(Utente c) {
		Map<String, Object> model = new HashMap<String, Object>();
		model.put("context", c.getId().toString() + "_" + c.getInternoString());
		model.put("id", c.getId().toString());
		model.put("interno", c.getInternoString());
		model.put("url", impostazioneDao.getWebUrl() + impostazioneDao.getLinkC(c));
		return mergeVelocityAsterisk("calendars",  model);
	}
	
	public void scaricaAllegato(RicezioneUtil item, DownloadBean downloadBean) {
		try {
			InputStream is = new ByteArrayInputStream(item.getContent());
			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(item.getNome());
			ApplicationUtil.copy(is, out);
			downloadBean.close();
			is.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	
	
	
	public Map<String, DatoAggiuntivo> loadValori(Contatto contatto) {
		if(contatto != null) {
			List <String> valoriValidi = new ArrayList<String>(ConstantsMap.anagraficaMap);
			Map<String, DatoAggiuntivo> valori = ApplicationUtil.convertiAnagrafica(contatto.getAnagrafica(), valoriValidi);
			List<DatoAggiuntivoAnagrafica> dettagliA = getDatiAggiuntiviAnagrafica(contatto.getAnagrafica().getId());
			if(ApplicationUtil.notEmpty(dettagliA)) {
				valori.putAll(ApplicationUtil.convertiDatoAggiuntivoA(dettagliA));
			}

			return valori;
		}
		return new HashMap<String, DatoAggiuntivo>();
	}
	
	
	public Map<String, DatoAggiuntivo> loadValori(Anagrafica anagrafica) {
		if(anagrafica != null) {
			List <String> valoriValidi = new ArrayList<String>(ConstantsMap.anagraficaMap);
			Map<String, DatoAggiuntivo> valori = ApplicationUtil.convertiAnagrafica(anagrafica, valoriValidi);
			List<DatoAggiuntivoAnagrafica> dettagliA = getDatiAggiuntiviAnagrafica(anagrafica.getId());
			if(ApplicationUtil.notEmpty(dettagliA)) {
				valori.putAll(ApplicationUtil.convertiDatoAggiuntivoA(dettagliA));
			}

			return valori;
		}
		return new HashMap<String, DatoAggiuntivo>();
	}
	
	
	public Map<String, DatoAggiuntivo> loadValori(Appuntamento appuntamento) {
		if(appuntamento != null) {
			List <String> valoriValidi = new ArrayList<String>(ConstantsMap.anagraficaMap);
			Map<String, DatoAggiuntivo> valori = ApplicationUtil.convertiAnagrafica(appuntamento.getContatto().getAnagrafica(), valoriValidi);
			List<DatoAggiuntivoAnagrafica> dettagliA = getDatiAggiuntiviAnagrafica(appuntamento.getContatto().getAnagrafica().getId());
			if(ApplicationUtil.notEmpty(dettagliA)) {
				valori.putAll(ApplicationUtil.convertiDatoAggiuntivoA(dettagliA));
			}

			return valori;
		}
		return new HashMap<String, DatoAggiuntivo>();
	}
	
	public Map<String, DatoAggiuntivo> loadValori(Contratto contratto) {
		if(contratto != null) {
			List <String> valoriValidi = new ArrayList<String>(ConstantsMap.anagraficaMap);
			Map<String, DatoAggiuntivo> valori = ApplicationUtil.convertiAnagrafica(contratto.getContatto().getAnagrafica(), valoriValidi);
			List<DatoAggiuntivoAnagrafica> dettagliA = getDatiAggiuntiviAnagrafica(contratto.getContatto().getAnagrafica().getId());
			if(ApplicationUtil.notEmpty(dettagliA)) {
				valori.putAll(ApplicationUtil.convertiDatoAggiuntivoA(dettagliA));
			}
			List<DatoAggiuntivoContratto> dettagliC = getDatiAggiuntiviContratto(contratto.getId());
			if(ApplicationUtil.notEmpty(dettagliC)) {
				valori.putAll(ApplicationUtil.convertiDatoAggiuntivoC(dettagliC));
			}
			return valori;
		}
		return new HashMap<String, DatoAggiuntivo>();
	}
	
	
	@SuppressWarnings("unchecked")
	public  List<DatoAggiuntivoAnagrafica> getDatiAggiuntiviAnagrafica(Long id) {
		if(id != null) {
			DetachedCriteria criteria = DetachedCriteria.forClass(DatoAggiuntivoAnagrafica.class);
			criteria.add(Restrictions.eq("anagrafica.id", id));
			return getList(criteria);
		}
		return new ArrayList<DatoAggiuntivoAnagrafica>();
	}
	@SuppressWarnings("unchecked")
	public List<DatoAggiuntivoContratto> getDatiAggiuntiviContratto(Long id) {
		if(id != null) {
			DetachedCriteria criteria = DetachedCriteria.forClass(DatoAggiuntivoContratto.class);
			criteria.add(Restrictions.eq("contratto.id", id));
			return getList(criteria);
		}
		return new ArrayList<DatoAggiuntivoContratto>();
	}
	
	
	public String scheda(DatoSondaggio dato, Map<String, DatoAggiuntivo> dati, Anagrafica anagrafica, Contatto contatto, Prodotto prodotto, String modulo) {
		if(modulo != null) {
			Map<String, Object> model = new HashMap<String, Object>();
			model.put("mappa", loadValori(anagrafica));
			model.put("dato", dato);
			model.put("dati", dati);
			model.put("anagrafica", anagrafica);
			model.put("contatto", contatto);
			model.put("prodotto", prodotto);
			model.put("dettagliAnagrafica", anagrafica != null ? pannelloDao.getDatiAggiuntiviAnagrafica(anagrafica.getId(), null) : null);
			return mergeVelocityPagina(modulo,  model);
		} 
		return null;
	}
	
	private void loadConfiguration(Map<String, Object> model) {
		model.put("appname", impostazioneDao.getApplicazione());
		model.put("link", impostazioneDao.getWebUrl());
		model.put("path", ConstantsGlobal.path_web);
		model.put("sdf", new DateFormatter(ConstantsFormat.dateTime.stampa()));
		model.put("sds", new DateFormatter(ConstantsFormat.dateSeconds.stampa()));
		model.put("data", new DateFormatter(ConstantsFormat.date.stampa()));
		model.put("ora", new DateFormatter(ConstantsFormat.time.stampa()));
		model.put("bean", new VelocityUtils());
		model.put("stampa", new StampaUtils());
	}
	
	public String mergeVelocityTemplate(String template, Map<String, Object> model) {
		String t = ConstantsGlobal.path_template + "/" + template + "." + ConstantsMap.html;
		String s = impostazioneDao.getHome() + ConstantsGlobal.path_template + "/" + template + "." + ConstantsMap.html;
		loadConfiguration(model);
		File f = new File(s);
		return velocityEngineUtil.useVelocityEngine(f.exists(), f.exists() ?  s : t, model);
	}
	
	public String mergeVelocityPagina(String template, Map<String, Object> model) {
		String s = impostazioneDao.getHome() + ConstantsGlobal.path_template + "/" + template + "." + ConstantsMap.html;
		loadConfiguration(model);
		File f = new File(s);
		if(f.exists()) {
			return velocityEngineUtil.useVelocityEngine(true, s, model);
		} 
		return "";
	}
	
	public String mergeVelocityManifest(String template, Map<String, Object> model) {
		String t = ConstantsGlobal.path_manifest + "/" + template;
		loadConfiguration(model);
		return velocityEngineUtil.useVelocityEngine(false, t, model);
	}
	
	public String mergeVelocityAsterisk(String template, Map<String, Object> model) {
		String t = ConstantsGlobal.path_centralino + "/" + template + "." + ConstantsMap.conf;
		loadConfiguration(model);
		return velocityEngineUtil.useVelocityEngine(false, t, model);
	}
	
	public String mergeVelocityMessage(String sorgente, Object o) {
		Map<String, Object> model = new HashMap<String, Object>();
		loadConfiguration(model);
		String s = ApplicationUtil.trimToEmpty(sorgente);
		if(s.contains("$!o")) {
			model.put("o", o);
			return velocityEngineUtil.useVelocityEngine(s, model);
		}
		return s;
	}

}