package it.callite.dao;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.OutputStream;
import java.io.Serializable;
import java.nio.file.DirectoryStream;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.SortedMap;

import javax.faces.model.SelectItem;

import org.apache.commons.net.ftp.FTPClient;
import org.apache.commons.net.ftp.FTPFile;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellType;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.ss.util.CellRangeAddress;
import org.hibernate.FetchMode;
import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.ProjectionList;
import org.hibernate.criterion.Projections;
import org.hibernate.criterion.Restrictions;
import org.hibernate.type.BooleanType;
import org.hibernate.type.IntegerType;
import org.hibernate.type.LongType;
import org.hibernate.type.Type;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import it.callite.action.DownloadBean;
import it.callite.chiamate.Chiamate;
import it.callite.constants.ConstantsEsito;
import it.callite.constants.ConstantsFormat;
import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.constants.ConstantsMap;
import it.callite.criteria.base.ExtendedCriteria;
import it.callite.criteria.report.BachecaCriteria;
import it.callite.criteria.report.ClassificaCriteria;
import it.callite.criteria.report.ProgressoCriteria;
import it.callite.criteria.report.ReportCriteria;
import it.callite.dao.generic.GenericDao;
import it.callite.log.ErrorLog;
import it.callite.model.Agente;
import it.callite.model.Appuntamento;
import it.callite.model.AutoContatto;
import it.callite.model.Campagna;
import it.callite.model.Contatto;
import it.callite.model.Contratto;
import it.callite.model.DatoAggiuntivoAnagrafica;
import it.callite.model.DatoAggiuntivoContratto;
import it.callite.model.Email;
import it.callite.model.EsitoAppuntamento;
import it.callite.model.EsitoArchiviazione;
import it.callite.model.EsitoChiamata;
import it.callite.model.EsitoContratto;
import it.callite.model.Lista;
import it.callite.model.Marcatura;
import it.callite.model.Notifica;
import it.callite.model.Operazione;
import it.callite.model.Pausa;
import it.callite.model.Qualifica;
import it.callite.model.Rmail;
import it.callite.model.Sondaggio;
import it.callite.model.StoriaAppuntamento;
import it.callite.model.StoriaContatto;
import it.callite.model.StoriaContratto;
import it.callite.model.Team;
import it.callite.model.Telecom;
import it.callite.model.Ufficio;
import it.callite.model.Utente;
import it.callite.model.Zona;
import it.callite.model.report.AgenteCampagnaBacheca;
import it.callite.model.report.AgenteCampagnaClassifica;
import it.callite.model.report.AgenteQualificaBacheca;
import it.callite.model.report.AgenteQualificaClassifica;
import it.callite.model.report.AgenteUfficioBacheca;
import it.callite.model.report.AgenteUfficioClassifica;
import it.callite.model.report.AgenteZonaBacheca;
import it.callite.model.report.AgenteZonaClassifica;
import it.callite.model.report.CampagnaBacheca;
import it.callite.model.report.Classifica;
import it.callite.model.report.ColonnaModel;
import it.callite.model.report.ConteggioModel;
import it.callite.model.report.Dettaglio;
import it.callite.model.report.DettaglioLavorato;
import it.callite.model.report.DettaglioLavoratoList;
import it.callite.model.report.OperatoreCampagnaBacheca;
import it.callite.model.report.OperatoreCampagnaClassifica;
import it.callite.model.report.OperatoreTeamBacheca;
import it.callite.model.report.OperatoreTeamClassifica;
import it.callite.model.report.PannelloModel;
import it.callite.model.report.PannelloReport;
import it.callite.model.report.ReportAppuntamento;
import it.callite.model.report.ReportAppuntamentoC;
import it.callite.model.report.ReportAppuntamentoCResultSet;
import it.callite.model.report.ReportAppuntamentoResultSet;
import it.callite.model.report.ReportContattiOdierni;
import it.callite.model.report.ReportContratto;
import it.callite.model.report.ReportContrattoC;
import it.callite.model.report.ReportContrattoCResultSet;
import it.callite.model.report.ReportContrattoResultSet;
import it.callite.model.report.ReportEsiti;
import it.callite.model.report.ReportEsitiCampagna;
import it.callite.model.report.ReportGenerale;
import it.callite.model.report.ReportGeneraleResultSet;
import it.callite.model.report.ReportLavorato;
import it.callite.model.report.ReportLavoratoResultSet;
import it.callite.model.report.ReportLista;
import it.callite.model.report.ReportListaEsito;
import it.callite.model.report.ReportListaEsitoResultSet;
import it.callite.model.report.ReportListaResultSet;
import it.callite.model.report.ReportOperatore;
import it.callite.model.report.ReportOperatoreResultSet;
import it.callite.model.report.ReportOrario;
import it.callite.model.report.ReportOrarioInserimento;
import it.callite.model.report.ReportSondaggio;
import it.callite.model.report.RiepilogoCampagna;
import it.callite.model.report.RiepilogoCampagnaList;
import it.callite.model.report.RiepilogoEsito;
import it.callite.model.report.RiepilogoEsitoAppoggio;
import it.callite.model.report.RiepilogoSondaggio;
import it.callite.model.report.SemaforoOperatore;
import it.callite.model.sip.Cdr;
import it.callite.utility.Allegato;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.ExcelFormat;
import it.callite.utility.ExcelFormatList;
import it.callite.utility.ExcelStyle;
import it.callite.utility.ExportUtility;
import it.callite.utility.MyFile;
import it.callite.utility.StatoIntervallo;
import it.callite.utility.UtilFTP;
import it.callite.utility.ZipPassword;
import it.callite.utility.item.CampagnaSI;
import it.callite.utility.item.TeamSI;
import it.callite.utility.utils.ModuloFileFilter;
@SuppressWarnings("serial")
@Repository("reportDao")
public class ReportDao extends GenericDao implements Serializable {
	@Autowired
	private CampagnaDao campagnaDao;
	@Autowired
	private UserDao userDao;
	@Autowired
	private ImpostazioneDao impostazioneDao;
	@Autowired
	private DataDao dataDao;
	@Autowired
	private DownloadDao downloadDao;
	
	@SuppressWarnings({ "unchecked", "unused" })
	private List<RiepilogoEsitoAppoggio> getRiepilogoEsiti(ReportCriteria r) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class, "cont");
		DetachedCriteria criteriaE= criteria.createCriteria("esitoChiamata", "esit");
		DetachedCriteria criteriaA= criteria.createCriteria("esitoArchiviazione", "arch",  ApplicationUtil.leftJoin());
		DetachedCriteria criteriaCA = criteria.createCriteria("campagna", "camp");
		DetachedCriteria criteriaLI = criteria.createCriteria("lista", "lista");
		if(r != null) {
			ApplicationUtil.intervallo(ApplicationUtil.isTrue(r.getInserimento()), r.getInizio(), r.getFine(), criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, criteriaLI);
			if(ApplicationUtil.gtZero(r.getLista())) {
				criteriaLI.add(Restrictions.idEq(r.getLista()));
			}
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("esit.id").as("eid")))
				.add(Projections.groupProperty("esit.etichetta").as("etichetta"))
				.add(Projections.groupProperty("esit.codice").as("codice"))
				.add(Projections.groupProperty("esit.colore").as("colore"))
				.add(Projections.sqlGroupProjection("positivo", "positivo", new String [] {"positivo"}, new Type []{BooleanType.INSTANCE}))
				.add(Projections.countDistinct("cont.id").as("numero"))
				);
		criteria.addOrder(Order.asc("esit.etichetta"));
		return getList(criteria, RiepilogoEsitoAppoggio.class);
	}
	@SuppressWarnings({ "unchecked", "unused" })
	private List<RiepilogoEsito> getRiepilogoEsitiChiamataStoria(ReportCriteria r){
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContatto.class, "sc");
		DetachedCriteria criteriaCO = criteria.createCriteria("contatto", "cont");
		DetachedCriteria criteriaE= criteria.createCriteria("esito", "esit");
		DetachedCriteria criteriaCA = criteriaCO.createCriteria("campagna", "camp");
		DetachedCriteria criteriaLI = criteriaCO.createCriteria("lista", "lista");
		if(r != null) {
			ApplicationUtil.pannello(r.getPannello(), criteria);
			ApplicationUtil.dataIntervalD(r.getInizio(), r.getFine(), "data", criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, criteriaLI);
			if(ApplicationUtil.gtZero(r.getLista())) {
				criteriaLI.add(Restrictions.idEq(r.getLista()));
			}
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("esit.id").as("eid")))
				.add(Projections.groupProperty("esit.etichetta").as("esito"))
				.add(Projections.groupProperty("esit.codice").as("codice"))
				.add(Projections.groupProperty("esit.colore").as("colore"))
				.add(Projections.countDistinct("sc.id").as("numero"))
				);
		criteria.addOrder(Order.asc("esit.etichetta"));
		return getList(criteria, RiepilogoEsito.class);
	}
	@SuppressWarnings({ "unchecked", "unused" })
	private List<RiepilogoEsito> getRiepilogoEsitiChiamataAutodialer(ReportCriteria r){
		DetachedCriteria criteria = DetachedCriteria.forClass(AutoContatto.class, "sc");
		DetachedCriteria criteriaCO = criteria.createCriteria("contatto", "cont");
		DetachedCriteria criteriaE= criteriaCO.createCriteria("esitoChiamata", "esit");
		DetachedCriteria criteriaCA = criteriaCO.createCriteria("campagna", "camp");
		DetachedCriteria criteriaLI = criteriaCO.createCriteria("lista", "lista");
		if(r != null) {
			ApplicationUtil.dataIntervalD(r.getInizio(), r.getFine(), "data_chiamata", criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, criteriaLI);
			if(ApplicationUtil.gtZero(r.getLista())) {
				criteriaLI.add(Restrictions.idEq(r.getLista()));
			}
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("esit.id").as("eid")))
				.add(Projections.groupProperty("esit.etichetta").as("esito"))
				.add(Projections.groupProperty("esit.codice").as("codice"))
				.add(Projections.groupProperty("esit.colore").as("colore"))
				.add(Projections.countDistinct("sc.id").as("numero"))
				);
		criteria.addOrder(Order.asc("esit.etichetta"));
		return getList(criteria, RiepilogoEsito.class);
	}
	@SuppressWarnings({ "unchecked", "unused" })
	private List<RiepilogoEsito> getRiepilogoEsitiChiamataAutodialer(Utente user, ReportCriteria r){
		DetachedCriteria criteria = DetachedCriteria.forClass(AutoContatto.class, "sc");
		DetachedCriteria criteriaCO = criteria.createCriteria("contatto", "cont");
		DetachedCriteria criteriaE= criteriaCO.createCriteria("esitoChiamata", "esit");
		DetachedCriteria criteriaCA = criteriaCO.createCriteria("campagna", "camp");
		DetachedCriteria criteriaLI = criteriaCO.createCriteria("lista", "lista");
		impostazioneDao.subCriteriaCampagnaSupervisore(user.getId(), criteriaCA, "id");
		if(r != null) {
			ApplicationUtil.dataIntervalD(r.getInizio(), r.getFine(), "data_chiamata", criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, criteriaLI);
			if(ApplicationUtil.gtZero(r.getLista())) {
				criteriaLI.add(Restrictions.idEq(r.getLista()));
			}
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("esit.id").as("eid")))
				.add(Projections.groupProperty("esit.etichetta").as("esito"))
				.add(Projections.groupProperty("esit.codice").as("codice"))
				.add(Projections.groupProperty("esit.colore").as("colore"))
				.add(Projections.countDistinct("sc.id").as("numero"))
				);
		criteria.addOrder(Order.asc("esit.etichetta"));
		return getList(criteria, RiepilogoEsito.class);
	}
	@SuppressWarnings({ "unchecked", "unused" })
	private List<RiepilogoEsito> getRiepilogoEsitiChiamataAutodialerPartner(Utente user, ReportCriteria r){
		DetachedCriteria criteria = DetachedCriteria.forClass(AutoContatto.class, "sc");
		DetachedCriteria criteriaCO = criteria.createCriteria("contatto", "cont");
		DetachedCriteria criteriaE= criteriaCO.createCriteria("esitoChiamata", "esit");
		DetachedCriteria criteriaCA = criteriaCO.createCriteria("campagna", "camp");
		DetachedCriteria criteriaLI = criteriaCO.createCriteria("lista", "lista");
		impostazioneDao.subCriteriaCampagnaPartner(user.getId(), criteriaCA, "id");
		if(r != null) {
			ApplicationUtil.dataIntervalD(r.getInizio(), r.getFine(), "data_chiamata", criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, criteriaLI);
			if(ApplicationUtil.gtZero(r.getLista())) {
				criteriaLI.add(Restrictions.idEq(r.getLista()));
			}
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("esit.id").as("eid")))
				.add(Projections.groupProperty("esit.etichetta").as("esito"))
				.add(Projections.groupProperty("esit.codice").as("codice"))
				.add(Projections.groupProperty("esit.colore").as("colore"))
				.add(Projections.countDistinct("sc.id").as("numero"))
				);
		criteria.addOrder(Order.asc("esit.etichetta"));
		return getList(criteria, RiepilogoEsito.class);
	}
	@SuppressWarnings({ "unchecked", "unused" })
	private List<RiepilogoEsitoAppoggio> getRiepilogoEsitiArchiviazioneStoria(ReportCriteria r) {
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContatto.class, "sc");
		DetachedCriteria criteriaCO = criteria.createCriteria("contatto", "cont");
		DetachedCriteria criteriaA= criteria.createCriteria("archiviazione", "arch");
		DetachedCriteria criteriaCA = criteriaCO.createCriteria("campagna", "camp");
		DetachedCriteria criteriaLI = criteriaCO.createCriteria("lista", "lista");
		if(r != null) {
			ApplicationUtil.pannello(r.getPannello(), criteria);
			ApplicationUtil.dataIntervalD(r.getInizio(), r.getFine(), "data", criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, criteriaLI);
			if(ApplicationUtil.gtZero(r.getLista())) {
				criteriaLI.add(Restrictions.idEq(r.getLista()));
			}
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("arch.id").as("eid")))
				.add(Projections.groupProperty("arch.etichetta").as("etichetta"))
				.add(Projections.groupProperty("arch.codice").as("codice"))
				.add(Projections.groupProperty("arch.colore").as("colore"))
				.add(Projections.groupProperty("arch.positivo").as("positivo"))
				.add(Projections.groupProperty("arch.utile").as("utile"))
				.add(Projections.countDistinct("sc.id").as("numero"))
				);
		criteria.addOrder(Order.desc("arch.id"));
		return getList(criteria, RiepilogoEsitoAppoggio.class);
	}
	@SuppressWarnings({ "unchecked", "unused" })
	private List<RiepilogoEsitoAppoggio> getCountEsitiArchiviazioneByCriteria(Integer pannello, Long campagnaId, Date dataInizio, Date dataFine) {
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContatto.class, "sc");
		DetachedCriteria criteriaA= criteria.createCriteria("archiviazione", "arch",  ApplicationUtil.leftJoin());
		DetachedCriteria criteriaC = criteria.createCriteria("contatto", "cont");
		DetachedCriteria criteriaCA = criteriaC.createCriteria("campagna", "camp");
		criteriaCA.add(Restrictions.idEq(campagnaId));
		ApplicationUtil.dataIntervalD(dataInizio, dataFine, "data", criteria);
		ApplicationUtil.pannello(pannello, criteria);
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("arch.positivo").as("positivo")))
				.add(Projections.groupProperty("arch.utile").as("utile"))
				.add(Projections.countDistinct("sc.id").as("numero"))
				);
		criteria.addOrder(Order.desc("numero"));
		return getList(criteria, RiepilogoEsitoAppoggio.class);
	}
	@SuppressWarnings({ "unchecked" })
	public List<AgenteZonaClassifica> getZonaClassifica(Long zonaId, Date dataInizio, Date dataFine,  Integer limite, Integer tipo) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class, "sc");
		DetachedCriteria criteriaZ = criteria.createCriteria("zone", "z");
		criteriaZ.add(Restrictions.idEq(zonaId));
		if(ApplicationUtil.eq(tipo,1)) {
			DetachedCriteria criteriaC = criteria.createCriteria("contatti", "ap");
			criteriaC.add(Restrictions.eq("tipo", ConstantsGlobal.inviato));
			ApplicationUtil.intervallo(true, dataInizio, dataFine, criteriaC);
		}
		if(ApplicationUtil.eq(tipo,2)) {
			DetachedCriteria criteriaC = criteria.createCriteria("appuntamenti", "ap");
			DetachedCriteria criteriaA = criteriaC.createCriteria("esito");
			criteriaA.add(Restrictions.eq("confermato", Boolean.TRUE));
			ApplicationUtil.intervallo(true, dataInizio, dataFine, criteriaC);
		}
		if(ApplicationUtil.eq(tipo,3)) {
			DetachedCriteria criteriaC = criteria.createCriteria("contratti", "ap");
			DetachedCriteria criteriaA = criteriaC.createCriteria("esito");
			criteriaA.add(Restrictions.eq("confermato", Boolean.TRUE));
			ApplicationUtil.intervallo(true, dataInizio, dataFine, criteriaC);
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("sc.id").as("id")))
				.add(Projections.groupProperty("sc.codice").as("codice"))
				.add(Projections.groupProperty("sc.nome").as("nome"))
				.add(Projections.groupProperty("sc.cognome").as("cognome"))
				.add(Projections.groupProperty("sc.pf").as("pf"))
				.add(Projections.groupProperty("sc.nome_completo").as("nome_completo"))
				.add(Projections.groupProperty("sc.ragione_sociale").as("ragione_sociale"))
				.add(Projections.countDistinct("ap.id").as("numero")));
		criteria.addOrder(Order.desc("numero")).addOrder(Order.asc("sc.id"));
		return getList(criteria, 0, limite, AgenteZonaClassifica.class);
	}
	@SuppressWarnings({ "unchecked" })
	public List<OperatoreTeamClassifica> getTeamClassificaO(Long teamId, Date dataInizio, Date dataFine,  Integer limite, Integer tipo) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class, "sc");
		DetachedCriteria criteriaZ = criteria.createCriteria("team", "z");
		criteriaZ.add(Restrictions.idEq(teamId));
		if(ApplicationUtil.eq(tipo,1)) {
			DetachedCriteria criteriaC = criteria.createCriteria("contatti", "ap");
			ApplicationUtil.intervallo(true, dataInizio, dataFine, criteriaC);
		}
		if(ApplicationUtil.eq(tipo,2)) {
			DetachedCriteria criteriaC = criteria.createCriteria("appuntamenti", "ap");
			ApplicationUtil.intervallo(true, dataInizio, dataFine, criteriaC);
		}
		if(ApplicationUtil.eq(tipo,3)) {
			DetachedCriteria criteriaC = criteria.createCriteria("contratti", "ap");
			ApplicationUtil.intervallo(true, dataInizio, dataFine, criteriaC);
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("sc.id").as("id")))
				.add(Projections.groupProperty("sc.nome").as("nome"))
				.add(Projections.groupProperty("sc.cognome").as("cognome"))
				.add(Projections.groupProperty("sc.username").as("username"))
				.add(Projections.countDistinct("ap.id").as("numero")));
		criteria.addOrder(Order.desc("numero")).addOrder(Order.asc("sc.id"));
		return getList(criteria, 0, limite, OperatoreTeamClassifica.class);
	}
	@SuppressWarnings({ "unchecked" })
	public List<AgenteCampagnaClassifica> getCampagnaClassifica(Long campagnaId, Date dataInizio, Date dataFine,  Integer limite, Integer tipo) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class, "sc");
		DetachedCriteria criteriaZ = criteria.createCriteria("zone").createCriteria("campagne", "q");
		criteriaZ.add(Restrictions.idEq(campagnaId));
		if(ApplicationUtil.eq(tipo,1)) {
			DetachedCriteria criteriaC = criteria.createCriteria("contatti", "ap");
			criteriaC.add(Restrictions.eq("campagna.id",campagnaId));
			criteriaC.add(Restrictions.eq("tipo", ConstantsGlobal.inviato));
			ApplicationUtil.intervallo(true, dataInizio, dataFine, criteriaC);
		}
		if(ApplicationUtil.eq(tipo,2)) {
			DetachedCriteria criteriaC = criteria.createCriteria("appuntamenti", "ap");
			criteriaC.createCriteria("contatto").add(Restrictions.eq("campagna.id",campagnaId));
			DetachedCriteria criteriaA = criteriaC.createCriteria("esito");
			criteriaA.add(Restrictions.eq("confermato", Boolean.TRUE));
			ApplicationUtil.intervallo(true, dataInizio, dataFine, criteriaC);
		}
		if(ApplicationUtil.eq(tipo,3)) {
			DetachedCriteria criteriaC = criteria.createCriteria("contratti", "ap");
			criteriaC.createCriteria("contatto").add(Restrictions.eq("campagna.id",campagnaId));
			DetachedCriteria criteriaA = criteriaC.createCriteria("esito");
			criteriaA.add(Restrictions.eq("confermato", Boolean.TRUE));
			ApplicationUtil.intervallo(true, dataInizio, dataFine, criteriaC);
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("sc.id").as("id")))
				.add(Projections.groupProperty("sc.codice").as("codice"))
				.add(Projections.groupProperty("sc.nome").as("nome"))
				.add(Projections.groupProperty("sc.cognome").as("cognome"))
				.add(Projections.groupProperty("sc.pf").as("pf"))
				.add(Projections.groupProperty("sc.nome_completo").as("nome_completo"))
				.add(Projections.groupProperty("sc.ragione_sociale").as("ragione_sociale"))
				.add(Projections.countDistinct("ap.id").as("numero")));
		criteria.addOrder(Order.desc("numero")).addOrder(Order.asc("sc.id"));
		return getList(criteria, 0, limite, AgenteCampagnaClassifica.class);
	}
	@SuppressWarnings({ "unchecked" })
	public List<AgenteQualificaClassifica> getQualificaClassifica(Long qualificaId, Date dataInizio, Date dataFine,  Integer limite, Integer tipo) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class, "sc");
		DetachedCriteria criteriaZ = criteria.createCriteria("qualifica", "q", ApplicationUtil.leftJoin());
		criteriaZ.add(Restrictions.idEq(qualificaId));
		if(ApplicationUtil.eq(tipo,1)) {
			DetachedCriteria criteriaC = criteria.createCriteria("contatti", "ap");
			criteriaC.add(Restrictions.eq("tipo", ConstantsGlobal.inviato));
			ApplicationUtil.intervallo(true, dataInizio, dataFine, criteriaC);
		}
		if(ApplicationUtil.eq(tipo,2)) {
			DetachedCriteria criteriaC = criteria.createCriteria("appuntamenti", "ap");
			DetachedCriteria criteriaA = criteriaC.createCriteria("esito");
			criteriaA.add(Restrictions.eq("confermato", Boolean.TRUE));
			ApplicationUtil.intervallo(true, dataInizio, dataFine, criteriaC);
		}
		if(ApplicationUtil.eq(tipo,3)) {
			DetachedCriteria criteriaC = criteria.createCriteria("contratti", "ap");
			DetachedCriteria criteriaA = criteriaC.createCriteria("esito");
			criteriaA.add(Restrictions.eq("confermato", Boolean.TRUE));
			ApplicationUtil.intervallo(true, dataInizio, dataFine, criteriaC);
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("sc.id").as("id")))
				.add(Projections.groupProperty("sc.codice").as("codice"))
				.add(Projections.groupProperty("sc.nome").as("nome"))
				.add(Projections.groupProperty("sc.cognome").as("cognome"))
				.add(Projections.groupProperty("sc.pf").as("pf"))
				.add(Projections.groupProperty("sc.nome_completo").as("nome_completo"))
				.add(Projections.groupProperty("sc.ragione_sociale").as("ragione_sociale"))
				.add(Projections.countDistinct("ap.id").as("numero")));
		criteria.addOrder(Order.desc("numero")).addOrder(Order.asc("sc.id"));
		return getList(criteria, 0, limite, AgenteQualificaClassifica.class);
	}
	@SuppressWarnings({ "unchecked" })
	public List<AgenteUfficioClassifica> getUfficioClassifica(Long ufficioId, Date dataInizio, Date dataFine,  Integer limite, Integer tipo) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class, "sc");
		DetachedCriteria criteriaZ = criteria.createCriteria("ufficio", "q", ApplicationUtil.leftJoin());
		criteriaZ.add(Restrictions.idEq(ufficioId));
		if(ApplicationUtil.eq(tipo,1)) {
			DetachedCriteria criteriaC = criteria.createCriteria("contatti", "ap");
			criteriaC.add(Restrictions.eq("tipo", ConstantsGlobal.inviato));
			ApplicationUtil.intervallo(true, dataInizio, dataFine, criteriaC);
		}
		if(ApplicationUtil.eq(tipo,2)) {
			DetachedCriteria criteriaC = criteria.createCriteria("appuntamenti", "ap");
			DetachedCriteria criteriaA = criteriaC.createCriteria("esito");
			criteriaA.add(Restrictions.eq("confermato", Boolean.TRUE));
			ApplicationUtil.intervallo(true, dataInizio, dataFine, criteriaC);
		}
		if(ApplicationUtil.eq(tipo,3)) {
			DetachedCriteria criteriaC = criteria.createCriteria("contratti", "ap");
			DetachedCriteria criteriaA = criteriaC.createCriteria("esito");
			criteriaA.add(Restrictions.eq("confermato", Boolean.TRUE));
			ApplicationUtil.intervallo(true, dataInizio, dataFine, criteriaC);
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("sc.id").as("id")))
				.add(Projections.groupProperty("sc.codice").as("codice"))
				.add(Projections.groupProperty("sc.nome").as("nome"))
				.add(Projections.groupProperty("sc.cognome").as("cognome"))
				.add(Projections.groupProperty("sc.pf").as("pf"))
				.add(Projections.groupProperty("sc.nome_completo").as("nome_completo"))
				.add(Projections.groupProperty("sc.ragione_sociale").as("ragione_sociale"))
				.add(Projections.countDistinct("ap.id").as("numero")));
		criteria.addOrder(Order.desc("numero")).addOrder(Order.asc("sc.id"));
		return getList(criteria, 0, limite, AgenteUfficioClassifica.class);
	}
	@SuppressWarnings({ "unchecked" })
	public List<AgenteCampagnaClassifica> getCampagnaClassificaR(Long campagnaId, Date dataInizio, Date dataFine,  Integer limite, Integer tipo) {
		DetachedCriteria criteriaAg = DetachedCriteria.forClass(Agente.class, "sc");
		DetachedCriteria criteriaCl = criteriaAg.createCriteria("closure_agenti_padre", "cl");
		criteriaCl.add(Restrictions.ge("distanza", 1));
		Integer max = userDao.getMaxLivelloRete();
		if(ApplicationUtil.gtZero(max)) {
			criteriaCl.add(Restrictions.le("distanza", max));
		}
		DetachedCriteria criteria = criteriaCl.createCriteria("agente", "agf");
		DetachedCriteria criteriaZ = criteria.createCriteria("zone").createCriteria("campagne", "q");
		criteriaZ.add(Restrictions.idEq(campagnaId));
		if(ApplicationUtil.eq(tipo,1)) {
			DetachedCriteria criteriaC = criteria.createCriteria("contatti", "ap");
			criteriaC.add(Restrictions.eq("campagna.id",campagnaId));
			criteriaC.add(Restrictions.eq("tipo", ConstantsGlobal.inviato));
			ApplicationUtil.intervallo(true, dataInizio, dataFine, criteriaC);
		}
		if(ApplicationUtil.eq(tipo,2)) {
			DetachedCriteria criteriaC = criteria.createCriteria("appuntamenti", "ap");
			criteriaC.createCriteria("contatto").add(Restrictions.eq("campagna.id",campagnaId));
			DetachedCriteria criteriaA = criteriaC.createCriteria("esito");
			criteriaA.add(Restrictions.eq("confermato", Boolean.TRUE));
			ApplicationUtil.intervallo(true, dataInizio, dataFine, criteriaC);
		}
		if(ApplicationUtil.eq(tipo,3)) {
			DetachedCriteria criteriaC = criteria.createCriteria("contratti", "ap");
			criteriaC.createCriteria("contatto").add(Restrictions.eq("campagna.id",campagnaId));
			DetachedCriteria criteriaA = criteriaC.createCriteria("esito");
			criteriaA.add(Restrictions.eq("confermato", Boolean.TRUE));
			ApplicationUtil.intervallo(true, dataInizio, dataFine, criteriaC);
		}
		criteriaAg.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("sc.id").as("id")))
				.add(Projections.groupProperty("sc.codice").as("codice"))
				.add(Projections.groupProperty("sc.nome").as("nome"))
				.add(Projections.groupProperty("sc.cognome").as("cognome"))
				.add(Projections.groupProperty("sc.pf").as("pf"))
				.add(Projections.groupProperty("sc.nome_completo").as("nome_completo"))
				.add(Projections.groupProperty("sc.ragione_sociale").as("ragione_sociale"))
				.add(Projections.countDistinct("ap.id").as("numero")));
		criteria.addOrder(Order.desc("numero")).addOrder(Order.asc("sc.id"));
		return getList(criteriaAg, 0, limite, AgenteCampagnaClassifica.class);
	}
	@SuppressWarnings({ "unchecked" })
	public List<AgenteZonaClassifica> getZonaClassificaR(Long zonaId, Date dataInizio, Date dataFine,  Integer limite, Integer tipo) {
		DetachedCriteria criteriaAg = DetachedCriteria.forClass(Agente.class, "sc");
		DetachedCriteria criteriaCl = criteriaAg.createCriteria("closure_agenti_padre", "cl");
		criteriaCl.add(Restrictions.ge("distanza", 1));
		Integer max = userDao.getMaxLivelloRete();
		if(ApplicationUtil.gtZero(max)) {
			criteriaCl.add(Restrictions.le("distanza", max));
		}
		DetachedCriteria criteria = criteriaCl.createCriteria("agente", "agf");
		DetachedCriteria criteriaZ = criteria.createCriteria("zone", "z");
		criteriaZ.add(Restrictions.idEq(zonaId));
		if(ApplicationUtil.eq(tipo,1)) {
			DetachedCriteria criteriaC = criteria.createCriteria("contatti", "ap");
			criteriaC.add(Restrictions.eq("tipo", ConstantsGlobal.inviato));
			ApplicationUtil.intervallo(true, dataInizio, dataFine, criteriaC);
		}
		if(ApplicationUtil.eq(tipo,2)) {
			DetachedCriteria criteriaC = criteria.createCriteria("appuntamenti", "ap");
			DetachedCriteria criteriaA = criteriaC.createCriteria("esito");
			criteriaA.add(Restrictions.eq("confermato", Boolean.TRUE));
			ApplicationUtil.intervallo(true, dataInizio, dataFine, criteriaC);
		}
		if(ApplicationUtil.eq(tipo,3)) {
			DetachedCriteria criteriaC = criteria.createCriteria("contratti", "ap");
			DetachedCriteria criteriaA = criteriaC.createCriteria("esito");
			criteriaA.add(Restrictions.eq("confermato", Boolean.TRUE));
			ApplicationUtil.intervallo(true, dataInizio, dataFine, criteriaC);
		}
		criteriaAg.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("sc.id").as("id")))
				.add(Projections.groupProperty("sc.codice").as("codice"))
				.add(Projections.groupProperty("sc.nome").as("nome"))
				.add(Projections.groupProperty("sc.cognome").as("cognome"))
				.add(Projections.groupProperty("sc.pf").as("pf"))
				.add(Projections.groupProperty("sc.nome_completo").as("nome_completo"))
				.add(Projections.groupProperty("sc.ragione_sociale").as("ragione_sociale"))
				.add(Projections.countDistinct("ap.id").as("numero")));
		criteria.addOrder(Order.desc("numero")).addOrder(Order.asc("sc.id"));
		return getList(criteriaAg, 0, limite, AgenteZonaClassifica.class);
	}
	@SuppressWarnings({ "unchecked" })
	public List<OperatoreCampagnaClassifica> getCampagnaClassificaO(Long campagnaId, Date dataInizio, Date dataFine,  Integer limite, Integer tipo) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class, "sc");
		DetachedCriteria criteriaZ = criteria.createCriteria("team").createCriteria("campagne", "q");
		criteriaZ.add(Restrictions.idEq(campagnaId));
		if(ApplicationUtil.eq(tipo,1)) {
			DetachedCriteria criteriaC = criteria.createCriteria("contatti", "ap");
			criteriaC.add(Restrictions.eq("campagna.id",campagnaId));
			ApplicationUtil.intervallo(true, dataInizio, dataFine, criteriaC);
		}
		if(ApplicationUtil.eq(tipo,2)) {
			DetachedCriteria criteriaC = criteria.createCriteria("appuntamenti", "ap");
			criteriaC.createCriteria("contatto").add(Restrictions.eq("campagna.id",campagnaId));
			ApplicationUtil.intervallo(true, dataInizio, dataFine, criteriaC);
		}
		if(ApplicationUtil.eq(tipo,3)) {
			DetachedCriteria criteriaC = criteria.createCriteria("contratti", "ap");
			criteriaC.createCriteria("contatto").add(Restrictions.eq("campagna.id",campagnaId));
			ApplicationUtil.intervallo(true, dataInizio, dataFine, criteriaC);
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("sc.id").as("id")))
				.add(Projections.groupProperty("sc.nome").as("nome"))
				.add(Projections.groupProperty("sc.cognome").as("cognome"))
				.add(Projections.groupProperty("sc.username").as("username"))
				.add(Projections.countDistinct("ap.id").as("numero")));
		criteria.addOrder(Order.desc("numero")).addOrder(Order.asc("sc.id"));
		return getList(criteria, 0, limite, OperatoreCampagnaClassifica.class);
	}
	@SuppressWarnings({ "unchecked" })
	public List<AgenteQualificaClassifica> getQualificaClassificaR(Long qualificaId, Date dataInizio, Date dataFine,  Integer limite, Integer tipo) {
		DetachedCriteria criteriaAg = DetachedCriteria.forClass(Agente.class, "sc");
		DetachedCriteria criteriaCl = criteriaAg.createCriteria("closure_agenti_padre", "cl");
		criteriaCl.add(Restrictions.ge("distanza", 1));
		Integer max = userDao.getMaxLivelloRete();
		if(ApplicationUtil.gtZero(max)) {
			criteriaCl.add(Restrictions.le("distanza", max));
		}
		DetachedCriteria criteria = criteriaCl.createCriteria("agente", "agf");
		DetachedCriteria criteriaZ = criteriaAg.createCriteria("qualifica", "q", ApplicationUtil.leftJoin());
		criteriaZ.add(Restrictions.idEq(qualificaId));
		if(ApplicationUtil.eq(tipo,1)) {
			DetachedCriteria criteriaC = criteria.createCriteria("contatti", "ap");
			criteriaC.add(Restrictions.eq("tipo", ConstantsGlobal.inviato));
			ApplicationUtil.intervallo(true, dataInizio, dataFine, criteriaC);
		}
		if(ApplicationUtil.eq(tipo,2)) {
			DetachedCriteria criteriaC = criteria.createCriteria("appuntamenti", "ap");
			DetachedCriteria criteriaA = criteriaC.createCriteria("esito");
			criteriaA.add(Restrictions.eq("confermato", Boolean.TRUE));
			ApplicationUtil.intervallo(true, dataInizio, dataFine, criteriaC);
		}
		if(ApplicationUtil.eq(tipo,3)) {
			DetachedCriteria criteriaC = criteria.createCriteria("contratti", "ap");
			DetachedCriteria criteriaA = criteriaC.createCriteria("esito");
			criteriaA.add(Restrictions.eq("confermato", Boolean.TRUE));
			ApplicationUtil.intervallo(true, dataInizio, dataFine, criteriaC);
		}
		criteriaAg.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("sc.id").as("id")))
				.add(Projections.groupProperty("sc.codice").as("codice"))
				.add(Projections.groupProperty("sc.nome").as("nome"))
				.add(Projections.groupProperty("sc.cognome").as("cognome"))
				.add(Projections.groupProperty("sc.pf").as("pf"))
				.add(Projections.groupProperty("sc.nome_completo").as("nome_completo"))
				.add(Projections.groupProperty("sc.ragione_sociale").as("ragione_sociale"))
				.add(Projections.countDistinct("ap.id").as("numero")));
		criteria.addOrder(Order.desc("numero")).addOrder(Order.asc("sc.id"));
		return getList(criteriaAg, 0, limite, AgenteQualificaClassifica.class);
	}
	@SuppressWarnings({ "unchecked" })
	public List<AgenteUfficioClassifica> getUfficioClassificaR(Long ufficioId, Date dataInizio, Date dataFine,  Integer limite, Integer tipo) {
		DetachedCriteria criteriaAg = DetachedCriteria.forClass(Agente.class, "sc");
		DetachedCriteria criteriaCl = criteriaAg.createCriteria("closure_agenti_padre", "cl");
		criteriaCl.add(Restrictions.ge("distanza", 1));
		Integer max = userDao.getMaxLivelloRete();
		if(ApplicationUtil.gtZero(max)) {
			criteriaCl.add(Restrictions.le("distanza", max));
		}
		DetachedCriteria criteria = criteriaCl.createCriteria("agente", "agf");
		DetachedCriteria criteriaZ = criteriaAg.createCriteria("ufficio", "q", ApplicationUtil.leftJoin());
		criteriaZ.add(Restrictions.idEq(ufficioId));
		if(ApplicationUtil.eq(tipo,1)) {
			DetachedCriteria criteriaC = criteria.createCriteria("contatti", "ap");
			criteriaC.add(Restrictions.eq("tipo", ConstantsGlobal.inviato));
			ApplicationUtil.intervallo(true, dataInizio, dataFine, criteriaC);
		}
		if(ApplicationUtil.eq(tipo,2)) {
			DetachedCriteria criteriaC = criteria.createCriteria("appuntamenti", "ap");
			DetachedCriteria criteriaA = criteriaC.createCriteria("esito");
			criteriaA.add(Restrictions.eq("confermato", Boolean.TRUE));
			ApplicationUtil.intervallo(true, dataInizio, dataFine, criteriaC);
		}
		if(ApplicationUtil.eq(tipo,3)) {
			DetachedCriteria criteriaC = criteria.createCriteria("contratti", "ap");
			DetachedCriteria criteriaA = criteriaC.createCriteria("esito");
			criteriaA.add(Restrictions.eq("confermato", Boolean.TRUE));
			ApplicationUtil.intervallo(true, dataInizio, dataFine, criteriaC);
		}
		criteriaAg.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("sc.id").as("id")))
				.add(Projections.groupProperty("sc.codice").as("codice"))
				.add(Projections.groupProperty("sc.nome").as("nome"))
				.add(Projections.groupProperty("sc.cognome").as("cognome"))
				.add(Projections.groupProperty("sc.pf").as("pf"))
				.add(Projections.groupProperty("sc.nome_completo").as("nome_completo"))
				.add(Projections.groupProperty("sc.ragione_sociale").as("ragione_sociale"))
				.add(Projections.countDistinct("ap.id").as("numero")));
		criteria.addOrder(Order.desc("numero")).addOrder(Order.asc("sc.id"));
		return getList(criteriaAg, 0, limite, AgenteUfficioClassifica.class);
	}
	@SuppressWarnings("unchecked")
	public List<Campagna> getCampagneAttive(Long team_id, int size) {
		Boolean ts = impostazioneDao.getEstrazioneStato();
		DetachedCriteria campagnaC = DetachedCriteria.forClass(Campagna.class);
		if(ApplicationUtil.gtZero(team_id)){
			campagnaC.createCriteria("team").add(Restrictions.idEq(team_id));
		}
		ApplicationUtil.criteriaStato(campagnaC, true, ApplicationUtil.isTrue(ts));
		ApplicationUtil.orderC(campagnaC);
		return getList(campagnaC, 0, size);
	}
	@SuppressWarnings("unchecked")
	public List<Zona> getZone(Long zona_id, int size) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Zona.class);
		if(ApplicationUtil.gtZero(zona_id)){
			criteria.add(Restrictions.idEq(zona_id));
		}
		ApplicationUtil.orderZ(criteria);
		return getList(criteria, 0, size);
	}
	@SuppressWarnings("unchecked")
	public List<Ufficio> getUffici(Long ufficio_id, int size) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Ufficio.class);
		if(ApplicationUtil.gtZero(ufficio_id)){
			criteria.add(Restrictions.idEq(ufficio_id));
		}
		return getList(criteria, 0, size);
	}
	@SuppressWarnings("unchecked")
	public List<Qualifica> getQualifiche(Long qualifica_id, int size) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Qualifica.class);
		if(ApplicationUtil.gtZero(qualifica_id)){
			criteria.add(Restrictions.idEq(qualifica_id));
		}
		return getList(criteria, 0, size);
	}
	@SuppressWarnings("unchecked")
	public List<Team> getTeam(Long team_id, int size) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Team.class);
		if(ApplicationUtil.gtZero(team_id)){
			criteria.add(Restrictions.idEq(team_id));
		}
		ApplicationUtil.orderT(criteria);
		return getList(criteria, 0, size);
	}
	@SuppressWarnings("unchecked")
	public List<Campagna> getCampagne(Long campagna_id, int size) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		if(ApplicationUtil.gtZero(campagna_id)){
			criteria.add(Restrictions.idEq(campagna_id));
		}
		ApplicationUtil.orderC(criteria);
		return getList(criteria, 0, size);
	}
	public int getRichiamiPubbliciSize(Long campagnaId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class);
		criteria.add(Restrictions.eq("campagna.id",campagnaId));
		criteria.add(Restrictions.isNull("proprietario.id"));
		ApplicationUtil.archiviazione(criteria, ApplicationUtil.date(), false);
		return getListSize(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Campagna> getCampagneAttiveReport(int start, int size) {
		Boolean ts = impostazioneDao.getEstrazioneStato();
		DetachedCriteria campagnaC = DetachedCriteria.forClass(Campagna.class);
		ApplicationUtil.criteriaStato(campagnaC, true, ApplicationUtil.isTrue(ts));
		return getList(campagnaC);
	}
	@SuppressWarnings({ "unchecked", "unused" })
	public List<Classifica> getClassificaByCampagnaId(Integer pannello, Long campagnaId, BachecaCriteria c) {
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContatto.class, "sc");
		ApplicationUtil.pannello(pannello, criteria);
		DetachedCriteria criteriaA = criteria.createCriteria("archiviazione", "arch",  ApplicationUtil.leftJoin());
		DetachedCriteria criteriaC = criteria.createCriteria("contatto", "cont");
		DetachedCriteria criteriaO = criteria.createCriteria("operatore", "op");
		DetachedCriteria criteriaCA = criteriaC.createCriteria("campagna", "camp");
		if(ApplicationUtil.gtZero(c.getTipologiaEsito())) {
			if(ApplicationUtil.eq(c.getTipologiaEsito(),1)) {
				criteriaA.add(Restrictions.eq("positivo", Boolean.TRUE));
			}
			if(ApplicationUtil.eq(c.getTipologiaEsito(),2)) {
				criteriaA.add(Restrictions.eq("utile", Boolean.TRUE));
			}
		}
		if(ApplicationUtil.gtZero(c.getTipologiaOperatore())) {
			Date oggi = ApplicationUtil.date();
			Date dataInizio = ApplicationUtil.dataInizio(c.getTipologiaOperatore(), oggi);
			ApplicationUtil.dataDate( dataInizio, null, "data", criteria);
		}
		criteriaCA.add(Restrictions.idEq(campagnaId));
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("op.username").as("username")))
				.add(Projections.property("op.cognome").as("cognome"))
				.add(Projections.property("op.nome").as("nome"))
				.add(Projections.countDistinct("sc.id").as("numero"))
				);
		criteria.addOrder(Order.desc("numero"));
		return getList(criteria, 0, c.getLimiteOperatore(), Classifica.class);
	}
	@SuppressWarnings({ "unchecked", "unused" })
	public List<Classifica> getEsitiXSecondi(Integer pannello, int i,  Integer tipologiaEsitoPopup) {
		Date dataFine = ApplicationUtil.date();
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContatto.class, "sc");
		ApplicationUtil.pannello(pannello, criteria);
		DetachedCriteria criteriaA = criteria.createCriteria("archiviazione", "arch",  ApplicationUtil.leftJoin());
		DetachedCriteria criteriaO = criteria.createCriteria("operatore", "op");
		ApplicationUtil.dataDate(ApplicationUtil.rimuoviSecondi(dataFine, i), dataFine, "data", criteria);
		if(ApplicationUtil.gtZero(tipologiaEsitoPopup)) {
			if(ApplicationUtil.eq(tipologiaEsitoPopup,1)) {
				criteriaA.add(Restrictions.eq("positivo", Boolean.TRUE));
			}
			if(ApplicationUtil.eq(tipologiaEsitoPopup,2)) {
				criteriaA.add(Restrictions.eq("utile", Boolean.TRUE));
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.property("op.username").as("username")))
				.add(Projections.property("op.cognome").as("cognome"))
				.add(Projections.property("op.nome").as("nome"))
				);
		return getList(criteria, Classifica.class);
	}
	@SuppressWarnings({ "unchecked", "unused" })
	public List<Classifica> getEsitiXSecondiByCampagnaId(Integer pannello, int i, Long campagnaId,  Integer tipologiaEsitoPopup) {
		Date dataInizio = ApplicationUtil.date();
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContatto.class, "sc");
		ApplicationUtil.pannello(pannello, criteria);
		DetachedCriteria criteriaA = criteria.createCriteria("archiviazione", "arch",  ApplicationUtil.leftJoin());
		DetachedCriteria criteriaO = criteria.createCriteria("operatore", "op");
		DetachedCriteria criteriaC = criteria.createCriteria("contatto", "cont");
		DetachedCriteria criteriaCA = criteriaC.createCriteria("campagna", "camp");
		criteriaCA.add(Restrictions.idEq(campagnaId));
		ApplicationUtil.dataDate(ApplicationUtil.rimuoviSecondi(dataInizio, i), dataInizio, "data", criteria);
		if(ApplicationUtil.gtZero(tipologiaEsitoPopup)) {
			if(ApplicationUtil.eq(tipologiaEsitoPopup,1)) {
				criteriaA.add(Restrictions.eq("positivo", Boolean.TRUE));
			}
			if(ApplicationUtil.eq(tipologiaEsitoPopup,2)) {
				criteriaA.add(Restrictions.eq("utile", Boolean.TRUE));
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.property("op.username").as("username")))
				.add(Projections.property("op.cognome").as("cognome"))
				.add(Projections.property("op.nome").as("nome"))
				);
		return getList(criteria, Classifica.class);
	}
	@SuppressWarnings("unchecked")
	public List<Long>  trovaUserBySupervisoreId(Utente user) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);					
		criteria.createCriteria("team").createCriteria("supervisore").add(Restrictions.idEq(user.getId()));
		criteria.setProjection(Projections.distinct(Projections.property("id")));
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Team> trovaTeamId() {
		DetachedCriteria criteria = DetachedCriteria.forClass(Team.class);
		ApplicationUtil.orderT(criteria);
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Zona> trovaZonaId() {
		DetachedCriteria criteria = DetachedCriteria.forClass(Zona.class);	
		ApplicationUtil.orderZ(criteria);
		return getList(criteria);
	}
	public int getArchiviazioniOdierneOperatore(Integer pannello, Long operatoreId, Long campagna, int tipo) {
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContatto.class);
		ApplicationUtil.pannello(pannello, criteria);
		if(ApplicationUtil.gtZero(campagna)) {
			criteria.createCriteria("contatto").add(Restrictions.eq("campagna.id", campagna));
		}
		criteria.add(Restrictions.eq("operatore.id",operatoreId));
		criteria.add(Restrictions.isNotNull("archiviazione.id"));
		Date dataInizio = ApplicationUtil.dataInizio(tipo, ApplicationUtil.date());
		criteria.add(Restrictions.gt("data", dataInizio));
		return getListSize(criteria);
	}
	public int getArchiviazioniOdierneOperatori(Integer pannello, List<Long> utenti, Long campagna, int tipo) {
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContatto.class);
		ApplicationUtil.pannello(pannello, criteria);
		if(ApplicationUtil.gtZero(campagna)) {
			criteria.createCriteria("contatto").add(Restrictions.eq("campagna.id", campagna));
		}
		criteria.add(Restrictions.in("operatore.id", utenti));
		criteria.add(Restrictions.isNotNull("archiviazione.id"));
		Date dataInizio = ApplicationUtil.dataInizio(tipo, ApplicationUtil.date());
		criteria.add(Restrictions.gt("data", dataInizio));
		return getListSize(criteria);
	}
	public int getContattiOdierneOperatore(Integer pannello,  Long operatoreId, Long campagna, int tipo) {
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContatto.class);
		ApplicationUtil.pannello(pannello, criteria);
		if(ApplicationUtil.gtZero(campagna)) {
			criteria.createCriteria("contatto").add(Restrictions.eq("campagna.id", campagna));
		}
		criteria.add(Restrictions.eq("operatore.id",operatoreId));
		Date dataInizio = ApplicationUtil.dataInizio(tipo, ApplicationUtil.date());
		criteria.add(Restrictions.gt("data", dataInizio));
		return getListSize(criteria);
	}
	public int getContattiOdierneOperatori(Integer pannello, List<Long> utenti, Long campagna, int tipo) {
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContatto.class);
		ApplicationUtil.pannello(pannello, criteria);
		if(ApplicationUtil.gtZero(campagna)) {
			criteria.createCriteria("contatto").add(Restrictions.eq("campagna.id", campagna));
		}
		criteria.add(Restrictions.in("operatore.id", utenti));
		Date dataInizio = ApplicationUtil.dataInizio(tipo, ApplicationUtil.date());
		criteria.add(Restrictions.gt("data", dataInizio));
		return getListSize(criteria);
	}
	public int getConversazioniOdierneOperatore(Integer pannello, Long operatoreId, Long campagna, int tipo) {
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContatto.class);
		ApplicationUtil.pannello(pannello, criteria);
		if(ApplicationUtil.gtZero(campagna)) {
			criteria.createCriteria("contatto").add(Restrictions.eq("campagna.id", campagna));
		}
		criteria.add(Restrictions.gt("secondi_conv_eff", 0));
		criteria.add(Restrictions.eq("operatore.id",operatoreId));
		Date dataInizio = ApplicationUtil.dataInizio(tipo, ApplicationUtil.date());
		criteria.add(Restrictions.gt("data", dataInizio));
		return getListSize(criteria);
	}
	public int getAppuntamentiOdierneOperatore(Long operatoreId, Long campagna, int tipo) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Appuntamento.class);
		if(ApplicationUtil.gtZero(campagna)) {
			criteria.createCriteria("contatto").add(Restrictions.eq("campagna.id", campagna));
		}
		criteria.add(Restrictions.eq("operatore.id",operatoreId));
		ApplicationUtil.intervallo(true, tipo, criteria);
		return getListSize(criteria);
	}
	public int getContrattiOdierneOperatore(Long operatoreId, Long campagna, int tipo) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Contratto.class);
		if(ApplicationUtil.gtZero(campagna)) {
			criteria.createCriteria("contatto").add(Restrictions.eq("campagna.id", campagna));
		}
		criteria.add(Restrictions.eq("operatore.id",operatoreId));
		ApplicationUtil.intervallo(true, tipo, criteria);
		return getListSize(criteria);
	}
	public int getConversazioniOdierneOperatori(Integer pannello, List<Long> utenti, Long campagna, int tipo) {
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContatto.class);
		ApplicationUtil.pannello(pannello, criteria);
		if(ApplicationUtil.gtZero(campagna)) {
			criteria.createCriteria("contatto").add(Restrictions.eq("campagna.id", campagna));
		}
		criteria.add(Restrictions.gt("secondi_conv_eff", 0));
		criteria.add(Restrictions.in("operatore.id", utenti));
		Date dataInizio = ApplicationUtil.dataInizio(tipo, ApplicationUtil.date());
		criteria.add(Restrictions.gt("data", dataInizio));
		return getListSize(criteria);
	}
	public int getAppuntamentiOdierneOperatori(List<Long> utenti, Long campagna, int tipo) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Appuntamento.class);
		if(ApplicationUtil.gtZero(campagna)) {
			criteria.createCriteria("contatto").add(Restrictions.eq("campagna.id", campagna));
		}
		criteria.add(Restrictions.in("operatore.id", utenti));
		ApplicationUtil.intervallo(true, tipo, criteria);
		return getListSize(criteria);
	}
	public int getContrattiOdierneOperatori(List<Long> utenti, Long campagna, int tipo) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Contratto.class);
		if(ApplicationUtil.gtZero(campagna)) {
			criteria.createCriteria("contatto").add(Restrictions.eq("campagna.id", campagna));
		}
		criteria.add(Restrictions.in("operatore.id", utenti));
		ApplicationUtil.intervallo(true, tipo, criteria);
		return getListSize(criteria);
	}
	@SuppressWarnings("unchecked")
	private List<RiepilogoEsitoAppoggio> getRiepilogoStati(ReportCriteria r) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class, "cont");
		DetachedCriteria criteriaCA = criteria.createCriteria("campagna", "camp");
		DetachedCriteria criteriaLI = criteria.createCriteria("lista", "lista");
		if(r != null) {
			ApplicationUtil.intervallo(ApplicationUtil.isTrue(r.getInserimento()), r.getInizio(), r.getFine(), criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, criteriaLI);
			if(ApplicationUtil.gtZero(r.getLista())) {
				criteriaLI.add(Restrictions.idEq(r.getLista()));
			}
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("cont.archiviato").as("positivo")))
				.add(Projections.countDistinct("cont.id").as("numero"))
				);
		criteria.addOrder(Order.desc("numero"));
		return getList(criteria, RiepilogoEsitoAppoggio.class);
	}
	@SuppressWarnings("unchecked")
	private List<RiepilogoEsitoAppoggio> getRiepilogoStatiStoria(Utente user, ReportCriteria r) {
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContatto.class, "cont");
		DetachedCriteria criteriaCo = criteria.createCriteria("contatto");
		DetachedCriteria criteriaCA = criteriaCo.createCriteria("campagna", "camp");
		if(user!= null) {
			DetachedCriteria criteriaUU = criteria.createCriteria("operatore", "oper");
			impostazioneDao.subCriteriaRoleSupervisore(user.getId(), criteria, "camp", "oper");
			if(ApplicationUtil.isTrue(sd)) {
				impostazioneDao.subCriteriaSupervisore(user.getId(), criteriaUU, "id");
			}
			impostazioneDao.subCriteriaCampagnaSupervisore(user.getId(), criteriaCA, "id");
		}
		if(r != null) {
			DetachedCriteria criteriaLI = criteriaCo.createCriteria("lista", "lista");
			ApplicationUtil.pannello(r.getPannello(), criteria);
			ApplicationUtil.dataIntervalD(r.getInizio(), r.getFine(), "data", criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, criteriaLI);
			if(ApplicationUtil.gtZero(r.getLista())) {
				criteriaLI.add(Restrictions.idEq(r.getLista()));
			}
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("cont.archiviato").as("positivo")))
				.add(Projections.countDistinct("cont.id").as("numero"))
				);
		criteria.addOrder(Order.desc("numero"));
		return getList(criteria, RiepilogoEsitoAppoggio.class);
	}
	@SuppressWarnings("unchecked")
	private List<RiepilogoEsitoAppoggio> getRiepilogoStatiStoriaPartner(Utente user, ReportCriteria r) {
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContatto.class, "cont");
		DetachedCriteria criteriaCo = criteria.createCriteria("contatto");
		DetachedCriteria criteriaCA = criteriaCo.createCriteria("campagna", "camp");
		DetachedCriteria criteriaLI = criteriaCo.createCriteria("lista", "lista");
		impostazioneDao.subCriteriaCampagnaPartner(user.getId(), criteriaCA, "id");
		if(r != null) {
			ApplicationUtil.pannello(r.getPannello(), criteria);
			ApplicationUtil.dataIntervalD(r.getInizio(), r.getFine(), "data", criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, criteriaLI);
			if(ApplicationUtil.gtZero(r.getLista())) {
				criteriaLI.add(Restrictions.idEq(r.getLista()));
			}
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("cont.archiviato").as("positivo")))
				.add(Projections.countDistinct("cont.id").as("numero"))
				);
		criteria.addOrder(Order.desc("numero"));
		return getList(criteria, RiepilogoEsitoAppoggio.class);
	}
	@SuppressWarnings("unchecked")
	public List<Dettaglio> getEsitiAppuntamento() {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoAppuntamento.class);
		ApplicationUtil.orderEsito(criteria);
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.property("id").as("esito_id")))
				.add(Projections.property("etichetta").as("etichetta"))
				.add(Projections.property("codice").as("codice"))
				.add(Projections.property("colore").as("colore")));
		return getList(criteria, Dettaglio.class);
	}
	@SuppressWarnings("unchecked")
	public List<Dettaglio> getEsitiContratto() {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoContratto.class);
		ApplicationUtil.orderEsito(criteria);
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.property("id").as("esito_id")))
				.add(Projections.property("etichetta").as("etichetta"))
				.add(Projections.property("codice").as("codice"))
				.add(Projections.property("colore").as("colore")));
		return getList(criteria, Dettaglio.class);
	}
	@SuppressWarnings("unchecked")
	private List<RiepilogoEsitoAppoggio> getRiepilogoStati(Utente user,  ReportCriteria r) {
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class, "cont");
		DetachedCriteria criteriaCA = criteria.createCriteria("campagna", "camp");
		DetachedCriteria criteriaLI = criteria.createCriteria("lista", "lista");
		if(ApplicationUtil.isTrue(sd)) {
			DetachedCriteria criteriaUU = criteria.createCriteria("utente");
			impostazioneDao.subCriteriaSupervisore(user.getId(), criteriaUU, "id");
		}
		impostazioneDao.subCriteriaCampagnaSupervisore(user.getId(), criteriaCA, "id");
		if(r != null) {
			ApplicationUtil.intervallo(ApplicationUtil.isTrue(r.getInserimento()), r.getInizio(), r.getFine(), criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, criteriaLI);
			if(ApplicationUtil.gtZero(r.getLista())) {
				criteriaLI.add(Restrictions.idEq(r.getLista()));
			}
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("cont.archiviato").as("positivo")))
				.add(Projections.countDistinct("cont.id").as("numero"))
				);
		criteria.addOrder(Order.desc("numero"));
		return getList(criteria, RiepilogoEsitoAppoggio.class);
	}
	@SuppressWarnings("unchecked")
	private List<RiepilogoEsitoAppoggio> getRiepilogoStatiPartner(Utente user, ReportCriteria r) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class, "cont");
		DetachedCriteria criteriaCA = criteria.createCriteria("campagna", "camp");
		DetachedCriteria criteriaLI = criteria.createCriteria("lista", "lista");
		impostazioneDao.subCriteriaCampagnaPartner(user.getId(), criteriaCA, "id");
		if(r != null) {
			ApplicationUtil.intervallo(ApplicationUtil.isTrue(r.getInserimento()), r.getInizio(), r.getFine(), criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, criteriaLI);
			if(ApplicationUtil.gtZero(r.getLista())) {
				criteriaLI.add(Restrictions.idEq(r.getLista()));
			}
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("cont.archiviato").as("positivo")))
				.add(Projections.countDistinct("cont.id").as("numero"))
				);
		criteria.addOrder(Order.desc("numero"));
		return getList(criteria, RiepilogoEsitoAppoggio.class);
	}
	@SuppressWarnings({ "unchecked", "unused" })
	private List<RiepilogoEsitoAppoggio> getRiepilogoEsiti(Utente user, ReportCriteria r) {
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class, "cont");
		DetachedCriteria criteriaE= criteria.createCriteria("esitoChiamata", "esit");
		DetachedCriteria criteriaA= criteria.createCriteria("esitoArchiviazione", "arch",  ApplicationUtil.leftJoin());
		DetachedCriteria criteriaCA = criteria.createCriteria("campagna", "camp");
		DetachedCriteria criteriaLI = criteria.createCriteria("lista", "lista");
		if(ApplicationUtil.isTrue(sd)) {
			DetachedCriteria criteriaUU = criteria.createCriteria("utente");
			impostazioneDao.subCriteriaSupervisore(user.getId(), criteriaUU, "id");
		}
		impostazioneDao.subCriteriaCampagnaSupervisore(user.getId(), criteriaCA, "id");
		if(r != null) {
			ApplicationUtil.intervallo(ApplicationUtil.isTrue(r.getInserimento()), r.getInizio(), r.getFine(), criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, criteriaLI);
			if(ApplicationUtil.gtZero(r.getLista())) {
				criteriaLI.add(Restrictions.idEq(r.getLista()));
			}
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("esit.id").as("eid")))
				.add(Projections.groupProperty("esit.etichetta").as("etichetta"))
				.add(Projections.groupProperty("esit.codice").as("codice"))
				.add(Projections.groupProperty("esit.colore").as("colore"))
				.add(Projections.sqlGroupProjection("positivo", "positivo", new String [] {"positivo"}, new Type []{BooleanType.INSTANCE}))
				.add(Projections.countDistinct("cont.id").as("numero"))
				);
		criteria.addOrder(Order.asc("esit.etichetta"));
		return getList(criteria, RiepilogoEsitoAppoggio.class);
	}
	@SuppressWarnings({ "unchecked", "unused" })
	private List<RiepilogoEsitoAppoggio> getRiepilogoEsitiPartner(Utente user, ReportCriteria r) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class, "cont");
		DetachedCriteria criteriaE= criteria.createCriteria("esitoChiamata", "esit");
		DetachedCriteria criteriaA= criteria.createCriteria("esitoArchiviazione", "arch",  ApplicationUtil.leftJoin());
		DetachedCriteria criteriaCA = criteria.createCriteria("campagna", "camp");
		DetachedCriteria criteriaLI = criteria.createCriteria("lista", "lista");
		impostazioneDao.subCriteriaCampagnaPartner(user.getId(), criteriaCA, "id");
		if(r != null) {
			ApplicationUtil.intervallo(ApplicationUtil.isTrue(r.getInserimento()), r.getInizio(), r.getFine(), criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, criteriaLI);
			if(ApplicationUtil.gtZero(r.getLista())) {
				criteriaLI.add(Restrictions.idEq(r.getLista()));
			}
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("esit.id").as("eid")))
				.add(Projections.groupProperty("esit.etichetta").as("etichetta"))
				.add(Projections.groupProperty("esit.codice").as("codice"))
				.add(Projections.groupProperty("esit.colore").as("colore"))
				.add(Projections.sqlGroupProjection("positivo", "positivo", new String [] {"positivo"}, new Type []{BooleanType.INSTANCE}))
				.add(Projections.countDistinct("cont.id").as("numero"))
				);
		criteria.addOrder(Order.asc("esit.etichetta"));
		return getList(criteria, RiepilogoEsitoAppoggio.class);
	}
	@SuppressWarnings({ "unchecked", "unused" })
	private List<RiepilogoEsito> getRiepilogoEsitiChiamataStoria(Utente user,  ReportCriteria r){
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContatto.class, "sc");
		DetachedCriteria criteriaCO = criteria.createCriteria("contatto", "cont");
		DetachedCriteria criteriaE= criteria.createCriteria("esito", "esit");
		DetachedCriteria criteriaCA = criteriaCO.createCriteria("campagna", "camp");
		if(user != null)  {
			DetachedCriteria criteriaOp = criteria.createCriteria("operatore", "oper");
			impostazioneDao.subCriteriaRoleSupervisore(user.getId(), criteria, "camp", "oper");
			impostazioneDao.subCriteriaSupervisore(user.getId(), criteriaOp, "id");
		}
		if(r != null) {
			DetachedCriteria criteriaLI = criteriaCO.createCriteria("lista", "lista");
			ApplicationUtil.pannello(r.getPannello(), criteria);
			ApplicationUtil.dataIntervalD(r.getInizio(), r.getFine(), "data", criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, criteriaLI);
			if(ApplicationUtil.gtZero(r.getLista())) {
				criteriaLI.add(Restrictions.idEq(r.getLista()));
			}
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
			if(user != null && ApplicationUtil.ltZero(r.getCampagna())) {
				if(ApplicationUtil.isTrue(sd)) {
					impostazioneDao.subCriteriaCampagnaSupervisore(user.getId(), criteriaCA, "id");
				}
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("esit.id").as("eid")))
				.add(Projections.groupProperty("esit.etichetta").as("esito"))
				.add(Projections.groupProperty("esit.codice").as("codice"))
				.add(Projections.groupProperty("esit.colore").as("colore"))
				.add(Projections.countDistinct("sc.id").as("numero"))
				);
		criteria.addOrder(Order.asc("esit.etichetta"));
		return getList(criteria, RiepilogoEsito.class);
	}
	@SuppressWarnings({ "unchecked", "unused" })
	private List<RiepilogoEsito> getRiepilogoEsitiChiamataStoriaPartner(Utente user, ReportCriteria r){
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContatto.class, "sc");
		DetachedCriteria criteriaCO = criteria.createCriteria("contatto", "cont");
		DetachedCriteria criteriaE= criteria.createCriteria("esito", "esit");
		DetachedCriteria criteriaCA = criteriaCO.createCriteria("campagna", "camp");
		DetachedCriteria criteriaLI = criteriaCO.createCriteria("lista", "lista");
		DetachedCriteria criteriaOp = criteria.createCriteria("operatore", "operatore");
		impostazioneDao.subCriteriaCampagnaPartner(user.getId(), criteriaCA, "id");
		if(r != null) {
			ApplicationUtil.pannello(r.getPannello(), criteria);
			ApplicationUtil.dataIntervalD(r.getInizio(), r.getFine(), "data", criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, criteriaLI);
			if(ApplicationUtil.gtZero(r.getLista())) {
				criteriaLI.add(Restrictions.idEq(r.getLista()));
			}
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("esit.id").as("eid")))
				.add(Projections.groupProperty("esit.etichetta").as("esito"))
				.add(Projections.groupProperty("esit.codice").as("codice"))
				.add(Projections.groupProperty("esit.colore").as("colore"))
				.add(Projections.countDistinct("sc.id").as("numero"))
				);
		criteria.addOrder(Order.asc("esit.etichetta"));
		return getList(criteria, RiepilogoEsito.class);
	}
	@SuppressWarnings({ "unchecked", "unused" })
	private List<RiepilogoEsito> getRiepilogoEsitiChiamata(Utente user, ReportCriteria r){
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class, "sc");
		DetachedCriteria criteriaE= criteria.createCriteria("esitoChiamata", "esit");
		DetachedCriteria criteriaCA = criteria.createCriteria("campagna", "camp");
		DetachedCriteria criteriaLI = criteria.createCriteria("lista", "lista");
		if(ApplicationUtil.isTrue(sd)) {
			DetachedCriteria criteriaUU = criteria.createCriteria("utente");
			impostazioneDao.subCriteriaSupervisore(user.getId(), criteriaUU, "id");
		}
		impostazioneDao.subCriteriaCampagnaSupervisore(user.getId(), criteriaCA, "id");
		if(r != null) {
			ApplicationUtil.intervallo(ApplicationUtil.isTrue(r.getInserimento()), r.getInizio(), r.getFine(), criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, criteriaLI);
			if(ApplicationUtil.gtZero(r.getLista())) {
				criteriaLI.add(Restrictions.idEq(r.getLista()));
			}
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("esit.id").as("eid")))
				.add(Projections.groupProperty("esit.etichetta").as("esito"))
				.add(Projections.groupProperty("esit.codice").as("codice"))
				.add(Projections.groupProperty("esit.colore").as("colore"))
				.add(Projections.countDistinct("sc.id").as("numero"))
				);
		criteria.addOrder(Order.asc("esit.etichetta"));
		return getList(criteria, RiepilogoEsito.class);
	}
	@SuppressWarnings({ "unchecked", "unused" })
	private List<RiepilogoEsito> getRiepilogoEsitiChiamataPartner(Utente user, ReportCriteria r){
		DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class, "sc");
		DetachedCriteria criteriaE= criteria.createCriteria("esitoChiamata", "esit");
		DetachedCriteria criteriaCA = criteria.createCriteria("campagna", "camp");
		DetachedCriteria criteriaLI = criteria.createCriteria("lista", "lista");
		impostazioneDao.subCriteriaCampagnaPartner(user.getId(), criteriaCA, "id");
		if(r != null) {
			ApplicationUtil.intervallo(ApplicationUtil.isTrue(r.getInserimento()), r.getInizio(), r.getFine(), criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, criteriaLI);
			if(ApplicationUtil.gtZero(r.getLista())) {
				criteriaLI.add(Restrictions.idEq(r.getLista()));
			}
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("esit.id").as("eid")))
				.add(Projections.groupProperty("esit.etichetta").as("esito"))
				.add(Projections.groupProperty("esit.codice").as("codice"))
				.add(Projections.groupProperty("esit.colore").as("colore"))
				.add(Projections.countDistinct("sc.id").as("numero"))
				);
		criteria.addOrder(Order.asc("esit.etichetta"));
		return getList(criteria, RiepilogoEsito.class);
	}
	@SuppressWarnings({ "unchecked", "unused" })
	private List<RiepilogoEsito> getRiepilogoEsitiChiamata(ReportCriteria r){
		DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class, "sc");
		DetachedCriteria criteriaE= criteria.createCriteria("esitoChiamata", "esit");
		DetachedCriteria criteriaCA = criteria.createCriteria("campagna", "camp");
		DetachedCriteria criteriaLI = criteria.createCriteria("lista", "lista");
		if(r != null) {
			ApplicationUtil.intervallo(ApplicationUtil.isTrue(r.getInserimento()), r.getInizio(), r.getFine(), criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, criteriaLI);
			if(ApplicationUtil.gtZero(r.getLista())) {
				criteriaLI.add(Restrictions.idEq(r.getLista()));
			}
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("esit.id").as("eid")))
				.add(Projections.groupProperty("esit.etichetta").as("esito"))
				.add(Projections.groupProperty("esit.codice").as("codice"))
				.add(Projections.groupProperty("esit.colore").as("colore"))
				.add(Projections.countDistinct("sc.id").as("numero"))
				);
		criteria.addOrder(Order.asc("esit.etichetta"));
		return getList(criteria, RiepilogoEsito.class);
	}
	@SuppressWarnings({ "unchecked", "unused" })
	private List<RiepilogoEsitoAppoggio> getRiepilogoEsitiArchiviazioneStoria(Utente user,  ReportCriteria r) {
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContatto.class, "sc");
		DetachedCriteria criteriaCO = criteria.createCriteria("contatto", "cont");
		DetachedCriteria criteriaA = criteria.createCriteria("archiviazione", "arch");
		DetachedCriteria criteriaCA = criteriaCO.createCriteria("campagna", "camp");
		DetachedCriteria criteriaOp = criteria.createCriteria("operatore", "oper");
		if(user != null)  {
			impostazioneDao.subCriteriaRoleSupervisore(user.getId(), criteria, "camp", "oper");
			impostazioneDao.subCriteriaSupervisore(user.getId(), criteriaOp, "id");
		}
		if(r != null) {
			DetachedCriteria criteriaLI = criteriaCO.createCriteria("lista", "lista");
			ApplicationUtil.pannello(r.getPannello(), criteria);
			ApplicationUtil.dataIntervalD(r.getInizio(), r.getFine(), "data", criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, criteriaLI);
			if(ApplicationUtil.gtZero(r.getLista())) {
				criteriaLI.add(Restrictions.idEq(r.getLista()));
			}
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
			if(user != null && ApplicationUtil.ltZero(r.getCampagna())) {
				if(ApplicationUtil.isTrue(sd)) {
					impostazioneDao.subCriteriaCampagnaSupervisore(user.getId(), criteriaCA, "id");
				}
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("arch.id").as("eid")))
				.add(Projections.groupProperty("arch.etichetta").as("etichetta"))
				.add(Projections.groupProperty("arch.codice").as("codice"))
				.add(Projections.groupProperty("arch.colore").as("colore"))
				.add(Projections.groupProperty("arch.positivo").as("positivo"))
				.add(Projections.groupProperty("arch.utile").as("utile"))
				.add(Projections.countDistinct("sc.id").as("numero"))
				);
		criteria.addOrder(Order.desc("arch.id"));
		return getList(criteria, RiepilogoEsitoAppoggio.class);
	}
	@SuppressWarnings({ "unchecked", "unused" })
	private List<RiepilogoEsitoAppoggio> getRiepilogoEsitiArchiviazioneStoriaPartner(Utente user, ReportCriteria r) {
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContatto.class, "sc");
		DetachedCriteria criteriaCO = criteria.createCriteria("contatto", "cont");
		DetachedCriteria criteriaA = criteria.createCriteria("archiviazione", "arch");
		DetachedCriteria criteriaCA = criteriaCO.createCriteria("campagna", "camp");
		DetachedCriteria criteriaLI = criteriaCO.createCriteria("lista", "lista");
		DetachedCriteria criteriaOp = criteria.createCriteria("operatore", "operatore");
		impostazioneDao.subCriteriaCampagnaPartner(user.getId(), criteriaCA, "id");
		if(r != null) {
			ApplicationUtil.pannello(r.getPannello(), criteria);
			ApplicationUtil.dataIntervalD(r.getInizio(), r.getFine(), "data", criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, criteriaLI);
			if(ApplicationUtil.gtZero(r.getLista())) {
				criteriaLI.add(Restrictions.idEq(r.getLista()));
			}
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("arch.id").as("eid")))
				.add(Projections.groupProperty("arch.etichetta").as("etichetta"))
				.add(Projections.groupProperty("arch.positivo").as("positivo"))
				.add(Projections.groupProperty("arch.codice").as("codice"))
				.add(Projections.groupProperty("arch.colore").as("colore"))
				.add(Projections.groupProperty("arch.utile").as("utile"))
				.add(Projections.countDistinct("sc.id").as("numero"))
				);
		criteria.addOrder(Order.desc("arch.id"));
		return getList(criteria, RiepilogoEsitoAppoggio.class);
	}
	@SuppressWarnings({ "unchecked", "unused" })
	private List<RiepilogoEsitoAppoggio> getRiepilogoEsitiArchiviazione(Utente user, ReportCriteria r) {
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class, "sc");
		DetachedCriteria criteriaA = criteria.createCriteria("esitoArchiviazione", "arch");
		DetachedCriteria criteriaCA = criteria.createCriteria("campagna", "camp");
		DetachedCriteria criteriaLI = criteria.createCriteria("lista", "lista");
		if(ApplicationUtil.isTrue(sd)) {
			DetachedCriteria criteriaUU = criteria.createCriteria("utente");
			impostazioneDao.subCriteriaSupervisore(user.getId(), criteriaUU, "id");
		}
		impostazioneDao.subCriteriaCampagnaSupervisore(user.getId(), criteriaCA, "id");
		if(r != null) {
			ApplicationUtil.intervallo(ApplicationUtil.isTrue(r.getInserimento()), r.getInizio(), r.getFine(), criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, criteriaLI);
			if(ApplicationUtil.gtZero(r.getLista())) {
				criteriaLI.add(Restrictions.idEq(r.getLista()));
			}
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("arch.id").as("eid")))
				.add(Projections.groupProperty("arch.etichetta").as("etichetta"))
				.add(Projections.groupProperty("arch.codice").as("codice"))
				.add(Projections.groupProperty("arch.colore").as("colore"))
				.add(Projections.groupProperty("arch.positivo").as("positivo"))
				.add(Projections.groupProperty("arch.utile").as("utile"))
				.add(Projections.countDistinct("sc.id").as("numero"))
				);
		criteria.addOrder(Order.desc("arch.id"));
		return getList(criteria, RiepilogoEsitoAppoggio.class);
	}
	@SuppressWarnings({ "unchecked", "unused" })
	private List<RiepilogoEsitoAppoggio> getRiepilogoEsitiArchiviazionePartner(Utente user, ReportCriteria r) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class, "sc");
		DetachedCriteria criteriaA = criteria.createCriteria("esitoArchiviazione", "arch");
		DetachedCriteria criteriaCA = criteria.createCriteria("campagna", "camp");
		DetachedCriteria criteriaLI = criteria.createCriteria("lista", "lista");
		impostazioneDao.subCriteriaCampagnaPartner(user.getId(), criteriaCA, "id");
		if(r != null) {
			ApplicationUtil.intervallo(ApplicationUtil.isTrue(r.getInserimento()), r.getInizio(), r.getFine(), criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, criteriaLI);
			if(ApplicationUtil.gtZero(r.getLista())) {
				criteriaLI.add(Restrictions.idEq(r.getLista()));
			}
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("arch.id").as("eid")))
				.add(Projections.groupProperty("arch.etichetta").as("etichetta"))
				.add(Projections.groupProperty("arch.codice").as("codice"))
				.add(Projections.groupProperty("arch.colore").as("colore"))
				.add(Projections.groupProperty("arch.positivo").as("positivo"))
				.add(Projections.groupProperty("arch.utile").as("utile"))
				.add(Projections.countDistinct("sc.id").as("numero"))
				);
		criteria.addOrder(Order.desc("arch.id"));
		return getList(criteria, RiepilogoEsitoAppoggio.class);
	}
	@SuppressWarnings({ "unchecked", "unused" })
	private List<RiepilogoEsitoAppoggio> getRiepilogoEsitiArchiviazione(ReportCriteria r) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class, "sc");
		DetachedCriteria criteriaA = criteria.createCriteria("esitoArchiviazione", "arch");
		DetachedCriteria criteriaCA = criteria.createCriteria("campagna", "camp");
		DetachedCriteria criteriaLI = criteria.createCriteria("lista", "lista");
		if(r != null) {
			ApplicationUtil.intervallo(ApplicationUtil.isTrue(r.getInserimento()), r.getInizio(), r.getFine(), criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, criteriaLI);
			if(ApplicationUtil.gtZero(r.getLista())) {
				criteriaLI.add(Restrictions.idEq(r.getLista()));
			}
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("arch.id").as("eid")))
				.add(Projections.groupProperty("arch.etichetta").as("etichetta"))
				.add(Projections.groupProperty("arch.codice").as("codice"))
				.add(Projections.groupProperty("arch.colore").as("colore"))
				.add(Projections.groupProperty("arch.positivo").as("positivo"))
				.add(Projections.groupProperty("arch.utile").as("utile"))
				.add(Projections.countDistinct("sc.id").as("numero"))
				);
		criteria.addOrder(Order.desc("arch.id"));
		return getList(criteria, RiepilogoEsitoAppoggio.class);
	}
	@SuppressWarnings("unchecked")
	public List<Dettaglio> getReportEsitiChiamata() {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoChiamata.class);
		criteria.add(Restrictions.ne("id", ConstantsEsito.ID_OK));
		if(!impostazioneDao.getResetContatto()) {
			criteria.add(Restrictions.ne("id", ConstantsEsito.ID_DA_CHIAMARE));
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.property("id").as("esito_id")))
				.add(Projections.property("etichetta").as("etichetta"))
				.add(Projections.property("codice").as("codice"))
				.add(Projections.property("colore").as("colore")));
		ApplicationUtil.orderEsito(criteria);
		return getList(criteria, Dettaglio.class);
	}
	public void getRegistrazioniFTP(Contatto contatto, List<MyFile> fils, DownloadBean downloadBean) {
		List<Allegato> files = new ArrayList<Allegato>();
		String b = impostazioneDao.audioSupportato();
		String monitor = ConstantsGlobal.path_monitor;
		if(ApplicationUtil.isNotTrue(impostazioneDao.getFTPEnabled())) {
			if (ApplicationUtil.notEmpty(fils)) {
				String home = impostazioneDao.getHome();
				Iterator<MyFile> itS = fils.iterator();
				while (itS.hasNext()) {
					MyFile file = (MyFile) itS.next();
					try {
						ApplicationUtil.outFileRCodec(contatto != null ? contatto.getId() : file.getId(), files, home + monitor+"/"+file.getName(), b, file.getModified());
					} catch (Exception e) {
						ErrorLog.log(null, getClass(), e);
					}
				}
			}
		} else {
			UtilFTP ut = impostazioneDao.getUtilFTP();
			FTPClient vmftp = ApplicationUtil.createClient();
			if (ApplicationUtil.connect(vmftp, ut)) {
				try {
					if(ApplicationUtil.notEmpty(fils)) {
						Iterator<MyFile> fIt = fils.iterator();
						while (fIt.hasNext()) {
							MyFile myFile = (MyFile) fIt.next();
							ApplicationUtil.outFileRCodec(vmftp, contatto != null ? contatto.getId() : myFile.getId(), files, monitor + "/" + myFile.getName(), b, myFile.getModified());
						}
					}
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				} finally {
					ApplicationUtil.disconnect(vmftp);
				}
			}
		}
		try {
			if (ApplicationUtil.empty(files)) {
				ApplicationUtil.fileWriter(files, ConstantsLanguage.NO_REGISTRAZIONE.stampa(), ConstantsLanguage.NO_REGISTRAZIONI.stampa());
			}

			String file_header = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.REGISTRAZIONI.stampa(), ConstantsLanguage.CONTATTO.stampa(), contatto.getAnagrafica().getDenominazione()) + "." + ConstantsMap.zip;
			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file_header, files);
			String p = ConstantsGlobal.secretZ;

			ApplicationUtil.zip(null, files, out, p);
			downloadBean.close();

		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void getRegistrazioniFTPA(Contatto contatto, List<MyFile> fils, DownloadBean downloadBean) {
		List<Allegato> files = new ArrayList<Allegato>();
		String b = impostazioneDao.audioSupportato();
		String monitor = ConstantsGlobal.path_monitor;
		if(ApplicationUtil.isNotTrue(impostazioneDao.getFTPEnabled())) {
			if (ApplicationUtil.notEmpty(fils)) {
				String home = impostazioneDao.getHome();
				Iterator<MyFile> itS = fils.iterator();
				while (itS.hasNext()) {
					MyFile file = (MyFile) itS.next();
					try {
						ApplicationUtil.outFileRCodec(contatto != null ? contatto.getId() : file.getId(), files, home + monitor+"/"+file.getName(), b, file.getModified());
					} catch (Exception e) {
						ErrorLog.log(null, getClass(), e);
					}
				}
			}
		} else {
			UtilFTP ut = impostazioneDao.getUtilFTP();
			FTPClient vmftp = ApplicationUtil.createClient();
			if (ApplicationUtil.connect(vmftp, ut)) {
				try {
					if(ApplicationUtil.notEmpty(fils)) {
						Iterator<MyFile> fIt = fils.iterator();
						while (fIt.hasNext()) {
							MyFile myFile = (MyFile) fIt.next();
							ApplicationUtil.outFileRCodec(vmftp, contatto != null ? contatto.getId() : myFile.getId(), files, monitor + "/" + myFile.getName(), b, myFile.getModified());
						}
					}
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				} finally {
					ApplicationUtil.disconnect(vmftp);
				}
			}
		}
		try {
			if (ApplicationUtil.notEmpty(files)) {


				String file_header = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.REGISTRAZIONI.stampa()) + "." + ConstantsMap.zip;
				OutputStream out = downloadBean.outputStream();
				downloadBean.filename(file_header, files);
				String p = ConstantsGlobal.secretZ;

				ApplicationUtil.zip(null, files, out, p);
				downloadBean.close();

			}
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void getAtteseFTP(List<MyFile> fils, DownloadBean downloadBean) {
		List<Allegato> files = new ArrayList<Allegato>();
		if(ApplicationUtil.isNotTrue(impostazioneDao.getFTPEnabled())) {
			if (ApplicationUtil.notEmpty(fils)) {
				String home = impostazioneDao.getHome();
				Iterator<MyFile> itS = fils.iterator();
				while (itS.hasNext()) {
					MyFile file = (MyFile) itS.next();
					ApplicationUtil.addFile(files, file.getName(),  home + ConstantsGlobal.path_attesa + "/" + file.getName(), file.getModified());
				}
			}
		} else {
			UtilFTP ut = impostazioneDao.getUtilFTP();
			FTPClient vmftp = ApplicationUtil.createClient();
			if (ApplicationUtil.connect(vmftp, ut)) {
				try {
					if(ApplicationUtil.notEmpty(fils)) {
						Iterator<MyFile> fIt = fils.iterator();
						while (fIt.hasNext()) {
							MyFile myFile = (MyFile) fIt.next();
							String path = ConstantsGlobal.path_attesa + "/" + myFile.getName();
							ApplicationUtil.addFile(vmftp, files, path, myFile.getModified());
						}
					}
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				} finally {
					ApplicationUtil.disconnect(vmftp);
				}
			}
		}
		try {
			if (ApplicationUtil.notEmpty(files)) {


				String file_header = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.ATTESE.stampa()) + "." + ConstantsMap.zip;
				OutputStream out = downloadBean.outputStream();
				downloadBean.filename(file_header, files);
				String p = ConstantsGlobal.secretZ;

				ApplicationUtil.zip(null, files, out, p);
				downloadBean.close();

			}
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void getSuoniFTP(List<MyFile> fils, DownloadBean downloadBean) {
		List<Allegato> files = new ArrayList<Allegato>();
		if(ApplicationUtil.isNotTrue(impostazioneDao.getFTPEnabled())) {
			if (ApplicationUtil.notEmpty(fils)) {
				String home = impostazioneDao.getHome();
				Iterator<MyFile> itS = fils.iterator();
				while (itS.hasNext()) {
					MyFile file = (MyFile) itS.next();
					ApplicationUtil.addFile(files, file.getName(),  home + ConstantsGlobal.path_sounds + "/"+ file.getName(), file.getModified());
				}
			}
		} else {
			UtilFTP ut = impostazioneDao.getUtilFTP();
			FTPClient vmftp = ApplicationUtil.createClient();
			if (ApplicationUtil.connect(vmftp, ut)) {
				try {
					if(ApplicationUtil.notEmpty(fils)) {
						Iterator<MyFile> fIt = fils.iterator();
						while (fIt.hasNext()) {
							MyFile myFile = (MyFile) fIt.next();
							String path = ConstantsGlobal.path_sounds + "/" + myFile.getName();
							ApplicationUtil.addFile(vmftp, files, path, myFile.getModified());
						}
					}
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				} finally {
					ApplicationUtil.disconnect(vmftp);
				}
			}
		}
		try {
			if (ApplicationUtil.notEmpty(files)) {


				String file_header = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.AUDIO.stampa()) + "." + ConstantsMap.zip;
				OutputStream out = downloadBean.outputStream();
				downloadBean.filename(file_header, files);
				String p = ConstantsGlobal.secretZ;

				ApplicationUtil.zip(null, files, out, p);
				downloadBean.close();

			}
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void getRegistrazioniFTP(Contatto contatto, DownloadBean downloadBean) {
		List<Allegato> files = new ArrayList<Allegato>();
		String b = impostazioneDao.audioSupportato();
		String monitor = ConstantsGlobal.path_monitor;
		if(ApplicationUtil.isNotTrue(impostazioneDao.getFTPEnabled())) {
			String home = impostazioneDao.getHome();
			File perc = new File(home + monitor);
			if(perc.exists()) {
				Path folder = Paths.get(perc.getAbsolutePath());
				DirectoryStream<Path> stream = ApplicationUtil.streamFile(folder, "_" + contatto.getId() + "." + ConstantsMap.wav);
				if(stream != null) {
					for(Path entry : stream) {
						File file = entry.toFile();
						try {
							ApplicationUtil.outFileRCodec(contatto != null ? contatto.getId() : null, files, home + monitor+"/"+file.getName(), b, file.lastModified());
						} catch (Exception e) {
							ErrorLog.log(null, getClass(), e);
						}
					}
				}
			}
		} else {
			UtilFTP ut = impostazioneDao.getUtilFTP();
			FTPClient vmftp = ApplicationUtil.createClient();
			if (ApplicationUtil.connect(vmftp, ut)) {
				try {
					FTPFile[] wavFiles = ApplicationUtil.listMonitor(vmftp, monitor + "/", contatto.getId(), ConstantsMap.wav);
					if (ApplicationUtil.notEmpty(wavFiles)) {
						for(FTPFile file : wavFiles) {
							ApplicationUtil.outFileRCodec(vmftp, contatto != null ? contatto.getId() : null, files, monitor + "/" + file.getName(), b, ApplicationUtil.time(file.getTimestamp()));
						}
					}
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				} finally {
					ApplicationUtil.disconnect(vmftp);
				}
			}
		}
		try {
			if (ApplicationUtil.empty(files)) {
				ApplicationUtil.fileWriter(files, ConstantsLanguage.NO_REGISTRAZIONE.stampa(), ConstantsLanguage.NO_REGISTRAZIONI.stampa());
			}

			String file_header = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.REGISTRAZIONI.stampa(), ConstantsLanguage.CONTATTO.stampa(), contatto.getAnagrafica().getDenominazione()) + "." + ConstantsMap.zip;
			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file_header, files);
			String p = ConstantsGlobal.secretZ;

			ApplicationUtil.zip(null, files, out, p);
			downloadBean.close();

		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void getRegistrazioniUnico(Contatto contatto, List<MyFile> fils, DownloadBean downloadBean) {
		List<Allegato> files = new ArrayList<Allegato>();
		String b = impostazioneDao.audioSupportato();
		String monitor = ConstantsGlobal.path_monitor;
		if(ApplicationUtil.isNotTrue(impostazioneDao.getFTPEnabled())) {
			if (ApplicationUtil.notEmpty(fils)) {
				String home = impostazioneDao.getHome();
				Iterator<MyFile> itS = fils.iterator();
				while (itS.hasNext()) {
					MyFile file = (MyFile) itS.next();
					try {
						ApplicationUtil.outFileRCodec(contatto != null ? contatto.getId() : file.getId(), files,  home + monitor+"/"+file.getName(), b, file.getModified());
					} catch (Exception e) {
						ErrorLog.log(null, getClass(), e);
					}
				}
			}
		} else {
			UtilFTP ut = impostazioneDao.getUtilFTP();
			FTPClient vmftp = ApplicationUtil.createClient();
			if (ApplicationUtil.connect(vmftp, ut)) {
				try {
					if(ApplicationUtil.notEmpty(fils)) {
						Iterator<MyFile> fIt = fils.iterator();
						while (fIt.hasNext()) {
							MyFile myFile = (MyFile) fIt.next();
							ApplicationUtil.outFileRCodec(vmftp, contatto != null ? contatto.getId() : myFile.getId(), files, monitor + "/" + myFile.getName(), b, myFile.getModified());
						}
					}
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				} finally {
					ApplicationUtil.disconnect(vmftp);
				}
			}
		}
		try {
			String s = ConstantsGlobal.secretT;


			String ext = b != null ? b : ConstantsMap.wav;
			String file_header = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.REGISTRAZIONI.stampa(), ConstantsLanguage.CONTATTO.stampa(), contatto.getAnagrafica().getDenominazione()) + "." + (s != null ? ConstantsMap.zip : ext);
			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file_header, files);

			if(s != null) {
				ByteArrayOutputStream bos = new ByteArrayOutputStream();
				ApplicationUtil.concatena(files, bos, b);
				ZipPassword.zip(null, bos.toByteArray(), out, ext, s);
				bos.close();
			} else {
				ApplicationUtil.concatena(files, out, b);
			}
			downloadBean.close();

		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void getRegistrazioniUnicoA(Contatto contatto, List<MyFile> fils, DownloadBean downloadBean) {
		List<Allegato> files = new ArrayList<Allegato>();
		String b = impostazioneDao.audioSupportato();
		String monitor = ConstantsGlobal.path_monitor;
		if(ApplicationUtil.isNotTrue(impostazioneDao.getFTPEnabled())) {
			if (ApplicationUtil.notEmpty(fils)) {
				String home = impostazioneDao.getHome();
				Iterator<MyFile> itS = fils.iterator();
				while (itS.hasNext()) {
					MyFile file = (MyFile) itS.next();
					try {
						ApplicationUtil.outFileRCodec(contatto != null ? contatto.getId() : file.getId(), files,  home + monitor+"/"+file.getName(), b, file.getModified());
					} catch (Exception e) {
						ErrorLog.log(null, getClass(), e);
					}
				}
			}
		} else {
			UtilFTP ut = impostazioneDao.getUtilFTP();
			FTPClient vmftp = ApplicationUtil.createClient();
			if (ApplicationUtil.connect(vmftp, ut)) {
				try {
					if(ApplicationUtil.notEmpty(fils)) {
						Iterator<MyFile> fIt = fils.iterator();
						while (fIt.hasNext()) {
							MyFile myFile = (MyFile) fIt.next();
							ApplicationUtil.outFileRCodec(vmftp, contatto != null ? contatto.getId() : myFile.getId(), files, monitor + "/" + myFile.getName(), b, myFile.getModified());
						}
					}
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				} finally {
					ApplicationUtil.disconnect(vmftp);
				}
			}
		}
		try {
			String s = ConstantsGlobal.secretT;

			String ext = b != null ? b : ConstantsMap.wav;
			String file_header = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.REGISTRAZIONI.stampa()) + "." + (s != null ? ConstantsMap.zip : ext);
			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file_header, files);

			if(s != null) {
				ByteArrayOutputStream bos = new ByteArrayOutputStream();
				ApplicationUtil.concatena(files, bos, b);
				ZipPassword.zip(null, bos.toByteArray(), out, ext, s);
				bos.close();
			} else {
				ApplicationUtil.concatena(files, out, b);
			}
			downloadBean.close();

		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void getRegistrazioniUnico(Contatto contatto, DownloadBean downloadBean) {
		List<Allegato> files = new ArrayList<Allegato>();
		String b = impostazioneDao.audioSupportato();
		String monitor = ConstantsGlobal.path_monitor;
		if(ApplicationUtil.isNotTrue(impostazioneDao.getFTPEnabled())) {
			String home = impostazioneDao.getHome();

			File perc = new File(home + monitor);
			if(perc.exists()) {
				Path folder = Paths.get(perc.getAbsolutePath());
				DirectoryStream<Path> stream = ApplicationUtil.streamFile(folder, "_" + contatto.getId() + "." + ConstantsMap.wav);
				if(stream != null) {
					for(Path entry : stream) {
						File file = entry.toFile();
						try {
							ApplicationUtil.outFileRCodec(contatto != null ? contatto.getId() : null, files,  home + monitor+"/"+file.getName(), b, file.lastModified());
						} catch (Exception e) {
							ErrorLog.log(null, getClass(), e);
						}
					}
				}
			}
		} else {
			UtilFTP ut = impostazioneDao.getUtilFTP();
			FTPClient vmftp = ApplicationUtil.createClient();
			if (ApplicationUtil.connect(vmftp, ut)) {
				try {
					FTPFile[] wavFiles = ApplicationUtil.listMonitor(vmftp, monitor + "/", contatto.getId(), ConstantsMap.wav);
					if (ApplicationUtil.notEmpty(wavFiles)) {
						for(FTPFile file : wavFiles) {
							ApplicationUtil.outFileRCodec(vmftp, contatto != null ? contatto.getId() : null, files, monitor + "/" + file.getName(), b, ApplicationUtil.time(file.getTimestamp()));
						}
					}
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				} finally {
					ApplicationUtil.disconnect(vmftp);
				}
			}
		}
		try {
			String s = ConstantsGlobal.secretT;
			String ext = b != null ? b : ConstantsMap.wav;
			String file_header = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.REGISTRAZIONI.stampa(), ConstantsLanguage.CONTATTO.stampa(), contatto.getAnagrafica().getDenominazione()) + "." + (s != null ? ConstantsMap.zip : ext);
			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file_header, files);

			if(s != null) {
				ByteArrayOutputStream bos = new ByteArrayOutputStream();
				ApplicationUtil.concatena(files, bos, b);
				ZipPassword.zip(null, bos.toByteArray(), out, ext, s);
				bos.close();
			} else {
				ApplicationUtil.concatena(files, out, b);
			}
			downloadBean.close();

		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public ReportContattiOdierni getContattiOdierniOperatore(Long operatoreId, Long campagna, int tipo) {
		ReportContattiOdierni rcoo = new ReportContattiOdierni();
		Integer pannello = 1;
		rcoo.setArchiviazioni(getArchiviazioniOdierneOperatore(pannello, operatoreId, campagna, tipo));
		rcoo.setContatti(getContattiOdierneOperatore(pannello, operatoreId, campagna, tipo));
		rcoo.setConversazioni(getConversazioniOdierneOperatore(pannello, operatoreId, campagna, tipo));
		rcoo.setAppuntamenti(getAppuntamentiOdierneOperatore(operatoreId, campagna, tipo));
		rcoo.setContratti(getContrattiOdierneOperatore(operatoreId, campagna, tipo));
		return rcoo;
	}
	public ReportContattiOdierni getContattiOdierniOperatori(Utente user, Long campagna, int tipo) {
		ReportContattiOdierni rcoo = new ReportContattiOdierni();
		Integer pannello = 1;
		List<Long> utenti = trovaUserBySupervisoreId(user);
		if(ApplicationUtil.notEmpty(utenti)) {
			rcoo.setArchiviazioni(getArchiviazioniOdierneOperatori(pannello, utenti, campagna, tipo));
			rcoo.setContatti(getContattiOdierneOperatori(pannello, utenti, campagna, tipo));
			rcoo.setConversazioni(getConversazioniOdierneOperatori(pannello, utenti, campagna, tipo));
			rcoo.setAppuntamenti(getAppuntamentiOdierneOperatori(utenti, campagna, tipo));
			rcoo.setContratti(getContrattiOdierneOperatori(utenti, campagna, tipo));
		}
		return rcoo;
	}
	public int getArchiviazioniOdierneOperatoriTeam(Integer pannello, Long team, int tipo) {
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContatto.class);
		DetachedCriteria criteriaCO = criteria.createCriteria("operatore", "op");
		DetachedCriteria criteriaCA = criteria.createCriteria("contatto").createCriteria("campagna", "camp");
		impostazioneDao.subCriteriaRoleTeam(team, criteria, "camp", "op");
		ApplicationUtil.pannello(pannello, criteria);
		if(ApplicationUtil.isTrue(sd)) {
			impostazioneDao.subCriteriaCampagnaTeam(team, criteriaCA, "id");
		}
		impostazioneDao.subCriteriaOperatoreTeam(team, criteriaCO, "id");
		criteria.add(Restrictions.isNotNull("archiviazione.id"));
		Date dataInizio = ApplicationUtil.dataInizio(tipo, ApplicationUtil.date());
		criteria.add(Restrictions.gt("data", dataInizio));
		return getListSize(criteria);
	}
	public int getContattiOdierneOperatoriTeam(Integer pannello, Long team, int tipo) {
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContatto.class);
		DetachedCriteria criteriaCO = criteria.createCriteria("operatore", "op");
		DetachedCriteria criteriaCA = criteria.createCriteria("contatto").createCriteria("campagna", "camp");
		impostazioneDao.subCriteriaRoleTeam(team, criteria, "camp", "op");
		ApplicationUtil.pannello(pannello, criteria);
		if(ApplicationUtil.isTrue(sd)) {
			impostazioneDao.subCriteriaCampagnaTeam(team, criteriaCA, "id");
		}
		impostazioneDao.subCriteriaOperatoreTeam(team, criteriaCO, "id");
		Date dataInizio = ApplicationUtil.dataInizio(tipo, ApplicationUtil.date());
		criteria.add(Restrictions.gt("data", dataInizio));
		return getListSize(criteria);
	}
	public int getContattiOdierneOperatoriZona(Long zona, int tipo) {
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class);
		DetachedCriteria criteriaAg = criteria.createCriteria("agente", "op");
		DetachedCriteria criteriaCA = criteria.createCriteria("campagna", "camp");
		impostazioneDao.subCriteriaRoleZona(zona, criteria, "camp", "op");
		if(ApplicationUtil.isTrue(sd)) {
			criteriaCA.createCriteria("zone", "TS").add(Restrictions.idEq(zona));
		}
		criteriaAg.createCriteria("zone", "TC").add(Restrictions.eq("id", zona));
		ApplicationUtil.intervallo(true, tipo, criteria);
		return getListSize(criteria);
	}
	public int getContrattiOdierneOperatoriTeam(Long team, int tipo) {
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(Contratto.class);
		DetachedCriteria criteriaCO = criteria.createCriteria("operatore", "op");
		DetachedCriteria criteriaCA = criteria.createCriteria("contatto").createCriteria("campagna", "camp");
		impostazioneDao.subCriteriaRoleTeam(team, criteria, "camp", "op");
		if(ApplicationUtil.isTrue(sd)) {
			impostazioneDao.subCriteriaCampagnaTeam(team, criteriaCA, "id");
		}
		impostazioneDao.subCriteriaOperatoreTeam(team, criteriaCO, "id");
		ApplicationUtil.intervallo(true, tipo, criteria);
		return getListSize(criteria);
	}
	public int getContrattiOdierneOperatoriZona(Long zona, int tipo) {
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(Contratto.class);
		DetachedCriteria criteriaAg = criteria.createCriteria("agente", "op");
		DetachedCriteria criteriaCA = criteria.createCriteria("contatto").createCriteria("campagna", "camp");
		impostazioneDao.subCriteriaRoleZona(zona, criteria, "camp", "op");
		if(ApplicationUtil.isTrue(sd)) {
			criteriaCA.createCriteria("zone", "TS").add(Restrictions.idEq(zona));
		}
		criteriaAg.createCriteria("zone", "TC").add(Restrictions.eq("id", zona));
		ApplicationUtil.intervallo(true, tipo, criteria);
		return getListSize(criteria);
	}
	public int getConversazioniOdierneOperatoriTeam(Integer pannello, Long team, int tipo) {
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContatto.class);
		DetachedCriteria criteriaCO = criteria.createCriteria("operatore", "op");
		DetachedCriteria criteriaCA = criteria.createCriteria("contatto").createCriteria("campagna", "camp");
		impostazioneDao.subCriteriaRoleTeam(team, criteria, "camp", "op");
		ApplicationUtil.pannello(pannello, criteria);
		if(ApplicationUtil.isTrue(sd)) {
			impostazioneDao.subCriteriaCampagnaTeam(team, criteriaCA, "id");
		}
		impostazioneDao.subCriteriaOperatoreTeam(team, criteriaCO, "id");
		criteria.add(Restrictions.gt("secondi_conv_eff", 0));
		Date dataInizio = ApplicationUtil.dataInizio(tipo, ApplicationUtil.date());
		criteria.add(Restrictions.gt("data", dataInizio));
		return getListSize(criteria);
	}
	public int getAppuntamentiOdierneOperatoriZona(Long zona, int tipo) {
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(Appuntamento.class);
		DetachedCriteria criteriaAg = criteria.createCriteria("agente", "op");
		DetachedCriteria criteriaCA = criteria.createCriteria("contatto").createCriteria("campagna", "camp");
		impostazioneDao.subCriteriaRoleZona(zona, criteria, "camp", "op");
		if(ApplicationUtil.isTrue(sd)) {
			criteriaCA.createCriteria("zone", "TS").add(Restrictions.idEq(zona));
		}
		criteriaAg.createCriteria("zone", "TC").add(Restrictions.eq("id", zona));
		ApplicationUtil.intervallo(true, tipo, criteria);
		return getListSize(criteria);
	}
	public int getAppuntamentiOdierneOperatoriTeam(Long team, int tipo) {
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(Appuntamento.class);
		DetachedCriteria criteriaCO = criteria.createCriteria("operatore", "op");
		DetachedCriteria criteriaCA = criteria.createCriteria("contatto").createCriteria("campagna", "camp");
		impostazioneDao.subCriteriaRoleTeam(team, criteria, "camp", "op");
		if(ApplicationUtil.isTrue(sd)) {
			impostazioneDao.subCriteriaCampagnaTeam(team, criteriaCA, "id");
		}
		impostazioneDao.subCriteriaOperatoreTeam(team, criteriaCO, "id");
		ApplicationUtil.intervallo(true, tipo, criteria);
		return getListSize(criteria);
	}
	public void getZipContratto(Contratto contratto, DownloadBean downloadBean) {
		try {
			String path = ApplicationUtil.normalizeNoEndSeparator(impostazioneDao.getPathBase(ConstantsGlobal.path_pda));
			File dir = new File(path);
			List<Allegato> files = new ArrayList<Allegato>();
			if(dir.exists() && dir.isDirectory()) {
				File[] fileS = dir.listFiles(new ModuloFileFilter(ConstantsGlobal.pda + "_" +contratto.getId()+"_", true));
				if (ApplicationUtil.notEmpty(fileS)) {
					for(File file : fileS) {
						MyFile pda = campagnaDao.reloadFileP(contratto != null ? contratto.getId() : null, path + "/" + ApplicationUtil.getNameN(file.getName()));
						if(pda.getData() == null) {
							ApplicationUtil.addFile(files, ApplicationUtil.getNameN(pda.getName()), pda.getName(), pda.getModified());
						} else {
							ApplicationUtil.addFile(files, ApplicationUtil.getNameN(pda.getName()), pda.getData(), pda.getModified());
						}
					}
				} 
			}
			if (ApplicationUtil.empty(files)) {
				ApplicationUtil.fileWriter(files, ConstantsLanguage.NO_DOCUMENTO.stampa(), ConstantsLanguage.NO_DOCUMENTI.stampa());
			}
			String fileName = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.CONTRATTO.stampa(), contratto.getId(), contratto.getContatto().getAnagrafica().getDenominazione()) +"." + ConstantsMap.zip;
			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(fileName, files);
			String p = ConstantsGlobal.secretZ;

			ApplicationUtil.zip(null, files, out, p);
			downloadBean.close();

		} catch(Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void getZipContratto(Contratto contratto, List<MyFile> mFiles, DownloadBean downloadBean) {
		try {
			String path = ApplicationUtil.normalizeNoEndSeparator(impostazioneDao.getPathBase(ConstantsGlobal.path_pda));
			List<Allegato> files = new ArrayList<Allegato>();
			if (ApplicationUtil.notEmpty(mFiles)) {
				Iterator<MyFile> itS = mFiles.iterator();
				while (itS.hasNext()) {
					MyFile file = (MyFile) itS.next();
					MyFile pda = campagnaDao.reloadFileP(contratto != null ? contratto.getId() : file.getId(), path + "/" + ApplicationUtil.getNameN(file.getName()));
					if(pda.getData() == null) {
						ApplicationUtil.addFile(files, ApplicationUtil.getNameN(pda.getName()), pda.getName(), pda.getModified());
					} else {
						ApplicationUtil.addFile(files, ApplicationUtil.getNameN(pda.getName()), pda.getData(), pda.getModified());
					}
				}
			}

			if (ApplicationUtil.empty(files)) {
				ApplicationUtil.fileWriter(files, ConstantsLanguage.NO_DOCUMENTO.stampa(), ConstantsLanguage.NO_DOCUMENTI.stampa());
			}
			String fileName = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.CONTRATTO.stampa(), contratto.getId(), contratto.getContatto().getAnagrafica().getDenominazione()) + "." + ConstantsMap.zip;
			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(fileName, files);
			String p = ConstantsGlobal.secretZ;

			ApplicationUtil.zip(null, files, out, p);
			downloadBean.close();

		} catch(Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void getZipContrattoA(Contratto contratto, List<MyFile> mFiles, DownloadBean downloadBean) {
		try {
			List<Allegato> files = new ArrayList<Allegato>();
			if (ApplicationUtil.notEmpty(mFiles)) {
				String path = ApplicationUtil.normalizeNoEndSeparator(impostazioneDao.getPathBase(ConstantsGlobal.path_pda));
				Iterator<MyFile> itS = mFiles.iterator();
				while (itS.hasNext()) {
					MyFile file = (MyFile) itS.next();
					MyFile pda = campagnaDao.reloadFileP(contratto != null ? contratto.getId() : file.getId(), path + "/" + ApplicationUtil.getNameN(file.getName()));
					if(pda.getData() == null) {
						ApplicationUtil.addFile(files, ApplicationUtil.getNameN(pda.getName()), pda.getName(), pda.getModified());
					} else {
						ApplicationUtil.addFile(files, ApplicationUtil.getNameN(pda.getName()), pda.getData(), pda.getModified());
					}
				}


				String fileName = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.PDA.stampa()) +"." + ConstantsMap.zip;
				OutputStream out = downloadBean.outputStream();
				downloadBean.filename(fileName, files);
				String p = ConstantsGlobal.secretZ;

				ApplicationUtil.zip(null, files, out, p);
				downloadBean.close();

			} 
		} catch(Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void getZipContrattoConcatenato(Contratto contratto, List<MyFile> mFiles, DownloadBean downloadBean) {
		try {
			List<Allegato> files = new ArrayList<Allegato>();
			if (ApplicationUtil.notEmpty(mFiles)) {
				String path = ApplicationUtil.normalizeNoEndSeparator(impostazioneDao.getPathBase(ConstantsGlobal.path_pda));
				String p = ConstantsGlobal.secretL;
				Iterator<MyFile> itS = mFiles.iterator();
				while (itS.hasNext()) {
					MyFile file = (MyFile) itS.next();
					MyFile pda = campagnaDao.reloadFileP(contratto != null ? contratto.getId() : file.getId(), path + "/" + ApplicationUtil.getNameN(file.getName()));
					if(pda.getData() == null) {
						ApplicationUtil.addFile(files, ApplicationUtil.getNameN(pda.getName()), pda.getName(), pda.getModified());
					} else {
						ApplicationUtil.addFile(files, ApplicationUtil.getNameN(pda.getName()), pda.getData(), pda.getModified());
					}
				}


				String fileName = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.PDA.stampa()) +"." + ConstantsMap.pdf;
				OutputStream out = downloadBean.outputStream();
				downloadBean.filename(fileName, files);
				String w = downloadDao.getWatermark();
				ApplicationUtil.concatenaPDF(w, files, out, p);
				downloadBean.close();

			} 
		} catch(Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public List<Dettaglio> getReportAppuntamenti() {
		return getEsitiAppuntamento(); 
	}
	public List<Dettaglio> getReportContratti() {
		return getEsitiContratto(); 
	}
	public List<String> getOperatori(BachecaCriteria lc, Integer tipologiaEsitoPopup) {
		List<Classifica> posizioni_esiti = getEsitiXSecondi(lc.getPannello(), ConstantsGlobal.bachecaSecondi, tipologiaEsitoPopup);
		List<String> operatori = new ArrayList<String>();
		for(Classifica c : posizioni_esiti)
			operatori.add(c.getDenominazione());
		return operatori;
	}
	public List<CampagnaBacheca> getBachecaContatto(BachecaCriteria lc) {
		List<CampagnaBacheca> bacheca = new ArrayList<CampagnaBacheca>();
		List<Campagna> campagne = getCampagneAttive(lc.getTeamCampagna(), lc.getLimiteCampagna());
		Date oggi = ApplicationUtil.date();
		Iterator<Campagna> campagneIterator = campagne.iterator();
		while(campagneIterator.hasNext()) {
			Campagna campagna = campagneIterator.next();
			CampagnaBacheca cw = new CampagnaBacheca();
			cw.setContatti(getRichiamiPubbliciSize(campagna.getId()));
			cw.setCampagna(campagna);
			//cw.setContent(campagna.getContent());
			List<RiepilogoEsitoAppoggio> ret = getCountEsitiArchiviazioneByCriteria(lc.getPannello(), campagna.getId(), null, null);
			int tot = 0, pos = 0, util = 0;
			for(RiepilogoEsitoAppoggio riga : ret) {
				if(ApplicationUtil.isTrue(riga.getPositivo()))
					pos = pos + riga.getNumero().intValue();
				if(ApplicationUtil.isTrue(riga.getUtile())) 
					util = util + riga.getNumero().intValue();
				tot = tot + riga.getNumero().intValue();
			}
			cw.setArchiviazioni_tot(tot);
			cw.setEsitiPositivi_tot(pos);
			cw.setEsitiUtili_tot(util);
			if(ApplicationUtil.gtZero(lc.getTipologia())) {
				Date dataInizio = ApplicationUtil.dataInizio(lc.getTipologia(), oggi);
				List<RiepilogoEsitoAppoggio> r = getCountEsitiArchiviazioneByCriteria(lc.getPannello(), campagna.getId(), dataInizio, null);
				tot = 0; pos = 0; util = 0;
				for(RiepilogoEsitoAppoggio riga : r) {
					if(ApplicationUtil.isTrue(riga.getPositivo()))
						pos = pos + riga.getNumero().intValue();
					if(ApplicationUtil.isTrue(riga.getUtile())) 
						util = util + riga.getNumero().intValue();
					tot = tot + riga.getNumero().intValue();
				}
				cw.setArchiviazioni(tot);
				cw.setEsitiPositivi(pos);
				cw.setEsitiUtili(util);
			}
			List<Classifica> posizioni_classifica = getClassificaByCampagnaId(lc.getPannello(), campagna.getId(), lc );
			int i = 1;
			StringBuilder sb = new StringBuilder();
			for(Classifica c : posizioni_classifica) {
				if(i!=1) {
					sb.append(" - ");
				}
				sb.append(i++).append(". ").append(c.getDenominazione()).append(" [").append(c.getNumero()).append("]");
			}
			cw.setClassifica(sb.toString());
			bacheca.add(cw);
		}
		return bacheca;
	}
	public List<String> getOperatori(BachecaCriteria lc, Long idCampagna, Integer tipologiaEsitoPopup) {
		if(ApplicationUtil.ltZero(idCampagna))
			return new ArrayList<String>();
		List<Classifica> posizioni_esiti = getEsitiXSecondiByCampagnaId(lc.getPannello(), ConstantsGlobal.bachecaSecondi, idCampagna, tipologiaEsitoPopup);
		List<String> operatori = new ArrayList<String>();
		for(Classifica c : posizioni_esiti)
			operatori.add(c.getDenominazione());
		return operatori;
	}
	public CampagnaBacheca getBachecaContatto(Long idCampagna, BachecaCriteria lc) {
		if(ApplicationUtil.ltZero(idCampagna))
			return new CampagnaBacheca();
		Date oggi = ApplicationUtil.date();
		Campagna campagna = campagnaDao.getCampagna(idCampagna);
		CampagnaBacheca cw = new CampagnaBacheca();
		cw.setContatti(getRichiamiPubbliciSize(campagna.getId()));
		cw.setCampagna(campagna);
		//cw.setContent(campagna.getContent());
		List<RiepilogoEsitoAppoggio> ret = getCountEsitiArchiviazioneByCriteria(lc.getPannello(), campagna.getId(), null, null);
		int tot = 0, pos = 0, util = 0;
		for(RiepilogoEsitoAppoggio riga : ret) {
			if(ApplicationUtil.isTrue(riga.getPositivo()))
				pos = pos + riga.getNumero().intValue();
			if(ApplicationUtil.isTrue(riga.getUtile())) 
				util = util + riga.getNumero().intValue();
			tot = tot + riga.getNumero().intValue();
		}
		cw.setArchiviazioni_tot(tot);
		cw.setEsitiPositivi_tot(pos);
		cw.setEsitiUtili_tot(util);
		if(ApplicationUtil.gtZero(lc.getTipologia())) {
			Date dataInizio = ApplicationUtil.dataInizio(lc.getTipologia(), oggi);
			List<RiepilogoEsitoAppoggio> r = getCountEsitiArchiviazioneByCriteria(lc.getPannello(), campagna.getId(), dataInizio, null);
			tot = 0; pos = 0; util = 0;
			for(RiepilogoEsitoAppoggio riga : r) {
				if(ApplicationUtil.isTrue(riga.getPositivo()))
					pos = pos + riga.getNumero().intValue();
				if(ApplicationUtil.isTrue(riga.getUtile())) 
					util = util + riga.getNumero().intValue();
				tot = tot + riga.getNumero().intValue();
			}
			cw.setArchiviazioni(tot);
			cw.setEsitiPositivi(pos);
			cw.setEsitiUtili(util);
		}
		List<Classifica> posizioni_classifica = getClassificaByCampagnaId(lc.getPannello(), campagna.getId(), lc );
		List<String> classificaList = new ArrayList<String>();
		int i = 1;
		String riga;
		StringBuilder sb = new StringBuilder();
		for(Classifica c : posizioni_classifica) {
			riga = new StringBuilder().append(i++).append(". ").append(c.getDenominazione()).append(" [").append(c.getNumero()).append("]").toString();
			sb.append(riga);
			classificaList.add(riga);
		}
		cw.setClassifica(sb.toString());
		cw.setClassificaList(classificaList);
		return cw;
	}
	@SuppressWarnings({ "unchecked" })
	private List<RiepilogoSondaggio> getReportSondaggioAnagraficaApp(Long utenteId, ReportCriteria r){
		DetachedCriteria criteria = DetachedCriteria.forClass(DatoAggiuntivoAnagrafica.class, "da");
		DetachedCriteria criteriaE = criteria.createCriteria("campo_aggiuntivo", "ca");
		ApplicationUtil.dataIntervalD(r.getInizio(), r.getFine(), "data_creazione", criteria);
		if (ApplicationUtil.gtZero(r.getSondaggio())) {
			criteriaE.createCriteria("domande").add(Restrictions.eq("sondaggio.id", r.getSondaggio()));
		} else {
			criteriaE.createCriteria("domande").add(Restrictions.isNotNull("sondaggio.id"));
		}
		if (ApplicationUtil.gtZero(r.getCampo())) {
			criteriaE.add(Restrictions.idEq(r.getCampo()));
		}
		if(ApplicationUtil.gtZero(utenteId)) {
			DetachedCriteria criteriaCA = criteria.createCriteria("anagrafica").createCriteria("contatti").createCriteria("campagna");
			impostazioneDao.subCriteriaCampagnaSupervisore(utenteId, criteriaCA, "id");
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("ca.id").as("eid")))
				.add(Projections.groupProperty("ca.etichetta").as("etichetta"))
				.add(Projections.groupProperty("ca.colore").as("colore"))
				.add(Projections.groupProperty("da.valore").as("risposta"))
				.add(Projections.countDistinct("da.id").as("numero"))
				);
		criteria.addOrder(Order.desc("numero"));
		return getList(criteria, RiepilogoSondaggio.class);
	}
	@SuppressWarnings({ "unchecked" })
	private List<RiepilogoSondaggio> getReportSondaggioAnagraficaAppPartner(Long utenteId, ReportCriteria r){
		DetachedCriteria criteria = DetachedCriteria.forClass(DatoAggiuntivoAnagrafica.class, "da");
		DetachedCriteria criteriaE = criteria.createCriteria("campo_aggiuntivo", "ca");
		ApplicationUtil.dataIntervalD(r.getInizio(), r.getFine(), "data_creazione", criteria);
		if (ApplicationUtil.gtZero(r.getSondaggio())) {
			criteriaE.createCriteria("domande").add(Restrictions.eq("sondaggio.id", r.getSondaggio()));
		} else {
			criteriaE.createCriteria("domande").add(Restrictions.isNotNull("sondaggio.id"));
		}
		if (ApplicationUtil.gtZero(r.getCampo())) {
			criteriaE.add(Restrictions.idEq(r.getCampo()));
		}
		if(ApplicationUtil.gtZero(utenteId)) {
			DetachedCriteria criteriaCA = criteria.createCriteria("anagrafica").createCriteria("contatti").createCriteria("campagna");
			impostazioneDao.subCriteriaCampagnaPartner(utenteId, criteriaCA, "id");
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("ca.id").as("eid")))
				.add(Projections.groupProperty("ca.etichetta").as("etichetta"))
				.add(Projections.groupProperty("ca.colore").as("colore"))
				.add(Projections.groupProperty("da.valore").as("risposta"))
				.add(Projections.countDistinct("da.id").as("numero"))
				);
		criteria.addOrder(Order.desc("numero"));
		return getList(criteria, RiepilogoSondaggio.class);
	}
	@SuppressWarnings({ "unchecked" })
	private List<RiepilogoSondaggio> getReportSondaggioContrattoApp(Long utenteId, ReportCriteria r){
		DetachedCriteria criteria = DetachedCriteria.forClass(DatoAggiuntivoContratto.class, "da");
		DetachedCriteria criteriaE = criteria.createCriteria("campo_aggiuntivo", "ca");
		ApplicationUtil.dataIntervalD(r.getInizio(), r.getFine(), "data_creazione", criteria);
		if (ApplicationUtil.gtZero(r.getSondaggio())) {
			criteriaE.createCriteria("domande").add(Restrictions.eq("sondaggio.id", r.getSondaggio()));
		} else {
			criteriaE.createCriteria("domande").add(Restrictions.isNotNull("sondaggio.id"));
		}
		if (ApplicationUtil.gtZero(r.getCampo())) {
			criteriaE.add(Restrictions.idEq(r.getCampo()));
		}
		if(ApplicationUtil.gtZero(utenteId)) {
			DetachedCriteria criteriaCA = criteria.createCriteria("contratto").createCriteria("contatto").createCriteria("campagna");
			impostazioneDao.subCriteriaCampagnaSupervisore(utenteId, criteriaCA, "id");
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("ca.id").as("eid")))
				.add(Projections.groupProperty("ca.etichetta").as("etichetta"))
				.add(Projections.groupProperty("ca.colore").as("colore"))
				.add(Projections.groupProperty("da.valore").as("risposta"))
				.add(Projections.countDistinct("da.id").as("numero"))
				);
		criteria.addOrder(Order.desc("numero"));
		return getList(criteria, RiepilogoSondaggio.class);
	}
	@SuppressWarnings({ "unchecked" })
	private List<RiepilogoSondaggio> getReportSondaggioContrattoAppPartner(Long utenteId, ReportCriteria r){
		DetachedCriteria criteria = DetachedCriteria.forClass(DatoAggiuntivoContratto.class, "da");
		DetachedCriteria criteriaE = criteria.createCriteria("campo_aggiuntivo", "ca");
		ApplicationUtil.dataIntervalD(r.getInizio(), r.getFine(), "data_creazione", criteria);
		if (ApplicationUtil.gtZero(r.getSondaggio())) {
				criteriaE.createCriteria("domande").add(Restrictions.eq("sondaggio.id", r.getSondaggio()));
		} else {
			criteriaE.createCriteria("domande").add(Restrictions.isNotNull("sondaggio.id"));
		}
		if (ApplicationUtil.gtZero(r.getCampo())) {
			criteriaE.add(Restrictions.idEq(r.getCampo()));
		}
		if(ApplicationUtil.gtZero(utenteId)) {
			DetachedCriteria criteriaCA = criteria.createCriteria("contratto").createCriteria("contatto").createCriteria("campagna");
			impostazioneDao.subCriteriaCampagnaPartner(utenteId, criteriaCA, "id");
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("ca.id").as("eid")))
				.add(Projections.groupProperty("ca.etichetta").as("etichetta"))
				.add(Projections.groupProperty("ca.colore").as("colore"))
				.add(Projections.groupProperty("da.valore").as("risposta"))
				.add(Projections.countDistinct("da.id").as("numero"))
				);
		criteria.addOrder(Order.desc("numero"));
		return getList(criteria, RiepilogoSondaggio.class);
	}
	public List<CampagnaBacheca> getBachecaAppuntamento(BachecaCriteria lc) {
		List<CampagnaBacheca> bacheca = new ArrayList<CampagnaBacheca>();
		List<Campagna> campagne = getCampagneAttive(lc.getTeamCampagna(), lc.getLimiteCampagna());
		Date oggi = ApplicationUtil.date();
		Iterator<Campagna> campagneIterator = campagne.iterator();
		while(campagneIterator.hasNext()) {
			Campagna campagna = campagneIterator.next();
			CampagnaBacheca cw = new CampagnaBacheca();
			cw.setContatti(getRichiamiPubbliciSize(campagna.getId()));
			cw.setCampagna(campagna);
			//cw.setContent(campagna.getContent());
			List<RiepilogoEsitoAppoggio> ret = getCountAppuntamentoByCriteria(campagna.getId(), null, null);
			int tot = 0, pos = 0, util = 0;
			for(RiepilogoEsitoAppoggio riga : ret) {
				if(ApplicationUtil.isTrue(riga.getPositivo()))
					pos = pos + riga.getNumero().intValue();
				if(ApplicationUtil.isTrue(riga.getUtile())) 
					util = util + riga.getNumero().intValue();
				tot = tot + riga.getNumero().intValue();
			}
			cw.setArchiviazioni_tot(tot);
			cw.setEsitiPositivi_tot(pos);
			cw.setEsitiUtili_tot(util);
			if(ApplicationUtil.gtZero(lc.getTipologia())) {
				Date dataInizio = ApplicationUtil.dataInizio(lc.getTipologia(), oggi);
				List<RiepilogoEsitoAppoggio> r = getCountAppuntamentoByCriteria(campagna.getId(), dataInizio, null);
				tot = 0; pos = 0; util = 0;
				for(RiepilogoEsitoAppoggio riga : r) {
					if(ApplicationUtil.isTrue(riga.getPositivo()))
						pos = pos + riga.getNumero().intValue();
					if(ApplicationUtil.isTrue(riga.getUtile())) 
						util = util + riga.getNumero().intValue();
					tot = tot + riga.getNumero().intValue();
				}
				cw.setArchiviazioni(tot);
				cw.setEsitiPositivi(pos);
				cw.setEsitiUtili(util);
			}
			List<Classifica> posizioni_classifica = getClassificaAppByCampagnaId(campagna.getId(), lc );
			int i = 1;
			StringBuilder sb = new StringBuilder();
			for(Classifica c : posizioni_classifica) {
				if(i!=1) {
					sb.append(" - ");
				}
				sb.append(i++).append(". ").append(c.getDenominazione()).append(" [").append(c.getNumero()).append("]");
			}
			cw.setClassifica(sb.toString());
			bacheca.add(cw);
		}
		return bacheca;
	}
	public CampagnaBacheca getBachecaAppuntamento(Long idCampagna, BachecaCriteria lc) {
		if(ApplicationUtil.ltZero(idCampagna))
			return new CampagnaBacheca();
		Date oggi = ApplicationUtil.date();
		Campagna campagna = campagnaDao.getCampagna(idCampagna);
		CampagnaBacheca cw = new CampagnaBacheca();
		cw.setContatti(getRichiamiPubbliciSize(campagna.getId()));
		cw.setCampagna(campagna);
		//cw.setContent(campagna.getContent());
		List<RiepilogoEsitoAppoggio> ret = getCountAppuntamentoByCriteria(campagna.getId(), null, null);
		int tot = 0, pos = 0, util = 0;
		for(RiepilogoEsitoAppoggio riga : ret) {
			if(ApplicationUtil.isTrue(riga.getPositivo()))
				pos = pos + riga.getNumero().intValue();
			if(ApplicationUtil.isTrue(riga.getUtile())) 
				util = util + riga.getNumero().intValue();
			tot = tot + riga.getNumero().intValue();
		}
		cw.setArchiviazioni_tot(tot);
		cw.setEsitiPositivi_tot(pos);
		cw.setEsitiUtili_tot(util);
		if(ApplicationUtil.gtZero(lc.getTipologia())) {
			Date dataInizio = ApplicationUtil.dataInizio(lc.getTipologia(), oggi);
			List<RiepilogoEsitoAppoggio> r = getCountAppuntamentoByCriteria(campagna.getId(), dataInizio, null);
			tot = 0; pos = 0; util = 0;
			for(RiepilogoEsitoAppoggio riga : r) {
				if(ApplicationUtil.isTrue(riga.getPositivo()))
					pos = pos + riga.getNumero().intValue();
				if(ApplicationUtil.isTrue(riga.getUtile())) 
					util = util + riga.getNumero().intValue();
				tot = tot + riga.getNumero().intValue();
			}
			cw.setArchiviazioni(tot);
			cw.setEsitiPositivi(pos);
			cw.setEsitiUtili(util);
		}
		List<Classifica> posizioni_classifica = getClassificaAppByCampagnaId(campagna.getId(), lc );
		List<String> classificaList = new ArrayList<String>();
		int i = 1;
		String riga;
		StringBuilder sb = new StringBuilder();
		for(Classifica c : posizioni_classifica) {
			riga = new StringBuilder().append(i++).append(". ").append(c.getDenominazione()).append(" [").append(c.getNumero()).append("]").toString();
			sb.append(riga);
			classificaList.add(riga);
		}
		cw.setClassifica(sb.toString());
		cw.setClassificaList(classificaList);
		return cw;
	}
	@SuppressWarnings({ "unchecked", "unused" })
	public List<RiepilogoEsitoAppoggio> getCountAppuntamentoByCriteria(Long campagnaId,  Date dateInizio, Date dateFine) {
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaAppuntamento.class, "sc");
		DetachedCriteria criteriaA= criteria.createCriteria("esito", "arch",  ApplicationUtil.leftJoin());
		DetachedCriteria criteriaCont = criteria.createCriteria("appuntamento", "cont");
		DetachedCriteria criteriaC = criteriaCont.createCriteria("contatto");
		DetachedCriteria criteriaCA = criteriaC.createCriteria("campagna", "camp");
		criteriaCA.add(Restrictions.idEq(campagnaId));
		ApplicationUtil.dataIntervalD(dateInizio, dateFine, "data", criteria);
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("arch.confermato").as("positivo")))
				.add(Projections.groupProperty("arch.utile").as("utile"))
				.add(Projections.countDistinct("sc.id").as("numero"))
				);
		criteria.addOrder(Order.desc("numero"));
		return getList(criteria, RiepilogoEsitoAppoggio.class);
	}
	@SuppressWarnings({ "unchecked", "unused" })
	public List<Classifica> getClassificaAppByCampagnaId(Long campagnaId, BachecaCriteria c) {
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaAppuntamento.class, "sc");
		DetachedCriteria criteriaA = criteria.createCriteria("esito", "arch",  ApplicationUtil.leftJoin());
		DetachedCriteria criteriaCont = criteria.createCriteria("appuntamento", "cont");
		DetachedCriteria criteriaC = criteriaCont.createCriteria("contatto");
		DetachedCriteria criteriaO = criteria.createCriteria("utente", "op");
		DetachedCriteria criteriaCA = criteriaC.createCriteria("campagna", "camp");
		if(ApplicationUtil.gtZero(c.getTipologiaEsito())) {
			if(ApplicationUtil.eq(c.getTipologiaEsito(),1)) {
				criteriaA.add(Restrictions.eq("confermato", Boolean.TRUE));
			}
			if(ApplicationUtil.eq(c.getTipologiaEsito(),2)) {
				criteriaA.add(Restrictions.eq("utile", Boolean.TRUE));
			}
		}
		if(ApplicationUtil.gtZero(c.getTipologiaOperatore())) {
			Date oggi = ApplicationUtil.date();
			Date dataInizio = ApplicationUtil.dataInizio(c.getTipologiaOperatore(), oggi);
			ApplicationUtil.dataDate( dataInizio, null, "data", criteria);
		}
		criteriaCA.add(Restrictions.idEq(campagnaId));
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("op.username").as("username")))
				.add(Projections.property("op.cognome").as("cognome"))
				.add(Projections.property("op.nome").as("nome"))
				.add(Projections.countDistinct("sc.id").as("numero"))
				);
		criteria.addOrder(Order.desc("numero"));
		return getList(criteria, 0, c.getLimiteOperatore(), Classifica.class);
	}
	@SuppressWarnings({ "unchecked", "unused" })
	public List<Classifica> getAppuntamentiXSecondi(int i,  Integer tipologiaEsitoPopup) {
		Date dataFine = ApplicationUtil.date();
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaAppuntamento.class, "sc");
		DetachedCriteria criteriaA = criteria.createCriteria("esito", "arch",  ApplicationUtil.leftJoin());
		DetachedCriteria criteriaO = criteria.createCriteria("utente", "op");
		ApplicationUtil.dataDate(ApplicationUtil.rimuoviSecondi(dataFine, i), dataFine, "data", criteria);
		if(ApplicationUtil.gtZero(tipologiaEsitoPopup)) {
			if(ApplicationUtil.eq(tipologiaEsitoPopup,1)) {
				criteriaA.add(Restrictions.eq("confermato", Boolean.TRUE));
			}
			if(ApplicationUtil.eq(tipologiaEsitoPopup,2)) {
				criteriaA.add(Restrictions.eq("utile", Boolean.TRUE));
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.property("op.username").as("username")))
				.add(Projections.property("op.cognome").as("cognome"))
				.add(Projections.property("op.nome").as("nome"))
				);
		return getList(criteria, Classifica.class);
	}
	@SuppressWarnings({ "unchecked", "unused" })
	public List<Classifica> getAppuntamentiXSecondiByCampagnaId(int i, Long campagnaId,  Integer tipologiaEsitoPopup) {
		Date dataInizio = ApplicationUtil.date();
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaAppuntamento.class, "sc");
		DetachedCriteria criteriaA = criteria.createCriteria("esito", "arch",  ApplicationUtil.leftJoin());
		DetachedCriteria criteriaO = criteria.createCriteria("utente", "op");
		DetachedCriteria criteriaCont = criteria.createCriteria("appuntamento", "cont");
		DetachedCriteria criteriaC = criteriaCont.createCriteria("contatto");
		DetachedCriteria criteriaCA = criteriaC.createCriteria("campagna", "camp");
		criteriaCA.add(Restrictions.idEq(campagnaId));
		ApplicationUtil.dataDate(ApplicationUtil.rimuoviSecondi(dataInizio, i), dataInizio, "data", criteria);
		if(ApplicationUtil.gtZero(tipologiaEsitoPopup)) {
			if(ApplicationUtil.eq(tipologiaEsitoPopup,1)) {
				criteriaA.add(Restrictions.eq("confermato", Boolean.TRUE));
			}
			if(ApplicationUtil.eq(tipologiaEsitoPopup,2)) {
				criteriaA.add(Restrictions.eq("utile", Boolean.TRUE));
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.property("op.username").as("username")))
				.add(Projections.property("op.cognome").as("cognome"))
				.add(Projections.property("op.nome").as("nome"))
				);
		return getList(criteria, Classifica.class);
	}
	public List<String> getOperatoriAppuntamento(Integer tipologiaEsitoPopup) {
		List<Classifica> posizioni_esiti = getAppuntamentiXSecondi(ConstantsGlobal.bachecaSecondi, tipologiaEsitoPopup);
		List<String> operatori = new ArrayList<String>();
		for(Classifica c : posizioni_esiti)
			operatori.add(c.getDenominazione());
		return operatori;
	}
	public List<String> getOperatoriAppuntamento(Long idCampagna, Integer tipologiaEsitoPopup) {
		if(ApplicationUtil.ltZero(idCampagna))
			return new ArrayList<String>();
		List<Classifica> posizioni_esiti = getAppuntamentiXSecondiByCampagnaId(ConstantsGlobal.bachecaSecondi, idCampagna, tipologiaEsitoPopup);
		List<String> operatori = new ArrayList<String>();
		for(Classifica c : posizioni_esiti)
			operatori.add(c.getDenominazione());
		return operatori;
	}
	public List<CampagnaBacheca> getBachecaContratto(BachecaCriteria lc) {
		List<CampagnaBacheca> bacheca = new ArrayList<CampagnaBacheca>();
		List<Campagna> campagne = getCampagneAttive(lc.getTeamCampagna(), lc.getLimiteCampagna());
		Date oggi = ApplicationUtil.date();
		Iterator<Campagna> campagneIterator = campagne.iterator();
		while(campagneIterator.hasNext()) {
			Campagna campagna = campagneIterator.next();
			CampagnaBacheca cw = new CampagnaBacheca();
			cw.setContatti(getRichiamiPubbliciSize(campagna.getId()));
			cw.setCampagna(campagna);
			//cw.setContent(campagna.getContent());
			List<RiepilogoEsitoAppoggio> ret = getCountContrattoByCriteria(campagna.getId(), null, null);
			int tot = 0, pos = 0, util = 0;
			for(RiepilogoEsitoAppoggio riga : ret) {
				if(ApplicationUtil.isTrue(riga.getPositivo()))
					pos = pos + riga.getNumero().intValue();
				if(ApplicationUtil.isTrue(riga.getUtile())) 
					util = util + riga.getNumero().intValue();
				tot = tot + riga.getNumero().intValue();
			}
			cw.setArchiviazioni_tot(tot);
			cw.setEsitiPositivi_tot(pos);
			cw.setEsitiUtili_tot(util);
			if(ApplicationUtil.gtZero(lc.getTipologia())) {
				Date dataInizio = ApplicationUtil.dataInizio(lc.getTipologia(), oggi);
				List<RiepilogoEsitoAppoggio> r = getCountContrattoByCriteria(campagna.getId(), dataInizio, null);
				tot = 0; pos = 0; util = 0;
				for(RiepilogoEsitoAppoggio riga : r) {
					if(ApplicationUtil.isTrue(riga.getPositivo()))
						pos = pos + riga.getNumero().intValue();
					if(ApplicationUtil.isTrue(riga.getUtile())) 
						util = util + riga.getNumero().intValue();
					tot = tot + riga.getNumero().intValue();
				}
				cw.setArchiviazioni(tot);
				cw.setEsitiPositivi(pos);
				cw.setEsitiUtili(util);
			}
			List<Classifica> posizioni_classifica = getClassificaContByCampagnaId(campagna.getId(), lc );
			int i = 1;
			StringBuilder sb = new StringBuilder();
			for(Classifica c : posizioni_classifica) {
				if(i!=1) {
					sb.append(" - ");
				}
				sb.append(i++).append(". ").append(c.getDenominazione()).append(" [").append(c.getNumero()).append("]");
			}
			cw.setClassifica(sb.toString());
			bacheca.add(cw);
		}
		return bacheca;
	}
	public CampagnaBacheca getBachecaContratto(Long idCampagna, BachecaCriteria lc) {
		if(ApplicationUtil.ltZero(idCampagna))
			return new CampagnaBacheca();
		Date oggi = ApplicationUtil.date();
		Campagna campagna = campagnaDao.getCampagna(idCampagna);
		CampagnaBacheca cw = new CampagnaBacheca();
		cw.setContatti(getRichiamiPubbliciSize(campagna.getId()));
		cw.setCampagna(campagna);
		//cw.setContent(campagna.getContent());
		List<RiepilogoEsitoAppoggio> ret = getCountContrattoByCriteria(campagna.getId(), null, null);
		int tot = 0, pos = 0, util = 0;
		for(RiepilogoEsitoAppoggio riga : ret) {
			if(ApplicationUtil.isTrue(riga.getPositivo()))
				pos = pos + riga.getNumero().intValue();
			if(ApplicationUtil.isTrue(riga.getUtile())) 
				util = util + riga.getNumero().intValue();
			tot = tot + riga.getNumero().intValue();
		}
		cw.setArchiviazioni_tot(tot);
		cw.setEsitiPositivi_tot(pos);
		cw.setEsitiUtili_tot(util);
		if(ApplicationUtil.gtZero(lc.getTipologia())) {
			Date dataInizio = ApplicationUtil.dataInizio(lc.getTipologia(), oggi);
			List<RiepilogoEsitoAppoggio> r = getCountContrattoByCriteria(campagna.getId(), dataInizio, null);
			tot = 0; pos = 0; util = 0;
			for(RiepilogoEsitoAppoggio riga : r) {
				if(ApplicationUtil.isTrue(riga.getPositivo()))
					pos = pos + riga.getNumero().intValue();
				if(ApplicationUtil.isTrue(riga.getUtile())) 
					util = util + riga.getNumero().intValue();
				tot = tot + riga.getNumero().intValue();
			}
			cw.setArchiviazioni(tot);
			cw.setEsitiPositivi(pos);
			cw.setEsitiUtili(util);
		}
		List<Classifica> posizioni_classifica = getClassificaContByCampagnaId(campagna.getId(), lc);
		List<String> classificaList = new ArrayList<String>();
		int i = 1;
		String riga;
		StringBuilder sb = new StringBuilder();
		for(Classifica c : posizioni_classifica) {
			riga = new StringBuilder().append(i++).append(". ").append(c.getDenominazione()).append(" [").append(c.getNumero()).append("]").toString();
			sb.append(riga);
			classificaList.add(riga);
		}
		cw.setClassifica(sb.toString());
		cw.setClassificaList(classificaList);
		return cw;
	}
	@SuppressWarnings({ "unchecked", "unused" })
	public List<RiepilogoEsitoAppoggio> getCountContrattoByCriteria(Long campagnaId, Date dateInizio, Date dateFine) {
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContratto.class, "sc");
		DetachedCriteria criteriaA= criteria.createCriteria("esito", "arch",  ApplicationUtil.leftJoin());
		DetachedCriteria criteriaCont = criteria.createCriteria("contratto", "cont");
		DetachedCriteria criteriaC = criteriaCont.createCriteria("contatto");
		DetachedCriteria criteriaCA = criteriaC.createCriteria("campagna", "camp");
		criteriaCA.add(Restrictions.idEq(campagnaId));
		ApplicationUtil.dataIntervalD(dateInizio, dateFine, "data", criteria);
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("arch.confermato").as("positivo")))
				.add(Projections.groupProperty("arch.utile").as("utile"))
				.add(Projections.countDistinct("sc.id").as("numero"))
				);
		criteria.addOrder(Order.desc("numero"));
		return getList(criteria, RiepilogoEsitoAppoggio.class);
	}
	@SuppressWarnings({ "unchecked", "unused" })
	public List<Classifica> getClassificaContByCampagnaId(Long campagnaId, BachecaCriteria c) {
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContratto.class, "sc");
		DetachedCriteria criteriaA = criteria.createCriteria("esito", "arch",  ApplicationUtil.leftJoin());
		DetachedCriteria criteriaCont = criteria.createCriteria("contratto", "cont");
		DetachedCriteria criteriaC = criteriaCont.createCriteria("contatto");
		DetachedCriteria criteriaO = criteria.createCriteria("utente", "op");
		DetachedCriteria criteriaCA = criteriaC.createCriteria("campagna", "camp");
		if(ApplicationUtil.gtZero(c.getTipologiaEsito())) {
			if(ApplicationUtil.eq(c.getTipologiaEsito(),1)) {
				criteriaA.add(Restrictions.eq("confermato", Boolean.TRUE));
			}
			if(ApplicationUtil.eq(c.getTipologiaEsito(),2)) {
				criteriaA.add(Restrictions.eq("utile", Boolean.TRUE));
			}
		}
		if(ApplicationUtil.gtZero(c.getTipologiaOperatore())) {
			Date oggi = ApplicationUtil.date();
			Date dataInizio = ApplicationUtil.dataInizio(c.getTipologiaOperatore(), oggi);
			ApplicationUtil.dataDate( dataInizio, null, "data", criteria);
		}
		criteriaCA.add(Restrictions.idEq(campagnaId));
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("op.username").as("username")))
				.add(Projections.property("op.cognome").as("cognome"))
				.add(Projections.property("op.nome").as("nome"))
				.add(Projections.countDistinct("sc.id").as("numero"))
				);
		criteria.addOrder(Order.desc("numero"));
		return getList(criteria, 0, c.getLimiteOperatore(), Classifica.class);
	}
	@SuppressWarnings({ "unchecked", "unused" })
	public List<Classifica> getContrattiXSecondi(int i,  Integer tipologiaEsitoPopup) {
		Date dataFine = ApplicationUtil.date();
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContratto.class, "sc");
		DetachedCriteria criteriaA = criteria.createCriteria("esito", "arch",  ApplicationUtil.leftJoin());
		DetachedCriteria criteriaO = criteria.createCriteria("utente", "op");
		ApplicationUtil.dataDate(ApplicationUtil.rimuoviSecondi(dataFine, i), dataFine, "data", criteria);
		if(ApplicationUtil.gtZero(tipologiaEsitoPopup)) {
			if(ApplicationUtil.eq(tipologiaEsitoPopup,1)) {
				criteriaA.add(Restrictions.eq("confermato", Boolean.TRUE));
			}
			if(ApplicationUtil.eq(tipologiaEsitoPopup,2)) {
				criteriaA.add(Restrictions.eq("utile", Boolean.TRUE));
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.property("op.username").as("username")))
				.add(Projections.property("op.cognome").as("cognome"))
				.add(Projections.property("op.nome").as("nome"))
				);
		return getList(criteria, Classifica.class);
	}
	@SuppressWarnings({ "unchecked", "unused" })
	public List<Classifica> getContrattiXSecondiByCampagnaId(int i, Long campagnaId,  Integer tipologiaEsitoPopup) {
		Date dataInizio = ApplicationUtil.date();
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContratto.class, "sc");
		DetachedCriteria criteriaA = criteria.createCriteria("esito", "arch",  ApplicationUtil.leftJoin());
		DetachedCriteria criteriaO = criteria.createCriteria("utente", "op");
		DetachedCriteria criteriaCont = criteria.createCriteria("contratto", "cont");
		DetachedCriteria criteriaC = criteriaCont.createCriteria("contatto");
		DetachedCriteria criteriaCA = criteriaC.createCriteria("campagna", "camp");
		criteriaCA.add(Restrictions.idEq(campagnaId));
		ApplicationUtil.dataDate(ApplicationUtil.rimuoviSecondi(dataInizio, i), dataInizio, "data", criteria);
		if(ApplicationUtil.gtZero(tipologiaEsitoPopup)) {
			if(ApplicationUtil.eq(tipologiaEsitoPopup,1)) {
				criteriaA.add(Restrictions.eq("confermato", Boolean.TRUE));
			}
			if(ApplicationUtil.eq(tipologiaEsitoPopup,2)) {
				criteriaA.add(Restrictions.eq("utile", Boolean.TRUE));
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.property("op.username").as("username")))
				.add(Projections.property("op.cognome").as("cognome"))
				.add(Projections.property("op.nome").as("nome"))
				);
		return getList(criteria, Classifica.class);
	}
	public List<String> getOperatoriContratto(Integer tipologiaEsitoPopup) {
		List<Classifica> posizioni_esiti = getContrattiXSecondi(ConstantsGlobal.bachecaSecondi, tipologiaEsitoPopup);
		List<String> operatori = new ArrayList<String>();
		for(Classifica c : posizioni_esiti)
			operatori.add(c.getDenominazione());
		return operatori;
	}
	public List<String> getOperatoriContratto(Long idCampagna, Integer tipologiaEsitoPopup) {
		if(ApplicationUtil.ltZero(idCampagna))
			return new ArrayList<String>();
		List<Classifica> posizioni_esiti = getContrattiXSecondiByCampagnaId(ConstantsGlobal.bachecaSecondi, idCampagna, tipologiaEsitoPopup);
		List<String> operatori = new ArrayList<String>();
		for(Classifica c : posizioni_esiti)
			operatori.add(c.getDenominazione());
		return operatori;
	}
	public List<AgenteZonaBacheca> classificaZona(ClassificaCriteria a) {
		List<AgenteZonaBacheca> bacheca = new ArrayList<AgenteZonaBacheca>();
		List<Zona> zone = getZone(a.getZonaId(), a.getLimiteZona());
		Date oggi = ApplicationUtil.date();
		Iterator<Zona> zonaIterator = zone.iterator();
		while(zonaIterator.hasNext()) {
			Zona zona = zonaIterator.next();
			if(ApplicationUtil.gtZero(a.getTipologia())) {
				Date dataInizio = ApplicationUtil.dataInizio(a.getTipologia(), oggi);
				List<AgenteZonaClassifica> num = getZonaClassifica(zona.getId(), dataInizio, null, a.getLimiteAgente(), a.getTipoClassifica());
				AgenteZonaBacheca cw = new AgenteZonaBacheca();
				cw.setClassifica(num);
				cw.setZona(zona);
				bacheca.add(cw);
			}
		}
		return bacheca;
	}
	public List<AgenteZonaBacheca> classificaZonaR(ClassificaCriteria a) {
		List<AgenteZonaBacheca> bacheca = new ArrayList<AgenteZonaBacheca>();
		List<Zona> zone = getZone(a.getZonaId(), a.getLimiteZona());
		Date oggi = ApplicationUtil.date();
		Iterator<Zona> zonaIterator = zone.iterator();
		while(zonaIterator.hasNext()) {
			Zona zona = zonaIterator.next();
			if(ApplicationUtil.gtZero(a.getTipologia())) {
				Date dataInizio = ApplicationUtil.dataInizio(a.getTipologia(), oggi);
				List<AgenteZonaClassifica> num = getZonaClassificaR(zona.getId(), dataInizio, null, a.getLimiteAgente(), a.getTipoClassifica());
				AgenteZonaBacheca cw = new AgenteZonaBacheca();
				cw.setClassifica(num);
				cw.setZona(zona);
				bacheca.add(cw);
			}
		}
		return bacheca;
	}
	public List<AgenteQualificaBacheca> classificaQualifica(ClassificaCriteria a) {
		List<AgenteQualificaBacheca> bacheca = new ArrayList<AgenteQualificaBacheca>();
		List<Qualifica> zone = getQualifiche(a.getQualificaId(), a.getLimiteQualifica());
		Date oggi = ApplicationUtil.date();
		Iterator<Qualifica> qualificaIterator = zone.iterator();
		while(qualificaIterator.hasNext()) {
			Qualifica qualifica = qualificaIterator.next();
			if(ApplicationUtil.gtZero(a.getTipologia())) {
				Date dataInizio = ApplicationUtil.dataInizio(a.getTipologia(), oggi);
				List<AgenteQualificaClassifica> num = getQualificaClassifica(qualifica.getId(), dataInizio, null, a.getLimiteAgente(), a.getTipoClassifica());
				AgenteQualificaBacheca cw = new AgenteQualificaBacheca();
				cw.setClassifica(num);
				cw.setQualifica(qualifica);
				bacheca.add(cw);
			}
		}
		return bacheca;
	}
	public List<AgenteQualificaBacheca> classificaQualificaR(ClassificaCriteria a) {
		List<AgenteQualificaBacheca> bacheca = new ArrayList<AgenteQualificaBacheca>();
		List<Qualifica> zone = getQualifiche(a.getQualificaId(), a.getLimiteQualifica());
		Date oggi = ApplicationUtil.date();
		Iterator<Qualifica> qualificaIterator = zone.iterator();
		while(qualificaIterator.hasNext()) {
			Qualifica qualifica = qualificaIterator.next();
			if(ApplicationUtil.gtZero(a.getTipologia())) {
				Date dataInizio = ApplicationUtil.dataInizio(a.getTipologia(), oggi);
				List<AgenteQualificaClassifica> num = getQualificaClassificaR(qualifica.getId(), dataInizio, null, a.getLimiteAgente(), a.getTipoClassifica());
				AgenteQualificaBacheca cw = new AgenteQualificaBacheca();
				cw.setClassifica(num);
				cw.setQualifica(qualifica);
				bacheca.add(cw);
			}
		}
		return bacheca;
	}
	public List<AgenteCampagnaBacheca> classificaCampagna(ClassificaCriteria a) {
		List<AgenteCampagnaBacheca> bacheca = new ArrayList<AgenteCampagnaBacheca>();
		List<Campagna> campagne = getCampagne(a.getCampagnaId(), a.getLimiteCampagna());
		Date oggi = ApplicationUtil.date();
		Iterator<Campagna> campagnaIterator = campagne.iterator();
		while(campagnaIterator.hasNext()) {
			Campagna campagna = campagnaIterator.next();
			if(ApplicationUtil.gtZero(a.getTipologia())) {
				Date dataInizio = ApplicationUtil.dataInizio(a.getTipologia(), oggi);
				List<AgenteCampagnaClassifica> num = getCampagnaClassifica(campagna.getId(), dataInizio, null, a.getLimiteAgente(), a.getTipoClassifica());
				AgenteCampagnaBacheca cw = new AgenteCampagnaBacheca();
				cw.setClassifica(num);
				cw.setCampagna(campagna);
				bacheca.add(cw);
			}
		}
		return bacheca;
	}
	public List<AgenteCampagnaBacheca> classificaCampagnaR(ClassificaCriteria a) {
		List<AgenteCampagnaBacheca> bacheca = new ArrayList<AgenteCampagnaBacheca>();
		List<Campagna> campagne = getCampagne(a.getCampagnaId(), a.getLimiteCampagna());
		Date oggi = ApplicationUtil.date();
		Iterator<Campagna> campagnaIterator = campagne.iterator();
		while(campagnaIterator.hasNext()) {
			Campagna campagna = campagnaIterator.next();
			if(ApplicationUtil.gtZero(a.getTipologia())) {
				Date dataInizio = ApplicationUtil.dataInizio(a.getTipologia(), oggi);
				List<AgenteCampagnaClassifica> num = getCampagnaClassificaR(campagna.getId(), dataInizio, null, a.getLimiteAgente(), a.getTipoClassifica());
				AgenteCampagnaBacheca cw = new AgenteCampagnaBacheca();
				cw.setClassifica(num);
				cw.setCampagna(campagna);
				bacheca.add(cw);
			}
		}
		return bacheca;
	}
	public List<OperatoreCampagnaBacheca> classificaCampagnaO(ClassificaCriteria a) {
		List<OperatoreCampagnaBacheca> bacheca = new ArrayList<OperatoreCampagnaBacheca>();
		List<Campagna> campagne = getCampagne(a.getCampagnaId(), a.getLimiteCampagna());
		Date oggi = ApplicationUtil.date();
		Iterator<Campagna> campagnaIterator = campagne.iterator();
		while(campagnaIterator.hasNext()) {
			Campagna campagna = campagnaIterator.next();
			if(ApplicationUtil.gtZero(a.getTipologia())) {
				Date dataInizio = ApplicationUtil.dataInizio(a.getTipologia(), oggi);
				List<OperatoreCampagnaClassifica> num = getCampagnaClassificaO(campagna.getId(), dataInizio, null, a.getLimiteOperatore(), a.getTipoClassifica());
				OperatoreCampagnaBacheca cw = new OperatoreCampagnaBacheca();
				cw.setClassifica(num);
				cw.setCampagna(campagna);
				bacheca.add(cw);
			}
		}
		return bacheca;
	}
	public List<OperatoreTeamBacheca> classificaTeamO(ClassificaCriteria a) {
		List<OperatoreTeamBacheca> bacheca = new ArrayList<OperatoreTeamBacheca>();
		List<Team> teams = getTeam(a.getTeamId(), a.getLimiteTeam());
		Date oggi = ApplicationUtil.date();
		Iterator<Team> teamIterator = teams.iterator();
		while(teamIterator.hasNext()) {
			Team team = teamIterator.next();
			if(ApplicationUtil.gtZero(a.getTipologia())) {
				Date dataInizio = ApplicationUtil.dataInizio(a.getTipologia(), oggi);
				List<OperatoreTeamClassifica> num = getTeamClassificaO(team.getId(), dataInizio, null, a.getLimiteOperatore(), a.getTipoClassifica());
				OperatoreTeamBacheca cw = new OperatoreTeamBacheca();
				cw.setClassifica(num);
				cw.setTeam(team);
				bacheca.add(cw);
			}
		}
		return bacheca;
	}
	public List<AgenteUfficioBacheca> classificaUfficio(ClassificaCriteria a) {
		List<AgenteUfficioBacheca> bacheca = new ArrayList<AgenteUfficioBacheca>();
		List<Ufficio> zone = getUffici(a.getUfficioId(), a.getLimiteUfficio());
		Date oggi = ApplicationUtil.date();
		Iterator<Ufficio> ufficioIterator = zone.iterator();
		while(ufficioIterator.hasNext()) {
			Ufficio ufficio = ufficioIterator.next();
			if(ApplicationUtil.gtZero(a.getTipologia())) {
				Date dataInizio = ApplicationUtil.dataInizio(a.getTipologia(), oggi);
				List<AgenteUfficioClassifica> num = getUfficioClassifica(ufficio.getId(), dataInizio, null, a.getLimiteAgente(), a.getTipoClassifica());
				AgenteUfficioBacheca cw = new AgenteUfficioBacheca();
				cw.setClassifica(num);
				cw.setUfficio(ufficio);
				bacheca.add(cw);
			}
		}
		return bacheca;
	}
	public List<AgenteUfficioBacheca> classificaUfficioR(ClassificaCriteria a) {
		List<AgenteUfficioBacheca> bacheca = new ArrayList<AgenteUfficioBacheca>();
		List<Ufficio> zone = getUffici(a.getUfficioId(), a.getLimiteUfficio());
		Date oggi = ApplicationUtil.date();
		Iterator<Ufficio> ufficioIterator = zone.iterator();
		while(ufficioIterator.hasNext()) {
			Ufficio ufficio = ufficioIterator.next();
			if(ApplicationUtil.gtZero(a.getTipologia())) {
				Date dataInizio = ApplicationUtil.dataInizio(a.getTipologia(), oggi);
				List<AgenteUfficioClassifica> num = getUfficioClassificaR(ufficio.getId(), dataInizio, null, a.getLimiteAgente(), a.getTipoClassifica());
				AgenteUfficioBacheca cw = new AgenteUfficioBacheca();
				cw.setClassifica(num);
				cw.setUfficio(ufficio);
				bacheca.add(cw);
			}
		}
		return bacheca;
	}
	@SuppressWarnings({ "unchecked" })
	private List<RiepilogoEsitoAppoggio> getRiepilogoEsitiC(Utente user, ReportCriteria r) {
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class, "cont");
		DetachedCriteria criteriaE = criteria.createCriteria("esitoChiamata", "esit");
		criteriaE.add(Restrictions.ne("id", ConstantsEsito.ID_OK));
		DetachedCriteria criteriaCA = criteria.createCriteria("campagna", "camp");
		DetachedCriteria criteriaLI = criteria.createCriteria("lista", "lista");
		if(user!=null) {
			if(ApplicationUtil.isTrue(sd)) {
				DetachedCriteria criteriaUU = criteria.createCriteria("utente");
				impostazioneDao.subCriteriaSupervisore(user.getId(), criteriaUU, "id");
			}
			impostazioneDao.subCriteriaCampagnaSupervisore(user.getId(), criteriaCA, "id");
		}
		if(r != null) {
			ApplicationUtil.intervallo(ApplicationUtil.isTrue(r.getInserimento()), r.getInizio(), r.getFine(), criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, criteriaLI);
			if(ApplicationUtil.gtZero(r.getLista())) {
				criteriaLI.add(Restrictions.idEq(r.getLista()));
			}
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("esit.id").as("eid")))
				.add(Projections.groupProperty("esit.etichetta").as("etichetta"))
				.add(Projections.groupProperty("esit.codice").as("codice"))
				.add(Projections.groupProperty("esit.colore").as("colore"))
				.add(Projections.countDistinct("cont.id").as("numero"))
				);
		criteria.addOrder(Order.asc("esit.etichetta"));
		return getList(criteria, RiepilogoEsitoAppoggio.class);
	}
	@SuppressWarnings({ "unchecked" })
	private List<RiepilogoEsitoAppoggio> getRiepilogoEsitiA(Utente user, ReportCriteria r) {
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class, "cont");
		DetachedCriteria criteriaE = criteria.createCriteria("esitoArchiviazione", "esit");
		criteriaE.add(Restrictions.isNotNull("id"));
		DetachedCriteria criteriaCA = criteria.createCriteria("campagna", "camp");
		DetachedCriteria criteriaLI = criteria.createCriteria("lista", "lista");
		if(user!=null) {
			if(ApplicationUtil.isTrue(sd)) {
				DetachedCriteria criteriaUU = criteria.createCriteria("utente");
				impostazioneDao.subCriteriaSupervisore(user.getId(), criteriaUU, "id");
			}
			impostazioneDao.subCriteriaCampagnaSupervisore(user.getId(), criteriaCA, "id");
		}
		if(r != null) {
			ApplicationUtil.intervallo(ApplicationUtil.isTrue(r.getInserimento()), r.getInizio(), r.getFine(), criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, criteriaLI);
			if(ApplicationUtil.gtZero(r.getLista())) {
				criteriaLI.add(Restrictions.idEq(r.getLista()));
			}
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("esit.id").as("eid")))
				.add(Projections.groupProperty("esit.etichetta").as("etichetta"))
				.add(Projections.groupProperty("esit.codice").as("codice"))
				.add(Projections.groupProperty("esit.colore").as("colore"))
				.add(Projections.countDistinct("cont.id").as("numero"))
				);
		criteria.addOrder(Order.asc("esit.etichetta"));
		return getList(criteria, RiepilogoEsitoAppoggio.class);
	}
	@SuppressWarnings({ "unchecked" })
	private List<RiepilogoEsitoAppoggio> getRiepilogoEsitiPartnerC(Utente user, ReportCriteria r) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class, "cont");
		DetachedCriteria criteriaE = criteria.createCriteria("esitoChiamata", "esit");
		criteriaE.add(Restrictions.ne("id", ConstantsEsito.ID_OK));
		DetachedCriteria criteriaCA = criteria.createCriteria("campagna", "camp");
		DetachedCriteria criteriaLI = criteria.createCriteria("lista", "lista");
		impostazioneDao.subCriteriaCampagnaPartner(user.getId(), criteriaCA, "id");
		if(r != null) {
			ApplicationUtil.intervallo(ApplicationUtil.isTrue(r.getInserimento()), r.getInizio(), r.getFine(), criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, criteriaLI);
			if(ApplicationUtil.gtZero(r.getLista())) {
				criteriaLI.add(Restrictions.idEq(r.getLista()));
			}
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("esit.id").as("eid")))
				.add(Projections.groupProperty("esit.etichetta").as("etichetta"))
				.add(Projections.groupProperty("esit.codice").as("codice"))
				.add(Projections.groupProperty("esit.colore").as("colore"))
				.add(Projections.countDistinct("cont.id").as("numero"))
				);
		criteria.addOrder(Order.asc("esit.etichetta"));
		return getList(criteria, RiepilogoEsitoAppoggio.class);
	}
	@SuppressWarnings({ "unchecked" })
	private List<RiepilogoEsitoAppoggio> getRiepilogoEsitiPartnerA(Utente user, ReportCriteria r) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class, "cont");
		DetachedCriteria criteriaE = criteria.createCriteria("esitoArchiviazione", "esit");
		criteriaE.add(Restrictions.isNotNull("id"));
		DetachedCriteria criteriaCA = criteria.createCriteria("campagna", "camp");
		DetachedCriteria criteriaLI = criteria.createCriteria("lista", "lista");
		impostazioneDao.subCriteriaCampagnaPartner(user.getId(), criteriaCA, "id");
		if(r != null) {
			ApplicationUtil.intervallo(ApplicationUtil.isTrue(r.getInserimento()), r.getInizio(), r.getFine(), criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, criteriaLI);
			if(ApplicationUtil.gtZero(r.getLista())) {
				criteriaLI.add(Restrictions.idEq(r.getLista()));
			}
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("esit.id").as("eid")))
				.add(Projections.groupProperty("esit.etichetta").as("etichetta"))
				.add(Projections.groupProperty("esit.codice").as("codice"))
				.add(Projections.groupProperty("esit.colore").as("colore"))
				.add(Projections.countDistinct("cont.id").as("numero"))
				);
		criteria.addOrder(Order.asc("esit.etichetta"));
		return getList(criteria, RiepilogoEsitoAppoggio.class);
	}
	@SuppressWarnings({ "unchecked", "unused" })
	private List<RiepilogoEsitoAppoggio> getRiepilogoEsitiStoria(Utente user, ReportCriteria r) {
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContatto.class, "cont");
		DetachedCriteria criteriaCo = criteria.createCriteria("contatto");
		DetachedCriteria criteriaE= criteria.createCriteria("esito", "esit");
		DetachedCriteria criteriaA= criteria.createCriteria("archiviazione", "arch",  ApplicationUtil.leftJoin());
		DetachedCriteria criteriaCA = criteriaCo.createCriteria("campagna", "camp");
		if(user!= null) {
			DetachedCriteria criteriaUU = criteria.createCriteria("operatore", "operatore");
			impostazioneDao.subCriteriaRoleSupervisore(user.getId(), criteria, "camp", "operatore");
			if(ApplicationUtil.isTrue(sd)) {
				impostazioneDao.subCriteriaSupervisore(user.getId(), criteriaUU, "id");
			}
			impostazioneDao.subCriteriaCampagnaSupervisore(user.getId(), criteriaCA, "id");
		}
		if(r != null) {
			DetachedCriteria criteriaLI = criteriaCo.createCriteria("lista", "lista");
			ApplicationUtil.pannello(r.getPannello(), criteria);
			ApplicationUtil.dataIntervalD(r.getInizio(), r.getFine(), "data", criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, criteriaLI);
			if(ApplicationUtil.gtZero(r.getLista())) {
				criteriaLI.add(Restrictions.idEq(r.getLista()));
			}
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("esit.id").as("eid")))
				.add(Projections.groupProperty("esit.etichetta").as("etichetta"))
				.add(Projections.groupProperty("esit.codice").as("codice"))
				.add(Projections.groupProperty("esit.colore").as("colore"))
				.add(Projections.sqlGroupProjection("positivo", "positivo", new String [] {"positivo"}, new Type []{BooleanType.INSTANCE}))
				.add(Projections.countDistinct("cont.id").as("numero"))
				);
		criteria.addOrder(Order.asc("esit.etichetta"));
		return getList(criteria, RiepilogoEsitoAppoggio.class);
	}
	@SuppressWarnings({ "unchecked" })
	private List<RiepilogoEsitoAppoggio> getRiepilogoEsitiStoriaC(Utente user, ReportCriteria r) {
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContatto.class, "cont");
		DetachedCriteria criteriaCo = criteria.createCriteria("contatto");
		DetachedCriteria criteriaE= criteria.createCriteria("esito", "esit");
		criteriaE.add(Restrictions.ne("id", ConstantsEsito.ID_OK));
		DetachedCriteria criteriaCA = criteriaCo.createCriteria("campagna", "camp");
		if(user!= null) {
			DetachedCriteria criteriaUU = criteria.createCriteria("operatore", "oper");
			impostazioneDao.subCriteriaRoleSupervisore(user.getId(), criteria, "camp", "oper");
			if(ApplicationUtil.isTrue(sd)) {
				impostazioneDao.subCriteriaSupervisore(user.getId(), criteriaUU, "id");
			}
			impostazioneDao.subCriteriaCampagnaSupervisore(user.getId(), criteriaCA, "id");
		}
		if(r != null) {
			DetachedCriteria criteriaLI = criteriaCo.createCriteria("lista", "lista");
			ApplicationUtil.pannello(r.getPannello(), criteria);
			ApplicationUtil.dataIntervalD(r.getInizio(), r.getFine(), "data", criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, criteriaLI);
			if(ApplicationUtil.gtZero(r.getLista())) {
				criteriaLI.add(Restrictions.idEq(r.getLista()));
			}
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("esit.id").as("eid")))
				.add(Projections.groupProperty("esit.etichetta").as("etichetta"))
				.add(Projections.groupProperty("esit.codice").as("codice"))
				.add(Projections.groupProperty("esit.colore").as("colore"))
				.add(Projections.countDistinct("cont.id").as("numero"))
				);
		criteria.addOrder(Order.asc("esit.etichetta"));
		return getList(criteria, RiepilogoEsitoAppoggio.class);
	}
	@SuppressWarnings({ "unchecked" })
	private List<RiepilogoEsitoAppoggio> getRiepilogoEsitiStoriaA(Utente user, ReportCriteria r) {
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContatto.class, "cont");
		DetachedCriteria criteriaCo = criteria.createCriteria("contatto");
		DetachedCriteria criteriaE= criteria.createCriteria("archiviazione", "esit");
		DetachedCriteria criteriaCA = criteriaCo.createCriteria("campagna", "camp");
		criteriaE.add(Restrictions.isNotNull("id"));
		if(user!=null) {
			DetachedCriteria criteriaUU = criteria.createCriteria("operatore", "oper");
			impostazioneDao.subCriteriaRoleSupervisore(user.getId(), criteria, "camp", "oper");
			if(ApplicationUtil.isTrue(sd)) {
				impostazioneDao.subCriteriaSupervisore(user.getId(), criteriaUU, "id");
			}
			impostazioneDao.subCriteriaCampagnaSupervisore(user.getId(), criteriaCA, "id");
		}
		if(r != null) {
			DetachedCriteria criteriaLI = criteriaCo.createCriteria("lista", "lista");
			ApplicationUtil.pannello(r.getPannello(), criteria);
			ApplicationUtil.dataIntervalD(r.getInizio(), r.getFine(), "data", criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, criteriaLI);
			if(ApplicationUtil.gtZero(r.getLista())) {
				criteriaLI.add(Restrictions.idEq(r.getLista()));
			}
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("esit.id").as("eid")))
				.add(Projections.groupProperty("esit.etichetta").as("etichetta"))
				.add(Projections.groupProperty("esit.codice").as("codice"))
				.add(Projections.groupProperty("esit.colore").as("colore"))
				.add(Projections.countDistinct("cont.id").as("numero"))
				);
		criteria.addOrder(Order.asc("esit.etichetta"));
		return getList(criteria, RiepilogoEsitoAppoggio.class);
	}
	@SuppressWarnings({ "unchecked", "unused" })
	private List<RiepilogoEsitoAppoggio> getRiepilogoEsitiStoriaPartner(Utente user, ReportCriteria r) {
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContatto.class, "cont");
		DetachedCriteria criteriaCo = criteria.createCriteria("contatto");
		DetachedCriteria criteriaE= criteria.createCriteria("esito", "esit");
		DetachedCriteria criteriaA= criteria.createCriteria("archiviazione", "arch",  ApplicationUtil.leftJoin());
		DetachedCriteria criteriaCA = criteriaCo.createCriteria("campagna", "camp");
		DetachedCriteria criteriaLI = criteriaCo.createCriteria("lista", "lista");
		impostazioneDao.subCriteriaCampagnaPartner(user.getId(), criteriaCA, "id");
		if(r != null) {
			ApplicationUtil.pannello(r.getPannello(), criteria);
			ApplicationUtil.dataIntervalD(r.getInizio(), r.getFine(), "data", criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, criteriaLI);
			if(ApplicationUtil.gtZero(r.getLista())) {
				criteriaLI.add(Restrictions.idEq(r.getLista()));
			}
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("esit.id").as("eid")))
				.add(Projections.groupProperty("esit.etichetta").as("etichetta"))
				.add(Projections.groupProperty("esit.codice").as("codice"))
				.add(Projections.groupProperty("esit.colore").as("colore"))
				.add(Projections.sqlGroupProjection("positivo", "positivo", new String [] {"positivo"}, new Type []{BooleanType.INSTANCE}))
				.add(Projections.countDistinct("cont.id").as("numero"))
				);
		criteria.addOrder(Order.asc("esit.etichetta"));
		return getList(criteria, RiepilogoEsitoAppoggio.class);
	}
	@SuppressWarnings({ "unchecked" })
	private List<RiepilogoEsitoAppoggio> getRiepilogoEsitiStoriaPartnerC(Utente user, ReportCriteria r) {
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContatto.class, "cont");
		DetachedCriteria criteriaCo = criteria.createCriteria("contatto");
		DetachedCriteria criteriaE= criteria.createCriteria("esito", "esit");
		criteriaE.add(Restrictions.ne("id", ConstantsEsito.ID_OK));
		DetachedCriteria criteriaCA = criteriaCo.createCriteria("campagna", "camp");
		DetachedCriteria criteriaLI = criteriaCo.createCriteria("lista", "lista");
		impostazioneDao.subCriteriaCampagnaPartner(user.getId(), criteriaCA, "id");
		if(r != null) {
			ApplicationUtil.pannello(r.getPannello(), criteria);
			ApplicationUtil.dataIntervalD(r.getInizio(), r.getFine(), "data", criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, criteriaLI);
			if(ApplicationUtil.gtZero(r.getLista())) {
				criteriaLI.add(Restrictions.idEq(r.getLista()));
			}
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("esit.id").as("eid")))
				.add(Projections.groupProperty("esit.etichetta").as("etichetta"))
				.add(Projections.groupProperty("esit.codice").as("codice"))
				.add(Projections.groupProperty("esit.colore").as("colore"))
				.add(Projections.countDistinct("cont.id").as("numero"))
				);
		criteria.addOrder(Order.asc("esit.etichetta"));
		return getList(criteria, RiepilogoEsitoAppoggio.class);
	}
	@SuppressWarnings({ "unchecked" })
	private List<RiepilogoEsitoAppoggio> getRiepilogoEsitiStoriaPartnerA(Utente user, ReportCriteria r) {
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContatto.class, "cont");
		DetachedCriteria criteriaCo = criteria.createCriteria("contatto");
		DetachedCriteria criteriaE= criteria.createCriteria("archiviazione", "esit");
		criteriaE.add(Restrictions.isNotNull("id"));
		DetachedCriteria criteriaCA = criteriaCo.createCriteria("campagna", "camp");
		DetachedCriteria criteriaLI = criteriaCo.createCriteria("lista", "lista");
		impostazioneDao.subCriteriaCampagnaPartner(user.getId(), criteriaCA, "id");
		if(r != null) {
			ApplicationUtil.pannello(r.getPannello(), criteria);
			ApplicationUtil.dataIntervalD(r.getInizio(), r.getFine(), "data", criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, criteriaLI);
			if(ApplicationUtil.gtZero(r.getLista())) {
				criteriaLI.add(Restrictions.idEq(r.getLista()));
			}
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("esit.id").as("eid")))
				.add(Projections.groupProperty("esit.etichetta").as("etichetta"))
				.add(Projections.groupProperty("esit.codice").as("codice"))
				.add(Projections.groupProperty("esit.colore").as("colore"))
				.add(Projections.countDistinct("cont.id").as("numero"))
				);
		criteria.addOrder(Order.asc("esit.etichetta"));
		return getList(criteria, RiepilogoEsitoAppoggio.class);
	}
	@SuppressWarnings({ "unchecked", "unused" })
	private List<ReportGeneraleResultSet> getReportGeneraleContattoStoriaResultset(Utente user, ReportCriteria r) {
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContatto.class, "sc");
		DetachedCriteria criteriaE= criteria.createCriteria("esito", "esit");
		DetachedCriteria criteriaA= criteria.createCriteria("archiviazione", "arch",  ApplicationUtil.leftJoin());
		DetachedCriteria criteriaO = criteria.createCriteria("operatore", "op");
		DetachedCriteria criteriaC = criteria.createCriteria("contatto", "cont");
		DetachedCriteria criteriaCA = criteriaC.createCriteria("campagna", "camp");
		if(user != null) {
			impostazioneDao.subCriteriaRoleSupervisore(user.getId(), criteria, "camp", "op");
			impostazioneDao.subCriteriaSupervisore(user.getId(), criteriaO, "id");
		}
		if(r != null) {
			DetachedCriteria criteriaLI = criteriaC.createCriteria("lista", "lista");
			ApplicationUtil.pannello(r.getPannello(), criteria);
			ApplicationUtil.dataIntervalD(r.getInizio(), r.getFine(), "data", criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, criteriaLI);
			if(ApplicationUtil.gtZero(r.getLista())) {
				criteriaLI.add(Restrictions.idEq(r.getLista()));
			}
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
			if(user != null && ApplicationUtil.ltZero(r.getCampagna())) {
				if(ApplicationUtil.isTrue(sd)) {
					impostazioneDao.subCriteriaCampagnaSupervisore(user.getId(), criteriaCA, "id");
				}
			}
			if (ApplicationUtil.gtZero(r.getTeam())){
				if(ApplicationUtil.isTrue(sd)) {
					impostazioneDao.subCriteriaCampagnaTeam(r.getTeam(), criteriaCA, "id");
				}
				impostazioneDao.subCriteriaOperatoreTeam(r.getTeam(), criteriaO, "id");
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("op.id").as("user_id")))
				.add(Projections.countDistinct("sc.id").as("totali"))
				.add(Projections.count("arch.id").as("archiviazioni"))
				.add(Projections.sum("sc.secondi_conv_eff").as("durata_conv_eff"))
				.add(Projections.sum("sc.secondi_tot").as("durata"))
				.add(Projections.sum("sc.secondi_conv_tot").as("durata_conv_tot"))
				.add(Projections.sum("sc.secondi_post").as("durata_post"))
				.add(Projections.sum("sc.secondi_pre").as("durata_pre"))
				.add(Projections.sum("sc.secondi_rec").as("durata_rec"))
				.add(Projections.sum("sc.secondi_attesa").as("durata_attesa"))
				.add(Projections.avg("sc.secondi_conv_eff").as("media_conv_eff"))
				.add(Projections.avg("sc.secondi_tot").as("media"))
				.add(Projections.avg("sc.secondi_conv_tot").as("media_conv_tot"))
				.add(Projections.avg("sc.secondi_post").as("media_post"))
				.add(Projections.avg("sc.secondi_pre").as("media_pre"))
				.add(Projections.avg("sc.secondi_rec").as("media_rec"))
				.add(Projections.avg("sc.secondi_attesa").as("media_attesa"))
				.add(Projections.sqlProjection("sum(if({alias}.archiviato, 1, 0)) as chiusi", new String [] {"chiusi"}, new Type []{LongType.INSTANCE}))
				.add(Projections.sqlProjection("sum(if(positivo, 1, 0)) as positivi", new String [] {"positivi"}, new Type []{LongType.INSTANCE}))
				.add(Projections.sqlProjection("sum(if(utile, 1, 0)) as utili", new String [] {"utili"}, new Type []{LongType.INSTANCE}))
				.add(Projections.sqlProjection("sum(if(secondi_conv_eff > 0, 1, 0)) as conversazioni", new String [] {"conversazioni"}, new Type []{LongType.INSTANCE}))
				);
		return getList(criteria, ReportGeneraleResultSet.class);
	}
	@SuppressWarnings({ "unchecked", "unused" })
	private List<ReportGeneraleResultSet> getReportGeneraleAppuntamentoStoriaResultset(Utente user, ReportCriteria r) {
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaAppuntamento.class, "sc");
		DetachedCriteria criteriaE= criteria.createCriteria("esito", "esit",  ApplicationUtil.leftJoin());
		DetachedCriteria criteriaO = criteria.createCriteria("utente", "op");
		DetachedCriteria criteriaApp = criteria.createCriteria("appuntamento", "r");
		DetachedCriteria criteriaC = criteriaApp.createCriteria("contatto", "cont");
		DetachedCriteria criteriaCA = criteriaC.createCriteria("campagna", "camp");
		if(user != null) {
			impostazioneDao.subCriteriaRoleSupervisore(user.getId(), criteria, "camp", "op");
			impostazioneDao.subCriteriaSupervisore(user.getId(), criteriaO, "id");
		}
		if(r != null) {
			DetachedCriteria criteriaLI = criteriaC.createCriteria("lista", "lista");
			ApplicationUtil.dataIntervalD(r.getInizio(), r.getFine(), "data", criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, criteriaLI);
			if(ApplicationUtil.gtZero(r.getLista())) {
				criteriaLI.add(Restrictions.idEq(r.getLista()));
			}
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
			if(user != null && ApplicationUtil.ltZero(r.getCampagna())) {
				if(ApplicationUtil.isTrue(sd)) {
					impostazioneDao.subCriteriaCampagnaSupervisore(user.getId(), criteriaCA, "id");
				}
			}
			if (ApplicationUtil.gtZero(r.getTeam())){
				if(ApplicationUtil.isTrue(sd)) {
					impostazioneDao.subCriteriaCampagnaTeam(r.getTeam(), criteriaCA, "id");
				}
				impostazioneDao.subCriteriaOperatoreTeam(r.getTeam(), criteriaO, "id");
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("op.id").as("user_id")))
				.add(Projections.countDistinct("sc.id").as("totali"))
				.add(Projections.sqlProjection("sum(if(confermato, 1, 0)) as confermati", new String [] {"confermati"}, new Type []{LongType.INSTANCE}))
				.add(Projections.sqlProjection("sum(if(utile, 1, 0)) as utili", new String [] {"utili"}, new Type []{LongType.INSTANCE}))
				);
		return getList(criteria, ReportGeneraleResultSet.class);
	}
	@SuppressWarnings({ "unchecked", "unused" })
	private List<ReportGeneraleResultSet> getReportGeneraleContrattoStoriaResultset(Utente user, ReportCriteria r) {
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContratto.class, "sc");
		DetachedCriteria criteriaE= criteria.createCriteria("esito", "esit",  ApplicationUtil.leftJoin());
		DetachedCriteria criteriaO = criteria.createCriteria("utente", "op");
		DetachedCriteria criteriaApp = criteria.createCriteria("contratto", "r");
		DetachedCriteria criteriaC = criteriaApp.createCriteria("contatto", "cont");
		DetachedCriteria criteriaCA = criteriaC.createCriteria("campagna", "camp");
		if(user != null) {
			impostazioneDao.subCriteriaRoleSupervisore(user.getId(), criteria, "camp", "op");
			impostazioneDao.subCriteriaSupervisore(user.getId(), criteriaO, "id");
		}
		if(r != null) {
			DetachedCriteria criteriaLI = criteriaC.createCriteria("lista", "lista");
			ApplicationUtil.dataIntervalD(r.getInizio(), r.getFine(), "data", criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, criteriaLI);
			if(ApplicationUtil.gtZero(r.getLista())) {
				criteriaLI.add(Restrictions.idEq(r.getLista()));
			}
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
			if(user != null && ApplicationUtil.ltZero(r.getCampagna())) {
				if(ApplicationUtil.isTrue(sd)) {
					impostazioneDao.subCriteriaCampagnaSupervisore(user.getId(), criteriaCA, "id");
				}
			}
			if (ApplicationUtil.gtZero(r.getTeam())){
				if(ApplicationUtil.isTrue(sd)) {
					impostazioneDao.subCriteriaCampagnaTeam(r.getTeam(), criteriaCA, "id");
				}
				impostazioneDao.subCriteriaOperatoreTeam(r.getTeam(), criteriaO, "id");
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("op.id").as("user_id")))
				.add(Projections.countDistinct("sc.id").as("totali"))
				.add(Projections.sqlProjection("sum(if(confermato, 1, 0)) as confermati", new String [] {"confermati"}, new Type []{LongType.INSTANCE}))
				.add(Projections.sqlProjection("sum(if(utile, 1, 0)) as utili", new String [] {"utili"}, new Type []{LongType.INSTANCE}))
				);
		return getList(criteria, ReportGeneraleResultSet.class);
	}
	@SuppressWarnings("unchecked")
	private List<ReportGeneraleResultSet> getReportUtenti(Utente user, ReportCriteria r) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		if(user != null || ApplicationUtil.gtZero(r.getTeam()) || ApplicationUtil.gtZero(r.getCampagna())) {
			DetachedCriteria criteriaT = criteria.createCriteria("team");
			if(user != null) {
				criteriaT.createCriteria("supervisore").add(Restrictions.idEq(user.getId()));	
			}
			if(ApplicationUtil.gtZero(r.getTeam())) {
				criteriaT.add(Restrictions.idEq(r.getTeam()));
			} 
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaT.createCriteria("campagne").add(Restrictions.idEq(r.getCampagna()));
			} 
		}
		criteria.setProjection(Projections.distinct(Projections.property("id").as("user_id")))
		;
		ApplicationUtil.orderU(criteria);
		return getList(criteria, ReportGeneraleResultSet.class);
	}
	public List<SelectItem> getCampagneSI(Utente utente, StatoIntervallo stato, Long teamId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		if(utente != null || ApplicationUtil.gtZero(teamId)) {
			DetachedCriteria criteriaT = criteria.createCriteria("team");
			if(utente != null) {
				criteriaT.createCriteria("supervisore").add(Restrictions.idEq(utente.getId()));
			}
			if(ApplicationUtil.gtZero(teamId)) {
				criteriaT.add(Restrictions.idEq(teamId));
			}
		}
		ApplicationUtil.statoLC(stato, criteria, null);
		List<CampagnaSI> lis = getCampagnaSI(criteria);
		return impostazioneDao.getCampagna(lis, true, false, false, null);
	}
	public List<SelectItem> getTeamSI(Utente utente, StatoIntervallo stato, Long campagnaId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Team.class);
		DetachedCriteria criteriaT = criteria.createCriteria("campagne");
		ApplicationUtil.statoLC(stato, criteriaT, null);
		if(ApplicationUtil.gtZero(campagnaId)) {
			criteriaT.add(Restrictions.idEq(campagnaId));
		}
		if(utente != null) {
			criteria.createCriteria("supervisore").add(Restrictions.idEq(utente.getId()));
		}
		List<TeamSI> lis = getTeamSI(criteria);
		return impostazioneDao.getTeam(lis, true, false, null);
	}
	@SuppressWarnings({ "unchecked", "unused" })
	private List<ReportGeneraleResultSet> getReportGeneraleContattoResultset(Utente user, ReportCriteria r) {
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class, "sc");
		DetachedCriteria criteriaE= criteria.createCriteria("esitoChiamata", "esit");
		DetachedCriteria criteriaA= criteria.createCriteria("esitoArchiviazione", "arch",  ApplicationUtil.leftJoin());
		DetachedCriteria criteriaO = criteria.createCriteria("utente", "op");
		if(user != null) {
			impostazioneDao.subCriteriaSupervisore(user.getId(), criteriaO, "id");
		}
		DetachedCriteria criteriaCA = criteria.createCriteria("campagna", "camp");
		DetachedCriteria criteriaLI = criteria.createCriteria("lista", "lista");
		if(r != null) {
			ApplicationUtil.intervallo(ApplicationUtil.isTrue(r.getInserimento()), r.getInizio(), r.getFine(), criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, criteriaLI);
			if(ApplicationUtil.gtZero(r.getLista())) {
				criteriaLI.add(Restrictions.idEq(r.getLista()));
			}
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
			if(user != null && ApplicationUtil.ltZero(r.getCampagna())) {
				if(ApplicationUtil.isTrue(sd)) {
					impostazioneDao.subCriteriaCampagnaSupervisore(user.getId(), criteriaCA, "id");
				}
			}
			if (ApplicationUtil.gtZero(r.getTeam())){
				if(ApplicationUtil.isTrue(sd)) {
					impostazioneDao.subCriteriaCampagnaTeam(r.getTeam(), criteriaCA, "id");
				}
				impostazioneDao.subCriteriaOperatoreTeam(r.getTeam(), criteriaO, "id");
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("op.id").as("user_id")))
				.add(Projections.countDistinct("sc.id").as("totali"))
				.add(Projections.count("arch.id").as("archiviazioni"))
				.add(Projections.sqlProjection("sum(if({alias}.archiviato, 1, 0)) as chiusi", new String [] {"chiusi"}, new Type []{LongType.INSTANCE}))
				.add(Projections.sqlProjection("sum(if(positivo, 1, 0)) as positivi", new String [] {"positivi"}, new Type []{LongType.INSTANCE}))
				.add(Projections.sqlProjection("sum(if(utile, 1, 0)) as utili", new String [] {"utili"}, new Type []{LongType.INSTANCE}))
				);
		return getList(criteria, ReportGeneraleResultSet.class);
	}
	@SuppressWarnings({ "unchecked", "unused" })
	private List<ReportGeneraleResultSet> getReportGeneraleAppuntamentoResultset(Utente user, ReportCriteria r) {
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(Appuntamento.class, "sc");
		DetachedCriteria criteriaE= criteria.createCriteria("esito", "esit",  ApplicationUtil.leftJoin());
		DetachedCriteria criteriaO = criteria.createCriteria("operatore", "op");
		DetachedCriteria criteriaC = criteria.createCriteria("contatto", "cont");
		DetachedCriteria criteriaCA = criteriaC.createCriteria("campagna", "camp");
		if(user != null) {
			impostazioneDao.subCriteriaRoleSupervisore(user.getId(), criteria, "camp", "op");
			impostazioneDao.subCriteriaSupervisore(user.getId(), criteriaO, "id");
		}
		if(r != null) {
			DetachedCriteria criteriaLI = criteriaC.createCriteria("lista", "lista");
			ApplicationUtil.intervallo(ApplicationUtil.isTrue(r.getInserimento()), r.getInizio(), r.getFine(), criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, criteriaLI);
			if(ApplicationUtil.gtZero(r.getLista())) {
				criteriaLI.add(Restrictions.idEq(r.getLista()));
			}
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
			if(user != null && ApplicationUtil.ltZero(r.getCampagna())) {
				if(ApplicationUtil.isTrue(sd)) {
					impostazioneDao.subCriteriaCampagnaSupervisore(user.getId(), criteriaCA, "id");
				}
			}
			if (ApplicationUtil.gtZero(r.getTeam())){
				if(ApplicationUtil.isTrue(sd)) {
					impostazioneDao.subCriteriaCampagnaTeam(r.getTeam(), criteriaCA, "id");
				}
				impostazioneDao.subCriteriaOperatoreTeam(r.getTeam(), criteriaO, "id");
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("op.id").as("user_id")))
				.add(Projections.countDistinct("sc.id").as("totali"))
				.add(Projections.sqlProjection("sum(if(confermato, 1, 0)) as confermati", new String [] {"confermati"}, new Type []{LongType.INSTANCE}))
				.add(Projections.sqlProjection("sum(if(utile, 1, 0)) as utili", new String [] {"utili"}, new Type []{LongType.INSTANCE}))
				);
		return getList(criteria, ReportGeneraleResultSet.class);
	}
	@SuppressWarnings({ "unchecked", "unused" })
	private List<ReportGeneraleResultSet> getReportGeneraleContrattoResultset(Utente user, ReportCriteria r) {
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(Contratto.class, "sc");
		DetachedCriteria criteriaE= criteria.createCriteria("esito", "esit",  ApplicationUtil.leftJoin());
		DetachedCriteria criteriaO = criteria.createCriteria("operatore", "op");
		DetachedCriteria criteriaC = criteria.createCriteria("contatto", "cont");
		DetachedCriteria criteriaCA = criteriaC.createCriteria("campagna", "camp");
		if(user != null) {
			impostazioneDao.subCriteriaRoleSupervisore(user.getId(), criteria, "camp", "op");
			impostazioneDao.subCriteriaSupervisore(user.getId(), criteriaO, "id");
		}
		if(r != null) {
			DetachedCriteria criteriaLI = criteriaC.createCriteria("lista", "lista");
			ApplicationUtil.intervallo(ApplicationUtil.isTrue(r.getInserimento()), r.getInizio(), r.getFine(), criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, criteriaLI);
			if(ApplicationUtil.gtZero(r.getLista())) {
				criteriaLI.add(Restrictions.idEq(r.getLista()));
			}
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
			if(user != null && ApplicationUtil.ltZero(r.getCampagna())) {
				if(ApplicationUtil.isTrue(sd)) {
					impostazioneDao.subCriteriaCampagnaSupervisore(user.getId(), criteriaCA, "id");
				}
			}
			if (ApplicationUtil.gtZero(r.getTeam())){
				if(ApplicationUtil.isTrue(sd)) {
					impostazioneDao.subCriteriaCampagnaTeam(r.getTeam(), criteriaCA, "id");
				}
				impostazioneDao.subCriteriaOperatoreTeam(r.getTeam(), criteriaO, "id");
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("op.id").as("user_id")))
				.add(Projections.countDistinct("sc.id").as("totali"))
				.add(Projections.sqlProjection("sum(if(confermato, 1, 0)) as confermati", new String [] {"confermati"}, new Type []{LongType.INSTANCE}))
				.add(Projections.sqlProjection("sum(if(utile, 1, 0)) as utili", new String [] {"utili"}, new Type []{LongType.INSTANCE}))
				);
		return getList(criteria, ReportGeneraleResultSet.class);
	}
	private DetachedCriteria  getOperatoreAppuntamentiStoriaByCriteria(Long user) {
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaAppuntamento.class);
		criteria.add(Restrictions.eq("utente.id",user));
		return criteria;
	}
	@SuppressWarnings("unchecked")
	public List<StoriaAppuntamento>  getOperatoreAppuntamentiStoria(Long user, int start, int size) {
		DetachedCriteria criteria = getOperatoreAppuntamentiStoriaByCriteria(user);
		criteria.setFetchMode("appuntamento", FetchMode.JOIN);
		criteria.addOrder(Order.desc("id"));
		return getList(criteria, start, size);
	}
	public int  getOperatoreAppuntamentiStoriaSize(Long user) {
		DetachedCriteria criteria = getOperatoreAppuntamentiStoriaByCriteria(user);
		return getListSize(criteria);
	}
	private DetachedCriteria  getOperatoreContrattiStoriaByCriteria(Long user) {
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContratto.class);
		criteria.add(Restrictions.eq("utente.id",user));
		return criteria;
	}
	@SuppressWarnings("unchecked")
	public List<StoriaContratto>  getOperatoreContrattiStoria(Long user, int start, int size) {
		DetachedCriteria criteria = getOperatoreContrattiStoriaByCriteria(user);
		criteria.setFetchMode("contratto", FetchMode.JOIN);
		criteria.addOrder(Order.desc("id"));
		return getList(criteria, start, size);
	}
	public int  getOperatoreContrattiStoriaSize(Long user) {
		DetachedCriteria criteria = getOperatoreContrattiStoriaByCriteria(user);
		return getListSize(criteria);
	}
	private DetachedCriteria  getOperatoreContattiStoriaByCriteria(Long user) {
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContatto.class);
		criteria.add(Restrictions.eq("operatore.id",user));
		return criteria;
	}
	@SuppressWarnings("unchecked")
	public List<StoriaContatto>  getOperatoreContattiStoria(Long user, int start, int size) {
		DetachedCriteria criteria = getOperatoreContattiStoriaByCriteria(user);
		criteria.setFetchMode("contatto", FetchMode.JOIN);
		criteria.addOrder(Order.desc("id"));
		return getList(criteria, start, size);
	}
	public int  getOperatoreContattiStoriaSize(Long user) {
		DetachedCriteria criteria = getOperatoreContattiStoriaByCriteria(user);
		return getListSize(criteria);
	}
	private DetachedCriteria  getOperatoreOperazioniByCriteria(Long user) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Operazione.class);
		criteria.add(Restrictions.eq("user.id",user));
		return criteria;
	}
	@SuppressWarnings("unchecked")
	public List<Operazione>  getOperatoreOperazioni(Long user, int start, int size) {
		DetachedCriteria criteria = getOperatoreOperazioniByCriteria(user);
		criteria.addOrder(Order.desc("id"));
		return getList(criteria, start, size);
	}
	public int  getOperatoreOperazioniSize(Long user) {
		DetachedCriteria criteria = getOperatoreOperazioniByCriteria(user);
		return getListSize(criteria);
	}
	private DetachedCriteria  getOperatorePauseByCriteria(Long user) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Pausa.class);
		criteria.add(Restrictions.eq("user.id",user));
		return criteria;
	}
	@SuppressWarnings("unchecked")
	public List<Pausa>  getOperatorePause(Long user, int start, int size) {
		DetachedCriteria criteria = getOperatorePauseByCriteria(user);
		criteria.addOrder(Order.desc("id"));
		return getList(criteria, start, size);
	}
	public int  getOperatorePauseSize(Long user) {
		DetachedCriteria criteria = getOperatorePauseByCriteria(user);
		return getListSize(criteria);
	}
	private DetachedCriteria  getOperatoreMarcatureByCriteria(Long user) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Marcatura.class);
		criteria.add(Restrictions.eq("user.id",user));
		return criteria;
	}
	@SuppressWarnings("unchecked")
	public List<Marcatura>  getOperatoreMarcature(Long user, int start, int size) {
		DetachedCriteria criteria = getOperatoreMarcatureByCriteria(user);
		criteria.addOrder(Order.desc("id"));
		return getList(criteria, start, size);
	}
	public int  getOperatoreMarcatureSize(Long user) {
		DetachedCriteria criteria = getOperatoreMarcatureByCriteria(user);
		return getListSize(criteria);
	}
	private DetachedCriteria  getOperatoreAppuntamentiByCriteria(Long user) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Appuntamento.class);
		criteria.add(Restrictions.eq("operatore.id",user));
		return criteria;
	}
	@SuppressWarnings("unchecked")
	public List<Appuntamento>  getOperatoreAppuntamenti(Long user, int start, int size) {
		DetachedCriteria criteria = getOperatoreAppuntamentiByCriteria(user);
		criteria.addOrder(Order.desc("id"));
		return getList(criteria, start, size);
	}
	public int  getOperatoreAppuntamentiSize(Long user) {
		DetachedCriteria criteria = getOperatoreAppuntamentiByCriteria(user);
		return getListSize(criteria);
	}
	private DetachedCriteria  getOperatoreContrattiByCriteria(Long user) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Contratto.class);
		criteria.add(Restrictions.eq("operatore.id",user));
		return criteria;
	}
	@SuppressWarnings("unchecked")
	public List<Contratto>  getOperatoreContratti(Long user, int start, int size) {
		DetachedCriteria criteria = getOperatoreContrattiByCriteria(user);
		criteria.addOrder(Order.desc("id"));
		return getList(criteria, start, size);
	}
	public int  getOperatoreContrattiSize(Long user) {
		DetachedCriteria criteria = getOperatoreContrattiByCriteria(user);
		return getListSize(criteria);
	}
	private DetachedCriteria  getOperatoreContattiByCriteria(Long user) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class);
		criteria.add(Restrictions.eq("proprietario.id",user));
		return criteria;
	}
	@SuppressWarnings("unchecked")
	public List<Contatto>  getOperatoreContatti(Long user, int start, int size) {
		DetachedCriteria criteria = getOperatoreContattiByCriteria(user);
		criteria.addOrder(Order.desc("id"));
		return getList(criteria, start, size);
	}
	public int  getOperatoreContattiSize(Long user) {
		DetachedCriteria criteria = getOperatoreContattiByCriteria(user);
		return getListSize(criteria);
	}
	private DetachedCriteria  getOperatoreEmailByCriteria(Long user) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Email.class);
		criteria.add(Restrictions.eq("utente.id",user));
		return criteria;
	}
	@SuppressWarnings("unchecked")
	public List<Email>  getOperatoreEmail(Long user, int start, int size) {
		DetachedCriteria criteria = getOperatoreEmailByCriteria(user);
		criteria.addOrder(Order.desc("id"));
		return getList(criteria, start, size);
	}
	public int  getOperatoreEmailSize(Long user) {
		DetachedCriteria criteria = getOperatoreEmailByCriteria(user);
		return getListSize(criteria);
	}

	private DetachedCriteria  getOperatoreRmailByCriteria(Long user) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Rmail.class);
		criteria.add(Restrictions.eq("utente.id",user));
		return criteria;
	}
	@SuppressWarnings("unchecked")
	public List<Rmail>  getOperatoreRmail(Long user, int start, int size) {
		DetachedCriteria criteria = getOperatoreRmailByCriteria(user);
		criteria.addOrder(Order.desc("id"));
		return getList(criteria, start, size);
	}
	public int  getOperatoreRmailSize(Long user) {
		DetachedCriteria criteria = getOperatoreRmailByCriteria(user);
		return getListSize(criteria);
	}
	
	private DetachedCriteria  getOperatoreTelecomByCriteria(Long user) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Telecom.class);
		criteria.add(Restrictions.eq("utente.id",user));
		return criteria;
	}
	@SuppressWarnings("unchecked")
	public List<Telecom>  getOperatoreTelecom(Long user, int start, int size) {
		DetachedCriteria criteria = getOperatoreTelecomByCriteria(user);
		criteria.addOrder(Order.desc("id"));
		return getList(criteria, start, size);
	}
	public int  getOperatoreTelecomSize(Long user) {
		DetachedCriteria criteria = getOperatoreTelecomByCriteria(user);
		return getListSize(criteria);
	}
	
	private DetachedCriteria  getOperatoreNotificaByCriteria(Long user) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Notifica.class);
		criteria.add(Restrictions.eq("utente.id",user));
		return criteria;
	}
	@SuppressWarnings("unchecked")
	public List<Notifica>  getOperatoreNotifica(Long user, int start, int size) {
		DetachedCriteria criteria = getOperatoreNotificaByCriteria(user);
		criteria.addOrder(Order.desc("id"));
		return getList(criteria, start, size);
	}
	public int  getOperatoreNotificaSize(Long user) {
		DetachedCriteria criteria = getOperatoreNotificaByCriteria(user);
		return getListSize(criteria);
	}
	
	private DetachedCriteria  getOperatoreCdrByCriteria(Long user) {
		DetachedCriteria criteriaP = DetachedCriteria.forClass(Cdr.class);
		criteriaP.add(Restrictions.eq("utente.id",user));
		criteriaP.setProjection(Projections.distinct(Projections.property("linkedid")));
		DetachedCriteria criteria = DetachedCriteria.forClass(Cdr.class);
		ApplicationUtil.lastapp(criteria);
		impostazioneDao.subPropertyInS(criteria, criteriaP, "linkedid");
		criteria.addOrder(Order.asc("calldate"));
		return criteria;
	}
	@SuppressWarnings("unchecked")
	public List<Cdr>  getOperatoreCdr(Long user, int start, int size) {
		DetachedCriteria criteria = getOperatoreCdrByCriteria(user);
		criteria.addOrder(Order.asc("calldate"));
		return getList(criteria, start, size);
	}
	public int  getOperatoreCdrSize(Long user) {
		DetachedCriteria criteria = getOperatoreCdrByCriteria(user);
		return getListSize(criteria);
	}
	private DetachedCriteria  getOperatoreAutoContattoByCriteria(Long user) {
		DetachedCriteria criteria = DetachedCriteria.forClass(AutoContatto.class);
		criteria.add(Restrictions.eq("utente.id",user));
		return criteria;
	}
	@SuppressWarnings("unchecked")
	public List<AutoContatto>  getOperatoreAutoContatto(Long user, int start, int size) {
		DetachedCriteria criteria = getOperatoreAutoContattoByCriteria(user);
		criteria.addOrder(Order.desc("id"));
		return getList(criteria, start, size);
	}
	public int  getOperatoreAutoContattoSize(Long user) {
		DetachedCriteria criteria = getOperatoreAutoContattoByCriteria(user);
		return getListSize(criteria);
	}
	@SuppressWarnings("unchecked")
	private List<ReportListaResultSet> getReportListe(Utente user, ReportCriteria r) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Lista.class);
		if(user != null || ApplicationUtil.gtZero(r.getCampagna())) {
			DetachedCriteria criteriaCA = criteria.createCriteria("campagna");
			if(user != null) {
				impostazioneDao.subCriteriaCampagnaSupervisore(user.getId(), criteriaCA, "id");
			}
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			} 
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, criteria);
		}
		criteria.setProjection(Projections.distinct(Projections.property("id").as("lista_id")));
		ApplicationUtil.orderC(criteria);
		return getList(criteria, ReportListaResultSet.class);
	}
	@SuppressWarnings({ "unchecked", "unused" })
	private List<ReportListaResultSet> getReportListaContattoResultset(Utente user, ReportCriteria r) {
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class, "sc");
		DetachedCriteria criteriaE= criteria.createCriteria("esitoChiamata", "esit");
		DetachedCriteria criteriaA= criteria.createCriteria("esitoArchiviazione", "arch",  ApplicationUtil.leftJoin());
		DetachedCriteria criteriaO = criteria.createCriteria("utente", "op");
		if(user != null) {
			impostazioneDao.subCriteriaSupervisore(user.getId(), criteriaO, "id");
		}
		DetachedCriteria criteriaCA = criteria.createCriteria("campagna", "camp");
		DetachedCriteria criteriaLI = criteria.createCriteria("lista", "lista");
		if(r != null) {
			ApplicationUtil.intervallo(ApplicationUtil.isTrue(r.getInserimento()), r.getInizio(), r.getFine(), criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, criteriaLI);
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
			if(user != null && ApplicationUtil.ltZero(r.getCampagna())) {
				if(ApplicationUtil.isTrue(sd)) {
					impostazioneDao.subCriteriaCampagnaSupervisore(user.getId(), criteriaCA, "id");
				}
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("lista.id").as("lista_id")))
				.add(Projections.countDistinct("sc.id").as("totali"))
				.add(Projections.count("arch.id").as("archiviazioni"))
				.add(Projections.sqlProjection("sum(if({alias}.archiviato, 1, 0)) as chiusi", new String [] {"chiusi"}, new Type []{LongType.INSTANCE}))
				.add(Projections.sqlProjection("sum(if(positivo, 1, 0)) as positivi", new String [] {"positivi"}, new Type []{LongType.INSTANCE}))
				.add(Projections.sqlProjection("sum(if(utile, 1, 0)) as utili", new String [] {"utili"}, new Type []{LongType.INSTANCE}))
				);
		return getList(criteria, ReportListaResultSet.class);
	}
	@SuppressWarnings({ "unchecked", "unused" })
	private List<ReportListaResultSet> getReportListaAppuntamentoResultset(Utente user, ReportCriteria r) {
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(Appuntamento.class, "sc");
		DetachedCriteria criteriaE= criteria.createCriteria("esito", "esit",  ApplicationUtil.leftJoin());
		DetachedCriteria criteriaO = criteria.createCriteria("operatore", "op");
		DetachedCriteria criteriaC = criteria.createCriteria("contatto", "cont");
		DetachedCriteria criteriaCA = criteriaC.createCriteria("campagna", "camp");
		if(user != null) {
			impostazioneDao.subCriteriaRoleSupervisore(user.getId(), criteria, "camp", "op");
			impostazioneDao.subCriteriaSupervisore(user.getId(), criteriaO, "id");
		}
		if(r != null) {
			DetachedCriteria criteriaLI = criteriaC.createCriteria("lista", "lista");
			ApplicationUtil.intervallo(ApplicationUtil.isTrue(r.getInserimento()), r.getInizio(), r.getFine(), criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, criteriaLI);
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
			if(user != null && ApplicationUtil.ltZero(r.getCampagna())) {
				if(ApplicationUtil.isTrue(sd)) {
					impostazioneDao.subCriteriaCampagnaSupervisore(user.getId(), criteriaCA, "id");
				}
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("lista.id").as("lista_id")))
				.add(Projections.countDistinct("sc.id").as("totali"))
				.add(Projections.sqlProjection("sum(if(confermato, 1, 0)) as confermati", new String [] {"confermati"}, new Type []{LongType.INSTANCE}))
				.add(Projections.sqlProjection("sum(if(utile, 1, 0)) as utili", new String [] {"utili"}, new Type []{LongType.INSTANCE}))
				);
		return getList(criteria, ReportListaResultSet.class);
	}
	@SuppressWarnings({ "unchecked", "unused" })
	private List<ReportListaResultSet> getReportListaContrattoResultset(Utente user, ReportCriteria r) {
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(Contratto.class, "sc");
		DetachedCriteria criteriaE= criteria.createCriteria("esito", "esit",  ApplicationUtil.leftJoin());
		DetachedCriteria criteriaO = criteria.createCriteria("operatore", "op");
		DetachedCriteria criteriaC = criteria.createCriteria("contatto", "cont");
		DetachedCriteria criteriaCA = criteriaC.createCriteria("campagna", "camp");
		if(user != null) {
			impostazioneDao.subCriteriaRoleSupervisore(user.getId(), criteria, "camp", "op");
			impostazioneDao.subCriteriaSupervisore(user.getId(), criteriaO, "id");
		}
		if(r != null) {
			DetachedCriteria criteriaLI = criteriaC.createCriteria("lista", "lista");
			ApplicationUtil.intervallo(ApplicationUtil.isTrue(r.getInserimento()), r.getInizio(), r.getFine(), criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, criteriaLI);
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
			if(user != null && ApplicationUtil.ltZero(r.getCampagna())) {
				if(ApplicationUtil.isTrue(sd)) {
					impostazioneDao.subCriteriaCampagnaSupervisore(user.getId(), criteriaCA, "id");
				}
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("lista.id").as("lista_id")))
				.add(Projections.countDistinct("sc.id").as("totali"))
				.add(Projections.sqlProjection("sum(if(confermato, 1, 0)) as confermati", new String [] {"confermati"}, new Type []{LongType.INSTANCE}))
				.add(Projections.sqlProjection("sum(if(utile, 1, 0)) as utili", new String [] {"utili"}, new Type []{LongType.INSTANCE}))
				);
		return getList(criteria, ReportListaResultSet.class);
	}
	@SuppressWarnings({ "unchecked", "unused" })
	private List<ReportListaResultSet> getReportListaContattoStoriaResultset(Utente user, ReportCriteria r) {
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContatto.class, "sc");
		DetachedCriteria criteriaE= criteria.createCriteria("esito", "esit");
		DetachedCriteria criteriaA= criteria.createCriteria("archiviazione", "arch",  ApplicationUtil.leftJoin());
		DetachedCriteria criteriaO = criteria.createCriteria("operatore", "op", ApplicationUtil.leftJoin());
		DetachedCriteria criteriaC = criteria.createCriteria("contatto", "cont");
		DetachedCriteria criteriaCA = criteriaC.createCriteria("campagna", "camp");
		if(user != null) {
			impostazioneDao.subCriteriaRoleSupervisore(user.getId(), criteria, "camp", "op");
			impostazioneDao.subCriteriaSupervisore(user.getId(), criteriaO, "id");
		}
		if(r != null) {
			DetachedCriteria criteriaLI = criteriaC.createCriteria("lista", "lista");
			ApplicationUtil.pannello(r.getPannello(), criteria);
			ApplicationUtil.dataIntervalD(r.getInizio(), r.getFine(), "data", criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, criteriaLI);
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
			if(user != null && ApplicationUtil.ltZero(r.getCampagna())) {
				if(ApplicationUtil.isTrue(sd)) {
					impostazioneDao.subCriteriaCampagnaSupervisore(user.getId(), criteriaCA, "id");
				}
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("lista.id").as("lista_id")))
				.add(Projections.countDistinct("sc.id").as("totali"))
				.add(Projections.count("arch.id").as("archiviazioni"))
				.add(Projections.sum("sc.secondi_conv_eff").as("durata_conv_eff"))
				.add(Projections.sum("sc.secondi_tot").as("durata"))
				.add(Projections.sum("sc.secondi_conv_tot").as("durata_conv_tot"))
				.add(Projections.sum("sc.secondi_post").as("durata_post"))
				.add(Projections.sum("sc.secondi_pre").as("durata_pre"))
				.add(Projections.sum("sc.secondi_rec").as("durata_rec"))
				.add(Projections.sum("sc.secondi_attesa").as("durata_attesa"))
				.add(Projections.avg("sc.secondi_conv_eff").as("media_conv_eff"))
				.add(Projections.avg("sc.secondi_tot").as("media"))
				.add(Projections.avg("sc.secondi_conv_tot").as("media_conv_tot"))
				.add(Projections.avg("sc.secondi_post").as("media_post"))
				.add(Projections.avg("sc.secondi_pre").as("media_pre"))
				.add(Projections.avg("sc.secondi_rec").as("media_rec"))
				.add(Projections.avg("sc.secondi_attesa").as("media_attesa"))
				.add(Projections.sqlProjection("sum(if({alias}.archiviato, 1, 0)) as chiusi", new String [] {"chiusi"}, new Type []{LongType.INSTANCE}))
				.add(Projections.sqlProjection("sum(if(positivo, 1, 0)) as positivi", new String [] {"positivi"}, new Type []{LongType.INSTANCE}))
				.add(Projections.sqlProjection("sum(if(utile, 1, 0)) as utili", new String [] {"utili"}, new Type []{LongType.INSTANCE}))
				.add(Projections.sqlProjection("sum(if(secondi_conv_eff > 0, 1, 0)) as conversazioni", new String [] {"conversazioni"}, new Type []{LongType.INSTANCE}))
				);
		return getList(criteria, ReportListaResultSet.class);
	}
	@SuppressWarnings({ "unchecked", "unused" })
	private List<ReportListaResultSet> getReportListaAppuntamentoStoriaResultset(Utente user, ReportCriteria r) {
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaAppuntamento.class, "sc");
		DetachedCriteria criteriaE= criteria.createCriteria("esito", "esit",  ApplicationUtil.leftJoin());
		DetachedCriteria criteriaO = criteria.createCriteria("utente", "op");
		DetachedCriteria criteriaApp = criteria.createCriteria("appuntamento", "r");
		DetachedCriteria criteriaC = criteriaApp.createCriteria("contatto", "cont");
		DetachedCriteria criteriaCA = criteriaC.createCriteria("campagna", "camp");
		if(user != null) {
			impostazioneDao.subCriteriaRoleSupervisore(user.getId(), criteria, "camp", "op");
			impostazioneDao.subCriteriaSupervisore(user.getId(), criteriaO, "id");
		}
		if(r != null) {
			DetachedCriteria criteriaLI = criteriaC.createCriteria("lista", "lista");
			ApplicationUtil.dataIntervalD(r.getInizio(), r.getFine(), "data", criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, criteriaLI);
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
			if(user != null && ApplicationUtil.ltZero(r.getCampagna())) {
				if(ApplicationUtil.isTrue(sd)) {
					impostazioneDao.subCriteriaCampagnaSupervisore(user.getId(), criteriaCA, "id");
				}
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("lista.id").as("lista_id")))
				.add(Projections.countDistinct("sc.id").as("totali"))
				.add(Projections.sqlProjection("sum(if(confermato, 1, 0)) as confermati", new String [] {"confermati"}, new Type []{LongType.INSTANCE}))
				.add(Projections.sqlProjection("sum(if(utile, 1, 0)) as utili", new String [] {"utili"}, new Type []{LongType.INSTANCE}))
				);
		return getList(criteria, ReportListaResultSet.class);
	}
	@SuppressWarnings({ "unchecked", "unused" })
	private List<ReportListaResultSet> getReportListaContrattoStoriaResultset(Utente user, ReportCriteria r) {
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContratto.class, "sc");
		DetachedCriteria criteriaE= criteria.createCriteria("esito", "esit",  ApplicationUtil.leftJoin());
		DetachedCriteria criteriaO = criteria.createCriteria("utente", "op");
		DetachedCriteria criteriaApp = criteria.createCriteria("contratto", "r");
		DetachedCriteria criteriaC = criteriaApp.createCriteria("contatto", "cont");
		DetachedCriteria criteriaCA = criteriaC.createCriteria("campagna", "camp");
		if(user != null) {
			impostazioneDao.subCriteriaRoleSupervisore(user.getId(), criteria, "camp", "op");
			impostazioneDao.subCriteriaSupervisore(user.getId(), criteriaO, "id");
		}
		if(r != null) {
			DetachedCriteria criteriaLI = criteriaC.createCriteria("lista", "lista");
			ApplicationUtil.dataIntervalD(r.getInizio(), r.getFine(), "data", criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, criteriaLI);
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
			if(user != null && ApplicationUtil.ltZero(r.getCampagna())) {
				if(ApplicationUtil.isTrue(sd)) {
					impostazioneDao.subCriteriaCampagnaSupervisore(user.getId(), criteriaCA, "id");
				}
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("lista.id").as("lista_id")))
				.add(Projections.countDistinct("sc.id").as("totali"))
				.add(Projections.sqlProjection("sum(if(confermato, 1, 0)) as confermati", new String [] {"confermati"}, new Type []{LongType.INSTANCE}))
				.add(Projections.sqlProjection("sum(if(utile, 1, 0)) as utili", new String [] {"utili"}, new Type []{LongType.INSTANCE}))
				);
		return getList(criteria, ReportListaResultSet.class);
	}
	public Map<Long, ConteggioModel> getListaContatti(List<Long> ids, Long campagnaId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class, "cont");
		if(ApplicationUtil.gtZero(campagnaId)) {
			criteria.add(Restrictions.eq("campagna.id", campagnaId));
		}
		criteria.add(Restrictions.in("lista.id", ids));
		return dataDao.evaluateConteggio(criteria, "cont.lista.id", "cont.id");
		
	}
	public Map<Long, ConteggioModel> getListaAnagrafiche(List<Long> ids, Long campagnaId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Lista.class, "cont");
		if(ApplicationUtil.gtZero(campagnaId)) {
			criteria.createCriteria("campagna").add(Restrictions.eq("id", campagnaId));
		}
		criteria.createCriteria("contatti", "st");
		criteria.add(Restrictions.in("id", ids));
		return dataDao.evaluateConteggio(criteria, "cont.id", "st.id");
	}
	public Map<Long, ConteggioModel> getListaCampagne(List<Long> ids, Long campagnaId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Lista.class, "cont");
		DetachedCriteria criteriaCA = criteria.createCriteria("campagna", "st");
		if(ApplicationUtil.gtZero(campagnaId)) {
			criteriaCA.add(Restrictions.eq("id", campagnaId));
		}
		criteria.add(Restrictions.in("id", ids));
		return dataDao.evaluateConteggio(criteria, "cont.id", "st.id");
	}
	@SuppressWarnings("unchecked")
	public RiepilogoCampagna getProgressoCampagne(Utente utente, ProgressoCriteria r) {
		Date data = ApplicationUtil.date();
		Boolean ts = impostazioneDao.getEstrazioneStato();
		RiepilogoCampagna rc = new RiepilogoCampagna(r.getContatti());
		DetachedCriteria criteriaCO = DetachedCriteria.forClass(Contatto.class, "cont");
		criteriaCO.add(Restrictions.eq("tipo", ConstantsGlobal.inviato));
		if(ApplicationUtil.isNotBlank(r.getTipo())) {
			criteriaCO.createCriteria("anagrafica").add(ApplicationUtil.isNotNull(r.getTipo()));
		}
		DetachedCriteria criteriaCA = criteriaCO.createCriteria("campagna", "camp");
		if(utente != null) {
			impostazioneDao.subCriteriaCampagnaSupervisore(utente.getId(), criteriaCA, "id");
		}
		if(ApplicationUtil.isTrue(r.getStatoC())) {
			ApplicationUtil.criteriaStato(criteriaCA, rc.getPubbliche(), ApplicationUtil.isTrue(ts));
		}
		if(ApplicationUtil.isTrue(r.getStatoL())) {
			Boolean s = impostazioneDao.getEstrazioneListe();
			if(ApplicationUtil.isTrue(s)) {
				DetachedCriteria criteriaLI = criteriaCO.createCriteria("lista", "list");
				ApplicationUtil.criteriaStato(criteriaLI, rc.getPubbliche(), ApplicationUtil.isTrue(ts));
			}
		}
		if(rc.getPubbliche()) {
			criteriaCO.add(Restrictions.isNull("proprietario.id"));
			if(!rc.getArchiviazione()) {
				criteriaCO.add(Restrictions.eq("archiviato", Boolean.FALSE));
			}
		} else {
			criteriaCO.add(Restrictions.isNotNull("proprietario.id"));
		}
		if(rc.getDisponibili() && !rc.getProssimi()) {
			criteriaCO.add(Restrictions.le("data_presentazione", data));
		}
		if(rc.getProssimi() && !rc.getDisponibili()) {
			criteriaCO.add(Restrictions.gt("data_presentazione", data));
		}
		ProjectionList p = Projections.projectionList();
		p.add(Projections.distinct(Projections.groupProperty("camp.id").as("eid")));
		p.add(Projections.property("camp.nome").as("esito"));
		p.add(Projections.property("camp.stato").as("stato"));
		p.add(Projections.property("camp.inverti").as("inverti"));
		p.add(Projections.property("camp.data_inizio").as("data_inizio"));
		p.add(Projections.property("camp.data_fine").as("data_fine"));
		p.add(Projections.property("camp.week_inizio").as("week_inizio"));
		p.add(Projections.property("camp.week_fine").as("week_fine"));
		p.add(Projections.property("camp.ora_inizio").as("ora_inizio"));
		p.add(Projections.property("camp.ora_fine").as("ora_fine"));
		p.add(Projections.property("camp.priorita").as("priorita"));
		p.add(Projections.property("camp.codice").as("codice"));
		if(rc.getPubbliche()) {
			if(rc.getArchiviazione()) {
				p.add(Projections.sqlProjection("sum(if(archiviato is true, 1, 0)) as archiviati", new String [] {"archiviati"}, new Type []{LongType.INSTANCE}));
			}
			if(rc.getDisponibili() || rc.getTutti()) {
				p.add(Projections.sqlProjection("sum(if(archiviato is false and data_presentazione <= now() and esitoChiamata_id = "+ConstantsEsito.ID_DA_CHIAMARE+", 1, 0)) as richiami1", new String [] {"richiami1"}, new Type []{LongType.INSTANCE}));
				p.add(Projections.sqlProjection("sum(if(archiviato is false and data_presentazione <= now() and esitoChiamata_id <> "+ConstantsEsito.ID_DA_CHIAMARE+", 1, 0)) as richiami2", new String [] {"richiami2"}, new Type []{LongType.INSTANCE}));
			}
			if(rc.getProssimi() || rc.getTutti()) {
				p.add(Projections.sqlProjection("sum(if(archiviato is false and data_presentazione > now() and esitoChiamata_id = "+ConstantsEsito.ID_DA_CHIAMARE+", 1, 0)) as richiami3", new String [] {"richiami3"}, new Type []{LongType.INSTANCE}));
				p.add(Projections.sqlProjection("sum(if(archiviato is false and data_presentazione > now() and esitoChiamata_id <> "+ConstantsEsito.ID_DA_CHIAMARE+", 1, 0)) as richiami4", new String [] {"richiami4"}, new Type []{LongType.INSTANCE}));
			}
		} else {
			if(rc.getDisponibili() || rc.getTutti()) {
				p.add(Projections.sqlProjection("sum(if(archiviato is false and data_presentazione <= now(), 1, 0)) as richiami1", new String [] {"richiami1"}, new Type []{LongType.INSTANCE}));
				p.add(Projections.sqlProjection("sum(if(archiviato is true and data_presentazione <= now(), 1, 0)) as richiami2", new String [] {"richiami2"}, new Type []{LongType.INSTANCE}));
			}
			if(rc.getProssimi() || rc.getTutti()) {
				p.add(Projections.sqlProjection("sum(if(archiviato is false and data_presentazione > now(), 1, 0)) as richiami3", new String [] {"richiami3"}, new Type []{LongType.INSTANCE}));
				p.add(Projections.sqlProjection("sum(if(archiviato is true and data_presentazione > now(), 1, 0)) as richiami4", new String [] {"richiami4"}, new Type []{LongType.INSTANCE}));
			}
		}
		criteriaCO.setProjection(p);
		ApplicationUtil.orderC(criteriaCA);
		List<RiepilogoCampagnaList> campagne = getList(criteriaCO, RiepilogoCampagnaList.class);
		rc.setLista(campagne);
		return rc;
	}
	@SuppressWarnings("unchecked")
	public RiepilogoCampagna getProgressoCampagnePartner(Utente utente, ProgressoCriteria r) {
		Date data = ApplicationUtil.date();
		Boolean ts = impostazioneDao.getEstrazioneStato();
		RiepilogoCampagna rc = new RiepilogoCampagna(r.getContatti());
		DetachedCriteria criteriaCO = DetachedCriteria.forClass(Contatto.class, "cont");
		criteriaCO.add(Restrictions.eq("tipo", ConstantsGlobal.inviato));
		if(ApplicationUtil.isNotBlank(r.getTipo())) {
			criteriaCO.createCriteria("anagrafica").add(ApplicationUtil.isNotNull(r.getTipo()));
		}
		DetachedCriteria criteriaCA = criteriaCO.createCriteria("campagna", "camp");
		if(utente != null) {
			impostazioneDao.subCriteriaCampagnaPartner(utente.getId(), criteriaCA, "id");
		}
		if(ApplicationUtil.isTrue(r.getStatoC())) {
			ApplicationUtil.criteriaStato(criteriaCA, rc.getPubbliche(), ApplicationUtil.isTrue(ts));
		}
		if(ApplicationUtil.isTrue(r.getStatoL())) {
			Boolean s = impostazioneDao.getEstrazioneListe();
			if(ApplicationUtil.isTrue(s)) {
				DetachedCriteria criteriaLI = criteriaCO.createCriteria("lista", "list");
				ApplicationUtil.criteriaStato(criteriaLI, rc.getPubbliche(), ApplicationUtil.isTrue(ts));
			}
		}
		if(rc.getPubbliche()) {
			criteriaCO.add(Restrictions.isNull("proprietario.id"));
			if(!rc.getArchiviazione()) {
				criteriaCO.add(Restrictions.eq("archiviato", Boolean.FALSE));
			}
		} else {
			criteriaCO.add(Restrictions.isNotNull("proprietario.id"));
		}
		if(rc.getDisponibili() && !rc.getProssimi()) {
			criteriaCO.add(Restrictions.le("data_presentazione", data));
		}
		if(rc.getProssimi() && !rc.getDisponibili()) {
			criteriaCO.add(Restrictions.gt("data_presentazione", data));
		}
		ProjectionList p = Projections.projectionList();
		p.add(Projections.distinct(Projections.groupProperty("camp.id").as("eid")));
		p.add(Projections.property("camp.nome").as("esito"));
		p.add(Projections.property("camp.stato").as("stato"));
		p.add(Projections.property("camp.inverti").as("inverti"));
		p.add(Projections.property("camp.data_inizio").as("data_inizio"));
		p.add(Projections.property("camp.data_fine").as("data_fine"));
		p.add(Projections.property("camp.week_inizio").as("week_inizio"));
		p.add(Projections.property("camp.week_fine").as("week_fine"));
		p.add(Projections.property("camp.ora_inizio").as("ora_inizio"));
		p.add(Projections.property("camp.ora_fine").as("ora_fine"));
		p.add(Projections.property("camp.priorita").as("priorita"));
		p.add(Projections.property("camp.codice").as("codice"));
		if(rc.getPubbliche()) {
			if(rc.getArchiviazione()) {
				p.add(Projections.sqlProjection("sum(if(archiviato is true, 1, 0)) as archiviati", new String [] {"archiviati"}, new Type []{LongType.INSTANCE}));
			}
			if(rc.getDisponibili() || rc.getTutti()) {
				p.add(Projections.sqlProjection("sum(if(archiviato is false and data_presentazione <= now() and esitoChiamata_id = "+ConstantsEsito.ID_DA_CHIAMARE+", 1, 0)) as richiami1", new String [] {"richiami1"}, new Type []{LongType.INSTANCE}));
				p.add(Projections.sqlProjection("sum(if(archiviato is false and data_presentazione <= now() and esitoChiamata_id <> "+ConstantsEsito.ID_DA_CHIAMARE+", 1, 0)) as richiami2", new String [] {"richiami2"}, new Type []{LongType.INSTANCE}));
			}
			if(rc.getProssimi() || rc.getTutti()) {
				p.add(Projections.sqlProjection("sum(if(archiviato is false and data_presentazione > now() and esitoChiamata_id = "+ConstantsEsito.ID_DA_CHIAMARE+", 1, 0)) as richiami3", new String [] {"richiami3"}, new Type []{LongType.INSTANCE}));
				p.add(Projections.sqlProjection("sum(if(archiviato is false and data_presentazione > now() and esitoChiamata_id <> "+ConstantsEsito.ID_DA_CHIAMARE+", 1, 0)) as richiami4", new String [] {"richiami4"}, new Type []{LongType.INSTANCE}));
			}
		} else {
			if(rc.getDisponibili() || rc.getTutti()) {
				p.add(Projections.sqlProjection("sum(if(archiviato is false and data_presentazione <= now(), 1, 0)) as richiami1", new String [] {"richiami1"}, new Type []{LongType.INSTANCE}));
				p.add(Projections.sqlProjection("sum(if(archiviato is true and data_presentazione <= now(), 1, 0)) as richiami2", new String [] {"richiami2"}, new Type []{LongType.INSTANCE}));
			}
			if(rc.getProssimi() || rc.getTutti()) {
				p.add(Projections.sqlProjection("sum(if(archiviato is false and data_presentazione > now(), 1, 0)) as richiami3", new String [] {"richiami3"}, new Type []{LongType.INSTANCE}));
				p.add(Projections.sqlProjection("sum(if(archiviato is true and data_presentazione > now(), 1, 0)) as richiami4", new String [] {"richiami4"}, new Type []{LongType.INSTANCE}));
			}
		}
		criteriaCO.setProjection(p);
		ApplicationUtil.orderC(criteriaCA);
		List<RiepilogoCampagnaList> campagne = getList(criteriaCO, RiepilogoCampagnaList.class);
		rc.setLista(campagne);
		return rc;
	}
	@SuppressWarnings("unchecked")
	public RiepilogoCampagna getProgressoListe(Long campagnaId, ProgressoCriteria r) {
		Date data = ApplicationUtil.date();
		RiepilogoCampagna rc = new RiepilogoCampagna(r.getContatti());
		DetachedCriteria criteriaCO = DetachedCriteria.forClass(Contatto.class, "cont");
		criteriaCO.add(Restrictions.eq("tipo", ConstantsGlobal.inviato));
		if(ApplicationUtil.isNotBlank(r.getTipo())) {
			criteriaCO.createCriteria("anagrafica").add(ApplicationUtil.isNotNull(r.getTipo()));
		}
		DetachedCriteria criteriaLI = criteriaCO.createCriteria("lista", "list");
		criteriaLI.createCriteria("campagna").add(Restrictions.eq("id", campagnaId));
		criteriaCO.add(Restrictions.eq("campagna.id",campagnaId));
		if(ApplicationUtil.isTrue(r.getStatoL())) {
			Boolean s = impostazioneDao.getEstrazioneListe();
			if(ApplicationUtil.isTrue(s)) {
				Boolean ts = impostazioneDao.getEstrazioneStato();
				ApplicationUtil.criteriaStato(criteriaLI, rc.getPubbliche(), ApplicationUtil.isTrue(ts));
			}
		}
		if(rc.getPubbliche()) {
			criteriaCO.add(Restrictions.isNull("proprietario.id"));
			if(!rc.getArchiviazione()) {
				criteriaCO.add(Restrictions.eq("archiviato", Boolean.FALSE));
			}
		} else {
			criteriaCO.add(Restrictions.isNotNull("proprietario.id"));
		}
		if(rc.getDisponibili() && !rc.getProssimi()) {
			criteriaCO.add(Restrictions.le("data_presentazione", data));
		}
		if(rc.getProssimi() && !rc.getDisponibili()) {
			criteriaCO.add(Restrictions.gt("data_presentazione", data));
		}
		ProjectionList p = Projections.projectionList();
		p.add(Projections.distinct(Projections.groupProperty("list.id").as("eid")));
		p.add(Projections.property("list.nome").as("esito"));
		p.add(Projections.property("list.stato").as("stato"));
		p.add(Projections.property("list.inverti").as("inverti"));
		p.add(Projections.property("list.data_inizio").as("data_inizio"));
		p.add(Projections.property("list.data_fine").as("data_fine"));
		p.add(Projections.property("list.week_inizio").as("week_inizio"));
		p.add(Projections.property("list.week_fine").as("week_fine"));
		p.add(Projections.property("list.ora_inizio").as("ora_inizio"));
		p.add(Projections.property("list.ora_fine").as("ora_fine"));
		p.add(Projections.property("list.priorita").as("priorita"));
		p.add(Projections.property("list.codice").as("codice"));
		if(rc.getPubbliche()) {
			if(rc.getArchiviazione()) {
				p.add(Projections.sqlProjection("sum(if(archiviato is true, 1, 0)) as archiviati", new String [] {"archiviati"}, new Type []{LongType.INSTANCE}));
			}
			if(rc.getDisponibili() || rc.getTutti()) {
				p.add(Projections.sqlProjection("sum(if(archiviato is false and data_presentazione <= now() and esitoChiamata_id = "+ConstantsEsito.ID_DA_CHIAMARE+", 1, 0)) as richiami1", new String [] {"richiami1"}, new Type []{LongType.INSTANCE}));
				p.add(Projections.sqlProjection("sum(if(archiviato is false and data_presentazione <= now() and esitoChiamata_id <> "+ConstantsEsito.ID_DA_CHIAMARE+", 1, 0)) as richiami2", new String [] {"richiami2"}, new Type []{LongType.INSTANCE}));
			}
			if(rc.getProssimi() || rc.getTutti()) {
				p.add(Projections.sqlProjection("sum(if(archiviato is false and data_presentazione > now() and esitoChiamata_id = "+ConstantsEsito.ID_DA_CHIAMARE+", 1, 0)) as richiami3", new String [] {"richiami3"}, new Type []{LongType.INSTANCE}));
				p.add(Projections.sqlProjection("sum(if(archiviato is false and data_presentazione > now() and esitoChiamata_id <> "+ConstantsEsito.ID_DA_CHIAMARE+", 1, 0)) as richiami4", new String [] {"richiami4"}, new Type []{LongType.INSTANCE}));
			}
		} else {
			if(rc.getDisponibili() || rc.getTutti()) {
				p.add(Projections.sqlProjection("sum(if(archiviato is false and data_presentazione <= now(), 1, 0)) as richiami1", new String [] {"richiami1"}, new Type []{LongType.INSTANCE}));
				p.add(Projections.sqlProjection("sum(if(archiviato is true and data_presentazione <= now(), 1, 0)) as richiami2", new String [] {"richiami2"}, new Type []{LongType.INSTANCE}));
			}
			if(rc.getProssimi() || rc.getTutti()) {
				p.add(Projections.sqlProjection("sum(if(archiviato is false and data_presentazione > now(), 1, 0)) as richiami3", new String [] {"richiami3"}, new Type []{LongType.INSTANCE}));
				p.add(Projections.sqlProjection("sum(if(archiviato is true and data_presentazione > now(), 1, 0)) as richiami4", new String [] {"richiami4"}, new Type []{LongType.INSTANCE}));
			}
		}
		criteriaCO.setProjection(p);
		ApplicationUtil.orderC(criteriaLI);
		List<RiepilogoCampagnaList> campagne = getList(criteriaCO, RiepilogoCampagnaList.class);
		rc.setLista(campagne);
		return rc;
	}
	public List<RiepilogoCampagnaList> completaAndamentoCampagne(String tipo, Utente utente, List<RiepilogoCampagnaList> campagne, int c) {
		if(ApplicationUtil.notEmpty(campagne)) {
			List<Long> utentiIdP = null;
			if(utente != null) {
				utentiIdP = new ArrayList<Long>();
				utentiIdP .addAll(campagnaDao.getIdUtentiBySupervisore(utente.getId()));
			}
			List<Long> campagneId = ApplicationUtil.prendiIdCampagnaReport(campagne);
			List<PannelloModel> d_a = Chiamate.getCodaContatti().get(c, utentiIdP);
			List<PannelloModel> d_b = Chiamate.getCodaAutodialer().list(c);
			List<PannelloModel> listaPubblici = getRichiamiPubblici(tipo, campagneId, c);
			List<PannelloModel> listaPersonali = getRichiamiPersonali(tipo, campagneId, utentiIdP, c);
			SortedMap<Long, ArrayList<PannelloModel>>  mappaListaPubblici = ApplicationUtil.collectionToHashPannello(listaPubblici, "getEsito_id");
			SortedMap<Long, ArrayList<PannelloModel>>  mappaListaPersonali = ApplicationUtil.collectionToHashPannello(listaPersonali, "getEsito_id");
			List<PannelloModel> listaNuovi = ApplicationUtil.getPannello(mappaListaPubblici, 0L);
			List<PannelloModel> listaRichiami = ApplicationUtil.getPannello(mappaListaPubblici, 1L);
			List<PannelloModel> listaPrivati = ApplicationUtil.getPannello(mappaListaPersonali, 0L);
			List<PannelloModel> listaArchiviati = ApplicationUtil.getPannello(mappaListaPersonali, 1L);
			List<PannelloModel> listaPrivatiAttiva = ApplicationUtil.getAttive(listaPrivati, d_a);
			List<PannelloModel> listaArchiviatiAttiva = ApplicationUtil.getAttive(listaArchiviati,  d_a);
			List<PannelloModel> listaNuoviAttiva = ApplicationUtil.getAttive(listaNuovi,  d_a);
			List<PannelloModel> listaNuoviAutoAttiva = ApplicationUtil.getAttive(listaNuovi,  d_b);
			List<PannelloModel> listaRichiamiAttiva = ApplicationUtil.getAttive(listaRichiami, d_a);
			List<PannelloModel> listaRichiamiAutoAttiva = ApplicationUtil.getAttive(listaRichiami,  d_b);
			SortedMap<Long, ArrayList<PannelloModel>>  mcNuovi = ApplicationUtil.collectionToHashPannello(listaNuovi, "getCampagna_id");
			SortedMap<Long, ArrayList<PannelloModel>>  mcRichiami = ApplicationUtil.collectionToHashPannello(listaRichiami, "getCampagna_id");
			SortedMap<Long, ArrayList<PannelloModel>>  mcPrivati = ApplicationUtil.collectionToHashPannello(listaPrivati, "getCampagna_id");
			SortedMap<Long, ArrayList<PannelloModel>>  mcArchiviati = ApplicationUtil.collectionToHashPannello(listaArchiviati, "getCampagna_id");
			SortedMap<Long, ArrayList<PannelloModel>>  mcPrivatiAttiva = ApplicationUtil.collectionToHashPannello(listaPrivatiAttiva, "getCampagna_id");
			SortedMap<Long, ArrayList<PannelloModel>>  mcArchiviatiAttiva = ApplicationUtil.collectionToHashPannello(listaArchiviatiAttiva, "getCampagna_id");
			SortedMap<Long, ArrayList<PannelloModel>>  mcNuoviAttiva = ApplicationUtil.collectionToHashPannello(listaNuoviAttiva, "getCampagna_id");
			SortedMap<Long, ArrayList<PannelloModel>>  mcNuoviAutoAttiva = ApplicationUtil.collectionToHashPannello(listaNuoviAutoAttiva, "getCampagna_id");
			SortedMap<Long, ArrayList<PannelloModel>>  mcRichiamiAttiva = ApplicationUtil.collectionToHashPannello(listaRichiamiAttiva, "getCampagna_id");
			SortedMap<Long, ArrayList<PannelloModel>>  mcRichiamiAutoAttiva = ApplicationUtil.collectionToHashPannello(listaRichiamiAutoAttiva, "getCampagna_id");
			Iterator<RiepilogoCampagnaList> objIter = campagne.iterator();
			while(objIter.hasNext()) {
				RiepilogoCampagnaList campagna = objIter.next();
				PannelloReport report = new PannelloReport();
				Integer contatti_nuovi = ApplicationUtil.getPannello(mcNuovi, campagna.getEid()).size();
				Integer attive_nuovi = ApplicationUtil.getPannello(mcNuoviAttiva, campagna.getEid()).size();
				Integer attive_nuovi_auto = ApplicationUtil.getPannello(mcNuoviAutoAttiva, campagna.getEid()).size();
				Integer contatti_richiami = ApplicationUtil.getPannello(mcRichiami, campagna.getEid()).size();
				Integer attive_richiami = ApplicationUtil.getPannello(mcRichiamiAttiva, campagna.getEid()).size();
				Integer attive_richiami_auto = ApplicationUtil.getPannello(mcRichiamiAutoAttiva, campagna.getEid()).size();
				report.setContatti(contatti_nuovi + contatti_richiami);
				report.setContatti_nuovi(contatti_nuovi);
				report.setContatti_richiami(contatti_richiami);
				report.setAttive(attive_nuovi + attive_richiami);
				report.setAttive_auto(attive_nuovi_auto + attive_richiami_auto);
				report.setAttive_nuovi(attive_nuovi);
				report.setAttive_nuovi_auto(attive_nuovi_auto);
				report.setAttive_richiami(attive_richiami);
				report.setAttive_richiami_auto(attive_richiami_auto);
				report.setContatti_privati( ApplicationUtil.getPannello(mcPrivati, campagna.getEid()).size());
				report.setAttive_privati( ApplicationUtil.getPannello(mcPrivatiAttiva, campagna.getEid()).size());
				report.setContatti_archiviati( ApplicationUtil.getPannello(mcArchiviati, campagna.getEid()).size());
				report.setAttive_archiviati( ApplicationUtil.getPannello(mcArchiviatiAttiva, campagna.getEid()).size());
				campagna.setReport(report);
			}
		}
		return campagne;
	}
	private DetachedCriteria trovaCampagneAttiveCriteria(Utente utente) {
		Boolean ts = impostazioneDao.getEstrazioneStato();
		DetachedCriteria campagnaC = DetachedCriteria.forClass(Campagna.class);
		ApplicationUtil.criteriaStato(campagnaC, true, ApplicationUtil.isTrue(ts));
		if(utente != null) {
			impostazioneDao.subCriteriaCampagnaUser(true, utente.getId(), campagnaC, "id");
		}
		return campagnaC;
	}
	@SuppressWarnings("unchecked")
	public List<RiepilogoCampagnaList> trovaCampagneAttive(String tipo, Utente utente, int start, int size) {
		DetachedCriteria criteria = trovaCampagneAttiveCriteria(utente);
		ProjectionList p = Projections.projectionList();
		p.add(Projections.distinct(Projections.groupProperty("id").as("eid")));
		p.add(Projections.property("nome").as("esito"));
		p.add(Projections.property("stato").as("stato"));
		p.add(Projections.property("inverti").as("inverti"));
		p.add(Projections.property("data_inizio").as("data_inizio"));
		p.add(Projections.property("data_fine").as("data_fine"));
		p.add(Projections.property("week_inizio").as("week_inizio"));
		p.add(Projections.property("week_fine").as("week_fine"));
		p.add(Projections.property("ora_inizio").as("ora_inizio"));
		p.add(Projections.property("ora_fine").as("ora_fine"));
		p.add(Projections.property("priorita").as("priorita"));
		p.add(Projections.property("codice").as("codice"));
		criteria.setProjection(p);
		ApplicationUtil.orderC(criteria);
		List<RiepilogoCampagnaList> campagne = getList(criteria, start, size, RiepilogoCampagnaList.class);
		return completaAndamentoCampagne(tipo, utente, campagne, 1);
	}
	public int trovaCampagneAttiveSize(Utente utente) {
		DetachedCriteria criteria = trovaCampagneAttiveCriteria(utente);
		return getListSize(criteria);
	}
	private DetachedCriteria trovaListeAttiveCriteria(RiepilogoCampagnaList campagna) {
		DetachedCriteria listaC = DetachedCriteria.forClass(Lista.class);
		listaC.createCriteria("campagna").add(Restrictions.eq("id", campagna.getEid()));
		Boolean s = impostazioneDao.getEstrazioneListe();
		if(ApplicationUtil.isTrue(s)) {
			Boolean ts = impostazioneDao.getEstrazioneStato();
			ApplicationUtil.criteriaStato(listaC, true, ApplicationUtil.isTrue(ts));
		}
		return listaC;
	}
	@SuppressWarnings("unchecked")
	public List<RiepilogoCampagnaList> trovaListeAttive(String tipo, Utente utente, RiepilogoCampagnaList campagna, int start, int size) {
		DetachedCriteria criteria = trovaListeAttiveCriteria(campagna);
		ProjectionList p = Projections.projectionList();
		p.add(Projections.distinct(Projections.groupProperty("id").as("eid")));
		p.add(Projections.property("nome").as("esito"));
		p.add(Projections.property("stato").as("stato"));
		p.add(Projections.property("inverti").as("inverti"));
		p.add(Projections.property("data_inizio").as("data_inizio"));
		p.add(Projections.property("data_fine").as("data_fine"));
		p.add(Projections.property("week_inizio").as("week_inizio"));
		p.add(Projections.property("week_fine").as("week_fine"));
		p.add(Projections.property("ora_inizio").as("ora_inizio"));
		p.add(Projections.property("ora_fine").as("ora_fine"));
		p.add(Projections.property("priorita").as("priorita"));
		p.add(Projections.property("codice").as("codice"));
		criteria.setProjection(p);
		ApplicationUtil.orderC(criteria);
		List<RiepilogoCampagnaList> campagne = getList(criteria, start, size, RiepilogoCampagnaList.class);
		return completaAndamentoCampagne(tipo, utente, campagne, 2);
	}
	public int trovaListeAttiveSize(Utente utente, RiepilogoCampagnaList campagna) {
		DetachedCriteria criteria = trovaListeAttiveCriteria(campagna);
		return getListSize(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<PannelloModel>  getRichiamiPubblici(String tipo, List<Long> campagnaId, int campagna) {
		if(ApplicationUtil.empty(campagnaId) && ApplicationUtil.ltZero(campagna)) {
			return new ArrayList<PannelloModel>();
		}
		DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class, "cont");
		if(ApplicationUtil.isNotBlank(tipo)) {
			criteria.createCriteria("anagrafica").add(ApplicationUtil.isNotNull(tipo));
		}
		criteria.add(Restrictions.eq("tipo", ConstantsGlobal.inviato));
		if(campagna == 1) {
			Boolean s = impostazioneDao.getEstrazioneListe();
			if(ApplicationUtil.isTrue(s)) {
				Boolean ts = impostazioneDao.getEstrazioneStato();
				DetachedCriteria listaC = criteria.createCriteria("lista");
				ApplicationUtil.criteriaStato(listaC, true, ApplicationUtil.isTrue(ts));
			}
			criteria.createCriteria("campagna", "camp").add(Restrictions.in("id",campagnaId));
		}
		if(campagna == 2) {
			criteria.createCriteria("lista", "camp").add(Restrictions.in("id",campagnaId));
		}
		criteria.add(Restrictions.isNull("proprietario.id"));
		ApplicationUtil.archiviazione(criteria, ApplicationUtil.date(), false);
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.property("cont.id").as("contatto_id")))
				.add(Projections.sqlProjection("if(esitoChiamata_id = "+ ConstantsEsito.ID_DA_CHIAMARE + ", 0, 1) as esito_id", new String [] {"esito_id"}, new Type []{LongType.INSTANCE}))
				.add(Projections.property("camp.id").as("campagna_id")));
		return getList(criteria, PannelloModel.class);
	}
	@SuppressWarnings("unchecked")
	public List<PannelloModel>  getRichiamiPersonali(String tipo, List<Long> campagnaId, List<Long> utentiId, int campagna) {
		if(ApplicationUtil.empty(campagnaId) && ApplicationUtil.ltZero(campagna)) {
			return new ArrayList<PannelloModel>();
		}
		DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class, "cont");
		criteria.add(Restrictions.eq("tipo", ConstantsGlobal.inviato));
		if(ApplicationUtil.isNotBlank(tipo)) {
			criteria.createCriteria("anagrafica").add(ApplicationUtil.isNotNull(tipo));
		}
		if(campagna == 1) {
			Boolean s = impostazioneDao.getEstrazioneListe();
			if(ApplicationUtil.isTrue(s)) {
				Boolean ts = impostazioneDao.getEstrazioneStato();
				DetachedCriteria listaC = criteria.createCriteria("lista");
				ApplicationUtil.criteriaStato(listaC, false, ApplicationUtil.isTrue(ts));
			}
			criteria.createCriteria("campagna", "camp").add(Restrictions.in("id",campagnaId));
		} 
		if(campagna == 2) {
			criteria.createCriteria("lista", "camp").add(Restrictions.in("id",campagnaId));
		}
		criteria.add(Restrictions.le("data_presentazione", ApplicationUtil.date()));
		if(ApplicationUtil.notEmpty(utentiId)) {
			criteria.add(Restrictions.in("proprietario.id", utentiId));
		} else {
			criteria.add(Restrictions.isNotNull("proprietario.id"));
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.property("cont.id").as("contatto_id")))
				.add(Projections.sqlProjection("if(archiviato is false, 0, 1) as esito_id", new String [] {"esito_id"}, new Type []{LongType.INSTANCE}))
				.add(Projections.property("camp.id").as("campagna_id")));
		return getList(criteria, PannelloModel.class);
	}
	public SemaforoOperatore getContattiSemaforo(Utente user, Campagna campagna, Integer a) {
		if(ApplicationUtil.gtZero(a)) {
			Date oggi = ApplicationUtil.date();
			Integer pannello = 1;
			int tipo = a.intValue();
			DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContatto.class, "sc");
			ApplicationUtil.pannello(pannello, criteria);
			if(campagna != null && ApplicationUtil.gtZero(campagna.getId()) && (tipo == 1 || tipo == 3 || tipo == 5 || tipo == 7)) {
				criteria.createCriteria("contatto").add(Restrictions.eq("campagna.id", campagna.getId()));
			}
			criteria.add(Restrictions.eq("operatore.id", user.getId()));
			if(tipo == 1 || tipo == 2) {
				criteria.add(Restrictions.gt("data", ApplicationUtil.inizioGiornata(oggi)));
			}
			if(tipo == 3 || tipo == 4) {
				criteria.add(Restrictions.gt("data", ApplicationUtil.inizioSettimana(oggi)));
			}
			if(tipo == 5 || tipo == 6) {
				criteria.add(Restrictions.gt("data", ApplicationUtil.inizioMese(oggi)));
			}
			if(tipo == 7 || tipo == 8) {
				criteria.add(Restrictions.gt("data", ApplicationUtil.inizioAnno(oggi)));
			}
			ProjectionList p = Projections.projectionList();
			p.add(Projections.countDistinct("sc.id").as("contatti"));
			p.add(Projections.sqlProjection("sum(if(archiviazione_id is not null, 1, 0)) as archiviazioni", new String [] {"archiviazioni"}, new Type []{LongType.INSTANCE}));
			p.add(Projections.sqlProjection("sum(if(secondi_conv_eff > 0, 1, 0)) as conversazioni", new String [] {"conversazioni"}, new Type []{LongType.INSTANCE}));
			criteria.setProjection(p);
			return (SemaforoOperatore) getUniqueResult(criteria, SemaforoOperatore.class);
		}
		return null;
	}
	public SemaforoOperatore getContattiSemaforo(Utente user, Campagna campagna, Integer a, SemaforoOperatore s, Contatto contatto) {
		if(contatto != null  && ApplicationUtil.gtZero(a)) {
			if(ApplicationUtil.isTrue(contatto.getAutodialerApertura())) {
				return getContattiSemaforo(user, campagna, a); 
			}
			return s;
		}
		return null;
	}
	@SuppressWarnings({ "unchecked" })
	private List<ReportOperatoreResultSet> getReportOperatoreResultset(Utente user, ReportCriteria r) {
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class, "sc");
		DetachedCriteria criteriaO = criteria.createCriteria("utente", "op");
		DetachedCriteria criteriaCA = criteria.createCriteria("campagna", "camp");
		DetachedCriteria criteriaLI = criteria.createCriteria("lista", "lista");
		if(user!=null) {
			impostazioneDao.subCriteriaSupervisore(user.getId(), criteriaO, "id");
		}
		if(r != null) {
			ApplicationUtil.intervallo(ApplicationUtil.isTrue(r.getInserimento()), r.getInizio(), r.getFine(), criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, criteriaLI);
			if(ApplicationUtil.gtZero(r.getTeam())) {
				impostazioneDao.subCriteriaOperatoreTeam(r.getTeam(), criteriaO, "id");
			} 
			if(ApplicationUtil.gtZero(r.getLista())) {
				criteriaLI.add(Restrictions.idEq(r.getLista()));
			}
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
			if(user != null && ApplicationUtil.ltZero(r.getCampagna())) {
				if(ApplicationUtil.isTrue(sd)) {
					impostazioneDao.subCriteriaCampagnaSupervisore(user.getId(), criteriaCA, "id");
				}
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("op.id").as("user_id")))
				.add(Projections.countDistinct("sc.id").as("chiamate"))
				);
		return getList(criteria, ReportOperatoreResultSet.class);
	}
	@SuppressWarnings({ "unchecked", "unused" })
	private List<ReportOperatoreResultSet> getReportOperatoreResultsetDati(Utente user, ReportCriteria r) {
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class, "sc");
		DetachedCriteria criteriaE= criteria.createCriteria("esitoChiamata", "esit");
		DetachedCriteria criteriaA= criteria.createCriteria("esitoArchiviazione", "arch",  ApplicationUtil.leftJoin());
		DetachedCriteria criteriaO = criteria.createCriteria("utente", "op");
		DetachedCriteria criteriaCA = criteria.createCriteria("campagna", "camp");
		DetachedCriteria criteriaLI = criteria.createCriteria("lista", "lista");
		if(user!=null) {
			impostazioneDao.subCriteriaSupervisore(user.getId(), criteriaO, "id");
		}
		if(r != null) {
			ApplicationUtil.intervallo(ApplicationUtil.isTrue(r.getInserimento()), r.getInizio(), r.getFine(), criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, criteriaLI);
			if(ApplicationUtil.gtZero(r.getTeam())) {
				impostazioneDao.subCriteriaOperatoreTeam(r.getTeam(), criteriaO, "id");
			} 
			if(ApplicationUtil.gtZero(r.getLista())) {
				criteriaLI.add(Restrictions.idEq(r.getLista()));
			}
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
			if(user != null && ApplicationUtil.ltZero(r.getCampagna())) {
				if(ApplicationUtil.isTrue(sd)) {
					impostazioneDao.subCriteriaCampagnaSupervisore(user.getId(), criteriaCA, "id");
				}
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("op.id").as("user_id")))
				.add(Projections.groupProperty("esit.id").as("esitoChiamata_id"))
				.add(Projections.groupProperty("arch.id").as("esitoArchiviazione_id"))
				.add(Projections.countDistinct("sc.id").as("chiamate"))
				);
		ApplicationUtil.orderR1(criteria);
		return getList(criteria, ReportOperatoreResultSet.class);
	}
	@SuppressWarnings({ "unchecked" })
	private List<ReportOperatoreResultSet> getReportOperatoreStoriaResultset(Utente user, ReportCriteria r) {
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContatto.class, "sc");
		DetachedCriteria criteriaO = criteria.createCriteria("operatore", "op");
		DetachedCriteria criteriaC = criteria.createCriteria("contatto", "cont");
		DetachedCriteria criteriaCA = criteriaC.createCriteria("campagna", "camp");
		if(user != null) {
			impostazioneDao.subCriteriaRoleSupervisore(user.getId(), criteria, "camp", "op");
			impostazioneDao.subCriteriaSupervisore(user.getId(), criteriaO, "id");
		}
		if(r != null) {
			DetachedCriteria criteriaLI = criteriaC.createCriteria("lista", "lista");
			ApplicationUtil.pannello(r.getPannello(), criteria);
			ApplicationUtil.dataIntervalD(r.getInizio(), r.getFine(), "data", criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, criteriaLI);
			if(ApplicationUtil.gtZero(r.getTeam())) {
				impostazioneDao.subCriteriaOperatoreTeam(r.getTeam(), criteriaO, "id");
			} 
			if(ApplicationUtil.gtZero(r.getLista())) {
				criteriaLI.add(Restrictions.idEq(r.getLista()));
			}
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
			if(user != null && ApplicationUtil.ltZero(r.getCampagna())) {
				if(ApplicationUtil.isTrue(sd)) {
					impostazioneDao.subCriteriaCampagnaSupervisore(user.getId(), criteriaCA, "id");
				}
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("op.id").as("user_id")))
				.add(Projections.countDistinct("sc.id").as("chiamate"))
				.add(Projections.sum("sc.secondi_conv_eff").as("durata_conv_eff"))
				.add(Projections.sum("sc.secondi_tot").as("durata"))
				.add(Projections.sum("sc.secondi_conv_tot").as("durata_conv_tot"))
				.add(Projections.sum("sc.secondi_post").as("durata_post"))
				.add(Projections.sum("sc.secondi_pre").as("durata_pre"))
				.add(Projections.sum("sc.secondi_rec").as("durata_rec"))
				.add(Projections.sum("sc.secondi_attesa").as("durata_attesa"))
				.add(Projections.avg("sc.secondi_conv_eff").as("media_conv_eff"))
				.add(Projections.avg("sc.secondi_tot").as("media"))
				.add(Projections.avg("sc.secondi_conv_tot").as("media_conv_tot"))
				.add(Projections.avg("sc.secondi_post").as("media_post"))
				.add(Projections.avg("sc.secondi_pre").as("media_pre"))
				.add(Projections.avg("sc.secondi_rec").as("media_rec"))
				.add(Projections.avg("sc.secondi_attesa").as("media_attesa"))
				);
		return getList(criteria, ReportOperatoreResultSet.class);
	}
	@SuppressWarnings({ "unchecked", "unused" })
	private List<ReportOperatoreResultSet> getReportOperatoreStoriaResultsetDati(Utente user, ReportCriteria r) {
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContatto.class, "sc");
		DetachedCriteria criteriaE= criteria.createCriteria("esito", "esit");
		DetachedCriteria criteriaA= criteria.createCriteria("archiviazione", "arch",  ApplicationUtil.leftJoin());
		DetachedCriteria criteriaO = criteria.createCriteria("operatore", "op");
		DetachedCriteria criteriaC = criteria.createCriteria("contatto", "cont");
		DetachedCriteria criteriaCA = criteriaC.createCriteria("campagna", "camp");
		if(user != null) {
			impostazioneDao.subCriteriaRoleSupervisore(user.getId(), criteria, "camp", "op");
			impostazioneDao.subCriteriaSupervisore(user.getId(), criteriaO, "id");
		}
		if(r != null) {
			DetachedCriteria criteriaLI = criteriaC.createCriteria("lista", "lista");
			ApplicationUtil.pannello(r.getPannello(), criteria);
			ApplicationUtil.dataIntervalD(r.getInizio(), r.getFine(), "data", criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, criteriaLI);
			if(ApplicationUtil.gtZero(r.getTeam())) {
				impostazioneDao.subCriteriaOperatoreTeam(r.getTeam(), criteriaO, "id");
			} 
			if(ApplicationUtil.gtZero(r.getLista())) {
				criteriaLI.add(Restrictions.idEq(r.getLista()));
			}
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
			if(user != null && ApplicationUtil.ltZero(r.getCampagna())) {
				if(ApplicationUtil.isTrue(sd)) {
					impostazioneDao.subCriteriaCampagnaSupervisore(user.getId(), criteriaCA, "id");
				}
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("op.id").as("user_id")))
				.add(Projections.groupProperty("esit.id").as("esitoChiamata_id"))
				.add(Projections.groupProperty("arch.id").as("esitoArchiviazione_id"))
				.add(Projections.countDistinct("sc.id").as("chiamate"))
				.add(Projections.sum("sc.secondi_conv_eff").as("durata_conv_eff"))
				.add(Projections.sum("sc.secondi_tot").as("durata"))
				.add(Projections.sum("sc.secondi_conv_tot").as("durata_conv_tot"))
				.add(Projections.sum("sc.secondi_post").as("durata_post"))
				.add(Projections.sum("sc.secondi_pre").as("durata_pre"))
				.add(Projections.sum("sc.secondi_rec").as("durata_rec"))
				.add(Projections.sum("sc.secondi_attesa").as("durata_attesa"))
				.add(Projections.avg("sc.secondi_conv_eff").as("media_conv_eff"))
				.add(Projections.avg("sc.secondi_tot").as("media"))
				.add(Projections.avg("sc.secondi_conv_tot").as("media_conv_tot"))
				.add(Projections.avg("sc.secondi_post").as("media_post"))
				.add(Projections.avg("sc.secondi_pre").as("media_pre"))
				.add(Projections.avg("sc.secondi_rec").as("media_rec"))
				.add(Projections.avg("sc.secondi_attesa").as("media_attesa"))
				);
		ApplicationUtil.orderR1(criteria);
		return getList(criteria, ReportOperatoreResultSet.class);
	}
	@SuppressWarnings({ "unchecked" })
	private List<ReportListaEsitoResultSet> getReportListaEsitoResultset(Utente user, ReportCriteria r) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class, "sc");
		DetachedCriteria criteriaLI = criteria.createCriteria("lista", "ls");
		DetachedCriteria criteriaCA = criteria.createCriteria("campagna", "camp");
		if(r != null) {
			ApplicationUtil.intervallo(ApplicationUtil.isTrue(r.getInserimento()), r.getInizio(), r.getFine(), criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, criteriaLI);
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
			if(user != null && ApplicationUtil.ltZero(r.getCampagna())) {
				impostazioneDao.subCriteriaCampagnaSupervisore(user.getId(), criteriaCA, "id");
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("ls.id").as("lista_id")))
				.add(Projections.countDistinct("sc.id").as("chiamate"))
				);
		return getList(criteria, ReportListaEsitoResultSet.class);
	}
	@SuppressWarnings({ "unchecked", "unused" })
	private List<ReportListaEsitoResultSet> getReportListaEsitoResultsetDati(Utente user, ReportCriteria r) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class, "sc");
		DetachedCriteria criteriaE= criteria.createCriteria("esitoChiamata", "esit");
		DetachedCriteria criteriaA= criteria.createCriteria("esitoArchiviazione", "arch",  ApplicationUtil.leftJoin());
		DetachedCriteria criteriaLI = criteria.createCriteria("lista", "ls");
		DetachedCriteria criteriaCA = criteria.createCriteria("campagna", "camp");
		if(r != null) {
			ApplicationUtil.intervallo(ApplicationUtil.isTrue(r.getInserimento()), r.getInizio(), r.getFine(), criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, criteriaLI);
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
			if(user != null && ApplicationUtil.ltZero(r.getCampagna())) {
				impostazioneDao.subCriteriaCampagnaSupervisore(user.getId(), criteriaCA, "id");
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("ls.id").as("lista_id")))
				.add(Projections.groupProperty("esit.id").as("esitoChiamata_id"))
				.add(Projections.groupProperty("arch.id").as("esitoArchiviazione_id"))
				.add(Projections.countDistinct("sc.id").as("chiamate"))
				);
		ApplicationUtil.orderR4(criteria);
		return getList(criteria, ReportListaEsitoResultSet.class);
	}
	@SuppressWarnings({ "unchecked" })
	private List<ReportListaEsitoResultSet> getReportListaEsitoStoriaResultset(Utente user, ReportCriteria r) {
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContatto.class, "sc");
		DetachedCriteria criteriaO = criteria.createCriteria("operatore", "op", ApplicationUtil.leftJoin());
		DetachedCriteria criteriaC = criteria.createCriteria("contatto", "cont");
		DetachedCriteria criteriaCA = criteriaC.createCriteria("campagna", "camp");
		if(user != null) {
			impostazioneDao.subCriteriaRoleSupervisore(user.getId(), criteria, "camp", "op");
			impostazioneDao.subCriteriaSupervisore(user.getId(), criteriaO, "id");
		}
		if(r != null) {
			DetachedCriteria criteriaLI = criteriaC.createCriteria("lista", "ls");
			ApplicationUtil.pannello(r.getPannello(), criteria);
			ApplicationUtil.dataIntervalD(r.getInizio(), r.getFine(), "data", criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, criteriaLI);
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
			if(user != null && ApplicationUtil.ltZero(r.getCampagna())) {
				if(ApplicationUtil.isTrue(sd)) {
					impostazioneDao.subCriteriaCampagnaSupervisore(user.getId(), criteriaCA, "id");
				}
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("ls.id").as("lista_id")))
				.add(Projections.countDistinct("sc.id").as("chiamate"))
				.add(Projections.sum("sc.secondi_conv_eff").as("durata_conv_eff"))
				.add(Projections.sum("sc.secondi_tot").as("durata"))
				.add(Projections.sum("sc.secondi_conv_tot").as("durata_conv_tot"))
				.add(Projections.sum("sc.secondi_post").as("durata_post"))
				.add(Projections.sum("sc.secondi_pre").as("durata_pre"))
				.add(Projections.sum("sc.secondi_rec").as("durata_rec"))
				.add(Projections.sum("sc.secondi_attesa").as("durata_attesa"))
				.add(Projections.avg("sc.secondi_conv_eff").as("media_conv_eff"))
				.add(Projections.avg("sc.secondi_tot").as("media"))
				.add(Projections.avg("sc.secondi_conv_tot").as("media_conv_tot"))
				.add(Projections.avg("sc.secondi_post").as("media_post"))
				.add(Projections.avg("sc.secondi_pre").as("media_pre"))
				.add(Projections.avg("sc.secondi_rec").as("media_rec"))
				.add(Projections.avg("sc.secondi_attesa").as("media_attesa"))
				);
		return getList(criteria, ReportListaEsitoResultSet.class);
	}
	@SuppressWarnings({ "unchecked", "unused" })
	private List<ReportListaEsitoResultSet> getReportListaEsitoStoriaResultsetDati(Utente user, ReportCriteria r) {
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContatto.class, "sc");
		DetachedCriteria criteriaE= criteria.createCriteria("esito", "esit");
		DetachedCriteria criteriaA= criteria.createCriteria("archiviazione", "arch",  ApplicationUtil.leftJoin());
		DetachedCriteria criteriaO = criteria.createCriteria("operatore", "op", ApplicationUtil.leftJoin());
		DetachedCriteria criteriaC = criteria.createCriteria("contatto", "cont");
		DetachedCriteria criteriaCA = criteriaC.createCriteria("campagna", "camp");
		if(user != null) {
			impostazioneDao.subCriteriaRoleSupervisore(user.getId(), criteria, "camp", "op");
			impostazioneDao.subCriteriaSupervisore(user.getId(), criteriaO, "id");
		}
		if(r != null) {
			DetachedCriteria criteriaLI = criteriaC.createCriteria("lista", "ls");
			ApplicationUtil.pannello(r.getPannello(), criteria);
			ApplicationUtil.dataIntervalD(r.getInizio(), r.getFine(), "data", criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, criteriaLI);
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
			if(user != null && ApplicationUtil.ltZero(r.getCampagna())) {
				if(ApplicationUtil.isTrue(sd)) {
					impostazioneDao.subCriteriaCampagnaSupervisore(user.getId(), criteriaCA, "id");
				}
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("ls.id").as("lista_id")))
				.add(Projections.groupProperty("esit.id").as("esitoChiamata_id"))
				.add(Projections.groupProperty("arch.id").as("esitoArchiviazione_id"))
				.add(Projections.countDistinct("sc.id").as("chiamate"))
				.add(Projections.sum("sc.secondi_conv_eff").as("durata_conv_eff"))
				.add(Projections.sum("sc.secondi_tot").as("durata"))
				.add(Projections.sum("sc.secondi_conv_tot").as("durata_conv_tot"))
				.add(Projections.sum("sc.secondi_post").as("durata_post"))
				.add(Projections.sum("sc.secondi_pre").as("durata_pre"))
				.add(Projections.sum("sc.secondi_rec").as("durata_rec"))
				.add(Projections.sum("sc.secondi_attesa").as("durata_attesa"))
				.add(Projections.avg("sc.secondi_conv_eff").as("media_conv_eff"))
				.add(Projections.avg("sc.secondi_tot").as("media"))
				.add(Projections.avg("sc.secondi_conv_tot").as("media_conv_tot"))
				.add(Projections.avg("sc.secondi_post").as("media_post"))
				.add(Projections.avg("sc.secondi_pre").as("media_pre"))
				.add(Projections.avg("sc.secondi_rec").as("media_rec"))
				.add(Projections.avg("sc.secondi_attesa").as("media_attesa"))
				);
		ApplicationUtil.orderR4(criteria);
		return getList(criteria, ReportListaEsitoResultSet.class);
	}
	private int getArchiviazioniOdierneOperatoreReport(Long operatore, Long supervisore, ReportCriteria r) {
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContatto.class);
		DetachedCriteria criteriaC = criteria.createCriteria("operatore", "op");
		DetachedCriteria criteriaCO = criteria.createCriteria("contatto");
		DetachedCriteria criteriaCA = criteriaCO.createCriteria("campagna", "camp");
		criteriaC.add(Restrictions.eq("id", operatore));
		if (ApplicationUtil.gtZero(supervisore)) {
			impostazioneDao.subCriteriaRoleSupervisore(supervisore, criteria, "camp", "op");
			impostazioneDao.subCriteriaSupervisore(supervisore, criteriaC, "id");
		}
		if(r != null) {
			DetachedCriteria criteriaLI = criteriaCO.createCriteria("lista", "lista");
			ApplicationUtil.pannello(r.getPannello(), criteria);
			ApplicationUtil.dataIntervalD(r.getInizio(), r.getFine(), "data", criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, criteriaLI);
			if(ApplicationUtil.gtZero(r.getLista())) {
				criteriaLI.add(Restrictions.idEq(r.getLista()));
			}
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
			if(ApplicationUtil.ltZero(r.getCampagna())) {
				if(ApplicationUtil.gtZero(supervisore)) {
					if(ApplicationUtil.isTrue(sd)) {
						impostazioneDao.subCriteriaCampagnaSupervisore(supervisore, criteriaCA, "id");
					}
				}
			}
		}
		criteria.add(Restrictions.isNotNull("archiviazione.id"));
		return getListSize(criteria);
	}
	private int getContattiOdierneOperatoreReport(Long operatore, Long supervisore, ReportCriteria r) {
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContatto.class);
		DetachedCriteria criteriaC = criteria.createCriteria("operatore", "op");
		DetachedCriteria criteriaCO = criteria.createCriteria("contatto");
		DetachedCriteria criteriaCA = criteriaCO.createCriteria("campagna", "camp");
		criteriaC.add(Restrictions.eq("id", operatore));
		if (ApplicationUtil.gtZero(supervisore)) {
			impostazioneDao.subCriteriaRoleSupervisore(supervisore, criteria, "camp", "op");
			impostazioneDao.subCriteriaSupervisore(supervisore, criteriaC, "id");
		}
		DetachedCriteria criteriaLI = criteriaCO.createCriteria("lista");
		if(r != null) {
			ApplicationUtil.pannello(r.getPannello(), criteria);
			ApplicationUtil.dataIntervalD(r.getInizio(), r.getFine(), "data", criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, criteriaLI);
			if(ApplicationUtil.gtZero(r.getLista())) {
				criteriaLI.add(Restrictions.idEq(r.getLista()));
			}
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
			if(ApplicationUtil.ltZero(r.getCampagna())) {
				if(ApplicationUtil.gtZero(supervisore)) {
					if(ApplicationUtil.isTrue(sd)) {
						impostazioneDao.subCriteriaCampagnaSupervisore(supervisore, criteriaCA, "id");
					}
				}
			}
		}
		return getListSize(criteria);
	}
	private int getConversazioniOdierneOperatoreReport(Long operatore, Long supervisore, ReportCriteria r) {
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContatto.class);
		DetachedCriteria criteriaC = criteria.createCriteria("operatore", "op");
		DetachedCriteria criteriaCO = criteria.createCriteria("contatto");
		DetachedCriteria criteriaCA = criteriaCO.createCriteria("campagna", "camp");
		criteriaC.add(Restrictions.eq("id", operatore));
		if (ApplicationUtil.gtZero(supervisore)) {
			impostazioneDao.subCriteriaRoleSupervisore(supervisore, criteria, "camp", "op");
			impostazioneDao.subCriteriaSupervisore(supervisore, criteriaC, "id");
		}
		if(r != null) {
			DetachedCriteria criteriaLI = criteriaCO.createCriteria("lista");
			ApplicationUtil.pannello(r.getPannello(), criteria);
			ApplicationUtil.dataIntervalD(r.getInizio(), r.getFine(), "data", criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, criteriaLI);
			if(ApplicationUtil.gtZero(r.getLista())) {
				criteriaLI.add(Restrictions.idEq(r.getLista()));
			}
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
			if(ApplicationUtil.ltZero(r.getCampagna())) {
				if(ApplicationUtil.gtZero(supervisore)) {
					if(ApplicationUtil.isTrue(sd)) {
						impostazioneDao.subCriteriaCampagnaSupervisore(supervisore, criteriaCA, "id");
					}
				}
			}
		}
		criteria.add(Restrictions.gt("secondi_conv_eff", 0));
		return getListSize(criteria);
	}
	private int getAppuntamentiOdierneOperatoreReport(Long operatore, Long supervisore, ReportCriteria r) {
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(Appuntamento.class);
		DetachedCriteria criteriaC = criteria.createCriteria("operatore", "op");
		DetachedCriteria criteriaCO = criteria.createCriteria("contatto");
		DetachedCriteria criteriaCA = criteriaCO.createCriteria("campagna", "camp");
		criteriaC.add(Restrictions.eq("id", operatore));
		if (ApplicationUtil.gtZero(supervisore)) {
			impostazioneDao.subCriteriaRoleSupervisore(supervisore, criteria, "camp", "op");
			impostazioneDao.subCriteriaSupervisore(supervisore, criteriaC, "id");
		}
		if(r != null) {
			DetachedCriteria criteriaLI = criteriaCO.createCriteria("lista");
			ApplicationUtil.intervallo(ApplicationUtil.isTrue(r.getInserimento()), r.getInizio(), r.getFine(), criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, criteriaLI);
			if(ApplicationUtil.gtZero(r.getLista())) {
				criteriaLI.add(Restrictions.idEq(r.getLista()));
			}
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
			if(ApplicationUtil.ltZero(r.getCampagna())) {
				if(ApplicationUtil.gtZero(supervisore)) {
					if(ApplicationUtil.isTrue(sd)) {
						impostazioneDao.subCriteriaCampagnaSupervisore(supervisore, criteriaCA, "id");
					}
				}
			}
		}
		return getListSize(criteria);
	}
	private int getContrattiOdierneOperatoreReport(Long operatore, Long supervisore, ReportCriteria r) {
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(Contratto.class);
		DetachedCriteria criteriaC = criteria.createCriteria("operatore", "op");
		DetachedCriteria criteriaCO = criteria.createCriteria("contatto");
		DetachedCriteria criteriaCA = criteriaCO.createCriteria("campagna", "camp");
		criteriaC.add(Restrictions.eq("id", operatore));
		if (ApplicationUtil.gtZero(supervisore)) {
			impostazioneDao.subCriteriaRoleSupervisore(supervisore, criteria, "camp", "op");
			impostazioneDao.subCriteriaSupervisore(supervisore, criteriaC, "id");
		}
		if(r != null) {
			DetachedCriteria criteriaLI = criteriaCO.createCriteria("lista");
			ApplicationUtil.intervallo(ApplicationUtil.isTrue(r.getInserimento()), r.getInizio(), r.getFine(), criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, criteriaLI);
			if(ApplicationUtil.gtZero(r.getLista())) {
				criteriaLI.add(Restrictions.idEq(r.getLista()));
			}
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
			if(ApplicationUtil.ltZero(r.getCampagna())) {
				if(ApplicationUtil.gtZero(supervisore)) {
					if(ApplicationUtil.isTrue(sd)) {
						impostazioneDao.subCriteriaCampagnaSupervisore(supervisore, criteriaCA, "id");
					}
				}
			}
		}
		return getListSize(criteria);
	}
	private int getArchiviazioniOdierneListaEsitoReport(Long lista, Long supervisore, ReportCriteria r) {
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContatto.class);
		DetachedCriteria criteriaC = criteria.createCriteria("operatore", "op");
		DetachedCriteria criteriaCO = criteria.createCriteria("contatto");
		DetachedCriteria criteriaCA = criteriaCO.createCriteria("campagna", "camp");
		if (ApplicationUtil.gtZero(supervisore)) {
			impostazioneDao.subCriteriaRoleSupervisore(supervisore, criteria, "camp", "op");
			impostazioneDao.subCriteriaSupervisore(supervisore, criteriaC, "id");
		}
		criteriaCO.add(Restrictions.eq("lista.id", lista));
		if(r != null) {
			ApplicationUtil.pannello(r.getPannello(), criteria);
			ApplicationUtil.dataIntervalD(r.getInizio(), r.getFine(), "data", criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, null);
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
			if(ApplicationUtil.ltZero(r.getCampagna())) {
				if(ApplicationUtil.gtZero(supervisore)) {
					if(ApplicationUtil.isTrue(sd)) {
						impostazioneDao.subCriteriaCampagnaSupervisore(supervisore, criteriaCA, "id");
					}
				}
			}
		}
		criteria.add(Restrictions.isNotNull("archiviazione.id"));
		return getListSize(criteria);
	}
	private int getContattiOdierneListaEsitoReport(Long lista, Long supervisore, ReportCriteria r) {
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContatto.class);
		DetachedCriteria criteriaC = criteria.createCriteria("operatore", "op");
		DetachedCriteria criteriaCO = criteria.createCriteria("contatto");
		DetachedCriteria criteriaCA = criteriaCO.createCriteria("campagna", "camp");
		if (ApplicationUtil.gtZero(supervisore)) {
			impostazioneDao.subCriteriaRoleSupervisore(supervisore, criteria, "camp", "op");
			impostazioneDao.subCriteriaSupervisore(supervisore, criteriaC, "id");
		}
		criteriaCO.add(Restrictions.eq("lista.id", lista));
		if(r != null) {
			ApplicationUtil.pannello(r.getPannello(), criteria);
			ApplicationUtil.dataIntervalD(r.getInizio(), r.getFine(), "data", criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, null);
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
			if(ApplicationUtil.ltZero(r.getCampagna())) {
				if(ApplicationUtil.gtZero(supervisore)) {
					if(ApplicationUtil.isTrue(sd)) {
						impostazioneDao.subCriteriaCampagnaSupervisore(supervisore, criteriaCA, "id");
					}
				}
			}
		}
		return getListSize(criteria);
	}
	private int getConversazioniOdierneListaEsitoReport(Long lista, Long supervisore, ReportCriteria r) {
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContatto.class);
		DetachedCriteria criteriaC = criteria.createCriteria("operatore", "op");
		DetachedCriteria criteriaCO = criteria.createCriteria("contatto");
		DetachedCriteria criteriaCA = criteriaCO.createCriteria("campagna", "camp");
		if (ApplicationUtil.gtZero(supervisore)) {
			impostazioneDao.subCriteriaRoleSupervisore(supervisore, criteria, "camp", "op");
			impostazioneDao.subCriteriaSupervisore(supervisore, criteriaC, "id");
		}
		criteriaCO.add(Restrictions.eq("lista.id", lista));
		if(r != null) {
			ApplicationUtil.pannello(r.getPannello(), criteria);
			ApplicationUtil.dataIntervalD(r.getInizio(), r.getFine(), "data", criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, null);
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
			if(ApplicationUtil.ltZero(r.getCampagna())) {
				if(ApplicationUtil.gtZero(supervisore)) {
					if(ApplicationUtil.isTrue(sd)) {
						impostazioneDao.subCriteriaCampagnaSupervisore(supervisore, criteriaCA, "id");
					}
				}
			}
		}
		criteria.add(Restrictions.gt("secondi_conv_eff", 0));
		return getListSize(criteria);
	}
	private int getAppuntamentiOdierneListaEsitoReport(Long lista, Long supervisore, ReportCriteria r) {
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(Appuntamento.class);
		DetachedCriteria criteriaC = criteria.createCriteria("operatore", "op");
		DetachedCriteria criteriaCO = criteria.createCriteria("contatto");
		DetachedCriteria criteriaCA = criteriaCO.createCriteria("campagna", "camp");
		if (ApplicationUtil.gtZero(supervisore)) {
			impostazioneDao.subCriteriaRoleSupervisore(supervisore, criteria, "camp", "op");
			impostazioneDao.subCriteriaSupervisore(supervisore, criteriaC, "id");
		}
		criteriaCO.add(Restrictions.eq("lista.id", lista));
		if(r != null) {
			ApplicationUtil.intervallo(ApplicationUtil.isTrue(r.getInserimento()), r.getInizio(), r.getFine(), criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, null);
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
			if(ApplicationUtil.ltZero(r.getCampagna())) {
				if(ApplicationUtil.gtZero(supervisore)) {
					if(ApplicationUtil.isTrue(sd)) {
						impostazioneDao.subCriteriaCampagnaSupervisore(supervisore, criteriaCA, "id");
					}
				}
			}
		}
		return getListSize(criteria);
	}
	private int getContrattiOdierneListaEsitoReport(Long lista, Long supervisore, ReportCriteria r) {
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(Contratto.class);
		DetachedCriteria criteriaC = criteria.createCriteria("operatore", "op");
		DetachedCriteria criteriaCO = criteria.createCriteria("contatto");
		DetachedCriteria criteriaCA = criteriaCO.createCriteria("campagna", "camp");
		if (ApplicationUtil.gtZero(supervisore)) {
			impostazioneDao.subCriteriaRoleSupervisore(supervisore, criteria, "camp", "op");
			impostazioneDao.subCriteriaSupervisore(supervisore, criteriaC, "id");
		}
		criteriaCO.add(Restrictions.eq("lista.id", lista));
	
		if(r != null) {
			ApplicationUtil.intervallo(ApplicationUtil.isTrue(r.getInserimento()), r.getInizio(), r.getFine(), criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, null);
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
			if(ApplicationUtil.ltZero(r.getCampagna())) {
				if(ApplicationUtil.gtZero(supervisore)) {
					if(ApplicationUtil.isTrue(sd)) {
						impostazioneDao.subCriteriaCampagnaSupervisore(supervisore, criteriaCA, "id");
					}
				}
			}
		}
		return getListSize(criteria);
	}
	@SuppressWarnings( "unchecked")
	private List<ReportLavoratoResultSet> getReportLavoratoResultsetMarcatura(Utente user, ReportCriteria r) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Marcatura.class, "a");
		criteria.add(Restrictions.isNotNull("data_inizio"));
		criteria.add(Restrictions.isNotNull("data_fine"));
		DetachedCriteria criteriaU = criteria.createCriteria("user");
		if(r != null) {
			ApplicationUtil.dataIntervalD(r.getInizio(), r.getFine(), "data_creazione", criteria);
			if(ApplicationUtil.gtZero(r.getOperatore())) {
				criteriaU.add(Restrictions.idEq(r.getOperatore()));
			}
			if(ApplicationUtil.gtZero(r.getTeam())) {
				impostazioneDao.subCriteriaOperatoreTeam(r.getTeam(), criteriaU, "id");
			}
		}
		if(user != null) {
			impostazioneDao.subCriteriaSupervisore(user.getId(), criteriaU, "id");
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("a.user.id").as("user_id")))
				.add(Projections.countDistinct("a.id").as("conteggio"))
				.add(Projections.sqlProjection("sum(TIMESTAMPDIFF(SECOND,{alias}.data_inizio, {alias}.data_fine)) as totale", new String [] {"totale"}, new Type []{LongType.INSTANCE}))
				);
		return getList(criteria, ReportLavoratoResultSet.class);
	}
	@SuppressWarnings( "unchecked")
	private List<ReportLavoratoResultSet> getReportLavoratoResultsetPause(Utente user, ReportCriteria r) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Pausa.class, "a");
		criteria.add(Restrictions.isNotNull("data_inizio"));
		DetachedCriteria criteriaU = criteria.createCriteria("user");
		if(r != null) {
			ApplicationUtil.dataIntervalD(r.getInizio(), r.getFine(), "data_creazione", criteria);
			if(ApplicationUtil.gtZero(r.getOperatore())) {
				criteriaU.add(Restrictions.idEq(r.getOperatore()));
			}
			if(ApplicationUtil.gtZero(r.getTeam())) {
				impostazioneDao.subCriteriaOperatoreTeam(r.getTeam(), criteriaU, "id");
			}
		}
		if(user != null) {
			impostazioneDao.subCriteriaSupervisore(user.getId(), criteriaU, "id");
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("a.user.id").as("user_id")))
				.add(Projections.groupProperty("a.chiave").as("pausa"))
				.add(Projections.groupProperty("a.positivo").as("positivo"))
				.add(Projections.countDistinct("a.id").as("conteggio"))
				.add(Projections.sqlProjection("sum(TIMESTAMPDIFF(SECOND,{alias}.data_inizio, {alias}.data_fine)) as totale", new String [] {"totale"}, new Type []{LongType.INSTANCE}))
				);
		return getList(criteria, ReportLavoratoResultSet.class);
	}
	@SuppressWarnings( "unchecked")
	private List<Pausa> getPause(Utente user, ReportCriteria r) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Pausa.class, "a");
		criteria.add(Restrictions.isNotNull("data_inizio"));
		DetachedCriteria criteriaU = criteria.createCriteria("user");
		if(r != null) {
			ApplicationUtil.dataIntervalD(r.getInizio(), r.getFine(), "data_creazione", criteria);
			if(ApplicationUtil.gtZero(r.getOperatore())) {
				criteriaU.add(Restrictions.idEq(r.getOperatore()));
			}
			if(ApplicationUtil.gtZero(r.getTeam())) {
				impostazioneDao.subCriteriaOperatoreTeam(r.getTeam(), criteriaU, "id");
			}
		}
		if(user != null) {
			impostazioneDao.subCriteriaSupervisore(user.getId(), criteriaU, "id");
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.property("chiave").as("chiave")))
				.add(Projections.property("positivo").as("positivo")));
		return getList(criteria, Pausa.class);
	}
	@SuppressWarnings("unchecked")
	public ReportOrarioInserimento getReportOrarioStoria(Utente user, ReportCriteria r) {
		if(r != null)  r = (ReportCriteria) r.loadCriteria();
		Boolean sd = impostazioneDao.separaDati();
		ReportOrarioInserimento rpo = new ReportOrarioInserimento();
		if(r == null || (r != null && r.getCerca())) {
			if(r != null && ApplicationUtil.isTrue(r.getAppuntamento())) {
				DetachedCriteria criteria = DetachedCriteria.forClass(StoriaAppuntamento.class, "sc");
				DetachedCriteria criteriaO = criteria.createCriteria("utente", "op");
				DetachedCriteria criteriaApp = criteria.createCriteria("appuntamento", "r");
				DetachedCriteria criteriaC = criteriaApp.createCriteria("contatto", "cont");
				DetachedCriteria criteriaCA = criteriaC.createCriteria("campagna", "camp");
				if(user != null) {
					impostazioneDao.subCriteriaRoleSupervisore(user.getId(), criteria, "camp", "op");
					impostazioneDao.subCriteriaSupervisore(user.getId(), criteriaO, "id");
				}
				if(r != null) {
					DetachedCriteria criteriaLI = criteriaC.createCriteria("lista", "lista");
					ApplicationUtil.dataIntervalD(r.getInizio(), r.getFine(), "data", criteria);
					ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, criteriaLI);
					if(ApplicationUtil.gtZero(r.getLista())) {
						criteriaLI.add(Restrictions.idEq(r.getLista()));
					}
					if(ApplicationUtil.gtZero(r.getCampagna())) {
						criteriaCA.add(Restrictions.idEq(r.getCampagna()));
					}
					if(user != null &&ApplicationUtil.ltZero(r.getCampagna())) {
						if(ApplicationUtil.isTrue(sd)) {
							impostazioneDao.subCriteriaCampagnaSupervisore(user.getId(), criteriaCA, "id");
						}
					}
				}
				criteria.setProjection(Projections.projectionList()
						.add(Projections.sqlGroupProjection("EXTRACT(HOUR FROM {alias}.data) as orario", "orario order by orario asc", new String [] {"orario"}, new Type []{IntegerType.INSTANCE}))
						.add(Projections.countDistinct("sc.id").as("numero"))
						); 
				rpo.setCampi(getList(criteria, ReportOrario.class));
			} else {
				DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContratto.class, "sc");
				DetachedCriteria criteriaO = criteria.createCriteria("utente", "op");
				if(user != null) {
					impostazioneDao.subCriteriaSupervisore(user.getId(), criteriaO, "id");
				}
				DetachedCriteria criteriaApp = criteria.createCriteria("contratto", "r");
				DetachedCriteria criteriaC = criteriaApp.createCriteria("contatto", "cont");
				DetachedCriteria criteriaCA = criteriaC.createCriteria("campagna", "camp");
				DetachedCriteria criteriaLI = criteriaC.createCriteria("lista", "lista");
				if(r != null) {
					ApplicationUtil.dataIntervalD(r.getInizio(), r.getFine(), "data", criteria);
					ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, criteriaLI);
					if(ApplicationUtil.gtZero(r.getLista())) {
						criteriaLI.add(Restrictions.idEq(r.getLista()));
					}
					if(ApplicationUtil.gtZero(r.getCampagna())) {
						criteriaCA.add(Restrictions.idEq(r.getCampagna()));
					}
					if(user != null &&ApplicationUtil.ltZero(r.getCampagna())) {
						if(ApplicationUtil.isTrue(sd)) {
							impostazioneDao.subCriteriaCampagnaSupervisore(user.getId(), criteriaCA, "id");
						}
					}
				}
				criteria.setProjection(Projections.projectionList()
						.add(Projections.sqlGroupProjection("EXTRACT(HOUR FROM {alias}.data) as orario", "orario order by orario asc", new String [] {"orario"}, new Type []{IntegerType.INSTANCE}))
						.add(Projections.countDistinct("sc.id").as("numero"))
						);
				rpo.setCampi(getList(criteria, ReportOrario.class));
			}
		}
		return rpo;
	}
	@SuppressWarnings("unchecked")
	public ReportOrarioInserimento getReportOrario(Utente user, ReportCriteria r) {
		if(r != null)  r = (ReportCriteria) r.loadCriteria(); 
		Boolean sd = impostazioneDao.separaDati();
		ReportOrarioInserimento rpo = new ReportOrarioInserimento();
		if(r == null || (r != null && r.getCerca())) {
			if(r != null && ApplicationUtil.isTrue(r.getAppuntamento())) {
				DetachedCriteria criteria = DetachedCriteria.forClass(Appuntamento.class, "sc");
				DetachedCriteria criteriaO = criteria.createCriteria("operatore", "op");
				DetachedCriteria criteriaC = criteria.createCriteria("contatto", "cont");
				DetachedCriteria criteriaCA = criteriaC.createCriteria("campagna", "camp");
				if(user != null) {
					impostazioneDao.subCriteriaRoleSupervisore(user.getId(), criteria, "camp", "op");
					impostazioneDao.subCriteriaSupervisore(user.getId(), criteriaO, "id");
				}
				if(r != null) {
					DetachedCriteria criteriaLI = criteriaC.createCriteria("lista", "lista");
					ApplicationUtil.intervallo(ApplicationUtil.isTrue(r.getInserimento()), r.getInizio(), r.getFine(), criteria);
					ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, criteriaLI);
					if(ApplicationUtil.gtZero(r.getLista())) {
						criteriaLI.add(Restrictions.idEq(r.getLista()));
					}
					if(ApplicationUtil.gtZero(r.getCampagna())) {
						criteriaCA.add(Restrictions.idEq(r.getCampagna()));
					}
					if(user != null && ApplicationUtil.ltZero(r.getCampagna())) {
						if(ApplicationUtil.isTrue(sd)) {
							impostazioneDao.subCriteriaCampagnaSupervisore(user.getId(), criteriaCA, "id");
						}
					}
				}
				String d = ApplicationUtil.isTrue(r.getInserimento()) ? "data_creazione" : "data_lavorazione";
				criteria.setProjection(Projections.projectionList()
						.add(Projections.sqlGroupProjection("EXTRACT(HOUR FROM {alias}."+d+") as orario", "orario order by orario asc", new String [] {"orario"}, new Type []{IntegerType.INSTANCE}))
						.add(Projections.countDistinct("sc.id").as("numero"))
						);
				rpo.setCampi(getList(criteria, ReportOrario.class));
			} else {
				DetachedCriteria criteria = DetachedCriteria.forClass(Contratto.class, "sc");
				DetachedCriteria criteriaO = criteria.createCriteria("operatore", "op");
				if(user != null) {
					impostazioneDao.subCriteriaSupervisore(user.getId(), criteriaO, "id");
				}
				DetachedCriteria criteriaC = criteria.createCriteria("contatto", "cont");
				DetachedCriteria criteriaCA = criteriaC.createCriteria("campagna", "camp");
				DetachedCriteria criteriaLI = criteriaC.createCriteria("lista", "lista");
				if(r != null) {
					ApplicationUtil.intervallo(ApplicationUtil.isTrue(r.getInserimento()), r.getInizio(), r.getFine(), criteria);
					ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, criteriaLI);
					if(ApplicationUtil.gtZero(r.getLista())) {
						criteriaLI.add(Restrictions.idEq(r.getLista()));
					}
					if(ApplicationUtil.gtZero(r.getCampagna())) {
						criteriaCA.add(Restrictions.idEq(r.getCampagna()));
					}
					if(user != null && ApplicationUtil.ltZero(r.getCampagna())) {
						if(ApplicationUtil.isTrue(sd)) {
							impostazioneDao.subCriteriaCampagnaSupervisore(user.getId(), criteriaCA, "id");
						}
					}
				}
				String d = ApplicationUtil.isTrue(r.getInserimento()) ? "data_creazione" : "data_lavorazione";
				criteria.setProjection(Projections.projectionList()
						.add(Projections.sqlGroupProjection("EXTRACT(HOUR FROM {alias}."+d+") as orario", "orario order by orario asc", new String [] {"orario"}, new Type []{IntegerType.INSTANCE}))
						.add(Projections.countDistinct("sc.id").as("numero"))
						);
				rpo.setCampi(getList(criteria, ReportOrario.class));
			}
		}
		return rpo;
	}	 
	public ReportEsitiCampagna getReportEsitiCampagna(ReportCriteria rr) {
		if(rr != null)  rr = (ReportCriteria) rr.loadCriteria(); 
		ReportEsitiCampagna report = new ReportEsitiCampagna();
		if(rr == null || (rr != null && rr.getCerca())) {
			List<RiepilogoEsito> esiti = new ArrayList<RiepilogoEsito>();
			List<RiepilogoEsitoAppoggio> query = getRiepilogoEsiti(rr);
			boolean ok = false, ko = false;
			for(RiepilogoEsitoAppoggio r : query) {
				if(!ApplicationUtil.equals(r.getEid(), ConstantsEsito.ID_OK) || r.getPositivo()!= null) { 
					RiepilogoEsito resito = new RiepilogoEsito(r.getEtichetta(), r.getCodice(), r.getColore());
					if(ApplicationUtil.equals(r.getEid(), ConstantsEsito.ID_OK) && ApplicationUtil.isTrue(r.getPositivo())) {
						ok = true;
						resito.setPositivo(true);
					}
					if(ApplicationUtil.equals(r.getEid(), ConstantsEsito.ID_OK) && ApplicationUtil.isNotTrue(r.getPositivo())) {
						ko = true;
						resito.setPositivo(false);
					} 
					resito.setNumero(r.getNumero());
					esiti.add(resito);
				}
			}
			if(ApplicationUtil.notEmpty(esiti)) {
				if(!ok) esiti.add(new RiepilogoEsito(ConstantsLanguage.OK.stampa(),  null, null,0L, true));
				if(!ko) esiti.add(new RiepilogoEsito(ConstantsLanguage.OK.stampa(),  null, null,0L, false));
				report.setEsiti(esiti);
			}
			List<RiepilogoEsito> stato = new ArrayList<RiepilogoEsito>();
			RiepilogoEsito ea = new RiepilogoEsito(ConstantsLanguage.APERTI.stampa(), null, null, 0L);
			RiepilogoEsito ec = new RiepilogoEsito(ConstantsLanguage.CHIUSI.stampa(), null, null, 0L);
			List<RiepilogoEsitoAppoggio> queryApp = getRiepilogoStati(rr);
			for(RiepilogoEsitoAppoggio rea : queryApp) {
				if(ApplicationUtil.isTrue(rea.getPositivo())) {
					ec.addNumero(rea.getNumero());
				} else {
					ea.addNumero(rea.getNumero());
				}
			}
			if(ApplicationUtil.gtZero(ea.getNumero()) || ApplicationUtil.gtZero(ec.getNumero())) {
				stato.add(ea);
				stato.add(ec);
				report.setStato(stato);
			}
		}
		return report;
	}
	public ReportEsitiCampagna getReportEsitiCampagna(Utente user,  ReportCriteria rr) {
		if(rr != null)  rr = (ReportCriteria) rr.loadCriteria(); 
		ReportEsitiCampagna report = new ReportEsitiCampagna();
		if(rr == null || (rr != null && rr.getCerca())) {
			List<RiepilogoEsito> esiti = new ArrayList<RiepilogoEsito>();
			List<RiepilogoEsitoAppoggio> query = getRiepilogoEsiti(user, rr);
			boolean ok = false, ko = false;
			for(RiepilogoEsitoAppoggio r : query) {
				if(!ApplicationUtil.equals(r.getEid(), ConstantsEsito.ID_OK) || r.getPositivo()!= null) { 
					RiepilogoEsito resito = new RiepilogoEsito(r.getEtichetta(), r.getCodice(), r.getColore());
					if(ApplicationUtil.equals(r.getEid(), ConstantsEsito.ID_OK) && ApplicationUtil.isTrue(r.getPositivo())) {
						ok = true;
						resito.setPositivo(true);
					}
					if(ApplicationUtil.equals(r.getEid(), ConstantsEsito.ID_OK) && ApplicationUtil.isNotTrue(r.getPositivo())) {
						ko = true;
						resito.setPositivo(false);
					}
					resito.setNumero(r.getNumero());
					esiti.add(resito);
				}
			}
			if(ApplicationUtil.notEmpty(esiti)) {
				if(!ok) esiti.add(new RiepilogoEsito(ConstantsLanguage.OK.stampa(), null, null, 0L, true));
				if(!ko) esiti.add(new RiepilogoEsito(ConstantsLanguage.OK.stampa(), null, null, 0L, false));
				report.setEsiti(esiti);
			}
			List<RiepilogoEsito> stato = new ArrayList<RiepilogoEsito>();
			RiepilogoEsito ea = new RiepilogoEsito(ConstantsLanguage.APERTI.stampa(),  null, null,0L);
			RiepilogoEsito ec = new RiepilogoEsito(ConstantsLanguage.CHIUSI.stampa(), null, null, 0L);
			List<RiepilogoEsitoAppoggio> queryApp = getRiepilogoStati(user, rr);
			for(RiepilogoEsitoAppoggio rea : queryApp) {
				if(ApplicationUtil.isTrue(rea.getPositivo())) {
					ec.addNumero(rea.getNumero());
				} else {
					ea.addNumero(rea.getNumero());
				}
			}
			if(ApplicationUtil.gtZero(ea.getNumero()) || ApplicationUtil.gtZero(ec.getNumero())) {
				stato.add(ea);
				stato.add(ec);
				report.setStato(stato);
			}
		}
		return report;
	}
	@SuppressWarnings("unchecked")
	public List<Dettaglio> getReportEsitiArchiviazione(ReportCriteria r) {
		if(r != null)  r = (ReportCriteria) r.loadCriteria(); 
		if(r == null || (r != null && r.getCerca())) {
			DetachedCriteria criteria = DetachedCriteria.forClass(EsitoArchiviazione.class);
			if(r != null) {
				DetachedCriteria criteriaCA = criteria.createCriteria("campagne");
				ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, null);
				if(ApplicationUtil.gtZero(r.getCampagna())) {
					criteriaCA.add(Restrictions.idEq(r.getCampagna()));
				}
			}
			ApplicationUtil.orderEsito(criteria);
			criteria.setProjection(Projections.projectionList()
					.add(Projections.distinct(Projections.property("id").as("esito_id")))
					.add(Projections.property("etichetta").as("etichetta"))
					.add(Projections.property("codice").as("codice"))
					.add(Projections.property("colore").as("colore")));
			return getList(criteria, Dettaglio.class);
		}
		return null;
	}
	public ReportEsiti getReportEsitiStoria(ReportCriteria rr) {
		if(rr != null)  rr = (ReportCriteria) rr.loadCriteria(); 
		ReportEsiti report = new ReportEsiti();		
		if(rr == null || (rr != null && rr.getCerca())) {
			List<RiepilogoEsito> arc = new ArrayList<RiepilogoEsito>();
			List<RiepilogoEsito> arc_utili = new ArrayList<RiepilogoEsito>();
			List<RiepilogoEsito> arc_positivi = new ArrayList<RiepilogoEsito>();
			List<RiepilogoEsito> chiamata = getRiepilogoEsitiChiamataStoria(rr);
			List<RiepilogoEsitoAppoggio> archiviazione = getRiepilogoEsitiArchiviazioneStoria(rr);
			int pos = 0, nonpos = 0, util = 0, inutil = 0; 
			for(RiepilogoEsitoAppoggio rea : archiviazione) {
				arc.add(rea.toRiepilogoEsito());
				if(ApplicationUtil.isTrue(rea.getPositivo()))
					pos = pos + rea.getNumero().intValue();
				else nonpos = nonpos + rea.getNumero().intValue();
				if(ApplicationUtil.isTrue(rea.getUtile())) 
					util = util + rea.getNumero().intValue();
				else inutil = inutil + rea.getNumero().intValue();
			}
			if(ApplicationUtil.gtZero(pos) || ApplicationUtil.gtZero(nonpos)) {
				arc_positivi.add(new RiepilogoEsito(ConstantsLanguage.POSITIVO.stampa(),  null, null,Long.valueOf(pos)));
				arc_positivi.add(new RiepilogoEsito(ConstantsLanguage.NEGATIVO.stampa(),  null, null,Long.valueOf(nonpos)));
			}
			if(ApplicationUtil.gtZero(util) || ApplicationUtil.gtZero(inutil)) {
				arc_utili.add(new RiepilogoEsito(ConstantsLanguage.UTILE.stampa(),  null, null,Long.valueOf(util)));
				arc_utili.add(new RiepilogoEsito(ConstantsLanguage.INUTILE.stampa(), null, null, Long.valueOf(inutil)));
			}
			report.setEsitiChiamata(chiamata);
			report.setEsitiArchiviazione(arc);
			report.setEsitiPositivi(arc_positivi);
			report.setEsitiUtili(arc_utili);
		}
		return report;
	}
	public ReportEsiti getReportEsitiStoria(Utente user,  ReportCriteria rr) {
		if(rr != null)  rr = (ReportCriteria) rr.loadCriteria(); 
		ReportEsiti report = new ReportEsiti();	
		if(rr == null || (rr != null && rr.getCerca())) {
			List<RiepilogoEsito> arc = new ArrayList<RiepilogoEsito>();
			List<RiepilogoEsito> arc_utili = new ArrayList<RiepilogoEsito>();
			List<RiepilogoEsito> arc_positivi = new ArrayList<RiepilogoEsito>();
			List<RiepilogoEsito> chiamata = getRiepilogoEsitiChiamataStoria(user, rr);
			List<RiepilogoEsitoAppoggio> archiviazione = getRiepilogoEsitiArchiviazioneStoria(user, rr);
			int pos = 0, nonpos = 0, util = 0, inutil = 0; 
			for(RiepilogoEsitoAppoggio rea : archiviazione) {
				arc.add(rea.toRiepilogoEsito());
				if(ApplicationUtil.isTrue(rea.getPositivo()))
					pos = pos + rea.getNumero().intValue();
				else nonpos = nonpos + rea.getNumero().intValue();
				if(ApplicationUtil.isTrue(rea.getUtile())) 
					util = util + rea.getNumero().intValue();
				else inutil = inutil + rea.getNumero().intValue();
			}
			if(ApplicationUtil.gtZero(pos) || ApplicationUtil.gtZero(nonpos)) {
				arc_positivi.add(new RiepilogoEsito(ConstantsLanguage.POSITIVO.stampa(), null, null, Long.valueOf(pos)));
				arc_positivi.add(new RiepilogoEsito(ConstantsLanguage.NEGATIVO.stampa(), null, null, Long.valueOf(nonpos)));
			}
			if(ApplicationUtil.gtZero(util) || ApplicationUtil.gtZero(inutil)) {
				arc_utili.add(new RiepilogoEsito(ConstantsLanguage.UTILE.stampa(),  null, null,Long.valueOf(util)));
				arc_utili.add(new RiepilogoEsito(ConstantsLanguage.INUTILE.stampa(),  null, null,Long.valueOf(inutil)));
			}
			report.setEsitiChiamata(chiamata);
			report.setEsitiArchiviazione(arc);
			report.setEsitiPositivi(arc_positivi);
			report.setEsitiUtili(arc_utili);
		}
		return report;
	}
	public ReportEsiti getReportEsitiStoriaPartner(Utente user, ReportCriteria rr) {
		if(rr != null)  rr = (ReportCriteria) rr.loadCriteria(); 
		ReportEsiti report = new ReportEsiti();		
		if(rr == null || (rr != null && rr.getCerca())) {
			List<RiepilogoEsito> arc = new ArrayList<RiepilogoEsito>();
			List<RiepilogoEsito> arc_utili = new ArrayList<RiepilogoEsito>();
			List<RiepilogoEsito> arc_positivi = new ArrayList<RiepilogoEsito>();
			List<RiepilogoEsito> chiamata = getRiepilogoEsitiChiamataStoriaPartner(user, rr);
			List<RiepilogoEsitoAppoggio> archiviazione = getRiepilogoEsitiArchiviazioneStoriaPartner(user, rr);
			int pos = 0, nonpos = 0, util = 0, inutil = 0; 
			for(RiepilogoEsitoAppoggio rea : archiviazione) {
				arc.add(rea.toRiepilogoEsito());
				if(ApplicationUtil.isTrue(rea.getPositivo()))
					pos = pos + rea.getNumero().intValue();
				else nonpos = nonpos + rea.getNumero().intValue();
				if(ApplicationUtil.isTrue(rea.getUtile())) 
					util = util + rea.getNumero().intValue();
				else inutil = inutil + rea.getNumero().intValue();
			}
			if(ApplicationUtil.gtZero(pos) || ApplicationUtil.gtZero(nonpos)) {
				arc_positivi.add(new RiepilogoEsito(ConstantsLanguage.POSITIVO.stampa(), null, null, Long.valueOf(pos)));
				arc_positivi.add(new RiepilogoEsito(ConstantsLanguage.NEGATIVO.stampa(),  null, null,Long.valueOf(nonpos)));
			}
			if(ApplicationUtil.gtZero(util) || ApplicationUtil.gtZero(inutil)) {
				arc_utili.add(new RiepilogoEsito(ConstantsLanguage.UTILE.stampa(),  null, null,Long.valueOf(util)));
				arc_utili.add(new RiepilogoEsito(ConstantsLanguage.INUTILE.stampa(),  null, null,Long.valueOf(inutil)));
			}
			report.setEsitiChiamata(chiamata);
			report.setEsitiArchiviazione(arc);
			report.setEsitiPositivi(arc_positivi);
			report.setEsitiUtili(arc_utili);
		}
		return report;
	}
	public ReportEsiti getReportEsiti(ReportCriteria rr) {
		if(rr != null)  rr = (ReportCriteria) rr.loadCriteria(); 
		ReportEsiti report = new ReportEsiti();	
		if(rr == null || (rr != null && rr.getCerca())) {
			List<RiepilogoEsito> arc = new ArrayList<RiepilogoEsito>();
			List<RiepilogoEsito> arc_utili = new ArrayList<RiepilogoEsito>();
			List<RiepilogoEsito> arc_positivi = new ArrayList<RiepilogoEsito>();
			List<RiepilogoEsito> chiamata = getRiepilogoEsitiChiamata(rr);
			List<RiepilogoEsitoAppoggio> archiviazione = getRiepilogoEsitiArchiviazione(rr);
			int pos = 0, nonpos = 0, util = 0, inutil = 0; 
			for(RiepilogoEsitoAppoggio rea : archiviazione) {
				arc.add(rea.toRiepilogoEsito());
				if(ApplicationUtil.isTrue(rea.getPositivo()))
					pos = pos + rea.getNumero().intValue();
				else nonpos = nonpos + rea.getNumero().intValue();
				if(ApplicationUtil.isTrue(rea.getUtile())) 
					util = util + rea.getNumero().intValue();
				else inutil = inutil + rea.getNumero().intValue();
			}
			if(ApplicationUtil.gtZero(pos) || ApplicationUtil.gtZero(nonpos)) {
				arc_positivi.add(new RiepilogoEsito(ConstantsLanguage.POSITIVO.stampa(), null, null, Long.valueOf(pos)));
				arc_positivi.add(new RiepilogoEsito(ConstantsLanguage.NEGATIVO.stampa(), null, null, Long.valueOf(nonpos)));
			}
			if(ApplicationUtil.gtZero(util) || ApplicationUtil.gtZero(inutil)) {
				arc_utili.add(new RiepilogoEsito(ConstantsLanguage.UTILE.stampa(),  null, null,Long.valueOf(util)));
				arc_utili.add(new RiepilogoEsito(ConstantsLanguage.INUTILE.stampa(), null, null, Long.valueOf(inutil)));
			}
			report.setEsitiChiamata(chiamata);
			report.setEsitiArchiviazione(arc);
			report.setEsitiPositivi(arc_positivi);
			report.setEsitiUtili(arc_utili);
		}
		return report;
	}
	public ReportEsiti getReportEsiti(Utente user, ReportCriteria rr) {
		if(rr != null)  rr = (ReportCriteria) rr.loadCriteria(); 
		ReportEsiti report = new ReportEsiti();		
		if(rr == null || (rr != null && rr.getCerca())) {
			List<RiepilogoEsito> arc = new ArrayList<RiepilogoEsito>();
			List<RiepilogoEsito> arc_utili = new ArrayList<RiepilogoEsito>();
			List<RiepilogoEsito> arc_positivi = new ArrayList<RiepilogoEsito>();
			List<RiepilogoEsito> chiamata = getRiepilogoEsitiChiamata(user, rr);
			List<RiepilogoEsitoAppoggio> archiviazione = getRiepilogoEsitiArchiviazione(user, rr);
			int pos = 0, nonpos = 0, util = 0, inutil = 0; 
			for(RiepilogoEsitoAppoggio rea : archiviazione) {
				arc.add(rea.toRiepilogoEsito());
				if(ApplicationUtil.isTrue(rea.getPositivo()))
					pos = pos + rea.getNumero().intValue();
				else nonpos = nonpos + rea.getNumero().intValue();
				if(ApplicationUtil.isTrue(rea.getUtile())) 
					util = util + rea.getNumero().intValue();
				else inutil = inutil + rea.getNumero().intValue();
			}
			if(ApplicationUtil.gtZero(pos) || ApplicationUtil.gtZero(nonpos)) {
				arc_positivi.add(new RiepilogoEsito(ConstantsLanguage.POSITIVO.stampa(), null, null, Long.valueOf(pos)));
				arc_positivi.add(new RiepilogoEsito(ConstantsLanguage.NEGATIVO.stampa(),  null, null,Long.valueOf(nonpos)));
			}
			if(ApplicationUtil.gtZero(util) || ApplicationUtil.gtZero(inutil)) {
				arc_utili.add(new RiepilogoEsito(ConstantsLanguage.UTILE.stampa(), null, null, Long.valueOf(util)));
				arc_utili.add(new RiepilogoEsito(ConstantsLanguage.INUTILE.stampa(), null, null, Long.valueOf(inutil)));
			}
			report.setEsitiChiamata(chiamata);
			report.setEsitiArchiviazione(arc);
			report.setEsitiPositivi(arc_positivi);
			report.setEsitiUtili(arc_utili);
		}
		return report;
	}
	public ReportEsiti getReportEsitiPartner(Utente user, ReportCriteria rr) {
		if(rr != null)  rr = (ReportCriteria) rr.loadCriteria(); 
		ReportEsiti report = new ReportEsiti();		
		if(rr == null || (rr != null && rr.getCerca())) {
			List<RiepilogoEsito> arc = new ArrayList<RiepilogoEsito>();
			List<RiepilogoEsito> arc_utili = new ArrayList<RiepilogoEsito>();
			List<RiepilogoEsito> arc_positivi = new ArrayList<RiepilogoEsito>();
			List<RiepilogoEsito> chiamata = getRiepilogoEsitiChiamataPartner(user, rr);
			List<RiepilogoEsitoAppoggio> archiviazione = getRiepilogoEsitiArchiviazionePartner(user, rr);
			int pos = 0, nonpos = 0, util = 0, inutil = 0; 
			for(RiepilogoEsitoAppoggio rea : archiviazione) {
				arc.add(rea.toRiepilogoEsito());
				if(ApplicationUtil.isTrue(rea.getPositivo()))
					pos = pos + rea.getNumero().intValue();
				else nonpos = nonpos + rea.getNumero().intValue();
				if(ApplicationUtil.isTrue(rea.getUtile())) 
					util = util + rea.getNumero().intValue();
				else inutil = inutil + rea.getNumero().intValue();
			}
			if(ApplicationUtil.gtZero(pos) || ApplicationUtil.gtZero(nonpos)) {
				arc_positivi.add(new RiepilogoEsito(ConstantsLanguage.POSITIVO.stampa(),  null, null,Long.valueOf(pos)));
				arc_positivi.add(new RiepilogoEsito(ConstantsLanguage.NEGATIVO.stampa(),  null, null,Long.valueOf(nonpos)));
			}
			if(ApplicationUtil.gtZero(util) || ApplicationUtil.gtZero(inutil)) {
				arc_utili.add(new RiepilogoEsito(ConstantsLanguage.UTILE.stampa(),  null, null,Long.valueOf(util)));
				arc_utili.add(new RiepilogoEsito(ConstantsLanguage.INUTILE.stampa(),  null, null,Long.valueOf(inutil)));
			}
			report.setEsitiChiamata(chiamata);
			report.setEsitiArchiviazione(arc);
			report.setEsitiPositivi(arc_positivi);
			report.setEsitiUtili(arc_utili);
		}
		return report;
	}
	public ReportEsiti getReportEsitiAutodialer(ReportCriteria r) {
		if(r != null)  r = (ReportCriteria) r.loadCriteria(); 
		ReportEsiti report = new ReportEsiti();		
		if(r == null || (r != null && r.getCerca())) {
			List<RiepilogoEsito> chiamata = getRiepilogoEsitiChiamataAutodialer(r);
			report.setEsitiChiamata(chiamata);
		}
		return report;
	}
	public ReportEsiti getReportEsitiAutodialer(Utente user, ReportCriteria r) {
		if(r != null)  r = (ReportCriteria) r.loadCriteria(); 
		ReportEsiti report = new ReportEsiti();		
		if(r == null || (r != null && r.getCerca())) {
			List<RiepilogoEsito> chiamata = getRiepilogoEsitiChiamataAutodialer(user, r);
			report.setEsitiChiamata(chiamata);
		}
		return report;
	}
	public ReportEsiti getReportEsitiAutodialerPartner(Utente user,ReportCriteria r) {
		if(r != null)  r = (ReportCriteria) r.loadCriteria(); 
		ReportEsiti report = new ReportEsiti();	
		if(r == null || (r != null && r.getCerca())) {
			List<RiepilogoEsito> chiamata = getRiepilogoEsitiChiamataAutodialerPartner(user, r);
			report.setEsitiChiamata(chiamata);
		}
		return report;
	}
	public ReportEsitiCampagna getReportEsitiCampagnaPartner(Utente user, ReportCriteria rr) {
		if(rr != null)  rr = (ReportCriteria) rr.loadCriteria(); 
		ReportEsitiCampagna report = new ReportEsitiCampagna();
		if(rr == null || (rr != null && rr.getCerca())) {
			List<RiepilogoEsito> esiti = new ArrayList<RiepilogoEsito>();
			List<RiepilogoEsitoAppoggio> query = getRiepilogoEsitiPartner(user, rr);
			boolean ok = false, ko = false;
			for(RiepilogoEsitoAppoggio r : query) {
				if(!ApplicationUtil.equals(r.getEid(), ConstantsEsito.ID_OK) || r.getPositivo()!= null) { 
					RiepilogoEsito resito = new RiepilogoEsito(r.getEtichetta(), r.getCodice(), r.getColore());
					if(ApplicationUtil.equals(r.getEid(), ConstantsEsito.ID_OK) && ApplicationUtil.isTrue(r.getPositivo())) {
						ok = true;
						resito.setPositivo(true);
					}
					if(ApplicationUtil.equals(r.getEid(), ConstantsEsito.ID_OK) && ApplicationUtil.isNotTrue(r.getPositivo())) {
						ko = true;
						resito.setPositivo(false);
					}
					resito.setNumero(r.getNumero());
					esiti.add(resito);
				}
			}
			if(ApplicationUtil.notEmpty(esiti)) {
				if(!ok) esiti.add(new RiepilogoEsito(ConstantsLanguage.OK.stampa(), null, null, 0L, true));
				if(!ko) esiti.add(new RiepilogoEsito(ConstantsLanguage.OK.stampa(), null, null, 0L, false));
				report.setEsiti(esiti);
			}
			List<RiepilogoEsito> stato = new ArrayList<RiepilogoEsito>();
			RiepilogoEsito ea = new RiepilogoEsito(ConstantsLanguage.APERTI.stampa(), null, null, 0L);
			RiepilogoEsito ec = new RiepilogoEsito(ConstantsLanguage.CHIUSI.stampa(), null, null, 0L);
			List<RiepilogoEsitoAppoggio> queryApp = getRiepilogoStatiPartner(user, rr);
			for(RiepilogoEsitoAppoggio rea : queryApp) {
				if(ApplicationUtil.isTrue(rea.getPositivo())) {
					ec.addNumero(rea.getNumero());
				} else {
					ea.addNumero(rea.getNumero());
				}
			}
			if(ApplicationUtil.gtZero(ea.getNumero()) || ApplicationUtil.gtZero(ec.getNumero())) {
				stato.add(ea);
				stato.add(ec);
				report.setStato(stato);
			}
		}
		return report;
	}
	public ReportSondaggio getReportSondaggioAnagrafica(Long utenteId, ReportCriteria r) {
		if(r != null)  r = (ReportCriteria) r.loadCriteria();
		ReportSondaggio report = new ReportSondaggio();	
		if(r == null || (r != null && r.getCerca())) {
			List<RiepilogoSondaggio> chiamata = getReportSondaggioAnagraficaApp(utenteId, r);
			report.setLista(chiamata);
		}
		return report;
	}
	public ReportSondaggio getReportSondaggioAnagraficaPartner(Long utenteId, ReportCriteria r) {
		if(r != null)  r = (ReportCriteria) r.loadCriteria(); 
		ReportSondaggio report = new ReportSondaggio();		
		if(r == null || (r != null && r.getCerca())) {
			List<RiepilogoSondaggio> chiamata = getReportSondaggioAnagraficaAppPartner(utenteId, r);
			report.setLista(chiamata);
		}
		return report;
	}
	public ReportSondaggio getReportSondaggioContratto(Long utenteId, ReportCriteria r) {
		if(r != null)  r = (ReportCriteria) r.loadCriteria(); 
		ReportSondaggio report = new ReportSondaggio();		
		if(r == null || (r != null && r.getCerca())) {
			List<RiepilogoSondaggio> chiamata = getReportSondaggioContrattoApp(utenteId, r);
			report.setLista(chiamata);
		}
		return report;
	}
	public ReportSondaggio getReportSondaggioContrattoPartner(Long utenteId, ReportCriteria r) {
		if(r != null)  r = (ReportCriteria) r.loadCriteria(); 
		ReportSondaggio report = new ReportSondaggio();	
		if(r == null || (r != null && r.getCerca())) {
			List<RiepilogoSondaggio> chiamata = getReportSondaggioContrattoAppPartner(utenteId, r);
			report.setLista(chiamata);
		}
		return report;
	}
	private DetachedCriteria getOperatoreArchiviatiFuturiCriteria(String tipo, Boolean supervisore, Integer t, Long userId, Long campagnaId) {
		Boolean ts = impostazioneDao.getEstrazioneStato();
		Date data = ApplicationUtil.date();
		DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class);
		
		criteria.add(Restrictions.eq("tipo", ConstantsGlobal.inviato));
		DetachedCriteria campagnaC = criteria.createCriteria("campagna");
		if(campagnaId != null) {
			campagnaC.add(Restrictions.idEq(campagnaId));
		} 
		ApplicationUtil.criteriaStato(campagnaC, false, ApplicationUtil.isTrue(ts));
		if(ApplicationUtil.isTrue(supervisore)) {
			impostazioneDao.subCriteriaSupervisore(userId, criteria.createCriteria("proprietario"), "id");
		} else {
			criteria.add(Restrictions.eq("proprietario.id",userId));
		}
		if(ApplicationUtil.isNotBlank(tipo)) {
			criteria.createCriteria("anagrafica").add(ApplicationUtil.isNotNull(tipo));
		}
		Boolean s = impostazioneDao.getEstrazioneListe();
		if(ApplicationUtil.isTrue(s)) {
			DetachedCriteria listaC = criteria.createCriteria("lista");
			ApplicationUtil.criteriaStato(listaC, false, ApplicationUtil.isTrue(ts));
		}
		ApplicationUtil.futuri(criteria, t, data, true);
		return criteria;
	}
	@SuppressWarnings("unchecked")
	public List<Contatto> getOperatoreArchiviatiFuturi(String tipo, Boolean supervisore, Integer t, Long userId, Long campagnaId,  int start, int size ) {
		DetachedCriteria criteria = getOperatoreArchiviatiFuturiCriteria(tipo, supervisore, t, userId, campagnaId);
		criteria.addOrder(Order.asc("data_presentazione"));
		return getList(criteria, start, size);
	}
	public int getOperatoreArchiviatiFuturiSize(String tipo, Boolean supervisore, Integer t, Long userId, Long campagnaId) {
		DetachedCriteria criteria = getOperatoreArchiviatiFuturiCriteria(tipo, supervisore, t, userId, campagnaId);
		return getListSize(criteria);
	}
	private DetachedCriteria getOperatorePrivatiFuturiCriteria(String tipo, Boolean supervisore, Integer t, Long userId, Long campagnaId) {
		Boolean ts = impostazioneDao.getEstrazioneStato();
		DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class);
		Date data = ApplicationUtil.date();
		criteria.add(Restrictions.eq("tipo", ConstantsGlobal.inviato));
		DetachedCriteria campagnaC = criteria.createCriteria("campagna");
		if(campagnaId != null) {
			campagnaC.add(Restrictions.idEq(campagnaId));
		} 
		ApplicationUtil.criteriaStato(campagnaC, false, ApplicationUtil.isTrue(ts));
		if(ApplicationUtil.isTrue(supervisore)) {
			impostazioneDao.subCriteriaSupervisore(userId, criteria.createCriteria("proprietario"), "id");
		} else {
			criteria.add(Restrictions.eq("proprietario.id",userId));
		}
		if(ApplicationUtil.isNotBlank(tipo)) {
			criteria.createCriteria("anagrafica").add(ApplicationUtil.isNotNull(tipo));
		}
		Boolean s = impostazioneDao.getEstrazioneListe();
		if(ApplicationUtil.isTrue(s)) {
			DetachedCriteria listaC = criteria.createCriteria("lista");
			ApplicationUtil.criteriaStato(listaC, false, ApplicationUtil.isTrue(ts));
		}
		ApplicationUtil.futuri(criteria, t, data, false);
		return criteria;
	}
	@SuppressWarnings("unchecked")
	public List<Contatto> getOperatorePrivatiFuturi(String tipo, Boolean supervisore, Integer t, Long userId, Long campagnaId,  int start, int size) {
		DetachedCriteria criteria = getOperatorePrivatiFuturiCriteria(tipo, supervisore, t, userId, campagnaId);
		criteria.addOrder(Order.asc("data_presentazione"));
		return getList(criteria, start, size);
	}
	public int getOperatorePrivatiFuturiSize(String tipo, Boolean supervisore, Integer t, Long userId, Long campagnaId) {
		DetachedCriteria criteria = getOperatorePrivatiFuturiCriteria(tipo, supervisore, t, userId, campagnaId);
		return getListSize(criteria);
	}
	public ReportEsitiCampagna getReportEsitiCampagnaCA(Utente user,  ReportCriteria rr) {
		if(rr != null)  rr = (ReportCriteria) rr.loadCriteria(); 
		ReportEsitiCampagna report = new ReportEsitiCampagna();
		if(rr == null || (rr != null && rr.getCerca())) {
			List<RiepilogoEsito> esiti = new ArrayList<RiepilogoEsito>();
			List<RiepilogoEsitoAppoggio> queryC = getRiepilogoEsitiC(user, rr);
			for(RiepilogoEsitoAppoggio r : queryC) {
				RiepilogoEsito resito = new RiepilogoEsito(r.getEtichetta(), r.getCodice(), r.getColore(), r.getNumero());
				esiti.add(resito);
			}
			List<RiepilogoEsitoAppoggio> queryA = getRiepilogoEsitiA(user, rr);
			for(RiepilogoEsitoAppoggio r : queryA) {
				RiepilogoEsito resito = new RiepilogoEsito(r.getEtichetta(), r.getCodice(), r.getColore(), r.getNumero());
				esiti.add(resito);
			}
			report.setEsiti(esiti);
		}
		return report;
	}
	public ReportEsitiCampagna getReportEsitiCampagnaPartnerCA(Utente user,  ReportCriteria rr) {
		if(rr != null)  rr = (ReportCriteria) rr.loadCriteria(); 
		ReportEsitiCampagna report = new ReportEsitiCampagna();
		if(rr == null || (rr != null && rr.getCerca())) {
			List<RiepilogoEsito> esiti = new ArrayList<RiepilogoEsito>();
			List<RiepilogoEsitoAppoggio> queryC = getRiepilogoEsitiPartnerC(user, rr);
			for(RiepilogoEsitoAppoggio r : queryC) {
				RiepilogoEsito resito = new RiepilogoEsito(r.getEtichetta(), r.getCodice(), r.getColore(), r.getNumero());
				esiti.add(resito);
			}
			List<RiepilogoEsitoAppoggio> queryA = getRiepilogoEsitiPartnerA(user, rr);
			for(RiepilogoEsitoAppoggio r : queryA) {
				RiepilogoEsito resito = new RiepilogoEsito(r.getEtichetta(), r.getCodice(), r.getColore(), r.getNumero());
				esiti.add(resito);
			}
			report.setEsiti(esiti);
		}
		return report;
	}
	public ReportEsitiCampagna getReportEsitiCampagnaStoria(Utente user,  ReportCriteria rr) {
		if(rr != null)  rr = (ReportCriteria) rr.loadCriteria(); 
		ReportEsitiCampagna report = new ReportEsitiCampagna();
		if(rr == null || (rr != null && rr.getCerca())) {
			List<RiepilogoEsito> esiti = new ArrayList<RiepilogoEsito>();
			List<RiepilogoEsitoAppoggio> query = getRiepilogoEsitiStoria(user, rr);
			boolean ok = false, ko = false;
			for(RiepilogoEsitoAppoggio r : query) {
				if(!ApplicationUtil.equals(r.getEid(), ConstantsEsito.ID_OK) || r.getPositivo()!= null) { 
					RiepilogoEsito resito = new RiepilogoEsito(r.getEtichetta(), r.getCodice(), r.getColore());
					if(ApplicationUtil.equals(r.getEid(), ConstantsEsito.ID_OK) && ApplicationUtil.isTrue(r.getPositivo())) {
						ok = true;
						resito.setPositivo(true);
					}
					if(ApplicationUtil.equals(r.getEid(), ConstantsEsito.ID_OK) && ApplicationUtil.isNotTrue(r.getPositivo())) {
						ko = true;
						resito.setPositivo(false);
					}
					resito.setNumero(r.getNumero());
					esiti.add(resito);
				}
			}
			if(ApplicationUtil.notEmpty(esiti)) {
				if(!ok) esiti.add(new RiepilogoEsito(ConstantsLanguage.OK.stampa(), null, null, 0L, true));
				if(!ko) esiti.add(new RiepilogoEsito(ConstantsLanguage.OK.stampa(), null, null, 0L, false));
				report.setEsiti(esiti);
			}
			List<RiepilogoEsito> stato = new ArrayList<RiepilogoEsito>();
			RiepilogoEsito ea = new RiepilogoEsito(ConstantsLanguage.APERTI.stampa(), null, null, 0L);
			RiepilogoEsito ec = new RiepilogoEsito(ConstantsLanguage.CHIUSI.stampa(), null, null, 0L);
			List<RiepilogoEsitoAppoggio> queryApp = getRiepilogoStatiStoria(user, rr);
			for(RiepilogoEsitoAppoggio rea : queryApp) {
				if(ApplicationUtil.isTrue(rea.getPositivo())) {
					ec.addNumero(rea.getNumero());
				} else {
					ea.addNumero(rea.getNumero());
				}
			}
			if(ApplicationUtil.gtZero(ea.getNumero()) || ApplicationUtil.gtZero(ec.getNumero())) {
				stato.add(ea);
				stato.add(ec);
				report.setStato(stato);
			}
		}
		return report;
	}
	public ReportEsitiCampagna getReportEsitiCampagnaStoriaCA(Utente user,  ReportCriteria rr) {
		if(rr != null)  rr = (ReportCriteria) rr.loadCriteria(); 
		ReportEsitiCampagna report = new ReportEsitiCampagna();
		if(rr == null || (rr != null && rr.getCerca())) {
			List<RiepilogoEsito> esiti = new ArrayList<RiepilogoEsito>();
			List<RiepilogoEsitoAppoggio> queryC = getRiepilogoEsitiStoriaC(user, rr);
			for(RiepilogoEsitoAppoggio r : queryC) {
				RiepilogoEsito resito = new RiepilogoEsito(r.getEtichetta(), r.getCodice(), r.getColore(), r.getNumero());
				esiti.add(resito);
			}
			List<RiepilogoEsitoAppoggio> queryA = getRiepilogoEsitiStoriaA(user, rr);
			for(RiepilogoEsitoAppoggio r : queryA) {
				RiepilogoEsito resito = new RiepilogoEsito(r.getEtichetta(), r.getCodice(), r.getColore(), r.getNumero());
				esiti.add(resito);
			}
			report.setEsiti(esiti);
		}
		return report;
	}
	public ReportEsitiCampagna getReportEsitiCampagnaStoriaPartner(Utente user,  ReportCriteria rr) {
		if(rr != null)  rr = (ReportCriteria) rr.loadCriteria(); 
		ReportEsitiCampagna report = new ReportEsitiCampagna();
		if(rr == null || (rr != null && rr.getCerca())) {
			List<RiepilogoEsito> esiti = new ArrayList<RiepilogoEsito>();
			List<RiepilogoEsitoAppoggio> query = getRiepilogoEsitiStoriaPartner(user, rr);
			boolean ok = false, ko = false;
			for(RiepilogoEsitoAppoggio r : query) {
				if(!ApplicationUtil.equals(r.getEid(), ConstantsEsito.ID_OK) || r.getPositivo()!= null) { 
					RiepilogoEsito resito = new RiepilogoEsito(r.getEtichetta(), r.getCodice(), r.getColore());
					if(ApplicationUtil.equals(r.getEid(), ConstantsEsito.ID_OK) && ApplicationUtil.isTrue(r.getPositivo())) {
						ok = true;
						resito.setPositivo(true);
					}
					if(ApplicationUtil.equals(r.getEid(), ConstantsEsito.ID_OK) && ApplicationUtil.isNotTrue(r.getPositivo())) {
						ko = true;
						resito.setPositivo(false);
					}
					resito.setNumero(r.getNumero());
					esiti.add(resito);
				}
			}
			if(ApplicationUtil.notEmpty(esiti)) {
				if(!ok) esiti.add(new RiepilogoEsito(ConstantsLanguage.OK.stampa(), null, null, 0L, true));
				if(!ko) esiti.add(new RiepilogoEsito(ConstantsLanguage.OK.stampa(),  null, null, 0L, false));
				report.setEsiti(esiti);
			}
			List<RiepilogoEsito> stato = new ArrayList<RiepilogoEsito>();
			RiepilogoEsito ea = new RiepilogoEsito(ConstantsLanguage.APERTI.stampa(), null, null, 0L);
			RiepilogoEsito ec = new RiepilogoEsito(ConstantsLanguage.CHIUSI.stampa(), null, null, 0L);
			List<RiepilogoEsitoAppoggio> queryApp = getRiepilogoStatiStoriaPartner(user, rr);
			for(RiepilogoEsitoAppoggio rea : queryApp) {
				if(ApplicationUtil.isTrue(rea.getPositivo())) {
					ec.addNumero(rea.getNumero());
				} else {
					ea.addNumero(rea.getNumero());
				}
			}
			if(ApplicationUtil.gtZero(ea.getNumero()) || ApplicationUtil.gtZero(ec.getNumero())) {
				stato.add(ea);
				stato.add(ec);
				report.setStato(stato);
			}
		}
		return report;
	}
	public ReportEsitiCampagna getReportEsitiCampagnaStoriaPartnerCA(Utente user,  ReportCriteria rr) {
		if(rr != null)  rr = (ReportCriteria) rr.loadCriteria(); 
		ReportEsitiCampagna report = new ReportEsitiCampagna();
		if(rr == null || (rr != null && rr.getCerca())) {
			List<RiepilogoEsito> esiti = new ArrayList<RiepilogoEsito>();
			List<RiepilogoEsitoAppoggio> queryC = getRiepilogoEsitiStoriaPartnerC(user, rr);
			for(RiepilogoEsitoAppoggio r : queryC) {
				RiepilogoEsito resito = new RiepilogoEsito(r.getEtichetta(), r.getCodice(), r.getColore(), r.getNumero());
				esiti.add(resito);
			}
			List<RiepilogoEsitoAppoggio> queryA = getRiepilogoEsitiStoriaPartnerA(user, rr);
			for(RiepilogoEsitoAppoggio r : queryA) {
				RiepilogoEsito resito = new RiepilogoEsito(r.getEtichetta(), r.getCodice(), r.getColore(), r.getNumero());
				esiti.add(resito);
			}
			report.setEsiti(esiti);
		}
		return report;
	}
	public List<ReportGenerale> getReportGeneraleStoria(Utente user, ReportCriteria r) {
		if(r != null)  r = (ReportCriteria) r.loadCriteria();
		List<ReportGenerale> report = new ArrayList<ReportGenerale>();
		if(r == null || (r != null && r.getCerca())) {
			List<ReportGeneraleResultSet> operatori = getReportUtenti(user, r);
			List<ReportGeneraleResultSet> contatti = getReportGeneraleContattoStoriaResultset(user, r);
			List<ReportGeneraleResultSet> appuntamenti = getReportGeneraleAppuntamentoStoriaResultset(user, r);
			List<ReportGeneraleResultSet> contratti = getReportGeneraleContrattoStoriaResultset(user, r);
			for(ReportGeneraleResultSet operatore : operatori) {
				boolean ok = false;
				ReportGenerale ro = new ReportGenerale();
				int a = contatti.indexOf(new ReportGeneraleResultSet(operatore.getUser_id()));
				if(a >= 0) {
					ReportGeneraleResultSet rs = contatti.get(a);
					if(!ok) {
						ok = ApplicationUtil.gtZero(rs.getTotali());
					}
					ro.setContatti(rs);
				} else {
					ro.setContatti(new ReportGeneraleResultSet());
				}
				int b = appuntamenti.indexOf(new ReportGeneraleResultSet(operatore.getUser_id()));
				if(b >= 0) {
					ReportGeneraleResultSet rs = appuntamenti.get(b);
					if(!ok) {
						ok = ApplicationUtil.gtZero(rs.getTotali());
					}
					ro.setAppuntamenti(rs);
				} else {
					ro.setAppuntamenti(new ReportGeneraleResultSet());
				}
				int c = contratti.indexOf(new ReportGeneraleResultSet(operatore.getUser_id()));
				if(c >= 0) {
					ReportGeneraleResultSet rs = contratti.get(c);
					if(!ok) {
						ok = ApplicationUtil.gtZero(rs.getTotali());
					}
					ro.setContratti(rs);
				} else {
					ro.setContratti(new ReportGeneraleResultSet());
				}
				if(ok) {
					Utente utente = userDao.getUser(operatore.getUser_id());
					ro.setOperatore(utente);
					report.add(ro);
				}
			}
		}
		return report;
	}
	public void exportReportGeneraleStoria(Utente user, ReportCriteria r, DownloadBean downloadBean) {
		//if(r != null)  r = (ReportCriteria) r.loadCriteria();
		if(r == null || (r != null && r.getCerca())) {
			String [] titles = new ExportUtility().titlesGeneraleStoria;
			Workbook wb = ApplicationUtil.workbookS();
			ExcelStyle s = new ExcelStyle(wb);
			Sheet sheet = wb.createSheet(ConstantsLanguage.REPORT.stampa() + " " + ConstantsLanguage.GENERALE.stampa());
			int k = 0;
			Row row = sheet.createRow(k++);
			for(int i = 0; i < titles.length; i++) {
				Cell cell = row.createCell(i, CellType.STRING);
				ApplicationUtil.headerCell(s, cell,  titles[i]);
			}
			sheet.setAutoFilter(new CellRangeAddress(0, 0, 0, titles.length-1));
			sheet.createFreezePane(0, 1);
			List<ReportGenerale> data = getReportGeneraleStoria(user, r);
			Iterator<ReportGenerale> objIter = data.iterator();
			while(objIter.hasNext()) {
				ReportGenerale dettaglio = objIter.next();
				row = sheet.createRow(k++);
				ExcelFormatList lista = new ExcelFormatList();
				lista.addValore(dettaglio.getOperatore().getId().toString());
				lista.addValore(dettaglio.getOperatore().getDenominazione());
				lista.addValore(dettaglio.getContatti().getTotali());
				lista.addValore(dettaglio.getContatti().getChiusi());
				lista.addValore(dettaglio.getContatti().getArchiviazioni());
				lista.addValore(dettaglio.getContatti().getPositivi());
				lista.addValore(dettaglio.getContatti().getUtili());
				lista.addValore(dettaglio.getContatti().getConversazioni());
				lista.addValore(dettaglio.getContatti().getDurata());
				lista.addValore(dettaglio.getContatti().getDurata_conv_eff());
				lista.addValore(dettaglio.getContatti().getDurata_conv_tot());
				lista.addValore(dettaglio.getContatti().getDurata_post());
				lista.addValore(dettaglio.getContatti().getDurata_pre());
				lista.addValore(dettaglio.getContatti().getDurata_rec());
				lista.addValore(dettaglio.getContatti().getDurata_attesa());
				lista.addValore(ConstantsFormat.seconds.stampa(), dettaglio.getContatti().getDurata());
				lista.addValore(ConstantsFormat.seconds.stampa(), dettaglio.getContatti().getDurata_conv_eff());
				lista.addValore(ConstantsFormat.seconds.stampa(), dettaglio.getContatti().getDurata_conv_tot());
				lista.addValore(ConstantsFormat.seconds.stampa(), dettaglio.getContatti().getDurata_post());
				lista.addValore(ConstantsFormat.seconds.stampa(), dettaglio.getContatti().getDurata_pre());
				lista.addValore(ConstantsFormat.seconds.stampa(), dettaglio.getContatti().getDurata_rec());
				lista.addValore(ConstantsFormat.seconds.stampa(), dettaglio.getContatti().getDurata_attesa());
				lista.addValore(dettaglio.getContatti().getMedia());
				lista.addValore(dettaglio.getContatti().getMedia_conv_eff());
				lista.addValore(dettaglio.getContatti().getMedia_conv_tot());
				lista.addValore(dettaglio.getContatti().getMedia_post());
				lista.addValore(dettaglio.getContatti().getMedia_pre());
				lista.addValore(dettaglio.getContatti().getMedia_rec());
				lista.addValore(dettaglio.getContatti().getMedia_attesa());
				lista.addValore(ConstantsFormat.seconds.stampa(), dettaglio.getContatti().getMedia());
				lista.addValore(ConstantsFormat.seconds.stampa(), dettaglio.getContatti().getMedia_conv_eff());
				lista.addValore(ConstantsFormat.seconds.stampa(), dettaglio.getContatti().getMedia_conv_tot());
				lista.addValore(ConstantsFormat.seconds.stampa(), dettaglio.getContatti().getMedia_post());
				lista.addValore(ConstantsFormat.seconds.stampa(), dettaglio.getContatti().getMedia_pre());
				lista.addValore(ConstantsFormat.seconds.stampa(), dettaglio.getContatti().getMedia_rec());	
				lista.addValore(ConstantsFormat.seconds.stampa(), dettaglio.getContatti().getMedia_attesa());
				lista.addValore(dettaglio.getAppuntamenti().getTotali());
				lista.addValore(dettaglio.getAppuntamenti().getConfermati());
				lista.addValore(dettaglio.getAppuntamenti().getUtili());
				lista.addValore(dettaglio.getContratti().getTotali());
				lista.addValore(dettaglio.getContratti().getConfermati());
				lista.addValore(dettaglio.getContratti().getUtili());
				for(int j = 0; j < titles.length; j++) {
					ExcelFormat el = lista.get(j);
					Cell cell = row.createCell(j, el.getCellType());
					ApplicationUtil.cellStamper(s, cell, el);
				}
			}
			sheet.setDefaultColumnWidth(20);
			try {
				String nome = ApplicationUtil.formatIntervallo(ConstantsLanguage.REPORT.stampa() + "_" + ConstantsLanguage.GENERALE.stampa(), r.getInizio(), r.getFine()) + ConstantsMap.xlsx; 
				OutputStream out = downloadBean.outputStream();
				downloadBean.filename(nome);

				String p = ConstantsGlobal.secretE;
				ApplicationUtil.passwordXLS(wb, out, p);
				downloadBean.close();

			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		}
	}
	public List<ReportGenerale> getReportGenerale(Utente user, ReportCriteria r) {
		if(r != null)  r = (ReportCriteria) r.loadCriteria(); 
		List<ReportGenerale> report = new ArrayList<ReportGenerale>();
		if(r == null || (r != null && r.getCerca())) {
			List<ReportGeneraleResultSet> operatori = getReportUtenti(user, r);
			List<ReportGeneraleResultSet> contatti = getReportGeneraleContattoResultset(user, r);
			List<ReportGeneraleResultSet> appuntamenti = getReportGeneraleAppuntamentoResultset(user, r);
			List<ReportGeneraleResultSet> contratti = getReportGeneraleContrattoResultset(user, r);
			for(ReportGeneraleResultSet operatore : operatori) {
				boolean ok = false;
				ReportGenerale ro = new ReportGenerale();
				int a = contatti.indexOf(new ReportGeneraleResultSet(operatore.getUser_id()));
				if(a >= 0) {
					ReportGeneraleResultSet rs = contatti.get(a);
					if(!ok) {
						ok = ApplicationUtil.gtZero(rs.getTotali());
					}
					ro.setContatti(rs);
				} else {
					ro.setContatti(new ReportGeneraleResultSet());
				}
				int b = appuntamenti.indexOf(new ReportGeneraleResultSet(operatore.getUser_id()));
				if(b >= 0) {
					ReportGeneraleResultSet rs = appuntamenti.get(b);
					if(!ok) {
						ok = ApplicationUtil.gtZero(rs.getTotali());
					}
					ro.setAppuntamenti(rs);
				} else {
					ro.setAppuntamenti(new ReportGeneraleResultSet());
				}
				int c = contratti.indexOf(new ReportGeneraleResultSet(operatore.getUser_id()));
				if(c >= 0) {
					ReportGeneraleResultSet rs = contratti.get(c);
					if(!ok) {
						ok = ApplicationUtil.gtZero(rs.getTotali());
					}
					ro.setContratti(rs);
				} else {
					ro.setContratti(new ReportGeneraleResultSet());
				}
				if(ok) {
					Utente utente = userDao.getUser(operatore.getUser_id());
					ro.setOperatore(utente);
					report.add(ro);
				}
			}
		}
		return report;
	}
	public void exportReportGenerale(Integer tipo, OutputStream result) throws Exception {
		Date dataInizio = ApplicationUtil.dataInizio(tipo, ApplicationUtil.date());
		ReportCriteria r = new ReportCriteria();
		r.setIntervallo(dataInizio, null);
		String [] titles = new ExportUtility().titlesGenerale;
		Workbook wb = ApplicationUtil.workbookS();
		ExcelStyle s = new ExcelStyle(wb);
		Sheet sheet = wb.createSheet(ConstantsLanguage.RIEPILOGO.stampa() + " " + ConstantsLanguage.GENERALE.stampa());
		int k = 0;
		Row row = sheet.createRow(k++);
		for(int i = 0; i < titles.length; i++) {
			Cell cell = row.createCell(i, CellType.STRING);
			ApplicationUtil.headerCell(s, cell,  titles[i]);
		}
		sheet.setAutoFilter(new CellRangeAddress(0, 0, 0, titles.length-1));
		sheet.createFreezePane(0, 1);
		List<ReportGenerale> data = getReportGenerale(null, r);
		Iterator<ReportGenerale> objIter = data.iterator();
		while(objIter.hasNext()) {
			ReportGenerale dettaglio = objIter.next();
			row = sheet.createRow(k++);
			ExcelFormatList lista = new ExcelFormatList();
			lista.addValore(dettaglio.getOperatore().getId().toString());
			lista.addValore(dettaglio.getOperatore().getDenominazione());
			lista.addValore(dettaglio.getContatti().getTotali());
			lista.addValore(dettaglio.getContatti().getChiusi());
			lista.addValore(dettaglio.getContatti().getArchiviazioni());
			lista.addValore(dettaglio.getContatti().getPositivi());
			lista.addValore(dettaglio.getContatti().getUtili());
			lista.addValore(dettaglio.getAppuntamenti().getTotali());
			lista.addValore(dettaglio.getAppuntamenti().getConfermati());
			lista.addValore(dettaglio.getAppuntamenti().getUtili());
			lista.addValore(dettaglio.getContratti().getTotali());
			lista.addValore(dettaglio.getContratti().getConfermati());
			lista.addValore(dettaglio.getContratti().getUtili());
			for(int j = 0; j < titles.length; j++) {
				ExcelFormat el = lista.get(j);
				Cell cell = row.createCell(j, el.getCellType());
				ApplicationUtil.cellStamper(s, cell, el);
			}
		}
		sheet.setDefaultColumnWidth(20);
		String p = ConstantsGlobal.secretE;
		ApplicationUtil.passwordXLS(wb, result, p);
	}
	public void exportReportGeneraleStoria(Integer tipo, OutputStream result) throws Exception {
		Date dataInizio = ApplicationUtil.dataInizio(tipo, ApplicationUtil.date());
		ReportCriteria r = new ReportCriteria();
		r.setIntervallo(dataInizio, null);
		String [] titles = new ExportUtility().titlesGeneraleStoria;
		Workbook wb = ApplicationUtil.workbookS();
		ExcelStyle s = new ExcelStyle(wb);
		Sheet sheet = wb.createSheet(ConstantsLanguage.REPORT.stampa() + " " + ConstantsLanguage.GENERALE.stampa());
		int k = 0;
		Row row = sheet.createRow(k++);
		for(int i = 0; i < titles.length; i++) {
			Cell cell = row.createCell(i, CellType.STRING);
			ApplicationUtil.headerCell(s, cell,  titles[i]);
		}
		sheet.setAutoFilter(new CellRangeAddress(0, 0, 0, titles.length-1));
		sheet.createFreezePane(0, 1);
		List<ReportGenerale> data = getReportGeneraleStoria(null, r);
		Iterator<ReportGenerale> objIter = data.iterator();
		while(objIter.hasNext()) {
			ReportGenerale dettaglio = objIter.next();
			row = sheet.createRow(k++);
			ExcelFormatList lista = new ExcelFormatList();
			lista.addValore(dettaglio.getOperatore().getId().toString());
			lista.addValore(dettaglio.getOperatore().getDenominazione());
			lista.addValore(dettaglio.getContatti().getTotali());
			lista.addValore(dettaglio.getContatti().getChiusi());
			lista.addValore(dettaglio.getContatti().getArchiviazioni());
			lista.addValore(dettaglio.getContatti().getPositivi());
			lista.addValore(dettaglio.getContatti().getUtili());
			lista.addValore(dettaglio.getContatti().getConversazioni());
			lista.addValore(dettaglio.getContatti().getDurata());
			lista.addValore(dettaglio.getContatti().getDurata_conv_eff());
			lista.addValore(dettaglio.getContatti().getDurata_conv_tot());
			lista.addValore(dettaglio.getContatti().getDurata_post());
			lista.addValore(dettaglio.getContatti().getDurata_pre());
			lista.addValore(dettaglio.getContatti().getDurata_rec());
			lista.addValore(dettaglio.getContatti().getDurata_attesa());
			lista.addValore(ConstantsFormat.seconds.stampa(), dettaglio.getContatti().getDurata());
			lista.addValore(ConstantsFormat.seconds.stampa(), dettaglio.getContatti().getDurata_conv_eff());
			lista.addValore(ConstantsFormat.seconds.stampa(), dettaglio.getContatti().getDurata_conv_tot());
			lista.addValore(ConstantsFormat.seconds.stampa(), dettaglio.getContatti().getDurata_post());
			lista.addValore(ConstantsFormat.seconds.stampa(), dettaglio.getContatti().getDurata_pre());
			lista.addValore(ConstantsFormat.seconds.stampa(), dettaglio.getContatti().getDurata_rec());
			lista.addValore(ConstantsFormat.seconds.stampa(), dettaglio.getContatti().getDurata_attesa());
			lista.addValore(dettaglio.getContatti().getMedia());
			lista.addValore(dettaglio.getContatti().getMedia_conv_eff());
			lista.addValore(dettaglio.getContatti().getMedia_conv_tot());
			lista.addValore(dettaglio.getContatti().getMedia_post());
			lista.addValore(dettaglio.getContatti().getMedia_pre());
			lista.addValore(dettaglio.getContatti().getMedia_rec());
			lista.addValore(dettaglio.getContatti().getMedia_attesa());
			lista.addValore(ConstantsFormat.seconds.stampa(), dettaglio.getContatti().getMedia());
			lista.addValore(ConstantsFormat.seconds.stampa(), dettaglio.getContatti().getMedia_conv_eff());
			lista.addValore(ConstantsFormat.seconds.stampa(), dettaglio.getContatti().getMedia_conv_tot());
			lista.addValore(ConstantsFormat.seconds.stampa(), dettaglio.getContatti().getMedia_post());
			lista.addValore(ConstantsFormat.seconds.stampa(), dettaglio.getContatti().getMedia_pre());
			lista.addValore(ConstantsFormat.seconds.stampa(), dettaglio.getContatti().getMedia_rec());	
			lista.addValore(ConstantsFormat.seconds.stampa(), dettaglio.getContatti().getMedia_attesa());
			lista.addValore(dettaglio.getAppuntamenti().getTotali());
			lista.addValore(dettaglio.getAppuntamenti().getConfermati());
			lista.addValore(dettaglio.getAppuntamenti().getUtili());
			lista.addValore(dettaglio.getContratti().getTotali());
			lista.addValore(dettaglio.getContratti().getConfermati());
			lista.addValore(dettaglio.getContratti().getUtili());
			for(int j = 0; j < titles.length; j++) {
				ExcelFormat el = lista.get(j);
				Cell cell = row.createCell(j, el.getCellType());
				ApplicationUtil.cellStamper(s, cell, el);
			}
		}
		sheet.setDefaultColumnWidth(20);
		String p = ConstantsGlobal.secretE;
		ApplicationUtil.passwordXLS(wb, result, p);
	}
	public void exportReportGenerale(Utente user, ReportCriteria r, DownloadBean downloadBean) {
		//if(r != null)  r = (ReportCriteria) r.loadCriteria();
		if(r == null || (r != null && r.getCerca())) {
			String [] titles = new ExportUtility().titlesGenerale;
			Workbook wb = ApplicationUtil.workbookS();
			ExcelStyle s = new ExcelStyle(wb);
			Sheet sheet = wb.createSheet(ConstantsLanguage.RIEPILOGO.stampa() + " " + ConstantsLanguage.GENERALE.stampa());
			int k = 0;
			Row row = sheet.createRow(k++);
			for(int i = 0; i < titles.length; i++) {
				Cell cell = row.createCell(i, CellType.STRING);
				ApplicationUtil.headerCell(s, cell,  titles[i]);
			}
			sheet.setAutoFilter(new CellRangeAddress(0, 0, 0, titles.length-1));
			sheet.createFreezePane(0, 1);
			List<ReportGenerale> data = getReportGenerale(user, r);
			Iterator<ReportGenerale> objIter = data.iterator();
			while(objIter.hasNext()) {
				ReportGenerale dettaglio = objIter.next();
				row = sheet.createRow(k++);
				ExcelFormatList lista = new ExcelFormatList();
				lista.addValore(dettaglio.getOperatore().getId().toString());
				lista.addValore(dettaglio.getOperatore().getDenominazione());
				lista.addValore(dettaglio.getContatti().getTotali());
				lista.addValore(dettaglio.getContatti().getChiusi());
				lista.addValore(dettaglio.getContatti().getArchiviazioni());
				lista.addValore(dettaglio.getContatti().getPositivi());
				lista.addValore(dettaglio.getContatti().getUtili());
				lista.addValore(dettaglio.getAppuntamenti().getTotali());
				lista.addValore(dettaglio.getAppuntamenti().getConfermati());
				lista.addValore(dettaglio.getAppuntamenti().getUtili());
				lista.addValore(dettaglio.getContratti().getTotali());
				lista.addValore(dettaglio.getContratti().getConfermati());
				lista.addValore(dettaglio.getContratti().getUtili());
				for(int j = 0; j < titles.length; j++) {
					ExcelFormat el = lista.get(j);
					Cell cell = row.createCell(j, el.getCellType());
					ApplicationUtil.cellStamper(s, cell, el);
				}
			}
			sheet.setDefaultColumnWidth(20);
			try {
				String nome = ApplicationUtil.formatIntervallo(ConstantsLanguage.RIEPILOGO.stampa() + "_" + ConstantsLanguage.GENERALE.stampa(), r.getInizio(), r.getFine()) + ConstantsMap.xlsx; 

				OutputStream out = downloadBean.outputStream();
				downloadBean.filename(nome);

				String p = ConstantsGlobal.secretE;
				ApplicationUtil.passwordXLS(wb, out, p);
				downloadBean.close();

			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		}
	}
	public List<ReportLista> getReportLista(Utente user, ReportCriteria r) {
		if(r != null)  r = (ReportCriteria) r.loadCriteria(); 
		List<ReportLista> report = new ArrayList<ReportLista>();
		if(r == null || (r != null && r.getCerca())) {
			List<Lista> listes = new ArrayList<Lista>();
			List<ReportListaResultSet> liste = getReportListe(user, r);
			List<ReportListaResultSet> contatti = getReportListaContattoResultset(user, r);
			List<ReportListaResultSet> appuntamenti = getReportListaAppuntamentoResultset(user, r);
			List<ReportListaResultSet> contratti = getReportListaContrattoResultset(user, r);
			for(ReportListaResultSet lista : liste) {
				boolean ok = false;
				ReportLista ro = new ReportLista();
				int a = contatti.indexOf(new ReportListaResultSet(lista.getLista_id()));
				if(a >= 0) {
					ReportListaResultSet rs = contatti.get(a);
					if(!ok) {
						ok = ApplicationUtil.gtZero(rs.getTotali());
					}
					ro.setContatti(rs);
				} else {
					ro.setContatti(new ReportListaResultSet());
				}
				int b = appuntamenti.indexOf(new ReportListaResultSet(lista.getLista_id()));
				if(b >= 0) {
					ReportListaResultSet rs = appuntamenti.get(b);
					if(!ok) {
						ok = ApplicationUtil.gtZero(rs.getTotali());
					}
					ro.setAppuntamenti(rs);
				} else {
					ro.setAppuntamenti(new ReportListaResultSet());
				}
				int c = contratti.indexOf(new ReportListaResultSet(lista.getLista_id()));
				if(c >= 0) {
					ReportListaResultSet rs = contratti.get(c);
					if(!ok) {
						ok = ApplicationUtil.gtZero(rs.getTotali());
					}
					ro.setContratti(rs);
				} else {
					ro.setContratti(new ReportListaResultSet());
				}
				if(ok) {
					Lista ls = (Lista) campagnaDao.get(Lista.class, lista.getLista_id());
					listes.add(ls);
					ro.setLista(ls);
					report.add(ro);
				}
			}
			impostaConteggioLista(listes, r!= null ? r.getCampagna() : null);
		}
		return report;
	}
	public List<ReportLista> getReportListaStoria(Utente user, ReportCriteria r) {
		if(r != null)  r = (ReportCriteria) r.loadCriteria(); 
		List<ReportLista> report = new ArrayList<ReportLista>();
		if(r == null || (r != null && r.getCerca())) {
			List<Lista> listes = new ArrayList<Lista>();
			List<ReportListaResultSet> liste = getReportListe(user, r);
			List<ReportListaResultSet> contatti = getReportListaContattoStoriaResultset(user, r);
			List<ReportListaResultSet> appuntamenti = getReportListaAppuntamentoStoriaResultset(user, r);
			List<ReportListaResultSet> contratti = getReportListaContrattoStoriaResultset(user, r);
			for(ReportListaResultSet lista : liste) {
				boolean ok = false;
				ReportLista ro = new ReportLista();
				int a = contatti.indexOf(new ReportListaResultSet(lista.getLista_id()));
				if(a >= 0) {
					ReportListaResultSet rs = contatti.get(a);
					if(!ok) {
						ok = ApplicationUtil.gtZero(rs.getTotali());
					}
					ro.setContatti(rs);
				} else {
					ro.setContatti(new ReportListaResultSet());
				}
				int b = appuntamenti.indexOf(new ReportListaResultSet(lista.getLista_id()));
				if(b >= 0) {
					ReportListaResultSet rs = appuntamenti.get(b);
					if(!ok) {
						ok = ApplicationUtil.gtZero(rs.getTotali());
					}
					ro.setAppuntamenti(rs);
				} else {
					ro.setAppuntamenti(new ReportListaResultSet());
				}
				int c = contratti.indexOf(new ReportListaResultSet(lista.getLista_id()));
				if(c >= 0) {
					ReportListaResultSet rs = contratti.get(c);
					if(!ok) {
						ok = ApplicationUtil.gtZero(rs.getTotali());
					}
					ro.setContratti(rs);
				} else {
					ro.setContratti(new ReportListaResultSet());
				}
				if(ok) {
					Lista ls = (Lista) campagnaDao.get(Lista.class, lista.getLista_id());
					listes.add(ls);
					ro.setLista(ls);
					report.add(ro);
				}
			}
			impostaConteggioLista(listes, r!= null ? r.getCampagna() : null);
		}
		return report;
	}
	public void exportReportListaStoria(Utente user, ReportCriteria r, DownloadBean downloadBean) {
		//if(r != null)  r = (ReportCriteria) r.loadCriteria();
		if(r == null || (r != null && r.getCerca())) {
			String [] titles = new ExportUtility().titlesListaStoria;
			Workbook wb = ApplicationUtil.workbookS();
			ExcelStyle s = new ExcelStyle(wb);
			Sheet sheet = wb.createSheet(ConstantsLanguage.REPORT.stampa() + " " + ConstantsLanguage.LISTA.stampa());
			int k = 0;
			Row row = sheet.createRow(k++);
			for(int i = 0; i < titles.length; i++) {
				Cell cell = row.createCell(i, CellType.STRING);
				ApplicationUtil.headerCell(s, cell,  titles[i]);
			}
			sheet.setAutoFilter(new CellRangeAddress(0, 0, 0, titles.length-1));
			sheet.createFreezePane(0, 1);
			List<ReportLista> data = getReportListaStoria(user, r);
			Iterator<ReportLista> objIter = data.iterator();
			while(objIter.hasNext()) {
				ReportLista dettaglio = objIter.next();
				row = sheet.createRow(k++);
				ExcelFormatList lista = new ExcelFormatList();
				ColonnaModel colonna = dettaglio.getLista().getModel();
				lista.addValore(dettaglio.getLista().getId().toString());
				lista.addValore(dettaglio.getLista_denominazione());
				lista.addValore(colonna.getNchiamate());
				lista.addValore(colonna.getNanagrafiche());
				lista.addValore(colonna.getNcampagne());
				lista.addValore(dettaglio.getContatti().getTotali());
				lista.addValore(dettaglio.getContatti().getChiusi());
				lista.addValore(dettaglio.getContatti().getArchiviazioni());
				lista.addValore(dettaglio.getContatti().getPositivi());
				lista.addValore(dettaglio.getContatti().getUtili());
				lista.addValore(dettaglio.getContatti().getConversazioni());
				lista.addValore(dettaglio.getContatti().getDurata());
				lista.addValore(dettaglio.getContatti().getDurata_conv_eff());
				lista.addValore(dettaglio.getContatti().getDurata_conv_tot());
				lista.addValore(dettaglio.getContatti().getDurata_post());
				lista.addValore(dettaglio.getContatti().getDurata_pre());
				lista.addValore(dettaglio.getContatti().getDurata_rec());
				lista.addValore(dettaglio.getContatti().getDurata_attesa());
				lista.addValore(ConstantsFormat.seconds.stampa(), dettaglio.getContatti().getDurata());
				lista.addValore(ConstantsFormat.seconds.stampa(), dettaglio.getContatti().getDurata_conv_eff());
				lista.addValore(ConstantsFormat.seconds.stampa(), dettaglio.getContatti().getDurata_conv_tot());
				lista.addValore(ConstantsFormat.seconds.stampa(), dettaglio.getContatti().getDurata_post());
				lista.addValore(ConstantsFormat.seconds.stampa(), dettaglio.getContatti().getDurata_pre());
				lista.addValore(ConstantsFormat.seconds.stampa(), dettaglio.getContatti().getDurata_rec());
				lista.addValore(ConstantsFormat.seconds.stampa(), dettaglio.getContatti().getDurata_attesa());
				lista.addValore(dettaglio.getContatti().getMedia());
				lista.addValore(dettaglio.getContatti().getMedia_conv_eff());
				lista.addValore(dettaglio.getContatti().getMedia_conv_tot());
				lista.addValore(dettaglio.getContatti().getMedia_post());
				lista.addValore(dettaglio.getContatti().getMedia_pre());
				lista.addValore(dettaglio.getContatti().getMedia_rec());
				lista.addValore(dettaglio.getContatti().getMedia_attesa());
				lista.addValore(ConstantsFormat.seconds.stampa(), dettaglio.getContatti().getMedia());
				lista.addValore(ConstantsFormat.seconds.stampa(), dettaglio.getContatti().getMedia_conv_eff());
				lista.addValore(ConstantsFormat.seconds.stampa(), dettaglio.getContatti().getMedia_conv_tot());
				lista.addValore(ConstantsFormat.seconds.stampa(), dettaglio.getContatti().getMedia_post());
				lista.addValore(ConstantsFormat.seconds.stampa(), dettaglio.getContatti().getMedia_pre());
				lista.addValore(ConstantsFormat.seconds.stampa(), dettaglio.getContatti().getMedia_rec());	
				lista.addValore(ConstantsFormat.seconds.stampa(), dettaglio.getContatti().getMedia_attesa());
				lista.addValore(dettaglio.getAppuntamenti().getTotali());
				lista.addValore(dettaglio.getAppuntamenti().getConfermati());
				lista.addValore(dettaglio.getAppuntamenti().getUtili());
				lista.addValore(dettaglio.getContratti().getTotali());
				lista.addValore(dettaglio.getContratti().getConfermati());
				lista.addValore(dettaglio.getContratti().getUtili());
				for(int j = 0; j < titles.length; j++) {
					ExcelFormat el = lista.get(j);
					Cell cell = row.createCell(j, el.getCellType());
					ApplicationUtil.cellStamper(s, cell, el);
				}
			}
			sheet.setDefaultColumnWidth(20);
			try {
				String nome = ApplicationUtil.formatIntervallo(ConstantsLanguage.REPORT.stampa() + "_" + ConstantsLanguage.LISTA.stampa(), r.getInizio(), r.getFine()) + ConstantsMap.xlsx; 


				OutputStream out = downloadBean.outputStream();
				downloadBean.filename(nome);

				String p = ConstantsGlobal.secretE;
				ApplicationUtil.passwordXLS(wb, out, p);
				downloadBean.close();

			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		}
	}
	public void exportReportLista(Utente user, ReportCriteria r, DownloadBean downloadBean) {
		//if(r != null)  r = (ReportCriteria) r.loadCriteria();
		if(r == null || (r != null && r.getCerca())) {
			String [] titles = new ExportUtility().titlesLista;
			Workbook wb = ApplicationUtil.workbookS();
			ExcelStyle s = new ExcelStyle(wb);
			Sheet sheet = wb.createSheet(ConstantsLanguage.RIEPILOGO.stampa() + " " + ConstantsLanguage.LISTA.stampa());
			int k = 0;
			Row row = sheet.createRow(k++);
			for(int i = 0; i < titles.length; i++) {
				Cell cell = row.createCell(i, CellType.STRING);
				ApplicationUtil.headerCell(s, cell,  titles[i]);
			}
			sheet.setAutoFilter(new CellRangeAddress(0, 0, 0, titles.length-1));
			sheet.createFreezePane(0, 1);
			List<ReportLista> data = getReportLista(user, r);
			Iterator<ReportLista> objIter = data.iterator();
			while(objIter.hasNext()) {
				ReportLista dettaglio = objIter.next();
				row = sheet.createRow(k++);
				ExcelFormatList lista = new ExcelFormatList();
				ColonnaModel colonna = dettaglio.getLista().getModel();
				lista.addValore(dettaglio.getLista().getId().toString());
				lista.addValore(dettaglio.getLista_denominazione());
				lista.addValore(colonna.getNchiamate());
				lista.addValore(colonna.getNanagrafiche());
				lista.addValore(colonna.getNcampagne());
				lista.addValore(dettaglio.getContatti().getTotali());
				lista.addValore(dettaglio.getContatti().getChiusi());
				lista.addValore(dettaglio.getContatti().getArchiviazioni());
				lista.addValore(dettaglio.getContatti().getPositivi());
				lista.addValore(dettaglio.getContatti().getUtili());
				lista.addValore(dettaglio.getAppuntamenti().getTotali());
				lista.addValore(dettaglio.getAppuntamenti().getConfermati());
				lista.addValore(dettaglio.getAppuntamenti().getUtili());
				lista.addValore(dettaglio.getContratti().getTotali());
				lista.addValore(dettaglio.getContratti().getConfermati());
				lista.addValore(dettaglio.getContratti().getUtili());
				for(int j = 0; j < titles.length; j++) {
					ExcelFormat el = lista.get(j);
					Cell cell = row.createCell(j, el.getCellType());
					ApplicationUtil.cellStamper(s, cell, el);
				}
			}
			sheet.setDefaultColumnWidth(20);
			try {
				String nome = ApplicationUtil.formatIntervallo(ConstantsLanguage.RIEPILOGO.stampa() + "_" + ConstantsLanguage.LISTA.stampa(), r.getInizio(), r.getFine()) + ConstantsMap.xlsx; 


				OutputStream out = downloadBean.outputStream();
				downloadBean.filename(nome);

				String p = ConstantsGlobal.secretE;
				ApplicationUtil.passwordXLS(wb, out, p);
				downloadBean.close();

			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		}
	}
	public void exportReportLista(Integer tipo, OutputStream result) throws Exception {
		Date dataInizio = ApplicationUtil.dataInizio(tipo, ApplicationUtil.date());
		ReportCriteria r = new ReportCriteria();
		r.setIntervallo(dataInizio, null);
		String [] titles = new ExportUtility().titlesLista;
		Workbook wb = ApplicationUtil.workbookS();
		ExcelStyle s = new ExcelStyle(wb);
		Sheet sheet = wb.createSheet(ConstantsLanguage.RIEPILOGO.stampa() + " " + ConstantsLanguage.LISTA.stampa());
		int k = 0;
		Row row = sheet.createRow(k++);
		for(int i = 0; i < titles.length; i++) {
			Cell cell = row.createCell(i, CellType.STRING);
			ApplicationUtil.headerCell(s, cell,  titles[i]);
		}
		sheet.setAutoFilter(new CellRangeAddress(0, 0, 0, titles.length-1));
		sheet.createFreezePane(0, 1);
		List<ReportLista> data = getReportLista(null, r);
		Iterator<ReportLista> objIter = data.iterator();
		while(objIter.hasNext()) {
			ReportLista dettaglio = objIter.next();
			row = sheet.createRow(k++);
			ExcelFormatList lista = new ExcelFormatList();
			ColonnaModel colonna = dettaglio.getLista().getModel();
			lista.addValore(dettaglio.getLista().getId().toString());
			lista.addValore(dettaglio.getLista_denominazione());
			lista.addValore(colonna.getNchiamate());
			lista.addValore(colonna.getNanagrafiche());
			lista.addValore(colonna.getNcampagne());
			lista.addValore(dettaglio.getContatti().getTotali());
			lista.addValore(dettaglio.getContatti().getChiusi());
			lista.addValore(dettaglio.getContatti().getArchiviazioni());
			lista.addValore(dettaglio.getContatti().getPositivi());
			lista.addValore(dettaglio.getContatti().getUtili());
			lista.addValore(dettaglio.getAppuntamenti().getTotali());
			lista.addValore(dettaglio.getAppuntamenti().getConfermati());
			lista.addValore(dettaglio.getAppuntamenti().getUtili());
			lista.addValore(dettaglio.getContratti().getTotali());
			lista.addValore(dettaglio.getContratti().getConfermati());
			lista.addValore(dettaglio.getContratti().getUtili());
			for(int j = 0; j < titles.length; j++) {
				ExcelFormat el = lista.get(j);
				Cell cell = row.createCell(j, el.getCellType());
				ApplicationUtil.cellStamper(s, cell, el);
			}
		}
		sheet.setDefaultColumnWidth(20);
		String p = ConstantsGlobal.secretE;
		ApplicationUtil.passwordXLS(wb, result, p);
	}
	public void exportReportListaStoria(Integer tipo, OutputStream result) throws Exception {
		Date dataInizio = ApplicationUtil.dataInizio(tipo, ApplicationUtil.date());
		ReportCriteria r = new ReportCriteria();
		r.setIntervallo(dataInizio, null);
		String [] titles = new ExportUtility().titlesListaStoria;
		Workbook wb = ApplicationUtil.workbookS();
		ExcelStyle s = new ExcelStyle(wb);
		Sheet sheet = wb.createSheet(ConstantsLanguage.REPORT.stampa() + " " + ConstantsLanguage.LISTA.stampa());
		int k = 0;
		Row row = sheet.createRow(k++);
		for(int i = 0; i < titles.length; i++) {
			Cell cell = row.createCell(i, CellType.STRING);
			ApplicationUtil.headerCell(s, cell,  titles[i]);
		}
		sheet.setAutoFilter(new CellRangeAddress(0, 0, 0, titles.length-1));
		sheet.createFreezePane(0, 1);
		List<ReportLista> data = getReportListaStoria(null, r);
		Iterator<ReportLista> objIter = data.iterator();
		while(objIter.hasNext()) {
			ReportLista dettaglio = objIter.next();
			row = sheet.createRow(k++);
			ExcelFormatList lista = new ExcelFormatList();
			ColonnaModel colonna = dettaglio.getLista().getModel();
			lista.addValore(dettaglio.getLista().getId().toString());
			lista.addValore(dettaglio.getLista_denominazione());
			lista.addValore(colonna.getNchiamate());
			lista.addValore(colonna.getNanagrafiche());
			lista.addValore(colonna.getNcampagne());
			lista.addValore(dettaglio.getContatti().getTotali());
			lista.addValore(dettaglio.getContatti().getChiusi());
			lista.addValore(dettaglio.getContatti().getArchiviazioni());
			lista.addValore(dettaglio.getContatti().getPositivi());
			lista.addValore(dettaglio.getContatti().getUtili());
			lista.addValore(dettaglio.getContatti().getConversazioni());
			lista.addValore(dettaglio.getContatti().getDurata());
			lista.addValore(dettaglio.getContatti().getDurata_conv_eff());
			lista.addValore(dettaglio.getContatti().getDurata_conv_tot());
			lista.addValore(dettaglio.getContatti().getDurata_post());
			lista.addValore(dettaglio.getContatti().getDurata_pre());
			lista.addValore(dettaglio.getContatti().getDurata_rec());
			lista.addValore(dettaglio.getContatti().getDurata_attesa());
			lista.addValore(ConstantsFormat.seconds.stampa(), dettaglio.getContatti().getDurata());
			lista.addValore(ConstantsFormat.seconds.stampa(), dettaglio.getContatti().getDurata_conv_eff());
			lista.addValore(ConstantsFormat.seconds.stampa(), dettaglio.getContatti().getDurata_conv_tot());
			lista.addValore(ConstantsFormat.seconds.stampa(), dettaglio.getContatti().getDurata_post());
			lista.addValore(ConstantsFormat.seconds.stampa(), dettaglio.getContatti().getDurata_pre());
			lista.addValore(ConstantsFormat.seconds.stampa(), dettaglio.getContatti().getDurata_rec());
			lista.addValore(ConstantsFormat.seconds.stampa(), dettaglio.getContatti().getDurata_attesa());
			lista.addValore(dettaglio.getContatti().getMedia());
			lista.addValore(dettaglio.getContatti().getMedia_conv_eff());
			lista.addValore(dettaglio.getContatti().getMedia_conv_tot());
			lista.addValore(dettaglio.getContatti().getMedia_post());
			lista.addValore(dettaglio.getContatti().getMedia_pre());
			lista.addValore(dettaglio.getContatti().getMedia_rec());
			lista.addValore(dettaglio.getContatti().getMedia_attesa());
			lista.addValore(ConstantsFormat.seconds.stampa(), dettaglio.getContatti().getMedia());
			lista.addValore(ConstantsFormat.seconds.stampa(), dettaglio.getContatti().getMedia_conv_eff());
			lista.addValore(ConstantsFormat.seconds.stampa(), dettaglio.getContatti().getMedia_conv_tot());
			lista.addValore(ConstantsFormat.seconds.stampa(), dettaglio.getContatti().getMedia_post());
			lista.addValore(ConstantsFormat.seconds.stampa(), dettaglio.getContatti().getMedia_pre());
			lista.addValore(ConstantsFormat.seconds.stampa(), dettaglio.getContatti().getMedia_rec());	
			lista.addValore(ConstantsFormat.seconds.stampa(), dettaglio.getContatti().getMedia_attesa());
			lista.addValore(dettaglio.getAppuntamenti().getTotali());
			lista.addValore(dettaglio.getAppuntamenti().getConfermati());
			lista.addValore(dettaglio.getAppuntamenti().getUtili());
			lista.addValore(dettaglio.getContratti().getTotali());
			lista.addValore(dettaglio.getContratti().getConfermati());
			lista.addValore(dettaglio.getContratti().getUtili());
			for(int j = 0; j < titles.length; j++) {
				ExcelFormat el = lista.get(j);
				Cell cell = row.createCell(j, el.getCellType());
				ApplicationUtil.cellStamper(s, cell, el);
			}
		}
		sheet.setDefaultColumnWidth(20);
		String p = ConstantsGlobal.secretE;
		ApplicationUtil.passwordXLS(wb, result, p);
	}
	public List<Lista> impostaConteggioLista(List<Lista> liste, Long campagnaId) {
		if(ApplicationUtil.notEmpty(liste))  {
			List<Long> ll = ApplicationUtil.prendiIdLista(liste);
			Map<Long, ConteggioModel> lc = getListaContatti(ll, campagnaId);
			Map<Long, ConteggioModel> ls = getListaCampagne(ll, campagnaId);
			Map<Long, ConteggioModel> la = getListaAnagrafiche(ll, campagnaId);
			Iterator<Lista> it = liste.iterator();
			while (it.hasNext()) {
				Lista lista = (Lista) it.next();
				ColonnaModel colonna = lista.getModel();
				colonna.setNchiamate(ApplicationUtil.getInMap(lc, lista.getId()));
				colonna.setNanagrafiche(ApplicationUtil.getInMap(la, lista.getId()));
				colonna.setNcampagne(ApplicationUtil.getInMap(ls, lista.getId()));
			}
		}
		return liste;
	}
	public void exportReportLavorato(Utente user, ReportCriteria r, DownloadBean downloadBean) {
		//if(r != null)  r = (ReportCriteria) r.loadCriteria();
		if(r == null || (r != null && r.getCerca())) {
			List<String> titles = new ArrayList<String>();
			titles.addAll(ApplicationUtil.asList(new ExportUtility().titlesLavorato));
			DettaglioLavoratoList dr = getReportLavorato(user, r);
			for(Pausa p : dr.getPause()) {
				titles.add(p.getChiaveS());
				titles.add(p.getChiaveS() + " " + ConstantsLanguage.SECONDIH.stampa());
				titles.add(p.getChiaveS() + " " + "["+ConstantsLanguage.CANCELLETTO.stampa()+"]");
			}
			titles.add(ConstantsLanguage.TEMPO_TOTH.stampa());
			titles.add(ConstantsLanguage.TEMPO_TOT.stampa());
			Workbook wb = ApplicationUtil.workbookS();
			ExcelStyle s = new ExcelStyle(wb);
			Sheet sheet = wb.createSheet(ConstantsLanguage.REPORT.stampa() + " " + ConstantsLanguage.LAVORATO.stampa());
			int k = 0;
			Row row = sheet.createRow(k++);
			for(int i = 0; i < titles.size() ; i++) {
				Cell cell = row.createCell(i, CellType.STRING);
				ApplicationUtil.headerCell(s, cell,  titles.get(i));
			}
			sheet.setAutoFilter(new CellRangeAddress(0, 0, 0, titles.size()-1));
			sheet.createFreezePane(0, 1);
			for(ReportLavorato dettaglio : dr.getReport()) {
				row = sheet.createRow(k++);
				ExcelFormatList lista = new ExcelFormatList();
				long totale = dettaglio.getMarcature().getTotaleS();
				lista.addValore(dettaglio.getOperatore().getId().toString());
				lista.addValore(dettaglio.getOperatore().getDenominazione());
				lista.addValore(dettaglio.getMarcature().getTotaleF());
				lista.addValore(dettaglio.getMarcature().getTotale());
				lista.addValore(dettaglio.getMarcature().getConteggio());
				
				for(DettaglioLavorato dfiglio : dettaglio.getPause()) {
					totale -= dfiglio.getTotaleS();
					lista.addValore(dfiglio.getTotaleF());
					lista.addValore(dfiglio.getTotale());
					lista.addValore(dfiglio.getConteggio());
				}
				lista.addValore(ApplicationUtil.format(totale));
				lista.addValore(totale);
				for(int j = 0; j < titles.size(); j++) {
					ExcelFormat el = lista.get(j);
					Cell cell = row.createCell(j, el.getCellType());
					ApplicationUtil.cellStamper(s, cell, el);
				}
			}
			sheet.setDefaultColumnWidth(20);
			try {
				String nome = ApplicationUtil.formatIntervallo(ConstantsLanguage.REPORT.stampa() + "_" + ConstantsLanguage.LAVORATO.stampa(), r.getInizio(), r.getFine()) + ConstantsMap.xlsx; 


				OutputStream out = downloadBean.outputStream();
				downloadBean.filename(nome);

				String p = ConstantsGlobal.secretE;
				ApplicationUtil.passwordXLS(wb, out, p);
				downloadBean.close();

			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		}
	}
	public void exportReportLavorato(Integer tipo, OutputStream result) throws Exception {
		Date dataInizio = ApplicationUtil.dataInizio(tipo, ApplicationUtil.date());
		ReportCriteria r = new ReportCriteria();
		r.setIntervallo(dataInizio, null);
		List<String> titles = new ArrayList<String>();
		titles.addAll(ApplicationUtil.asList(new ExportUtility().titlesLavorato));
		DettaglioLavoratoList dr = getReportLavorato(null, r);
		for(Pausa p : dr.getPause()) {
			titles.add(p.getChiaveS());
			titles.add(p.getChiaveS() + " " + ConstantsLanguage.SECONDIH.stampa());
			titles.add(p.getChiaveS() + " " + "["+ConstantsLanguage.CANCELLETTO.stampa()+"]");
		}
		titles.add(ConstantsLanguage.TEMPO_TOTH.stampa());
		titles.add(ConstantsLanguage.TEMPO_TOT.stampa());
		Workbook wb = ApplicationUtil.workbookS();
		ExcelStyle s = new ExcelStyle(wb);
		Sheet sheet = wb.createSheet(ConstantsLanguage.REPORT.stampa() + " " + ConstantsLanguage.LAVORATO.stampa());
		int k = 0;
		Row row = sheet.createRow(k++);
		for(int i = 0; i < titles.size() ; i++) {
			Cell cell = row.createCell(i, CellType.STRING);
			ApplicationUtil.headerCell(s, cell,  titles.get(i));
		}
		sheet.setAutoFilter(new CellRangeAddress(0, 0, 0, titles.size()-1));
		sheet.createFreezePane(0, 1);
		for(ReportLavorato dettaglio : dr.getReport()) {
			row = sheet.createRow(k++);
			ExcelFormatList lista = new ExcelFormatList();
			long totale = dettaglio.getMarcature().getTotaleS();
			lista.addValore(dettaglio.getOperatore().getId().toString());
			lista.addValore(dettaglio.getOperatore().getDenominazione());
			lista.addValore(dettaglio.getMarcature().getTotaleF());
			lista.addValore(dettaglio.getMarcature().getTotale());
			lista.addValore(dettaglio.getMarcature().getConteggio());
			for(DettaglioLavorato dfiglio : dettaglio.getPause()) {
				totale -= dfiglio.getTotaleS();
				lista.addValore(dfiglio.getTotaleF());
				lista.addValore(dfiglio.getTotale());
				lista.addValore(dfiglio.getConteggio());
			}
			lista.addValore(ApplicationUtil.format(totale));
			lista.addValore(totale);
			for(int j = 0; j < titles.size(); j++) {
				ExcelFormat el = lista.get(j);
				Cell cell = row.createCell(j, el.getCellType());
				ApplicationUtil.cellStamper(s, cell, el);
			}
		}
		sheet.setDefaultColumnWidth(20);
		String p = ConstantsGlobal.secretE;
		ApplicationUtil.passwordXLS(wb, result, p);
	}
	public List<ReportOperatore> getReportOperatore(Utente user, ReportCriteria r) {
		if(r != null)  r = (ReportCriteria) r.loadCriteria(); 
		List<ReportOperatore> report = new ArrayList<ReportOperatore>();
		if(r == null || (r != null && r.getCerca())) {
			List<Dettaglio> esitiChiamata = getReportEsitiChiamata();
			List<Dettaglio> esitiArchiviazione = getReportEsitiArchiviazione(r);
			List<ReportOperatoreResultSet> operatori = getReportOperatoreResultset(user, r);
			List<ReportOperatoreResultSet> dati = getReportOperatoreResultsetDati(user, r);
			for(ReportOperatoreResultSet operatore : operatori) {
				Utente utente = userDao.getUser(operatore.getUser_id());
				ReportOperatore report_op = new ReportOperatore();
				report_op.setOperatore(utente);
				report_op.setTotaleChiamate(new Dettaglio(operatore.getChiamate(),null, null,null,null, null, null, null, null, null, null, null, null, null, null));
				List<Dettaglio> esitiChiamataDettaglio = new ArrayList<Dettaglio>();
				List<Dettaglio> esitiArchiviazioneDettaglio = new ArrayList<Dettaglio>();
				for(Dettaglio ec : esitiChiamata) {
					Dettaglio d;
					int i = dati.indexOf(new ReportOperatoreResultSet(utente.getId(), ec.getEsito_id(), null));
					if(i >= 0) {
						ReportOperatoreResultSet dato = dati.get(i);
						d = new Dettaglio(dato.getChiamate(),null, null,null,null, null, null, null, null, null, null, null, null, null, null);
						d.p_numero(dato.getChiamate(), operatore.getChiamate());
					}
					else d = new Dettaglio(true);
					d.setEsito_id(ec.getEsito_id());
					d.setEtichetta(ec.getEtichetta());
					esitiChiamataDettaglio.add(d);
				}
				for(Dettaglio ea : esitiArchiviazione) {
					Dettaglio d;
					int i = dati.indexOf(new ReportOperatoreResultSet(utente.getId(), ConstantsEsito.ID_OK, ea.getEsito_id()));
					if(i >= 0) {
						ReportOperatoreResultSet dato = dati.get(i);
						d = new Dettaglio(dato.getChiamate(),null, null,null,null, null, null, null, null, null, null, null, null, null, null);
						d.p_numero(dato.getChiamate(), operatore.getChiamate());
					}
					else d = new Dettaglio(true);
					d.setEsito_id(ea.getEsito_id());
					d.setEtichetta(ea.getEtichetta());
					esitiArchiviazioneDettaglio.add(d);
				}
				report_op.setEsitiContattoList(esitiChiamataDettaglio);
				report_op.setEsitiArchiviazioneList(esitiArchiviazioneDettaglio);
				report.add(report_op);
			}
		}
		return report;
	}
	public void exportReportOperatore(Utente user, ReportCriteria r, DownloadBean downloadBean) {
		if(r != null)  r = (ReportCriteria) r.loadCriteria();
		if(r == null || (r != null && r.getCerca())) {
			int a = 1;
			List<Dettaglio> esitiChiamata = getReportEsitiChiamata();
			List<Dettaglio> esitiArchiviazione = getReportEsitiArchiviazione(r);
			List<ReportOperatoreResultSet> operatori = getReportOperatoreResultset(user, r);
			List<ReportOperatoreResultSet> dati = getReportOperatoreResultsetDati(user,r);
			Workbook wb = ApplicationUtil.workbookS();
			ExcelStyle s = new ExcelStyle(wb);
			List<Sheet> fogli = new ArrayList<Sheet>();
			fogli.add(wb.createSheet(ConstantsLanguage.NUMERO_CHIAMATE.stampa()));
			for(Sheet foglio : fogli) {
				int j = 0;
				Row riga = foglio.createRow(foglio.getPhysicalNumberOfRows());
				Cell cella = riga.createCell(j++, CellType.STRING);
				ApplicationUtil.headerCell(s, cella, ConstantsLanguage.OPERATORE.stampa());
				cella = riga.createCell(j++, CellType.STRING);
				ApplicationUtil.headerCell(s, cella, ConstantsLanguage.TOTALE.stampa());
				for(Dettaglio head : esitiChiamata) {
					cella = riga.createCell(j++, CellType.STRING);
					ApplicationUtil.headerCell(s, cella, head.getEtichetta());
				}
				for(Dettaglio head : esitiArchiviazione) {
					cella = riga.createCell(j++, CellType.STRING);
					ApplicationUtil.headerCell(s, cella, head.getEtichetta());
				}
				foglio.setAutoFilter(new CellRangeAddress(0, 0, 0, ApplicationUtil.size(esitiChiamata) + ApplicationUtil.size(esitiArchiviazione) + 1));
				foglio.createFreezePane(0, 1);
			}
			for(ReportOperatoreResultSet operatore : operatori) {
				int j = 0;
				Utente utente = userDao.getUser(operatore.getUser_id());
				Row riga[] = new Row[a];
				for(int i = 0; i < riga.length; i++) {
					riga[i] = fogli.get(i).createRow(fogli.get(i).getPhysicalNumberOfRows());
					ApplicationUtil.formatString(s, riga[i].createCell(j, CellType.STRING), utente.getDenominazione());
				}
				j++;
				ApplicationUtil.formatNumber(s, riga[0].createCell(j, CellType.NUMERIC), operatore.getChiamate());
				j++;
				for(Dettaglio ec : esitiChiamata) {
					int i = dati.indexOf(new ReportOperatoreResultSet(utente.getId(), ec.getEsito_id(), null));
					ReportOperatoreResultSet dato = new ReportOperatoreResultSet();
					if(i >= 0) {
						dato = dati.get(i);
					}
					ApplicationUtil.formatNumber(s, riga[0].createCell(j, CellType.NUMERIC), dato.getChiamate());
					j++;
				}
				for(Dettaglio ea : esitiArchiviazione) {
					int i = dati.indexOf(new ReportOperatoreResultSet(utente.getId(), ConstantsEsito.ID_OK, ea.getEsito_id()));
					ReportOperatoreResultSet dato = new ReportOperatoreResultSet();
					if(i >= 0) {
						dato = dati.get(i);
					}
					ApplicationUtil.formatNumber(s, riga[0].createCell(j, CellType.NUMERIC), dato.getChiamate());
					j++;
				}
			}
			try {
				String nome = ApplicationUtil.formatIntervallo(ConstantsLanguage.RIEPILOGO.stampa() + "_" + ConstantsLanguage.OPERATORI.stampa(), r.getInizio(), r.getFine()) + ConstantsMap.xlsx; 
				OutputStream out = downloadBean.outputStream();
				downloadBean.filename(nome);

				String p = ConstantsGlobal.secretE;
				ApplicationUtil.passwordXLS(wb, out, p);
				downloadBean.close();

			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		}
	}
	public void exportReportOperatore(Integer tipo, OutputStream result) throws Exception {
		int a = 1;
		Date dataInizio = ApplicationUtil.dataInizio(tipo, ApplicationUtil.date());
		ReportCriteria r = new ReportCriteria();
		r.setIntervallo(dataInizio, null);
		List<Dettaglio> esitiChiamata = getReportEsitiChiamata();
		List<Dettaglio> esitiArchiviazione = getReportEsitiArchiviazione(null);
		List<ReportOperatoreResultSet> operatori = getReportOperatoreResultset(null, r);
		List<ReportOperatoreResultSet> dati = getReportOperatoreResultsetDati(null, r);
		Workbook wb = ApplicationUtil.workbookS();
		ExcelStyle s = new ExcelStyle(wb);
		List<Sheet> fogli = new ArrayList<Sheet>();
		fogli.add(wb.createSheet(ConstantsLanguage.NUMERO_CHIAMATE.stampa()));
		for(Sheet foglio : fogli) {
			int j = 0;
			Row riga = foglio.createRow(foglio.getPhysicalNumberOfRows());
			Cell cella = riga.createCell(j++, CellType.STRING);
			ApplicationUtil.headerCell(s, cella, ConstantsLanguage.OPERATORE.stampa());
			cella = riga.createCell(j++, CellType.STRING);
			ApplicationUtil.headerCell(s, cella, ConstantsLanguage.TOTALE.stampa());
			for(Dettaglio head : esitiChiamata) {
				cella = riga.createCell(j++, CellType.STRING);
				ApplicationUtil.headerCell(s, cella, head.getEtichetta());
			}
			for(Dettaglio head : esitiArchiviazione) {
				cella = riga.createCell(j++, CellType.STRING);
				ApplicationUtil.headerCell(s, cella, head.getEtichetta());
			}
			foglio.setAutoFilter(new CellRangeAddress(0, 0, 0, ApplicationUtil.size(esitiChiamata) + ApplicationUtil.size(esitiArchiviazione) + 1));
			foglio.createFreezePane(0, 1);
		}
		for(ReportOperatoreResultSet operatore : operatori) {
			int j = 0;
			Utente utente = userDao.getUser(operatore.getUser_id());
			Row riga[] = new Row[a];
			for(int i = 0; i < riga.length; i++) {
				riga[i] = fogli.get(i).createRow(fogli.get(i).getPhysicalNumberOfRows());
				ApplicationUtil.formatString(s, riga[i].createCell(j, CellType.STRING), utente.getDenominazione());
			}
			j++;
			ApplicationUtil.formatNumber(s, riga[0].createCell(j, CellType.NUMERIC), operatore.getChiamate());
			j++;
			for(Dettaglio ec : esitiChiamata) {
				int i = dati.indexOf(new ReportOperatoreResultSet(utente.getId(), ec.getEsito_id(), null));
				ReportOperatoreResultSet dato = new ReportOperatoreResultSet();
				if(i >= 0) {
					dato = dati.get(i);
				}
				ApplicationUtil.formatNumber(s, riga[0].createCell(j, CellType.NUMERIC), dato.getChiamate());
				j++;
			}
			for(Dettaglio ea : esitiArchiviazione) {
				int i = dati.indexOf(new ReportOperatoreResultSet(utente.getId(), ConstantsEsito.ID_OK, ea.getEsito_id()));
				ReportOperatoreResultSet dato = new ReportOperatoreResultSet();
				if(i >= 0) {
					dato = dati.get(i);
				}
				ApplicationUtil.formatNumber(s, riga[0].createCell(j, CellType.NUMERIC), dato.getChiamate());
				j++;
			}
		}
		String p = ConstantsGlobal.secretE;
		ApplicationUtil.passwordXLS(wb, result, p);
	}
	public List<ReportOperatore> getReportOperatoreStoria(Utente user, ReportCriteria r) {
		if(r != null)  r = (ReportCriteria) r.loadCriteria(); 
		List<ReportOperatore> report = new ArrayList<ReportOperatore>();
		if(r == null || (r != null && r.getCerca())) {
			List<Dettaglio> esitiChiamata = getReportEsitiChiamata();
			List<Dettaglio> esitiArchiviazione = getReportEsitiArchiviazione(r);
			List<ReportOperatoreResultSet> operatori = getReportOperatoreStoriaResultset(user, r);
			List<ReportOperatoreResultSet> dati = getReportOperatoreStoriaResultsetDati(user, r);
			for(ReportOperatoreResultSet operatore : operatori) {
				Utente utente = userDao.getUser(operatore.getUser_id());
				ReportOperatore report_op = new ReportOperatore();
				report_op.setOperatore(utente);
				report_op.setTotaleChiamate(new Dettaglio(operatore.getChiamate(), ApplicationUtil.format(operatore.getDurata()), ApplicationUtil.format(operatore.getMedia()),  ApplicationUtil.format(operatore.getDurata_conv_eff()), ApplicationUtil.format(operatore.getMedia_conv_eff()), ApplicationUtil.format(operatore.getDurata_conv_tot()), ApplicationUtil.format(operatore.getMedia_conv_tot()), ApplicationUtil.format(operatore.getDurata_post()), ApplicationUtil.format(operatore.getMedia_post()), ApplicationUtil.format(operatore.getDurata_pre()), ApplicationUtil.format(operatore.getMedia_pre()), ApplicationUtil.format(operatore.getDurata_rec()), ApplicationUtil.format(operatore.getMedia_rec()), ApplicationUtil.format(operatore.getDurata_attesa()), ApplicationUtil.format(operatore.getMedia_attesa())));
				List<Dettaglio> esitiChiamataDettaglio = new ArrayList<Dettaglio>();
				List<Dettaglio> esitiArchiviazioneDettaglio = new ArrayList<Dettaglio>();
				for(Dettaglio ec : esitiChiamata) {
					Dettaglio d;
					int i = dati.indexOf(new ReportOperatoreResultSet(utente.getId(), ec.getEsito_id(), null));
					if(i >= 0) {
						ReportOperatoreResultSet dato = dati.get(i);
						d = new Dettaglio(dato.getChiamate(), ApplicationUtil.format(dato.getDurata()), ApplicationUtil.format(dato.getMedia()),  ApplicationUtil.format(dato.getDurata_conv_eff()), ApplicationUtil.format(dato.getMedia_conv_eff()), ApplicationUtil.format(dato.getDurata_conv_tot()), ApplicationUtil.format(dato.getMedia_conv_tot()), ApplicationUtil.format(dato.getDurata_post()), ApplicationUtil.format(dato.getMedia_post()), ApplicationUtil.format(dato.getDurata_pre()), ApplicationUtil.format(dato.getMedia_pre()), ApplicationUtil.format(dato.getDurata_rec()), ApplicationUtil.format(dato.getMedia_rec()), ApplicationUtil.format(dato.getDurata_attesa()), ApplicationUtil.format(dato.getMedia_attesa()));
						d.p_numero(dato.getChiamate(), operatore.getChiamate());
						d.p_tempo(dato.getDurata(), operatore.getDurata());
					}
					else d = new Dettaglio(true);
					d.setEsito_id(ec.getEsito_id());
					d.setEtichetta(ec.getEtichetta());
					esitiChiamataDettaglio.add(d);
				}
				for(Dettaglio ea : esitiArchiviazione) {
					Dettaglio d;
					int i = dati.indexOf(new ReportOperatoreResultSet(utente.getId(), ConstantsEsito.ID_OK, ea.getEsito_id()));
					if(i >= 0) {
						ReportOperatoreResultSet dato = dati.get(i);
						d = new Dettaglio(dato.getChiamate(), ApplicationUtil.format(dato.getDurata()), ApplicationUtil.format(dato.getMedia()),  ApplicationUtil.format(dato.getDurata_conv_eff()), ApplicationUtil.format(dato.getMedia_conv_eff()), ApplicationUtil.format(dato.getDurata_conv_tot()), ApplicationUtil.format(dato.getMedia_conv_tot()), ApplicationUtil.format(dato.getDurata_post()), ApplicationUtil.format(dato.getMedia_post()), ApplicationUtil.format(dato.getDurata_pre()), ApplicationUtil.format(dato.getMedia_pre()), ApplicationUtil.format(dato.getDurata_rec()), ApplicationUtil.format(dato.getMedia_rec()), ApplicationUtil.format(dato.getDurata_attesa()), ApplicationUtil.format(dato.getMedia_attesa()));
						d.p_numero(dato.getChiamate(), operatore.getChiamate());
						d.p_tempo(dato.getDurata(), operatore.getDurata());
					}
					else d = new Dettaglio(true);
					d.setEsito_id(ea.getEsito_id());
					d.setEtichetta(ea.getEtichetta());
					esitiArchiviazioneDettaglio.add(d);
				}
				report_op.setEsitiContattoList(esitiChiamataDettaglio);
				report_op.setEsitiArchiviazioneList(esitiArchiviazioneDettaglio);
				report.add(report_op);
			}
		}
		return report;
	}
	public void exportReportOperatoreStoria(Utente user, ReportCriteria r, DownloadBean downloadBean) {
		if(r != null)  r = (ReportCriteria) r.loadCriteria();
		if(r == null || (r != null && r.getCerca())) {
			boolean formatta = impostazioneDao.getFormattaTempi();
			int a = 15;
			List<Dettaglio> esitiChiamata = getReportEsitiChiamata();
			List<Dettaglio> esitiArchiviazione = getReportEsitiArchiviazione(r);
			List<ReportOperatoreResultSet> operatori = getReportOperatoreStoriaResultset(user, r);
			List<ReportOperatoreResultSet> dati = getReportOperatoreStoriaResultsetDati(user, r);
			Workbook wb = ApplicationUtil.workbookS();
			ExcelStyle s = new ExcelStyle(wb);
			List<Sheet> fogli = new ArrayList<Sheet>();
			fogli.add(wb.createSheet(ConstantsLanguage.NUMERO_CHIAMATE.stampa()));
			fogli.add(wb.createSheet(ConstantsLanguage.TEMPO_TOTH.stampa()));
			fogli.add(wb.createSheet(ConstantsLanguage.MEDIA_TOTH.stampa()));
			fogli.add(wb.createSheet(ConstantsLanguage.TEMPO_CONV_EFFH.stampa()));
			fogli.add(wb.createSheet(ConstantsLanguage.MEDIA_CONV_EFFH.stampa()));
			fogli.add(wb.createSheet(ConstantsLanguage.TEMPO_CONV_TOTH.stampa()));
			fogli.add(wb.createSheet(ConstantsLanguage.MEDIA_CONV_TOTH.stampa()));
			fogli.add(wb.createSheet(ConstantsLanguage.TEMPO_POSTH.stampa()));
			fogli.add(wb.createSheet(ConstantsLanguage.MEDIA_POSTH.stampa()));
			fogli.add(wb.createSheet(ConstantsLanguage.TEMPO_PREH.stampa()));
			fogli.add(wb.createSheet(ConstantsLanguage.MEDIA_PREH.stampa()));
			fogli.add(wb.createSheet(ConstantsLanguage.TEMPO_RECH.stampa()));
			fogli.add(wb.createSheet(ConstantsLanguage.MEDIA_RECH.stampa()));
			fogli.add(wb.createSheet(ConstantsLanguage.TEMPO_ATTESAH.stampa()));
			fogli.add(wb.createSheet(ConstantsLanguage.MEDIA_ATTESAH.stampa()));
			for(Sheet foglio : fogli) {
				int j = 0;
				Row riga = foglio.createRow(foglio.getPhysicalNumberOfRows());
				Cell cella = riga.createCell(j++, CellType.STRING);
				ApplicationUtil.headerCell(s, cella, ConstantsLanguage.OPERATORE.stampa());
				cella = riga.createCell(j++, CellType.STRING);
				ApplicationUtil.headerCell(s, cella, ConstantsLanguage.TOTALE.stampa());
				for(Dettaglio head : esitiChiamata) {
					cella = riga.createCell(j++, CellType.STRING);
					ApplicationUtil.headerCell(s, cella, head.getEtichetta());
				}
				for(Dettaglio head : esitiArchiviazione) {
					cella = riga.createCell(j++, CellType.STRING);
					ApplicationUtil.headerCell(s, cella, head.getEtichetta());
				}
				foglio.setAutoFilter(new CellRangeAddress(0, 0, 0, ApplicationUtil.size(esitiChiamata) + ApplicationUtil.size(esitiArchiviazione) + 1));
				foglio.createFreezePane(0, 1);
			}
			for(ReportOperatoreResultSet operatore : operatori) {
				int j = 0;
				Utente utente = userDao.getUser(operatore.getUser_id());
				Row riga[] = new Row[a];
				for(int i = 0; i < riga.length; i++) {
					riga[i] = fogli.get(i).createRow(fogli.get(i).getPhysicalNumberOfRows());
					ApplicationUtil.formatString(s, riga[i].createCell(j, CellType.STRING), utente.getDenominazione());
				}
				j++;
				ApplicationUtil.formatNumber(s, riga[0].createCell(j, CellType.NUMERIC), operatore.getChiamate());
				ApplicationUtil.formatTimestamp(formatta, s, riga[1].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), operatore.getDurata());
				ApplicationUtil.formatTimestamp(formatta, s, riga[2].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), operatore.getMedia());
				ApplicationUtil.formatTimestamp(formatta, s, riga[3].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), operatore.getDurata_conv_eff());
				ApplicationUtil.formatTimestamp(formatta, s, riga[4].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), operatore.getMedia_conv_eff());
				ApplicationUtil.formatTimestamp(formatta, s, riga[5].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), operatore.getDurata_conv_tot());
				ApplicationUtil.formatTimestamp(formatta, s, riga[6].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), operatore.getMedia_conv_tot());
				ApplicationUtil.formatTimestamp(formatta, s, riga[7].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), operatore.getDurata_post());
				ApplicationUtil.formatTimestamp(formatta, s, riga[8].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), operatore.getMedia_post());
				ApplicationUtil.formatTimestamp(formatta, s, riga[9].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), operatore.getDurata_pre());
				ApplicationUtil.formatTimestamp(formatta, s, riga[10].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), operatore.getMedia_pre());
				ApplicationUtil.formatTimestamp(formatta, s, riga[11].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), operatore.getDurata_rec());
				ApplicationUtil.formatTimestamp(formatta, s, riga[12].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), operatore.getMedia_rec());
				ApplicationUtil.formatTimestamp(formatta, s, riga[13].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), operatore.getDurata_attesa());
				ApplicationUtil.formatTimestamp(formatta, s, riga[14].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), operatore.getMedia_attesa());
				j++;
				for(Dettaglio ec : esitiChiamata) {
					int i = dati.indexOf(new ReportOperatoreResultSet(utente.getId(), ec.getEsito_id(), null));
					ReportOperatoreResultSet dato = new ReportOperatoreResultSet();
					if(i >= 0) {
						dato = dati.get(i);
					}
					ApplicationUtil.formatNumber(s, riga[0].createCell(j, CellType.NUMERIC), dato.getChiamate());
					ApplicationUtil.formatTimestamp(formatta, s, riga[1].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getDurata());
					ApplicationUtil.formatTimestamp(formatta, s, riga[2].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getMedia());
					ApplicationUtil.formatTimestamp(formatta, s, riga[3].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getDurata_conv_eff());
					ApplicationUtil.formatTimestamp(formatta, s, riga[4].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getMedia_conv_eff());
					ApplicationUtil.formatTimestamp(formatta, s, riga[5].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getDurata_conv_tot());
					ApplicationUtil.formatTimestamp(formatta, s, riga[6].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getMedia_conv_tot());
					ApplicationUtil.formatTimestamp(formatta, s, riga[7].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getDurata_post());
					ApplicationUtil.formatTimestamp(formatta, s, riga[8].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getMedia_post());
					ApplicationUtil.formatTimestamp(formatta, s, riga[9].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getDurata_pre());
					ApplicationUtil.formatTimestamp(formatta, s, riga[10].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getMedia_pre());
					ApplicationUtil.formatTimestamp(formatta, s, riga[11].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getDurata_rec());
					ApplicationUtil.formatTimestamp(formatta, s, riga[12].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getMedia_rec());
					ApplicationUtil.formatTimestamp(formatta, s, riga[13].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getDurata_attesa());
					ApplicationUtil.formatTimestamp(formatta, s, riga[14].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getMedia_attesa());
					j++;
				}
				for(Dettaglio ea : esitiArchiviazione) {
					int i = dati.indexOf(new ReportOperatoreResultSet(utente.getId(), ConstantsEsito.ID_OK, ea.getEsito_id()));
					ReportOperatoreResultSet dato = new ReportOperatoreResultSet();
					if(i >= 0) {
						dato = dati.get(i);
					}
					ApplicationUtil.formatNumber(s, riga[0].createCell(j, CellType.NUMERIC), dato.getChiamate());
					ApplicationUtil.formatTimestamp(formatta, s, riga[1].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getDurata());
					ApplicationUtil.formatTimestamp(formatta, s, riga[2].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getMedia());
					ApplicationUtil.formatTimestamp(formatta, s, riga[3].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getDurata_conv_eff());
					ApplicationUtil.formatTimestamp(formatta, s, riga[4].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getMedia_conv_eff());
					ApplicationUtil.formatTimestamp(formatta, s, riga[5].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getDurata_conv_tot());
					ApplicationUtil.formatTimestamp(formatta, s, riga[6].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getMedia_conv_tot());
					ApplicationUtil.formatTimestamp(formatta, s, riga[7].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getDurata_post());
					ApplicationUtil.formatTimestamp(formatta, s, riga[8].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getMedia_post());
					ApplicationUtil.formatTimestamp(formatta, s, riga[9].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getDurata_pre());
					ApplicationUtil.formatTimestamp(formatta, s, riga[10].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getMedia_pre());
					ApplicationUtil.formatTimestamp(formatta, s, riga[11].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getDurata_rec());
					ApplicationUtil.formatTimestamp(formatta, s, riga[12].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getMedia_rec());
					ApplicationUtil.formatTimestamp(formatta, s, riga[13].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getDurata_attesa());
					ApplicationUtil.formatTimestamp(formatta, s, riga[14].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getMedia_attesa());
					j++;
				}
			}
			try {
				String nome = ApplicationUtil.formatIntervallo(ConstantsLanguage.REPORT.stampa() + "_" + ConstantsLanguage.OPERATORI.stampa(), r.getInizio(), r.getFine()) + ConstantsMap.xlsx; 


				OutputStream out = downloadBean.outputStream();
				downloadBean.filename(nome);

				String p = ConstantsGlobal.secretE;
				ApplicationUtil.passwordXLS(wb, out, p);
				downloadBean.close();

			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		}
	}
	public void exportReportOperatoreStoria(Integer tipo, OutputStream result) throws Exception {
		boolean formatta = impostazioneDao.getFormattaTempi();
		int a = 15;
		Date dataInizio = ApplicationUtil.dataInizio(tipo, ApplicationUtil.date());
		ReportCriteria r = new ReportCriteria();
		r.setIntervallo(dataInizio, null);
		List<Dettaglio> esitiChiamata = getReportEsitiChiamata();
		List<Dettaglio> esitiArchiviazione = getReportEsitiArchiviazione(null);
		List<ReportOperatoreResultSet> operatori = getReportOperatoreStoriaResultset(null, r);
		List<ReportOperatoreResultSet> dati = getReportOperatoreStoriaResultsetDati(null, r);
		Workbook wb = ApplicationUtil.workbookS();
		ExcelStyle s = new ExcelStyle(wb);
		List<Sheet> fogli = new ArrayList<Sheet>();
		fogli.add(wb.createSheet(ConstantsLanguage.NUMERO_CHIAMATE.stampa()));
		fogli.add(wb.createSheet(ConstantsLanguage.TEMPO_TOTH.stampa()));
		fogli.add(wb.createSheet(ConstantsLanguage.MEDIA_TOTH.stampa()));
		fogli.add(wb.createSheet(ConstantsLanguage.TEMPO_CONV_EFFH.stampa()));
		fogli.add(wb.createSheet(ConstantsLanguage.MEDIA_CONV_EFFH.stampa()));
		fogli.add(wb.createSheet(ConstantsLanguage.TEMPO_CONV_TOTH.stampa()));
		fogli.add(wb.createSheet(ConstantsLanguage.MEDIA_CONV_TOTH.stampa()));
		fogli.add(wb.createSheet(ConstantsLanguage.TEMPO_POSTH.stampa()));
		fogli.add(wb.createSheet(ConstantsLanguage.MEDIA_POSTH.stampa()));
		fogli.add(wb.createSheet(ConstantsLanguage.TEMPO_PREH.stampa()));
		fogli.add(wb.createSheet(ConstantsLanguage.MEDIA_PREH.stampa()));
		fogli.add(wb.createSheet(ConstantsLanguage.TEMPO_RECH.stampa()));
		fogli.add(wb.createSheet(ConstantsLanguage.MEDIA_RECH.stampa()));
		fogli.add(wb.createSheet(ConstantsLanguage.TEMPO_ATTESAH.stampa()));
		fogli.add(wb.createSheet(ConstantsLanguage.MEDIA_ATTESAH.stampa()));
		for(Sheet foglio : fogli) {
			int j = 0;
			Row riga = foglio.createRow(foglio.getPhysicalNumberOfRows());
			Cell cella = riga.createCell(j++, CellType.STRING);
			ApplicationUtil.headerCell(s, cella, ConstantsLanguage.OPERATORE.stampa());
			cella = riga.createCell(j++, CellType.STRING);
			ApplicationUtil.headerCell(s, cella, ConstantsLanguage.TOTALE.stampa());
			for(Dettaglio head : esitiChiamata) {
				cella = riga.createCell(j++, CellType.STRING);
				ApplicationUtil.headerCell(s, cella, head.getEtichetta());
			}
			for(Dettaglio head : esitiArchiviazione) {
				cella = riga.createCell(j++, CellType.STRING);
				ApplicationUtil.headerCell(s, cella, head.getEtichetta());
			}
			foglio.setAutoFilter(new CellRangeAddress(0, 0, 0, ApplicationUtil.size(esitiChiamata) + ApplicationUtil.size(esitiArchiviazione) + 1));
			foglio.createFreezePane(0, 1);
		}
		for(ReportOperatoreResultSet operatore : operatori) {
			int j = 0;
			Utente utente = userDao.getUser(operatore.getUser_id());
			Row riga[] = new Row[a];
			for(int i = 0; i < riga.length; i++) {
				riga[i] = fogli.get(i).createRow(fogli.get(i).getPhysicalNumberOfRows());
				ApplicationUtil.formatString(s, riga[i].createCell(j, CellType.STRING), utente.getDenominazione());
			}
			j++;
			ApplicationUtil.formatNumber(s, riga[0].createCell(j, CellType.NUMERIC), operatore.getChiamate());
			ApplicationUtil.formatTimestamp(formatta, s, riga[1].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), operatore.getDurata());
			ApplicationUtil.formatTimestamp(formatta, s, riga[2].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), operatore.getMedia());
			ApplicationUtil.formatTimestamp(formatta, s, riga[3].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), operatore.getDurata_conv_eff());
			ApplicationUtil.formatTimestamp(formatta, s, riga[4].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), operatore.getMedia_conv_eff());
			ApplicationUtil.formatTimestamp(formatta, s, riga[5].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), operatore.getDurata_conv_tot());
			ApplicationUtil.formatTimestamp(formatta, s, riga[6].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), operatore.getMedia_conv_tot());
			ApplicationUtil.formatTimestamp(formatta, s, riga[7].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), operatore.getDurata_post());
			ApplicationUtil.formatTimestamp(formatta, s, riga[8].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), operatore.getMedia_post());
			ApplicationUtil.formatTimestamp(formatta, s, riga[9].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), operatore.getDurata_pre());
			ApplicationUtil.formatTimestamp(formatta, s, riga[10].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), operatore.getMedia_pre());
			ApplicationUtil.formatTimestamp(formatta, s, riga[11].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), operatore.getDurata_rec());
			ApplicationUtil.formatTimestamp(formatta, s, riga[12].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), operatore.getMedia_rec());
			ApplicationUtil.formatTimestamp(formatta, s, riga[13].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), operatore.getDurata_attesa());
			ApplicationUtil.formatTimestamp(formatta, s, riga[14].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), operatore.getMedia_attesa());
			j++;
			for(Dettaglio ec : esitiChiamata) {
				int i = dati.indexOf(new ReportOperatoreResultSet(utente.getId(), ec.getEsito_id(), null));
				ReportOperatoreResultSet dato = new ReportOperatoreResultSet();
				if(i >= 0) {
					dato = dati.get(i);
				}
				ApplicationUtil.formatNumber(s, riga[0].createCell(j, CellType.NUMERIC), dato.getChiamate());
				ApplicationUtil.formatTimestamp(formatta, s, riga[1].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getDurata());
				ApplicationUtil.formatTimestamp(formatta, s, riga[2].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getMedia());
				ApplicationUtil.formatTimestamp(formatta, s, riga[3].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getDurata_conv_eff());
				ApplicationUtil.formatTimestamp(formatta, s, riga[4].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getMedia_conv_eff());
				ApplicationUtil.formatTimestamp(formatta, s, riga[5].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getDurata_conv_tot());
				ApplicationUtil.formatTimestamp(formatta, s, riga[6].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getMedia_conv_tot());
				ApplicationUtil.formatTimestamp(formatta, s, riga[7].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getDurata_post());
				ApplicationUtil.formatTimestamp(formatta, s, riga[8].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getMedia_post());
				ApplicationUtil.formatTimestamp(formatta, s, riga[9].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getDurata_pre());
				ApplicationUtil.formatTimestamp(formatta, s, riga[10].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getMedia_pre());
				ApplicationUtil.formatTimestamp(formatta, s, riga[11].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getDurata_rec());
				ApplicationUtil.formatTimestamp(formatta, s, riga[12].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getMedia_rec());
				ApplicationUtil.formatTimestamp(formatta, s, riga[13].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getDurata_attesa());
				ApplicationUtil.formatTimestamp(formatta, s, riga[14].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getMedia_attesa());
				j++;
			}
			for(Dettaglio ea : esitiArchiviazione) {
				int i = dati.indexOf(new ReportOperatoreResultSet(utente.getId(), ConstantsEsito.ID_OK, ea.getEsito_id()));
				ReportOperatoreResultSet dato = new ReportOperatoreResultSet();
				if(i >= 0) {
					dato = dati.get(i);
				}
				ApplicationUtil.formatNumber(s, riga[0].createCell(j, CellType.NUMERIC), dato.getChiamate());
				ApplicationUtil.formatTimestamp(formatta, s, riga[1].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getDurata());
				ApplicationUtil.formatTimestamp(formatta, s, riga[2].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getMedia());
				ApplicationUtil.formatTimestamp(formatta, s, riga[3].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getDurata_conv_eff());
				ApplicationUtil.formatTimestamp(formatta, s, riga[4].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getMedia_conv_eff());
				ApplicationUtil.formatTimestamp(formatta, s, riga[5].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getDurata_conv_tot());
				ApplicationUtil.formatTimestamp(formatta, s, riga[6].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getMedia_conv_tot());
				ApplicationUtil.formatTimestamp(formatta, s, riga[7].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getDurata_post());
				ApplicationUtil.formatTimestamp(formatta, s, riga[8].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getMedia_post());
				ApplicationUtil.formatTimestamp(formatta, s, riga[9].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getDurata_pre());
				ApplicationUtil.formatTimestamp(formatta, s, riga[10].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getMedia_pre());
				ApplicationUtil.formatTimestamp(formatta, s, riga[11].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getDurata_rec());
				ApplicationUtil.formatTimestamp(formatta, s, riga[12].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getMedia_rec());
				ApplicationUtil.formatTimestamp(formatta, s, riga[13].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getDurata_attesa());
				ApplicationUtil.formatTimestamp(formatta, s, riga[14].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getMedia_attesa());
				j++;
			}
		}
		String p = ConstantsGlobal.secretE;
		ApplicationUtil.passwordXLS(wb, result, p);
	}
	private ExtendedCriteria getReportOperatoreEsitoTotaleStoriaCriteria(Long utente,  ReportCriteria r, Long operatoreId) {
		if(r != null)  r = (ReportCriteria) r.loadCriteria();
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContatto.class);
		DetachedCriteria criteriaOp = criteria.createCriteria("operatore", "op");
		DetachedCriteria criteriaC = criteria.createCriteria("contatto");
		DetachedCriteria criteriaCA = criteriaC.createCriteria("campagna", "camp");
		if (ApplicationUtil.gtZero(utente)) {
			impostazioneDao.subCriteriaRoleSupervisore(utente, criteria, "camp", "op");
		}
		if(r != null) {
			DetachedCriteria criteriaLI = criteriaC.createCriteria("lista");
			ApplicationUtil.pannello(r.getPannello(), criteria);
			ApplicationUtil.dataIntervalD(r.getInizio(), r.getFine(), "data", criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, criteriaLI);
			if(ApplicationUtil.gtZero(r.getLista())) {
				criteriaLI.add(Restrictions.idEq(r.getLista()));
			}
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
			if(ApplicationUtil.ltZero(r.getCampagna())) {
				if(ApplicationUtil.gtZero(utente)) {
					if(ApplicationUtil.isTrue(sd)) {
						impostazioneDao.subCriteriaCampagnaSupervisore(utente, criteriaCA, "id");
					}
				}
			}
		}
		criteriaOp.add(Restrictions.idEq(operatoreId));
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, r);
	}
	private ExtendedCriteria getReportOperatoreEsitoTotaleCriteria(Long utente,  ReportCriteria r, Long operatoreId) {
		if(r != null)  r = (ReportCriteria) r.loadCriteria();
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class);
		DetachedCriteria criteriaCA = criteria.createCriteria("campagna");
		DetachedCriteria criteriaLI = criteria.createCriteria("lista");
		if(r != null) {
			ApplicationUtil.intervallo(ApplicationUtil.isTrue(r.getInserimento()), r.getInizio(), r.getFine(), criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, criteriaLI);
			if(ApplicationUtil.gtZero(r.getLista())) {
				criteriaLI.add(Restrictions.idEq(r.getLista()));
			}
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
			if(ApplicationUtil.ltZero(r.getCampagna())) {
				if(ApplicationUtil.gtZero(utente)) {
					if(ApplicationUtil.isTrue(sd)) {
						impostazioneDao.subCriteriaCampagnaSupervisore(utente, criteriaCA, "id");
					}
				}
			}
		}
		criteria.add(Restrictions.eq("utente.id",operatoreId));
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, r);
	}
	@SuppressWarnings("unchecked")
	public List<StoriaContatto>  getReportOperatoreEsitoTotaleStoria(Long user,ReportCriteria r, Long operatoreId, int start, int size) {
		ExtendedCriteria criteria = getReportOperatoreEsitoTotaleStoriaCriteria(user, r, operatoreId);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	@SuppressWarnings("unchecked")
	public List<Contatto>  getReportOperatoreEsitoTotale(Long user,ReportCriteria r, Long operatoreId, int start, int size) {
		ExtendedCriteria criteria = getReportOperatoreEsitoTotaleCriteria(user, r, operatoreId);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int  getReportOperatoreEsitoTotaleSize(Long user,  ReportCriteria r, Long operatoreId) {
		ExtendedCriteria criteria = getReportOperatoreEsitoTotaleCriteria(user, r, operatoreId);
		return getListSize(criteria);
	}
	public int  getReportOperatoreEsitoTotaleStoriaSize(Long user,  ReportCriteria r, Long operatoreId) {
		ExtendedCriteria criteria = getReportOperatoreEsitoTotaleStoriaCriteria(user, r, operatoreId);
		return getListSize(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<StoriaContatto>  getReportOperatoreEsitoChiamataStoria(Long utente, ReportCriteria r,  Long esito, Long operatoreId, int start, int size) {
		ExtendedCriteria criteria = getReportOperatoreEsitoChiamataStoriaCriteria(utente, r, esito, operatoreId);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int  getReportOperatoreEsitoChiamataStoriaSize(Long utente, ReportCriteria r,  Long esito, Long operatoreId) {
		ExtendedCriteria criteria = getReportOperatoreEsitoChiamataStoriaCriteria(utente, r, esito, operatoreId);
		return getListSize(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Contatto>  getReportOperatoreEsitoChiamata(Long utente, ReportCriteria r,  Long esito, Long operatoreId, int start, int size) {
		ExtendedCriteria criteria = getReportOperatoreEsitoChiamataCriteria(utente, r, esito, operatoreId);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int  getReportOperatoreEsitoChiamataSize(Long utente, ReportCriteria r,  Long esito, Long operatoreId) {
		ExtendedCriteria criteria = getReportOperatoreEsitoChiamataCriteria(utente, r, esito, operatoreId);
		return getListSize(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Contatto>  getReportOperatoreEsitoArchivazione(Long utente, ReportCriteria r,  Long esito, Long operatoreId, int start, int size) {
		ExtendedCriteria criteria = getReportOperatoreEsitoArchivazioneCriteria(utente, r, esito, operatoreId);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int  getReportOperatoreEsitoArchivazioneSize(Long utente, ReportCriteria r,  Long esito, Long operatoreId) {
		ExtendedCriteria criteria = getReportOperatoreEsitoArchivazioneCriteria(utente, r, esito, operatoreId);
		return getListSize(criteria);
	}
	private ExtendedCriteria getReportOperatoreEsitoChiamataStoriaCriteria(Long utente, ReportCriteria r,  Long esito, Long operatoreId) {
		if(r != null)  r = (ReportCriteria) r.loadCriteria();
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContatto.class);
		DetachedCriteria criteriaOp = criteria.createCriteria("operatore", "op");
		DetachedCriteria criteriaCa = criteria.createCriteria("contatto");
		DetachedCriteria criteriaCA = criteriaCa.createCriteria("campagna", "camp");
		if (ApplicationUtil.gtZero(esito)) {
			criteria.add(Restrictions.eq("esito.id", esito));
		}
		if (ApplicationUtil.gtZero(utente)) {
			impostazioneDao.subCriteriaRoleSupervisore(utente, criteria, "camp", "op");
		}
		if(r != null) {
			DetachedCriteria criteriaLI = criteriaCa.createCriteria("lista");
			ApplicationUtil.pannello(r.getPannello(), criteria);
			ApplicationUtil.dataIntervalD(r.getInizio(), r.getFine(), "data", criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, criteriaLI);
			if(ApplicationUtil.gtZero(r.getLista())) {
				criteriaLI.add(Restrictions.idEq(r.getLista()));
			}
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
			if(ApplicationUtil.ltZero(r.getCampagna())) {
				if(ApplicationUtil.gtZero(utente)) {
					if(ApplicationUtil.isTrue(sd)) {
						impostazioneDao.subCriteriaCampagnaSupervisore(utente, criteriaCA, "id");
					}
				}
			}
		}
		criteriaOp.add(Restrictions.idEq(operatoreId));
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, r);
	}
	private ExtendedCriteria getReportOperatoreEsitoChiamataCriteria(Long utente, ReportCriteria r,  Long esito, Long operatoreId) {
		if(r != null)  r = (ReportCriteria) r.loadCriteria();
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class);
		DetachedCriteria criteriaCA = criteria.createCriteria("campagna");
		DetachedCriteria criteriaLI = criteria.createCriteria("lista");
		if (ApplicationUtil.gtZero(esito)) {
			criteria.add(Restrictions.eq("esitoChiamata.id", esito));
		}
		if(r != null) {
			ApplicationUtil.intervallo(ApplicationUtil.isTrue(r.getInserimento()), r.getInizio(), r.getFine(), criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, criteriaLI);
			if(ApplicationUtil.gtZero(r.getLista())) {
				criteriaLI.add(Restrictions.idEq(r.getLista()));
			}
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
			if(ApplicationUtil.ltZero(r.getCampagna())) {
				if(ApplicationUtil.gtZero(utente)) {
					if(ApplicationUtil.isTrue(sd)) {
						impostazioneDao.subCriteriaCampagnaSupervisore(utente, criteriaCA, "id");
					}
				}
			}
		}
		criteria.add(Restrictions.eq("utente.id",operatoreId));
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, r);
	}
	private ExtendedCriteria getReportOperatoreEsitoArchivazioneCriteria(Long utente, ReportCriteria r,  Long esito, Long operatoreId) {
		if(r != null)  r = (ReportCriteria) r.loadCriteria();
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class);
		DetachedCriteria criteriaCA = criteria.createCriteria("campagna");
		DetachedCriteria criteriaLI = criteria.createCriteria("lista");
		if (ApplicationUtil.gtZero(esito)) {
			criteria.add(Restrictions.eq("esitoArchiviazione.id", esito));
		}
		if(r != null) {
			ApplicationUtil.intervallo(ApplicationUtil.isTrue(r.getInserimento()), r.getInizio(), r.getFine(), criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, criteriaLI);
			if(ApplicationUtil.gtZero(r.getLista())) {
				criteriaLI.add(Restrictions.idEq(r.getLista()));
			}
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
			if(ApplicationUtil.ltZero(r.getCampagna())) {
				if(ApplicationUtil.gtZero(utente)) {
					if(ApplicationUtil.isTrue(sd)) {
						impostazioneDao.subCriteriaCampagnaSupervisore(utente, criteriaCA, "id");
					}
				}
			}
		}
		criteria.add(Restrictions.eq("utente.id",operatoreId));
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, r);
	}
	@SuppressWarnings("unchecked")
	public List<StoriaContatto>  getReportOperatoreEsitoArchivazioneStoria(Long utente, ReportCriteria r,  Long esito, Long operatoreId, int start, int size) {
		ExtendedCriteria criteria = getReportOperatoreEsitoArchivazioneStoriaCriteria(utente, r, esito, operatoreId);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int  getReportOperatoreEsitoArchivazioneStoriaSize(Long utente, ReportCriteria r,  Long esito, Long operatoreId) {
		ExtendedCriteria criteria = getReportOperatoreEsitoArchivazioneStoriaCriteria(utente, r, esito, operatoreId);
		return getListSize(criteria);
	}
	private ExtendedCriteria getReportOperatoreEsitoArchivazioneStoriaCriteria(Long utente, ReportCriteria r, Long esito, Long operatoreId) {
		if(r != null)  r = (ReportCriteria) r.loadCriteria();
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContatto.class);
		DetachedCriteria criteriaOp = criteria.createCriteria("operatore", "op");
		DetachedCriteria criteriaC = criteria.createCriteria("contatto");
		DetachedCriteria criteriaCA = criteriaC.createCriteria("campagna", "camp");
		if (ApplicationUtil.gtZero(esito)) {
			criteria.add(Restrictions.eq("archiviazione.id", esito));
		}
		if (ApplicationUtil.gtZero(utente)) {
			impostazioneDao.subCriteriaRoleSupervisore(utente, criteria, "camp", "op");
		}
		if(r != null) {
			DetachedCriteria criteriaLI = criteriaC.createCriteria("lista");
			ApplicationUtil.pannello(r.getPannello(), criteria);
			ApplicationUtil.dataIntervalD(r.getInizio(), r.getFine(), "data", criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, criteriaLI);
			if(ApplicationUtil.gtZero(r.getLista())) {
				criteriaLI.add(Restrictions.idEq(r.getLista()));
			}
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
			if(ApplicationUtil.ltZero(r.getCampagna())) {
				if(ApplicationUtil.gtZero(utente)) {
					if(ApplicationUtil.isTrue(sd)) {
						impostazioneDao.subCriteriaCampagnaSupervisore(utente, criteriaCA, "id");
					}
				}
			}
		}
		criteriaOp.add(Restrictions.idEq(operatoreId));
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, r);
	}
	public List<ReportListaEsito> getReportListaEsito(Utente user, ReportCriteria r) {
		if(r != null)  r = (ReportCriteria) r.loadCriteria(); 
		List<ReportListaEsito> report = new ArrayList<ReportListaEsito>();
		if(r == null || (r != null && r.getCerca())) {
			List<Dettaglio> esitiChiamata = getReportEsitiChiamata();
			List<Dettaglio> esitiArchiviazione = getReportEsitiArchiviazione(r);
			List<ReportListaEsitoResultSet> listeE = getReportListaEsitoResultset(user, r);
			List<ReportListaEsitoResultSet> dati = getReportListaEsitoResultsetDati(user, r);
			for(ReportListaEsitoResultSet listaE : listeE) {
				Lista lista = campagnaDao.getLista(listaE.getLista_id());
				ReportListaEsito report_op = new ReportListaEsito();
				report_op.setLista(lista);
				report_op.setTotaleChiamate(new Dettaglio(listaE.getChiamate(),null, null,null,null, null, null, null, null, null, null, null, null, null, null));
				List<Dettaglio> esitiChiamataDettaglio = new ArrayList<Dettaglio>();
				List<Dettaglio> esitiArchiviazioneDettaglio = new ArrayList<Dettaglio>();
				for(Dettaglio ec : esitiChiamata) {
					Dettaglio d;
					int i = dati.indexOf(new ReportListaEsitoResultSet(lista.getId(), ec.getEsito_id(), null));
					if(i >= 0) {
						ReportListaEsitoResultSet dato = dati.get(i);
						d = new Dettaglio(dato.getChiamate(),null, null,null,null, null, null, null, null, null, null, null, null, null, null);
						d.p_numero(dato.getChiamate(), listaE.getChiamate());
					}
					else d = new Dettaglio(true);
					d.setEsito_id(ec.getEsito_id());
					d.setEtichetta(ec.getEtichetta());
					esitiChiamataDettaglio.add(d);
				}
				for(Dettaglio ea : esitiArchiviazione) {
					Dettaglio d;
					int i = dati.indexOf(new ReportListaEsitoResultSet(lista.getId(), ConstantsEsito.ID_OK, ea.getEsito_id()));
					if(i >= 0) {
						ReportListaEsitoResultSet dato = dati.get(i);
						d = new Dettaglio(dato.getChiamate(),null, null,null,null, null, null, null, null, null, null, null, null, null, null);
						d.p_numero(dato.getChiamate(), listaE.getChiamate());
					}
					else d = new Dettaglio(true);
					d.setEsito_id(ea.getEsito_id());
					d.setEtichetta(ea.getEtichetta());
					esitiArchiviazioneDettaglio.add(d);
				}
				report_op.setEsitiContattoList(esitiChiamataDettaglio);
				report_op.setEsitiArchiviazioneList(esitiArchiviazioneDettaglio);
				report.add(report_op);
			}
		}
		return report;
	}
	public void exportReportListaEsito(Utente user, ReportCriteria r, DownloadBean downloadBean) {
		if(r != null)  r = (ReportCriteria) r.loadCriteria();
		if(r == null || (r != null && r.getCerca())) {
			int a = 1;
			List<Dettaglio> esitiChiamata = getReportEsitiChiamata();
			List<Dettaglio> esitiArchiviazione = getReportEsitiArchiviazione(r);
			List<ReportListaEsitoResultSet> listeE = getReportListaEsitoResultset(user, r);
			List<ReportListaEsitoResultSet> dati = getReportListaEsitoResultsetDati(user,r);
			Workbook wb = ApplicationUtil.workbookS();
			ExcelStyle s = new ExcelStyle(wb);
			List<Sheet> fogli = new ArrayList<Sheet>();
			fogli.add(wb.createSheet(ConstantsLanguage.NUMERO_CHIAMATE.stampa()));
			for(Sheet foglio : fogli) {
				int j = 0;
				Row riga = foglio.createRow(foglio.getPhysicalNumberOfRows());
				Cell cella = riga.createCell(j++, CellType.STRING);
				ApplicationUtil.headerCell(s, cella, ConstantsLanguage.LISTA.stampa());
				cella = riga.createCell(j++, CellType.STRING);
				ApplicationUtil.headerCell(s, cella, ConstantsLanguage.TOTALE.stampa());
				for(Dettaglio head : esitiChiamata) {
					cella = riga.createCell(j++, CellType.STRING);
					ApplicationUtil.headerCell(s, cella, head.getEtichetta());
				}
				for(Dettaglio head : esitiArchiviazione) {
					cella = riga.createCell(j++, CellType.STRING);
					ApplicationUtil.headerCell(s, cella, head.getEtichetta());
				}
				foglio.setAutoFilter(new CellRangeAddress(0, 0, 0, ApplicationUtil.size(esitiChiamata) + ApplicationUtil.size(esitiArchiviazione) + 1));
				foglio.createFreezePane(0, 1);
			}
			for(ReportListaEsitoResultSet listaE : listeE) {
				int j = 0;
				Lista lista = campagnaDao.getLista(listaE.getLista_id());
				Row riga[] = new Row[a];
				for(int i = 0; i < riga.length; i++) {
					riga[i] = fogli.get(i).createRow(fogli.get(i).getPhysicalNumberOfRows());
					ApplicationUtil.formatString(s, riga[i].createCell(j, CellType.STRING), lista.getDenominazione());
				}
				j++;
				ApplicationUtil.formatNumber(s, riga[0].createCell(j, CellType.NUMERIC), listaE.getChiamate());
				j++;
				for(Dettaglio ec : esitiChiamata) {
					int i = dati.indexOf(new ReportListaEsitoResultSet(lista.getId(), ec.getEsito_id(), null));
					ReportListaEsitoResultSet dato = new ReportListaEsitoResultSet();
					if(i >= 0) {
						dato = dati.get(i);
					}
					ApplicationUtil.formatNumber(s, riga[0].createCell(j, CellType.NUMERIC), dato.getChiamate());
					j++;
				}
				for(Dettaglio ea : esitiArchiviazione) {
					int i = dati.indexOf(new ReportListaEsitoResultSet(lista.getId(), ConstantsEsito.ID_OK, ea.getEsito_id()));
					ReportListaEsitoResultSet dato = new ReportListaEsitoResultSet();
					if(i >= 0) {
						dato = dati.get(i);
					}
					ApplicationUtil.formatNumber(s, riga[0].createCell(j, CellType.NUMERIC), dato.getChiamate());
					j++;
				}
			}
			try {
				String nome = ApplicationUtil.formatIntervallo(ConstantsLanguage.RIEPILOGO.stampa() + "_" + ConstantsLanguage.LISTE.stampa(), r.getInizio(), r.getFine()) + ConstantsMap.xlsx; 


				OutputStream out = downloadBean.outputStream();
				downloadBean.filename(nome);

				String p = ConstantsGlobal.secretE;
				ApplicationUtil.passwordXLS(wb, out, p);
				downloadBean.close();

			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		}
	}
	public void exportReportListaEsito(Integer tipo, OutputStream result) throws Exception {
		int a = 1;
		Date dataInizio = ApplicationUtil.dataInizio(tipo, ApplicationUtil.date());
		ReportCriteria r = new ReportCriteria();
		r.setIntervallo(dataInizio, null);
		List<Dettaglio> esitiChiamata = getReportEsitiChiamata();
		List<Dettaglio> esitiArchiviazione = getReportEsitiArchiviazione(null);
		List<ReportListaEsitoResultSet> listeE = getReportListaEsitoResultset(null, r);
		List<ReportListaEsitoResultSet> dati = getReportListaEsitoResultsetDati(null, r);
		Workbook wb = ApplicationUtil.workbookS();
		ExcelStyle s = new ExcelStyle(wb);
		List<Sheet> fogli = new ArrayList<Sheet>();
		fogli.add(wb.createSheet(ConstantsLanguage.NUMERO_CHIAMATE.stampa()));
		for(Sheet foglio : fogli) {
			int j = 0;
			Row riga = foglio.createRow(foglio.getPhysicalNumberOfRows());
			Cell cella = riga.createCell(j++, CellType.STRING);
			ApplicationUtil.headerCell(s, cella, ConstantsLanguage.LISTA.stampa());
			cella = riga.createCell(j++, CellType.STRING);
			ApplicationUtil.headerCell(s, cella, ConstantsLanguage.TOTALE.stampa());
			for(Dettaglio head : esitiChiamata) {
				cella = riga.createCell(j++, CellType.STRING);
				ApplicationUtil.headerCell(s, cella, head.getEtichetta());
			}
			for(Dettaglio head : esitiArchiviazione) {
				cella = riga.createCell(j++, CellType.STRING);
				ApplicationUtil.headerCell(s, cella, head.getEtichetta());
			}
			foglio.setAutoFilter(new CellRangeAddress(0, 0, 0, ApplicationUtil.size(esitiChiamata) + ApplicationUtil.size(esitiArchiviazione) + 1));
			foglio.createFreezePane(0, 1);
		}
		for(ReportListaEsitoResultSet listaE : listeE) {
			int j = 0;
			Lista lista = campagnaDao.getLista(listaE.getLista_id());
			Row riga[] = new Row[a];
			for(int i = 0; i < riga.length; i++) {
				riga[i] = fogli.get(i).createRow(fogli.get(i).getPhysicalNumberOfRows());
				ApplicationUtil.formatString(s, riga[i].createCell(j, CellType.STRING), lista.getDenominazione());
			}
			j++;
			ApplicationUtil.formatNumber(s, riga[0].createCell(j, CellType.NUMERIC), listaE.getChiamate());
			j++;
			for(Dettaglio ec : esitiChiamata) {
				int i = dati.indexOf(new ReportListaEsitoResultSet(lista.getId(), ec.getEsito_id(), null));
				ReportListaEsitoResultSet dato = new ReportListaEsitoResultSet();
				if(i >= 0) {
					dato = dati.get(i);
				}
				ApplicationUtil.formatNumber(s, riga[0].createCell(j, CellType.NUMERIC), dato.getChiamate());
				j++;
			}
			for(Dettaglio ea : esitiArchiviazione) {
				int i = dati.indexOf(new ReportListaEsitoResultSet(lista.getId(), ConstantsEsito.ID_OK, ea.getEsito_id()));
				ReportListaEsitoResultSet dato = new ReportListaEsitoResultSet();
				if(i >= 0) {
					dato = dati.get(i);
				}
				ApplicationUtil.formatNumber(s, riga[0].createCell(j, CellType.NUMERIC), dato.getChiamate());
				j++;
			}
		}
		String p = ConstantsGlobal.secretE;
		ApplicationUtil.passwordXLS(wb, result, p);
	}
	public void exportReportListaEsitoStoria(Utente user, ReportCriteria r, DownloadBean downloadBean) {
		if(r != null)  r = (ReportCriteria) r.loadCriteria();
		if(r == null || (r != null && r.getCerca())) {
			boolean formatta = impostazioneDao.getFormattaTempi();
			int a = 15;
			List<Dettaglio> esitiChiamata = getReportEsitiChiamata();
			List<Dettaglio> esitiArchiviazione = getReportEsitiArchiviazione(r);
			List<ReportListaEsitoResultSet> listeE = getReportListaEsitoStoriaResultset(user, r);
			List<ReportListaEsitoResultSet> dati = getReportListaEsitoStoriaResultsetDati(user, r);
			Workbook wb = ApplicationUtil.workbookS();
			ExcelStyle s = new ExcelStyle(wb);
			List<Sheet> fogli = new ArrayList<Sheet>();
			fogli.add(wb.createSheet(ConstantsLanguage.NUMERO_CHIAMATE.stampa()));
			fogli.add(wb.createSheet(ConstantsLanguage.TEMPO_TOTH.stampa()));
			fogli.add(wb.createSheet(ConstantsLanguage.MEDIA_TOTH.stampa()));
			fogli.add(wb.createSheet(ConstantsLanguage.TEMPO_CONV_EFFH.stampa()));
			fogli.add(wb.createSheet(ConstantsLanguage.MEDIA_CONV_EFFH.stampa()));
			fogli.add(wb.createSheet(ConstantsLanguage.TEMPO_CONV_TOTH.stampa()));
			fogli.add(wb.createSheet(ConstantsLanguage.MEDIA_CONV_TOTH.stampa()));
			fogli.add(wb.createSheet(ConstantsLanguage.TEMPO_POSTH.stampa()));
			fogli.add(wb.createSheet(ConstantsLanguage.MEDIA_POSTH.stampa()));
			fogli.add(wb.createSheet(ConstantsLanguage.TEMPO_PREH.stampa()));
			fogli.add(wb.createSheet(ConstantsLanguage.MEDIA_PREH.stampa()));
			fogli.add(wb.createSheet(ConstantsLanguage.TEMPO_RECH.stampa()));
			fogli.add(wb.createSheet(ConstantsLanguage.MEDIA_RECH.stampa()));
			fogli.add(wb.createSheet(ConstantsLanguage.TEMPO_ATTESAH.stampa()));
			fogli.add(wb.createSheet(ConstantsLanguage.MEDIA_ATTESAH.stampa()));
			for(Sheet foglio : fogli) {
				int j = 0;
				Row riga = foglio.createRow(foglio.getPhysicalNumberOfRows());
				Cell cella = riga.createCell(j++, CellType.STRING);
				ApplicationUtil.headerCell(s, cella, ConstantsLanguage.LISTA.stampa());
				cella = riga.createCell(j++, CellType.STRING);
				ApplicationUtil.headerCell(s, cella, ConstantsLanguage.TOTALE.stampa());
				for(Dettaglio head : esitiChiamata) {
					cella = riga.createCell(j++, CellType.STRING);
					ApplicationUtil.headerCell(s, cella, head.getEtichetta());
				}
				for(Dettaglio head : esitiArchiviazione) {
					cella = riga.createCell(j++, CellType.STRING);
					ApplicationUtil.headerCell(s, cella, head.getEtichetta());
				}
				foglio.setAutoFilter(new CellRangeAddress(0, 0, 0, ApplicationUtil.size(esitiChiamata) + ApplicationUtil.size(esitiArchiviazione) + 1));
				foglio.createFreezePane(0, 1);
			}
			for(ReportListaEsitoResultSet listaE : listeE) {
				int j = 0;
				Lista lista = campagnaDao.getLista(listaE.getLista_id());
				Row riga[] = new Row[a];
				for(int i = 0; i < riga.length; i++) {
					riga[i] = fogli.get(i).createRow(fogli.get(i).getPhysicalNumberOfRows());
					ApplicationUtil.formatString(s, riga[i].createCell(j, CellType.STRING), lista.getDenominazione());
				}
				j++;
				ApplicationUtil.formatNumber(s, riga[0].createCell(j, CellType.NUMERIC), listaE.getChiamate());
				ApplicationUtil.formatTimestamp(formatta, s, riga[1].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), listaE.getDurata());
				ApplicationUtil.formatTimestamp(formatta, s, riga[2].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), listaE.getMedia());
				ApplicationUtil.formatTimestamp(formatta, s, riga[3].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), listaE.getDurata_conv_eff());
				ApplicationUtil.formatTimestamp(formatta, s, riga[4].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), listaE.getMedia_conv_eff());
				ApplicationUtil.formatTimestamp(formatta, s, riga[5].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), listaE.getDurata_conv_tot());
				ApplicationUtil.formatTimestamp(formatta, s, riga[6].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), listaE.getMedia_conv_tot());
				ApplicationUtil.formatTimestamp(formatta, s, riga[7].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), listaE.getDurata_post());
				ApplicationUtil.formatTimestamp(formatta, s, riga[8].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), listaE.getMedia_post());
				ApplicationUtil.formatTimestamp(formatta, s, riga[9].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), listaE.getDurata_pre());
				ApplicationUtil.formatTimestamp(formatta, s, riga[10].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), listaE.getMedia_pre());
				ApplicationUtil.formatTimestamp(formatta, s, riga[11].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), listaE.getDurata_rec());
				ApplicationUtil.formatTimestamp(formatta, s, riga[12].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), listaE.getMedia_rec());
				ApplicationUtil.formatTimestamp(formatta, s, riga[13].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), listaE.getDurata_attesa());
				ApplicationUtil.formatTimestamp(formatta, s, riga[14].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), listaE.getMedia_attesa());
				j++;
				for(Dettaglio ec : esitiChiamata) {
					int i = dati.indexOf(new ReportListaEsitoResultSet(lista.getId(), ec.getEsito_id(), null));
					ReportListaEsitoResultSet dato = new ReportListaEsitoResultSet();
					if(i >= 0) {
						dato = dati.get(i);
					}
					ApplicationUtil.formatNumber(s, riga[0].createCell(j, CellType.NUMERIC), dato.getChiamate());
					ApplicationUtil.formatTimestamp(formatta, s, riga[1].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getDurata());
					ApplicationUtil.formatTimestamp(formatta, s, riga[2].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getMedia());
					ApplicationUtil.formatTimestamp(formatta, s, riga[3].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getDurata_conv_eff());
					ApplicationUtil.formatTimestamp(formatta, s, riga[4].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getMedia_conv_eff());
					ApplicationUtil.formatTimestamp(formatta, s, riga[5].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getDurata_conv_tot());
					ApplicationUtil.formatTimestamp(formatta, s, riga[6].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getMedia_conv_tot());
					ApplicationUtil.formatTimestamp(formatta, s, riga[7].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getDurata_post());
					ApplicationUtil.formatTimestamp(formatta, s, riga[8].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getMedia_post());
					ApplicationUtil.formatTimestamp(formatta, s, riga[9].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getDurata_pre());
					ApplicationUtil.formatTimestamp(formatta, s, riga[10].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getMedia_pre());
					ApplicationUtil.formatTimestamp(formatta, s, riga[11].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getDurata_rec());
					ApplicationUtil.formatTimestamp(formatta, s, riga[12].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getMedia_rec());
					ApplicationUtil.formatTimestamp(formatta, s, riga[13].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getDurata_attesa());
					ApplicationUtil.formatTimestamp(formatta, s, riga[14].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getMedia_attesa());
					j++;
				}
				for(Dettaglio ea : esitiArchiviazione) {
					int i = dati.indexOf(new ReportListaEsitoResultSet(lista.getId(), ConstantsEsito.ID_OK, ea.getEsito_id()));
					ReportListaEsitoResultSet dato = new ReportListaEsitoResultSet();
					if(i >= 0) {
						dato = dati.get(i);
					}
					ApplicationUtil.formatNumber(s, riga[0].createCell(j, CellType.NUMERIC), dato.getChiamate());
					ApplicationUtil.formatTimestamp(formatta, s, riga[1].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getDurata());
					ApplicationUtil.formatTimestamp(formatta, s, riga[2].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getMedia());
					ApplicationUtil.formatTimestamp(formatta, s, riga[3].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getDurata_conv_eff());
					ApplicationUtil.formatTimestamp(formatta, s, riga[4].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getMedia_conv_eff());
					ApplicationUtil.formatTimestamp(formatta, s, riga[5].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getDurata_conv_tot());
					ApplicationUtil.formatTimestamp(formatta, s, riga[6].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getMedia_conv_tot());
					ApplicationUtil.formatTimestamp(formatta, s, riga[7].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getDurata_post());
					ApplicationUtil.formatTimestamp(formatta, s, riga[8].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getMedia_post());
					ApplicationUtil.formatTimestamp(formatta, s, riga[9].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getDurata_pre());
					ApplicationUtil.formatTimestamp(formatta, s, riga[10].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getMedia_pre());
					ApplicationUtil.formatTimestamp(formatta, s, riga[11].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getDurata_rec());
					ApplicationUtil.formatTimestamp(formatta, s, riga[12].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getMedia_rec());
					ApplicationUtil.formatTimestamp(formatta, s, riga[13].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getDurata_attesa());
					ApplicationUtil.formatTimestamp(formatta, s, riga[14].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getMedia_attesa());
					j++;
				}
			}
			try {
				String nome = ApplicationUtil.formatIntervallo(ConstantsLanguage.REPORT.stampa() + "_" + ConstantsLanguage.LISTE.stampa(), r.getInizio(), r.getFine()) + ConstantsMap.xlsx; 


				OutputStream out = downloadBean.outputStream();
				downloadBean.filename(nome);

				String p = ConstantsGlobal.secretE;
				ApplicationUtil.passwordXLS(wb, out, p);
				downloadBean.close();

			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		}
	}
	public void exportReportListaEsitoStoria(Integer tipo, OutputStream result) throws Exception {
		boolean formatta = impostazioneDao.getFormattaTempi();
		int a = 15;
		Date dataInizio = ApplicationUtil.dataInizio(tipo, ApplicationUtil.date());
		ReportCriteria r = new ReportCriteria();
		r.setIntervallo(dataInizio, null);
		List<Dettaglio> esitiChiamata = getReportEsitiChiamata();
		List<Dettaglio> esitiArchiviazione = getReportEsitiArchiviazione(null);
		List<ReportListaEsitoResultSet> listeE = getReportListaEsitoStoriaResultset(null, r);
		List<ReportListaEsitoResultSet> dati = getReportListaEsitoStoriaResultsetDati(null, r);
		Workbook wb = ApplicationUtil.workbookS();
		ExcelStyle s = new ExcelStyle(wb);
		List<Sheet> fogli = new ArrayList<Sheet>();
		fogli.add(wb.createSheet(ConstantsLanguage.NUMERO_CHIAMATE.stampa()));
		fogli.add(wb.createSheet(ConstantsLanguage.TEMPO_TOTH.stampa()));
		fogli.add(wb.createSheet(ConstantsLanguage.MEDIA_TOTH.stampa()));
		fogli.add(wb.createSheet(ConstantsLanguage.TEMPO_CONV_EFFH.stampa()));
		fogli.add(wb.createSheet(ConstantsLanguage.MEDIA_CONV_EFFH.stampa()));
		fogli.add(wb.createSheet(ConstantsLanguage.TEMPO_CONV_TOTH.stampa()));
		fogli.add(wb.createSheet(ConstantsLanguage.MEDIA_CONV_TOTH.stampa()));
		fogli.add(wb.createSheet(ConstantsLanguage.TEMPO_POSTH.stampa()));
		fogli.add(wb.createSheet(ConstantsLanguage.MEDIA_POSTH.stampa()));
		fogli.add(wb.createSheet(ConstantsLanguage.TEMPO_PREH.stampa()));
		fogli.add(wb.createSheet(ConstantsLanguage.MEDIA_PREH.stampa()));
		fogli.add(wb.createSheet(ConstantsLanguage.TEMPO_RECH.stampa()));
		fogli.add(wb.createSheet(ConstantsLanguage.MEDIA_RECH.stampa()));
		fogli.add(wb.createSheet(ConstantsLanguage.TEMPO_ATTESAH.stampa()));
		fogli.add(wb.createSheet(ConstantsLanguage.MEDIA_ATTESAH.stampa()));
		for(Sheet foglio : fogli) {
			int j = 0;
			Row riga = foglio.createRow(foglio.getPhysicalNumberOfRows());
			Cell cella = riga.createCell(j++, CellType.STRING);
			ApplicationUtil.headerCell(s, cella, ConstantsLanguage.LISTA.stampa());
			cella = riga.createCell(j++, CellType.STRING);
			ApplicationUtil.headerCell(s, cella, ConstantsLanguage.TOTALE.stampa());
			for(Dettaglio head : esitiChiamata) {
				cella = riga.createCell(j++, CellType.STRING);
				ApplicationUtil.headerCell(s, cella, head.getEtichetta());
			}
			for(Dettaglio head : esitiArchiviazione) {
				cella = riga.createCell(j++, CellType.STRING);
				ApplicationUtil.headerCell(s, cella, head.getEtichetta());
			}
			foglio.setAutoFilter(new CellRangeAddress(0, 0, 0, ApplicationUtil.size(esitiChiamata) + ApplicationUtil.size(esitiArchiviazione) + 1));
			foglio.createFreezePane(0, 1);
		}
		for(ReportListaEsitoResultSet listaE : listeE) {
			int j = 0;
			Lista lista = campagnaDao.getLista(listaE.getLista_id());
			Row riga[] = new Row[a];
			for(int i = 0; i < riga.length; i++) {
				riga[i] = fogli.get(i).createRow(fogli.get(i).getPhysicalNumberOfRows());
				ApplicationUtil.formatString(s, riga[i].createCell(j, CellType.STRING), lista.getDenominazione());
			}
			j++;
			ApplicationUtil.formatNumber(s, riga[0].createCell(j, CellType.NUMERIC), listaE.getChiamate());
			ApplicationUtil.formatTimestamp(formatta, s, riga[1].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), listaE.getDurata());
			ApplicationUtil.formatTimestamp(formatta, s, riga[2].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), listaE.getMedia());
			ApplicationUtil.formatTimestamp(formatta, s, riga[3].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), listaE.getDurata_conv_eff());
			ApplicationUtil.formatTimestamp(formatta, s, riga[4].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), listaE.getMedia_conv_eff());
			ApplicationUtil.formatTimestamp(formatta, s, riga[5].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), listaE.getDurata_conv_tot());
			ApplicationUtil.formatTimestamp(formatta, s, riga[6].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), listaE.getMedia_conv_tot());
			ApplicationUtil.formatTimestamp(formatta, s, riga[7].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), listaE.getDurata_post());
			ApplicationUtil.formatTimestamp(formatta, s, riga[8].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), listaE.getMedia_post());
			ApplicationUtil.formatTimestamp(formatta, s, riga[9].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), listaE.getDurata_pre());
			ApplicationUtil.formatTimestamp(formatta, s, riga[10].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), listaE.getMedia_pre());
			ApplicationUtil.formatTimestamp(formatta, s, riga[11].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), listaE.getDurata_rec());
			ApplicationUtil.formatTimestamp(formatta, s, riga[12].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), listaE.getMedia_rec());
			ApplicationUtil.formatTimestamp(formatta, s, riga[13].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), listaE.getDurata_attesa());
			ApplicationUtil.formatTimestamp(formatta, s, riga[14].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), listaE.getMedia_attesa());
			j++;
			for(Dettaglio ec : esitiChiamata) {
				int i = dati.indexOf(new ReportListaEsitoResultSet(lista.getId(), ec.getEsito_id(), null));
				ReportListaEsitoResultSet dato = new ReportListaEsitoResultSet();
				if(i >= 0) {
					dato = dati.get(i);
				}
				ApplicationUtil.formatNumber(s, riga[0].createCell(j, CellType.NUMERIC), dato.getChiamate());
				ApplicationUtil.formatTimestamp(formatta, s, riga[1].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getDurata());
				ApplicationUtil.formatTimestamp(formatta, s, riga[2].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getMedia());
				ApplicationUtil.formatTimestamp(formatta, s, riga[3].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getDurata_conv_eff());
				ApplicationUtil.formatTimestamp(formatta, s, riga[4].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getMedia_conv_eff());
				ApplicationUtil.formatTimestamp(formatta, s, riga[5].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getDurata_conv_tot());
				ApplicationUtil.formatTimestamp(formatta, s, riga[6].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getMedia_conv_tot());
				ApplicationUtil.formatTimestamp(formatta, s, riga[7].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getDurata_post());
				ApplicationUtil.formatTimestamp(formatta, s, riga[8].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getMedia_post());
				ApplicationUtil.formatTimestamp(formatta, s, riga[9].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getDurata_pre());
				ApplicationUtil.formatTimestamp(formatta, s, riga[10].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getMedia_pre());
				ApplicationUtil.formatTimestamp(formatta, s, riga[11].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getDurata_rec());
				ApplicationUtil.formatTimestamp(formatta, s, riga[12].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getMedia_rec());
				ApplicationUtil.formatTimestamp(formatta, s, riga[13].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getDurata_attesa());
				ApplicationUtil.formatTimestamp(formatta, s, riga[14].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getMedia_attesa());
				j++;
			}
			for(Dettaglio ea : esitiArchiviazione) {
				int i = dati.indexOf(new ReportListaEsitoResultSet(lista.getId(), ConstantsEsito.ID_OK, ea.getEsito_id()));
				ReportListaEsitoResultSet dato = new ReportListaEsitoResultSet();
				if(i >= 0) {
					dato = dati.get(i);
				}
				ApplicationUtil.formatNumber(s, riga[0].createCell(j, CellType.NUMERIC), dato.getChiamate());
				ApplicationUtil.formatTimestamp(formatta, s, riga[1].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getDurata());
				ApplicationUtil.formatTimestamp(formatta, s, riga[2].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getMedia());
				ApplicationUtil.formatTimestamp(formatta, s, riga[3].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getDurata_conv_eff());
				ApplicationUtil.formatTimestamp(formatta, s, riga[4].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getMedia_conv_eff());
				ApplicationUtil.formatTimestamp(formatta, s, riga[5].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getDurata_conv_tot());
				ApplicationUtil.formatTimestamp(formatta, s, riga[6].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getMedia_conv_tot());
				ApplicationUtil.formatTimestamp(formatta, s, riga[7].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getDurata_post());
				ApplicationUtil.formatTimestamp(formatta, s, riga[8].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getMedia_post());
				ApplicationUtil.formatTimestamp(formatta, s, riga[9].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getDurata_pre());
				ApplicationUtil.formatTimestamp(formatta, s, riga[10].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getMedia_pre());
				ApplicationUtil.formatTimestamp(formatta, s, riga[11].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getDurata_rec());
				ApplicationUtil.formatTimestamp(formatta, s, riga[12].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getMedia_rec());
				ApplicationUtil.formatTimestamp(formatta, s, riga[13].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getDurata_attesa());
				ApplicationUtil.formatTimestamp(formatta, s, riga[14].createCell(j, CellType.NUMERIC), ConstantsFormat.seconds.stampa(), dato.getMedia_attesa());
				j++;
			}
		}
		String p = ConstantsGlobal.secretE;
		ApplicationUtil.passwordXLS(wb, result, p);
	}
	@SuppressWarnings("unused")
	private ExtendedCriteria getReportListaEsitoEsitoTotaleStoriaCriteria(Long utente,  ReportCriteria r, Long listaEId) {
		if(r != null)  r = (ReportCriteria) r.loadCriteria();
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContatto.class);
		DetachedCriteria criteriaOp = criteria.createCriteria("operatore", "op");
		DetachedCriteria criteriaC = criteria.createCriteria("contatto");
		DetachedCriteria criteriaCA = criteriaC.createCriteria("campagna", "camp");
		if (ApplicationUtil.gtZero(utente)) {
			impostazioneDao.subCriteriaRoleSupervisore(utente, criteria, "camp", "op");
		}
		if(r != null) {
			ApplicationUtil.pannello(r.getPannello(), criteria);
			ApplicationUtil.dataIntervalD(r.getInizio(), r.getFine(), "data", criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, null);
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
			if(ApplicationUtil.ltZero(r.getCampagna())) {
				if(ApplicationUtil.gtZero(utente)) {
					if(ApplicationUtil.isTrue(sd)) {
						impostazioneDao.subCriteriaCampagnaSupervisore(utente, criteriaCA, "id");
					}
				}
			}
		}
		criteriaC.add(Restrictions.eq("lista.id",listaEId));
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, r);
	}
	private ExtendedCriteria getReportListaEsitoEsitoTotaleCriteria(Long utente,  ReportCriteria r, Long listaEId) {
		if(r != null)  r = (ReportCriteria) r.loadCriteria();
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class);
		DetachedCriteria criteriaCA = criteria.createCriteria("campagna");
		if(r != null) {
			ApplicationUtil.intervallo(ApplicationUtil.isTrue(r.getInserimento()), r.getInizio(), r.getFine(), criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, null);
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
			if(ApplicationUtil.ltZero(r.getCampagna())) {
				if(ApplicationUtil.gtZero(utente)) {
					if(ApplicationUtil.isTrue(sd)) {
						impostazioneDao.subCriteriaCampagnaSupervisore(utente, criteriaCA, "id");
					}
				}
			}
		}
		criteria.add(Restrictions.eq("lista.id",listaEId));
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, r);
	}
	@SuppressWarnings("unchecked")
	public List<StoriaContatto>  getReportListaEsitoEsitoTotaleStoria(Long user,ReportCriteria r, Long listaEId, int start, int size) {
		ExtendedCriteria criteria = getReportListaEsitoEsitoTotaleStoriaCriteria(user, r, listaEId);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	@SuppressWarnings("unchecked")
	public List<Contatto>  getReportListaEsitoEsitoTotale(Long user,ReportCriteria r, Long listaEId, int start, int size) {
		ExtendedCriteria criteria = getReportListaEsitoEsitoTotaleCriteria(user, r, listaEId);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int  getReportListaEsitoEsitoTotaleSize(Long user,  ReportCriteria r, Long listaEId) {
		ExtendedCriteria criteria = getReportListaEsitoEsitoTotaleCriteria(user, r, listaEId);
		return getListSize(criteria);
	}
	public int  getReportListaEsitoEsitoTotaleStoriaSize(Long user,  ReportCriteria r, Long listaEId) {
		ExtendedCriteria criteria = getReportListaEsitoEsitoTotaleStoriaCriteria(user, r, listaEId);
		return getListSize(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<StoriaContatto>  getReportListaEsitoEsitoChiamataStoria(Long utente, ReportCriteria r,  Long esito, Long listaEId, int start, int size) {
		ExtendedCriteria criteria = getReportListaEsitoEsitoChiamataStoriaCriteria(utente, r, esito, listaEId);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int  getReportListaEsitoEsitoChiamataStoriaSize(Long utente, ReportCriteria r,  Long esito, Long listaEId) {
		ExtendedCriteria criteria = getReportListaEsitoEsitoChiamataStoriaCriteria(utente, r, esito, listaEId);
		return getListSize(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Contatto>  getReportListaEsitoEsitoChiamata(Long utente, ReportCriteria r,  Long esito, Long listaEId, int start, int size) {
		ExtendedCriteria criteria = getReportListaEsitoEsitoChiamataCriteria(utente, r, esito, listaEId);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int  getReportListaEsitoEsitoChiamataSize(Long utente, ReportCriteria r,  Long esito, Long listaEId) {
		ExtendedCriteria criteria = getReportListaEsitoEsitoChiamataCriteria(utente, r, esito, listaEId);
		return getListSize(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Contatto>  getReportListaEsitoEsitoArchivazione(Long utente, ReportCriteria r,  Long esito, Long listaEId, int start, int size) {
		ExtendedCriteria criteria = getReportListaEsitoEsitoArchivazioneCriteria(utente, r, esito, listaEId);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int  getReportListaEsitoEsitoArchivazioneSize(Long utente, ReportCriteria r,  Long esito, Long listaEId) {
		ExtendedCriteria criteria = getReportListaEsitoEsitoArchivazioneCriteria(utente, r, esito, listaEId);
		return getListSize(criteria);
	}
	@SuppressWarnings("unused")
	private ExtendedCriteria getReportListaEsitoEsitoChiamataStoriaCriteria(Long utente, ReportCriteria r,  Long esito, Long listaEId) {
		if(r != null)  r = (ReportCriteria) r.loadCriteria();
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContatto.class);
		DetachedCriteria criteriaOp = criteria.createCriteria("operatore", "op");
		DetachedCriteria criteriaC = criteria.createCriteria("contatto");
		DetachedCriteria criteriaCA = criteriaC.createCriteria("campagna", "camp");
		if (ApplicationUtil.gtZero(esito)) {
			criteria.add(Restrictions.eq("esito.id", esito));
		}
		if (ApplicationUtil.gtZero(utente)) {
			impostazioneDao.subCriteriaRoleSupervisore(utente, criteria, "camp", "op");
		}
		if(r != null) {
			ApplicationUtil.pannello(r.getPannello(), criteria);
			ApplicationUtil.dataIntervalD(r.getInizio(), r.getFine(), "data", criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, null);
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
			if(ApplicationUtil.ltZero(r.getCampagna())) {
				if(ApplicationUtil.gtZero(utente)) {
					if(ApplicationUtil.isTrue(sd)) {
						impostazioneDao.subCriteriaCampagnaSupervisore(utente, criteriaCA, "id");
					}
				}
			}
		}
		criteriaC.add(Restrictions.eq("lista.id",listaEId));
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, r);
	}
	private ExtendedCriteria getReportListaEsitoEsitoChiamataCriteria(Long utente, ReportCriteria r,  Long esito, Long listaEId) {
		if(r != null)  r = (ReportCriteria) r.loadCriteria();
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class);
		DetachedCriteria criteriaCA = criteria.createCriteria("campagna");
		if (ApplicationUtil.gtZero(esito)) {
			criteria.add(Restrictions.eq("esitoChiamata.id", esito));
		}
		if(r != null) {
			ApplicationUtil.intervallo(ApplicationUtil.isTrue(r.getInserimento()), r.getInizio(), r.getFine(), criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, null);
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
			if(ApplicationUtil.ltZero(r.getCampagna())) {
				if(ApplicationUtil.gtZero(utente)) {
					if(ApplicationUtil.isTrue(sd)) {
						impostazioneDao.subCriteriaCampagnaSupervisore(utente, criteriaCA, "id");
					}
				}
			}
		}
		criteria.add(Restrictions.eq("lista.id",listaEId));
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, r);
	}
	private ExtendedCriteria getReportListaEsitoEsitoArchivazioneCriteria(Long utente, ReportCriteria r,  Long esito, Long listaEId) {
		if(r != null)  r = (ReportCriteria) r.loadCriteria();
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class);
		DetachedCriteria criteriaCA = criteria.createCriteria("campagna");
		if (ApplicationUtil.gtZero(esito)) {
			criteria.add(Restrictions.eq("esitoArchiviazione.id", esito));
		}
		if(r != null) {
			ApplicationUtil.intervallo(ApplicationUtil.isTrue(r.getInserimento()), r.getInizio(), r.getFine(), criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, null);
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
			if(ApplicationUtil.ltZero(r.getCampagna())) {
				if(ApplicationUtil.gtZero(utente)) {
					if(ApplicationUtil.isTrue(sd)) {
						impostazioneDao.subCriteriaCampagnaSupervisore(utente, criteriaCA, "id");
					}
				}
			}
		}
		criteria.add(Restrictions.eq("lista.id",listaEId));
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, r);
	}
	@SuppressWarnings("unchecked")
	public List<StoriaContatto>  getReportListaEsitoEsitoArchivazioneStoria(Long utente, ReportCriteria r,  Long esito, Long listaEId, int start, int size) {
		ExtendedCriteria criteria = getReportListaEsitoEsitoArchivazioneStoriaCriteria(utente, r, esito, listaEId);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int  getReportListaEsitoEsitoArchivazioneStoriaSize(Long utente, ReportCriteria r,  Long esito, Long listaEId) {
		ExtendedCriteria criteria = getReportListaEsitoEsitoArchivazioneStoriaCriteria(utente, r, esito, listaEId);
		return getListSize(criteria);
	}
	@SuppressWarnings("unused")
	private ExtendedCriteria getReportListaEsitoEsitoArchivazioneStoriaCriteria(Long utente, ReportCriteria r, Long esito, Long listaEId) {
		if(r != null)  r = (ReportCriteria) r.loadCriteria();
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContatto.class);
		DetachedCriteria criteriaOp = criteria.createCriteria("operatore", "op");
		DetachedCriteria criteriaC = criteria.createCriteria("contatto");
		DetachedCriteria criteriaCA = criteriaC.createCriteria("campagna", "camp");
		if (ApplicationUtil.gtZero(esito)) {
			criteria.add(Restrictions.eq("archiviazione.id", esito));
		}
		if (ApplicationUtil.gtZero(utente)) {
			impostazioneDao.subCriteriaRoleSupervisore(utente, criteria, "camp", "op");
		}
		if(r != null) {
			ApplicationUtil.pannello(r.getPannello(), criteria);
			ApplicationUtil.dataIntervalD(r.getInizio(), r.getFine(), "data", criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, null);
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
			if(ApplicationUtil.ltZero(r.getCampagna())) {
				if(ApplicationUtil.gtZero(utente)) {
					if(ApplicationUtil.isTrue(sd)) {
						impostazioneDao.subCriteriaCampagnaSupervisore(utente, criteriaCA, "id");
					}
				}
			}
		}
		criteriaC.add(Restrictions.eq("lista.id",listaEId));
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, r);
	}
	public List<ReportListaEsito> getReportListaEsitoStoria(Utente user, ReportCriteria r) {
		if(r != null)  r = (ReportCriteria) r.loadCriteria(); 
		List<ReportListaEsito> report = new ArrayList<ReportListaEsito>();
		if(r == null || (r != null && r.getCerca())) {
			List<Dettaglio> esitiChiamata = getReportEsitiChiamata();
			List<Dettaglio> esitiArchiviazione = getReportEsitiArchiviazione(r);
			List<ReportListaEsitoResultSet> listeE = getReportListaEsitoStoriaResultset(user, r);
			List<ReportListaEsitoResultSet> dati = getReportListaEsitoStoriaResultsetDati(user, r);
			for(ReportListaEsitoResultSet listaE : listeE) {
				Lista lista = campagnaDao.getLista(listaE.getLista_id());
				ReportListaEsito report_op = new ReportListaEsito();
				report_op.setLista(lista);
				report_op.setTotaleChiamate(new Dettaglio(listaE.getChiamate(), ApplicationUtil.format(listaE.getDurata()), ApplicationUtil.format(listaE.getMedia()),  ApplicationUtil.format(listaE.getDurata_conv_eff()), ApplicationUtil.format(listaE.getMedia_conv_eff()), ApplicationUtil.format(listaE.getDurata_conv_tot()), ApplicationUtil.format(listaE.getMedia_conv_tot()), ApplicationUtil.format(listaE.getDurata_post()), ApplicationUtil.format(listaE.getMedia_post()), ApplicationUtil.format(listaE.getDurata_pre()), ApplicationUtil.format(listaE.getMedia_pre()), ApplicationUtil.format(listaE.getDurata_rec()), ApplicationUtil.format(listaE.getMedia_rec()), ApplicationUtil.format(listaE.getDurata_attesa()), ApplicationUtil.format(listaE.getMedia_attesa())));
				List<Dettaglio> esitiChiamataDettaglio = new ArrayList<Dettaglio>();
				List<Dettaglio> esitiArchiviazioneDettaglio = new ArrayList<Dettaglio>();
				for(Dettaglio ec : esitiChiamata) {
					Dettaglio d;
					int i = dati.indexOf(new ReportListaEsitoResultSet(lista.getId(), ec.getEsito_id(), null));
					if(i >= 0) {
						ReportListaEsitoResultSet dato = dati.get(i);
						d = new Dettaglio(dato.getChiamate(), ApplicationUtil.format(dato.getDurata()), ApplicationUtil.format(dato.getMedia()),  ApplicationUtil.format(dato.getDurata_conv_eff()), ApplicationUtil.format(dato.getMedia_conv_eff()), ApplicationUtil.format(dato.getDurata_conv_tot()), ApplicationUtil.format(dato.getMedia_conv_tot()), ApplicationUtil.format(dato.getDurata_post()), ApplicationUtil.format(dato.getMedia_post()), ApplicationUtil.format(dato.getDurata_pre()), ApplicationUtil.format(dato.getMedia_pre()), ApplicationUtil.format(dato.getDurata_rec()), ApplicationUtil.format(dato.getMedia_rec()), ApplicationUtil.format(dato.getDurata_attesa()), ApplicationUtil.format(dato.getMedia_attesa()));
						d.p_numero(dato.getChiamate(), listaE.getChiamate());
						d.p_tempo(dato.getDurata(), listaE.getDurata());
					}
					else d = new Dettaglio(true);
					d.setEsito_id(ec.getEsito_id());
					d.setEtichetta(ec.getEtichetta());
					esitiChiamataDettaglio.add(d);
				}
				for(Dettaglio ea : esitiArchiviazione) {
					Dettaglio d;
					int i = dati.indexOf(new ReportListaEsitoResultSet(lista.getId(), ConstantsEsito.ID_OK, ea.getEsito_id()));
					if(i >= 0) {
						ReportListaEsitoResultSet dato = dati.get(i);
						d = new Dettaglio(dato.getChiamate(), ApplicationUtil.format(dato.getDurata()), ApplicationUtil.format(dato.getMedia()),  ApplicationUtil.format(dato.getDurata_conv_eff()), ApplicationUtil.format(dato.getMedia_conv_eff()), ApplicationUtil.format(dato.getDurata_conv_tot()), ApplicationUtil.format(dato.getMedia_conv_tot()), ApplicationUtil.format(dato.getDurata_post()), ApplicationUtil.format(dato.getMedia_post()), ApplicationUtil.format(dato.getDurata_pre()), ApplicationUtil.format(dato.getMedia_pre()), ApplicationUtil.format(dato.getDurata_rec()), ApplicationUtil.format(dato.getMedia_rec()), ApplicationUtil.format(dato.getDurata_attesa()), ApplicationUtil.format(dato.getMedia_attesa()));
						d.p_numero(dato.getChiamate(), listaE.getChiamate());
						d.p_tempo(dato.getDurata(), listaE.getDurata());
					}
					else d = new Dettaglio(true);
					d.setEsito_id(ea.getEsito_id());
					d.setEtichetta(ea.getEtichetta());
					esitiArchiviazioneDettaglio.add(d);
				}
				report_op.setEsitiContattoList(esitiChiamataDettaglio);
				report_op.setEsitiArchiviazioneList(esitiArchiviazioneDettaglio);
				report.add(report_op);
			}
		}
		return report;
	}
	public ReportContattiOdierni getContattiOdierniOperatoreReport(Long operatore, Long supervisore, ReportCriteria r) {
		if(r != null)  r = (ReportCriteria) r.loadCriteria(); 
		ReportContattiOdierni rcoo = new ReportContattiOdierni();
		if(r == null || (r != null && r.getCerca())) {
			rcoo.setArchiviazioni(getArchiviazioniOdierneOperatoreReport(operatore, supervisore, r));
			rcoo.setContatti(getContattiOdierneOperatoreReport(operatore, supervisore, r));
			rcoo.setConversazioni(getConversazioniOdierneOperatoreReport(operatore, supervisore, r));
			rcoo.setAppuntamenti(getAppuntamentiOdierneOperatoreReport(operatore, supervisore, r));
			rcoo.setContratti(getContrattiOdierneOperatoreReport(operatore, supervisore, r));
		}
		return rcoo;
	}
	@SuppressWarnings("unchecked")
	public List<StoriaContatto>  getReportOperatoreArchiviazioni(Long user,  ReportCriteria r, Long operatoreId, int start, int size) {
		ExtendedCriteria criteria = getReportOperatoreArchiviazioniCriteria(user,  r, operatoreId);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int  getReportOperatoreArchiviazioniSize(Long user,  ReportCriteria r, Long operatoreId) {
		ExtendedCriteria criteria = getReportOperatoreArchiviazioniCriteria(user, r, operatoreId);
		return getListSize(criteria);
	}
	private ExtendedCriteria getReportOperatoreArchiviazioniCriteria(Long utente,  ReportCriteria r, Long operatoreId) {
		if(r != null)  r = (ReportCriteria) r.loadCriteria();
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContatto.class);
		DetachedCriteria criteriaOp = criteria.createCriteria("operatore", "op");
		DetachedCriteria criteriaC = criteria.createCriteria("contatto");
		DetachedCriteria criteriaCA = criteriaC.createCriteria("campagna", "camp");
		if (ApplicationUtil.gtZero(utente)) {
			impostazioneDao.subCriteriaRoleSupervisore(utente, criteria, "camp", "op");
		}
		if(r != null) {
			DetachedCriteria criteriaLI = criteriaC.createCriteria("lista");
			ApplicationUtil.pannello(r.getPannello(), criteria);
			ApplicationUtil.dataIntervalD(r.getInizio(), r.getFine(), "data", criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, criteriaLI);
			if(ApplicationUtil.gtZero(r.getLista())) {
				criteriaLI.add(Restrictions.idEq(r.getLista()));
			}
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
			if(ApplicationUtil.ltZero(r.getCampagna())) {
				if(ApplicationUtil.gtZero(utente)) {
					if(ApplicationUtil.isTrue(sd)) {
						impostazioneDao.subCriteriaCampagnaSupervisore(utente, criteriaCA, "id");
					}
				}
			}
		}
		criteriaOp.add(Restrictions.idEq(operatoreId));
		criteria.add(Restrictions.isNotNull("archiviazione.id"));
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, r);
	}
	@SuppressWarnings("unchecked")
	public List<StoriaContatto>  getReportListaEsitoArchiviazioni(Long user,  ReportCriteria r, Long listaId, int start, int size) {
		ExtendedCriteria criteria = getReportListaEsitoArchiviazioniCriteria(user,  r, listaId);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int  getReportListaEsitoArchiviazioniSize(Long user,  ReportCriteria r, Long listaId) {
		ExtendedCriteria criteria = getReportListaEsitoArchiviazioniCriteria(user, r, listaId);
		return getListSize(criteria);
	}
	@SuppressWarnings("unused")
	private ExtendedCriteria getReportListaEsitoArchiviazioniCriteria(Long user,  ReportCriteria r, Long listaId) {
		if(r != null)  r = (ReportCriteria) r.loadCriteria();
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContatto.class);
		DetachedCriteria criteriaOp = criteria.createCriteria("operatore", "op");
		ApplicationUtil.pannello(r.getPannello(), criteria);
		DetachedCriteria criteriaC = criteria.createCriteria("contatto");
		DetachedCriteria criteriaCA = criteriaC.createCriteria("campagna", "camp");
		if (ApplicationUtil.gtZero(user)) {
			impostazioneDao.subCriteriaRoleSupervisore(user, criteria, "camp", "op");
		}
		if(r != null) {
			DetachedCriteria criteriaLI = criteriaC.createCriteria("lista");
			ApplicationUtil.dataIntervalD(r.getInizio(), r.getFine(), "data", criteria);
			ApplicationUtil.statoLC(r.getStatoIntervallo(), criteriaCA, criteriaLI);
			if(ApplicationUtil.gtZero(r.getCampagna())) {
				criteriaCA.add(Restrictions.idEq(r.getCampagna()));
			}
			if(ApplicationUtil.ltZero(r.getCampagna())) {
				if(ApplicationUtil.gtZero(user)) {
					if(ApplicationUtil.isTrue(sd)) {
						impostazioneDao.subCriteriaCampagnaSupervisore(user, criteriaCA, "id");
					}
				}
			}
		}
		criteriaC.add(Restrictions.eq("lista.id",listaId));
		criteria.add(Restrictions.isNotNull("archiviazione.id"));
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, r);
	}
	public ReportContattiOdierni getContattiOdierniListaEsitoReport(Long lista, Long supervisore, ReportCriteria r) {
		if(r != null)  r = (ReportCriteria) r.loadCriteria(); 
		ReportContattiOdierni rcoo = new ReportContattiOdierni();
		if(r == null || (r != null && r.getCerca())) {
			rcoo.setArchiviazioni(getArchiviazioniOdierneListaEsitoReport(lista, supervisore, r));
			rcoo.setContatti(getContattiOdierneListaEsitoReport(lista, supervisore, r));
			rcoo.setConversazioni(getConversazioniOdierneListaEsitoReport(lista, supervisore, r));
			rcoo.setAppuntamenti(getAppuntamentiOdierneListaEsitoReport(lista, supervisore, r));
			rcoo.setContratti(getContrattiOdierneListaEsitoReport(lista, supervisore, r));
		}
		return rcoo;
	}
	public DettaglioLavoratoList getReportLavorato(Utente user, ReportCriteria r) {
		if(r != null)  r = (ReportCriteria) r.loadCriteria(); 
		DettaglioLavoratoList dd = new DettaglioLavoratoList();
		if(r == null || (r != null && r.getCerca())) {
			List<ReportLavorato> report = new ArrayList<ReportLavorato>();
			List<Pausa> pauseS = getPause(user, r);
			dd.setPause(pauseS);
			List<ReportLavoratoResultSet> marcature = getReportLavoratoResultsetMarcatura(user, r);
			List<ReportLavoratoResultSet> dati = getReportLavoratoResultsetPause(user, r);
			for(ReportLavoratoResultSet o : marcature) {
				Utente utente = userDao.getUser(o.getUser_id());
				ReportLavorato report_op = new ReportLavorato();
				report_op.setOperatore(utente);
				DettaglioLavorato m = new DettaglioLavorato();
				m.setConteggio(o.getConteggio());
				m.setTotale(o.getTotale());
				long totale = o.getTotaleS();
				report_op.setMarcature(m);
				List<DettaglioLavorato> pauseDettaglio = new ArrayList<DettaglioLavorato>();
				for(Pausa ec : pauseS) {
					DettaglioLavorato d = new DettaglioLavorato();
					d.setPausa(ec.getChiave());
					d.setPositivo(ec.getPositivo());
					int i = dati.indexOf(new ReportLavoratoResultSet(utente.getId(), ec.getChiave(), ec.getPositivo()));
					if(i >= 0) {
						ReportLavoratoResultSet dato = dati.get(i);
						d.setConteggio(dato.getConteggio());
						d.setTotale(dato.getTotale());
						totale -= dato.getTotaleS();
					} else {
						d.setConteggio(0L);
						d.setTotale(0L);
						totale -= 0L;
					}
					pauseDettaglio.add(d);
				}
				m.setSottrazione(totale);
				report_op.setPause(pauseDettaglio);
				report.add(report_op);
				dd.setReport(report);
			}
		}
		return dd;
	}
	@SuppressWarnings("unchecked")
	private List<ReportContrattoResultSet> getReportContrattoStoriaResultset(Long userId, ReportCriteria r) {
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContratto.class, "sa");
		DetachedCriteria criteriaO = criteria.createCriteria("contratto", "app");
		DetachedCriteria criteriaP = criteriaO.createCriteria("prodotto");
		DetachedCriteria criteriaC = criteria.createCriteria("utente", "op");
		DetachedCriteria criteriaA = criteriaO.createCriteria("agente", "ag", ApplicationUtil.leftJoin());
		DetachedCriteria criteriaCA = criteriaO.createCriteria("contatto").createCriteria("campagna", "camp");
		if(ApplicationUtil.gtZero(userId)) {
			impostazioneDao.subCriteriaRoleSupervisore(userId, criteria, "camp", "op");
			impostazioneDao.subCriteriaSupervisore(userId, criteriaC, "id");
			if(ApplicationUtil.isTrue(sd)) {
				impostazioneDao.subCriteriaCampagnaSupervisore(userId, criteriaCA, "id");
			}
		}
		if(r != null) { 
			ApplicationUtil.dataIntervalD(r.getInizio(), r.getFine(), "data", criteria);
			if(ApplicationUtil.gtZero(r.getAgente())) {
				criteriaA.add(Restrictions.idEq(r.getAgente()));
			}
			if (ApplicationUtil.gtZero(r.getTeam())){
				if(ApplicationUtil.isTrue(sd)) {
					impostazioneDao.subCriteriaCampagnaTeam(r.getTeam(), criteriaCA, "id");
				}
				impostazioneDao.subCriteriaOperatoreTeam(r.getTeam(), criteriaC, "id");
			}
			if (ApplicationUtil.gtZero(r.getProdottoId())) {
				criteriaP.add(Restrictions.idEq(r.getProdottoId()));
			}
			if (ApplicationUtil.gtZero(r.getCategoriaId())) {
				criteriaP.add(Restrictions.eq("categoria.id",r.getCategoriaId()));
			}

		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("op.id").as("user_id")))
				.add(Projections.countDistinct("sa.id").as("contratti"))
				);
		ApplicationUtil.orderR3(criteria);
		return getList(criteria, ReportContrattoResultSet.class);
	}
	@SuppressWarnings({ "unchecked", "unused" })
	private List<ReportContrattoResultSet> getReportContrattoStoriaResultsetDati(Long userId, ReportCriteria r) {
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContratto.class, "sa");
		DetachedCriteria criteriaO = criteria.createCriteria("contratto", "app");
		DetachedCriteria criteriaP = criteriaO.createCriteria("prodotto");
		DetachedCriteria criteriaC = criteria.createCriteria("utente", "op");
		DetachedCriteria criteriaA = criteriaO.createCriteria("agente", "ag", ApplicationUtil.leftJoin());
		DetachedCriteria criteriaE = criteria.createCriteria("esito", "esit",  ApplicationUtil.leftJoin());
		DetachedCriteria criteriaCA = criteriaO.createCriteria("contatto").createCriteria("campagna", "camp");
		if(ApplicationUtil.gtZero(userId)) {
			impostazioneDao.subCriteriaRoleSupervisore(userId, criteria, "camp", "op");
			impostazioneDao.subCriteriaSupervisore(userId, criteriaC, "id");
			if(ApplicationUtil.isTrue(sd)) {
				impostazioneDao.subCriteriaCampagnaSupervisore(userId, criteriaCA, "id");
			}
		}
		if(r != null) {
			ApplicationUtil.dataIntervalD(r.getInizio(), r.getFine(), "data", criteria);
			if(ApplicationUtil.gtZero(r.getAgente())) {
				criteriaA.add(Restrictions.idEq(r.getAgente()));
			} 
			if (ApplicationUtil.gtZero(r.getTeam())){
				if(ApplicationUtil.isTrue(sd)) {
					impostazioneDao.subCriteriaCampagnaTeam(r.getTeam(), criteriaCA, "id");
				}
				impostazioneDao.subCriteriaOperatoreTeam(r.getTeam(), criteriaC, "id");
			}
			if (ApplicationUtil.gtZero(r.getProdottoId())) {
				criteriaP.add(Restrictions.idEq(r.getProdottoId()));
			}
			if (ApplicationUtil.gtZero(r.getCategoriaId())) {
				criteriaP.add(Restrictions.eq("categoria.id",r.getCategoriaId()));
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("op.id").as("user_id")))
				.add(Projections.groupProperty("esit.id").as("esito_id"))
				.add(Projections.countDistinct("sa.id").as("contratti"))
				);
		ApplicationUtil.orderR2(criteria);
		return getList(criteria, ReportContrattoResultSet.class);
	}
	public void exportReportContratto(Integer tipo, OutputStream result) throws Exception {
		int a = 1;
		Date dataInizio = ApplicationUtil.dataInizio(tipo, ApplicationUtil.date());
		List<Dettaglio> esitiContratto = getReportContratti();
		ReportCriteria r = new ReportCriteria();
		r.setIntervallo(dataInizio, null);
		List<ReportContrattoResultSet> operatori = getReportContrattoResultset(null, r);
		List<ReportContrattoResultSet> dati = getReportContrattoResultsetDati(null, r);
		Workbook wb = ApplicationUtil.workbookS();
		ExcelStyle s = new ExcelStyle(wb);
		List<Sheet> fogli = new ArrayList<Sheet>();
		fogli.add(wb.createSheet(ConstantsLanguage.NUMERO_CONTRATTI.stampa()));
		for(Sheet foglio : fogli) {
			int j = 0;
			Row riga = foglio.createRow(foglio.getPhysicalNumberOfRows());
			Cell cella = riga.createCell(j++, CellType.STRING);
			ApplicationUtil.headerCell(s, cella, ConstantsLanguage.OPERATORE.stampa());
			cella = riga.createCell(j++, CellType.STRING);
			ApplicationUtil.headerCell(s, cella, ConstantsLanguage.TOTALE.stampa());
			for(Dettaglio head : esitiContratto) {
				cella = riga.createCell(j++, CellType.STRING);
				ApplicationUtil.headerCell(s, cella, head.getEtichetta());
			}
			foglio.setAutoFilter(new CellRangeAddress(0, 0, 0, ApplicationUtil.size(esitiContratto)));
			foglio.createFreezePane(0, 1);
		}
		for(ReportContrattoResultSet operatore : operatori) {
			int j = 0;
			Utente utente = userDao.getUser(operatore.getUser_id());
			Row riga[] = new Row[a];
			for(int i = 0; i < riga.length; i++) {
				riga[i] = fogli.get(i).createRow(fogli.get(i).getPhysicalNumberOfRows());
				ApplicationUtil.formatString(s, riga[i].createCell(j, CellType.STRING), utente.getDenominazione());
			}
			j++;
			ApplicationUtil.formatNumber(s, riga[0].createCell(j, CellType.NUMERIC), operatore.getContratti());
			j++;
			for(Dettaglio ec : esitiContratto) {
				int i = dati.indexOf(new ReportContrattoResultSet(utente.getId(), ec.getEsito_id()));
				ReportContrattoResultSet dato = new ReportContrattoResultSet();
				if(i >= 0) {
					dato = dati.get(i);
				}
				ApplicationUtil.formatNumber(s, riga[0].createCell(j, CellType.NUMERIC), dato.getContratti());
				j++;
			}
		}
		String p = ConstantsGlobal.secretE;
		ApplicationUtil.passwordXLS(wb, result, p);
	}
	public void exportReportContrattoStoria(Integer tipo, OutputStream result) throws Exception {
		int a = 1;
		Date dataInizio = ApplicationUtil.dataInizio(tipo, ApplicationUtil.date());
		List<Dettaglio> esitiContratto = getReportContratti();
		ReportCriteria r = new ReportCriteria();
		r.setIntervallo(dataInizio, null);
		List<ReportContrattoResultSet> operatori = getReportContrattoStoriaResultset(null, r);
		List<ReportContrattoResultSet> dati = getReportContrattoStoriaResultsetDati(null, r);
		Workbook wb = ApplicationUtil.workbookS();
		ExcelStyle s = new ExcelStyle(wb);
		List<Sheet> fogli = new ArrayList<Sheet>();
		fogli.add(wb.createSheet(ConstantsLanguage.NUMERO_CONTRATTI.stampa()));
		for(Sheet foglio : fogli) {
			int j = 0;
			Row riga = foglio.createRow(foglio.getPhysicalNumberOfRows());
			Cell cella = riga.createCell(j++, CellType.STRING);
			ApplicationUtil.headerCell(s, cella, ConstantsLanguage.OPERATORE.stampa());
			cella = riga.createCell(j++, CellType.STRING);
			ApplicationUtil.headerCell(s, cella, ConstantsLanguage.TOTALE.stampa());
			for(Dettaglio head : esitiContratto) {
				cella = riga.createCell(j++, CellType.STRING);
				ApplicationUtil.headerCell(s, cella, head.getEtichetta());
			}
			foglio.setAutoFilter(new CellRangeAddress(0, 0, 0, ApplicationUtil.size(esitiContratto)));
			foglio.createFreezePane(0, 1);
		}
		for(ReportContrattoResultSet operatore : operatori) {
			int j = 0;
			Utente utente = userDao.getUser(operatore.getUser_id());
			Row riga[] = new Row[a];
			for(int i = 0; i < riga.length; i++) {
				riga[i] = fogli.get(i).createRow(fogli.get(i).getPhysicalNumberOfRows());
				ApplicationUtil.formatString(s, riga[i].createCell(j, CellType.STRING), utente.getDenominazione());
			}
			j++;
			ApplicationUtil.formatNumber(s, riga[0].createCell(j, CellType.NUMERIC), operatore.getContratti());
			j++;
			for(Dettaglio ec : esitiContratto) {
				int i = dati.indexOf(new ReportContrattoResultSet(utente.getId(), ec.getEsito_id()));
				ReportContrattoResultSet dato = new ReportContrattoResultSet();
				if(i >= 0) {
					dato = dati.get(i);
				}
				ApplicationUtil.formatNumber(s, riga[0].createCell(j, CellType.NUMERIC), dato.getContratti());
				j++;
			}
		}
		String p = ConstantsGlobal.secretE;
		ApplicationUtil.passwordXLS(wb, result, p);
	}
	@SuppressWarnings("unchecked")
	private List<ReportContrattoResultSet> getReportContrattoResultset(Long userId, ReportCriteria r) {
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(Contratto.class, "sa");
		DetachedCriteria criteriaP = criteria.createCriteria("prodotto");
		DetachedCriteria criteriaC = criteria.createCriteria("operatore", "op");
		DetachedCriteria criteriaA = criteria.createCriteria("agente", "ag", ApplicationUtil.leftJoin());
		DetachedCriteria criteriaCA = criteria.createCriteria("contatto").createCriteria("campagna", "camp");
		if(ApplicationUtil.gtZero(userId)) {
			impostazioneDao.subCriteriaRoleSupervisore(userId, criteria, "camp", "op");
			impostazioneDao.subCriteriaSupervisore(userId, criteriaC, "id");
			if(ApplicationUtil.isTrue(sd)) {
				impostazioneDao.subCriteriaCampagnaSupervisore(userId, criteriaCA, "id");
			}
		}
		if(r != null) {
			ApplicationUtil.intervallo(ApplicationUtil.isTrue(r.getInserimento()), r.getInizio(), r.getFine(), criteria);
			if(ApplicationUtil.gtZero(r.getAgente())) {
				criteriaA.add(Restrictions.idEq(r.getAgente()));
			}
			if (ApplicationUtil.gtZero(r.getTeam())){
				if(ApplicationUtil.isTrue(sd)) {
					impostazioneDao.subCriteriaCampagnaTeam(r.getTeam(), criteriaCA, "id");
				}
				impostazioneDao.subCriteriaOperatoreTeam(r.getTeam(), criteriaC, "id");
			}
			if (ApplicationUtil.gtZero(r.getProdottoId())) {
				criteriaP.add(Restrictions.idEq(r.getProdottoId()));
			}
			if (ApplicationUtil.gtZero(r.getCategoriaId())) {
				criteriaP.add(Restrictions.eq("categoria.id",r.getCategoriaId()));
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("op.id").as("user_id")))
				.add(Projections.countDistinct("sa.id").as("contratti"))
				);
		ApplicationUtil.orderR3(criteria);
		return getList(criteria, ReportContrattoResultSet.class);
	}
	@SuppressWarnings({ "unchecked", "unused" })
	private List<ReportContrattoResultSet> getReportContrattoResultsetDati(Long userId, ReportCriteria r) {
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(Contratto.class, "sa");
		DetachedCriteria criteriaP = criteria.createCriteria("prodotto");
		DetachedCriteria criteriaC = criteria.createCriteria("operatore", "op");
		DetachedCriteria criteriaE = criteria.createCriteria("esito", "esit",  ApplicationUtil.leftJoin());
		DetachedCriteria criteriaA = criteria.createCriteria("agente", "ag", ApplicationUtil.leftJoin());
		DetachedCriteria criteriaCA = criteria.createCriteria("contatto").createCriteria("campagna", "camp");
		if(ApplicationUtil.gtZero(userId)) {
			impostazioneDao.subCriteriaRoleSupervisore(userId, criteria, "camp", "op");
			impostazioneDao.subCriteriaSupervisore(userId, criteriaC, "id");
			if(ApplicationUtil.isTrue(sd)) {
				impostazioneDao.subCriteriaCampagnaSupervisore(userId, criteriaCA, "id");
			}
		}
		if(r != null) {
			ApplicationUtil.intervallo(ApplicationUtil.isTrue(r.getInserimento()), r.getInizio(), r.getFine(), criteria);
			if(ApplicationUtil.gtZero(r.getAgente())) {
				criteriaA.add(Restrictions.idEq(r.getAgente()));
			}  
			if (ApplicationUtil.gtZero(r.getTeam())){
				if(ApplicationUtil.isTrue(sd)) {
					impostazioneDao.subCriteriaCampagnaTeam(r.getTeam(), criteriaCA, "id");
				}
				impostazioneDao.subCriteriaOperatoreTeam(r.getTeam(), criteriaC, "id");
			}
			if (ApplicationUtil.gtZero(r.getProdottoId())) {
				criteriaP.add(Restrictions.idEq(r.getProdottoId()));
			}
			if (ApplicationUtil.gtZero(r.getCategoriaId())) {
				criteriaP.add(Restrictions.eq("categoria.id",r.getCategoriaId()));
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("op.id").as("user_id")))
				.add(Projections.groupProperty("esit.id").as("esito_id"))
				.add(Projections.countDistinct("sa.id").as("contratti"))
				);
		ApplicationUtil.orderR2(criteria);
		return getList(criteria, ReportContrattoResultSet.class);
	}
	public List<ReportContratto> getReportContrattoStoria(Long userid, ReportCriteria r) {
		if(r != null)  r = (ReportCriteria) r.loadCriteria(); 
		List<ReportContratto> report = new ArrayList<ReportContratto>();
		if(r == null || (r != null && r.getCerca())) {
			List<Dettaglio> esitiContratto = getReportContratti();
			List<ReportContrattoResultSet> operatori = getReportContrattoStoriaResultset(userid, 	r);
			List<ReportContrattoResultSet> dati = getReportContrattoStoriaResultsetDati(userid, r);
			for(ReportContrattoResultSet operatore : operatori) {
				Utente utente = userDao.getUser(operatore.getUser_id());
				ReportContratto report_app = new ReportContratto();
				report_app.setOperatore(utente);
				report_app.setTotaleContratti(new Dettaglio(operatore.getContratti(),null, null,null,null, null, null, null, null, null, null, null, null, null, null));
				List<Dettaglio> esitiContrattoDettaglio = new ArrayList<Dettaglio>();
				for(Dettaglio ec : esitiContratto) {
					Dettaglio d;
					int i = dati.indexOf(new ReportContrattoResultSet(utente.getId(), ec.getEsito_id()));
					if(i >= 0) {
						ReportContrattoResultSet dato = dati.get(i);
						d = new Dettaglio(dato.getContratti(),null, null,null,null, null, null, null, null, null, null, null, null, null, null);
						d.p_numero(dato.getContratti(), operatore.getContratti());
					}
					else d = new Dettaglio(true);
					d.setEsito_id(ec.getEsito_id());
					d.setEtichetta(ec.getEtichetta());
					esitiContrattoDettaglio.add(d);
				}
				report_app.setEsitiContrattiList(esitiContrattoDettaglio);
				report.add(report_app);
			}
		}
		return report;
	}
	public void exportReportContrattiStoria(Long userId, ReportCriteria r, DownloadBean downloadBean) {
		if(r != null)  r = (ReportCriteria) r.loadCriteria();
		if(r == null || (r != null && r.getCerca())) {
			int a = 1;
			List<Dettaglio> esitiContratto = getReportContratti();
			List<ReportContrattoResultSet> operatori = getReportContrattoStoriaResultset(userId, r);
			List<ReportContrattoResultSet> dati = getReportContrattoStoriaResultsetDati(userId, r);
			Workbook wb = ApplicationUtil.workbookS();
			ExcelStyle s = new ExcelStyle(wb);
			List<Sheet> fogli = new ArrayList<Sheet>();
			fogli.add(wb.createSheet(ConstantsLanguage.NUMERO_CONTRATTI.stampa()));
			for(Sheet foglio : fogli) {
				int j = 0;
				Row riga = foglio.createRow(foglio.getPhysicalNumberOfRows());
				Cell cella = riga.createCell(j++, CellType.STRING);
				ApplicationUtil.headerCell(s, cella, ConstantsLanguage.OPERATORE.stampa());
				cella = riga.createCell(j++, CellType.STRING);
				ApplicationUtil.headerCell(s, cella, ConstantsLanguage.TOTALE.stampa());
				for(Dettaglio head : esitiContratto) {
					cella = riga.createCell(j++, CellType.STRING);
					ApplicationUtil.headerCell(s, cella, head.getEtichetta());
				}
				foglio.setAutoFilter(new CellRangeAddress(0, 0, 0, ApplicationUtil.size(esitiContratto)));
				foglio.createFreezePane(0, 1);
			}
			for(ReportContrattoResultSet operatore : operatori) {
				int j = 0;
				Utente utente = userDao.getUser(operatore.getUser_id());
				Row riga[] = new Row[a];
				for(int i = 0; i < riga.length; i++) {
					riga[i] = fogli.get(i).createRow(fogli.get(i).getPhysicalNumberOfRows());
					ApplicationUtil.formatString(s, riga[i].createCell(j, CellType.STRING), utente.getDenominazione());
				}
				j++;
				ApplicationUtil.formatNumber(s, riga[0].createCell(j, CellType.NUMERIC), operatore.getContratti());
				j++;
				for(Dettaglio ec : esitiContratto) {
					int i = dati.indexOf(new ReportContrattoResultSet(utente.getId(), ec.getEsito_id()));
					ReportContrattoResultSet dato = new ReportContrattoResultSet();
					if(i >= 0) {
						dato = dati.get(i);
					}
					ApplicationUtil.formatNumber(s, riga[0].createCell(j, CellType.NUMERIC), dato.getContratti());
					j++;
				}
			}
			try {
				String nome = ApplicationUtil.formatIntervallo(ConstantsLanguage.REPORT.stampa() + "_" + ConstantsLanguage.CONTRATTI.stampa(), r.getInizio(), r.getFine()) + ConstantsMap.xlsx;


				OutputStream out = downloadBean.outputStream();
				downloadBean.filename(nome);

				String p = ConstantsGlobal.secretE;
				ApplicationUtil.passwordXLS(wb, out, p);
				downloadBean.close();

			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		}
	}
	public List<ReportContratto> getReportContratto(Long userid, ReportCriteria r) {
		if(r != null)  r = (ReportCriteria) r.loadCriteria(); 
		List<ReportContratto> report = new ArrayList<ReportContratto>();
		if(r == null || (r != null && r.getCerca())) {
			List<Dettaglio> esitiContratto = getReportContratti();
			List<ReportContrattoResultSet> operatori = getReportContrattoResultset(userid, 	r);
			List<ReportContrattoResultSet> dati = getReportContrattoResultsetDati(userid, r);
			for(ReportContrattoResultSet operatore : operatori) {
				Utente utente = userDao.getUser(operatore.getUser_id());
				ReportContratto report_app = new ReportContratto();
				report_app.setOperatore(utente);
				report_app.setTotaleContratti(new Dettaglio(operatore.getContratti(),null, null,null,null, null, null, null, null, null, null, null, null, null, null));
				List<Dettaglio> esitiContrattoDettaglio = new ArrayList<Dettaglio>();
				for(Dettaglio ec : esitiContratto) {
					Dettaglio d;
					int i = dati.indexOf(new ReportContrattoResultSet(utente.getId(), ec.getEsito_id()));
					if(i >= 0) {
						ReportContrattoResultSet dato = dati.get(i);
						d = new Dettaglio(dato.getContratti(),null, null,null,null, null, null, null, null, null, null, null, null, null, null);
						d.p_numero(dato.getContratti(), operatore.getContratti());
					}
					else d = new Dettaglio(true);
					d.setEsito_id(ec.getEsito_id());
					d.setEtichetta(ec.getEtichetta());
					esitiContrattoDettaglio.add(d);
				}
				report_app.setEsitiContrattiList(esitiContrattoDettaglio);
				report.add(report_app);
			}
		}
		return report;
	}
	public void exportReportContratti(Long userId, ReportCriteria r, DownloadBean downloadBean) {
		if(r != null)  r = (ReportCriteria) r.loadCriteria();
		if(r == null || (r != null && r.getCerca())) {
			int a = 1;
			List<Dettaglio> esitiContratto = getReportContratti();
			List<ReportContrattoResultSet> operatori = getReportContrattoResultset(userId, r);
			List<ReportContrattoResultSet> dati = getReportContrattoResultsetDati(userId, r);
			Workbook wb = ApplicationUtil.workbookS();
			ExcelStyle s = new ExcelStyle(wb);
			List<Sheet> fogli = new ArrayList<Sheet>();
			fogli.add(wb.createSheet(ConstantsLanguage.NUMERO_CONTRATTI.stampa()));
			for(Sheet foglio : fogli) {
				int j = 0;
				Row riga = foglio.createRow(foglio.getPhysicalNumberOfRows());
				Cell cella = riga.createCell(j++, CellType.STRING);
				ApplicationUtil.headerCell(s, cella, ConstantsLanguage.OPERATORE.stampa());
				cella = riga.createCell(j++, CellType.STRING);
				ApplicationUtil.headerCell(s, cella, ConstantsLanguage.TOTALE.stampa());
				for(Dettaglio head : esitiContratto) {
					cella = riga.createCell(j++, CellType.STRING);
					ApplicationUtil.headerCell(s, cella, head.getEtichetta());
				}
				foglio.setAutoFilter(new CellRangeAddress(0, 0, 0, ApplicationUtil.size(esitiContratto)));
				foglio.createFreezePane(0, 1);
			}
			for(ReportContrattoResultSet operatore : operatori) {
				int j = 0;
				Utente utente = userDao.getUser(operatore.getUser_id());
				Row riga[] = new Row[a];
				for(int i = 0; i < riga.length; i++) {
					riga[i] = fogli.get(i).createRow(fogli.get(i).getPhysicalNumberOfRows());
					ApplicationUtil.formatString(s, riga[i].createCell(j, CellType.STRING), utente.getDenominazione());
				}
				j++;
				ApplicationUtil.formatNumber(s, riga[0].createCell(j, CellType.NUMERIC), operatore.getContratti());
				j++;
				for(Dettaglio ec : esitiContratto) {
					int i = dati.indexOf(new ReportContrattoResultSet(utente.getId(), ec.getEsito_id()));
					ReportContrattoResultSet dato = new ReportContrattoResultSet();
					if(i >= 0) {
						dato = dati.get(i);
					}
					ApplicationUtil.formatNumber(s, riga[0].createCell(j, CellType.NUMERIC), dato.getContratti());
					j++;
				}
			}
			try {
				String nome = ApplicationUtil.formatIntervallo(ConstantsLanguage.RIEPILOGO.stampa() + "_" + ConstantsLanguage.CONTRATTI.stampa(), r.getInizio(), r.getFine()) + ConstantsMap.xlsx;


				OutputStream out = downloadBean.outputStream();
				downloadBean.filename(nome);

				String p = ConstantsGlobal.secretE;
				ApplicationUtil.passwordXLS(wb, out, p);
				downloadBean.close();

			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		}
	}
	private ExtendedCriteria  getReportContrattiStoriaByCriteria(Long user, Long esitoId, ReportCriteria r, Long operatoreId) {
		if(r != null)  r = (ReportCriteria) r.loadCriteria();
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContratto.class);
		DetachedCriteria criteriaCa = criteria.createCriteria("contratto");
		DetachedCriteria criteriaP = criteriaCa.createCriteria("prodotto");
		DetachedCriteria criteriaC = criteriaCa.createCriteria("contatto");
		DetachedCriteria criteriaUU = criteria.createCriteria("utente", "op");
		DetachedCriteria criteriaCm = criteriaC.createCriteria("campagna", "camp");
		if(ApplicationUtil.gtZero(user)) {
			impostazioneDao.subCriteriaRoleSupervisore(user, criteria, "camp", "op");
			if(ApplicationUtil.isTrue(sd)) {
				impostazioneDao.subCriteriaSupervisore(user, criteriaUU, "id");
			}
			impostazioneDao.subCriteriaCampagnaSupervisore(user, criteriaCm, "id");
		}
		if(r != null) {
			ApplicationUtil.dataIntervalD(r.getInizio(), r.getFine(), "data", criteria);
			if(ApplicationUtil.gtZero(r.getAgente())) {
				criteria.add(Restrictions.eq("agente.id",r.getAgente()));
			}
			if (ApplicationUtil.gtZero(r.getTeam())){
				if(ApplicationUtil.isTrue(sd)) {
					impostazioneDao.subCriteriaCampagnaTeam(r.getTeam(), criteriaCm, "id");
				}
				impostazioneDao.subCriteriaOperatoreTeam(r.getTeam(), criteriaUU, "id");
			}
			if (ApplicationUtil.gtZero(r.getProdottoId())) {
				criteriaP.add(Restrictions.idEq(r.getProdottoId()));
			}
			if (ApplicationUtil.gtZero(r.getCategoriaId())) {
				criteriaP.add(Restrictions.eq("categoria.id",r.getCategoriaId()));
			}
		}
		criteriaUU.add(Restrictions.idEq(operatoreId));
		if(ApplicationUtil.gtZero(esitoId)) {
			criteria.add(Restrictions.eq("esito.id",esitoId));
		}
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, r);
	}
	@SuppressWarnings("unchecked")
	public List<StoriaContratto>  getReportContrattiStoria(Long user, Long esito, ReportCriteria r, Long operatoreId, int start, int size) {
		ExtendedCriteria criteria = getReportContrattiStoriaByCriteria(user, esito, r, operatoreId);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int  getReportContrattiStoriaSize(Long user, Long esito, ReportCriteria r, Long operatoreId) {
		ExtendedCriteria criteria = getReportContrattiStoriaByCriteria(user, esito, r, operatoreId);
		return getListSize(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Contratto>  getReportContratti(Long user, Long esitoId, ReportCriteria r, Long operatoreId, int start, int size) {
		ExtendedCriteria criteria = getReportContrattiByCriteria(user, esitoId, r, operatoreId);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int  getReportContrattiSize(Long user,  Long esitoId,  ReportCriteria r, Long operatoreId) {
		ExtendedCriteria criteria = getReportContrattiByCriteria(user, esitoId, r, operatoreId);
		return getListSize(criteria);
	}
	private ExtendedCriteria  getReportContrattiByCriteria(Long user, Long esitoId,  ReportCriteria r, Long operatoreId) {
		if(r != null)  r = (ReportCriteria) r.loadCriteria();
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(Contratto.class);
		DetachedCriteria criteriaP = criteria.createCriteria("prodotto");
		DetachedCriteria criteriaC = criteria.createCriteria("contatto");
		DetachedCriteria criteriaUU = criteria.createCriteria("operatore", "op");
		DetachedCriteria criteriaCm = criteriaC.createCriteria("campagna", "camp");
		criteriaUU.add(Restrictions.idEq(operatoreId));
		if(ApplicationUtil.gtZero(esitoId)) {
			criteria.add(Restrictions.eq("esito.id",esitoId));
		}
		if(ApplicationUtil.gtZero(user)) {
			impostazioneDao.subCriteriaRoleSupervisore(user, criteria, "camp", "op");
			if(ApplicationUtil.isTrue(sd)) {
				impostazioneDao.subCriteriaSupervisore(user, criteriaUU, "id");
			}
			impostazioneDao.subCriteriaCampagnaSupervisore(user, criteriaCm, "id");
		}
		if(r != null) {
			ApplicationUtil.intervallo(ApplicationUtil.isTrue(r.getInserimento()), r.getInizio(), r.getFine(), criteria);
			if(ApplicationUtil.gtZero(r.getAgente())) {
				criteria.add(Restrictions.eq("agente.id",r.getAgente()));
			}
			if (ApplicationUtil.gtZero(r.getTeam())){
				if(ApplicationUtil.isTrue(sd)) {
					impostazioneDao.subCriteriaCampagnaTeam(r.getTeam(), criteriaCm, "id");
				}
				impostazioneDao.subCriteriaOperatoreTeam(r.getTeam(), criteriaUU, "id");
			}
			if (ApplicationUtil.gtZero(r.getProdottoId())) {
				criteriaP.add(Restrictions.idEq(r.getProdottoId()));
			}
			if (ApplicationUtil.gtZero(r.getCategoriaId())) {
				criteriaP.add(Restrictions.eq("categoria.id",r.getCategoriaId()));
			}
		}
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, r);
	}
	@SuppressWarnings("unchecked")
	private List<ReportAppuntamentoResultSet> getReportAppuntamentoStoriaResultset(Long userId, ReportCriteria r) {
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaAppuntamento.class, "sa");
		DetachedCriteria criteriaO = criteria.createCriteria("appuntamento", "app");
		DetachedCriteria criteriaC = criteria.createCriteria("utente", "op");
		DetachedCriteria criteriaA = criteriaO.createCriteria("agente", "ag", ApplicationUtil.leftJoin());
		DetachedCriteria criteriaCA = criteriaO.createCriteria("contatto").createCriteria("campagna", "camp");
		if(ApplicationUtil.gtZero(userId)) {
			impostazioneDao.subCriteriaRoleSupervisore(userId, criteria, "camp", "op");
			impostazioneDao.subCriteriaSupervisore(userId, criteriaC, "id");
			if(ApplicationUtil.isTrue(sd)) {
				impostazioneDao.subCriteriaCampagnaSupervisore(userId, criteriaCA, "id");
			}
		}
		if(r != null) {
			ApplicationUtil.dataIntervalD(r.getInizio(), r.getFine(), "data", criteria);
			if(ApplicationUtil.gtZero(r.getAgente())) {
				criteriaA.add(Restrictions.idEq(r.getAgente()));
			}
			if (ApplicationUtil.gtZero(r.getTeam())){
				if(ApplicationUtil.isTrue(sd)) {
					impostazioneDao.subCriteriaCampagnaTeam(r.getTeam(), criteriaCA, "id");
				}
				impostazioneDao.subCriteriaOperatoreTeam(r.getTeam(), criteriaC, "id");
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("op.id").as("user_id")))
				.add(Projections.countDistinct("sa.id").as("appuntamenti"))
				);
		ApplicationUtil.orderR3(criteria);
		return getList(criteria, ReportAppuntamentoResultSet.class);
	}
	@SuppressWarnings({ "unchecked", "unused" })
	private List<ReportAppuntamentoResultSet> getReportAppuntamentoStoriaResultsetDati(Long userId, ReportCriteria r) {
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaAppuntamento.class, "sa");
		DetachedCriteria criteriaO = criteria.createCriteria("appuntamento", "app");
		DetachedCriteria criteriaC = criteria.createCriteria("utente", "op");
		DetachedCriteria criteriaA = criteriaO.createCriteria("agente", "ag", ApplicationUtil.leftJoin());
		DetachedCriteria criteriaE = criteria.createCriteria("esito", "esit",  ApplicationUtil.leftJoin());
		DetachedCriteria criteriaCA = criteriaO.createCriteria("contatto").createCriteria("campagna", "camp");
		if(ApplicationUtil.gtZero(userId)) {
			impostazioneDao.subCriteriaRoleSupervisore(userId, criteria, "camp", "op");
			impostazioneDao.subCriteriaSupervisore(userId, criteriaC, "id");
			if(ApplicationUtil.isTrue(sd)) {
				impostazioneDao.subCriteriaCampagnaSupervisore(userId, criteriaCA, "id");
			}
		}
		if(r != null) {
			ApplicationUtil.dataIntervalD(r.getInizio(), r.getFine(), "data", criteria);
			if(ApplicationUtil.gtZero(r.getAgente())) {
				criteriaA.add(Restrictions.idEq(r.getAgente()));
			} 
			if (ApplicationUtil.gtZero(r.getTeam())){
				if(ApplicationUtil.isTrue(sd)) {
					impostazioneDao.subCriteriaCampagnaTeam(r.getTeam(), criteriaCA, "id");
				}
				impostazioneDao.subCriteriaOperatoreTeam(r.getTeam(), criteriaC, "id");
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("op.id").as("user_id")))
				.add(Projections.groupProperty("esit.id").as("esito_id"))
				.add(Projections.countDistinct("sa.id").as("appuntamenti"))
				);
		ApplicationUtil.orderR2(criteria);
		return getList(criteria, ReportAppuntamentoResultSet.class);
	}
	public void exportReportAppuntamento(Integer tipo, OutputStream result) throws Exception {
		int a = 1;
		Date dataInizio = ApplicationUtil.dataInizio(tipo, ApplicationUtil.date());
		List<Dettaglio> esitiAppuntamento = getReportAppuntamenti();
		ReportCriteria r = new ReportCriteria();
		r.setIntervallo(dataInizio, null);
		List<ReportAppuntamentoResultSet> operatori = getReportAppuntamentoResultset(null, r);
		List<ReportAppuntamentoResultSet> dati = getReportAppuntamentoResultsetDati(null, r);
		Workbook wb = ApplicationUtil.workbookS();
		ExcelStyle s = new ExcelStyle(wb);
		List<Sheet> fogli = new ArrayList<Sheet>();
		fogli.add(wb.createSheet(ConstantsLanguage.NUMERO_APPUNTAMENTI.stampa()));
		for(Sheet foglio : fogli) {
			int j = 0;
			Row riga = foglio.createRow(foglio.getPhysicalNumberOfRows());
			Cell cella = riga.createCell(j++, CellType.STRING);
			ApplicationUtil.headerCell(s, cella, ConstantsLanguage.OPERATORE.stampa());
			cella = riga.createCell(j++, CellType.STRING);
			ApplicationUtil.headerCell(s, cella, ConstantsLanguage.TOTALE.stampa());
			for(Dettaglio head : esitiAppuntamento) {
				cella = riga.createCell(j++, CellType.STRING);
				ApplicationUtil.headerCell(s, cella, head.getEtichetta());
			}
			foglio.setAutoFilter(new CellRangeAddress(0, 0, 0, ApplicationUtil.size(esitiAppuntamento)));
			foglio.createFreezePane(0, 1);
		}
		for(ReportAppuntamentoResultSet operatore : operatori) {
			int j = 0;
			Utente utente = userDao.getUser(operatore.getUser_id());
			Row riga[] = new Row[a];
			for(int i = 0; i < riga.length; i++) {
				riga[i] = fogli.get(i).createRow(fogli.get(i).getPhysicalNumberOfRows());
				ApplicationUtil.formatString(s, riga[i].createCell(j, CellType.STRING), utente.getDenominazione());
			}
			j++;
			ApplicationUtil.formatNumber(s, riga[0].createCell(j, CellType.NUMERIC), operatore.getAppuntamenti());
			j++;
			for(Dettaglio ec : esitiAppuntamento) {
				int i = dati.indexOf(new ReportAppuntamentoResultSet(utente.getId(), ec.getEsito_id()));
				ReportAppuntamentoResultSet dato = new ReportAppuntamentoResultSet();
				if(i >= 0) {
					dato = dati.get(i);
				}
				ApplicationUtil.formatNumber(s, riga[0].createCell(j, CellType.NUMERIC), dato.getAppuntamenti());
				j++;
			}
		}
		String p = ConstantsGlobal.secretE;
		ApplicationUtil.passwordXLS(wb, result, p);
	}
	public void exportReportAppuntamentoStoria(Integer tipo, OutputStream result) throws Exception {
		int a = 1;
		Date dataInizio = ApplicationUtil.dataInizio(tipo, ApplicationUtil.date());
		List<Dettaglio> esitiAppuntamento = getReportAppuntamenti();
		ReportCriteria r = new ReportCriteria();
		r.setIntervallo(dataInizio, null);
		List<ReportAppuntamentoResultSet> operatori = getReportAppuntamentoStoriaResultset(null, r);
		List<ReportAppuntamentoResultSet> dati = getReportAppuntamentoStoriaResultsetDati(null, r);
		Workbook wb = ApplicationUtil.workbookS();
		ExcelStyle s = new ExcelStyle(wb);
		List<Sheet> fogli = new ArrayList<Sheet>();
		fogli.add(wb.createSheet(ConstantsLanguage.NUMERO_APPUNTAMENTI.stampa()));
		for(Sheet foglio : fogli) {
			int j = 0;
			Row riga = foglio.createRow(foglio.getPhysicalNumberOfRows());
			Cell cella = riga.createCell(j++, CellType.STRING);
			ApplicationUtil.headerCell(s, cella, ConstantsLanguage.OPERATORE.stampa());
			cella = riga.createCell(j++, CellType.STRING);
			ApplicationUtil.headerCell(s, cella, ConstantsLanguage.TOTALE.stampa());
			for(Dettaglio head : esitiAppuntamento) {
				cella = riga.createCell(j++, CellType.STRING);
				ApplicationUtil.headerCell(s, cella, head.getEtichetta());
			}
			foglio.setAutoFilter(new CellRangeAddress(0, 0, 0, ApplicationUtil.size(esitiAppuntamento)));
			foglio.createFreezePane(0, 1);
		}
		for(ReportAppuntamentoResultSet operatore : operatori) {
			int j = 0;
			Utente utente = userDao.getUser(operatore.getUser_id());
			Row riga[] = new Row[a];
			for(int i = 0; i < riga.length; i++) {
				riga[i] = fogli.get(i).createRow(fogli.get(i).getPhysicalNumberOfRows());
				ApplicationUtil.formatString(s, riga[i].createCell(j, CellType.STRING), utente.getDenominazione());
			}
			j++;
			ApplicationUtil.formatNumber(s, riga[0].createCell(j, CellType.NUMERIC), operatore.getAppuntamenti());
			j++;
			for(Dettaglio ec : esitiAppuntamento) {
				int i = dati.indexOf(new ReportAppuntamentoResultSet(utente.getId(), ec.getEsito_id()));
				ReportAppuntamentoResultSet dato = new ReportAppuntamentoResultSet();
				if(i >= 0) {
					dato = dati.get(i);
				}
				ApplicationUtil.formatNumber(s, riga[0].createCell(j, CellType.NUMERIC), dato.getAppuntamenti());
				j++;
			}
		}
		String p = ConstantsGlobal.secretE;
		ApplicationUtil.passwordXLS(wb, result, p);
	}
	@SuppressWarnings("unchecked")
	private List<ReportAppuntamentoResultSet> getReportAppuntamentoResultset(Long userId, ReportCriteria r) {
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(Appuntamento.class, "sa");
		DetachedCriteria criteriaC = criteria.createCriteria("operatore", "op");
		DetachedCriteria criteriaA = criteria.createCriteria("agente", "ag", ApplicationUtil.leftJoin());
		DetachedCriteria criteriaCA = criteria.createCriteria("contatto").createCriteria("campagna", "camp");
		if(ApplicationUtil.gtZero(userId)) {
			impostazioneDao.subCriteriaRoleSupervisore(userId, criteria, "camp", "op");
			impostazioneDao.subCriteriaSupervisore(userId, criteriaC, "id");
			if(ApplicationUtil.isTrue(sd)) {
				impostazioneDao.subCriteriaCampagnaSupervisore(userId, criteriaCA, "id");
			}
		}
		if(r != null) {
			ApplicationUtil.intervallo(ApplicationUtil.isTrue(r.getInserimento()), r.getInizio(), r.getFine(), criteria);
			if(ApplicationUtil.gtZero(r.getAgente())) {
				criteriaA.add(Restrictions.idEq(r.getAgente()));
			}
			if (ApplicationUtil.gtZero(r.getTeam())){
				if(ApplicationUtil.isTrue(sd)) {
					impostazioneDao.subCriteriaCampagnaTeam(r.getTeam(), criteriaCA, "id");
				}
				impostazioneDao.subCriteriaOperatoreTeam(r.getTeam(), criteriaC, "id");
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("op.id").as("user_id")))
				.add(Projections.countDistinct("sa.id").as("appuntamenti"))
				);
		ApplicationUtil.orderR3(criteria);
		return getList(criteria, ReportAppuntamentoResultSet.class);
	}
	@SuppressWarnings({ "unchecked", "unused" })
	private List<ReportAppuntamentoResultSet> getReportAppuntamentoResultsetDati(Long userId, ReportCriteria r) {
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(Appuntamento.class, "sa");
		DetachedCriteria criteriaC = criteria.createCriteria("operatore", "op");
		DetachedCriteria criteriaE = criteria.createCriteria("esito", "esit", ApplicationUtil.leftJoin());
		DetachedCriteria criteriaA = criteria.createCriteria("agente", "ag", ApplicationUtil.leftJoin());
		DetachedCriteria criteriaCA = criteria.createCriteria("contatto").createCriteria("campagna", "camp");
		if(ApplicationUtil.gtZero(userId)) {
			impostazioneDao.subCriteriaRoleSupervisore(userId, criteria, "camp", "op");
			impostazioneDao.subCriteriaSupervisore(userId, criteriaC, "id");
			if(ApplicationUtil.isTrue(sd)) {
				impostazioneDao.subCriteriaCampagnaSupervisore(userId, criteriaCA, "id");
			}
		}
		if(r != null) {
			ApplicationUtil.intervallo(ApplicationUtil.isTrue(r.getInserimento()), r.getInizio(), r.getFine(), criteria);
			if(ApplicationUtil.gtZero(r.getAgente())) {
				criteriaA.add(Restrictions.idEq(r.getAgente()));
			}  
			if (ApplicationUtil.gtZero(r.getTeam())){
				if(ApplicationUtil.isTrue(sd)) {
					impostazioneDao.subCriteriaCampagnaTeam(r.getTeam(), criteriaCA, "id");
				}
				impostazioneDao.subCriteriaOperatoreTeam(r.getTeam(), criteriaC, "id");
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("op.id").as("user_id")))
				.add(Projections.groupProperty("esit.id").as("esito_id"))
				.add(Projections.countDistinct("sa.id").as("appuntamenti"))
				);
		ApplicationUtil.orderR2(criteria);
		return getList(criteria, ReportAppuntamentoResultSet.class);
	}
	public List<ReportAppuntamento> getReportAppuntamentoStoria(Long userid, ReportCriteria r) {
		if(r != null)  r = (ReportCriteria) r.loadCriteria(); 
		List<ReportAppuntamento> report = new ArrayList<ReportAppuntamento>();
		if(r == null || (r != null && r.getCerca())) {
			List<Dettaglio> esitiAppuntamento = getReportAppuntamenti();
			List<ReportAppuntamentoResultSet> operatori = getReportAppuntamentoStoriaResultset(userid, 	r);
			List<ReportAppuntamentoResultSet> dati = getReportAppuntamentoStoriaResultsetDati(userid, r);
			for(ReportAppuntamentoResultSet operatore : operatori) {
				Utente utente = userDao.getUser(operatore.getUser_id());
				ReportAppuntamento report_app = new ReportAppuntamento();
				report_app.setOperatore(utente);
				report_app.setTotaleAppuntamenti(new Dettaglio(operatore.getAppuntamenti(),null, null,null,null, null, null, null, null, null, null, null, null, null, null));
				List<Dettaglio> esitiAppuntamentoDettaglio = new ArrayList<Dettaglio>();
				for(Dettaglio ec : esitiAppuntamento) {
					Dettaglio d;
					int i = dati.indexOf(new ReportAppuntamentoResultSet(utente.getId(), ec.getEsito_id()));
					if(i >= 0) {
						ReportAppuntamentoResultSet dato = dati.get(i);
						d = new Dettaglio(dato.getAppuntamenti(),null, null,null,null, null, null, null, null, null, null, null, null, null, null);
						d.p_numero(dato.getAppuntamenti(), operatore.getAppuntamenti());
					}
					else d = new Dettaglio(true);
					d.setEsito_id(ec.getEsito_id());
					d.setEtichetta(ec.getEtichetta());
					esitiAppuntamentoDettaglio.add(d);
				}
				report_app.setEsitiAppuntamentiList(esitiAppuntamentoDettaglio);
				report.add(report_app);
			}
		}
		return report;
	}
	public void exportReportAppuntamentiStoria(Long userId, ReportCriteria r, DownloadBean downloadBean) {
		if(r != null)  r = (ReportCriteria) r.loadCriteria();
		if(r == null || (r != null && r.getCerca())) {
			int a = 1;
			List<Dettaglio> esitiAppuntamento = getReportAppuntamenti();
			List<ReportAppuntamentoResultSet> operatori = getReportAppuntamentoStoriaResultset(userId, r);
			List<ReportAppuntamentoResultSet> dati = getReportAppuntamentoStoriaResultsetDati(userId, r);
			Workbook wb = ApplicationUtil.workbookS();
			ExcelStyle s = new ExcelStyle(wb);
			List<Sheet> fogli = new ArrayList<Sheet>();
			fogli.add(wb.createSheet(ConstantsLanguage.NUMERO_APPUNTAMENTI.stampa()));
			for(Sheet foglio : fogli) {
				int j = 0;
				Row riga = foglio.createRow(foglio.getPhysicalNumberOfRows());
				Cell cella = riga.createCell(j++, CellType.STRING);
				ApplicationUtil.headerCell(s, cella, ConstantsLanguage.OPERATORE.stampa());
				cella = riga.createCell(j++, CellType.STRING);
				ApplicationUtil.headerCell(s, cella, ConstantsLanguage.TOTALE.stampa());
				for(Dettaglio head : esitiAppuntamento) {
					cella = riga.createCell(j++, CellType.STRING);
					ApplicationUtil.headerCell(s, cella, head.getEtichetta());
				}
				foglio.setAutoFilter(new CellRangeAddress(0, 0, 0, ApplicationUtil.size(esitiAppuntamento)));
				foglio.createFreezePane(0, 1);
			}
			for(ReportAppuntamentoResultSet operatore : operatori) {
				int j = 0;
				Utente utente = userDao.getUser(operatore.getUser_id());
				Row riga[] = new Row[a];
				for(int i = 0; i < riga.length; i++) {
					riga[i] = fogli.get(i).createRow(fogli.get(i).getPhysicalNumberOfRows());
					ApplicationUtil.formatString(s, riga[i].createCell(j, CellType.STRING), utente.getDenominazione());
				}
				j++;
				ApplicationUtil.formatNumber(s, riga[0].createCell(j, CellType.NUMERIC), operatore.getAppuntamenti());
				j++;
				for(Dettaglio ec : esitiAppuntamento) {
					int i = dati.indexOf(new ReportAppuntamentoResultSet(utente.getId(), ec.getEsito_id()));
					ReportAppuntamentoResultSet dato = new ReportAppuntamentoResultSet();
					if(i >= 0) {
						dato = dati.get(i);
					}
					ApplicationUtil.formatNumber(s, riga[0].createCell(j, CellType.NUMERIC), dato.getAppuntamenti());
					j++;
				}
			}
			try {
				String nome = ApplicationUtil.formatIntervallo(ConstantsLanguage.REPORT.stampa() + "_" + ConstantsLanguage.APPUNTAMENTI.stampa(), r.getInizio(), r.getFine()) + ConstantsMap.xlsx;


				OutputStream out = downloadBean.outputStream();
				downloadBean.filename(nome);

				String p = ConstantsGlobal.secretE;
				ApplicationUtil.passwordXLS(wb, out, p);
				downloadBean.close();

			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		}
	}
	public List<ReportAppuntamento> getReportAppuntamento(Long userid, ReportCriteria r) {
		if(r != null)  r = (ReportCriteria) r.loadCriteria(); 
		List<ReportAppuntamento> report = new ArrayList<ReportAppuntamento>();
		if(r == null || (r != null && r.getCerca())) {
			List<Dettaglio> esitiAppuntamento = getReportAppuntamenti();
			List<ReportAppuntamentoResultSet> operatori = getReportAppuntamentoResultset(userid, 	r);
			List<ReportAppuntamentoResultSet> dati = getReportAppuntamentoResultsetDati(userid, r);
			for(ReportAppuntamentoResultSet operatore : operatori) {
				Utente utente = userDao.getUser(operatore.getUser_id());
				ReportAppuntamento report_app = new ReportAppuntamento();
				report_app.setOperatore(utente);
				report_app.setTotaleAppuntamenti(new Dettaglio(operatore.getAppuntamenti(),null, null,null,null, null, null, null, null, null, null, null, null, null, null));
				List<Dettaglio> esitiAppuntamentoDettaglio = new ArrayList<Dettaglio>();
				for(Dettaglio ec : esitiAppuntamento) {
					Dettaglio d;
					int i = dati.indexOf(new ReportAppuntamentoResultSet(utente.getId(), ec.getEsito_id()));
					if(i >= 0) {
						ReportAppuntamentoResultSet dato = dati.get(i);
						d = new Dettaglio(dato.getAppuntamenti(),null, null,null,null, null, null, null, null, null, null, null, null, null, null);
						d.p_numero(dato.getAppuntamenti(), operatore.getAppuntamenti());
					}
					else d = new Dettaglio(true);
					d.setEsito_id(ec.getEsito_id());
					d.setEtichetta(ec.getEtichetta());
					esitiAppuntamentoDettaglio.add(d);
				}
				report_app.setEsitiAppuntamentiList(esitiAppuntamentoDettaglio);
				report.add(report_app);
			}
		}
		return report;
	}
	public void exportReportAppuntamenti(Long userId, ReportCriteria r, DownloadBean downloadBean) {
		if(r != null)  r = (ReportCriteria) r.loadCriteria();
		if(r == null || (r != null && r.getCerca())) {
			List<Dettaglio> esitiAppuntamento = getReportAppuntamenti();
			List<ReportAppuntamentoResultSet> operatori = getReportAppuntamentoResultset(userId, r);
			List<ReportAppuntamentoResultSet> dati = getReportAppuntamentoResultsetDati(userId, r);
			Workbook wb = ApplicationUtil.workbookS();
			ExcelStyle s = new ExcelStyle(wb);
			List<Sheet> fogli = new ArrayList<Sheet>();
			fogli.add(wb.createSheet(ConstantsLanguage.NUMERO_APPUNTAMENTI.stampa()));
			for(Sheet foglio : fogli) {
				int j = 0;
				Row riga = foglio.createRow(foglio.getPhysicalNumberOfRows());
				Cell cella = riga.createCell(j++, CellType.STRING);
				ApplicationUtil.headerCell(s, cella, ConstantsLanguage.OPERATORE.stampa());
				cella = riga.createCell(j++, CellType.STRING);
				ApplicationUtil.headerCell(s, cella, ConstantsLanguage.TOTALE.stampa());
				for(Dettaglio head : esitiAppuntamento) {
					cella = riga.createCell(j++, CellType.STRING);
					ApplicationUtil.headerCell(s, cella, head.getEtichetta());
				}
				foglio.setAutoFilter(new CellRangeAddress(0, 0, 0, ApplicationUtil.size(esitiAppuntamento)));
				foglio.createFreezePane(0, 1);
			}
			for(ReportAppuntamentoResultSet operatore : operatori) {
				int a = 1;
				int j = 0;
				Utente utente = userDao.getUser(operatore.getUser_id());
				Row riga[] = new Row[a];
				for(int i = 0; i < riga.length; i++) {
					riga[i] = fogli.get(i).createRow(fogli.get(i).getPhysicalNumberOfRows());
					ApplicationUtil.formatString(s, riga[i].createCell(j, CellType.STRING), utente.getDenominazione());
				}
				j++;
				ApplicationUtil.formatNumber(s, riga[0].createCell(j, CellType.NUMERIC), operatore.getAppuntamenti());
				j++;
				for(Dettaglio ec : esitiAppuntamento) {
					int i = dati.indexOf(new ReportAppuntamentoResultSet(utente.getId(), ec.getEsito_id()));
					ReportAppuntamentoResultSet dato = new ReportAppuntamentoResultSet();
					if(i >= 0) {
						dato = dati.get(i);
					}
					ApplicationUtil.formatNumber(s, riga[0].createCell(j, CellType.NUMERIC), dato.getAppuntamenti());
					j++;
				}
			}
			try {
				String nome = ApplicationUtil.formatIntervallo(ConstantsLanguage.RIEPILOGO.stampa() + "_" + ConstantsLanguage.APPUNTAMENTI.stampa(), r.getInizio(), r.getFine()) + ConstantsMap.xlsx;


				OutputStream out = downloadBean.outputStream();
				downloadBean.filename(nome);

				String p = ConstantsGlobal.secretE;
				ApplicationUtil.passwordXLS(wb, out, p);
				downloadBean.close();

			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		}
	}
	@SuppressWarnings("unchecked")
	public List<StoriaAppuntamento>  getReportAppuntamentiStoria(Long user, Long esitoId, ReportCriteria r, Long operatoreId, int start, int size) {
		ExtendedCriteria criteria = getReportAppuntamentiStoriaByCriteria(user, esitoId,r, operatoreId);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int  getReportAppuntamentiStoriaSize(Long user, Long esitoId, ReportCriteria r, Long operatoreId) {
		ExtendedCriteria criteria = getReportAppuntamentiStoriaByCriteria(user, esitoId,r, operatoreId);
		return getListSize(criteria);
	}
	private ExtendedCriteria  getReportAppuntamentiStoriaByCriteria(Long userId, Long esitoId, ReportCriteria r, Long operatoreId) {
		if(r != null)  r = (ReportCriteria) r.loadCriteria();
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaAppuntamento.class);
		DetachedCriteria criteriaCa = criteria.createCriteria("appuntamento");
		DetachedCriteria criteriaC = criteriaCa.createCriteria("contatto");
		DetachedCriteria criteriaUU = criteria.createCriteria("utente", "op");
		DetachedCriteria criteriaCm = criteriaC.createCriteria("campagna", "camp");
		if(ApplicationUtil.gtZero(userId)) {
			impostazioneDao.subCriteriaRoleSupervisore(userId, criteria, "camp", "op");
			if(ApplicationUtil.isTrue(sd)) {
				impostazioneDao.subCriteriaSupervisore(userId, criteriaUU, "id");
			}
			impostazioneDao.subCriteriaCampagnaSupervisore(userId, criteriaCm, "id");
		}
		if(r != null) {
			ApplicationUtil.dataIntervalD(r.getInizio(), r.getFine(), "data", criteria);
			if(ApplicationUtil.gtZero(r.getAgente())) {
				criteria.add(Restrictions.eq("agente.id",r.getAgente()));
			}
			if (ApplicationUtil.gtZero(r.getTeam())){
				if(ApplicationUtil.isTrue(sd)) {
					impostazioneDao.subCriteriaCampagnaTeam(r.getTeam(), criteriaCm, "id");
				}
				impostazioneDao.subCriteriaOperatoreTeam(r.getTeam(), criteriaUU, "id");
			}
		}
		criteriaUU.add(Restrictions.idEq(operatoreId));
		if(ApplicationUtil.gtZero(esitoId)) {
			criteria.add(Restrictions.eq("esito.id",esitoId));
		}
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, r);
	}
	@SuppressWarnings("unchecked")
	public List<Appuntamento>  getReportAppuntamenti(Long user, Long esitoId, ReportCriteria r, Long operatoreId, int start, int size) {
		ExtendedCriteria criteria = getReportAppuntamentiByCriteria(user, esitoId,r, operatoreId);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int  getReportAppuntamentiSize(Long user, Long esitoId, ReportCriteria r, Long operatoreId) {
		ExtendedCriteria criteria = getReportAppuntamentiByCriteria(user, esitoId, r, operatoreId);
		return getListSize(criteria);
	}
	private ExtendedCriteria  getReportAppuntamentiByCriteria(Long user, Long esitoId, ReportCriteria r, Long operatoreId) {
		if(r != null)  r = (ReportCriteria) r.loadCriteria();
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(Appuntamento.class);
		DetachedCriteria criteriaC = criteria.createCriteria("contatto");
		DetachedCriteria criteriaUU = criteria.createCriteria("operatore", "op");
		DetachedCriteria criteriaCm = criteriaC.createCriteria("campagna",  "camp");
		criteriaUU.add(Restrictions.idEq(operatoreId));
		if(ApplicationUtil.gtZero(esitoId)) {
			criteria.add(Restrictions.eq("esito.id",esitoId));
		}
		if(ApplicationUtil.gtZero(user)) {
			impostazioneDao.subCriteriaRoleSupervisore(user, criteria, "camp", "op");
			if(ApplicationUtil.isTrue(sd)) {
				impostazioneDao.subCriteriaSupervisore(user, criteriaUU, "id");
			}
			impostazioneDao.subCriteriaCampagnaSupervisore(user, criteriaCm, "id");
		}
		if(r != null) {
			ApplicationUtil.intervallo(ApplicationUtil.isTrue(r.getInserimento()), r.getInizio(), r.getFine(), criteria);
			if(ApplicationUtil.gtZero(r.getAgente())) {
				criteria.add(Restrictions.eq("agente.id",r.getAgente()));
			}
			if (ApplicationUtil.gtZero(r.getTeam())){
				if(ApplicationUtil.isTrue(sd)) {
					impostazioneDao.subCriteriaCampagnaTeam(r.getTeam(), criteriaCm, "id");
				}
				impostazioneDao.subCriteriaOperatoreTeam(r.getTeam(), criteriaUU, "id");
			}
		}
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, r);
	}
	public void exportReportAppuntamentoC(Integer tipo, OutputStream result) throws Exception {
		int a = 1;
		Date dataInizio = ApplicationUtil.dataInizio(tipo, ApplicationUtil.date());
		List<Dettaglio> esitiAppuntamento = getReportAppuntamenti();
		ReportCriteria r = new ReportCriteria();
		r.setIntervallo(dataInizio, null);
		List<ReportAppuntamentoCResultSet> campagne = getReportAppuntamentoCResultset(null, r);
		List<ReportAppuntamentoCResultSet> dati = getReportAppuntamentoCResultsetDati(null, r);
		Workbook wb = ApplicationUtil.workbookS();
		ExcelStyle s = new ExcelStyle(wb);
		List<Sheet> fogli = new ArrayList<Sheet>();
		fogli.add(wb.createSheet(ConstantsLanguage.NUMERO_APPUNTAMENTI.stampa()));
		for(Sheet foglio : fogli) {
			int j = 0;
			Row riga = foglio.createRow(foglio.getPhysicalNumberOfRows());
			Cell cella = riga.createCell(j++, CellType.STRING);
			ApplicationUtil.headerCell(s, cella, ConstantsLanguage.CAMPAGNA.stampa());
			cella = riga.createCell(j++, CellType.STRING);
			ApplicationUtil.headerCell(s, cella, ConstantsLanguage.TOTALE.stampa());
			for(Dettaglio head : esitiAppuntamento) {
				cella = riga.createCell(j++, CellType.STRING);
				ApplicationUtil.headerCell(s, cella, head.getEtichetta());
			}
			foglio.setAutoFilter(new CellRangeAddress(0, 0, 0, ApplicationUtil.size(esitiAppuntamento)));
			foglio.createFreezePane(0, 1);
		}
		for(ReportAppuntamentoCResultSet c : campagne) {
			int j = 0;
			Campagna campagna = getCampagna(c.getCampagna_id());
			Row riga[] = new Row[a];
			for(int i = 0; i < riga.length; i++) {
				riga[i] = fogli.get(i).createRow(fogli.get(i).getPhysicalNumberOfRows());
				ApplicationUtil.formatString(s, riga[i].createCell(j, CellType.STRING), campagna.getDenominazione());
			}
			j++;
			ApplicationUtil.formatNumber(s, riga[0].createCell(j, CellType.NUMERIC), c.getAppuntamenti());
			j++;
			for(Dettaglio ec : esitiAppuntamento) {
				int i = dati.indexOf(new ReportAppuntamentoCResultSet(campagna.getId(), ec.getEsito_id()));
				ReportAppuntamentoCResultSet dato = new ReportAppuntamentoCResultSet();
				if(i >= 0) {
					dato = dati.get(i);
				}
				ApplicationUtil.formatNumber(s, riga[0].createCell(j, CellType.NUMERIC), dato.getAppuntamenti());
				j++;
			}
		}
		String p = ConstantsGlobal.secretE;
		ApplicationUtil.passwordXLS(wb, result, p);
	}
	public void exportReportAppuntamentoCStoria(Integer tipo, OutputStream result) throws Exception {
		int a = 1;
		Date dataInizio = ApplicationUtil.dataInizio(tipo, ApplicationUtil.date());
		List<Dettaglio> esitiAppuntamento = getReportAppuntamenti();
		ReportCriteria r = new ReportCriteria();
		r.setIntervallo(dataInizio, null);
		List<ReportAppuntamentoCResultSet> campagne = getReportAppuntamentoCStoriaResultset(null, r);
		List<ReportAppuntamentoCResultSet> dati = getReportAppuntamentoCStoriaResultsetDati(null, r);
		Workbook wb = ApplicationUtil.workbookS();
		ExcelStyle s = new ExcelStyle(wb);
		List<Sheet> fogli = new ArrayList<Sheet>();
		fogli.add(wb.createSheet(ConstantsLanguage.NUMERO_APPUNTAMENTI.stampa()));
		for(Sheet foglio : fogli) {
			int j = 0;
			Row riga = foglio.createRow(foglio.getPhysicalNumberOfRows());
			Cell cella = riga.createCell(j++, CellType.STRING);
			ApplicationUtil.headerCell(s, cella, ConstantsLanguage.CAMPAGNA.stampa());
			cella = riga.createCell(j++, CellType.STRING);
			ApplicationUtil.headerCell(s, cella, ConstantsLanguage.TOTALE.stampa());
			for(Dettaglio head : esitiAppuntamento) {
				cella = riga.createCell(j++, CellType.STRING);
				ApplicationUtil.headerCell(s, cella, head.getEtichetta());
			}
			foglio.setAutoFilter(new CellRangeAddress(0, 0, 0, ApplicationUtil.size(esitiAppuntamento)));
			foglio.createFreezePane(0, 1);
		}
		for(ReportAppuntamentoCResultSet c : campagne) {
			int j = 0;
			Campagna campagna = getCampagna(c.getCampagna_id());
			Row riga[] = new Row[a];
			for(int i = 0; i < riga.length; i++) {
				riga[i] = fogli.get(i).createRow(fogli.get(i).getPhysicalNumberOfRows());
				ApplicationUtil.formatString(s, riga[i].createCell(j, CellType.STRING), campagna.getDenominazione());
			}
			j++;
			ApplicationUtil.formatNumber(s, riga[0].createCell(j, CellType.NUMERIC), c.getAppuntamenti());
			j++;
			for(Dettaglio ec : esitiAppuntamento) {
				int i = dati.indexOf(new ReportAppuntamentoCResultSet(campagna.getId(), ec.getEsito_id()));
				ReportAppuntamentoCResultSet dato = new ReportAppuntamentoCResultSet();
				if(i >= 0) {
					dato = dati.get(i);
				}
				ApplicationUtil.formatNumber(s, riga[0].createCell(j, CellType.NUMERIC), dato.getAppuntamenti());
				j++;
			}
		}
		String p = ConstantsGlobal.secretE;
		ApplicationUtil.passwordXLS(wb, result, p);
	}
	public List<ReportAppuntamentoC> getReportAppuntamentoCStoria(Long userid, ReportCriteria r) {
		if(r != null)  r = (ReportCriteria) r.loadCriteria(); 
		List<ReportAppuntamentoC> report = new ArrayList<ReportAppuntamentoC>();
		if(r == null || (r != null && r.getCerca())) {
			List<Dettaglio> esitiAppuntamento = getReportAppuntamenti();
			List<ReportAppuntamentoCResultSet> campagne = getReportAppuntamentoCStoriaResultset(userid, 	r);
			List<ReportAppuntamentoCResultSet> dati = getReportAppuntamentoCStoriaResultsetDati(userid, r);
			for(ReportAppuntamentoCResultSet c : campagne) {
				Campagna campagna = getCampagna(c.getCampagna_id());
				ReportAppuntamentoC report_app = new ReportAppuntamentoC();
				report_app.setCampagna(campagna);
				report_app.setTotaleAppuntamenti(new Dettaglio(c.getAppuntamenti(),null, null,null,null, null, null, null, null, null, null, null, null, null, null));
				List<Dettaglio> esitiAppuntamentoDettaglio = new ArrayList<Dettaglio>();
				for(Dettaglio ec : esitiAppuntamento) {
					Dettaglio d;
					int i = dati.indexOf(new ReportAppuntamentoCResultSet(campagna.getId(), ec.getEsito_id()));
					if(i >= 0) {
						ReportAppuntamentoCResultSet dato = dati.get(i);
						d = new Dettaglio(dato.getAppuntamenti(),null, null,null,null, null, null, null, null, null, null, null, null, null, null);
						d.p_numero(dato.getAppuntamenti(), c.getAppuntamenti());
					}
					else d = new Dettaglio(true);
					d.setEsito_id(ec.getEsito_id());
					d.setEtichetta(ec.getEtichetta());
					esitiAppuntamentoDettaglio.add(d);
				}
				report_app.setEsitiAppuntamentiList(esitiAppuntamentoDettaglio);
				report.add(report_app);
			}
		}
		return report;
	}
	public void exportReportAppuntamentiCStoria(Long userId, ReportCriteria r, DownloadBean downloadBean) {
		if(r != null)  r = (ReportCriteria) r.loadCriteria();
		if(r == null || (r != null && r.getCerca())) {
			int a = 1;
			List<Dettaglio> esitiAppuntamento = getReportAppuntamenti();
			List<ReportAppuntamentoCResultSet> campagne = getReportAppuntamentoCStoriaResultset(userId, r);
			List<ReportAppuntamentoCResultSet> dati = getReportAppuntamentoCStoriaResultsetDati(userId, r);
			Workbook wb = ApplicationUtil.workbookS();
			ExcelStyle s = new ExcelStyle(wb);
			List<Sheet> fogli = new ArrayList<Sheet>();
			fogli.add(wb.createSheet(ConstantsLanguage.NUMERO_APPUNTAMENTI.stampa()));
			for(Sheet foglio : fogli) {
				int j = 0;
				Row riga = foglio.createRow(foglio.getPhysicalNumberOfRows());
				Cell cella = riga.createCell(j++, CellType.STRING);
				ApplicationUtil.headerCell(s, cella, ConstantsLanguage.CAMPAGNA.stampa());
				cella = riga.createCell(j++, CellType.STRING);
				ApplicationUtil.headerCell(s, cella, ConstantsLanguage.TOTALE.stampa());
				for(Dettaglio head : esitiAppuntamento) {
					cella = riga.createCell(j++, CellType.STRING);
					ApplicationUtil.headerCell(s, cella, head.getEtichetta());
				}
				foglio.setAutoFilter(new CellRangeAddress(0, 0, 0, ApplicationUtil.size(esitiAppuntamento)));
				foglio.createFreezePane(0, 1);
			}
			for(ReportAppuntamentoCResultSet c : campagne) {
				int j = 0;
				Campagna campagna = getCampagna(c.getCampagna_id());
				Row riga[] = new Row[a];
				for(int i = 0; i < riga.length; i++) {
					riga[i] = fogli.get(i).createRow(fogli.get(i).getPhysicalNumberOfRows());
					ApplicationUtil.formatString(s, riga[i].createCell(j, CellType.STRING), campagna.getDenominazione());
				}
				j++;
				ApplicationUtil.formatNumber(s, riga[0].createCell(j, CellType.NUMERIC), c.getAppuntamenti());
				j++;
				for(Dettaglio ec : esitiAppuntamento) {
					int i = dati.indexOf(new ReportAppuntamentoCResultSet(campagna.getId(), ec.getEsito_id()));
					ReportAppuntamentoCResultSet dato = new ReportAppuntamentoCResultSet();
					if(i >= 0) {
						dato = dati.get(i);
					}
					ApplicationUtil.formatNumber(s, riga[0].createCell(j, CellType.NUMERIC), dato.getAppuntamenti());
					j++;
				}
			}
			try {
				String nome = ApplicationUtil.formatIntervallo(ConstantsLanguage.REPORT.stampa() + "_" + ConstantsLanguage.APPUNTAMENTI.stampa(), r.getInizio(), r.getFine()) + ConstantsMap.xlsx;


				OutputStream out = downloadBean.outputStream();
				downloadBean.filename(nome);

				String p = ConstantsGlobal.secretE;
				ApplicationUtil.passwordXLS(wb, out, p);
				downloadBean.close();

			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		}
	}
	public List<ReportAppuntamentoC> getReportAppuntamentoC(Long userid, ReportCriteria r) {
		if(r != null)  r = (ReportCriteria) r.loadCriteria(); 
		List<ReportAppuntamentoC> report = new ArrayList<ReportAppuntamentoC>();
		if(r == null || (r != null && r.getCerca())) {
			List<Dettaglio> esitiAppuntamento = getReportAppuntamenti();
			List<ReportAppuntamentoCResultSet> campagne = getReportAppuntamentoCResultset(userid, 	r);
			List<ReportAppuntamentoCResultSet> dati = getReportAppuntamentoCResultsetDati(userid, r);
			for(ReportAppuntamentoCResultSet c : campagne) {
				Campagna campagna = getCampagna(c.getCampagna_id());
				ReportAppuntamentoC report_app = new ReportAppuntamentoC();
				report_app.setCampagna(campagna);
				report_app.setTotaleAppuntamenti(new Dettaglio(c.getAppuntamenti(),null, null,null,null, null, null, null, null, null, null, null, null, null, null));
				List<Dettaglio> esitiAppuntamentoDettaglio = new ArrayList<Dettaglio>();
				for(Dettaglio ec : esitiAppuntamento) {
					Dettaglio d;
					int i = dati.indexOf(new ReportAppuntamentoCResultSet(campagna.getId(), ec.getEsito_id()));
					if(i >= 0) {
						ReportAppuntamentoCResultSet dato = dati.get(i);
						d = new Dettaglio(dato.getAppuntamenti(),null, null,null,null, null, null, null, null, null, null, null, null, null, null);
						d.p_numero(dato.getAppuntamenti(), c.getAppuntamenti());
					}
					else d = new Dettaglio(true);
					d.setEsito_id(ec.getEsito_id());
					d.setEtichetta(ec.getEtichetta());
					esitiAppuntamentoDettaglio.add(d);
				}
				report_app.setEsitiAppuntamentiList(esitiAppuntamentoDettaglio);
				report.add(report_app);
			}
		}
		return report;
	}
	public void exportReportAppuntamentiC(Long userId, ReportCriteria r, DownloadBean downloadBean) {
		if(r != null)  r = (ReportCriteria) r.loadCriteria();
		if(r == null || (r != null && r.getCerca())) {
			List<Dettaglio> esitiAppuntamento = getReportAppuntamenti();
			List<ReportAppuntamentoCResultSet> campagne = getReportAppuntamentoCResultset(userId, r);
			List<ReportAppuntamentoCResultSet> dati = getReportAppuntamentoCResultsetDati(userId, r);
			Workbook wb = ApplicationUtil.workbookS();
			ExcelStyle s = new ExcelStyle(wb);
			List<Sheet> fogli = new ArrayList<Sheet>();
			fogli.add(wb.createSheet(ConstantsLanguage.NUMERO_APPUNTAMENTI.stampa()));
			for(Sheet foglio : fogli) {
				int j = 0;
				Row riga = foglio.createRow(foglio.getPhysicalNumberOfRows());
				Cell cella = riga.createCell(j++, CellType.STRING);
				ApplicationUtil.headerCell(s, cella, ConstantsLanguage.CAMPAGNA.stampa());
				cella = riga.createCell(j++, CellType.STRING);
				ApplicationUtil.headerCell(s, cella, ConstantsLanguage.TOTALE.stampa());
				for(Dettaglio head : esitiAppuntamento) {
					cella = riga.createCell(j++, CellType.STRING);
					ApplicationUtil.headerCell(s, cella, head.getEtichetta());
				}
				foglio.setAutoFilter(new CellRangeAddress(0, 0, 0, ApplicationUtil.size(esitiAppuntamento)));
				foglio.createFreezePane(0, 1);
			}
			for(ReportAppuntamentoCResultSet c : campagne) {
				int a = 1;
				int j = 0;
				Campagna campagna = getCampagna(c.getCampagna_id());
				Row riga[] = new Row[a];
				for(int i = 0; i < riga.length; i++) {
					riga[i] = fogli.get(i).createRow(fogli.get(i).getPhysicalNumberOfRows());
					ApplicationUtil.formatString(s, riga[i].createCell(j, CellType.STRING), campagna.getDenominazione());
				}
				j++;
				ApplicationUtil.formatNumber(s, riga[0].createCell(j, CellType.NUMERIC), c.getAppuntamenti());
				j++;
				for(Dettaglio ec : esitiAppuntamento) {
					int i = dati.indexOf(new ReportAppuntamentoCResultSet(campagna.getId(), ec.getEsito_id()));
					ReportAppuntamentoCResultSet dato = new ReportAppuntamentoCResultSet();
					if(i >= 0) {
						dato = dati.get(i);
					}
					ApplicationUtil.formatNumber(s, riga[0].createCell(j, CellType.NUMERIC), dato.getAppuntamenti());
					j++;
				}
			}
			try {
				String nome = ApplicationUtil.formatIntervallo(ConstantsLanguage.RIEPILOGO.stampa() + "_" + ConstantsLanguage.APPUNTAMENTI.stampa(), r.getInizio(), r.getFine()) + ConstantsMap.xlsx;


				OutputStream out = downloadBean.outputStream();
				downloadBean.filename(nome);

				String p = ConstantsGlobal.secretE;
				ApplicationUtil.passwordXLS(wb, out, p);
				downloadBean.close();

			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		}
	}
	@SuppressWarnings("unchecked")
	public List<StoriaAppuntamento>  getReportAppuntamentiCStoria(Long user, Long esitoId, ReportCriteria r, Long campagnaID, int start, int size) {
		ExtendedCriteria criteria = getReportAppuntamentiCStoriaByCriteria(user, esitoId,r, campagnaID);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int  getReportAppuntamentiCStoriaSize(Long user, Long esitoId, ReportCriteria r, Long campagnaID) {
		ExtendedCriteria criteria = getReportAppuntamentiCStoriaByCriteria(user, esitoId,r, campagnaID);
		return getListSize(criteria);
	}
	private ExtendedCriteria  getReportAppuntamentiCStoriaByCriteria(Long userId, Long esitoId, ReportCriteria r, Long campagnaID) {
		if(r != null)  r = (ReportCriteria) r.loadCriteria();
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaAppuntamento.class);
		DetachedCriteria criteriaCa = criteria.createCriteria("appuntamento");
		DetachedCriteria criteriaC = criteriaCa.createCriteria("contatto");
		DetachedCriteria criteriaCm = criteriaC.createCriteria("campagna", "camp");
		DetachedCriteria criteriaUU = criteria.createCriteria("utente", "op");
		criteriaCm.add(Restrictions.idEq(campagnaID));
		if(ApplicationUtil.gtZero(userId)) {
			impostazioneDao.subCriteriaRoleSupervisore(userId, criteria, "camp", "op");
			if(ApplicationUtil.isTrue(sd)) {
				impostazioneDao.subCriteriaSupervisore(userId, criteriaUU, "id");
			}
			impostazioneDao.subCriteriaCampagnaSupervisore(userId, criteriaCm, "id");
		}
		if(r != null) {
			ApplicationUtil.dataIntervalD(r.getInizio(), r.getFine(), "data", criteria);
			if(ApplicationUtil.gtZero(r.getAgente())) {
				criteria.add(Restrictions.eq("agente.id",r.getAgente()));
			}
			if (ApplicationUtil.gtZero(r.getTeam())){
				if(ApplicationUtil.isTrue(sd)) {
					impostazioneDao.subCriteriaCampagnaTeam(r.getTeam(), criteriaCm, "id");
				}
				impostazioneDao.subCriteriaOperatoreTeam(r.getTeam(), criteriaUU, "id");
			} 
		}
		if(ApplicationUtil.gtZero(esitoId)) {
			criteria.add(Restrictions.eq("esito.id",esitoId));
		}
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, r);
	}
	@SuppressWarnings("unchecked")
	public List<Appuntamento>  getReportAppuntamentiC(Long user, Long esitoId, ReportCriteria r, Long campagnaID, int start, int size) {
		ExtendedCriteria criteria = getReportAppuntamentiCByCriteria(user, esitoId,r, campagnaID);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int  getReportAppuntamentiCSize(Long user, Long esitoId, ReportCriteria r, Long campagnaID) {
		ExtendedCriteria criteria = getReportAppuntamentiCByCriteria(user, esitoId, r, campagnaID);
		return getListSize(criteria);
	}
	private ExtendedCriteria  getReportAppuntamentiCByCriteria(Long user, Long esitoId, ReportCriteria r, Long campagnaID) {
		if(r != null)  r = (ReportCriteria) r.loadCriteria();
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(Appuntamento.class);
		DetachedCriteria criteriaC = criteria.createCriteria("contatto");
		DetachedCriteria criteriaCm = criteriaC.createCriteria("campagna", "camp");
		DetachedCriteria criteriaUU = criteria.createCriteria("operatore", "op");
		criteriaCm.add(Restrictions.idEq(campagnaID));
		if(ApplicationUtil.gtZero(esitoId)) {
			criteria.add(Restrictions.eq("esito.id",esitoId));
		}
		if(ApplicationUtil.gtZero(user)) {
			impostazioneDao.subCriteriaRoleSupervisore(user, criteria, "camp", "op");
			if(ApplicationUtil.isTrue(sd)) {
				impostazioneDao.subCriteriaSupervisore(user, criteriaUU, "id");
			}
			impostazioneDao.subCriteriaCampagnaSupervisore(user, criteriaCm, "id");
		}
		if(r != null) {
			ApplicationUtil.intervallo(ApplicationUtil.isTrue(r.getInserimento()), r.getInizio(), r.getFine(), criteria);
			if(ApplicationUtil.gtZero(r.getAgente())) {
				criteria.add(Restrictions.eq("agente.id",r.getAgente()));
			}
			if (ApplicationUtil.gtZero(r.getTeam())){
				if(ApplicationUtil.isTrue(sd)) {
					impostazioneDao.subCriteriaCampagnaTeam(r.getTeam(), criteriaCm, "id");
				}
				impostazioneDao.subCriteriaOperatoreTeam(r.getTeam(), criteriaUU, "id");
			}
		}
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, r);
	}
	public void exportReportContrattoC(Integer tipo, OutputStream result) throws Exception {
		int a = 1;
		Date dataInizio = ApplicationUtil.dataInizio(tipo, ApplicationUtil.date());
		List<Dettaglio> esitiContratto = getReportContratti();
		ReportCriteria r = new ReportCriteria();
		r.setIntervallo(dataInizio, null);
		List<ReportContrattoCResultSet> campagne = getReportContrattoCResultset(null, r);
		List<ReportContrattoCResultSet> dati = getReportContrattoCResultsetDati(null, r);
		Workbook wb = ApplicationUtil.workbookS();
		ExcelStyle s = new ExcelStyle(wb);
		List<Sheet> fogli = new ArrayList<Sheet>();
		fogli.add(wb.createSheet(ConstantsLanguage.NUMERO_CONTRATTI.stampa()));
		for(Sheet foglio : fogli) {
			int j = 0;
			Row riga = foglio.createRow(foglio.getPhysicalNumberOfRows());
			Cell cella = riga.createCell(j++, CellType.STRING);
			ApplicationUtil.headerCell(s, cella, ConstantsLanguage.CAMPAGNA.stampa());
			cella = riga.createCell(j++, CellType.STRING);
			ApplicationUtil.headerCell(s, cella, ConstantsLanguage.TOTALE.stampa());
			for(Dettaglio head : esitiContratto) {
				cella = riga.createCell(j++, CellType.STRING);
				ApplicationUtil.headerCell(s, cella, head.getEtichetta());
			}
			foglio.setAutoFilter(new CellRangeAddress(0, 0, 0, ApplicationUtil.size(esitiContratto)));
			foglio.createFreezePane(0, 1);
		}
		for(ReportContrattoCResultSet c : campagne) {
			int j = 0;
			Campagna campagna = getCampagna(c.getCampagna_id());
			Row riga[] = new Row[a];
			for(int i = 0; i < riga.length; i++) {
				riga[i] = fogli.get(i).createRow(fogli.get(i).getPhysicalNumberOfRows());
				ApplicationUtil.formatString(s, riga[i].createCell(j, CellType.STRING), campagna.getDenominazione());
			}
			j++;
			ApplicationUtil.formatNumber(s, riga[0].createCell(j, CellType.NUMERIC), c.getContratti());
			j++;
			for(Dettaglio ec : esitiContratto) {
				int i = dati.indexOf(new ReportContrattoCResultSet(campagna.getId(), ec.getEsito_id()));
				ReportContrattoCResultSet dato = new ReportContrattoCResultSet();
				if(i >= 0) {
					dato = dati.get(i);
				}
				ApplicationUtil.formatNumber(s, riga[0].createCell(j, CellType.NUMERIC), dato.getContratti());
				j++;
			}
		}
		String p = ConstantsGlobal.secretE;
		ApplicationUtil.passwordXLS(wb, result, p);
	}
	public void exportReportContrattoCStoria(Integer tipo, OutputStream result) throws Exception {
		int a = 1;
		Date dataInizio = ApplicationUtil.dataInizio(tipo, ApplicationUtil.date());
		List<Dettaglio> esitiContratto = getReportContratti();
		ReportCriteria r = new ReportCriteria();
		r.setIntervallo(dataInizio, null);
		List<ReportContrattoCResultSet> campagne = getReportContrattoCStoriaResultset(null, r);
		List<ReportContrattoCResultSet> dati = getReportContrattoCStoriaResultsetDati(null, r);
		Workbook wb = ApplicationUtil.workbookS();
		ExcelStyle s = new ExcelStyle(wb);
		List<Sheet> fogli = new ArrayList<Sheet>();
		fogli.add(wb.createSheet(ConstantsLanguage.NUMERO_CONTRATTI.stampa()));
		for(Sheet foglio : fogli) {
			int j = 0;
			Row riga = foglio.createRow(foglio.getPhysicalNumberOfRows());
			Cell cella = riga.createCell(j++, CellType.STRING);
			ApplicationUtil.headerCell(s, cella, ConstantsLanguage.CAMPAGNA.stampa());
			cella = riga.createCell(j++, CellType.STRING);
			ApplicationUtil.headerCell(s, cella, ConstantsLanguage.TOTALE.stampa());
			for(Dettaglio head : esitiContratto) {
				cella = riga.createCell(j++, CellType.STRING);
				ApplicationUtil.headerCell(s, cella, head.getEtichetta());
			}
			foglio.setAutoFilter(new CellRangeAddress(0, 0, 0, ApplicationUtil.size(esitiContratto)));
			foglio.createFreezePane(0, 1);
		}
		for(ReportContrattoCResultSet c : campagne) {
			int j = 0;
			Campagna campagna = getCampagna(c.getCampagna_id());
			Row riga[] = new Row[a];
			for(int i = 0; i < riga.length; i++) {
				riga[i] = fogli.get(i).createRow(fogli.get(i).getPhysicalNumberOfRows());
				ApplicationUtil.formatString(s, riga[i].createCell(j, CellType.STRING), campagna.getDenominazione());
			}
			j++;
			ApplicationUtil.formatNumber(s, riga[0].createCell(j, CellType.NUMERIC), c.getContratti());
			j++;
			for(Dettaglio ec : esitiContratto) {
				int i = dati.indexOf(new ReportContrattoCResultSet(campagna.getId(), ec.getEsito_id()));
				ReportContrattoCResultSet dato = new ReportContrattoCResultSet();
				if(i >= 0) {
					dato = dati.get(i);
				}
				ApplicationUtil.formatNumber(s, riga[0].createCell(j, CellType.NUMERIC), dato.getContratti());
				j++;
			}
		}
		String p = ConstantsGlobal.secretE;
		ApplicationUtil.passwordXLS(wb, result, p);
	}
	public List<ReportContrattoC> getReportContrattoCStoria(Long userid, ReportCriteria r) {
		if(r != null)  r = (ReportCriteria) r.loadCriteria(); 
		List<ReportContrattoC> report = new ArrayList<ReportContrattoC>();
		if(r == null || (r != null && r.getCerca())) {
			List<Dettaglio> esitiContratto = getReportContratti();
			List<ReportContrattoCResultSet> campagne = getReportContrattoCStoriaResultset(userid, r);
			List<ReportContrattoCResultSet> dati = getReportContrattoCStoriaResultsetDati(userid, r);
			for(ReportContrattoCResultSet c : campagne) {
				Campagna campagna = getCampagna(c.getCampagna_id());
				ReportContrattoC report_app = new ReportContrattoC();
				report_app.setCampagna(campagna);
				report_app.setTotaleContratti(new Dettaglio(c.getContratti(),null, null,null,null, null, null, null, null, null, null, null, null, null, null));
				List<Dettaglio> esitiContrattoDettaglio = new ArrayList<Dettaglio>();
				for(Dettaglio ec : esitiContratto) {
					Dettaglio d;
					int i = dati.indexOf(new ReportContrattoCResultSet(campagna.getId(), ec.getEsito_id()));
					if(i >= 0) {
						ReportContrattoCResultSet dato = dati.get(i);
						d = new Dettaglio(dato.getContratti(),null, null,null,null, null, null, null, null, null, null, null, null, null, null);
						d.p_numero(dato.getContratti(), c.getContratti());
					}
					else d = new Dettaglio(true);
					d.setEsito_id(ec.getEsito_id());
					d.setEtichetta(ec.getEtichetta());
					esitiContrattoDettaglio.add(d);
				}
				report_app.setEsitiContrattiList(esitiContrattoDettaglio);
				report.add(report_app);
			}
		}
		return report;
	}
	public void exportReportContrattiCStoria(Long userId, ReportCriteria r, DownloadBean downloadBean) {
		if(r != null)  r = (ReportCriteria) r.loadCriteria();
		if(r == null || (r != null && r.getCerca())) {
			int a = 1;
			List<Dettaglio> esitiContratto = getReportContratti();
			List<ReportContrattoCResultSet> campagne = getReportContrattoCStoriaResultset(userId, r);
			List<ReportContrattoCResultSet> dati = getReportContrattoCStoriaResultsetDati(userId, r);
			Workbook wb = ApplicationUtil.workbookS();
			ExcelStyle s = new ExcelStyle(wb);
			List<Sheet> fogli = new ArrayList<Sheet>();
			fogli.add(wb.createSheet(ConstantsLanguage.NUMERO_CONTRATTI.stampa()));
			for(Sheet foglio : fogli) {
				int j = 0;
				Row riga = foglio.createRow(foglio.getPhysicalNumberOfRows());
				Cell cella = riga.createCell(j++, CellType.STRING);
				ApplicationUtil.headerCell(s, cella, ConstantsLanguage.CAMPAGNA.stampa());
				cella = riga.createCell(j++, CellType.STRING);
				ApplicationUtil.headerCell(s, cella, ConstantsLanguage.TOTALE.stampa());
				for(Dettaglio head : esitiContratto) {
					cella = riga.createCell(j++, CellType.STRING);
					ApplicationUtil.headerCell(s, cella, head.getEtichetta());
				}
				foglio.setAutoFilter(new CellRangeAddress(0, 0, 0, ApplicationUtil.size(esitiContratto)));
				foglio.createFreezePane(0, 1);
			}
			for(ReportContrattoCResultSet c : campagne) {
				int j = 0;
				Campagna campagna = getCampagna(c.getCampagna_id());
				Row riga[] = new Row[a];
				for(int i = 0; i < riga.length; i++) {
					riga[i] = fogli.get(i).createRow(fogli.get(i).getPhysicalNumberOfRows());
					ApplicationUtil.formatString(s, riga[i].createCell(j, CellType.STRING), campagna.getDenominazione());
				}
				j++;
				ApplicationUtil.formatNumber(s, riga[0].createCell(j, CellType.NUMERIC), c.getContratti());
				j++;
				for(Dettaglio ec : esitiContratto) {
					int i = dati.indexOf(new ReportContrattoCResultSet(campagna.getId(), ec.getEsito_id()));
					ReportContrattoCResultSet dato = new ReportContrattoCResultSet();
					if(i >= 0) {
						dato = dati.get(i);
					}
					ApplicationUtil.formatNumber(s, riga[0].createCell(j, CellType.NUMERIC), dato.getContratti());
					j++;
				}
			}
			try {
				String nome = ApplicationUtil.formatIntervallo(ConstantsLanguage.REPORT.stampa() + "_" + ConstantsLanguage.CONTRATTI.stampa(), r.getInizio(), r.getFine()) + ConstantsMap.xlsx;


				OutputStream out = downloadBean.outputStream();
				downloadBean.filename(nome);

				String p = ConstantsGlobal.secretE;
				ApplicationUtil.passwordXLS(wb, out, p);
				downloadBean.close();

			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		}
	}
	public List<ReportContrattoC> getReportContrattoC(Long userid, ReportCriteria r) {
		if(r != null)  r = (ReportCriteria) r.loadCriteria(); 
		List<ReportContrattoC> report = new ArrayList<ReportContrattoC>();
		if(r == null || (r != null && r.getCerca())) {
			List<Dettaglio> esitiContratto = getReportContratti();
			List<ReportContrattoCResultSet> campagne = getReportContrattoCResultset(userid, 	r);
			List<ReportContrattoCResultSet> dati = getReportContrattoCResultsetDati(userid, r);
			for(ReportContrattoCResultSet c : campagne) {
				Campagna campagna = getCampagna(c.getCampagna_id());
				ReportContrattoC report_app = new ReportContrattoC();
				report_app.setCampagna(campagna);
				report_app.setTotaleContratti(new Dettaglio(c.getContratti(),null, null,null,null, null, null, null, null, null, null, null, null, null, null));
				List<Dettaglio> esitiContrattoDettaglio = new ArrayList<Dettaglio>();
				for(Dettaglio ec : esitiContratto) {
					Dettaglio d;
					int i = dati.indexOf(new ReportContrattoCResultSet(campagna.getId(), ec.getEsito_id()));
					if(i >= 0) {
						ReportContrattoCResultSet dato = dati.get(i);
						d = new Dettaglio(dato.getContratti(),null, null,null,null, null, null, null, null, null, null, null, null, null, null);
						d.p_numero(dato.getContratti(), c.getContratti());
					}
					else d = new Dettaglio(true);
					d.setEsito_id(ec.getEsito_id());
					d.setEtichetta(ec.getEtichetta());
					esitiContrattoDettaglio.add(d);
				}
				report_app.setEsitiContrattiList(esitiContrattoDettaglio);
				report.add(report_app);
			}
		}
		return report;
	}
	public void exportReportContrattiC(Long userId, ReportCriteria r, DownloadBean downloadBean) {
		if(r != null)  r = (ReportCriteria) r.loadCriteria();
		if(r == null || (r != null && r.getCerca())) {
			int a = 1;
			List<Dettaglio> esitiContratto = getReportContratti();
			List<ReportContrattoCResultSet> campagne = getReportContrattoCResultset(userId, r);
			List<ReportContrattoCResultSet> dati = getReportContrattoCResultsetDati(userId, r);
			Workbook wb = ApplicationUtil.workbookS();
			ExcelStyle s = new ExcelStyle(wb);
			List<Sheet> fogli = new ArrayList<Sheet>();
			fogli.add(wb.createSheet(ConstantsLanguage.NUMERO_CONTRATTI.stampa()));
			for(Sheet foglio : fogli) {
				int j = 0;
				Row riga = foglio.createRow(foglio.getPhysicalNumberOfRows());
				Cell cella = riga.createCell(j++, CellType.STRING);
				ApplicationUtil.headerCell(s, cella, ConstantsLanguage.CAMPAGNA.stampa());
				cella = riga.createCell(j++, CellType.STRING);
				ApplicationUtil.headerCell(s, cella, ConstantsLanguage.TOTALE.stampa());
				for(Dettaglio head : esitiContratto) {
					cella = riga.createCell(j++, CellType.STRING);
					ApplicationUtil.headerCell(s, cella, head.getEtichetta());
				}
				foglio.setAutoFilter(new CellRangeAddress(0, 0, 0, ApplicationUtil.size(esitiContratto)));
				foglio.createFreezePane(0, 1);
			}
			for(ReportContrattoCResultSet c : campagne) {
				int j = 0;
				Campagna campagna = getCampagna(c.getCampagna_id());
				Row riga[] = new Row[a];
				for(int i = 0; i < riga.length; i++) {
					riga[i] = fogli.get(i).createRow(fogli.get(i).getPhysicalNumberOfRows());
					ApplicationUtil.formatString(s, riga[i].createCell(j, CellType.STRING), campagna.getDenominazione());
				}
				j++;
				ApplicationUtil.formatNumber(s, riga[0].createCell(j, CellType.NUMERIC), c.getContratti());
				j++;
				for(Dettaglio ec : esitiContratto) {
					int i = dati.indexOf(new ReportContrattoCResultSet(campagna.getId(), ec.getEsito_id()));
					ReportContrattoCResultSet dato = new ReportContrattoCResultSet();
					if(i >= 0) {
						dato = dati.get(i);
					}
					ApplicationUtil.formatNumber(s, riga[0].createCell(j, CellType.NUMERIC), dato.getContratti());
					j++;
				}
			}
			try {
				String nome = ApplicationUtil.formatIntervallo(ConstantsLanguage.RIEPILOGO.stampa() + "_" + ConstantsLanguage.CONTRATTI.stampa(), r.getInizio(), r.getFine()) + ConstantsMap.xlsx;


				OutputStream out = downloadBean.outputStream();
				downloadBean.filename(nome);

				String p = ConstantsGlobal.secretE;
				ApplicationUtil.passwordXLS(wb, out, p);
				downloadBean.close();

			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		}
	}
	private ExtendedCriteria  getReportContrattiCStoriaByCriteria(Long user, Long esitoId, ReportCriteria r, Long campagnaID) {
		if(r != null)  r = (ReportCriteria) r.loadCriteria();
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContratto.class);
		DetachedCriteria criteriaCa = criteria.createCriteria("contratto");
		DetachedCriteria criteriaP = criteriaCa.createCriteria("prodotto");
		DetachedCriteria criteriaC = criteriaCa.createCriteria("contatto");
		DetachedCriteria criteriaCm = criteriaC.createCriteria("campagna", "ca");
		DetachedCriteria criteriaUU = criteria.createCriteria("utente", "op");
		criteriaCm.add(Restrictions.idEq(campagnaID));
		if(ApplicationUtil.gtZero(user)) {
			impostazioneDao.subCriteriaRoleSupervisore(user, criteria, "ca", "op");
			if(ApplicationUtil.isTrue(sd)) {
				impostazioneDao.subCriteriaSupervisore(user, criteriaUU, "id");
			}
			impostazioneDao.subCriteriaCampagnaSupervisore(user, criteriaCm, "id");
		}
		if(r != null) {
			ApplicationUtil.dataIntervalD(r.getInizio(), r.getFine(), "data", criteria);
			if(ApplicationUtil.gtZero(r.getAgente())) {
				criteria.add(Restrictions.eq("agente.id",r.getAgente()));
			}
			if (ApplicationUtil.gtZero(r.getTeam())){
				if(ApplicationUtil.isTrue(sd)) {
					impostazioneDao.subCriteriaCampagnaTeam(r.getTeam(), criteriaCm, "id");
				}
				impostazioneDao.subCriteriaOperatoreTeam(r.getTeam(), criteriaUU, "id");
			}
			if (ApplicationUtil.gtZero(r.getProdottoId())) {
				criteriaP.add(Restrictions.idEq(r.getProdottoId()));
			}
			if (ApplicationUtil.gtZero(r.getCategoriaId())) {
				criteriaP.add(Restrictions.eq("categoria.id",r.getCategoriaId()));
			}
		}
		if(ApplicationUtil.gtZero(esitoId)) {
			criteria.add(Restrictions.eq("esito.id",esitoId));
		}
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, r);
	}
	@SuppressWarnings("unchecked")
	public List<StoriaContratto>  getReportContrattiCStoria(Long user, Long esito, ReportCriteria r, Long campagnaID, int start, int size) {
		ExtendedCriteria criteria = getReportContrattiCStoriaByCriteria(user, esito, r, campagnaID);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int  getReportContrattiCStoriaSize(Long user, Long esito, ReportCriteria r, Long campagnaID) {
		ExtendedCriteria criteria = getReportContrattiCStoriaByCriteria(user, esito, r, campagnaID);
		return getListSize(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Contratto>  getReportContrattiC(Long user, Long esitoId, ReportCriteria r, Long campagnaID, int start, int size) {
		ExtendedCriteria criteria = getReportContrattiCByCriteria(user, esitoId, r, campagnaID);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int  getReportContrattiCSize(Long user,  Long esitoId,  ReportCriteria r, Long campagnaID) {
		ExtendedCriteria criteria = getReportContrattiCByCriteria(user, esitoId, r, campagnaID);
		return getListSize(criteria);
	}
	private ExtendedCriteria  getReportContrattiCByCriteria(Long user, Long esitoId,  ReportCriteria r, Long campagnaID) {
		if(r != null)  r = (ReportCriteria) r.loadCriteria();
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(Contratto.class);
		DetachedCriteria criteriaP = criteria.createCriteria("prodotto");
		DetachedCriteria criteriaC = criteria.createCriteria("contatto");
		DetachedCriteria criteriaCm = criteriaC.createCriteria("campagna", "ca");
		DetachedCriteria criteriaUU = criteria.createCriteria("operatore", "op");
		criteriaCm.add(Restrictions.idEq(campagnaID));
		if(ApplicationUtil.gtZero(esitoId)) {
			criteria.add(Restrictions.eq("esito.id",esitoId));
		}
		if(ApplicationUtil.gtZero(user)) {
			impostazioneDao.subCriteriaRoleSupervisore(user, criteria, "ca", "op");
			if(ApplicationUtil.isTrue(sd)) {
				impostazioneDao.subCriteriaSupervisore(user, criteriaUU, "id");
			}
			impostazioneDao.subCriteriaCampagnaSupervisore(user, criteriaCm, "id");
		}
		if(r != null) {
			ApplicationUtil.intervallo(ApplicationUtil.isTrue(r.getInserimento()), r.getInizio(), r.getFine(), criteria);
			if(ApplicationUtil.gtZero(r.getAgente())) {
				criteria.add(Restrictions.eq("agente.id",r.getAgente()));
			}
			if (ApplicationUtil.gtZero(r.getTeam())){
				if(ApplicationUtil.isTrue(sd)) {
					impostazioneDao.subCriteriaCampagnaTeam(r.getTeam(), criteriaCm, "id");
				}
				impostazioneDao.subCriteriaOperatoreTeam(r.getTeam(), criteriaUU, "id");
			}
			if (ApplicationUtil.gtZero(r.getProdottoId())) {
				criteriaP.add(Restrictions.idEq(r.getProdottoId()));
			}
			if (ApplicationUtil.gtZero(r.getCategoriaId())) {
				criteriaP.add(Restrictions.eq("categoria.id",r.getCategoriaId()));
			}
		}
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, r);
	}
	public Campagna getCampagna(Long ca) {
		return  (Campagna) get(Campagna.class, ca);
	}
	@SuppressWarnings("unchecked")
	private List<ReportAppuntamentoCResultSet> getReportAppuntamentoCStoriaResultset(Long userId, ReportCriteria r) {
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaAppuntamento.class, "sa");
		DetachedCriteria criteriaO = criteria.createCriteria("appuntamento", "app");
		DetachedCriteria criteriaCA = criteriaO.createCriteria("contatto").createCriteria("campagna", "ca");
		DetachedCriteria criteriaA = criteriaO.createCriteria("agente", "ag", ApplicationUtil.leftJoin());
		DetachedCriteria criteriaC = criteria.createCriteria("utente", "op");
		if(ApplicationUtil.gtZero(userId)) {
			impostazioneDao.subCriteriaRoleSupervisore(userId, criteria, "ca", "op");
			impostazioneDao.subCriteriaSupervisore(userId, criteriaC, "id");
			if(ApplicationUtil.isTrue(sd)) {
				impostazioneDao.subCriteriaCampagnaSupervisore(userId, criteriaCA, "id");
			}
		}
		if(r != null) {
			ApplicationUtil.dataIntervalD(r.getInizio(), r.getFine(), "data", criteria);
			if(ApplicationUtil.gtZero(r.getAgente())) {
				criteriaA.add(Restrictions.idEq(r.getAgente()));
			}
			if (ApplicationUtil.gtZero(r.getTeam())){
				if(ApplicationUtil.isTrue(sd)) {
					impostazioneDao.subCriteriaCampagnaTeam(r.getTeam(), criteriaCA, "id");
				}
				impostazioneDao.subCriteriaOperatoreTeam(r.getTeam(), criteriaC, "id");
			}
 
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("ca.id").as("campagna_id")))
				.add(Projections.countDistinct("sa.id").as("appuntamenti"))
				);
		ApplicationUtil.orderC3(criteria);
		return getList(criteria, ReportAppuntamentoCResultSet.class);
	}
	@SuppressWarnings({ "unchecked", "unused" })
	private List<ReportAppuntamentoCResultSet> getReportAppuntamentoCStoriaResultsetDati(Long userId, ReportCriteria r) {
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaAppuntamento.class, "sa");
		DetachedCriteria criteriaO = criteria.createCriteria("appuntamento", "app");
		DetachedCriteria criteriaE = criteria.createCriteria("esito", "esit",  ApplicationUtil.leftJoin());
		DetachedCriteria criteriaCA = criteriaO.createCriteria("contatto").createCriteria("campagna", "ca");
		DetachedCriteria criteriaA = criteriaO.createCriteria("agente", "ag", ApplicationUtil.leftJoin());
		DetachedCriteria criteriaC = criteria.createCriteria("utente", "op");
		if(ApplicationUtil.gtZero(userId)) {
			impostazioneDao.subCriteriaRoleSupervisore(userId, criteria, "ca", "op");
			impostazioneDao.subCriteriaSupervisore(userId, criteriaC, "id");
			if(ApplicationUtil.isTrue(sd)) {
				impostazioneDao.subCriteriaCampagnaSupervisore(userId, criteriaCA, "id");
			}
		}
		if(r != null) {
			ApplicationUtil.dataIntervalD(r.getInizio(), r.getFine(), "data", criteria);
			if(ApplicationUtil.gtZero(r.getAgente())) {
				criteriaA.add(Restrictions.idEq(r.getAgente()));
			}  
			if (ApplicationUtil.gtZero(r.getTeam())){
				if(ApplicationUtil.isTrue(sd)) {
					impostazioneDao.subCriteriaCampagnaTeam(r.getTeam(), criteriaCA, "id");
				}
				impostazioneDao.subCriteriaOperatoreTeam(r.getTeam(), criteriaC, "id");
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("ca.id").as("campagna_id")))
				.add(Projections.groupProperty("esit.id").as("esito_id"))
				.add(Projections.countDistinct("sa.id").as("appuntamenti"))
				);
		ApplicationUtil.orderC2(criteria);
		return getList(criteria, ReportAppuntamentoCResultSet.class);
	}
	@SuppressWarnings("unchecked")
	private List<ReportAppuntamentoCResultSet> getReportAppuntamentoCResultset(Long userId, ReportCriteria r) {
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(Appuntamento.class, "sa");
		DetachedCriteria criteriaCA = criteria.createCriteria("contatto").createCriteria("campagna", "ca");
		DetachedCriteria criteriaA = criteria.createCriteria("agente", "ag", ApplicationUtil.leftJoin());
		DetachedCriteria criteriaC = criteria.createCriteria("operatore", "op");
		if(ApplicationUtil.gtZero(userId)) {
			impostazioneDao.subCriteriaRoleSupervisore(userId, criteria, "ca", "op");
			impostazioneDao.subCriteriaSupervisore(userId, criteriaC, "id");
			if(ApplicationUtil.isTrue(sd)) {
				impostazioneDao.subCriteriaCampagnaSupervisore(userId, criteriaCA, "id");
			}
		}
		if(r != null) {
			ApplicationUtil.intervallo(ApplicationUtil.isTrue(r.getInserimento()), r.getInizio(), r.getFine(), criteria);
			if(r != null && ApplicationUtil.gtZero(r.getAgente())) {
				criteriaA.add(Restrictions.idEq(r.getAgente()));
			}
			if (ApplicationUtil.gtZero(r.getTeam())){
				if(ApplicationUtil.isTrue(sd)) {
					impostazioneDao.subCriteriaCampagnaTeam(r.getTeam(), criteriaCA, "id");
				}
				impostazioneDao.subCriteriaOperatoreTeam(r.getTeam(), criteriaC, "id");
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("ca.id").as("campagna_id")))
				.add(Projections.countDistinct("sa.id").as("appuntamenti"))
				);
		ApplicationUtil.orderC3(criteria);
		return getList(criteria, ReportAppuntamentoCResultSet.class);
	}
	@SuppressWarnings({ "unchecked", "unused" })
	private List<ReportAppuntamentoCResultSet> getReportAppuntamentoCResultsetDati(Long userId, ReportCriteria r) {
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(Appuntamento.class, "sa");
		DetachedCriteria criteriaE = criteria.createCriteria("esito", "esit", ApplicationUtil.leftJoin());
		DetachedCriteria criteriaCA = criteria.createCriteria("contatto").createCriteria("campagna", "ca");
		DetachedCriteria criteriaA = criteria.createCriteria("agente", "ag", ApplicationUtil.leftJoin());
		DetachedCriteria criteriaC = criteria.createCriteria("operatore", "op");
		if(ApplicationUtil.gtZero(userId)) {
			impostazioneDao.subCriteriaRoleSupervisore(userId, criteria, "ca", "op");
			impostazioneDao.subCriteriaSupervisore(userId, criteriaC, "id");
			if(ApplicationUtil.isTrue(sd)) {
				impostazioneDao.subCriteriaCampagnaSupervisore(userId, criteriaCA, "id");
			}
		}
		if(r != null) {
			ApplicationUtil.intervallo(ApplicationUtil.isTrue(r.getInserimento()), r.getInizio(), r.getFine(), criteria);
			if(ApplicationUtil.gtZero(r.getAgente())) {
				criteriaA.add(Restrictions.idEq(r.getAgente()));
			}  
			if (ApplicationUtil.gtZero(r.getTeam())){
				if(ApplicationUtil.isTrue(sd)) {
					impostazioneDao.subCriteriaCampagnaTeam(r.getTeam(), criteriaCA, "id");
				}
				impostazioneDao.subCriteriaOperatoreTeam(r.getTeam(), criteriaC, "id");
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("ca.id").as("campagna_id")))
				.add(Projections.groupProperty("esit.id").as("esito_id"))
				.add(Projections.countDistinct("sa.id").as("appuntamenti"))
				);
		ApplicationUtil.orderC2(criteria);
		return getList(criteria, ReportAppuntamentoCResultSet.class);
	}
	@SuppressWarnings("unchecked")
	private List<ReportContrattoCResultSet> getReportContrattoCStoriaResultset(Long userId, ReportCriteria r) {
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContratto.class, "sa");
		DetachedCriteria criteriaO = criteria.createCriteria("contratto", "app");
		DetachedCriteria criteriaP = criteriaO.createCriteria("prodotto");
		DetachedCriteria criteriaCA = criteriaO.createCriteria("contatto").createCriteria("campagna", "ca");
		DetachedCriteria criteriaA = criteriaO.createCriteria("agente", "ag", ApplicationUtil.leftJoin());
		DetachedCriteria criteriaC = criteria.createCriteria("utente", "op");
		if(ApplicationUtil.gtZero(userId)) {
			impostazioneDao.subCriteriaRoleSupervisore(userId, criteria, "ca", "op");
			impostazioneDao.subCriteriaSupervisore(userId, criteriaC, "id");
			if(ApplicationUtil.isTrue(sd)) {
				impostazioneDao.subCriteriaCampagnaSupervisore(userId, criteriaCA, "id");
			}
		}
		if(r != null) {
			ApplicationUtil.dataIntervalD(r.getInizio(), r.getFine(), "data", criteria);
			if(ApplicationUtil.gtZero(r.getAgente())) {
				criteriaA.add(Restrictions.idEq(r.getAgente()));
			}
			if (ApplicationUtil.gtZero(r.getTeam())){
				if(ApplicationUtil.isTrue(sd)) {
					impostazioneDao.subCriteriaCampagnaTeam(r.getTeam(), criteriaCA, "id");
				}
				impostazioneDao.subCriteriaOperatoreTeam(r.getTeam(), criteriaC, "id");
			}
			if (ApplicationUtil.gtZero(r.getProdottoId())) {
				criteriaP.add(Restrictions.idEq(r.getProdottoId()));
			}
			if (ApplicationUtil.gtZero(r.getCategoriaId())) {
				criteriaP.add(Restrictions.eq("categoria.id",r.getCategoriaId()));
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("ca.id").as("campagna_id")))
				.add(Projections.countDistinct("sa.id").as("contratti"))
				);
		ApplicationUtil.orderC3(criteria);
		return getList(criteria, ReportContrattoCResultSet.class);
	}
	@SuppressWarnings({ "unchecked", "unused" })
	private List<ReportContrattoCResultSet> getReportContrattoCStoriaResultsetDati(Long userId, ReportCriteria r) {
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContratto.class, "sa");
		DetachedCriteria criteriaO = criteria.createCriteria("contratto", "app");
		DetachedCriteria criteriaP = criteriaO.createCriteria("prodotto");
		DetachedCriteria criteriaCA = criteriaO.createCriteria("contatto").createCriteria("campagna", "ca");
		DetachedCriteria criteriaE = criteria.createCriteria("esito", "esit",  ApplicationUtil.leftJoin());
		DetachedCriteria criteriaA = criteriaO.createCriteria("agente", "ag", ApplicationUtil.leftJoin());
		DetachedCriteria criteriaC = criteria.createCriteria("utente", "op");
		if(ApplicationUtil.gtZero(userId)) {
			impostazioneDao.subCriteriaRoleSupervisore(userId, criteria, "ca", "op");
			impostazioneDao.subCriteriaSupervisore(userId, criteriaC, "id");
			if(ApplicationUtil.isTrue(sd)) {
				impostazioneDao.subCriteriaCampagnaSupervisore(userId, criteriaCA, "id");
			}
		}
		if(r != null) {
			ApplicationUtil.dataIntervalD(r.getInizio(), r.getFine(), "data", criteria);
			if(ApplicationUtil.gtZero(r.getAgente())) {
				criteriaA.add(Restrictions.idEq(r.getAgente()));
			}  
			if (ApplicationUtil.gtZero(r.getTeam())){
				if(ApplicationUtil.isTrue(sd)) {
					impostazioneDao.subCriteriaCampagnaTeam(r.getTeam(), criteriaCA, "id");
				}
				impostazioneDao.subCriteriaOperatoreTeam(r.getTeam(), criteriaC, "id");
			}
			if (ApplicationUtil.gtZero(r.getProdottoId())) {
				criteriaP.add(Restrictions.idEq(r.getProdottoId()));
			}
			if (ApplicationUtil.gtZero(r.getCategoriaId())) {
				criteriaP.add(Restrictions.eq("categoria.id",r.getCategoriaId()));
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("ca.id").as("campagna_id")))
				.add(Projections.groupProperty("esit.id").as("esito_id"))
				.add(Projections.countDistinct("sa.id").as("contratti"))
				);
		ApplicationUtil.orderC2(criteria);
		return getList(criteria, ReportContrattoCResultSet.class);
	}
	@SuppressWarnings("unchecked")
	private List<ReportContrattoCResultSet> getReportContrattoCResultset(Long userId, ReportCriteria r) {
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(Contratto.class, "sa");
		DetachedCriteria criteriaP = criteria.createCriteria("prodotto");
		DetachedCriteria criteriaCA = criteria.createCriteria("contatto").createCriteria("campagna", "ca");
		DetachedCriteria criteriaA = criteria.createCriteria("agente", "ag", ApplicationUtil.leftJoin());
		DetachedCriteria criteriaC = criteria.createCriteria("operatore", "op");
		if(ApplicationUtil.gtZero(userId)) {
			impostazioneDao.subCriteriaRoleSupervisore(userId, criteria, "ca", "op");
			impostazioneDao.subCriteriaSupervisore(userId, criteriaC, "id");
			if(ApplicationUtil.isTrue(sd)) {
				impostazioneDao.subCriteriaCampagnaSupervisore(userId, criteriaCA, "id");
			}
		}
		if(r != null) {
			ApplicationUtil.intervallo(ApplicationUtil.isTrue(r.getInserimento()), r.getInizio(), r.getFine(), criteria);
			if(ApplicationUtil.gtZero(r.getAgente())) {
				criteriaA.add(Restrictions.idEq(r.getAgente()));
			}
			if (ApplicationUtil.gtZero(r.getTeam())){
				if(ApplicationUtil.isTrue(sd)) {
					impostazioneDao.subCriteriaCampagnaTeam(r.getTeam(), criteriaCA, "id");
				}
				impostazioneDao.subCriteriaOperatoreTeam(r.getTeam(), criteriaC, "id");
			}
			if (ApplicationUtil.gtZero(r.getProdottoId())) {
				criteriaP.add(Restrictions.idEq(r.getProdottoId()));
			}
			if (ApplicationUtil.gtZero(r.getCategoriaId())) {
				criteriaP.add(Restrictions.eq("categoria.id",r.getCategoriaId()));
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("ca.id").as("campagna_id")))
				.add(Projections.countDistinct("sa.id").as("contratti"))
				);
		ApplicationUtil.orderC3(criteria);
		return getList(criteria, ReportContrattoCResultSet.class);
	}
	@SuppressWarnings({ "unchecked", "unused" })
	private List<ReportContrattoCResultSet> getReportContrattoCResultsetDati(Long userId, ReportCriteria r) {
		Boolean sd = impostazioneDao.separaDati();
		DetachedCriteria criteria = DetachedCriteria.forClass(Contratto.class, "sa");
		DetachedCriteria criteriaP = criteria.createCriteria("prodotto");
		DetachedCriteria criteriaCA = criteria.createCriteria("contatto").createCriteria("campagna", "ca");
		DetachedCriteria criteriaE = criteria.createCriteria("esito", "esit",  ApplicationUtil.leftJoin());
		DetachedCriteria criteriaA = criteria.createCriteria("agente", "ag", ApplicationUtil.leftJoin());
		DetachedCriteria criteriaC = criteria.createCriteria("operatore", "op");
		if(ApplicationUtil.gtZero(userId)) {
			impostazioneDao.subCriteriaRoleSupervisore(userId, criteria, "ca", "op");
			impostazioneDao.subCriteriaSupervisore(userId, criteriaC, "id");
			if(ApplicationUtil.isTrue(sd)) {
				impostazioneDao.subCriteriaCampagnaSupervisore(userId, criteriaCA, "id");
			}
		}
		if(r != null) {
			ApplicationUtil.intervallo(ApplicationUtil.isTrue(r.getInserimento()), r.getInizio(), r.getFine(), criteria);
			if(ApplicationUtil.gtZero(r.getAgente())) {
				criteriaA.add(Restrictions.idEq(r.getAgente()));
			}  
			if (ApplicationUtil.gtZero(r.getTeam())){
				if(ApplicationUtil.isTrue(sd)) {
					impostazioneDao.subCriteriaCampagnaTeam(r.getTeam(), criteriaCA, "id");
				}
				impostazioneDao.subCriteriaOperatoreTeam(r.getTeam(), criteriaC, "id");
			}
			if (ApplicationUtil.gtZero(r.getProdottoId())) {
				criteriaP.add(Restrictions.idEq(r.getProdottoId()));
			}
			if (ApplicationUtil.gtZero(r.getCategoriaId())) {
				criteriaP.add(Restrictions.eq("categoria.id",r.getCategoriaId()));
			}
		}
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty("ca.id").as("campagna_id")))
				.add(Projections.groupProperty("esit.id").as("esito_id"))
				.add(Projections.countDistinct("sa.id").as("contratti"))
				);
		ApplicationUtil.orderC2(criteria);
		return getList(criteria, ReportContrattoCResultSet.class);
	}
	
	
	private DetachedCriteria  getOperatoreSondaggioByCriteria(Long user) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Sondaggio.class);
		criteria.add(Restrictions.eq("utente.id",user));
		return criteria;
	}
	@SuppressWarnings("unchecked")
	public List<Sondaggio>  getOperatoreSondaggio(Long user, int start, int size) {
		DetachedCriteria criteria = getOperatoreSondaggioByCriteria(user);
		criteria.addOrder(Order.desc("id"));
		return getList(criteria, start, size);
	}
	public int  getOperatoreSondaggioSize(Long user) {
		DetachedCriteria criteria = getOperatoreSondaggioByCriteria(user);
		return getListSize(criteria);
	}
	public void  salvaCampagna(RiepilogoCampagnaList rch) {
		Campagna c = (Campagna) get(Campagna.class, rch.getEid());
		c.setStato(rch.getStato());
		c.setPriorita(rch.getPriorita());
		salvaAggiorna(c);
	}
	public void  salvaLista(Long campagnaId, RiepilogoCampagnaList rch) {
		Lista c = (Lista) get(Lista.class, rch.getEid());
		c.setStato(rch.getStato());
		c.setPriorita(rch.getPriorita());
		salvaAggiorna(c);
	}
}